<?php
/**
 * Seotoaster 2.0 plugin bootstrap.
 *
 * @todo Add more comments
 * @author Seotoaster core team <core.team@seotoaster.com>
 */

class Api extends Tools_Plugins_Abstract {

    const APIURL = 'https://mojo.seosamba.com/plugin/api/run/'; // http://samba3com.seosalsa.com  // http://www.mojo.com https://mojo.seosamba.com

	const ACTION_POSTFIX = 'Action';

	const PLUGIN_ENABLED = 'enabled';

    const PAGE_SAMPLE_NUMBER = 25;

    const PAGE_TYPE_CATEGORY = 'category';

    const PAGE_TYPE_PAGE = 'page';

    const PAGE_TYPE_NOCATEGORY = 'noCategory';

    const PAGE_TYPE_NEWS = 'news';

    const PAGE_TYPE_PRODUCT = 'product';

    const PAGE_INDEX_HTML = 'index.html';

    const SEO_DATA_NETCONTENT_START = '<!-- SeoData_Netcontent-->';

    const SEO_DATA_NETCONTENT_END = '<!-- END_SeoData_Netcontent-->';

    const SEO_DATA_SAMBA_ANALYTIC_START = '<!-- SeoData_SambaAnalytic-->';

    const SEO_DATA_SAMBA_ANALYTIC_END = '<!-- END_SeoData_SambaAnalytic-->';

    const SEO_DATA_GOOGLE_ANALYTIC_START = '<!-- SeoData_GoogleAnalytic-->';

    const SEO_DATA_GOOGLE_ANALYTIC_END = '<!-- END_SeoData_GoogleAnalytic-->';

    const REDIRECT_LIMIT = 50;

	private $_authenticateAdapter = null;

	private $_call   = null;

	private $_params = array();
	
	private static $_encryptMessageFlag = true;

	private $_productPageUrl = null;

	protected $_options = null;
	
	protected $_isPromoInstalled = false;
	
	protected $_isShoppingInstalled = null;
	
	private $_zendDb = null;

    protected $_toasterPages = array();

    protected $_indexPage = null;

	/**
	 * List of action that should be allowed to specific roles
	 *
	 * By default all of actions of your plugin are available to the guest user
	 * @var array
	 */
	protected $_securedActions = array(
		Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'secured'
        )
	);

	/**
	 * Init method.
	 *
	 * Use this method to init your plugin's data and variables
	 * Use this method to init specific helpers, view, etc...
	 */
	protected function _init() {
		$this->_zendDb = new Zend_Db_Table();
		$this->_view->setScriptPath(__DIR__ . '/system/views/');
	}

	public function  __construct($options, $seotoasterData) {
		parent::__construct($options, $seotoasterData);
		$this->_request = new Zend_Controller_Request_Http();
		$this->_options = $options;
	}
	/**
	 * Main entry point
	 *
	 * @param array $requestedParams
	 * @return mixed $dispatcherResult
	 */
	public function run($requestedParams = array()) {
		$this->_requestedParams = $requestedParams;
		if(!isset($requestedParams['run']) || !$requestedParams['run']) {
			throw new Exceptions_SeotoasterException('Wrong api call');
		}
		$this->_call = $requestedParams['run'];
		unset ($requestedParams['run']);
		$this->_params = $requestedParams;
		$headers = array();
		foreach ($_SERVER as $k => $v)
		{
			if (substr($k, 0, 5) == "HTTP_")
			{
				$k = str_replace('_', ' ', substr($k, 5));
				$k = str_replace(' ', '-', ucwords(strtolower($k)));
				$headers[$k] = $v;
			}
		}
		$sambaToken = self::decryptSambaToken($headers['Encrypted-Samba-Token'], $headers['Auth-Token'], $headers['Iv']);
		$toasterAccess = $this->_toasterAccess($sambaToken);
		if($toasterAccess != false) {
			$this->_response($this->_callDispatcher());
		}
		else {
			$data['done'] = false;
			$data['message'] = 'Wrong TOASTER TOKEN';
			$this->_response($data);
		}
	}

	private function _getMethod() {
		return strtolower($_SERVER['REQUEST_METHOD']);
	}

	private function _callDispatcher() {
		$callAction = '_'. $this->_call . ucfirst($this->_getMethod()) . self::ACTION_POSTFIX;
		if($this->_request->isPut() || $this->_request->isDelete()) {
			parse_str($this->_request->getRawBody(), $params);
			if(isset($params['auth']) && isset($params['iv']) && isset($params['message']) ) {
				$params = self::decryptMessage($params['message'], $params['auth'], $params['iv']);
			}
            foreach($params as $key => $value) {
                $this->_params[$key] = $value;
            }
		}
		else {
			if(isset($this->_params['auth']) && isset($this->_params['iv']) && isset($this->_params['message']) ) {
				$this->_params = self::decryptMessage($this->_params['message'], $this->_params['auth'], $this->_params['iv']);
			}
		}

		if(method_exists($this, $callAction)) {
			return $this->$callAction();
		}
        else {
            $plugins = Application_Model_Mappers_PluginMapper::getInstance()->findEnabled();
            if (!empty($plugins)) {
                $actionExists = false;
                $pluginError = false;
                $errorMessage = '';
                $pluginResponses = array();
                foreach ($plugins as $plugin) {
                    $pluginClassName = $plugin->getName();
                    $pluginClass = new Zend_Reflection_Class($pluginClassName);
                    $pluginApiActionsMethod = 'pluginApiActions';
                    $apiActions = $pluginClass->hasMethod($pluginApiActionsMethod);
                    if (!$apiActions) {
                        continue;
                    }
                    $verifyApiActions = $pluginClass->getMethod($pluginApiActionsMethod);
                    if (!$verifyApiActions->isStatic()) {
                        continue;
                    }
                    $apiExistingActions = $pluginClassName::$pluginApiActionsMethod();
                    $pluginAction = $this->_call;
                    if (in_array($pluginAction, $apiExistingActions)) {
                        try {
                            $verifyFinalMethod = $pluginClass->getMethod($pluginAction);
                        } catch (Exception $e) {
                            $pluginError = true;
                            $errorMessage = 'Method '. $this->_call . ' does not exist'.' in the plugin '.$pluginClassName.' with message '. $e->getMessage();
                            break;
                        }
                        if (!$verifyFinalMethod->isStatic()) {
                            continue;
                        }
                        $actionExists = true;
                        try {
                            $pluginResponse = $pluginClassName::$pluginAction($this->_params);
                            if (!empty($pluginResponse)) {
                                $pluginResponses[$pluginClassName] = $pluginResponse;
                            }
                        } catch (Exception $e) {
                            $pluginError = true;
                            $errorMessage = 'Error in method '.$this->_call.' in the plugin '.$pluginClassName.' with message '. $e->getMessage();
                            break;
                        }
                    }
                }

                if ($pluginError === true) {
                    return array('done' => false, 'message' => $errorMessage);
                }

                if ($actionExists) {
                    if (!empty($pluginResponses)) {
                        return $pluginResponses;
                    } else {
                        return array('done' => true);
                    }
                }
            }

            return array('done' => false, 'message' => 'Method '. $this->_call . ' does not exist');
        }
	}

	public static function request($method, $action, $data ,$url = self::APIURL) {
		$encryptedSambaToken = array();
		if(extension_loaded('mcrypt')) {
			$encryptedSambaToken = (isset($data['sambaToken'])) ? self::encryptSambaToken($data['sambaToken']) : array();
		}
		unset ($data['sambaToken']);
		//$data = self::encryptMessage($data);
		if(strtoupper($method) == 'GET') {
			foreach ($data as $k=>$v) {
                if(is_array($v)) {$v = serialize($v);}
				$action .= '/' . $k . '/' . $v;
			}
		}
		$curl = curl_init();
		curl_setopt($curl, CURLOPT_URL, $url.$action);
		curl_setopt($curl, CURLOPT_HEADER, false);
		curl_setopt($curl, CURLOPT_HTTPHEADER, $encryptedSambaToken);
		curl_setopt($curl, CURLOPT_VERBOSE, 1);
		curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, FALSE);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
		if(strtoupper($method) == 'POST') {
			curl_setopt($curl, CURLOPT_POST, 1);
			$nvpRequest = http_build_query($data);
			curl_setopt($curl, CURLOPT_POSTFIELDS, $nvpRequest);
		}
		else if( (strtoupper($method) == 'PUT') || (strtoupper($method) == 'DELETE') ) {
			curl_setopt($curl, CURLOPT_CUSTOMREQUEST, strtoupper($method));
			$nvpRequest = http_build_query($data);
			curl_setopt($curl, CURLOPT_POSTFIELDS, $nvpRequest);
		}
		$response = curl_exec($curl);
		$responseSet = json_decode($response, true);
		if(isset($responseSet['auth']) && isset($responseSet['iv']) && isset($responseSet['message']) ) {
			if(extension_loaded('mcrypt')) {
				$response = self::decryptMessage($responseSet['message'], $responseSet['auth'], $responseSet['iv']);
			}
			else {
				$response = array('done' => false, 'message' => 'Need php mcrypt extention!');
			}
		}
		else {$response = $responseSet;}
		return $response;
	}

	public static function _response($data, $code = 200, $contentType = 'application/json') {
		$response = new Zend_Controller_Response_Http();
		//$response->setHeader('Content-Type', $contentType);
		$response->setHttpResponseCode($code);
		/*if(!$body) {
			$body = Tools_Codes::$codes[$code];
		}*/
		if(self::$_encryptMessageFlag) {
			$data = self::encryptMessage($data);
		}
		$response->setBody(json_encode($data));
		$response->sendResponse();
		exit;
	}

	public static function encryptSambaToken($sambaToken){
		$authToken = md5(uniqid());
		$initVectorSize = mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_CFB);
		$initVector = mcrypt_create_iv($initVectorSize, MCRYPT_RAND);
		$key = substr($authToken, 14, -1) . substr($authToken, 7, 7);
		$encrypted = mcrypt_encrypt(MCRYPT_RIJNDAEL_256, $key, $sambaToken, MCRYPT_MODE_CFB, $initVector);
		return array('Encrypted-Samba-Token: ' . urlencode($encrypted), 'Auth-Token: ' . $authToken, 'Iv:' . urlencode($initVector) );
	}

	public static function decryptSambaToken($sambaToken, $authToken ,$ivDecode) {
		$key = substr($authToken, 14, -1) . substr($authToken, 7, 7);
		$initVector = urldecode($ivDecode);
		$decripted = mcrypt_decrypt(MCRYPT_RIJNDAEL_256, $key, urldecode($sambaToken), MCRYPT_MODE_CFB, $initVector);
		return $decripted;
	}

	public static function encryptMessage($message) {
		$authToken = md5(uniqid());
		$message = json_encode($message);
		$initVectorSize = mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_CFB);
		$initVector = mcrypt_create_iv($initVectorSize, MCRYPT_RAND);
		$key = substr($authToken, 14, -1) . strrev(substr($authToken, 16, -1));
		$encrypted = mcrypt_encrypt(MCRYPT_RIJNDAEL_256, $key, $message, MCRYPT_MODE_CFB, $initVector);
		return array('message' => urlencode($encrypted), 'auth' => $authToken, 'iv' => urlencode($initVector) );
	}

	public static function decryptMessage($message, $authToken ,$ivDecode) {
		$key = substr($authToken, 14, -1) . strrev(substr($authToken, 16, -1));
		$initVector = urldecode($ivDecode);
		$decripted = mcrypt_decrypt(MCRYPT_RIJNDAEL_256, $key, urldecode($message), MCRYPT_MODE_CFB, $initVector);
		$decripted = json_decode($decripted, true);
		return $decripted;
	}

    /**
     * Get system or plugin current version
     *
     * @return mixed
     */
    private function _versionGetAction()
    {
        $versionTypes = array('system', 'plugin');
        if (empty($this->_params['versionType']) || !in_array($this->_params['versionType'], $versionTypes, true)) {
            return array('done' => false, 'message' => 'Please specify version type');
        }

        $versionType = $this->_params['versionType'];

        if ($versionType === 'plugin' && empty($this->_params['pluginName'])) {
            return array('done' => false, 'message' => 'Please specify plugin name');
        }

        if ($versionType === 'plugin') {
            $pluginName = $this->_params['pluginName'];
            $pluginMapper = Application_Model_Mappers_PluginMapper::getInstance();
            $pluginModel = $pluginMapper->findByName($pluginName);
            if (!$pluginModel instanceof Application_Model_Models_Plugin) {
                return array('done' => false, 'message' => 'Plugin not found');
            }

            $pluginStatus = $pluginModel->getStatus();
            if ($pluginStatus === Application_Model_Models_Plugin::DISABLED) {
                return array('done' => false, 'message' => 'Plugin disabled');
            }
            $pluginVersion = $pluginModel->getVersion();

            return array('done' => true, 'version' => $pluginVersion);

        } else {
            $systemConfig = Application_Model_Mappers_ConfigMapper::getInstance()->getConfig();
            if (empty($systemConfig) || empty($systemConfig['version'])) {
                return array('done' => false, 'message' => 'Version doesn\'t exist');
            }

            return array('done' => true, 'version' => $systemConfig['version']);
        }

    }


    public function isShoppingInstalled(){
        return $this->_isShoppingInstalled();
    }

    private function _pluginPutAction() {
		if(!isset($this->_params['license'])) {
			$data['done'] = false;
			return $data;
		}

		$plugin = Application_Model_Mappers_PluginMapper::getInstance()->findByName(strtolower($this->_params['license']['pluginName']));

        if(empty($plugin)) {
            return array('done' => false);
        }

		$salt = strrev(md5(strrev(Api_Tools_System::getDomain())
            . base64_encode(ucfirst($plugin->getName()))));
		$license = json_encode($this->_params['license']);

		$pluginDbTable = new Application_Model_DbTable_Plugin();
        $pluginDbTable->update(array("license" => new Zend_Db_Expr("AES_ENCRYPT('".addslashes($license)."','".$salt."')" )), array('name = ?' => $plugin->getName()));
		$data['done'] = true;
		return $data;
	}

	private function _websiteIdCardPutAction() {
		$widcardPlugin = Tools_Factory_PluginFactory::createPlugin('widcard',array(), array('websiteUrl' => $this->_websiteUrl));
		$data = $widcardPlugin->sambaUpdateWebsiteIdCard($this->_params);
		return $data;
	}

    private function _websiteIdCardPhoneNumberReplacePostAction() {
        $widcardPlugin = Tools_Factory_PluginFactory::createPlugin('widcard',array(), array('websiteUrl' => $this->_websiteUrl));
        $data = $widcardPlugin->sambaReplacePhoneNumber($this->_params);
        return $data;
    }

    private function _websiteIdCardPhoneNumberReplaceDeleteAction() {
        $widcardPlugin = Tools_Factory_PluginFactory::createPlugin('widcard',array(), array('websiteUrl' => $this->_websiteUrl));
        $data = $widcardPlugin->sambaReplacePhoneNumber($this->_params);
        return $data;
    }

	private function _netContentPostAction() {
		$netcontentPlugin = Tools_Factory_PluginFactory::createPlugin('netcontent',array(), array('websiteUrl' => $this->_websiteUrl));
		$data = $netcontentPlugin->sambaNetContentWidget($this->_params);
		return $data;
	}

	private function _netContentDeleteAction() {
		$netcontentPlugin = Tools_Factory_PluginFactory::createPlugin('netcontent',array(), array('websiteUrl' => $this->_websiteUrl));
		$data = $netcontentPlugin->deleteSambaNetContentWidget($this->_params);
		return $data;
	}

    protected function _getExcludePages($excludeIndexPage = false) {
        $excludePages = array();
        $notFoundPage = Application_Model_Mappers_PageMapper::getInstance()->fetchByOption(Application_Model_Models_Page::OPT_404PAGE, false);

        if(!empty($notFoundPage)) {

            foreach($notFoundPage as $page) {
                array_push($excludePages, $page->getId());
            }
        }

        if($this->_isShoppingInstalled() !== null) {
            array_push($excludePages, $this->_isShoppingInstalled);
        }

        $pageDbTable = new Application_Model_DbTable_Page();
        $whereSimpleLink = $pageDbTable->getAdapter()->quoteInto("external_link_status = ?", '1');
        $simpleLinkPages = Application_Model_Mappers_PageMapper::getInstance()->fetchAll($whereSimpleLink);

        if(!empty($simpleLinkPages)) {

            foreach($simpleLinkPages as $linkPage) {
                array_push($excludePages, $linkPage->getId());
            }
        }

        // Exclude index page if index page is under NoCategory
        if($excludeIndexPage) {
            $indexPage = $this->_zendDb->getAdapter()->fetchRow(
                $this->_zendDb->getAdapter()->select()->from(array('p' => 'page'), array('id'))->where(
                $this->_zendDb->getAdapter()->quoteInto('p.url = ?', self::PAGE_INDEX_HTML)
            )->group('p.id'));

            if(!empty($indexPage)) {
                array_push($excludePages, $indexPage['id']);
            }
        }

        return $excludePages;
    }

    private function _findPagesGetAction() {
        $data['done'] = true;
        $data['pages'] = Api_Tools_System::findUrls($this->_params['query']);
        return $data;
    }

	private function _pagesGetAction() {
		self::$_encryptMessageFlag = false;
		if(isset($this->_params['list'])) {
            if(!isset($this->_params['forDeepLinks'])) {
                $limit = (!empty($this->_params['limit'])) ? $this->_params['limit'] : self::REDIRECT_LIMIT;
                $offset = (!empty($this->_params['offset'])) ? $this->_params['offset'] : null;
                $data['pages'] = Api_Tools_System::getUrls($limit, $offset);
                $data['count'] = Api_Tools_System::getUrls(null, null, true);
            }else {
                $data['pages'] = Application_Model_Mappers_PageMapper::getInstance()->fetchIdUrlPairs();
            }
            $data['done'] = true;
            return $data;
		}
		else {
            try{
                $pagesTable = new Application_Model_DbTable_Page();
                $overwrite = null;
                $pulledPages = array();
                if(isset($this->_params['overwrite']) ) {
                    $overwrite = true;
                }
                $fields = json_decode($this->_params['fields'],true);
                /*
                 * Prepare set of pages which must not be pulled to toaster
                 * Get shopping category page ID (if it exists)
                 * */
                 $excludePages = $this->_getExcludePages();

                if(isset($this->_params['pulledPages'])) {
                    $pulledPages = json_decode($this->_params['pulledPages'], true);
                }

                $queue = new Api_Tools_Queue($this->_params, $this->_websiteUrl);
                /*
                 * Select and add to returning collection the index.html page
                 * index.html selects and process separate from other pages!!!
                 * */
                $queue->findIndexPage($fields, $excludePages, $overwrite);
                if(!isset($this->_params['forceId'])) {
                    $queue->pullCategoryPages($fields, $excludePages, $overwrite);
                    $queue->pullRegularPages($fields, $excludePages, $overwrite, $pulledPages);
                    $queue->pullNoCategoryPages($fields, $excludePages, $overwrite, $pulledPages);
                    $queue->pullNewsPages($fields, $excludePages, $overwrite, $pulledPages);
                    $queue->pullProductPages($fields, $excludePages, $overwrite, $pulledPages);

                    if(!empty($pulledPages)) {
                        $queue->_toasterPages['originalPagesToSyncDelete'] = array($queue->_indexPage['id']);
                        $whereExclude = null;
                        if(!empty($excludePages)) {
                            $whereExclude = $pagesTable->getAdapter()->quoteInto("page.id NOT IN(?)", $excludePages);
                        }

                        if(isset($pulledPages[self::PAGE_TYPE_CATEGORY]) && !empty($pulledPages[self::PAGE_TYPE_CATEGORY])) {
                        /*
                         * Collect all categories to sync/delete
                         * */
                            $select = $pagesTable->getAdapter()->select()->from('page', array('id'));
                            if($whereExclude !== null) {
                                $select->where($whereExclude);
                            }
                            $select->where($pagesTable->getAdapter()->quoteInto("page.parent_id = ?", 0));
                            $pulledCategoryId = array();
                            foreach($pulledPages[self::PAGE_TYPE_CATEGORY] as $catId => $pageId) {
                                array_push($pulledCategoryId, $catId);
                            }
                            $select->where($pagesTable->getAdapter()->quoteInto("page.id IN(?)", $pulledCategoryId));

                            $pulledCategoryId = $pagesTable->getAdapter()->fetchCol($select);
                            if(!empty($pulledCategoryId)) {
                                $queue->_toasterPages['originalPagesToSyncDelete'] = array_merge($queue->_toasterPages['originalPagesToSyncDelete'], $pulledCategoryId);
                            }
                         /*
                         * Collect all regular pages to sync/delete
                         * */
                            $pageIds = array();
                            $select = $pagesTable->getAdapter()->select()->from('page', array('id'));

                            foreach($pulledPages[self::PAGE_TYPE_CATEGORY] as $catId => $pageId) {
                                if(!empty($pageId)) {
                                    $pageIds = array_merge($pageIds, $pageId);
                                    $whereCondition = $pagesTable->getAdapter()->quoteInto("page.parent_id = ?", $catId);
                                    $whereCondition .= $pagesTable->getAdapter()->quoteInto(" AND page.id IN(?)", $pageId);

                                    if($whereExclude !== null) {
                                        $whereCondition .= $pagesTable->getAdapter()->quoteInto(" AND page.id NOT IN(?)", $whereExclude);
                                    }

                                    $wherePages[] = '(' . $whereCondition . ')';
                                }
                            }

                            if(!empty($pageIds)) {
                                $wherePages[] = $pagesTable->getAdapter()->quoteInto("( page.id IN (?) )", $pageIds);
                            }

                            if(isset($wherePages) && !empty($wherePages)) {
                                $select->where(implode(' OR ', $wherePages));
                            }

                            $pulledPageId = $pagesTable->getAdapter()->fetchCol($select);
                            if(!empty($pulledPageId)) {
                                $queue->_toasterPages['originalPagesToSyncDelete'] = array_merge($queue->_toasterPages['originalPagesToSyncDelete'], $pulledPageId);
                            }
                        }
                        /*
                         * Collect all NO-CATEGORY pages to sync/delete
                         * */
                        if(isset($pulledPages[self::PAGE_TYPE_NOCATEGORY]) && !empty($pulledPages[self::PAGE_TYPE_NOCATEGORY])) {
                            $select = $pagesTable->getAdapter()->select()->from('page', array('id'));
                            $whereCondition = $pagesTable->getAdapter()->quoteInto("page.parent_id = ?", -1);
                            $whereCondition .= $pagesTable->getAdapter()->quoteInto(" AND page.id IN(?)", $pulledPages[self::PAGE_TYPE_NOCATEGORY]);
                            if($whereExclude !== null) {
                                $whereCondition .= $pagesTable->getAdapter()->quoteInto(" AND page.id NOT IN(?)", $whereExclude);
                            }
                            $select->where($whereCondition);
                            $pulledNoCategoryId = $pagesTable->getAdapter()->fetchCol($select);
                            if(!empty($pulledNoCategoryId)) {
                                $queue->_toasterPages['originalPagesToSyncDelete'] = array_merge($queue->_toasterPages['originalPagesToSyncDelete'], $pulledNoCategoryId);
                            }
                        }
                        /*
                         * Collect all NEWS pages to sync/delete
                         * */
                        if(isset($pulledPages[self::PAGE_TYPE_NEWS]) && !empty($pulledPages[self::PAGE_TYPE_NEWS])) {
                            $whereCondition = $pagesTable->getAdapter()->quoteInto("p.id IN(?)", $pulledPages[self::PAGE_TYPE_NEWS]);

                            if($whereExclude !== null) {
                                $whereCondition .= $pagesTable->getAdapter()->quoteInto(" AND p.id NOT IN(?)", $whereExclude);
                            }

                            $select = $this->_zendDb->getAdapter()->select()
                                ->from(array('pnn' => 'plugin_newslog_news'), array('pageId' => 'page_id'))
                                ->join(array('p' => 'page'), 'p.id = pnn.page_id', NULL)
                                ->where($whereCondition)
                                ->order(array('p.id ASC'));
                            $select->where($whereCondition);

                            $pulledNewsId = $pagesTable->getAdapter()->fetchCol($select);
                            if(!empty($pulledNoCategoryId)) {
                                $queue->_toasterPages['originalPagesToSyncDelete'] = array_merge($queue->_toasterPages['originalPagesToSyncDelete'], $pulledNewsId);
                            }
                        }
                        /*
                         * Collect all PRODUCT pages to sync/delete
                         * */
                        if(isset($pulledPages[self::PAGE_TYPE_PRODUCT]) && !empty($pulledPages[self::PAGE_TYPE_PRODUCT]) && $this->_isShoppingInstalled() !== null) {
                            $select = $pagesTable->getAdapter()->select()->from('page', array('id'));
                            $whereCondition = $pagesTable->getAdapter()->quoteInto("page.parent_id = ?", $this->_isShoppingInstalled);
                            $whereCondition .= $pagesTable->getAdapter()->quoteInto(" AND page.id IN(?)", $pulledPages[self::PAGE_TYPE_PRODUCT]);
                            if($whereExclude !== null) {
                                $whereCondition .= $pagesTable->getAdapter()->quoteInto(" AND page.id NOT IN(?)", $whereExclude);
                            }
                            $select->where($whereCondition);
                            $pulledProductId = $pagesTable->getAdapter()->fetchCol($select);
                            if(!empty($pulledProductId)) {
                                $queue->_toasterPages['originalPagesToSyncDelete'] = array_merge($queue->_toasterPages['originalPagesToSyncDelete'], $pulledProductId);
                            }
                        }
                    }
                }else {
                    $methodName = 'pull' . ucfirst($this->_params['forceType']).'Pages';
                    $forceId = (integer)$this->_params['forceId'];
                    $queue->$methodName($fields, $excludePages, $overwrite, $pulledPages, $forceId);
                }
                return $queue->_toasterPages;

            }catch (Exception $e) {
                error_log($e->getMessage());
                return array('done' => false, 'message' => $e->getMessage());
            }
		}
	}

	private function _optimizeTableDeleteAction() {
        $queue = new Api_Tools_Queue($this->_params, $this->_websiteUrl);
        return $queue->cleanOptimizedPages($this->_params);
	}

	private function _deeplinkListGetAction() {
		$data = $this->_getDeeplinksList();
		return $data;
	}

	private function _getDeeplinksList() {
		$list = array_reverse(Application_Model_Mappers_DeeplinkMapper::getInstance()->fetchAll(null, array('id')));
		if(is_array($list) && count($list) > 0) {
			foreach ($list as $objectItem) {
				$deeplink['id'] = $objectItem->getId();
				$deeplink['name'] = $objectItem->getName();
				$deeplink['url'] = $objectItem->getUrl();
				$deeplink['type'] = $objectItem->getType();
				$deeplink['banned'] = $objectItem->getBanned();
				$deeplink['nofollow'] = $objectItem->getNofollow();
				$deeplink['pageId'] = $objectItem->getPageId();
				$deeplink['pageId'] = $objectItem->getPageId();
				$deeplinks[] = $deeplink;
			}
			$data = $deeplinks;
		}
		else {
			$data = false;
		}
		return $data;
	}

	private function _deeplinkPostAction() {
		if($this->_request->isPost()) {
			$autoDeeplinks = (isset($this->_params['autoDeeplinks'])) ? $this->_params['autoDeeplinks'] : false;
			if($autoDeeplinks != false) {
				$deeplink       = new Application_Model_Models_Deeplink();
				$deeplink->registerObserver(new Tools_Seo_Watchdog());
				$pageMapper       = Application_Model_Mappers_PageMapper::getInstance();
				foreach ($autoDeeplinks as $autoDeeplink) {
						$deeplink->setType(Application_Model_Models_Deeplink::TYPE_INTERNAL);
						$page = $pageMapper->find($autoDeeplink['url']);
						$deeplink->setPageId($autoDeeplink['url']);
						$deeplink->setUrl($page->getUrl());
						$deeplink->setName($autoDeeplink['anchorText']);
						$deeplink->setBanned(false);
						$deeplink->setNofollow($autoDeeplink['nofollow']);
						Application_Model_Mappers_DeeplinkMapper::getInstance()->save($deeplink);
						$anchors[] = $autoDeeplink['anchorText'];
				}
				$deeplink->notifyObservers();
				$deeplinksDbTable = new Application_Model_DbTable_Deeplink();
				$where = $deeplinksDbTable->getAdapter()->quoteInto('name IN(?)', $anchors);
				$autoDplId = $deeplinksDbTable->getAdapter()->fetchAll($deeplinksDbTable->getAdapter()->select()->from('deeplink', array('id','name'))->where($where));
				$data['error'] = 0;
				$data['responseText'] = 'Autodeeplinks added.';
				$data['autoDplId'] = $autoDplId;
			return $data;
			}
			$deeplinksForm    = new Application_Form_Deeplink();
            $deeplinksForm->removeElement(Tools_System_Tools::CSRF_SECURE_TOKEN);
			if($deeplinksForm->isValid($this->_params)) {
				$pageMapper       = Application_Model_Mappers_PageMapper::getInstance();
				$data           = $deeplinksForm->getValues();
				$deeplink       = new Application_Model_Models_Deeplink();
				if(intval($data['url'])) {
					$deeplink->setType(Application_Model_Models_Deeplink::TYPE_INTERNAL);
					$page = $pageMapper->find($data['url']);
					$deeplink->setPageId($data['url']);
					$deeplink->setUrl($page->getUrl());
				}
				else {
					$deeplink->setType(Application_Model_Models_Deeplink::TYPE_EXTERNAL);
					$urlValidator = new Validators_UrlRegex();
					if(!$urlValidator->isValid($data['url'])) {
						$data['error'] = 1;
						$data['responseText'] = 'External url <br />' . implode('<br />', $urlValidator->getMessages());
						return $data;
					}
					$deeplink->setUrl($data['url']);
					$deeplink->setPageId(null);
				}
				$deeplink->setName($data['anchorText']);
				$deeplink->setBanned(false);
				$deeplink->setNofollow($data['nofollow']);
				$deeplink->registerObserver(new Tools_Seo_Watchdog());
				Application_Model_Mappers_DeeplinkMapper::getInstance()->save($deeplink);
				$deeplink->notifyObservers();
				$data['error'] = 0;
				$data['responseText'] = 'Deeplink added.';
			}
			else {
				$data['error'] = 1;
				$data['responseText'] = Tools_Content_Tools::proccessFormMessagesIntoHtml($deeplinksForm->getMessages(), get_class($deeplinksForm));
			}
		}
		return $data;
	}

	private function _deeplinkDeleteAction() {
		$ids = $this->_params['deeplinkId'];
		if(is_array($ids)) {
			foreach ($ids as $id) {
				$this->_removeDeeplink($id);
			}
			$data['done'] = true;
			$data['message'] = 'Deeplinks Removed.';
		}
		else {
			$this->_removeDeeplink($ids);
			$data['done'] = true;
			$data['message'] = 'Deeplink Removed.';
		}
		return $data;
	}

	private function _removeDeeplink($deeplinkId) {
		$deeplinkMapper = Application_Model_Mappers_DeeplinkMapper::getInstance();
		$deeplink       = $deeplinkMapper->find($deeplinkId);
		$deeplink->registerObserver(new Tools_Deeplink_GarbageCollector(array(
			'action' => Tools_Deeplink_GarbageCollector::CLEAN_ONDELETE
		)));
		return $deeplinkMapper->delete($deeplink);
	}

	private function _robotsGetAction() {
		 $data['data'] = Tools_Filesystem_Tools::getFile('robots.txt');
		 return $data;
	}

	private function _robotsPutAction() {
		$robotsForm = new Application_Form_Robots();
        $robotsForm->removeElement(Tools_System_Tools::CSRF_SECURE_TOKEN);
		if($robotsForm->isValid($this->_params)) {
			$robotsData = $robotsForm->getValues();
			$filePath = Zend_Registry::get('website');
			$filePath = $filePath['path'];
			try{
				Tools_Filesystem_Tools::saveFile($filePath.'robots.txt', $robotsData['content']);
				$data['done'] = true;
				$data['message'] = 'Robots.txt updated.';

			}
			catch (Exception $e) {
				$data['done'] = false;
				$data['message'] = 'Can not update Robots.txt.';
			}
			return $data;
		}
	}


	private function _redirectGetAction() {
		if($this->_params['list']) {
            $limit = (!empty($this->_params['limit'])) ? $this->_params['limit'] : self::REDIRECT_LIMIT;
            $offset = (!empty($this->_params['offset'])) ? $this->_params['offset'] : null;
			$redirects = Api_Tools_System::getRedirects($limit, $offset);
            $data = array(
                'done' => true,
                'redirects' => $redirects,
                'count' => Api_Tools_System::getRedirects(null, null, true)
            );
        }

		if(!empty($this->_params['count'])) {
            $data = array('count' => Api_Tools_System::getRedirects(null, null, true));
        }

        return $data;
	}

	private function _redirectPostAction() {
		if($this->_request->isPost()) {
			$redirectForm   = new Application_Form_Redirect();
            $redirectForm->removeElement(Tools_System_Tools::CSRF_SECURE_TOKEN);
			if($redirectForm->isValid($this->_params)) {
				$data          = $redirectForm->getValues();
				$redirect      = new Application_Model_Models_Redirect();
				$fromUrlPath   = Tools_System_Tools::getUrlPath($data['fromUrl']);
				$inDbValidator = new Zend_Validate_Db_NoRecordExists(array(
					'table' => 'redirect',
					'field' => 'from_url'
				));
				if(!$inDbValidator->isValid($fromUrlPath)) {
					$data['done'] = false;
					$data['message'] = implode('<br />', $inDbValidator->getMessages());
					return $data;
				}

				$fromUrl = Tools_System_Tools::getUrlPath($data['fromUrl']);
				$redirect->setFromUrl($fromUrl);
				$redirect->setDomainFrom(Tools_System_Tools::getUrlScheme($data['fromUrl']) . '://' . Tools_System_Tools::getUrlHost($data['fromUrl']) . '/');

				if(intval($data['toUrl'])) {
					$pageMapper = Application_Model_Mappers_PageMapper::getInstance();
					$fromPage = $pageMapper->findByUrl($fromUrl);
					$toPage = $pageMapper->find($data['toUrl']);
					$redirect->setDomainTo($this->_websiteUrl);
					$redirect->setToUrl($toPage->getUrl());
					$redirect->setPageId($fromPage->getId());
				}
				else {
					$urlValidator = new Validators_UrlRegex();
					if(!$urlValidator->isValid($data['toUrl'])) {
						$data['done'] = false;
						$data['message'] = 'External url <br />' . implode('<br />', $urlValidator->getMessages());
						return $data;
					}
					$redirect->setDomainTo(Tools_System_Tools::getUrlScheme($data['toUrl']) . '://' . Tools_System_Tools::getUrlHost($data['toUrl']) . '/');
					$redirect->setToUrl(Tools_System_Tools::getUrlPath($data['toUrl']));
					$redirect->setPageId(null);
				}
				$redirectMapper = Application_Model_Mappers_RedirectMapper::getInstance();
				$redirectMapper->save($redirect);
				$helper = Zend_Controller_Action_HelperBroker::getExistingHelper('cache');
				$helper->clean('toaster_301redirects', '301redirects');
				$data['done'] = true;
				$data['message'] = 'Redirect saved.';
			}
			else {
				$data['done'] = false;
				$data['message'] = Tools_Content_Tools::proccessFormMessagesIntoHtml($redirectForm->getMessages(), get_class($redirectForm));
			}
			return $data;
		}
	}

	private function _redirectDeleteAction() {
		$ids = $this->_params['redirectId'];
		$redirectMapper = Application_Model_Mappers_RedirectMapper::getInstance();
		if(is_array($ids)) {
			foreach ($ids as $id) {
				$redirectMapper->delete($redirectMapper->find($id));
			}
			$data['message'] = 'Redirects Removed.';
		}
		else {
			$redirectMapper->delete($redirectMapper->find($ids));
			$data['message'] = 'Redirect Removed.';
		}
		$helper = Zend_Controller_Action_HelperBroker::getExistingHelper('cache');
		$helper->clean('toaster_301redirects', '301redirects');
		$data['done'] = true;
		return $data;
	}

	private function _toasterPagesContentGetAction() {

		return false;
	}

	private function _seoBottomPutAction() {
		return $this->_saveSeoCode('bottom', $this->_params['seoBottomScripts']);
	}

	private function _seoTopPutAction() {
		return $this->_saveSeoCode('top', $this->_params['seoTopScripts']);
	}

	private function _seoHeadPutAction() {
		return $this->_saveSeoCode('head', $this->_params['seoHeadScripts']);
	}

	private function _saveSeoCode($type, $data) {
        $data = "\r\n" . $data . "\r\n";
		$seoDataDbTable = new Application_Model_DbTable_Seodata();
		$seoData = $seoDataDbTable->fetchRow();
		if($seoData != null) {
			$seoDataModel = new Application_Model_Models_Seodata($seoData->toArray());
		}
		else {
			$seoDataModel = new Application_Model_Models_Seodata();
		}
		switch ($type) {
			case 'top':
			    $seoTopData = $seoDataModel->getSeoTop();
			    if(!empty($seoTopData)){
                        $seoTopData = str_replace(' {$form:conversioncode}', '', $seoTopData);
                        $seoTopData = Api_Tools_System::checkSnippetType($seoTopData, $data);
                }
                else{
                    $seoTopData = $data;
                }

                $seoTopData = $seoTopData . ' {$form:conversioncode}';
                $seoTopData = preg_replace("~(\r\n){2,}~", "\r\n", $seoTopData);
				$seoDataModel->setSeoTop($seoTopData);
			break;
			case 'head':
                $seoHeadData = $seoDataModel->getSeoHead();
                if(!empty($seoHeadData)){
                    $seoHeadData = Api_Tools_System::checkSnippetType($seoHeadData, $data);
                }
                else{
                    $seoHeadData = $data;
                }
                $seoHeadData = preg_replace("~(\r\n){2,}~", "\r\n", $seoHeadData);
				$seoDataModel->setSeoHead($seoHeadData);
			break;
			case 'bottom':
                $seoBottomData = $seoDataModel->getSeoBottom();
                if(!empty($seoBottomData)){
                    $seoBottomData = Api_Tools_System::checkSnippetType($seoBottomData, $data);
                }
                else{
                    $seoBottomData = $data;
                }
                $seoBottomData = preg_replace("~(\r\n){2,}~", "\r\n", $seoBottomData);
                $seoDataModel->setSeoBottom($seoBottomData);
			break;
		}
		$seoData = Application_Model_Mappers_SeodataMapper::getInstance()->save($seoDataModel);
		unset($data);
		$data['done'] = true;
		return $data;
	}

	public function putSeoData($type, $data) {
		$this->_saveSeoCode($type, $data);
	}

	public function _whitelabelPostAction() {
		$whiteLabelLogo = $this->_params['logo'];
		$mojoCompanyAgencyName = $this->_params['name'];

        $dataToSave = array(
            'whiteLabelLogo' => $whiteLabelLogo,
            'mojoCompanyAgencyName' => $mojoCompanyAgencyName
        );

		$data['done'] = (Application_Model_Mappers_ConfigMapper::getInstance()->save($dataToSave)) ? true : false;

		return $data;
	}

	public function _whitelabelDeleteAction() {
		$configDbTable = new Application_Model_DbTable_Config();
		$where = $configDbTable->getAdapter()->quoteInto('name = ?', 'whiteLabelLogo');
		$configDbTable->getAdapter()->delete('config', $where);
        Application_Model_Mappers_ConfigMapper::getInstance()->save(array('mojoCompanyAgencyName' => Tools_System_Tools::DEFAULT_MOJO_COMPANY_AGENCY_NAME));
		$data['done'] = true;
		return $data;
	}

	private function _commandQueuePutAction() {
		self::$_encryptMessageFlag = false;
		$queue = new Api_Tools_Queue($this->_params, $this->_websiteUrl);
		$data = $queue->processQueue();
		return $data;
	}

	protected function _isShoppingInstalled() {
		$shoppingInstalled = null;
		$plugins = Application_Model_Mappers_PluginMapper::getInstance()->findByName('shopping');
		if($plugins != null) {
			$shoppingInstalled = ( $plugins->getStatus() == self::PLUGIN_ENABLED )? true : false;
		}

		if($shoppingInstalled == true) {
			$this->_productPageUrl = Shopping::PRODUCT_CATEGORY_URL;
			$productPage = Application_Model_Mappers_PageMapper::getInstance()->findByUrl($this->_productPageUrl);
			if( $productPage != null ) {
				$this->_isShoppingInstalled = (integer)$productPage->getId();
			}
		}
		return $this->_isShoppingInstalled;
	}
	
	protected function _isPromoInstalled() {
		$plugins = Application_Model_Mappers_PluginMapper::getInstance()->findByName('promo');
		if($plugins != null) {
			$this->_isPromoInstalled = ( $plugins->getStatus() == self::PLUGIN_ENABLED )? true : false;
		}
	}

	private function _toasterAccess($sambaToken) {
		$this->_authenticateAdapter = new Zend_Auth_Adapter_DbTable(
			Zend_Registry::get('dbAdapter'),
			'config',
			'value',
			'value'
		);
		$this->_authenticateAdapter->setIdentity($sambaToken)->setCredential($sambaToken);
		$authResult = $this->_authenticateAdapter->authenticate();
		if( $authResult->isValid() ) {
			return array('authenticate'=> true);
		}
		else {
			return false;
		}
	}

    /**
     * News receiving action. Called when news items distributed from mojo
     *
     * @return array
     */
    protected function _newsPostAction() {
        return $this->_newsAction(Newslog_Tools_Distributor::ACTION_RECEIVE);
    }

    /**
     * News removing action. Called when news delete is requested from mojo
     *
     * @return array
     */
    protected function _newsDeleteAction() {
        return $this->_newsAction(Newslog_Tools_Distributor::ACTION_REMOVE);
    }

    /**
     * Get remote authorization token
     *
     * @return array
     * @throws Exception
     * @throws Exceptions_SeotoasterException
     */
    protected function _remoteAuthorizationTokenPostAction()
    {

        return Tools_Remoteauthorization_Tools::postRemoteAuthorizationToken($this->_params);
    }


    /**
     * Add form blacklist config
     *
     * @return array
     */
    protected function _blacklistPostAction()
    {
        $formBlacklistRulesMapper = Application_Model_Mappers_FormBlacklistRulesMapper::getInstance();
        $formBlacklistRulesMapper->deleteAllData();
        if (!empty($this->_params['data'])) {
            $data = $this->_params['data'];
            $importData = array();
            foreach ($this->_params['data'] as $paramData) {
                $importData[] = $paramData['type'];
                $importData[] = $paramData['value'];

            }
            $headers = array('type', 'value');
            $columnNames = implode($headers, ', ');
            $whereStm = substr(str_repeat('?,', count($headers)), 0, -1);
            $quantity = count($importData) / count($headers);
            $values = implode(
                ',',
                array_fill(0, $quantity, '(' . $whereStm . ')')
            );
            $importStmt = $formBlacklistRulesMapper->getDbTable()->getAdapter()
                ->prepare(
                    'INSERT INTO ' . $formBlacklistRulesMapper->getDbTable()->info(Zend_Db_Table::NAME) . ' (' . $columnNames . ') VALUES ' . $values . ''
                );
            try {
                $importStmt->execute($importData);
                $data['done'] = true;
            } catch (Exception $e) {
                $data['done'] = false;
            }
        }
        return $data;

    }

	/**
	 * Getting website news template action. Called when website is checked in mojo for sending news
	 *
	 * @return array
	*/
	protected function _templateGetAction(){
		$templatesArr = array();
		try {
			$templatesModel = Application_Model_Mappers_TemplateMapper::getInstance()->findByType(Newslog::TEMPLATE_TYPE);
			foreach ($templatesModel as $model) {
				$templatesArr[] = $model->getName();
			}
			$data['done'] = true;
			$data['result'] = $templatesArr;
		}
		catch (Exception $e){
			$data['done'] = false;
			$data['message'] = $e->getMessage();
		}
		return $data;
	}
    /**
     * News context action dispatcher.
     *
     * @param $action
     * @return array
     */
    protected function _newsAction($action) {
        $result = array('error' => false, 'message' => '');
        try {
            Newslog_Tools_Distributor::getInstance()->$action($this->_params['data']);
        } catch (Exceptions_NewslogException $nle) {
            $result = array(
                'error'   => true,
                'message' => $nle->getMessage()
            );
        }
        return $result;
    }

    private function _gwmtSiteVerificationCodePostAction() {
        try {
            if(isset($this->_params['gwmtVerificationCodeClient'])) {
                Application_Model_Mappers_ConfigMapper::getInstance()->save(array('gwmtVerificationCodeClient' => $this->_params['gwmtVerificationCodeClient']));
            }
            if(isset($this->_params['gwmtVerificationCodeAgency'])) {
                Application_Model_Mappers_ConfigMapper::getInstance()->save(array('gwmtVerificationCodeAgency' => $this->_params['gwmtVerificationCodeAgency']));
            }
            $data = array('done' => true, 'message' => 'Google Webmaster Tools website verification code has been added');
        }
        catch(Exception $e) {
            $data = array('done' => false, 'message' => $e->getMessage());
        }
        return $data;
    }

    private function _formsGetAction(){
        $allForms = Application_Model_Mappers_FormMapper::getInstance()->fetchAll();
        $result = array();
        foreach ($allForms as $formData){
            $result[] = $formData->toArray();
        }
        return $result;
    }

	public function afterController() {
        $layout = Zend_Layout::getMvcInstance();
        $view = $layout->getView();
        if($view->pageData['optimized'] === true) {
            $seoIntro = new Api_Tools_SeoIntro($view, $this->_websiteUrl);
            $content = $seoIntro->insertSeoIntro();
            if($content) {
                $this->_response->setBody($content);
            }
        }
	}

}
