<?php

class Api_Tools_PurchaseWatchdog implements Interfaces_Observer
{

    /**
     * Send information about successful action
     */
    const SAVE_SALES_STATS_ACTION = 'saveSalesStats';

    /**
     * Allowed statuses
     *
     * @var array
     */
    public static $_allowedStatuses = array(
        Models_Model_CartSession::CART_STATUS_COMPLETED,
        Models_Model_CartSession::CART_STATUS_SHIPPED,
        Models_Model_CartSession::CART_STATUS_DELIVERED,
        Models_Model_CartSession::CART_STATUS_REFUNDED
    );


    /**
     * Send to the mojo only orders with status completed, shipped, delivered, refunded!!!
     *
     * @param $cartSessionModel
     * @return string
     */
    public function notify($cartSessionModel)
    {
        if ($cartSessionModel instanceof Models_Model_CartSession) {
            $cartStatus = $cartSessionModel->getStatus();
            if (in_array($cartStatus, self::$_allowedStatuses)) {
                $userId = $cartSessionModel->getUserId();
                $userModel = Application_Model_Mappers_UserMapper::getInstance()->find($userId);
                $billingAddressId = $cartSessionModel->getBillingAddressId();
                $shippingAddressId = $cartSessionModel->getShippingAddressId();
                $billingAddress = Tools_ShoppingCart::getAddressById($billingAddressId);
                $shippingAddress = Tools_ShoppingCart::getAddressById($shippingAddressId);
                if (!empty($billingAddress)) {
                    $billingStateCode = $billingAddress['state'];
                    if (!empty($billingStateCode) && is_numeric($billingStateCode)) {
                        $stateInfo = Tools_Geo::getStateById($billingStateCode);
                        if (!empty($stateInfo)) {
                            $billingAddress['state'] = $stateInfo['state'];
                        }
                    }
                }

                if (!empty($shippingAddress)) {
                    $shippingStateCode = $shippingAddress['state'];
                    if (!empty($shippingStateCode) && is_numeric($shippingStateCode)) {
                        $stateInfo = Tools_Geo::getStateById($shippingStateCode);
                        if (!empty($stateInfo)) {
                            $shippingAddress['state'] = $stateInfo['state'];
                        }
                    }
                }

                $websiteConfig = Zend_Controller_Action_HelperBroker::getStaticHelper('config')->getConfig();
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();

                $fullPurchaseData = $cartSessionModel->toArray();
                $fullPurchaseData['billingAddress'] = $billingAddress;
                $fullPurchaseData['shippingAddress'] = $shippingAddress;

                $userInfo = array();

                if($userModel instanceof Application_Model_Models_User) {
                    $userInfo = $userModel->toArray();
                }

                $fullPurchaseData['userInfo'] = $userInfo;
                $productMapper = Models_Mapper_ProductMapper::getInstance();
                $storeCurrencyCode = Models_Mapper_ShoppingConfig::getInstance()->getConfigParam('currency');
                $fullPurchaseData['currency'] = $storeCurrencyCode;
                foreach ($fullPurchaseData['cartContent'] as $key => $purchaseData) {
                    $productId = $purchaseData['product_id'];
                    if (empty($productId)) {
                        $productId = $purchaseData['id'];
                    }
                    $productModel = $productMapper->find($productId);
                    if ($productModel instanceof Models_Model_Product) {
                        $fullPurchaseData['cartContent'][$key]['shortDescription'] = $productModel->getShortDescription();
                        $fullPurchaseData['cartContent'][$key]['fullDescription'] = $productModel->getFullDescription();
                        $fullPurchaseData['cartContent'][$key]['brandName'] = $productModel->getBrand();
                        $photo = $productModel->getPhoto();
                    }

                    if (!empty($photo)) {
                        $fullPurchaseData['cartContent'][$key]['photo'] = Tools_Misc::prepareProductImage($photo);
                    } else {
                        $fullPurchaseData['cartContent'][$key]['photo'] = $websiteUrl. 'system/images/noimage.png';
                    }
                }

                if (isset($websiteConfig['sambaToken']) && (isset($websiteConfig['websiteId']))) {
                    $url = parse_url($websiteUrl);
                    $data['websiteUrl'] = $url['host'];
                    $data['websiteId'] = $websiteConfig['websiteId'];
                    $data['sambaToken'] = $websiteConfig['sambaToken'];
                    $data['command'] = '';
                    $data['fullPurchaseData'] = $fullPurchaseData;
                    return Api::request('POST', self::SAVE_SALES_STATS_ACTION, $data);
                }

            }

        }

    }
}

