<?php
/**
 * Description of queue
*/
class Api_Tools_Queue extends Api {

	private $_queue = null;
	private $_pageHelper = null;
	private $_productPageUrl = null;
	private $_productPages = null;
	private $_widcard = null;
	private $_zendDb = null;
	protected $_websiteUrl = null;
	protected $_productPageIds = array();
	protected $_websiteHelper = null;
	protected $_cacheHelper = null;
    protected $_optimizedPages = array();

    private $_tagsFieldIndex = false;

	const INDEX_URL = 'index.html';

    const SEO_RULE = 'seoRule';

    const URL_RULE = 'urlRule';

    const PAGES_APPLY_RULE_LIMIT = 300;

	public function __construct($queue, $websiteUrl) {
		parent::__construct($queue, $websiteUrl);
		$this->_queue = $queue;
        $this->_zendDb = new Zend_Db_Table();
		$this->_pageHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('Page');
		$this->_websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('Website');
		$this->_cacheHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('Cache');
		$this->_websiteUrl = $websiteUrl;
	}

	public function processQueue() {
		$result = array();
		$this->_isPromoInstalled();
		$this->_isShoppingInstalled();
		$this->_getWidcard();
		if($this->_isShoppingInstalled != null) {
			$this->_productPageIds = $this->_zendDb->getAdapter()->fetchCol(
		        $this->_zendDb->getAdapter()->select()
                ->from('shopping_product', array('page_id'))->order(array('page_id ASC'))
            );
		}
		foreach ($this->_queue['queue'] as $command) {
			if( isset($command['params'])) {
				$process = '_'.$command['name'];
				$result[$command['name']] = $this->$process($command['params']);
			}
			else {
				$result[$command['name']] = array('done' => true, 'message' => 'No parameters sent');
			}
		}
		return $result;
	}
	
	private function _getWidcard() {
		$wicData = Application_Model_Mappers_ConfigMapper::getInstance()->getConfig();
		$industry = Tools_Factory_PluginFactory::createPlugin('api',array(), array('websiteUrl' => $this->_websiteUrl));
		$industryList = $industry::request('get' , 'industryList', array());
		
		if( isset($wicData['wicCountryState']) && ($wicData['wicCountryState'] != '') ) {
            $states = array();

            if($wicData['wicOrganizationCountry'] == 'US') {
                $states = self::$stateListUSA;
            }

            if($wicData['wicOrganizationCountry'] == 'CA') {
                $states = self::$stateListCanada;
            }

            if($wicData['wicOrganizationCountry'] == 'AU') {
                $states = self::$stateListAustralia;
            }

			$wicData['wicCountryState'] = $states[$wicData['wicCountryState']];
		}
		else {
			$wicData['wicCountryState'] = '';
		}
		$websiteIndustries = unserialize($wicData['wicIndustryType']);
		if($websiteIndustries != '') {
			$wicData['wicIndustryType'] = $industryList[$websiteIndustries[0]];
		}
		else {
			$wicData['wicIndustryType'] = '';
		}
		$this->_widcard = $wicData;
	}
	
	private function _cleanPagesCache($pageId) {
		$tags = array (
			'pageid_'. $pageId,
			'Widgets_Menu_Menu',
			'Widgets_Related_Related',
			'pageTags',
			'Widgets_List_List',
			'sitemaps'
		);
		$this->_cacheHelper->clean(false, false, $tags);
	}

    private function _cleanCache() {
        $this->_cacheHelper->clean();
    }

    private function _cleanContainerCache($containerId) {
        $container = Application_Model_Mappers_ContainerMapper::getInstance()->find($containerId);
        if($container !== null) {
            $cacheTags = array(
                $container->getName() .'_'. $container->getContainerType() .'_pid_'. intval($container->getPageId())
            );
            $this->_cacheHelper->clean(false, false, $cacheTags);
        }
    }

    private function _getMassApplyPages($params) {
        /*
         * If massApply key and regular pages under sent categoryId or pages under NO_CATEGORY
         * */
        if((integer)$params['massApply']['categoryId'] > 0 || (integer)$params['massApply']['categoryId'] === -1) {
            $where = $this->_zendDb->getAdapter()->quoteInto("p.parent_id = ?", (integer)$params['massApply']['categoryId']);
            $where .= $this->_zendDb->getAdapter()->quoteInto(" AND p.draft = ?", '0');

            $excludeIndexPage = ((integer)$params['massApply']['categoryId'] === -1);
            $excludePages = $this->_getExcludePages($excludeIndexPage);

            if(!empty($params['massApply']['excludePageId'])) {
                $excludePages = array_merge($excludePages, $params['massApply']['excludePageId']);
            }
            if(count($excludePages)) {
                $where .= $this->_zendDb->getAdapter()->quoteInto(" AND p.id NOT IN(?)", $excludePages);
            }
            $select = $this->_zendDb->getAdapter()->select()
                ->from(array('p' => 'page'), array('ids' => 'id'))->where($where)
                ->order(array('id ASC'));
            if(isset($params['massApply']['offset'])) {
                $select->limit(self::PAGES_APPLY_RULE_LIMIT, $params['massApply']['offset']);
            }else {
                $select->limit(self::PAGES_APPLY_RULE_LIMIT);
            }
            $pageIds = $this->_zendDb->getAdapter()->fetchCol($select);
            $pageCount = $this->_zendDb->getAdapter()->fetchOne(
                $this->_zendDb->getAdapter()->select()
                    ->from(array('p' => 'page'), array('count' => 'COUNT(*)'))->where($where)
            );
        }elseif((integer)$params['massApply']['categoryId'] === -2) { // Page type product
            /*
             * If massApply key and product pages
             * */
            if($this->_isShoppingInstalled != null) {
                $where = $this->_zendDb->getAdapter()->quoteInto("p.draft = ?", '0');

                if(!empty($params['massApply']['excludePageId'])) {
                    $where .= $this->_zendDb->getAdapter()->quoteInto(" AND p.id NOT IN(?)", $params['massApply']['excludePageId']);
                }

                $select = $this->_zendDb->getAdapter()->select()
                    ->from(array('sp' => 'shopping_product'), array('ids' => 'page_id'))
                    ->join(array('p' => 'page'), 'p.id = sp.page_id', null)
                    ->where($where)
                    ->order(array('p.id ASC'));
                if(isset($params['massApply']['offset'])) {
                    $select->limit(self::PAGES_APPLY_RULE_LIMIT, $params['massApply']['offset']);
                }else {
                    $select->limit(self::PAGES_APPLY_RULE_LIMIT);
                }
                $pageIds = $this->_zendDb->getAdapter()->fetchCol($select);
                $pageCount = $this->_zendDb->getAdapter()->fetchOne(
                    $this->_zendDb->getAdapter()->select()
                        ->from(array('sp' => 'shopping_product'), array('count' => new Zend_Db_Expr('COUNT(*)')))
                        ->join(array('p' => 'page'), 'p.id = sp.page_id', null)
                );
                $this->_productPageIds = $pageIds;
            }
        }elseif ((integer)$params['massApply']['categoryId'] === -3) { // News pages
            $where = $this->_zendDb->getAdapter()->quoteInto("p.draft = ?", '0');

            $excludeIndexPage = ((integer)$params['massApply']['categoryId'] === -3);
            $excludePages = $this->_getExcludePages($excludeIndexPage);

            if(!empty($params['massApply']['excludePageId'])) {
                $excludePages = array_merge($excludePages, $params['massApply']['excludePageId']);
            }
            if(count($excludePages)) {
                $where .= $this->_zendDb->getAdapter()->quoteInto(" AND p.id NOT IN(?)", $excludePages);
            }

            $select = $this->_zendDb->getAdapter()->select()
                ->from(array('pnn' => 'plugin_newslog_news'), array('ids' => 'page_id'))
                ->join(array('p' => 'page'), 'p.id = pnn.page_id', NULL)
                ->where($where)
                ->order(array('p.id ASC'));

            if(isset($params['massApply']['offset'])) {
                $select->limit(self::PAGES_APPLY_RULE_LIMIT, $params['massApply']['offset']);
            }else {
                $select->limit(self::PAGES_APPLY_RULE_LIMIT);
            }
            $pageIds = $this->_zendDb->getAdapter()->fetchCol($select);
            $pageCount = $this->_zendDb->getAdapter()->fetchOne(
                $this->_zendDb->getAdapter()->select()
                    ->from(array('pnn' => 'plugin_newslog_news'), array('pageNumber' => 'COUNT(DISTINCT p.id)'))
                    ->join(array('p' => 'page'), 'p.id = pnn.page_id', NULL)
                    ->where($where)
            );
        }
        return array('pageIds' => $pageIds, 'count' => $pageCount);
    }

	private function _applyUrlRule($params) {
		if(isset($params['pageIds']) || isset($params['massApply'])) {
			$pageIds = (isset($params['pageIds']))? $params['pageIds'] : array();
			unset ($params['pageIds']);
			if(isset($params['massApply'])) {
                $massApplyPages = $this->_getMassApplyPages($params);
                $pageIds = $massApplyPages['pageIds'];
                $pageCount = $massApplyPages['count'];
                if(empty($pageIds)) {
                    return  array('done' => true, 'offset' => null, 'message' => 'There are no pages to apply the URL-rule');
                }
            }
			foreach($pageIds as $pageId) {
				if($pageId != NULL) {
					$ruleParams = (isset($params[$pageId])) ? $params[$pageId] : $params['massApply']['rule'];
					$tweakedRule = (isset($ruleParams['url'])) ? $ruleParams['url'] : null;
					$tkp = (isset($ruleParams['state']['tkp'])) ? $ruleParams['state']['tkp'] : null;
					try {
                        $this->_collectOptimizedPages($pageId, $ruleParams['state']['Id'], self::URL_RULE);
						$this->_createRule($ruleParams['ruleValue'], $ruleParams['delimiter'], $pageId, $ruleParams['state']['Id'], $ruleParams['state']['Status'], $tweakedRule, $tkp);
					}
					catch (Exception $e) {
                        error_log($e->getMessage());
                        return array('done' => false, 'message' => 'PAGE #: '.$pageId.' --- '. $e->getMessage());
                    }
				}
			}

            if(isset($params['massApply'])) {
                $generalOffset = isset($params['massApply']['offset'])? (integer)$params['massApply']['offset'] + self::PAGES_APPLY_RULE_LIMIT : self::PAGES_APPLY_RULE_LIMIT;
                if((integer)$pageCount > $generalOffset) {
                    if(isset($params['massApply']['offset'])) {
                        $offset = $generalOffset;
                    }else {
                        $offset = self::PAGES_APPLY_RULE_LIMIT;
                    }
                    $data = array(
                        'done' => true,
                        'offset' => $offset,
                        'optimizedPageIds' => $this->_optimizedPages
                    );
                }else {
                    $data = array(
                        'done' => true,
                        'offset' => null,
                        'optimizedPageIds' => $this->_optimizedPages
                    );
                    $data['pageCount'] = ($pageCount <= self::PAGES_APPLY_RULE_LIMIT) ? $pageCount: $pageCount - $params['massApply']['offset'];
                }
            }else {
                $data = array('done' => true, 'message' => 'Url-rules have been applied');
            }
            $this->_cleanCache();
            return $data;
		}
	}

	private function _applySeoRule($params) {
		if(isset($params['pageIds']) || isset($params['massApply']) ) {
			$pageIds = (isset($params['pageIds']))? $params['pageIds'] : array();
			unset ($params['pageIds']);

            if(isset($params['massApply'])) {
                try{
                $massApplyPages = $this->_getMassApplyPages($params);
                $pageIds = $massApplyPages['pageIds'];
                $pageCount = $massApplyPages['count'];
                if(empty($pageIds)) {
                    return array('done' => true, 'offset' => null, 'message' => 'There are no pages to apply the SEO-rule');
                }
                }catch (Exception $e) {return $e->getMessage();}
            }
			foreach ($pageIds as $pageId) {
				if($pageId != NULL) {
					$ruleParams = (isset($params[$pageId])) ? $params[$pageId] : $params['massApply']['rule'];
					$pageStatus = $ruleParams['state']['Status'];
					$pageSeoRuleId = $ruleParams['state']['Id'];
					$tkp = (isset($ruleParams['state']['tkp'])) ? $ruleParams['state']['tkp'] : null;
					unset($ruleParams['state']);
					try {
                        $this->_collectOptimizedPages($pageId, $pageSeoRuleId, self::SEO_RULE);
						$this->_createSeoRule($ruleParams, $pageId, $pageStatus, $pageSeoRuleId, $tkp);
						//$this->_cleanPagesCache($pageId);
					}
					catch (Exception $e){
                        error_log($e->getMessage() .' -> '. $pageId); return $e->getMessage();
                        return array('done' => false, 'message' => 'PAGE #: '.$pageId.' --- '. $e->getMessage());
                    }
				}
			}
            if(isset($params['massApply'])) {
                $generalOffset = isset($params['massApply']['offset'])? (integer)$params['massApply']['offset'] + self::PAGES_APPLY_RULE_LIMIT : self::PAGES_APPLY_RULE_LIMIT;
                if((integer)$pageCount > $generalOffset) {
                    if(isset($params['massApply']['offset'])) {
                        $offset = $generalOffset;
                    }else {
                        $offset = self::PAGES_APPLY_RULE_LIMIT;
                    }
                    $data = array(
                        'done' => true,
                        'offset' => $offset,
                        'optimizedPageIds' => $this->_optimizedPages
                    );
                }else {
                    $data = array(
                        'done' => true,
                        'offset' => null,
                        'optimizedPageIds' => $this->_optimizedPages
                    );
                    $data['pageCount'] = ($pageCount <= self::PAGES_APPLY_RULE_LIMIT) ? $pageCount: ($pageCount - $params['massApply']['offset']);
                }
            }else {
                $data = array('done' => true, 'message' => 'Mass apply SEO-rules have been applied');
            }
		}
		if(isset($params['tweaks'])) {
			foreach ($params['tweaks'] as $pageId => $tweakedPage) {
				try {
                    $metaDescriptionTweaked = new Zend_Db_Expr('NULL');

                    if(!empty($tweakedPage['metaDescription'])) {
                        $metaDescriptionTweaked = "'" . str_replace("'", "''", htmlspecialchars(str_replace('"', '', $tweakedPage['metaDescription']), ENT_COMPAT, 'UTF-8')) . "'";
                    }

                    $metaKeywordsTweaked = new Zend_Db_Expr('NULL');
                    if(!empty($tweakedPage['metaKeywords'])) {
                        $metaKeywordsTweaked = "'" . str_replace("'", "''", htmlspecialchars(str_replace('"', '', $tweakedPage['metaKeywords']), ENT_COMPAT, 'UTF-8')) . "'";
                    }

                    $teaserTextTweaked = new Zend_Db_Expr('NULL');
                    if(!empty($tweakedPage['teaserText'])) {
                        $teaserTextTweaked = "'" . str_replace("'", "''", $tweakedPage['teaserText']) . "'";
                    }

                    $seoIntro = new Zend_Db_Expr('NULL');
                    if(!empty($tweakedPage['seoIntro'])) {
                        $seoIntro = "'" . str_replace("'", "''", $tweakedPage['seoIntro']) . "'";
                    }

                    $seoIntroTarget = new Zend_Db_Expr('NULL');
                    if($seoIntroTarget) {
                        $seoIntroTarget = "'" . str_replace("'", "''", $tweakedPage['seoIntroTarget']) . "'";
                    }

					$this->_zendDb->getAdapter()->query("insert into optimized (`page_id`, `h1`, `header_title`, `nav_name`,`meta_description`,`meta_keywords`, `targeted_key_phrase`, `teaser_text`, `status`, `seo_rule_id`, `seo_intro`, `seo_intro_target`)
					VALUES('".$pageId."','".str_replace("'", "''", $tweakedPage['h1'])
					    . "','" . str_replace("'", "''", $tweakedPage['headerTitle'])
					    . "','" . str_replace("'", "''", $tweakedPage['navName'])
					    . "',". $metaDescriptionTweaked
					    . "," . $metaKeywordsTweaked
					    . ",'" . str_replace("'", "''", $tweakedPage['targetedKeyPhrase'])
					    . "'," . $teaserTextTweaked
					    . ",'" . $tweakedPage['status']
					    . "','" . $tweakedPage['seoRuleId']
					    . "'," . $seoIntro
                        . "," . $seoIntroTarget .")

					ON DUPLICATE KEY UPDATE h1 = '".str_replace("'", "''", $tweakedPage['h1'])
					    ."', header_title = '".str_replace("'", "''", $tweakedPage['headerTitle'])
					    ."', nav_name = '".str_replace("'", "''", $tweakedPage['navName'])
					    ."', meta_description = " . $metaDescriptionTweaked
					    .", meta_keywords = " . $metaKeywordsTweaked
					    .", targeted_key_phrase = '".str_replace("'", "''", $tweakedPage['targetedKeyPhrase'])
					    ."', teaser_text = " . $teaserTextTweaked
					    .", status = '".$tweakedPage['status']
					    ."', seo_rule_id = '".$tweakedPage['seoRuleId']
					    ."', seo_intro = " . $seoIntro
                        .", seo_intro_target = " . $seoIntroTarget);
					//$this->_cleanPagesCache($pageId);
				}
				catch (Exception $e){
                    error_log($e->getMessage()); return $e->getMessage();
                    return array('done' => false, 'message' => 'PAGE #: '.$pageId.' --- '. $e->getMessage());
                }
			}
            $data = array('done' => true, 'message' => 'Seo-rules have been applied');
		}
        $this->_cleanCache();
        return $data;
	}

	private function _createRule($ruleValue, $delimiter, $pageId, $ruleId, $pageStatus, $tweakedUrl, $tkp) {
		if($ruleValue === '0') {
			$oldUrl = $this->_zendDb->getAdapter()->fetchRow("SELECT IF( (SELECT count(*) FROM optimized where page_id = '$pageId' AND url IS NOT NULL), (SELECT url from optimized where page_id = '$pageId') , false) as oldUrl");
			if($oldUrl['oldUrl'] != false) {
				$this->_zendDb->getAdapter()->query("insert into redirect (`page_id`, `from_url`, `to_url`, `domain_to`, `domain_from`) VALUES('$pageId','".$oldUrl['oldUrl']."', (SELECT url from page where id = '$pageId'), '".$this->_websiteHelper->getUrl()."', '".$this->_websiteHelper->getUrl()."')");
				$whereUrl = $this->_zendDb->getAdapter()->quoteInto("from_url IN(?)", $this->_zendDb->getAdapter()->fetchRow("SELECT url from page where id = '$pageId'"));
				$wherePageId = $this->_zendDb->getAdapter()->quoteInto('page_id = ?', $pageId);
				$this->_zendDb->getAdapter()->delete('redirect', array($whereUrl, $wherePageId));

                $newPageUrl = $this->_zendDb->getAdapter()->fetchRow("SELECT url from page where id = '$pageId'");
                $this->_updateRedirectsToUrl($pageId, $newPageUrl['url'], $this->_websiteHelper->getUrl());
                Tools_Page_Tools::processPagePreviewImage($newPageUrl['url'], Tools_Page_Tools::processPagePreviewImage($oldUrl['oldUrl']));
                $this->_zendDb->getAdapter()->query("UPDATE `page` set `preview_image` =
                                    (CONCAT('".substr($newPageUrl['url'] ,0 , strrpos($newPageUrl['url'], '.'))."' ,
                                             SUBSTRING(`preview_image`, LOCATE('.', `preview_image`), LENGTH(`preview_image`) )
                                    ) ) where `id` = '".$pageId."'");
                $pageData = $this->_zendDb->getAdapter()->fetchRow(
                    $this->_zendDb->getAdapter()->select()
                        ->from(array('o' => 'optimized'), array('seoRuleId' => 'seo_rule_id', 'pageStatus' => 'status'))
                        ->where($wherePageId)
                );
                if(!empty($pageData)) {
                    if($pageData['seoRuleId'] === null) {
                        $this->_zendDb->getAdapter()->delete('optimized', array('page_id = ?' => $pageId));
                    }else {
                        $pageStatus = $pageData['pageStatus'];
                        $this->_zendDb->getAdapter()->query("UPDATE optimized set `url` = NULL, `url_rule_id` = NULL , `status` = '$pageStatus' where page_id = '$pageId'");
                    }
                }
			}
			return;
		}
		if($tweakedUrl == null) {
			$concats = null;
			$duplicateConcats = null;
			$rule = implode($delimiter, unserialize($ruleValue));
			$inPageIdsArray = in_array($pageId, $this->_productPageIds);
			$optimizedUrl = array(
				'{h1}' => "',`h1`,'",
				'{navigation_label}' => "',`nav_name`,'",
                '{cat_index_nav_label}'=> "',(SELECT (CASE
    WHEN (`parent_id` = '0') THEN (SELECT IFNULL(`nav_name`,\"\") FROM `page` WHERE `id` = '".$pageId."')
    WHEN (`parent_id` != '0' AND `parent_id` > '0' AND `parent_id` != IFNULL((SELECT `id` from `page` WHERE `url` = 'product-pages.html'),0) )
    THEN (SELECT IFNULL(`nav_name`,\"\") FROM `page` WHERE `id` = (SELECT `parent_id` FROM `page` WHERE `id` = '".$pageId."') )
    ELSE ''
  END) AS glueStringIndex
FROM `page` where `id` = '".$pageId."'),'",
                '{page_tags}' => "',(SELECT IFNULL(GROUP_CONCAT(`fa`.`name` SEPARATOR ', '), \"\") FROM `featured_area` AS fa JOIN `page_fa` as pfa ON fa.id = pfa.fa_id WHERE `pfa`.`page_id` = '".$pageId."'),'",
				'{page_teaser}' => "',`teaser_text`,'",
				'{targeted_key_phrase}' => "',`targeted_key_phrase`,'",
				'{meta_description}' => "',`meta_description`,'",
				'{org_name}' => addslashes($this->_widcard['wicOrganizationName']),
				'{org_description}' => addslashes($this->_widcard['wicOrganizationDescription']),
				'{local_phone}' => addslashes($this->_widcard['wicPhone']),
				'{city}' => addslashes($this->_widcard['wicCity']),
				'{MSA}' => addslashes($this->_widcard['wicMSA']),
				'{state}' => addslashes($this->_widcard['wicCountryState']),
				'{zip}' => addslashes($this->_widcard['wicZip']),
				'{industry}' => $this->_widcard['wicIndustryType'],
				'}product_name{' => "',`pr`.`name`,'",
				'}brand{' => "',`b`.`name`,'",
				'}product_tags{'=> "',(select IFNULL(group_concat(t.name), \"\") from shopping_tags as t join shopping_product_has_tag as p2t on p2t.tag_id = t.id join shopping_product as p ON p2t.product_id = p.id where ".$this->_zendDb->getAdapter()->quoteInto('p.page_id = ?', $pageId)."),'",
				'}product_price{' => "',TRUNCATE(`pr`.`price`, 2),'",
				'}product_SKU{' => "',`pr`.`sku`,'",
                '}product_MPN{' => "',`pr`.`mpn`,'",
				'}promo_price{' => "',IFNULL(`promo`.`promo_price`,\"\"),'",
				'}promo_exp_date{' => "',IFNULL(`promo`.`promo_due`,\"\"),'"
			);

			$optimizedPageUpdate = array(
				'{h1}' => "',`p`.`h1`,'",
				'{navigation_label}' => "',`p`.`nav_name`,'",
                '{cat_index_nav_label}'=> "',(SELECT (CASE
    WHEN (`parent_id` = '0') THEN (SELECT IFNULL(`nav_name`,\"\") FROM `page` WHERE `id` = '".$pageId."')
    WHEN (`parent_id` != '0' AND `parent_id` > '0' AND `parent_id` != IFNULL((SELECT `id` from `page` WHERE `url` = 'product-pages.html'),0) )
    THEN (SELECT IFNULL(`nav_name`,\"\") FROM `page` WHERE `id` = (SELECT `parent_id` FROM `page` WHERE `id` = '".$pageId."') )
    ELSE ''
  END) AS glueStringIndex
FROM `page` where `id` = '".$pageId."'),'",
                '{page_tags}' => "',(SELECT IFNULL(GROUP_CONCAT(`fa`.`name` SEPARATOR ', '), \"\") FROM `featured_area` AS fa JOIN `page_fa` as pfa ON fa.id = pfa.fa_id WHERE `pfa`.`page_id` = '".$pageId."'),'",
				'{page_teaser}' => "',`p`.`teaser_text`,'",
				'{targeted_key_phrase}' => "',`p`.`targeted_key_phrase`,'",
				'{meta_description}' => "',`p`.`meta_description`,'",
				'{org_name}' => addslashes($this->_widcard['wicOrganizationName']),
				'{org_description}' => addslashes($this->_widcard['wicOrganizationDescription']),
				'{local_phone}' => addslashes($this->_widcard['wicPhone']),
				'{city}' => addslashes($this->_widcard['wicCity']),
				'{state}' => addslashes($this->_widcard['wicCountryState']),
				'{MSA}' => addslashes($this->_widcard['wicMSA']),
				'{zip}' => addslashes($this->_widcard['wicZip']),
				'{industry}' => $this->_widcard['wicIndustryType'],
				'}product_name{' => "',`pr`.`name`,'",
				'}brand{' => "',`b`.`name`,'",
				'}product_tags{'=> "',(select IFNULL(group_concat(t.name), \"\") from shopping_tags as t join shopping_product_has_tag as p2t on p2t.tag_id = t.id join shopping_product as p ON p2t.product_id = p.id where ".$this->_zendDb->getAdapter()->quoteInto('p.page_id = ?', $pageId)."),'",
				'}product_price{' => "',TRUNCATE(`pr`.`price`, 2),'",
				'}product_SKU{' => "',`pr`.`sku`,'",
                '}product_MPN{' => "',`pr`.`mpn`,'",
				'}promo_price{' => "',IFNULL(`promo`.`promo_price`,\"\"),'",
				'}promo_exp_date{' => "',IFNULL(`promo`.`promo_due`,\"\"),'"
				);
				if(!$this->_isPromoInstalled && $inPageIdsArray) {
					$rule = str_replace(array('{promo_price}','{promo_exp_date}'), '', $rule);
				}
				$rule = str_replace("'", "''", $rule);
				$replaceSymbols = array('^',';','<','>','[',']','{','}','|','=',')','(','!','@','#','%','$','+','&',':','?','–','/','.','\\\'',',','`','"','*','\\\\','  ','   ',''.$delimiter.$delimiter.$delimiter.'',''.$delimiter.$delimiter.'');
				$startReplace = '';
				$endReplace = '';
				for ($i = 0; $i<count($replaceSymbols); $i++) {
					if( ($replaceSymbols[$i] == '/') || ($replaceSymbols[$i] == '\\\\') || ($replaceSymbols[$i] == '?') ||
                        ($replaceSymbols[$i] == '.') || ($replaceSymbols[$i] == ',') || ($replaceSymbols[$i] == '&') ||
                        ($replaceSymbols[$i] == ':') || ($replaceSymbols[$i] == '*') || ($replaceSymbols[$i] == '+') || ($replaceSymbols[$i] == '^') ||
                        ($replaceSymbols[$i] == ';') || ($replaceSymbols[$i] == '<') || ($replaceSymbols[$i] == '>') || ($replaceSymbols[$i] == '[') ||
                        ($replaceSymbols[$i] == ']') || ($replaceSymbols[$i] == '{') || ($replaceSymbols[$i] == '}') || ($replaceSymbols[$i] == '|') ||
                        ($replaceSymbols[$i] == '=') || ($replaceSymbols[$i] == '(') || ($replaceSymbols[$i] == ')') || ($replaceSymbols[$i] == '!') ||
                        ($replaceSymbols[$i] == '@') || ($replaceSymbols[$i] == '#') || ($replaceSymbols[$i] == '%') || ($replaceSymbols[$i] == '$') ||
                        ($replaceSymbols[$i] == $delimiter.$delimiter) || ($replaceSymbols[$i] == $delimiter.$delimiter.$delimiter) ) {
                        $replacer = $delimiter;
                    }else {$replacer = '';}
					$startReplace .= "REPLACE( ";
					$endReplace .= ", '$replaceSymbols[$i]', '$replacer') ";
				}
				$fieldConcat = str_replace(array("'',",",''"), '', "'" . strtr($rule, $optimizedUrl)."'");

				$concats[] = ' IF ( CONCAT('. $fieldConcat . ') = \'\', NULL, CONCAT( TRIM(BOTH \''.$delimiter.'\' FROM LOWER( REPLACE( '.$startReplace.' TRIM(CONCAT('. $fieldConcat . ')), \' \', \''.$delimiter.'\' ) '.$endReplace.' ) ), \'.html\')     ) as url'; //

                $noDuplicateConcat = '';

                /* CHECK FOR DUPLICATED URL's FOR PRODUCT PAGES WHEN AUTO RULE APPLYING */
                if(!empty($this->_productPageIds) && $inPageIdsArray) {
                    $selectDuplicate = $this->_zendDb->getAdapter()->select()->from(array('p' => 'page'), array(new Zend_Db_Expr("" . implode(',',$concats) . "")))->where('p.id = ?', $pageId);
                    $selectDuplicate->joinLeft(array('pr' => 'shopping_product'), '`p`.`id` = `pr`.`page_id`', array(null))->
                        joinLeft(array('b' => 'shopping_brands'), '`pr`.`brand_id` = `b`.`id`', array(null));
                    if($this->_isPromoInstalled && $inPageIdsArray) {
                        $selectDuplicate->joinLeft(array('promo' => 'plugin_promo'), '`pr`.`id` = `promo`.`product_id`', array(null));
                    }
                    $urlQuery = $selectDuplicate->assemble();
                    $isDuplicateUrl = $this->_zendDb->getAdapter()->fetchRow("SELECT IF ( EXISTS
                        (SELECT url from optimized where optimized.url =
                        ($urlQuery) AND (page_id != $pageId) ) ,1,0 ) as duplicate");
                    if($isDuplicateUrl['duplicate']) {
                        unset($concats);
                        $noDuplicateConcat = ",'".$delimiter."',`pr`.`sku`";
                        $concats[] = ' IF ( CONCAT('. $fieldConcat . ') = \'\', NULL, CONCAT( TRIM(BOTH \''.$delimiter.'\' FROM LOWER( REPLACE( '.$startReplace.' TRIM(CONCAT('. $fieldConcat . ')), \' \', \''.$delimiter.'\' ) '.$endReplace.' ) )'.$noDuplicateConcat.' ,  \'.html\')     ) as url';
                    }
                }

				$duplicateFieldConcat = str_replace(array("'',",",''"), '', "'" . strtr($rule, $optimizedPageUpdate) . "'");
				$duplicateConcats[] = '`url` = IF ( CONCAT(' . $duplicateFieldConcat . ') = \'\' , NULL, CONCAT( TRIM(BOTH \''.$delimiter.'\' FROM LOWER( REPLACE( '.$startReplace.' TRIM(CONCAT('. $duplicateFieldConcat . ')), \' \', \''.$delimiter.'\' ) '.$endReplace.' ) )'.$noDuplicateConcat.', \'.html\') ) ';

				$select = $this->_zendDb->getAdapter()->select()->from(array('p' => 'page'), array(new Zend_Db_Expr("`p`.`id` as 'page_id', " . implode(',',$concats) . " , CONCAT('$pageStatus') as 'status', CONCAT('$ruleId') as 'url_rule_id'")))->where('p.id = ?', $pageId);

				if(!empty($this->_productPageIds) && $inPageIdsArray) {
					$select->joinLeft(array('pr' => 'shopping_product'), '`p`.`id` = `pr`.`page_id`', array(null))->
					joinLeft(array('b' => 'shopping_brands'), '`pr`.`brand_id` = `b`.`id`', array(null));
				}
				if($this->_isPromoInstalled && $inPageIdsArray) {
					$select->joinLeft(array('promo' => 'plugin_promo'), '`pr`.`id` = `promo`.`product_id`', array(null));
				}
				//return "insert into optimized (`page_id`, `url`, `status`, `url_rule_id`) " . $select->assemble() . " ON DUPLICATE KEY UPDATE " . implode(',',$duplicateConcats) . ", `status` = CONCAT('$pageStatus'), `url_rule_id` = CONCAT('$ruleId')";
				$urlToRedirect = $this->_zendDb->getAdapter()->fetchRow("SELECT IF( (SELECT count(*) FROM optimized where page_id = '$pageId' AND url IS NOT NULL), (SELECT url from optimized where page_id = '$pageId') , (SELECT url from page where id = '$pageId')) as urlToRedirect");
				if($tkp != null) {
					$this->_zendDb->getAdapter()->query("UPDATE `page` AS `p` INNER JOIN `page` AS `sp` ON `p`.`id` = `sp`.`id` SET `p`.`targeted_key_phrase` = '".addslashes($tkp)."' where `p`.`id` = ".$pageId."");
				}

                /*
                 * Insert or update new URL to optimized DB-table
                 * */
				$this->_zendDb->getAdapter()->query("insert into optimized (`page_id`, `url`, `status`, `url_rule_id`) " . $select->assemble() . " ON DUPLICATE KEY UPDATE " . implode(',',$duplicateConcats) . ", `status` = CONCAT('$pageStatus'), `url_rule_id` = CONCAT('$ruleId')");
				$urls = $this->_zendDb->getAdapter()->fetchRow($this->_zendDb->getAdapter()->select()->from(array('p' => 'page'), array('oldUrl' => 'p.url'))->joinLeft(array('o' => 'optimized'), 'p.id = o.page_id', array('newUrl' => 'o.url'))->where('p.id = ?', $pageId));
                if(($urlToRedirect['urlToRedirect'] != $urls['newUrl'])) {
                    $this->_updateInternalLinks($urlToRedirect, $urls, $pageId);
                }
				$whereUrl = $this->_zendDb->getAdapter()->quoteInto('from_url = ?', $urls['newUrl']);
				//$wherePageId = $this->_zendDb->getAdapter()->quoteInto('page_id = ?', $pageId);
                $whereUrlTo = $this->_zendDb->getAdapter()->quoteInto('to_url = ?', $urlToRedirect['urlToRedirect']);
				$this->_zendDb->getAdapter()->delete('redirect', array($whereUrl, $whereUrlTo));//$wherePageId
				$rowExists = $this->_zendDb->getAdapter()->fetchRow("SELECT IF( (SELECT count(*) FROM redirect WHERE page_id = '$pageId' AND from_url = '".$urlToRedirect['urlToRedirect']."' AND to_url = '".$urls['newUrl']."' AND domain_to = '".$this->_websiteHelper->getUrl()."' AND domain_from = '".$this->_websiteHelper->getUrl()."'), true, false ) rowExists");
				if(!$rowExists['rowExists'] && ($urlToRedirect['urlToRedirect'] != $urls['newUrl']) ) {
					$this->_zendDb->getAdapter()->query("insert into redirect (`page_id`, `from_url`, `to_url`, `domain_to`, `domain_from`) VALUES('$pageId','".$urlToRedirect['urlToRedirect']."', '".$urls['newUrl']."', '".$this->_websiteHelper->getUrl()."', '".$this->_websiteHelper->getUrl()."')");
                    $this->_updateRedirectsToUrl($pageId, $urls['newUrl'], $this->_websiteHelper->getUrl());
                    $this->_zendDb->getAdapter()->query("UPDATE `page` set `preview_image` =
                                    CASE
                                    WHEN (`preview_image` IS NOT NULL AND `preview_image` != '')
                                    THEN (CONCAT('".substr($urls['newUrl'] ,0 , strrpos($urls['newUrl'], '.'))."' ,
                                             SUBSTRING(`preview_image`, LOCATE('.', `preview_image`), LENGTH(`preview_image`) )
                                    ) )
                                    ELSE NULL
                                    END
                                     where `id` = '".$pageId."'");
                    Tools_Page_Tools::processPagePreviewImage($urls['newUrl'], Tools_Page_Tools::processPagePreviewImage($urlToRedirect['urlToRedirect']));
				}
		}
		else {
            // $urlToRedirect => Old URL
			$urlToRedirect = $this->_zendDb->getAdapter()->fetchRow("SELECT IF( (SELECT count(*) FROM optimized where page_id = '$pageId' AND url IS NOT NULL), (SELECT url from optimized where page_id = '$pageId') , (SELECT url from page where id = '$pageId')) as urlToRedirect");
			if($tkp != null) {
				$this->_zendDb->getAdapter()->query("UPDATE `page` AS `p` INNER JOIN `page` AS `sp` ON `p`.`id` = `sp`.`id` SET `p`.`targeted_key_phrase` = '".addslashes($tkp)."' where `p`.`id` = ".$pageId."");
			}
			$this->_zendDb->getAdapter()->query("insert into optimized (`page_id`, `url`, `status`, `url_rule_id`) VALUES ('$pageId', '$tweakedUrl', '$pageStatus', '$ruleId') ON DUPLICATE KEY UPDATE `url` = CONCAT('$tweakedUrl') , `status` = CONCAT('$pageStatus'), `url_rule_id` = CONCAT('$ruleId')");
			$urls = $this->_zendDb->getAdapter()->fetchRow($this->_zendDb->getAdapter()->select()->from(array('p' => 'page'), array('oldUrl' => 'p.url'))->joinLeft(array('o' => 'optimized'), 'p.id = o.page_id', array('newUrl' => 'o.url'))->where('p.id = ?', $pageId));
            if(($urlToRedirect['urlToRedirect'] != $urls['newUrl'])) {
                $this->_updateInternalLinks($urlToRedirect, $urls, $pageId);
            }
			$whereUrl = $this->_zendDb->getAdapter()->quoteInto('from_url = ?', $urls['newUrl']);
			//$wherePageId = $this->_zendDb->getAdapter()->quoteInto('page_id = ?', $pageId);
            $whereUrlTo = $this->_zendDb->getAdapter()->quoteInto('to_url = ?', $urlToRedirect['urlToRedirect']);
			$this->_zendDb->getAdapter()->delete('redirect', array($whereUrl, $whereUrlTo));//$wherePageId
			$rowExists = $this->_zendDb->getAdapter()->fetchRow("SELECT IF( (SELECT count(*) FROM redirect WHERE page_id = '$pageId' AND from_url = '".$urlToRedirect['urlToRedirect']."' AND to_url = '".$urls['newUrl']."' AND domain_to = '".$this->_websiteHelper->getUrl()."' AND domain_from = '".$this->_websiteHelper->getUrl()."'), true, false ) rowExists");
			if(!$rowExists['rowExists'] && ($urlToRedirect['urlToRedirect'] != $urls['newUrl']) ) {
				$this->_zendDb->getAdapter()->query("insert into redirect (`page_id`, `from_url`, `to_url`, `domain_to`, `domain_from`) VALUES('$pageId','".$urlToRedirect['urlToRedirect']."', '".$urls['newUrl']."', '".$this->_websiteHelper->getUrl()."', '".$this->_websiteHelper->getUrl()."')");
                $this->_updateRedirectsToUrl($pageId, $urls['newUrl'], $this->_websiteHelper->getUrl());
                $this->_zendDb->getAdapter()->query("UPDATE `page` set `preview_image` =
                                    CASE
                                    WHEN (`preview_image` IS NOT NULL AND `preview_image` != '')
                                    THEN (CONCAT('".substr($urls['newUrl'] ,0 , strrpos($urls['newUrl'], '.'))."' ,
                                             SUBSTRING(`preview_image`, LOCATE('.', `preview_image`), LENGTH(`preview_image`) )
                                    ) )
                                    ELSE NULL
                                    END
                                     where `id` = '".$pageId."'");
                Tools_Page_Tools::processPagePreviewImage($urls['newUrl'], Tools_Page_Tools::processPagePreviewImage($urlToRedirect['urlToRedirect']));
			}
		}
	}

	private function _createSeoRule($params, $pageId, $pageStatus, $pageSeoRuleId, $tkp) {
		if($pageSeoRuleId === '0') {
			if($this->_zendDb->getAdapter()->fetchRow("SELECT IF( (SELECT count(*) FROM optimized where page_id = $pageId), true , false)")) {
                $pageData = $this->_zendDb->getAdapter()->fetchRow(
                    $this->_zendDb->getAdapter()->select()
                        ->from(array('o' => 'optimized'), array('urlRuleId' => 'url_rule_id', 'pageStatus' => 'status'))
                        ->where($this->_zendDb->getAdapter()->quoteInto("page_id = ?", $pageId))
                );
                if($pageData['urlRuleId'] === null) {
                    $this->_zendDb->getAdapter()->delete('optimized', array('page_id = ?' => $pageId));
                }else {
                    $pageStatus = $pageData['pageStatus'];
				    $this->_zendDb->getAdapter()->query("UPDATE optimized set `h1` = NULL, `seo_rule_id` = NULL, `header_title` = NULL, `nav_name` = NULL, `targeted_key_phrase` = NULL, `meta_description` = NULL, `meta_keywords` = NULL, `teaser_text` = NULL, `seo_intro` = NULL, `seo_intro_target` = NULL, `status` = '$pageStatus' where page_id = '$pageId'");
                }
			}
			return;
		}
		$concats = null;
		$duplicateConcats = null;
		$optimizedPage = array(
			'{h1}' => "',`h1`,'",
			'{navigation_label}' => "',`nav_name`,'",
			'{page_teaser}' => "',`teaser_text`,'",
			'{targeted_key_phrase}' => "',`targeted_key_phrase`,'",
            '{cat_pages_nav_label}'=> "',(SELECT (CASE
    WHEN (`parent_id` = '0')
    THEN (SELECT IFNULL(GROUP_CONCAT(`nav_name` SEPARATOR ', '),\"\") FROM `page` WHERE `parent_id` = '".$pageId."')
    WHEN (`parent_id` != '0' AND `parent_id` > '0' AND `parent_id` != IFNULL((SELECT `id` from `page` WHERE `url` = 'product-pages.html'),0) )
    THEN (SELECT IFNULL(GROUP_CONCAT(`nav_name` SEPARATOR ', '),\"\") FROM `page` WHERE `parent_id` = (SELECT `parent_id` FROM `page` WHERE `id` = '".$pageId."')  AND `id` != '".$pageId."')
    ELSE ''
  END) AS glueString
FROM `page` where `id` = '".$pageId."'),'",
            '{cat_index_nav_label}'=> "',(SELECT (CASE
    WHEN (`parent_id` = '0') THEN (SELECT IFNULL(`nav_name`,\"\") FROM `page` WHERE `id` = '".$pageId."')
    WHEN (`parent_id` != '0' AND `parent_id` > '0' AND `parent_id` != IFNULL((SELECT `id` from `page` WHERE `url` = 'product-pages.html'),0) )
    THEN (SELECT IFNULL(`nav_name`,\"\") FROM `page` WHERE `id` = (SELECT `parent_id` FROM `page` WHERE `id` = '".$pageId."') )
    ELSE ''
  END) AS glueStringIndex
FROM `page` where `id` = '".$pageId."'),'",
            '{page_tags}' => "',(SELECT IFNULL(GROUP_CONCAT(`fa`.`name` SEPARATOR ', '), \"\") FROM `featured_area` AS fa JOIN `page_fa` as pfa ON fa.id = pfa.fa_id WHERE `pfa`.`page_id` = '".$pageId."'),'",
			'{meta_description}' => "',`meta_description`,'",
            '{meta_keywords}' => "',`meta_keywords`,'",
			'{org_name}' => addslashes($this->_widcard['wicOrganizationName']),
			'{org_description}' => addslashes($this->_widcard['wicOrganizationDescription']),
			'{local_phone}' => addslashes($this->_widcard['wicPhone']),
			'{city}' => addslashes($this->_widcard['wicCity']),
			'{state}' => addslashes($this->_widcard['wicCountryState']),
			'{MSA}' => addslashes($this->_widcard['wicMSA']),
			'{zip}' => addslashes($this->_widcard['wicZip']),
			'{industry}' => addslashes($this->_widcard['wicIndustryType']),
			'{product_name}' => "',`pr`.`name`,'",
			'{brand}' => "',`b`.`name`,'",
			'{product_tags}'=> "',(select IFNULL(group_concat(t.name), \"\") from shopping_tags as t join shopping_product_has_tag as p2t on p2t.tag_id = t.id join shopping_product as p ON p2t.product_id = p.id where ".$this->_zendDb->getAdapter()->quoteInto('p.page_id = ?', $pageId)."),'",
			'{product_price}' => "',ROUND(`pr`.`price`, 2),'",
			'{product_SKU}' => "',`pr`.`sku`,'",
			'{promo_price}' => "',IFNULL(`promo`.`promo_price`,\"\"),'",
			'{promo_exp_date}' => "',IFNULL(`promo`.`promo_due`,\"\"),'"
		);

        $optimizedPage = $this->_addWidcardCustomFields($optimizedPage);

		$optimizedPageUpdate = array(
			'{h1}' => "',`p`.`h1`,'",
			'{navigation_label}' => "',`p`.`nav_name`,'",
			'{page_teaser}' => "',`p`.`teaser_text`,'",
			'{targeted_key_phrase}' => "',`p`.`targeted_key_phrase`,'",
            '{cat_pages_nav_label}'=> "',(SELECT (CASE
    WHEN (`parent_id` = '0')
    THEN (SELECT IFNULL(GROUP_CONCAT(`nav_name` SEPARATOR ', '),\"\") FROM `page` WHERE `parent_id` = '".$pageId."')
    WHEN (`parent_id` != '0' AND `parent_id` > '0' AND `parent_id` != IFNULL((SELECT `id` from `page` WHERE `url` = 'product-pages.html'),0) )
    THEN (SELECT IFNULL(GROUP_CONCAT(`nav_name` SEPARATOR ', '),\"\") FROM `page` WHERE `parent_id` = (SELECT `parent_id` FROM `page` WHERE `id` = '".$pageId."')  AND `id` != '".$pageId."')
    ELSE ''
  END) AS glueString
FROM `page` where `id` = '".$pageId."'),'",
            '{cat_index_nav_label}'=> "',(SELECT (CASE
    WHEN (`parent_id` = '0') THEN (SELECT IFNULL(`nav_name`,\"\") FROM `page` WHERE `id` = '".$pageId."')
    WHEN (`parent_id` != '0' AND `parent_id` > '0' AND `parent_id` != IFNULL((SELECT `id` from `page` WHERE `url` = 'product-pages.html'),0) )
    THEN (SELECT IFNULL(`nav_name`,\"\") FROM `page` WHERE `id` = (SELECT `parent_id` FROM `page` WHERE `id` = '".$pageId."') )
    ELSE ''
  END) AS glueStringIndex
FROM `page` where `id` = '".$pageId."'),'",
            '{page_tags}' => "',(SELECT IFNULL(GROUP_CONCAT(`fa`.`name` SEPARATOR ', '), \"\") FROM `featured_area` AS fa JOIN `page_fa` as pfa ON fa.id = pfa.fa_id WHERE `pfa`.`page_id` = '".$pageId."'),'",
			'{meta_description}' => "',`p`.`meta_description`,'",
            '{meta_keywords}' => "',`p`.`meta_keywords`,'",
			'{org_name}' => addslashes($this->_widcard['wicOrganizationName']),
			'{org_description}' => addslashes($this->_widcard['wicOrganizationDescription']),
			'{local_phone}' => addslashes($this->_widcard['wicPhone']),
			'{city}' => addslashes($this->_widcard['wicCity']),
			'{state}' => addslashes($this->_widcard['wicCountryState']),
			'{zip}' => addslashes($this->_widcard['wicZip']),
			'{MSA}' => addslashes($this->_widcard['wicMSA']),
			'{industry}' => $this->_widcard['wicIndustryType'],
			'{product_name}' => "',`pr`.`name`,'",
			'{brand}' => "',`b`.`name`,'",
			'{product_tags}'=> "',(select IFNULL(group_concat(t.name), \"\") from shopping_tags as t join shopping_product_has_tag as p2t on p2t.tag_id = t.id join shopping_product as p ON p2t.product_id = p.id where ".$this->_zendDb->getAdapter()->quoteInto('p.page_id = ?', $pageId)."),'",
			'{product_price}' => "',ROUND(`pr`.`price`, 2),'",
			'{product_SKU}' => "',`pr`.`sku`,'",
			'{promo_price}' => "',IFNULL(`promo`.`promo_price`,\"\"),'",
			'{promo_exp_date}' => "',IFNULL(`promo`.`promo_due`,\"\"),'"
		);

        $optimizedPageUpdate = $this->_addWidcardCustomFields($optimizedPageUpdate);

        $replaceSeoRulesSign = array('<' => '&lt;', '>' => '&gt;', '"' => '&quot;', '\\\'' => '&#039;');
        $startReplace = '';
        $endReplace = '';
        foreach ($replaceSeoRulesSign as $k => $v) {
            $startReplace .= "REPLACE( ";
            $endReplace .= ", '$k', '$v') ";
        }
		$inPageIdsArray = in_array($pageId, $this->_productPageIds);
		foreach ($params as $pageFieldName => $pageField) {
            $fieldParts = explode("'", $pageField);
            $fieldPartsDuplicate = array();
            foreach($fieldParts as $i => $part) {
                $fieldParts[$i] = str_replace("'", "''", $part);

                if (!$this->_isPromoInstalled && $inPageIdsArray) {
                    $fieldParts[$i] = str_replace(array('{promo_price}', '{promo_exp_date}'), '', $part);
                    $fieldPartsDuplicate[$i] = $fieldParts[$i];
                }

                $fieldParts[$i] = str_replace(array("'',", ",''"), '', "'" . strtr($part, $optimizedPage) . "'");
                $fieldPartsDuplicate[$i] = str_replace(array("'',", ",''"), '', "'" . strtr($part, $optimizedPageUpdate) . "'");
            }

            $fieldConcat = implode(",'\\'',", $fieldParts);
            $duplicateFieldConcat = implode(",'\\'',", $fieldPartsDuplicate);

            if( ($pageFieldName == 'meta_description') || ($pageFieldName == 'meta_keywords') ) {
                $concats[] = ' IF ( CONCAT('. $fieldConcat . ') = \'\', NULL, ' . $startReplace . 'CONCAT(' . $fieldConcat . ')' . $endReplace . ' ) as \'' . $pageFieldName .'\'';
                $duplicateConcats[] = '`' . $pageFieldName . '` = IF ( CONCAT(' . $duplicateFieldConcat . ') = \'\' , NULL, ' . $startReplace . 'CONCAT(' . $duplicateFieldConcat . ')' . $endReplace . ' ) ';
            }
            else {
			    $concats[] = ' IF ( CONCAT('. $fieldConcat . ') = \'\', NULL, CONCAT('. $fieldConcat . ') ) as \'' . $pageFieldName .'\'';
                $duplicateConcats[] = '`' . $pageFieldName . '` = IF ( CONCAT(' . $duplicateFieldConcat . ') = \'\' , NULL, CONCAT(' . $duplicateFieldConcat . ') ) ';
            }
		}
		$select = $this->_zendDb->getAdapter()->select()->from(array('p' => 'page'), array(new Zend_Db_Expr("`p`.`id` as 'page_id', " . implode(',',$concats) . " , IF ( CONCAT(`p`.`targeted_key_phrase`) IS NULL, CONCAT(`p`.`h1`), CONCAT(`p`.`targeted_key_phrase`)) as 'targeted_key_phrase', CONCAT('$pageStatus') as 'status', CONCAT('$pageSeoRuleId') as 'seo_rule_id'")))->where('p.id = ?', $pageId);

		if(!empty($this->_productPageIds) && $inPageIdsArray) {
			$select->joinLeft(array('pr' => 'shopping_product'), '`p`.`id` = `pr`.`page_id`', array(null))->
				joinLeft(array('b' => 'shopping_brands'), '`pr`.`brand_id` = `b`.`id`', array(null));
		}

		if($this->_isPromoInstalled && $inPageIdsArray) {
			$select->joinLeft(array('promo' => 'plugin_promo'), '`pr`.`id` = `promo`.`product_id`', array(null));
		}

		if($tkp != null) {
			$this->_zendDb->getAdapter()->query("UPDATE `page` AS `p` INNER JOIN `page` AS `sp` ON `p`.`id` = `sp`.`id` SET `p`.`targeted_key_phrase` = '".addslashes($tkp)."' where `p`.`id` = ".$pageId."");
		}

		$this->_zendDb->getAdapter()->query("insert into optimized (`page_id`, `h1`, `header_title`, `nav_name`,`meta_description`,`meta_keywords`,  `teaser_text`, `seo_intro`, `seo_intro_target`,`targeted_key_phrase`, `status`, `seo_rule_id`) "
            . $select->assemble()
            . " ON DUPLICATE KEY UPDATE "
            . implode(',',$duplicateConcats)
            . ", `targeted_key_phrase` = IF ( CONCAT(`p`.`targeted_key_phrase`) IS NULL, CONCAT(`p`.`h1`), CONCAT(`p`.`targeted_key_phrase`)) , `status` = CONCAT('$pageStatus'), `seo_rule_id` = CONCAT('$pageSeoRuleId')");
	}

	private function _addWidcardCustomFields($optimizationParams) {
        $customParams = Widcard_Models_Mappers_WidcardCustomParamsMapper::getInstance()->getCustomParams();

        if(!empty($customParams)) {
            foreach($customParams as $key => $value) {
                $optimizationParams['{' . $key . '}'] = addslashes($value);
            }
        }

        return $optimizationParams;
    }

	private function _removeOptimized($params) {
		if(isset($params['pageIds']) || ($params['massApply']) ) {
			$pageIds = (isset($params['pageIds']))? $params['pageIds'] : array();
			unset ($params['pageIds']);
			if(isset($params['massApply'])) {
                $massCleanPages = $this->_getMassApplyPages($params);
                $pageIds = $massCleanPages['pageIds'];
                $pageCount = $massCleanPages['count'];
                if(empty($pageIds)) {
                    return array('done' => true, 'offset' => null, 'message' => 'There are no pages to clean');
                }
            }
			try{
                $this->_handleUrlDeOptimization($pageIds);
		    } catch (Exception $e){error_log($e->getMessage()); return $e->getMessage();}
			$optimizedDbTable = new Application_Model_DbTable_Optimized();
			$where = $optimizedDbTable->getAdapter()->quoteInto('page_id IN(?)', $pageIds);
			$optimizedDbTable->delete($where);
			//$this->_cacheHelper->clean('toaster_301redirects', '301redirects');
		}
        if(isset($params['massApply'])) {
            $generalOffset = isset($params['massApply']['offset'])? (integer)$params['massApply']['offset'] + self::PAGES_APPLY_RULE_LIMIT : self::PAGES_APPLY_RULE_LIMIT;
            if((integer)$pageCount > $generalOffset) {
                if(isset($params['massApply']['offset'])) {
                    $offset = $generalOffset;
                }else {
                    $offset = self::PAGES_APPLY_RULE_LIMIT;
                }
                $data = array('done' => true, 'offset' => $offset);
            }else {
                $data = array('done' => true, 'offset' => null);
            }
        }else {
            $data = array('done' => true, 'message' => 'Cleaning has been done');
        }
        $this->_cleanCache();
		return $data;
	}

	public function cleanOptimizedPages($params) {
        $selectLimit = 100;
        $optimizedDbTable = new Application_Model_DbTable_Optimized();

        if(empty($params['totalOptimized'])) {
            $pageCountSelect = $this->_zendDb->getAdapter()->select()
                ->from($optimizedDbTable->info('name'), 'COUNT("page_id") as totalOptimized');
            $totalOptimizedPages = $optimizedDbTable->getAdapter()->fetchOne($pageCountSelect);
            $params['totalOptimized'] = $totalOptimizedPages;
        }


        $pageIdSelect = $this->_zendDb->getAdapter()->select()
            ->from($optimizedDbTable->info('name'))->limit($selectLimit);
        $pageIds = $optimizedDbTable->getAdapter()->fetchCol($pageIdSelect);

        if(!empty($pageIds)) {
            try {
                $this->_handleUrlDeOptimization($pageIds);
            } catch (Exception $e) {
                error_log($e->getMessage());
                return $e->getMessage();
            }

            $optimizedDbTable = new Application_Model_DbTable_Optimized();
            $where = $optimizedDbTable->getAdapter()->quoteInto('page_id IN(?)', $pageIds);
            $optimizedDbTable->delete($where);

            $this->_cacheHelper->clean('toaster_301redirects', '301redirects');
        }

        $data = array(
            'done' => true,
            'totalOptimized' => $params['totalOptimized'],
            'shift' => count($pageIds),
            'message' => 'Cleaning has been done'
        );
        $this->_cleanCache();

        return $data;
    }

    private function _handleUrlDeOptimization($pageIds) {
        foreach ($pageIds as $pageId) {
            if ($pageId != NULL) {
                $urlToRedirect = $this->_zendDb->getAdapter()->fetchRow(
                    "SELECT IF( (
                        SELECT count(*) FROM optimized where page_id = '$pageId' AND url IS NOT NULL), 
                            (SELECT url FROM optimized where page_id = '$pageId') , (false) 
                        ) as urlToRedirect");
                if ($urlToRedirect['urlToRedirect']) {
                    $this->_zendDb->getAdapter()->query(
                        "insert into redirect (`page_id`, `from_url`, `to_url`, `domain_to`, `domain_from`)
                        VALUES(
                            '$pageId',
                            '" . $urlToRedirect['urlToRedirect'] . "', 
                            (SELECT url from page where id = '$pageId'),
                            '" . $this->_websiteHelper->getUrl() . "',
                            '" . $this->_websiteHelper->getUrl() . "')"
                    );

                    $whereUrl = $this->_zendDb->getAdapter()->quoteInto("from_url IN(?)",
                        $this->_zendDb->getAdapter()->fetchRow("SELECT url from page where id = '$pageId'")
                    );
                    $wherePageId = $this->_zendDb->getAdapter()->quoteInto('page_id = ?', $pageId);
                    //$whereUrlTo = $this->_zendDb->getAdapter()->quoteInto("to_url = ?", $urlToRedirect['urlToRedirect']);
                    $this->_zendDb->getAdapter()->delete('redirect', array($whereUrl, $wherePageId));
                    $newPageUrl = $this->_zendDb->getAdapter()->fetchRow("SELECT url from page where id = '$pageId'");
                    $this->_updateRedirectsToUrl($pageId, $newPageUrl['url'], $this->_websiteHelper->getUrl());
                    $this->_zendDb->getAdapter()->query("UPDATE `page` set `preview_image` =
                                (CONCAT('" . substr($newPageUrl['url'], 0, strrpos($newPageUrl['url'], '.')) . "' ,
                                         SUBSTRING(`preview_image`, LOCATE('.', `preview_image`), LENGTH(`preview_image`) )
                                ) ) where `id` = '" . $pageId . "'");
                    Tools_Page_Tools::processPagePreviewImage($newPageUrl['url'],
                        Tools_Page_Tools::processPagePreviewImage($urlToRedirect['urlToRedirect'])
                    );

                    if ($urlToRedirect['urlToRedirect'] != $newPageUrl['url']) {
                        $this->_updateInternalLinks($urlToRedirect, array('newUrl' => $newPageUrl['url']), $pageId, true);
                    }
                }
            }
        }
    }

    private function _updateInternalLinks($urlToRedirect, $urls, $pageId, $cleanFlag = false) {
        try {
            $internalLink = $this->_zendDb->getAdapter()->fetchCol("SELECT `id_container` FROM `link_container` WHERE `link` = '" . $this->_websiteUrl.$urlToRedirect['urlToRedirect'] . "'");
            if(!empty($internalLink)) {
                $this->_zendDb->getAdapter()->query("UPDATE `link_container` SET `link` = '" . $this->_websiteUrl.$urls['newUrl'] . "' WHERE `link` = '" . $this->_websiteUrl.$urlToRedirect['urlToRedirect'] . "'");
                $this->_zendDb->getAdapter()->query("UPDATE `container` SET `content` = REPLACE(`content`, '" . $this->_websiteUrl.$urlToRedirect['urlToRedirect'] . "', '" . $this->_websiteUrl.$urls['newUrl'] . "') WHERE `id` IN(" . implode(',', $internalLink) . ")");
            }
        }catch (Exception $e){error_log($e->getMessage());}
    }

    private function _updateRedirectsToUrl($pageId, $toUrl, $domainTo) {
        $this->_zendDb->getAdapter()->query("UPDATE `redirect` SET
        `to_url` = '" . $toUrl . "',
        `domain_to` = '" . $domainTo . "'
        WHERE `page_id` = '" . $pageId ."'");
    }

    private function _joinTags($select) {
        return $select->joinLeft(array('pfa' => 'page_fa'), 'p.id = pfa.page_id', null)
            ->joinLeft(array('fa' => 'featured_area'), 'pfa.fa_id = fa.id', null);
    }

    private function _joinNewsTags($select) {
        return $select->joinLeft(array('pnnht' => 'plugin_newslog_news_has_tag'), 'pnn.id = pnnht.news_id', null)
            ->joinLeft(array('pnt' => 'plugin_newslog_tag'), 'pnt.id = pnnht.tag_id', null);
    }

    private function _processTagsField($fields) {
        $tagsFieldIndex = array_search('tags', $fields);

        if($tagsFieldIndex !== false) {
            $fields[$tagsFieldIndex] = "IFNULL(GROUP_CONCAT(fa.name), '') AS tags";
            $this->_tagsFieldIndex = $tagsFieldIndex;
        }
        return $fields;
    }

    private function _processNewsTagsField($fields) {
        $tagsFieldIndex = array_search('tags', $fields);

        if($tagsFieldIndex !== false) {
            $fields[$tagsFieldIndex] = "IFNULL(GROUP_CONCAT(pnt.name), '') AS tags";
            $this->_tagsFieldIndex = $tagsFieldIndex;
        }
        return $fields;
    }

    /*
     * Pull toaster pages section
     * */
    /*
     * Get index.html page, return it as category page, even if it belongs to another level
     * */
    public function findIndexPage($fields, $excludePages, $overwrite = null) {
        $fields = $this->_processTagsField($fields);
        $indexSelect = $this->_zendDb->getAdapter()->select()->from(array('p' => 'page'), $fields)->where(
            $this->_zendDb->getAdapter()->quoteInto('p.url = ?', self::PAGE_INDEX_HTML)
        )->group('p.id');

        if($this->_tagsFieldIndex !== false) {
            $indexSelect = $this->_joinTags($indexSelect);
        }

        if($overwrite) {
            $indexSelect = $this->_joinOptimizedToSelect($indexSelect);
        }

        $this->_indexPage = $this->_zendDb->getAdapter()->fetchRow($indexSelect);
        $this->_indexPage['native_parent_id'] = $this->_indexPage['parent_id'];
        $this->_indexPage['parent_id'] = 0;
        $fields[$this->_tagsFieldIndex] = 'tags';
        foreach($fields as $field) {
            $this->_toasterPages[self::PAGE_TYPE_CATEGORY][$this->_indexPage['id']][$field] = $this->_indexPage[$field];
        }
        /*
         * Set optimized values for index.html if those exist
         * */
        if(isset($this->_indexPage['status']) && $this->_indexPage['status'] !== null) {
            $this->_setOptimizedPageValues($this->_indexPage, self::PAGE_TYPE_CATEGORY);
        }
        /*
         * Get regular pages under INDEX.html, IF INDEX.html is category
         * */
        if((integer)$this->_indexPage['native_parent_id'] === 0) {
            $countWhere = $this->_zendDb->getAdapter()->quoteInto('p.parent_id = ?', $this->_indexPage['id']);
            $countWhere .= $this->_zendDb->getAdapter()->quoteInto(" AND p.draft = ?", '0');
            if(count($excludePages)) {
                $countWhere .= $this->_zendDb->getAdapter()->quoteInto(" AND p.id NOT IN(?)", $excludePages);
            }
            $countPagesUnderCategory = $this->_zendDb->getAdapter()->fetchOne(
                $this->_zendDb->getAdapter()->select()->from(array('p' => 'page'), array('pageNumber' => 'COUNT(DISTINCT p.id)'))
                    ->where($countWhere)
            );

            $this->_toasterPages['pagesNumber'][] = array(
                'pageType' => self::PAGE_TYPE_CATEGORY,
                'pageId' => $this->_indexPage['id'],
                'pageNumber' => $countPagesUnderCategory
            );
        }
    }

    private function _joinOptimizedToSelect($zendSelect) {
        $optimizePrefix = 'Optm_';
        return $zendSelect->joinLeft(array('o'=>'optimized'), 'p.id = o.page_id',
            array(
                $optimizePrefix.'h1' => 'h1',
                $optimizePrefix.'nav_name' => 'nav_name',
                $optimizePrefix.'header_title' => 'header_title',
                $optimizePrefix.'url' => 'url',
                $optimizePrefix.'meta_description' => 'meta_description',
                $optimizePrefix.'meta_keywords' => 'meta_keywords',
                $optimizePrefix.'targeted_key_phrase' => 'targeted_key_phrase',
                $optimizePrefix.'teaser_text' => 'teaser_text',
                'status',
                'seo_rule_id',
                'url_rule_id'
            )
        );
    }

    private function _setOptimizedPageValues($page, $pageType) {
        $optimizePrefix = 'Optm_';
        $this->_toasterPages[$pageType][$page['id']]['status'] = $page['status'];
        $this->_toasterPages[$pageType][$page['id']]['seo_rule_id'] = $page['seo_rule_id'];
        $this->_toasterPages[$pageType][$page['id']]['url_rule_id'] = $page['url_rule_id'];
        if($page['status'] == 'tweaked') {
            $this->_toasterPages[$pageType][$page['id']]['tweaked']['h1'] = $page[$optimizePrefix.'h1'];
            $this->_toasterPages[$pageType][$page['id']]['tweaked']['header_title'] = $page[$optimizePrefix.'header_title'];
            $this->_toasterPages[$pageType][$page['id']]['tweaked']['nav_name'] = $page[$optimizePrefix.'nav_name'];
            $this->_toasterPages[$pageType][$page['id']]['tweaked']['url'] = ($page[$optimizePrefix.'url'] == null) ? $page['url'] : $page[$optimizePrefix.'url'];
            $this->_toasterPages[$pageType][$page['id']]['tweaked']['meta_description'] = $page[$optimizePrefix.'meta_description'];
            $this->_toasterPages[$pageType][$page['id']]['tweaked']['meta_keywords'] = $page[$optimizePrefix.'meta_keywords'];
            $this->_toasterPages[$pageType][$page['id']]['tweaked']['targeted_key_phrase'] = $page[$optimizePrefix.'targeted_key_phrase'];
            $this->_toasterPages[$pageType][$page['id']]['tweaked']['teaser_text'] = $page[$optimizePrefix.'teaser_text'];
        }
    }

    /**
     * Collect all category pages
     * @param $fields array - page fields collection
     * @param $excludePages array() - page ID's which shouldn't be selected
     * @param $overwrite bool - flag, to get optimized values for particular page, if those exist
     * @return NO RETURNING VALUE
     * */
    public function pullCategoryPages($fields, $excludePages, $overwrite = null) {
        /*
         * Get categories
         * */
        array_push($excludePages, $this->_indexPage['id']);
        $where = $this->_zendDb->getAdapter()->quoteInto("p.parent_id = ?", 0);
        $where .= $this->_zendDb->getAdapter()->quoteInto(" AND p.draft = ?", '0');
        if(count($excludePages)) {
            $where .= $this->_zendDb->getAdapter()->quoteInto(" AND p.id NOT IN(?)", $excludePages);
        }

        $fields = $this->_processTagsField($fields);

        $select = $this->_zendDb->getAdapter()->select()->from(array('p' => 'page'), $fields)->where($where)->order(array('p.id ASC'))->group('p.id');

        if($this->_tagsFieldIndex !== false) {
            $select = $this->_joinTags($select);
        }

        if($overwrite) {
            $select = $this->_joinOptimizedToSelect($select);
        }

        $categoryPages = $this->_zendDb->getAdapter()->fetchAll($select);

        $fields[$this->_tagsFieldIndex] = 'tags';
        /*
         * Collect all category pages
         * Fetch regular pages number under existing categories
         * */
        if(!empty($categoryPages)) {
            foreach($categoryPages as $categoryPage) {
                foreach($fields as $field) {
                    $this->_toasterPages[self::PAGE_TYPE_CATEGORY][$categoryPage['id']][$field] = $categoryPage[$field];
                }
                if(isset($categoryPage['status']) && $categoryPage['status'] !== null) {
                    $this->_setOptimizedPageValues($categoryPage, self::PAGE_TYPE_CATEGORY);
                }
                $countWhere = $this->_zendDb->getAdapter()->quoteInto('p.parent_id = ?', $categoryPage['id']);
                $countWhere .= $this->_zendDb->getAdapter()->quoteInto(" AND p.draft = ?", '0');
                if(count($excludePages)) {
                    $countWhere .= $this->_zendDb->getAdapter()->quoteInto(" AND p.id NOT IN(?)", $excludePages);
                }
                $countPagesUnderCategory = $this->_zendDb->getAdapter()->fetchOne(
                    $this->_zendDb->getAdapter()->select()->from(array('p' => 'page'), array('pageNumber' => 'COUNT(DISTINCT p.id)'))
                        ->where($countWhere)
                );

                $this->_toasterPages['pagesNumber'][] = array(
                    'pageType' => self::PAGE_TYPE_CATEGORY,
                    'pageId' => $categoryPage['id'],
                    'pageNumber' => $countPagesUnderCategory
                );
            }
        }
    }

    /**
     * Collect all regular pages
     * @param $fields array - page fields collection
     * @param $excludePages array() - page ID's which shouldn't be selected
     * @param $overwrite bool - flag, to get optimized values for particular page, if those exist
     * @param $pulledPages array - pages pulled from toaster
     * @param $forcePull - bool - flag to pull another portion of self::PAGE_SAMPLE_NUMBER = (25)
     * @return NO RETURNING VALUE
     * */
    public function pullRegularPages($fields, $excludePages, $overwrite = null, $pulledPages = array(), $forcePull = false) {
        $unionQueries = array();
        if((integer)$this->_indexPage['native_parent_id'] !== 0) {
            array_push($excludePages, $this->_indexPage['id']);
        }
        /*
         * Get category pages
         * */
        if($forcePull === false) {
            $where = $this->_zendDb->getAdapter()->quoteInto("p.parent_id = ?", 0);
        }
        elseif($forcePull !== false) {
            $where = $this->_zendDb->getAdapter()->quoteInto("p.id = ?", $forcePull);
        }
        if(count($excludePages)) {
            $where .= $this->_zendDb->getAdapter()->quoteInto(" AND p.id NOT IN(?)", $excludePages);
        }

        foreach($fields as $fld) {
            if($fld !== 'tags') {
                $fieldsToSelect[] = '`p`.`'.$fld.'`';
            }
        }

        $fields = $this->_processTagsField($fields);

        $categoryPages = $this->_zendDb->getAdapter()->fetchAll("SELECT pageCn.pagesInCategory, " . implode(', ', $fieldsToSelect) . "
            FROM `page` AS `p`
              INNER JOIN (SELECT `parent_id`, GROUP_CONCAT(`page`.`id`) AS pagesInCategory
              FROM `page` WHERE page.parent_id NOT IN(0) GROUP BY page.parent_id ORDER BY page.id ASC) AS pageCn ON `p`.`id` = pageCn.parent_id
            WHERE $where GROUP BY `p`.`id` ORDER BY `p`.`id` ASC");
        /*
         * If categories exist, collect regular pages
         * */
        if(!empty($categoryPages)) {
            foreach($categoryPages as $categoryPage) {
                $where = $this->_zendDb->getAdapter()->quoteInto("p.parent_id = ?", $categoryPage['id']);
                $where .= $this->_zendDb->getAdapter()->quoteInto(" AND p.draft = ?", '0');
                if(count($excludePages)) {
                    $where .= $this->_zendDb->getAdapter()->quoteInto(" AND p.id NOT IN(?)", $excludePages);
                }
                /*
                 * Get before pulled regular pages, if there are some (depend on category), or FORCE pull action
                 * */
                $pulledNum = 0;

                if(!empty($pulledPages[self::PAGE_TYPE_CATEGORY][$categoryPage['id']])) {
                    $pulledNum = count($pulledPages[self::PAGE_TYPE_CATEGORY][$categoryPage['id']]);
                }

                $sliceLimit = ($pulledNum <= self::PAGE_SAMPLE_NUMBER) ? self::PAGE_SAMPLE_NUMBER : $pulledNum;

                if(!empty($pulledPages) &&
                    array_key_exists($categoryPage['id'], $pulledPages[self::PAGE_TYPE_CATEGORY]) &&
                    !empty($pulledPages[self::PAGE_TYPE_CATEGORY][$categoryPage['id']]) && $forcePull === false) {
                    $pagesInCategory = explode(',', $categoryPage['pagesInCategory']);
                    $pagesToSelect = array_diff_key($pagesInCategory, $pulledPages[self::PAGE_TYPE_CATEGORY][$categoryPage['id']]);

                    if(!empty($pagesToSelect)) {
                        $pulledPages[self::PAGE_TYPE_CATEGORY][$categoryPage['id']] += $pagesToSelect;
                        $pulledPages[self::PAGE_TYPE_CATEGORY][$categoryPage['id']] = array_slice($pulledPages[self::PAGE_TYPE_CATEGORY][$categoryPage['id']], 0 ,$sliceLimit);
                    }

                    $whereCategory = $where = $this->_zendDb->getAdapter()->quoteInto("p.parent_id = ?", $categoryPage['id']);

                    if(!empty($pulledPages[self::PAGE_TYPE_CATEGORY][$categoryPage['id']])) {
                        $whereCategory .= $this->_zendDb->getAdapter()->quoteInto(" AND p.id NOT IN(?)", $pulledPages[self::PAGE_TYPE_CATEGORY][$categoryPage['id']]);
                    }

                    $whereCategory .= $this->_zendDb->getAdapter()->quoteInto(" AND p.draft = ?", '0');
                    $selectCategoryRegularPages = $this->_zendDb->getAdapter()->select()->from(array('p' => 'page'), 'id')
                        ->where($whereCategory)
                        ->order(array('p.id ASC'))->limit($sliceLimit);
                    $categoryRegularPages = $this->_zendDb->getAdapter()->fetchCol($selectCategoryRegularPages->assemble());

                    if(!empty($categoryRegularPages)) {
                        $pulledPages[self::PAGE_TYPE_CATEGORY][$categoryPage['id']] = array_merge($pulledPages[self::PAGE_TYPE_CATEGORY][$categoryPage['id']], $categoryRegularPages);
                    }

                    $where .= $this->_zendDb->getAdapter()->quoteInto(" AND p.id IN(?)", $pulledPages[self::PAGE_TYPE_CATEGORY][$categoryPage['id']]);
                }elseif($forcePull !== false) {
                    $where .= $this->_zendDb->getAdapter()->quoteInto(" AND p.id NOT IN(?)", $pulledPages[self::PAGE_TYPE_CATEGORY][$categoryPage['id']]);
                }

                $select = $this->_zendDb->getAdapter()->select()->from(array('p' => 'page'), $fields)
                    ->where($where)
                    ->order(array('p.id ASC'))->group('p.id');

                if($this->_tagsFieldIndex !== false) {
                    $select = $this->_joinTags($select);
                }

                if(empty($pulledPages) ||
                    !array_key_exists($categoryPage['id'], $pulledPages[self::PAGE_TYPE_CATEGORY]) ||
                    empty($pulledPages[self::PAGE_TYPE_CATEGORY][$categoryPage['id']]) || $forcePull !== false) {
                    $select->limit(self::PAGE_SAMPLE_NUMBER);
                }

                if($overwrite) {
                    $select = $this->_joinOptimizedToSelect($select);
                }

                $unionQueries[] = '('.$select->assemble().')';
            }
            $regularPages = $this->_zendDb->getAdapter()->fetchAll(implode(' UNION ', $unionQueries));
            $fields[$this->_tagsFieldIndex] = 'tags';

            if(!empty($regularPages)) {
                foreach($regularPages as $regularPage) {
                    foreach($fields as $field) {
                        $this->_toasterPages[self::PAGE_TYPE_PAGE][$regularPage['id']][$field] = $regularPage[$field];
                    }
                    if(isset($regularPage['status']) && $regularPage['status'] !== null) {
                        $this->_setOptimizedPageValues($regularPage, self::PAGE_TYPE_PAGE);
                    }
                }
            }
        }
    }

    protected function _getNewsPageIds() {
        $newsPlugin = Application_Model_Mappers_PluginMapper::getInstance()->findByName('newslog');
        $pagesInNewsCategory = array();

        if(!empty($newsPlugin)) {
            $where = $this->_zendDb->getAdapter()->quoteInto('p.draft = ?', '0');
            $newsPageIds = $this->_zendDb->getAdapter()->select()
                ->from(array('pnn' => 'plugin_newslog_news'), array('pageId' => 'page_id'))
                ->join(array('p' => 'page'), 'p.id = pnn.page_id', NULL)
                ->where($where)
                ->order(array('p.id ASC'));

            $pagesInNewsCategory = $this->_zendDb->getAdapter()->fetchCol($newsPageIds);
        }

        return $pagesInNewsCategory;
    }

    public function pullNewsPages($fields, $excludePages, $overwrite = null, $pulledPages = array(), $forcePull = false) {
        $where = $this->_zendDb->getAdapter()->quoteInto('p.draft = ?', '0');

        if(!empty($pulledPages) && !empty($pulledPages[self::PAGE_TYPE_NEWS]) && $forcePull === false) {
            $pagesInCategory = $this->_getNewsPageIds();

            if(!empty($pagesInCategory)) {
                $pulledNum = count($pulledPages[self::PAGE_TYPE_NEWS]);
                $sliceLimit = ($pulledNum <= self::PAGE_SAMPLE_NUMBER) ? self::PAGE_SAMPLE_NUMBER : $pulledNum;
                $pulledPages[self::PAGE_TYPE_NEWS] += $pagesInCategory;
                $pulledPages[self::PAGE_TYPE_NEWS] = array_slice($pulledPages[self::PAGE_TYPE_NEWS], 0 ,$sliceLimit);
            }
            $where .= $this->_zendDb->getAdapter()->quoteInto(" AND p.id IN(?)", $pulledPages[self::PAGE_TYPE_NEWS]);
        }elseif($forcePull !== false) {
            $where .= $this->_zendDb->getAdapter()->quoteInto(" AND p.id NOT IN(?)", $pulledPages[self::PAGE_TYPE_NEWS]);
        }

        $fields = $this->_processNewsTagsField($fields);
        $select = $this->_zendDb->getAdapter()->select()
            ->from(array('pnn' => 'plugin_newslog_news'), NULL)
            ->join(array('p' => 'page'), 'p.id = pnn.page_id', $fields)
            ->where($where)
            ->order(array('p.id ASC'))->group('p.id');

        if($this->_tagsFieldIndex !== false) {
            $select = $this->_joinNewsTags($select);
        }

        if(empty($pulledPages) || empty($pulledPages[self::PAGE_TYPE_NEWS]) || $forcePull !== false) {
            $select->limit(self::PAGE_SAMPLE_NUMBER);
        }
        /*
         * Add optimized values to page data
         * */
        if($overwrite) {
            $select = $this->_joinOptimizedToSelect($select);
        }

        $pagesInNewsCategory = $this->_zendDb->getAdapter()->fetchAll($select);
        $fields[$this->_tagsFieldIndex] = 'tags';

        if(!empty($pagesInNewsCategory)) {
            /*
             * Get noCategory pages number
             * */
            $countWhere = $this->_zendDb->getAdapter()->quoteInto("p.draft = ?", '0');

            if(count($excludePages)) {
                $countWhere .= $this->_zendDb->getAdapter()->quoteInto(" AND p.id NOT IN(?)", $excludePages);
            }

            $countPagesUnderNewsCategory = $this->_zendDb->getAdapter()->fetchOne(
                $this->_zendDb->getAdapter()->select()
                    ->from(array('pnn' => 'plugin_newslog_news'), array('pageNumber' => 'COUNT(DISTINCT p.id)'))
                    ->join(array('p' => 'page'), 'p.id = pnn.page_id', NULL)
                    ->where($countWhere)
            );
            /*
             * Collect noCategory pages
             * */
            foreach($pagesInNewsCategory as $newsPage) {
                foreach($fields as $field) {
                    $this->_toasterPages[self::PAGE_TYPE_NEWS][$newsPage['id']][$field] = $newsPage[$field];
                }
                if(isset($newsPage['status']) && $newsPage['status'] !== null) {
                    $this->_setOptimizedPageValues($newsPage, self::PAGE_TYPE_NEWS);
                }
            }
            $this->_toasterPages['pagesNumber'][] = array(
                'pageType' => self::PAGE_TYPE_NEWS,
                'pageId' => null,
                'pageNumber' => $countPagesUnderNewsCategory
            );
        }
    }

    /**
     * Collect all noCategory pages
     * @param $fields array - page fields collection
     * @param $excludePages array() - page ID's which shouldn't be selected
     * @param $overwrite bool - flag, to get optimized values for particular page, if those exist
     * @param $pulledPages array - pages pulled from toaster
     * @param $forcePull - bool - flag to pull another portion of self::PAGE_SAMPLE_NUMBER = (25)
     * @return NO RETURNING VALUE
     * */
    public function pullNoCategoryPages($fields, $excludePages, $overwrite = null, $pulledPages = array(), $forcePull = false) {
        array_push($excludePages, $this->_indexPage['id']);
        $newsPageIds = $this->_getNewsPageIds();

        if(!empty($newsPageIds)) {
            $excludePages = array_merge($excludePages, $newsPageIds);
        }

        /*
         * Fetch and collect noCategory pages
         * */
        $where = $this->_zendDb->getAdapter()->quoteInto('p.parent_id = ?', -1);
        $where .= $this->_zendDb->getAdapter()->quoteInto(' AND p.draft = ?', '0');
        if(count($excludePages)) {
            $where .= $this->_zendDb->getAdapter()->quoteInto(" AND p.id NOT IN(?)", $excludePages);
        }
        /*
         * Get only pulled before pages, if there are some, and compare with current page state
         * */
        if(!empty($pulledPages) && !empty($pulledPages[self::PAGE_TYPE_NOCATEGORY]) && $forcePull === false) {
            $pagesInCategorySelect = $this->_zendDb->getAdapter()->select()->
                from(array('p' => 'page', array('id')))->where($where)->order(array('p.id ASC'));
            $pagesInCategory = $this->_zendDb->getAdapter()->fetchCol($pagesInCategorySelect);
            if(!empty($pagesInCategory)) {
                $pulledNum = count($pulledPages[self::PAGE_TYPE_NOCATEGORY]);
                $sliceLimit = ($pulledNum <= self::PAGE_SAMPLE_NUMBER) ? self::PAGE_SAMPLE_NUMBER : $pulledNum;
                $pulledPages[self::PAGE_TYPE_NOCATEGORY] += $pagesInCategory;
                $pulledPages[self::PAGE_TYPE_NOCATEGORY] = array_slice($pulledPages[self::PAGE_TYPE_NOCATEGORY], 0 ,$sliceLimit);
            }
            $where .= $this->_zendDb->getAdapter()->quoteInto(" AND p.id IN(?)", $pulledPages[self::PAGE_TYPE_NOCATEGORY]);
        }elseif($forcePull !== false) {
            $where .= $this->_zendDb->getAdapter()->quoteInto(" AND p.id NOT IN(?)", $pulledPages[self::PAGE_TYPE_NOCATEGORY]);
        }

        $fields = $this->_processTagsField($fields);
        $select = $this->_zendDb->getAdapter()->select()->from(array('p' => 'page'), $fields)
            ->where($where)
            ->order(array('p.id ASC'))->group('p.id');

        if($this->_tagsFieldIndex !== false) {
            $select = $this->_joinTags($select);
        }

        if(empty($pulledPages) || empty($pulledPages[self::PAGE_TYPE_NOCATEGORY]) || $forcePull !== false) {
            $select->limit(self::PAGE_SAMPLE_NUMBER);
        }
        /*
         * Add optimized values to page data
         * */
        if($overwrite) {
            $select = $this->_joinOptimizedToSelect($select);
        }

        $noCategoryPages = $this->_zendDb->getAdapter()->fetchAll($select);
        $fields[$this->_tagsFieldIndex] = 'tags';

        if(!empty($noCategoryPages)) {
            /*
             * Get noCategory pages number
             * */
            $countWhere = $this->_zendDb->getAdapter()->quoteInto('p.parent_id = ?', -1);
            $countWhere .= $this->_zendDb->getAdapter()->quoteInto(" AND p.draft = ?", '0');
            if(count($excludePages)) {
                $countWhere .= $this->_zendDb->getAdapter()->quoteInto(" AND p.id NOT IN(?)", $excludePages);
            }
            $countPagesUnderNoCategory = $this->_zendDb->getAdapter()->fetchOne(
                $this->_zendDb->getAdapter()->select()->from(array('p' => 'page'), array('pageNumber' => 'COUNT(DISTINCT p.id)'))
                    ->where($countWhere)
            );
            /*
             * Collect noCategory pages
             * */
            foreach($noCategoryPages as $noCategoryPage) {
                foreach($fields as $field) {
                    $this->_toasterPages[self::PAGE_TYPE_NOCATEGORY][$noCategoryPage['id']][$field] = $noCategoryPage[$field];
                }
                if(isset($noCategoryPage['status']) && $noCategoryPage['status'] !== null) {
                    $this->_setOptimizedPageValues($noCategoryPage, self::PAGE_TYPE_NOCATEGORY);
                }
            }
            $this->_toasterPages['pagesNumber'][] = array(
                'pageType' => self::PAGE_TYPE_NOCATEGORY,
                'pageId' => null,
                'pageNumber' => $countPagesUnderNoCategory
            );
        }
    }

    /**
     * Collect all product pages
     * @param $fields array - page fields collection
     * @param $excludePages array() - page ID's which shouldn't be selected
     * @param $overwrite bool - flag, to get optimized values for particular page, if those exist
     * @param $pulledPages array - pages pulled from toaster
     * @param $forcePull - bool - flag to pull another portion of self::PAGE_SAMPLE_NUMBER = (25)
     * @return NO RETURNING VALUE
     * */
    public function pullProductPages($fields, $excludePages, $overwrite = null, $pulledPages = array(), $forcePull = false) {
        /*
         * Fetch and collect product pages and additional product data (if products exist)
         * */
        $this->_isPromoInstalled();
        $this->_isShoppingInstalled();
        if(!empty($this->_isShoppingInstalled)) {
            $where = $this->_zendDb->getAdapter()->quoteInto("p.parent_id = ?", $this->_isShoppingInstalled);
            $where .= $this->_zendDb->getAdapter()->quoteInto(' AND p.draft = ?', '0');
            /*
             * Get only pulled before pages, if there are some
             * */
            if(!empty($pulledPages) && !empty($pulledPages[self::PAGE_TYPE_PRODUCT]) && $forcePull === false) {
                $pagesInCategorySelectLimit = count($pulledPages[self::PAGE_TYPE_PRODUCT]) + self::PAGE_SAMPLE_NUMBER;
                $pagesInCategorySelect = $this->_zendDb->getAdapter()->select()->
                    from(array('p' => 'page', array('id')))->
                    where($where)->limit($pagesInCategorySelectLimit)->order(array('p.id ASC'));
                $pagesInCategory = $this->_zendDb->getAdapter()->fetchCol($pagesInCategorySelect);
                if(!empty($pagesInCategory)) {
                    $pulledNum = count($pulledPages[self::PAGE_TYPE_PRODUCT]);
                    $sliceLimit = ($pulledNum <= self::PAGE_SAMPLE_NUMBER) ? self::PAGE_SAMPLE_NUMBER : $pulledNum;
                    $pulledPages[self::PAGE_TYPE_PRODUCT] += $pagesInCategory;
                    $pulledPages[self::PAGE_TYPE_PRODUCT] = array_slice($pulledPages[self::PAGE_TYPE_PRODUCT], 0 ,$sliceLimit);
                }
                $where .= $this->_zendDb->getAdapter()->quoteInto(" AND p.id IN(?)", $pulledPages[self::PAGE_TYPE_PRODUCT]);
            }elseif($forcePull !== false) {
                $where .= $this->_zendDb->getAdapter()->quoteInto(" AND p.id NOT IN(?)", $pulledPages[self::PAGE_TYPE_PRODUCT]);
            }

            if(($index = array_search('tags', $fields)) !== false) {
                unset($fields[$index]);
            }

            $select = $this->_zendDb->getAdapter()->select()->from(array('p' => 'page'), $fields)
                ->where($where)
                ->order(array('p.id ASC'));
            if(empty($pulledPages) || empty($pulledPages[self::PAGE_TYPE_PRODUCT]) || $forcePull !== false) {
                $select->limit(self::PAGE_SAMPLE_NUMBER);
            }
            if($overwrite) {
                $select = $this->_joinOptimizedToSelect($select);
            }

            $productPages = $this->_zendDb->getAdapter()->fetchAll($select);

            if(!empty($productPages)) {
                $productPagesIds = array();
                $countWhere = $this->_zendDb->getAdapter()->quoteInto("p.parent_id = ?", $this->_isShoppingInstalled);
                $countWhere .= $this->_zendDb->getAdapter()->quoteInto(' AND p.draft = ?', '0');
                $countPagesUnderProduct = $this->_zendDb->getAdapter()->fetchOne(
                    $this->_zendDb->getAdapter()->select()->from(array('p' => 'page'), array('pageNumber' => 'COUNT(DISTINCT p.id)'))
                        ->where($countWhere)
                );
                $this->_toasterPages['pagesNumber'][] = array(
                    'pageType' => self::PAGE_TYPE_PRODUCT,
                    'pageId' => null,
                    'pageNumber' => $countPagesUnderProduct
                );
                foreach($productPages as $productPage) {
                    foreach($fields as $field) {
                        $this->_toasterPages[self::PAGE_TYPE_PRODUCT][$productPage['id']][$field] = $productPage[$field];
                    }
                    if(isset($productPage['status']) && $productPage['status'] !== null) {
                        $this->_setOptimizedPageValues($productPage, self::PAGE_TYPE_PRODUCT);
                    }
                    array_push($productPagesIds, $productPage['id']);
                }

                $productDbTable = new Models_DbTable_Product();
                $where = $productDbTable->getAdapter()->quoteInto('page_id IN(?)', $productPagesIds);
                $products = Models_Mapper_ProductMapper::getInstance()->fetchAll($where);
                foreach ($products as $product) {
                    $this->_toasterPages[self::PAGE_TYPE_PRODUCT][$product->getPage()->getId()]['productData']['productName'] = $product->getName();
                    $this->_toasterPages[self::PAGE_TYPE_PRODUCT][$product->getPage()->getId()]['productData']['productPrice'] = $product->getPrice();
                    $this->_toasterPages[self::PAGE_TYPE_PRODUCT][$product->getPage()->getId()]['productData']['brandName'] = $product->getBrand();
                    if($this->_isPromoInstalled) {
                        $promo = $product->getExtraProperties();
                        if(!empty($promo)) {
                            $this->_toasterPages[self::PAGE_TYPE_PRODUCT][$product->getPage()->getId()]['productData']['promoPrice'] = $promo[0]['g:sale_price'];
                            $this->_toasterPages[self::PAGE_TYPE_PRODUCT][$product->getPage()->getId()]['productData']['promoExpireDate'] = substr($promo[0]['g:sale_price_effective_date'], strrpos($promo[0]['g:sale_price_effective_date'], '/')+1, strpos($promo[0]['g:sale_price_effective_date'], 'T') );
                        }
                    }
                    $tags = $product->getTags();
                    if( is_array($tags) && (count($tags) > 0) ) {
                        $tagsArr = array();
                        foreach ($tags as $tag) {
                            $tagsArr[] = $tag['name'];
                        }
                        $productTags = implode(',', $tagsArr);
                        $this->_toasterPages[self::PAGE_TYPE_PRODUCT][$product->getPage()->getId()]['productData']['productTags'] = $productTags;
                    }
                    $this->_toasterPages[self::PAGE_TYPE_PRODUCT][$product->getPage()->getId()]['productData']['productSku'] = $product->getSku();
                    $this->_toasterPages[self::PAGE_TYPE_PRODUCT][$product->getPage()->getId()]['productData']['productMpn'] = $product->getMpn();
                }
            }
        }
    }

    private function _collectOptimizedPages($pageId, $ruleId, $ruleType) {
        $page = Application_Model_Mappers_PageMapper::getInstance()->find($pageId);
        if(!empty($page) && $ruleId != 0) {
            if($page->getOptimized() === false) {
                array_push($this->_optimizedPages, $pageId);
                return;
            }

            $wherePageId = $this->_zendDb->getAdapter()->quoteInto('page_id = ?', $pageId);
            $optimized = $pageData = $this->_zendDb->getAdapter()->fetchRow(
                $this->_zendDb->getAdapter()->select()
                    ->from(array('o' => 'optimized'), array(
                        'seoRuleId' => 'seo_rule_id',
                        'urlRuleId' => 'url_rule_id',
                        'pageStatus' => 'status')
                    )->where($wherePageId)
            );

            if(!empty($optimized) && !in_array($pageId, $this->_optimizedPages)) {
                if ($ruleType === self::SEO_RULE && $optimized['seoRuleId'] != $ruleId) {
                    array_push($this->_optimizedPages, $pageId);
                } elseif ($ruleType === self::URL_RULE && $optimized['urlRuleId'] != $ruleId) {
                    array_push($this->_optimizedPages, $pageId);
                }
            }
        }
    }

    public static $stateListUSA = array(
        'AL'=>"Alabama",
        'AK'=>"Alaska",
        'AZ'=>"Arizona",
        'AR'=>"Arkansas",
        'CA'=>"California",
        'CO'=>"Colorado",
        'CT'=>"Connecticut",
        'DE'=>"Delaware",
        'DC'=>"District Of Columbia",
        'FL'=>"Florida",
        'GA'=>"Georgia",
        'HI'=>"Hawaii",
        'ID'=>"Idaho",
        'IL'=>"Illinois",
        'IN'=>"Indiana",
        'IA'=>"Iowa",
        'KS'=>"Kansas",
        'KY'=>"Kentucky",
        'LA'=>"Louisiana",
        'ME'=>"Maine",
        'MD'=>"Maryland",
        'MA'=>"Massachusetts",
        'MI'=>"Michigan",
        'MN'=>"Minnesota",
        'MS'=>"Mississippi",
        'MO'=>"Missouri",
        'MT'=>"Montana",
        'NE'=>"Nebraska",
        'NV'=>"Nevada",
        'NH'=>"New Hampshire",
        'NJ'=>"New Jersey",
        'NM'=>"New Mexico",
        'NY'=>"New York",
        'NC'=>"North Carolina",
        'ND'=>"North Dakota",
        'OH'=>"Ohio",
        'OK'=>"Oklahoma",
        'OR'=>"Oregon",
        'PA'=>"Pennsylvania",
        'RI'=>"Rhode Island",
        'SC'=>"South Carolina",
        'SD'=>"South Dakota",
        'TN'=>"Tennessee",
        'TX'=>"Texas",
        'UT'=>"Utah",
        'VT'=>"Vermont",
        'VA'=>"Virginia",
        'WA'=>"Washington",
        'WV'=>"West Virginia",
        'WI'=>"Wisconsin",
        'WY'=>"Wyoming");

    public static $stateListCanada = array(
        'AB'	=>	'Alberta',
        'BC'	=>	'British Columbia',
        'MB'	=>	'Manitoba',
        'NB'	=>	'New Brunswick',
        'NF'	=>	'Newfoundland and Labrador',
        'NT'	=>	'Northwest Territories',
        'NS'	=>	'Nova Scotia',
        'NU'	=>	'Nunavut',
        'ON'	=>	'Ontario',
        'PE'	=>	'Prince Edward Island',
        'QC'	=>	'Quebec',
        'SK'	=>	'Saskatchewan',
        'YT'	=>	'Yukon Territory'
    );

    public static $stateListAustralia = array(
        'ACT' => 'Australian Capital Territory',
        'NSW' => 'New South Wales',
        'NT' => 'Northern Territory',
        'QLD' => 'Queensland',
        'SA' => 'South Australia',
        'TAS' => 'Tasmania',
        'VIC' => 'Victoria',
        'WA' => 'Western Australia'
    );

}
