<?php
/**
 * Description of Api_Tools_System
*/
class Api_Tools_System extends Api {

	public function __construct($queue, $websiteUrl) {
		parent::__construct($queue, $websiteUrl);
	}

	public static function getDomain() {
        $url = Zend_Controller_Action_HelperBroker::getStaticHelper('website')->getUrl();
        return preg_replace('/https?:\/\//', '', $url);
    }

    public static function replaceSnippet($str, $needleStart, $needleEnd, $replacement) {
        $pos = strpos($str, $needleStart);
        $start = $pos === false ? 0 : $pos; //+ strlen($needleStart);
        $pos = strpos($str, $needleEnd, $start);
        $end = $start === false ? strlen($str) : $pos + strlen($needleEnd);
        return substr_replace($str, $replacement, $start, $end - $start);
    }

    public static function checkSnippetType($oldSnippet, $newSnippet){
        if(!empty(preg_match('/http:\/\/sa.seotoaster.com/', $oldSnippet)) || !empty(preg_match('/' . self::SEO_DATA_SAMBA_ANALYTIC_START . '/', $oldSnippet))){
            if (empty(preg_match('/' . self::SEO_DATA_SAMBA_ANALYTIC_START . '/', $oldSnippet))){
                $oldSnippet =  self::SEO_DATA_SAMBA_ANALYTIC_START . $oldSnippet . self::SEO_DATA_SAMBA_ANALYTIC_END;
            }
            if(!empty(preg_match('/' . self::SEO_DATA_SAMBA_ANALYTIC_START . '/', $newSnippet))){
                return self::replaceSnippet($oldSnippet, self::SEO_DATA_SAMBA_ANALYTIC_START, self::SEO_DATA_SAMBA_ANALYTIC_END,  $newSnippet);
            }
        }

        if (!empty(preg_match('/google-analytics.com\/ga.js/', $oldSnippet)) || !empty(preg_match('/' . self::SEO_DATA_GOOGLE_ANALYTIC_START . '/', $oldSnippet))) {
            if (empty(preg_match('/' . self::SEO_DATA_GOOGLE_ANALYTIC_START . '/', $oldSnippet))) {
                $oldSnippet = self::SEO_DATA_GOOGLE_ANALYTIC_START . $oldSnippet . self::SEO_DATA_GOOGLE_ANALYTIC_END;
            }
            if (!empty(preg_match('/' . self::SEO_DATA_GOOGLE_ANALYTIC_START . '/', $newSnippet))) {
                return self::replaceSnippet($oldSnippet, self::SEO_DATA_GOOGLE_ANALYTIC_START, self::SEO_DATA_GOOGLE_ANALYTIC_END, $newSnippet);
            }
        }

        if (empty(preg_match('/' . self::SEO_DATA_NETCONTENT_START . '/', $oldSnippet)) &&
            empty(preg_match('/' . self::SEO_DATA_GOOGLE_ANALYTIC_START . '/', $oldSnippet)) &&
            empty(preg_match('/' . self::SEO_DATA_SAMBA_ANALYTIC_START . '/', $oldSnippet))){
            $oldSnippet =  self::SEO_DATA_NETCONTENT_START . $oldSnippet . self::SEO_DATA_NETCONTENT_END;
        }
        if (!empty(preg_match('/' . self::SEO_DATA_NETCONTENT_START . '/', $oldSnippet)) && !empty(preg_match('/' .  self::SEO_DATA_NETCONTENT_START . '/', $newSnippet))) {
            return self::replaceSnippet($oldSnippet,  self::SEO_DATA_NETCONTENT_START, self::SEO_DATA_NETCONTENT_END, $newSnippet);
        }
        return $oldSnippet . $newSnippet;
    }

    public static function getRedirects($limit, $offset, $count = false) {
        $dbTable = new Zend_Db_Table();
        if(!$count) {
            $select = $dbTable->getAdapter()->select()->from('redirect', array(
                'id' => 'id',
                'pageId' => 'page_id',
                'fromUrl' => 'from_url',
                'toUrl' => 'to_url',
                'domainTo' => 'domain_to',
                'domainFrom' => 'domain_from'
            ))
                ->order(array('page_id', 'id'));
            if (!empty($offset)) {
                $select->limit($limit, $offset);
            } else {
                $select->limit($limit);
            }

            return $dbTable->getAdapter()->fetchAll($select);
        }

        return count(
            $dbTable->getAdapter()->fetchCol(
                $dbTable->getAdapter()->select()->from('redirect', array('count' => 'COUNT(id)'))->group('id')
            )
        );
    }

    public static function getUrls($limit, $offset, $count = false) {
        $dbTable = new Zend_Db_Table();
        $where = $dbTable->getAdapter()->quoteInto("system = ?", '0');
        $where .= $dbTable->getAdapter()->quoteInto(" AND draft = ?", '0');
        $where .= $dbTable->getAdapter()->quoteInto(" AND page_type NOT IN(?)", '4');

        if(!$count) {
            $select = $dbTable->getAdapter()->select()->from(array('p' => 'page'), null)
                ->joinLeft(array('o' => 'optimized'), 'p.id = o.page_id', null)
                ->columns(array(
                    'id' => 'p.id',
                    'url' => new Zend_Db_Expr('COALESCE(o.url, p.url)')
                ))->order(array('url'))
            ->where($where);

            if (!empty($offset)) {
                $select->limit($limit, $offset);
            } else {
                $select->limit($limit);
            }

            return $dbTable->getAdapter()->fetchAll($select);
        }

        return count(
            $dbTable->getAdapter()->fetchCol(
                $dbTable->getAdapter()->select()->from('page', array('count' => 'COUNT(id)'))->group('id')
                ->where($where)
            )
        );
    }

    public static function findUrls($query, $limit = 10) {
        $dbTable = new Zend_Db_Table();
        $where = $dbTable->getAdapter()->quoteInto("system = ?", '0');
        $where .= $dbTable->getAdapter()->quoteInto(" AND draft = ?", '0');
        $where .= $dbTable->getAdapter()->quoteInto(" AND page_type NOT IN(?)", '4');
        $where .= $dbTable->getAdapter()->quoteInto(" AND (COALESCE(o.url, p.url) LIKE ?)", $query . '%');

        $select = $dbTable->getAdapter()->select()->from(array('p' => 'page'), null)
            ->joinLeft(array('o' => 'optimized'), 'p.id = o.page_id', null)
            ->columns(array(
                'id' => 'p.id',
                'url' => new Zend_Db_Expr('COALESCE(o.url, p.url)')
            ))->order(array('url'))
            ->where($where)->limit($limit);

        return $dbTable->getAdapter()->fetchAll($select);
    }

}
