<?php

class Api_Crm_Auth extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * System session helper
     *
     * @var null
     */
    protected $_sessionHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();

    /**
     * @var array
     */
    public static $_errorCodes = array(
        '1' => 'Please specify login',
        '2' => 'Please specify password',
        '3' => 'Please check you credentials',
    );


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('post')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('post')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('post')
        ),
        Tools_Security_Acl::ROLE_GUEST => array(
            'allow' => array('post')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_sessionHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
    }


    /**
     *
     * Resource:
     * : /api/crm/auth/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

    }

    /**
     *
     * Resource:
     * : /api/crm/auth/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        $login = $this->_request->getParam('login');
        $password = $this->_request->getParam('password');
        if (empty($login)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes['1']));
        }

        if (empty($password)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes['2']));
        }

        $token = Tools_LeadExternalApiTools::authorize($login, $password);
        if (empty($token)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes['3']), self::REST_STATUS_UNAUTHORIZED);
        }

        return array('status' => true, 'token' => $token);
    }

    /**
     *
     * Resource:
     * : /api/crm/auth/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     *
     * @return JSON
     */
    public function putAction()
    {

    }

    /**
     * Resource:
     * : /api/crm/auth/
     *
     * HttpMethod:
     * : DELETE
     *
     *
     * @return JSON
     */
    public function deleteAction()
    {

    }

}
