<?php

class Api_Crm_Leadcustomparams extends Api_Leads_Leadcustomparams
{

    /**
     * @var array
     */
    public static $_errorCodes = array(
        '1' => 'Access denied',
        '2' => 'Missing lead id',
        '3' => 'Lead not found',
        '4' => 'Record not found',
        '5' => 'id is missing',
        '6' => 'Custom param id is missing',
        '7' => 'Custom params option id not found'
    );

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('param_id', 'lead_id', 'secureToken');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_GUEST => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $translator = Zend_Registry::get('Zend_Translate');
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $authHeader = $this->_request->getHeader('Authorization');
        if (empty($authHeader)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[1]), self::REST_STATUS_UNAUTHORIZED);
        }
        $token = Tools_LeadExternalApiTools::verify($authHeader);
        if ($token === false) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[1]), self::REST_STATUS_UNAUTHORIZED);
        }
    }

    /**
     *
     * Resource:
     * : /api/crm/leadcustomparams/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order'), FILTER_SANITIZE_STRING);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $leadId = filter_var($this->_request->getParam('leadId'), FILTER_SANITIZE_NUMBER_INT);
        $grid = filter_var($this->_request->getParam('grid'), FILTER_SANITIZE_STRING);
        $leadCustomTabId = filter_var($this->_request->getParam('customTabId'), FILTER_SANITIZE_NUMBER_INT);

        Tools_LeadExternalApiTools::logRequest($this->_request->getHeader('Authorization'),
            $this->_request->getParams(), Tools_LeadExternalApiTools::LEAD_CUSTOM_PARAMS_ENDPOINT, 'GET');

        $leadCustomParamMapper = Leads_Mapper_LeadsCustomParamsMapper::getInstance();
        if ($id) {
            $where = $leadCustomParamMapper->getDbTable()->getAdapter()->quoteInto('pllccpd.id = ?', $id);
            $data = $leadCustomParamMapper->fetchAll($where);
        } elseif(!empty($grid) && !empty($leadId)) {
            $configData = $leadCustomParamMapper->fetchAllData(null, $sortOrder, $limit, $offset);
            $paramsData = $leadCustomParamMapper->getParamsForGrid($leadId);

            if(!empty($leadCustomTabId)) {
                $leadsCustomTabsHasCustomParamsMapper = Leads_Mapper_LeadsCustomTabsHasCustomParamsMapper::getInstance();
                $selectedCustomParamsData = $leadsCustomTabsHasCustomParamsMapper->getSelectedCustomTabsParams($leadCustomTabId, Api_Leads_Leadcustomtabsfields::CUSTOM_TAB_TYPE_LEAD);
            }

            if (!empty($configData['data'])) {
                foreach ($configData['data'] as $key => $confData) {
                    $paramId = $confData['param_id'];

                    if(!empty($leadCustomTabId)) {
                        $configData['data'][$key]['visible'] = false;
                        if(!empty($selectedCustomParamsData)) {
                            if(array_key_exists($paramId, $selectedCustomParamsData)) {
                                $configData['data'][$key]['visible'] = true;
                            }
                        }
                    }

                    if (array_key_exists($paramId, $paramsData)) {
                        $configData['data'][$key]['param_value'] = $paramsData[$paramId]['param_value'];
                        $configData['data'][$key]['id'] =  $paramsData[$paramId]['id'];
                    } else {
                        $configData['data'][$key]['param_value'] = null;
                        $configData['data'][$key]['id'] =  null;
                    }
                }
                $data = $configData;
            } else {
                $data = array();
            }

        } else {
            $data = $leadCustomParamMapper->fetchAll(null, $sortOrder, $limit, $offset);
        }

        return $data;
    }

    /**
     *
     * Resource:
     * : /api/crm/leadcustomparams/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if(empty($data)) {
            $data = $this->getRequest()->getParams();
        }
        $translator = Zend_Registry::get('Zend_Translate');

        $data = Tools_LeadTools::camelCaseKeysToUnderscore($data, true);

        if (empty($data['lead_id'])) {
            return array(
                'status' => false,
                'errorCode' => '2',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[2])
            );
        }

        if (empty($data['param_id'])) {
            return array(
                'status' => false,
                'errorCode' => '6',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[6])
            );
        }

        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
        $leadsModel = $leadsMapper->find($data['lead_id']);

        if (!$leadsModel instanceof Leads_Model_LeadsModel) {
            return array(
                'status' => false,
                'errorCode' => '3',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[3])
            );
        }

        if($data['param_type'] == Tools_LeadImportTools::ATTRIBUTE_TYPE_DATE){
            $defaultTimezone = 'UTC';
            $currentTime = Tools_System_Tools::convertDateFromTimezone('now', $defaultTimezone, Tools_System_Tools::getUserTimezone(), 'H:i:s');
            $formDate = Tools_System_Tools::convertDateFromTimezone($data['param_value'], $defaultTimezone, Tools_System_Tools::getUserTimezone(), 'Y-m-d ') . $currentTime;

            $data['param_value'] = $formDate;
        }

        if($data['param_type'] == Tools_LeadImportTools::ATTRIBUTE_TYPE_SELECT || $data['param_type'] == Tools_LeadImportTools::ATTRIBUTE_TYPE_CHECKBOX || $data['param_type'] == Tools_LeadImportTools::ATTRIBUTE_TYPE_RADIO){
            $leadCustomParamOptionsDataMapper = Leads_Mapper_LeadsCustomParamsOptionsDataMapper::getInstance();

            $leadCustomParamOptionsData = $leadCustomParamOptionsDataMapper->findOptionIdByCustomParamId($data['param_id'], $data['param_value']);

            if($leadCustomParamOptionsData instanceof Leads_Model_LeadsCustomParamsOptionsDataModel) {
                $data['params_option_id'] = $leadCustomParamOptionsData->getId();
            } else {
                return array(
                    'status' => false,
                    'errorCode' => '7',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[7])
                );
            }
        }

        $leadCustomParamMapper = Leads_Mapper_LeadsCustomParamsMapper::getInstance();

        $leadCustomParamModel = new Leads_Model_LeadsCustomParamsModel();
        $leadCustomParamModel->setOptions($data);

        Tools_LeadExternalApiTools::logRequest($this->_request->getHeader('Authorization'),
            $this->_request->getParams(), Tools_LeadExternalApiTools::LEAD_CUSTOM_PARAMS_ENDPOINT, 'POST');

        $leadCustomParamModel = $leadCustomParamMapper->save($leadCustomParamModel);

        return array(
            'status' => true,
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead custom param has been added'),
            'customParamData' => $leadCustomParamModel->toArray()
        );

    }

    /**
     *
     * Resource:
     * : /api/crm/leadcustomparams/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     * : lead custom param id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if(empty($data)) {
            $data = $this->getRequest()->getParams();
        }
        $translator = Zend_Registry::get('Zend_Translate');

        if (!empty($data['id'])) {
            $data = Tools_LeadTools::camelCaseKeysToUnderscore($data, true);

            $customParamId = filter_var($data['id'], FILTER_SANITIZE_NUMBER_INT);
            $leadCustomParamMapper = Leads_Mapper_LeadsCustomParamsMapper::getInstance();
            $leadCustomParamModel = $leadCustomParamMapper->find($customParamId);

            if (!$leadCustomParamModel instanceof Leads_Model_LeadsCustomParamsModel) {
                return array(
                    'status' => false,
                    'errorCode' => '4',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[4])
                );
            }

            if (empty($data['param_id'])) {
                return array(
                    'status' => false,
                    'errorCode' => '6',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[6])
                );
            }

            if($data['param_type'] == Tools_LeadImportTools::ATTRIBUTE_TYPE_DATE){
                $defaultTimezone = 'UTC';
                $currentTime = Tools_System_Tools::convertDateFromTimezone('now', $defaultTimezone, Tools_System_Tools::getUserTimezone(), 'H:i:s');
                $formDate = Tools_System_Tools::convertDateFromTimezone($data['param_value'], $defaultTimezone, Tools_System_Tools::getUserTimezone(), 'Y-m-d ') . $currentTime;

                $data['param_value'] = $formDate;
            }

            if($data['param_type'] == Tools_LeadImportTools::ATTRIBUTE_TYPE_SELECT || $data['param_type'] == Tools_LeadImportTools::ATTRIBUTE_TYPE_CHECKBOX || $data['param_type'] == Tools_LeadImportTools::ATTRIBUTE_TYPE_RADIO){
                $leadCustomParamOptionsDataMapper = Leads_Mapper_LeadsCustomParamsOptionsDataMapper::getInstance();

                $leadCustomParamOptionsData = $leadCustomParamOptionsDataMapper->findOptionIdByCustomParamId($data['param_id'], $data['param_value']);

                if($leadCustomParamOptionsData instanceof Leads_Model_LeadsCustomParamsOptionsDataModel){
                    $data['params_option_id'] = $leadCustomParamOptionsData->getId();
                }else{
                    $leadCustomParamModel->setParamsOptionId(null);
                }
            }

            $leadCustomParamModel->setOptions($data);
            $leadCustomParamMapper->save($leadCustomParamModel);

            Tools_LeadExternalApiTools::logRequest($this->_request->getHeader('Authorization'),
                $this->_request->getRawBody(), Tools_LeadExternalApiTools::LEAD_CUSTOM_PARAMS_ENDPOINT, 'PUT');

            return array(
                'status' => true,
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead custom param has been updated'),
                'customParamData' => $leadCustomParamModel->toArray()
            );
        }

        return array(
            'status' => false,
            'errorCode' => '5',
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[5])
        );


    }

    /**
     *
     * Resource:
     * : /api/crm/leadcustomparams/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     * : lead custom param id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $translator = Zend_Registry::get('Zend_Translate');

        if (!$id) {
            return array(
                'status' => false,
                'errorCode' => '5',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[5])
            );
        }

        $leadCustomParamMapper = Leads_Mapper_LeadsCustomParamsMapper::getInstance();
        $leadCustomParamModel = $leadCustomParamMapper->find($id);
        if ($leadCustomParamModel instanceof Leads_Model_LeadsCustomParamsModel) {
            $leadCustomParamMapper->delete($id);

            Tools_LeadExternalApiTools::logRequest($this->_request->getHeader('Authorization'),
                $this->_request->getParams(), Tools_LeadExternalApiTools::LEAD_CUSTOM_PARAMS_ENDPOINT, 'DELETE');

            return array(
                'status' => true,
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead custom param has been deleted.')
            );
        }

        return array(
            'status' => false,
            'errorCode' => '4',
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[4])
        );
    }

}
