<?php

class Api_Crm_Leadcustomparamsconfig extends Api_Service_Abstract
{
    /**
     * @var array
     */
    public static $_errorCodes = array(
        '1' => 'Access denied'
    );

    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_GUEST => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $translator = Zend_Registry::get('Zend_Translate');
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $authHeader = $this->_request->getHeader('Authorization');
        if (empty($authHeader)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[1]), self::REST_STATUS_UNAUTHORIZED);
        }
        $token = Tools_LeadExternalApiTools::verify($authHeader);
        if ($token === false) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[1]), self::REST_STATUS_UNAUTHORIZED);
        }
    }


    /**
     *
     * Resource:
     * : /api/crm/leadcustomparamsconfig/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order', 'pllcpc.param_name'), FILTER_SANITIZE_STRING);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);

        Tools_LeadExternalApiTools::logRequest($this->_request->getHeader('Authorization'),
            $this->_request->getParams(), Tools_LeadExternalApiTools::LEAD_CUSTOM_PARAMS_CONFIG_ENDPOINT, 'GET');

        $leadCustomParamConfigMapper = Leads_Mapper_LeadsCustomParamsConfigMapper::getInstance();
        if ($id) {
            $where = $leadCustomParamConfigMapper->getDbTable()->getAdapter()->quoteInto('pllcpc.id = ?', $id);
            $data = $leadCustomParamConfigMapper->fetchAll($where);
        } else {
            $data = $leadCustomParamConfigMapper->fetchAll(null, $sortOrder, $limit, $offset);
        }

        if (empty($data)) {
            $data = array();
        } else {
            foreach ($data['data'] as $key => $datum) {
                if(!is_null($datum['option_values']) && !is_null($datum['option_ids'])) {
                    $optionValues = explode(',', $datum['option_values']);
                    $optionIds = explode(',', $datum['option_ids']);

                    $data['data'][$key]['option_values'] = $optionValues;
                    $data['data'][$key]['option_ids'] = $optionIds;

                    $data['data'][$key]['options'] = array_combine($optionIds, $optionValues);
                }
            }

            $data['status'] = true;
        }

        return $data;
    }

    /**
     *
     * Resource:
     * : /api/crm/leadcustomparamsconfig/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {

    }

    /**
     *
     * Resource:
     * : /api/crm/leadcustomparamsconfig/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     * : lead custom param id to update
     *
     * @return JSON
     */
    public function putAction()
    {

    }

    /**
     *
     * Resource:
     * : /api/crm/leadcustomparamsconfig/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     * : lead custom param id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {

    }

}
