<?php

class Api_Crm_Leadnotes extends Api_Leads_Leadnotes
{

    /**
     * @var array
     */
    public static $_errorCodes = array(
        '1' => 'Access denied',
        '2' => 'Missing lead id',
        '3' => 'Lead not found',
        '4' => 'Record not found',
        '5' => 'id is missing',
        '6' => 'Note is missing'
    );

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('note');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_GUEST => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )

    );

    public function init()
    {
        parent::init();
        $translator = Zend_Registry::get('Zend_Translate');
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $authHeader = $this->_request->getHeader('Authorization');
        if (empty($authHeader)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[1]), self::REST_STATUS_UNAUTHORIZED);
        }
        $token = Tools_LeadExternalApiTools::verify($authHeader);
        if ($token === false) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[1]), self::REST_STATUS_UNAUTHORIZED);
        }

    }


    /**
     *
     * Resource:
     * : /api/crm/leadnotes/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order', 'pln.date_created DESC'), FILTER_SANITIZE_STRING);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $leadId = filter_var($this->_request->getParam('leadId', false), FILTER_SANITIZE_NUMBER_INT);

        $leadNotesDataMapper = Leads_Mapper_LeadsNoteMapper::getInstance();

        Tools_LeadExternalApiTools::logRequest($this->_request->getHeader('Authorization'),
            $this->_request->getParams(), Tools_LeadExternalApiTools::LEAD_NOTES_ENDPOINT, 'GET');

        if (!empty($leadId)) {
            $whereLeadId = $leadNotesDataMapper->getDbTable()->getAdapter()->quoteInto('pln.lead_id = ?',
                    $leadId);
        }

        if ($id) {
            $where = $leadNotesDataMapper->getDbTable()->getAdapter()->quoteInto('pln.id = ?', $id);
            if (!empty($whereLeadId)) {
                $where .= ' AND  ' . $whereLeadId;
            }
            $data = $leadNotesDataMapper->fetchAllData($where);
        } else {
            $where = null;
            if (!empty($whereLeadId)) {
                $where = $whereLeadId;
            }
            $data = $leadNotesDataMapper->fetchAllData($where, $sortOrder, $limit, $offset, true);
        }

        return $data;
    }

    /**
     *
     * Resource:
     * : /api/crm/leadnotes/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if(empty($data)) {
            $data = $this->getRequest()->getParams();
        }
        $translator = Zend_Registry::get('Zend_Translate');
        $data = Tools_LeadTools::camelCaseKeysToUnderscore($data, true);
        if (empty($data['lead_id'])) {
            return array(
                'status' => false,
                'errorCode' => '2',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[2])
            );
        }

        if (empty($data['note'])) {
            return array(
                'status' => false,
                'errorCode' => '6',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[6])
            );
        }

        $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
        $leadModel = $leadMapper->find($data['lead_id']);
        if (!$leadModel instanceof Leads_Model_LeadsModel) {
            return array(
                'status' => false,
                'errorCode' => '3',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[3])
            );
        }

        $leadsUsersApiTokensMapper = Leads_Mapper_LeadsUsersApiTokensMapper::getInstance();
        $tokenModel = $leadsUsersApiTokensMapper->findByToken($this->_request->getHeader('Authorization'));
        $apiUserId = $tokenModel->getApiUserId();
        $leadsUsersApiAccessModel =  Leads_Mapper_LeadsUsersApiAccessMapper::getInstance()->find($apiUserId);
        $userId = $leadsUsersApiAccessModel->getUserId();
        $data['user_id'] = $userId;

        $leadsNotesMapper = Leads_Mapper_LeadsNoteMapper::getInstance();
        $leadsNoteModel = new Leads_Model_LeadsNoteModel();
        $leadsNoteModel->setOptions($data);
        $leadsNoteModel->setDateCreated(Tools_System_Tools::convertDateFromTimezone('now'));

        Tools_LeadExternalApiTools::logRequest($this->_request->getHeader('Authorization'),
            $this->_request->getParams(), Tools_LeadExternalApiTools::LEAD_NOTES_ENDPOINT, 'POST');

        $leadsNotesMapper->save($leadsNoteModel);

        return array(
            'status' => true,
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Note has been added'),
            'notesData' => $leadsNoteModel->toArray()
        );

    }

    /**
     *
     * Resource:
     * : /api/crm/leadnotes/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     * : lead note id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if(empty($data)) {
            $data = $this->getRequest()->getParams();
        }
        $translator = Zend_Registry::get('Zend_Translate');
        if (!empty($data['id'])) {
            $data = Tools_LeadTools::camelCaseKeysToUnderscore($data, true);
            $id = filter_var($data['id'], FILTER_SANITIZE_NUMBER_INT);
            $leadNoteMapper = Leads_Mapper_LeadsNoteMapper::getInstance();
            $leadNoteModel = $leadNoteMapper->find($id);
            if (!$leadNoteModel instanceof Leads_Model_LeadsNoteModel) {
                return array(
                    'status' => false,
                    'errorCode' => '4',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[4])
                );
            }

            if (empty($data['note'])) {
                return array(
                    'status' => false,
                    'errorCode' => '6',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[6])
                );
            }

            $leadNoteModel->setOptions($data);
            $leadNoteMapper->save($leadNoteModel);

            Tools_LeadExternalApiTools::logRequest($this->_request->getHeader('Authorization'),
                $this->_request->getRawBody(), Tools_LeadExternalApiTools::LEAD_NOTES_ENDPOINT, 'PUT');

            return array(
                'status' => true,
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Note has been updated'),
                'notesData' => $leadNoteModel->toArray()
            );
        }

        return array(
            'status' => false,
            'errorCode' => '5',
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[5])
        );

    }

    /**
     * Delete lead stage
     *
     * Resource:
     * : /api/leads/leadnotes/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     * : lead note id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $translator = Zend_Registry::get('Zend_Translate');
        if (!$id) {
            return array(
                'status' => false,
                'errorCode' => '5',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[5])
            );
        }
        $leadsNoteMapper = Leads_Mapper_LeadsNoteMapper::getInstance();
        $leadNoteModel = $leadsNoteMapper->find($id);
        if ($leadNoteModel instanceof Leads_Model_LeadsNoteModel) {
            $leadsNoteMapper->delete($id);

            Tools_LeadExternalApiTools::logRequest($this->_request->getHeader('Authorization'),
                $this->_request->getParams(), Tools_LeadExternalApiTools::LEAD_NOTES_ENDPOINT, 'DELETE');

            return array(
                'status' => true,
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Note has been deleted.')
            );
        }

        return array(
            'status' => false,
            'errorCode' => '4',
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[4])
        );

    }

}
