<?php

class Api_Crm_Leadorganization extends Api_Leads_Leadorganization
{


    /**
     * @var array
     */
    public static $_errorCodes = array(
        '1' => 'Access denied',
        '2' => 'customerGroupId is invalid',
        '3' => 'Organization name already in use',
        '4' => 'Missing organization name',
        '5' => 'Organization not found',
        '6' => 'Another organization has such name',
        '7' => 'industryId doesn\'t exist',
        '8' => 'id is missing',
        '12' => 'Organization not found'
    );


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('organization_name');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_GUEST => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $authHeader = $this->_request->getHeader('Authorization');
        $translator = Zend_Registry::get('Zend_Translate');
        if (empty($authHeader)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[1]), self::REST_STATUS_UNAUTHORIZED);
        }
        $token = Tools_LeadExternalApiTools::verify($authHeader);
        if ($token === false) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[1]), self::REST_STATUS_UNAUTHORIZED);
        }

    }


    /**
     * Get lead organization by id
     *
     * Resource:
     * : /api/crm/leadorganization/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order', 'plo.created_at DESC'), FILTER_SANITIZE_STRING);
        $forGrid = (bool)filter_var($this->_request->getParam('isGrid', false), FILTER_SANITIZE_STRING);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $searchTerm = filter_var($this->_request->getParam('searchTerm'), FILTER_SANITIZE_STRING);
        $detailedInfo = filter_var($this->_request->getParam('detailedInfo'), FILTER_SANITIZE_STRING);
        $withOpportunityAmount = false;
        $withSpentPurchaseAmount = false;
        $withNextActivityDate = false;
        $having = null;
        $withLeadsData = false;
        $includesCustomOrganizationFields = false;

        Tools_LeadExternalApiTools::logRequest($this->_request->getHeader('Authorization'),
            $this->_request->getParams(), Tools_LeadExternalApiTools::LEAD_ORGANIZATIONS_ENDPOINT, 'GET');

        $leadOrganizationDataMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
        if ($id && $forGrid) {
            $where = $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.id = ?', $id);
            $data = $leadOrganizationDataMapper->fetchAll($where, null, null, null, true, true);
        } elseif ($id) {
            $data = $leadOrganizationDataMapper->find($id);
            if ($data instanceof Leads_Model_LeadsOrganizationModel) {
                $data = $data->toArray();
            } else {
                $data = array();
            }
        } else {
            $filterParams = filter_var_array($this->_request->getParam('filter', array()), FILTER_SANITIZE_STRING);
            $where = null;
            if (!empty($filterParams)) {
                $filterParams['lead-organization-attribute'] = str_replace('*-amp-*', '&', $filterParams['lead-organization-attribute']);
                $where = '';
                if (!empty($filterParams['lead-organization-attribute'])) {
                    $filterParams['lead-organization-attribute'] = trim($filterParams['lead-organization-attribute'],
                        " \t\n\r\x0B");
                    if (strpos($filterParams['lead-organization-attribute'], '*') !== false) {
                        $attributeValue = str_replace('*', '%', $filterParams['lead-organization-attribute']);
                        $where .= ' (' . $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_name LIKE ?',
                                $attributeValue . '%');
                    } else {
                        $where .= ' (' . $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_name LIKE ?',
                                '%' . $filterParams['lead-organization-attribute'] . '%');
                    }

                    if (!empty($filterParams['search_includes_notes'])) {
                        if (strpos($filterParams['lead-organization-attribute'], '*') !== false) {
                            $attributeValue = str_replace('*', '%', $filterParams['lead-organization-attribute']);
                            $where .= ' OR ' . $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_notes LIKE ?',
                                    $attributeValue . '%');
                        } else {
                            $where .= ' OR ' . $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_notes LIKE ?',
                                    '%' . $filterParams['lead-organization-attribute'] . '%');
                        }
                    }

                    if (!empty($filterParams['search_includes_custom_organization_fields'])) {
                        $includesCustomOrganizationFields = true;
                        if (strpos($filterParams['lead-organization-attribute'], '*') !== false) {
                            $attributeValue = str_replace('*', '%', $filterParams['lead-organization-attribute']);
                            $where .= ' OR ' . $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plocpd.param_value LIKE ?',
                                    $attributeValue . '%');
                        } else {
                            $where .= ' OR ' . $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plocpd.param_value LIKE ?',
                                    '%' . $filterParams['lead-organization-attribute'] . '%');
                        }
                    }
                    $where .= ')';
                }

                if (!empty($filterParams['organization_zip'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $organizationZips = explode(',', $filterParams['organization_zip']);
                    $organizationZips = array_diff($organizationZips, array(''));
                    $organizationZips = array_map('trim', $organizationZips);

                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_zip IN (?)', $organizationZips);
                }

                if (!empty($filterParams['organization_city'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_city = ?',
                        $filterParams['organization_city']);
                }

                if (!empty($filterParams['organization_country'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_country_code = ?',
                        $filterParams['organization_country']);
                }

                if (!empty($filterParams['organization_country_state']) && is_array($filterParams['organization_country_state'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_state_code IN (?)',
                        $filterParams['organization_country_state']);
                }

                if (!empty($filterParams['created_at_from'])) {
                    $createdAt = Tools_System_Tools::convertDateFromTimezone($filterParams['created_at_from']);
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    if (!empty($filterParams['last_updated']) && $filterParams['last_updated'] === 'last_updated') {
                        $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.updated_at >= ?',
                            $createdAt);
                    } else {
                        $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.created_at >= ?',
                            $createdAt);
                    }

                }
                if (!empty($filterParams['created_at_to'])) {
                    $updatedAt = Tools_System_Tools::convertDateFromTimezone($filterParams['created_at_to'] . ' 23:59:59');
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    if (!empty($filterParams['last_updated']) && $filterParams['last_updated'] === 'last_updated') {
                        $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.updated_at <= ?',
                            $updatedAt);
                    } else {
                        $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.created_at <= ?',
                            $updatedAt);
                    }

                }

                if (!empty($filterParams['customer_group_id'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.customer_group_id = ?',
                        $filterParams['customer_group_id']);
                }

                if (!empty($filterParams['industry_id'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_industry_id = ?',
                        $filterParams['industry_id']);
                }

                if (!empty($filterParams['organization_domain'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_domain = ?',
                        $filterParams['organization_domain']);
                }

                if (!empty($filterParams['organization_domain1'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_domain1 = ?',
                        $filterParams['organization_domain1']);
                }

                if (!empty($filterParams['last_interaction_id'])) {
                    $withLeadsData = true;
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('pld.last_activity_type_id = ?',
                        $filterParams['last_interaction_id']);
                }

                if (!empty($filterParams['total_opportunity_amount_from']) || !empty($filterParams['total_opportunity_amount_to']) || !empty($filterParams['lead_opportunity_stages']) || !empty($filterParams['opportunity_type_id'])) {
                    $withOpportunityAmount = true;
                }

                if (!empty($filterParams['total_spent_purchase_amount_from']) || !empty($filterParams['total_spent_purchase_amount_to']) || !empty($filterParams['special_filter_type']) && $filterParams['special_filter_type'] === Api_Leads_Leads::FILTER_EXISTING_CLIENT) {
                    $withSpentPurchaseAmount = true;

                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('scs.status IN (?)',
                        array(
                            Models_Model_CartSession::CART_STATUS_COMPLETED,
                            Models_Model_CartSession::CART_STATUS_SHIPPED,
                            Models_Model_CartSession::CART_STATUS_DELIVERED
                        ));
                }

                if ($filterParams['special_filter_type'] === Api_Leads_Leads::FILTER_PROSPECT) {
                    $withSpentPurchaseAmount = true;

                }

                if (!empty($filterParams['lead_opportunity_stages'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plop.opportunity_stage_id IN (?)',
                        $filterParams['lead_opportunity_stages']);
                }

                if (!empty($filterParams['lead_score_from'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('pld.score >= ?',
                        (int)$filterParams['lead_score_from']);
                    $withLeadsData = true;
                }

                if (!empty($filterParams['lead_score_to'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('pld.score <= ?',
                        (int)$filterParams['lead_score_to']);
                    $withLeadsData = true;
                }

                if (!empty($filterParams['special_filter_type'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    if ($filterParams['special_filter_type'] === Api_Leads_Leads::FILTER_EXISTING_CLIENT) {
                        $where .= new Zend_Db_Expr('pld.user_id IS NOT NULL');
                    }
                    if ($filterParams['special_filter_type'] === Api_Leads_Leads::FILTER_PROSPECT) {
                        $where .= new Zend_Db_Expr('pld.user_id IS NULL');
                        $where .= ' AND ' . new Zend_Db_Expr('pld.id IS NOT NULL');
                    }
                    if ($filterParams['special_filter_type'] === Api_Leads_Leads::FILTER_NO_EMAIL) {
                        $withLeadsData = true;
                        $where .= new Zend_Db_Expr('pld.lead_email IS NULL');
                        $where .= ' AND ' . new Zend_Db_Expr('pld.id IS NOT NULL');
                    }
                }

                if (!empty($filterParams['opportunity_type_id'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plop.opportunity_type_id = ?',
                        $filterParams['opportunity_type_id']);
                }

                if (!empty($filterParams['total_opportunity_amount_from'])) {
                    if (!empty($having)) {
                        $having .= ' AND ';
                    }
                    $having .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('SUM(plop.opportunity_value) >= ?',
                        $filterParams['total_opportunity_amount_from']);
                }
                if (!empty($filterParams['total_opportunity_amount_to'])) {
                    if (!empty($having)) {
                        $having .= ' AND ';
                    }
                    $having .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('SUM(plop.opportunity_value) <= ?',
                        $filterParams['total_opportunity_amount_to']);
                }

                if (!empty($filterParams['total_spent_purchase_amount_from'])) {
                    if (!empty($having)) {
                        $having .= ' AND ';
                    }
                    $having .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('SUM(scs.total) >= ?',
                        $filterParams['total_spent_purchase_amount_from']);
                }

                if (!empty($filterParams['total_spent_purchase_amount_to'])) {
                    if (!empty($having)) {
                        $having .= ' AND ';
                    }
                    $having .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('SUM(scs.total) <= ?',
                        $filterParams['total_spent_purchase_amount_to']);
                }

            } elseif (!empty($searchTerm)) {
                $where = $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_name LIKE ?',
                    $searchTerm . '%');
            }


            $data = $leadOrganizationDataMapper->fetchAll($where, $sortOrder, $limit, $offset, false, false,
                    $withOpportunityAmount, $having, $withNextActivityDate, $withLeadsData,
                    $includesCustomOrganizationFields, $withSpentPurchaseAmount);

        }

         return $data;
    }

    /**
     * Create new lead organization
     *
     * Resource:
     * : /api/crm/leadorganization/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if(empty($data)) {
            $data = $this->getRequest()->getParams();
        }
        $translator = Zend_Registry::get('Zend_Translate');
        $data = Tools_LeadTools::camelCaseKeysToUnderscore($data, true);

        if (empty($data['organization_name'])) {
            return array(
                'status' => false,
                'errorCode' => '4',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[4])
            );
        }

        if (!empty($data['organization_mobile'])) {
            $data['organization_mobile'] = Tools_LeadTools::cleanPhoneAddress($data['organization_mobile']);
        }

        if (!empty($data['organization_phone'])) {
            $data['organization_phone'] = Tools_LeadTools::cleanPhoneAddress($data['organization_phone']);
        }

        if (empty($data['customer_group_id'])) {
            $data['customer_group_id'] = null;
        }

        if (empty($data['organization_industry_id'])) {
            $data['organization_industry_id'] = null;
        } else {
            $organizationIndustryModel = Leads_Mapper_LeadsIndustryMapper::getInstance()->find($data['organization_industry_id']);
            if(!$organizationIndustryModel instanceof Leads_Model_LeadsIndustryModel) {
                return array(
                    'status' => false,
                    'errorCode' => '7',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[7])
                );
            }
        }

        if (empty($data['organization_country_code'])) {
            $data['organization_country_code'] = null;
        }

        $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
        $userDefaultMobileCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');
        if (empty($data['organization_phone_country_code']) && !empty($userDefaultMobileCountryCode)) {
            $data['organization_phone_country_code'] = $userDefaultMobileCountryCode;
        }

        if (empty($data['organization_mobile_country_code']) && !empty($userDefaultMobileCountryCode)) {
            $data['organization_mobile_country_code'] = $userDefaultMobileCountryCode;
        }

        if (empty($data['primary_contact_id'])) {
            $data['primary_contact_id'] = null;
        }

        if (!empty($data['organization_website'])) {
            $data['organization_website'] = Tools_LeadTools::prepareWebsiteUrl($data['organization_website']);
            $domain = Tools_LeadTools::getDomainFromUrl($data['organization_website']);
            if (!empty($domain)) {
                $data['organization_domain'] = $domain;
            } else {
                $data['organization_domain'] = '';
            }
        } else {
            $data['organization_domain'] = '';
        }

        if (!empty($data['organization_website1'])) {
            $data['organization_website1'] = Tools_LeadTools::prepareWebsiteUrl($data['organization_website1']);
            $domain = Tools_LeadTools::getDomainFromUrl($data['organization_website1']);
            if (!empty($domain)) {
                $data['organization_domain1'] = $domain;
            } else {
                $data['organization_domain1'] = '';
            }
        } else {
            $data['organization_domain1'] = '';
        }

        $leadsOrganizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
        $leadsOrganizationModel = new Leads_Model_LeadsOrganizationModel();
        $leadsOrganizationModel->setOptions($data);

        $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();
        if (!empty($shoppingConfig['country'])) {
            $defaultCountryCode = $shoppingConfig['country'];
            if (empty($data['organization_country_code'])) {
                $leadsOrganizationModel->setOrganizationCountryCode($defaultCountryCode);
            }
        }

        if (!empty($data['customer_group_id'])) {
            $groupId = filter_var($data['customer_group_id'], FILTER_SANITIZE_NUMBER_INT);
            if (!empty($groupId)) {
                $validator = new Zend_Validate_Db_NoRecordExists(array(
                    'table' => 'shopping_group',
                    'field' => 'id'
                ));

                if ($validator->isValid($groupId)) {
                    return array(
                        'status' => false,
                        'errorCode' => '2',
                        'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[2])
                    );
                }
            }
        }

        $validator = new Zend_Validate_Db_NoRecordExists(array(
            'table' => 'plugin_lead_organization',
            'field' => 'organization_name'
        ));

        if (!$validator->isValid($data['organization_name'])) {
            return array(
                'status' => false,
                'errorCode' => '3',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[3])
            );
        }

        $createdAt = Tools_System_Tools::convertDateFromTimezone('now');

        $leadsOrganizationModel->setCreatedAt($createdAt);
        $leadsOrganizationModel = $leadsOrganizationMapper->save($leadsOrganizationModel);

        Tools_LeadExternalApiTools::logRequest($this->_request->getHeader('Authorization'),
            $this->_request->getParams(), Tools_LeadExternalApiTools::LEAD_ORGANIZATIONS_ENDPOINT, 'POST');

        $orgData = $leadsOrganizationModel->toArray();
        unset($orgData['organizationUrl']);

        return array(
            'status' => true,
            'organizationInfo' => $orgData,
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Organization has been created')
        );
    }

    /**
     * Update lead organization data
     *
     * Resource:
     * : /api/crm/leadorganization/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     * : lead organization id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if(empty($data)) {
            $data = $this->getRequest()->getParams();
        }
        if (!empty($data['id'])) {
            $data = Tools_LeadTools::camelCaseKeysToUnderscore($data, true);
            $translator = Zend_Registry::get('Zend_Translate');
            $organizationId = filter_var($data['id'], FILTER_SANITIZE_NUMBER_INT);
            $leadOrganizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
            $leadOrganizationModel = $leadOrganizationMapper->find($organizationId);
            if (!$leadOrganizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                return array(
                    'status' => false,
                    'errorCode' => '5',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[5])
                );
            }

            $oldOrganizationName = $leadOrganizationModel->getOrganizationName();
            $currentOrganizationName = $data['organization_name'];

            if (!empty($data['organization_mobile'])) {
                $data['organization_mobile'] = Tools_LeadTools::cleanPhoneAddress($data['organization_mobile']);
            }

            if (!empty($data['organization_phone'])) {
                $data['organization_phone'] = Tools_LeadTools::cleanPhoneAddress($data['organization_phone']);
            }

            if (empty($data['customer_group_id'])) {
                $data['customer_group_id'] = null;
            }

            if (empty($data['organization_industry_id'])) {
                $data['organization_industry_id'] = null;
            } else {
                $organizationIndustryModel = Leads_Mapper_LeadsIndustryMapper::getInstance()->find($data['organization_industry_id']);
                if(!$organizationIndustryModel instanceof Leads_Model_LeadsIndustryModel) {
                    return array(
                        'status' => false,
                        'errorCode' => '7',
                        'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[7])
                    );
                }
            }

            $clearStateCode = false;

            if (empty($data['organization_country_code'])) {
                $data['organization_country_code'] = null;
            } else {
                $stateCodes = Tools_LeadTools::getStates($data['organization_country_code'], true, true);
                if (!empty($stateCodes) && empty($data['organization_state_code'])) {
                    $stateCodesKeys = array_keys($stateCodes);
                    $data['organization_state_code'] = $stateCodesKeys[0];
                } elseif (empty($stateCodes)) {
                    $clearStateCode = true;
                }
            }

            if (empty($data['organization_state_code'])) {
                $data['organization_state_code'] = null;
            } elseif ($clearStateCode) {
                $data['organization_state_code'] = null;
            } elseif (!empty($stateCodes) && !empty($data['organization_state_code']) && empty($stateCodes[$data['organization_state_code']])) {
                $stateCodesKeys = array_keys($stateCodes);
                $data['organization_state_code'] = $stateCodesKeys[0];
            }

            if (empty($data['primary_contact_id'])) {
                $data['primary_contact_id'] = null;
            }

            if (!empty($data['organization_website'])) {
                $data['organization_website'] = Tools_LeadTools::prepareWebsiteUrl($data['organization_website']);
                $domain = Tools_LeadTools::getDomainFromUrl($data['organization_website']);
                if (!empty($domain)) {
                    $data['organization_domain'] = $domain;
                } else {
                    $data['organization_domain'] = '';
                }
            } else {
                $data['organization_domain'] = '';
            }

            if (!empty($data['organization_website1'])) {
                $data['organization_website1'] = Tools_LeadTools::prepareWebsiteUrl($data['organization_website1']);
                $domain = Tools_LeadTools::getDomainFromUrl($data['organization_website1']);
                if (!empty($domain)) {
                    $data['organization_domain1'] = $domain;
                } else {
                    $data['organization_domain1'] = '';
                }
            } else {
                $data['organization_domain1'] = '';
            }

            $leadOrganizationModel->setOptions($data);

            if ($oldOrganizationName !== $currentOrganizationName) {
                $validateOrganizationNameExists = new Zend_Validate_Db_RecordExists(array(
                    'table' => 'plugin_lead_organization',
                    'field' => 'organization_name'
                ));
                if ($validateOrganizationNameExists->isValid($currentOrganizationName)) {
                    $leadOrganization = $leadOrganizationMapper->findByName($currentOrganizationName);
                    if ($leadOrganization instanceof Leads_Model_LeadsOrganizationModel) {
                        return array(
                            'status' => false,
                            'errorCode' => '6',
                            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[6])
                        );
                    }
                }
            }
            $updatedAt = Tools_System_Tools::convertDateFromTimezone('now');
            $leadOrganizationModel->setUpdatedAt($updatedAt);

            $organizationModel = $leadOrganizationMapper->save($leadOrganizationModel);

            Tools_LeadExternalApiTools::logRequest($this->_request->getHeader('Authorization'),
                $this->_request->getRawBody(), Tools_LeadExternalApiTools::LEAD_ORGANIZATIONS_ENDPOINT, 'PUT');

            $orgData = $organizationModel->toArray();
            unset($orgData['organizationUrl']);

            return array(
                'status' => true,
                'organizationInfo' => $orgData,
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Organization data has been updated')
            );
        }

    }

    /**
     * Delete lead organization
     *
     * Resource:
     * : /api/crm/leadorganization/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     * : lead organization id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $translator = Zend_Registry::get('Zend_Translate');
        if (!$id) {
            return array(
                'status' => false,
                'errorCode' => '8',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[8])
            );
        }
        $leadsOrganizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
        $leadOrganizationModel = $leadsOrganizationMapper->find($id);
        if ($leadOrganizationModel instanceof Leads_Model_LeadsOrganizationModel) {
            $leadsOrganizationMapper->delete($id);
            $lastViewedLeadsAndOrgMapper = Leads_Mapper_LeadsLastViewedMapper::getInstance();
            $lastViewedLeadsAndOrgMapper->delete($id,
                Leads_Model_LeadsLastViewedModel::LEAD_LAST_VIEWED_TYPE_ORGANIZATION);

            Tools_LeadExternalApiTools::logRequest($this->_request->getHeader('Authorization'),
                $this->_request->getParams(), Tools_LeadExternalApiTools::LEAD_ORGANIZATIONS_ENDPOINT, 'DELETE');
            return array(
                'status' => true,
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Organization has been deleted.')
            );

        }
        return array(
            'status' => false,
            'errorCode' => '12',
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[12])
        );
    }

}
