<?php

class Api_Crm_Leads extends Api_Leads_Leads
{

    /**
     * @var array
     */
    public static $_errorCodes = array(
        '1' => 'Access denied',
        '2' => 'Please specify either leadEmail or leadPhone or leadFirstName, leadLastName and leadZip',
        '4' => 'Group is invalid.',
        '5' => 'Email already in use',
        '6' => 'You already have a lead with such phone number',
        '7' => 'You already have a lead with such name and zip',
        '8' => 'Not valid email address',
        '9' => 'Specify ownerId',
        '10' => 'Lead owner not found',
        '11' => 'Lead not found',
        '12' => 'You can\'t change user email. Real user already assigned to the lead.',
        '13' => 'id is missing',
        '14' => 'Lead stage not found',
        '15' => 'Organization not found'
    );


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_GUEST => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );


    public function init()
    {
        parent::init();
        $translator = Zend_Registry::get('Zend_Translate');
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $authHeader = $this->_request->getHeader('Authorization');
        if (empty($authHeader)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[1]), self::REST_STATUS_UNAUTHORIZED);
        }
        $token = Tools_LeadExternalApiTools::verify($authHeader);
        if ($token === false) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[1]), self::REST_STATUS_UNAUTHORIZED);
        }

    }


    /**
     * Get lead by id
     *
     * Resource:
     * : /api/crm/leads/
     *
     * HttpMethod:
     * : GET
     * @return array
     * @throws Zend_Exception
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order', 'pld.created_at DESC'), FILTER_SANITIZE_STRING);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $searchTerm = filter_var($this->_request->getParam('searchTerm'), FILTER_SANITIZE_STRING);
        $email = filter_var($this->_request->getParam('email'), FILTER_SANITIZE_STRING);
        $phone = filter_var($this->_request->getParam('phone'), FILTER_SANITIZE_NUMBER_INT);
        $phoneCountryCode = filter_var($this->_request->getParam('phoneCountryCode'), FILTER_SANITIZE_STRING);
        $leadFirstName = filter_var($this->_request->getParam('leadFirstName'), FILTER_SANITIZE_STRING);
        $leadLastName = filter_var($this->_request->getParam('leadLastName'), FILTER_SANITIZE_STRING);
        $leadZip = filter_var($this->_request->getParam('leadZip'), FILTER_SANITIZE_NUMBER_INT);
        $leadDataMapper = Leads_Mapper_LeadsMapper::getInstance();
        $withOpportunityAmount = false;
        $withSpentPurchaseAmount = false;
        $withNextActivityDate = false;
        $withTasks = false;
        $having = null;
        $callDuration = false;
        $includesNotes = false;
        $includesCustomLeadFields = false;
        $includesCustomOrganizationFields = false;
        $withOpportunityExpected = false;

        Tools_LeadExternalApiTools::logRequest($this->_request->getHeader('Authorization'),
            $this->_request->getParams(), Tools_LeadExternalApiTools::LEADS_ENDPOINT, 'GET');

        if ($id) {
            $where = $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.id = ?', $id);
            $data = $leadDataMapper->fetchAll($where, null, null, null, true, true);
        } else {
            $filterParams = $this->_request->getParam('filter', array());

            foreach ($filterParams as $key => $param) {
                if (!is_array($param)) {
                    $filterParams[$key] = Tools_LeadTools::validateTextField($param);
                }
            }

            $where = null;
            if (!empty($filterParams)) {
                $filterParams['lead-attribute'] = str_replace('*-amp-*', '&', $filterParams['lead-attribute']);
                $where = '';
                if (!empty($filterParams['lead-attribute']) && $filterParams['filter-by'] === 'attributes') {
                    $filterParams['lead-attribute'] = trim($filterParams['lead-attribute'], " \t\n\r\x0B");
                    if (strpos($filterParams['lead-attribute'], '*') !== false) {
                        $attributeValue = str_replace('*', '%', $filterParams['lead-attribute']);
                        $where .= ' (' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_first_name LIKE ?',
                                $attributeValue . '%');
                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_last_name LIKE ?',
                                $attributeValue . '%');
                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_email LIKE ?',
                                $attributeValue . '%');
                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_secondary_email LIKE ?',
                                $attributeValue . '%');
                    } else {
                        $where .= ' (' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_first_name LIKE ?',
                                $filterParams['lead-attribute'] . '%');
                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_last_name LIKE ?',
                                $filterParams['lead-attribute'] . '%');
                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_email LIKE ?',
                                $filterParams['lead-attribute'] . '%');
                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_secondary_email LIKE ?',
                                $filterParams['lead-attribute'] . '%');
                    }

                    if (!empty($filterParams['search_includes_notes'])) {
                        $includesNotes = true;
                        if (strpos($filterParams['lead-attribute'], '*') !== false) {
                            $attributeValue = str_replace('*', '%', $filterParams['lead-attribute']);
                            $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_notes LIKE ?',
                                    $attributeValue . '%');
                            $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pln.note LIKE ?',
                                    $attributeValue . '%');
                        } else {

                            $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_notes LIKE ?',
                                    '%' . $filterParams['lead-attribute'] . '%');
                            $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pln.note LIKE ?',
                                    '%' . $filterParams['lead-attribute'] . '%');
                        }
                    }

                    if (!empty($filterParams['search_includes_custom_lead_fields'])) {
                        $includesCustomLeadFields = true;
                        if (strpos($filterParams['lead-attribute'], '*') !== false) {
                            $attributeValue = str_replace('*', '%', $filterParams['lead-attribute']);
                            $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pllcpd.param_value LIKE ?',
                                    $attributeValue . '%');
                        } else {
                            $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pllcpd.param_value LIKE ?',
                                    '%' . $filterParams['lead-attribute'] . '%');
                        }
                    }

                    $leadPhone = Tools_LeadTools::cleanPhoneAddress($filterParams['lead-attribute']);
                    $leadPhoneIsNumber = mb_substr(preg_replace('~[^0-9A-Za-z]~ui', '', $filterParams['lead-attribute']), 0, 1);
                    if (strpos($filterParams['lead-attribute'], '*') !== false) {
                        $attributeValue = str_replace('*', '%', $filterParams['lead-attribute']);
                        if(!empty($leadPhone) && is_numeric($leadPhoneIsNumber)) {
                            $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_phone LIKE ?',
                                    $leadPhone . '%');
                            $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_mobile LIKE ?',
                                    $leadPhone . '%');
                        }

                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr('CONCAT(pld.lead_first_name, " ", pld.lead_last_name) LIKE ?'),
                                $attributeValue . '%');
                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr('CONCAT(pld.lead_first_name, pld.lead_last_name) LIKE ?'),
                                $attributeValue . '%') . ') ';
                    } else {
                        if(!empty($leadPhone) && is_numeric($leadPhoneIsNumber)) {
                            $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_phone LIKE ?',
                                    $leadPhone . '%');
                            $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_mobile LIKE ?',
                                    $leadPhone . '%');
                        }

                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr('CONCAT(pld.lead_first_name, " ", pld.lead_last_name) LIKE ?'),
                                $filterParams['lead-attribute'] . '%');
                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr('CONCAT(pld.lead_first_name, pld.lead_last_name) LIKE ?'),
                                $filterParams['lead-attribute'] . '%') . ') ';
                    }
                }
                if (!empty($filterParams['lead-attribute']) && $filterParams['filter-by'] === 'organization') {
                    $filterParams['lead-attribute'] = trim($filterParams['lead-attribute'], " \t\n\r\x0B");
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_name LIKE ?',
                        $filterParams['lead-attribute'] . '%');

                    if (!empty($filterParams['search_includes_notes'])) {
                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_notes LIKE ?',
                                '%' . $filterParams['lead-attribute'] . '%');
                    }

                    if (!empty($filterParams['search_includes_custom_lead_fields'])) {
                        $includesCustomOrganizationFields = true;
                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plocpd.param_value LIKE ?',
                                '%' . $filterParams['lead-attribute'] . '%');
                    }
                }

                if (!empty($filterParams['lead_zip'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $leadZips = explode(',', $filterParams['lead_zip']);
                    $leadZips = array_diff($leadZips, array(''));
                    $leadZips = array_map('trim', $leadZips);

                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_zip IN (?)', $leadZips);
                }

                if (!empty($filterParams['lead_score_from'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.score >= ?',
                        (int)$filterParams['lead_score_from']);
                }

                if (!empty($filterParams['lead_score_to'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.score <= ?',
                        (int)$filterParams['lead_score_to']);
                }

                if (!empty($filterParams['lead_city'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_city = ?',
                        $filterParams['lead_city']);
                }

                if (!empty($filterParams['lead_country'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_country_code = ?',
                        $filterParams['lead_country']);
                }

                if (!empty($filterParams['lead_country_state']) && is_array($filterParams['lead_country_state'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_state_code IN (?)',
                        $filterParams['lead_country_state']);
                }

                if (!empty($filterParams['created_at_from'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $createdAt = Tools_System_Tools::convertDateFromTimezone($filterParams['created_at_from']);
                    if (!empty($filterParams['last_updated']) && $filterParams['last_updated'] === 'last_updated') {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.updated_at >= ?',
                            $createdAt);
                    } else {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.created_at >= ?',
                            $createdAt);
                    }
                }
                if (!empty($filterParams['created_at_to'])) {
                    $updatedAt = Tools_System_Tools::convertDateFromTimezone($filterParams['created_at_to'] . ' 23:59:59');
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    if (!empty($filterParams['last_updated']) && $filterParams['last_updated'] === 'last_updated') {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.updated_at <= ?',
                            $updatedAt);
                    } else {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.created_at <= ?',
                            $updatedAt);
                    }
                }

                if (!empty($filterParams['customer_group_id'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.customer_group_id = ?',
                        $filterParams['customer_group_id']);
                }
                if (!empty($filterParams['stage_id'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.stage_id IN (?)',
                        $filterParams['stage_id']);
                }
                if (!empty($filterParams['lead_source_type'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_type_id = ?',
                        $filterParams['lead_source_type']);

                    $leadTypeModel = Leads_Mapper_LeadsTypeMapper::getInstance()->find($filterParams['lead_source_type']);
                    $callDurationParam = false;
                    if ($leadTypeModel instanceof Leads_Model_LeadsTypeModel) {
                        if ($leadTypeModel->getTypeName() == Leads_Model_LeadsTypeModel::LEAD_TYPE_INBOUND_CALL || $leadTypeModel->getTypeName() == Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_CALL) {
                            $callDurationParam = true;
                        }
                    }

                    if (!empty($filterParams['call_duration']) && !empty($callDurationParam)) {
                        $callDuration = true;
                        $filterParams['call_duration'] = (int)$filterParams['call_duration'];

                        $where .= ' AND ';

                        if (!empty($filterParams['more_less_call_duration']) && $filterParams['more_less_call_duration'] == 'less') {
                            $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plcl.call_duration <= ?',
                                $filterParams['call_duration']);
                        } else {
                            $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plcl.call_duration >= ?',
                                $filterParams['call_duration']);
                        }
                    }
                }
                if (!empty($filterParams['lead_source'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_source LIKE ?',
                        '%' . $filterParams['lead_source'] . '%');
                }
                if (!empty($filterParams['owner_id'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?',
                        $filterParams['owner_id']);
                }
                if (!empty($filterParams['opportunity_owner_id'])) {
                    $withOpportunityExpected = true;
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plop.creator_id = ?',
                        $filterParams['opportunity_owner_id']);
                }

                if (!empty($filterParams['special_filter_type'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    if ($filterParams['special_filter_type'] === self::FILTER_EXISTING_CLIENT) {
                        $where .= new Zend_Db_Expr('pld.user_id IS NOT NULL');
                    }
                    if ($filterParams['special_filter_type'] === self::FILTER_PROSPECT) {
                        $where .= new Zend_Db_Expr('pld.user_id IS NULL');
                    }
                    if ($filterParams['special_filter_type'] === self::FILTER_NO_EMAIL) {
                        $where .= new Zend_Db_Expr('pld.lead_email IS NULL');
                    }
                }

                if (!empty($filterParams['next_activity_from']) || !empty($filterParams['next_activity_to'])) {
                    $withNextActivityDate = true;
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plal.event_type = ?',
                        Leads_Model_LeadsActionLogModel::EVENT_LOG_UPCOMING);
                    $where .= ' AND ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plal.created_at > ?',
                            Tools_System_Tools::convertDateFromTimezone('now'));

                }

                if (!empty($filterParams['next_activity_from'])) {
                    $activityFromDate = Tools_System_Tools::convertDateFromTimezone($filterParams['next_activity_from']);
                    $having = $leadDataMapper->getDbTable()->getAdapter()->quoteInto('MIN(plal.created_at) > ?',
                        $activityFromDate);
                }

                if (!empty($filterParams['next_activity_to'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $activityToDate = Tools_System_Tools::convertDateFromTimezone($filterParams['next_activity_to']);
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plal.created_at < ?',
                        $activityToDate);
                }

                if (!empty($filterParams['opportunity_date_from']) || !empty($filterParams['opportunity_date_to'])) {
                    $withOpportunityExpected = true;
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $opportunityFromDate = Tools_System_Tools::convertDateFromTimezone($filterParams['opportunity_date_from']);
                    if (!empty($filterParams['expected_at'])) {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plop.expected_close_date >= ?',
                            $opportunityFromDate);
                    } elseif (!empty($filterParams['creation_at'])) {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plop.created_at >= ?',
                            $opportunityFromDate);
                    } elseif (!empty($filterParams['state_change_at'])) {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plop.change_stage_date >= ?',
                            $opportunityFromDate);
                    }

                    if (!empty($filterParams['opportunity_date_to'])) {
                        if (!empty($where)) {
                            $where .= ' AND ';
                        }

                        $opportunityToDate = Tools_System_Tools::convertDateFromTimezone($filterParams['opportunity_date_to'] . ' 23:59:59');
                        if (!empty($filterParams['expected_at'])) {
                            $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plop.expected_close_date <= ?',
                                $opportunityToDate);
                        } elseif (!empty($filterParams['creation_at'])) {
                            $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plop.created_at <= ?',
                                $opportunityToDate);
                        } elseif (!empty($filterParams['state_change_at'])) {
                            $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plop.change_stage_date <= ?',
                                $opportunityToDate);
                        }
                    }
                }


                if (!empty($filterParams['lead_tags']) && is_array($filterParams['lead_tags'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plht.tag_id IN (?)',
                        $filterParams['lead_tags']);
                }

                if (!empty($filterParams['lead_tags_exclude']) && is_array($filterParams['lead_tags_exclude'])) {
                    $subSelect = $leadDataMapper->getDbTable()->select(Zend_Db_Table::SELECT_WITHOUT_FROM_PART)
                        ->distinct()
                        ->setIntegrityCheck(false)
                        ->from(array('ltags' => 'plugin_leads_has_tags'), array('ltags.lead_id'));
                    $subSelectWhere = $leadDataMapper->getDbTable()->getAdapter()->quoteInto('tag_id IN (?)', $filterParams['lead_tags_exclude']);
                    $subSelect->where($subSelectWhere);
                    $subSelectIds = $leadDataMapper->getDbTable()->getAdapter()->fetchCol($subSelect);
                    if (!empty($subSelectIds)) {
                        if (!empty($where)) {
                            $where .= ' AND ';
                        }
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.id NOT IN (?)', $subSelectIds);
                    }
                }

                if (!empty($filterParams['last_interaction_id'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.last_activity_type_id = ?',
                        $filterParams['last_interaction_id']);
                }

                if (!empty($filterParams['total_opportunity_amount_from']) || !empty($filterParams['total_opportunity_amount_to']) || !empty($filterParams['lead_opportunity_stages']) || !empty($filterParams['opportunity_type_id'])) {
                    $withOpportunityAmount = true;
                }

                if (!empty($filterParams['total_spent_purchase_amount_from']) || !empty($filterParams['total_spent_purchase_amount_to']) && !empty($filterParams['special_filter_type']) && $filterParams['special_filter_type'] === self::FILTER_EXISTING_CLIENT) {
                    $withSpentPurchaseAmount = true;

                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('scs.status IN (?)', array(
                        Models_Model_CartSession::CART_STATUS_COMPLETED,
                        Models_Model_CartSession::CART_STATUS_SHIPPED,
                        Models_Model_CartSession::CART_STATUS_DELIVERED
                    ));
                }

                if (!empty($filterParams['lead_opportunity_stages'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plop.opportunity_stage_id IN (?)',
                        $filterParams['lead_opportunity_stages']);
                }

                if (!empty($filterParams['opportunity_type_id'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plop.opportunity_type_id IN (?)',
                        $filterParams['opportunity_type_id']);
                }

                if (!empty($filterParams['lead_task'])) {
                    $withTasks = true;
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    if ($filterParams['lead_task'] === self::FILTER_TASK_BOTH) {
                        $where .= new Zend_Db_Expr('plta.id IS NOT NULL');
                    }
                    if ($filterParams['lead_task'] === self::FILTER_TASK_OPENED) {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plta.is_task_completed = ?',
                            '0');
                    }
                    if ($filterParams['lead_task'] === self::FILTER_TASK_CLOSED) {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plta.is_task_completed = ?',
                            '1');
                    }
                    if ($filterParams['lead_task'] === self::FILTER_TASK_OVERDUE) {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plta.is_task_overdue = ?', '1');
                        $where .= ' AND '. $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plta.is_task_completed = ?', '0');
                    }
                    if ($filterParams['lead_task'] === self::FILTER_TASK_NONE) {
                        $where .= new Zend_Db_Expr('plta.id IS NULL');
                    }

                }

                if (!empty($filterParams['total_opportunity_amount_from'])) {
                    if (!empty($having)) {
                        $having .= ' AND ';
                    }
                    if (empty($filterParams['next_activity_from'])) {
                        $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('SUM(plop.opportunity_value)/ COUNT(DISTINCT(ifnull(CONCAT(`plht`.`tag_id`, `plht`.`lead_id`), 1))) >= ?',
                            $filterParams['total_opportunity_amount_from']);
                    } else {
                        $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('(SUM(plop.opportunity_value)/ COUNT(DISTINCT(ifnull(CONCAT(`plht`.`tag_id`, `plht`.`lead_id`), 1)))/COUNT(DISTINCT(ifnull(CONCAT(`plal`.`id`, `plal`.`lead_id`), 1)))) >= ?',
                            $filterParams['total_opportunity_amount_from']);
                    }
                }
                if (!empty($filterParams['total_opportunity_amount_to'])) {
                    if (!empty($having)) {
                        $having .= ' AND ';
                    }
                    if (empty($filterParams['next_activity_from'])) {
                        $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('SUM(plop.opportunity_value)/ COUNT(DISTINCT(ifnull(CONCAT(`plht`.`tag_id`, `plht`.`lead_id`), 1))) <= ?',
                            $filterParams['total_opportunity_amount_to']);
                    } else {
                        $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('(SUM(plop.opportunity_value)/ COUNT(DISTINCT(ifnull(CONCAT(`plht`.`tag_id`, `plht`.`lead_id`), 1)))/COUNT(DISTINCT(ifnull(CONCAT(`plal`.`id`, `plal`.`lead_id`), 1)))) <= ?',
                            $filterParams['total_opportunity_amount_to']);
                    }
                }

                if (!empty($filterParams['total_spent_purchase_amount_from'])) {
                    if (!empty($having)) {
                        $having .= ' AND ';
                    }
                    $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('SUM(scs.total)/ COUNT(DISTINCT(ifnull(CONCAT(`plht`.`tag_id`, `plht`.`lead_id`), 1))) >= ?',
                        $filterParams['total_spent_purchase_amount_from']);
                }

                if (!empty($filterParams['total_spent_purchase_amount_to'])) {
                    if (!empty($having)) {
                        $having .= ' AND ';
                    }
                    $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('SUM(scs.total)/ COUNT(DISTINCT(ifnull(CONCAT(`plht`.`tag_id`, `plht`.`lead_id`), 1))) <= ?',
                        $filterParams['total_spent_purchase_amount_to']);
                }

                if (!empty($filterParams['interaction_value'])) {
                    if (in_array($filterParams['interaction_value'], self::$_sortingTypes, true)) {
                        $sortOrder = 'pld.' . $filterParams['interaction_value'] . ' DESC';
                        if ($filterParams['interaction_value'] === self::SORT_ORDER_LAST_ACTIVITY_INBOUND || $filterParams['interaction_value'] === self::SORT_ORDER_LAST_ACTIVITY_OUTBOUND) {
                            $orderInteractionParam = $filterParams['interaction_value'];
                        }
                    }
                }
            } elseif (!empty($searchTerm)) {
                $where = $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_email LIKE ?',
                    $searchTerm . '%');
            } elseif (!empty($email)) {
                $where = $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_email = ?', $email);
            } elseif (!empty($phone)) {
                if (!empty($phoneCountryCode)) {
                    $phoneCountryPhoneCode = Zend_Locale::getTranslation($phoneCountryCode, 'phoneToTerritory');
                    $phoneCountryCode = '+' . $phoneCountryPhoneCode;

                    $where = '(' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr("CONCAT(`pld`.`lead_mobile_country_code_value`, `pld`.`lead_mobile`) = ?"),
                            $phoneCountryCode . $phone);
                    $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr("CONCAT(`pld`.`lead_phone_country_code_value`, `pld`.`lead_phone`) = ?"),
                            $phoneCountryCode . $phone) . ')';
                }
            } elseif (!empty($leadFirstName) && !empty($leadLastName) && !empty($leadZip)) {
                $where = $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_first_name = ?',
                    $leadFirstName);
                $where .= ' AND ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_last_name = ?',
                        $leadLastName);
                $where .= ' AND ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_zip = ?',
                        $leadZip);
            }

            $data = $leadDataMapper->fetchAll($where, $sortOrder, $limit, $offset, false, false, $withOpportunityAmount,
                $having, $withNextActivityDate, $withTasks, $callDuration, $includesNotes, $includesCustomLeadFields,
                $includesCustomOrganizationFields, $withOpportunityExpected, $withSpentPurchaseAmount);

        }

        if (empty($data)) {
            $data = array();
        } else {
            $data['status'] = true;
        }

        return $data;
    }

    /**
     * New lead creation
     *
     * Resource:
     * : /api/crm/leads/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if(empty($data)) {
            $data = $this->getRequest()->getParams();
        }
        $translator = Zend_Registry::get('Zend_Translate');
        $data = Tools_LeadTools::camelCaseKeysToUnderscore($data, true);

        if (isset($data['lead_email'])) {
            $data['lead_email'] = trim($data['lead_email']);
        }

        $allowedToCreate = false;
        if (!empty($data['lead_email'])) {
            $allowedToCreate = true;
        } elseif (!empty($data['lead_phone'])) {
            $allowedToCreate = true;
        } elseif (!empty($data['lead_first_name']) && !empty($data['lead_last_name']) && !empty($data['lead_zip'])) {
            $allowedToCreate = true;
        }

        if (!$allowedToCreate) {
            return array('status' => false,  'errorCode' => '2', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes['2']));
        }

        if (empty($data['owner_id'])) {
            return array('status' => false,  'errorCode' => '9', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes['9']));
        }

        $owners = Leads_Mapper_LeadsMapper::getInstance()->getAllUsers(true, true);
        if (!array_key_exists($data['owner_id'], $owners)) {
            return array('status' => false,  'errorCode' => '10', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes['10']));
        }


        $leadAdditionAvailable = Tools_LeadLicenseTools::checkLimit();
        if ($leadAdditionAvailable['error'] === true) {
            return array('status' => false, 'errorCode' => '3', 'message' => $translator->translate($leadAdditionAvailable['message']));
        }

        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
        $leadsModel = new Leads_Model_LeadsModel();

        if (!empty($data['lead_mobile'])) {
            $data['lead_mobile'] = Tools_LeadTools::cleanPhoneAddress($data['lead_mobile']);
        }

        if (!empty($data['lead_phone'])) {
            $data['lead_phone'] = Tools_LeadTools::cleanPhoneAddress($data['lead_phone']);
        }

        if (!empty($data['lead_phone_country_code'])) {
            $phoneCountryPhoneCode = Zend_Locale::getTranslation($data['lead_phone_country_code'], 'phoneToTerritory');
            $data['lead_phone_country_code_value'] = '+' . $phoneCountryPhoneCode;
        }

        if (!empty($data['lead_mobile_country_code'])) {
            $mobileCountryPhoneCode = Zend_Locale::getTranslation($data['lead_mobile_country_code'],
                'phoneToTerritory');
            $data['lead_mobile_country_code_value'] = '+' . $mobileCountryPhoneCode;
        }

        if (empty($data['lead_mobile'])) {
            $data['lead_mobile'] = null;
            $data['lead_mobile_country_code_value'] = null;
        }

        if (empty($data['lead_phone'])) {
            $data['lead_phone'] = null;
            $data['lead_phone_country_code_value'] = null;
        }

        $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
        $userDefaultMobileCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');
        if (empty($data['lead_mobile_country_code']) && !empty($userDefaultMobileCountryCode)) {
            $data['lead_mobile_country_code'] = $userDefaultMobileCountryCode;
        }

        if (empty($data['lead_phone_country_code']) && !empty($userDefaultMobileCountryCode)) {
            $data['lead_phone_country_code'] = $userDefaultMobileCountryCode;
        }

        $leadOrganizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
        if (!empty($data['organization_id'])) {
            $organizationModel = $leadOrganizationMapper->find($data['organization_id']);
            if ($organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                $leadsModel->setOrganizationId($organizationModel->getId());
            } else {
                return array('status' => false, 'errorCode' => '15', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[15]));
            }
        }

        if (empty($data['organization_id'])) {
            $data['organization_id'] = null;
        }

        if (!empty($data['customer_group_id'])) {
            $groupId = filter_var($data['customer_group_id'], FILTER_SANITIZE_NUMBER_INT);
            if (!empty($groupId)) {
                $validator = new Zend_Validate_Db_NoRecordExists(array(
                    'table' => 'shopping_group',
                    'field' => 'id'
                ));

                if ($validator->isValid($groupId)) {
                    return array('status' => false, 'errorCode' => '4', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[4]));
                }
            }
        }

        if (empty($data['customer_group_id'])) {
            $data['customer_group_id'] = null;
        }

        $leadsModel->setOptions($data);
        $leadsModel->setProductIds(array_filter(explode(',', $leadsModel->getProductIds())));
        $leadsModel->setCreationType(Leads_Model_LeadsModel::CREATION_TYPE_API);

        $assignPrimaryContactId = false;
        $organizationModel = null;

        /**
         * Create organization if not exists
         */
        if (!empty($data['organization_name'])) {
            $organizationName = $data['organization_name'];
            $organizationModel = $leadOrganizationMapper->findByName($organizationName);
            if (!$organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                $assignPrimaryContactId = true;
                $organizationModel = new Leads_Model_LeadsOrganizationModel();
                $organizationModel->setOrganizationName($organizationName);
                $createdAt = Tools_System_Tools::convertDateFromTimezone('now');
                $organizationModel->setCreatedAt($createdAt);
                $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
                $userDefaultMobileCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');
                if (!empty($userDefaultMobileCountryCode)) {
                    $organizationModel->setOrganizationMobileCountryCode($userDefaultMobileCountryCode);
                    $organizationModel->setOrganizationPhoneCountryCode($userDefaultMobileCountryCode);
                }

                $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();
                if (!empty($shoppingConfig['country'])) {
                    $defaultCountryCode = $shoppingConfig['country'];
                    $organizationModel->setOrganizationCountryCode($defaultCountryCode);
                }
                $organizationModel = $leadOrganizationMapper->save($organizationModel);
            }
            $leadsModel->setOrganizationId($organizationModel->getId());

        }

        if (!empty($data['lead_email'])) {
            $emailValidator = new Tools_System_CustomEmailValidator();

            if (!$emailValidator->isValid($data['lead_email'])) {
                return array('status' => false, 'errorCode' => '8', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[8]));
            }

            $validator = new Zend_Validate_Db_NoRecordExists(array(
                'table' => 'plugin_leads_data',
                'field' => 'lead_email'
            ));

            if (!$validator->isValid($data['lead_email'])) {
                return array('status' => false, 'errorCode' => '5', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[5]));
            }
        } else {
            $leadsModel->setLeadEmail(null);
        }

        if (!empty($data['lead_phone_country_code_value']) && !empty($data['lead_phone'])) {
            $leadsModelWithPhone = Tools_LeadTools::findLeadByPhoneMobileNumber($data['lead_phone_country_code_value'] . $data['lead_phone'],
                true);
            if ($leadsModelWithPhone instanceof Leads_Model_LeadsModel) {
                return array('status' => false, 'errorCode' => '6', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[6]));
            }
        }

        if (!empty($data['lead_first_name']) && !empty($data['lead_last_name']) && !empty($data['lead_zip'])) {
            $leadsModelWithFirstLastNameZip = $leadsMapper->findByFirstNameLastNameZip($data['lead_first_name'],
                $data['lead_last_name'], $data['lead_zip']);

            if ($leadsModelWithFirstLastNameZip instanceof Leads_Model_LeadsModel) {
                return array('status' => false, 'errorCode' => '7', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[7]));
            }

            $leadsModel->setLeadZip($data['lead_zip']);
        }

        $createdAt = Tools_System_Tools::convertDateFromTimezone('now');
        $leadsModel->setCreatedAt($createdAt);

        $leadTypeModel = Leads_Mapper_LeadsTypeMapper::getInstance()->getDefaultType();
        if ($leadTypeModel instanceof Leads_Model_LeadsTypeModel && empty($data['lead_type_id'])) {
            $leadsModel->setLeadTypeId($leadTypeModel->getId());
        }

        $leadStageModel = Leads_Mapper_LeadsStageMapper::getInstance()->getDefaultStage();
        if ($leadStageModel instanceof Leads_Model_LeadsStageModel && empty($data['stage_id'])) {
            $leadsModel->setStageId($leadStageModel->getId());
        }

        $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();

        if (!empty($data['lead_phone_country_code'])) {
            $leadsModel->setLeadCountryCode($data['lead_phone_country_code']);
        } elseif (!empty($shoppingConfig['country'])) {
            $defaultCountryCode = $shoppingConfig['country'];
            if (empty($data['lead_country_code'])) {
                $leadsModel->setLeadCountryCode($defaultCountryCode);
            }
        }

        if (!empty($shoppingConfig['state'])) {
            $defaultStateId = $shoppingConfig['state'];
            if (empty($data['lead_state_code'])) {
                $allStates = Tools_LeadTools::getStates($leadsModel->getLeadCountryCode(), true, false, 'state');
                if (array_key_exists($defaultStateId, $allStates)) {
                    $leadsModel->setLeadStateCode($allStates[$defaultStateId]);
                }
            }
        }

        $leadsModel = $leadsMapper->save($leadsModel);

        Tools_LeadExternalApiTools::logRequest($this->_request->getHeader('Authorization'),
            $this->_request->getParams(), Tools_LeadExternalApiTools::LEADS_ENDPOINT, 'POST');

        /**
         * Assign primary contact id if it's new organization
         */
        if ($assignPrimaryContactId === true) {
            $organizationModel->setPrimaryContactId($leadsModel->getId());
            $leadOrganizationMapper->save($organizationModel);
        }

        if ($organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
            $firstContactOrganizationDate = $organizationModel->getFirstContactCreationDate();
            if (empty($firstContactOrganizationDate)) {
                $organizationModel->setFirstContactCreationDate($leadsModel->getCreatedAt());
                $leadOrganizationMapper->save($organizationModel);
            }
        }

        return array(
            'status' => true,
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead has been created'),
            'leadId' => $leadsModel->getId()
        );
    }

    /**
     * Update lead data
     *
     * Resource:
     * : /api/crm/leads/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     * : leads id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if(empty($data)) {
            $data = $this->getRequest()->getParams();
        }
        if (!empty($data['id'])) {
            $data = Tools_LeadTools::camelCaseKeysToUnderscore($data, true);
            $translator = Zend_Registry::get('Zend_Translate');
            $leadId = filter_var($data['id'], FILTER_SANITIZE_NUMBER_INT);
            $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
            $leadModel = $leadMapper->find($leadId);
            if (!$leadModel instanceof Leads_Model_LeadsModel) {
                return array(
                    'status' => false,
                    'errorCode' => '11',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[11])
                );
            }

            $currentLeadEmail = $leadModel->getLeadEmail();
            $currentMobilePhone = $leadModel->getLeadMobile();
            $currentPhone = $leadModel->getLeadPhone();
            if (empty($currentLeadEmail) && empty($currentMobilePhone) && empty($currentPhone) && (!empty($data['lead_email']) || !empty($data['lead_mobile']) || !empty($data['lead_phone']))) {
                $leadAdditionAvailable = Tools_LeadLicenseTools::checkLimit();
                if ($leadAdditionAvailable['error'] === true) {
                    $this->_error($translator->translate($leadAdditionAvailable['message']));
                    return array(
                        'status' => false,
                        'errorCode' => '3',
                        'message' => $translator->translate($leadAdditionAvailable['message'])
                    );
                }
            }

            $oldEmail = $leadModel->getLeadEmail();
            $currentEmail = $data['lead_email'];
            if (!empty($data['lead_phone_country_code'])) {
                $phoneCountryPhoneCode = Zend_Locale::getTranslation($data['lead_phone_country_code'],
                    'phoneToTerritory');
                $data['lead_phone_country_code_value'] = '+' . $phoneCountryPhoneCode;
            }

            if (!empty($data['lead_mobile_country_code'])) {
                $mobileCountryPhoneCode = Zend_Locale::getTranslation($data['lead_mobile_country_code'],
                    'phoneToTerritory');
                $data['lead_mobile_country_code_value'] = '+' . $mobileCountryPhoneCode;
            }

            if (!empty($data['customer_group_id'])) {
                $data['customer_group_id'] = filter_var($data['customer_group_id'], FILTER_SANITIZE_NUMBER_INT);
                if (!empty($data['customer_group_id'])) {
                    $groupId = $data['customer_group_id'];
                    if (!empty($groupId)) {
                        $validator = new Zend_Validate_Db_NoRecordExists(array(
                            'table' => 'shopping_group',
                            'field' => 'id'
                        ));

                        if ($validator->isValid($groupId)) {
                            return array('status' => false, 'errorCode' => '4', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[4]));
                        }
                    }
                }
            }

            if (!empty($data['organization_id'])) {
                $data['organization_id'] = filter_var($data['organization_id'], FILTER_SANITIZE_NUMBER_INT);
                if (!empty($data['organization_id'])) {
                    $organizationId = $data['organization_id'];
                    if (!empty($organizationId)) {
                        $validator = new Zend_Validate_Db_NoRecordExists(array(
                            'table' => 'plugin_lead_organization',
                            'field' => 'id'
                        ));

                        if ($validator->isValid($organizationId)) {
                            return array('status' => false, 'errorCode' => '15', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[15]));
                        }
                    }
                }
            }

            if (empty($data['organization_id'])) {
                $data['organization_id'] = null;
            }

            if (!empty($data['user_id'])) {
                $data['user_id'] = filter_var($data['user_id'], FILTER_SANITIZE_NUMBER_INT);
            }

            if (empty($data['user_id'])) {
                $currentUserId = $leadModel->getUserId();
                if (!empty($currentUserId)) {
                    $data['user_id'] = $currentUserId;
                } else {
                    $data['user_id'] = null;
                }
            }

            if (!empty($data['stage_id'])) {
                $data['stage_id'] = filter_var($data['stage_id'], FILTER_SANITIZE_NUMBER_INT);
                $leadStageModel = Leads_Mapper_LeadsStageMapper::getInstance()->find($data['stage_id']);
                if (!$leadStageModel instanceof Leads_Model_LeadsStageModel) {
                    return array('status' => false, 'errorCode' => '14', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[14]));
                }
            }

            if (empty($data['stage_id'])) {
                $data['stage_id'] = null;
            }

            $clearStateCode = false;

            if (empty($data['lead_country_code'])) {
                $data['lead_country_code'] = null;
            } else {
                $stateCodes = Tools_LeadTools::getStates($data['lead_country_code'], true, true);

                if (!empty($stateCodes) && empty($data['lead_state_code'])) {
                    $stateCodesKeys = array_keys($stateCodes);
                    $data['lead_state_code'] = $stateCodesKeys[0];
                } elseif (empty($stateCodes)) {
                    $clearStateCode = true;
                }
            }

            if (empty($data['lead_state_code'])) {
                $data['lead_state_code'] = null;
            } elseif ($clearStateCode) {
                $data['lead_state_code'] = null;
            } elseif (!empty($stateCodes) && !empty($data['lead_state_code']) && empty($stateCodes[$data['lead_state_code']])) {
                $stateCodesKeys = array_keys($stateCodes);
                $data['lead_state_code'] = $stateCodesKeys[0];
            }

            if (!empty($data['lead_mobile'])) {
                $data['lead_mobile'] = Tools_LeadTools::cleanPhoneAddress($data['lead_mobile']);
            }

            if (!empty($data['lead_phone'])) {
                $data['lead_phone'] = Tools_LeadTools::cleanPhoneAddress($data['lead_phone']);
            }

            if (empty($data['lead_mobile'])) {
                $data['lead_mobile'] = null;
                $data['lead_mobile_country_code_value'] = null;
            }

            if (empty($data['lead_phone'])) {
                $data['lead_phone'] = null;
                $data['lead_phone_country_code_value'] = null;
            }

            if (!empty($data['lead_phone_country_code_value']) && !empty($data['lead_phone'])) {
                $leadsModelWithPhone = Tools_LeadTools::findLeadByPhoneMobileNumber($data['lead_phone_country_code_value'] . $data['lead_phone'],
                    true, $data['id']);
                if ($leadsModelWithPhone instanceof Leads_Model_LeadsModel) {
                    return array(
                        'status' => false,
                        'errorCode' => '30',
                        'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'You have another') . ' <u><a target="_blank" href="' . Zend_Controller_Action_HelperBroker::getStaticHelper('website')->getUrl() . 'dashboard/leads/#lead/' . $leadsModelWithPhone->getId() . '">' . 'lead</a></u>' . ' ' . $translator->translate(Leads::TRANSLATION_PREFIX.'with such phone.')
                    );
                }
            }

            if (!empty($data['lead_mobile_country_code_value']) && !empty($data['lead_mobile'])) {
                $leadsModelWithMobile = Tools_LeadTools::findLeadByPhoneMobileNumber($data['lead_mobile_country_code_value'] . $data['lead_mobile'],
                    true, $data['id']);
                if ($leadsModelWithMobile instanceof Leads_Model_LeadsModel) {
                    return array(
                        'status' => false,
                        'errorCode' => '30',
                        'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'You have another') . ' <u><a target="_blank" href="' . Zend_Controller_Action_HelperBroker::getStaticHelper('website')->getUrl() . 'dashboard/leads/#lead/' . $leadsModelWithPhone->getId() . '">' . 'lead</a></u>' . ' ' . $translator->translate(Leads::TRANSLATION_PREFIX.'with such phone.')
                    );
                }
            }

            if (!empty($data['lead_phone_country_code'])) {
                $phoneCountryPhoneCode = Zend_Locale::getTranslation($data['lead_phone_country_code'],
                    'phoneToTerritory');
                $data['lead_phone_country_code_value'] = '+' . $phoneCountryPhoneCode;
            }

            if (!empty($data['lead_mobile_country_code'])) {
                $mobileCountryPhoneCode = Zend_Locale::getTranslation($data['lead_mobile_country_code'],
                    'phoneToTerritory');
                $data['lead_mobile_country_code_value'] = '+' . $mobileCountryPhoneCode;
            }

            $leadUserId = $leadModel->getUserId();

            if (!empty($currentEmail)) {
                $emailValidator = new Tools_System_CustomEmailValidator();

                if (!$emailValidator->isValid($currentEmail)) {
                    return array(
                        'status' => false,
                        'errorCode' => '8',
                        'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[8])
                    );
                }
            }

            if (empty($currentEmail)) {
                $currentEmail = null;
                $data['lead_email'] = null;
            }

            if ($oldEmail !== $currentEmail) {
                $validateEmailExists = new Zend_Validate_Db_RecordExists(array(
                    'table' => 'plugin_leads_data',
                    'field' => 'lead_email'
                ));
                if ($validateEmailExists->isValid($currentEmail)) {
                    $duplicatedLeadModel = $leadMapper->findByEmail($currentEmail);
                    if ($duplicatedLeadModel instanceof Leads_Model_LeadsModel) {
                        return array(
                            'status' => false,
                            'errorCode' => '30',
                            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'You have another') . ' <u><a target="_blank" href="' . Zend_Controller_Action_HelperBroker::getStaticHelper('website')->getUrl() . 'dashboard/leads/#lead/' . $leadsModelWithPhone->getId() . '">' . 'lead</a></u>' . ' ' . $translator->translate(Leads::TRANSLATION_PREFIX.'with such email.')
                        );
                    }
                }
            }
            if (!empty($leadUserId) && $oldEmail !== $currentEmail) {
                return array(
                    'status' => false,
                    'errorCode' => '12',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[12])
                );
            }


            if (empty($data['customer_group_id'])) {
                $data['customer_group_id'] = null;
            }

            $leadModel->setOptions($data);
//            $createdAt = Tools_System_Tools::convertDateFromTimezone('now');

//            /**
//             * Create organization if not exists
//             */
//            if (!empty($data['organization_name'])) {
//                $organizationName = filter_var($data['organization_name'], FILTER_SANITIZE_STRING);
//                if (!empty($organizationName)) {
//                    $leadOrganizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
//                    $organizationModel = $leadOrganizationMapper->findByName($organizationName);
//                    if (!$organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
//                        $organizationModel = new Leads_Model_LeadsOrganizationModel();
//                        $organizationModel->setOrganizationName($organizationName);
//                        $organizationModel->setCreatedAt($createdAt);
//
//                        $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
//                        $userDefaultMobileCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');
//                        if (!empty($userDefaultMobileCountryCode)) {
//                            $organizationModel->setOrganizationMobileCountryCode($userDefaultMobileCountryCode);
//                            $organizationModel->setOrganizationPhoneCountryCode($userDefaultMobileCountryCode);
//                        }
//
//                        $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();
//                        if (!empty($shoppingConfig['country'])) {
//                            $defaultCountryCode = $shoppingConfig['country'];
//                            $organizationModel->setOrganizationCountryCode($defaultCountryCode);
//                        }
//
//                        /**
//                         * Assign primary contact id if it's new organization
//                         */
//                        $organizationModel->setPrimaryContactId($leadId);
//                        $organizationModel->setFirstContactCreationDate($data['created_at']);
//                        $organizationModel = $leadOrganizationMapper->save($organizationModel);
//                    }
//                    $leadModel->setOrganizationId($organizationModel->getId());
//                    $organizationModelWithThisLeadContactId = $leadOrganizationMapper->findByPrimaryContactId($leadModel->getId());
//                    if ($organizationModelWithThisLeadContactId instanceof Leads_Model_LeadsOrganizationModel) {
//                        $previousPrimaryOrganizationId = $organizationModelWithThisLeadContactId->getId();
//                        $currentOrganizationId = $leadModel->getOrganizationId();
//                        if ($previousPrimaryOrganizationId != $currentOrganizationId) {
//                            $organizationModelWithThisLeadContactId->setPrimaryContactId(null);
//                            $firstContactOrganizationDate = $organizationModel->getFirstContactCreationDate();
//                            if (empty($firstContactOrganizationDate)) {
//                                $organizationModel->setFirstContactCreationDate($data['created_at']);
//                            }
//                            $leadOrganizationMapper->save($organizationModelWithThisLeadContactId);
//                        }
//                    }
//
//                }
//            } else {
//                $leadModel->setOrganizationId(null);
//            }

            $updatedAt = Tools_System_Tools::convertDateFromTimezone('now');

            $leadModel->setUpdatedAt($updatedAt);

            $leadModel = $leadMapper->save($leadModel);

            Tools_LeadExternalApiTools::logRequest($this->_request->getHeader('Authorization'),
                $this->_request->getRawBody(), Tools_LeadExternalApiTools::LEADS_ENDPOINT, 'PUT');

            return array(
                'status' => true,
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead has been updated'),
                'leadId' => $leadModel->toArray()
            );
        }
    }

    /**
     * Delete lead
     *
     * Resource:
     * : /api/crm/leads/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     * : leads id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $translator = Zend_Registry::get('Zend_Translate');
        if (!$id) {
            return array(
                'status' => false,
                'errorCode' => '13',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[13])
            );
        }
        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
        $leadModel = $leadsMapper->find($id);

        if ($leadModel instanceof Leads_Model_LeadsModel) {

            $emailSequenceEnabled = Tools_LeadTools::verifyPluginEnabled(Tools_LeadTools::SEQUENCE_PLUGIN);
            if ($emailSequenceEnabled) {
                $leadEmailHasSequenceMapper = Leads_Mapper_LeadsHasEmailSequenceMapper::getInstance();
                $leadHasSequences = $leadEmailHasSequenceMapper->getLeadSequenceByLeadId($id);

                if (!empty($leadHasSequences)) {
                    foreach ($leadHasSequences as $leadHasSequence) {
                        $sequenceId = $leadHasSequence['sequence_id'];
                        $leadId = $leadHasSequence['lead_id'];

                        Emailsequence_Models_Mapper_EmailsequenceQueueMapper::getInstance()->deleteQueueBySequenceIdAndExternalId($sequenceId,
                            $leadId, 'lead', array('0'));
                    }
                }
            }


            $leadsMapper->delete($id);
            $lastViewedLeadsAndOrgMapper = Leads_Mapper_LeadsLastViewedMapper::getInstance();
            $lastViewedLeadsAndOrgMapper->delete($id, Leads_Model_LeadsLastViewedModel::LEAD_LAST_VIEWED_TYPE_LEAD);

            Tools_LeadExternalApiTools::logRequest($this->_request->getHeader('Authorization'),
                $this->_request->getParams(), Tools_LeadExternalApiTools::LEADS_ENDPOINT, 'DELETE');

            return array(
                'status' => true,
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead has been deleted.')
            );
        }

        return array(
            'status' => false,
            'errorCode' => '11',
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[11])
        );

    }


}
