<?php

class Api_Crm_Leadtask extends Api_Leads_Leadtask
{

    /**
     * @var array
     */
    public static $_errorCodes = array(
        '1' => 'Access denied',
        '2' => 'leadId is missing',
        '3' => 'Lead not found',
        '4' => 'Task title is missing',
        '5' => 'Task notes is missing',
        '6' => 'id is missing',
        '7' => 'Task not found',
        '8' => 'userId is missing',
        '9' => 'wrong userId'
    );

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('title', 'expired_at');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_GUEST => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $translator = Zend_Registry::get('Zend_Translate');
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $authHeader = $this->_request->getHeader('Authorization');
        if (empty($authHeader)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[1]), self::REST_STATUS_UNAUTHORIZED);
        }
        $token = Tools_LeadExternalApiTools::verify($authHeader);
        if ($token === false) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[1]), self::REST_STATUS_UNAUTHORIZED);
        }

    }


    /**
     * Get lead task by id
     *
     * Resource:
     * : /api/leads/leadtask/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

    }

    /**
     * Create new lead task
     *
     * Resource:
     * : /api/crm/leadtask/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if(empty($data)) {
            $data = $this->getRequest()->getParams();
        }
        $translator = Zend_Registry::get('Zend_Translate');
        $data = Tools_LeadTools::camelCaseKeysToUnderscore($data, true);

        $leadsTaskMapper = Leads_Mapper_LeadsTaskMapper::getInstance();
        $leadsTaskModel = new Leads_Model_LeadsTaskModel();

        $data['expired_at'] = Tools_System_Tools::convertDateFromTimezone($data['expired_at']);
        if (empty($data['lead_id'])) {
            return array(
                'status' => false,
                'errorCode' => '2',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[2])
            );
        }

        $leadModel = Leads_Mapper_LeadsMapper::getInstance()->find($data['lead_id']);
        if (!$leadModel instanceof Leads_Model_LeadsModel) {
            return array(
                'status' => false,
                'errorCode' => '3',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[3])
            );
        }

        if (empty($data['title'])) {
            return array(
                'status' => false,
                'errorCode' => '4',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[4])
            );
        }

        if (empty($data['notes'])) {
            return array(
                'status' => false,
                'errorCode' => '5',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[5])
            );
        }

        if (empty($data['user_id'])) {
            return array(
                'status' => false,
                'errorCode' => '8',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[8])
            );
        }

        $owners = Leads_Mapper_LeadsMapper::getInstance()->getAllUsers(true, true);
        if (!array_key_exists($data['user_id'], $owners)) {
            return array('status' => false,  'errorCode' => '9', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes['9']));
        }


        $leadsUsersApiTokensMapper = Leads_Mapper_LeadsUsersApiTokensMapper::getInstance();
        $tokenModel = $leadsUsersApiTokensMapper->findByToken($this->_request->getHeader('Authorization'));
        $apiUserId = $tokenModel->getApiUserId();
        $leadsUsersApiAccessModel =  Leads_Mapper_LeadsUsersApiAccessMapper::getInstance()->find($apiUserId);
        $userId = $leadsUsersApiAccessModel->getUserId();
        $data['creator_id'] = $userId;

        $leadsData = $leadModel->toArray();
        $organizationData = array();
        $dictionary = array();
        $dictionary = Tools_LeadsMailWatchdog::generateDictionary($dictionary, $leadsData, 'lead');
        if (!empty($leadsData['organizationId'])) {
            $organizationModel = Leads_Mapper_LeadsOrganizationMapper::getInstance()->find($leadsData['organizationId']);
            if ($organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                $organizationData = $organizationModel->toArray();
                $dictionary = Tools_LeadsMailWatchdog::generateDictionary($dictionary, $organizationData, 'organization');
            }
        }

        $data['title'] = Tools_LeadsMailWatchdog::parseEmailLexems($data['title'], $dictionary);
        $data['notes'] = Tools_LeadsMailWatchdog::parseEmailLexems($data['notes'], $dictionary);

        if (!empty($data['immediate'])) {
            $expiredAt = Tools_System_Tools::convertDateFromTimezone('now');
            $data['expired_at'] = $expiredAt;
        }

        $leadsTaskModel->setOptions($data);

        $leadTaskModel = $leadsTaskMapper->save($leadsTaskModel);

        $createdAt = Tools_System_Tools::convertDateFromTimezone('now');

        $leadId = filter_var($data['lead_id'], FILTER_SANITIZE_NUMBER_INT);
        $leadModel = Leads_Mapper_LeadsMapper::getInstance()->find($leadId);
        if (!$leadModel instanceof Leads_Model_LeadsModel) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Lead doesn\'t exists'));
        }

        $actionLogExpiredModel = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_TASK, $leadId,
            Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND, $createdAt, Leads_Model_LeadsTypeModel::LEAD_OUTBOUND_TASK);

        $actionLogUpcomingModel = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_TASK, $leadId,
            Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND, $data['expired_at'], Leads_Model_LeadsTypeModel::LEAD_OUTBOUND_TASK, Leads_Model_LeadsActionLogModel::EVENT_LOG_UPCOMING);

        $leadTaskId = $leadTaskModel->getId();

        $leadsTaskLogMapper = Leads_Mapper_LeadsTasksLogMapper::getInstance();
        $leadsTaskLogModel = new Leads_Model_LeadsTasksLogModel();
        $leadsTaskLogModel->setOptions($data);
        $leadsTaskLogModel->setActionLogId($actionLogExpiredModel->getId());
        $leadsTaskLogModel->setTaskId($leadTaskId);
        $leadsTaskLogModel->setLeadId($leadId);
        $leadsTaskLogMapper->save($leadsTaskLogModel);


        $leadsTaskLogModel = new Leads_Model_LeadsTasksLogModel();
        $leadsTaskLogModel->setOptions($data);
        $leadsTaskLogModel->setActionLogId($actionLogUpcomingModel->getId());
        $leadsTaskLogModel->setTaskId($leadTaskId);
        $leadsTaskLogModel->setLeadId($leadId);
        $leadsTaskLogMapper->save($leadsTaskLogModel);

        if (!empty($data['immediate_email']) && empty($data['immediate'])) {
            $leadTaskModel = Leads_Mapper_LeadsTaskMapper::getInstance()->find($leadTaskId);
            if ($leadTaskModel instanceof Leads_Model_LeadsTaskModel) {
                $leadTaskModel->registerObserver(new Tools_Mail_Watchdog(array(
                    'trigger' => Tools_LeadsMailWatchdog::LEADS_TASK_CREATED,
                    'leadData' => $leadsData,
                    'organizationData' => $organizationData
                )));

                $leadTaskModel->notifyObservers();
            }
        }

//        if (!empty($data['immediate'])) {
//            $leadTaskQueueMapper = Leads_Mapper_LeadsTaskQueueMapper::getInstance();
//            $taskReminderInQueueModel = $leadTaskQueueMapper->findByTaskIdQueueType($leadTaskId, Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_TYPE_FIRST_REMINDER);
//            if (!$taskReminderInQueueModel instanceof Leads_Model_LeadsTaskQueueModel) {
//                $taskReminderInQueueModel = new Leads_Model_LeadsTaskQueueModel();
//                $taskReminderInQueueModel->setStatus(Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_STATUS_SENT);
//                $taskReminderInQueueModel->setQueueType(Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_TYPE_FIRST_REMINDER);
//                $taskReminderInQueueModel->setTaskId($leadTaskId);
//                $taskReminderInQueueModel->setToSendDate($expiredAt);
//                $taskReminderInQueueModel->setSentDate($expiredAt);
//                $leadTaskQueueMapper->save($taskReminderInQueueModel);
//            }
//            $taskReminderInQueueModel = $leadTaskQueueMapper->findByTaskIdQueueType($leadTaskId, Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_TYPE_SECOND_REMINDER);
//            if (!$taskReminderInQueueModel instanceof Leads_Model_LeadsTaskQueueModel) {
//                $taskReminderInQueueModel = new Leads_Model_LeadsTaskQueueModel();
//                $taskReminderInQueueModel->setStatus('processing');
//                $taskReminderInQueueModel->setQueueType(Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_TYPE_SECOND_REMINDER);
//                $taskReminderInQueueModel->setTaskId($leadTaskId);
//                $taskReminderInQueueModel->setToSendDate($expiredAt);
//                $taskReminderInQueueModel->setSentDate($expiredAt);
//                $taskReminderInQueueModel = $leadTaskQueueMapper->save($taskReminderInQueueModel);
//            }
//
//            $leadTaskModel = Leads_Mapper_LeadsTaskMapper::getInstance()->find($leadTaskId);
//            if ($leadTaskModel instanceof Leads_Model_LeadsTaskModel) {
//                $leadTaskModel->registerObserver(new Tools_Mail_Watchdog(array(
//                    'trigger' => Tools_LeadsMailWatchdog::LEADS_TASK_REMINDER,
//                    'queueId' => $taskReminderInQueueModel->getId(),
//                    'leadData' => $leadsData,
//                    'organizationData' => $organizationData
//                )));
//
//                $leadTaskModel->notifyObservers();
//            }
//        }

        Tools_LeadExternalApiTools::logRequest($this->_request->getHeader('Authorization'),
            $this->_request->getParams(), Tools_LeadExternalApiTools::LEAD_TASK_ENDPOINT, 'POST');

        return array(
            'status' => true,
            'taskInfo' => $leadTaskModel->toArray(),
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead task has been created')
        );

    }

    /**
     * Update lead task data
     *
     * Resource:
     * : /api/crm/leadtask/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     * : lead task id to update
     *
     * @return JSON
     */
    public function putAction()
    {

    }

    /**
     * Delete lead task
     *
     * Resource:
     * : /api/crm/leadtask/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     * : lead task id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $translator = Zend_Registry::get('Zend_Translate');
        if (!$id) {
            return array(
                'status' => false,
                'errorCode' => '6',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[6])
            );
        }
        $leadsTaskMapper = Leads_Mapper_LeadsTaskMapper::getInstance();
        $leadTaskModel = $leadsTaskMapper->find($id);
        if ($leadTaskModel instanceof Leads_Model_LeadsTaskModel) {
            $leadsTaskMapper->delete($id);
            Tools_LeadExternalApiTools::logRequest($this->_request->getHeader('Authorization'),
                $this->_request->getParams(), Tools_LeadExternalApiTools::LEAD_TASK_ENDPOINT, 'DELETE');
            return array('status' => true, 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Task has been deleted'));
        }

        return array(
            'status' => false,
            'errorCode' => '7',
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[7])
        );

    }

}
