<?php

class Api_Crm_Organizationindustries extends Api_Service_Abstract
{

    /**
     * @var array
     */
    public static $_errorCodes = array(
        '1' => 'Access denied'
    );


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_GUEST => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );


    public function init()
    {
        parent::init();
        $translator = Zend_Registry::get('Zend_Translate');
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $authHeader = $this->_request->getHeader('Authorization');
        if (empty($authHeader)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[1]), self::REST_STATUS_UNAUTHORIZED);
        }
        $token = Tools_LeadExternalApiTools::verify($authHeader);
        if ($token === false) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[1]), self::REST_STATUS_UNAUTHORIZED);
        }

    }


    /**
     * Get lead organization industries
     *
     * Resource:
     * : /api/crm/organizationindustries/
     *
     * HttpMethod:
     * : GET
     * @return array
     * @throws Zend_Exception
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order', 'pli.id DESC'), FILTER_SANITIZE_STRING);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $having = null;
        $leadsIndustryMapper = Leads_Mapper_LeadsIndustryMapper::getInstance();

        Tools_LeadExternalApiTools::logRequest($this->_request->getHeader('Authorization'),
            $this->_request->getParams(), Tools_LeadExternalApiTools::ORGANIZATION_INDUSTRIES_ENDPOINT, 'GET');

        if ($id) {
            $where = $leadsIndustryMapper->getDbTable()->getAdapter()->quoteInto('pli.id = ?', $id);
            $data = $leadsIndustryMapper->fetchAllData($where, null, null, null, true, true);
        } else {
            $where = null;
            $data = $leadsIndustryMapper->fetchAllData($where, $sortOrder, $limit, $offset);
        }

        if (empty($data)) {
            $data = array();
        } else {
            $data['status'] = true;
        }

        return $data;
    }

    /**
     *
     * Resource:
     * : /api/crm/organizationindustries/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {

    }

    /**
     *
     * Resource:
     * : /api/crm/organizationindustries/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     * : leads id to update
     *
     * @return JSON
     */
    public function putAction()
    {

    }

    /**
     *
     * Resource:
     * : /api/crm/organizationindustries/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     * : leads id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {

    }


}
