<?php

class Api_Leads_Eventrules extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;


    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('event_id', 'rule_name');

    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    /**
     * User restricted id access
     *
     * @var null
     */
    protected $_restrictedUserId = null;


    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
    }


    /**
     * Get event rules
     *
     * Resource:
     * : /api/leads/eventrules/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order'), FILTER_SANITIZE_STRING);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $formName = filter_var($this->_request->getParam('formName'), FILTER_SANITIZE_STRING);
        $forGrid = (bool)filter_var($this->_request->getParam('isGrid', false), FILTER_SANITIZE_STRING);
        $detailedInfo = (bool)filter_var($this->_request->getParam('detailedInfo', false), FILTER_SANITIZE_STRING);

        if (empty($sortOrder)) {
            $sortOrder = 'plergc.created_at DESC';
        }

        $leadsEventRulesGeneralConfigMapper = Leads_Mapper_LeadsEventsRulesGeneralConfigMapper::getInstance();

        $where = null;
        $withTimezoneOffset = false;
        if ($detailedInfo || $forGrid) {
            $withTimezoneOffset = true;
        }
        if ($id && $forGrid) {
            $where = $leadsEventRulesGeneralConfigMapper->getDbTable()->getAdapter()->quoteInto('plergc.id = ?', $id);
            $data = $leadsEventRulesGeneralConfigMapper->fetchAll($where, null, null, null, true, true, $withTimezoneOffset);
        } elseif ($id) {
            $data = $leadsEventRulesGeneralConfigMapper->find($id);
        } elseif(!empty($formName) && !empty($formType)) {

        } elseif (!empty($forGrid)) {
            $data = $leadsEventRulesGeneralConfigMapper->fetchAll($where, $sortOrder, $limit, $offset, false, false, $withTimezoneOffset);
        }

        if (empty($data)) {
            $data = array();
        }

        if ($forGrid && !empty($detailedInfo)) {
            $ruleId = $data['id'];
            $leadEventRulesConfigMapper = Leads_Mapper_LeadsEventRulesConfigMapper::getInstance();
            $actionsData =  $leadsEventRulesGeneralConfigMapper->getEventActionsByRuleId($ruleId);
            $fieldsData =  $leadEventRulesConfigMapper->getFormFieldsByRuleId($ruleId);
            $data['fieldsData'] = $fieldsData;
            $data['formFields'] = Tools_LeadFormTools::getDefaultFieldsList();

            $data['actionsData'] = $actionsData;
            $data['leadOwners'] = Leads_Mapper_LeadsMapper::getInstance()->getLeadOwnersFullList();
            $data['allUsers'] = Leads_Mapper_LeadsMapper::getInstance()->getAllUsers(true);
            $currentUser = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser();
            $data['currentUserId'] = $currentUser->getId();
            $data['userTimezoneOffset'] = Tools_System_Tools::getUtcOffset('P');
            $data['userTimezone'] = $currentUser->getTimezone();
            $leadTagsDataMapper = Leads_Mapper_LeadsTagsMapper::getInstance();
            $leadStageDataMapper = Leads_Mapper_LeadsStageMapper::getInstance();
            $leadSequenceMapper = Emailsequence_Models_Mapper_EmailSequenceSettingsMapper::getInstance();
            $data['lifecycleStages'] = $leadStageDataMapper->getStages();
            $data['leadTags'] = $leadTagsDataMapper->getTagsPairs();
            $data['emailSequencies'] = $leadSequenceMapper->getSequencesPairs();
            $data['emailSequenciesArchived'] = $leadSequenceMapper->getArchivedSequences();
            $emailTemplates = Leads_Mapper_LeadsMapper::getInstance()->getEmailTemplates(true);
            $data['emailTemplates'] =  $emailTemplates;
            $data['emailServices'] = $enabledServices = Tools_LeadTools::getEnabledServices('email');
            $data['limitedAccess'] =  $this->_restrictedUserId;
            $advertisingList = Tools_LeadTools::$_advertisingTypes;
            foreach ($advertisingList as $advKey => $advAccount) {
                $services = Tools_LeadTools::sambastatsConnectionStatusGet($advKey);

                if (!empty($services['data'])) {
                    $adAccountId = $services['data']['adAccountId'];
                    if(empty($services['data']['connectionStatus'])) {
                        $adAccountId = 'disconnected';
                    }
                    $advertisingList[$advKey] .= ' (' . $adAccountId . ')';
                }
            }
            $data['advertisingList'] = $advertisingList;
            $data['cronScriptPathAudiences'] = Zend_Registry::get('website')['path'].Tools_LeadRemarketingAudiencesTools::LEAD_REMARKETING_AUDIENCES_PROCESS_PATH;
        }

        return $data;
    }

    /**
     * New lead form rule general config
     *
     * Resource:
     * : /api/leads/eventrules/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        if (!empty($this->_restrictedUserId)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'You don\'t have permission to add events'));
        }
        $data = $this->getRequest()->getParams();


        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'));
        }

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'/go">here</a>'));
        }

        $leadsEventRulesGeneralConfigMapper = Leads_Mapper_LeadsEventsRulesGeneralConfigMapper::getInstance();
        $leadsEventRulesActionsMapper = Leads_Mapper_LeadsEventRulesActionsMapper::getInstance();

        $ruleName = $data['rule_name'];
        $eventId = $data['event_id'];

        $ruleModel = $leadsEventRulesGeneralConfigMapper->checkRuleExist($eventId, $ruleName);
        if ($ruleModel instanceof Leads_Model_LeadsEventsRulesGeneralConfigModel) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Rule name already exists'));
        }

        $ruleModel = new Leads_Model_LeadsEventsRulesGeneralConfigModel();

        $createdAt = Tools_System_Tools::convertDateFromTimezone('now');
        if (empty($data['user_id'])) {
            $currentUserId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();
        } else {
            $currentUserId = $data['user_id'];
        }

        $ruleModel->setOptions($data);
        $ruleModel->setCreatorId($currentUserId);
        $ruleModel->setCreatedAt($createdAt);

        $ruleModel = $leadsEventRulesGeneralConfigMapper->save($ruleModel);
        $ruleId = $ruleModel->getId();
        $loggedUserId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();
        if (!empty($data['fieldsData']) && is_array($data['fieldsData'])) {
            $leadEventRulesConfigMapper = Leads_Mapper_LeadsEventRulesConfigMapper::getInstance();
            foreach ($data['fieldsData'] as $fieldData) {
                $leadEventRulesConfigModel = new Leads_Model_LeadsEventRulesConfigModel();
                $leadEventRulesConfigModel->setFieldName($fieldData['field_name']);
                $leadEventRulesConfigModel->setFieldValue($fieldData['field_value']);
                $leadEventRulesConfigModel->setRuleId($ruleId);
                $leadEventRulesConfigModel->setRuleComparisonOperator($fieldData['rule_comparison_operator']);
                $leadEventRulesConfigMapper->save($leadEventRulesConfigModel);
            }
        }

        if (!empty($data['actionsData']) && is_array($data['actionsData'])) {
            $rawActionsData = $this->getRequest()->getParam('actionsData');
            foreach ($data['actionsData'] as $index => $actionData) {
                if ($actionData['actionType'] === 'assign_task' && isset($rawActionsData[$index]['notes'])) {
                    $actionData['notes'] = $rawActionsData[$index]['notes'];
                }
                if ($actionData['actionType'] === 'assign_task' && empty($actionData['creator_id'])) {
                    $actionData['creator_id'] = $loggedUserId;
                }
                if ($actionData['actionType'] === 'assign_task' && empty($actionData['user_id'])) {
                    $actionData['user_id'] = $loggedUserId;
                }
                if($actionData['actionType'] === 'assign_email_copy') {
                    $actionData['emailsToCopy'] = Tools_LeadTools::trimValues($actionData['emailsToCopy'], true);
                }

                if ($actionData['actionType'] === 'assign_task' && empty($actionData['taskImmediate'])) {
                    $shoppingConfigMapper =  Models_Mapper_ShoppingConfig::getInstance();
                    $storeTimezone = $shoppingConfigMapper->getConfigParam('timezone');
                    if (!empty($actionData['taskTimezone'])) {
                        $timezone = $actionData['taskTimezone'];
                    } elseif(!empty($storeTimezone)) {
                        $timezone = $storeTimezone;
                    } else {
                        $timezone = 'UTC';
                    }
                    $hoursShift = Tools_EmailSequenceTools::getTimezoneShift('UTC', $timezone);
                    if ($hoursShift > 0) {
                        $hoursShift = '-' . $hoursShift;
                    } else {
                        $hoursShift = '+' . abs($hoursShift);
                    }
                    $actionData['hoursShift'] = $hoursShift;
                }

                if ($actionData['actionType'] === 'assign_remarketing_list') {
                    $actionData['lists'] = $rawActionsData[$index]['lists'];
                    $actionData['listsLabels'] = $rawActionsData[$index]['listsLabels'];
                }

                $leadsEventRulesActionModel = new Leads_Model_LeadsEventRulesActionModel();
                $leadsEventRulesActionModel->setRuleId($ruleId);
                $leadsEventRulesActionModel->setActionType($actionData['actionType']);
                unset($actionData['actionType']);
                $leadsEventRulesActionModel->setActionConfig(json_encode($actionData));
                $leadsEventRulesActionsMapper->save($leadsEventRulesActionModel);
            }
        }

        $this->_responseHelper->success(array(
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Rule has been created')
        ));

    }

    /**
     * Update lead form rule general config
     *
     * Resource:
     * : /api/leads/eventrules/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     * : lead form rule general config id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        if (!empty($this->_restrictedUserId)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'You don\'t have permission to edit event'));
        }
        parse_str($this->_request->getRawBody(), $data);
        if (!empty($data['ruleId']) && !empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {

            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();
                $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
            }

            $leadsEventRulesGeneralConfigMapper = Leads_Mapper_LeadsEventsRulesGeneralConfigMapper::getInstance();
            $leadsEventRulesActionsMapper = Leads_Mapper_LeadsEventRulesActionsMapper::getInstance();

            $ruleId = $data['ruleId'];

            $leadsEventRulesGeneralConfigModel = $leadsEventRulesGeneralConfigMapper->find($ruleId);
            if (!$leadsEventRulesGeneralConfigModel instanceof Leads_Model_LeadsEventsRulesGeneralConfigModel) {
                $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Rule id is missing'));
            }

            $eventId = $leadsEventRulesGeneralConfigModel->getEventId();
            $ruleName = $leadsEventRulesGeneralConfigModel->getRuleName();
            $data['event_id'] = $eventId;

            $leadsFormRulesGeneralConfigModelExisting = $leadsEventRulesGeneralConfigMapper->checkRuleExist($eventId, $ruleName);
            if ($leadsFormRulesGeneralConfigModelExisting instanceof Leads_Model_LeadsFormRulesGeneralConfigModel) {
                $otherRuleId = $leadsFormRulesGeneralConfigModelExisting->getId();
                if ($otherRuleId != $ruleId) {
                    $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Rule with such name already exist'));
                }
            }

            $loggedUserId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();
            $updatedAt = Tools_System_Tools::convertDateFromTimezone('now');
            if (empty($data['editor_id'])) {
                $editorUserId = $loggedUserId;
            } else {
                $editorUserId = $data['editor_id'];
            }

            $leadsEventRulesGeneralConfigModel->setOptions($data);
            $leadsEventRulesGeneralConfigModel->setEditorId($editorUserId);
            $leadsEventRulesGeneralConfigModel->setUpdatedAt($updatedAt);

            $leadsEventRulesGeneralConfigModel = $leadsEventRulesGeneralConfigMapper->save($leadsEventRulesGeneralConfigModel);
            $ruleId = $leadsEventRulesGeneralConfigModel->getId();
            $leadEventRulesConfigMapper = Leads_Mapper_LeadsEventRulesConfigMapper::getInstance();

            $leadsEventRulesActionsMapper->deleteByRuleId($ruleId);
            $leadEventRulesConfigMapper->deleteByRuleId($ruleId);

            if (!empty($data['fieldsData']) && is_array($data['fieldsData'])) {
                foreach ($data['fieldsData'] as $fieldData) {
                    $leadEventRulesConfigModel = new Leads_Model_LeadsEventRulesConfigModel();
                    $leadEventRulesConfigModel->setFieldName($fieldData['field_name']);
                    $leadEventRulesConfigModel->setFieldValue($fieldData['field_value']);
                    $leadEventRulesConfigModel->setRuleId($ruleId);
                    $leadEventRulesConfigModel->setRuleComparisonOperator($fieldData['rule_comparison_operator']);
                    $leadEventRulesConfigMapper->save($leadEventRulesConfigModel);
                }
            }

            if (!empty($data['actionsData']) && is_array($data['actionsData'])) {
                foreach ($data['actionsData'] as $actionData) {
                    $leadsEventRulesActionModel = new Leads_Model_LeadsEventRulesActionModel();
                    if ($actionData['actionType'] === 'assign_task' && empty($actionData['creator_id'])) {
                        $actionData['creator_id'] = $loggedUserId;
                    }
                    if ($actionData['actionType'] === 'assign_task' && empty($actionData['user_id'])) {
                        $actionData['user_id'] = $loggedUserId;
                    }
                    if($actionData['actionType'] === 'assign_email_copy') {
                        $actionData['emailsToCopy'] = Tools_LeadTools::trimValues($actionData['emailsToCopy'], true);
                    }

                    if ($actionData['actionType'] === 'assign_task' && empty($actionData['taskImmediate'])) {
                        $shoppingConfigMapper =  Models_Mapper_ShoppingConfig::getInstance();
                        $storeTimezone = $shoppingConfigMapper->getConfigParam('timezone');
                        if (!empty($actionData['taskTimezone']) && $actionData['taskTimezone'] !== 'null') {
                            $timezone = $actionData['taskTimezone'];
                        } elseif(!empty($storeTimezone)) {
                            $timezone = $storeTimezone;
                        } else {
                            $timezone = 'UTC';
                        }
                        $hoursShift = Tools_EmailSequenceTools::getTimezoneShift('UTC', $timezone);
                        if ($hoursShift > 0) {
                            $hoursShift = '-' . $hoursShift;
                        } else {
                            $hoursShift = '+' . abs($hoursShift);
                        }
                        $actionData['hoursShift'] = $hoursShift;
                    }
                    $leadsEventRulesActionModel->setRuleId($ruleId);
                    $leadsEventRulesActionModel->setActionType($actionData['actionType']);
                    unset($actionData['actionType']);
                    $leadsEventRulesActionModel->setActionConfig(json_encode($actionData));
                    $leadsEventRulesActionsMapper->save($leadsEventRulesActionModel);
                }
            }


            $this->_responseHelper->success(array(
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Rule has been updated')
            ));
        }

    }

    /**
     * Delete lead event rule general config
     *
     * Resource:
     * : /api/leads/eventrules/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     * : lead event rule general config to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        if (!empty($this->_restrictedUserId)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'You don\'t have permission to delete event'));
        }
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        if (!$id) {
            $this->_error();
        }
        $leadsEventRulesGeneralConfigMapper = Leads_Mapper_LeadsEventsRulesGeneralConfigMapper::getInstance();
        $leadsEventRulesGeneralConfigModel = $leadsEventRulesGeneralConfigMapper->find($id);

        if ($leadsEventRulesGeneralConfigModel instanceof Leads_Model_LeadsEventsRulesGeneralConfigModel) {
            $leadsEventRulesGeneralConfigMapper->delete($id);
        } else {
            $this->_error();
        }

    }

}
