<?php

class Api_Leads_Externalform extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_GUEST => array(
            'allow' => array('post')
        )
    );


    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_redirector = new Zend_Controller_Action_Helper_Redirector();
    }


    /**
     *
     * Resource:
     * : /api/leads/externalforms/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {


    }

    public function postAction()
    {
        $formData = $this->_request->getParams();
        $translator = Zend_Registry::get('Zend_Translate');
        if (empty($formData)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Data missing'));
        }

        if (empty($formData['formSignature'])) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Access not allowed'));
        }

        unset($formData['plugin']);
        unset($formData['service']);
        unset($formData['controller']);
        unset($formData['action']);

        $leadsFormBuilderGeneralConfigMapper = Leads_Mapper_LeadsBuilderFormGeneralConfigMapper::getInstance();
        $leadsFormBuilderGeneralConfigModel = $leadsFormBuilderGeneralConfigMapper->findBySignature($formData['formSignature']);
        if (!$leadsFormBuilderGeneralConfigModel instanceof Leads_Model_LeadsBuilderFormGeneralConfigModel) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Form not found'));
        }

        $failUrl = $leadsFormBuilderGeneralConfigModel->getFailUrl();
        $domain = $leadsFormBuilderGeneralConfigModel->getFormDomain();
        $formConfigSignature = $leadsFormBuilderGeneralConfigModel->getFormSignature();
        $fullDomain = 'http://'.strtr($domain, array('http://' => '', 'https://'  => ''));
        $formHost = parse_url($fullDomain, PHP_URL_HOST);

        if (empty($formData['g-recaptcha-response'])) {
            if (empty($failUrl)) {
                $this->_redirector->gotoUrl($fullDomain);
            }
            $this->_redirector->gotoUrl($fullDomain.'/'.$failUrl);
        }

        $recaptchaResponse = $formData['g-recaptcha-response'];

        unset($formData['g-recaptcha-response']);


        if ($formConfigSignature !== $formData['formSignature']) {
           $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Access not allowed'));
        }

        unset($formData['formSignature']);

        if (empty($formData['email'])) {
            if (empty($failUrl)) {
                $this->_redirector->gotoUrl($fullDomain);
            }
            $this->_redirector->gotoUrl($fullDomain.'/'.$failUrl);
        }

        $formId = $leadsFormBuilderGeneralConfigModel->getId();
        $recaptchaPrivateKey = $leadsFormBuilderGeneralConfigModel->getRecaptchaPrivateKey();
        $successUrl = $leadsFormBuilderGeneralConfigModel->getSuccessUrl();

        $leadBuilderFormFieldsMapper = Leads_Mapper_LeadsBuilderFormFieldsMapper::getInstance();
        $originalFieldsData = $leadBuilderFormFieldsMapper->getFieldsData($formId);

        if (!empty($originalFieldsData)) {
            foreach ($originalFieldsData as $originalFieldData) {
                $fieldsData = json_decode($originalFieldData['field_data'], true);
                if ($fieldsData['fieldType'] === Tools_LeadFormBuilderTools::FORM_FIELD_TYPE_CHECKBOX) {
                    if (!isset($formData[$fieldsData['fieldName']])) {
                        unset($originalFieldsData[$fieldsData['fieldName']]);
                    }
                }

                if ($fieldsData['fieldType'] === Tools_LeadFormBuilderTools::FORM_FIELD_TYPE_RADIO) {
                    if (!isset($formData[$fieldsData['fieldName']])) {
                        unset($originalFieldsData[$fieldsData['fieldName']]);
                    }
                }
            }
        }


        if (!empty($originalFieldsData) && array_key_exists(Tools_LeadFormBuilderTools::FORM_FIELD_SUBTYPE_MOBILE,
                $originalFieldsData) && $originalFieldsData[Tools_LeadFormBuilderTools::FORM_FIELD_SUBTYPE_MOBILE]['system_type'] === Tools_LeadFormBuilderTools::FORM_FIELD_SYSTEM_TYPE_CRM
        ) {
            $originalFieldsData['mobile-country-code'] = array();
        }

        if (!empty($originalFieldsData) && array_key_exists(Tools_LeadFormBuilderTools::FORM_FIELD_SUBTYPE_PHONE,
                $originalFieldsData) && $originalFieldsData[Tools_LeadFormBuilderTools::FORM_FIELD_SUBTYPE_PHONE]['system_type'] === Tools_LeadFormBuilderTools::FORM_FIELD_SYSTEM_TYPE_CRM
        ) {
            $originalFieldsData['phone-country-code'] = array();
        }

        if (empty($originalFieldsData['email'])) {
            $originalFieldsData['email'] = array();
        }

        // Check if all fields exists
        $allFieldsExist = array_intersect_key($originalFieldsData, $formData);
        if (count($allFieldsExist) !== count($originalFieldsData)) {
            if (empty($failUrl)) {
                $this->_redirector->gotoUrl($fullDomain);
            }
            $this->_redirector->gotoUrl($fullDomain.'/'.$failUrl);
        }

        //Filter out fake data
        $processedFields = array();
        foreach ($formData as $fieldName => $formFieldValue) {
            if (array_key_exists($fieldName, $originalFieldsData)) {
                $value = $formFieldValue;
                if (!empty($originalFieldsData[$fieldName]['field_data'])) {
                    $fieldsData = json_decode($originalFieldsData[$fieldName]['field_data'], true);

                    if ($fieldsData['fieldType'] === Tools_LeadFormBuilderTools::FORM_FIELD_TYPE_CHECKBOX) {
                        if (!empty($formFieldValue)) {
                            $value = implode(',', array_intersect($fieldsData['fieldOptions'], $formFieldValue));
                        } else {
                            $value = '';
                        }
                    }

                    if ($fieldsData['fieldType'] === Tools_LeadFormBuilderTools::FORM_FIELD_TYPE_DROPDOWN) {
                        if (!empty($formFieldValue) && in_array($formFieldValue, $fieldsData['fieldOptions'], true)) {
                            $value = $formFieldValue;
                        } else {
                            $value = '';
                        }
                    }

                    if ($fieldsData['fieldType'] === Tools_LeadFormBuilderTools::FORM_FIELD_TYPE_RADIO) {
                        if (!empty($formFieldValue) && in_array($formFieldValue, $fieldsData['fieldOptions'], true)) {
                            $value = $formFieldValue;
                        } else {
                            $value = '';
                        }
                    }
                }

                $processedFields[$fieldName] = $value;

            }
        }

        if (!empty($_SERVER['HTTP_REFERER'])) {
            $processedFields['formUrl'] = $_SERVER['HTTP_REFERER'];
        } else {
            $processedFields['formUrl'] = $fullDomain;
        }

        $processedFields['formName'] = $leadsFormBuilderGeneralConfigModel->getFormName();
        $processedFields['externalFormSignature'] = $formConfigSignature;

        $url = 'https://www.google.com/recaptcha/api/siteverify'. "?secret=" . $recaptchaPrivateKey . "&response=" . $recaptchaResponse;
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_TIMEOUT, 10);
        $res = curl_exec($curl);
        curl_close($curl);

        $res = json_decode($res, true);
        if (is_array($res) && !empty($res['success']) && !empty($res['hostname'])) {
             if ($res['success'] && $res['hostname'] == $formHost) {
                 if (Tools_System_FormBlacklist::isBlacklisted($formData['email'], $formData)) {
                     if (empty($successUrl)) {
                         $this->_redirector->gotoUrl($fullDomain);
                     }
                     $this->_redirector->gotoUrl($fullDomain.'/'.$successUrl);
                 }

                 if (Tools_System_FormBlacklist::isSpam($formData)) {
                     if (empty($successUrl)) {
                         $this->_redirector->gotoUrl($fullDomain);
                     }
                     $this->_redirector->gotoUrl($fullDomain.'/'.$successUrl);
                 }
                Tools_LeadExternalFormTools::sendEmails($leadsFormBuilderGeneralConfigModel, $processedFields);
                Tools_LeadExternalFormTools::sendSms($leadsFormBuilderGeneralConfigModel, $processedFields);
                $formWatchdog = new Tools_LeadFormWatchdog();
                $formWatchdog->notify($processedFields);
                if (!empty($successUrl)) {
                    $this->_redirector->gotoUrl($fullDomain.'/'.$successUrl);
                }
                $this->_redirector->gotoUrl($fullDomain);
            }
        }

        if (empty($failUrl)) {
            $this->_redirector->gotoUrl($fullDomain);
        }
        $this->_redirector->gotoUrl($fullDomain.'/'.$failUrl);

    }


    public function putAction()
    {

    }

    public function deleteAction()
    {

    }

}
