<?php

class Api_Leads_Formbuilder extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;


    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('form_name', 'form_type', 'field_data');

    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    /**
     * User restricted id access
     *
     * @var null
     */
    protected $_restrictedUserId = null;


    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
    }


    /**
     * Get form builder records
     *
     * Resource:
     * : /api/leads/formbuilder/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order'), FILTER_SANITIZE_STRING);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $formName = filter_var($this->_request->getParam('formName'), FILTER_SANITIZE_STRING);
        $forGrid = (bool)filter_var($this->_request->getParam('isGrid', false), FILTER_SANITIZE_STRING);
        $detailedInfo = (bool)filter_var($this->_request->getParam('detailedInfo', false), FILTER_SANITIZE_STRING);

        if (empty($sortOrder)) {
            $sortOrder = 'plbfgc.created_at DESC';
        }

        $leadsFormBuilderGeneralConfigMapper = Leads_Mapper_LeadsBuilderFormGeneralConfigMapper::getInstance();

        $where = null;
        $withTimezoneOffset = false;
        if ($detailedInfo || $forGrid) {
            $withTimezoneOffset = true;
        }
        if ($id && $forGrid) {
            $where = $leadsFormBuilderGeneralConfigMapper->getDbTable()->getAdapter()->quoteInto('plbfgc.id = ?', $id);
            $data = $leadsFormBuilderGeneralConfigMapper->fetchAll($where, null, null, null, true, true, $withTimezoneOffset);
        } elseif ($id) {
            $data = $leadsFormBuilderGeneralConfigMapper->find($id);
        } elseif(!empty($formName) && !empty($formType)) {

        } elseif (!empty($forGrid)) {
            $data = $leadsFormBuilderGeneralConfigMapper->fetchAll($where, $sortOrder, $limit, $offset, false, false, $withTimezoneOffset);
        }

        if (empty($data)) {
            $data = array();
        }

        if ($forGrid && !empty($detailedInfo)) {
            $builderConfigId = $data['id'];
            $leadsBuilderFieldsMapper = Leads_Mapper_LeadsBuilderFormFieldsMapper::getInstance();
            $fieldsData = $leadsBuilderFieldsMapper->getFieldsData($builderConfigId);
            if (empty($fieldsData)) {
                $fieldsData = array();
            }
            $data['field_data'] = $fieldsData;
            $data['leadOwners'] = Leads_Mapper_LeadsMapper::getInstance()->getLeadOwnersFullList();
            $data['allUsers'] = Leads_Mapper_LeadsMapper::getInstance()->getAllUsers(true);
            $currentUser = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser();
            $data['currentUserId'] = $currentUser->getId();
            $data['userTimezoneOffset'] = Tools_System_Tools::getUtcOffset('P');
            $data['userTimezone'] = $currentUser->getTimezone();

            $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
            $userDefaultPhoneMobileCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');
            if (!empty($userDefaultPhoneMobileCountryCode)) {
                $mobilePhoneCountryCode = $userDefaultPhoneMobileCountryCode;
            } else {
                $mobilePhoneCountryCode = 'US';
            }

            $defaultCountryCode = 'US';
            $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();
            if (!empty($shoppingConfig['country'])) {
                if (empty($countryCode)) {
                    $defaultCountryCode = $shoppingConfig['country'];
                }
            }

            $additionalFieldGroupParams['defaultCountryCode'] = $defaultCountryCode;
            $additionalFieldGroupParams['defaultPhoneMobileCountryCode'] = $mobilePhoneCountryCode;

            $stateCodes = Tools_LeadTools::getStates($defaultCountryCode, true, true);

            $leadDataMapper = Leads_Mapper_LeadsMapper::getInstance();
            $tagMapper = Leads_Mapper_LeadsTagsMapper::getInstance();
            $additionalFieldGroupParams['phoneCountryCodes'] =  Tools_System_Tools::getFullCountryPhoneCodesList(true, array(), true);
            $additionalFieldGroupParams['phoneCountryCodesClearCodes'] =  Tools_System_Tools::getFullCountryPhoneCodesList(false, array(), true);
            $customerGroups = $leadDataMapper->getCustomerGroups();
            $leadTags = $tagMapper->getTagsPairs();
            asort($customerGroups);
            asort($leadTags);
            $additionalFieldGroupParams['customerGroups'] = $customerGroups;
            $additionalFieldGroupParams['countryCodes'] = Tools_Geo::getCountries(true);
            $additionalFieldGroupParams['leadTags'] = $leadTags;
            $additionalFieldGroupParams['stateCodes'] = $stateCodes;

            $emailTemplates = Leads_Mapper_LeadsMapper::getInstance()->getEmailTemplates(true);

            $data['emailTemplates'] = $emailTemplates;
            $data['additionalFieldGroupParams'] = $additionalFieldGroupParams;

            $customLeadsFields = Leads_Mapper_LeadsCustomParamsConfigMapper::getInstance()->fetchAll();
            $customOrganizationsFields = Leads_Mapper_LeadsOrganizationCustomParamsConfigMapper::getInstance()->fetchAll();

            if(!empty($customLeadsFields['data'])) {
                $data['additionalLeadsCustomFields'] = $customLeadsFields['data'];
            }
            if(!empty($customOrganizationsFields['data'])) {
                $data['additionalOrganizationsCustomFields'] = $customOrganizationsFields['data'];
            }

            $data['limitedAccess'] =  $this->_restrictedUserId;
        }

        return $data;
    }

    /**
     * New form creation
     *
     * Resource:
     * : /api/leads/formbuilder/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        if (!empty($this->_restrictedUserId)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'You don\'t have permission to add form config'));
        }

        $data = filter_var_array(json_decode($this->_request->getParam('formFieldsData'), true));


        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'));
        }

        $secureToken = $data['secureToken'];
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'invalid token'));
        }

        $leadsBuilderFormGeneralConfigMapper = Leads_Mapper_LeadsBuilderFormGeneralConfigMapper::getInstance();
        $leadsBuilderFieldsMapper = Leads_Mapper_LeadsBuilderFormFieldsMapper::getInstance();

        $formName = $data['form_name'];

        $builderFormGeneralModel = $leadsBuilderFormGeneralConfigMapper->findByFormName($formName);
        if ($builderFormGeneralModel instanceof Leads_Model_LeadsBuilderFormGeneralConfigModel) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'A form with this name already exists. Please pick another name'));
        }

        $builderFormGeneralModel = new Leads_Model_LeadsBuilderFormGeneralConfigModel();

        $createdAt = Tools_System_Tools::convertDateFromTimezone('now');
        if (empty($data['user_id'])) {
            $currentUserId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();
        } else {
            $currentUserId = $data['user_id'];
        }

        $builderFormGeneralModel->setOptions($data);
        $builderFormGeneralModel->setCreatorId($currentUserId);
        $builderFormGeneralModel->setCreatedAt($createdAt);
        $builderFormGeneralModel->setEditorId($currentUserId);
        $builderFormGeneralModel->setExternalId($data['externalId']);

        if (!empty($data['sendEmail']) && !empty($data['sendEmailConfig'])) {
            $builderFormGeneralModel->setSendEmailConfig(json_encode($data['sendEmailConfig']));
        }

        if (!empty($data['sendAutoReply']) && !empty($data['sendAutoReplyConfig'])) {
            $builderFormGeneralModel->setSendAutoReplyConfig(json_encode($data['sendAutoReplyConfig']));
        }

        $formSignature = sha1($formName . uniqid(microtime()));
        $formCode = Tools_LeadFormBuilderTools::generatePreviewCode($data['field_data'], $data['form_type'], $formSignature, $data);

        $builderFormGeneralModel->setFormCode($formCode);
        $builderFormGeneralModel->setFormSignature($formSignature);

        $builderFormGeneralModel = $leadsBuilderFormGeneralConfigMapper->save($builderFormGeneralModel);
        $formBuilderConfigId = $builderFormGeneralModel->getId();
        if (!empty($data['field_data']) && is_array($data['field_data'])) {
            foreach ($data['field_data'] as $fieldData) {
                $leadBuilderFormFieldsModel = new Leads_Model_LeadsBuilderFormFieldsModel();
                $leadBuilderFormFieldsModel->setFormId($formBuilderConfigId);
                $leadBuilderFormFieldsModel->setFieldName($fieldData['fieldName']);
                if (!empty($fieldData['fieldValue'])) {
                    $leadBuilderFormFieldsModel->setFieldValue($fieldData['fieldValue']);
                }
                if (empty($fieldData['fieldLabel'])) {
                    $fieldData['fieldLabel'] = '';
                }
                $leadBuilderFormFieldsModel->setFieldLabel($fieldData['fieldLabel']);
                $leadBuilderFormFieldsModel->setFieldType($fieldData['fieldType']);
                $leadBuilderFormFieldsModel->setFieldData(json_encode($fieldData));
                $leadBuilderFormFieldsModel->setSystemType($fieldData['systemType']);
                $leadsBuilderFieldsMapper->save($leadBuilderFormFieldsModel);
            }
        }

        $this->_responseHelper->success(array(
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Form has been created'), 'model' => $builderFormGeneralModel->toArray()
        ));

    }

    /**
     * Update form config
     *
     * Resource:
     * : /api/leads/formbuilder/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     * : lead form config id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        if (!empty($this->_restrictedUserId)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'You don\'t have permission to edit form config'));
        }

        $data = json_decode($this->_request->getRawBody(), true);
        if (!empty($data['formId']) && !empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {

            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'invalid token'));
            }

            $leadsBuilderFormGeneralConfigMapper = Leads_Mapper_LeadsBuilderFormGeneralConfigMapper::getInstance();
            $leadsBuilderFieldsMapper = Leads_Mapper_LeadsBuilderFormFieldsMapper::getInstance();

            $formId = $data['formId'];

            $builderFormGeneralModel = $leadsBuilderFormGeneralConfigMapper->find($formId);
            if (!$builderFormGeneralModel instanceof Leads_Model_LeadsBuilderFormGeneralConfigModel) {
                $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Form id is missing'));
            }

            $formName = $data['form_name'];

            $builderFormGeneralModelExisting = $leadsBuilderFormGeneralConfigMapper->findByFormName($formName);
            if ($builderFormGeneralModelExisting instanceof Leads_Model_LeadsBuilderFormGeneralConfigModel) {
                $otherFormId = $builderFormGeneralModelExisting->getId();
                if ($otherFormId != $formId) {
                    $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'A form with this name already exists. Please pick another name'));
                }
            }

            $loggedUserId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();
            $updatedAt = Tools_System_Tools::convertDateFromTimezone('now');
            if (empty($data['editor_id'])) {
                $editorUserId = $loggedUserId;
            } else {
                $editorUserId = $data['editor_id'];
            }

            $builderFormGeneralModel->setOptions($data);
            $builderFormGeneralModel->setEditorId($editorUserId);
            $builderFormGeneralModel->setUpdatedAt($updatedAt);
            $builderFormGeneralModel->setExternalId($data['externalId']);
            $formCode = Tools_LeadFormBuilderTools::generatePreviewCode($data['field_data'], $data['form_type'], $builderFormGeneralModel->getFormSignature(), $data);

            if (!empty($data['sendEmail']) && !empty($data['sendEmailConfig'])) {
                $builderFormGeneralModel->setSendEmailConfig(json_encode($data['sendEmailConfig']));
            }

            if (!empty($data['sendAutoReply']) && !empty($data['sendAutoReplyConfig'])) {
                $builderFormGeneralModel->setSendAutoReplyConfig(json_encode($data['sendAutoReplyConfig']));
            }

            $builderFormGeneralModel->setFormCode($formCode);

            $builderFormGeneralConfigModel = $leadsBuilderFormGeneralConfigMapper->save($builderFormGeneralModel);
            $formId = $builderFormGeneralConfigModel->getId();

            $leadsBuilderFieldsMapper->deleteByFormId($formId);

            $formBuilderConfigId = $builderFormGeneralModel->getId();
            if (!empty($data['field_data']) && is_array($data['field_data'])) {
                foreach ($data['field_data'] as $fieldData) {
                    $leadBuilderFormFieldsModel = new Leads_Model_LeadsBuilderFormFieldsModel();
                    $leadBuilderFormFieldsModel->setFormId($formBuilderConfigId);
                    $leadBuilderFormFieldsModel->setFieldName($fieldData['fieldName']);
                    if (!empty($fieldData['fieldValue'])) {
                        $leadBuilderFormFieldsModel->setFieldValue($fieldData['fieldValue']);
                    }
                    if (empty($fieldData['fieldLabel'])) {
                        $fieldData['fieldLabel'] = '';
                    }
                    $leadBuilderFormFieldsModel->setFieldLabel($fieldData['fieldLabel']);
                    $leadBuilderFormFieldsModel->setFieldData(json_encode($fieldData));
                    $leadBuilderFormFieldsModel->setFieldType($fieldData['fieldType']);
                    $leadBuilderFormFieldsModel->setSystemType($fieldData['systemType']);
                    $leadsBuilderFieldsMapper->save($leadBuilderFormFieldsModel);
                }
            }

            $this->_responseHelper->success(array(
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Form has been updated'), 'model' => $builderFormGeneralModel->toArray()
            ));
        }

    }

    /**
     * Delete form config
     *
     * Resource:
     * : /api/leads/formbuilder/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     * : form config id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        if (!empty($this->_restrictedUserId)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'You don\'t have permission to delete form config'));
        }

        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        if (!$id) {
            $this->_error();
        }
        $leadsBuilderFormGeneralConfigConfigMapper = Leads_Mapper_LeadsBuilderFormGeneralConfigMapper::getInstance();
        $leadsBuilderFormGeneralConfigModel = $leadsBuilderFormGeneralConfigConfigMapper->find($id);

        if ($leadsBuilderFormGeneralConfigModel instanceof Leads_Model_LeadsBuilderFormGeneralConfigModel) {
            $leadsBuilderFormGeneralConfigConfigMapper->delete($id);
        } else {
            $this->_error();
        }

    }

}
