<?php

class Api_Leads_Formbuilderconfig extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    /**
     * User restricted id access
     *
     * @var null
     */
    protected $_restrictedUserId = null;

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
    }


    /**
     * Get event by id
     *
     * Resource:
     * : /api/leads/formbuilderconfig/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order'), FILTER_SANITIZE_STRING);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $forGrid = (bool)filter_var($this->_request->getParam('isGrid', false), FILTER_SANITIZE_STRING);

        $additionalFieldGroupParams = array();
        if ($forGrid) {
            $leadDataMapper = Leads_Mapper_LeadsMapper::getInstance();
            $tagMapper = Leads_Mapper_LeadsTagsMapper::getInstance();
            $data['leadOwners'] = Leads_Mapper_LeadsMapper::getInstance()->getLeadOwnersFullList();
            $currentUser = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser();
            $data['currentUserId'] = $currentUser->getId();
            $additionalFieldGroupParams['phoneCountryCodes'] =  Tools_System_Tools::getFullCountryPhoneCodesList(true, array(), true);
            $additionalFieldGroupParams['phoneCountryCodesClearCodes'] =  Tools_System_Tools::getFullCountryPhoneCodesList(false, array(), true);
            $customerGroups = $leadDataMapper->getCustomerGroups();
            $leadTags = $tagMapper->getTagsPairs();
            asort($customerGroups);
            asort($leadTags);
            $additionalFieldGroupParams['customerGroups'] = $customerGroups;
            $additionalFieldGroupParams['countryCodes'] = Tools_Geo::getCountries(true);
            $additionalFieldGroupParams['leadTags'] = $leadTags;

            $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
            $userDefaultPhoneMobileCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');
            if (!empty($userDefaultPhoneMobileCountryCode)) {
                $mobilePhoneCountryCode = $userDefaultPhoneMobileCountryCode;
            } else {
                $mobilePhoneCountryCode = 'US';
            }

            $defaultCountryCode = 'US';
            $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();
            if (!empty($shoppingConfig['country'])) {
                if (empty($countryCode)) {
                    $defaultCountryCode = $shoppingConfig['country'];
                }
            }

            $additionalFieldGroupParams['defaultCountryCode'] = $defaultCountryCode;
            $additionalFieldGroupParams['defaultPhoneMobileCountryCode'] = $mobilePhoneCountryCode;

            $emailTemplates = Leads_Mapper_LeadsMapper::getInstance()->getEmailTemplates(true);

            $stateCodes = Tools_LeadTools::getStates($defaultCountryCode, true, true);
            $additionalFieldGroupParams['stateCodes'] = $stateCodes;
            $data['additionalFieldGroupParams'] = $additionalFieldGroupParams;
            $data['emailTemplates'] = $emailTemplates;

            $customLeadsFields = Leads_Mapper_LeadsCustomParamsConfigMapper::getInstance()->fetchAll();
            $customOrganizationsFields = Leads_Mapper_LeadsOrganizationCustomParamsConfigMapper::getInstance()->fetchAll();

            if(!empty($customLeadsFields['data'])) {
                $data['additionalLeadsCustomFields'] = $customLeadsFields['data'];
            } else {
                $data['additionalLeadsCustomFields'] = array();
            }
            if(!empty($customOrganizationsFields['data'])) {
                $data['additionalOrganizationsCustomFields'] = $customOrganizationsFields['data'];
            } else {
                $data['additionalOrganizationsCustomFields'] = array();
            }

            $data['limitedAccess'] =  $this->_restrictedUserId;

        }

        return $data;
    }

    public function postAction()
    {

    }

    public function putAction()
    {

    }

    public function deleteAction()
    {

    }

}
