<?php

class Api_Leads_Imaprules extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     *
     * Resource:
     * : /api/leads/imaprules/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $where = '';

        $leadsImapRulesConfigMapper = Leads_Mapper_LeadsImapRulesConfigMapper::getInstance();
        if (!empty($id)) {
            $where = $leadsImapRulesConfigMapper->getDbTable()->getAdapter()->quoteInto('plirc.id = ?', $id);
            $data = $leadsImapRulesConfigMapper->fetchAll($where, null, null, null, true, true);
            if (!empty($data)) {
                $data['subRulesConfig'] = json_decode($data['subRulesConfig'], true);
            }
        }  else {
            $data = $leadsImapRulesConfigMapper->fetchAll($where, null, $limit, $offset, false, false);
        }

        $leadImapDataMapper = Leads_Mapper_LeadsImapConfigMapper::getInstance();
        $imapRules = $leadImapDataMapper->getRules();
        $data['additionalInfo']['imapRules'] = $imapRules;
        $data['additionalInfo']['assignToAttributes'] = Tools_LeadInboundEmailTools::getListOfAssignToAttributes();

        return $data;
    }

    /**
     *
     * Resource:
     * : /api/leads/imaprules/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING);
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'));
        }

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
        }

        $leadsImapRulesConfigMapper = Leads_Mapper_LeadsImapRulesConfigMapper::getInstance();
//        $leadsImapRulesConfigModel = $leadsImapRulesConfigMapper->findByImapConfigId($data['imapConfigId']);
//        if ($leadsImapRulesConfigModel instanceof Leads_Model_LeadsImapRulesConfigModel) {
//            return array(
//                'error' => '1',
//                'message' => $translator->translate('You already have a rule for this imap config')
//            );
//        }

        if (!empty($data['ruleEmail'])) {
            $isEmailValid = Tools_System_Tools::isEmailValid($data['ruleEmail']);
            if ($isEmailValid === false) {
                return array(
                    'error' => '1',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Wrong from email format')
                );
            }
        }

        $session = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
        $userId = $session->getCurrentUser()->getId();

        $leadsImapRulesConfigModel = new Leads_Model_LeadsImapRulesConfigModel();
        $leadsImapRulesConfigModel->setCreatedAt(Tools_System_Tools::convertDateFromTimezone('now'));
        $leadsImapRulesConfigModel->setUpdatedAt(null);
        $leadsImapRulesConfigModel->setRuleName($data['ruleName']);
        $leadsImapRulesConfigModel->setCreatorId($userId);
        $leadsImapRulesConfigModel->setImapConfigId($data['imapConfigId']);
        $leadsImapRulesConfigModel->setSubRulesConfig(json_encode($data['ruleElements']));
        $leadsImapRulesConfigModel->setRuleEmail($data['ruleEmail']);
        $leadsImapRulesConfigModel->setEditorId(null);
        $leadsImapRulesConfigMapper->save($leadsImapRulesConfigModel);

        return array(
            'error' => '0',
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Rule has been added')
        );
    }

    /**
     *
     * Resource:
     * : /api/leads/imaprules/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     * : lead tag id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if (!empty($data['id']) && !empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {
            $translator = Zend_Registry::get('Zend_Translate');
            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();
                $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
            }

            if (!empty($data['ruleEmail'])) {
                $isEmailValid = Tools_System_Tools::isEmailValid($data['ruleEmail']);
                if ($isEmailValid === false) {
                    return array(
                        'error' => '1',
                        'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Wrong from email format')
                    );
                }
            }

            $leadsImapRulesConfigMapper = Leads_Mapper_LeadsImapRulesConfigMapper::getInstance();

            $leadsImapRulesConfigModel = $leadsImapRulesConfigMapper->find($data['id']);
            if ($leadsImapRulesConfigModel instanceof Leads_Model_LeadsImapRulesConfigModel) {
                $session = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
                $editorId = $session->getCurrentUser()->getId();

                $leadsImapRulesConfigModel->setUpdatedAt(Tools_System_Tools::convertDateFromTimezone('now'));
                $leadsImapRulesConfigModel->setRuleName($data['ruleName']);
                $leadsImapRulesConfigModel->setImapConfigId($data['imapConfigId']);
                $leadsImapRulesConfigModel->setSubRulesConfig(json_encode($data['ruleElements']));
                $leadsImapRulesConfigModel->setRuleEmail($data['ruleEmail']);
                $leadsImapRulesConfigModel->setEditorId($editorId);
                $leadsImapRulesConfigMapper->save($leadsImapRulesConfigModel);

                return array(
                    'error' => '0',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Rule has been updated')
                );

            }
        }

        return array(
            'error' => '1',
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Rule not found')
        );

    }

    /**
     *
     * Resource:
     * : /api/leads/imaprules/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     * : lead tag id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        if (!$id) {
            $this->_error();
        }
        $translator = Zend_Registry::get('Zend_Translate');
        $leadsImapRulesConfigMapper = Leads_Mapper_LeadsImapRulesConfigMapper::getInstance();
        $leadsImapRulesConfigModel = $leadsImapRulesConfigMapper->find($id);
        if ($leadsImapRulesConfigModel instanceof Leads_Model_LeadsImapRulesConfigModel) {
            $leadsImapRulesConfigMapper->delete($id);
            return array(
                'error' => '0',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Rule has been deleted')
            );
        } else {
            $this->_error();
        }

    }

}
