<?php

/**
 * Class Api_Leads_Leadadvertiseraccountsconfig
 */
class Api_Leads_Leadadvertiseraccountsconfig extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * System session helper
     *
     * @var null
     */
    protected $_sessionHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }

    /**
     *
     * Resource:
     * : /api/leads/leadadvertiseraccountsconfig/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $limit = filter_var($this->_request->getParam('limit', null), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset', null), FILTER_SANITIZE_NUMBER_INT);
        $order = filter_var($this->_request->getParam('order', 'plaac.adv_id DESC'), FILTER_SANITIZE_STRING);

        $leadsAdvertiserAccountsConfigMapper = Leads_Mapper_LeadsAdvertiserAccountsConfigMapper::getInstance();


        $servicesIds = array();
        foreach (Tools_LeadTools::$_advertisingTypes as $advKey => $advAccount) {
            $services = Tools_LeadTools::sambastatsConnectionStatusGet($advKey);

            /*$services = array(
                    'error' => 0,
                    'data' => array(
                        'status' => '1',
                        //'status' => '0',
                        'adAccountId' => '1234567',
                        'name' => 'Some company name',
                        'canDelete' => true,
                        'connectionStatus' => 1 //0 //check if account connected
                    ),
                    'link' => 'http://google.com/');*/

            /*$services = array(
                'error' => 1,
                'message' => 'Website is not connected'
            );*/

            if(!empty($services['error']) || $services['done'] === false) {
                $mojoConnectionError = false;
                if($services['done'] === false) {
                    $mojoConnectionError = true;
                }
                $this->_error(array('status' => 'error', 'message' => $services['message'], 'errorMojoConnection' => $mojoConnectionError,  'errorCode' => '1'), 200);
            }

            if (!empty($services['data'])) {
                $account = $leadsAdvertiserAccountsConfigMapper->findByAdvId($services['data']['adAccountId']);

                $servicesIds[$advKey] = $services['data']['adAccountId'];

                $accountStatus = 'pending';

                if(!empty($services['data']['status'])) {
                    $accountStatus = 'connected';
                }

                if(empty($services['data']['connectionStatus'])) {
                    $accountStatus = 'disconnected';
                }

                if(!$account instanceof Leads_Model_LeadsAdvertiserAccountsConfigModel) {
                    $account = new Leads_Model_LeadsAdvertiserAccountsConfigModel();
                }

                $account->setAdvStatus($accountStatus);
                $account->setAdvName($services['data']['name']);
                $account->setAdvType($advKey);
                $account->setAdvId($services['data']['adAccountId']);
                $account->setAdvVerificationLink($services['link']);

                $canDelete = 0;
                if($services['data']['canDelete']) {
                    $canDelete = 1;
                }

                $account->setAllowDelete($canDelete);

                $leadsAdvertiserAccountsConfigMapper->save($account);

            } elseif(empty($services['data'])) {
                $leadsAdvertiserAccountsConfigMapper->deleteByAccountType($advKey);
            }
        }

        $advertiserAccountsConfigData = $leadsAdvertiserAccountsConfigMapper->fetchAll(null, $order, $limit, $offset);

        //$advertiserAccountsConfigData['data'] = array();
        //$advertiserAccountsConfigData['totalRecords'] = 0;

        $advertiserTypes = $leadsAdvertiserAccountsConfigMapper->getAdvTypes();

        $advertiserTypesData = array();
        if(!empty($advertiserTypes)) {
            foreach (Tools_LeadTools::$_advertisingTypes as $key => $type) {
                if(in_array($key, $advertiserTypes)) {
                    if(!empty($servicesIds[$key])) {
                        $type .= ' (' . $servicesIds[$key] . ')';
                    }
                    $advertiserTypesData[$key] = $type;
                }
            }
        }

        return array('advertiserAccountsConfig' => $advertiserAccountsConfigData, 'advertiserTypes' => $advertiserTypesData, 'status' => 'ok');
    }

    /**
     *
     * * * Resource:
     * : /api/leads/leadadvertiseraccountsconfig/
     *
     * @return array
     */
    public function postAction()
    {
        $translator  = Zend_Registry::get('Zend_Translate');
        $secureToken = $this->_request->getParam('secureToken', false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken,Leads::LEADS_SECURE_TOKEN);
        if ($tokenValid !== true) {
            $this->_error(array('status' => 'error', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Access denied'), 'errorCode' => '1'), 200);
        }
        $adAccountId = filter_var($this->_request->getParam('adAccountId', false), FILTER_SANITIZE_STRING);
        $account = filter_var($this->_request->getParam('account', false), FILTER_SANITIZE_STRING);

        if(!empty($account) && !empty($adAccountId)) {
            $leadsAdvertiserAccountsConfigMapper = Leads_Mapper_LeadsAdvertiserAccountsConfigMapper::getInstance();

            $advAccount = $leadsAdvertiserAccountsConfigMapper->getByIdType($adAccountId, $account);

            if($advAccount instanceof Leads_Model_LeadsAdvertiserAccountsConfigModel) {
                $this->_error(array('status' => 'error', 'message' => $account .' already exist', 'errorCode' => '1'), 200);
            } else {
                $services = Apps::apiCall('POST', 'sambastatsConnectionRequest', array(), array('type' => $account, 'adAccountId' => $adAccountId));

                /*$services = array(
                    'error' => 1,
                    'message' => 'Website is not connected'
                );*/

                /*$services = array(
                    'error' => 0,
                    'data' => array(
                        'status' => '0',
                        'adAccountId' => '1234567',
                        'name' => 'Some company name'),
                    'link' => 'http://google.com/');*/

                if(!empty($services['error'])) {
                    $this->_error(array('status' => 'error', 'message' => $services['message'], 'errorCode' => '1'), 200);
                } else {
                    $leadsAdvertiserAccountsConfigModel = new Leads_Model_LeadsAdvertiserAccountsConfigModel();

                    $leadsAdvertiserAccountsConfigModel->setAdvId($services['data']['adAccountId']);
                    $leadsAdvertiserAccountsConfigModel->setAdvName($services['data']['name']);
                    $leadsAdvertiserAccountsConfigModel->setAdvType($account);

                    if($services['data']['status']) {
                        $leadsAdvertiserAccountsConfigModel->setAdvStatus('connected');
                    } else {
                        $leadsAdvertiserAccountsConfigModel->setAdvStatus('pending');
                    }
                    $leadsAdvertiserAccountsConfigModel->setAdvVerificationLink($services['link']);

                    $leadsAdvertiserAccountsConfigMapper->save($leadsAdvertiserAccountsConfigModel);
                }

                return array('error' => '0', 'accountVerificationLink' => $services['link'], 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Remarketing campaign has been saved.'));
            }
        } else {
            $this->_error(array('status' => 'error', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Make sure Ad account nb field is not empty instead!'), 'errorCode' => '1'), 200);
        }
    }

    /**
     *
     * * * Resource:
     * : /api/leads/leadadvertiseraccountsconfig/
     *
     *
     */
    public function putAction()
    {

    }

    /**
     *
     * * Resource:
     * : /api/leads/leadadvertiseraccountsconfig/
     *
     * @return array
     */
    public function deleteAction()
    {
        $translator  = Zend_Registry::get('Zend_Translate');
        $secureToken = $this->_request->getParam('secureToken', false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken,Leads::LEADS_SECURE_TOKEN);
        if ($tokenValid !== true) {
            $this->_error(array('status' => 'error', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Access denied'), 'errorCode' => '1'), 200);
        }

        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);

        $leadsAdvertiserAccountsConfigMapper = Leads_Mapper_LeadsAdvertiserAccountsConfigMapper::getInstance();
        $leadsAdvertiserAccountsConfigModel = $leadsAdvertiserAccountsConfigMapper->find($id);
        if (!$leadsAdvertiserAccountsConfigModel instanceof Leads_Model_LeadsAdvertiserAccountsConfigModel) {
            $this->_error(array('status' => 'error', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Can\'t delete advertiser configuration'), 'errorCode' => '1'), 200);
        }

        $services = Apps::apiCall('DELETE', 'sambastatsConnectionRequest', array(), array(
            'type' => $leadsAdvertiserAccountsConfigModel->getAdvType(),
            'adAccountId' => $leadsAdvertiserAccountsConfigModel->getAdvId()
        ));

        /*$services = array(
                    'error' => 0,
                    'message' => 'Success'
                );*/

        if(!empty($services['error'])) {
            $this->_error(array('status' => 'error', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'You Can\'t delete advertiser account!'), 'errorCode' => '1'), 200);
        }

        $leadsAdvertiserAccountsConfigMapper->delete($id);

        return array('error' => '0', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Record has been deleted'));
    }

}
