<?php

class Api_Leads_Leadapiaccess extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * System session helper
     *
     * @var null
     */
    protected $_sessionHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_sessionHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
    }

    /**
     *
     * Resource:
     * : /api/leads/leadapiaccess/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $limit = filter_var($this->_request->getParam('limit', null), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset', null), FILTER_SANITIZE_NUMBER_INT);
        $order = filter_var($this->_request->getParam('order', 'pluac.id DESC'), FILTER_SANITIZE_STRING);

        $leadsUsersApiAccessMapper = Leads_Mapper_LeadsUsersApiAccessMapper::getInstance();

        $apiAccessData = $leadsUsersApiAccessMapper->getConfigInfo(null, $order, $limit, $offset);
        $allUsers = Leads_Mapper_LeadsMapper::getInstance()->getAllUsers(true, true);
        $apiAccessData['usersList'] = $allUsers;


        return array('apiAccessData' => $apiAccessData, 'status' => 'ok');
    }

    /**
     * Save custom tab fields
     *
     * * * Resource:
     * : /api/leads/leadapiaccess/
     *
     * @return array
     */
    public function postAction()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        $secureToken = $this->_request->getParam('secureToken', false);
        $userId = filter_var($this->_request->getParam('userId', false), FILTER_SANITIZE_NUMBER_INT);
        $ipAddresses = trim(filter_var($this->_request->getParam('ipAddresses', false), FILTER_SANITIZE_STRING));
        $tokenValid = Tools_System_Tools::validateToken($secureToken, Leads::LEADS_SECURE_TOKEN);
        if ($tokenValid !== true) {
            $this->_error(array(
                'status' => 'error',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Access denied'),
                'errorCode' => '1'
            ), 200);
        }

        if (empty($userId)) {
            $this->_error(array(
                'status' => 'error',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'User id missing'),
                'errorCode' => '1'
            ), 200);
        }

        $userModel = Application_Model_Mappers_UserMapper::getInstance()->find($userId);
        if (!$userModel instanceof Application_Model_Models_User) {
            $this->_error(array(
                'status' => 'error',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'User not found'),
                'errorCode' => '1'
            ), 200);
        }

        $leadsUsersApiAccessMapper = Leads_Mapper_LeadsUsersApiAccessMapper::getInstance();
        $leadsUsersApiAccessModel = $leadsUsersApiAccessMapper->findByUserId($userId);
        if ($leadsUsersApiAccessModel instanceof Leads_Model_LeadsUsersApiAccessModel) {
            $this->_error(array(
                'status' => 'error',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'User already has api access'),
                'errorCode' => '1'
            ), 200);
        }
        $leadsUsersApiAccessModel = new Leads_Model_LeadsUsersApiAccessModel();
        $leadsUsersApiAccessModel->setUserId($userId);
        $leadsUsersApiAccessModel->setIpAddresses($ipAddresses);
        $leadsUsersApiAccessMapper->save($leadsUsersApiAccessModel);

        return array('error' => '0', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Access has been added.'));

    }

    public function putAction()
    {

    }

    /**
     * Delete record
     *
     * * * Resource:
     * : /api/leads/leadapiaccess/
     *
     * @return array
     */
    public function deleteAction()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        $secureToken = $this->_request->getParam('secureToken', false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, Leads::LEADS_SECURE_TOKEN);
        if ($tokenValid !== true) {
            $this->_error(array(
                'status' => 'error',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Access denied'),
                'errorCode' => '1'
            ), 200);
        }

        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);

        $leadsUsersApiAccessMapper = Leads_Mapper_LeadsUsersApiAccessMapper::getInstance();
        $leadsUsersApiAccessModel = $leadsUsersApiAccessMapper->find($id);
        if (!$leadsUsersApiAccessModel instanceof Leads_Model_LeadsUsersApiAccessModel) {
            $this->_error(array(
                'status' => 'error',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Record not found'),
                'errorCode' => '1'
            ), 200);
        }
        $leadsUsersApiAccessMapper->delete($id);

        return array('error' => '0', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Record has been deleted'));
    }

}
