<?php

class Api_Leads_Leadcall extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * System session helper
     *
     * @var null
     */
    protected $_sessionHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_sessionHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
    }


    /**
     *
     * Resource:
     * : /api/leads/leadcall/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

    }

    /**
     *
     * Resource:
     * : /api/leads/leadcall/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {

    }

    /**
     *
     * Resource:
     * : /api/leads/leadcall/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if (!empty($data['id']) && !empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {
            $translator = Zend_Registry::get('Zend_Translate');
            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();
                $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
            }

            $callLogMapper = Leads_Mapper_LeadsCallLogMapper::getInstance();
            $callLogModel = $callLogMapper->find($data['id']);
            if (!$callLogModel instanceof Leads_Model_LeadsCallLogModel) {
                $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Wrong id'));
            }
            $callLogModel->setDetails($data['details']);
            $callLogMapper->save($callLogModel);
            $type = $callLogModel->getType();
            $this->_responseHelper->success($translator->translate(ucfirst($type). 'log updated'));
        }
        $this->_error('Wrong params');
    }

    /**
     * Delete lead lead call
     *
     * Resource:
     * : /api/leads/leadcall/
     *
     * HttpMethod:
     * : DELETE
     *
     *
     * @return JSON
     */
    public function deleteAction()
    {

    }

}
