<?php

class Api_Leads_Leadcustomtabsconfig extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * System session helper
     *
     * @var null
     */
    protected $_sessionHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_sessionHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
    }

    /**
     *
     * Resource:
     * : /api/leads/leadcustomtabsconfig/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $limit = filter_var($this->_request->getParam('limit', null), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset', null), FILTER_SANITIZE_NUMBER_INT);
        $order = filter_var($this->_request->getParam('order', 'lct.tab_name DESC'), FILTER_SANITIZE_STRING);

        $leadsCustomTabsConfigMapper = Leads_Mapper_LeadsCustomTabsConfigMapper::getInstance();

        $customTabsData = $leadsCustomTabsConfigMapper->getCustomTabsConfigInfo(null, $order, $limit, $offset);

        return array('customTabsData' => $customTabsData, 'status' => 'ok');
    }

    /**
     * Save custom tab
     *
     * * * Resource:
     * : /api/leads/leadcustomtabsconfig/
     *
     * @return array
     */
    public function postAction()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        $secureToken = $this->_request->getParam('secureToken', false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken,Leads::LEADS_SECURE_TOKEN);
        if ($tokenValid !== true) {
            $this->_error(array('status' => 'error', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Access denied'), 'errorCode' => '1'), 200);
        }
        $customTabFieldType = filter_var($this->_request->getParam('customTabFieldType', false), FILTER_SANITIZE_STRING);
        $customTabName = filter_var($this->_request->getParam('customTabName', false), FILTER_SANITIZE_STRING);
        $customTabLabel = filter_var($this->_request->getParam('customTabLabel', false), FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES);

        if(!empty($customTabName) && !empty($customTabFieldType)) {
            $leadsCustomTabsConfigMapper = Leads_Mapper_LeadsCustomTabsConfigMapper::getInstance();

            $foundCustomTab = $leadsCustomTabsConfigMapper->findByTabNameAndByTabType($customTabName, $customTabFieldType);

            if($foundCustomTab instanceof Leads_Model_LeadsCustomTabsConfigModel) {
                $this->_error(array('status' => 'error', 'message' => $customTabName . ' ' . $translator->translate(Leads::TRANSLATION_PREFIX.'already exist'), 'errorCode' => '1'), 200);
            } else {
                $leadsCustomTabsModel = new Leads_Model_LeadsCustomTabsConfigModel();

                $leadsCustomTabsModel->setTabName($customTabName);
                $leadsCustomTabsModel->setTabLabel($customTabLabel);
                $leadsCustomTabsModel->setTabType($customTabFieldType);

                $leadsCustomTabsConfigMapper->save($leadsCustomTabsModel);
            }

            return array('error' => '0', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Custom tab has been saved.'));
        } else {
            $this->_error(array('status' => 'error', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Can\'t create tab'), 'errorCode' => '1'), 200);
        }
    }

    /**
     * Update custom tab
     *
     * * * Resource:
     * : /api/leads/leadcustomtabsconfig/
     *
     *
     */
    public function putAction()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        $data = json_decode($this->_request->getRawBody(), true);

        $tokenValid = Tools_System_Tools::validateToken($data['secureToken'],Leads::LEADS_SECURE_TOKEN);
        if ($tokenValid !== true) {
            $this->_error(array('status' => 'error', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Access denied'), 'errorCode' => '1'), 200);
        }

        $id = filter_var($data['id'], FILTER_SANITIZE_NUMBER_INT);
        $tabLabel = filter_var($data['tabLabel'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES);

        if(!empty($id) && !empty($tabLabel)) {
            $leadsCustomTabsConfigMapper = Leads_Mapper_LeadsCustomTabsConfigMapper::getInstance();

            $foundCustomTab = $leadsCustomTabsConfigMapper->find($id);

            if($foundCustomTab instanceof Leads_Model_LeadsCustomTabsConfigModel) {
                $foundCustomTab->setTabLabel($tabLabel);

                $leadsCustomTabsConfigMapper->save($foundCustomTab);

                return array('error' => '0', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Custom tab has been updated'));
            }
        }

        $this->_error(array('status' => 'error', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Can\'t update tab label'), 'errorCode' => '1'), 200);

    }

    /**
     * Delete custom tab
     *
     * * Resource:
     * : /api/leads/leadcustomtabsconfig/
     *
     * @return array
     */
    public function deleteAction()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        $secureToken = $this->_request->getParam('secureToken', false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken,Leads::LEADS_SECURE_TOKEN);
        if ($tokenValid !== true) {
            $this->_error(array('status' => 'error', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Access denied'), 'errorCode' => '1'), 200);
        }

        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);

        $leadsCustomTabsConfigMapper = Leads_Mapper_LeadsCustomTabsConfigMapper::getInstance();
        $leadsCustomTabsConfigModel = $leadsCustomTabsConfigMapper->find($id);
        if (!$leadsCustomTabsConfigModel instanceof Leads_Model_LeadsCustomTabsConfigModel) {
            $this->_error(array('status' => 'error', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Can\'t delete tab'), 'errorCode' => '1'), 200);
        }
        $leadsCustomTabsConfigMapper->delete($id);

        return array('error' => '0', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Record has been deleted'));
    }

}
