<?php

class Api_Leads_Leadcustomtabsfields extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    const CUSTOM_TAB_TYPE_LEAD = 'lead';

    const CUSTOM_TAB_TYPE_ORGANIZATION = 'organization';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * System session helper
     *
     * @var null
     */
    protected $_sessionHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_sessionHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
    }

    /**
     *
     * Resource:
     * : /api/leads/leadcustomtabsfields/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $secureToken = $this->_request->getParam('secureToken', false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken,Leads::LEADS_SECURE_TOKEN);
        if ($tokenValid !== true) {
            $this->_error(array('status' => 'error', 'message' => 'Access denied', 'errorCode' => '1'), 200);
        }

        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);

        $leadsCustomTabsConfigMapper = Leads_Mapper_LeadsCustomTabsConfigMapper::getInstance();

        if(!empty($id)) {
            $foundCustomTab = $leadsCustomTabsConfigMapper->find($id);

            if($foundCustomTab instanceof Leads_Model_LeadsCustomTabsConfigModel) {
                $tabType = $foundCustomTab->getTabType();

                $leadsCustomTabsHasCustomParamsMapper = Leads_Mapper_LeadsCustomTabsHasCustomParamsMapper::getInstance();

                if(!empty($tabType)) {
                    $leadsCustomParamsResult = array();
                    $leadsCustomParams = array();
                    $selectedCustomParamsData = array();

                    if($tabType == self::CUSTOM_TAB_TYPE_LEAD) {
                        $leadsCustomParamsMapper = Leads_Mapper_LeadsCustomParamsConfigMapper::getInstance();
                        $leadsCustomParams = $leadsCustomParamsMapper->getCustomParamsPairs();
                    } elseif ($tabType == self::CUSTOM_TAB_TYPE_ORGANIZATION) {
                        $leadsOrganizationCustomParamsMapper = Leads_Mapper_LeadsOrganizationCustomParamsConfigMapper::getInstance();
                        $leadsCustomParams = $leadsOrganizationCustomParamsMapper->getCustomParamsPairs();
                    }

                    if($tabType == self::CUSTOM_TAB_TYPE_LEAD || $tabType == self::CUSTOM_TAB_TYPE_ORGANIZATION) {
                        $selectedCustomParamsData = $leadsCustomTabsHasCustomParamsMapper->getSelectedCustomTabsParams($id, $tabType);
                    }

                    if(!empty($leadsCustomParams)) {
                        foreach ($leadsCustomParams as $key => $param) {
                            $checked = false;

                            if(array_key_exists($key, $selectedCustomParamsData)) {
                                $checked = true;
                            }

                            $leadsCustomParamsResult[$key] = array('id' => $key, 'name' => $param, 'checked' => $checked);
                        }
                        $leadsCustomParams = $leadsCustomParamsResult;
                    }

                    return array('tabType' => $tabType, 'customParams' => $leadsCustomParams ,'selectedCustomParamsData' => $selectedCustomParamsData, 'status' => 'ok');
                }
            }
        }

        $this->_error(array('status' => 'error', 'message' => 'Can\'t edit tab', 'errorCode' => '1'), 200);
    }

    /**
     * Save custom tab fields
     *
     * * * Resource:
     * : /api/leads/leadcustomtabsfields/
     *
     * @return array
     */
    public function postAction()
    {
        $secureToken = $this->_request->getParam('secureToken', false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken,Leads::LEADS_SECURE_TOKEN);
        if ($tokenValid !== true) {
            $this->_error(array('status' => 'error', 'message' => 'Access denied', 'errorCode' => '1'), 200);
        }
        $customTabId = filter_var($this->_request->getParam('customTabId', false), FILTER_SANITIZE_STRING);
        $customFieldId = filter_var($this->_request->getParam('customFieldId', false), FILTER_SANITIZE_STRING);
        $tabType = filter_var($this->_request->getParam('tabType', false), FILTER_SANITIZE_STRING);

        $leadsCustomTabsHasCustomParamsMapper = Leads_Mapper_LeadsCustomTabsHasCustomParamsMapper::getInstance();

        if(!empty($customTabId) && !empty($customFieldId) && ($tabType == self::CUSTOM_TAB_TYPE_LEAD || $tabType == self::CUSTOM_TAB_TYPE_ORGANIZATION)) {
            $customParamData = $leadsCustomTabsHasCustomParamsMapper->getTabCustomParams($customTabId, $customFieldId);

            if($customParamData instanceof Leads_Model_LeadsCustomTabsHasCustomParamsModel) {
                $leadsCustomTabsHasCustomParamsMapper->delete($customTabId, $customFieldId);

                return array('error' => '0', 'message' => 'deleted');
            } else {
                $leadsCustomTabsHasCustomParamsModel = new Leads_Model_LeadsCustomTabsHasCustomParamsModel();

                $leadsCustomTabsHasCustomParamsModel->setTabId($customTabId);
                $leadsCustomTabsHasCustomParamsModel->setCustomParamId($customFieldId);

                $leadsCustomTabsHasCustomParamsMapper->save($leadsCustomTabsHasCustomParamsModel);

                return array('error' => '0', 'message' => 'updated');
            }
        }

        $this->_error(array('status' => 'error', 'message' => 'Can\'t update custom params', 'errorCode' => '1'), 200);
    }

    public function putAction()
    {

    }

    public function deleteAction()
    {

    }

}
