<?php

class Api_Leads_Leaddocuments extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }

    public function getAction()
    {
        $leadId = filter_var($this->_request->getParam('lead_id'), FILTER_SANITIZE_NUMBER_INT);
        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order', 'pldoc.id'), FILTER_SANITIZE_STRING);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);

        if(!empty($leadId)) {
            $leadsDocumentsMapper = Leads_Mapper_LeadsDocumentsMapper::getInstance();

            $where = $leadsDocumentsMapper->getDbTable()->getAdapter()->quoteInto('pldoc.lead_id = ?', $leadId);
            $documentsData = $leadsDocumentsMapper->fetchAllData($where, $sortOrder, $limit, $offset);

            return $documentsData;
        }
    }

    public function postAction()
    {
        $data = filter_var_array($this->getRequest()->getPost(), FILTER_SANITIZE_STRING);
        if (empty($data) || empty($data['leadId'])) {
            $this->_error();
        }

        if(!Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $this->_error();
        }

        $tokenToValidate = $this->_request->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $valid = Tools_System_Tools::validateToken($tokenToValidate,
            self::LEADS_SECURE_TOKEN);
        if (!$valid) {
            $this->_error();
        }

        $translator = Zend_Registry::get('Zend_Translate');
        $sessionHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('session');

        $userRoleId = $sessionHelper->getCurrentUser()->getRoleId();
        $leadId = filter_var($data['leadId'], FILTER_SANITIZE_NUMBER_INT);

        $isAllow = true;
        if($userRoleId == Shopping::ROLE_SALESPERSON) {
            $isAllow = Tools_LeadTools::allowSales($leadId);
        }

        if(!$isAllow) {
            return array('error' => 1, 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Permission deny'));
        }

        $leadExists = Leads_Mapper_LeadsMapper::getInstance()->find($leadId);
        if (!$leadExists instanceof Leads_Model_LeadsModel) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Lead not exists'));
        }

        $uploader = new Zend_File_Transfer_Adapter_Http();
        $uploader->clearValidators();
        $uploader->clearFilters();
        $fileInfo = $uploader->getFileInfo();
        if(!empty($fileInfo)) {
            $file = reset($fileInfo);

            if(!empty($file)) {
                $storedData = Tools_LeadDocumentsTools::generateStoredName($file);

                if(isset($storedData['error'])) {
                    return array('error' => 1, 'message' => $storedData['error']['result'], 'fileName' => $file['name']);
                }

                $savePath = Tools_LeadDocumentsTools::getFilePath($storedData['fileStoredName']);
                $uploader->addFilter('Rename', array(
                    'target' => $savePath,
                    'overwrite' => true
                ));
                $acceptedFileTypes = Tools_LeadDocumentsTools::$_acceptedFileTypes;
                $acceptedMimeTypes = Tools_LeadDocumentsTools::$_acceptedFilesMimeTypes;

                //Adding file extension validation
                $uploader->addValidator('Extension', false, $acceptedFileTypes);
                //Adding mime types validation
                $uploader->addValidator('MimeType', true, $acceptedMimeTypes);

                if ($uploader->isUploaded() && $uploader->isValid()) {
                    try {
                        $uploader->receive();
                        $leadsDocumentsMapper = Leads_Mapper_LeadsDocumentsMapper::getInstance();
                        $leadsDocumentsModel = new Leads_Model_LeadsDocumentsModel();
                        $leadsDocumentsModel->setLeadId($leadId);
                        $leadsDocumentsModel->setFileStoredName($storedData['fileStoredName']);
                        $leadsDocumentsModel->setFileHash($storedData['fileHash']);
                        $leadsDocumentsModel->setOriginalFileName($storedData['fileName'] . '.' . $storedData['fileExtension']);
                        $leadsDocumentsModel->setDisplayFileName($storedData['fileName']);
                        $leadsDocumentsModel->setUploadedAt(date(Tools_System_Tools::DATE_MYSQL));

                        $leadsDocumentsMapper->save($leadsDocumentsModel);

                        return array('error' => 0, 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'File uploaded'));

                    } catch (Exceptions_SeotoasterException $e) {
                        $this->_error($e->getMessage());
                    }
                } else {
                    $uploaderErrorMsg =  $uploader->getMessages();
                    return array('error' => 1, 'message' => $uploaderErrorMsg, 'fileName' => $file['name']);
                }
            }
        }
        return array('error' => 1, 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'File not attached'));
    }

    public function putAction()
    {

    }

    public function deleteAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $translator = Zend_Registry::get('Zend_Translate');
        $sessionHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('session');

        if (!$id) {
            $this->_error();
        }

        if(!Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $this->_error();
        }

        $userRoleId = $sessionHelper->getCurrentUser()->getRoleId();

        $leadsDocumentsMapper = Leads_Mapper_LeadsDocumentsMapper::getInstance();
        $leadsDocumentsModel = $leadsDocumentsMapper->find($id);
        if ($leadsDocumentsModel instanceof Leads_Model_LeadsDocumentsModel) {
            $leadId = $leadsDocumentsModel->getLeadId();

            $isAllow = true;
            if($userRoleId == Shopping::ROLE_SALESPERSON) {
                $isAllow = Tools_LeadTools::allowSales($leadId);
            }

            if(!$isAllow) {
                return array('error' => 1, 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Permission deny'));
            }

            $fileStoredName = $leadsDocumentsModel->getFileStoredName();
            if (empty($leadsDocumentsModel->getExternalUrl())) {
                $savePath = Tools_LeadDocumentsTools::getFilePath($fileStoredName);
                if (file_exists($savePath)) {
                    if (!Tools_Filesystem_Tools::deleteFile($savePath)) {
                        $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'File remove issue'));
                    }
                }
            }
            $leadsDocumentsMapper->delete($id);

        } else {
            $this->_error();
        }
    }

}
