<?php

class Api_Leads_Leademailsequence extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('lead_id');

    /**
     * System translator
     *
     * @var null
     */
    protected $_translator = null;


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    /**
     * init
     */
    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_translator = Zend_Registry::get('Zend_Translate');
    }


    /**
     * Get
     *
     * Resource:
     * : /api/leads/leademailsequence/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $leadId = filter_var($this->_request->getParam('leadId', false), FILTER_SANITIZE_NUMBER_INT);
        $leadEmailHasSequenceMapper = Leads_Mapper_LeadsHasEmailSequenceMapper::getInstance();
        $sequenceData = Tools_LeadTools::getEmailSequenceList();
        $subscribeInfo = array();
        $subscribeSmsInfo = array();

        if (!empty($leadId)) {
            $data = $leadEmailHasSequenceMapper->getLeadSequenceByLeadId($leadId, true);

            $leadsMapperLeadsMapper = Leads_Mapper_LeadsMapper::getInstance();
            $currentLead = $leadsMapperLeadsMapper->find($leadId);

            if($currentLead instanceof Leads_Model_LeadsModel) {
               $leadEmail = $currentLead->getLeadEmail();

               $mobilePhoneLead = Tools_EmailSequenceTools::glueUserPhone(
                   $currentLead->getLeadMobileCountryCodeValue(),
                   $currentLead->getLeadMobile(),
                   $currentLead->getLeadPhoneCountryCodeValue(),
                   $currentLead->getLeadPhone()
               );
               if (!empty($mobilePhoneLead)) {
                   $cartStatusSubscribeSmsMapper = Emailsequence_Models_Mapper_SubscribeSmsMapper::getInstance();
                   $subscribeSmsModel = $cartStatusSubscribeSmsMapper->findByMobile($mobilePhoneLead);
                   if ($subscribeSmsModel instanceof Emailsequence_Models_Models_SubscribeSmsModel) {
                       $subscribeSmsInfo = array('status' => $subscribeSmsModel->getStatus());
                   }
               } else {
                   $subscribeSmsInfo = array('status' => Emailsequence_Models_Models_SubscribeSmsModel::STATUS_SUBSCRIBED);
               }
               if(!empty($leadEmail)) {
                   $cartStatusSubscribeMapper = Emailsequence_Models_Mapper_SubscribeMapper::getInstance();
                   $subscribeExists = $cartStatusSubscribeMapper->findByExternalLeadEmail($leadEmail);

                   if(!empty($subscribeExists)) {
                       $subscribeInfo = $subscribeExists;
                   }

                   if(!empty($data)) {
                       $sequenceIds = array();
                       foreach ($data as $sequence) {
                           array_push($sequenceIds, $sequence['sequence_id']);
                       }

                       if(!empty($sequenceIds)) {
                           $sequenceIds = implode(',', $sequenceIds);
                       }

                       if(!empty($sequenceIds) && !empty($subscribeInfo)) {
                           $subscribeInfo['sequenceIds'] = is_string($sequenceIds) ? $sequenceIds : '';;
                       }
                   }

               }
            }
        }

        if (empty($data)) {
            $response = array();
        } else {
            $response['data'] = $data;
        }

        $response['sequenceList'] = $sequenceData;
        $response['subscribeInfo'] = $subscribeInfo;
        $response['subscribeSmsInfo'] = $subscribeSmsInfo;

        return $response;
    }

    /**
     *
     * Resource:
     * : /api/leads/leademailsequence/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING);
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'));
        }

        if (empty($data['sequence_id'])) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Please choose sequence name'));
        }

        $data['sequence_id'] = trim(filter_var($data['sequence_id'], FILTER_SANITIZE_STRING));

        $leadEmailHasSequenceMapper = Leads_Mapper_LeadsHasEmailSequenceMapper::getInstance();
        $emailSequenceSettingsMapper = Emailsequence_Models_Mapper_EmailSequenceSettingsMapper::getInstance();
        $sequenceModel = $emailSequenceSettingsMapper->find($data['sequence_id']);
        if (!$sequenceModel instanceof Emailsequence_Models_Models_EmailSequenceSettingsModel) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Sequence doesn\'t exist'));
        }

        $leadEmailHasModel = $leadEmailHasSequenceMapper->findByLeadIdSequenceId($data['lead_id'], $sequenceModel->getId());
        if ($leadEmailHasModel instanceof Leads_Model_LeadsHasEmailSequenceModel) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Sequence already assigned'));
        }

        $sequenceAppliedDate = date(Tools_System_Tools::DATE_MYSQL, strtotime('now'));

        $leadEmailHasModel = new Leads_Model_LeadsHasEmailSequenceModel();
        $leadEmailHasModel->setSequenceId($sequenceModel->getId());
        $leadEmailHasModel->setLeadId($data['lead_id']);
        $leadEmailHasModel->setSequenceDate($sequenceAppliedDate);

        $leadEmailHasSequenceMapper->save($leadEmailHasModel);
        $cacheHelper = Zend_Controller_Action_HelperBroker::getExistingHelper('cache');
        $cacheHelper->clean('email_sequence_lead_status_data_no_cart', 'email_sequence_lead_status_no_cart');

        $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Sequence has been added'));

    }

    /**
     *
     * Resource:
     * : /api/leads/leademailsequence/
     *
     * HttpMethod:
     * : PUT
     *
     * @return JSON
     */
    public function putAction()
    {
    }

    /**
     * Delete email sequence from the lead
     *
     * Resource:
     * : /api/leads/leademailsequence/
     *
     * HttpMethod:
     * : DELETE
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $leadId = filter_var($this->_request->getParam('lead_id'), FILTER_SANITIZE_NUMBER_INT);
        $unsubscribeFlag = filter_var($this->_request->getParam('unsubscribeFlag'), FILTER_SANITIZE_STRING);
        if(!empty($unsubscribeFlag)) {
            $sequenceId = filter_var($this->_request->getParam('sequence_id'), FILTER_SANITIZE_STRING);
        } else {
            $sequenceId = filter_var($this->_request->getParam('sequence_id'), FILTER_SANITIZE_NUMBER_INT);
        }

        if (empty($leadId)) {
            $this->_responseHelper->fail($this->_translator->translate(Leads::TRANSLATION_PREFIX.'Lead id is missing'));
        }
        $leadsHasEmailSequenceMapper = Leads_Mapper_LeadsHasEmailSequenceMapper::getInstance();

        if(!empty($unsubscribeFlag)) {
            $leadsMapperLeadsMapper = Leads_Mapper_LeadsMapper::getInstance();
            $currentLead = $leadsMapperLeadsMapper->find($leadId);

            $leadEmail = $currentLead->getLeadEmail();

            $cartStatusSubscribeMapper = Emailsequence_Models_Mapper_SubscribeMapper::getInstance();
            $subscribeExists = $cartStatusSubscribeMapper->findByExternalLeadEmail($leadEmail);

            if(!empty($subscribeExists)) {
                if(!empty($sequenceId)) {
                    $sequenceId = explode(',', $sequenceId);
                    foreach ($sequenceId as $seqId) {
                        $leadHasSequenceModel = $leadsHasEmailSequenceMapper->getLeadSequence($leadId, $seqId);

                        if(!empty($subscribeExists) && $leadHasSequenceModel instanceof Leads_Model_LeadsHasEmailSequenceModel) {
                            $notSendEmailsInQueue = $leadsHasEmailSequenceMapper->getFromCartStatusQueue($leadId, 'lead', $seqId, array('0'));
                            if (!empty($notSendEmailsInQueue)) {
                                $leadsHasEmailSequenceMapper->delete($leadId, $seqId);
                                $leadEmailSequenceStatusMapper = Leads_Mapper_LeadsEmailSequenceStatusMapper::getInstance();
                                $leadEmailSequenceStatusMapper->deleteByLeadIdSequenceId($leadId, $seqId);
                                $leadsHasEmailSequenceMapper->deleteFromCartStatusQueue($leadId, 'lead', $seqId, '0');
                            }
                        }
                    }
                }

                if(!empty($subscribeExists) && $subscribeExists['status'] == Emailsequence::SUBSCRIBED_STATUS_SUBSCRIBED) {
                    $cartStatusSubscribeMapper->updateStatus($subscribeExists['code'], Emailsequence::SUBSCRIBED_STATUS_UNSUBSCRIBED);
                }
            } else {
                $cartStatusSubscribeModel = new Emailsequence_Models_Models_SubscribeModel();
                $cartStatusSubscribeModel->setUserId(0);
                $cartStatusSubscribeModel->setCode(sha1(uniqid(strtotime('now'))));
                $cartStatusSubscribeModel->setStatus(Emailsequence::SUBSCRIBED_STATUS_UNSUBSCRIBED);
                $cartStatusSubscribeModel->setExternal(1);
                $cartStatusSubscribeModel->setExternalEmail($leadEmail);
                $cartStatusSubscribeMapper->save($cartStatusSubscribeModel);
            }

            $this->_responseHelper->success($this->_translator->translate(Leads::TRANSLATION_PREFIX.'Unsubscribed'));

        } else {
            $leadHasSequenceModel = $leadsHasEmailSequenceMapper->getLeadSequence($leadId, $sequenceId);

            if ($leadHasSequenceModel instanceof Leads_Model_LeadsHasEmailSequenceModel) {
                $leadsHasEmailSequenceMapper->delete($leadId, $sequenceId);
                $notSendEmailsInQueue = $leadsHasEmailSequenceMapper->getFromCartStatusQueue($leadId, 'lead', $sequenceId, array('0'));
                if (!empty($notSendEmailsInQueue)) {
                    $leadEmailSequenceStatusMapper = Leads_Mapper_LeadsEmailSequenceStatusMapper::getInstance();
                    $leadEmailSequenceStatusMapper->deleteByLeadIdSequenceId($leadId, $sequenceId);
                    $leadsHasEmailSequenceMapper->deleteFromCartStatusQueue($leadId, 'lead', $sequenceId, '0');
                }

                $this->_responseHelper->success($this->_translator->translate(Leads::TRANSLATION_PREFIX.'Sequence has been deleted'));
            } else {
                $this->_responseHelper->fail();
            }
        }
    }

}
