<?php

class Api_Leads_Leademailsequencelog extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('lead_id');

    /**
     * System translator
     *
     * @var null
     */
    protected $_translator = null;


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    /**
     * init
     */
    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_translator = Zend_Registry::get('Zend_Translate');
    }


    /**
     * Get
     *
     * Resource:
     * : /api/leads/leademailsequencelog/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $leadId = filter_var($this->_request->getParam('leadId', false), FILTER_SANITIZE_NUMBER_INT);
        $leadEmailHasSequenceMapper = Leads_Mapper_LeadsHasEmailSequenceMapper::getInstance();
        $fromYear = filter_var($this->_request->getParam('fromYear'), FILTER_SANITIZE_STRING);
        $fromMonth = filter_var($this->_request->getParam('fromMonth'), FILTER_SANITIZE_STRING);

//        $defaultTimezone = date_default_timezone_get();
//        if (empty($defaultTimezone)) {
//            $defaultTimezone = 'UTC';
//        }

        $defaultTimezone = 'UTC';

        if (!empty($leadId)) {
            if (!empty($fromYear) && !empty($fromMonth)) {
                $currentYear = $fromYear;
                $currentMonth = $fromMonth;
                $sentFrom = $fromYear.'-'.$fromMonth.'-01'.' 00:00:00';
                $sentAtTo = Tools_System_Tools::convertDateFromTimezone($sentFrom, $defaultTimezone, 'UTC', 'Y-m-t 23:59:59');
                $sentAtFrom = $sentFrom;
            } else {
                $sentFrom = 'now';
                $currentYear = Tools_System_Tools::convertDateFromTimezone($sentFrom, $defaultTimezone, Tools_System_Tools::getUserTimezone(), 'Y');
                $currentMonth = Tools_System_Tools::convertDateFromTimezone($sentFrom, $defaultTimezone, Tools_System_Tools::getUserTimezone(), 'm');
                $sentAtTo = Tools_System_Tools::convertDateFromTimezone($sentFrom, $defaultTimezone, Tools_System_Tools::getUserTimezone(), 'Y-m-t 23:59:59');
                $sentAtFrom = Tools_System_Tools::convertDateFromTimezone($sentFrom, $defaultTimezone, Tools_System_Tools::getUserTimezone(), 'Y-m-01 00:00:00');
            }


            $whereSent = $leadEmailHasSequenceMapper->getDbTable()->getAdapter()->quoteInto('pcq.externalId = ?', $leadId);
            $whereSent .= ' AND '. $leadEmailHasSequenceMapper->getDbTable()->getAdapter()->quoteInto('pcq.sentAt > ?', $sentAtFrom);
            $whereSent .= ' AND '. $leadEmailHasSequenceMapper->getDbTable()->getAdapter()->quoteInto('pcq.sentAt < ?', $sentAtTo);
            $dataSent = $leadEmailHasSequenceMapper->getEmailSequenceLog($whereSent, 'pcq.id', null, null, true);

            $whereDelayed = $leadEmailHasSequenceMapper->getDbTable()->getAdapter()->quoteInto('pcq.externalId = ?', $leadId);
            $whereDelayed .= ' AND '. $leadEmailHasSequenceMapper->getDbTable()->getAdapter()->quoteInto('pcq.delayedTo > ?', $sentAtFrom);
            $whereDelayed .= ' AND '. $leadEmailHasSequenceMapper->getDbTable()->getAdapter()->quoteInto('pcq.delayedTo < ?', $sentAtTo);
            $whereDelayed .= ' AND '. $leadEmailHasSequenceMapper->getDbTable()->getAdapter()->quoteInto('pcq.status = ?', '0');

            $dataDelayed = $leadEmailHasSequenceMapper->getEmailSequenceLog($whereDelayed, 'pcq.delayedTo', null, null, true);

            $data = array_merge($dataSent, $dataDelayed);
        } else {
            $this->_error('missing lead id');
        }

        if (empty($data)) {
            $response = array();
        } else {
            $response['data'] = $data;
        }

        /**
         * Prepare the list of days for the chosen month
         */
        $listOfDays = array();
        $startDate = "01-" . $currentMonth . "-" . $currentYear;
        $startTime = strtotime($startDate);
        $endTime = strtotime("+1 month", $startTime);
        $previousWeekOfTheMonth = false;
        $previousDateOfTheWeek = false;
        $weeksInTheMonth = array();
        $totalEmailsSent = 0;
        $totalSmsSent = 0;
        $totalEmailsDelayed = 0;
        $totalSmsDelayed = 0;
        for ($i = $startTime; $i < $endTime; $i += 86400) {
            $dayOfTheMonth = date('Y-m-d', $i);
            $listOfDays[$dayOfTheMonth] = array('day' => $dayOfTheMonth, 'weekAbbreviation' => date('D', $i), 'weekNumber' =>  intval(date('W', $i)));
            $weekInTheMonth = intval(date('W', $i));
            if (empty($previousWeekOfTheMonth)) {
                $weeksInTheMonth[$weekInTheMonth] = array('dayFrom' => date('Y-m-d', $i));
            } elseif ($previousWeekOfTheMonth !== $weekInTheMonth) {
                $weeksInTheMonth[$previousWeekOfTheMonth]['dayTo'] = $previousDateOfTheWeek;
                $weeksInTheMonth[$weekInTheMonth]['dayFrom'] = $dayOfTheMonth;
            }

            if ($endTime >= $i + 86400) {
                $weeksInTheMonth[$weekInTheMonth]['dayTo'] = $dayOfTheMonth;
            }

            $previousDateOfTheWeek = $dayOfTheMonth;
            $previousWeekOfTheMonth = $weekInTheMonth;
        }

        if (!empty($response['data'])) {
            foreach ($response['data'] as $emailData) {
                if ($emailData['status'] === '1') {
                    if (array_key_exists($emailData['sentAtSingleDate'], $listOfDays)) {
                        $weekId = intval(date('W', strtotime($emailData['sentAtSingleDate']. '00:00:00')));

                        if($emailData['type'] == 'email') {
                            if (empty($weeksInTheMonth[$weekId]['emailsSent'])) {
                                $weeksInTheMonth[$weekId]['emailsSent'] = 1;
                            } else {
                                $weeksInTheMonth[$weekId]['emailsSent'] =  $weeksInTheMonth[$weekId]['emailsSent'] + 1;
                            }

                            $totalEmailsSent = $totalEmailsSent + 1;
                        } else {
                            if (empty($weeksInTheMonth[$weekId]['smsSent'])) {
                                $weeksInTheMonth[$weekId]['smsSent'] = 1;
                            } else {
                                $weeksInTheMonth[$weekId]['smsSent'] =  $weeksInTheMonth[$weekId]['smsSent'] + 1;
                            }

                            $totalSmsSent = $totalSmsSent + 1;
                        }

                        $listOfDays[$emailData['sentAtSingleDate']]['emailsData'][] = $emailData;
                    }
                } else {
                    if (array_key_exists($emailData['delayedAtSingleDate'], $listOfDays)) {
                        $weekId = intval(date('W', strtotime($emailData['delayedAtSingleDate']. '00:00:00')));
                        if($emailData['type'] == 'email') {
                            if (empty($weeksInTheMonth[$weekId]['emailsDelayed'])) {
                                $weeksInTheMonth[$weekId]['emailsDelayed'] = 1;
                            } else {
                                $weeksInTheMonth[$weekId]['emailsDelayed'] =  $weeksInTheMonth[$weekId]['emailsDelayed'] + 1;
                            }

                            $totalEmailsDelayed = $totalEmailsDelayed + 1;
                        } else {
                            if (empty($weeksInTheMonth[$weekId]['smsDelayed'])) {
                                $weeksInTheMonth[$weekId]['smsDelayed'] = 1;
                            } else {
                                $weeksInTheMonth[$weekId]['smsDelayed'] =  $weeksInTheMonth[$weekId]['smsDelayed'] + 1;
                            }

                            $totalSmsDelayed = $totalSmsDelayed + 1;
                        }

                        $listOfDays[$emailData['delayedAtSingleDate']]['emailsData'][] = $emailData;
                    }
                }
            }
        }

        $response['currentYear'] = $currentYear;
        $response['currentMonth'] = $currentMonth;
        $response['listOfDays'] = $listOfDays;
        $response['totalEmailsSent'] = $totalEmailsSent;
        $response['totalSmsSent'] = $totalSmsSent;
        $response['totalEmailsDelayed'] = $totalEmailsDelayed;
        $response['totalSmsDelayed'] = $totalSmsDelayed;
        $response['listOfWeeks'] = $weeksInTheMonth;
        $response['dateNow'] = Tools_System_Tools::convertDateFromTimezone('now', $defaultTimezone, Tools_System_Tools::getUserTimezone(), 'Y-m-d');
        $response['fullMonthName'] = Tools_System_Tools::convertDateFromTimezone($sentFrom, $defaultTimezone, Tools_System_Tools::getUserTimezone(), 'F');
        $response['currentUserTimeZone'] = Tools_System_Tools::convertDateFromTimezone('now', 'UTC', Tools_System_Tools::getUserTimezone(), 'T');

        return $response;
    }

    public function postAction()
    {
    }

    public function putAction()
    {
    }

    public function deleteAction()
    {
    }

}
