<?php

class Api_Leads_Leadfilterpreset extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * System session helper
     *
     * @var null
     */
    protected $_sessionHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('filter_preset_name', 'filter_preset_type');

    /**
     * Existing preset types
     *
     * @var array
     */
    protected $_allowedPresetTypes = array(
        Leads_Model_LeadsFilterPresetModel::FILTER_PRESET_TYPE_ORGANIZATION,
        Leads_Model_LeadsFilterPresetModel::FILTER_PRESET_TYPE_LEAD
    );


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_sessionHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
    }


    /**
     * Get lead filter preset by id
     *
     * Resource:
     * : /api/leads/leadfilterpreset/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order'), FILTER_SANITIZE_STRING);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);

        $leadFilterPresetDataMapper = Leads_Mapper_LeadsFilterPresetMapper::getInstance();
        $currentUser = $this->_sessionHelper->getCurrentUser();
        $userId = 0;
        if ($id) {
            $data = $leadFilterPresetDataMapper->findByIdRole($id, $userId);
        } else {
            $data = $leadFilterPresetDataMapper->fetchAll(null, $sortOrder, $limit, $offset);
        }

        return $data;
    }

    /**
     * Create new lead filter preset
     *
     * Resource:
     * : /api/leads/leadfilterpreset/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = filter_var_array($this->_request->getParams(), FILTER_SANITIZE_STRING);
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        $presetType = $data['filter_preset_type'];

        if (!in_array($presetType, $this->_allowedPresetTypes)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing preset type'));
        }

        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'));
        }

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
        }

        $leadsFilterPresetMapper = Leads_Mapper_LeadsFilterPresetMapper::getInstance();
        $leadsFilterPresetModel = new Leads_Model_LeadsFilterPresetModel();
        $data['filter_preset_name'] = preg_replace('~[^\w\s\_]~ui', '', $this->_request->getParam('filter_preset_name'));
        if (!empty($data['filter_preset_data'])) {
            if(!empty($data['filter_preset_data']['lead_source_type']) && !empty($data['filter_preset_data']['call_duration'])) {
                $leadTypeModel = Leads_Mapper_LeadsTypeMapper::getInstance()->find($data['filter_preset_data']['lead_source_type']);
                if($leadTypeModel instanceof Leads_Model_LeadsTypeModel) {
                    if($leadTypeModel->getTypeName() == Leads_Model_LeadsTypeModel::LEAD_TYPE_INBOUND_CALL || $leadTypeModel->getTypeName() == Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_CALL) {
                        $data['filter_preset_data']['lead_source_type_name'] = $leadTypeModel->getTypeName();
                    } else {
                        unset($data['filter_preset_data']['call_duration']);
                    }
                }
            }

            if(!empty($data['filter_preset_data']['last_interaction_id']) && !empty($data['filter_preset_data']['outbound_emails_sub_type'])) {
                $leadTypeModel = Leads_Mapper_LeadsTypeMapper::getInstance()->find($data['filter_preset_data']['last_interaction_id']);
                if($leadTypeModel instanceof Leads_Model_LeadsTypeModel) {
                    if($leadTypeModel->getTypeName() == Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_EMAIL) {
                        $data['filter_preset_data']['lead_last_interaction_name'] = $leadTypeModel->getTypeName();
                    } else {
                        unset($data['filter_preset_data']['outbound_emails_sub_type']);
                    }
                }
            }

            $data['filter_preset_data'] = json_encode($data['filter_preset_data']);
        }
        $leadsFilterPresetModel->setOptions($data);

        $leadPresetExistingModel = $leadsFilterPresetMapper->getByNameAndType($data['filter_preset_name'], $presetType);
        if (!empty($leadPresetExistingModel)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Lead filter preset with such name already exists'));
        }

        $userId = $this->_sessionHelper->getCurrentUser()->getId();
        if (!empty($data['is_default'])) {
            $leadsFilterPresetMapper->resetDefaultByCreatorId($userId, $presetType);
        }

        $leadsFilterPresetModel->setCreatorId($userId);
        $leadsFilterPresetModel = $leadsFilterPresetMapper->save($leadsFilterPresetModel);

        $this->_responseHelper->success(array(
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Filter has been created'),
            'id' => $leadsFilterPresetModel->getId()
        ));
    }

    /**
     * Update lead preset data
     *
     * Resource:
     * : /api/leads/leadfilterpreset/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     * : lead filter preset id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if (!empty($data['id']) && !empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {
            $translator = Zend_Registry::get('Zend_Translate');
            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();
                $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
            }
            $presetId = filter_var($data['id'], FILTER_SANITIZE_NUMBER_INT);
            $leadFilterPresetMapper = Leads_Mapper_LeadsFilterPresetMapper::getInstance();
            $filterPresetModel = $leadFilterPresetMapper->find($presetId);
            if (!$filterPresetModel instanceof Leads_Model_LeadsFilterPresetModel) {
                $this->_responseHelper->fail($translator->translate(Leads::TRANSLATION_PREFIX.'Config doesn\'t exists'));
            }

            $currentUser = $this->_sessionHelper->getCurrentUser();
            $userId = $currentUser->getId();
            $userRole = $currentUser->getRoleId();
            if ($userRole !== Tools_Security_Acl::ROLE_SUPERADMIN && $userRole !== Tools_Security_Acl::ROLE_ADMIN) {
                $creatorId = $filterPresetModel->getCreatorId();
                if ($creatorId != $userId) {
                    $this->_error('Editing not allowed');
                }
            }

            $presetType = $data['filter_preset_type'];

            if (!in_array($presetType, $this->_allowedPresetTypes)) {
                $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing preset type'));
            }

            $oldFilterPresetName = $filterPresetModel->getFilterPresetName();
            $data['filter_preset_name'] = preg_replace('~[^\w\s\_]~ui', '', $data['filter_preset_name']);
            $newFilterPresetName = $data['filter_preset_name'];
            if ($oldFilterPresetName !== $newFilterPresetName) {
                $leadPresetExistingModel = $leadFilterPresetMapper->getByNameAndType($data['filter_preset_name'], $presetType);
                if (!empty($leadPresetExistingModel)) {
                    $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Lead filter preset with such name already exists'));
                }
            }

            $userId = $this->_sessionHelper->getCurrentUser()->getId();
            if (!empty($data['is_default'])) {
                $leadFilterPresetMapper->resetDefaultByCreatorId($userId, $presetType);
            }

            if (!empty($data['filter_preset_data'])) {
                if(!empty($data['filter_preset_data']['lead_source_type']) && !empty($data['filter_preset_data']['call_duration'])) {
                    $leadTypeModel = Leads_Mapper_LeadsTypeMapper::getInstance()->find($data['filter_preset_data']['lead_source_type']);
                    if($leadTypeModel instanceof Leads_Model_LeadsTypeModel) {
                        if($leadTypeModel->getTypeName() == Leads_Model_LeadsTypeModel::LEAD_TYPE_INBOUND_CALL || $leadTypeModel->getTypeName() == Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_CALL) {
                            $data['filter_preset_data']['lead_source_type_name'] = $leadTypeModel->getTypeName();
                        } else {
                            unset($data['filter_preset_data']['call_duration']);
                        }
                    }
                }

                if(!empty($data['filter_preset_data']['last_interaction_id']) && !empty($data['filter_preset_data']['outbound_emails_sub_type'])) {
                    $leadTypeModel = Leads_Mapper_LeadsTypeMapper::getInstance()->find($data['filter_preset_data']['last_interaction_id']);
                    if($leadTypeModel instanceof Leads_Model_LeadsTypeModel) {
                        if($leadTypeModel->getTypeName() == Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_EMAIL) {
                            $data['filter_preset_data']['lead_last_interaction_name'] = $leadTypeModel->getTypeName();
                        } else {
                            unset($data['filter_preset_data']['outbound_emails_sub_type']);
                        }
                    }
                }

                $data['filter_preset_data'] = json_encode($data['filter_preset_data']);
            }

            $filterPresetModel->setOptions($data);
            $leadFilterPresetMapper->save($filterPresetModel);
            $this->_responseHelper->success(array(
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Filter updated'),
                'id' => $filterPresetModel->getId()
            ));
        }

    }

    /**
     * Delete lead filter preset
     *
     * Resource:
     * : /api/leads/leadfilterpreset/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     * : lead filter preset id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $currentUser = $this->_sessionHelper->getCurrentUser();
        $userId = $currentUser->getId();
        $userRole = $currentUser->getRoleId();

        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        if (!$id) {
            $this->_responseHelper->fail('');
        }
        $translator = Zend_Registry::get('Zend_Translate');
        $leadsFilterPresetMapper = Leads_Mapper_LeadsFilterPresetMapper::getInstance();
        $leadFilterPresetModel = $leadsFilterPresetMapper->find($id);
        if ($leadFilterPresetModel instanceof Leads_Model_LeadsFilterPresetModel) {
            if ($userRole !== Tools_Security_Acl::ROLE_SUPERADMIN && $userRole !== Tools_Security_Acl::ROLE_ADMIN) {
                $creatorId = $leadFilterPresetModel->getCreatorId();
                if ($creatorId != $userId) {
                    $this->_responseHelper->fail('Editing not allowed');
                }
            }

            $leadsFilterPresetMapper->delete($id);

            $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Filter has been deleted'));
        } else {
            $this->_responseHelper->fail('');
        }

    }

}
