<?php

class Api_Leads_Leadgeneralconfig extends Api_Service_Abstract
{

    /**
     * Tasks reminders
     */
    const FIRST_TASK_REMINDER = 'first_task_reminder';

    const SECOND_TASK_REMINDER = 'second_task_reminder';

    /**
     * Sales person access limitation
     */
    const SALES_PERSON_LIMITED_ACCESS = 'sales_person_limited_access';

    /**
     * Process product tags successful purchase
     */
    const PROCESS_PRODUCT_TAGS = 'process_product_tags';

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('paramName');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    public function getAction()
    {
    }

    public function postAction()
    {
        $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING);
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'));
        }

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
        }

        $paramName = trim(filter_var($data['paramName'], FILTER_SANITIZE_STRING));
        $paramValue = trim(filter_var($data['paramValue'], FILTER_SANITIZE_STRING));

        if ($paramName === self::FIRST_TASK_REMINDER || $paramName === self::SECOND_TASK_REMINDER) {
            $leadTaskQueueMapper = Leads_Mapper_LeadsTaskQueueMapper::getInstance();
            if ($paramName === self::FIRST_TASK_REMINDER) {
                $leadTaskQueueMapper->deleteByQueueType(self::FIRST_TASK_REMINDER, Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_STATUS_PROCESSING);
            }
            if ($paramName === self::SECOND_TASK_REMINDER) {
                $leadTaskQueueMapper->deleteByQueueType(self::SECOND_TASK_REMINDER, Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_STATUS_PROCESSING);
            }
            $cacheHelper = Zend_Controller_Action_HelperBroker::getExistingHelper('cache');
            $cacheHelper->clean('lead_task_queue_data', 'leads');
        }

        if ($paramName === 'profile_google_api_key') {
            $config['crmGoogleMapKey'] = $paramValue;
            Application_Model_Mappers_ConfigMapper::getInstance()->save($config);
            $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Data has been updated'));
        }

        if ($paramName === 'profile_google_map_default_zoom') {
            if(!empty($paramValue)){
                $tmpValue = filter_var($paramValue, FILTER_SANITIZE_NUMBER_INT);
                if(empty($tmpValue) || ($tmpValue < 1 ||  $tmpValue > 22)){
                    $this->_responseHelper->fail($translator->translate(Leads::TRANSLATION_PREFIX.'Default zoom parameter not valid'));
                }
            }

            $config['crmGoogleMapDefaultZoom'] = empty($paramValue) ? '' : $paramValue;
            Application_Model_Mappers_ConfigMapper::getInstance()->save($config);
            $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Data has been updated'));
        }

        if ($paramName === 'redirectAdminAfterLogin') {
            $configMapper = Application_Model_Mappers_ConfigMapper::getInstance();
            $redirect = '';
            if (!empty($paramValue)) {
                $redirect = 'dashboard';
            }
            $configMapper->save(array($paramName => $redirect));
            $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Data has been updated'));
        }

        if ($paramName === 'enableLeadOrgGeotagging') {
            $configMapper = Application_Model_Mappers_ConfigMapper::getInstance();
            $enableLeadOrgGeotagging = '';
            if (!empty($paramValue)) {
                $enableLeadOrgGeotagging = '1';
            }
            $configMapper->save(array($paramName => $enableLeadOrgGeotagging));
            $this->_responseHelper->success($translator->translate('Data has been updated'));
        }

        $leadGeneralConfigMapper = Leads_Mapper_LeadsConfigMapper::getInstance();
        $leadGeneralConfigMapper->save(array($paramName => $paramValue));

        $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Data has been updated'));
    }

    public function putAction()
    {
    }

    public function deleteAction()
    {
    }

}
