<?php

class Api_Leads_Leadgeneralreports extends Api_Service_Abstract
{
    /**
     * Existing stat types
     */
    const LEAD_STAT_TYPE_CALLS_PLACED = 'callsPlaced';

    const LEAD_STAT_TYPE_CALLS_RECEIVED = 'callsReceived';

    const LEAD_STAT_TYPE_EMAILS_SENT = 'emailsSent';

    const LEAD_STAT_TYPE_EMAILS_RECEIVED = 'emailsReceived';

    const LEAD_STAT_TYPE_MEETINGS_HELD = 'meetingsHeld';

    const LEAD_STAT_TYPE_TASKS_COMPLETED = 'tasksCompleted';

    /**
     * Lead reports secure token
     */
    const LEAD_REPORTS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     *
     * Resource:
     * : /api/leads/leadgeneralreports/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $leadId = filter_var($this->_request->getParam('leadId'), FILTER_SANITIZE_NUMBER_INT);
        $statsTypes = $this->_request->getParam('statsTypes');
        $tabContentName = filter_var($this->_request->getParam('tabContentName'), FILTER_SANITIZE_STRING);
        $leadReportsCompare = filter_var($this->_request->getParam('leadReportsCompare'), FILTER_SANITIZE_NUMBER_INT);
        $leadReportsFilterStartDate = filter_var($this->_request->getParam('leadReportsFilterStartDate'), FILTER_SANITIZE_STRING);
        $leadReportsFilterEndDate = filter_var($this->_request->getParam('leadReportsFilterEndDate'), FILTER_SANITIZE_STRING);
        $leadReportsCompareFilterStartDate = filter_var($this->_request->getParam('leadReportsCompareFilterStartDate'), FILTER_SANITIZE_STRING);
        $leadReportsCompareFilterEndDate = filter_var($this->_request->getParam('leadReportsCompareFilterEndDate'), FILTER_SANITIZE_STRING);

        $leadReportsMapper = Leads_Mapper_LeadsReportsMapper::getInstance();

        $leadReportsFilterStartDate = Tools_System_Tools::convertDateFromTimezone($leadReportsFilterStartDate. ' 00:00:00', '', 'UTC', 'Y-m-d H:i:s');
        $leadReportsFilterEndDate = Tools_System_Tools::convertDateFromTimezone($leadReportsFilterEndDate. ' 23:59:59', '', 'UTC', 'Y-m-d H:i:s');
        $leadReportsCompareFilterStartDate = Tools_System_Tools::convertDateFromTimezone($leadReportsCompareFilterStartDate. ' 00:00:00', '', 'UTC', 'Y-m-d H:i:s');
        $leadReportsCompareFilterEndDate = Tools_System_Tools::convertDateFromTimezone($leadReportsCompareFilterEndDate. ' 23:59:59', '', 'UTC', 'Y-m-d H:i:s');

        $currency = Zend_Registry::get('Zend_Currency');

        $data = array();

        if (in_array(self::LEAD_STAT_TYPE_CALLS_PLACED, $statsTypes)) {
            $originalData = $leadReportsMapper->getCallsStats($leadReportsFilterStartDate, $leadReportsFilterEndDate,
                Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND);
            $compareData = array();
            if (!empty($leadReportsCompare)) {
                $compareData = $leadReportsMapper->getCallsStats($leadReportsCompareFilterStartDate,
                    $leadReportsCompareFilterEndDate, Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND);
            }

            $data['stats'][self::LEAD_STAT_TYPE_CALLS_PLACED] = Tools_LeadReportsTools::prepareStatsData($originalData,
                $leadReportsCompare, $compareData);
        }

        if (in_array(self::LEAD_STAT_TYPE_CALLS_RECEIVED, $statsTypes)) {
            $originalData = $leadReportsMapper->getCallsStats($leadReportsFilterStartDate, $leadReportsFilterEndDate,
                Leads_Model_LeadsActionLogModel::ACTION_TYPE_INBOUND);
            $compareData = array();
            if (!empty($leadReportsCompare)) {
                $compareData = $leadReportsMapper->getCallsStats($leadReportsCompareFilterStartDate,
                    $leadReportsCompareFilterEndDate, Leads_Model_LeadsActionLogModel::ACTION_TYPE_INBOUND);
            }

            $data['stats'][self::LEAD_STAT_TYPE_CALLS_RECEIVED] = Tools_LeadReportsTools::prepareStatsData($originalData,
                $leadReportsCompare, $compareData);
        }

        if (in_array(self::LEAD_STAT_TYPE_EMAILS_SENT, $statsTypes)) {
            //sent
            $originalDataEmailSent = $leadReportsMapper->getEmailsStats($leadReportsFilterStartDate,
                $leadReportsFilterEndDate, Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_SENT);
            //sent
            $originalDataEmailSequenceSent = $leadReportsMapper->getEmailsSequenceStats($leadReportsFilterStartDate,
                $leadReportsFilterEndDate);
            //sent
            $originalDataEmailNewsletterSent = $leadReportsMapper->getEmailsStats($leadReportsFilterStartDate,
                $leadReportsFilterEndDate, Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_SENT, 'sambaEmailMarketing', false);

            //opened
            $originalDataEmailOpened = $leadReportsMapper->getEmailsStats($leadReportsFilterStartDate,
                $leadReportsFilterEndDate, Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_OPENED);
            //opened
            $originalDataEmailSequenceOpened = $leadReportsMapper->getEmailsSequenceStatsOpened($leadReportsFilterStartDate,
                $leadReportsFilterEndDate);
            //opened
            $originalDataEmailNewsletterOpened = $leadReportsMapper->getEmailsStats($leadReportsFilterStartDate,
                $leadReportsFilterEndDate, Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_OPENED, 'sambaEmailMarketing', false);

            if(!empty($originalDataEmailSent) && !empty($originalDataEmailSequenceSent)) {
                $originalDataEmailSent['quantity'] -= $originalDataEmailSequenceSent['quantity'];
            }

            if(!empty($originalDataEmailOpened) && !empty($originalDataEmailSequenceOpened)) {
                $originalDataEmailOpened['quantity'] -= $originalDataEmailSequenceOpened['quantity'];
            }

            $compareDataSent = array();
            $originalData = array();
            $originalData['quantity'] = $originalDataEmailSent['quantity'] + $originalDataEmailSequenceSent['quantity'] + $originalDataEmailNewsletterSent['quantity'];
            if (!empty($leadReportsCompare)) {
                //sent
                $compareDataEmailSent = $leadReportsMapper->getEmailsStats($leadReportsCompareFilterStartDate,
                    $leadReportsCompareFilterEndDate, Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_SENT);
                //sent
                $compareDataEmailSequenceSent = $leadReportsMapper->getEmailsSequenceStats($leadReportsCompareFilterStartDate,
                    $leadReportsCompareFilterEndDate);
                //sent
                $compareDataEmailNewsletterSent = $leadReportsMapper->getEmailsStats($leadReportsCompareFilterStartDate,
                    $leadReportsCompareFilterEndDate, Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_SENT,
                    'sambaEmailMarketing', false);

                if(!empty($compareDataEmailSent) && !empty($compareDataEmailSequenceSent)) {
                    $compareDataEmailSent['quantity'] -= $compareDataEmailSequenceSent['quantity'];
                }

                $compareDataSent['quantity'] = $compareDataEmailSent['quantity'] + $compareDataEmailSequenceSent['quantity'] + $compareDataEmailNewsletterSent['quantity'];
            }

            //sent
            $result = Tools_LeadReportsTools::prepareStatsData($originalData,
                $leadReportsCompare, $compareDataSent);

            $result['manual'] = $originalDataEmailSent['quantity'];
            $result['auto'] = $originalDataEmailSequenceSent['quantity'];
            $result['newsletter'] = $originalDataEmailNewsletterSent['quantity'];

            //opened
            $compareDataOpened = array();
            $originalData['quantity'] = $originalDataEmailOpened['quantity'] + $originalDataEmailSequenceOpened['quantity'] + $originalDataEmailNewsletterOpened['quantity'];
            if (!empty($leadReportsCompare)) {
                //opened
                $compareDataEmailOpened = $leadReportsMapper->getEmailsStats($leadReportsCompareFilterStartDate,
                    $leadReportsCompareFilterEndDate, Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_OPENED);
                //opened
                $compareDataEmailSequenceOpened = $leadReportsMapper->getEmailsSequenceStatsOpened($leadReportsCompareFilterStartDate,
                    $leadReportsCompareFilterEndDate);
                //opened
                $compareDataEmailNewsletterOpened = $leadReportsMapper->getEmailsStats($leadReportsCompareFilterStartDate,
                    $leadReportsCompareFilterEndDate, Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_OPENED,
                    'sambaEmailMarketing', false);

                if(!empty($compareDataEmailOpened) && !empty($compareDataEmailSequenceOpened)) {
                    $compareDataEmailOpened['quantity'] -= $compareDataEmailSequenceOpened['quantity'];
                }

                $compareDataOpened['quantity'] = $compareDataEmailOpened['quantity'] + $compareDataEmailSequenceOpened['quantity'] + $compareDataEmailNewsletterOpened['quantity'];
            }

            $quantityOpened = Tools_LeadReportsTools::prepareStatsData($originalData,
                $leadReportsCompare, $compareDataOpened);

            $result['incrementOpened'] = $quantityOpened['increment'];
            $result['incrementSymbolOpened'] = $quantityOpened['incrementSymbol'];
            $result['incrementSymbolLabelOpened'] = $quantityOpened['incrementSymbolLabel'];
            $result['quantityOpened'] = $quantityOpened['quantity'];

            $result['manualOpened'] = $originalDataEmailOpened['quantity'];
            $result['autoOpened'] = $originalDataEmailSequenceOpened['quantity'];
            $result['newsletterOpened'] = $originalDataEmailNewsletterOpened['quantity'];

            $data['stats'][self::LEAD_STAT_TYPE_EMAILS_SENT] = $result;
        }

        if (in_array(self::LEAD_STAT_TYPE_EMAILS_RECEIVED, $statsTypes)) {
            $originalData = $leadReportsMapper->getEmailsStats($leadReportsFilterStartDate, $leadReportsFilterEndDate,
                Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_RECEIVED);
            $compareData = array();
            if (!empty($leadReportsCompare)) {
                $compareData = $leadReportsMapper->getEmailsStats($leadReportsCompareFilterStartDate,
                    $leadReportsCompareFilterEndDate, Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_RECEIVED);
            }

            $data['stats'][self::LEAD_STAT_TYPE_EMAILS_RECEIVED] = Tools_LeadReportsTools::prepareStatsData($originalData,
                $leadReportsCompare, $compareData);
        }

        if (in_array(self::LEAD_STAT_TYPE_MEETINGS_HELD, $statsTypes)) {
            $originalData = $leadReportsMapper->getMeetings($leadReportsFilterStartDate, $leadReportsFilterEndDate, array('finished'));
            $compareData = array();
            if (!empty($leadReportsCompare)) {
                $compareData = $leadReportsMapper->getMeetings($leadReportsCompareFilterStartDate, $leadReportsCompareFilterEndDate, array('finished'));
            }

            $data['stats'][self::LEAD_STAT_TYPE_MEETINGS_HELD] = Tools_LeadReportsTools::prepareStatsData($originalData, $leadReportsCompare, $compareData);
        }

        if (in_array(self::LEAD_STAT_TYPE_TASKS_COMPLETED, $statsTypes)) {
            $originalData = $leadReportsMapper->getTasks($leadReportsFilterStartDate, $leadReportsFilterEndDate, array(), array(Leads_Model_LeadsActionLogModel::ACTION_TASK_COMPLETED));
            $compareData = array();
            if (!empty($leadReportsCompare)) {
                $compareData = $leadReportsMapper->getTasks($leadReportsCompareFilterStartDate, $leadReportsCompareFilterEndDate, array(), array(Leads_Model_LeadsActionLogModel::ACTION_TASK_COMPLETED));
            }

            $data['stats'][self::LEAD_STAT_TYPE_TASKS_COMPLETED] = Tools_LeadReportsTools::prepareStatsData($originalData, $leadReportsCompare, $compareData);
        }

        $data['tabContentName'] = $tabContentName;
        $data['currencySymbol'] = preg_replace('~[\w]~', '', $currency->getSymbol());
        $data['comparedTo'] = $leadReportsCompare;


        return $data;
    }

    /**
     * Resource:
     * : /api/leads/leadgeneralreports/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
    }

    /**
     *
     * Resource:
     * : /api/leads/leadgeneralreports/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     *
     * @return JSON
     */
    public function putAction()
    {

    }

    /**
     *
     * Resource:
     * : /api/leads/leadgeneralreports/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     *
     * @return JSON
     */
    public function deleteAction()
    {
    }

}
