<?php

class Api_Leads_Leadgoal extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Get lead goal config
     *
     * Resource:
     * : /api/leads/leadgoal/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $leadGoalDetailed = filter_var($this->_request->getParam('leadGoalDetailed'), FILTER_SANITIZE_NUMBER_INT);

        $leadsOwnerGoalsMapper = Leads_Mapper_LeadsOwnerGoalsMapper::getInstance();
        $leadsOwnerGoalsOpportunitiesMapper = Leads_Mapper_LeadsOwnerGoalsOpportunitiesMapper::getInstance();

        $where = null;
        if (!empty($id)) {
            $where = $leadsOwnerGoalsMapper->getDbTable()->getAdapter()->quoteInto('plog.id = ?', $id);
            $ownersData = $leadsOwnerGoalsMapper->fetchAllData($where, null, null, null, true, true);
        } else {
            $ownersData = $leadsOwnerGoalsMapper->fetchAllData($where, null, null, null, true);
        }

        $leadOwners = Leads_Mapper_LeadsMapper::getInstance()->getLeadOwnersFullList(0, array(), true);


        $data = array();
        if (empty($ownersData)) {
            $data['data'] = array();
        } else {
            $data['data'] = $ownersData;
            foreach ($ownersData as $ownerData) {
                unset($leadOwners[$ownerData['user_id']]);
            }

            if (!empty($leadGoalDetailed)) {
                $data['goalOppStats'] = Tools_LeadGoalsTools::calculateGoalOppStats($ownersData['id']);
            }
        }

        $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');

        $companyName = $configHelper->getConfig('wicOrganizationName');
        $currency = Zend_Registry::get('Zend_Currency');
        $currencySymbol = preg_replace('~[\w]~', '', $currency->getSymbol());
        $globalStats = Tools_LeadGoalsTools::calculateGlobalStats();

        $leadsOpportunityTypeMapper = Leads_Mapper_LeadsOpportunityTypeMapper::getInstance();
        $opportunityTypes = $leadsOpportunityTypeMapper->getOpportunityTypes();

        $data['additionalInfo'] = array(
            'opportunityTypes' => $opportunityTypes,
            'companyName' => $companyName,
            'currencySymbol' => $currencySymbol,
            'leadOwners' => $leadOwners
        );

        $data['globalStats'] = $globalStats;

        $shoppingConfigMapper = Models_Mapper_ShoppingConfig::getInstance()->getConfigParam('fiscalYearStart');

        $data['fiscalYearStart'] = $shoppingConfigMapper;

        return $data;
    }

    /**
     * Create new goal config
     *
     * Resource:
     * : /api/leads/leadgoal/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING);
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error(array(
                'status' => 'error',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'),
                'errorCode' => '1'
            ), 200);
        }

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in ' . '<a href="' . $websiteUrl . 'go">here</a>'));
        }

        if (empty($data['userId'])) {
            return array(
                'error' => '1',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Missing user id')
            );
        }

        $userId = $data['userId'];
        $leadsOwnerGoalsMapper = Leads_Mapper_LeadsOwnerGoalsMapper::getInstance();
        $leadsOwnerGoalsModel = $leadsOwnerGoalsMapper->findByUserId($userId);
        if ($leadsOwnerGoalsModel instanceof Leads_Model_LeadsOwnerGoalsModel) {
            return array(
                'error' => '1',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'User already exists')
            );
        }

        $leadGoalDataModel = new Leads_Model_LeadsOwnerGoalsModel();
        $leadGoalDataModel->setUserId($userId);
        $leadGoalDataModel->setCallsPlaced(0);
        $leadGoalDataModel->setEmailsSent(0);
        $leadGoalDataModel->setMeetingsHeld(0);
        $leadGoalDataModel->setQuotesAmount(0);
        $leadGoalDataModel->setQuoteObjective(0);

        $leadsOwnerGoalsMapper->save($leadGoalDataModel);

        return array(
            'error' => '0',
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Goals have been saved')
        );
    }

    /**
     *
     * Resource:
     * : /api/leads/leadgoal/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error(array(
                'status' => 'error',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'),
                'errorCode' => '1'
            ), 200);
        }

        if (!empty($data['id']) && !empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {
            $translator = Zend_Registry::get('Zend_Translate');
            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();
                $this->_error($translator->translate('Your session has timed-out. Please Log back in ' . '<a href="' . $websiteUrl . 'go">here</a>'));
            }
            $id = filter_var($data['id'], FILTER_SANITIZE_NUMBER_INT);

            $leadsOwnerGoalsMapper = Leads_Mapper_LeadsOwnerGoalsMapper::getInstance();
            $leadsOwnerGoalsModel = $leadsOwnerGoalsMapper->find($id);
            if (!$leadsOwnerGoalsModel instanceof Leads_Model_LeadsOwnerGoalsModel) {
                $this->_error(array(
                    'status' => 'error',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Goal config doesn\'t exist'),
                    'errorCode' => '1'
                ), 200);
            }

            $leadsOwnerGoalsModel->setMeetingsHeld($data['meetings_held']);
            $leadsOwnerGoalsModel->setQuotesAmount($data['quotes_amount']);
            $leadsOwnerGoalsModel->setCallsPlaced($data['calls_placed']);
            $leadsOwnerGoalsModel->setEmailsSent($data['emails_sent']);
            $leadsOwnerGoalsModel->setQuoteObjective($data['quote_objective']);
            $leadsOwnerGoalsMapper->save($leadsOwnerGoalsModel);

            return array(
                'error' => '0',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'User has been added.')
            );
        }

        if (!empty($fieldDataMissing)) {
            $this->_error(array(
                'status' => 'error',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'),
                'errorCode' => '1'
            ), 200);
        }

    }

    /**
     *
     * Resource:
     * : /api/leads/leadgoal/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function deleteAction()
    {

        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $translator = Zend_Registry::get('Zend_Translate');
        if (!empty($id)) {
            $leadsOwnerGoalsMapper = Leads_Mapper_LeadsOwnerGoalsMapper::getInstance();
            $leadsOwnerGoalsMapper->delete($id);

            return array(
                'error' => '0',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead owner config has been deleted')
            );
        } else {
            $this->_error();
        }

    }

}
