<?php

class Api_Leads_Leadgoalopportunities extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Get lead goal opportunities config
     *
     * Resource:
     * : /api/leads/leadgoalopportunities/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $goalId = filter_var($this->_request->getParam('goalId'), FILTER_SANITIZE_NUMBER_INT);
        $opportunityId = filter_var($this->_request->getParam('opportunityId'), FILTER_SANITIZE_NUMBER_INT);

        $leadsOwnerGoalsOpportunitiesMapper = Leads_Mapper_LeadsOwnerGoalsOpportunitiesMapper::getInstance();

        $leadsOwnerGoalsOpportunitiesModel = $leadsOwnerGoalsOpportunitiesMapper->findByGoalAndOppId($goalId, $opportunityId);
        if ($leadsOwnerGoalsOpportunitiesModel instanceof Leads_Model_LeadsOwnerGoalsOpportunitiesModel) {
            $data['data'] = $leadsOwnerGoalsOpportunitiesModel->toArray();
        } else {
            $data = array(
                'data' =>
                    array(
                        'opportunityAmount' => 0,
                        'opportunityCount' => 0
                    )
            );
        }

        $goalOppStats = Tools_LeadGoalsTools::calculateGoalOppStats($goalId);
        $globalStats = Tools_LeadGoalsTools::calculateGlobalStats();

        $currency = Zend_Registry::get('Zend_Currency');
        $currencySymbol = preg_replace('~[\w]~', '', $currency->getSymbol());

        $data['additionalInfo'] = array(
            'currencySymbol' => $currencySymbol
        );

        $data['goalOppStats'] = $goalOppStats;
        $data['globalStats'] = $globalStats;

        return $data;
    }

    /**
     * Create new goal opporotunity config
     *
     * Resource:
     * : /api/leads/leadgoalopportunities/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING);
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error(array(
                'status' => 'error',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'),
                'errorCode' => '1'
            ), 200);
        }

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in ' . '<a href="' . $websiteUrl . 'go">here</a>'));
        }

        if (empty($data['goalId'])) {
            return array(
                'error' => '1',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Missing goal id')
            );
        }


        if (empty($data['opportunityId'])) {
            return array(
                'error' => '1',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Missing opportunity id')
            );
        }

        $leadsOwnerGoalsOpportunitiesMapper = Leads_Mapper_LeadsOwnerGoalsOpportunitiesMapper::getInstance();
        $leadsOwnerGoalsOpportunitiesModel = $leadsOwnerGoalsOpportunitiesMapper->findByGoalAndOppId($data['goalId'], $data['opportunityId']);
        if (!$leadsOwnerGoalsOpportunitiesModel instanceof Leads_Model_LeadsOwnerGoalsOpportunitiesModel) {
            $leadsOwnerGoalsOpportunitiesModel =  new Leads_Model_LeadsOwnerGoalsOpportunitiesModel();
            $leadsOwnerGoalsOpportunitiesModel->setOwnerGoalsId($data['goalId']);
            $leadsOwnerGoalsOpportunitiesModel->setOpportunityTypeId($data['opportunityId']);
        }

        $leadsOwnerGoalsOpportunitiesModel->setOpportunityAmount((int) $data['oppAmount']);
        $leadsOwnerGoalsOpportunitiesModel->setOpportunityCount((int) $data['oppCount']);
        $leadsOwnerGoalsOpportunitiesMapper->save($leadsOwnerGoalsOpportunitiesModel);

        return array(
            'error' => '0',
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Opportunity data has been saved')
        );
    }

    /**
     *
     * Resource:
     * : /api/leads/leadgoalopportunities/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function putAction()
    {


    }

    /**
     *
     * Resource:
     * : /api/leads/leadgoalopportunities/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     * : lead mobile app config id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {


    }

}
