<?php

class Api_Leads_Leadhassocialservice extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('lead_id', 'service_id');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     *
     * Resource:
     * : /api/leads/leadhassocialservice/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

    }

    /**
     *
     * Resource:
     * : /api/leads/leadhassocialservice/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {

    }

    /**
     * Update lead social service info
     *
     * Resource:
     * : /api/leads/leadhassocialservice/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     * : lead id to update
     * : social service id
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        $translator = Zend_Registry::get('Zend_Translate');
        if (empty($data)) {
            $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING);
        }
        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });
        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'));
        }
        if (!empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {
            $leadId = $data['lead_id'];
            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();
                $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
            }
            $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
            $leadModel = $leadMapper->find($leadId);
            if (!$leadModel instanceof Leads_Model_LeadsModel) {
                $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Lead doesn\'t exists'));
            }

            $leadMapper->addSocialService($leadId, $data['service_id'], $data['service_link']);

            $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Service link has been saved'));
        }
        $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'error'));

    }

    /**
     * Delete lead social service
     *
     * Resource:
     * : /api/leads/leadhassocialservice/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     * : lead_id to delete
     * : service_id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $leadId = filter_var($this->_request->getParam('lead_id'), FILTER_SANITIZE_NUMBER_INT);
        $serviceId = filter_var($this->_request->getParam('service_id'), FILTER_SANITIZE_NUMBER_INT);
        if (!$leadId || !$serviceId) {
            $this->_error();
        }
        $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
        $leadTagExists = $leadMapper->getByLeadIdTagId($leadId, $serviceId);
        if (!empty($leadTagExists)) {
            $leadMapper->deleteSocialService($leadId, $serviceId);
        } else {
            $this->_error();
        }

    }

}
