<?php

class Api_Leads_Leadhastags extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('lead_id', 'tag_id');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Get lead tags by id
     *
     * Resource:
     * : /api/leads/leadhastags/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

    }

    /**
     * Assign new tag to lead
     *
     * Resource:
     * : /api/leads/leadhastags/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {

    }

    /**
     * Update lead tag data
     *
     * Resource:
     * : /api/leads/leadhastags/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     * : lead id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        $translator = Zend_Registry::get('Zend_Translate');
        if (empty($data)) {
            $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING);
        }
        if (!empty($data['lead_id']) && !empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {
            $leadId = $data['lead_id'];
            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();
                $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
            }
            $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
            $leadModel = $leadMapper->find($leadId);
            if (!$leadModel instanceof Leads_Model_LeadsModel) {
                $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Lead doesn\'t exists'));
            }
            if (empty($data['tag_id'])) {
                $leadMapper->deleteTag($leadId);
            } else {
                $leadMapper->addTag($leadId, $data['tag_id']);
            }

            $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Tag has been updated'));
        }
        $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'error'));

    }

    /**
     * Delete lead tag
     *
     * Resource:
     * : /api/leads/leadhastags/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     * : lead_id to delete
     * : tag_id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $leadId = filter_var($this->_request->getParam('lead_id'), FILTER_SANITIZE_NUMBER_INT);
        $tagId = filter_var($this->_request->getParam('tag_id'), FILTER_SANITIZE_NUMBER_INT);
        if (!$leadId || !$tagId) {
            $this->_error();
        }
        $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
        $leadTagExists = $leadMapper->getByLeadIdTagId($leadId, $tagId);
        if (!empty($leadTagExists)) {
            $leadMapper->deleteTag($leadId, $tagId);
        } else {
            $this->_error();
        }

    }

}
