<?php

class Api_Leads_Leadip extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Get lead profile by
     *
     * Resource:
     * : /api/leads/leadip/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {


    }

    /**
     *
     * Resource:
     * : /api/leads/leadip/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING);
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'));
        }

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
        }

        if (empty($data['leadId'])) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Lead id is missing'));
        }

        if (empty($data['ipAddress'])) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Please specify ip address'));
        }

        $ipAddress = filter_var(trim($data['ipAddress']),FILTER_VALIDATE_IP);
        if (empty($ipAddress)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Please specify valid ip address'));
        }

        $leadId = $data['leadId'];

        $leadIpsMapper = Leads_Mapper_LeadsIpsMapper::getInstance();
        $leadIpRecordExists = $leadIpsMapper->getByLeadIdAndIp($leadId, $ipAddress);
        if ($leadIpRecordExists instanceof Leads_Model_LeadsIpsModel) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Ip is already assigned'));
        }

        $leadsIpsModel = Tools_LeadTools::addIpAddressToLead($leadId, $ipAddress);
        if ($leadsIpsModel instanceof Leads_Model_LeadsIpsModel) {
            Tools_LeadTools::addIpToMojo($leadsIpsModel);
        }

        $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Ip address has been added'));

    }

    /**
     * Resource:
     * : api/leads/leadip/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     *
     * @return JSON
     */
    public function putAction()
    {

    }

    /**
     *
     * Resource:
     * : api/leads/leadip/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $leadId = filter_var($this->_request->getParam('leadId'), FILTER_SANITIZE_NUMBER_INT);
        $ipAddress = filter_var($this->_request->getParam('ipAddress'), FILTER_SANITIZE_STRING);
        if (!$leadId || !$ipAddress) {
            $this->_error();
        }
        $translator = Zend_Registry::get('Zend_Translate');
        $leadIpsMapper = Leads_Mapper_LeadsIpsMapper::getInstance();
        $leadIpRecordExists = $leadIpsMapper->getByLeadIdAndIp($leadId, $ipAddress);
        if ($leadIpRecordExists instanceof Leads_Model_LeadsIpsModel) {
            $leadsIpUserProfileLogMapper = Leads_Mapper_LeadsIpUserProfileLogMapper::getInstance();
            $leadsIpUserProfileLogMapper->deleteByIpLeadId($ipAddress, $leadId);
            $leadIpsMapper->deleteByLeadIdIp($leadId, $ipAddress);
            $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Ip address has been deleted'));
        } else {
            $this->_error();
        }
    }

}
