<?php

class Api_Leads_Leadmeeting extends Api_Service_Abstract
{

    const MEETING_DEFAULT_EMAIL_SUFFIX = 'meeting';

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('start_time_date', 'end_time_date');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Get lead meeting by id
     *
     * Resource:
     * : /api/leads/leadmeeting/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
    }

    /**
     * Create new lead meeting
     *
     * Resource:
     * : /api/leads/leadmeeting/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = $this->getRequest()->getParams();
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'));
        }

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
        $websiteUrl = $websiteHelper->getUrl();
        if (!$tokenValid) {
            $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
        }

        $leadId = filter_var($data['lead_id'], FILTER_SANITIZE_NUMBER_INT);
        $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
        $leadModel = $leadMapper->find($leadId);
        if (!$leadModel instanceof Leads_Model_LeadsModel) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Lead doesn\'t exists'));
        }

        $leadsMeetingMapper = Leads_Mapper_LeadsMeetingMapper::getInstance();
        $leadsMeetingModel = new Leads_Model_LeadsMeetingModel();
        $session = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
        $currentLoggedUserId = $session->getCurrentUser()->getId();
        if (empty($data['user_id'])) {
            $data['user_id'] = $currentLoggedUserId;
        }
        $data['creator_id'] = $currentLoggedUserId;
        $data['organizer'] = $session->getCurrentUser()->getFullName();

        $defaultMeetingEmail = Leads_Mapper_LeadsConfigMapper::getInstance()->getConfigParam('defaultMeetingEmail');

        if (empty($defaultMeetingEmail)) {
            $currentHost = explode('.', parse_url($websiteUrl, PHP_URL_HOST));
            $domain1 = array_pop($currentHost);
            $domain2 = array_pop($currentHost);
            $data['organizerEmail'] = self::MEETING_DEFAULT_EMAIL_SUFFIX.'@'.$domain2.'.'.$domain1;
        } else {
            $data['organizerEmail'] = $defaultMeetingEmail;
        }

        $data['invite_attendees'] = implode(',' ,array_filter(array_unique(array_map('trim',explode(',', $data['invite_attendees'])))));
        if (!empty($data['invite_attendees'])) {
            $data['invite_attendees'] = $data['invite_attendees'].','.$data['organizerEmail'];
        } else {
            $data['invite_attendees'] = $data['organizerEmail'];
        }

        $leadEmail = $leadModel->getLeadEmail();
        if (!empty($leadEmail)) {
            $data['invite_attendees'] = trim($data['invite_attendees'] . ',' . $leadEmail, ',');
        }

        if (empty($data['invite_attendees'])) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'You must specify lead email or attendees email address.'));
        }

        $createdAt = Tools_System_Tools::convertDateFromTimezone('now');

        $leadTypeModel = Leads_Mapper_LeadsTypeMapper::getInstance()->findByTypeName(Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_MEETING);
        if ($leadTypeModel instanceof Leads_Model_LeadsTypeModel) {
            $leadModel->setLastActivityTypeId($leadTypeModel->getId());
            $leadModel->setLastActivityOutbound($createdAt);
            $leadModel->setLastActivitySubtype('');
            $leadModel = $leadMapper->save($leadModel);
        }

        if (empty($data['meeting_type'])) {
            $data['meeting_type'] = Leads_Model_LeadsMeetingModel::LEAD_MEETING_TYPE_AUTO;
        }

        if (empty($data['google_calendar'])) {
            $data['google_calendar'] = 0;
        }

        $leadsMeetingModel->setOptions($data);

        $startMeetingTime = Tools_System_Tools::convertDateFromTimezone($data['start_time_date']);
        $endMeetingTime =  Tools_System_Tools::convertDateFromTimezone($data['end_time_date']);
        $leadsMeetingModel->setStartTimeDate($startMeetingTime);
        $leadsMeetingModel->setEndTimeDate($endMeetingTime);

        if (empty($data['firstNotification'])) {
            $leadsMeetingModel->setFirstNotificationDate(null);
        } else {
            $firstNotificationDate = date(Tools_System_Tools::DATE_MYSQL,
                strtotime($startMeetingTime . '- '.$data['firstNotification'] . ' ' . $data['firstNotificationUnits']));
            $leadsMeetingModel->setFirstNotificationDate($firstNotificationDate);

        }

        if (empty($data['secondNotification'])) {
            $leadsMeetingModel->setSecondNotificationDate(null);
        } else {
            $secondNotificationDate = date(Tools_System_Tools::DATE_MYSQL,
                strtotime($startMeetingTime . '- '.$data['secondNotification'] . ' ' . $data['secondNotificationUnits']));
            $leadsMeetingModel->setSecondNotificationDate($secondNotificationDate);
        }

        $sendSms = false;
        $phoneNumber = '';
        if (!empty($data['sendSms'])) {
            $leadMobile = $leadModel->getLeadMobile();
            if (empty($leadMobile)) {
                $leadPhone = $leadModel->getLeadPhone();
                if (!empty($phoneNumber)) {
                    $phoneNumber = $leadModel->getLeadPhoneCountryCodeValue() . $leadPhone;
                }
            } else {
                $phoneNumber = $leadModel->getLeadMobileCountryCodeValue() . $leadMobile;
            }

            $phoneNumber = Apps_Tools_Twilio::normalizePhoneNumberToE164($phoneNumber);
            if (!empty($phoneNumber)) {
                $sendSms = true;
            } else {
                $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Not valid lead phone number'));
            }
        }

        $leadsMeetingModel->registerObserver(new Tools_Mail_Watchdog(array(
            'trigger' => Tools_LeadsMailWatchdog::LEADS_MEETING,
            'params' => $data,
            'leadModel' => $leadModel,
            'sendSms' => $sendSms,
            'smsPhoneNumber' => $phoneNumber,
        )));

        $leadsMeetingModel = $leadsMeetingMapper->save($leadsMeetingModel);

        if (empty($data['manualMeeting'])) {
            $leadsMeetingModel->notifyObservers();
        }

        $actionLogCreatedModel = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_MEETING, $leadId,
            Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND, $createdAt, Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_MEETING);

        $actionLogExpiredModel = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_MEETING, $leadId,
            Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND, $endMeetingTime,  Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_MEETING);

        $actionLogUpcomingModel = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_MEETING, $leadId,
            Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND, $startMeetingTime,  Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_MEETING, Leads_Model_LeadsActionLogModel::EVENT_LOG_UPCOMING);

        $meetingLogMapper = Leads_Mapper_LeadsMeetingLogMapper::getInstance();
        $meetingLogModel = new Leads_Model_LeadsMeetingLogModel();
        $meetingLogModel->setOptions($data);
        $meetingLogModel->setActionLogId($actionLogCreatedModel->getId());
        $meetingLogModel->setMeetingId($leadsMeetingModel->getId());
        $meetingLogModel->setMeetingStatus('created');
        $meetingLogModel->setCreatorId($currentLoggedUserId);
        $meetingLogModel->setLeadId($leadId);
        $meetingLogModel->setStartTimeDate($startMeetingTime);
        $meetingLogModel->setEndTimeDate($endMeetingTime);
        $meetingLogMapper->save($meetingLogModel);

        $meetingLogModel = new Leads_Model_LeadsMeetingLogModel();
        $meetingLogModel->setOptions($data);
        $meetingLogModel->setActionLogId($actionLogUpcomingModel->getId());
        $meetingLogModel->setMeetingId($leadsMeetingModel->getId());
        $meetingLogModel->setMeetingStatus('upcoming');
        $meetingLogModel->setCreatorId($currentLoggedUserId);
        $meetingLogModel->setLeadId($leadId);
        $meetingLogModel->setStartTimeDate($startMeetingTime);
        $meetingLogModel->setEndTimeDate($endMeetingTime);
        $meetingLogMapper->save($meetingLogModel);

        $meetingLogModel = new Leads_Model_LeadsMeetingLogModel();
        $meetingLogModel->setOptions($data);
        $meetingLogModel->setActionLogId($actionLogExpiredModel->getId());
        $meetingLogModel->setMeetingId($leadsMeetingModel->getId());
        $meetingLogModel->setMeetingStatus('finished');
        $meetingLogModel->setCreatorId($currentLoggedUserId);
        $meetingLogModel->setLeadId($leadId);
        $meetingLogModel->setStartTimeDate($startMeetingTime);
        $meetingLogModel->setEndTimeDate($endMeetingTime);
        $meetingLogMapper->save($meetingLogModel);

        $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Meeting has been created'));
    }

    /**
     * Update lead meeting data
     *
     * Resource:
     * : /api/leads/leadmeeting/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     * : lead meeting id to update
     *
     * @return JSON
     */
    public function putAction()
    {

    }

    /**
     * Delete lead meeting
     *
     * Resource:
     * : /api/leads/leadmeeting/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     * : lead meeting id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        if (!$id) {
            $this->_error();
        }
        $leadsMeetingMapper = Leads_Mapper_LeadsMeetingMapper::getInstance();
        $leadMeetingModel = $leadsMeetingMapper->find($id);
        if ($leadMeetingModel instanceof Leads_Model_LeadsMeetingModel) {
            $leadsMeetingMapper->delete($id);
        } else {
            $this->_error();
        }

    }

}
