<?php

class Api_Leads_Leadmobileapp extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Get lead mobile app config
     *
     * Resource:
     * : /api/leads/leadmobileapp/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = '';
        $sessionHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
        $currentUser = $sessionHelper->getCurrentUser();
        $userRoleId = $currentUser->getRoleId();
        $userId = $currentUser->getId();

        $leadMobileAppDataMapper = Leads_Mapper_LeadsMobileAppConfigMapper::getInstance();
        $where = null;
        if ($userRoleId == Shopping::ROLE_SALESPERSON) {
            $where = $leadMobileAppDataMapper->getDbTable()->getAdapter()->quoteInto('plmac.user_id = ?', $userId);
        }

        $data = $leadMobileAppDataMapper->fetchAllData($where, $sortOrder, $limit, $offset);

        if (empty($data)) {
            $data = array();
        }

        $data['leadOwners'] = Leads_Mapper_LeadsMapper::getInstance()->getLeadOwnersFullList(0, array(), true);
        if ($userRoleId == Shopping::ROLE_SALESPERSON) {
            $data['leadOwners'] = Leads_Mapper_LeadsMapper::getInstance()->getLeadOwnersFullList(0, array($userId), true);
        }

        if (!empty($data) && !empty($data['leadOwners'])) {
            $uniqueLogins = $leadMobileAppDataMapper->getUniqueLogins();
            $leadOwnerDropdown = $data['leadOwners'];
            foreach ($leadOwnerDropdown as $key => $leadOwner) {
                if (array_key_exists($leadOwner['email'], $uniqueLogins)) {
                    unset($leadOwnerDropdown[$key]);
                }
            }

            $data['leadOwnersDropdown'] = $leadOwnerDropdown;
        }

        return $data;
    }

    /**
     * Create new imap config
     *
     * Resource:
     * : /api/leads/leadmobileapp/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING);
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error(array(
                'status' => 'error',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'),
                'errorCode' => '1'
            ), 200);
        }

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in ' . '<a href="' . $websiteUrl . 'go">here</a>'));
        }

        $leadOwnerId = filter_var($this->_request->getParam('leadOwnerId'), FILTER_SANITIZE_NUMBER_INT);
        if (empty($leadOwnerId)) {
            $this->_error(array(
                'status' => 'error',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Please specify lead owner'),
                'errorCode' => '1'
            ), 200);
        }

        $userModel = Application_Model_Mappers_UserMapper::getInstance()->find($leadOwnerId);
        if (!$userModel instanceof Application_Model_Models_User) {
            $this->_error(array(
                'status' => 'error',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'User not found'),
                'errorCode' => '1'
            ), 200);
        }

        $sessionHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
        $currentUser = $sessionHelper->getCurrentUser();
        $userRoleId = $currentUser->getRoleId();
        $userId = $currentUser->getId();
        if ($userRoleId == Shopping::ROLE_SALESPERSON) {
            if ($userId != $leadOwnerId) {
                $this->_error(array(
                    'status' => 'error',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'You don\'t have enough permissions to grant access'),
                    'errorCode' => '1'
                ), 200);
            }
        }

        $leadMobileAppDataMapper = Leads_Mapper_LeadsMobileAppConfigMapper::getInstance();
        $leadMobileAppDataModel = $leadMobileAppDataMapper->findByEmail($userModel->getEmail());
        if ($leadMobileAppDataModel instanceof Leads_Model_LeadsMobileAppConfigModel) {
            $this->_error(array(
                'status' => 'error',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'You already granted access to this user'),
                'errorCode' => '1'
            ), 200);
        }

        $password = $userModel->getPassword();
        $login = $userModel->getEmail();
        $userId = $userModel->getId();
        $params = array('userId' => $userId, 'login' => $login, 'password' => $password,
            'fullName' => $userModel->getFullName());
        $response = Apps::apiCall('POST', 'mobileAppRemoteUser', '', $params);
        if (empty($response)) {
            $this->_error(array(
                'status' => 'error',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'No connection with authorization server'),
                'errorCode' => '1'
            ), 200);
        }

        if (empty($response['done'])) {
            $this->_error(array(
                'status' => 'error',
                'message' => $response['message'],
                'errorCode' => '1'
            ), 200);
        }

        $leadMobileAppDataModel = new Leads_Model_LeadsMobileAppConfigModel();
        $leadMobileAppDataModel->setUserId($leadOwnerId);
        $leadMobileAppDataModel->setStatus(Leads_Model_LeadsMobileAppConfigModel::STATUS_ENABLED);
        if (!empty($password)) {
            $leadMobileAppDataModel->setPassword($password);
        }
        $leadMobileAppDataModel->setLogin($login);
        $leadMobileAppDataModel->setCreatedAt(Tools_System_Tools::convertDateFromTimezone('now'));
        if (!empty($response['socialAppLoginDisabled'])) {
            $leadMobileAppDataModel->setSocialAppStatus(Leads_Model_LeadsMobileAppConfigModel::STATUS_SOCIAL_APP_DISABLED);
        }

        $leadMobileAppDataModel = $leadMobileAppDataMapper->save($leadMobileAppDataModel);

        $message = $translator->translate(Leads::TRANSLATION_PREFIX.'CRM\'s mobile APP access has been enabled for this account');
        if (!empty($response['message'])) {
            $message = $response['message'];
        }

        return array(
            'error' => '0',
            'message' => $message,
            'imapInfo' => $leadMobileAppDataModel->toArray()
        );
    }

    /**
     * Update lead mobile app data
     *
     * Resource:
     * : /api/leads/leadmobileapp/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     * : lead mobile app id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if (!empty($data['id']) && !empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {
            $translator = Zend_Registry::get('Zend_Translate');
            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();
                $this->_error($translator->translate('Your session has timed-out. Please Log back in ' . '<a href="' . $websiteUrl . 'go">here</a>'));
            }
            $mobileAppRecordId = filter_var($data['id'], FILTER_SANITIZE_NUMBER_INT);
            $leadMobileAppDataMapper = Leads_Mapper_LeadsMobileAppConfigMapper::getInstance();
            $leadMobileAppDataModel = $leadMobileAppDataMapper->find($mobileAppRecordId);
            if (!$leadMobileAppDataModel instanceof Leads_Model_LeadsMobileAppConfigModel) {
                $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Record not found'));
            }

            $userId = $leadMobileAppDataModel->getUserId();
            $userModel = Application_Model_Mappers_UserMapper::getInstance()->find($userId);
            if (!$userModel instanceof Application_Model_Models_User) {
                $this->_error(array(
                    'status' => 'error',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'User not found'),
                    'errorCode' => '1'
                ), 200);
            }


            $sessionHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
            $currentUser = $sessionHelper->getCurrentUser();
            $userRoleId = $currentUser->getRoleId();
            $userId = $currentUser->getId();
            if ($userRoleId == Shopping::ROLE_SALESPERSON) {
                if ($userId != $leadMobileAppDataModel->getUserId()) {
                    $this->_error(array(
                        'status' => 'error',
                        'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'You don\'t have enough permissions to grant access'),
                        'errorCode' => '1'
                    ), 200);
                }
            }

            $password = $userModel->getPassword();
            $login = $userModel->getEmail();
            $userId = $userModel->getId();

            $params = array('userId' => $userId, 'login' => $login, 'password' => $password,
                'fullName' => $userModel->getFullName());

            //Enable or disable app access
            if (isset($data['status'])) {
                if (!empty($data['status'])) {
                    if ($data['status'] === 'enabled') {
                        $response = Apps::apiCall('POST', 'mobileAppRemoteUser', '', $params);
                        if (empty($response)) {
                            $this->_error(array(
                                'status' => 'error',
                                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'No connection with authorization server'),
                                'errorCode' => '1'
                            ), 200);
                        }

                        if (empty($response['done'])) {
                            $this->_error(array(
                                'status' => 'error',
                                'message' => $response['message'],
                                'errorCode' => '1'
                            ), 200);
                        }

                        $leadMobileAppDataModel->setStatus($data['status']);
                        $leadMobileAppDataModel->setPassword($userModel->getPassword());
                        $leadMobileAppDataModel->setUpdatedAt(Tools_System_Tools::convertDateFromTimezone('now'));
                        $leadMobileAppDataMapper->save($leadMobileAppDataModel);

                        return array(
                            'error' => '0',
                            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'CRM\'s mobile APP access has been enabled for this account'),
                            'imapInfo' => $leadMobileAppDataModel->toArray()
                        );

                    } else {
                        $response = Apps::apiCall('DELETE', 'mobileAppRemoteUser', '', array('userId' => $userId));
                        if (empty($response)) {
                            $this->_error(array(
                                'status' => 'error',
                                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'No connection with authorization server'),
                                'errorCode' => '1'
                            ), 200);
                        }

                        if (empty($response['done'])) {
                            $this->_error(array(
                                'status' => 'error',
                                'message' => $response['message'],
                                'errorCode' => '1'
                            ), 200);
                        }

                        $message = $translator->translate(Leads::TRANSLATION_PREFIX.'CRM\'s mobile APP access has been disabled for this account');
                        if (!empty($response['message'])) {
                            $message = $response['message'];
                        }

                        $leadMobileAppDataModel->setSocialAppStatus('disabled');
                        $leadMobileAppDataModel->setStatus($data['status']);
                        $leadMobileAppDataModel->setPassword($userModel->getPassword());
                        $leadMobileAppDataModel->setUpdatedAt(Tools_System_Tools::convertDateFromTimezone('now'));
                        $leadMobileAppDataMapper->save($leadMobileAppDataModel);

                        return array(
                            'error' => '0',
                            'message' => $message,
                            'imapInfo' => $leadMobileAppDataModel->toArray()
                        );
                    }
                }
            }

            //Enable or disable social app access
            if (isset($data['socialAppStatus'])) {
                $currentStatus = $leadMobileAppDataModel->getStatus();
                if ($currentStatus === 'disabled') {
                    $this->_error(array(
                        'status' => 'error',
                        'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'You can\'t enable social APP access if account disabled.'),
                        'errorCode' => '1'
                    ), 200);
                }

                if (!empty($data['socialAppStatus'])) {
                    $socialAppLoginDisabled = '1';
                    $socialMessage = $translator->translate(Leads::TRANSLATION_PREFIX.'Social APP access has been disabled for this account');
                    if ($data['socialAppStatus'] === 'enabled') {
                        $socialAppLoginDisabled = '0';
                        $socialMessage = $translator->translate(Leads::TRANSLATION_PREFIX.'Social APP access has been enabled for this account');
                    }

                    $params = array(
                        'userId' => $userId,
                        'login' => $login,
                        'password' => $password,
                        'socialAppLoginDisabled' => $socialAppLoginDisabled,
                        'fullName' => $userModel->getFullName()
                    );
                    $response = Apps::apiCall('PUT', 'mobileAppRemoteUser', '', $params);
                    if (empty($response)) {
                        $this->_error(array(
                            'status' => 'error',
                            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'No connection with authorization server'),
                            'errorCode' => '1'
                        ), 200);
                    }

                    if (empty($response['done'])) {
                        $this->_error(array(
                            'status' => 'error',
                            'message' => $response['message'],
                            'errorCode' => '1'
                        ), 200);
                    }

                    $leadMobileAppDataModel->setSocialAppStatus($data['socialAppStatus']);
                    $leadMobileAppDataModel->setPassword($userModel->getPassword());
                    $leadMobileAppDataModel->setUpdatedAt(Tools_System_Tools::convertDateFromTimezone('now'));
                    $leadMobileAppDataMapper->save($leadMobileAppDataModel);

                    return array(
                        'error' => '0',
                        'message' => $socialMessage,
                        'imapInfo' => $leadMobileAppDataModel->toArray()
                    );
                }
            }

            $response = Apps::apiCall('PUT', 'mobileAppRemoteUser', '', $params);
            if (empty($response)) {
                $this->_error(array(
                    'status' => 'error',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'No connection with authorization server'),
                    'errorCode' => '1'
                ), 200);
            }

            if (empty($response['done'])) {
                $this->_error(array(
                    'status' => 'error',
                    'message' => $response['message'],
                    'errorCode' => '1'
                ), 200);
            }

            $status = $leadMobileAppDataModel->getStatus();
            if ($status === 'disabled') {
                $this->_error(array(
                    'status' => 'error',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'You can\'t update password if account disabled'),
                    'errorCode' => '1'
                ), 200);
            }

            if (!empty($response['socialAppLoginDisabled'])) {
                $leadMobileAppDataModel->setSocialAppStatus(Leads_Model_LeadsMobileAppConfigModel::STATUS_SOCIAL_APP_DISABLED);
            }

            $leadMobileAppDataModel->setPassword($userModel->getPassword());
            $leadMobileAppDataModel->setUpdatedAt(Tools_System_Tools::convertDateFromTimezone('now'));
            $leadMobileAppDataMapper->save($leadMobileAppDataModel);

            $message = $translator->translate(Leads::TRANSLATION_PREFIX.'Access to the mobile APP has been updated!');
            if (!empty($response['message'])) {
                $message = $response['message'];
            }

            return array(
                'error' => '0',
                'message' => $message,
                'imapInfo' => $leadMobileAppDataModel->toArray()
            );
        }


    }

    /**
     * Delete lead mobile app  record
     *
     * Resource:
     * : /api/leads/leadmobileapp/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     * : lead mobile app config id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {


    }

}
