<?php

class Api_Leads_Leadopportunitystage extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('opportunityStage');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Get lead opportunity stage by id
     *
     * Resource:
     * : /api/leads/leadopportunitystage/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);

        $leadOpportunityDataMapper = Leads_Mapper_LeadsOpportunityStageMapper::getInstance();
        $data = array();
        if ($id) {
            $where = $leadOpportunityDataMapper->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $data = $leadOpportunityDataMapper->fetchAll($where);
        } else {
            $stages = $leadOpportunityDataMapper->fetchAll();
            if(!empty($stages)) {
                $data = array_map(function ($stage) {
                    return $stage->toArray();
                }, $stages);
            }
        }

        return $data;
    }


    public function postAction() {}


    /**
     * Update lead opportunity stage data
     *
     * Resource:
     * : /api/leads/leadopportunitystage/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     * : lead opportunity id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = array();
        parse_str($this->_request->getRawBody(), $data);
        if (!empty($data['id']) && !empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {
            $translator = Zend_Registry::get('Zend_Translate');
            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();
                $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
            }
            $id = filter_var($data['id'], FILTER_SANITIZE_NUMBER_INT);
            $description = filter_var($data['description'], FILTER_SANITIZE_STRING);
            $leadOpportunityStageMapper = Leads_Mapper_LeadsOpportunityStageMapper::getInstance();
            $leadOpportunityStageModel = $leadOpportunityStageMapper->find($id);
            if (!$leadOpportunityStageModel instanceof Leads_Model_LeadsOpportunityStageModel) {
                $this->_responseHelper->fail($translator->translate(Leads::TRANSLATION_PREFIX.'Config doesn\'t exists'));
            }
            $leadOpportunityStageModel->setDescription($description);
            $leadOpportunityStageMapper->save($leadOpportunityStageModel);
            $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Opportunity stage has been added'));
        }

    }

    /**
     * Delete lead opportunity stage
     *
     * Resource:
     * : /api/leads/leadopportunitystage/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     * : lead opportunity id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        if (!$id) {
            $this->_error();
        }
        $leadsOpportunityStageMapper = Leads_Mapper_LeadsOpportunityStageMapper::getInstance();
        $leadOpportunityStageModel = $leadsOpportunityStageMapper->find($id);
        if ($leadOpportunityStageModel instanceof Leads_Model_LeadsOpportunityStageModel) {
            $leadOpportunityStageModel->setDescription('');
            $leadsOpportunityStageMapper->save($leadOpportunityStageModel);
            $translator = Zend_Registry::get('Zend_Translate');
            $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Opportunity has been deleted'));
        } else {
            $this->_error();
        }

    }

}
