<?php

class Api_Leads_Leadopportunitytype extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('type_name', 'type_label');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Get lead opportunity type by id
     *
     * Resource:
     * : /api/leads/leadopportunitytype/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order', 'plot.type_name'), FILTER_SANITIZE_STRING);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $searchTerm = filter_var($this->_request->getParam('searchTerm'), FILTER_SANITIZE_STRING);

        $opportunityTypeDataMapper = Leads_Mapper_LeadsOpportunityTypeMapper::getInstance();
        if ($id) {
            $where = $opportunityTypeDataMapper->getDbTable()->getAdapter()->quoteInto('plt.id = ?', $id);
            $data = $opportunityTypeDataMapper->fetchAll($where);
        }  elseif (!empty($searchTerm)) {
            $where = $opportunityTypeDataMapper->getDbTable()->getAdapter()->quoteInto('plot.type_name LIKE ?',
                $searchTerm . '%');
            $data = $opportunityTypeDataMapper->fetchAll($where, $sortOrder, $limit, $offset, true);
        } else {
            $data = $opportunityTypeDataMapper->fetchAll(null, $sortOrder, $limit, $offset);
        }

        return $data;
    }

    /**
     * Create new lead opportunity type
     *
     * Resource:
     * : /api/leads/leadopportunitytype/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING);
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'));
        }

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
        }

        $data['type_name'] = rtrim(preg_replace('~[^\w\s\_]~ui', '', $this->_request->getParam('type_name')));
        $data['type_label'] = filter_var($this->_request->getParam('type_label'), FILTER_SANITIZE_STRING);

        $opportunityTypeMapper = Leads_Mapper_LeadsOpportunityTypeMapper::getInstance();
        $leadsOpportunityTypeModel = new Leads_Model_LeadsOpportunityTypeModel();
        $leadsOpportunityTypeModel->setOptions($data);

        $validator = new Zend_Validate_Db_NoRecordExists(array(
            'table' => 'plugin_leads_opportunity_type',
            'field' => 'type_name'
        ));

        if (!$validator->isValid($data['type_name'])) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Opportunity type with such name already exists'));
        }

        $validator = new Zend_Validate_Db_NoRecordExists(array(
            'table' => 'plugin_leads_opportunity_type',
            'field' => 'type_label'
        ));

        if (!$validator->isValid($data['type_label'])) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Opportunity label with such name already exists'));
        }


        $leadsOpportunityTypeModel = $opportunityTypeMapper->save($leadsOpportunityTypeModel);

        $this->_responseHelper->success(array('opportunityTypeInfo' => $leadsOpportunityTypeModel->toArray(), 'message' =>$translator->translate(Leads::TRANSLATION_PREFIX.'Opportunity type has been created')));
    }

    /**
     * Update lead opportunity type data
     *
     * Resource:
     * : /api/leads/leadopportunitytype/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     * : lead opportunity id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if (!empty($data['id']) && !empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {
            $translator = Zend_Registry::get('Zend_Translate');
            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();
                $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
            }
            $leadOpportunityId = filter_var($data['id'], FILTER_SANITIZE_NUMBER_INT);
            $leadOpportunityTypeMapper = Leads_Mapper_LeadsOpportunityTypeMapper::getInstance();
            $leadOpportunityTypeModel = $leadOpportunityTypeMapper->find($leadOpportunityId);
            if (!$leadOpportunityTypeModel instanceof Leads_Model_LeadsOpportunityTypeModel) {
                $this->_responseHelper->fail($translator->translate(Leads::TRANSLATION_PREFIX.'Config doesn\'t exists'));
            }

            $data['type_name'] = rtrim(preg_replace('~[^\w\s\_]~ui', '', $data['type_name']));
            $data['type_label'] = rtrim(trim(filter_var($data['type_label'], FILTER_SANITIZE_STRING)));
            $oldOpportunityType = $leadOpportunityTypeModel->getTypeName();
            $currentOpportunityType = $data['type_name'];
            $oldOpportunityTypeLabel =  $leadOpportunityTypeModel->getTypeLabel();
            $currentOpportunityLabel = $data['type_label'];
            if ($oldOpportunityType !== $currentOpportunityType) {
                $validateOpportunityTypeExists = new Zend_Validate_Db_RecordExists(array(
                    'table' => 'plugin_leads_opportunity_type',
                    'field' => 'type_name'
                ));
                if ($validateOpportunityTypeExists->isValid($currentOpportunityType)) {
                    $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'You have another opportunity type with such name'));
                }
            }
            if ($oldOpportunityTypeLabel !== $currentOpportunityLabel) {
                $validateOpportunityTypeExists = new Zend_Validate_Db_RecordExists(array(
                    'table' => 'plugin_leads_opportunity_type',
                    'field' => 'type_label'
                ));
                if ($validateOpportunityTypeExists->isValid($currentOpportunityLabel)) {
                    $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'You have another opportunity type with such label'));
                }
            }

            $leadOpportunityTypeModel->setOptions($data);
            $leadOpportunityTypeMapper->save($leadOpportunityTypeModel);
            $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Opportunity type has been updated'));
        }

    }

    /**
     * Delete lead opportunity type
     *
     * Resource:
     * : /api/leads/leadopportunitytype/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     * : lead opportunity type id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        if (!$id) {
            $this->_error();
        }
        $leadsOpportunityTypeMapper = Leads_Mapper_LeadsOpportunityTypeMapper::getInstance();
        $leadOpportunityTypeModel = $leadsOpportunityTypeMapper->find($id);
        if ($leadOpportunityTypeModel instanceof Leads_Model_LeadsOpportunityTypeModel) {
            $leadsOpportunityTypeMapper->delete($id);
        } else {
            $this->_error();
        }

    }

}
