<?php

class Api_Leads_Leadorganization extends Api_Service_Abstract
{


    /**
     * comparison operators
     */
    const RULE_COMPARISON_OPERATOR_EQUAL = 'equal';

    const RULE_COMPARISON_OPERATOR_NOTEQUAL = 'notequal';

    const RULE_COMPARISON_OPERATOR_LIKE = 'like';

    const RULE_COMPARISON_OPERATOR_IN = 'in';

    const RULE_COMPARISON_GREATER_THAN = 'greaterthan';

    const RULE_COMPARISON_LESS_THAN = 'lessthan';

    const RULE_COMPARISON_NOT_IN = 'notin';


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('organization_name');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    /**
     * User restricted id access
     *
     * @var null
     */
    protected $_restrictedUserId = null;

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
    }


    /**
     * Get lead organization by id
     *
     * Resource:
     * : /api/leads/leadorganization/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order'), FILTER_SANITIZE_STRING);
        $forGrid = (bool)filter_var($this->_request->getParam('isGrid', false), FILTER_SANITIZE_STRING);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $searchTerm = filter_var($this->_request->getParam('searchTerm'), FILTER_SANITIZE_STRING);
        $detailedInfo = filter_var($this->_request->getParam('detailedInfo'), FILTER_SANITIZE_STRING);
        $withOpportunityAmount = false;
        $withSpentPurchaseAmount = false;
        $withNextActivityDate = false;
        $having = null;
        $withLeadsData = false;
        $sortOrder = 'plo.created_at DESC';
        $includesCustomOrganizationFields = false;
        $additionalOrganizationParamsForSelect = array();

        $customPaginate = filter_var($this->_request->getParam('customPaginate'), FILTER_SANITIZE_NUMBER_INT);

        if(!empty($customPaginate)) {
            $limit = $customPaginate;
        }

        $leadOrganizationDataMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
        $leadDataMapper = Leads_Mapper_LeadsMapper::getInstance();
        if ($id && $forGrid) {
            $where = $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.id = ?', $id);
            if (!empty($this->_restrictedUserId)) {
                $where .= ' AND ('.  $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('pldm.owner_id = ?', $this->_restrictedUserId);
                $where .= ' OR pldm.owner_id IS NULL)';
            }
            $data = $leadOrganizationDataMapper->fetchAll($where, null, null, null, true, true);
        } elseif ($id) {
            $data = $leadOrganizationDataMapper->find($id);
            if ($data instanceof Leads_Model_LeadsOrganizationModel) {
                $data = $data->toArray();
            } else {
                $data =array();
            }
        } else {
            $filterParams = $this->_request->getParam('filter', array());

            foreach ($filterParams as $key => $param) {
                if (!is_array($param)) {
                    $filterParams[$key] = Tools_LeadTools::validateTextField($param);
                }
            }
            $where = null;
            if (!empty($filterParams)) {
                $filterParams['lead-organization-attribute'] = str_replace('*-amp-*', '&', $filterParams['lead-organization-attribute']);
                $where = '';
                if (!empty($filterParams['lead-organization-attribute'])) {
                    $filterParams['lead-organization-attribute'] = trim($filterParams['lead-organization-attribute'], " \t\n\r\x0B");

                    $organizationPhone = Tools_LeadTools::cleanPhoneAddress($filterParams['lead-organization-attribute']);
                    $organizationPhoneIsNumber = mb_substr(preg_replace('~[^0-9A-Za-z]~ui', '', $filterParams['lead-organization-attribute']), 0, 1);
                    if (strpos($filterParams['lead-organization-attribute'], '*') !== false) {
                        $attributeValue = str_replace('*', '%', $filterParams['lead-organization-attribute']);
                        $where .= ' (' . $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_name LIKE ?',
                                $attributeValue . '%');
                        if(!empty($organizationPhone) && is_numeric($organizationPhoneIsNumber)) {
                            $where .= ' OR ' . $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_phone LIKE ?',
                                    $organizationPhone . '%');
                            $where .= ' OR ' . $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_mobile LIKE ?',
                                    $organizationPhone . '%');
                        }
                    } else {
                        $attributeValues = explode(' ', $filterParams['lead-organization-attribute']);
                        $where .= ' ('.'(';

                        foreach ($attributeValues as $key => $attrVal) {
                            $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_name LIKE ?',
                                    '%' . $attrVal . '%');

                            if (count($attributeValues) > $key+1) {
                                $where .= ' AND ';
                            }

                        }

                        $where .= ')';


                        if(!empty($organizationPhone) && is_numeric($organizationPhoneIsNumber)) {
                            $where .= ' OR ' . $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_phone LIKE ?',
                                    $organizationPhone . '%');
                            $where .= ' OR ' . $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_mobile LIKE ?',
                                    $organizationPhone . '%');
                        }
                    }

                    if (!empty($filterParams['search_includes_notes'])) {
                        if (strpos($filterParams['lead-organization-attribute'], '*') !== false) {
                            $attributeValue = str_replace('*', '%', $filterParams['lead-organization-attribute']);
                            $where .= ' OR ' . $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_notes LIKE ?',
                                    $attributeValue . '%');
                        } else {
                            $attributeValues = explode(' ', $filterParams['lead-organization-attribute']);
                            $where .= ' OR ( ';

                            foreach ($attributeValues as $key => $attrVal) {
                                $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_notes LIKE ?',
                                    '%' . $attrVal . '%');

                                if (count($attributeValues) > $key+1) {
                                    $where .= ' AND ';
                                }
                            }

                            $where .= ')';

                        }
                    }

                    if (!empty($filterParams['search_includes_custom_organization_fields'])) {
                        $includesCustomOrganizationFields = true;
                        if (strpos($filterParams['lead-organization-attribute'], '*') !== false) {
                            $attributeValue = str_replace('*', '%', $filterParams['lead-organization-attribute']);
                            $where .= ' OR ' . $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plocpd.param_value LIKE ?',
                                    $attributeValue . '%');
                        } else {
                            $attributeValues = explode(' ', $filterParams['lead-organization-attribute']);
                            $where .= ' OR ( ';

                            foreach ($attributeValues as $key => $attrVal) {
                                $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plocpd.param_value LIKE ?',
                                    '%' . $attrVal . '%');

                                if (count($attributeValues) > $key+1) {
                                    $where .= ' AND ';
                                }
                            }

                            $where .= ')';
                        }
                    }
                    $where .=  ')';
                }

                if (!empty($this->_restrictedUserId)) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= ' ('.  $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('pldm.owner_id = ?', $this->_restrictedUserId);
                    $where .= ' OR pldm.owner_id IS NULL)';
                }

                if (!empty($filterParams['organization_without_leads'])) {
                    $organizationWithLeads = $leadOrganizationDataMapper->getOrganizationWithLeads();
                    if (!empty($organizationWithLeads)) {
                        if (!empty($where)) {
                            $where .= ' AND ';
                        }

                        $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.id NOT IN (?)', $organizationWithLeads);
                    }
                }

                if (!empty($filterParams['organization_zip'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $organizationZips = explode(',', $filterParams['organization_zip']);
                    $organizationZips = array_diff($organizationZips, array(''));
                    $organizationZips = array_map('trim', $organizationZips);

                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_zip IN (?)', $organizationZips);
                }

                if (!empty($filterParams['organization_city'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_city = ?', $filterParams['organization_city']);
                }

                if (!empty($filterParams['organization_country'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_country_code = ?', $filterParams['organization_country']);
                }

                if (!empty($filterParams['organization_country_state']) && is_array($filterParams['organization_country_state'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_state_code IN (?)',
                        $filterParams['organization_country_state']);
                }

                if(!empty($filterParams['created_at_from'])) {
                    $createdAt = Tools_System_Tools::convertDateFromTimezone($filterParams['created_at_from']);
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    if (!empty($filterParams['last_updated']) && $filterParams['last_updated'] === 'last_updated') {
                        $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.updated_at >= ?', $createdAt);
                    } else {
                        $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.created_at >= ?', $createdAt);
                    }

                }
                if(!empty($filterParams['created_at_to'])) {
                    $updatedAt =  Tools_System_Tools::convertDateFromTimezone($filterParams['created_at_to'].' 23:59:59');
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    if (!empty($filterParams['last_updated']) && $filterParams['last_updated'] === 'last_updated') {
                        $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.updated_at <= ?', $updatedAt);
                    } else {
                        $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.created_at <= ?', $updatedAt);
                    }

                }

                if (!empty($filterParams['customer_group_id'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.customer_group_id = ?', $filterParams['customer_group_id']);
                }

                if (!empty($filterParams['industry_id'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_industry_id = ?', $filterParams['industry_id']);
                }

                if (!empty($filterParams['organization_domain'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_domain = ?', $filterParams['organization_domain']);
                }

                if (!empty($filterParams['organization_domain1'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_domain1 = ?', $filterParams['organization_domain1']);
                }

                if (!empty($filterParams['last_interaction_id'])) {
                    $withLeadsData = true;
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('pld.last_activity_type_id = ?', $filterParams['last_interaction_id']);
                }

                if (!empty($filterParams['last_interaction_id']) && !empty($filterParams['outbound_emails_sub_type'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('pld.last_activity_subtype = ?', $filterParams['outbound_emails_sub_type']);
                }

                if (!empty($filterParams['total_opportunity_amount_from']) || !empty($filterParams['total_opportunity_amount_to']) || !empty($filterParams['lead_opportunity_stages']) || !empty($filterParams['opportunity_type_id'])) {
                    $withOpportunityAmount = true;
                }

                if (!empty($filterParams['total_spent_purchase_amount_from']) || !empty($filterParams['total_spent_purchase_amount_to']) || !empty($filterParams['special_filter_type']) && $filterParams['special_filter_type'] === Api_Leads_Leads::FILTER_EXISTING_CLIENT) {
                    $withSpentPurchaseAmount = true;

                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('scs.status IN (?)', array(Models_Model_CartSession::CART_STATUS_COMPLETED, Models_Model_CartSession::CART_STATUS_SHIPPED, Models_Model_CartSession::CART_STATUS_DELIVERED));
                }

                if($filterParams['special_filter_type'] === Api_Leads_Leads::FILTER_PROSPECT) {
                    $withSpentPurchaseAmount = true;

                }

                if (!empty($filterParams['lead_opportunity_stages'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plop.opportunity_stage_id IN (?)', $filterParams['lead_opportunity_stages']);
                }

                if (!empty($filterParams['lead_score_from'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('pld.score >= ?',
                        (int) $filterParams['lead_score_from']);
                    $withLeadsData = true;
                }

                if (!empty($filterParams['lead_score_to'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('pld.score <= ?',
                        (int) $filterParams['lead_score_to']);
                    $withLeadsData = true;
                }

                if (!empty($filterParams['organization_tags']) && is_array($filterParams['organization_tags'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plht.tag_id IN (?)', $filterParams['organization_tags']);
                    $withLeadsData = true;
                    $withLeadsTags = true;
                }
                if (!empty($filterParams['organization_tags_exclude']) && is_array($filterParams['organization_tags_exclude'])) {
                    $subSelect = $leadDataMapper->getDbTable()->select(Zend_Db_Table::SELECT_WITHOUT_FROM_PART)
                        ->distinct()
                        ->setIntegrityCheck(false)
                        ->from(array('ltags' => 'plugin_leads_has_tags'), array('ltags.lead_id'));
                    $subSelectWhere = $leadDataMapper->getDbTable()->getAdapter()->quoteInto('tag_id IN (?)', $filterParams['organization_tags_exclude']);
                    $subSelect->where($subSelectWhere);
                    $subSelectIds = $leadDataMapper->getDbTable()->getAdapter()->fetchCol($subSelect);
                    if (!empty($subSelectIds)) {
                        if (!empty($where)) {
                            $where .= ' AND ';
                        }
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.id NOT IN (?)', $subSelectIds);
                        $withLeadsData = true;
                        $withLeadsTags = true;
                    }
                }

                if (!empty($filterParams['special_filter_type'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    if ($filterParams['special_filter_type'] === Api_Leads_Leads::FILTER_EXISTING_CLIENT) {
                        $where .= new Zend_Db_Expr('pld.user_id IS NOT NULL');
                    }
                    if ($filterParams['special_filter_type'] === Api_Leads_Leads::FILTER_PROSPECT) {
                        $where .= new Zend_Db_Expr('pld.user_id IS NULL');
                        $where .= ' AND '. new Zend_Db_Expr('pld.id IS NOT NULL');
                    }
                    if ($filterParams['special_filter_type'] === Api_Leads_Leads::FILTER_NO_EMAIL) {
                        $withLeadsData = true;
                        $where .= new Zend_Db_Expr('pld.lead_email IS NULL');
                        $where .= ' AND '. new Zend_Db_Expr('pld.id IS NOT NULL');
                    }
                }

                if (!empty($filterParams['opportunity_type_id'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plop.opportunity_type_id = ?', $filterParams['opportunity_type_id']);
                }

                if (!empty($filterParams['total_opportunity_amount_from'])) {
                    if (!empty($having)) {
                        $having .= ' AND ';
                    }
                    $having .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('SUM(plop.opportunity_value) >= ?', $filterParams['total_opportunity_amount_from']);
                }
                if (!empty($filterParams['total_opportunity_amount_to'])) {
                    if (!empty($having)) {
                        $having .= ' AND ';
                    }
                    $having .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('SUM(plop.opportunity_value) <= ?', $filterParams['total_opportunity_amount_to']);
                }

                if (!empty($filterParams['total_spent_purchase_amount_from'])) {
                    if (!empty($having)) {
                        $having .= ' AND ';
                    }
                    $having .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('SUM(scs.total) >= ?',
                        $filterParams['total_spent_purchase_amount_from']);
                }

                if (!empty($filterParams['total_spent_purchase_amount_to'])) {
                    if (!empty($having)) {
                        $having .= ' AND ';
                    }
                    $having .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('SUM(scs.total) <= ?',
                        $filterParams['total_spent_purchase_amount_to']);
                }

                if (!empty($filterParams['custom_organization_params_search']) && is_array($filterParams['custom_organization_params_search'])) {
                    $leadsOrganizationCustomParamsMapper = Leads_Mapper_LeadsOrganizationCustomParamsConfigMapper::getInstance();
                    $tempParamCounter = 1;
                    foreach ($filterParams['custom_organization_params_search'] as  $searchCustomParamsData) {
                        $searchParamId = $searchCustomParamsData['id'];
                        $organizationCustomParamsModel = $leadsOrganizationCustomParamsMapper->find($searchParamId);
                        if ($organizationCustomParamsModel instanceof Leads_Model_LeadsOrganizationCustomParamsConfigModel) {
                            $includesCustomOrganizationFields = true;
                            $tempParamName = 'customOrganizationParamVal' . $tempParamCounter;
                            if (!empty($having)) {
                                $having .= ' AND ';
                            }
                            $operator = $searchCustomParamsData['operator'];
                            $paramOperator = '=';
                            if ($operator === self::RULE_COMPARISON_OPERATOR_EQUAL) {
                                $paramOperator = '=';
                            }
                            if ($operator === self::RULE_COMPARISON_OPERATOR_NOTEQUAL) {
                                $paramOperator = '<>';
                            }
                            if ($operator === self::RULE_COMPARISON_GREATER_THAN) {
                                $paramOperator = '>';
                                $searchCustomParamsData['value'] = floatval($searchCustomParamsData['value']);
                            }
                            if ($operator === self::RULE_COMPARISON_LESS_THAN) {
                                $paramOperator = '<';
                                $searchCustomParamsData['value'] = floatval($searchCustomParamsData['value']);
                            }

                            if ($operator === self::RULE_COMPARISON_OPERATOR_LIKE) {
                                $paramOperator = 'LIKE';
                                $having .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('(' . $tempParamName . ') ' . $paramOperator . ' ?',
                                    '%'. $searchCustomParamsData['value']. '%');
                            } elseif ($operator === self::RULE_COMPARISON_OPERATOR_EQUAL) {
                                if (empty($searchCustomParamsData['value'])) {
                                    $having .= '('.$leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('(' . $tempParamName . ') ' . $paramOperator . ' ?',
                                            $searchCustomParamsData['value']).' OR ';
                                    $having .= '(' . $tempParamName . ') '. 'IS NULL';

                                    $having .=   ')';
                                } else {
                                    $having .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('(' . $tempParamName . ') ' . $paramOperator . ' ?',
                                        $searchCustomParamsData['value']);
                                }
                            } elseif ($operator === self::RULE_COMPARISON_GREATER_THAN || $operator === self::RULE_COMPARISON_LESS_THAN) {
                                $having .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('CAST(' . $tempParamName . '  AS DECIMAL(10,2))' . $paramOperator . ' ?',
                                    $searchCustomParamsData['value']);
                            } else {
                                $having .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('(' . $tempParamName . ') ' . $paramOperator . ' ?',
                                    $searchCustomParamsData['value']);
                            }
                            $additionalOrganizationParamsForSelect = array_merge($additionalOrganizationParamsForSelect,
                                array($tempParamName => new Zend_Db_Expr('GROUP_CONCAT(DISTINCT(CASE WHEN plocpd.param_id=' . $searchParamId . ' THEN plocpd.param_value END))')));
                            $tempParamCounter++;
                        }
                    }
                }

            } elseif (!empty($searchTerm)) {
                $where = $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_name LIKE ?',
                    $searchTerm . '%');
            }


            if ($forGrid) {
                $data = $leadOrganizationDataMapper->fetchAll($where, $sortOrder, $limit, $offset, false, false, $withOpportunityAmount, $having, $withNextActivityDate, $withLeadsData, $includesCustomOrganizationFields, $withSpentPurchaseAmount, $additionalOrganizationParamsForSelect, $withLeadsTags);
            } else {
                $data = $leadOrganizationDataMapper->fetchAll($where, null, null, null, true, false, false, $withOpportunityAmount, $having, $withNextActivityDate, $withLeadsData, $withSpentPurchaseAmount, $additionalOrganizationParamsForSelect, $withLeadsTags);
            }


        }

        $currency = Zend_Registry::get('Zend_Currency');

        if ($forGrid) {
            $moneyFormat = Tools_LeadTools::getCurrencyFormat();
            $data['moneyFormat'] = json_encode($moneyFormat);
            $listMasksMapper = Application_Model_Mappers_MasksListMapper::getInstance();
            $data['mobileMasks'] = $listMasksMapper->getListOfMasksByType(Application_Model_Models_MaskList::MASK_TYPE_MOBILE);
            $data['desktopMasks'] = $listMasksMapper->getListOfMasksByType(Application_Model_Models_MaskList::MASK_TYPE_DESKTOP);
            $data['groups'] = Leads_Mapper_LeadsMapper::getInstance()->getCustomerGroups();
            $data['industries'] = Leads_Mapper_LeadsIndustryMapper::getInstance()->getIndustriesPairs();
            if ($detailedInfo) {
                $data['countries'] = Tools_Geo::getCountries(true);
                if (!empty($data['organization_country_code'])) {
                    $data['states'] = Tools_LeadTools::getStates($data['organization_country_code'], true, true);
                }
                $leadOpportunityDataMapper = Leads_Mapper_LeadsOpportunityMapper::getInstance();
                $data['phoneCountryCodes'] =  Tools_System_Tools::getFullCountryPhoneCodesList(true, array(), true);
                $data['lastContacted'] = $leadOrganizationDataMapper->getLastContactedLead($id);
                $data['firstPurchaseDate'] = $leadOrganizationDataMapper->getFirstPurchaseDate($id);
                $data['organizationLeads'] = $leadOrganizationDataMapper->getOrganizationLeads($id);
                $data['leadOpportunityStats'] =  $leadOpportunityDataMapper->getLeadOrganizationOpportunityQuoteStatSalesDetailedView($id);
                $data['isQuizzEnabled'] = Tools_LeadTools::isPluginEnabled('Quizz');
                $totalSales = $leadOpportunityDataMapper->getLeadOrganizationTotalSalesDetailedView($id);
                $opportunitiesWhichNotSold = $leadOpportunityDataMapper->getOpportunitiesOrganizationWhichNotSold($data['id']);

                $leadsCustomTabsConfigMapper = Leads_Mapper_LeadsCustomTabsConfigMapper::getInstance();

                $customTabsWhere = $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('lct.tab_type = ?', Api_Leads_Leadcustomtabsfields::CUSTOM_TAB_TYPE_ORGANIZATION);
                $customConfigsTabs = $leadsCustomTabsConfigMapper->getCustomTabsConfigInfo($customTabsWhere, null, null, null, null, true);

                if(!empty($customConfigsTabs)) {
                    $data['customConfigsTabs'] = $customConfigsTabs;
                }

                if (!empty($opportunitiesWhichNotSold)) {
                    foreach ($opportunitiesWhichNotSold as $opportunitiesWhichNotSoldSingle) {
                        if (!empty($totalSales)) {
                            $totalSales['count'] = $totalSales['count'] + 1;
                            $totalSales['total'] = $totalSales['total'] + $opportunitiesWhichNotSoldSingle['opportunity_value'];
                        }   else {
                            $totalSales = array(
                                'count' => 1,
                                'total' => $opportunitiesWhichNotSoldSingle['opportunity_value']
                            );
                        }
                    }
                }
                $opportunitiesWithoutQuotes = $leadOpportunityDataMapper->getOpportunitiesOrganizationWithoutQuotes($data['id']);
                if (!empty($opportunitiesWithoutQuotes)) {
                    foreach ($opportunitiesWithoutQuotes as $opportunitiesWithoutQuotesSingle) {
                        if (!empty($totalSales)) {
                            $totalSales['count'] = $totalSales['count'] + 1;
                            $totalSales['total'] = $totalSales['total'] + $opportunitiesWithoutQuotesSingle['opportunity_value'];
                        } else {
                            $totalSales = array(
                                'count' => 1,
                                'total' => $opportunitiesWithoutQuotesSingle['opportunity_value']
                            );
                        }
                    }
                }

//                if (!empty($data['leadOpportunityStats']) && !empty($data['leadOpportunityStats']['100'])) {
//                    if (!empty($totalSales)) {
//                        $totalSales['count'] = $totalSales['count'] + $data['leadOpportunityStats']['100']['count'];
//                        $totalSales['total'] = $totalSales['total'] + $data['leadOpportunityStats']['100']['total'];
//                    } else {
//                        $totalSales = array(
//                            'count' => $data['leadOpportunityStats']['100']['count'],
//                            'total' => $data['leadOpportunityStats']['100']['total']
//                        );
//                    }
//                }
//                $leadOpportunityStatsExclude = $leadOpportunityDataMapper->getCartStatsForOpportunityStageOrganization($data['id'], array('100'));
//                if (!empty($totalSales) && !empty($leadOpportunityStatsExclude)) {
//                    $totalSales['count'] = $totalSales['count'] - $leadOpportunityStatsExclude['count'];
//                    $totalSales['total'] = $totalSales['total'] - $leadOpportunityStatsExclude['total'];
//                }
                if (!empty($totalSales)) {
                    $data['totalSales'] = $totalSales;
                } else {
                    $data['totalSales'] = '';
                }
                $data['userPrefixes'] = Tools_LeadTools::$userPrefixes;
            }
        }

        if (!empty($data)){
            $data['currencySymbol'] = preg_replace('~[\w]~', '', $currency->getSymbol());
            $data['userTimezoneOffset'] = Tools_System_Tools::convertDateFromTimezone('now', 'UTC', Tools_System_Tools::getUserTimezone(), 'T');
        }

        return $data;
    }

    /**
     * Create new lead organization
     *
     * Resource:
     * : /api/leads/leadorganization/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING);
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'));
        }

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
        }

        if (!empty($data['organization_mobile'])) {
            $data['organization_mobile'] = Tools_LeadTools::cleanPhoneAddress($data['organization_mobile']);
        }

        if (!empty($data['organization_phone'])) {
            $data['organization_phone'] = Tools_LeadTools::cleanPhoneAddress($data['organization_phone']);
        }

        if (empty($data['customer_group_id'])) {
            $data['customer_group_id'] = null;
        }

        if (empty($data['organization_industry_id'])) {
            $data['organization_industry_id'] = null;
        }

        if (empty($data['organization_country_code'])) {
            $data['organization_country_code'] = null;
        }

        $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
        $userDefaultMobileCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');
        if (empty($data['organization_phone_country_code']) && !empty($userDefaultMobileCountryCode)) {
            $data['organization_phone_country_code'] = $userDefaultMobileCountryCode;

            if (!empty($data['organization_mobile_country_code'])) {
                $mobileCountryPhoneCode = Zend_Locale::getTranslation($data['organization_mobile_country_code'],
                    'phoneToTerritory');
                $data['organization_mobile_country_code_value'] = '+' . $mobileCountryPhoneCode;
            }
        }

        if (empty($data['organization_mobile_country_code']) && !empty($userDefaultMobileCountryCode)) {
            $data['organization_mobile_country_code'] = $userDefaultMobileCountryCode;

            if (!empty($data['organization_phone_country_code'])) {
                $phoneCountryPhoneCode = Zend_Locale::getTranslation($data['organization_phone_country_code'],
                    'phoneToTerritory');
                $data['organization_phone_country_code_value'] = '+' . $phoneCountryPhoneCode;
            }
        }

        if (empty($data['primary_contact_id'])) {
            $data['primary_contact_id'] = null;
        }

        if (!empty($data['organization_website'])) {
            $data['organization_website'] = Tools_LeadTools::prepareWebsiteUrl($data['organization_website']);
            $domain = Tools_LeadTools::getDomainFromUrl($data['organization_website']);
            if (!empty($domain)) {
                $data['organization_domain'] = $domain;
            } else {
                $data['organization_domain'] = '';
            }
        } else {
            $data['organization_domain'] = '';
        }

        if (!empty($data['organization_website1'])) {
            $data['organization_website1'] = Tools_LeadTools::prepareWebsiteUrl($data['organization_website1']);
            $domain = Tools_LeadTools::getDomainFromUrl($data['organization_website1']);
            if (!empty($domain)) {
                $data['organization_domain1'] = $domain;
            } else {
                $data['organization_domain1'] = '';
            }
        } else {
            $data['organization_domain1'] = '';
        }

        $leadsOrganizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
        $leadsOrganizationModel = new Leads_Model_LeadsOrganizationModel();
        $leadsOrganizationModel->setOptions($data);

        $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();
        if (!empty($shoppingConfig['country'])) {
            $defaultCountryCode = $shoppingConfig['country'];
            if (empty($data['organization_country_code'])) {
                $leadsOrganizationModel->setOrganizationCountryCode($defaultCountryCode);
            }
        }

        if (!empty($data['customer_group_id'])) {
            $groupId = filter_var($data['customer_group_id'], FILTER_SANITIZE_NUMBER_INT);
            if (!empty($groupId)) {
                $validator = new Zend_Validate_Db_NoRecordExists(array(
                    'table' => 'shopping_group',
                    'field' => 'id'
                ));

                if ($validator->isValid($groupId)) {
                    $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Group is invalid.'));
                }
            }
        }

        $validator = new Zend_Validate_Db_NoRecordExists(array(
            'table' => 'plugin_lead_organization',
            'field' => 'organization_name'
        ));

        if (!$validator->isValid($data['organization_name'])) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Organization name already in use'));
        }

        $createdAt = Tools_System_Tools::convertDateFromTimezone('now');

        $leadsOrganizationModel->setCreatedAt($createdAt);
        $leadsOrganizationMapper->save($leadsOrganizationModel);

        $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Organization has been created'));
    }

    /**
     * Update lead organization data
     *
     * Resource:
     * : /api/leads/leadorganization/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     * : lead organization id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if (!empty($data['id']) && !empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {
            $translator = Zend_Registry::get('Zend_Translate');
            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();
                $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
            }
            $organizationId = filter_var($data['id'], FILTER_SANITIZE_NUMBER_INT);
            $leadOrganizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
            $leadOrganizationModel = $leadOrganizationMapper->find($organizationId);
            if (!$leadOrganizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                $this->_responseHelper->fail($translator->translate(Leads::TRANSLATION_PREFIX.'Config doesn\'t exists'));
            }

            if (!empty($data['mergeRecordWithId'])) {
                if (!empty($this->_restrictedUserId)) {
                    $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'You don\'t have permission to merge organizations'));
                }
                $mergeOrganizationToId = $data['mergeRecordWithId'];
                $leadOrganizationModel = $leadOrganizationMapper->find($mergeOrganizationToId);
                if (!$leadOrganizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                    $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Organization can\'t be merged'));
                }

                $mergeResult = Tools_LeadMergeTools::mergeOrganization($data['id'], $mergeOrganizationToId, true);
                if ($mergeResult['error'] === 1){
                    $this->_error($mergeResult['message']);
                }
                $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Organization data has been merged'));

            } else {
                $oldOrganizationName = $leadOrganizationModel->getOrganizationName();
                $currentOrganizationName = $data['organization_name'];

                if (!empty($data['organization_mobile'])) {
                    $data['organization_mobile'] = Tools_LeadTools::cleanPhoneAddress($data['organization_mobile']);
                }

                if (!empty($data['organization_phone'])) {
                    $data['organization_phone'] = Tools_LeadTools::cleanPhoneAddress($data['organization_phone']);
                }

                if (empty($data['customer_group_id'])) {
                    $data['customer_group_id'] = null;
                }

                if (empty($data['organization_industry_id'])) {
                    $data['organization_industry_id'] = null;
                }

                if (!empty($data['organization_mobile_country_code'])) {
                    $mobileCountryPhoneCode = Zend_Locale::getTranslation($data['organization_mobile_country_code'],
                        'phoneToTerritory');
                    $data['organization_mobile_country_code_value'] = '+' . $mobileCountryPhoneCode;
                }

                if (!empty($data['organization_phone_country_code'])) {
                    $phoneCountryPhoneCode = Zend_Locale::getTranslation($data['organization_phone_country_code'],
                        'phoneToTerritory');
                    $data['organization_phone_country_code_value'] = '+' . $phoneCountryPhoneCode;
                }

                $clearStateCode = false;

                if (empty($data['organization_country_code'])) {
                    $data['organization_country_code'] = null;
                } else {
                    $stateCodes = Tools_LeadTools::getStates($data['organization_country_code'], true, true);

                    if(!empty($stateCodes) && empty($data['organization_state_code'])) {
                        $stateCodesKeys = array_keys($stateCodes);
                        $data['organization_state_code'] = $stateCodesKeys[0];
                    } elseif(empty($stateCodes)) {
                        $clearStateCode = true;
                    }
                }

                if (empty($data['organization_state_code'])) {
                    $data['organization_state_code'] = null;
                } elseif ($clearStateCode) {
                    $data['organization_state_code'] = null;
                } elseif (!empty($stateCodes) && !empty($data['organization_state_code']) && empty($stateCodes[$data['organization_state_code']])) {
                    $stateCodesKeys = array_keys($stateCodes);
                    $data['organization_state_code'] = $stateCodesKeys[0];
                }

                if (empty($data['primary_contact_id'])) {
                    $data['primary_contact_id'] = null;
                }

                if (!empty($data['organization_website'])) {
                    $data['organization_website'] = Tools_LeadTools::prepareWebsiteUrl($data['organization_website']);
                    $domain = Tools_LeadTools::getDomainFromUrl($data['organization_website']);
                    if (!empty($domain)) {
                        $data['organization_domain'] = $domain;
                    } else {
                        $data['organization_domain'] = '';
                    }
                } else {
                    $data['organization_domain'] = '';
                }

                if (!empty($data['organization_website1'])) {
                    $data['organization_website1'] = Tools_LeadTools::prepareWebsiteUrl($data['organization_website1']);
                    $domain = Tools_LeadTools::getDomainFromUrl($data['organization_website1']);
                    if (!empty($domain)) {
                        $data['organization_domain1'] = $domain;
                    } else {
                        $data['organization_domain1'] = '';
                    }
                } else {
                    $data['organization_domain1'] = '';
                }

                $leadOrganizationModel->setOptions($data);

                if ($oldOrganizationName !== $currentOrganizationName) {
                    $validateOrganizationNameExists = new Zend_Validate_Db_RecordExists(array(
                        'table' => 'plugin_lead_organization',
                        'field' => 'organization_name'
                    ));
                    if ($validateOrganizationNameExists->isValid($currentOrganizationName)) {
                        $leadOrganization = $leadOrganizationMapper->findByName($currentOrganizationName);
                        if ($leadOrganization instanceof Leads_Model_LeadsOrganizationModel) {
                            $this->_responseHelper->fail(array(
                                'duplicatedOrganizationName' => '1',
                                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'You have another').' <u><a target="_blank" href="'.Zend_Controller_Action_HelperBroker::getStaticHelper('website')->getUrl().'dashboard/organizations/#organization/'.$leadOrganization->getId().'">'.'organization</a></u> with such name.',
                                'mergeToId' => $leadOrganization->getId()
                            ));
                        }
                    }
                }
                $updatedAt = Tools_System_Tools::convertDateFromTimezone('now');
                $leadOrganizationModel->setUpdatedAt($updatedAt);

                $organizationModel = $leadOrganizationMapper->save($leadOrganizationModel);
                $this->_responseHelper->success(array(
                    'organizationModel' => $organizationModel->toArray(),
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Organization data has been updated')
                ));
            }
        }


    }

    /**
     * Delete lead organization
     *
     * Resource:
     * : /api/leads/leadorganization/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     * : lead organization id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        if (!$id) {
            $this->_error();
        }
        $leadsOrganizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
        $leadOrganizationModel = $leadsOrganizationMapper->find($id);
        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
        if ($leadOrganizationModel instanceof Leads_Model_LeadsOrganizationModel) {
            $leadsOrganizationDocumentsMapper = Leads_Mapper_LeadsOrganizationDocumentsMapper::getInstance();

            $existedDocuments = $leadsOrganizationDocumentsMapper->findDocumentsByOrganizationId($leadOrganizationModel->getId());

            if(!empty($existedDocuments)) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websitePath = $websiteHelper->getPath();
                $organizationLogosPath = $websitePath . Leads::ORGANIZATION_LOGOS_IMAGES_PATH;

                foreach ($existedDocuments as $document) {
                    $filePath = $organizationLogosPath . DIRECTORY_SEPARATOR . $document['fileStoredName'];

                    if(is_file($filePath)) {
                        unlink($filePath);
                    }
                }
            }

            $leadsMapper->removeOrganizationId($leadOrganizationModel->getId());
            $leadsOrganizationMapper->delete($id);
            $lastViewedLeadsAndOrgMapper = Leads_Mapper_LeadsLastViewedMapper::getInstance();
            $lastViewedLeadsAndOrgMapper->delete($id, Leads_Model_LeadsLastViewedModel::LEAD_LAST_VIEWED_TYPE_ORGANIZATION);
        } else {
            $this->_error();
        }

    }

}
