<?php

class Api_Leads_Leadprofile extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Get lead profile by
     *
     * Resource:
     * : /api/leads/leadprofile/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order'), FILTER_SANITIZE_STRING);
        $map = filter_var($this->_request->getParam('map'), FILTER_SANITIZE_STRING);
        $leadId = filter_var($this->_request->getParam('leadId'), FILTER_SANITIZE_NUMBER_INT);
        $positionInGridLeadId = filter_var($this->_request->getParam('positionInGridLeadId'), FILTER_SANITIZE_NUMBER_INT);
        $calculatePositionInGrid = filter_var($this->_request->getParam('calculatePositionInGrid'), FILTER_SANITIZE_STRING);
        $visitLogId = filter_var($this->_request->getParam('visitLogId'), FILTER_SANITIZE_STRING);
        $leadsProfileMapper = Leads_Mapper_LeadsIpUserProfileLogMapper::getInstance();
        $leadsProfileVisitMapper = Leads_Mapper_LeadsIpUserProfileLogActionDetailsMapper::getInstance();
        $leadIpsMapper = Leads_Mapper_LeadsIpsMapper::getInstance();
        if ($leadId) {
            $where = $leadsProfileMapper->getDbTable()->getAdapter()->quoteInto('pliupl.lead_id = ?', $leadId);
            $filterParams = filter_var_array($this->_request->getParam('filter', array()), FILTER_SANITIZE_STRING);
            if (!empty($filterParams)) {

            }
            if (empty($sortOrder)) {
                $sortOrder = 'pliupl.first_action_timestamp';
            }
            if (!empty($map)) {
                $group = array('latitude', 'longitude');
                $mapsData = $leadsProfileMapper->fetchAllForVisitsMap($where, $sortOrder, $limit, $offset, $group);
            } else {
                $data = $leadsProfileMapper->fetchAllForGrid($where, $sortOrder, $limit, $offset);
            }
        }

        if (!empty($calculatePositionInGrid) && !empty($visitLogId) && !empty($positionInGridLeadId)) {
            $totalRecordsData = $leadsProfileMapper->getTotalVisitsForLead($positionInGridLeadId, 'pliupl.first_action_timestamp');
            if (!empty($totalRecordsData['totalRecords'])) {
                $visitPosition = array_search($visitLogId, $totalRecordsData['data']);
                $positionPageInGrid = ceil($visitPosition/10);
                if ($positionPageInGrid > 0) {
                    $positionPageInGrid = $positionPageInGrid - 1;
                }
                $data['visitPageNumber'] = $positionPageInGrid;
            }
        }

        if (empty($data)) {
            $data = array();
        }

        if (!empty($data) && empty($map) && !empty($leadId)) {
            $data['stats'] = $leadsProfileMapper->getStatsData($leadId);
            $data['firstVisitInfo'] = $leadsProfileMapper->getStatsFirstDateInfo($leadId);
            $data['lastVisitInfo'] = $leadsProfileMapper->getStatsLastDateInfo($leadId);
            $data['currentDate'] = Tools_System_Tools::convertDateFromTimezone('now');
            $data['pageLoadAverage'] = $leadsProfileVisitMapper->getPageAverageLoad($leadId);
        }

        if (!empty($mapsData) && !empty($map) && !empty($leadId)) {
            $data['data'] = $mapsData;
        }

        if (!empty($data)) {
            $data['countryCodes'] = Tools_Geo::getCountries(true);
            $data['googleApiKey'] = Zend_Controller_Action_HelperBroker::getStaticHelper('config')->getConfig('crmGoogleMapKey');
            $data['googleMapDefaultZoom'] = Zend_Controller_Action_HelperBroker::getStaticHelper('config')->getConfig('crmGoogleMapDefaultZoom');
            $data['ipAddressesData'] = $leadIpsMapper->getIpsByLeadId($leadId);
        }


        return $data;
    }

    /**
     *
     * Resource:
     * : /api/leads/leadprofile/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {

    }

    /**
     * Resource:
     * : /api/leads/leadprofile/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     *
     * @return JSON
     */
    public function putAction()
    {

    }

    /**
     *
     * Resource:
     * : /api/leads/leadprofile/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     *
     * @return JSON
     */
    public function deleteAction()
    {

    }

}
