<?php

class Api_Leads_Leadquote extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('opportunity_id', 'lead_id', 'quote_title');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Resource:
     * : /api/leads/leadquote/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

    }


    public function postAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);

        if (empty($data)) {
            $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING);
        }
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            if (empty($data['quote_title'])) {
                $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Please specify quote title'));
            }

            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'));
        }

        $templateMapper = Application_Model_Mappers_TemplateMapper::getInstance();
        $quoteTemplates = $templateMapper->findByType(Quote::QUOTE_TEPMPLATE_TYPE);
        if (empty($quoteTemplates)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Quote template is missing'));
        }


        $secureToken = filter_var($data[Tools_System_Tools::CSRF_SECURE_TOKEN], FILTER_SANITIZE_STRING);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
        }

        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
        $leadModel = $leadsMapper->find($data['lead_id']);
        if (!$leadModel instanceof Leads_Model_LeadsModel) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Lead doesn\'t exists'));
        }
        $leadEmail = $leadModel->getLeadEmail();
        if (empty($leadEmail)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Lead doesn\'t have email'));
        }

        $duplicateQuote = false;
        if (!empty($data['duplicate_quote_id'])) {
            $quoteMapper = Quote_Models_Mapper_QuoteMapper::getInstance();
            $duplicateQuoteModel = $quoteMapper->find($data['duplicate_quote_id']);
            if (!$duplicateQuoteModel instanceof Quote_Models_Model_Quote) {
                $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Quote for duplication not found'));
            }
            $duplicateQuote = true;
        }

        $useOrganizationAddress =  $leadModel->getUseOrganizationAddress();

        $dataForQuoteCreation = array();
        $dataForQuoteCreation['firstname'] = $leadModel->getLeadFirstName();
        $dataForQuoteCreation['lastname'] = $leadModel->getLeadLastName();
        $dataForQuoteCreation['email'] = $leadEmail;
        $dataForQuoteCreation['mobile'] = $leadModel->getLeadMobile();
        $dataForQuoteCreation['mobilecountrycode'] = $leadModel->getLeadMobileCountryCode();
        $dataForQuoteCreation['phonecountrycode'] = $leadModel->getLeadPhoneCountryCode();
        $dataForQuoteCreation['mobile_country_code_value'] = $leadModel->getLeadMobileCountryCodeValue();
        $dataForQuoteCreation['phone_country_code_value'] = $leadModel->getLeadPhoneCountryCodeValue();
        $dataForQuoteCreation['address1'] = $leadModel->getLeadAddress();
        $dataForQuoteCreation['address2'] = $leadModel->getLeadAddress1();
        $dataForQuoteCreation['city'] = $leadModel->getLeadCity();
        $dataForQuoteCreation['phone'] = $leadModel->getLeadPhone();
        $dataForQuoteCreation['zip'] = $leadModel->getLeadZip();
        $dataForQuoteCreation['country'] = $leadModel->getLeadCountryCode();
        $dataForQuoteCreation['state'] = $leadModel->getLeadStateCode();
        $dataForQuoteCreation['company'] = '';

        $leadPrefix = $leadModel->getPrefix();
        if(!empty($leadPrefix) && in_array($leadPrefix, Tools_LeadTools::$userPrefixes)) {
            $dataForQuoteCreation['prefix'] = $leadPrefix;
        }

        $dataForQuoteCreation['position'] = $leadModel->getLeadPosition();

        $leadOrganizationId = $leadModel->getOrganizationId();
        if (!empty($leadOrganizationId)) {
            $leadOrganization = Leads_Mapper_LeadsOrganizationMapper::getInstance()->find($leadOrganizationId);
            if ($leadOrganization instanceof Leads_Model_LeadsOrganizationModel) {
                $dataForQuoteCreation['company'] = $leadOrganization->getOrganizationName();
            }

            if (!empty($useOrganizationAddress) && $leadOrganization instanceof Leads_Model_LeadsOrganizationModel) {
                $dataForQuoteCreation['address1'] = $leadOrganization->getOrganizationAddress();
                $dataForQuoteCreation['address2'] =  $leadOrganization->getOrganizationAddress1();
                $dataForQuoteCreation['city'] = $leadOrganization->getOrganizationCity();
                $dataForQuoteCreation['zip'] =  $leadOrganization->getOrganizationZip();
                $dataForQuoteCreation['country'] = $leadOrganization->getOrganizationCountryCode();
                $dataForQuoteCreation['state'] = $leadOrganization->getOrganizationStateCode();
            }
        }

        if (!empty($dataForQuoteCreation['state'])) {
            $leadStateExists = Tools_LeadTools::findStateByStateAbbr($dataForQuoteCreation['state']);
            if (!empty($leadStateExists)) {
                $dataForQuoteCreation['state'] = $leadStateExists['id'];
            }
        }

        $cartMapper = Models_Mapper_CartSessionMapper::getInstance();
        if ($duplicateQuote === false) {
            $cartSessionModel = new Models_Model_CartSession();
        } else {
            $cartSessionModel = $cartMapper->find($duplicateQuoteModel->getCartId());
            if ($cartSessionModel instanceof Models_Model_CartSession){
                $cartSessionModel->setId(null);
            }
        }

        $cartSessionModel->setDiscountTaxRate(1);
        $cartSessionModel->setShippingPrice(0);
        $cartSessionModel->setStatus(Quote_Models_Model_Quote::STATUS_NEW);
        $cartSessionModel->setPartialPaidAmount('0');
        $cartSessionModel->setPurchasedOn(null);
        $cartSessionModel->setPartialPurchasedOn(null);
        $cartSessionModel->setIsFirstPaymentManuallyPaid('0');
        $cartSessionModel->setIsSecondPaymentManuallyPaid('0');
        $cartSessionModel->setIsFullOrderManuallyPaid('0');
        $cartSessionModel->setFirstPaymentGateway('');
        $cartSessionModel->setSecondPaymentGateway('');
        $cartSessionModel->setFirstPartialPaidAmount(0);
        $cartSessionModel->setSecondPartialPaidAmount(0);
        $cartSessionModel->setPurchaseErrorMessage('');
        $cartSessionModel->setPartialNotificationDate(null);

        $cart = $cartMapper->save($cartSessionModel);

        $customer = Quote_Tools_Tools::processCustomer($dataForQuoteCreation);

        if (!$cart) {
            $this->_error('Unable to create quote');
        }

        $cart = $cartMapper->save(
            $cart->setBillingAddressId(Quote_Tools_Tools::addAddress($dataForQuoteCreation,
                Models_Model_Customer::ADDRESS_TYPE_BILLING, $customer))
                ->setShippingAddressId(Quote_Tools_Tools::addAddress($dataForQuoteCreation,
                    Models_Model_Customer::ADDRESS_TYPE_SHIPPING, $customer))
                ->setUserId($customer->getId())
        );

        $creatorUser = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser();
        $creatorName = $creatorUser->getFullName();
        $creatorId = $creatorUser->getId();

        $quoteTitle = '';
        if (!empty($data['quote_title'])) {
            $quoteTitle = $data['quote_title'];
        }

        $pageMapper = Application_Model_Mappers_PageMapper::getInstance();

        if ($duplicateQuote === true) {
            $duplicateQuotePage = $pageMapper->findByUrl($duplicateQuoteModel->getId() . '.html');
            $oldPageId = 0;
            if ($duplicateQuotePage instanceof Application_Model_Models_Page) {
                $oldPageId = $duplicateQuotePage->getId();
            }

            $quoteModel = Quote_Tools_Tools::createQuote($cart,
                array(
                    'editedBy' => $creatorName,
                    'creatorId' => $creatorId,
                    'disclaimer' => '',
                    'quoteTitle' => $quoteTitle,
                    'oldPageId' => $oldPageId,
                    'oldQuoteId' => $duplicateQuoteModel->getId(),
                    'actionType' => Quote::QUOTE_TYPE_CLONE
                ));
        } else {

            $quoteParams = array(
                'editedBy' => $creatorName,
                'creatorId' => $creatorId,
                'disclaimer' => '',
                'quoteTitle' => $quoteTitle
            );

            if (!empty($data['quote_template_name'])) {
                $quoteParams['templateName'] = $data['quote_template_name'];
            }

            $quoteModel = Quote_Tools_Tools::createQuote($cart,
                $quoteParams);
        }

        if ($quoteModel instanceof Quote_Models_Model_Quote) {
            if ($duplicateQuote === true) {
                $quotePage = $pageMapper->findByUrl($quoteModel->getId() . '.html');
                $pageModelWithContainers = $pageMapper->findByUrl($duplicateQuoteModel->getId() . '.html');
                if ($pageModelWithContainers instanceof Application_Model_Models_Page && $quotePage instanceof Application_Model_Models_Page) {
                    $containers = $pageModelWithContainers->getContainers();
                    if (!empty($containers)) {
                        $containerMapper = Application_Model_Mappers_ContainerMapper::getInstance();
                        foreach ($containers as $container) {
                            if (!empty($container['page_id'])) {
                                $containerModel = new Application_Model_Models_Container();
                                $containerModel->setName($container['name']);
                                $containerModel->setContainerType($container['container_type']);
                                $containerModel->setContent($container['content']);
                                $containerModel->setPublished($container['published']);
                                $containerModel->setPublishingDate($container['published_date']);
                                $containerModel->setPageId($quotePage->getId());
                                $containerMapper->save($containerModel);
                            }
                        }
                    }
                }

                $quoteDraggableMapper = Quote_Models_Mapper_QuoteDraggableMapper::getInstance();
                $quoteDraggableModelToClone = $quoteDraggableMapper->findByQuoteId($duplicateQuoteModel->getId());
                if ($quoteDraggableModelToClone instanceof Quote_Models_Model_QuoteDraggableModel) {
                    $quoteDraggableModel = new Quote_Models_Model_QuoteDraggableModel();
                    $quoteDraggableModel->setData($quoteDraggableModelToClone->getData());
                    $quoteDraggableModel->setQuoteId($quoteModel->getId());
                    $quoteDraggableMapper->save($quoteDraggableModel);
                }

            }

            $leadUserId = $leadModel->getUserId();
            if (empty($leadUserId)) {
                $leadModel->setUserId($quoteModel->getUserId());
                $leadsMapper->save($leadModel);
            }

            $quoteId = $quoteModel->getId();
            $createdAt = Tools_System_Tools::convertDateFromTimezone('now');
            $leadsOpportunityQuoteMapper = Leads_Mapper_LeadsOpportunityQuoteMapper::getInstance();
            $leadOpportunityQuoteModel = new Leads_Model_LeadsOpportunityQuoteModel();
            $leadOpportunityQuoteModel->setOpportunityId($data['opportunity_id']);
            $leadOpportunityQuoteModel->setQuoteId($quoteId);
            $leadOpportunityQuoteModel->setLeadId($data['lead_id']);
            $leadOpportunityQuoteModel->setCartId($cart->getId());
            $leadOpportunityQuoteModel->setCreatedAt($createdAt);
            $leadsOpportunityQuoteMapper->save($leadOpportunityQuoteModel);
            $this->_responseHelper->success(array('message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Quote has been created.').
                    ' <a href="'.Zend_Controller_Action_HelperBroker::getStaticHelper('website')->getUrl().$quoteId.'.html" target="_blank">'. $translator->translate(Leads::TRANSLATION_PREFIX.'Click to see newly created quote.') .' '.$quoteId.'.html</a>', 'quoteLink' => $quoteId.'.html'));
        }

        $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Error during quote creation'));

    }


    /**
     * Resource:
     * : /api/leads/leadquote/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     * : lead opportunity id to update
     *
     * @return JSON
     */
    public function putAction()
    {

    }

    /**
     * Delete lead quote
     *
     * Resource:
     * : /api/leads/leadquote/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     *
     * @return JSON
     */
    public function deleteAction()
    {

    }

}
