<?php

class Api_Leads_Leads extends Api_Service_Abstract
{

    /**
     * Filter by existing client
     */
    const FILTER_EXISTING_CLIENT = 'existing_client';

    /**
     * Filter for leads without completed orders
     */
    const FILTER_PROSPECT = 'prospect';

    /**
     * Filter for leads without emails
     */
    const FILTER_NO_EMAIL = 'no_email';

    /**
     * comparison operators
     */
    const RULE_COMPARISON_OPERATOR_EQUAL = 'equal';

    const RULE_COMPARISON_OPERATOR_NOTEQUAL = 'notequal';

    const RULE_COMPARISON_OPERATOR_LIKE = 'like';

    const RULE_COMPARISON_OPERATOR_IN = 'in';

    const RULE_COMPARISON_GREATER_THAN = 'greaterthan';

    const RULE_COMPARISON_LESS_THAN = 'lessthan';

    const RULE_COMPARISON_NOT_IN = 'notin';

    /**
     * Sorting constants
     */
    const SORT_ORDER_CREATED_AT = 'created_at';

    const SORT_ORDER_UPDATED_AT = 'updated_at';

    const SORT_ORDER_LAST_ACTIVITY_INBOUND = 'last_activity';

    const SORT_ORDER_LAST_ACTIVITY_OUTBOUND = 'last_activity_outbound';

    const SORT_ORDER_LEAD_NAME = 'lead_first_name';

    /**
     * Sorting tasks constants
     */
    const FILTER_TASK_OPENED = 'open';

    const FILTER_TASK_CLOSED = 'closed';

    const FILTER_TASK_BOTH = 'both';

    const FILTER_TASK_NONE = 'none';

    const FILTER_TASK_OVERDUE = 'overdue';

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * Allowed sorting types
     *
     * @var array
     */
    public static $_sortingTypes = array(
        self::SORT_ORDER_CREATED_AT,
        self::SORT_ORDER_UPDATED_AT,
        self::SORT_ORDER_LAST_ACTIVITY_INBOUND,
        self::SORT_ORDER_LAST_ACTIVITY_OUTBOUND,
        self::SORT_ORDER_LEAD_NAME
    );

    /**
     * tasks filter
     *
     * @var array
     */
    public static $_tasksFilter = array(
        'open' => 'Open',
        'closed' => 'Closed',
        'both' => 'Both',
        'overdue' => 'Overdue',
        'none' => 'None'
    );


    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    /**
     * User restricted id access
     *
     * @var null
     */
    protected $_restrictedUserId = null;

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
    }


    /**
     * Get lead by id
     *
     * Resource:
     * : /api/leads/leads/
     *
     * HttpMethod:
     * : GET
     * @return array
     * @throws Zend_Exception
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order'), FILTER_SANITIZE_STRING);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $forGrid = (bool)filter_var($this->_request->getParam('isGrid', false), FILTER_SANITIZE_STRING);
        $searchTerm = filter_var($this->_request->getParam('searchTerm'), FILTER_SANITIZE_STRING);
        $email = filter_var($this->_request->getParam('email'), FILTER_SANITIZE_STRING);
        $mobile = filter_var($this->_request->getParam('mobile'), FILTER_SANITIZE_NUMBER_INT);
        $mobileCountryCode = filter_var($this->_request->getParam('mobileCountryCode'), FILTER_SANITIZE_STRING);
        $leadFirstName = filter_var($this->_request->getParam('leadFirstName'), FILTER_SANITIZE_STRING);
        $leadLastName = filter_var($this->_request->getParam('leadLastName'), FILTER_SANITIZE_STRING);
        $leadZip = filter_var($this->_request->getParam('leadZip'), FILTER_SANITIZE_NUMBER_INT);
        $detailedInfo = filter_var($this->_request->getParam('detailedInfo'), FILTER_SANITIZE_STRING);
        $leadDataMapper = Leads_Mapper_LeadsMapper::getInstance();
        $withOpportunityAmount = false;
        $withSpentPurchaseAmount = false;
        $withNextActivityDate = false;
        $includePluginLeadsTypeCpecificColl = false;
        $withTasks = false;
        $having = null;
        $currency = Zend_Registry::get('Zend_Currency');
        $callDuration = false;
        $includesNotes = false;
        $includesCustomLeadFields = false;
        $includesCustomOrganizationFields = false;
        $withOpportunityExpected = false;
        $withEmailLog = false;
        $withOrganizationIndustry = false;
        $additionalParamsForSelect = array();
        $additionalOrganizationParamsForSelect = array();
        $withEmailSmsSequence = false;

        $customPaginate = filter_var($this->_request->getParam('customPaginate'), FILTER_SANITIZE_NUMBER_INT);

        if(!empty($customPaginate)) {
            $limit = $customPaginate;
        }

        $sortOrder = 'pld.created_at DESC';
        $orderInteractionParam = '';
        if ($id && $forGrid) {
            $where = $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.id = ?', $id);
            if (!empty($this->_restrictedUserId)) {
                $where .= ' AND '.  $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $this->_restrictedUserId);
            }
            $data = $leadDataMapper->fetchAll($where, null, null, null, true, true);
        } elseif ($id) {
            if (!empty($this->_restrictedUserId)) {
                $data = $leadDataMapper->findByIdOwnerId($id, $this->_restrictedUserId);
            } else {
                $data = $leadDataMapper->find($id);
            }
        } else {
            $filterParams = $this->_request->getParam('filter', array());

            foreach ($filterParams as $key => $param) {
                if (!is_array($param)) {
                    $filterParams[$key] = Tools_LeadTools::validateTextField($param);
                }
            }
            $where = null;
            if (!empty($this->_restrictedUserId)) {
                $filterParams['owner_id'] = $this->_restrictedUserId;
            }

            if (!empty($filterParams)) {
                $filterParams['lead-attribute'] = str_replace('*-amp-*', '&', $filterParams['lead-attribute']);
                $where = '';
                if (!empty($filterParams['lead-attribute']) && $filterParams['filter-by'] === 'attributes') {
                    $filterParams['lead-attribute'] = trim($filterParams['lead-attribute'], " \t\n\r\x0B");
                    if (strpos($filterParams['lead-attribute'], '*') !== false) {
                        $attributeValue = str_replace('*', '%', $filterParams['lead-attribute']);
                        $where .= ' (' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_first_name LIKE ?',
                                $attributeValue . '%');
                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_last_name LIKE ?',
                                $attributeValue . '%');
                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_email LIKE ?',
                                $attributeValue . '%');
                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_secondary_email LIKE ?',
                                $attributeValue . '%');
                    } else {
                        $attributeValues = explode(' ', $filterParams['lead-attribute']);
                        $where .= ' ('.'(';
                        foreach ($attributeValues as $key => $attrVal) {
                            $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_first_name LIKE ?',
                                '%'. $attrVal . '%');

                            if (count($attributeValues) > $key+1) {
                                $where .= ' AND ';
                            }
                        }

                        $where .= ') OR ( ';

                        foreach ($attributeValues as $key => $attrVal) {
                            $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_last_name LIKE ?',
                                '%'. $attrVal . '%');

                            if (count($attributeValues) > $key+1) {
                                $where .= ' AND ';
                            }
                        }

                        $where .= ') OR ( ';
                        foreach ($attributeValues as $key => $attrVal) {
                            $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_email LIKE ?',
                                '%'. $attrVal . '%');

                            if (count($attributeValues) > $key+1) {
                                $where .= ' AND ';
                            }
                        }

                        $where .= ') OR ( ';
                        foreach ($attributeValues as $key => $attrVal) {
                            $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_secondary_email LIKE ?',
                                '%'. $attrVal . '%');

                            if (count($attributeValues) > $key+1) {
                                $where .= ' AND ';
                            }
                        }

                        $where .= ')';

                    }

                    if (!empty($filterParams['search_includes_notes'])) {
                        $includesNotes = true;
                        if (strpos($filterParams['lead-attribute'], '*') !== false) {
                            $attributeValue = str_replace('*', '%', $filterParams['lead-attribute']);
                            $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_notes LIKE ?',
                                    $attributeValue . '%');
                            $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pln.note LIKE ?',
                                    $attributeValue . '%');
                        } else {
                            $attributeValues = explode(' ', $filterParams['lead-attribute']);
                            $where .= ' OR ( ';
                            foreach ($attributeValues as $key => $attrVal) {
                                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_notes LIKE ?',
                                        '%' . $attrVal . '%');

                                if (count($attributeValues) > $key+1) {
                                    $where .= ' AND ';
                                }
                            }

                            $where .= ') OR ( ';

                            foreach ($attributeValues as $key => $attrVal) {
                                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pln.note LIKE ?',
                                        '%' . $attrVal . '%');

                                if (count($attributeValues) > $key+1) {
                                    $where .= ' AND ';
                                }
                            }
                            $where .= ')';
                        }
                    }

                    if (!empty($filterParams['search_includes_custom_lead_fields'])) {
                        $includesCustomLeadFields = true;
                        if (strpos($filterParams['lead-attribute'], '*') !== false) {
                            $attributeValue = str_replace('*', '%', $filterParams['lead-attribute']);
                            $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pllcpd.param_value LIKE ?',
                                    $attributeValue . '%');
                        } else {
                            $attributeValues = explode(' ', $filterParams['lead-attribute']);

                            $where .= ' OR ( ';
                            foreach ($attributeValues as $key => $attrVal) {
                                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pllcpd.param_value LIKE ?',
                                    '%' . $attrVal. '%');
                                if (count($attributeValues) > $key + 1) {
                                    $where .= ' AND ';
                                }
                            }
                            $where .= ')';
                        }
                    }

                    $leadPhone = Tools_LeadTools::cleanPhoneAddress($filterParams['lead-attribute']);
                    $leadPhoneIsNumber = mb_substr(preg_replace('~[^0-9A-Za-z]~ui', '', $filterParams['lead-attribute']), 0, 1);
                    if (strpos($filterParams['lead-attribute'], '*') !== false) {
                        $attributeValue = str_replace('*', '%', $filterParams['lead-attribute']);

                        if(!empty($leadPhone) && is_numeric($leadPhoneIsNumber)) {
                            $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_phone LIKE ?',
                                    $leadPhone . '%');
                            $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_mobile LIKE ?',
                                    $leadPhone . '%');
                        }

                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr('CONCAT(pld.lead_first_name, " ", pld.lead_last_name) LIKE ?'),
                                $attributeValue . '%');
                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr('CONCAT(pld.lead_first_name, pld.lead_last_name) LIKE ?'),
                                $attributeValue . '%') . ') ';
                    } else {
                        if(!empty($leadPhone) && is_numeric($leadPhoneIsNumber)) {
                            $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_phone LIKE ?',
                                    $leadPhone . '%');
                            $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_mobile LIKE ?',
                                    $leadPhone . '%');
                            $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr('CONCAT(pld.lead_mobile_country_code_value, pld.lead_mobile) LIKE ?'),
                                    '%'. $leadPhone . '%');
                            $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr('CONCAT(pld.lead_phone_country_code_value, pld.lead_phone) LIKE ?'),
                                     '%'. $leadPhone . '%');
                        }

                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr('CONCAT(pld.lead_first_name, " ", pld.lead_last_name) LIKE ?'),
                                $filterParams['lead-attribute'] . '%');
                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr('CONCAT(pld.lead_first_name, pld.lead_last_name) LIKE ?'),
                                $filterParams['lead-attribute'] . '%') . ') ';
                    }
                }
                if (!empty($filterParams['lead-attribute']) && $filterParams['filter-by'] === 'organization') {
                    $filterParams['lead-attribute'] = trim($filterParams['lead-attribute'], " \t\n\r\x0B");
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= ' (' .$leadDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_name LIKE ?',
                        $filterParams['lead-attribute'] . '%');

                    if (!empty($filterParams['search_includes_notes'])) {
                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_notes LIKE ?',
                                '%'. $filterParams['lead-attribute']. '%');
                    }

                    if (!empty($filterParams['search_includes_custom_lead_fields'])) {
                        $includesCustomOrganizationFields = true;
                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plocpd.param_value LIKE ?',
                                '%'. $filterParams['lead-attribute'] . '%');
                    }

                    $where .= ') ';
                }

                if (!empty($filterParams['lead_titles']) && !empty($filterParams['lead_title_operator'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $operator = $filterParams['lead_title_operator'];

                    if ($operator === self::RULE_COMPARISON_OPERATOR_EQUAL) {
                        $paramOperator = '=';
                    }
                    if ($operator === self::RULE_COMPARISON_OPERATOR_NOTEQUAL) {
                        $paramOperator = '<>';
                    }

                    if ($operator === self::RULE_COMPARISON_OPERATOR_LIKE) {
                        $paramOperator = 'LIKE';
                        $filterParams['lead_titles'] = $filterParams['lead_titles'].'%';
                    }

                    if (!empty($paramOperator)) {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_position ' . $paramOperator . ' ?',
                            $filterParams['lead_titles']);
                    }
                }

                if (isset($filterParams['nps_score']) && is_numeric($filterParams['nps_score']) && !empty($filterParams['lead_filter_nps_score_operator'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $operator = $filterParams['lead_filter_nps_score_operator'];

                    if ($operator === self::RULE_COMPARISON_OPERATOR_EQUAL) {
                        $paramOperator = '=';
                    }
                    if ($operator === self::RULE_COMPARISON_GREATER_THAN) {
                        $paramOperator = '>=';
                    }

                    if ($operator === self::RULE_COMPARISON_LESS_THAN) {
                        $paramOperator = '<=';
                    }

                    if (!empty($paramOperator)) {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plrr.score ' . $paramOperator . ' ?',
                            $filterParams['nps_score']);
                        $where .= ' AND '. $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plrr.status = ?',
                            'reviewed');
                    }
                }

                if (!empty($filterParams['lead_zip'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $leadZips = explode(',', $filterParams['lead_zip']);
                    $leadZips = array_diff($leadZips, array(''));
                    $leadZips = array_map('trim', $leadZips);

                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_zip IN (?)', $leadZips);
                }

                if (!empty($filterParams['lead_score_from'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.score >= ?',
                        (int) $filterParams['lead_score_from']);
                }

                if (!empty($filterParams['lead_score_to'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.score <= ?',
                        (int) $filterParams['lead_score_to']);
                }

                if (!empty($filterParams['lead_city'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_city = ?',
                        $filterParams['lead_city']);
                }

                if (!empty($filterParams['lead_country'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_country_code = ?',
                        $filterParams['lead_country']);
                }

                if (!empty($filterParams['lead_country_state']) && is_array($filterParams['lead_country_state'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_state_code IN (?)',
                        $filterParams['lead_country_state']);
                }

                if (!empty($filterParams['created_at_from'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $createdAt =  Tools_System_Tools::convertDateFromTimezone($filterParams['created_at_from']);
                    if (!empty($filterParams['last_updated']) && $filterParams['last_updated'] === 'last_updated') {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.updated_at >= ?',
                            $createdAt);
                    } else {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.created_at >= ?',
                            $createdAt);
                    }
                }
                if (!empty($filterParams['created_at_to'])) {
                    $updatedAt =  Tools_System_Tools::convertDateFromTimezone($filterParams['created_at_to'].' 23:59:59');
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    if (!empty($filterParams['last_updated']) && $filterParams['last_updated'] === 'last_updated') {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.updated_at <= ?',
                            $updatedAt);
                    } else {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.created_at <= ?',
                            $updatedAt);
                    }
                }

                if (!empty($filterParams['customer_group_id'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.customer_group_id = ?',
                        $filterParams['customer_group_id']);
                }

                if (!empty($filterParams['email_validation_status'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.email_validated_status = ?',
                        $filterParams['email_validation_status']);
                    $where .= ' AND '.$leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_email <> ?',
                        '');
                }

                if (!empty($filterParams['stage_id'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.stage_id IN (?)',
                        $filterParams['stage_id']);
                }
                if (!empty($filterParams['lead_source_type'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_type_id = ?',
                        $filterParams['lead_source_type']);

                    $leadTypeModel = Leads_Mapper_LeadsTypeMapper::getInstance()->find($filterParams['lead_source_type']);
                    $callDurationParam = false;
                    if($leadTypeModel instanceof Leads_Model_LeadsTypeModel) {
                        if($leadTypeModel->getTypeName() == Leads_Model_LeadsTypeModel::LEAD_TYPE_INBOUND_CALL || $leadTypeModel->getTypeName() == Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_CALL) {
                            $callDurationParam = true;
                        }
                    }

                    if (!empty($filterParams['call_duration']) && !empty($callDurationParam)) {
                        $callDuration = true;
                        $filterParams['call_duration'] = (int)$filterParams['call_duration'];

                        $where .= ' AND ';

                        if(!empty($filterParams['more_less_call_duration']) && $filterParams['more_less_call_duration'] == 'less') {
                            $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plcl.call_duration <= ?',
                                $filterParams['call_duration']);
                        } else {
                            $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plcl.call_duration >= ?',
                                $filterParams['call_duration']);
                        }
                    }

                }
                if (!empty($filterParams['lead_source'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_source LIKE ?',
                        '%'. $filterParams['lead_source']. '%');
                }
                if (!empty($filterParams['owner_id'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?',
                        $filterParams['owner_id']);
                }
                if (!empty($filterParams['opportunity_owner_id'])) {
                    $withOpportunityExpected = true;
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plop.creator_id = ?',
                        $filterParams['opportunity_owner_id']);
                }

                if (!empty($filterParams['special_filter_type'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    if ($filterParams['special_filter_type'] === self::FILTER_EXISTING_CLIENT) {
                        $where .= new Zend_Db_Expr('pld.user_id IS NOT NULL');
                    }
                    if ($filterParams['special_filter_type'] === self::FILTER_PROSPECT) {
                        $where .= new Zend_Db_Expr('pld.user_id IS NULL');
                    }
                    if ($filterParams['special_filter_type'] === self::FILTER_NO_EMAIL) {
                        $where .= '('. new Zend_Db_Expr('pld.lead_email IS NULL');
                        $where .= ' OR '.$leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_email = ?', '').')';
                    }
                }

                if (!empty($filterParams['activity_id']) && is_array($filterParams['activity_id'])) {
                    $withNextActivityDate = true;
                    $includePluginLeadsTypeCpecificColl = true;
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plt.id IN (?)', $filterParams['activity_id']);
                }

                if (!empty($filterParams['next_activity_from'])) {
                    $withNextActivityDate = true;
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $currentDate = Tools_System_Tools::convertDateFromTimezone('now');
                    $activityFromDate = Tools_System_Tools::convertDateFromTimezone($filterParams['next_activity_from']);

                    if($activityFromDate > $currentDate) {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plal.event_type = ?',
                            Leads_Model_LeadsActionLogModel::EVENT_LOG_UPCOMING);
                        $where .= ' AND ' .$leadDataMapper->getDbTable()->getAdapter()->quoteInto('plal.created_at > ?', $currentDate);

                    } else {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plal.created_at > ?', $activityFromDate);
                    }

                    $having = $leadDataMapper->getDbTable()->getAdapter()->quoteInto('MIN(plal.created_at) > ?', $activityFromDate);
                }

                if(!empty($filterParams['next_activity_to'])) {
                    $withNextActivityDate = true;
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $activityToDate = Tools_System_Tools::convertDateFromTimezone($filterParams['next_activity_to'].' 23:59:59');
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plal.created_at < ?', $activityToDate);
                }

                if (!empty($filterParams['opportunity_date_from']) || !empty($filterParams['opportunity_date_to'])) {
                    $withOpportunityExpected = true;
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $opportunityFromDate = Tools_System_Tools::convertDateFromTimezone($filterParams['opportunity_date_from']);
                    if(!empty($filterParams['expected_at'])) {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plop.expected_close_date >= ?', $opportunityFromDate);
                    } elseif (!empty($filterParams['creation_at'])) {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plop.created_at >= ?', $opportunityFromDate);
                    } elseif (!empty($filterParams['state_change_at'])) {
                        $createdAtComparisonField = new Zend_Db_Expr('COALESCE(plop.change_stage_date, plop.created_at)');
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto($createdAtComparisonField.' >= ?', $opportunityFromDate);
                    }

                    if (!empty($filterParams['opportunity_date_to'])) {
                        if (!empty($where)) {
                            $where .= ' AND ';
                        }

                        $opportunityToDate = Tools_System_Tools::convertDateFromTimezone($filterParams['opportunity_date_to'] .' 23:59:59');
                        if(!empty($filterParams['expected_at'])) {
                            $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plop.expected_close_date <= ?', $opportunityToDate);
                        } elseif (!empty($filterParams['creation_at'])) {
                            $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plop.created_at <= ?', $opportunityToDate);
                        } elseif (!empty($filterParams['state_change_at'])) {
                            $dateToComparisonField = new Zend_Db_Expr('COALESCE(plop.change_stage_date, plop.created_at)');
                            $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto($dateToComparisonField.' <= ?', $opportunityToDate);
                        }
                    }
                }


                if (!empty($filterParams['lead_tags']) && is_array($filterParams['lead_tags'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plht.tag_id IN (?)', $filterParams['lead_tags']);
                }

                if (!empty($filterParams['email_sms_sequences']) && is_array($filterParams['email_sms_sequences'])) {
                    $withEmailSmsSequence = true;
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plhes.sequence_id IN (?)', $filterParams['email_sms_sequences']);

                    if (!empty($filterParams['email_sequence_type']) && ($filterParams['email_sequence_type'] === 'sms' ||  $filterParams['email_sequence_type'] === 'email') &&
                        !empty($filterParams['email_sequence_type_action']) && ($filterParams['email_sequence_type_action'] === 'sent' || $filterParams['email_sequence_type_action'] === 'opened' || $filterParams['email_sequence_type_action'] === 'all')) {
                        $withNextActivityDate = true;
                        $emailSequenceTypeAction = $filterParams['email_sequence_type_action'];

                        if (!empty((int)$filterParams['email_sms_sequences_type_period'])) {
                            $periodDays = (int)$filterParams['email_sms_sequences_type_period'];
                            $periodDaysEnd = Tools_System_Tools::convertDateFromTimezone('now');
                            $periodDaysStart = Tools_System_Tools::convertDateFromTimezone('-'.$periodDays.' days');
                        }

                        if ($filterParams['email_sequence_type'] === 'sms') {
                            if (!empty($where)) {
                                $where .= ' AND ';
                            }

                            if ($emailSequenceTypeAction === 'all') {
                                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.status IN (?)', array('0', '1', '2'));
                                $where .= ' AND '. $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.type = ?', 'sms');
                                $where .= ' AND '. $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.sequence_id IN (?)',  $filterParams['email_sms_sequences']);
                                if (!empty($periodDays)) {
                                    if (!empty($where)) {
                                        $where .= ' AND ';
                                    }

                                    $where .= '(('.$leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.sentAt >= ?', $periodDaysStart);
                                    $where .= ' AND '. $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.sentAt <= ?', $periodDaysEnd).')';
                                    $where .= ' OR ('.$leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.delayedTo >= ?', $periodDaysStart);
                                    $where .= ' AND '.$leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.delayedTo <= ?', $periodDaysEnd).'))';
                                }
                            } else {
                                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.status = ?', '1');
                                $where .= ' AND '. $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.type = ?', 'sms');
                                $where .= ' AND '. $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.sequence_id IN (?)',  $filterParams['email_sms_sequences']);
                                if (!empty($periodDays)) {
                                    if (!empty($where)) {
                                        $where .= ' AND ';
                                    }

                                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.sentAt >= ?', $periodDaysStart);
                                    $where .= ' AND '. $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.sentAt <= ?', $periodDaysEnd);
                                }
                            }


                        }

                        if ($filterParams['email_sequence_type'] === 'email') {
                            if ($emailSequenceTypeAction === 'all') {
                                if (!empty($where)) {
                                    $where .= ' AND ';
                                }
                                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.status IN (?)', array('0', '1', '2'));
                                $where .= ' AND '. $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.type = ?', 'email');
                            }
                            if ($emailSequenceTypeAction === 'sent') {
                                if (!empty($where)) {
                                    $where .= ' AND ';
                                }
                                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plel.email_status = ?', 'sent');
                                $where .= ' AND '. $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plel.follow_up_sequences_id IN (?)',  $filterParams['email_sms_sequences']);
                            }
                            if ($emailSequenceTypeAction === 'opened') {
                                if (!empty($where)) {
                                    $where .= ' AND ';
                                }
                                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plel.email_status = ?', 'opened');
                                $where .= ' AND '. $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plel.follow_up_sequences_id IN (?)',  $filterParams['email_sms_sequences']);

                            }

                            if (!empty($periodDays) && ($emailSequenceTypeAction === 'sent' || $emailSequenceTypeAction === 'opened')) {
                                if (!empty($where)) {
                                    $where .= ' AND ';
                                }

                                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plal.created_at  >= ?', $periodDaysStart);
                                $where .= ' AND '. $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plal.created_at  <= ?', $periodDaysEnd);
                            } elseif (!empty($periodDays)){
                                if (!empty($where)) {
                                    $where .= ' AND ';
                                }

                                $where .= '(('.$leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.sentAt >= ?', $periodDaysStart);
                                $where .= ' AND '. $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.sentAt <= ?', $periodDaysEnd).')';
                                $where .= ' OR ('.$leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.delayedTo >= ?', $periodDaysStart);
                                $where .= ' AND '.$leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.delayedTo <= ?', $periodDaysEnd).'))';
                            }

                        }
                    }
                }

                if (!empty($filterParams['lead_tags_exclude']) && is_array($filterParams['lead_tags_exclude'])) {
                    $subSelect = $leadDataMapper->getDbTable()->select(Zend_Db_Table::SELECT_WITHOUT_FROM_PART)
                        ->distinct()
                        ->setIntegrityCheck(false)
                        ->from(array('ltags' => 'plugin_leads_has_tags'), array('ltags.lead_id'));
                    $subSelectWhere = $leadDataMapper->getDbTable()->getAdapter()->quoteInto('tag_id IN (?)', $filterParams['lead_tags_exclude']);
                    $subSelect->where($subSelectWhere);
                    $subSelectIds = $leadDataMapper->getDbTable()->getAdapter()->fetchCol($subSelect);
                    if (!empty($subSelectIds)) {
                        if (!empty($where)) {
                            $where .= ' AND ';
                        }
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.id NOT IN (?)', $subSelectIds);
                    }
                }

                if (!empty($filterParams['last_interaction_id'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.last_activity_type_id = ?', $filterParams['last_interaction_id']);
                }

                if (!empty($filterParams['last_interaction_id']) && !empty($filterParams['outbound_emails_sub_type'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.last_activity_subtype = ?', $filterParams['outbound_emails_sub_type']);
                }

                if (!empty($filterParams['total_opportunity_amount_from']) || !empty($filterParams['total_opportunity_amount_to']) || !empty($filterParams['lead_opportunity_stages']) || !empty($filterParams['opportunity_type_id'])) {
                    $withOpportunityAmount = true;
                }

                if (!empty($filterParams['total_spent_purchase_amount_from']) || !empty($filterParams['total_spent_purchase_amount_to']) && !empty($filterParams['special_filter_type']) && $filterParams['special_filter_type'] === self::FILTER_EXISTING_CLIENT) {
                    $withSpentPurchaseAmount = true;

                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('scs.status IN (?)', array(Models_Model_CartSession::CART_STATUS_COMPLETED, Models_Model_CartSession::CART_STATUS_SHIPPED, Models_Model_CartSession::CART_STATUS_DELIVERED));
                }

                if (!empty($filterParams['lead_opportunity_stages'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plop.opportunity_stage_id IN (?)', $filterParams['lead_opportunity_stages']);
                }

                if (!empty($filterParams['opportunity_type_id'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plop.opportunity_type_id IN (?)', $filterParams['opportunity_type_id']);
                }

                if (!empty($filterParams['lead_task'])) {
                    $withTasks = true;
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    if ($filterParams['lead_task'] === self::FILTER_TASK_BOTH) {
                        $where .= new Zend_Db_Expr('plta.id IS NOT NULL');
                    }
                    if ($filterParams['lead_task'] === self::FILTER_TASK_OPENED) {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plta.is_task_completed = ?', '0');
                    }
                    if ($filterParams['lead_task'] === self::FILTER_TASK_CLOSED) {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plta.is_task_completed = ?', '1');
                    }
                    if ($filterParams['lead_task'] === self::FILTER_TASK_OVERDUE) {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plta.is_task_overdue = ?', '1');
                        $where .= ' AND '. $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plta.is_task_completed = ?', '0');
                    }
                    if ($filterParams['lead_task'] === self::FILTER_TASK_NONE) {
                        $where .= new Zend_Db_Expr('plta.id IS NULL');
                    }

                }

                if (!empty($filterParams['custom_organization_params_search']) && is_array($filterParams['custom_organization_params_search'])) {
                    $leadsOrganizationCustomParamsMapper = Leads_Mapper_LeadsOrganizationCustomParamsConfigMapper::getInstance();
                    $tempParamCounter = 1;
                    foreach ($filterParams['custom_organization_params_search'] as  $searchCustomParamsData) {
                        $searchParamId = $searchCustomParamsData['id'];
                        $organizationCustomParamsModel = $leadsOrganizationCustomParamsMapper->find($searchParamId);
                        if ($organizationCustomParamsModel instanceof Leads_Model_LeadsOrganizationCustomParamsConfigModel) {
                            $includesCustomOrganizationFields = true;
                            $tempParamName = 'customOrganizationParamVal' . $tempParamCounter;
                            if (!empty($having)) {
                                $having .= ' AND ';
                            }
                            $operator = $searchCustomParamsData['operator'];
                            $paramOperator = '=';
                            if ($operator === self::RULE_COMPARISON_OPERATOR_EQUAL) {
                                $paramOperator = '=';
                            }
                            if ($operator === self::RULE_COMPARISON_OPERATOR_NOTEQUAL) {
                                $paramOperator = '<>';
                            }
                            if ($operator === self::RULE_COMPARISON_GREATER_THAN) {
                                $paramOperator = '>';
                                $searchCustomParamsData['value'] = floatval($searchCustomParamsData['value']);
                            }
                            if ($operator === self::RULE_COMPARISON_LESS_THAN) {
                                $paramOperator = '<';
                                $searchCustomParamsData['value'] = floatval($searchCustomParamsData['value']);
                            }

                            if ($operator === self::RULE_COMPARISON_OPERATOR_LIKE) {
                                $paramOperator = 'LIKE';
                                $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('(' . $tempParamName . ') ' . $paramOperator . ' ?',
                                    '%'. $searchCustomParamsData['value']. '%');
                            } elseif ($operator === self::RULE_COMPARISON_OPERATOR_EQUAL) {
                                if (empty($searchCustomParamsData['value'])) {
                                    $having .= '('.$leadDataMapper->getDbTable()->getAdapter()->quoteInto('(' . $tempParamName . ') ' . $paramOperator . ' ?',
                                            $searchCustomParamsData['value']).' OR ';
                                    $having .= '(' . $tempParamName . ') '. 'IS NULL';

                                    $having .=   ')';
                                } else {
                                    $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('(' . $tempParamName . ') ' . $paramOperator . ' ?',
                                        $searchCustomParamsData['value']);
                                }
                            } elseif ($operator === self::RULE_COMPARISON_GREATER_THAN || $operator === self::RULE_COMPARISON_LESS_THAN) {
                                $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('CAST(' . $tempParamName . '  AS DECIMAL(10,2))' . $paramOperator . ' ?',
                                    $searchCustomParamsData['value']);
                            } else {
                                $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('(' . $tempParamName . ') ' . $paramOperator . ' ?',
                                    $searchCustomParamsData['value']);
                            }
                            $additionalOrganizationParamsForSelect = array_merge($additionalOrganizationParamsForSelect,
                                array($tempParamName => new Zend_Db_Expr('GROUP_CONCAT(DISTINCT(CASE WHEN plocpd.param_id=' . $searchParamId . ' THEN plocpd.param_value END))')));
                            $tempParamCounter++;
                        }
                    }
                }

                if (!empty($filterParams['custom_params_search']) && is_array($filterParams['custom_params_search'])) {
                    $leadsCustomParamsMapper = Leads_Mapper_LeadsCustomParamsConfigMapper::getInstance();
                    $tempParamCounter = 1;
                    foreach ($filterParams['custom_params_search'] as  $searchCustomParamsData) {
                        $searchParamId = $searchCustomParamsData['id'];
                        $leadsCustomParamsModel = $leadsCustomParamsMapper->find($searchParamId);
                        if ($leadsCustomParamsModel instanceof Leads_Model_LeadsCustomParamsConfigModel) {
                            $includesCustomLeadFields = true;
                            $tempParamName = 'customParamVal' . $tempParamCounter;
                            if (!empty($having)) {
                                $having .= ' AND ';
                            }
                            $operator = $searchCustomParamsData['operator'];
                            $paramOperator = '=';
                            if ($operator === self::RULE_COMPARISON_OPERATOR_EQUAL) {
                                $paramOperator = '=';
                            }
                            if ($operator === self::RULE_COMPARISON_OPERATOR_NOTEQUAL) {
                                $paramOperator = '<>';
                            }
                            if ($operator === self::RULE_COMPARISON_GREATER_THAN) {
                                $paramOperator = '>';
                                $searchCustomParamsData['value'] = floatval($searchCustomParamsData['value']);
                            }
                            if ($operator === self::RULE_COMPARISON_LESS_THAN) {
                                $paramOperator = '<';
                                $searchCustomParamsData['value'] = floatval($searchCustomParamsData['value']);
                            }

                            if ($operator === self::RULE_COMPARISON_OPERATOR_LIKE) {
                                $paramOperator = 'LIKE';
                                $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('(' . $tempParamName . ') ' . $paramOperator . ' ?',
                                    '%'. $searchCustomParamsData['value']. '%');
                            } elseif ($operator === self::RULE_COMPARISON_OPERATOR_EQUAL) {
                                if (empty($searchCustomParamsData['value'])) {
                                    $having .= '('.$leadDataMapper->getDbTable()->getAdapter()->quoteInto('(' . $tempParamName . ') ' . $paramOperator . ' ?',
                                        $searchCustomParamsData['value']).' OR ';
                                    $having .= '(' . $tempParamName . ') '. 'IS NULL';

                                    $having .=   ')';
                                } else {
                                    $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('(' . $tempParamName . ') ' . $paramOperator . ' ?',
                                        $searchCustomParamsData['value']);
                                }
                            } elseif ($operator === self::RULE_COMPARISON_GREATER_THAN || $operator === self::RULE_COMPARISON_LESS_THAN) {
                                $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('CAST(' . $tempParamName . '  AS DECIMAL(10,2))' . $paramOperator . ' ?',
                                    $searchCustomParamsData['value']);
                            } else {
                                $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('(' . $tempParamName . ') ' . $paramOperator . ' ?',
                                    $searchCustomParamsData['value']);
                            }
                            $additionalParamsForSelect = array_merge($additionalParamsForSelect,
                                array($tempParamName => new Zend_Db_Expr('GROUP_CONCAT(DISTINCT(CASE WHEN pllcpd.param_id=' . $searchParamId . ' THEN pllcpd.param_value END))')));
                            $tempParamCounter++;
                        }
                    }
                }

                if (!empty($filterParams['email_status'])) {
                    $withEmailLog = true;

                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plel.email_status = ? ', $filterParams['email_status']);
                }


                if(!empty($filterParams['industry_id'])) {
                    $withOrganizationIndustry = true;
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pli.id = ?', $filterParams['industry_id']);
                }

                if (!empty($filterParams['total_opportunity_amount_from'])) {
                    if (!empty($having)) {
                        $having .= ' AND ';
                    }
                    if (empty($filterParams['next_activity_from'])) {
                        $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('SUM(plop.opportunity_value)/ COUNT(DISTINCT(ifnull(CONCAT(`plht`.`tag_id`, `plht`.`lead_id`), 1))) >= ?',
                            $filterParams['total_opportunity_amount_from']);
                    } else {
                        $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('(SUM(plop.opportunity_value)/ COUNT(DISTINCT(ifnull(CONCAT(`plht`.`tag_id`, `plht`.`lead_id`), 1)))/COUNT(DISTINCT(ifnull(CONCAT(`plal`.`id`, `plal`.`lead_id`), 1)))) >= ?',
                            $filterParams['total_opportunity_amount_from']);
                    }
                }
                if (!empty($filterParams['total_opportunity_amount_to'])) {
                    if (!empty($having)) {
                        $having .= ' AND ';
                    }
                    if (empty($filterParams['next_activity_from'])) {
                        $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('SUM(plop.opportunity_value)/ COUNT(DISTINCT(ifnull(CONCAT(`plht`.`tag_id`, `plht`.`lead_id`), 1))) <= ?',
                            $filterParams['total_opportunity_amount_to']);
                    } else {
                        $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('(SUM(plop.opportunity_value)/ COUNT(DISTINCT(ifnull(CONCAT(`plht`.`tag_id`, `plht`.`lead_id`), 1)))/COUNT(DISTINCT(ifnull(CONCAT(`plal`.`id`, `plal`.`lead_id`), 1)))) <= ?',
                            $filterParams['total_opportunity_amount_to']);
                    }
                }

                if (!empty($filterParams['total_spent_purchase_amount_from'])) {
                    if (!empty($having)) {
                        $having .= ' AND ';
                    }
                    $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('SUM(scs.total)/ COUNT(DISTINCT(ifnull(CONCAT(`plht`.`tag_id`, `plht`.`lead_id`), 1))) >= ?',
                        $filterParams['total_spent_purchase_amount_from']);
                }

                if (!empty($filterParams['total_spent_purchase_amount_to'])) {
                    if (!empty($having)) {
                        $having .= ' AND ';
                    }
                    $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('SUM(scs.total)/ COUNT(DISTINCT(ifnull(CONCAT(`plht`.`tag_id`, `plht`.`lead_id`), 1))) <= ?',
                        $filterParams['total_spent_purchase_amount_to']);
                }

                if (!empty($filterParams['interaction_value'])) {
                    if (in_array($filterParams['interaction_value'], self::$_sortingTypes, true)) {
                        if ($filterParams['interaction_value'] === self::SORT_ORDER_LEAD_NAME) {
                            $sortOrder = 'pld.' . $filterParams['interaction_value'] . ' ASC';
                        } else {
                            $sortOrder = 'pld.' . $filterParams['interaction_value'] . ' DESC';
                        }
                        if ($filterParams['interaction_value'] === self::SORT_ORDER_LAST_ACTIVITY_INBOUND || $filterParams['interaction_value'] === self::SORT_ORDER_LAST_ACTIVITY_OUTBOUND) {
                            $orderInteractionParam = $filterParams['interaction_value'];
                        }
                    }
                }
            } elseif (!empty($searchTerm)) {
                $where = $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_email LIKE ?',
                    $searchTerm . '%');
            } elseif (!empty($email)) {
                $where = $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_email = ?', $email);
            } elseif (!empty($mobile) && !empty($mobileCountryCode)) {
                    $mobileCountryPhoneCode = Zend_Locale::getTranslation($mobileCountryCode, 'phoneToTerritory');
                    $mobileCountryCode = '+'.$mobileCountryPhoneCode;
                    $mobile = Tools_LeadTools::cleanPhoneAddress($mobile);
                    $where = '('.$leadDataMapper->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr("CONCAT(`pld`.`lead_mobile_country_code_value`, `pld`.`lead_mobile`) = ?"), $mobileCountryCode . $mobile);
                    $where .= ' OR ' .$leadDataMapper->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr("CONCAT(`pld`.`lead_phone_country_code_value`, `pld`.`lead_phone`) = ?"), $mobileCountryCode . $mobile).')';

            } elseif (!empty($leadFirstName) && !empty($leadLastName) && !empty($leadZip)) {
                $where = $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_first_name = ?', $leadFirstName);
                $where .= ' AND ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_last_name = ?', $leadLastName);
                $where .= ' AND ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_zip = ?', $leadZip);
            }

            $data = $leadDataMapper->fetchAll(
                $where,
                $sortOrder,
                $limit,
                $offset,
                false,
                false,
                $withOpportunityAmount,
                $having,
                $withNextActivityDate,
                $withTasks,
                $callDuration,
                $includesNotes,
                $includesCustomLeadFields,
                $includesCustomOrganizationFields,
                $withOpportunityExpected,
                $withSpentPurchaseAmount,
                $withEmailLog,
                $withOrganizationIndustry,
                $additionalParamsForSelect,
                $additionalOrganizationParamsForSelect,
                true,
                $includePluginLeadsTypeCpecificColl,
                $withEmailSmsSequence
            );

        }

        if (empty($data)) {
            $data = array();
        }

        $currentUser = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser();
        $leadOrganizationDataMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();

        if ($forGrid) {
            $data['organizationModel'] = array();
            if (!empty($data['use_organization_address'])) {
                $organizationModel = $leadOrganizationDataMapper->find($data['organization_id']);
                if ($organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                    $data['organizationModel'] = $organizationModel->toArray();

                    if(!empty($data['organizationModel']['organizationIndustryId'])) {
                        $organizationIndustry = Leads_Mapper_LeadsIndustryMapper::getInstance()->find($data['organizationModel']['organizationIndustryId']);
                        if($organizationIndustry instanceof Leads_Model_LeadsIndustryModel) {
                            $data['organizationModel']['organizationIndustry'] = $organizationIndustry->getLabel();
                        }
                    }
                }
            }
            $data['groups'] = $leadDataMapper->getCustomerGroups();
            $data['stages'] = Leads_Mapper_LeadsStageMapper::getInstance()->getStages();
            $leadTypeMapper = Leads_Mapper_LeadsTypeMapper::getInstance();
            $data['types'] = Leads_Mapper_LeadsTypeMapper::getInstance()->getTypes();
            $moneyFormat = Tools_LeadTools::getCurrencyFormat();
            $listMasksMapper = Application_Model_Mappers_MasksListMapper::getInstance();
            $data['mobileMasks'] = $listMasksMapper->getListOfMasksByType(Application_Model_Models_MaskList::MASK_TYPE_MOBILE);
            $data['desktopMasks'] = $listMasksMapper->getListOfMasksByType(Application_Model_Models_MaskList::MASK_TYPE_DESKTOP);
            $data['moneyFormat'] = json_encode($moneyFormat);
            $data['phoneCountryCodes'] =  Tools_System_Tools::getFullCountryPhoneCodesList(true, array(), true);
            if (!empty($data['types']) && !empty($data['last_activity_type_id'])) {
                $leadTypeModel = $leadTypeMapper->find($data['last_activity_type_id']);
                if ($leadTypeModel instanceof Leads_Model_LeadsTypeModel) {
                    $data['leadTypeName'] = $leadTypeModel->getTypeName();
                }
            }
            $data['tags'] = Leads_Mapper_LeadsTagsMapper::getInstance()->getTags();
            $data['socialServices'] = Leads_Mapper_LeadsSocialServicesConfigMapper::getInstance()->getServices();
            $data['currentDesktopCountryCode'] = $currentUser->getDesktopCountryCode();
            $data['currentDesktopPhone'] = $currentUser->getDesktopPhone();
            $data['currentVoipPhone'] = $currentUser->getVoipPhone();
            $data['currentMobileCountryCode'] = $currentUser->getMobileCountryCode();
            $data['currentMobilePhone'] = $currentUser->getMobilePhone();
            $translator = Zend_Registry::get('Zend_Translate');
            $data['reviewSystems'] = array(
                'fb' => $translator->translate(Leads::TRANSLATION_PREFIX.'Facebook'),
                'gmb' => $translator->translate(Leads::TRANSLATION_PREFIX.'GMB')
            );

            $data['limitedAccess'] =  $this->_restrictedUserId;

            if(!empty($orderInteractionParam)){
                $data['interactionParam'] = $orderInteractionParam;
            }

            if (!empty($data['data']) && is_array($data['data'])) {
                foreach ($data['data'] as $key => $leadData) {
                    $data['data'][$key]['nextActionOn'] = $this->_getUpcomingEvent($leadData['id']);
                }
            } elseif(!empty($data['id'])) {
                $nextActionData = $this->_getUpcomingEvent($data['id'], true);
                $nextActionDate = '';
                $nextActionType = '';
                if (!empty($nextActionData)) {
                    $nextActionDate = $nextActionData['created_at'];
                    $nextActionType = $nextActionData['actionType'];
                }

                $data['reviewRequestSendAgain'] = false;
                if (!empty($data['reviewStatus']) && $data['reviewStatus'] === Leads_Model_LeadsReviewRequestModel::STATUS_SENT) {
                    if (!empty($data['reviewSentAt'])) {
                        if (strtotime($data['reviewSentAt']) < strtotime(Tools_System_Tools::convertDateFromTimezone(date('Y-m-d', strtotime('-1 month')). ' 00:00:00'))){
                            $data['reviewRequestSendAgain'] = true;
                        }
                    }
                }

                $data['realReviewRequestSendAgain'] = false;
                if (!empty($data['reviewRequestStatus']) && $data['reviewRequestStatus'] === Leads_Model_LeadsReviewRequestModel::STATUS_SENT) {
                    $reviewRequestSentAt = $data['reviewRequestSentAt'];
                    if (strtotime($reviewRequestSentAt) < strtotime(Tools_System_Tools::convertDateFromTimezone(date('Y-m-d', strtotime('-1 month')). ' 00:00:00'))){
                        $data['realReviewRequestSendAgain'] = true;
                    }
                }

                $data['taskCronscriptLink'] = Zend_Registry::get('website')['path'].Leads_Model_LeadsTaskModel::LEAD_TASK_PROCESS_PATH;
                $data['meetingscriptLink'] = Zend_Registry::get('website')['path'].Leads_Model_LeadsMeetingModel::LEAD_MEETING_PROCESS_PATH;
                $session = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
                $userRole = $session->getCurrentUser()->getRoleId();
                $data['isSuperAdminLogged'] = '0';
                if ($userRole === Tools_Security_Acl::ROLE_SUPERADMIN) {
                     $data['isSuperAdminLogged'] = '1';
                }

                $data['nextActionOn'] = $nextActionDate;
                $data['nextActionType'] = $nextActionType;
                $data['currentUserId'] = $currentUser->getId();
                $data['currentUserEmail'] = $currentUser->getEmail();
                $data['leadSocialServices'] = $leadDataMapper->getLeadSocialServices($data['id']);
                $leadOpportunityDataMapper = Leads_Mapper_LeadsOpportunityMapper::getInstance();
                $data['leadOpportunityStats'] = $leadOpportunityDataMapper->getLeadOpportunityQuoteStatSalesDetailedView($data['id']);
                $data['currencySymbol'] = preg_replace('~[\w]~', '', $currency->getSymbol());
                $data['firstPurchaseDate'] = $leadOpportunityDataMapper->getFirstPurchaseDate($data['id']);
                $data['isQuizzEnabled'] = Tools_LeadTools::isPluginEnabled('Quizz');
                $totalSales = $leadOpportunityDataMapper->getLeadTotalSalesDetailedView($data['id']);
                $opportunitiesWhichNotSold = $leadOpportunityDataMapper->getOpportunitiesWhichNotSold($data['id']);

                $leadsCustomTabsConfigMapper = Leads_Mapper_LeadsCustomTabsConfigMapper::getInstance();

                $customTabsWhere = $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('lct.tab_type = ?', Api_Leads_Leadcustomtabsfields::CUSTOM_TAB_TYPE_LEAD);
                $customConfigsTabs = $leadsCustomTabsConfigMapper->getCustomTabsConfigInfo($customTabsWhere, null, null, null, null, true);

                if(!empty($customConfigsTabs)) {
                    $data['customConfigsTabs'] = $customConfigsTabs;
                }

                if (!empty($opportunitiesWhichNotSold)) {
                    foreach ($opportunitiesWhichNotSold as $opportunitiesWhichNotSoldSingle) {
                        if (!empty($totalSales)) {
                            $totalSales['count'] = $totalSales['count'] + 1;
                            $totalSales['total'] = $totalSales['total'] + $opportunitiesWhichNotSoldSingle['opportunity_value'];
                        }   else {
                            $totalSales = array(
                                'count' => 1,
                                'total' => $opportunitiesWhichNotSoldSingle['opportunity_value']
                            );
                    }
                    }
                }
                $opportunitiesWithoutQuotes = $leadOpportunityDataMapper->getOpportunitiesWithoutQuotes($data['id']);
                if (!empty($opportunitiesWithoutQuotes)) {
                    foreach ($opportunitiesWithoutQuotes as $opportunitiesWithoutQuotesSingle) {
                        if (!empty($totalSales)) {
                            $totalSales['count'] = $totalSales['count'] + 1;
                            $totalSales['total'] = $totalSales['total'] + $opportunitiesWithoutQuotesSingle['opportunity_value'];
                        } else {
                            $totalSales = array(
                                'count' => 1,
                                'total' => $opportunitiesWithoutQuotesSingle['opportunity_value']
                            );
                        }
                    }
                }


//                if (!empty($data['leadOpportunityStats']) && !empty($data['leadOpportunityStats']['100'])) {
//                    if (!empty($totalSales)) {
//                        $totalSales['count'] = $totalSales['count'] + $data['leadOpportunityStats']['100']['count'];
//                        $totalSales['total'] = $totalSales['total'] + $data['leadOpportunityStats']['100']['total'];
//                    } else {
//                        $totalSales = array(
//                            'count' => $data['leadOpportunityStats']['100']['count'],
//                            'total' => $data['leadOpportunityStats']['100']['total']
//                        );
//                    }
//                }
//
//                $leadOpportunityStatsExclude = $leadOpportunityDataMapper->getCartStatsForOpportunityStage($data['id'], array('100'));
//                if (!empty($totalSales) && !empty($leadOpportunityStatsExclude)) {
//                    $totalSales['count'] = $totalSales['count'] - $leadOpportunityStatsExclude['count'];
//                    $totalSales['total'] = $totalSales['total'] - $leadOpportunityStatsExclude['total'];
//                }

                if (!empty($totalSales)) {
                    $data['totalSales'] = $totalSales;
                } else {
                    $data['totalSales'] = '';
                }
            }
            $data['userTimezoneOffset'] = Tools_System_Tools::convertDateFromTimezone('now', 'UTC', Tools_System_Tools::getUserTimezone(), 'T');

            if ($detailedInfo) {
                $data['countries'] = Tools_Geo::getCountries(true);
                if (!empty($data['use_organization_address']) && !empty($data['organizationModel']) && !empty($data['organizationModel']['organizationCountryCode'])) {
                    $data['states'] = Tools_LeadTools::getStates($data['organizationModel']['organizationCountryCode'], true, true);
                } elseif(!empty($data['lead_country_code'])) {
                    $data['states'] = Tools_LeadTools::getStates($data['lead_country_code'], true, true);
                }


                $data['sequenceData'] = Leads_Mapper_LeadsHasEmailSequenceMapper::getInstance()->getLeadSequenceByLeadId($id);
            }

            $data['mimeTypes'] = Tools_LeadDocumentsTools::$_acceptedFileTypesArray;
            $uploadMaxSize = ini_get('upload_max_filesize');
            $data['uploadMaxSize'] = !empty($uploadMaxSize) ? $uploadMaxSize : '';

            // max upload file size and files count
            $data['allowedUploadData'] = Tools_System_Tools::getAllowedUploadData();

            $data['leadsOwners'] = array();
            if (empty($this->_restrictedUserId)) {
                $owners = Leads_Mapper_LeadsMapper::getInstance()->getLeadOwnersFullList();
                $allUsers = Leads_Mapper_LeadsMapper::getInstance()->getAllUsers(true);
                $data['allUsers'] = $allUsers;
                if(!empty($owners)){
                    $data['leadsOwners'] = $owners;
                }
            }
        }

        return $data;
    }

    /**
     * New lead creation
     *
     * Resource:
     * : /api/leads/leads/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING);
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'));
        }

        /*if (empty($data['leadEmail']) && empty($data['leadPhone'])) {
            $this->_error($translator->translate('Please specify either email or phone number'));
        }*/

        if (isset($data['leadEmail'])) {
            $data['leadEmail'] = trim($data['leadEmail']);
        }

        if (isset($data['leadSecondaryEmail'])) {
            $data['leadSecondaryEmail'] = trim($data['leadSecondaryEmail']);
        }

        $allowedToCreate = false;
        if (!empty($data['leadEmail'])) {
            $allowedToCreate = true;
        } elseif (!empty($data['leadPhone'])) {
            $allowedToCreate = true;
        } elseif (!empty($data['leadMobile'])) {
            $allowedToCreate = true;
        }
        elseif (!empty($data['leadFirstName']) && !empty($data['leadLastName']) && !empty($data['leadZip'])) {
            $allowedToCreate = true;
        }

        if(!$allowedToCreate) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Please specify either email or phone number or first|last name and zip code'));
        }

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
        }

        $leadAdditionAvailable = Tools_LeadLicenseTools::checkLimit();
        if ($leadAdditionAvailable['error'] === true) {
            $this->_error($translator->translate($leadAdditionAvailable['message']));
        }

        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
        $leadsModel = new Leads_Model_LeadsModel();

        if (!empty($data['lead_mobile'])) {
            $data['lead_mobile'] = Tools_LeadTools::cleanPhoneAddress($data['lead_mobile']);
        }


        if (!empty($data['lead_phone'])) {
            $data['lead_phone'] = Tools_LeadTools::cleanPhoneAddress($data['lead_phone']);
        }

        if (!empty($data['lead_phone_country_code'])) {
            $phoneCountryPhoneCode = Zend_Locale::getTranslation($data['lead_phone_country_code'], 'phoneToTerritory');
            $data['lead_phone_country_code_value'] = '+'.$phoneCountryPhoneCode;
        }

        if (!empty($data['lead_mobile_country_code'])) {
            $mobileCountryPhoneCode = Zend_Locale::getTranslation($data['lead_mobile_country_code'], 'phoneToTerritory');
            $data['lead_mobile_country_code_value'] = '+'.$mobileCountryPhoneCode;
        }

        if (empty($data['lead_mobile'])) {
            $data['lead_mobile'] = null;
            $data['lead_mobile_country_code_value'] = null;
        }

        if (!empty($data['leadPhone'])) {
            $data['lead_phone'] = Tools_LeadTools::cleanPhoneAddress($data['leadPhone']);
        }

        if (!empty($data['leadMobile'])) {
            $data['lead_mobile'] = Tools_LeadTools::cleanPhoneAddress($data['leadMobile']);
        }

        if (empty($data['lead_phone'])) {
            $data['lead_phone'] = null;
            $data['lead_phone_country_code_value'] = null;
        }

        if (empty($data['lead_mobile'])) {
            $data['lead_mobile'] = null;
            $data['lead_mobile_country_code_value'] = null;
        }

        $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
        $userDefaultMobileCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');
        if (empty($data['lead_mobile_country_code']) && !empty($userDefaultMobileCountryCode)) {
            $data['lead_mobile_country_code'] = $userDefaultMobileCountryCode;
        }

        if (empty($data['lead_phone_country_code']) && !empty($userDefaultMobileCountryCode)) {
            $data['lead_phone_country_code'] = $userDefaultMobileCountryCode;
        }

        $leadsModel->setOptions($data);
        $leadsModel->setProductIds(array_filter(explode(',', $leadsModel->getProductIds())));

        if (!empty($data['customerGroupId'])) {
            $groupId = filter_var($data['customerGroupId'], FILTER_SANITIZE_NUMBER_INT);
            if (!empty($groupId)) {
                $validator = new Zend_Validate_Db_NoRecordExists(array(
                    'table' => 'shopping_group',
                    'field' => 'id'
                ));

                if ($validator->isValid($groupId)) {
                    $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Group is invalid.'));
                }
            }
        }

        $assignPrimaryContactId = false;
        $leadOrganizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
        $organizationModel = null;

        if (!empty($data['organization_id'])) {
            $organizationModel = $leadOrganizationMapper->find($data['organization_id']);
            if ($organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                $leadsModel->setOrganizationId($organizationModel->getId());
            }
        }

        /**
         * Create organization if not exists
         */
        if (!empty($data['organizationName'])) {
            $organizationName = $data['organizationName'];
            $organizationModel = $leadOrganizationMapper->findByName($organizationName);
            if (!$organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                $assignPrimaryContactId = true;
                $organizationModel = new Leads_Model_LeadsOrganizationModel();
                $organizationModel->setOrganizationName($organizationName);
                $createdAt = Tools_System_Tools::convertDateFromTimezone('now');
                $organizationModel->setCreatedAt($createdAt);
                $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
                $userDefaultMobileCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');
                if (!empty($userDefaultMobileCountryCode)) {
                    $organizationModel->setOrganizationMobileCountryCode($userDefaultMobileCountryCode);
                    $organizationModel->setOrganizationPhoneCountryCode($userDefaultMobileCountryCode);
                }

                $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();
                if (!empty($shoppingConfig['country'])) {
                    $defaultCountryCode = $shoppingConfig['country'];
                    $organizationModel->setOrganizationCountryCode($defaultCountryCode);
                }
                $organizationModel = $leadOrganizationMapper->save($organizationModel);
            }
            $leadsModel->setOrganizationId($organizationModel->getId());

        }

        if (!empty($data['leadEmail'])) {
            $emailValidator = new Tools_System_CustomEmailValidator();

            if (!$emailValidator->isValid($data['leadEmail'])) {
                $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Not valid email address'));
            }

            $validator = new Zend_Validate_Db_NoRecordExists(array(
                'table' => 'plugin_leads_data',
                'field' => 'lead_email'
            ));

            if (!$validator->isValid($data['leadEmail'])) {
                $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Email already in use'));
            }
        } else {
            $leadsModel->setLeadEmail(null);
        }


        $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
        $websiteUrl = $websiteHelper->getUrl();

        if (!empty($data['lead_mobile_country_code_value']) && !empty($data['lead_mobile'])) {
            $leadsModelWithPhone = Tools_LeadTools::findLeadByPhoneMobileNumber($data['lead_mobile_country_code_value'] . $data['lead_mobile']);
            if ($leadsModelWithPhone instanceof Leads_Model_LeadsModel) {
                $leadLink = "<u><a target='_blank' href=".$websiteUrl."dashboard/leads/#lead/".$leadsModelWithPhone->getId().">".$translator->translate(Leads::TRANSLATION_PREFIX."Lead")."</a></u>";
                $this->_error($translator->translate(Leads::TRANSLATION_PREFIX."You already have a"). ' '.$leadLink. ' '. $translator->translate(Leads::TRANSLATION_PREFIX."with such phone number"));
            }
        }

        if(!empty($data['leadFirstName']) && !empty($data['leadLastName']) && !empty($data['leadZip'])) {
            $leadsModelWithFirstLastNameZip = $leadsMapper->findByFirstNameLastNameZip($data['leadFirstName'], $data['leadLastName'], $data['leadZip']);

            if($leadsModelWithFirstLastNameZip instanceof Leads_Model_LeadsModel) {
                $leadLink = "<u><a target='_blank' href=".$websiteUrl."dashboard/leads/#lead/".$leadsModelWithFirstLastNameZip->getId().">".$translator->translate(Leads::TRANSLATION_PREFIX."Lead")."</a></u>";
                $this->_error($translator->translate(Leads::TRANSLATION_PREFIX."You already have a"). ' '.$leadLink. ' '. $translator->translate(Leads::TRANSLATION_PREFIX."with such name and zip"));
            }

            $leadsModel->setLeadZip($data['leadZip']);
        }

        $session = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
        $userId = $session->getCurrentUser()->getId();

        $createdAt = Tools_System_Tools::convertDateFromTimezone('now');

        $leadsModel->setOwnerId($userId);
        $leadsModel->setCreatedAt($createdAt);

        $leadTypeModel = Leads_Mapper_LeadsTypeMapper::getInstance()->getDefaultType();
        if ($leadTypeModel instanceof Leads_Model_LeadsTypeModel && empty($data['lead_type_id'])) {
            $leadsModel->setLeadTypeId($leadTypeModel->getId());
        }

        $leadStageModel = Leads_Mapper_LeadsStageMapper::getInstance()->getDefaultStage();
        if ($leadStageModel instanceof Leads_Model_LeadsStageModel && empty($data['stage_id'])) {
            $leadsModel->setStageId($leadStageModel->getId());
        }

        $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();

        if (!empty($data['lead_mobile_country_code'])) {
            $leadsModel->setLeadCountryCode($data['lead_mobile_country_code']);
        } elseif (!empty($shoppingConfig['country'])) {
            $defaultCountryCode = $shoppingConfig['country'];
            if (empty($data['lead_country_code'])) {
                $leadsModel->setLeadCountryCode($defaultCountryCode);
            }
        }

        if (!empty($shoppingConfig['state'])) {
            $defaultStateId = $shoppingConfig['state'];
            if (empty($data['lead_state_code'])) {
                $allStates = Tools_LeadTools::getStates($leadsModel->getLeadCountryCode(), true, false, 'state');
                if (array_key_exists($defaultStateId, $allStates)) {
                    $leadsModel->setLeadStateCode($allStates[$defaultStateId]);
                }
            }
        }

        $leadsModel = $leadsMapper->save($leadsModel);

        /**
         * Assign primary contact id if it's new organization
         */
        if ($assignPrimaryContactId === true) {
            $organizationModel->setPrimaryContactId($leadsModel->getId());
            $leadOrganizationMapper->save($organizationModel);
        }

        if ($organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
            $firstContactOrganizationDate = $organizationModel->getFirstContactCreationDate();
            if (empty($firstContactOrganizationDate)) {
                $organizationModel->setFirstContactCreationDate($leadsModel->getCreatedAt());
                $leadOrganizationMapper->save($organizationModel);
            }
        }

        $this->_responseHelper->success(array('message' =>$translator->translate(Leads::TRANSLATION_PREFIX.'Lead has been created'), 'leadId' => $leadsModel->getId()));
    }

    /**
     * Update lead data
     *
     * Resource:
     * : /api/leads/leads/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     * : leads id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if (!empty($data['id']) && !empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {
            $translator = Zend_Registry::get('Zend_Translate');
            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();
                $this->_error($translator->translate('Your session has timed-out. Please Log back in ' . '<a href="' . $websiteUrl . 'go">here</a>'));
            }
            $leadId = filter_var($data['id'], FILTER_SANITIZE_NUMBER_INT);
            $leadMapper = Leads_Mapper_LeadsMapper::getInstance();

            if (!empty($this->_restrictedUserId)) {
                $leadModel = $leadMapper->findByIdOwnerId($leadId, $this->_restrictedUserId);
            } else {
                $leadModel = $leadMapper->find($leadId);
            }
            if (!$leadModel instanceof Leads_Model_LeadsModel) {
                $this->_responseHelper->fail($translator->translate(Leads::TRANSLATION_PREFIX.'Config doesn\'t exists'));
            }

            if (isset($data['lead_secondary_email'])) {
                $data['lead_secondary_email'] = trim($data['lead_secondary_email']);
            }

            if (!empty($data['lead_secondary_email'])) {
                $emailValidator = new Tools_System_CustomEmailValidator();
                if (!$emailValidator->isValid($data['lead_secondary_email'])) {
                    $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Not valid secondary email address'));
                }
            }

            if (!empty($data['mergeRecordWithId']) && !empty($data['mergeBasedOn'])) {
                if (!empty($this->_restrictedUserId)) {
                    $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'You don\'t have permission to merge leads'));
                }
                $mergeRecordWithId = $data['mergeRecordWithId'];
                $leadLeadModel = $leadMapper->find($mergeRecordWithId);
                if (!$leadLeadModel instanceof Leads_Model_LeadsModel) {
                    $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Lead can\'t be merged'));
                }

                $mergeResult = Tools_LeadMergeTools::mergeLead($data['id'], $mergeRecordWithId, $data['mergeBasedOn'], true);
                if ($mergeResult['error'] === 1) {
                    $this->_error($mergeResult['message']);
                }
                $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Organization data has been merged'));

            } else {

                $currentLeadEmail = $leadModel->getLeadEmail();
                $currentMobilePhone = $leadModel->getLeadMobile();
                $currentPhone = $leadModel->getLeadPhone();
                if (empty($currentLeadEmail) && empty($currentMobilePhone) && empty($currentPhone) && (!empty($data['lead_email']) || !empty($data['lead_mobile']) || !empty($data['lead_phone']))) {
                    $leadAdditionAvailable = Tools_LeadLicenseTools::checkLimit();
                    if ($leadAdditionAvailable['error'] === true) {
                        $this->_error($translator->translate($leadAdditionAvailable['message']));
                    }
                }

                $oldEmail = $leadModel->getLeadEmail();
                $currentEmail = $data['lead_email'];
                if (!empty($data['lead_phone_country_code'])) {
                    $phoneCountryPhoneCode = Zend_Locale::getTranslation($data['lead_phone_country_code'],
                        'phoneToTerritory');
                    $data['lead_phone_country_code_value'] = '+' . $phoneCountryPhoneCode;
                }

                if (!empty($data['lead_mobile_country_code'])) {
                    $mobileCountryPhoneCode = Zend_Locale::getTranslation($data['lead_mobile_country_code'],
                        'phoneToTerritory');
                    $data['lead_mobile_country_code_value'] = '+' . $mobileCountryPhoneCode;
                }
                if (empty($data['customer_group_id'])) {
                    $data['customer_group_id'] = null;
                }

                if (empty($data['organization_id'])) {
                    $data['organization_id'] = null;
                }

                if (empty($data['user_id'])) {
                    $data['user_id'] = null;
                }

                if (empty($data['stage_id'])) {
                    $data['stage_id'] = null;
                }

                $clearStateCode = false;

                if (empty($data['lead_country_code'])) {
                    $data['lead_country_code'] = null;
                } else {
                    $stateCodes = Tools_LeadTools::getStates($data['lead_country_code'], true, true);

                    if(!empty($stateCodes) && empty($data['lead_state_code'])) {
                        $stateCodesKeys = array_keys($stateCodes);
                        $data['lead_state_code'] = $stateCodesKeys[0];
                    } elseif(empty($stateCodes)) {
                        $clearStateCode = true;
                    }
                }

                if (empty($data['lead_state_code'])) {
                    $data['lead_state_code'] = null;
                } elseif ($clearStateCode) {
                    $data['lead_state_code'] = null;
                } elseif (!empty($stateCodes) && !empty($data['lead_state_code']) && empty($stateCodes[$data['lead_state_code']])) {
                    $stateCodesKeys = array_keys($stateCodes);
                    $data['lead_state_code'] = $stateCodesKeys[0];
                }

                if (!empty($data['lead_mobile'])) {
                    $data['lead_mobile'] = Tools_LeadTools::cleanPhoneAddress($data['lead_mobile']);
                }

                if (!empty($data['lead_phone'])) {
                    $data['lead_phone'] = Tools_LeadTools::cleanPhoneAddress($data['lead_phone']);
                }

                if (empty($data['lead_mobile'])) {
                    $data['lead_mobile'] = null;
                    $data['lead_mobile_country_code_value'] = null;
                }

                if (empty($data['lead_phone'])) {
                    $data['lead_phone'] = null;
                    $data['lead_phone_country_code_value'] = null;
                }

                if (!empty($data['lead_phone_country_code_value']) && !empty($data['lead_phone'])) {
                    $leadsModelWithPhone = Tools_LeadTools::findLeadByPhoneMobileNumber($data['lead_phone_country_code_value'] . $data['lead_phone'],
                        true, $data['id']);
                    if ($leadsModelWithPhone instanceof Leads_Model_LeadsModel) {
                        $message = '';

                        if($data['lead_last_name'] != $leadsModelWithPhone->getLeadLastName()) {
                            $message = ' ' . $translator->translate(Leads::TRANSLATION_PREFIX.'Also Lead has different last name'). ' "' . $leadsModelWithPhone->getLeadLastName() . '".';
                        }

                        $this->_responseHelper->fail(array(
                            'duplicatedLeadPhone' => '1',
                            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'You have another') . ' <u><a target="_blank" href="' . Zend_Controller_Action_HelperBroker::getStaticHelper('website')->getUrl() . 'dashboard/leads/#lead/' . $leadsModelWithPhone->getId() . '">' . 'Lead</a></u>' . ' ' . $translator->translate(Leads::TRANSLATION_PREFIX.'with such phone.'). $message,
                            'mergeToId' => $leadsModelWithPhone->getId(),
                            'mergeBasedOn' => Tools_LeadMergeTools::MERGE_LEAD_BASED_ON_PHONE
                        ));
                    }
                }

                if (!empty($data['lead_mobile_country_code_value']) && !empty($data['lead_mobile'])) {
                    $leadsModelWithMobile = Tools_LeadTools::findLeadByPhoneMobileNumber($data['lead_mobile_country_code_value'] . $data['lead_mobile'],
                        true, $data['id']);
                    if ($leadsModelWithMobile instanceof Leads_Model_LeadsModel) {
                        $message = '';

                        if($data['lead_last_name'] != $leadsModelWithMobile->getLeadLastName()) {
                            $message = ' ' . $translator->translate(Leads::TRANSLATION_PREFIX.'Also Lead has different last name'). ' "' . $leadsModelWithMobile->getLeadLastName() . '".';
                        }

                        $this->_responseHelper->fail(array(
                            'duplicatedLeadMobile' => '1',
                            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'You have another') . ' <u><a target="_blank" href="' . Zend_Controller_Action_HelperBroker::getStaticHelper('website')->getUrl() . 'dashboard/leads/#lead/' . $leadsModelWithMobile->getId() . '">' . 'Lead</a></u>' . ' ' . $translator->translate(Leads::TRANSLATION_PREFIX.'with such phone.'). $message,
                            'mergeToId' => $leadsModelWithMobile->getId(),
                            'mergeBasedOn' => Tools_LeadMergeTools::MERGE_LEAD_BASED_ON_MOBILE
                        ));
                    }
                }

                if (!empty($data['lead_phone_country_code'])) {
                    $phoneCountryPhoneCode = Zend_Locale::getTranslation($data['lead_phone_country_code'],
                        'phoneToTerritory');
                    $data['lead_phone_country_code_value'] = '+' . $phoneCountryPhoneCode;
                }

                if (!empty($data['lead_mobile_country_code'])) {
                    $mobileCountryPhoneCode = Zend_Locale::getTranslation($data['lead_mobile_country_code'],
                        'phoneToTerritory');
                    $data['lead_mobile_country_code_value'] = '+' . $mobileCountryPhoneCode;
                }

                $leadUserId = $leadModel->getUserId();

                if (!empty($currentEmail)) {
                    $emailValidator = new Tools_System_CustomEmailValidator();

                    if (!$emailValidator->isValid($currentEmail)) {
                        $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Not valid email address'));
                    }
                }

                if (empty($currentEmail)) {
                    $currentEmail = null;
                    $data['lead_email'] = null;
                }

                if ($oldEmail !== $currentEmail) {
                    $validateEmailExists = new Zend_Validate_Db_RecordExists(array(
                        'table' => 'plugin_leads_data',
                        'field' => 'lead_email'
                    ));
                    if ($validateEmailExists->isValid($currentEmail)) {
                        $duplicatedLeadModel = $leadMapper->findByEmail($currentEmail);
                        if ($duplicatedLeadModel instanceof Leads_Model_LeadsModel) {
                            $this->_responseHelper->fail(array(
                                'duplicatedLeadEmail' => '1',
                                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'You have another') . ' <u><a target="_blank" href="' . Zend_Controller_Action_HelperBroker::getStaticHelper('website')->getUrl() . 'dashboard/leads/#lead/' . $duplicatedLeadModel->getId() . '">' . 'lead</a></u>' . ' ' . $translator->translate(Leads::TRANSLATION_PREFIX.'with such email.'),
                                'mergeToId' => $duplicatedLeadModel->getId(),
                                'mergeBasedOn' => Tools_LeadMergeTools::MERGE_LEAD_BASED_ON_EMAIL
                            ));
                        }
                    }
                }
                if (!empty($leadUserId) && $oldEmail !== $currentEmail) {
                    $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'You can\'t change user email. Real user already assigned to the lead.'));
                }

                if (!empty($leadUserId)) {
                    Tools_LeadTools::syncUserGroup($leadUserId, $data['customer_group_id']);
                }

                $leadModel->setOptions($data);
                $createdAt = Tools_System_Tools::convertDateFromTimezone('now');

                /**
                 * Create organization if not exists
                 */
                if (!empty($data['organization_name'])) {
                    $organizationName = filter_var($data['organization_name'], FILTER_SANITIZE_STRING);
                    if (!empty($organizationName)) {
                        $leadOrganizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
                        $organizationModel = $leadOrganizationMapper->findByName($organizationName);
                        if (!$organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                            $organizationModel = new Leads_Model_LeadsOrganizationModel();
                            $organizationModel->setOrganizationName($organizationName);
                            $organizationModel->setCreatedAt($createdAt);

                            $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
                            $userDefaultMobileCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');
                            if (!empty($userDefaultMobileCountryCode)) {
                                $organizationModel->setOrganizationMobileCountryCode($userDefaultMobileCountryCode);
                                $organizationModel->setOrganizationPhoneCountryCode($userDefaultMobileCountryCode);
                            }

                            $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();
                            if (!empty($shoppingConfig['country'])) {
                                $defaultCountryCode = $shoppingConfig['country'];
                                $organizationModel->setOrganizationCountryCode($defaultCountryCode);
                            }

                            /**
                             * Assign primary contact id if it's new organization
                             */
                            $organizationModel->setPrimaryContactId($leadId);
                            $organizationModel->setFirstContactCreationDate($data['created_at']);
                            $organizationModel = $leadOrganizationMapper->save($organizationModel);
                        }
                        $leadModel->setOrganizationId($organizationModel->getId());
                        $organizationModelWithThisLeadContactId = $leadOrganizationMapper->findByPrimaryContactId($leadModel->getId());
                        if ($organizationModelWithThisLeadContactId instanceof Leads_Model_LeadsOrganizationModel) {
                            $previousPrimaryOrganizationId = $organizationModelWithThisLeadContactId->getId();
                            $currentOrganizationId = $leadModel->getOrganizationId();
                            if ($previousPrimaryOrganizationId != $currentOrganizationId) {
                                $organizationModelWithThisLeadContactId->setPrimaryContactId(null);
                                $firstContactOrganizationDate = $organizationModel->getFirstContactCreationDate();
                                if (empty($firstContactOrganizationDate)) {
                                    $organizationModel->setFirstContactCreationDate($data['created_at']);
                                }
                                $leadOrganizationMapper->save($organizationModelWithThisLeadContactId);
                            }
                        }

                    }
                } else {
                    $leadModel->setOrganizationId(null);
                }

                $updatedAt = Tools_System_Tools::convertDateFromTimezone('now');

                $leadModel->setUpdatedAt($updatedAt);

                $leadModel = $leadMapper->save($leadModel);
                $this->_responseHelper->success(array(
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead data has been updated'),
                    'model' => $leadModel->toArray()
                ));
            }
        }

    }

    /**
     * Delete lead
     *
     * Resource:
     * : /api/leads/leads/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     * : leads id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $deleteAndMarkAsSpam = $this->_request->getParam('deleteAndMarkAsSpam');
        if (!$id) {
            $this->_error();
        }
        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
        if (!empty($this->_restrictedUserId)) {
            $leadModel = $leadsMapper->findByIdOwnerId($id, $this->_restrictedUserId);
        } else {
            $leadModel = $leadsMapper->find($id);
        }
        if ($leadModel instanceof Leads_Model_LeadsModel) {

            $emailSequenceEnabled = Tools_LeadTools::verifyPluginEnabled(Tools_LeadTools::SEQUENCE_PLUGIN);
            if($emailSequenceEnabled) {
                $leadEmailHasSequenceMapper = Leads_Mapper_LeadsHasEmailSequenceMapper::getInstance();
                $leadHasSequences = $leadEmailHasSequenceMapper->getLeadSequenceByLeadId($id);

                if (!empty($leadHasSequences)) {
                    foreach ($leadHasSequences as $leadHasSequence) {
                        $sequenceId = $leadHasSequence['sequence_id'];
                        $leadId = $leadHasSequence['lead_id'];

                        Emailsequence_Models_Mapper_EmailsequenceQueueMapper::getInstance()->deleteQueueBySequenceIdAndExternalId($sequenceId, $leadId, 'lead', array('0'));
                    }
                }
            }

            $pluginLeadsRegistrationLogMapper = Leads_Mapper_LeadsRegistrationLogMapper::getInstance();
            $leadRegistrationModel = $pluginLeadsRegistrationLogMapper->findByLeadId($id);
            if ($leadRegistrationModel instanceof Leads_Model_LeadsRegistrationLogModel) {
                $pluginLeadsRegistrationLogMapper->delete($leadRegistrationModel->getId());
            }

            if (!empty($deleteAndMarkAsSpam)) {
                $leadEmail = $leadModel->getLeadEmail();
                $phoneNumber = '';
                $ipAddress = array();
                if (!empty($leadModel->getLeadMobile())) {
                    $phoneNumber = $leadModel->getLeadMobileCountryCodeValue().$leadModel->getLeadMobile();
                }

                $leadIpsMapper = Leads_Mapper_LeadsIpsMapper::getInstance();
                $leadIpRecordExists = $leadIpsMapper->getByLeadId($id);
                if ($leadIpRecordExists instanceof Leads_Model_LeadsIpsModel) {
                    $ipAddress = $leadIpRecordExists->getLeadIpAddress();
                }

                if (!empty($leadEmail) || !empty($phoneNumber) || !empty($ipAddress)) {
                    $response = Apps::apiCall('POST', 'appsAddLeadToSpam', array(), array(
                        'data' => array(
                            'email' => $leadEmail,
                            'phoneNumber' => $phoneNumber,
                            'ipAddress' => $ipAddress
                        )
                    ), 1);
                }

            }

            $leadsMapper->delete($id);
            $lastViewedLeadsAndOrgMapper = Leads_Mapper_LeadsLastViewedMapper::getInstance();
            $lastViewedLeadsAndOrgMapper->delete($id, Leads_Model_LeadsLastViewedModel::LEAD_LAST_VIEWED_TYPE_LEAD);
        } else {
            $this->_error();
        }

    }

    /**
     * Get upcoming event
     *
     * @param int $id lead id
     * @param bool $allData all data flag
     * @return string
     */
    private function _getUpcomingEvent($id, $allData = false)
    {
        $leadActionLogMapper = Leads_Mapper_LeadsActionLogMapper::getInstance();
        $upcomingEventData = $leadActionLogMapper->getUpcomingEvent($id);
        if (!empty($upcomingEventData)) {
           if ($allData === false) {
               return $upcomingEventData['created_at'];
           }

           return $upcomingEventData;
        }
        return '';
    }

}
