<?php

class Api_Leads_Leadscorerulesattr extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;


    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();

    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    /**
     * User restricted id access
     *
     * @var null
     */
    protected $_restrictedUserId = null;

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
    }


    /**
     * Get by lead form rules general config id
     *
     * Resource:
     * : /api/leads/leadscorerulesattr/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order'), FILTER_SANITIZE_STRING);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $formName = filter_var($this->_request->getParam('formName'), FILTER_SANITIZE_STRING);
        $forGrid = (bool)filter_var($this->_request->getParam('isGrid', false), FILTER_SANITIZE_STRING);
        $detailedInfo = (bool)filter_var($this->_request->getParam('detailedInfo', false), FILTER_SANITIZE_STRING);

        if (empty($data)) {
            $data = array();
        }
        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();

        if ($forGrid) {
            $currentUser = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser();
            $scoreActions = Tools_LeadScoreTools::getScoreActions();
            asort($scoreActions);
            $data['currentUserId'] = $currentUser->getId();
            $data['userTimezoneOffset'] = Tools_System_Tools::getUtcOffset('P');
            $data['userTimezone'] = $currentUser->getTimezone();
            $data['scorePropertyActions'] = Tools_LeadScoreTools::getScorePropertyActions();
            $data['scoreActions'] = $scoreActions;
        }

        if (!empty($data)) {
            $customerGroups = $leadsMapper->getCustomerGroups();
            asort($customerGroups);
            $data['customerGroups'] = $customerGroups;
            $countryCodes = Tools_Geo::getCountries(true);
            asort($countryCodes);
            $data['countryCodes'] = $countryCodes;
            $leadOwners = $leadsMapper->getLeadOwnersFullList();
            asort($countryCodes);
            $data['leadOwners'] = $leadOwners;
            $leadStages = Leads_Mapper_LeadsStageMapper::getInstance()->getStages();
            asort($leadStages);
            $data['stages'] = $leadStages;
            $industries = Leads_Mapper_LeadsIndustryMapper::getInstance()->getIndustriesPairs();
            asort($industries);
            $data['industries'] = $industries;
            $leadTags = Leads_Mapper_LeadsTagsMapper::getInstance()->getTagsPairs();
            asort($leadTags);
            $data['leadTags'] = $leadTags;
            $data['leadsPredefinedValues'] = Tools_LeadScoreTools::getLeadAttributesPredefinedValuesList();
            $opportunityStages = Leads_Mapper_LeadsOpportunityStageMapper::getInstance()->getOpportunityStages();
            asort($opportunityStages);
            $data['opportunityStages'] = $opportunityStages;
            $opportunityTypes = Leads_Mapper_LeadsOpportunityTypeMapper::getInstance()->getOpportunityTypesPairs();
            asort($opportunityTypes);
            $data['opportunityTypes'] = $opportunityTypes;
            $data['cronScriptPath'] = Zend_Registry::get('website')['path'].Tools_LeadScoreTools::LEAD_SCORE_PROCESS_PATH;
            $data['cronScriptPathAudiences'] = Zend_Registry::get('website')['path'].Tools_LeadRemarketingAudiencesTools::LEAD_REMARKETING_AUDIENCES_PROCESS_PATH;
            $data['limitedAccess'] =  $this->_restrictedUserId;

            $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
            $listMasksMapper = Application_Model_Mappers_MasksListMapper::getInstance();
            $data['desktopMasks'] = $listMasksMapper->getListOfMasksByType(Application_Model_Models_MaskList::MASK_TYPE_DESKTOP);
            $data['phoneCountryCodes'] =  Tools_System_Tools::getFullCountryPhoneCodesList(true, array(), true);

            $userDefaultPhoneMobileCode = $configHelper->getConfig('userDefaultPhoneMobileCode');
            if(!empty($userDefaultPhoneMobileCode)) {
                $data['userDefaultPhoneMobileCode'] = $userDefaultPhoneMobileCode;
            }

        }

        return $data;
    }

    /**
     *
     * Resource:
     * : /api/leads/leadscorerulesattr/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {

    }

    /**
     *
     * Resource:
     * : /api/leads/leadscorerulesattr/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     * : lead form rule general config id to update
     *
     * @return JSON
     */
    public function putAction()
    {
    }

    /**
     *
     * Resource:
     * : /api/leads/leadscorerulesattr/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     *
     * @return JSON
     */
    public function deleteAction()
    {

    }

}
