<?php

class Api_Leads_Leadsformrules extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;


    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('form_name', 'form_type', 'rule_name');

    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    /**
     * User restricted id access
     *
     * @var null
     */
    protected $_restrictedUserId = null;

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);

    }


    /**
     * Get by lead form rules general config id
     *
     * Resource:
     * : /api/leads/leadsformrules/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order'), FILTER_SANITIZE_STRING);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $formName = filter_var($this->_request->getParam('formName'), FILTER_SANITIZE_STRING);
        $forGrid = (bool)filter_var($this->_request->getParam('isGrid', false), FILTER_SANITIZE_STRING);
        $detailedInfo = (bool)filter_var($this->_request->getParam('detailedInfo', false), FILTER_SANITIZE_STRING);
        $formType = filter_var($this->_request->getParam('formType', Leads_Model_LeadsFormRulesGeneralConfigModel::FORM_TYPE_INTERNAL), FILTER_SANITIZE_STRING);

        $leadsFormRulesGeneralConfigMapper = Leads_Mapper_LeadsFormRulesGeneralConfigMapper::getInstance();

        $where = null;
        $withTimezoneOffset = false;
        if ($detailedInfo || $forGrid) {
            $withTimezoneOffset = true;
        }
        if ($id && $forGrid) {
            $where = $leadsFormRulesGeneralConfigMapper->getDbTable()->getAdapter()->quoteInto('plfrgc.id = ?', $id);
            $data = $leadsFormRulesGeneralConfigMapper->fetchAll($where, null, null, null, true, true, $withTimezoneOffset);
        } elseif ($id) {
            $data = $leadsFormRulesGeneralConfigMapper->find($id);
        } elseif(!empty($formName) && !empty($formType)) {

        } elseif (!empty($forGrid)) {
            $data = $leadsFormRulesGeneralConfigMapper->fetchAll($where, $sortOrder, $limit, $offset, false, false, $withTimezoneOffset);
        }

        if (empty($data)) {
            $data = array();
        }

        if ($forGrid && !empty($detailedInfo)) {
            $ruleId = $data['id'];
            $fieldsData =  $leadsFormRulesGeneralConfigMapper->getFormFieldsByRuleId($ruleId);
            $actionsData =  $leadsFormRulesGeneralConfigMapper->getFormActionsByRuleId($ruleId);
            $data['fieldsData'] = $fieldsData;
            $data['actionsData'] = $actionsData;
            if ($data['form_type'] === Leads_Model_LeadsFormRulesGeneralConfigModel::FORM_TYPE_INTERNAL) {
                $fieldNames =  Tools_LeadFormTools::getSystemFormFields($data['form_name']);
            } else {
                $fieldNames =  Tools_LeadFormTools::getExternalFormFields($data['form_name']);
            }
            $data['formFields'] = $fieldNames;
            $data['leadOwners'] = Leads_Mapper_LeadsMapper::getInstance()->getLeadOwnersFullList();
            $data['allUsers'] = Leads_Mapper_LeadsMapper::getInstance()->getAllUsers(true);
            $currentUser = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser();
            $data['currentUserId'] = $currentUser->getId();
            $data['userTimezoneOffset'] = Tools_System_Tools::getUtcOffset('P');
            $data['userTimezone'] = $currentUser->getTimezone();
            $leadTagsDataMapper = Leads_Mapper_LeadsTagsMapper::getInstance();
            $leadStageDataMapper = Leads_Mapper_LeadsStageMapper::getInstance();
            $leadSequenceMapper = Emailsequence_Models_Mapper_EmailSequenceSettingsMapper::getInstance();
            $data['lifecycleStages'] = $leadStageDataMapper->getStages();
            $data['leadTags'] = $leadTagsDataMapper->getTagsPairs();
            $data['emailSequencies'] = $leadSequenceMapper->getSequencesPairs();
            $data['emailServices'] = Tools_LeadTools::getEnabledServices('email');
            $data['emailSequenciesArchived'] = $leadSequenceMapper->getArchivedSequences();
            $data['limitedAccess'] =  $this->_restrictedUserId;
            
            $advertisingList = Tools_LeadTools::$_advertisingTypes;
            foreach ($advertisingList as $advKey => $advAccount) {
                $services = Tools_LeadTools::sambastatsConnectionStatusGet($advKey);

                if (!empty($services['data'])) {
                    $adAccountId = $services['data']['adAccountId'];
                    if(empty($services['data']['connectionStatus'])) {
                        $adAccountId = 'disconnected';
                    }
                    $advertisingList[$advKey] .= ' (' . $adAccountId . ')';
                }
            }
            $data['advertisingList'] = $advertisingList;
            $data['cronScriptPathAudiences'] = Zend_Registry::get('website')['path'].Tools_LeadRemarketingAudiencesTools::LEAD_REMARKETING_AUDIENCES_PROCESS_PATH;
        }

        return $data;
    }

    /**
     * New lead form rule general config
     *
     * Resource:
     * : /api/leads/leadsformrules/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        if (!empty($this->_restrictedUserId)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'You don\'t have permission to add form rule'));
        }

        $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING);

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'));
        }

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
        }

        $leadsFormRulesGeneralConfigMapper = Leads_Mapper_LeadsFormRulesGeneralConfigMapper::getInstance();
        $leadFormRulesConfigMapper = Leads_Mapper_LeadsFormRulesConfigMapper::getInstance();
        $leadsFormRulesActions = Leads_Mapper_LeadsFormRulesActionsMapper::getInstance();

        $ruleName = $data['rule_name'];
        $formType = $data['form_type'];
        $formName = $data['form_name'];

        $ruleModel = $leadsFormRulesGeneralConfigMapper->checkRuleExist($formName, $formType, $ruleName);
        if ($ruleModel instanceof Leads_Model_LeadsFormRulesGeneralConfigModel) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Rule name already exists'));
        }

        $ruleModel = new Leads_Model_LeadsFormRulesGeneralConfigModel();

        $createdAt = Tools_System_Tools::convertDateFromTimezone('now');
        if (empty($data['user_id'])) {
            $currentUserId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();
        } else {
            $currentUserId = $data['user_id'];
        }

        $ruleModel->setOptions($data);
        $ruleModel->setCreatorId($currentUserId);
        $ruleModel->setCreatedAt($createdAt);

        $ruleModel = $leadsFormRulesGeneralConfigMapper->save($ruleModel);
        $ruleId = $ruleModel->getId();
        $loggedUserId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();

        if (!empty($data['fieldsData']) && is_array($data['fieldsData'])) {
            foreach ($data['fieldsData'] as $fieldData) {
                $leadFormRulesConfigModel = new Leads_Model_LeadsFormRulesConfigModel();
                $leadFormRulesConfigModel->setFieldName($fieldData['field_name']);
                $leadFormRulesConfigModel->setFieldValue($fieldData['field_value']);
                $leadFormRulesConfigModel->setRuleId($ruleId);
                $leadFormRulesConfigModel->setRuleComparisonOperator($fieldData['rule_comparison_operator']);
                $leadFormRulesConfigMapper->save($leadFormRulesConfigModel);
            }
        }

        if (!empty($data['actionsData']) && is_array($data['actionsData'])) {
            $rawActionsData = $this->getRequest()->getParam('actionsData');
            foreach ($data['actionsData'] as $index => $actionData) {
                if ($actionData['actionType'] === 'assign_task' && isset($rawActionsData[$index]['notes'])) {
                    $actionData['notes'] = $rawActionsData[$index]['notes'];
                }
                if ($actionData['actionType'] === 'assign_task' && empty($actionData['creator_id'])) {
                    $actionData['creator_id'] = $loggedUserId;
                }
                if ($actionData['actionType'] === 'assign_task' && empty($actionData['user_id'])) {
                    $actionData['user_id'] = $loggedUserId;
                }
                if($actionData['actionType'] === 'assign_email_copy') {
                    $actionData['emailsToCopy'] = Tools_LeadTools::trimValues($actionData['emailsToCopy'], true);
                }

                if ($actionData['actionType'] === 'assign_task' && empty($actionData['taskImmediate'])) {
                    $shoppingConfigMapper =  Models_Mapper_ShoppingConfig::getInstance();
                    $storeTimezone = $shoppingConfigMapper->getConfigParam('timezone');
                    if (!empty($actionData['taskTimezone'])) {
                        $timezone = $actionData['taskTimezone'];
                    } elseif(!empty($storeTimezone)) {
                        $timezone = $storeTimezone;
                    } else {
                        $timezone = 'UTC';
                    }
                    $hoursShift = Tools_EmailSequenceTools::getTimezoneShift('UTC', $timezone);
                    if ($hoursShift > 0) {
                        $hoursShift = '-' . $hoursShift;
                    } else {
                        $hoursShift = '+' . abs($hoursShift);
                    }
                    $actionData['hoursShift'] = $hoursShift;
                }

                if ($actionData['actionType'] === 'assign_remarketing_list') {
                    $actionData['lists'] = $rawActionsData[$index]['lists'];
                    $actionData['listsLabels'] = $rawActionsData[$index]['listsLabels'];
                }

                $leadsFormRulesActionModel = new Leads_Model_LeadsFormRulesActionModel();
                $leadsFormRulesActionModel->setRuleId($ruleId);
                $leadsFormRulesActionModel->setActionType($actionData['actionType']);
                unset($actionData['actionType']);
                $leadsFormRulesActionModel->setActionConfig(json_encode($actionData));
                $leadsFormRulesActions->save($leadsFormRulesActionModel);
            }
        }

        $this->_responseHelper->success(array(
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Rule has been created')
        ));

    }

    /**
     * Update lead form rule general config
     *
     * Resource:
     * : /api/leads/leadsformrules/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     * : lead form rule general config id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        if (!empty($this->_restrictedUserId)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'You don\'t have permission to edit form rule'));
        }
        parse_str($this->_request->getRawBody(), $data);
        if (!empty($data['ruleId']) && !empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {
            $translator = Zend_Registry::get('Zend_Translate');
            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();
                $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
            }
            $leadsFormRulesGeneralConfigMapper = Leads_Mapper_LeadsFormRulesGeneralConfigMapper::getInstance();
            $leadFormRulesConfigMapper = Leads_Mapper_LeadsFormRulesConfigMapper::getInstance();
            $leadsFormRulesActions = Leads_Mapper_LeadsFormRulesActionsMapper::getInstance();

            $ruleId = $data['ruleId'];
            $ruleName = $data['rule_name'];
            $formType = $data['form_type'];
            $formName = $data['form_name'];

            $leadsFormRulesGeneralConfigModel = $leadsFormRulesGeneralConfigMapper->find($ruleId);
            if (!$leadsFormRulesGeneralConfigModel instanceof Leads_Model_LeadsFormRulesGeneralConfigModel) {
                $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Rule id is missing'));
            }

            $leadsFormRulesGeneralConfigModelExisting = $leadsFormRulesGeneralConfigMapper->checkRuleExist($formName, $formType, $ruleName);
            if ($leadsFormRulesGeneralConfigModelExisting instanceof Leads_Model_LeadsFormRulesGeneralConfigModel) {
                $otherRuleId = $leadsFormRulesGeneralConfigModelExisting->getId();
                if ($otherRuleId != $ruleId) {
                    $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Rule with such name already exist'));
                }
            }

            $loggedUserId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();
            $updatedAt = Tools_System_Tools::convertDateFromTimezone('now');
            if (empty($data['editor_id'])) {
                $editorUserId = $loggedUserId;
            } else {
                $editorUserId = $data['editor_id'];
            }

            $leadsFormRulesGeneralConfigModel->setOptions($data);
            $leadsFormRulesGeneralConfigModel->setEditorId($editorUserId);
            $leadsFormRulesGeneralConfigModel->setUpdatedAt($updatedAt);

            $leadsFormRulesGeneralConfigModel = $leadsFormRulesGeneralConfigMapper->save($leadsFormRulesGeneralConfigModel);
            $ruleId = $leadsFormRulesGeneralConfigModel->getId();

            $leadFormRulesConfigMapper->deleteByRuleId($ruleId);
            $leadsFormRulesActions->deleteByRuleId($ruleId);

            if (!empty($data['fieldsData']) && is_array($data['fieldsData'])) {
                foreach ($data['fieldsData'] as $fieldData) {
                    $leadFormRulesConfigModel = new Leads_Model_LeadsFormRulesConfigModel();
                    $leadFormRulesConfigModel->setFieldName($fieldData['field_name']);
                    $leadFormRulesConfigModel->setFieldValue($fieldData['field_value']);
                    $leadFormRulesConfigModel->setRuleId($ruleId);
                    $leadFormRulesConfigModel->setRuleComparisonOperator($fieldData['rule_comparison_operator']);
                    $leadFormRulesConfigMapper->save($leadFormRulesConfigModel);
                }
            }

            if (!empty($data['actionsData']) && is_array($data['actionsData'])) {
                foreach ($data['actionsData'] as $actionData) {
                    $leadsFormRulesActionModel = new Leads_Model_LeadsFormRulesActionModel();
                    if ($actionData['actionType'] === 'assign_task' && empty($actionData['creator_id'])) {
                        $actionData['creator_id'] = $loggedUserId;
                    }
                    if ($actionData['actionType'] === 'assign_task' && empty($actionData['user_id'])) {
                        $actionData['user_id'] = $loggedUserId;
                    }
                    if($actionData['actionType'] === 'assign_email_copy') {
                        $actionData['emailsToCopy'] = Tools_LeadTools::trimValues($actionData['emailsToCopy'], true);
                    }

                    if ($actionData['actionType'] === 'assign_task' && empty($actionData['taskImmediate'])) {
                        $shoppingConfigMapper =  Models_Mapper_ShoppingConfig::getInstance();
                        $storeTimezone = $shoppingConfigMapper->getConfigParam('timezone');
                        if (!empty($actionData['taskTimezone'])) {
                            $timezone = $actionData['taskTimezone'];
                        } elseif(!empty($storeTimezone)) {
                            $timezone = $storeTimezone;
                        } else {
                            $timezone = 'UTC';
                        }
                        $hoursShift = Tools_EmailSequenceTools::getTimezoneShift('UTC', $timezone);
                        if ($hoursShift > 0) {
                            $hoursShift = '-' . $hoursShift;
                        } else {
                            $hoursShift = '+' . abs($hoursShift);
                        }
                        $actionData['hoursShift'] = $hoursShift;
                    }
                    $leadsFormRulesActionModel->setRuleId($ruleId);
                    $leadsFormRulesActionModel->setActionType($actionData['actionType']);
                    unset($actionData['actionType']);
                    $leadsFormRulesActionModel->setActionConfig(json_encode($actionData));
                    $leadsFormRulesActions->save($leadsFormRulesActionModel);
                }
            }


            $this->_responseHelper->success(array(
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Rule has been updated')
            ));
        }

    }

    /**
     * Delete lead form rule general config
     *
     * Resource:
     * : /api/leads/leadsformrules/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     * : lead form rule general config to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        if (!empty($this->_restrictedUserId)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'You don\'t have permission to delete form rule'));
        }

        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        if (!$id) {
            $this->_error();
        }
        $leadsFormRulesGeneralConfigMapper = Leads_Mapper_LeadsFormRulesGeneralConfigMapper::getInstance();
        $leadsFormRulesGeneralConfigModel = $leadsFormRulesGeneralConfigMapper->find($id);

        if ($leadsFormRulesGeneralConfigModel instanceof Leads_Model_LeadsFormRulesGeneralConfigModel) {
            $leadsFormRulesGeneralConfigMapper->delete($id);
        } else {
            $this->_error();
        }

    }

}
