<?php

class Api_Leads_Leadsgridstats extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('leadEmail');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Get different lead stats data
     *
     * Resource:
     * : /api/leads/leadsgridstats/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $leadIds = filter_var($this->_request->getParam('leadIds'), FILTER_SANITIZE_STRING);
        $dateType = filter_var($this->_request->getParam('dateType'), FILTER_SANITIZE_STRING);
        $dateFrom = filter_var($this->_request->getParam('dateFrom'), FILTER_SANITIZE_STRING);
        $dateTo = filter_var($this->_request->getParam('dateTo'), FILTER_SANITIZE_STRING);
        $leadOpportunityStages = filter_var_array($this->_request->getParam('leadOpportunityStages'), FILTER_SANITIZE_STRING);
        $leadOpportunityType = filter_var($this->_request->getParam('leadOpportunityType'), FILTER_SANITIZE_STRING);
        $leadOpportunityOwner = filter_var($this->_request->getParam('leadOpportunityOwner'), FILTER_SANITIZE_STRING);
        $leadOpportunityDataMapper = Leads_Mapper_LeadsOpportunityMapper::getInstance();
        $data = array();
        if (!empty($leadIds)) {
            $leadIds = array_filter(array_map('trim', explode(',', $leadIds)));
            $data['totalSpentInfo'] = $leadOpportunityDataMapper->getLeadTotalSales($leadIds);
            if (!empty($dateType) && (!empty($dateFrom) || !empty($dateTo))) {
                if (!empty($dateFrom)) {
                    $dateFrom = Tools_System_Tools::convertDateFromTimezone($dateFrom);
                }

                if (!empty($dateTo)) {
                    $dateTo = Tools_System_Tools::convertDateFromTimezone($dateTo .' 23:59:59');
                }

                $data['totalOpportunitiesInfo'] = $leadOpportunityDataMapper->getLeadTotalOpportunities($leadIds, true, $dateFrom, $dateTo, $dateType, $leadOpportunityType, $leadOpportunityStages, $leadOpportunityOwner);
            } else {
                $data['totalOpportunitiesInfo'] = $leadOpportunityDataMapper->getLeadTotalOpportunities($leadIds);
            }
            $currency = Zend_Registry::get('Zend_Currency');
            $data['currencySymbol'] = preg_replace('~[\w]~', '', $currency->getSymbol());
        }

        return $data;
    }

    public function postAction(){}

    public function putAction(){}

    public function deleteAction(){}


}
