<?php

class Api_Leads_Leadsms extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * System session helper
     *
     * @var null
     */
    protected $_sessionHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_sessionHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
    }


    /**
     *
     * Resource:
     * : /api/leads/leadsms/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $leadId = filter_var($this->_request->getParam('lead_id'), FILTER_SANITIZE_NUMBER_INT);

        $sequenceData = Tools_LeadTools::getEmailSequenceList();

        if (!empty($leadId)) {
            $sequenceInUse = Leads_Mapper_LeadsHasEmailSequenceMapper::getInstance()->getLeadSequenceByLeadId($leadId);
        }

        if (empty($sequenceInUse)) {
            $sequenceInUse = array();
        }

        return array(
            'sequenceData' => $sequenceData,
            'sequenceInUse' => $sequenceInUse
        );
    }

    /**
     *
     * Resource:
     * : /api/leads/leadsms/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = $this->getRequest()->getParams();
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'));
        }

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in ' . '<a href="' . $websiteUrl . 'go">here</a>'));
        }

        $leadId = $data['leadId'];

        $leadModel = Leads_Mapper_LeadsMapper::getInstance()->find($leadId);
        if (!$leadModel instanceof Leads_Model_LeadsModel) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Lead not found'));
        }

        $userId = $this->_sessionHelper->getCurrentUser()->getId();

        $storeAdminUserModel = Application_Model_Mappers_UserMapper::getInstance()->find($userId);
        if (!$storeAdminUserModel instanceof Application_Model_Models_User) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'user not authorized'));
        }

        $testSmsFlag = $this->_request->getParam('testSms');

        $receiverPhoneNumber = filter_var($this->_request->getParam('lead-call-number', ''),
            FILTER_SANITIZE_STRING);
        $receiverCountryCode = filter_var($this->_request->getParam('lead-call-country-code', ''),
            FILTER_SANITIZE_STRING);

        if(!empty($testSmsFlag)){
            $userMobileCode = $storeAdminUserModel->getMobileCountryCode();
            $userMobile = $storeAdminUserModel->getMobilePhone();

            if(!empty($userMobileCode) && !empty($userMobile)) {
                $receiverCountryCode = $userMobileCode;
                $receiverPhoneNumber = $userMobile;
            }
        }

        $countryPhoneCode = '+' . Zend_Locale::getTranslation($receiverCountryCode, 'phoneToTerritory');

        $receiverPhoneNumber = $countryPhoneCode . $receiverPhoneNumber;

        $receiverPhoneNumber = Apps_Tools_Twilio::normalizePhoneNumberToE164($receiverPhoneNumber);

        if(!empty($testSmsFlag) && empty($receiverPhoneNumber)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Can\'t send test SMS. Wrong user mobile number'));
        } elseif (empty($receiverPhoneNumber)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Wrong lead phone number'));
        }

        $emailSequenceId =  filter_var($this->_request->getParam('lead-sms-follow-up-sequences', false), FILTER_SANITIZE_NUMBER_INT);

        if (!empty($emailSequenceId) && empty($testSmsFlag)) {
            $emailSequenceMapper = Emailsequence_Models_Mapper_EmailSequenceSettingsMapper::getInstance();
            $emailSequenceModel = $emailSequenceMapper->find($emailSequenceId);
            $sequenceAppliedDate = date(Tools_System_Tools::DATE_MYSQL, strtotime('now'));
            if ($emailSequenceModel instanceof Emailsequence_Models_Models_EmailSequenceSettingsModel) {
                $leadHasEmailSequenceMapper = Leads_Mapper_LeadsHasEmailSequenceMapper::getInstance();
                $leadHasEmailSequenceModel = new Leads_Model_LeadsHasEmailSequenceModel();
                $leadHasEmailSequenceModel->setLeadId($leadId);
                $leadHasEmailSequenceModel->setSequenceId($emailSequenceId);
                $leadHasEmailSequenceModel->setSequenceDate($sequenceAppliedDate);
                $leadHasEmailSequenceMapper->save($leadHasEmailSequenceModel);
            }
        }

        $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
        $leadType = Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_SMS;
        $type = Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND;
        $createdAt = Tools_System_Tools::convertDateFromTimezone('now');
        $leadTypeId = Tools_LeadTools::getLeadTypeIdByLeadTypeName($leadType);

        if (!empty($leadTypeId) && empty($testSmsFlag)) {
            $leadModel->setLastActivityTypeId($leadTypeId);
            $leadModel->setLastActivitySubtype('');
            if ($type == Leads_Model_LeadsActionLogModel::ACTION_TYPE_INBOUND) {
                $leadModel->setLastActivity($createdAt);
            } else {
                $leadModel->setLastActivityOutbound($createdAt);
            }
            $leadMapper->save($leadModel);
        }

        $leadId = $leadModel->getId();
        $smsContent = $data['smsContent'];

        $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
        if($leadModel instanceof Leads_Model_LeadsModel) {
            $smsContent = Tools_LeadSmsTools::addDictionarySmsFields($smsContent, $leadModel, $websiteHelper->getUrl());
        }

        $subscriber['subscriber']['user'] = array(
            'phone' => array($receiverPhoneNumber),
            'message' => $smsContent,
            'owner_type' => Apps::SMS_OWNER_TYPE_USER,
            'custom_params' => array(),
            'sms_from_type' => 'crm',
            'lead_first_name' => $leadModel->getLeadFirstName(),
            'lead_last_name' => $leadModel->getLeadLastName()
        );

        Apps::apiCall('POST', 'apps', array('twilioSms'), $subscriber);

        if(empty($testSmsFlag)) {
            $actionLedLogModel = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_SMS,
                $leadId, $type, $createdAt, $leadType);
            $callLogMapper = Leads_Mapper_LeadsCallLogMapper::getInstance();
            $callLogModel = new Leads_Model_LeadsCallLogModel();
            $callLogModel->setActionLogId($actionLedLogModel->getId());
            $callLogModel->setType(Leads_Model_LeadsCallLogModel::TYPE_SMS);
            $callLogModel->setStatus('completed');
            $callLogModel->setScore('valid');
            $callLogModel->setCreatorId($userId);
            $callLogModel->setDetails($smsContent);
            $callLogModel->setLeadId($leadId);
            $callLogModel->setIsVoip('0');
            $callLogModel->setCaller('');
            $callLogModel->setReceiver($receiverPhoneNumber);
            $callLogModel->setTimestamp($createdAt);
            $callLogMapper->save($callLogModel);
        }

        $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Sms has been sent'));

    }

    /**
     *
     * Resource:
     * : /api/leads/leadsms/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     *
     * @return JSON
     */
    public function putAction()
    {

    }

    /**
     * Delete lead lead call
     *
     * Resource:
     * : /api/leads/leadsms/
     *
     * HttpMethod:
     * : DELETE
     *
     *
     * @return JSON
     */
    public function deleteAction()
    {

    }

}
