<?php

class Api_Leads_Leadsorganizationgridstats extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('leadEmail');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Get different lead stats data
     *
     * Resource:
     * : /api/leads/leadsorganizationgridstats/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $organizationIds = filter_var($this->_request->getParam('organizationIds'), FILTER_SANITIZE_STRING);
        $leadOpportunityDataMapper = Leads_Mapper_LeadsOpportunityMapper::getInstance();
        $leadOrganizationDataMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
        $data = array();
        if (!empty($organizationIds)) {
            $organizationIds = array_filter(array_map('trim', explode(',', $organizationIds)));
            $data['totalSpentInfo'] = $leadOpportunityDataMapper->getLeadOrganizationTotalSales($organizationIds);
            $data['totalOpportunitiesInfo'] = $leadOpportunityDataMapper->getLeadOrganizationTotalOpportunities($organizationIds);
            $data['interactionInfo'] = array();
            $leadTypeData = Leads_Mapper_LeadsTypeMapper::getInstance()->getTypesAndLabelsPairs();
            foreach ($organizationIds as $organizationId) {
                $lastContactedLeadInfo = $leadOrganizationDataMapper->getLastContactedLead($organizationId);
                if (!empty($lastContactedLeadInfo)) {
                    if (array_key_exists($lastContactedLeadInfo['lead_type'], $leadTypeData)) {
                        $lastContactedLeadInfo['action_type_label'] = $leadTypeData[$lastContactedLeadInfo['lead_type']];
                        $data['interactionInfo'][$organizationId] = $lastContactedLeadInfo;
                    }
                }
            }
            $currency = Zend_Registry::get('Zend_Currency');
            $data['currencySymbol'] = preg_replace('~[\w]~', '', $currency->getSymbol());
        }

        return $data;
    }

    public function postAction(){}

    public function putAction(){}

    public function deleteAction(){}


}
