<?php

class Api_Leads_Leadstimeline extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Get lead by id
     *
     * Resource:
     * : /api/leads/leadstimeline/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = 'plal.created_at DESC';
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $timeLineType = filter_var($this->_request->getParam('timelineType'), FILTER_SANITIZE_STRING);
        $leadId = filter_var($this->_request->getParam('leadId'), FILTER_SANITIZE_NUMBER_INT);
        $dateTodayEnd = Tools_System_Tools::convertDateFromTimezone(date('Y-m-d', strtotime('+1 day')). ' 00:00:00');
        $dateTodayStart = Tools_System_Tools::convertDateFromTimezone(date('Y-m-d').'00:00:00');
        $currency = Zend_Registry::get('Zend_Currency');
        $filterParams = filter_var_array($this->_request->getParam('filter', array()), FILTER_SANITIZE_STRING);

        if ($timeLineType === Tools_LeadTools::TIMELINE_TYPE_TODAY) {
            $limit = null;
        }

        $data = Tools_LeadTools::getTimelineData($limit, $offset, $sortOrder, $id, $timeLineType, $leadId, $dateTodayEnd, $dateTodayStart, $filterParams);

        if (!empty($data)) {
            $data['users'] = Leads_Mapper_LeadsMapper::getInstance()->getAllUsers(true);
            $data['currentUserId'] = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();
            $data['currencySymbol'] = preg_replace('~[\w]~', '', $currency->getSymbol());
            $moneyFormat = Tools_LeadTools::getCurrencyFormat();
            $data['moneyFormat'] = json_encode($moneyFormat);
        } else {
            $data = array();
        }

        if (!empty($data['data']) && is_array($data['data'])) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $leadMapper = Leads_Mapper_LeadsMapper::getInstance();

            foreach ($data['data'] as $key => $dataEl) {
                if (!empty($data['data'][$key]['emailUid'])) {
                    $canBeEncoded = json_encode(base64_decode($data['data'][$key]['encodedContent']));
                    if ($canBeEncoded === false) {
                        if (function_exists('utf8_decode')) {
                            $data['data'][$key]['encodedContent'] = utf8_decode(base64_decode($data['data'][$key]['encodedContent']));
                        } else {
                            $data['data'][$key]['encodedContent'] = '';
                        }
                    } else {
                        $data['data'][$key]['encodedContent'] = base64_decode($data['data'][$key]['encodedContent']);
                    }
                }

                if(!empty($data['data'][$key]['meetingDetails'])) {
                    $leadModel = $leadMapper->find($leadId);
                    $leadsData = $leadModel->toArray();
                    $organizationData = array();

                    $dictionary = array();
                    $leadsData['leadlink'] = $websiteHelper->getUrl().'crmLogin/tabName/dashboard/subTab/leads/paramHashName/lead/paramHash/'.$leadModel->getId().'/';
                    $dictionary = Tools_LeadsMailWatchdog::generateDictionary($dictionary, $leadsData, 'lead');
                    $dictionary = Tools_LeadsMailWatchdog::generateDictionary($dictionary, ['organizationName' => '', 'orglink' => $leadsData['leadlink']], 'organization');
                    if (!empty($leadsData['organizationId'])) {
                        $organizationModel = Leads_Mapper_LeadsOrganizationMapper::getInstance()->find($leadsData['organizationId']);
                        if ($organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                            $organizationData = $organizationModel->toArray();
                            $organizationData['orglink'] = $websiteHelper->getUrl() . 'crmLogin/tabName/dashboard/subTab/organizations/paramHashName/organization/paramHash/' . $organizationModel->getId() . '/';
                            $dictionary = Tools_LeadsMailWatchdog::generateDictionary($dictionary, $organizationData, 'organization');
                        }
                    }
                    $data['data'][$key]['meetingDetails'] = Tools_LeadsMailWatchdog::parseEmailLexems($data['data'][$key]['meetingDetails'], $dictionary);
                    $data['data'][$key]['meetingDetails'] = Tools_LeadTools::parseLexemsAndContent($data['data'][$key]['meetingDetails'], $leadModel);
                }
            }
        }

        $isQuizzInstalled = Tools_LeadTools::isPluginInstalled('Quizz');
        if ($isQuizzInstalled === true) {
            $isQuizzEnabled = Tools_LeadTools::isPluginEnabled('Quizz');
            if (!$isQuizzEnabled) {
                $data['data'] = $this->_removeQuizzItem($data['data']);
                $data['totalRecords'] = count($data['data']);
            }
        }

        $data['userTimezoneOffset'] = Tools_System_Tools::convertDateFromTimezone('now', 'UTC', Tools_System_Tools::getUserTimezone(), 'T');

        return $data;
    }

    protected function _removeQuizzItem($data)
    {
        $result = array();
        if (!empty($data) && is_array($data)) {
            foreach ($data as $item) {
                if ($item['type'] != 'quizz') {
                    $result[] = $item;
                }
            }
        }
        return $result;
    }

    /**
     *
     * Resource:
     * : /api/leads/leadstimeline/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
    }

    /**
     *
     * Resource:
     * : /api/leads/leadstimeline/
     *
     * HttpMethod:
     * : PUT
     *
     * @return JSON
     */
    public function putAction()
    {


    }

    /**
     *
     * Resource:
     * : /api/leads/leadstimeline/
     *
     * HttpMethod:
     * : DELETE
     *
     *
     * @return JSON
     */
    public function deleteAction()
    {

    }

}
