<?php

class Api_Leads_Leadtask extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('title', 'expired_at');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Get lead task by id
     *
     * Resource:
     * : /api/leads/leadtask/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
    }

    /**
     * Create new lead task
     *
     * Resource:
     * : /api/leads/leadtask/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING);
        $translator = Zend_Registry::get('Zend_Translate');
        $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');

        $mandatory = $this->_mandatoryParams;
        if (!empty($data['immediate'])) {
            unset($mandatory[1]);
        }

        $fieldDataMissing = array_filter($mandatory, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Please fill all missing fields'));
        }

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
        }

        $leadsTaskMapper = Leads_Mapper_LeadsTaskMapper::getInstance();
        $leadsTaskModel = new Leads_Model_LeadsTaskModel();
        $session = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
        $currentLoggedUserId = $session->getCurrentUser()->getId();
        if (empty($data['user_id'])) {
            $data['user_id'] = $currentLoggedUserId;
        }
        $data['creator_id'] = $currentLoggedUserId;

        $data['expired_at'] = Tools_System_Tools::convertDateFromTimezone($data['expired_at']);
        $data['due_date'] = Tools_System_Tools::convertDateFromTimezone($data['due_date']);
        $data['notes'] = $this->getRequest()->getParam('notes');
        $leadId = $data['lead_id'];
        $leadModel = Leads_Mapper_LeadsMapper::getInstance()->find($leadId);
        $leadsData = $leadModel->toArray();
        $organizationData = array();
        $dictionary = array();
        $leadsData['leadlink'] = $websiteHelper->getUrl().'crmLogin/tabName/dashboard/subTab/leads/paramHashName/lead/paramHash/'.$leadModel->getId().'/';
        $dictionary = Tools_LeadsMailWatchdog::generateDictionary($dictionary, $leadsData, 'lead');
        $dictionary = Tools_LeadsMailWatchdog::generateDictionary($dictionary, ['organizationName' => '', 'orglink' => $leadsData['leadlink']], 'organization');
        if (!empty($leadsData['organizationId'])) {
            $organizationModel = Leads_Mapper_LeadsOrganizationMapper::getInstance()->find($leadsData['organizationId']);
            if ($organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                $organizationData = $organizationModel->toArray();
                $organizationData['orglink'] = $websiteHelper->getUrl() . 'crmLogin/tabName/dashboard/subTab/organizations/paramHashName/organization/paramHash/' . $organizationModel->getId() . '/';
                $dictionary = Tools_LeadsMailWatchdog::generateDictionary($dictionary, $organizationData, 'organization');
            }
        }

        $data['title'] = Tools_LeadsMailWatchdog::parseEmailLexems($data['title'], $dictionary);
        $data['notes'] = Tools_LeadsMailWatchdog::parseEmailLexems($data['notes'], $dictionary);
        $data['notes'] = Tools_LeadTools::parseLexemsAndContent($data['notes'], $leadModel);

        if (!empty($data['immediate'])) {
            $expiredAt = Tools_System_Tools::convertDateFromTimezone('now');
            $data['expired_at'] = $expiredAt;
        }

        $leadsTaskModel->setOptions($data);

        $leadTaskModel = $leadsTaskMapper->save($leadsTaskModel);

        $createdAt = Tools_System_Tools::convertDateFromTimezone('now');

        $leadId = filter_var($data['lead_id'], FILTER_SANITIZE_NUMBER_INT);
        $leadModel = Leads_Mapper_LeadsMapper::getInstance()->find($leadId);
        if (!$leadModel instanceof Leads_Model_LeadsModel) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Lead doesn\'t exists'));
        }

        $actionLogExpiredModel = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_TASK, $leadId,
            Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND, $createdAt, Leads_Model_LeadsTypeModel::LEAD_OUTBOUND_TASK);

        $actionLogUpcomingModel = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_TASK, $leadId,
            Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND, $data['expired_at'], Leads_Model_LeadsTypeModel::LEAD_OUTBOUND_TASK, Leads_Model_LeadsActionLogModel::EVENT_LOG_UPCOMING);

        $leadTaskId = $leadTaskModel->getId();

        $leadsTaskLogMapper = Leads_Mapper_LeadsTasksLogMapper::getInstance();
        $leadsTaskLogModel = new Leads_Model_LeadsTasksLogModel();
        $leadsTaskLogModel->setOptions($data);
        $leadsTaskLogModel->setActionLogId($actionLogExpiredModel->getId());
        $leadsTaskLogModel->setTaskId($leadTaskId);
        $leadsTaskLogModel->setLeadId($leadId);
        $leadsTaskLogModel->setDueDate($data['due_date']);
        $leadsTaskLogModel->setIsTaskOverdue('0');
        $leadsTaskLogMapper->save($leadsTaskLogModel);


        $leadsTaskLogModel = new Leads_Model_LeadsTasksLogModel();
        $leadsTaskLogModel->setOptions($data);
        $leadsTaskLogModel->setActionLogId($actionLogUpcomingModel->getId());
        $leadsTaskLogModel->setTaskId($leadTaskId);
        $leadsTaskLogModel->setLeadId($leadId);
        $leadsTaskLogModel->setDueDate($data['due_date']);
        $leadsTaskLogModel->setIsTaskOverdue('0');
        $leadsTaskLogMapper->save($leadsTaskLogModel);

        if (!empty($data['immediateEmail']) || !empty($data['immediate'])) {
            $leadTaskModel = Leads_Mapper_LeadsTaskMapper::getInstance()->find($leadTaskId);
            if ($leadTaskModel instanceof Leads_Model_LeadsTaskModel) {
                $leadTaskModel->registerObserver(new Tools_Mail_Watchdog(array(
                    'trigger' => Tools_LeadsMailWatchdog::LEADS_TASK_CREATED,
                    'leadData' => $leadsData,
                    'organizationData' => $organizationData
                )));

                $leadTaskModel->notifyObservers();
            }
        }

        $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Lead task has been created'));
    }

    /**
     * Update lead task data
     *
     * Resource:
     * : /api/leads/leadtask/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     * : lead task id to update
     *
     * @return JSON
     */
    public function putAction()
    {

    }

    /**
     * Delete lead task
     *
     * Resource:
     * : /api/leads/leadtask/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     * : lead task id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        if (!$id) {
            $this->_error();
        }
        $leadsTaskMapper = Leads_Mapper_LeadsTaskMapper::getInstance();
        $leadTaskModel = $leadsTaskMapper->find($id);
        if ($leadTaskModel instanceof Leads_Model_LeadsTaskModel) {
            $leadsTaskMapper->delete($id);
        } else {
            $this->_error();
        }

    }

}
