<?php

class Api_Leads_Leadworkflow extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;


    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();

    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    /**
     * User restricted id access
     *
     * @var null
     */
    protected $_restrictedUserId = null;


    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
    }


    /**
     * Get by lead score rules general config id
     *
     * Resource:
     * : /api/leads/leadworkflow/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order'), FILTER_SANITIZE_STRING);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $forGrid = (bool)filter_var($this->_request->getParam('isGrid', false), FILTER_SANITIZE_STRING);
        $detailedInfo = (bool)filter_var($this->_request->getParam('detailedInfo', false), FILTER_SANITIZE_STRING);
        $leadsWorkflowGeneralConfigMapper = Leads_Mapper_LeadsWorkflowGeneralConfigMapper::getInstance();
        $customPaginate = filter_var($this->_request->getParam('customPaginate'), FILTER_SANITIZE_NUMBER_INT);

        if(!empty($customPaginate)) {
            $limit = $customPaginate;
        }
        $where = null;
        $withTimezoneOffset = false;
        if ($detailedInfo || $forGrid) {
            $withTimezoneOffset = true;
        }
        if ($id && $forGrid) {
            $where = $leadsWorkflowGeneralConfigMapper->getDbTable()->getAdapter()->quoteInto('plwgc.id = ?', $id);
            $data = $leadsWorkflowGeneralConfigMapper->fetchAll($where, null, null, null, true, true, $withTimezoneOffset);
        } elseif ($id) {
            $data = $leadsWorkflowGeneralConfigMapper->find($id);
        } elseif(!empty($formName) && !empty($formType)) {

        } elseif (!empty($forGrid)) {
            $data = $leadsWorkflowGeneralConfigMapper->fetchAll($where, $sortOrder, $limit, $offset, false, false, $withTimezoneOffset);
        }

        if (empty($data)) {
            $data = array();
        }

        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
        if ($forGrid && !empty($detailedInfo)) {
            $leadsWorkflowActionConfigMapper = Leads_Mapper_LeadsWorkflowActionsMapper::getInstance();
            $actionsData =  $leadsWorkflowActionConfigMapper->getActionByRuleIds($id);
            $data['actionsData'] = $actionsData;
        }

        if (!empty($data['data'])) {
            $data = $this->_addScoreRulesToResult($data);
        }


        if (!empty($id)) {
            $data = $this->_addScoreRulesToResult($data, $id);
        }

        if (!empty($data)) {
            $leadAttributeLegends = Tools_LeadScoreTools::getLeadAttributeLegends();
            $organizationAttributeLegends = Tools_LeadScoreTools::getOrganizationAttributeLegends();
            $actionTypesAttributeLegends = Tools_LeadScoreTools::getActionTypesAttributeLegends();
            $comparisonOperatorLegend = Tools_LeadScoreTools::getComparisonOperatorLegend();
            $leadOpportunitiesLegends = Tools_LeadScoreTools::getLeadOpportunitiesLegends();
            $leadCustomAttributesLegends = Tools_LeadScoreTools::getLeadScoreCustomAttributes(Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_LEAD_CUSTOM_ATTRIBUTES);
            $organizationCustomAttributeLegends = Tools_LeadScoreTools::getLeadScoreCustomAttributes(Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_ORGANIZATION_CUSTOM_ATTRIBUTES);
            $data['allUsers'] = Leads_Mapper_LeadsMapper::getInstance()->getAllUsers(true);
            $data['ruleSetLegends'] = array(
                'lead_attribute' => $leadAttributeLegends,
                'organization_attribute' => $organizationAttributeLegends,
                'action_type' => $actionTypesAttributeLegends,
                'comparisonOperatorLegend' => $comparisonOperatorLegend,
                'lead_opportunities' => $leadOpportunitiesLegends,
                'lead_custom_attributes' => $leadCustomAttributesLegends,
                'organization_custom_attributes' => $organizationCustomAttributeLegends,
                'custom_legends' =>
                array('quote_sent' => array('Yes' => 'The quote is sent to lead', 'No' => 'No quotes sent'),
                'quote_view' => array('Yes' => 'Lead viewed a quote', 'No' => 'No quotes viewed'),
                'registered' => array('Yes' => 'Lead was registered', 'No' => 'Lead wasn\'t registered'))
            );

            $data['leadOwners'] = $leadsMapper->getLeadOwnersFullList();
            $currentUser = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser();
            $scoreActions = Tools_LeadScoreTools::getScoreActions();
            asort($scoreActions);
            $data['currentUserId'] = $currentUser->getId();
            $data['userTimezoneOffset'] = Tools_System_Tools::getUtcOffset('P');
            $data['userTimezone'] = $currentUser->getTimezone();
            $data['scorePropertyActions'] = Tools_LeadScoreTools::getScorePropertyActions();
            $data['scoreActions'] = $scoreActions;
            $data['scoreAttributesLabels']['lead_organization_attributes'] = array_merge(Tools_LeadScoreTools::getLeadScoreLeadAttributes(),
                Tools_LeadScoreTools::getLeadScoreOrganizationAttributes(), Tools_LeadScoreTools::getLeadScoreOpportunitiesAttributes(), Tools_LeadScoreTools::getFormsLabels());
            $data['scoreAttributesLabels']['actions_attributes'] = Tools_LeadScoreTools::getAllActionAttributes();
            $data['scoreAttributesLabels'][Leads_Model_LeadsWorkflowConfigModel::RULE_TYPE_LEAD_CUSTOM_ATTRIBUTES] = Tools_LeadScoreTools::getLeadScoreCustomAttributes(Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_LEAD_CUSTOM_ATTRIBUTES);
            $data['scoreAttributesLabels'][Leads_Model_LeadsWorkflowConfigModel::RULE_TYPE_ORGANIZATION_CUSTOM_ATTRIBUTES] = Tools_LeadScoreTools::getLeadScoreCustomAttributes(Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_ORGANIZATION_CUSTOM_ATTRIBUTES);
            $customerGroups = $leadsMapper->getCustomerGroups();
            asort($customerGroups);
            $data['customerGroups'] = $customerGroups;
            $forms = Tools_LeadScoreTools::getForms(true);
            asort($forms);
            $data['forms'] = $forms;
            $countryCodes = Tools_Geo::getCountries(true);
            asort($countryCodes);
            $data['countryCodes'] = $countryCodes;
            $leadOwners = $leadsMapper->getLeadOwnersFullList();
            asort($countryCodes);
            $data['leadOwners'] = $leadOwners;
            $leadStages = Leads_Mapper_LeadsStageMapper::getInstance()->getStages();
            asort($leadStages);
            $data['stages'] = $leadStages;
            $industries = Leads_Mapper_LeadsIndustryMapper::getInstance()->getIndustriesPairs();
            asort($industries);
            $data['industries'] = $industries;
            $leadTags = Leads_Mapper_LeadsTagsMapper::getInstance()->getTagsPairs();
            asort($leadTags);
            $data['leadTags'] = $leadTags;
            $data['leadsPredefinedValues'] = Tools_LeadScoreTools::getLeadAttributesPredefinedValuesList();
            $opportunityStages = Leads_Mapper_LeadsOpportunityStageMapper::getInstance()->getOpportunityStages();
            asort($opportunityStages);
            $data['opportunityStages'] = $opportunityStages;
            $opportunityTypes = Leads_Mapper_LeadsOpportunityTypeMapper::getInstance()->getOpportunityTypesPairs();
            asort($opportunityTypes);
            $data['opportunityTypes'] = $opportunityTypes;
            $leadStageDataMapper = Leads_Mapper_LeadsStageMapper::getInstance();
            $leadSequenceMapper = Emailsequence_Models_Mapper_EmailSequenceSettingsMapper::getInstance();
            $data['lifecycleStages'] = $leadStageDataMapper->getStages();
            $data['emailSequencies'] = $leadSequenceMapper->getSequencesPairs(true);
            $emailTemplates = Leads_Mapper_LeadsMapper::getInstance()->getEmailTemplates(true);
            $data['emailTemplates'] =  $emailTemplates;
            $data['emailServices'] = Tools_LeadTools::getEnabledServices('email');
            $data['emailSequenciesArchived'] = $leadSequenceMapper->getArchivedSequences();
            $data['limitedAccess'] =  $this->_restrictedUserId;

            $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
            $listMasksMapper = Application_Model_Mappers_MasksListMapper::getInstance();
            $data['desktopMasks'] = $listMasksMapper->getListOfMasksByType(Application_Model_Models_MaskList::MASK_TYPE_DESKTOP);
            $data['phoneCountryCodes'] =  Tools_System_Tools::getFullCountryPhoneCodesList(true, array(), true);

            $userDefaultPhoneMobileCode = $configHelper->getConfig('userDefaultPhoneMobileCode');
            if(!empty($userDefaultPhoneMobileCode)) {
                $data['userDefaultPhoneMobileCode'] = $userDefaultPhoneMobileCode;
            }

            $advertisingList = Tools_LeadTools::$_advertisingTypes;
            foreach ($advertisingList as $advKey => $advAccount) {
                $services = Tools_LeadTools::sambastatsConnectionStatusGet($advKey);

                if (!empty($services['data'])) {
                    $adAccountId = $services['data']['adAccountId'];
                    if(empty($services['data']['connectionStatus'])) {
                        $adAccountId = 'disconnected';
                    }
                    $advertisingList[$advKey] .= ' (' . $adAccountId . ')';
                }
            }
            $data['advertisingList'] = $advertisingList;
            $data['cronScriptPathAudiences'] = Zend_Registry::get('website')['path'].Tools_LeadRemarketingAudiencesTools::LEAD_REMARKETING_AUDIENCES_PROCESS_PATH;

        }

        return $data;
    }

    /**
     * New lead workflow rule general config
     *
     * Resource:
     * : /api/leads/leadworkflow/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        if (!empty($this->_restrictedUserId)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'You don\'t have permission to add workflow'));
        }
        $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING);

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'));
        }

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
        }

        $leadsWorkflowGeneralConfigMapper = Leads_Mapper_LeadsWorkflowGeneralConfigMapper::getInstance();
        $leadWorkflowConfigMapper = Leads_Mapper_LeadsWorkflowConfigMapper::getInstance();
        $leadsWorkflowActions = Leads_Mapper_LeadsWorkflowActionsMapper::getInstance();

        $ruleName = $data['rule_name'];

        $ruleModel = $leadsWorkflowGeneralConfigMapper->checkRuleExist($ruleName);
        if ($ruleModel instanceof Leads_Model_LeadsWorkflowGeneralConfigModel) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Rule name already exists'));
        }

        $ruleModel = new Leads_Model_LeadsWorkflowGeneralConfigModel();

        $createdAt = Tools_System_Tools::convertDateFromTimezone('now');
        if (empty($data['user_id'])) {
            $currentUserId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();
        } else {
            $currentUserId = $data['user_id'];
        }

        $ruleModel->setOptions($data);
        $ruleModel->setCreatorId($currentUserId);
        $ruleModel->setCreatedAt($createdAt);

        $ruleModel = $leadsWorkflowGeneralConfigMapper->save($ruleModel);
        $ruleId = $ruleModel->getId();

        if (!empty($data['fieldsData']) && is_array($data['fieldsData'])) {
            foreach ($data['fieldsData'] as $fieldData) {
                if (empty($fieldData['field_name'])) {
                    continue;
                }
                $leadWorkflowConfigModel = new Leads_Model_LeadsWorkflowConfigModel();
                $leadWorkflowConfigModel->setFieldName($fieldData['field_name']);
                if (is_array($fieldData['field_value'])) {
                   $fieldValue = implode(',', array_map('trim', $fieldData['field_value']));
                } else {
                    $fieldValue = implode(',', array_map('trim', explode(',', $fieldData['field_value'])));
                }

                if($fieldData['field_name'] == Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_LEAD_PHONE || $fieldData['field_name'] == Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_TRACKING_NUMBER) {
                    if($fieldData['rule_comparison_operator'] == Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_EQUAL || $fieldData['rule_comparison_operator'] == Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_NOTEQUAL) {
                        $fieldValue = json_encode(array('country_code' => $fieldData['country_code'], $fieldData['field_name'] => $fieldValue));
                    }
                }

                $leadWorkflowConfigModel->setFieldValue($fieldValue);
                $leadWorkflowConfigModel->setRuleId($ruleId);
                $leadWorkflowConfigModel->setRuleComparisonOperator($fieldData['rule_comparison_operator']);
                $leadWorkflowConfigModel->setRuleSubtype($fieldData['rule_subtype']);
                $leadWorkflowConfigModel->setRuleType($fieldData['rule_type']);
                $leadWorkflowConfigMapper->save($leadWorkflowConfigModel);
            }
        }

        if (!empty($data['actionsData']) && is_array($data['actionsData'])) {
            $rawActionsData = $this->getRequest()->getParam('actionsData');
            foreach ($data['actionsData'] as $index => $actionData) {
                if ($actionData['actionType'] === 'assign_task' && isset($rawActionsData[$index]['notes'])) {
                    $actionData['notes'] = $rawActionsData[$index]['notes'];
                }
                if ($actionData['actionType'] === 'assign_task' && empty($actionData['creator_id'])) {
                    $actionData['creator_id'] = $currentUserId;
                }
                if ($actionData['actionType'] === 'assign_task' && empty($actionData['user_id'])) {
                    $actionData['user_id'] = $currentUserId;
                }
                if($actionData['actionType'] === 'assign_email_copy') {
                    $actionData['emailsToCopy'] = Tools_LeadTools::trimValues($actionData['emailsToCopy'], true);
                }

                if ($actionData['actionType'] === 'assign_task' && empty($actionData['taskImmediate'])) {
                    $shoppingConfigMapper =  Models_Mapper_ShoppingConfig::getInstance();
                    $storeTimezone = $shoppingConfigMapper->getConfigParam('timezone');
                    if (!empty($actionData['taskTimezone'])) {
                        $timezone = $actionData['taskTimezone'];
                    } elseif(!empty($storeTimezone)) {
                        $timezone = $storeTimezone;
                    } else {
                        $timezone = 'UTC';
                    }
                    $hoursShift = Tools_EmailSequenceTools::getTimezoneShift('UTC', $timezone);
                    if ($hoursShift > 0) {
                        $hoursShift = '-' . $hoursShift;
                    } else {
                        $hoursShift = '+' . abs($hoursShift);
                    }
                    $actionData['hoursShift'] = $hoursShift;
                }

                if ($actionData['actionType'] === 'assign_remarketing_list') {
                    $actionData['lists'] = $rawActionsData[$index]['lists'];
                    $actionData['listsLabels'] = $rawActionsData[$index]['listsLabels'];
                }

                $leadsWorkflowActionModel = new Leads_Model_LeadsWorkflowActionModel();
                $leadsWorkflowActionModel->setRuleId($ruleId);
                $leadsWorkflowActionModel->setActionType($actionData['actionType']);
                unset($actionData['actionType']);
                $leadsWorkflowActionModel->setActionConfig(json_encode($actionData));
                $leadsWorkflowActions->save($leadsWorkflowActionModel);
            }
        }

        $this->_responseHelper->success(array(
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Rule has been created')
        ));

    }

    /**
     * Update lead workflow rule general config
     *
     * Resource:
     * : /api/leads/leadworkflow/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     * : lead score rule general config id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        if (!empty($this->_restrictedUserId)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'You don\'t have permission to edit workflow'));
        }

        parse_str($this->_request->getRawBody(), $data);
        if (!empty($data['ruleId']) && !empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {
            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();
                $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
            }
            $leadsWorkflowGeneralConfigMapper = Leads_Mapper_LeadsWorkflowGeneralConfigMapper::getInstance();
            $leadWorkflowConfigMapper = Leads_Mapper_LeadsWorkflowConfigMapper::getInstance();
            $leadsWorkflowActions = Leads_Mapper_LeadsWorkflowActionsMapper::getInstance();

            $ruleId = $data['ruleId'];
            $ruleName = $data['rule_name'];

            $leadsWorkflowGeneralConfigModel = $leadsWorkflowGeneralConfigMapper->find($ruleId);
            if (!$leadsWorkflowGeneralConfigModel instanceof Leads_Model_LeadsWorkflowGeneralConfigModel) {
                $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Rule id is missing'));
            }

            $leadsWorkflowGeneralConfigModelExisting = $leadsWorkflowGeneralConfigMapper->checkRuleExist($ruleName);
            if ($leadsWorkflowGeneralConfigModelExisting instanceof Leads_Model_LeadsScoreRulesGeneralConfigModel) {
                $otherRuleId = $leadsWorkflowGeneralConfigModelExisting->getId();
                if ($otherRuleId != $ruleId) {
                    $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Rule with such name already exist'));
                }
            }

            $loggedUserId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();
            $updatedAt = Tools_System_Tools::convertDateFromTimezone('now');
            if (empty($data['editor_id'])) {
                $editorUserId = $loggedUserId;
            } else {
                $editorUserId = $data['editor_id'];
            }

            $leadWorkflowConfigMapper->deleteByRuleId($ruleId);
            $leadsWorkflowActions->deleteByRuleId($ruleId);

            if (!empty($data['fieldsData']) && is_array($data['fieldsData'])) {
                foreach ($data['fieldsData'] as $fieldData) {
                    if(!empty($fieldData['field_name']) && !empty($fieldData['rule_type'])) {
                        $leadWorkflowConfigModel = new Leads_Model_LeadsWorkflowConfigModel();
                        $leadWorkflowConfigModel->setFieldName($fieldData['field_name']);
                        if (is_array($fieldData['field_value'])) {
                            $fieldValue = implode(',', array_map('trim', $fieldData['field_value']));
                        } else {
                            $fieldValue = implode(',', array_map('trim', explode(',', $fieldData['field_value'])));
                        }

                        if($fieldData['field_name'] == Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_LEAD_PHONE || $fieldData['field_name'] == Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_TRACKING_NUMBER) {
                            if($fieldData['rule_comparison_operator'] == Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_EQUAL || $fieldData['rule_comparison_operator'] == Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_NOTEQUAL) {
                                $fieldValue = json_encode(array('country_code' => $fieldData['country_code'], $fieldData['field_name'] => $fieldValue));
                            }
                        }

                        $leadWorkflowConfigModel->setFieldValue($fieldValue);
                        $leadWorkflowConfigModel->setRuleId($ruleId);
                        $leadWorkflowConfigModel->setRuleType($fieldData['rule_type']);
                        $leadWorkflowConfigModel->setRuleSubtype($fieldData['rule_subtype']);
                        $leadWorkflowConfigModel->setRuleComparisonOperator($fieldData['rule_comparison_operator']);

                        $leadWorkflowConfigMapper->save($leadWorkflowConfigModel);
                    } else {
                        $leadsWorkflowGeneralConfigModel->setRuleName($ruleName);
                        $leadsWorkflowGeneralConfigModel->setScore($fieldData['field_value']);
                        $leadsWorkflowGeneralConfigModel->setScoreComparisonOperator($fieldData['rule_comparison_operator']);

                        $leadsWorkflowGeneralConfigModel->setEditorId($editorUserId);
                        $leadsWorkflowGeneralConfigModel->setUpdatedAt($updatedAt);

                        $leadsWorkflowGeneralConfigModel = $leadsWorkflowGeneralConfigMapper->save($leadsWorkflowGeneralConfigModel);
                    }
                }
            }

            if (!empty($data['actionsData']) && is_array($data['actionsData'])) {
                foreach ($data['actionsData'] as $actionData) {
                    $leadsWorkflowActionModel = new Leads_Model_LeadsWorkflowActionModel();
                    if ($actionData['actionType'] === 'assign_task' && empty($actionData['creator_id'])) {
                        $actionData['creator_id'] = $loggedUserId;
                    }
                    if ($actionData['actionType'] === 'assign_task' && empty($actionData['user_id'])) {
                        $actionData['user_id'] = $loggedUserId;
                    }
                    if($actionData['actionType'] === 'assign_email_copy') {
                        $actionData['emailsToCopy'] = Tools_LeadTools::trimValues($actionData['emailsToCopy'], true);
                    }

                    if ($actionData['actionType'] === 'assign_task' && empty($actionData['taskImmediate'])) {
                        $shoppingConfigMapper =  Models_Mapper_ShoppingConfig::getInstance();
                        $storeTimezone = $shoppingConfigMapper->getConfigParam('timezone');
                        if (!empty($actionData['taskTimezone'])) {
                            $timezone = $actionData['taskTimezone'];
                        } elseif(!empty($storeTimezone)) {
                            $timezone = $storeTimezone;
                        } else {
                            $timezone = 'UTC';
                        }
                        $hoursShift = Tools_EmailSequenceTools::getTimezoneShift('UTC', $timezone);
                        if ($hoursShift > 0) {
                            $hoursShift = '-' . $hoursShift;
                        } else {
                            $hoursShift = '+' . abs($hoursShift);
                        }
                        $actionData['hoursShift'] = $hoursShift;
                    }
                    $leadsWorkflowActionModel->setRuleId($ruleId);
                    $leadsWorkflowActionModel->setActionType($actionData['actionType']);
                    unset($actionData['actionType']);
                    $leadsWorkflowActionModel->setActionConfig(json_encode($actionData));
                    $leadsWorkflowActions->save($leadsWorkflowActionModel);
                }
            }

            $reprocessLeadWorkflowRule = $data['reprocessLeadWorkflowRule'];

            if (!empty($reprocessLeadWorkflowRule)) {
                $leadsWorkflowRulesLogMapper = Leads_Mapper_LeadsWorkflowRulesLogMapper::getInstance();
                $leadsWorkflowRulesLogModel = $leadsWorkflowRulesLogMapper->findByRuleId($ruleId);
                if ($leadsWorkflowRulesLogModel instanceof Leads_Model_LeadsWorkflowRulesLogModel) {
                    $leadsWorkflowRulesLogMapper->deleteByRuleId($ruleId);
                }
            }

            $this->_responseHelper->success(array(
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Rule has been updated')
            ));
        }

    }

    /**
     * Delete lead workflow rule general config
     *
     * Resource:
     * : /api/leads/leadworkflow/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     * : lead workflow rule general config to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        if (!empty($this->_restrictedUserId)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'You don\'t have permission to delete workflow'));
        }

        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        if (!$id) {
            $this->_error();
        }
        $leadsWorkflowGeneralConfigMapper = Leads_Mapper_LeadsWorkflowGeneralConfigMapper::getInstance();
        $leadsWorkflowGeneralConfigModel = $leadsWorkflowGeneralConfigMapper->find($id);

        if ($leadsWorkflowGeneralConfigModel instanceof Leads_Model_LeadsWorkflowGeneralConfigModel) {
            $leadsWorkflowGeneralConfigMapper->delete($id);
        } else {
            $this->_error();
        }

    }

    /**
     * @param array $data data
     * @param int $id score id
     * @return mixed
     */
    protected function _addScoreRulesToResult($data, $id = 0)
    {
        $leadsWorkflowConfigMapper = Leads_Mapper_LeadsWorkflowConfigMapper::getInstance();
        if (!empty($id)) {
            $scoreRules = $leadsWorkflowConfigMapper->getScoreRules($id);
            foreach ($scoreRules as $scoreRule) {
                if($scoreRule['field_name'] == Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_LEAD_PHONE || $scoreRule['field_name'] == Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_TRACKING_NUMBER) {
                    if($scoreRule['rule_comparison_operator'] == Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_EQUAL || $scoreRule['rule_comparison_operator'] == Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_NOTEQUAL) {
                        $scoreRule['field_value'] = json_decode($scoreRule['field_value']);
                    }
                }

                $data['rulesData'][] = $scoreRule;
            }
        } else {
            $scoreRules = $leadsWorkflowConfigMapper->getScoreRules();
            foreach ($data['data'] as $recordNumber => $singleDataResult) {
                $keys = preg_grep('~^' . $singleDataResult['id'] . '$~ui', array_column($scoreRules, 'rule_id'));
                if (!empty($keys)) {
                    foreach ($keys as $key => $val) {
                        $data['data'][$recordNumber]['rulesData'][] = $scoreRules[$key];
                    }
                }
            }
        }

        return $data;
    }

}
