<?php

class Api_Leads_Opportunitytimeline extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Get lead opportunity by id
     *
     * Resource:
     * : /api/leads/opportunitytimeline/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $leadId = filter_var($this->_request->getParam('lead_id'), FILTER_SANITIZE_NUMBER_INT);
        $currency = Zend_Registry::get('Zend_Currency');
        $leadOpportunityDataMapper = Leads_Mapper_LeadsOpportunityMapper::getInstance();
        $filterBy = filter_var_array($this->_request->getParam('filter'), FILTER_SANITIZE_STRING);
        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order'), FILTER_SANITIZE_STRING);
        $data = array();
        if (!empty($leadId)) {
            $where = $leadOpportunityDataMapper->getDbTable()->getAdapter()->quoteInto('plo.lead_id = ?', $leadId);
            if (!empty($filterBy['filter-by-opportunity-stage'])) {
                $where .= ' AND '. $leadOpportunityDataMapper->getDbTable()->getAdapter()->quoteInto('plos.id = ?', $filterBy['filter-by-opportunity-stage']);
            }
            $data = $leadOpportunityDataMapper->getOpportunitiesForTimeline($where, $sortOrder, $limit, $offset, false, false, 'plo.id');
            if (!empty($data['data'])) {
                foreach ($data['data'] as $key => $opportunityInfo) {
                    $where = $leadOpportunityDataMapper->getDbTable()->getAdapter()->quoteInto('plo.lead_id = ?', $leadId);
                    $where .= ' AND '. $leadOpportunityDataMapper->getDbTable()->getAdapter()->quoteInto('ploq.opportunity_id = ?', $opportunityInfo['opportunity_id']);
                    $data['data'][$key]['opportunityQuoteInfo'] = $leadOpportunityDataMapper->getOpportunitiesForTimeline($where, null, null, null, true, false);
                }
            }

            $leadOpportunityDataMapper = Leads_Mapper_LeadsOpportunityStageMapper::getInstance();
            $data['opportunityStages'] = $leadOpportunityDataMapper->getOpportunityStages();
            $data['users'] = Leads_Mapper_LeadsMapper::getInstance()->getLeadOwnersFullList();
            $data['currentUserId'] = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();
            $data['currencySymbol'] = preg_replace('~[\w]~', '', $currency->getSymbol());
            $moneyFormat = Tools_LeadTools::getCurrencyFormat();
            $data['moneyFormat'] = json_encode($moneyFormat);
            $data['quoteTemplates'] = array_merge(
                array(0 => 'Select quote template'),
                Tools_System_Tools::getTemplatesHash(Quote_Models_Model_Quote::TEMPLATE_TYPE_QUOTE));
        }

        return $data;
    }


    public function postAction()
    {

    }
    public function putAction()
    {

    }

    /**
     * Delete lead opportunity
     *
     * Resource:
     * : /api/leads/opportunitytimeline/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     * : lead opportunity id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        if (empty($id)) {
            $this->_error();
        }
        $leadsOpportunityMapper = Leads_Mapper_LeadsOpportunityMapper::getInstance();
        $leadOpportunityModel = $leadsOpportunityMapper->find($id);
        if ($leadOpportunityModel instanceof Leads_Model_LeadsOpportunityModel) {
            $leadOpportunityModel->setDescription('');
            $leadsOpportunityMapper->save($leadOpportunityModel);
            $translator = Zend_Registry::get('Zend_Translate');
            $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Opportunity has been deleted'));
        }

        $this->_error();

    }

}
