<?php

class Api_Leads_Organizationcustomparams extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('param_id', 'organization_id', 'secureToken');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Get organization custom param by id
     *
     * Resource:
     * : /api/leads/organizationcustomparams/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order'), FILTER_SANITIZE_STRING);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $organizationId = filter_var($this->_request->getParam('organizationId'), FILTER_SANITIZE_NUMBER_INT);
        $grid = filter_var($this->_request->getParam('grid'), FILTER_SANITIZE_STRING);
        $organizationCustomTabId = filter_var($this->_request->getParam('customTabId'), FILTER_SANITIZE_NUMBER_INT);

        $organizationCustomParamMapper = Leads_Mapper_LeadsOrganizationCustomParamsMapper::getInstance();
        if ($id) {
            $where = $organizationCustomParamMapper->getDbTable()->getAdapter()->quoteInto('ploccpd.id = ?', $id);
            $data = $organizationCustomParamMapper->fetchAll($where);
        } elseif(!empty($grid) && !empty($organizationId)) {
            $configData = $organizationCustomParamMapper->fetchAllData(null, $sortOrder, $limit, $offset);
            $paramsData = $organizationCustomParamMapper->getParamsForGrid($organizationId);

            if(!empty($organizationCustomTabId)) {
                $leadsCustomTabsHasCustomParamsMapper = Leads_Mapper_LeadsCustomTabsHasCustomParamsMapper::getInstance();
                $selectedCustomParamsData = $leadsCustomTabsHasCustomParamsMapper->getSelectedCustomTabsParams($organizationCustomTabId, Api_Leads_Leadcustomtabsfields::CUSTOM_TAB_TYPE_ORGANIZATION);
            }

            if (!empty($configData['data'])) {
                foreach ($configData['data'] as $key => $confData) {
                    $paramId = $confData['param_id'];

                    if(!empty($organizationCustomTabId)) {
                        $configData['data'][$key]['visible'] = false;
                        if(!empty($selectedCustomParamsData)) {
                            if(array_key_exists($paramId, $selectedCustomParamsData)) {
                                $configData['data'][$key]['visible'] = true;
                            }
                        }
                    }

                    if (array_key_exists($paramId, $paramsData)) {
                        $configData['data'][$key]['param_value'] = $paramsData[$paramId]['param_value'];
                        $configData['data'][$key]['id'] =  $paramsData[$paramId]['id'];
                    } else {
                        $configData['data'][$key]['param_value'] = null;
                        $configData['data'][$key]['id'] =  null;
                    }
                }
                $data = $configData;
            } else {
                $data = array();
            }

        } else {
            $data = $organizationCustomParamMapper->fetchAll(null, $sortOrder, $limit, $offset);
        }

        return $data;
    }

    /**
     * Create new organization custom param config
     *
     * Resource:
     * : /api/leads/organizationcustomparams/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {

        $data = json_decode($this->_request->getRawBody(), true);
        if(empty($data)) {
            $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING);
        }
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'));
        }

        $secureToken = $data['secureToken'];
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
        }

        if($data['param_type'] == Tools_LeadImportTools::ATTRIBUTE_TYPE_DATE){
            $defaultTimezone = 'UTC';
            $currentTime = Tools_System_Tools::convertDateFromTimezone('now', $defaultTimezone, Tools_System_Tools::getUserTimezone(), 'H:i:s');
            $formDate = Tools_System_Tools::convertDateFromTimezone($data['param_value'], $defaultTimezone, Tools_System_Tools::getUserTimezone(), 'Y-m-d ') . $currentTime;

            $data['param_value'] = $formDate;
        }

        if($data['param_type'] == Tools_LeadImportTools::ATTRIBUTE_TYPE_SELECT || $data['param_type'] == Tools_LeadImportTools::ATTRIBUTE_TYPE_CHECKBOX || $data['param_type'] == Tools_LeadImportTools::ATTRIBUTE_TYPE_RADIO){
            $leadOrganizationCustomParamOptionsDataMapper = Leads_Mapper_LeadsOrganizationCustomParamsOptionsDataMapper::getInstance();

            $leadOrganizationCustomParamOptionsData = $leadOrganizationCustomParamOptionsDataMapper->findOptionIdByCustomParamId($data['param_id'], $data['param_value']);

            $data['params_option_id'] = $leadOrganizationCustomParamOptionsData->getId();
        }

        $organizationCustomParamMapper = Leads_Mapper_LeadsOrganizationCustomParamsMapper::getInstance();

        $organizationCustomParamModel = new Leads_Model_LeadsOrganizationCustomParamsModel();
        $organizationCustomParamModel->setOptions($data);

        $organizationCustomParamModel = $organizationCustomParamMapper->save($organizationCustomParamModel);

        $this->_responseHelper->success(array('message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Saved'), 'id' => $organizationCustomParamModel->getId()));
    }

    /**
     * Update organization custom param config
     *
     * Resource:
     * : /api/leads/organizationcustomparams/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     * : organization custom param id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if (!empty($data['id']) && !empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {
            $translator = Zend_Registry::get('Zend_Translate');
            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();
                $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
            }
            $customParamId = filter_var($data['id'], FILTER_SANITIZE_NUMBER_INT);
            $organizationCustomParamMapper = Leads_Mapper_LeadsOrganizationCustomParamsMapper::getInstance();
            $organizationCustomParamModel = $organizationCustomParamMapper->find($customParamId);

            if (!$organizationCustomParamModel instanceof Leads_Model_LeadsOrganizationCustomParamsModel) {
                $this->_responseHelper->fail($translator->translate(Leads::TRANSLATION_PREFIX.'Param doesn\'t exists'));
            }

            $notValid = Tools_LeadTools::customParamValidate($data['param_value'], '~[^\'\s\w-.]~ui');

            if($notValid) {
                $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Invalid param name. You can use only alphabet and digits. You can also use "-".'));
            }

            if($data['param_type'] == Tools_LeadImportTools::ATTRIBUTE_TYPE_DATE){
                if(!empty($data['param_value'])) {
                    $defaultTimezone = 'UTC';
                    $currentTime = Tools_System_Tools::convertDateFromTimezone('now', $defaultTimezone, Tools_System_Tools::getUserTimezone(), 'H:i:s');
                    $formDate = Tools_System_Tools::convertDateFromTimezone($data['param_value'], $defaultTimezone, Tools_System_Tools::getUserTimezone(), 'Y-m-d ') . $currentTime;

                    $data['param_value'] = $formDate;
                }
            }

            if($data['param_type'] == Tools_LeadImportTools::ATTRIBUTE_TYPE_SELECT || $data['param_type'] == Tools_LeadImportTools::ATTRIBUTE_TYPE_CHECKBOX || $data['param_type'] == Tools_LeadImportTools::ATTRIBUTE_TYPE_RADIO){
                $leadOrganizationCustomParamOptionsDataMapper = Leads_Mapper_LeadsOrganizationCustomParamsOptionsDataMapper::getInstance();

                $leadOrganizationCustomParamOptionsData = $leadOrganizationCustomParamOptionsDataMapper->findOptionIdByCustomParamId($data['param_id'], $data['param_value']);

                if($leadOrganizationCustomParamOptionsData instanceof Leads_Model_LeadsOrganizationCustomParamsOptionsDataModel){
                    $data['params_option_id'] = $leadOrganizationCustomParamOptionsData->getId();
                }else{
                    $organizationCustomParamModel->setParamsOptionId(null);
                }
            }

            $organizationCustomParamModel->setOptions($data);
            $organizationCustomParamMapper->save($organizationCustomParamModel);
            $this->_responseHelper->success(array('message' =>$translator->translate(Leads::TRANSLATION_PREFIX.'Saved')));
        }

    }

    /**
     * Delete organization custom param
     *
     * Resource:
     * : /api/leads/organizationcustomparams/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     * : organization custom param id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        if (!$id) {
            $this->_error();
        }
        $organizationCustomParamMapper = Leads_Mapper_LeadsOrganizationCustomParamsMapper::getInstance();
        $organizationCustomParamModel = $organizationCustomParamMapper->find($id);
        if ($organizationCustomParamModel instanceof Leads_Model_LeadsOrganizationCustomParamsModel) {
            $organizationCustomParamMapper->delete($id);
        } else {
            $this->_error();
        }

    }

}
