<?php

class Api_Leads_Organizationcustomparamsconfig extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('label', 'param_type', 'param_name');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Get organization custom param config by id
     *
     * Resource:
     * : /api/leads/organizationcustomparamsconfig/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order', 'plocpc.param_name'), FILTER_SANITIZE_STRING);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);

        $organizationCustomParamConfigMapper = Leads_Mapper_LeadsOrganizationCustomParamsConfigMapper::getInstance();
        if ($id) {
            $where = $organizationCustomParamConfigMapper->getDbTable()->getAdapter()->quoteInto('plocpc.id = ?', $id);
            $data = $organizationCustomParamConfigMapper->fetchAll($where);
        } else {
            $data = $organizationCustomParamConfigMapper->fetchAll(null, $sortOrder, $limit, $offset);
        }

        return $data;
    }

    /**
     * Create new organization custom param config
     *
     * Resource:
     * : /api/leads/organizationcustomparamsconfig/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = $this->getRequest()->getParams();
        $translator = Zend_Registry::get('Zend_Translate');
        $dropdownOptionValues = array();
        $dropdownOptionIds = array();

        if(!empty($data['custom_param_name'])){
            $dropdownOptionValues = array_unique(explode(',' , $data['custom_param_name']));

            if(!empty($dropdownOptionValues)) {
                foreach ($dropdownOptionValues as $value) {
                    $notValid = Tools_LeadTools::customParamValidate($value);

                    if($notValid) {
                        $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Invalid param name. You can use only alphabet and digits. You can also use "-".'));
                    }
                }
            }
        }

        if(!empty($data['custom_param_options_ids'])){
            $dropdownOptionIds = array_unique(explode(',' , $data['custom_param_options_ids']));
        }

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'));
        }

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
        }

        if(preg_match('~[^\w-]~ui', $data['param_name'])) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Invalid param name. You can use only alphabet and digits. You can also use "-". White Spaces not allowed'));
        }

        $organizationCustomParamConfigMapper = Leads_Mapper_LeadsOrganizationCustomParamsConfigMapper::getInstance();

        if(!empty($data['custom_param_id'])){
            $organizationCustomParamConfigModel = $organizationCustomParamConfigMapper->findById($data['custom_param_id']);
            if (!$organizationCustomParamConfigModel instanceof Leads_Model_LeadsOrganizationCustomParamsConfigModel) {
                $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Custom param with such name already exists'));
            }
        }else{
            $organizationCustomParamConfigModel = $organizationCustomParamConfigMapper->getByName($data['param_name']);
            if ($organizationCustomParamConfigModel instanceof Leads_Model_LeadsOrganizationCustomParamsConfigModel) {
                $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Custom param with such name already exists'));
            }

            $organizationCustomParamConfigModel = new Leads_Model_LeadsOrganizationCustomParamsConfigModel();
        }

        $organizationCustomParamConfigModel->setOptions($data);
        $organizationCustomParamConfigMapper->save($organizationCustomParamConfigModel);

        if ($organizationCustomParamConfigModel instanceof Leads_Model_LeadsOrganizationCustomParamsConfigModel) {
            $organizationCustomParamOptionsDataMapper = Leads_Mapper_LeadsOrganizationCustomParamsOptionsDataMapper::getInstance();

            $organizationCustomParamId = $organizationCustomParamConfigModel->getId();

            if ($data['param_type'] == Tools_LeadImportTools::ATTRIBUTE_TYPE_CHECKBOX) {
                $defCheckboxValues = array();

                $notValid = Tools_LeadTools::customParamValidate($data['checkbox_yes']);

                if($notValid) {
                    $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Invalid param name. You can use only alphabet and digits. You can also use "-".'));
                }

                array_push($defCheckboxValues, $data['checkbox_yes']);

                if (empty($data['checkbox_yes'])) {
                    $defCheckboxValues[0] = 'yes';
                }

                $data = $organizationCustomParamOptionsDataMapper->findByCustomParamId($organizationCustomParamId);

                Tools_LeadTools::processCustomParamsOptions($data, $defCheckboxValues, Leads::ORGANIZATIONS_TYPE, $organizationCustomParamId);
            } elseif ($data['param_type'] == Tools_LeadImportTools::ATTRIBUTE_TYPE_SELECT || $data['param_type'] == Tools_LeadImportTools::ATTRIBUTE_TYPE_RADIO) {
                $data = $organizationCustomParamOptionsDataMapper->findByCustomParamId($organizationCustomParamId);

                foreach ($data as $key => $customParam) {
                    $customParamId = $customParam->getId();
                    if(in_array($customParamId, $dropdownOptionIds)) {
                        $optionValue = $dropdownOptionValues[$key];
                        $data[$key]->setOptionValue($optionValue);
                    }
                }

                Tools_LeadTools::processCustomParamsOptions($data, $dropdownOptionValues, Leads::ORGANIZATIONS_TYPE, $organizationCustomParamId);
            }
        }

        $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Custom param has been created'));
    }

    /**
     * Update organization custom param config
     *
     * Resource:
     * : /api/leads/organizationcustomparamsconfig/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     * : organization custom param id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if (!empty($data['id']) && !empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {
            $translator = Zend_Registry::get('Zend_Translate');
            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();
                $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
            }
            $customParamId = filter_var($data['id'], FILTER_SANITIZE_NUMBER_INT);
            $organizationCustomParamConfigMapper = Leads_Mapper_LeadsOrganizationCustomParamsConfigMapper::getInstance();
            $organizationCustomParamConfigModel = $organizationCustomParamConfigMapper->find($customParamId);
            if (!$organizationCustomParamConfigModel instanceof Leads_Model_LeadsOrganizationCustomParamsConfigModel) {
                $this->_responseHelper->fail($translator->translate(Leads::TRANSLATION_PREFIX.'Config doesn\'t exists'));
            }

            $oldParamName = $organizationCustomParamConfigModel->getParamName();

            if(preg_match('~[^\w-]~ui', $data['param_name'])) {
                $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Invalid param name. You can use only alphabet and digits. You can also use "-". White Spaces not allowed'));
            }
            $currentParamName = $data['param_name'];
            if ($oldParamName !== $currentParamName) {
                $validateTypeExists = new Zend_Validate_Db_RecordExists(array(
                    'table' => 'plugin_leads_lead_custom_params_config',
                    'field' => 'param_name'
                ));
                if ($validateTypeExists->isValid($currentParamName)) {
                    $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'You have another custom param with such name'));
                }
            }

            $organizationCustomParamConfigModel->setOptions($data);
            $organizationCustomParamConfigMapper->save($organizationCustomParamConfigModel);
            $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Custom param has been updated'));
        }

    }

    /**
     * Delete organization custom param config
     *
     * Resource:
     * : /api/leads/organizationcustomparamsconfig/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     * : organization custom param id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $customDropdownSelectionFlag = $this->_request->getParam('customDropdownSelectionFlag');
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);

        if (!$id) {
            $this->_error();
        }

        if(!empty($customDropdownSelectionFlag)){
            $organizationCustomParamOptionsMapper = Leads_Mapper_LeadsOrganizationCustomParamsOptionsDataMapper::getInstance();
            $organizationCustomParamOptionsModel = $organizationCustomParamOptionsMapper->find($id);
            if ($organizationCustomParamOptionsModel instanceof Leads_Model_LeadsOrganizationCustomParamsOptionsDataModel) {
                $organizationCustomParamOptionsMapper->delete($id);
            } else {
                $this->_error();
            }
        }else{
            $organizationCustomParamConfigMapper = Leads_Mapper_LeadsOrganizationCustomParamsConfigMapper::getInstance();
            $organizationCustomParamConfigModel = $organizationCustomParamConfigMapper->find($id);
            if ($organizationCustomParamConfigModel instanceof Leads_Model_LeadsOrganizationCustomParamsConfigModel) {
                $organizationCustomParamConfigMapper->delete($id);
            } else {
                $this->_error();
            }
        }
    }

}
