<?php

class Api_Leads_Organizationstimeline extends Api_Service_Abstract
{

    /**
     * Current day log
     */
    const TIMELINE_TYPE_TODAY = 'today';

    /**
     * Upcoming events
     */
    const TIMELINE_TYPE_UPCOMING = 'upcoming';

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Get lead by id
     *
     * Resource:
     * : /api/leads/organizationstimeline/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = 'plal.created_at DESC';
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $timeLineType = filter_var($this->_request->getParam('timelineType'), FILTER_SANITIZE_STRING);
        $organizationId = filter_var($this->_request->getParam('organizationId'), FILTER_SANITIZE_NUMBER_INT);
        $dateTodayEnd = Tools_System_Tools::convertDateFromTimezone(date('Y-m-d', strtotime('+1 day')). ' 00:00:00');
        $dateTodayStart = Tools_System_Tools::convertDateFromTimezone(date('Y-m-d').'00:00:00');
        $currency = Zend_Registry::get('Zend_Currency');
        $filterParams = filter_var_array($this->_request->getParam('filter', array()), FILTER_SANITIZE_STRING);
        $leadId = filter_var_array($this->_request->getParam('leadId', array()), FILTER_SANITIZE_NUMBER_INT);

        $organizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
        $leadsOrganizations = $organizationMapper->getOrganizationLeads($organizationId);

        $data = array();

        if(!empty($leadsOrganizations)){
            if(!empty($leadId)){
                $filteredLeadsOrganizations = array();
                foreach ($leadId as $lId){
                    if(array_key_exists($lId, $leadsOrganizations)){
                        $filteredLeadsOrganizations[$lId] = $leadsOrganizations[$lId];
                    }
                }
                if(!empty($filteredLeadsOrganizations)){
                    $leadsOrganizations = $filteredLeadsOrganizations;
                }
            }

            $leadsOrganizationsIds = array_keys($leadsOrganizations);

            $timelineData = Tools_LeadTools::getTimelineData($limit, $offset, $sortOrder, $id, $timeLineType, $leadsOrganizationsIds, $dateTodayEnd, $dateTodayStart, $filterParams);

            if(!empty($timelineData)){
                $data[] = $timelineData;
            }
        }

        if (!empty($data)) {
            $timelineDataFull = array();

            foreach ($data as $key => $value) {
                $timelineDataFull['totalRecords'] += $value['totalRecords'];
                foreach ($value['data'] as $v) {
                    $timelineDataFull['data'][] = $v;
                }
            }

            $users = Leads_Mapper_LeadsMapper::getInstance()->getAllUsers(true);
            $currentUserId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();
            $currencySymbol = preg_replace('~[\w]~', '', $currency->getSymbol());
            $moneyFormat = Tools_LeadTools::getCurrencyFormat();
            $dataMoneyFormat = json_encode($moneyFormat);

            $timelineDataFull['users'] = $users;
            $timelineDataFull['currentUserId'] = $currentUserId;
            $timelineDataFull['currencySymbol'] = $currencySymbol;
            $timelineDataFull['moneyFormat'] = $dataMoneyFormat;
            $timelineDataFull['offset'] = $value['offset'];
            $timelineDataFull['limit'] = $value['limit'];
            $timelineDataFull['leadInfo'] = !empty($leadsOrganizations) ? $leadsOrganizations : array();
            $timelineDataFull['userTimezoneOffset'] = Tools_System_Tools::convertDateFromTimezone('now', 'UTC', Tools_System_Tools::getUserTimezone(), 'T');

            $data = $timelineDataFull;
        } else {
            $data = array();
        }

        return $data;
    }

    /**
     *
     * Resource:
     * : /api/leads/rganizationstimeline/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
    }

    /**
     *
     * Resource:
     * : /api/leads/organizationstimeline/
     *
     * HttpMethod:
     * : PUT
     *
     * @return JSON
     */
    public function putAction()
    {


    }

    /**
     *
     * Resource:
     * : /api/leads/organizationstimeline/
     *
     * HttpMethod:
     * : DELETE
     *
     *
     * @return JSON
     */
    public function deleteAction()
    {

    }

}
