<?php

class Api_Leads_Ruleforms extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    /**
     * User restricted id access
     *
     * @var null
     */
    protected $_restrictedUserId = null;

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
    }


    /**
     * Get form by id
     *
     * Resource:
     * : /api/leads/ruleforms/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order'), FILTER_SANITIZE_STRING);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $forGrid = (bool)filter_var($this->_request->getParam('isGrid', false), FILTER_SANITIZE_STRING);

        $forms = Leads_Mapper_LeadsFormRulesGeneralConfigMapper::getInstance()->getAllForms();

        $data['forms'] = $forms;
        if (empty($data['forms'])) {
            $data = array();
        }

        if ($forGrid && !empty($data)) {
            $data['leadOwners'] = Leads_Mapper_LeadsMapper::getInstance()->getLeadOwnersFullList();
            $currentUser = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser();
            $data['currentUserId'] = $currentUser->getId();
            $data['userTimezoneOffset'] = Tools_System_Tools::getUtcOffset('P');
            $data['userTimezone'] = $currentUser->getTimezone();
            $leadTagsDataMapper = Leads_Mapper_LeadsTagsMapper::getInstance();
            $leadStageDataMapper = Leads_Mapper_LeadsStageMapper::getInstance();
            $leadSequenceMapper = Emailsequence_Models_Mapper_EmailSequenceSettingsMapper::getInstance();
            $data['lifecycleStages'] = $leadStageDataMapper->getStages();
            $data['leadTags'] = $leadTagsDataMapper->getTagsPairs();
            $data['emailServices'] = $enabledServices = Tools_LeadTools::getEnabledServices('email');
            $data['emailSequencies'] = $leadSequenceMapper->getSequencesPairs(true);
            $data['limitedAccess'] =  $this->_restrictedUserId;

            $advertisingList = Tools_LeadTools::$_advertisingTypes;
            foreach ($advertisingList as $advKey => $advAccount) {
                $services = Tools_LeadTools::sambastatsConnectionStatusGet($advKey);

                if (!empty($services['data'])) {
                    $adAccountId = $services['data']['adAccountId'];
                    if(empty($services['data']['connectionStatus'])) {
                        $adAccountId = 'disconnected';
                    }
                    $advertisingList[$advKey] .= ' (' . $adAccountId . ')';
                }
            }
            $data['advertisingList'] = $advertisingList;
            $data['cronScriptPathAudiences'] = Zend_Registry::get('website')['path'].Tools_LeadRemarketingAudiencesTools::LEAD_REMARKETING_AUDIENCES_PROCESS_PATH;
        }

        return $data;
    }

    public function postAction()
    {

    }

    public function putAction()
    {

    }

    public function deleteAction()
    {

    }

}
