<?php

class Api_Leads_Tablecolumnorder extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('tableName');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     *
     * Resource:
     * : /api/leads/tablecolumnorder/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {


    }

    /**
     *
     * Resource:
     * : /api/leads/tablecolumnorder/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING,
            FILTER_FLAG_NO_ENCODE_QUOTES);
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'));
        }

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            return array(
                'error' => '0',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Wrong token')
            );
        }

        $tableName = $data['tableName'];
        $tableColumnOrder = array_filter($data['tableColumnOrder']);
        if (!empty($tableColumnOrder)) {
            $fieldsData = json_encode($tableColumnOrder);
        } else {
            $fieldsData = '';
        }

        $presetId = $data['presetId'];
        $presetName = filter_var($data['presetName'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES);

        $session = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
        $userId = $session->getCurrentUser()->getId();
        $date = Tools_System_Tools::convertDateFromTimezone('now');

        $leadsTableColumnsOrderMapper = Leads_Mapper_LeadsTableColumnsOrderMapper::getInstance();
        if (!isset($presetId)) {
            return array(
                'error' => '0',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Parameter missing')
            );
        }

        if (!empty($presetId)) {
            $leadsTableColumnsOrderModel = $leadsTableColumnsOrderMapper->find($presetId);
            if (!$leadsTableColumnsOrderModel instanceof Leads_Model_LeadsTableColumnsOrderModel) {
                return array(
                    'error' => '0',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Record not found')
                );
            }

            $leadsTableColumnsOrderModel->setFieldsData($fieldsData);
        } elseif ($presetId == '0') {
            $leadsTableColumnsOrderModel = $leadsTableColumnsOrderMapper->findDefaultPresetModel($tableName);
            if (!$leadsTableColumnsOrderModel instanceof Leads_Model_LeadsTableColumnsOrderModel) {
                return array(
                    'error' => '0',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Record not found')
                );
            }

            $leadsTableColumnsOrderModel->setFieldsData($fieldsData);

        } else {
            $existingPreset = $leadsTableColumnsOrderMapper->findUserPreset($tableName, $presetName, $userId);
            if ($existingPreset instanceof Leads_Model_LeadsTableColumnsOrderModel) {
                return array(
                    'error' => '0',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Preset with this name already exists')
                );
            }

            $leadsTableColumnsOrderModel = new Leads_Model_LeadsTableColumnsOrderModel();
            $leadsTableColumnsOrderModel->setTableName($tableName);
            $leadsTableColumnsOrderModel->setCreatedAt($date);
            $leadsTableColumnsOrderModel->setIsDefault('0');
            $leadsTableColumnsOrderModel->setUserId($userId);
            $leadsTableColumnsOrderModel->setPresetName($presetName);
            $leadsTableColumnsOrderModel->setFieldsData($fieldsData);
        }

        $leadsTableColumnsOrderMapper->save($leadsTableColumnsOrderModel);

        return array(
            'error' => '0',
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Table column order has been saved')
        );
    }

    /**
     *
     * Resource:
     * : /api/leads/tablecolumnorder/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     * : lead tag id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = filter_var_array(json_decode($this->_request->getRawBody(), true), FILTER_SANITIZE_STRING,
            FILTER_FLAG_NO_ENCODE_QUOTES);

        $translator = Zend_Registry::get('Zend_Translate');

        $secureToken = $data['secureToken'];
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in ' . '<a href="' . $websiteUrl . 'go">here</a>'));
        }

        $id = $data['id'];
        $isDefault = '0';
        if (!empty($data['isDefault'])) {
            $isDefault = '1';
        }

        $session = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
        $userId = $session->getCurrentUser()->getId();
        $date = Tools_System_Tools::convertDateFromTimezone('now');

        $leadsTableColumnsOrderMapper = Leads_Mapper_LeadsTableColumnsOrderMapper::getInstance();
        $leadsTableColumnsOrderModel = $leadsTableColumnsOrderMapper->find($id);
        if (!$leadsTableColumnsOrderModel instanceof Leads_Model_LeadsTableColumnsOrderModel) {
            return array(
                'error' => '1',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Record not found')
            );
        }

        if ($userId != $leadsTableColumnsOrderModel->getUserId()) {
            return array(
                'error' => '1',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Access not allowed')
            );
        }

        $leadsTableColumnsOrderMapper->disableDefaultPreset($userId, $data['tableName']);

        $leadsTableColumnsOrderModel->setIsDefault($isDefault);

        $leadsTableColumnsOrderMapper->save($leadsTableColumnsOrderModel);

        return array(
            'error' => '0',
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Has been changed')
        );
    }

    /**
     *
     * Resource:
     * : /api/leads/tablecolumnorder/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     * : lead tag id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {

        $id = $this->_request->getParam('id');

        $translator = Zend_Registry::get('Zend_Translate');

        if (empty($id)) {
            return array(
                'error' => '1',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Record not found')
            );
        }

        $session = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
        $userId = $session->getCurrentUser()->getId();

        $leadsTableColumnsOrderMapper = Leads_Mapper_LeadsTableColumnsOrderMapper::getInstance();
        $leadsTableColumnsOrderModel = $leadsTableColumnsOrderMapper->find($id);
        if (!$leadsTableColumnsOrderModel instanceof Leads_Model_LeadsTableColumnsOrderModel) {
            return array(
                'error' => '1',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Record not found')
            );
        }

        if ($leadsTableColumnsOrderModel->getUserId() != $userId) {
            return array(
                'error' => '1',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Record not found')
            );
        }

        $leadsTableColumnsOrderMapper->delete($id);

        return array(
            'error' => '0',
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Preset has been deleted')
        );
    }

}
