<?php

class Api_Leadsdashboard_Engagedleads extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    const ACTION_TYPE_ALL = 'all';

    const ACTION_TYPE_OPENED_EMAILS = 'open_email';

    const ACTION_TYPE_WEBSITE_VISIT = 'inbound_website_visit';

    const ACTION_TYPE_CALLED = 'inbound_call';

    const ACTION_TYPE_EMAILED = 'outbound_email';

    const ACTION_TYPE_CHATTED = 'inbound_chat';

    const ACTION_TYPE_SMS = 'inbound_sms';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();

    /**
     * User restricted id access
     *
     * @var null
     */
    protected $_restrictedUserId = null;

    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );


    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
        $this->_loggedUserRoleId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getRoleId();

    }


    /**
     * Welcome screen
     *
     * Resource:
     * : /api/leadsdashboard/engagedleads/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $userId = filter_var($this->_request->getParam('userId'), FILTER_SANITIZE_NUMBER_INT);
        $actionType = filter_var($this->_request->getParam('actionType'), FILTER_SANITIZE_STRING);
        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = 'pld.last_activity DESC';


        $restrictedAccess = Tools_LeadWelcomeScreenTools::isRestrictedAccess();
        $currentUserId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();

        if ($userId == '-2') {
            $userId = $currentUserId;
        }

        $showAllUsers = false;
        if ($userId == '-1') {
            $showAllUsers = true;
        }


        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();

        $where = $leadsMapper->getDbTable()->getAdapter()->quoteInto('pld.last_activity <> ?',
            '0000-00-00 00:00:00');
        if ($showAllUsers === false) {
            if (!empty($where)) {
                $where .= ' AND ';
            }
            if (!empty($userId) && empty($restrictedAccess)) {
                $where .= $leadsMapper->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?',
                    $userId);
            } else {
                $where .= $leadsMapper->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?',
                    $currentUserId);
            }
        }

        if (!empty($actionType) && $actionType !== self::ACTION_TYPE_ALL) {
            $leadTypes = Leads_Mapper_LeadsTypeMapper::getInstance()->getTypes(true, array(), array(),
                array('type_name', 'id'));

            if ($actionType === self::ACTION_TYPE_OPENED_EMAILS) {
                $actionTypeSubtype = 'opened';
                $actionType = self::ACTION_TYPE_EMAILED;
            }

            if (!empty($leadTypes) && !empty($leadTypes[$actionType])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $actionTypeId = $leadTypes[$actionType]['id'];
                if ($actionType === self::ACTION_TYPE_OPENED_EMAILS) {
                    $actionTypeSubtype = 'opened';
                }

                if (!empty($actionTypeId)) {

                    $where .= $leadsMapper->getDbTable()->getAdapter()->quoteInto('pld.last_activity_type_id = ?',
                        $actionTypeId);


                    if (!empty($actionTypeSubtype)) {
                        if (!empty($where)) {
                            $where .= ' AND ';
                        }

                        $where .= $leadsMapper->getDbTable()->getAdapter()->quoteInto('pld.last_activity_subtype = ?',
                            $actionTypeSubtype);
                    }
                }
            }
        }

        $data = $leadsMapper->fetchAll($where, $sortOrder, $limit, $offset);


        if (empty($data)) {
            $data['data'] = array();
        }

        if (empty($data['totalRecords'])) {
            $data['totalRecords'] = 0;
        }

        $leadsIds = array();
        $oppStats = array();
        if (!empty($data['data'])) {
            foreach ($data['data'] as $leadsData) {
                $leadsIds[] = $leadsData['id'];
            }

            if (!empty($leadsIds)) {
                $oppStats = Tools_LeadWelcomeScreenTools::getTotalOppStats($leadsIds);
            }
        }

        if (!empty($data['data'])) {
            $leadsCustomParamsMapper = Leads_Mapper_LeadsCustomParamsMapper::getInstance();

            foreach ($data['data'] as $key => $leadsData) {
                $countryStates = Tools_LeadTools::getStates($leadsData['lead_country_code'], true, true);
                if (!empty($leadsData['lead_state_code'])) {
                    $data['data'][$key]['lead_state_code'] = $countryStates[$leadsData['lead_state_code']];
                }

                $leadCustomParams = $leadsCustomParamsMapper->getParamsDataWithConfigInfo($leadsData['id'], array('text'));

                if(!empty($leadCustomParams)) {
                    foreach ($leadCustomParams as $cParam) {
                        $data['data'][$key][Tools_LeadInboundEmailTools::LEAD_CUSTOM_ATTRIBUTES_PREFIX.'fields'][Tools_LeadInboundEmailTools::LEAD_CUSTOM_ATTRIBUTES_PREFIX.$cParam['param_type'].'_'.$cParam['param_name']] = $cParam['param_value'];
                    }
                }
            }
        }

        $leadDataMapper = Leads_Mapper_LeadsMapper::getInstance();

        $tableColumnsInfo = Tools_LeadTablesTools::getTableColumnsInfo(Tools_LeadTablesTools::ENGAGED_LEAD_TABLE_COLUMN_ORDER_NAME);
        $data['tableColumnsInfo'] = $tableColumnsInfo;
        $data['engagedLeadsAdditionalInfo']['currencyInfo'] = Tools_LeadWelcomeScreenTools::getCurrencyInfo();
        $data['engagedLeadsAdditionalInfo']['countryCodes'] = Tools_Geo::getCountries(true);
        $data['engagedLeadsAdditionalInfo']['industryIds'] = Leads_Mapper_LeadsIndustryMapper::getInstance()->getIndustriesPairs(false);
        $data['engagedLeadsAdditionalInfo']['customerGroups'] = $leadDataMapper->getCustomerGroups();
        $data['currencyInfo'] = Tools_LeadWelcomeScreenTools::getCurrencyInfo();

        $data['oppStats'] = $oppStats;

        return $data;
    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {

    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function putAction()
    {

    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function deleteAction()
    {


    }

}
