<?php

class Api_Leadsdashboard_Graphs extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';


    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();

    /**
     * User restricted id access
     *
     * @var null
     */
    protected $_restrictedUserId = null;

    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );


    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
        $this->_loggedUserRoleId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getRoleId();

    }


    /**
     * Welcome screen
     *
     * Resource:
     * : /api/leadsdashboard/graphs/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $userId = filter_var($this->_request->getParam('userId'), FILTER_SANITIZE_NUMBER_INT);
        $graphType = $this->_request->getParam('graphType');
        $leadReportsFilterStartDateOrigin =  filter_var($this->_request->getParam('filterDateFrom'), FILTER_SANITIZE_STRING);
        $leadReportsFilterEndDateOrigin =  filter_var($this->_request->getParam('filterDateTo'), FILTER_SANITIZE_STRING);
        $leadReportsFunnelDateOrigin =  filter_var($this->_request->getParam('leadReportsFunnelDateOrigin'), FILTER_SANITIZE_STRING);
        $leadReportsFunnelToDateOrigin =  filter_var($this->_request->getParam('leadReportsFunnelToDateOrigin'), FILTER_SANITIZE_STRING);

        $restrictedAccess = Tools_LeadWelcomeScreenTools::isRestrictedAccess();
        $currentUserId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();

        if ($userId == '-2') {
            $userId = $currentUserId;
        }

        $showAllUsers = false;
        if ($userId == '-1') {
            $showAllUsers = true;
        }

        $restrictedUserId = 0;

        if ($showAllUsers === false) {
            if (!empty($userId) && empty($restrictedAccess)) {
                $restrictedUserId = $userId;
            } else {
                $restrictedUserId = $currentUserId;
            }
        }

        $storageName = '';

        if ($graphType === Tools_LeadDashboardGraphTools::LEAD_REPORT_OPPORTUNITY_STAGE_PER_SOURCE) {
            if (empty($leadReportsFilterStartDateOrigin)) {
                $leadReportsFilterStartDateOrigin = date('Y-m-01', strtotime('now'));
            } else {
                $leadReportsFilterStartDateOrigin = date('Y-m-d', strtotime($leadReportsFilterStartDateOrigin));
            }

            if (empty($leadReportsFilterEndDateOrigin)) {
                $leadReportsFilterEndDateOrigin = date('Y-m-t', strtotime('now'));
            } else {
                $leadReportsFilterEndDateOrigin = date('Y-m-d', strtotime($leadReportsFilterEndDateOrigin));
            }

            $storageName = 'opportunityStagePerSourceGraphData';
            $data['graphs'][Tools_LeadDashboardGraphTools::LEAD_REPORT_OPPORTUNITY_STAGE_PER_SOURCE] = Tools_LeadDashboardGraphTools::getGraphData($graphType,
                $leadReportsFilterStartDateOrigin, $leadReportsFilterEndDateOrigin, '', '', $restrictedUserId);
        }

        if ($graphType === Tools_LeadDashboardGraphTools::LEAD_REPORT_OPPORTUNITY_FUNNEL_STAGE_ORIGINAL) {
            if (empty($leadReportsFunnelDateOrigin)) {
                $leadReportsFunnelDateOrigin = date('Y-m-d', strtotime('now'));
            } else {
                $leadReportsFunnelDateOrigin = date('Y-m-d', strtotime($leadReportsFunnelDateOrigin));
            }

            if (empty($leadReportsFunnelToDateOrigin)) {
                $leadReportsFunnelToDateOrigin = date('Y-m-d', strtotime('+100 years'));
            } else {
                $leadReportsFunnelToDateOrigin = date('Y-m-d', strtotime($leadReportsFunnelToDateOrigin));
            }

            $storageName = 'funnelGraphData';
            $data['graphs'][Tools_LeadDashboardGraphTools::LEAD_REPORT_OPPORTUNITY_FUNNEL_STAGE_ORIGINAL] = Tools_LeadDashboardGraphTools::getGraphData($graphType,
                $leadReportsFunnelDateOrigin, $leadReportsFunnelToDateOrigin, '', '', $restrictedUserId);
        }

        $data['storageName'] = $storageName;

        return $data;
    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {

    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function putAction()
    {

    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function deleteAction()
    {


    }

}
