<?php

class Api_Leadsdashboard_Latestleads extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();

    /**
     * User restricted id access
     *
     * @var null
     */
    protected $_restrictedUserId = null;

    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );


    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
        $this->_loggedUserRoleId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getRoleId();

    }


    /**
     * Welcome screen
     *
     * Resource:
     * : /api/leadsdashboard/latestleads/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $userId = filter_var($this->_request->getParam('userId'), FILTER_SANITIZE_NUMBER_INT);
        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = 'pld.created_at DESC';


        $restrictedAccess = Tools_LeadWelcomeScreenTools::isRestrictedAccess();
        $currentUserId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();

        if ($userId == '-2') {
            $userId = $currentUserId;
        }

        $showAllUsers = false;
        if ($userId == '-1') {
            $showAllUsers = true;
        }

        $where = null;
        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();

        if ($showAllUsers === false) {
            if (!empty($userId) && empty($restrictedAccess)) {
                $where = $leadsMapper->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?',
                    $userId);
            } else {
                $where = $leadsMapper->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?',
                    $currentUserId);
            }
        }


        $data = $leadsMapper->fetchAll($where, $sortOrder, $limit, $offset);


        if (empty($data)) {
            $data['data'] = array();
        }

        if (empty($data['totalRecords'])) {
            $data['totalRecords'] = 0;
        }

        if (!empty($data['data'])) {
            $leadsCustomParamsMapper = Leads_Mapper_LeadsCustomParamsMapper::getInstance();

            foreach ($data['data'] as $key => $leadsData) {
                $countryStates = Tools_LeadTools::getStates($leadsData['lead_country_code'], true, true);
                if (!empty($leadsData['lead_state_code'])) {
                    $data['data'][$key]['lead_state_code'] = $countryStates[$leadsData['lead_state_code']];
                }

                $leadCustomParams = $leadsCustomParamsMapper->getParamsDataWithConfigInfo($leadsData['id'], array('text'));

                if(!empty($leadCustomParams)) {
                    foreach ($leadCustomParams as $cParam) {
                        $data['data'][$key][Tools_LeadInboundEmailTools::LEAD_CUSTOM_ATTRIBUTES_PREFIX.'fields'][Tools_LeadInboundEmailTools::LEAD_CUSTOM_ATTRIBUTES_PREFIX.$cParam['param_type'].'_'.$cParam['param_name']] = $cParam['param_value'];
                    }
                }
            }
        }

        $leadDataMapper = Leads_Mapper_LeadsMapper::getInstance();

        $tableColumnsInfo = Tools_LeadTablesTools::getTableColumnsInfo(Tools_LeadTablesTools::LATEST_LEAD_TABLE_COLUMN_ORDER_NAME);
        $data['tableColumnsInfo'] = $tableColumnsInfo;
        $data['latestLeadsAdditionalInfo']['currencyInfo'] = Tools_LeadWelcomeScreenTools::getCurrencyInfo();
        $data['latestLeadsAdditionalInfo']['countryCodes'] = Tools_Geo::getCountries(true);
        $data['latestLeadsAdditionalInfo']['industryIds'] = Leads_Mapper_LeadsIndustryMapper::getInstance()->getIndustriesPairs(false);
        $data['latestLeadsAdditionalInfo']['customerGroups'] = $leadDataMapper->getCustomerGroups();


        return $data;
    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {

    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function putAction()
    {

    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function deleteAction()
    {


    }

}
