<?php

class Api_Leadsdashboard_Opportunitiesclosingsoon extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();

    /**
     * User restricted id access
     *
     * @var null
     */
    protected $_restrictedUserId = null;

    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );


    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
        $this->_loggedUserRoleId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getRoleId();

    }


    /**
     * Welcome screen
     *
     * Resource:
     * : /api/leadsdashboard/opportunitiesclosingsoon/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $userId = filter_var($this->_request->getParam('userId'), FILTER_SANITIZE_NUMBER_INT);
        $oppStage = filter_var($this->_request->getParam('oppStage'), FILTER_SANITIZE_NUMBER_INT);
        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = 'plo.expected_close_date DESC';
        $subType = filter_var($this->_request->getParam('subType', 'open'), FILTER_SANITIZE_STRING);

        $where = null;
        $restrictedAccess = Tools_LeadWelcomeScreenTools::isRestrictedAccess();
        $currentUserId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();
        $dateTodayEnd = Tools_System_Tools::convertDateFromTimezone(date('Y-m-d', strtotime('+1 day')) . ' 00:00:00');
        $dateTodayStart = Tools_System_Tools::convertDateFromTimezone(date('Y-m-d') . '00:00:00');

        if ($userId == '-2') {
            $userId = $currentUserId;
        }

        $showAllUsers = false;
        if ($userId == '-1') {
            $showAllUsers = true;
        }


        $leadsOpportunityMapper = Leads_Mapper_LeadsOpportunityMapper::getInstance();
        if ($showAllUsers === false) {
            if (!empty($userId) && empty($restrictedAccess)) {
                $where = $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.creator_id = ?',
                    $userId);
            } else {
                $where = $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.creator_id = ?',
                    $currentUserId);
            }
        }

        if (!empty($oppStage)) {
            if (!empty($where)) {
                $where .= ' AND ';
            }

            if($oppStage == '-1') {
                $where .= $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.opportunity_stage_id NOT IN (?)',
                    array('1', '100'));
            } else {
                $where .= $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.opportunity_stage_id = ?',
                    $oppStage);
            }
        }

        if (!empty($where)) {
            $where .= ' AND ';
        }

        if ($subType === 'open') {
            $where .= $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.expected_close_date >= ?', Tools_System_Tools::convertDateFromTimezone('+1 day'));
            $data = $leadsOpportunityMapper->getOpportunitiesForDashboard($where, $sortOrder, $limit, $offset);
        } else {
            $where .= $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.expected_close_date <= ?', Tools_System_Tools::convertDateFromTimezone('+1 day'));
            $data = $leadsOpportunityMapper->getOpportunitiesForDashboard($where, $sortOrder, $limit, $offset);
        }

        $opportunityStagesInfo = Leads_Mapper_LeadsOpportunityStageMapper::getInstance()->getOpportunityStagesData(true);

        if (empty($data)) {
            $data['data'] = array();
        } else {
            $leadsOpportunityQuoteMapper = Leads_Mapper_LeadsOpportunityQuoteMapper::getInstance();
            foreach ($data['data'] as $key => $opportunityStage) {
                $opportunitiesQuotes = $leadsOpportunityQuoteMapper->getOpportunities($opportunityStage['oppId']);
                if (!empty($opportunitiesQuotes)) {
                    $data['data'][$key]['opportunities'] = $opportunitiesQuotes;
                } else {
                    $data['data'][$key]['opportunities'] = array();
                }

                $data['data'][$key]['quoteCount'] = count($data['data'][$key]['opportunities']);
            }
        }

        $data['opportunitiesAdditionalInfo'] = array(
            'opportunityStagesInfo' => $opportunityStagesInfo
        );

        return $data;
    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {

    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function putAction()
    {

    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function deleteAction()
    {


    }

}
