<?php

class Api_Leadsdashboard_Opportunitygoal extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();

    /**
     * User restricted id access
     *
     * @var null
     */
    protected $_restrictedUserId = null;

    protected $_loggedUserRoleId = null;

    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );


    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
        $this->_loggedUserRoleId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getRoleId();

    }


    /**
     * Welcome screen
     *
     * Resource:
     * : /api/leadsdashboard/opportunitygoal/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $userId = filter_var($this->_request->getParam('userId'), FILTER_SANITIZE_NUMBER_INT);
        $filterDateFrom = filter_var($this->_request->getParam('filterDateFrom'), FILTER_SANITIZE_STRING);
        $filterDateTo = filter_var($this->_request->getParam('filterDateTo'), FILTER_SANITIZE_STRING);
        $queryType = filter_var($this->_request->getParam('queryType'), FILTER_SANITIZE_STRING);
        $oppType = filter_var($this->_request->getParam('oppType'), FILTER_SANITIZE_STRING);
        $filterPeriodMonths = filter_var($this->_request->getParam('filterPeriodMonths'), FILTER_SANITIZE_STRING);

        $where = null;
        $restrictedAccess = Tools_LeadWelcomeScreenTools::isRestrictedAccess();
        $currentUserId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();

        if ($userId == '-2') {
            $userId = $currentUserId;
        }

        $showAllUsers = false;
        if ($userId == '-1') {
            $showAllUsers = true;
        }

        $data['data'] = array();

        if (empty($filterDateFrom) && empty($filterDateTo)) {
            $filterPeriodMonths = 1;
        }

        if (empty($filterDateFrom)) {
            $filterDateFrom = date('Y-m-01', strtotime('now'));
        } else {
            $filterDateFrom = date('Y-m-d', strtotime($filterDateFrom));
        }

        if (empty($filterDateTo)) {
            $filterDateTo = date('Y-m-t', strtotime('now'));
        } else {
            $filterDateTo = date('Y-m-d', strtotime($filterDateTo));
        }

        $restrictedUserId = 0;

        if ($showAllUsers === false) {
            if (!empty($userId) && empty($restrictedAccess)) {
                $restrictedUserId = $userId;
            } else {
                $restrictedUserId = $currentUserId;
            }
        }

        $data['opportunitiesAdditionalInfo']['displayRank'] = '0';
        if (!empty($restrictedUserId)) {
            $data['opportunitiesAdditionalInfo']['displayRank'] = '1';
            $data['opportunitiesAdditionalInfo']['rank'] = Tools_LeadStatsTools::calculateSalesRank($filterDateFrom, $filterDateTo, $restrictedUserId, $queryType, Tools_LeadStatsTools::LEAD_STAT_TYPE_SUB_TYPE_OPEN, $oppType);
        }

        $additionalParams = array();
        $additionalParams['oppType'] = $oppType;

        if ($queryType === 'amount') {
            $graphType = Tools_LeadDashboardGraphTools::LEAD_REPORT_OPPORTUNITY_PIE_AMOUNT;
            $data['graphs'][Tools_LeadDashboardGraphTools::LEAD_REPORT_OPPORTUNITY_PIE_AMOUNT] = Tools_LeadDashboardGraphTools::getGraphData($graphType,
                $filterDateFrom, $filterDateTo, '', '', $restrictedUserId, $additionalParams);

            $totalAmount = 0;
            $openAmount = 0;
            $lostAmount = 0;
            $wonAmount = 0;
            $data['opportunitiesAdditionalInfo']['statsInfo'] = array();
            if (!empty($data['graphs'][Tools_LeadDashboardGraphTools::LEAD_REPORT_OPPORTUNITY_PIE_AMOUNT]['data'])) {
                foreach ($data['graphs'][Tools_LeadDashboardGraphTools::LEAD_REPORT_OPPORTUNITY_PIE_AMOUNT]['data'] as $key => $statGraphInfo) {
                    $totalAmount += $statGraphInfo['amount'];
                    if ($key == '1') {
                        $lostAmount = $statGraphInfo['amount'];
                    }

                    if ($key == '99') {
                        $openAmount = $statGraphInfo['amount'];
                    }

                    if ($key == '100') {
                        $wonAmount = $statGraphInfo['amount'];
                    }
                }
            }

            $data['opportunitiesAdditionalInfo']['statsInfo']['totalAmount'] = $totalAmount;
            $data['opportunitiesAdditionalInfo']['statsInfo']['openAmount'] = $openAmount;
            $data['opportunitiesAdditionalInfo']['statsInfo']['lostAmount'] = $lostAmount;
            $data['opportunitiesAdditionalInfo']['statsInfo']['wonAmount'] = $wonAmount;
        }

        if ($queryType === 'count') {
            $graphType = Tools_LeadDashboardGraphTools::LEAD_REPORT_OPPORTUNITY_PIE_COUNT;
            $data['graphs'][Tools_LeadDashboardGraphTools::LEAD_REPORT_OPPORTUNITY_PIE_COUNT] = Tools_LeadDashboardGraphTools::getGraphData($graphType,
                $filterDateFrom, $filterDateTo, '', '', $restrictedUserId, $additionalParams);

            $totalCount = 0;
            $openCount = 0;
            $lostCount = 0;
            $wonCount = 0;
            $data['opportunitiesAdditionalInfo']['statsInfo'] = array();
            if (!empty($data['graphs'][Tools_LeadDashboardGraphTools::LEAD_REPORT_OPPORTUNITY_PIE_COUNT]['data'])) {
                foreach ($data['graphs'][Tools_LeadDashboardGraphTools::LEAD_REPORT_OPPORTUNITY_PIE_COUNT]['data'] as $key => $statGraphInfo) {
                    $totalCount += $statGraphInfo['count'];
                    if ($key == '1') {
                        $lostCount = $statGraphInfo['count'];
                    }

                    if ($key == '99') {
                        $openCount = $statGraphInfo['count'];
                    }

                    if ($key == '100') {
                        $wonCount = $statGraphInfo['count'];
                    }
                }
            }

            $data['opportunitiesAdditionalInfo']['statsInfo']['totalCount'] = $totalCount;
            $data['opportunitiesAdditionalInfo']['statsInfo']['openCount'] = $openCount;
            $data['opportunitiesAdditionalInfo']['statsInfo']['lostCount'] = $lostCount;
            $data['opportunitiesAdditionalInfo']['statsInfo']['wonCount'] = $wonCount;
        }

        $statsData = Tools_LeadStatsTools::calculateOpportunityGoals($data['opportunitiesAdditionalInfo']['statsInfo'], $filterPeriodMonths,
                $filterDateFrom, $filterDateTo, $restrictedUserId, $queryType, $oppType);

        $data['opportunitiesAdditionalInfo']['statsInfo'] = $statsData;

        $opportunityTypesInfo = Leads_Mapper_LeadsOpportunityTypeMapper::getInstance()->getOpportunityTypes();

        $data['opportunitiesAdditionalInfo']['opportunityTypesInfo'] = $opportunityTypesInfo;


        return $data;
    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {

    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function putAction()
    {

    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function deleteAction()
    {


    }

}
