<?php

class Api_Leadsdashboard_Oppsalesgeneralinfo extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();

    /**
     * User restricted id access
     *
     * @var null
     */
    protected $_restrictedUserId = null;

    protected $_loggedUserRoleId = null;

    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );


    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
        $this->_loggedUserRoleId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getRoleId();

    }


    /**
     * Welcome screen
     *
     * Resource:
     * : /api/leadsdashboard/oppsalesgeneralinfo/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $userId = filter_var($this->_request->getParam('userId'), FILTER_SANITIZE_NUMBER_INT);
        $filterDateFrom = filter_var($this->_request->getParam('filterDateFrom'), FILTER_SANITIZE_STRING);
        $filterDateTo = filter_var($this->_request->getParam('filterDateTo'), FILTER_SANITIZE_STRING);
        $filterPeriodMonths = filter_var($this->_request->getParam('filterPeriodMonths'), FILTER_SANITIZE_STRING);

        $where = null;
        $restrictedAccess = Tools_LeadWelcomeScreenTools::isRestrictedAccess();
        $currentUserId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();
        $userTimezone = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getTimezone();

        if ($userId == '-2') {
            $userId = $currentUserId;
        }

        $showAllUsers = false;
        if ($userId == '-1') {
            $showAllUsers = true;
        }

        $data['data'] = array();

        if (empty($filterDateFrom) && empty($filterDateTo)) {
            $filterPeriodMonths = 1;
        }

        if (empty($filterDateFrom)) {
            $filterDateFrom = Tools_System_Tools::convertDateFromTimezone(Tools_System_Tools::convertDateFromTimezone('now',
                $userTimezone, 'UTC', 'Y-m-01 00:00:00'));
        } else {
            $filterDateFrom = Tools_System_Tools::convertDateFromTimezone($filterDateFrom, $userTimezone, 'UTC',
                'Y-m-d H:i:s');
        }

        if (empty($filterDateTo)) {
            $filterDateTo = Tools_System_Tools::convertDateFromTimezone(Tools_System_Tools::convertDateFromTimezone('now',
                $userTimezone, 'UTC', 'Y-m-t 23:59:59'));
        } else {
            $filterDateTo = Tools_System_Tools::convertDateFromTimezone($filterDateTo, $userTimezone, 'UTC',
                'Y-m-d H:i:s');
        }

        $statsTypes = array(
            0 => 'callsPlaced',
            1 => 'callsReceived',
            2 => 'emailsSent',
            3 => 'emailsReceived',
            4 => 'meetingsHeld',
            5 => 'tasksCompleted',
        );



        if ($showAllUsers === false) {
            if (!empty($userId) && empty($restrictedAccess)) {
                $userIdToAnalyze = $userId;
                $statsData = Tools_LeadStatsTools::getGeneralStats($filterDateFrom, $filterDateTo, $statsTypes, $userId);
            } else {
                $userIdToAnalyze = $currentUserId;
                $statsData = Tools_LeadStatsTools::getGeneralStats($filterDateFrom, $filterDateTo, $statsTypes, $currentUserId);
            }
        } else {
            $userIdToAnalyze = 0;
            $statsData = Tools_LeadStatsTools::getGeneralStats($filterDateFrom, $filterDateTo, $statsTypes);
        }

        $statsData = Tools_LeadStatsTools::calculateGoalStats($statsData['stats'], $filterPeriodMonths, $filterDateFrom, $filterDateTo, $userIdToAnalyze);

        $data['data']['stats'] = $statsData;

        return $data;
    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {

    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function putAction()
    {

    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function deleteAction()
    {


    }

}
