<?php

class Api_Leadsdashboard_Salesgoal extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();

    /**
     * User restricted id access
     *
     * @var null
     */
    protected $_restrictedUserId = null;

    protected $_loggedUserRoleId = null;

    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );


    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
        $this->_loggedUserRoleId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getRoleId();

    }


    /**
     * Welcome screen
     *
     * Resource:
     * : /api/leadsdashboard/salesgoal/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $userId = filter_var($this->_request->getParam('userId'), FILTER_SANITIZE_NUMBER_INT);
        $filterDateFrom = filter_var($this->_request->getParam('filterDateFrom'), FILTER_SANITIZE_STRING);
        $filterDateTo = filter_var($this->_request->getParam('filterDateTo'), FILTER_SANITIZE_STRING);
        $filterPeriodMonths = filter_var($this->_request->getParam('filterPeriodMonths'), FILTER_SANITIZE_STRING);

        $where = null;
        $restrictedAccess = Tools_LeadWelcomeScreenTools::isRestrictedAccess();
        $currentUserId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();

        if ($userId == '-2') {
            $userId = $currentUserId;
        }

        $showAllUsers = false;
        if ($userId == '-1') {
            $showAllUsers = true;
        }

        $data['data'] = array();

        if (empty($filterDateFrom) && empty($filterDateTo)) {
            $filterPeriodMonths = 1;
        }

        if (empty($filterDateFrom)) {
            $filterDateFrom = date('Y-m-01', strtotime('now'));
        } else {
            $filterDateFrom = date('Y-m-d', strtotime($filterDateFrom));
        }

        if (empty($filterDateTo)) {
            $filterDateTo = date('Y-m-t', strtotime('now'));
        } else {
            $filterDateTo = date('Y-m-d', strtotime($filterDateTo));
        }

        $restrictedUserId = 0;

        if ($showAllUsers === false) {
            if (!empty($userId) && empty($restrictedAccess)) {
                $restrictedUserId = $userId;
            } else {
                $restrictedUserId = $currentUserId;
            }
        }

        $data['salesAdditionalInfo']['displayRank'] = '0';
        if (!empty($restrictedUserId)) {
            $data['salesAdditionalInfo']['displayRank'] = '1';
            $data['salesAdditionalInfo']['rank'] = Tools_LeadStatsTools::calculateSalesRank($filterDateFrom, $filterDateTo, $restrictedUserId);

        }

        $amountInfo = Tools_LeadDashboardGraphTools::getGraphData(Tools_LeadDashboardGraphTools::LEAD_REPORT_OPPORTUNITY_PIE_AMOUNT,
            $filterDateFrom, $filterDateTo, '', '', $restrictedUserId);

        $wonAmount = 0;
        $data['salesAdditionalInfo']['statsInfo'] = array();
        if (!empty($amountInfo['data'])) {
            foreach ($amountInfo['data'] as $key => $statGraphInfo) {
                if ($key == '100') {
                    $wonAmount = $statGraphInfo['amount'];
                }
            }
        }

        $data['salesAdditionalInfo']['statsInfo']['totalAmount'] = $wonAmount;

        $statsData = Tools_LeadStatsTools::calculateSalesGoals($data['salesAdditionalInfo']['statsInfo'], $filterPeriodMonths,
            $filterDateFrom, $filterDateTo, $restrictedUserId);

        $data['salesAdditionalInfo']['statsInfo'] = $statsData;

        $additionalParams = array();
        if (!empty($filterPeriodMonths)) {
            $additionalParams['filterPeriodMonths'] = $filterPeriodMonths;
        }

        $data['graphs'][Tools_LeadDashboardGraphTools::LEAD_REPORT_OPPORTUNITY_QUOTE_ATTAINMENT] = Tools_LeadDashboardGraphTools::getGraphData(Tools_LeadDashboardGraphTools::LEAD_REPORT_OPPORTUNITY_QUOTE_ATTAINMENT,
            $filterDateFrom, $filterDateTo, '', '', $restrictedUserId, $additionalParams);


        return $data;
    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {

    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function putAction()
    {

    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function deleteAction()
    {


    }

}
