<?php

class Api_Leadsdashboard_Taskmeetingsnotes extends Api_Service_Abstract
{


    const TASKS_TYPE = 'tasks';

    const MEETINGS_TYPE = 'meetings';

    const NOTES_TYPE = 'notes';

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();

    /**
     * User restricted id access
     *
     * @var null
     */
    protected $_restrictedUserId = null;

    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );


    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
        $this->_loggedUserRoleId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getRoleId();

    }


    /**
     * Welcome screen
     *
     * Resource:
     * : /api/leadsdashboard/taskmeetingsnotes/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $userId = filter_var($this->_request->getParam('userId'), FILTER_SANITIZE_NUMBER_INT);
        $queryType = filter_var($this->_request->getParam('queryType', self::TASKS_TYPE), FILTER_SANITIZE_STRING);
        $subtype = filter_var($this->_request->getParam('querySubType'), FILTER_SANITIZE_STRING);
        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);

        $where = null;
        $restrictedAccess = Tools_LeadWelcomeScreenTools::isRestrictedAccess();
        $currentUserId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();
        $dateTodayEnd = Tools_System_Tools::convertDateFromTimezone(date('Y-m-d', strtotime('+1 day')). ' 00:00:00');
        $dateTodayStart = Tools_System_Tools::convertDateFromTimezone(date('Y-m-d').'00:00:00');

        if ($userId == '-2') {
            $userId = $currentUserId;
        }

        $showAllUsers = false;
        if ($userId == '-1') {
            $showAllUsers = true;
        }

        if ($queryType === self::TASKS_TYPE) {
            $leadsTaskMapper = Leads_Mapper_LeadsTaskMapper::getInstance();
            if ($showAllUsers === false) {
                if (!empty($userId) && empty($restrictedAccess)) {
                    $where = $leadsTaskMapper->getDbTable()->getAdapter()->quoteInto('plt.user_id = ?', $userId);
                } else {
                    $where = $leadsTaskMapper->getDbTable()->getAdapter()->quoteInto('plt.user_id = ?',
                        $currentUserId);
                }
            }

            $sortOrder = 'plt.due_date DESC';

            if (!empty($where)) {
                $where .= ' AND ';
            }

            $where .= $leadsTaskMapper->getDbTable()->getAdapter()->quoteInto('plt.is_task_completed <> ?', '1');

            if ($subtype === Tools_LeadTools::TIMELINE_TYPE_UPCOMING) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }

                $where .= $leadsTaskMapper->getDbTable()->getAdapter()->quoteInto('plt.due_date > ?', $dateTodayEnd);
            } elseif($subtype === Api_Leads_Leads::FILTER_TASK_OVERDUE) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $leadsTaskMapper->getDbTable()->getAdapter()->quoteInto('plt.is_task_overdue = ?', '1');
            } else {
                if (!empty($where)) {
                    $where .= ' AND ';
                }

                $where .= $leadsTaskMapper->getDbTable()->getAdapter()->quoteInto('plt.due_date < ?', $dateTodayEnd);
            }


            $data = $leadsTaskMapper->fetchAllData($where, $sortOrder, $limit, $offset);
        }

        if ($queryType === self::MEETINGS_TYPE) {
            $sortOrder = 'plm.start_time_date DESC';
            $leadsMeetingMapper = Leads_Mapper_LeadsMeetingMapper::getInstance();
            if ($showAllUsers === false) {
                if (!empty($userId) && empty($restrictedAccess)) {
                    $where = $leadsMeetingMapper->getDbTable()->getAdapter()->quoteInto('plm.creator_id = ?', $userId);
                } else {
                    $where = $leadsMeetingMapper->getDbTable()->getAdapter()->quoteInto('plm.creator_id = ?',
                        $currentUserId);
                }
            }

            if ($subtype === Tools_LeadTools::TIMELINE_TYPE_UPCOMING) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }

                $where .= $leadsMeetingMapper->getDbTable()->getAdapter()->quoteInto('plm.start_time_date > ?', $dateTodayEnd);
            }

            $data = $leadsMeetingMapper->fetchAllData($where, $sortOrder, $limit, $offset);
        }

        if ($queryType === self::NOTES_TYPE) {
            $sortOrder = 'pln.date_created DESC';
            $leadNotesDataMapper = Leads_Mapper_LeadsNoteMapper::getInstance();
            if ($showAllUsers === false) {
                if (!empty($userId) && empty($restrictedAccess)) {
                    $where = $leadNotesDataMapper->getDbTable()->getAdapter()->quoteInto('pln.user_id = ?', $userId);
                } else {
                    $where = $leadNotesDataMapper->getDbTable()->getAdapter()->quoteInto('pln.user_id = ?',
                        $currentUserId);
                }
            }

            $data = $leadNotesDataMapper->fetchAllDataForDashboard($where, $sortOrder, $limit, $offset);
        }

        $leadOwners = Leads_Mapper_LeadsMapper::getInstance()->getLeadOwnersFullList(0, array(), true);

        if (empty($data)) {
            $data['data'] = array();
        }

        $data['additionalInfo'] = array(
            'leadOwners' => $leadOwners
        );

        return $data;
    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {

    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function putAction()
    {

    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function deleteAction()
    {


    }

}
