<?php

class Api_Leadsopenapi_Leadorganization extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('organization_name');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get')
        ),
        Tools_Security_Acl::ROLE_GUEST => array(
            'allow' => array('get')
        ),
    );

    /**
     * User restricted id access
     *
     * @var null
     */
    protected $_restrictedUserId = null;

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Get lead organization by id
     *
     * Resource:
     * : /api/leadsopenapi/leadorganization/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $sortOrder = filter_var($this->_request->getParam('order'), FILTER_SANITIZE_STRING);
        $searchTerm = filter_var($this->_request->getParam('searchTerm'), FILTER_SANITIZE_STRING);
        $having = null;

        $leadsConfigMapper = Leads_Mapper_LeadsConfigMapper::getInstance();
        $enableOpenApi = $leadsConfigMapper->getConfigParam('enableOpenApi');
        $translator = Zend_Registry::get('Zend_Translate');
        if (empty($enableOpenApi)) {
            $this->_responseHelper->fail($translator->translate(Leads::TRANSLATION_PREFIX.'Access denied'));
        }

        if (empty($sortOrder)) {
            $sortOrder = 'plo.created_at DESC';
        } else {
            $sortOrder = 'plo.'.$sortOrder.' DESC';
        }

        $leadOrganizationDataMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();

        $where = null;
        if (!empty($searchTerm)) {
            $where = $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_name LIKE ?',
                $searchTerm . '%');
        }


        $data = $leadOrganizationDataMapper->fetchAll($where, $sortOrder, null, null, true, false, false);

        return $data;
    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     * : lead organization id to update
     *
     * @return JSON
     */
    public function putAction()
    {

    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     * : lead organization id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {

    }

}
