<?php

class Api_Leadsopportunities_Hideopportunities extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();

    /**
     * User restricted id access
     *
     * @var null
     */
    protected $_restrictedUserId = null;

    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
        $this->_loggedUserRoleId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getRoleId();

    }


    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */

    public function getAction()
    {

    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        if (!empty($this->_restrictedUserId)) {
            return array(
                'status' => 'error',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'You don\'t have permission to hide opportunity'));
        }

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'/go">here</a>'));
        }

        $opportunity = filter_var($this->getRequest()->getParam('opportunityId'), FILTER_SANITIZE_STRING);

        $hideAllOppotunities = false;
        if($opportunity == 'hideAll') {
            $hideAllOppotunities = true;
        }

        $leadsOpportunityMapper = Leads_Mapper_LeadsOpportunityMapper::getInstance();
        $leadsHiddenOpportunitiesMapper = Leads_Mapper_LeadsHiddenOpportunitiesMapper::getInstance();

        if($hideAllOppotunities) {
            $userId = filter_var($this->_request->getParam('userId'), FILTER_SANITIZE_NUMBER_INT);

            $restrictedAccess = Tools_LeadWelcomeScreenTools::isRestrictedAccess();
            $currentUserId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();

            if ($userId == '-2') {
                $userId = $currentUserId;
            }

            $showAllUsers = false;
            if ($userId == '-1') {
                $showAllUsers = true;
            }

            if ($showAllUsers === false) {
                if (!empty($userId) && empty($restrictedAccess)) {
                    $where = $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.creator_id = ?',
                        $userId);
                } else {
                    $where = $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.creator_id = ?',
                        $currentUserId);
                }
            }

            $opportunityStageId = filter_var($this->_request->getParam('opportunityStageId'), FILTER_SANITIZE_NUMBER_INT);

            if(!empty($opportunityStageId)) {
                if(!empty($where)) {
                    $where .= ' AND ';
                }

                $where .= $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.opportunity_stage_id = ?',
                    $opportunityStageId);
            }

            $hiddenOpportunities = $leadsHiddenOpportunitiesMapper->fetchAll();

            if(!empty($hiddenOpportunities)) {
                if(!empty($where)) {
                    $where .= ' AND ';
                }

                $where .= $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.id NOT IN (?)', $hiddenOpportunities);
            }

            $oppData = $leadsOpportunityMapper->getOpportunitiesForOpportunitiesTab($where);

            if(!empty($oppData)) {
                foreach ($oppData as $opp) {
                    $leadsHiddenOpportunitiesModel = new Leads_Model_LeadsHiddenOpportunitiesModel();
                    $leadsHiddenOpportunitiesModel->setOpportunityId($opp['oppId']);

                    $leadsHiddenOpportunitiesMapper->save($leadsHiddenOpportunitiesModel);
                }

                return array('status'=> true);
            } else {
                return array(
                    'status' => 'error',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Nothing to hide.'));
            }
        } else {
            if(!empty($opportunity)) {
                $where =  $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('opportunity_id = ?', $opportunity);

                $hiddenOpportunity = $leadsHiddenOpportunitiesMapper->fetchAll($where);

                if(empty($hiddenOpportunity)) {
                    $leadsHiddenOpportunitiesModel = new Leads_Model_LeadsHiddenOpportunitiesModel();
                    $leadsHiddenOpportunitiesModel->setOpportunityId($opportunity);

                    $leadsHiddenOpportunitiesMapper->save($leadsHiddenOpportunitiesModel);

                    return array('status'=> true);
                }
            }
        }

        return array(
            'status' => 'error',
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Can\t hide opportunity'));

    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function putAction()
    {

    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function deleteAction()
    {


    }

}
