<?php

class Api_Leadsopportunities_Opportunitiesinfo extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();

    /**
     * User restricted id access
     *
     * @var null
     */
    protected $_restrictedUserId = null;

    protected $_loggedUserRoleId = null;

    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );


    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $this->_restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
        $this->_loggedUserRoleId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getRoleId();

    }


    /**
     * Opportunities info
     *
     * Resource:
     * : /api/leadsopportunities/opportunitiesinfo/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $userId = filter_var($this->_request->getParam('userId'), FILTER_SANITIZE_NUMBER_INT);
        $filterStartDateOrigin =  filter_var($this->_request->getParam('filterDateFrom'), FILTER_SANITIZE_STRING);
        $filterEndDateOrigin =  filter_var($this->_request->getParam('filterDateTo'), FILTER_SANITIZE_STRING);

        if (empty($filterStartDateOrigin)) {
            $filterStartDateOrigin = date('Y-m-01', strtotime('now'));
        } else {
            $filterStartDateOrigin = date('Y-m-d', strtotime($filterStartDateOrigin));
        }

        if (empty($filterEndDateOrigin)) {
            $filterEndDateOrigin = date('Y-m-t', strtotime('now'));
        } else {
            $filterEndDateOrigin = date('Y-m-d', strtotime($filterEndDateOrigin));
        }

        $filterStartDate = Tools_System_Tools::convertDateFromTimezone($filterStartDateOrigin . ' 00:00:00',
            false, 'UTC', 'Y-m-d H:i:s');
        $filterEndDate = Tools_System_Tools::convertDateFromTimezone($filterEndDateOrigin . ' 23:59:59',
            false, 'UTC', 'Y-m-d H:i:s');

        $where = null;
        $restrictedAccess = Tools_LeadWelcomeScreenTools::isRestrictedAccess();
        $currentUserId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();

        if ($userId == '-2') {
            $userId = $currentUserId;
        }

        $showAllUsers = false;
        if ($userId == '-1') {
            $showAllUsers = true;
        }

        $leadsOpportunityMapper = Leads_Mapper_LeadsOpportunityMapper::getInstance();
        if ($showAllUsers === false) {
            if (!empty($userId) && empty($restrictedAccess)) {
                $where = $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.creator_id = ?',
                    $userId);
            } else {
                $where = $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.creator_id = ?',
                    $currentUserId);
            }
        }

        $leadsHiddenOpportunitiesMapper = Leads_Mapper_LeadsHiddenOpportunitiesMapper::getInstance();

        $hiddenOpportunities = $leadsHiddenOpportunitiesMapper->fetchAll();

        if(!empty($hiddenOpportunities)) {
            if(!empty($where)) {
                $where .= ' AND ';
            }

            $where .= $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.id NOT IN (?)', $hiddenOpportunities);
        }

        if(!empty($where)) {
            $where .= ' AND ';
        }

        $where .= $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.expected_close_date >= ?', $filterStartDate);
        $where .= ' AND ' . $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.expected_close_date <= ?', $filterEndDate);

        $oppData = $leadsOpportunityMapper->getOpportunitiesForOpportunitiesTab($where);

        $opportunityStages = array(
            10 => 'Unqualified',
            20 => 'NeedsAssessment',
            30 => 'Qualified',
            40 => 'MeetingSet',
            50 => 'MeetingHeld',
            60 => 'ProposalSent',
            70 => 'ProposalAcknowledged',
            80 => 'ObjectionsRaised',
            90 => 'ObjectionsOvercame',
            1 =>  'Lost',
            100 =>'Won'
        );

        $opportunitiesDataFinal = array(
            'opportunities' => array(
                'Unqualified' => array(),
                'NeedsAssessment' => array(),
                'Qualified' => array(),
                'MeetingSet' => array(),
                'MeetingHeld' => array(),
                'ProposalSent' => array(),
                'ProposalAcknowledged' => array(),
                'ObjectionsRaised' => array(),
                'ObjectionsOvercame' => array(),
                'Lost' => array(),
                'Won' => array()
            )
        );

        $forecastedSales = '0';
        $pipelineSize = '0';
        if(!empty($oppData)) {
            $leadsOpportunityQuoteMapper = Leads_Mapper_LeadsOpportunityQuoteMapper::getInstance();

            foreach ($oppData as $opp) {
                if($opp['opportunityStageId'] != '1' && $opp['opportunityStageId'] != '100') {
                    $forecastedSales += Tools_LeadReportsTools::prepareSalesExpectation(array($opp));
                    $pipelineSize += Tools_LeadReportsTools::preparePipelineSize(array($opp));
                }

                $oppQuotesData = $leadsOpportunityQuoteMapper->getOppQuoteAndCartInfoByOppId($opp['oppId'], $opp['leadId']);

                if(!empty($oppQuotesData)) {
                    $opp['oppQuoteInfo'] = $oppQuotesData;
                }

                if(array_key_exists($opp['opportunityStageId'],$opportunityStages )) {
                    $opportunitiesDataFinal['opportunities'][$opportunityStages[$opp['opportunityStageId']]][] = $opp;
                }
            }
        }

        $opportunitiesDataFinal['currencyInfo'] = Tools_LeadWelcomeScreenTools::getCurrencyInfo();
        $opportunitiesDataFinal['forecastedSales'] = $forecastedSales;
        $opportunitiesDataFinal['pipelineSize'] = $pipelineSize;

        $opportunityStagesInfo = Leads_Mapper_LeadsOpportunityStageMapper::getInstance()->getOpportunityStagesData(true);

        $opportunitiesDataFinal['opportunitiesAdditionalInfo'] = array(
            'opportunityStagesInfo' => $opportunityStagesInfo,
            'allUsers' => (int) $showAllUsers
        );

        return $opportunitiesDataFinal;
    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        if (!empty($this->_restrictedUserId)) {
            return array(
                'status' => 'error',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'You don\'t have permission to change opportunity'));
        }

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'/go">here</a>'));
        }

        $opportunity = filter_var($this->getRequest()->getParam('opportunityId'), FILTER_SANITIZE_NUMBER_INT);
        $opportunityToDraggableStageId = filter_var($this->getRequest()->getParam('toDraggableStageId'), FILTER_SANITIZE_NUMBER_INT);

        $filterStartDateOrigin =  filter_var($this->_request->getParam('filterDateFrom'), FILTER_SANITIZE_STRING);
        $filterEndDateOrigin =  filter_var($this->_request->getParam('filterDateTo'), FILTER_SANITIZE_STRING);

        if (empty($filterStartDateOrigin)) {
            $filterStartDateOrigin = date('Y-m-01', strtotime('now'));
        } else {
            $filterStartDateOrigin = date('Y-m-d', strtotime($filterStartDateOrigin));
        }

        if (empty($filterEndDateOrigin)) {
            $filterEndDateOrigin = date('Y-m-t', strtotime('now'));
        } else {
            $filterEndDateOrigin = date('Y-m-d', strtotime($filterEndDateOrigin));
        }

        $filterStartDate = Tools_System_Tools::convertDateFromTimezone($filterStartDateOrigin . ' 00:00:00',
            false, 'UTC', 'Y-m-d H:i:s');
        $filterEndDate = Tools_System_Tools::convertDateFromTimezone($filterEndDateOrigin . ' 23:59:59',
            false, 'UTC', 'Y-m-d H:i:s');

        if(!empty($opportunity) && !empty($opportunityToDraggableStageId)) {
            $leadOpportunityMapper = Leads_Mapper_LeadsOpportunityMapper::getInstance();

            $leadOpportunity = $leadOpportunityMapper->find($opportunity);

            if($leadOpportunity instanceof Leads_Model_LeadsOpportunityModel) {
                $stageIdFrom = $leadOpportunity->getOpportunityStageId();

                $leadOpportunityMapper->changeStageId($stageIdFrom, $opportunityToDraggableStageId, $opportunity);

                $userId = filter_var($this->_request->getParam('userId'), FILTER_SANITIZE_NUMBER_INT);

                $where = null;
                $restrictedAccess = Tools_LeadWelcomeScreenTools::isRestrictedAccess();
                $currentUserId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();

                if ($userId == '-2') {
                    $userId = $currentUserId;
                }

                $showAllUsers = false;
                if ($userId == '-1') {
                    $showAllUsers = true;
                }

                $leadsOpportunityMapper = Leads_Mapper_LeadsOpportunityMapper::getInstance();
                if ($showAllUsers === false) {
                    if (!empty($userId) && empty($restrictedAccess)) {
                        $where = $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.creator_id = ?',
                            $userId);
                    } else {
                        $where = $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.creator_id = ?',
                            $currentUserId);
                    }
                }

                $leadsHiddenOpportunitiesMapper = Leads_Mapper_LeadsHiddenOpportunitiesMapper::getInstance();

                $hiddenOpportunities = $leadsHiddenOpportunitiesMapper->fetchAll();

                if(!empty($hiddenOpportunities)) {
                    if(!empty($where)) {
                        $where .= ' AND ';
                    }

                    $where .= $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.id NOT IN (?)', $hiddenOpportunities);
                }

                if(!empty($where)) {
                    $where .= ' AND ';
                }

                $where .= $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.expected_close_date >= ?', $filterStartDate);
                $where .= ' AND ' . $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.expected_close_date <= ?', $filterEndDate);

                $oppData = $leadsOpportunityMapper->getOpportunitiesForOpportunitiesTab($where);

                $forecastedSales = '0';
                $pipelineSize = '0';
                if(!empty($oppData)) {
                    foreach ($oppData as $opp) {

                        if($opp['opportunityStageId'] != '1' && $opp['opportunityStageId'] != '100') {
                            $forecastedSales += Tools_LeadReportsTools::prepareSalesExpectation(array($opp));
                            $pipelineSize += Tools_LeadReportsTools::preparePipelineSize(array($opp));
                        }
                    }
                }

                $opportunitiesForecastData['forecastedSales'] = $forecastedSales;
                $opportunitiesForecastData['pipelineSize'] = $pipelineSize;

                return array('status'=> true, 'opportunitiesForecastData' => $opportunitiesForecastData);
            }
        } else {
            return array('status' => 'nothingChange');
        }

        return array(
            'status' => 'error',
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Can\'t update opportunity'));
    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function putAction()
    {

    }

    /**
     *
     * Resource:
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     *
     * @return JSON
     */
    public function deleteAction()
    {


    }

}
