<?php

/**
 * LeadsActionLogMapper.php
 *
 *
 * @method Leads_Mapper_LeadsActionLogMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsActionLogMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsActionLogModel';

    protected $_dbTable = 'Leads_DbTable_LeadsActionLogDbTable';

    /**
     * Save action log model
     * @param $model Leads_Model_LeadsActionLogModel
     * @return Leads_Model_LeadsActionLogModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'type' => $model->getType(),
            'lead_id' => $model->getLeadId(),
            'created_at' => $model->getCreatedAt(),
            'action_type' => $model->getActionType(),
            'additional_content' => $model->getAdditionalContent(),
            'event_type' => $model->getEventType(),
            'lead_type' => $model->getLeadType()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Get lead data with info for groups etc..
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param bool $withTimezoneOffset select data with processed dates based on logged user timezone
     * @param string $groupBy mysql group by
     * @param string $having mysql having
     * @return array
     */
    public function fetchAll(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $withoutCount = false,
        $singleRecord = false,
        $withTimezoneOffset = true,
        $groupBy = '',
        $having = ''
    ) {
        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');

        $createdAt = 'plal.created_at';
        $meetingEndTimeDate = 'plml.end_time_date';
        $meetingStartTimeDate = 'plml.start_time_date';

        if ($withTimezoneOffset === true) {
            $createdAt = new Zend_Db_Expr("CONVERT_TZ(`plal`.`created_at`, '+00:00', '$mysqlOffset')");
            $meetingEndTimeDate = new Zend_Db_Expr("CONVERT_TZ(`plml`.`end_time_date`, '+00:00', '$mysqlOffset')");
            $meetingStartTimeDate = new Zend_Db_Expr("CONVERT_TZ(`plml`.`start_time_date`, '+00:00', '$mysqlOffset')");
        }

        $selectFields =  array(
            'plal.id',
            'plal.type',
            'plal.lead_id',
            'created_at'=> $createdAt,
            'plal.action_type',
            'plal.event_type',
            'plal.additional_content',
            'leadUserId' => 'plrl.user_id',
            'leadFirstName' => 'pld.lead_first_name',
            'leadLastName' => 'pld.lead_last_name',
            'quoteId' => 'plql.quote_id',
            'quoteTitle' => 'plql.title',
            'quoteStatus' => 'plql.status',
            'quoteCreatorId' => 'plql.creator_id',
            'quoteLink' => 'plql.quote_link',
            'quoteTotal' => 'plql.total',
            'quoteSent' => 'plql.quote_sent',
            'quoteIsDeleted' => 'plql.is_deleted',
            'quoteDeletedBy' => 'plql.deleted_by',
            'purchaseSignedOnlyBy' => 'plpl.signed_only_by',
            'formName' => 'plfl.form_name',
            'formContent' => 'plfl.content',
            'downloaderContent' => 'pldl.content',
            'formUrl' => 'plfl.form_url',
            'emailContent' => 'plel.content',
            'emailSubject' => 'plel.email_subject',
            'emailTo' => 'plel.email_to',
            'emailStatus' => 'plel.email_status',
            'meetingEndTimeDate' => $meetingEndTimeDate,
            'meetingStartTimeDate' => $meetingStartTimeDate,
            'meetingSubject' => 'plml.subject',
            'meetingDetails' => 'plml.details',
            'meetingInviteAttendees' => 'plml.invite_attendees',
            'meetingLocation' => 'plml.location',
            'meetingStatus' => 'plml.meeting_status',
            'meetingId' => 'plml.meeting_id',
            'meetingCreatorId' => 'plml.creator_id',
            'meetingLogId' => 'plml.id',
            'meetingAdditionalDetails' => 'plml.additional_content',
            'taskTitle' => 'pltl.title',
            'taskNotes' => 'pltl.notes',
            'taskCreatorId' => 'pltl.creator_id',
            'taskAssignedToId' => 'pltl.user_id',
            'taskId' => 'pltl.task_id',
            'isTaskCompleted' => 'pltl.is_task_completed',
            'isTaskOverdue' => 'pltl.is_task_overdue',
            'taskCompletedBy' => 'pltl.task_completed_by',
            'taskCompletionComment' => 'pltl.completion_comment',
            'callDuration' => 'plcl.call_duration',
            'callLink' => 'plcl.link',
            'callCreatorId' => 'plcl.creator_id',
            'callDetails' => 'plcl.details',
            'callStatus' => 'plcl.status',
            'caller' => 'plcl.caller',
            'callLogId' => 'plcl.id',
            'callSource' => 'plcl.call_source',
            'callReceiver' => 'plcl.receiver',
            'callLogSubType' => 'plcl.sub_type',
            'purchaseCartId' => 'plpl.cart_id',
            'purchaseTotal' => 'plpl.total',
            'purchaseStatus' => 'plpl.status',
            'purchaseIsPartial' => 'plpl.is_partial',
            'purchasePartialPaidAmount' => 'plpl.partially_paid_amount',
            'purchaseGateway' => 'plpl.gateway',
            'purchaseIsManuallyPaid' => 'plpl.is_manually_paid',
            'downloadedFile' => 'pldl.file_name',
            'downloadedFilePath' => 'pldl.file_path',
            'downloadedPageUrl' => 'pldl.page_url',
            'downloadedPageId' => 'pldl.page_id',
            'registrationLogUserRole' => 'plrl.user_role',
            'registrationPageUrl' => 'plrl.registration_page_url',
            'registrationPageId' => 'plrl.registration_page_id',
            'visitTotalTime' => 'pliupl.visit_duration',
            'visitTotalCount' => 'pliupl.visit_count',
            'visitTotalActions' => 'pliupl.actions',
            'visitId' => 'pliupl.id',
            'idVisit' => 'pliupl.id_visit',
            'visitLastActionDateTime' => 'pliupl.last_action_date_time',
            'emailAttachmentFileName' => 'pleal.file_name',
            'emailAttachmentFileFolder' => 'pleal.file_folder_name',
            'emailAttachmentFileLink' => 'pleal.file_link',
            'emailFromName' => 'plel.email_from_name',
            'emailFromEmail' => 'plel.email_from',
            'emailSourceType' => 'plel.email_source_type',
            'emailSourceName' => 'plel.external_source_name',
            'emailCreatorId'  => 'plel.creator_id',
            'emailReplyId' => 'plel.reply_id',
            'encodedContent' => new Zend_Db_Expr(
                $this->getDbTable()->getAdapter()->quoteInto("AES_DECRYPT(plel.encoded_content,?)",
                    Tools_LeadInboundEmailTools::getPasswordKey())),
            'emailUid' => 'plel.uid',
            'originalPurchaseTotal' => 'scs.total'
        );

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plal' => 'plugin_leads_action_log'),
                $selectFields
            )
            ->joinLeft(array('plql' => 'plugin_leads_quote_log'), 'plql.action_log_id=plal.id', array())
            ->joinLeft(array('plfl' => 'plugin_leads_form_log'), 'plfl.action_log_id=plal.id', array())
            ->joinLeft(array('plel' => 'plugin_leads_email_log'), 'plel.action_log_id=plal.id', array())
            ->joinLeft(array('pleal' => 'plugin_leads_email_link_attachment_log'), 'plel.attachment_id=pleal.id', array())
            ->joinLeft(array('plml' => 'plugin_leads_meeting_log'), 'plml.action_log_id=plal.id', array())
            ->joinLeft(array('pltl' => 'plugin_leads_tasks_log'), 'pltl.action_log_id=plal.id', array())
            ->joinLeft(array('plcl' => 'plugin_leads_call_log'), 'plcl.action_log_id=plal.id', array())
            ->joinLeft(array('plpl' => 'plugin_leads_purchase_log'), 'plpl.action_log_id=plal.id', array())
            ->joinLeft(array('scs' => 'shopping_cart_session'), 'plpl.cart_id =scs.id', array())
            ->joinLeft(array('pldl' => 'plugin_leads_downloader_log'), 'pldl.action_log_id=plal.id', array())
            ->joinLeft(array('plrl' => 'plugin_leads_registration_log'), 'plrl.action_log_id=plal.id', array())
            ->joinLeft(array('pliupl' => 'plugin_leads_ip_user_profile_log'), 'pliupl.action_log_id=plal.id', array())
            ->joinLeft(array('pld' => 'plugin_leads_data'), 'plal.lead_id=pld.id', array());
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        if (!empty($groupBy)) {
            $select->group($groupBy);
        }

        if (!empty($having)) {
            $select->having($having);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::GROUP);

            $select->from(array('plal' => 'plugin_leads_action_log'), array('count' => 'COUNT(plal.id)'))
                ->joinLeft(array('plql' => 'plugin_leads_quote_log'), 'plql.action_log_id=plal.id', array())
                ->joinLeft(array('plfl' => 'plugin_leads_form_log'), 'plfl.action_log_id=plal.id', array())
                ->joinLeft(array('plel' => 'plugin_leads_email_log'), 'plel.action_log_id=plal.id', array())
                ->joinLeft(array('pleal' => 'plugin_leads_email_link_attachment_log'), 'plel.attachment_id=pleal.id', array())
                ->joinLeft(array('plml' => 'plugin_leads_meeting_log'), 'plml.action_log_id=plal.id', array())
                ->joinLeft(array('pltl' => 'plugin_leads_tasks_log'), 'pltl.action_log_id=plal.id', array())
                ->joinLeft(array('plcl' => 'plugin_leads_call_log'), 'plcl.action_log_id=plal.id', array())
                ->joinLeft(array('plpl' => 'plugin_leads_purchase_log'), 'plpl.action_log_id=plal.id', array())
                ->joinLeft(array('pldl' => 'plugin_leads_downloader_log'), 'pldl.action_log_id=plal.id', array())
                ->joinLeft(array('plrl' => 'plugin_leads_registration_log'), 'plrl.action_log_id=plal.id', array())
                ->joinLeft(array('pliupl' => 'plugin_leads_ip_user_profile_log'), 'pliupl.action_log_id=plal.id', array());
            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }
    /**
     * Get quote data info
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param string $groupBy mysql group by
     * @param string $having mysql having
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param bool $withTimezoneOffset select data with processed dates based on logged user timezone
     * @return array
     */
    public function getQuotesDataInfo(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $groupBy = '',
        $having = '',
        $withoutCount = false,
        $singleRecord = false,
        $withTimezoneOffset = true
    )
    {
        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');

        $createdAt = 'pld.created_at';

        if ($withTimezoneOffset === true) {
            $createdAt = new Zend_Db_Expr("CONVERT_TZ(`pld`.`created_at`, '+00:00', '$mysqlOffset')");
        }

        $selectFields =  array(
            'pld.id'
        );

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pld' => 'plugin_leads_data'),
                $selectFields
            )
            ->join(array('sq' => 'shopping_quote'), 'pld.user_id=sq.user_id', array())
            ->join(array('scs' => 'shopping_cart_session'), 'scs.id=sq.cart_id', array());

        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        if (!empty($groupBy)) {
            $select->group($groupBy);
        }

        if (!empty($having)) {
            $select->having($having);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::GROUP);

            $select->from(array('pld' => 'plugin_leads_data'), array('count' => 'COUNT(plal.id)'))
                ->join(array('sq' => 'shopping_quote'), 'pld.user_id=sq.user_id', array())
                ->join(array('scs' => 'shopping_cart_session'), 'scs.id=sq.cart_id', array());

            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }

    /**
     * Get cart data info
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param string $groupBy mysql group by
     * @param string $having mysql having
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param bool $withTimezoneOffset select data with processed dates based on logged user timezone
     * @param bool $withProducts select data with products in cart
     * @return array
     */
    public function getCartDataInfo(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $groupBy = '',
        $having = '',
        $withoutCount = false,
        $singleRecord = false,
        $withTimezoneOffset = true,
        $withProducts = false
    )
    {
        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');

        $createdAt = 'pld.created_at';

        if ($withTimezoneOffset === true) {
            $createdAt = new Zend_Db_Expr("CONVERT_TZ(`pld`.`created_at`, '+00:00', '$mysqlOffset')");
        }

        $selectFields =  array(
            'pld.id'
        );

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pld' => 'plugin_leads_data'),
                $selectFields
            )
            ->join(array('scs' => 'shopping_cart_session'), 'pld.user_id=scs.user_id', array());

        if ($withProducts === true) {
            $select->join(array('scsc' => 'shopping_cart_session_content'), 'scsc.cart_id=scs.id', array());
            $select->join(array('sp' => 'shopping_product'), 'sp.id=scsc.product_id', array());
        }

        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        if (!empty($groupBy)) {
            $select->group($groupBy);
        }

        if (!empty($having)) {
            $select->having($having);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::GROUP);

            $select->from(array('pld' => 'plugin_leads_data'), array('count' => 'COUNT(scs.id)'))
                ->join(array('scs' => 'shopping_cart_session'), 'pld.user_id=scs.id', array());

            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }


    /**
     * Get upcoming id
     *
     * @param int $leadId lead id
     * @return null
     */
    public function getUpcomingEvent($leadId)
    {
       $mysqlOffset = Tools_System_Tools::getUtcOffset('P');
       $createdAt =  Tools_System_Tools::convertDateFromTimezone('now');
       $createdAtSelect = new Zend_Db_Expr("CONVERT_TZ(`created_at`, '+00:00', '$mysqlOffset') as `created_at`");
       $where = $this->getDbTable()->getAdapter()->quoteInto('event_type = ?', Leads_Model_LeadsActionLogModel::EVENT_LOG_UPCOMING);
       $where .= ' AND ' .$this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);
       $where .= ' AND ' .$this->getDbTable()->getAdapter()->quoteInto('created_at >= ?', $createdAt);
       $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_action_log', array($createdAtSelect, 'actionType' => 'type'))->order('created_at ASC')->limit(1)->where($where);
       return $this->getDbTable()->getAdapter()->fetchRow($select);

    }


    /**
     * Change lead id
     *
     * @param array $leadIds lead ids
     * @param int $newLeadId new lead id
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeLeadId($leadIds, $newLeadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id IN (?)', $leadIds);
        $data = array('lead_id' => $newLeadId);
        $this->getDbTable()->getAdapter()->update('plugin_leads_action_log', $data, $where);
    }



    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_action_log', $where);
    }


}
