<?php

/**
 * LeadsAdditionalEmailLogMapper.php
 *
 *
 * @method Leads_Mapper_LeadsAdditionalEmailLogMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsAdditionalEmailLogMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsAdditionalEmailLogModel';

    protected $_dbTable = 'Leads_DbTable_LeadsAdditionalEmailLogDbTable';

    /**
     * Save email log model to DB
     * @param $model Leads_Model_LeadsAdditionalEmailLogModel
     * @return Leads_Model_LeadsAdditionalEmailLogModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'email_subject' => $model->getEmailSubject(),
            'content' => $model->getContent(),
            'email_from' => $model->getEmailFrom(),
            'email_from_name' => $model->getEmailFromName(),
            'email_to' => $model->getEmailTo(),
            'email_status' => $model->getEmailStatus(),
            'email_bcc' => $model->getEmailBcc(),
            'email_source_type' => $model->getEmailSourceType(),
            'message_id' => $model->getMessageId(),
            'reply_id' => $model->getReplyId(),
            'references' => $model->getReferences(),
            'uid' => $model->getUid(),
            'message_number' => $model->getMessageNumber(),
            'account_id' => $model->getAccountId(),
            'received_at' => $model->getReceivedAt(),
            'encoded_content' => new Zend_Db_Expr("AES_ENCRYPT('" . base64_encode($model->getEncodedContent()) . "', '" . Tools_LeadInboundEmailTools::getPasswordKey(). "')")
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }


    /**
     * Find by message id
     *
     * @param int $messageId attachment id
     * @return null
     */
    public function findByMessageId($messageId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('message_id = ?', $messageId);
        return $this->_findWhere($where);
    }

    /**
     * Find by message id
     *
     * @param int $messageId attachment id
     * @return null
     */
    public function findByMessageIdDecoded($messageId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('message_id = ?', $messageId);
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_additional_email_log',
            array(
                'id',
                'email_subject',
                'content',
                'email_from',
                'email_from_name',
                'email_to',
                'lead_id',
                'email_bcc',
                'email_source_type',
                'external_source_name',
                'message_id',
                'reply_id',
                'references',
                'uid',
                'message_number',
                'account_id',
                'received_at',
                'encoded_content' => new Zend_Db_Expr(
                    $this->getDbTable()->getAdapter()->quoteInto("AES_DECRYPT(encoded_content,?)",
                        Tools_LeadInboundEmailTools::getPasswordKey()))
            ))
            ->where($where);

        return $this->getDbTable()->getAdapter()->fetchRow($select);

    }
    /**
     * Find by reply id
     *
     * @param int $replyId reply id
     * @return null
     */
    public function findByReplyId($replyId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('reply_id = ?', $replyId);
        return $this->_findWhere($where);
    }

    /**
     * Delete lead email log record
     *
     * @param int $id lead id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_email_log', $where);

    }

}
