<?php

class Leads_Mapper_LeadsAdvertiserAccountsConfigMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsAdvertiserAccountsConfigModel';

    protected $_dbTable = 'Leads_DbTable_LeadsAdvertiserAccountsConfigDbTable';

    /**
     * Save leads advertiser accounts config model to DB
     * @param $model Leads_Model_LeadsAdvertiserAccountsConfigModel
     * @return Leads_Model_LeadsAdvertiserAccountsConfigModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'adv_id'                => $model->getAdvId(),
            'adv_name'              => $model->getAdvName(),
            'adv_type'              => $model->getAdvType(),
            'adv_status'            => $model->getAdvStatus(),
            'adv_verification_link' => $model->getAdvVerificationLink(),
            'allow_delete'          => $model->getAllowDelete()

        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    public function fetchAll(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $withoutCount = false,
        $singleRecord = false
    ) {
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plaac' => 'plugin_leads_advertiser_account_config'),
                array(
                    'plaac.id',
                    'plaac.adv_id',
                    'plaac.adv_name',
                    'plaac.adv_type',
                    'plaac.adv_status',
                    'plaac.adv_verification_link',
                    'plaac.allow_delete'
                )
            );
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::GROUP);

            $select->from(array('plaac' => 'plugin_leads_advertiser_account_config'),
                array('count' => 'COUNT(plaac.id)'));
            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }

    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_advertiser_account_config', $where);
    }

    public function deleteByAccountType($advType)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('adv_type = ?', $advType);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_advertiser_account_config', $where);
    }

    public function findById($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->_findWhere($where);
    }

    public function findByAdvId($advId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('adv_id = ?', $advId);

        return $this->_findWhere($where);
    }

    public function getByIdType($adAccountId, $account)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('adv_id = ?', $adAccountId);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('adv_type = ?', $account);

        return $this->_findWhere($where);
    }

    public function getAdvTypes()
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('adv_status = ?', 'connected');

        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_advertiser_account_config',
            array('adv_type', 'adv_type'))
            ->where($where);;

        return $this->getDbTable()->getAdapter()->fetchPairs($select);
    }

}
