<?php

/**
 * LeadsApiAccessListMapper.php
 *
 *
 * @method Leads_Mapper_LeadsApiAccessListMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsApiAccessListMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsApiAccessListModel';

    protected $_dbTable = 'Leads_DbTable_LeadsApiAccessListDbTable';

    /**
     * Save action log model
     * @param $model Leads_Model_LeadsApiAccessListModel
     * @return Leads_Model_LeadsApiAccessListModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'api_user_id' => $model->getApiUserId(),
            'resource' => $model->getResource()
        );

        $modelExists = $this->getByUserIdResourceId($data['api_user_id'], $data['resource']);
        if (!$modelExists instanceof Leads_Model_LeadsApiAccessListModel) {
            $this->getDbTable()->insert($data);
        } else {
            $where = $this->getDbTable()->getAdapter()->quoteInto('api_user_id = ?', $data['api_user_id']);
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('resource = ?', $data['resource']);
            $this->getDbTable()->update($data, $where);
        }

        return $model;
    }


    /**
     * Get by user id and resource id
     *
     * @param int $userId api user id
     * @param string $resource resource
     * @return null
     */
    public function getByUserIdResourceId($userId, $resource)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('api_user_id = ?', $userId);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('resource = ?', $resource);

        return $this->_findWhere($where);
    }


    /**
     * Delete by api user id
     *
     * @param int $id api user id
     * @return int
     */
    public function deleteByUserId($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('api_user_id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_api_access_list', $where);
    }

    /**
     * Delete by resource and user id
     *
     * @param int $userId api user id
     * @param string $resource resource
     * @return int
     */
    public function deleteByResourceUserId($userId, $resource)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('api_user_id = ?', $userId);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('resource = ?', $resource);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_api_access_list', $where);
    }


}
