<?php

/**
 * LeadsBuilderFormFieldsMapper.php
 *
 *
 * @method Leads_Mapper_LeadsBuilderFormFieldsMapper getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsBuilderFormFieldsMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsBuilderFormFieldsModel';

    protected $_dbTable = 'Leads_DbTable_LeadsBuilderFormFieldsDbTable';

    /**
     * Save lead form builder general config model to DB
     * @param $model Leads_Model_LeadsBuilderFormFieldsModel
     * @return Leads_Model_LeadsBuilderFormFieldsModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'form_id' => $model->getFormId(),
            'field_name' => $model->getFieldName(),
            'field_label' => $model->getFieldLabel(),
            'field_value' => $model->getFieldValue(),
            'field_data' => $model->getFieldData(),
            'field_type' => $model->getFieldType(),
            'system_type' => $model->getSystemType()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Delete by form general config id
     *
     * @param int $formId form id
     * @return int
     */
    public function deleteByFormId($formId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('form_id = ?', $formId);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_builder_form_fields', $where);
    }

    /**
     * get by form general config id
     *
     * @param int $formId form id
     * @return array
     */
    public function getFieldsData($formId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('form_id = ?', $formId);
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_builder_form_fields', array('field_name', 'field_label', 'id', 'field_value', 'field_data', 'system_type'))
            ->where($where)->order('id DESC');

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }


    /**
     * get by form general config id
     *
     * @param int $formId form id
     * @param array $fieldTypes field types
     * @return array
     */
    public function getFieldsDataNames($formId, $fieldTypes = array())
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('form_id = ?', $formId);
        if (!empty($fieldTypes)) {
            $where .= ' AND '.  $this->getDbTable()->getAdapter()->quoteInto('field_type IN (?)', $fieldTypes);
        }
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_builder_form_fields', array('field_name', 'field_name'))
            ->where($where)->order('field_label');

        return $this->getDbTable()->getAdapter()->fetchPairs($select);
    }



    /**
     * Delete form fields form config
     *
     * @param int $id config id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_builder_form_fields', $where);

    }

}
