<?php

/**
 * LeadsBuilderFormGeneralConfigMapper.php
 *
 *
 * @method Leads_Mapper_LeadsBuilderFormGeneralConfigMapper getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsBuilderFormGeneralConfigMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsBuilderFormGeneralConfigModel';

    protected $_dbTable = 'Leads_DbTable_LeadsBuilderFormGeneralConfigDbTable';

    /**
     * Save lead form builder general config model to DB
     * @param $model Leads_Model_LeadsBuilderFormGeneralConfigModel
     * @return Leads_Model_LeadsBuilderFormGeneralConfigModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'form_name' => $model->getFormName(),
            'form_type' => $model->getFormType(),
            'form_title' => $model->getFormTitle(),
            'form_domain' => $model->getFormDomain(),
            'form_button_label' => $model->getFormButtonLabel(),
            'form_code' => $model->getFormCode(),
            'form_signature' => $model->getFormSignature(),
            'created_at' => $model->getCreatedAt(),
            'creator_id' => $model->getCreatorId(),
            'editor_id' => $model->getEditorId(),
            'updated_at' => $model->getUpdatedAt(),
            'send_email' => $model->getSendEmail(),
            'send_auto_reply' => $model->getSendAutoReply(),
            'send_email_config' => $model->getSendEmailConfig(),
            'send_auto_reply_config' => $model->getSendAutoReplyConfig(),
            'recaptcha_public_key' => $model->getRecaptchaPublicKey(),
            'recaptcha_private_key' => $model->getRecaptchaPrivateKey(),
            'success_url' => $model->getSuccessUrl(),
            'fail_url' => $model->getFailUrl(),
            'sms_mobile' => $model->getSmsMobile(),
            'external_id' => $model->getExternalId()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Get form builder data
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param bool $withTimezoneOffset select data with processed dates based on logged user timezone
     * @return array
     */
    public function fetchAll(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $withoutCount = false,
        $singleRecord = false,
        $withTimezoneOffset = false
    ) {

        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');

        $createdAt = 'plbfgc.created_at';

        if ($withTimezoneOffset === true) {
            $createdAt = new Zend_Db_Expr("CONVERT_TZ(`plbfgc`.`created_at`, '+00:00', '$mysqlOffset')");
        }

        $updatedAt = 'plbfgc.updated_at';

        if ($withTimezoneOffset === true) {
            $updatedAt = new Zend_Db_Expr("CONVERT_TZ(`plbfgc`.`updated_at`, '+00:00', '$mysqlOffset')");
        }

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plbfgc' => 'plugin_leads_builder_form_general_config'),
                array(
                    'plbfgc.id',
                    'plbfgc.form_name',
                    'plbfgc.form_type',
                    'plbfgc.form_title',
                    'plbfgc.form_domain',
                    'plbfgc.form_button_label',
                    'plbfgc.form_code',
                    'plbfgc.form_signature',
                    'plbfgc.sms_mobile',
                    'plbfgc.creator_id',
                    'createdAt' => $createdAt,
                    'plbfgc.editor_id',
                    'updatedAt' => $updatedAt,
                    'creatorName' => 'u.full_name',
                    'editorName' => 'uc.full_name',
                    'plbfgc.recaptcha_public_key',
                    'plbfgc.recaptcha_private_key',
                    'plbfgc.success_url',
                    'plbfgc.fail_url',
                    'plbfgc.send_email',
                    'plbfgc.send_auto_reply',
                    'plbfgc.send_email_config',
                    'plbfgc.send_auto_reply_config',
                    'plbfgc.external_id'
                )
            )
            ->joinLeft(array('plbff' => 'plugin_leads_builder_form_fields'), 'plbff.form_id=plbfgc.id', array())
            ->joinLeft(array('u' => 'user'), 'u.id=plbfgc.creator_id', array())
            ->joinLeft(array('uc' => 'user'), 'uc.id=plbfgc.editor_id', array());


        $select->group('plbfgc.id');
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::LIMIT_COUNT);

            $count = array('count' => new Zend_Db_Expr('COUNT(DISTINCT(plbfgc.id))'));

            $select->from(array('plbfgc' => 'plugin_leads_builder_form_general_config'), $count)
                ->joinLeft(array('plbff' => 'plugin_leads_builder_form_fields'), 'plbff.form_id=plbfgc.id', array())
                ->joinLeft(array('u' => 'user'), 'u.id=plbfgc.creator_id', array())
                ->joinLeft(array('uc' => 'user'), 'uc.id=plbfgc.editor_id', array());


            $select =  $this->getDbTable()->getAdapter()->select()
                ->from(
                    array('subres' => $select),
                    array('count' => 'SUM(count)')
                );

            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }

    /**
     *
     * Find form name
     *
     * @param string $formName form name
     * @return int
     */
    public function findByFormName($formName)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('form_name = ?', $formName);

        return $this->_findWhere($where);
    }


    /**
     *
     * Find form name and type
     *
     * @param string $formName form name
     * @param string $formType form name
     * @return int
     */
    public function findByFormNameAndType($formName, $formType = Leads_Model_LeadsFormRulesGeneralConfigModel::FORM_TYPE_INTERNAL)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('form_name = ?', $formName);
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('form_type = ?', $formType);

        return $this->_findWhere($where);
    }

    /**
     *
     * Find form by signature
     *
     * @param string $signature form name
     * @return int
     */
    public function findBySignature($signature)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('form_signature = ?', $signature);

        return $this->_findWhere($where);
    }

    /**
     * Set not assigned creator id
     *
     * @param int $creatorId creator id
     * @throws Zend_Db_Adapter_Exception
     */
    public function setNotAssignedCreator($creatorId)
    {
        $where = new Zend_Db_Expr('creator_id IS NULL');

        $data = array('creator_id' => $creatorId);
        $this->getDbTable()->getAdapter()->update('plugin_leads_builder_form_general_config', $data, $where);
    }

    /**
     * Set not assigned editor id
     *
     * @param int $editorId editor id
     * @throws Zend_Db_Adapter_Exception
     */
    public function setNotAssignedEditor($editorId)
    {
        $where = new Zend_Db_Expr('editor_id IS NULL');

        $data = array('editor_id' => $editorId);
        $this->getDbTable()->getAdapter()->update('plugin_leads_builder_form_general_config', $data, $where);
    }

    /**
     * Delete form builder general config record
     *
     * @param int $id config id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_builder_form_general_config', $where);

    }

}
