<?php

/**
 * LeadsCallLogMapper.php
 *
 *
 * @method Leads_Mapper_LeadsCallLogMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsCallLogMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsCallLogModel';

    protected $_dbTable = 'Leads_DbTable_LeadsCallLogDbTable';

    /**
     * Save call log model to DB
     * @param $model Leads_Model_LeadsCallLogModel
     * @return Leads_Model_LeadsCallLogModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'call_sid' => $model->getCallSid(),
            'creator_id' => $model->getCreatorId(),
            'type' => $model->getType(),
            'sub_type' => $model->getSubType(),
            'account_sid' => $model->getAccountSid(),
            'caller' => $model->getCaller(),
            'timestamp' => $model->getTimestamp(),
            'status' => $model->getStatus(),
            'score' => $model->getScore(),
            'link' => $model->getLink(),
            'action_log_id' => $model->getActionLogId(),
            'call_duration' => $model->getCallDuration(),
            'lead_id' => $model->getLeadId(),
            'details' => $model->getDetails(),
            'call_source' => $model->getCallSource(),
            'receiver' => $model->getReceiver(),
            'is_voip' => $model->getIsVoip()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }


    /**
     * Change lead id
     *
     * @param array $leadIds lead ids
     * @param int $newLeadId new lead id
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeLeadId($leadIds, $newLeadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id IN (?)', $leadIds);
        $data = array('lead_id' => $newLeadId);
        $this->getDbTable()->getAdapter()->update('plugin_leads_call_log', $data, $where);
    }

    /**
     * Change creator id for call
     *
     * @param int $creatorIdFrom creator id from (system user id)
     * @param int $creatorIdTo creator id  to (system user id)
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeCreatorId($creatorIdFrom, $creatorIdTo)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('creator_id = ?', $creatorIdFrom);
        $data = array('creator_id' => $creatorIdTo);
        $this->getDbTable()->getAdapter()->update('plugin_leads_call_log', $data, $where);
    }


    /**
     * Delete lead call log record
     *
     * @param int $id lead id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_call_log', $where);

    }

}
