<?php

/**
 * LeadsCustomParamsMapper.php
 *
 *
 * @method Leads_Mapper_LeadsCustomParamsMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsCustomParamsMapper extends Application_Model_Mappers_Abstract implements Leads_LeadsCustomParamsMapperInterface
{

    protected $_model = 'Leads_Model_LeadsCustomParamsModel';

    protected $_dbTable = 'Leads_DbTable_LeadsCustomParamsDbTable';

    /**
     * Save lead custom params model to DB
     * @param $model Leads_Model_LeadsCustomParamsModel
     * @return Leads_Model_LeadsCustomParamsModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'param_id' => $model->getParamId(),
            'lead_id' => $model->getLeadId(),
            'param_value' => $model->getParamValue(),
            'params_option_id' => $model->getParamsOptionId()
        );

        $id = $model->getId();

        if (empty($id)) {
            $existingModel = $this->getParamByIdAndParamId($data['lead_id'], $data['param_id']);
            if ($existingModel instanceof Leads_Model_LeadsCustomParamsModel) {
                $id = $existingModel->getId();
            }
        }

        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    public function getParamByIdAndParamId($leadId, $paramId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('param_id = ?', $paramId);
        return $this->_findWhere($where);
    }

    /**
     * Get lead custom params
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @return array
     */
    public function fetchAll(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $withoutCount = false,
        $singleRecord = false
    ) {
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pllccpd' => 'plugin_leads_lead_custom_params_data'),
                array(
                    'pllccpd.id',
                    'pllccpd.param_id',
                    'pllccpd.lead_id',
                    'pllccpd.param_value',
                )
            );
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::GROUP);

            $select->from(array('pllccpd' => 'plugin_leads_lead_custom_params_data'), array('count' => 'COUNT(pllccpd.id)'));
            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }

    /**
     * Get lead custom params and original config params
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @return array
     */
    public function fetchAllData(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $withoutCount = false,
        $singleRecord = false
    ) {
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pllcpc' => 'plugin_leads_lead_custom_params_config'),
                array(
                    'param_id' => 'pllcpc.id',
                    'pllcpc.param_type',
                    'pllcpc.param_name',
                    'pllcpc.label',
                    'option_values' => new Zend_Db_Expr('GROUP_CONCAT(pllcpod.option_value)')
                )
            )
            ->joinLeft(array('pllcpod' => 'plugin_leads_lead_custom_params_options_data'), 'pllcpod.custom_param_id = pllcpc.id', array())
            ->group('pllcpc.id');
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::GROUP);

            $select->from(array('pllcpc' => 'plugin_leads_lead_custom_params_config'), array('count' => 'COUNT(pllcpc.id)'));
            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }

    /**
     * Get params prepared for grid (associative array which has as a key - custom param config id)
     *
     * @param int $leadId lead id
     * @return mixed
     */
    public function getParamsForGrid($leadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pllcpd' => 'plugin_leads_lead_custom_params_data'), array('param_id', 'lead_id', 'id', 'param_value'))
            ->where($where);
        return  $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Update selections option param values
     *
     * @param $optionId
     * @param $optionValue
     */
    public function selectUpdateOptionsValues($optionId , $optionValue)
    {
        if(!empty($optionValue)){
            $data = array('param_value' => $optionValue);

            $where = $this->getDbTable()->getAdapter()->quoteInto('params_option_id = ?', $optionId);
            $select = $this->getDbTable()->getAdapter()->select()
                ->from(array('pllcpd' => 'plugin_leads_lead_custom_params_data'), array('params_option_id'))
                ->where($where);
            $resultRow =  $this->getDbTable()->getAdapter()->fetchRow($select);

            if(!empty($resultRow)){
                $this->getDbTable()->update($data , $where);
            }
        }
    }

    /**
     * Get params (associative array which has as a key - custom param config id)
     *
     * @param int $leadId lead id
     * @return mixed
     */
    public function getAllParams($leadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pllcpd' => 'plugin_leads_lead_custom_params_data'), array('param_id', 'lead_id', 'id', 'param_value', 'params_option_id'))
            ->where($where);
        return  $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Delete lead custom param record
     *
     * @param int $id lead custom param id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_lead_custom_params_data', $where);

    }

    /**
     * Get lead custom fields with param_value
     *
     * @param $leadId
     * @param $allowedTypes
     * @return array
     */
    public function getParamsDataWithConfigInfo($leadId, $allowedTypes = array())
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('pllcpd.lead_id = ?', $leadId);

        if(!empty($allowedTypes)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pllcpc.param_type IN (?)', $allowedTypes);
        }

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pllcpd' => 'plugin_leads_lead_custom_params_data'), array(
                'pllcpc.param_type',
                'pllcpc.param_name',
                'pllcpc.label',
                'pllcpd.param_value'
                ))
            ->joinLeft(array('pllcpc' => 'plugin_leads_lead_custom_params_config'), 'pllcpc.id = pllcpd.param_id', array())
            ->where($where);
        return  $this->getDbTable()->getAdapter()->fetchAll($select);
    }

}
