<?php

class Leads_Mapper_LeadsCustomTabsConfigMapper extends Application_Model_Mappers_Abstract
{

    protected $_dbTable = 'Leads_DbTable_LeadsCustomTabsConfigDbTable';

    protected $_model = 'Leads_Model_LeadsCustomTabsConfigModel';

    /**
     * Save form stage
     *
     * @param Leads_Model_LeadsCustomTabsConfigModel $model
     * @throws Exception
     * @throws Exceptions_SeotoasterException
     * @return Leads_Model_LeadsCustomTabsConfigModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            throw new Exceptions_SeotoasterException('Given parameter should be ' . $this->_model . ' instance');
        }
        $data = array(
            'tab_name'  => $model->getTabName(),
            'tab_label' => $model->getTabLabel(),
            'tab_type'  => $model->getTabType(),
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto("id=?", $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    public function getCustomTabsConfigInfo(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $having = null,
        $withoutCount = false,
        $singleRecord = false
    ) {

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('lct' => 'plugin_leads_custom_tabs_config'),
                array('lct.*'));

        if (!empty($where)) {
            $select->where($where);
        }

        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($having)) {
            $select->having($having);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::LIMIT_COUNT);


            $count = array('count' => new Zend_Db_Expr('COUNT(DISTINCT(lct.id))'));

            $select->from(array('lct' => 'plugin_leads_custom_tabs_config'), $count);

            $select = $this->getDbTable()->getAdapter()->select()
                ->from(
                    array('subres' => $select),
                    array('count' => 'SUM(count)')
                );

            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }

    }

    public function findByTabNameAndByTabType($tabName, $tabType)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('tab_name = ?', $tabName);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('tab_type = ?', $tabType);
        return $this->_findWhere($where);
    }

    /**
     * Delete record id
     *
     * @param int $id record id
     * @return int
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
        return $this->getDbTable()->getAdapter()->delete('plugin_leads_custom_tabs_config', $where);
    }

}

