<?php

class Leads_Mapper_LeadsDocumentsMapper extends Application_Model_Mappers_Abstract
{
    protected $_model = 'Leads_Model_LeadsDocumentsModel';

    protected $_dbTable = 'Leads_DbTable_LeadsDocumentsDbTable';

    /**
     * @param $model
     * @return mixed
     * @throws Exception
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'lead_id'            => $model->getLeadId(),
            'file_stored_name'   => $model->getFileStoredName(),
            'file_hash'          => $model->getFileHash(),
            'original_file_name' => $model->getOriginalFileName(),
            'display_file_name'  => $model->getDisplayFileName(),
            'uploaded_at'        => $model->getUploadedAt(),
            'external_url'       => $model->getExternalUrl()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * @param null $where
     * @param null $order
     * @param null $limit
     * @param null $offset
     * @param null $withTimezoneOffset
     * @return array
     * @throws Exception
     */
    public function fetchAllData($where = null, $order = null, $limit = null, $offset = null,  $withTimezoneOffset = true)
    {

        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');
        if ($withTimezoneOffset === true) {
            $uploadedAt = new Zend_Db_Expr("CONVERT_TZ(`pldoc`.`uploaded_at`, '+00:00', '$mysqlOffset')");
        } else{
            $uploadedAt = 'pldoc.uploaded_at';
        }

        $select = $this->getDbTable()->getAdapter()->select()->from(array('pldoc' => 'plugin_leads_documents') ,
            array(
                'pldoc.id',
                'leadId'           => 'pldoc.lead_id',
                'fileStoredName'   => 'pldoc.file_stored_name',
                'fileHash'         => 'pldoc.file_hash',
                'originalFileName' => 'pldoc.original_file_name',
                'displayfilename'  => 'pldoc.display_file_name',
                'uploadedAt'       => $uploadedAt,
                'externalUrl'      => 'pldoc.external_url'
            ));

        if (!empty($order)) {
            $select->order($order . ' DESC');
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        $data = $this->getDbTable()->getAdapter()->fetchAll($select);

        $select->reset(Zend_Db_Select::COLUMNS);
        $select->reset(Zend_Db_Select::FROM);
        $select->reset(Zend_Db_Select::LIMIT_OFFSET);
        $select->reset(Zend_Db_Select::GROUP);

        $select->from(array('pldoc' => 'plugin_leads_documents'), array('count' => 'COUNT(pldoc.id)'));
        $count = $this->getDbTable()->getAdapter()->fetchRow($select);

        return array(
            'totalRecords' => $count['count'],
            'data' => $data,
            'offset' => $offset,
            'limit' => $limit
        );
    }

    /**
     * @param $leadId
     * @return array|null
     * @throws Exception
     */
    public function findFilesByLeadId($leadId)
    {
        $select = $this->getDbTable()->getAdapter()->select(Zend_Db_Table::SELECT_WITHOUT_FROM_PART)->from(array('pldoc' => 'plugin_leads_documents') ,
            array(
                'pldoc.id',
                'leadId'           => 'pldoc.lead_id',
                'fileStoredName'   => 'pldoc.file_stored_name',
                'fileHash'         => 'pldoc.file_hash',
                'originalFileName' => 'pldoc.original_file_name',
                'displayfilename'  => 'pldoc.display_file_name',
                'uploadedAt'       => 'pldoc.uploaded_at',
                'externalUrl'      => 'pldoc.external_url'
            ));
        $where = $this->getDbTable()->getAdapter()->quoteInto('pldoc.lead_id = ?', $leadId);
        $select->where($where);
        $select->order('pldoc.uploaded_at DESC');

       return $this->getDbTable()->getAdapter()->fetchAll($select);
    }

    /**
     * Change lead id
     *
     * @param array $leadIds lead ids
     * @param int $newLeadId new lead id
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeLeadId($leadIds, $newLeadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id IN (?)', $leadIds);
        $data = array('lead_id' => $newLeadId);
        $this->getDbTable()->getAdapter()->update('plugin_leads_documents', $data, $where);
    }

    /**
     * @param $id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
        return $this->getDbTable()->getAdapter()->delete('plugin_leads_documents', $where);
    }


}
