<?php

/**
 * LeadsDownloaderLogMapper.php
 *
 *
 * @method Leads_Mapper_LeadsDownloaderLogMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsDownloaderLogMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsDownloaderLogModel';

    protected $_dbTable = 'Leads_DbTable_LeadsDownloaderLogDbTable';

    /**
     * Save lead downloader log model to DB
     * @param $model Leads_Model_LeadsDownloaderLogModel
     * @return Leads_Model_LeadsDownloaderLogModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'file_name' => $model->getFileName(),
            'file_path' => $model->getFilePath(),
            'page_url' => $model->getPageUrl(),
            'action_log_id' => $model->getActionLogId(),
            'lead_id' => $model->getLeadId(),
            'page_id' => $model->getPageId(),
            'content' => $model->getContent()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }


    /**
     * Change lead id
     *
     * @param array $leadIds lead ids
     * @param int $newLeadId new lead id
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeLeadId($leadIds, $newLeadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id IN (?)', $leadIds);
        $data = array('lead_id' => $newLeadId);
        $this->getDbTable()->getAdapter()->update('plugin_leads_downloader_log', $data, $where);
    }

    /**
     * Delete lead form log record
     *
     * @param int $id lead id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_downloader_log', $where);

    }

}
