<?php

/**
 * LeadsEmailLinkAttachmentLogMapper.php
 *
 *
 * @method Leads_Mapper_LeadsEmailLinkAttachmentLogMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsEmailLinkAttachmentLogMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsEmailLinkAttachmentLogModel';

    protected $_dbTable = 'Leads_DbTable_LeadsEmailLinkAttachmentLogDbTable';

    /**
     * Save email link attachment log model
     * @param $model Leads_Model_LeadsEmailLinkAttachmentLogModel
     * @return Leads_Model_LeadsEmailLinkAttachmentLogModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'file_name' => $model->getFileName(),
            'file_link' => $model->getFileLink(),
            'file_folder_name' => $model->getFileFolderName(),
            'email_log_sent_hash' => $model->getEmailLogSentHash(),
            'file_hash' => $model->getFileHash(),
            'source_type' => $model->getSourceType(),
            'link_type' => $model->getLinkType(),
            'created_datetime' => $model->getCreatedDateTime(),
            'archived' => $model->getArchived(),
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Get by file hash and email log sent hash
     *
     * @param $emailLogSentHash
     * @param $fileHash
     * @return null
     */
    public function getByFileHashSentHash($emailLogSentHash, $fileHash)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('email_log_sent_hash = ?', $emailLogSentHash);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('file_hash = ?', $fileHash);

        return $this->_findWhere($where);
    }

    /**
     * Set archived by email hashed
     *
     * @param $emailLogSentHash
     * @return null
     */
    public function setArchivedByEmailHashed($emailLogSentHash)
    {
        $data = array('archived' => '1');
        $where = $this->getDbTable()->getAdapter()->quoteInto('email_log_sent_hash = ?', $emailLogSentHash);
        $this->getDbTable()->getAdapter()->update('plugin_leads_email_link_attachment_log', $data, $where);
    }


}
