<?php

/**
 * LeadsEmailSequenceStatusMapper.php
 *
 * @method Leads_Mapper_LeadsEmailSequenceStatusMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsEmailSequenceStatusMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsEmailSequenceStatusModel';

    protected $_dbTable = 'Leads_DbTable_LeadsEmailSequenceStatusDbTable';

    /**
     * Save email sequence to DB
     * @param $model Leads_Model_LeadsEmailSequenceStatusModel
     * @return Leads_Model_LeadsEmailSequenceStatusModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'lead_id' => $model->getLeadId(),
            'sequence_id' => $model->getSequenceId(),
            'cart_status_settings_id' => $model->getCartStatusSettingsId(),
            'sequence_applied_date' => $model->getSequenceAppliedDate()
        );

        $sequenceModel = $this->getLeadSequence($data['lead_id'], $data['sequence_id'], $data['cart_status_settings_id']);
        if (!$sequenceModel instanceof Leads_Model_LeadsEmailSequenceStatusModel) {
            $this->getDbTable()->insert($data);
        }

        return $model;
    }

    /**
     * Get lead sequence by lead id and sequence id and settings id
     *
     * @param int $leadId lead id
     * @param int $sequenceId sequence id
     * @param int $cartStatusSettingsId cart status id
     *
     * @return array
     */
    public function getLeadSequence($leadId, $sequenceId, $cartStatusSettingsId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('sequence_id = ?', $sequenceId);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('cart_status_settings_id = ?', $cartStatusSettingsId);

        return $this->_findWhere($where);
    }

    /**
     * Delete lead sequence by lead id
     *
     * @param int $leadId lead id

     * @return int
     */
    public function deleteByLeadId($leadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_email_sequence_status', $where);
    }

    /**
     * Delete lead sequence by lead id, sequence id
     *
     * @param int $leadId lead id
     * @param int $sequenceId sequence id
     * @return int
     */
    public function deleteByLeadIdSequenceId($leadId, $sequenceId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('sequence_id = ?', $sequenceId);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_email_sequence_status', $where);
    }

    /**
     * Change lead id
     *
     * @param int $oldLeadId old lead id
     * @param int $newLeadId new lead id
     * @param int $sequenceIds sequence ids
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeEmailSequenceLead($oldLeadId, $newLeadId, $sequenceIds)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $oldLeadId);
        $data = array('lead_id' => $newLeadId);
        $where .= ' AND '.$this->getDbTable()->getAdapter()->quoteInto('sequence_id IN (?)', $sequenceIds);
        $this->getDbTable()->getAdapter()->update('plugin_leads_email_sequence_status', $data, $where);
    }

    /**
     * Delete lead sequence by lead id, sequence id, cart status id
     *
     * @param int $leadId lead id
     * @param int $sequenceId sequence id
     * @param int $cartStatusSettingsId cart status id
     * @return mixed
     * @throws Exception
     */
    public function delete($leadId, $sequenceId, $cartStatusSettingsId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('sequence_id = ?', $sequenceId);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('cart_status_settings_id = ?', $cartStatusSettingsId);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_email_sequence_status', $where);
    }

}
