<?php

/**
 * LeadsEventRulesConfigMapper.php
 *
 *
 * @method Leads_Mapper_LeadsEventRulesConfigMapper  getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsEventRulesConfigMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsEventRulesConfigModel';

    protected $_dbTable = 'Leads_DbTable_LeadsEventRulesConfigDbTable';

    /**
     * Save lead note model to DB
     * @param $model Leads_Model_LeadsEventRulesConfigModel
     * @return Leads_Model_LeadsEventRulesConfigModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'rule_id' => $model->getRuleId(),
            'field_name' => $model->getFieldName(),
            'rule_comparison_operator' => $model->getRuleComparisonOperator(),
            'field_value' => $model->getFieldValue()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * get form rules
     * @param int $ruleId rule id
     * @return array
     */
    public function getFormFieldsByRuleId($ruleId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plerc.rule_id = ?', $ruleId);

        $select = $this->getDbTable()->getAdapter()->select()->from(array('plerc' => 'plugin_leads_event_rules_config'),
            array('plerc.field_name', 'plerc.id', 'plerc.rule_comparison_operator', 'plerc.field_value'))
            ->where($where)->order('plerc.field_name');

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Delete by rule id
     *
     * @param int $ruleId rule id
     * @return int
     */
    public function deleteByRuleId($ruleId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('rule_id = ?', $ruleId);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_event_rules_config', $where);
    }

    /**
     * Delete form rules config record
     *
     * @param int $id config id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_event_rules_config', $where);

    }

}
