<?php

/**
 * LeadsFilterPresetMapper.php
 *
 *
 * @method Leads_Mapper_LeadsFilterPresetMapper  getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsFilterPresetMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsFilterPresetModel';

    protected $_dbTable = 'Leads_DbTable_LeadsFilterPresetDbTable';

    /**
     * Save lead stage model to DB
     * @param $model Leads_Model_LeadsFilterPresetModel
     * @return Leads_Model_LeadsFilterPresetModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'creator_id' => $model->getCreatorId(),
            'filter_preset_name' => $model->getFilterPresetName(),
            'filter_preset_data' => $model->getFilterPresetData(),
            'is_default' => $model->getIsDefault(),
            'access' => $model->getAccess(),
            'filter_preset_type' => $model->getFilterPresetType()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Find preset by id and creator id
     *
     * @param int $id preset id
     * @param int $creatorId system user id
     * @return null
     */
    public function findByIdRole($id, $creatorId = 0)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
        if (!empty($creatorId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('creator_id = ?', $creatorId);
        }

        return $this->_findWhere($where);
    }

    /**
     * Reset default preset state
     *
     * @param int $creatorId system user id
     * @param string $presetType filter preset type
     * @return int
     * @throws Zend_Db_Adapter_Exception
     */
    public function resetDefaultByCreatorId($creatorId, $presetType)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('creator_id = ?', $creatorId);
        $where .= ' AND '.$this->getDbTable()->getAdapter()->quoteInto('filter_preset_type = ?', $presetType);

        return $this->getDbTable()->getAdapter()->update('plugin_leads_filter_preset', array('is_default' => '0'),
            $where);
    }

    /**
     * Get default preset by creator id
     *
     * @param int $creatorId system user id
     * @param string $presetType preset type
     * @return null
     */
    public function getDefaultPreset($creatorId, $presetType)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('creator_id = ?', $creatorId);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('is_default = ?', '1');
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('filter_preset_type = ?', $presetType);

        return $this->_findWhere($where);
    }

    /**
     * Find preset by name using preset type
     *
     * @param string $presetName filter preset name
     * @param string $presetType filter preset type
     * @return null
     */
    public function getByNameAndType($presetName, $presetType)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('filter_preset_name = ?', $presetName);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('filter_preset_type = ?', $presetType);

        return $this->_findWhere($where);
    }

    /**
     * Delete lead record
     *
     * @param int $id lead id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_filter_preset', $where);

    }

}
