<?php

/**
 * LeadsFormRulesConfigMapper.php
 *
 *
 * @method Leads_Mapper_LeadsFormRulesConfigMapper  getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsFormRulesConfigMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsFormRulesConfigModel';

    protected $_dbTable = 'Leads_DbTable_LeadsFormRulesConfigDbTable';

    /**
     * Save lead note model to DB
     * @param $model Leads_Model_LeadsFormRulesConfigModel
     * @return Leads_Model_LeadsFormRulesConfigModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'rule_id' => $model->getRuleId(),
            'field_name' => $model->getFieldName(),
            'rule_comparison_operator' => $model->getRuleComparisonOperator(),
            'field_value' => $model->getFieldValue()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Delete by rule id
     *
     * @param int $ruleId rule id
     * @return int
     */
    public function deleteByRuleId($ruleId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('rule_id = ?', $ruleId);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_form_rules_config', $where);
    }

    /**
     * Delete form rules config record
     *
     * @param int $id config id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_form_rules_config', $where);

    }

}
