<?php

/**
 * LeadsFormRulesGeneralConfigMapper.php
 *
 *
 * @method Leads_Mapper_LeadsFormRulesGeneralConfigMapper  getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsFormRulesGeneralConfigMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsFormRulesGeneralConfigModel';

    protected $_dbTable = 'Leads_DbTable_LeadsFormRulesGeneralConfigDbTable';

    /**
     * Save lead note model to DB
     * @param $model Leads_Model_LeadsFormRulesGeneralConfigModel
     * @return Leads_Model_LeadsFormRulesGeneralConfigModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'form_name' => $model->getFormName(),
            'form_type' => $model->getFormType(),
            'rule_name' => $model->getRuleName(),
            'created_at' => $model->getCreatedAt(),
            'creator_id' => $model->getCreatorId(),
            'updated_at' => $model->getUpdatedAt(),
            'editor_id' => $model->getEditorId()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Get form rules data
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param bool $withTimezoneOffset select data with processed dates based on logged user timezone
     * @return array
     */
    public function fetchAll(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $withoutCount = false,
        $singleRecord = false,
        $withTimezoneOffset = false
    ) {

        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');

        $createdAt = 'plfrgc.created_at';

        if ($withTimezoneOffset === true) {
            $createdAt = new Zend_Db_Expr("CONVERT_TZ(`plfrgc`.`created_at`, '+00:00', '$mysqlOffset')");
        }

        $updatedAt = 'plfrgc.updated_at';

        if ($withTimezoneOffset === true) {
            $updatedAt = new Zend_Db_Expr("CONVERT_TZ(`plfrgc`.`updated_at`, '+00:00', '$mysqlOffset')");
        }

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plfrgc' => 'plugin_leads_form_rules_general_config'),
                array(
                    'plfrgc.id',
                    'plfrgc.form_name',
                    'plfrgc.form_type',
                    'plfrgc.rule_name',
                    'plfrgc.creator_id',
                    'createdAt' => $createdAt,
                    'plfrgc.editor_id',
                    'updatedAt' => $updatedAt,
                    'creatorName' => 'u.full_name',
                    'editorName' => 'uc.full_name',
                    'actionTypes' => new Zend_Db_Expr('GROUP_CONCAT(DISTINCT(plfra.action_type))')
                )
            )
            ->joinLeft(array('plfra' => 'plugin_leads_form_rules_actions'), 'plfra.rule_id=plfrgc.id', array())
            ->joinLeft(array('u' => 'user'), 'u.id=plfrgc.creator_id', array())
            ->joinLeft(array('uc' => 'user'), 'uc.id=plfrgc.editor_id', array());


        $select->group('plfrgc.id');
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::LIMIT_COUNT);

            $count = array('count' => new Zend_Db_Expr('COUNT(DISTINCT(plfrgc.id))'));

            $select->from(array('plfrgc' => 'plugin_leads_form_rules_general_config'), $count)
                ->joinLeft(array('plfra' => 'plugin_leads_form_rules_actions'), 'plfra.rule_id=plfrgc.id', array())
                ->joinLeft(array('u' => 'user'), 'u.id=plfrgc.creator_id', array())
                ->joinLeft(array('uc' => 'user'), 'uc.id=plfrgc.editor_id', array());


            $select =  $this->getDbTable()->getAdapter()->select()
                ->from(
                    array('subres' => $select),
                    array('count' => 'SUM(count)')
                );

            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }

    /**
     * Get assigned form rules
     *
     * @param string $formName form name
     * @param string $formType form type
     * @return array
     */
    public function getFormRules($formName, $formType)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plfrgc.form_name = ?', $formName);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plfrgc.form_type = ?', $formType);

        $select = $this->getDbTable()->getAdapter()->select()->from(array('plfrgc' => 'plugin_leads_form_rules_general_config'),
            array('uniqueKey' => new Zend_Db_Expr("CONCAT(plfrgc.id, '_', plfrc.id)"), 'plfrgc.id', 'plfrgc.form_name', 'plfrc.field_name', 'plfrc.rule_comparison_operator', 'plfrc.field_value'))
            ->join(array('plfrc' => 'plugin_leads_form_rules_config'), 'plfrc.rule_id=plfrgc.id', array())
            ->where($where);

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * get form rules
     * @param int $ruleId rule id
     * @return array
     */
    public function getFormFieldsByRuleId($ruleId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plfrc.rule_id = ?', $ruleId);

        $select = $this->getDbTable()->getAdapter()->select()->from(array('plfrc' => 'plugin_leads_form_rules_config'),
            array('plfrc.field_name', 'plfrc.id', 'plfrc.rule_comparison_operator', 'plfrc.field_value'))
            ->where($where)->order('plfrc.field_name');

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * get form rules
     * @param int $ruleId rule id
     * @return array
     */
    public function getFormActionsByRuleId($ruleId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plfra.rule_id = ?', $ruleId);

        $select = $this->getDbTable()->getAdapter()->select()->from(array('plfra' => 'plugin_leads_form_rules_actions'),
            array('plfra.action_type', 'plfra.id', 'plfra.action_config'))
            ->where($where);

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Get system forms
     * @return array
     */
    public function getSystemForms()
    {
        $select = $this->getDbTable()->getAdapter()->select()->from('form', array('key' => new Zend_Db_Expr("CONCAT(name, '_', 'internal')"), 'name', 'formType' => new Zend_Db_Expr('CONCAT("internal")')));
        $select->order('name');
        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    public function getAllForms()
    {
        $selectInternalForms = $this->getDbTable()->getAdapter()->select()->from('form', array('key' => new Zend_Db_Expr("CONCAT(name, '_', 'internal')"), 'name', 'formType' => new Zend_Db_Expr('CONCAT("internal")')));

        $where = $this->getDbTable()->getAdapter()->quoteInto('form_type = ?', Leads_Model_LeadsFormRulesGeneralConfigModel::FORM_TYPE_EXTERNAL);
        $selectExternalForms = $this->getDbTable()->getAdapter()->select()->from(array('plbfgc' => 'plugin_leads_builder_form_general_config'), array('key' => new Zend_Db_Expr("CONCAT(form_name, '_', 'external')"), 'name' => 'form_name', 'formType' => new Zend_Db_Expr('CONCAT("external")')));
        $selectExternalForms->where($where);
        $select = $this->getDbTable()->select()
            ->union(array($selectInternalForms, $selectExternalForms))->order('key');

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Check if rule already exists
     *
     * @param string $formName form name
     * @param string $formType form type (internal, external)
     * @param string $ruleName unique rule name
     * @return null
     */
    public function checkRuleExist($formName, $formType, $ruleName)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('form_name = ?', $formName);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('form_type = ?', $formType);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('rule_name = ?', $ruleName);

        return $this->_findWhere($where);
    }


    /**
     * Set not assigned creator id
     *
     * @param int $creatorId creator id
     * @throws Zend_Db_Adapter_Exception
     */
    public function setNotAssignedCreator($creatorId)
    {
        $where = new Zend_Db_Expr('creator_id IS NULL');

        $data = array('creator_id' => $creatorId);
        $this->getDbTable()->getAdapter()->update('plugin_leads_form_rules_general_config', $data, $where);
    }

    /**
     * Set not assigned editor id
     *
     * @param int $editorId editor id
     * @throws Zend_Db_Adapter_Exception
     */
    public function setNotAssignedEditor($editorId)
    {
        $where = new Zend_Db_Expr('editor_id IS NULL');

        $data = array('editor_id' => $editorId);
        $this->getDbTable()->getAdapter()->update('plugin_leads_form_rules_general_config', $data, $where);
    }

    /**
     * Delete form rules general config record
     *
     * @param int $id config id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_form_rules_general_config', $where);

    }

}
