<?php

/**
 * LeadsHasEmailSequenceMapper.php
 *
 * @method Leads_Mapper_LeadsHasEmailSequenceMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsHasEmailSequenceMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsHasEmailSequenceModel';

    protected $_dbTable = 'Leads_DbTable_LeadsHasEmailSequenceDbTable';

    /**
     * Save email sequence to DB
     * @param $model Leads_Model_LeadsHasEmailSequenceModel
     * @return Leads_Model_LeadsHasEmailSequenceModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'lead_id' => $model->getLeadId(),
            'sequence_id' => $model->getSequenceId(),
            'sequence_date' => $model->getSequenceDate()
        );

        $sequenceModel = $this->getLeadSequence($data['lead_id'], $data['sequence_id']);
        if (!$sequenceModel instanceof Leads_Model_LeadsHasEmailSequenceModel) {
            $this->getDbTable()->insert($data);
        }

        return $model;
    }

    /**
     * Get whole sequence config
     *
     * @param bool $onlyCartStatuses flag to get only cart statuses config
     * @param array $cartStatuses limit cart statuses
     *
     * @return array
     */
    public function getEmailSequenceConfig($onlyCartStatuses = true, $cartStatuses = array())
    {
        $emailSequenceEnabled = Tools_LeadTools::verifyPluginEnabled(Tools_LeadTools::SEQUENCE_PLUGIN);
        if ($emailSequenceEnabled) {
            $this->getDbTable()->getAdapter()->query(new Zend_Db_Expr('SET SESSION group_concat_max_len = 1000000;'));
            $where = $this->getDbTable()->getAdapter()->quoteInto('pet.trigger_enabled = ?', '1');
            $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('pcs.sequenceOnly = ?', '1');
            if ($onlyCartStatuses === true) {
                $where .= ' AND '.$this->getDbTable()->getAdapter()->quoteInto('pcs.cartStatus IN (?)', Emailsequence::$cartStatuses);
            } else {
                $where .= ' AND '.$this->getDbTable()->getAdapter()->quoteInto('pcs.cartStatus IN (?)', $cartStatuses);
            }
            $select = $this->getDbTable()->getAdapter()->select()
                ->from(array('plhes' => 'plugin_leads_has_email_sequence'), array(
                    'lead_emails' => new Zend_Db_Expr('GROUP_CONCAT(DISTINCT (pld.lead_email))'),
                    'pet.trigger_id', 'pcs.cartStatus',
                    'pcs.period', 'pcs.productsIds', 'pcs.emailTemplate', 'pcs.emailFrom', 'pcs.productsRule',
                    'pcs.emailMessage', 'pcs.schedulePeriod', 'pcs.lagType', 'pcs.sequenceOnly', 'pcs.triggerName', 'pcs.schedulePeriodType',
                    'sequenceId' => 'pet.sequence_settings_id', 'leadIds' => new Zend_Db_Expr('GROUP_CONCAT(DISTINCT(plhes.lead_id))'), 'pcs.subject', 'pcs.preheader'
                ))
                ->join(array('pld' => 'plugin_leads_data'), 'plhes.lead_id=pld.id', array())
                ->join(array('pet' => 'plugin_emailsequence_triggers'), 'pet.sequence_settings_id=plhes.sequence_id', array())
                ->join(array('pcs' => 'plugin_cartstatusemail_settings'), 'pcs.id=pet.trigger_id', array())
                ->where($where)
                ->group('pet.trigger_id');

            return $this->getDbTable()->getAdapter()->fetchAll($select);
        }
        return array();

    }


    /**
     * Get whole sequence config
     *
     * @param bool $onlyCartStatuses flag to get only cart statuses config
     * @param array $cartStatuses limit cart statuses
     *
     * @return array
     */
    public function getEmailSequenceConfigNoCart($onlyCartStatuses = true, $cartStatuses = array())
    {
        $emailSequenceEnabled = Tools_LeadTools::verifyPluginEnabled(Tools_LeadTools::SEQUENCE_PLUGIN);
        if ($emailSequenceEnabled) {
            $this->getDbTable()->getAdapter()->query(new Zend_Db_Expr('SET SESSION group_concat_max_len = 1000000;'));
            $where = $this->getDbTable()->getAdapter()->quoteInto('pet.trigger_enabled = ?', '1');
            $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('pcs.sequenceOnly = ?', '1');
            if ($onlyCartStatuses === true) {
                $where .= ' AND '.$this->getDbTable()->getAdapter()->quoteInto('pcs.cartStatus IN (?)', Emailsequence::$cartStatuses);
            } else {
                $where .= ' AND '.$this->getDbTable()->getAdapter()->quoteInto('pcs.cartStatus IN (?)', $cartStatuses);
            }
            $where .= ' AND '.new Zend_Db_Expr('sequence_date >  CASE pcs.lagType WHEN "hour" THEN date_sub(NOW(), INTERVAL pcs.period HOUR) WHEN "day" THEN date_sub(NOW(), INTERVAL pcs.period DAY)  WHEN "month" THEN date_sub(NOW(), INTERVAL pcs.period MONTH) END');

            $select = $this->getDbTable()->getAdapter()->select()
                ->from(array('plhes' => 'plugin_leads_has_email_sequence'), array(new Zend_Db_Expr('CONCAT(plhes.lead_id, pet.sequence_settings_id, pet.trigger_id)'),
                    'pld.lead_email',
                    'pld.lead_first_name',
                    'pld.lead_last_name',
                    'pld.lead_mobile_country_code_value',
                    'pld.lead_mobile',
                    'pld.lead_phone_country_code_value',
                    'pld.lead_phone',
                    'pet.trigger_id', 'pcs.cartStatus',
                    'pcs.period', 'pcs.productsIds', 'pcs.emailTemplate', 'pcs.emailFrom', 'pcs.productsRule',
                    'pcs.emailMessage', 'pcs.schedulePeriod', 'pcs.lagType', 'pcs.sequenceOnly', 'pcs.triggerName', 'pcs.schedulePeriodType',
                    'sequenceId' => 'pet.sequence_settings_id', 'plhes.lead_id', 'plhes.sequence_date','pcs.subject', 'pcs.type', 'pcs.preheader')
                )
                ->join(array('pld' => 'plugin_leads_data'), 'plhes.lead_id=pld.id', array())
                ->join(array('pet' => 'plugin_emailsequence_triggers'), 'pet.sequence_settings_id=plhes.sequence_id', array())
                ->join(array('pcs' => 'plugin_cartstatusemail_settings'), 'pcs.id=pet.trigger_id', array())
                ->where($where);

            return $this->getDbTable()->getAdapter()->fetchAssoc($select);
        }
        return array();

    }

    /**
     * get all processed triggers using prepared keys
     *
     * @param array $triggerSequenceKeys trigger sequence keys
     * @return array
     */
    public function getProcessedTriggers($triggerSequenceKeys)
    {
        $where =  new Zend_Db_Expr('CONCAT(lead_id, sequence_id, cart_status_settings_id) IN'). $this->getDbTable()->getAdapter()->quoteInto('(?)', $triggerSequenceKeys);
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plhes' => 'plugin_leads_email_sequence_status'),
                array(new Zend_Db_Expr('CONCAT(lead_id, sequence_id, cart_status_settings_id)')))
            ->where($where);

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Check cart status queue
     *
     * @param int $externalId external id
     * @param string $externalType external type
     * @param int $sequenceId sequence id
     * @param int $cartStatusId cart status id
     * @return mixed
     */
    public function checkCartStatusQueue($externalId, $externalType, $sequenceId, $cartStatusId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('externalId = ?', $externalId);
        $where .= ' AND '.$this->getDbTable()->getAdapter()->quoteInto('externalType = ?', $externalType);
        $where .= ' AND '.$this->getDbTable()->getAdapter()->quoteInto('sequence_id = ?', $sequenceId);
        $where .= ' AND '.$this->getDbTable()->getAdapter()->quoteInto('cartStatusId = ?', $cartStatusId);

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plq' => 'plugin_cartstatusemail_queue'),
                array('id'))
            ->where($where);

        return $this->getDbTable()->getAdapter()->fetchRow($select);
    }

    /**
     * Get all leads with sequence id
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount without count flag
     * @param bool $singleRecord if true return single record
     * @return array
     */
    public function fetchAll(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $withoutCount = false,
        $singleRecord = false
    ) {
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plhes' => 'plugin_leads_has_email_sequence'),
                array(
                    'plhes.lead_id',
                    'plhes.sequence_id'
                )
            );

        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRecord === true) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::GROUP);

            $select->from(array('plhes' => 'plugin_leads_has_email_sequence'),
                array('count' => 'COUNT(plhes.lead_id)'));
            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        }

        return $data;
    }


    /**
     * Get lead sequence by lead id and sequence id
     *
     * @param int $leadId lead id
     * @param int $sequenceId sequence id
     *
     * @return array
     */
    public function getLeadSequence($leadId, $sequenceId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('sequence_id = ?', $sequenceId);

        return $this->_findWhere($where);
    }

    /**
     * Get sequence by lead id
     *
     * @param int $leadId lead id
     * @param bool $fetchAll fetch all mode
     * @return array
     */
    public function getLeadSequenceByLeadId($leadId, $fetchAll = false)
    {
        $pluginExistsAndEnabled = Tools_LeadTools::verifyPluginEnabled(Tools_LeadTools::SEQUENCE_PLUGIN);
        if ($pluginExistsAndEnabled === true) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('plhes.lead_id = ?', $leadId);
            $select = $this->getDbTable()->getAdapter()->select()->from(array('plhes' => 'plugin_leads_has_email_sequence'),
                array('plhes.sequence_id', 'plhes.lead_id', 'pes.sequence_name'))
                ->join(array('pes' => 'plugin_emailsequence_settings'), 'pes.id=plhes.sequence_id', array())
                ->where($where);
            if ($fetchAll === false) {
                return $this->getDbTable()->getAdapter()->fetchAssoc($select);
            }
            return $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        return array();
    }

    /**
     * Delete lead sequence
     *
     * @param int $leadId lead id
     * @param int $sequenceId sequence id
     * @return int
     */
    public function findByLeadIdSequenceId($leadId, $sequenceId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('sequence_id = ?', $sequenceId);

        return $this->_findWhere($where);
    }

    /**
     * Delete lead sequence
     *
     * @param int $leadId lead id
     * @param int $sequenceId sequence id
     * @return mixed
     * @throws Exception
     */
    public function delete($leadId, $sequenceId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('sequence_id = ?', $sequenceId);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_has_email_sequence', $where);
    }

    /**
     * Get data from cart status queue
     *
     * @param int $externalId external id (lead id)
     * @param string $externalType external type
     * @param int $sequenceId sequence id
     * @param array $status statuses
     * @return array
     */
    public function getFromCartStatusQueue($externalId, $externalType, $sequenceId, $status = array())
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('externalId = ?', $externalId);
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('externalType = ?', $externalType);
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('sequence_id = ?', $sequenceId);
        if (!empty($status)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('status IN (?)', $status);
        }

        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_cartstatusemail_queue')->where($where);

        return $this->getDbTable()->getAdapter()->fetchAll($select);
    }

    /**
     * Delete from cart status queue
     *
     * @param int $externalId external id (lead id)
     * @param string $externalType external type
     * @param int $sequenceId sequence id
     * @param string $status status
     * @return int
     */
    public function deleteFromCartStatusQueue($externalId, $externalType, $sequenceId, $status)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('externalId = ?', $externalId);
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('externalType = ?', $externalType);
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('sequence_id = ?', $sequenceId);
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('status = ?', $status);

        return $this->getDbTable()->getAdapter()->delete('plugin_cartstatusemail_queue', $where);
    }

    /**
     * email sequence log part
     *
     */

    /**
     * Get all email sequence log data
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount without count flag
     * @param bool $withTimezoneOffset select data with processed dates based on logged user timezone
     * @return array
     */
    public function getEmailSequenceLog (
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $withoutCount = false,
        $withTimezoneOffset = true
    ) {

        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');

        /**
         * Server offset
         */
//        $defaultTimezone = date_default_timezone_get();
//        if (empty($defaultTimezone)) {
            $defaultTimezone = 'UTC';
      //  }

        $serverOffset = '+00:00';
//        if ($defaultTimezone !== 'UTC') {
//            $userTimeZone = new DateTimeZone($defaultTimezone);
//            $currentUserDate = new DateTime('now', $userTimeZone);
//            $serverOffset = $currentUserDate->format('P');
       // }
        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');
        $sentAt = 'pcq.sentAt';
        $delayedTo = 'pcq.delayedTo';

        if ($withTimezoneOffset === true) {
            $sentAt = new Zend_Db_Expr("CONVERT_TZ(`pcq`.`sentAt`, '$serverOffset', '$mysqlOffset')");
            $delayedTo = new Zend_Db_Expr("CONVERT_TZ(`pcq`.`delayedTo`, '$serverOffset', '$mysqlOffset')");
            $sentAtSingleDate = new Zend_Db_Expr("DATE_FORMAT($sentAt,'%Y-%m-%d')");
            $delayedAtSingleDate = new Zend_Db_Expr("DATE_FORMAT($delayedTo,'%Y-%m-%d')");
        }

        $columns = array(
            'pcq.id',
            'pcq.cartStatusId',
            'pcq.cartStatus',
            'pcq.cartId',
            'pcq.userEmail',
            'pcq.userFullName',
            'pcq.externalId',
            'pcq.externalType',
            'pcq.sequence_id',
            'pcq.status',
            'sentAt' => $sentAt,
            'delayedTo' => $delayedTo,
            'sentAtSingleDate' => $sentAtSingleDate,
            'delayedAtSingleDate' => $delayedAtSingleDate,
            'sequenceName' => 'pes.sequence_name',
            'pcs.triggerName',
            'pcs.type',
            'pcs.preheader'
        );


        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pcq' => 'plugin_cartstatusemail_queue'),
                $columns
            )->joinLeft(array('pes' => 'plugin_emailsequence_settings'), 'pes.id=pcq.sequence_id', array())
            ->joinLeft(array('pcs' => 'plugin_cartstatusemail_settings'), 'pcq.cartStatusId=pcs.id', array());

        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        $data = $this->getDbTable()->getAdapter()->fetchAll($select);

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::GROUP);

            $select->from(array('pcq' => 'plugin_cartstatusemail_queue'),
                array('count' => 'COUNT(pcq.id)'));
            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        }

        return $data;
    }

    /**
     * Change lead id
     *
     * @param int $oldLeadId old lead id
     * @param int $newLeadId new lead id
     * @param int $sequenceIds sequence ids
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeEmailSequenceLead($oldLeadId, $newLeadId, $sequenceIds)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $oldLeadId);
        $data = array('lead_id' => $newLeadId);
        $where .= ' AND '.$this->getDbTable()->getAdapter()->quoteInto('sequence_id IN (?)', $sequenceIds);
        $this->getDbTable()->getAdapter()->update('plugin_leads_has_email_sequence', $data, $where);
    }

}
