<?php

/**
 * LeadsImapConfigMapper.php
 *
 *
 * @method Leads_Mapper_LeadsImapConfigMapper  getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsImapConfigMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsImapConfigModel';

    protected $_dbTable = 'Leads_DbTable_LeadsImapConfigDbTable';

    /**
     * Save lead imap config to DB
     * @param $model Leads_Model_LeadsImapConfigModel
     * @return Leads_Model_LeadsImapConfigModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'service_name' => $model->getServiceName(),
            'owner_id' => $model->getOwnerId(),
            'host'     => $model->getHost(),
            'encryption' => $model->getEncryption(),
            'login'    => $model->getLogin(),
            'password' =>  new Zend_Db_Expr("AES_ENCRYPT('" . addslashes($model->getPassword()) . "', '" . Tools_LeadInboundEmailTools::getPasswordKey(). "')"),
            'initial_data_processed' => $model->getInitialDataProcessed(),
            'last_processed_date' => $model->getLastProcessedDate(),
            'process_all_emails' => $model->getProcessAllEmails(),
            'notification_sent' => $model->getNotificationSent()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Get lead imap config data with info
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $singleRow single row result
     * @return array
     */
    public function fetchAllData($where = null, $order = null, $limit = null, $offset = null, $singleRow = false, $withPassword = false)
    {
        $fieldsToSelect = array(
            'plic.id',
            'plic.service_name',
            'plic.owner_id',
            'plic.host',
            'plic.encryption',
            'plic.login',
            'plic.initial_data_processed',
            'plic.last_processed_date',
            'plic.process_all_emails',
            'plic.notification_sent'
        );

        if ($withPassword === true) {
            $fieldsToSelect['password'] = new Zend_Db_Expr(
                $this->getDbTable()->getAdapter()->quoteInto("AES_DECRYPT(plic.password,?)",
                    Tools_LeadInboundEmailTools::getPasswordKey())
            );
        }

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plic' => 'plugin_leads_imap_config'),
                $fieldsToSelect
            );


        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRow === true) {
            return  $this->getDbTable()->getAdapter()->fetchRow($select);
        }

        $data = $this->getDbTable()->getAdapter()->fetchAll($select);

        $select->reset(Zend_Db_Select::COLUMNS);
        $select->reset(Zend_Db_Select::FROM);
        $select->reset(Zend_Db_Select::LIMIT_OFFSET);
        $select->reset(Zend_Db_Select::GROUP);

        $select->from(array('plic' => 'plugin_leads_imap_config'), array('count' => 'COUNT(plic.id)'));
        $count = $this->getDbTable()->getAdapter()->fetchRow($select);

        return array(
            'totalRecords' => $count['count'],
            'data' => $data,
            'offset' => $offset,
            'limit' => $limit
        );
    }

    public function getRules()
    {
        $select = $this->getDbTable()->getAdapter()->select()->from(array('plic' => 'plugin_leads_imap_config'),
            array('id', 'service_name', 'u.full_name'))
            ->joinLeft(array('u' => 'user'), 'u.id=plic.owner_id', array());

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Delete lead type record
     *
     * @param int $id lead type id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_imap_config', $where);

    }

}
