<?php

/**
 * LeadsImapRulesConfigMapper.php
 *
 *
 * @method Leads_Mapper_LeadsImapRulesConfigMapper  getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsImapRulesConfigMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsImapRulesConfigModel';

    protected $_dbTable = 'Leads_DbTable_LeadsImapRulesConfigDbTable';

    /**
     * Save lead note model to DB
     * @param $model Leads_Model_LeadsImapRulesConfigModel
     * @return Leads_Model_LeadsImapRulesConfigModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'rule_name' => $model->getRuleName(),
            'imap_config_id' => $model->getImapConfigId(),
            'rule_email' => $model->getRuleEmail(),
            'created_at' => $model->getCreatedAt(),
            'creator_id' => $model->getCreatorId(),
            'updated_at' => $model->getUpdatedAt(),
            'editor_id' => $model->getEditorId(),
            'sub_rules_config' => $model->getSubRulesConfig()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Get imap rules
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param bool $withTimezoneOffset select data with processed dates based on logged user timezone
     * @return array
     */
    public function fetchAll(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $withoutCount = false,
        $singleRecord = false,
        $withTimezoneOffset = false
    ) {

        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');

        $createdAt = 'plirc.created_at';

        if ($withTimezoneOffset === true) {
            $createdAt = new Zend_Db_Expr("CONVERT_TZ(`plirc`.`created_at`, '+00:00', '$mysqlOffset')");
        }

        $updatedAt = 'plirc.updated_at';

        if ($withTimezoneOffset === true) {
            $updatedAt = new Zend_Db_Expr("CONVERT_TZ(`plirc`.`updated_at`, '+00:00', '$mysqlOffset')");
        }

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plirc' => 'plugin_leads_imap_rules_config'),
                array(
                    'plirc.id',
                    'plirc.rule_name',
                    'plirc.creator_id',
                    'plirc.rule_email',
                    'createdAt' => $createdAt,
                    'plirc.editor_id',
                    'updatedAt' => $updatedAt,
                    'creatorName' => 'u.full_name',
                    'editorName' => 'uc.full_name',
                    'serviceName' => 'plic.service_name',
                    'subRulesConfig' => 'plirc.sub_rules_config',
                    'imapConfigId' => 'plirc.imap_config_id',
                )
            )
            ->joinLeft(array('u' => 'user'), 'u.id=plirc.creator_id', array())
            ->joinLeft(array('uc' => 'user'), 'uc.id=plirc.editor_id', array())
            ->joinLeft(array('plic' => 'plugin_leads_imap_config'), 'plic.id=plirc.imap_config_id', array());


        $select->group('plirc.id');
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::LIMIT_COUNT);

            $count = array('count' => new Zend_Db_Expr('COUNT(DISTINCT(plirc.id))'));

            $select->from(array('plirc' => 'plugin_leads_imap_rules_config'), $count)
                ->joinLeft(array('u' => 'user'), 'u.id=plirc.creator_id', array())
                ->joinLeft(array('uc' => 'user'), 'uc.id=plirc.editor_id', array());


            $select = $this->getDbTable()->getAdapter()->select()
                ->from(
                    array('subres' => $select),
                    array('count' => 'SUM(count)')
                );

            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            if (empty($count['count'])) {
                $count['count'] = 0;
            }

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }

    /**
     * Find by imap config id
     *
     * @param int $imapConfigId imap config id
     * @return null
     */
    public function findByImapConfigId($imapConfigId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('imap_config_id = ?', $imapConfigId);
        return $this->_findWhere($where);
    }

    /**
     * Delete form rules config record
     *
     * @param int $id config id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_imap_rules_config', $where);

    }

}
