<?php

/**
 * LeadsIndustryMapper.php
 *
 *
 * @method Leads_Mapper_LeadsIndustryMapper  getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsIndustryMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsIndustryModel';

    protected $_dbTable = 'Leads_DbTable_LeadsIndustryDbTable';

    /**
     * Save industry model to DB
     * @param $model Leads_Model_LeadsIndustryModel
     * @return Leads_Model_LeadsIndustryModel
     */

    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'type' => $model->getType(),
            'label' => $model->getLabel()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }


    /**
     * Delete industry record
     *
     * @param int $id industry id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_industry', $where);

    }

    /**
     * Get lead type data with info
     *
     * @param string $search search keyword
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @return array
     */
    public function fetchAllData($where = null, $order = null, $limit = null, $offset = null, $withoutCount = false,
        $singleRecord = false, $search = null)
    {
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pli' => 'plugin_leads_industry'),
                array(
                    'pli.id',
                    'pli.type',
                    'pli.label'
                )
            );

        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($search)) {
            $select->where('type LIKE  ?', '%'.$search.'%');
            $select->orWhere('label LIKE  ?', '%'.$search.'%');
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);
        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }
        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::GROUP);

            $select->from(array('pli' => 'plugin_leads_industry'), array('count' => 'COUNT(pli.id)'));
            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        }

        return $data;
    }

    /**
     * Get all industries ids and labels as key pair
     * @param bool $keyAsType return industry type as key
     * @param array $order order by
     * @return array
     */
    public function getIndustriesPairs($keyAsType = false, $order = array())
    {
        if ($keyAsType) {
            $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_industry', array('type', 'id'));
        } else {
            $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_industry', array('id', 'label'));
        }

        if (!empty($order)) {
            $select->order($order);
        }

        return $this->getDbTable()->getAdapter()->fetchPairs($select);
    }
}

