<?php

/**
 * LeadsInternalNotificationsMapper.php
 *
 *
 * @method Leads_Mapper_LeadsInternalNotificationsMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsInternalNotificationsMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsInternalNotificationsModel';

    protected $_dbTable = 'Leads_DbTable_LeadsInternalNotificationsDbTable';

    /**
     * Save lead lead external notifications
     * @param $model Leads_Model_LeadsInternalNotificationsModel
     * @return Leads_Model_LeadsInternalNotificationsModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'status' => $model->getStatus(),
            'lead_id' => $model->getLeadId(),
            'additional_info' => $model->getAdditionalInfo(),
            'created_at' => $model->getCreatedAt(),
            'updated_at' => $model->getUpdatedAt()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    public function getNotSendNotifications()
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('status = ?', 'scheduled');
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plen' => 'plugin_leads_internal_notifications'),
                array(
                    'plen.*'
                )
            );

        $select->where($where);

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * @param int $id record id
     * @param string $status notification status
     */
    public function changeStatus($id, $status)
    {
        $data = array('status' => $status);
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
        $this->getDbTable()->update($data, $where);
    }

    /**
     * @param array $ids
     * @param string $status notification status
     */
    public function changeStatuses($ids, $status)
    {
        $data = array('status' => $status);
        $where = $this->getDbTable()->getAdapter()->quoteInto('id IN (?)', $ids);
        $this->getDbTable()->update($data, $where);
    }

}
