<?php

/**
 * LeadsIpUserProfileLogActionDetailsMapper.php
 *
 *
 * @method Leads_Mapper_LeadsIpUserProfileLogActionDetailsMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsIpUserProfileLogActionDetailsMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsIpUserProfileLogActionDetailsModel';

    protected $_dbTable = 'Leads_DbTable_LeadsIpUserProfileLogActionDetailsDbTable';

    /**
     * Save ip log action details model to DB
     * @param $model Leads_Model_LeadsIpUserProfileLogActionDetailsModel
     * @return Leads_Model_LeadsIpUserProfileLogActionDetailsModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
           'type' => $model->getType(),
           'domain_name' => $model->getDomainName(),
           'page_url' => $model->getPageUrl(),
           'page_title' => $model->getPageTitle(),
           'time_spent' => $model->getTimeSpent(),
           'page_generation_time' => $model->getPageGenerationTime(),
           'page_id_action' => $model->getPageIdAction(),
           'action_datetime' => $model->getActionDatetime(),
           'user_profile_log_id' => $model->getUserProfileLogId()
        );

        $id = $this->getDbTable()->insert($data);
        $model->setId($id);

        return $model;
    }


    /**
     * Get lead action details info for visits etc..
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param bool $withTimezoneOffset select data with processed dates based on logged user timezone
     * @return array
     */
    public function fetchAllForGrid(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $withoutCount = false,
        $singleRecord = false,
        $withTimezoneOffset = true
    ) {
        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');

        $createdAt = 'pliuplad.action_datetime';

        if ($withTimezoneOffset === true) {
            $createdAt = new Zend_Db_Expr("CONVERT_TZ(`pliuplad`.`action_datetime`, '+00:00', '$mysqlOffset')");
        }

        $selectFields =  array(
            'pliuplad.id',
            'userProfileLogId' => 'pliuplad.user_profile_log_id',
            'actionDateTime'=> $createdAt,
            'domainName' => 'pliuplad.domain_name',
            'pageUrl' => 'pliuplad.page_url',
            'pageTitle' => 'pliuplad.page_title',
            'timeSpent' => 'pliuplad.time_spent',
            'pageGenerationTime' => 'pliuplad.page_generation_time',
            'pageIdAction' => 'pliuplad.page_id_action'
        );

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pliuplad' => 'plugin_leads_ip_user_profile_log_actions_details'),
                $selectFields
            );
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::GROUP);

            $select->from(array('pliuplad' => 'plugin_leads_ip_user_profile_log_actions_details'), array('count' => 'COUNT(pliuplad.id)'));
            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }

    /**
     * Get lead action details info for visits etc..
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param bool $withTimezoneOffset select data with processed dates based on logged user timezone
     * @return array
     */
    public function fetchAllForGridWithActionLog(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $withoutCount = false,
        $singleRecord = false,
        $withTimezoneOffset = true
    ) {
        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');

        $createdAt = 'pliuplad.action_datetime';

        if ($withTimezoneOffset === true) {
            $createdAt = new Zend_Db_Expr("CONVERT_TZ(`pliuplad`.`action_datetime`, '+00:00', '$mysqlOffset')");
        }

        $selectFields =  array(
            'pliuplad.id',
            'userProfileLogId' => 'pliuplad.user_profile_log_id',
            'actionDateTime'=> $createdAt,
            'domainName' => 'pliuplad.domain_name',
            'pageUrl' => 'pliuplad.page_url',
            'pageTitle' => 'pliuplad.page_title',
            'timeSpent' => 'pliuplad.time_spent',
            'pageGenerationTime' => 'pliuplad.page_generation_time',
            'pageIdAction' => 'pliuplad.page_id_action'
        );

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pliuplad' => 'plugin_leads_ip_user_profile_log_actions_details'),
                $selectFields
            )->joinLeft(array('pliupl' => 'plugin_leads_ip_user_profile_log'), 'pliuplad.user_profile_log_id=pliupl.id', array());
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);
     
        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        return $data;

    }


    /**
     * get average page load
     *
     * @param int $leadId lead id
     * @return mixed
     */
    public function getPageAverageLoad($leadId)
    {
        $selectFields =  array(
            'averagePageGenerationTime' => new Zend_Db_Expr('SUM(pliuplad.page_generation_time)/COUNT(pliuplad.id)'),
        );
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pliupl' => 'plugin_leads_ip_user_profile_log'),
                $selectFields
            )->join(array('pliuplad' => 'plugin_leads_ip_user_profile_log_actions_details'), 'pliupl.id=pliuplad.user_profile_log_id', array());

        $where = $this->getDbTable()->getAdapter()->quoteInto('pliupl.lead_id = ?', $leadId);

        $select->where($where);

        return $this->getDbTable()->getAdapter()->fetchRow($select);
    }

    /**
     * Delete by record id
     *
     * @param int $id record id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_ip_user_profile_log_actions_details', $where);

    }

}
