<?php

/**
 * LeadsIpUserProfileLogMapper.php
 *
 *
 * @method Leads_Mapper_LeadsIpUserProfileLogMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsIpUserProfileLogMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsIpUserProfileLogModel';

    protected $_dbTable = 'Leads_DbTable_LeadsIpUserProfileLogDbTable';

    /**
     * Save lead task model to DB
     * @param $model Leads_Model_LeadsIpUserProfileLogModel
     * @return Leads_Model_LeadsIpUserProfileLogModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'visit_ip_address' => $model->getVisitIpAddress(),
            'website_id' => $model->getWebsiteId(),
            'visitor_id' => $model->getVisitorId(),
            'id_visit' => $model->getIdVisit(),
            'goal_conversions' => $model->getGoalConversions(),
            'site_currency' => $model->getSiteCurrency(),
            'server_date' => $model->getServerDate(),
            'piwik_user_id' => $model->getPiwikUserId(),
            'last_action_date_time' => $model->getLastActionDateTime(),
            'visitor_type' => $model->getVisitorType(),
            'visit_converted' => $model->getVisitConverted(),
            'visit_count' => $model->getVisitCount(),
            'first_action_timestamp' => $model->getFirstActionTimestamp(),
            'visit_ecommerce_status' => $model->getVisitEcommerceStatus(),
            'days_since_first_visit' => $model->getDaysSinceFirstVisit(),
            'days_since_last_ecommerce_order' => $model->getDaysSinceLastEcommerceOrder(),
            'visit_duration' => $model->getVisitDuration(),
            'searches' => $model->getSearches(),
            'actions' => $model->getActions(),
            'referrer_type' => $model->getReferrerType(),
            'referrer_type_name' => $model->getReferrerTypeName(),
            'referrer_name' => $model->getReferrerName(),
            'referrer_keyword' => $model->getReferrerKeyword(),
            'referrer_keyword_position' => $model->getReferrerKeywordPosition(),
            'referrer_url' => $model->getReferrerUrl(),
            'referrer_search_engine_url' => $model->getReferrerSearchEngineUrl(),
            'language_code' => $model->getLanguageCode(),
            'device_type' => $model->getDeviceType(),
            'device_brand' => $model->getDeviceBrand(),
            'device_model' => $model->getDeviceModel(),
            'operating_system' => $model->getOperatingSystem(),
            'operating_system_name' => $model->getOperatingSystemName(),
            'operating_system_code' => $model->getOperatingSystemCode(),
            'operating_system_version' => $model->getOperatingSystemVersion(),
            'browser_family' => $model->getBrowserFamily(),
            'browser_family_description' => $model->getBrowserFamilyDescription(),
            'browser' => $model->getBrowser(),
            'browser_name' => $model->getBrowserName(),
            'browser_code' => $model->getBrowserCode(),
            'browser_version' => $model->getBrowserVersion(),
            'events' => $model->getEvents(),
            'continent_code' => $model->getContinentCode(),
            'country_code' => $model->getCountryCode(),
            'region' => $model->getRegion(),
            'city' => $model->getCity(),
            'location' => $model->getLocation(),
            'latitude' => $model->getLatitude(),
            'longitude' => $model->getLongitude(),
            'resolution' => $model->getResolution(),
            'server_timestamp' => $model->getServerTimestamp(),
            'days_since_last_visit' => $model->getDaysSinceLastVisit(),
            'action_log_id' => $model->getActionLogId(),
            'lead_id' => $model->getLeadId()
        );

        $id = $this->getDbTable()->insert($data);
        $model->setId($id);

        return $model;
    }

    /**
     * Get by piwik visit id
     *
     * @param int $visitId visit id
     * @param int $leadId lead id
     * @return null
     */
    public function getByIdVisit($visitId, $leadId = 0)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id_visit = ?', $visitId);
        if (!empty($leadId)) {
            $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);
        }

        return $this->_findWhere($where);
    }

    /**
     * Get total visits for lead
     *
     * @param int $leadId lead id
     * @param string $order OPTIONAL An SQL ORDER clause.
     *
     * @return mixed
     */
    public function getTotalVisitsForLead($leadId, $order)
    {
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pliupl' => 'plugin_leads_ip_user_profile_log'), array('count' => 'COUNT(pliupl.id)'));

        $selectFields =  array('pliupl.id');

        $select->order($order);
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pliupl' => 'plugin_leads_ip_user_profile_log'),
                $selectFields
            );
        if (!empty($order)) {
            $select->order($order);
        }

        $select->where($this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId));

        $data = $this->getDbTable()->getAdapter()->fetchCol($select);

        $select->reset(Zend_Db_Select::COLUMNS);
        $select->reset(Zend_Db_Select::FROM);
        $select->reset(Zend_Db_Select::LIMIT_OFFSET);
        $select->reset(Zend_Db_Select::GROUP);

        $select->from(array('pliupl' => 'plugin_leads_ip_user_profile_log'), array('count' => 'COUNT(pliupl.id)'));
        $count = $this->getDbTable()->getAdapter()->fetchRow($select);

        return array(
            'totalRecords' => $count['count'],
            'data' => $data
        );

    }

    /**
     * Get visits info etc..
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL OFFSET offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param bool $withTimezoneOffset select data with processed dates based on logged user timezone
     * @return array
     */
    public function fetchAllForGrid(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $withoutCount = false,
        $singleRecord = false,
        $withTimezoneOffset = true
    ) {
        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');

        $createdAt = 'pliupl.first_action_timestamp';

        if ($withTimezoneOffset === true) {
            $createdAt = new Zend_Db_Expr("CONVERT_TZ(`pliupl`.`first_action_timestamp`, '+00:00', '$mysqlOffset')");
        }

        $selectFields =  array(
            'pliupl.id',
            'leadId' => 'pliupl.lead_id',
            'firstActionTimestamp'=> $createdAt,
            'visitCount' => 'pliupl.visit_count',
            'visitConverted' => 'pliupl.visit_converted',
            'visitDuration' => 'pliupl.visit_duration',
            'actions' => 'pliupl.actions',
            'referrerTypeName' => 'pliupl.referrer_type_name',
            'referrerName' => 'pliupl.referrer_name',
            'referrerKeyword' => 'pliupl.referrer_keyword',
            'referrerUrl' => 'pliupl.referrer_url',
            'language_code' => 'pliupl.language_code',
            'deviceBrand' => 'pliupl.device_brand',
            'deviceModel' => 'pliupl.device_model',
            'operatingSystemName' => 'pliupl.operating_system_name',
            'operatingSystemVersion' => 'pliupl.operating_system_version',
            'browserName' => 'pliupl.browser_name',
            'browserCode' => 'pliupl.browser_code',
            'operatingSystemCode' => 'pliupl.operating_system_code',
            'countryCode' => 'pliupl.country_code',
            'region' => 'pliupl.region',
            'city' => 'pliupl.city',
            'location' => 'pliupl.location',
            'latitude' => 'pliupl.latitude',
            'longitude' => 'pliupl.longitude',
            'actionLogId' => 'pliupl.action_log_id',
            'resolution' => 'pliupl.resolution'
        );

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pliupl' => 'plugin_leads_ip_user_profile_log'),
                $selectFields
            );
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::GROUP);

            $select->from(array('pliupl' => 'plugin_leads_ip_user_profile_log'), array('count' => 'COUNT(pliupl.id)'));
            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }

    /**
     * Get visits info for map
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL OFFSET offset.
     * @param string $group OPTIONAL An SQL GROUP BY group by.
     * @param bool $singleRecord flag fetch single record
     * @param bool $withTimezoneOffset select data with processed dates based on logged user timezone
     * @return array
     */
    public function fetchAllForVisitsMap(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $group = null,
        $singleRecord = false,
        $withTimezoneOffset = true
    ) {
        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');

        $createdAt = 'pliupl.first_action_timestamp';

        if ($withTimezoneOffset === true) {
            $createdAt = new Zend_Db_Expr("CONVERT_TZ(MIN(`pliupl`.`first_action_timestamp`), '+00:00', '$mysqlOffset')");
        }

        $selectFields =  array(
            'pliupl.id',
            'leadId' => 'pliupl.lead_id',
            'firstActionTimestamp'=> $createdAt,
            'visitCount' => new Zend_Db_Expr('SUM(pliupl.visit_count)'),
            'visitConverted' => new Zend_Db_Expr('SUM(pliupl.visit_converted)'),
            'visitDuration' => 'pliupl.visit_duration',
            'actions' => 'pliupl.actions',
            'referrerTypeName' => 'pliupl.referrer_type_name',
            'referrerName' => new Zend_Db_Expr('GROUP_CONCAT(DISTINCT(pliupl.referrer_name))'),
            'referrerKeyword' => 'pliupl.referrer_keyword',
            'referrerUrl' => 'pliupl.referrer_url',
            'language_code' => 'pliupl.language_code',
            'deviceBrand' => 'pliupl.device_brand',
            'deviceModel' => 'pliupl.device_model',
            'operatingSystemName' => 'pliupl.operating_system_name',
            'operatingSystemVersion' => 'pliupl.operating_system_version',
            'browserName' => new Zend_Db_Expr('GROUP_CONCAT(DISTINCT(pliupl.browser_name))'),
            'countryCode' => 'pliupl.country_code',
            'region' => 'pliupl.region',
            'city' => 'pliupl.city',
            'location' => 'pliupl.location',
            'latitude' => 'pliupl.latitude',
            'longitude' => 'pliupl.longitude',
            'actionLogId' => 'pliupl.action_log_id',
            'resolution' => 'pliupl.resolution',
            'locationsQuantity' => new Zend_Db_Expr('COUNT(pliupl.id)')
        );

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pliupl' => 'plugin_leads_ip_user_profile_log'),
                $selectFields
            );
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        if (!empty($group)) {
            $select->group($group);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        return $data;

    }

    /**
     * Get ip user log aggregated stats
     *
     * @param int $leadId lead id
     * @return mixed
     */
    public function getStatsData($leadId)
    {

        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);
        $selectFields = array(
            'pliupl.id',
            'firstVisitDate' => new Zend_Db_Expr('MIN(first_action_timestamp)'),
            'lastVisitDate' => new Zend_Db_Expr('MAX(last_action_date_time)'),
            'operatingSystems' => new Zend_Db_Expr('GROUP_CONCAT(DISTINCT(CONCAT(operating_system_name, " ", operating_system_version)))'),
            'operatingSystemsCode' => new Zend_Db_Expr('GROUP_CONCAT(DISTINCT(CONCAT(operating_system_code)))'),
            'operatingSystemsCodePlusVersion' => new Zend_Db_Expr('GROUP_CONCAT(DISTINCT(CONCAT(operating_system_code, "|", operating_system_version)))'),
            'resolutions' => new Zend_Db_Expr('GROUP_CONCAT(DISTINCT(resolution))'),
            'ipAddresses' => new Zend_Db_Expr('GROUP_CONCAT(DISTINCT(visit_ip_address))'),
            'browserCodes' => new Zend_Db_Expr('GROUP_CONCAT(DISTINCT(pliupl.browser_code))'),
            'totalSpentTime' => new Zend_Db_Expr('SUM(visit_duration)'),
            'convertedGoals' => new Zend_Db_Expr('SUM(goal_conversions)'),
            'visitsCount' => new Zend_Db_Expr('COUNT(DISTINCT(id_visit))'),
            'viewedPages' => new Zend_Db_Expr('SUM(actions)')
        );

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pliupl' => 'plugin_leads_ip_user_profile_log'),
                $selectFields)->where($where);

        return $this->getDbTable()->getAdapter()->fetchRow($select);
    }

    /**
     * Get first date action taken
     *
     * @param int $leadId lead id
     * @return mixed
     */
    public function getStatsFirstDateInfo($leadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);
        $selectFields = array(
            'pliupl.id',
            'firstVisitDate' => new Zend_Db_Expr('MIN(first_action_timestamp)'),
            'referrerName' => new Zend_Db_Expr('pliupl.referrer_name')
        );

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pliupl' => 'plugin_leads_ip_user_profile_log'),
                $selectFields)->where($where);

        return $this->getDbTable()->getAdapter()->fetchRow($select);
    }

    /**
     * Get last date action taken
     *
     * @param int $leadId lead id
     * @return mixed
     */
    public function getStatsLastDateInfo($leadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);
        $selectFields = array(
            'pliupl.id',
            'lastVisitDate' => new Zend_Db_Expr('MAX(last_action_date_time)'),
            'referrerName' => new Zend_Db_Expr('pliupl.referrer_name')
        );

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pliupl' => 'plugin_leads_ip_user_profile_log'),
                $selectFields)->where($where);

        return $this->getDbTable()->getAdapter()->fetchRow($select);
    }

    /**
     * Change lead id
     *
     * @param array $leadIds lead ids
     * @param int $newLeadId new lead id
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeLeadId($leadIds, $newLeadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id IN (?)', $leadIds);
        $data = array('lead_id' => $newLeadId);
        $this->getDbTable()->getAdapter()->update('plugin_leads_ip_user_profile_log', $data, $where);
    }

    /**
     * Delete by record id
     *
     * @param int $id record id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_ip_user_profile_log', $where);

    }

    /**
     * Delete by lead id and ip
     *
     * @param string $ip ip address
     * @param int $leadId lead id
     * @return mixed
     * @throws Exception
     */
    public function deleteByIpLeadId($ip, $leadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('visit_ip_address = ?', $ip);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_ip_user_profile_log', $where);

    }

    /**
     * @param $leadId
     * @return mixed
     */
    public function findProfileDataByLeadId($leadId) {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);

        $select = $this->getDbTable()->getAdapter()->select()->distinct()
            ->from(array('plugin_leads_ip_user_profile_log'), array('lead_id'))->where($where);

        return $this->getDbTable()->getAdapter()->fetchRow($select);
    }

}
