<?php

/**
 * LeadsIpsMapper.php
 *
 *
 * @method Leads_Mapper_LeadsIpsMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsIpsMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsIpsModel';

    protected $_dbTable = 'Leads_DbTable_LeadsIpsDbTable';

    /**
     * Save lead task model to DB
     * @param $model Leads_Model_LeadsIpsModel
     * @return Leads_Model_LeadsIpsModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'lead_id' => $model->getLeadId(),
            'lead_ip_address' => $model->getLeadIpAddress(),
            'ip_quantity' => $model->getIpQuantity()
        );

        $leadId = $data['lead_id'];
        $ip = $data['lead_ip_address'];
        $leadIpModel = $this->getByLeadIdAndIp($leadId, $ip);
        if (!$leadIpModel instanceof Leads_Model_LeadsIpsModel) {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        } else {
            $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);
            $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('lead_ip_address = ?', $ip);
            $this->getDbTable()->update($data, $where);
        }

        return $model;
    }

    /**
     * Get lead ip address
     *
     * @param int $leadId lead id
     * @param string $ip ip address
     * @return null
     */
    public function getByLeadIdAndIp($leadId, $ip)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('lead_ip_address = ?', $ip);
        return $this->_findWhere($where);
    }

    /**
     * Get by ip excluding
     *
     * @param int $leadId lead id
     * @param string $ip ip address
     * @return null
     */
    public function getByIpExcludeLeadId($leadId, $ip)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id <> ?', $leadId);
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('lead_ip_address = ?', $ip);
        return $this->_findWhere($where);
    }

    /**
     * Get by ip address
     * @param string $ip ip address
     * @return array|null
     */
    public function getByIp($ip)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_ip_address = ?', $ip);
        return $this->fetchAll($where);
    }

    /**
     * Delete lead ips records by lead id
     *
     * @param int $id lead id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_ips', $where);

    }

    /**
     * Delete ip record
     *
     * @param int $leadId lead id
     * @param string $ip lead ip address
     * @return int
     */
    public function deleteByLeadIdIp($leadId, $ip)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);
        $where .=  ' AND '. $this->getDbTable()->getAdapter()->quoteInto('lead_ip_address = ?', $ip);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_ips', $where);
    }


    /**
     * @param $leadId
     * @return |null
     */
    public function getByLeadId($leadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);
        return $this->_findWhere($where);
    }

    /**
     * Get lead attached ips
     *
     * @param int $leadId lead id
     * @return array
     */
    public function getIpsByLeadId($leadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_ips',
            array('lead_ip_address', 'lead_id'))
            ->where($where);

        return $this->getDbTable()->getAdapter()->fetchAll($select);

    }

}
