<?php

/**
 * LeadsLastViewedMapper.php
 *
 *
 * @method Leads_Mapper_LeadsLastViewedMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsLastViewedMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsLastViewedModel';

    protected $_dbTable = 'Leads_DbTable_LeadsLastViewedDbTable';

    /**
     * Save lead last viewed model to DB
     * @param $model Leads_Model_LeadsLastViewedModel
     * @return Leads_Model_LeadsLastViewedModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'id' => $model->getId(),
            'lead_type' => $model->getLeadType(),
            'viewed_at' => $model->getViewedAt(),
            'user_id' => $model->getUserId()
        );

        $this->getDbTable()->insert($data);

        return $model;
    }

    /**
     * Get lead last viewed leads and organizations
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @return array
     */
    public function fetchAll($where = null, $order = null, $limit = null, $offset = null)
    {
        $andOrganization = "pllv.lead_type='".Leads_Model_LeadsLastViewedModel::LEAD_LAST_VIEWED_TYPE_ORGANIZATION."'";
        $andLead = "pllv.lead_type='".Leads_Model_LeadsLastViewedModel::LEAD_LAST_VIEWED_TYPE_LEAD."'";
        $select = $this->getDbTable()->getAdapter()->select()->from(array('pllv' => 'plugin_leads_last_viewed'),
            array(
                'leadId' => 'pld.id',
                'organizationId' => 'plo.id',
                'leadFirstName' => 'pld.lead_first_name',
                'leadLastName' => 'pld.lead_last_name',
                'organizationName' => 'plo.organization_name',
                'viewedDate' => 'pllv.viewed_at'
            ))
            ->joinLeft(array('pld' => 'plugin_leads_data'),
                'pllv.id=pld.id AND '.$andLead,
                array())
            ->joinLeft(array('plo' => 'plugin_lead_organization'),
                'pllv.id=plo.id AND '.$andOrganization,
                array())
            ->where($where)
            ->limit($limit, $offset)
            ->order($order);

        $data = $this->getDbTable()->getAdapter()->fetchAll($select);

        $select->reset(Zend_Db_Select::COLUMNS);
        $select->reset(Zend_Db_Select::FROM);
        $select->reset(Zend_Db_Select::LIMIT_OFFSET);
        $select->reset(Zend_Db_Select::GROUP);

        $select->from(array('pllv' => 'plugin_leads_last_viewed'), array('count' => 'COUNT(pllv.id)'));
        $count = $this->getDbTable()->getAdapter()->fetchRow($select);

        return array(
            'totalRecords' => $count['count'],
            'data' => $data,
            'offset' => $offset,
            'limit' => $limit
        );
    }

    /**
     * Delete last viewed record
     *
     * @param int $id last viewed item id
     * @param string $leadType lead type (lead, organization)
     * @param int $userId system user id
     * @return mixed
     * @throws Exception
     */
    public function findBy($id, $leadType, $userId = 0)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
        $where .= ' AND '.$this->getDbTable()->getAdapter()->quoteInto('lead_type = ?', $leadType);
        if (!empty($userId)) {
            $where .= ' AND '.$this->getDbTable()->getAdapter()->quoteInto('user_id = ?', $userId);
        }

        return $this->_findWhere($where);
    }


    /**
     * Delete last viewed record
     *
     * @param int $id last viewed item id
     * @param string $leadType lead type (lead, organization)
     * @param int $userId system user id
     * @return mixed
     * @throws Exception
     */
    public function delete($id, $leadType, $userId = 0)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('lead_type = ?', $leadType);
        if (!empty($userId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('user_id = ?', $userId);
        }

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_last_viewed', $where);

    }

}
