<?php

/**
 * LeadsMapper.php
 *
 *
 * @method Leads_Mapper_LeadsMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsModel';

    protected $_dbTable = 'Leads_DbTable_LeadsDbTable';

    /**
     * Save lead model to DB
     * @param $model Leads_Model_LeadsModel
     * @return Leads_Model_LeadsModel
     */
    public function save($model)
    {
        $oldLeadModel = null;
        if ($model instanceof $this->_model) {
            $oldLeadId = $model->getId();
            if (!empty($oldLeadId)) {
                $oldLeadModel = $this->find($oldLeadId);
            }
        }

        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        if (empty($model->getSkipCoordinatesGeneration())) {
            $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
            $enableLeadOrgGeotagging = $configHelper->getConfig('enableLeadOrgGeotagging');
            if (!empty($enableLeadOrgGeotagging)) {
                $model = Tools_LeadGeolocationTools::generateAddressHashFromLeadModelAndAssignData($model);
            }
        }

        $data = array(
            'lead_first_name'                => $model->getLeadFirstName(),
            'lead_last_name'                 => $model->getLeadLastName(),
            'lead_email'                     => $model->getLeadEmail(),
            'lead_phone'                     => $model->getLeadPhone(),
            'lead_source'                    => $model->getLeadSource(),
            'lead_notes'                     => $model->getLeadNotes(),
            'organization_id'                => $model->getOrganizationId(),
            'user_id'                        => $model->getUserId(),
            'stage_id'                       => $model->getStageId(),
            'customer_group_id'              => $model->getCustomerGroupId(),
            'created_at'                     => $model->getCreatedAt(),
            'updated_at'                     => $model->getUpdatedAt(),
            'lead_zip'                       => $model->getLeadZip(),
            'lead_address'                   => $model->getLeadAddress(),
            'lead_address1'                  => $model->getLeadAddress1(),
            'last_activity'                  => $model->getLastActivity(),
            'owner_id'                       => $model->getOwnerId(),
            'lead_type_id'                   => $model->getLeadTypeId(),
            'lead_mobile'                    => $model->getLeadMobile(),
            'lead_position'                  => $model->getLeadPosition(),
            'lead_country_code'              => $model->getLeadCountryCode(),
            'lead_state_code'                => $model->getLeadStateCode(),
            'middle_name'                    => $model->getMiddleName(),
            'prefix'                         => $model->getPrefix(),
            'lead_phone_country_code'        => $model->getLeadPhoneCountryCode(),
            'lead_phone_country_code_value'  => $model->getLeadPhoneCountryCodeValue(),
            'lead_mobile_country_code'       => $model->getLeadMobileCountryCode(),
            'lead_mobile_country_code_value' => $model->getLeadMobileCountryCodeValue(),
            'lead_phone_extension'           => $model->getLeadPhoneExtension(),
            'lead_city'                      => $model->getLeadCity(),
            'external_id'                    => $model->getExternalId(),
            'creation_type'                  => $model->getCreationType(),
            'last_activity_type_id'          => $model->getLastActivityTypeId(),
            'use_organization_address'       => $model->getUseOrganizationAddress(),
            'last_activity_subtype'          => $model->getLastActivitySubtype(),
            'last_activity_outbound'         => $model->getLastActivityOutbound(),
            'score'                          => $model->getScore(),
            'score_processed_at'             => $model->getScoreProcessedAt(),
            'remote_authorization_token'     => $model->getRemoteAuthorizationToken(),
            'lead_secondary_email'           => $model->getLeadSecondaryEmail(),
            'email_validated_status'         => $model->getEmailValidatedStatus(),
            'email_validated_date'           => $model->getEmailValidatedDate(),
            'email_validated_score'          => $model->getEmailValidatedScore(),
            'address_hash'                   => $model->getAddressHash(),
            'lat'                            => $model->getLat(),
            'lng'                            => $model->getLng(),
            'last_coordinates_processed_date'=> $model->getLastCoordinatesProcessedDate()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        if ($oldLeadModel instanceof $this->_model) {
            Tools_LeadExternalNotificationsTools::addToExternalNotificationsLog($model, $oldLeadModel);
        }

        Tools_LeadInternalNotificationsTools::addToInternalNotificationsLog($model, $oldLeadModel);

        $tagIds = $model->getLeadTags();
        $this->processTags($id, $tagIds);

        return $model;
    }

    /**
     * @return array
     */
    public function getLeadTitles()
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_position <> ?', '');
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_data',
            array('id'=>'lead_position', 'label' => new Zend_Db_Expr('CONCAT(lead_position," (",COUNT(lead_position), ") ")')))
            ->group('lead_position')
            ->where($where);
        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Add tags to lead
     *
     * @param int $leadId lead id
     * @param array $tagIds tag ids
     */
    public function processTags($leadId, $tagIds)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);
        if (!empty($tagIds) && is_array($tagIds)) {
            $this->getDbTable()->getAdapter()->delete('plugin_leads_has_tags', $where);
            foreach ($tagIds as $tagId) {
                $data = array(
                    'lead_id' => $leadId,
                    'tag_id' => $tagId
                );
                $this->getDbTable()->getAdapter()->insert('plugin_leads_has_tags', $data);
            }
        }
    }

    /**
     * Add single tag to the lead
     *
     * @param int $leadId lead id
     * @param int $tagId tag id
     * @throws Zend_Db_Adapter_Exception
     */
    public function addTag($leadId, $tagId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);
        $where .= ' AND '.$this->getDbTable()->getAdapter()->quoteInto('tag_id = ?', $tagId);
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_has_tags')->where($where);
        $tagAlreadyExists = $this->getDbTable()->getAdapter()->fetchRow($select);
        if (empty($tagAlreadyExists)) {
            $data = array(
                'lead_id' => $leadId,
                'tag_id' => $tagId
            );
            $this->getDbTable()->getAdapter()->insert('plugin_leads_has_tags', $data);
        }
    }

    /**
     * Add social service to the lead
     *
     * @param int $leadId lead id
     * @param int $serviceId service id
     * @param string $serviceLink service link
     * @throws Zend_Db_Adapter_Exception
     */
    public function addSocialService($leadId, $serviceId, $serviceLink)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);
        $where .= ' AND '.$this->getDbTable()->getAdapter()->quoteInto('service_id = ?', $serviceId);
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_has_social_service')->where($where);
        $serviceExists = $this->getDbTable()->getAdapter()->fetchRow($select);
        $data = array(
            'lead_id' => $leadId,
            'service_id' => $serviceId,
            'service_link' => $serviceLink
        );
        if (empty($serviceExists)) {
            $this->getDbTable()->getAdapter()->insert('plugin_leads_has_social_service', $data);
        } else {
            $this->getDbTable()->getAdapter()->update('plugin_leads_has_social_service', $data, $where);
        }
    }

    /**
     * Get lead data with info for groups etc..
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param bool $withOpportunityAmount select with opportunity amount
     * @param string $having mysql having
     * @param bool $nextActivityDate enable search by next activity date
     * @param bool $withTasks enable search by tasks
     * @param bool $callDuration call log table records duration
     * @param bool $includesNotes search includes notes fields
     * @param bool $includesCustomLeadFields search includes custom fields
     * @param bool $includesCustomOrganizationFields search includes organization custom fields
     * @param bool $withOpportunityExpected search with opportunity expected
     * @param bool $withSpentPurchaseAmount search with spent purchase amount
     * @param bool $withEmailLog search with email log
     * @param bool $withOrganizationIndustry with organization industry flag
     * @param array $additionalParamsForSelect select query params
     * @param array $additionalOrganizationParamsForSelect select query params
     * @param bool $withTimezoneOffset with timezone offset
     * @param bool $includePluginLeadsTypeCpecificColl include plugin leads colls
     * @param bool $withEmailSmsSequence with email sms sequnce flag
     *
     * @return array
     */
    public function fetchAll(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $withoutCount = false,
        $singleRecord = false,
        $withOpportunityAmount = false,
        $having = '',
        $nextActivityDate = false,
        $withTasks = false,
        $callDuration = false,
        $includesNotes = false,
        $includesCustomLeadFields = false,
        $includesCustomOrganizationFields = false,
        $withOpportunityExpected = false,
        $withSpentPurchaseAmount = false,
        $withEmailLog = false,
        $withOrganizationIndustry = false,
        $additionalParamsForSelect = array(),
        $additionalOrganizationParamsForSelect = array(),
        $withTimezoneOffset = true,
        $includePluginLeadsTypeCpecificColl = false,
        $withEmailSmsSequence = false
    ) {

        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');
        if ($withTimezoneOffset === true) {
            $reviewedFacebookDate = new Zend_Db_Expr("CONVERT_TZ(`plrr`.`request_reviewed_facebook_date`, '+00:00', '$mysqlOffset')");
            $reviewedGmbDate = new Zend_Db_Expr("CONVERT_TZ(`plrr`.`request_reviewed_gmb_date`, '+00:00', '$mysqlOffset')");
            $requestSentAt = new Zend_Db_Expr("CONVERT_TZ(`plrr`.`review_request_sent_at`, '+00:00', '$mysqlOffset')");
            $reviewedAt = new Zend_Db_Expr("CONVERT_TZ(`plrr`.`reviewed_at`, '+00:00', '$mysqlOffset')");
            $sentAt = new Zend_Db_Expr("CONVERT_TZ(`plrr`.`sent_at`, '+00:00', '$mysqlOffset')");
            $lastActivity = new Zend_Db_Expr("CONVERT_TZ(`pld`.`last_activity`, '+00:00', '$mysqlOffset')");
            $lastActivityOutbound = new Zend_Db_Expr("CONVERT_TZ(`pld`.`last_activity_outbound`, '+00:00', '$mysqlOffset')");
            $emailValidatedDate = new Zend_Db_Expr("CONVERT_TZ(`pld`.`email_validated_date`, '+00:00', '$mysqlOffset')");
        } else{
            $reviewedFacebookDate = 'plrr.request_reviewed_facebook_date';
            $reviewedGmbDate = 'plrr.request_reviewed_gmb_date';
            $requestSentAt = 'plrr.review_request_sent_at';
            $reviewedAt = 'plrr.reviewed_at';
            $sentAt = 'plrr.sent_at';
            $lastActivity = 'pld.last_activity';
            $lastActivityOutbound = 'pld.last_activity_outbound';
            $emailValidatedDate = 'pld.email_validated_date';
        }

        $params = array(
            'pld.id',
            'pld.lead_first_name',
            'pld.lead_last_name',
            'pld.lead_email',
            'pld.lead_phone',
            'pld.lead_source',
            'pld.lead_notes',
            'pld.lead_zip',
            'pld.lead_address',
            'pld.lead_address1',
            'pld.organization_id',
            'pld.user_id',
            'pld.stage_id',
            'pld.customer_group_id',
            'pld.owner_id',
            'pld.lead_type_id',
            'pld.lead_country_code',
            'pld.lead_state_code',
            'pld.middle_name',
            'pld.prefix',
            'pld.lead_phone_country_code',
            'pld.lead_phone_country_code_value',
            'pld.lead_mobile_country_code',
            'pld.lead_mobile_country_code_value',
            'pld.lead_phone_extension',
            'pld.last_activity',
            'lastActivity' => $lastActivity,
            'lifecycleName' => 'pls.stage_name',
            'pld.last_activity_type_id',
            'pld.lead_city',
            'pld.use_organization_address',
            'pld.created_at',
            'pld.last_activity_subtype',
            'pld.last_activity_outbound',
            'lastActivityOutbound' => $lastActivityOutbound,
            'pld.lead_secondary_email',
            'pld.lat',
            'pld.lng',
            'leadTypeLabel' => 'plt.type_label',
            'owner_name' => 'u.full_name',
            'group_name' => new Zend_Db_Expr('GROUP_CONCAT(DISTINCT(shgr.groupName))'),
            'tagIds' => new Zend_Db_Expr('GROUP_CONCAT(DISTINCT(plht.tag_id))'),
            'organization_name' => new Zend_Db_Expr('GROUP_CONCAT(DISTINCT(plo.organization_name))'),
            'pld.lead_mobile',
            'pld.lead_position',
            'pld.score',
            'pld.email_validated_status',
            'pld.email_validated_score',
            'emailValidatedDate' => $emailValidatedDate,
            'reviewScore' => 'plrr.score',
            'reviewSentAt' => $sentAt,
            'reviewReviewedAt' => $reviewedAt,
            'reviewStatus' => 'plrr.status',
            'reviewRequestSentAt' => $requestSentAt,
            'reviewRequestReviewedFacebookDate' => $reviewedFacebookDate,
            'reviewRequestReviewedGmbDate' => $reviewedGmbDate,
            'reviewRequestStatus' => 'plrr.review_status',
            'plo.organization_phone_country_code',
            'plo.organization_phone',
            'plo.organization_mobile_country_code',
            'plo.organization_mobile_country_code_value',
            'plo.organization_mobile',
            'plo.organization_address',
            'plo.organization_address1',
            'plo.organization_city',
            'plo.organization_zip',
            'plo.organization_email',
            'plo.organization_country_code',
            'plo.organization_website',
            'plo.organization_industry_id',
            'organization_group_id' => 'plo.customer_group_id'
        );


        $params = array_merge($additionalParamsForSelect, $params);
        $params = array_merge($additionalOrganizationParamsForSelect, $params);

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pld' => 'plugin_leads_data'),
                $params
            )
            ->joinLeft(array('shgr' => 'shopping_group'), 'shgr.id=pld.customer_group_id', array())
            ->joinLeft(array('plo' => 'plugin_lead_organization'), 'plo.id=pld.organization_id', array())
            ->joinLeft(array('plht' => 'plugin_leads_has_tags'), 'plht.lead_id=pld.id', array())
            ->joinLeft(array('u' => 'user'), 'u.id=pld.owner_id', array())
            ->joinLeft(array('plrr' => 'plugin_leads_review_request'), 'pld.id=plrr.lead_id', array())
            ->joinLeft(array('pls' => 'plugin_leads_stage'), 'pls.id=pld.stage_id', array());

        if (true === $withEmailSmsSequence) {
            $select->joinLeft(array('plhes' => 'plugin_leads_has_email_sequence'), 'plhes.lead_id=pld.id', array());
            $select->joinLeft(array('plel' => 'plugin_leads_email_log'), 'plel.lead_id=plhes.lead_id', array());
            $select->joinLeft(array('pcq' => 'plugin_cartstatusemail_queue'), 'pcq.externalId=plhes.lead_id', array());
        }

        if (true === $withOpportunityAmount || true === $withOpportunityExpected) {
            $select->join(array('plop' => 'plugin_leads_opportunity'), 'plop.lead_id=pld.id', array());
        }

        if (true === $nextActivityDate) {
            $select->joinLeft(array('plal' => 'plugin_leads_action_log'), 'plal.lead_id=pld.id', array());
        }

        if (false === $includePluginLeadsTypeCpecificColl) {
            $select->joinLeft(array('plt' => 'plugin_leads_type'), 'plt.id=pld.lead_type_id', array());
        } else {
            $select->joinLeft(array('plt' => 'plugin_leads_type'), 'plt.type_name=plal.lead_type', array());
        }

        if (true === $withTasks) {
            $select->joinLeft(array('plta' => 'plugin_leads_tasks'), 'plta.lead_id=pld.id', array());
        }

        if (true === $callDuration) {
            $select->joinLeft(array('plcl' => 'plugin_leads_call_log'), 'plcl.lead_id=pld.id', array());
        }
        if (true === $includesNotes) {
            $select->joinLeft(array('pln' => 'plugin_leads_notes'), 'pln.lead_id=pld.id', array());
        }
        if (true === $includesCustomLeadFields) {
            $select->joinLeft(array('pllcpd' => 'plugin_leads_lead_custom_params_data'), 'pllcpd.lead_id=pld.id', array());
        }
        if (true === $includesCustomOrganizationFields) {
            $select->joinLeft(array('plocpd' => 'plugin_leads_organization_custom_params_data'), 'plocpd.organization_id=plo.id', array());
        }
        if (true === $withSpentPurchaseAmount) {
            $select->joinLeft(array('scs' => 'shopping_cart_session'), 'scs.user_id=pld.user_id', array());
        }
        if (true === $withEmailLog) {
            $select->joinLeft(array('plel' => 'plugin_leads_email_log'), 'plel.lead_id=pld.id', array());
        }
        if (true === $withOrganizationIndustry) {
            $select->joinLeft(array('pli' => 'plugin_leads_industry'), 'pli.id=plo.organization_industry_id', array());
        }

        if (!empty($having)) {
            $select->having($having);
        }

        $select->group('pld.id');
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::LIMIT_COUNT);

            $count = array('count' => new Zend_Db_Expr('COUNT(DISTINCT(pld.id))'));
            $count = array_merge($count, $additionalParamsForSelect);
            $count = array_merge($count, $additionalOrganizationParamsForSelect);

            $select->from(array('pld' => 'plugin_leads_data'), $count)
                ->joinLeft(array('shgr' => 'shopping_group'), 'shgr.id=pld.customer_group_id', array())
                ->joinLeft(array('plo' => 'plugin_lead_organization'), 'plo.id=pld.organization_id', array())
                ->joinLeft(array('plht' => 'plugin_leads_has_tags'), 'plht.lead_id=pld.id', array())
                ->joinLeft(array('u' => 'user'), 'u.id=pld.owner_id', array())
                ->joinLeft(array('plrr' => 'plugin_leads_review_request'), 'pld.id=plrr.lead_id', array())
                ->joinLeft(array('pls' => 'plugin_leads_stage'), 'pls.id=pld.stage_id', array());

            if (true === $nextActivityDate) {
                $select->joinLeft(array('plal' => 'plugin_leads_action_log'), 'plal.lead_id=pld.id', array());
            }

            if (false === $includePluginLeadsTypeCpecificColl) {
                $select->joinLeft(array('plt' => 'plugin_leads_type'), 'plt.id=pld.lead_type_id', array());
            } else {
                $select->joinLeft(array('plt' => 'plugin_leads_type'), 'plt.type_name=plal.lead_type', array());
            }

            if (true === $withOpportunityAmount || true === $withOpportunityExpected) {
                $select->join(array('plop' => 'plugin_leads_opportunity'), 'plop.lead_id=pld.id', array());
            }

            if (true === $withEmailSmsSequence) {
                $select->joinLeft(array('plhes' => 'plugin_leads_has_email_sequence'), 'plhes.lead_id=pld.id', array());
                $select->joinLeft(array('plel' => 'plugin_leads_email_log'), 'plel.lead_id=plhes.lead_id', array());
                $select->joinLeft(array('pcq' => 'plugin_cartstatusemail_queue'), 'pcq.externalId=plhes.lead_id', array());
            }

            if (true === $withTasks) {
                $select->joinLeft(array('plta' => 'plugin_leads_tasks'), 'plta.lead_id=pld.id', array());
            }

            if (true === $callDuration) {
                $select->joinLeft(array('plcl' => 'plugin_leads_call_log'), 'plcl.lead_id=pld.id', array());
            }

            if (true === $includesNotes) {
                $select->joinLeft(array('pln' => 'plugin_leads_notes'), 'pln.lead_id=pld.id', array());
            }

            if (true === $includesCustomLeadFields) {
                $select->joinLeft(array('pllcpd' => 'plugin_leads_lead_custom_params_data'), 'pllcpd.lead_id=pld.id', array());
            }

            if (true === $includesCustomOrganizationFields) {
                $select->joinLeft(array('plocpd' => 'plugin_leads_organization_custom_params_data'), 'plocpd.organization_id=plo.id', array());
            }

            if (true === $withSpentPurchaseAmount) {
                $select->joinLeft(array('scs' => 'shopping_cart_session'), 'scs.user_id=pld.user_id', array());
            }

            if (true === $withEmailLog) {
                $select->joinLeft(array('plel' => 'plugin_leads_email_log'), 'plel.lead_id=pld.id', array());
            }

            if (true === $withOrganizationIndustry) {
                $select->joinLeft(array('pli' => 'plugin_leads_industry'), 'pli.id=plo.organization_industry_id', array());
            }

            $select =  $this->getDbTable()->getAdapter()->select()
                    ->from(
                        array('subres' => $select),
                        array('count' => 'SUM(count)')
                    );

            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }

    /**
     * Get lead data with info for groups etc..
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param string $having mysql having
     *
     * @return array
     */
    public function fetchAllGroups(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $withoutCount = false,
        $singleRecord = false,
        $having = ''
    ) {
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('sp' => 'shopping_group'),
                array(
                    'sp.id',
                    'group_name' => 'sp.groupName',
                )
            );

        if (!empty($having)) {
            $select->having($having);
        }

        $select->group('sp.id');
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::LIMIT_COUNT);

            $count = array('count' => new Zend_Db_Expr('COUNT(DISTINCT(sp.id))'));

            $select->from(array('sp' => 'shopping_group'), $count);

            $select =  $this->getDbTable()->getAdapter()->select()
                ->from(
                    array('subres' => $select),
                    array('count' => 'SUM(count)')
                );

            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }


    /**
     * Get all groups pairs
     *
     * @return array
     */
    public function getCustomerGroups()
    {
        $select = $this->getDbTable()->getAdapter()->select()->from('shopping_group', array('id', 'groupName'));

        return $this->getDbTable()->getAdapter()->fetchPairs($select);
    }

    /**
     * Get lead owner info info
     * @param bool $pairs return result in pairs key => value
     *
     * @return array
     */
    public function getLeadOwners($pairs = false)
    {
        $select = $this->getDbTable()->getAdapter()->select()->from(array('pld' => 'plugin_leads_data'),
            array('u.id', 'u.full_name'))->joinLeft(array('u' => 'user'), 'u.id=pld.owner_id')->group('u.id')->where('u.id IS NOT NULL')->order('u.full_name');

        if ($pairs ===true) {
            return $this->getDbTable()->getAdapter()->fetchPairs($select);
        } else {
            return $this->getDbTable()->getAdapter()->fetchAssoc($select);
        }
    }

    /**
     * Get lead owner info info
     * @param bool $pairs return result in pairs key => value
     * @param bool $adminGroup admin group user only flag
     *
     * @return array
     */
    public function getAllUsers($pairs = false, $adminGroup = false)
    {
        $select = $this->getDbTable()->getAdapter()->select()->from(array('u' => 'user'),
            array('id', 'full_name'))->order('full_name');

        if ($adminGroup === true) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('role_id IN (?)',
                array(Tools_Security_Acl::ROLE_SUPERADMIN,
                    Tools_Security_Acl::ROLE_ADMIN,
                    Shopping::ROLE_SALESPERSON
                )
            );
            $select->where($where);
        }

        if ($pairs ===true) {
            return $this->getDbTable()->getAdapter()->fetchPairs($select);
        } else {
            return $this->getDbTable()->getAdapter()->fetchAssoc($select);
        }
    }

    /**
     * Get users data
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param string $having mysql having
     *
     * @return array
     */
    public function getUsersData(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $withoutCount = false,
        $singleRecord = false,
        $having = ''
    ) {
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('u' => 'user'),
                array(
                    'u.id',
                    'u.role_id',
                    'u.email',
                    'u.prefix',
                    'u.full_name',
                    'u.timezone',
                    'u.mobile_country_code',
                    'u.mobile_country_code_value',
                    'u.desktop_phone',
                    'u.desktop_country_code',
                    'u.desktop_country_code_value',
                )
            );


        if (!empty($having)) {
            $select->having($having);
        }

        $select->group('u.id');
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::LIMIT_COUNT);

            $count = array('count' => new Zend_Db_Expr('COUNT(DISTINCT(u.id))'));

            $select->from(array('u' => 'user'), $count);


            $select =  $this->getDbTable()->getAdapter()->select()
                ->from(
                    array('subres' => $select),
                    array('count' => 'SUM(count)')
                );

            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }


    /**
     * Get lead social services
     * @param int $leadId lead id
     * @return array
     */
    public function getLeadSocialServices($leadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_has_social_service', array('service_id', 'service_link'))->where($where);
        return $this->getDbTable()->getAdapter()->fetchPairs($select);
    }

    /**
     * Get lead social services
     * @param int $leadId lead id
     * @return array
     */
    public function getLeadSocialServicesData($leadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_has_social_service', array('service_id', 'service_link'))->where($where);
        return $this->getDbTable()->getAdapter()->fetchAll($select);
    }

    /**
     * Get lead social services
     * @param int $leadId lead id
     * @return array
     */
    public function getLeadSocialServicesNames($leadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plhcs.lead_id = ?', $leadId);
        $select = $this->getDbTable()->getAdapter()->select()->from(array('plhcs' => 'plugin_leads_has_social_service'), array('plscs.service_name', 'plhcs.service_link'))
            ->join(array('plscs' => 'plugin_leads_social_services_config'), 'plhcs.service_id=plscs.id', array())
            ->where($where);
        return $this->getDbTable()->getAdapter()->fetchPairs($select);
    }

    /**
     * Get lead social services
     * @param int $leadId lead id
     * @return array
     */
    public function getLeadSocialServicesNamesData($leadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plhcs.lead_id = ?', $leadId);
        $select = $this->getDbTable()->getAdapter()->select()->from(array('plhcs' => 'plugin_leads_has_social_service'), array('plscs.service_name', 'plhcs.service_link'))
            ->join(array('plscs' => 'plugin_leads_social_services_config'), 'plhcs.service_id=plscs.id', array())
            ->where($where);
        return $this->getDbTable()->getAdapter()->fetchAll($select);
    }


    /**
     * Change lead id
     *
     * @param array $leadIds lead ids
     * @param int $newLeadId new lead id
     * @param array $serviceIds service ids
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeSocialServiceLeadId($leadIds, $newLeadId, $serviceIds = array())
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id IN (?)', $leadIds);
        $data = array('lead_id' => $newLeadId);
        if (!empty($serviceIds)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('service_id IN (?)', $serviceIds);
        }
        $this->getDbTable()->getAdapter()->update('plugin_leads_has_social_service', $data, $where);
    }

    /**
     * Get all possible owners of the leads
     * @param int $excludeId exclude owner id
     * @param array $ids user ids
     * @param bool $fullInfo full info
     * @return array
     */
    public function getLeadOwnersFullList($excludeId = 0, $ids = array(), $fullInfo = false)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('role_id IN (?)',
            array(Tools_Security_Acl::ROLE_ADMIN, Shopping::ROLE_SALESPERSON, Tools_Security_Acl::ROLE_SUPERADMIN));
        if ($excludeId) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('id <> ?', $excludeId);
        }

        if (!empty($ids)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('id IN (?)', $ids);
        }

        if (!empty($fullInfo)) {
            $select = $this->getDbTable()->getAdapter()->select()->from(array('u' => 'user'),
                array('id', 'full_name', 'role_id', 'email'))->where($where);
            return $this->getDbTable()->getAdapter()->fetchAssoc($select);

        } else {
            $select = $this->getDbTable()->getAdapter()->select()->from(array('u' => 'user'),
                array('id', 'full_name'))->where($where);
            return $this->getDbTable()->getAdapter()->fetchPairs($select);
        }

    }

    /**
     * Get all possible owners of the leads
     * @param array $ids user ids
     * @return array
     */
    public function getLeadOwnersList($ids = array())
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('role_id IN (?)',
            array(Tools_Security_Acl::ROLE_ADMIN, Shopping::ROLE_SALESPERSON, Tools_Security_Acl::ROLE_SUPERADMIN));

        if (!empty($ids)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('id IN (?)', $ids);
        }

        $select = $this->getDbTable()->getAdapter()->select()->from(array('u' => 'user'),
            array('id', 'full_name', 'email'))->where($where);

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }


    /**
     * Find superadmin user
     *
     * @return null
     */
    public function findSuperAdminUser()
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto("role_id = ?", 'superadmin');
        $select = $this->getDbTable()->getAdapter()->select()->from('user')->where($where);
        $row = $this->getDbTable()->getAdapter()->fetchRow($select);
        if(!$row) {
            return null;
        }
        return new Application_Model_Models_User($row);
    }

    /**
     * Find lead by email
     *
     * @param string $email lead email
     * @return null
     */
    public function findByEmail($email)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_email = ?', $email);

        return $this->_findWhere($where);
    }

    /**
     * Find lead by emails
     *
     * @param string $emails lead email
     * @return null
     */
    public function findByEmails($emails)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_email IN (?)', $emails);
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_data', array(
            'lead_email',
            'lead_first_name',
            'lead_last_name'
        ))->where($where);

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }


    /**
     * Find lead by external id
     *
     * @param string $externalId lead external id
     * @return null
     */
    public function findByExternalId($externalId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('external_id = ?', $externalId);

        return $this->_findWhere($where);
    }

    /**
     * Find lead by user id
     *
     * @param string $userId system user id
     * @return null
     */
    public function findByUserId($userId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('user_id = ?', $userId);

        return $this->_findWhere($where);
    }

    /**
     * Get prepared for email service lead data
     *
     * @param array $leadIds lead ids
     * @return array
     */
    public function findByLeadIdsForEmailService(array $leadIds)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id IN (?) ', $leadIds);
        $where .= ' AND  '.new Zend_Db_Expr('lead_email IS NOT NULL');
        $select = $this->getDbTable()->getAdapter()->select()
            ->from('plugin_leads_data', array('lead_email', new Zend_Db_Expr("CONCAT(lead_first_name, lead_last_name)")))
            ->where($where);
        return $this->getDbTable()->getAdapter()->fetchPairs($select);
    }

    /**
     * Find by user ids
     *
     * @param array $leadIds lead ids
     * @return array
     */
    public function findByUserIds(array $leadIds)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('user_id IN (?) ', $leadIds);
        $select = $this->getDbTable()->getAdapter()->select()
            ->from('plugin_leads_data')
            ->where($where);
        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Find by lead ids
     *
     * @param array $leadIds lead ids
     * @return array
     */
    public function findByLeadIds(array $leadIds)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id IN (?) ', $leadIds);
        $select = $this->getDbTable()->getAdapter()->select()
            ->from('plugin_leads_data')
            ->where($where);
        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Find lead by mobile or phone number
     *
     * @param string $phone
     * @param bool $fullPhone search by full phone number
     * @param int $excludeId exclude id
     * @return null
     */
    public function findByPhoneNumberOrMobile($phone, $fullPhone = true, $excludeId = 0)
    {
        if ($fullPhone === true) {
            $where = '('.$this->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr('CONCAT(`lead_mobile_country_code_value`, `lead_mobile`) = ?'), $phone);
            $where .= ' OR ' .$this->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr('CONCAT(`lead_phone_country_code_value`, `lead_phone`) = ?'), $phone).')';
        } else {
            $where =  '('.$this->getDbTable()->getAdapter()->quoteInto('lead_mobile = ?', $phone);
            $where .= ' OR ' .$this->getDbTable()->getAdapter()->quoteInto('lead_phone = ?', $phone).')';
        }

        if (!empty($excludeId)) {
            $where .= ' AND ' .$this->getDbTable()->getAdapter()->quoteInto('id <> ?', $excludeId);
        }

        return $this->_findWhere($where);
    }

    /**
     * Get lead by first name, last name, postal code
     *
     * @param string $firstName first name
     * @param string $lastName last name
     * @param string $zip postal code
     * @param int $excludeLeadId exclude lead id
     * @return null
     */
    public function findByFirstNameLastNameZip($firstName, $lastName, $zip, $excludeLeadId = 0)
    {
        $where =  $this->getDbTable()->getAdapter()->quoteInto('lead_first_name = ?', $firstName);
        $where .= ' AND ' .$this->getDbTable()->getAdapter()->quoteInto('lead_last_name = ?', $lastName);
        $where .= ' AND ' .$this->getDbTable()->getAdapter()->quoteInto('lead_zip = ?', $zip);

        if (!empty($excludeLeadId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('id <> ?', $excludeLeadId);
        }

        return $this->_findWhere($where);
    }

    /**
     * Find if lead has tag
     *
     * @param int $leadId lead id
     * @param int $tagId tag id
     * @return mixed
     */
    public function getByLeadIdTagId($leadId, $tagId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);
        $where .= ' AND '.$this->getDbTable()->getAdapter()->quoteInto('tag_id = ?', $tagId);
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_has_tags')->where($where);
        return $this->getDbTable()->getAdapter()->fetchRow($select);

    }

    /**
     * Delete social service
     *
     * @param int $leadId lead id
     * @param int $serviceId service id
     * @return int
     */
    public function deleteSocialService($leadId, $serviceId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('service_id = ?', $serviceId);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_has_social_service', $where);
    }

    /**
     * Delete lead tag
     *
     * @param int $leadId lead id
     * @param int $tagId tag id
     * @return int
     */
    public function deleteTag($leadId, $tagId = 0)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);
        if (!empty($tagId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('tag_id = ?', $tagId);
        }
        return $this->getDbTable()->getAdapter()->delete('plugin_leads_has_tags', $where);
    }

    /**
     * Change lead owner for leads
     *
     * @param array $leadIds
     * @param int $ownerId owner id (system user id)
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeLeadOwner(array $leadIds, $ownerId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id IN (?)', $leadIds);
        $data = array('owner_id' => $ownerId);
        $this->getDbTable()->getAdapter()->update('plugin_leads_data', $data, $where);
    }

    /**
     * Change lead owner for leads
     *
     * @param int $ownerIdFrom owner id from (system user id)
     * @param int $ownerIdTo owner id  to (system user id)
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeLeadOwnerByOwnerId($ownerIdFrom, $ownerIdTo)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('owner_id = ?', $ownerIdFrom);
        $data = array('owner_id' => $ownerIdTo);
        $this->getDbTable()->getAdapter()->update('plugin_leads_data', $data, $where);
    }

    /**
     * Change lead organization for leads
     *
     * @param int $oldOrganizationId old organization id
     * @param int $organizationId organization id (organization id)
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeOrganization($oldOrganizationId, $organizationId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('organization_id = ?', $oldOrganizationId);
        $data = array('organization_id' => $organizationId);
        $this->getDbTable()->getAdapter()->update('plugin_leads_data', $data, $where);
    }

    /**
     * Delete lead record
     *
     * @param int $id lead id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_data', $where);

    }

    /**
     * Get quantity of leads with not empty mobile, phone, email address
     *
     * @return mixed
     */
    public function calculateLeadsWithNonEmptyEmailPhone()
    {
        $where = '(' . $this->getDbTable()->getAdapter()->quoteInto('lead_phone <> ?', '');
        $where .= ' OR ' . new Zend_Db_Expr('lead_phone IS NOT NULL') . ')';
        $where .= ' OR ';
        $where .= '(' . $this->getDbTable()->getAdapter()->quoteInto('lead_mobile <> ?', '');
        $where .= ' OR ' . new Zend_Db_Expr('lead_mobile IS NOT NULL') . ')';
        $where .= ' OR ';
        $where .= '(' . $this->getDbTable()->getAdapter()->quoteInto('lead_email <> ?', '');
        $where .= ' OR ' . new Zend_Db_Expr('lead_email IS NOT NULL') . ')';

        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_data',
            array('quantity' => new Zend_Db_Expr('COUNT(id)')))->where($where);

        return $this->getDbTable()->getAdapter()->fetchRow($select);
    }

    /**
     * Find first lead created under organization
     *
     * @param int $organizationId organization id
     * @return mixed
     */
    public function findFirstCreatedOrganizationContact($organizationId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('organization_id = ?', $organizationId);

        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_data')->where($where)->order('created_at')->limit(1);

        return $this->getDbTable()->getAdapter()->fetchRow($select);
    }

    /**
     * @param array $leadsIds
     * @param array $excludeFields
     * @return array
     */
    public function fetchLeadsForExport($leadsIds = array(), $excludeFields = array(), $useCustomParams = false)
    {
        $defaultFields = array(
            'lead_id'                        => 'lead.id',
            'lead_first_name'                => 'lead.lead_first_name',
            'lead_last_name'                 => 'lead.lead_last_name',
            'lead_email'                     => 'lead.lead_email',
            'lead_mobile'                    => 'lead.lead_mobile',
            'organization_name'              => 'organization.organization_name',
            'organization_website'           => 'organization.organization_website',
            'organization_website1'           => 'organization.organization_website1',
            'middle_name'                    => 'lead.middle_name',
            'prefix'                         => 'lead.prefix',
            'lead_phone'                     => 'lead.lead_phone',
            'lead_phone_country_code'        => 'lead.lead_phone_country_code',
            'lead_phone_country_code_value'  => 'lead.lead_phone_country_code_value',
            'lead_phone_extension'           => 'lead.lead_phone_extension',
            'lead_mobile_country_code'       => 'lead.lead_mobile_country_code',
            'lead_mobile_country_code_value' => 'lead.lead_mobile_country_code_value',
            'lead_source'                    => 'lead.lead_source',
            'lead_notes'                     => 'lead.lead_notes',
            'lead_position'                  => 'lead.lead_position',
            'lead_zip'                       => 'lead.lead_zip',
            'lead_city'                      => 'lead.lead_city',
            'lead_address'                   => 'lead.lead_address',
            'lead_address1'                  => 'lead.lead_address1',
            'lead_country_code'              => 'lead.lead_country_code',
            'lead_state_code'                => 'lead.lead_state_code',
            'created_at'                     => 'lead.created_at',
            'last_activity'                  => 'lead.last_activity',
            'external_id'                    => 'lead.external_id',
            'lead_tags'                      => '(GROUP_CONCAT(tags.tag_name))',
            'lead_type_name'                 => 'type.type_name',
            'customer_group'                 => 'group.groupName',
            'lifecycle_name'                 => 'stage.stage_name',
            'lead_created_by_name'           => 'user.full_name',
            'lead_created_by_email'          => 'user.email',
            'lead_secondary_email'           => 'lead.lead_secondary_email'
        );

        if (!empty($excludeFields)) {
            foreach ($excludeFields as $fieldName => $fieldValue) {
                unset($defaultFields[$fieldName]);
            }
        }

        if($useCustomParams) {
            $defaultFields = array('id' => 'lead.id') + $defaultFields;
        }

        $select = $this->getDbTable()->select(Zend_Db_Table::SELECT_WITHOUT_FROM_PART)
            ->setIntegrityCheck(false)
            ->from(
                array('lead' => 'plugin_leads_data'),
                $defaultFields
            )
            ->joinLeft(array('organization' => 'plugin_lead_organization'), 'organization.id = lead.organization_id', array(''))
            ->joinLeft(array('type' => 'plugin_leads_type'), 'type.id = lead.lead_type_id', array(''))
            ->joinLeft(array('group' => 'shopping_group'), 'group.id = lead.customer_group_id', array(''))
            ->joinLeft(array('leadshtags' => 'plugin_leads_has_tags'), 'lead.id = leadshtags.lead_id', array(''))
            ->joinLeft(array('tags' => 'plugin_leads_tags'), 'leadshtags.tag_id = tags.id', array(''))
            ->joinLeft(array('stage' => 'plugin_leads_stage'), 'stage.id = lead.stage_id', array(''))
            ->joinLeft(array('user' => 'user'), 'user.id = lead.owner_id', array(''))
            ->group('lead.id');
        if (!empty($leadsIds)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('lead.id IN (?)', $leadsIds);
            $select->where($where);
        }
        if($useCustomParams) {
            return $this->getDbTable()->getAdapter()->fetchAssoc($select);
        }

        return $this->getDbTable()->getAdapter()->fetchAll($select);
    }


    /**
     * Find by lead id and owner id
     *
     * @param int $id lead id
     * @param int $ownerId owner id
     *
     * @return Leads_Model_LeadsModel
     */
    public function findByIdOwnerId($id, $ownerId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('owner_id = ?', $ownerId);
        return $this->_findWhere($where);
    }

    /**
     * Find lead owner by user email
     *
     * @param $leadEmail
     * @return mixed
     */
    public function findOwnerByUserEmail($leadEmail)
    {
        $select = $this->getDbTable()->select(Zend_Db_Table::SELECT_WITHOUT_FROM_PART)
            ->setIntegrityCheck(false)
            ->from(
                array('lead' => 'plugin_leads_data'), array(
                    'email' => 'user.email',
                    'fullname' => 'user.full_name',
                    'mobilephone' => 'user.mobile_phone',
                    'mobilecountrycode' => 'user.mobile_country_code',
                    'desktopcountrycode' => 'user.desktop_country_code',
                    'desktopphone' => 'user.desktop_phone',
                    'fulldesktopphone' => new Zend_Db_Expr('CONCAT(user.desktop_country_code_value, user.desktop_phone)'),
                    'fullmobilephone' => new Zend_Db_Expr('CONCAT(user.mobile_country_code_value, user.mobile_phone)'),
                    'signature' => 'user.signature',
                    'personalcalendarurl' => 'user.personal_calendar_url',
                    'avatarlink' => 'user.avatar_link',
                )
            )
            ->join(array('user' => 'user'), 'user.id = lead.owner_id', array(''));
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead.lead_email = ?', $leadEmail);
        $select->where($where);

        return $this->getDbTable()->getAdapter()->fetchRow($select);
    }

    /**
     * Find lead organization name by user email
     *
     * @param $email
     * @return mixed
     */
    public function findByLeadEmail($email)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_email = ?', $email);

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pld' => 'plugin_leads_data'), array(
                'plo.organization_name',
                'plo.organization_website'
            ))
            ->join(array('plo' => 'plugin_lead_organization'), 'pld.organization_id = plo.id', array())
            ->where($where);

        return $this->getDbTable()->getAdapter()->fetchRow($select);
    }

    /**
     * Find lead organization documents data by user email
     *
     * @param $email
     * @return mixed
     */
    public function findOrganizationDocument($email)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_email = ?', $email);

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pld' => 'plugin_leads_data'), array(
                'plod.file_stored_name',
                'plod.file_hash',
                'plod.original_file_name',
                'plod.display_file_name',
                'plod.uploaded_at'
            ))
            ->join(array('plo' => 'plugin_lead_organization'), 'pld.organization_id = plo.id', array())
            ->join(array('plod' => 'plugin_leads_organization_documents'), 'plo.id = plod.organization_id', array())
            ->where($where);

        return $this->getDbTable()->getAdapter()->fetchRow($select);
    }

    /**
     * Find lead organization documents data by user ID
     *
     * @param $email
     * @return mixed
     */
    public function findOrganizationDocumentByUserId($userId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('pld.user_id = ?', $userId);

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pld' => 'plugin_leads_data'), array(
                'plod.file_stored_name',
                'plod.file_hash',
                'plod.original_file_name',
                'plod.display_file_name',
                'plod.uploaded_at'
            ))
            ->join(array('plo' => 'plugin_lead_organization'), 'pld.organization_id = plo.id', array())
            ->join(array('plod' => 'plugin_leads_organization_documents'), 'plo.id = plod.organization_id', array())
            ->where($where);

        return $this->getDbTable()->getAdapter()->fetchRow($select);
    }

    /**
     * Get all email templates
     * @param bool $pairs select templates as key pairs
     *
     * @return array
     */
    public function getEmailTemplates($pairs = false)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('type = ?', 'typemail');
        if ($pairs === true) {
            $select = $this->getDbTable()->getAdapter()->select()->from(array('t' => 'template'), array('name', 'name'))->where($where);
            return $this->getDbTable()->getAdapter()->fetchPairs($select);
        } else {
            $select = $this->getDbTable()->getAdapter()->select()->from(array('t' => 'template'), array('name', 'content', 'type'))->where($where);
            return $this->getDbTable()->getAdapter()->fetchAssoc($select);
        }
    }

    /**
     * Get leads extended info for the email service
     *
     * @param array $leadIds lead ids
     * @return array
     */
    public function getExtendedLeadInfoForEmailService(array $leadIds)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('pld.id IN (?) ', $leadIds);
        $where .= ' AND  ' . new Zend_Db_Expr('pld.lead_email IS NOT NULL');
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pld'=>'plugin_leads_data'),
                array(
                    'email' => 'pld.lead_email',
                    'firstName' => 'pld.lead_first_name',
                    'lastName' => 'pld.lead_last_name',
                    'city' => 'pld.lead_city',
                    'state' => 'pld.lead_state_code',
                    'country' => 'pld.lead_country_code',
                    'zip' => 'pld.lead_zip',
                    'phoneCountryCode' => 'pld.lead_phone_country_code',
                    'phoneNumber' => 'pld.lead_phone',
                    'mobileCode' => 'pld.lead_mobile_country_code',
                    'mobileNumber' => 'pld.lead_mobile',
                    'externalId' => 'pld.id',
                    'position' => 'pld.lead_position',
                    'organization' => 'plo.organization_name'
                ))
            ->joinLeft(array('plo' => 'plugin_lead_organization'), 'plo.id=pld.organization_id', array())
            ->where($where);

        return $this->getDbTable()->getAdapter()->fetchAll($select);
    }

    /**
     * Get leads extended info for the Ads remarketing service
     *
     * @param array $leadIds lead ids
     * @return array
     */
    public function getExtendedLeadInfoForRemarketingLists(array $leadIds)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('pld.id IN (?) ', $leadIds);
        $where .= ' AND  ' . new Zend_Db_Expr('pld.lead_email IS NOT NULL');
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pld'=>'plugin_leads_data'),
                array(
                    'id'               => 'pld.id',
                    'email'            => 'pld.lead_email',
                    'phoneNumberCode'  => 'pld.lead_phone_country_code_value',
                    'phoneNumber'      => 'pld.lead_phone',
                    'mobileNumberCode' => 'pld.lead_mobile_country_code_value',
                    'mobileNumber'     => 'pld.lead_mobile',
                    'city'             => 'pld.lead_city',
                    'state'            => 'pld.lead_state_code',
                    'country'          => 'pld.lead_country_code',
                    'zip'              => 'pld.lead_zip',
                    'firstName'        => 'pld.lead_first_name',
                    'lastName'         => 'pld.lead_last_name'
                ))->where($where);

        return $this->getDbTable()->getAdapter()->fetchAll($select);
    }

    /**
     * Assign Default stage_id for leads
     *
     * @param $stageId
     * @param $oldStageId
     * @throws Zend_Db_Adapter_Exception
     */
    public function assignDefaultStageForLeads($stageId, $oldStageId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('stage_id = ?', $oldStageId);
        $data = array('stage_id' => $stageId);
        $this->getDbTable()->getAdapter()->update('plugin_leads_data', $data, $where);
    }

    /**
     * Set lead score to initial value 0
     *
     * @throws Zend_Db_Adapter_Exception
     */
    public function flushScore()
    {
        $data = array('score' => '0', 'score_processed_at' => Tools_System_Tools::convertDateFromTimezone('now'));
        $this->getDbTable()->getAdapter()->update('plugin_leads_data', $data);
    }

    /**
     * Update lead score
     *
     * @param int $leadId lead id
     * @param $score
     * @throws Zend_Db_Adapter_Exception
     */
    public function updateScore($leadId, $score)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $leadId);
        $data = array('score' => $score, 'score_processed_at' => Tools_System_Tools::convertDateFromTimezone('now'));
        $this->getDbTable()->getAdapter()->update('plugin_leads_data', $data, $where);
    }


    /**
     * Find by remote auth token
     *
     * @param string $token remote auth token
     * @return null
     * @throws Exception
     */
    public function findByRemoteAuthToken($token)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('remote_authorization_token = ?', $token);
        return $this->_findWhere($where);
    }

    /**
     * Get leads with mobile or lead organization mobile numbers
     *
     * @param $leadIds
     * @param $limit
     * @param $offset
     * @return array
     */
    public function getLeadAndOrganizationMobileByLeadIds($leadIds, $limit, $offset)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('pld.id IN (?)', $leadIds);
        $where .= ' AND ';

        $where .=  '(('.new Zend_Db_Expr('pld.lead_mobile_country_code_value IS NOT NULL') . ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.lead_mobile_country_code_value <> ?', '');
        $where .= ' AND ' .new Zend_Db_Expr('pld.lead_mobile IS NOT NULL')  . ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.lead_mobile <> ?', '') .')';
        $where .= ' OR ';
        $where .=  '('.new Zend_Db_Expr('plo.organization_mobile_country_code_value IS NOT NULL') . ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plo.organization_mobile_country_code_value <> ?', '');
        $where .= ' AND ' .new Zend_Db_Expr('plo.organization_mobile IS NOT NULL') . ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plo.organization_mobile <> ?', '') .'))';

        $data = $this->fetchAll($where, null, $limit, $offset, true);

        return $data;
    }

    /**
     * Remove organization id
     *
     * @param int $organizationId organization id
     * @param $score
     * @throws Zend_Db_Adapter_Exception
     */
    public function removeOrganizationId($organizationId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('organization_id = ?', $organizationId);
        $data = array('organization_id' => null);
        $this->getDbTable()->getAdapter()->update('plugin_leads_data', $data, $where);
    }

    /**
     * Find leads in radius using coordinates
     *
     * @param string $lat latitude
     * @param string $lng longitude
     * @param string $radius radius
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @return mixed
     * @throws Exception
     */
    public function findInRadius($lat, $lng, $radius, $where = null, $order = null, $limit = null, $offset = null,
                                 $withoutCount = false, $singleRecord = false)
    {

        $coordinatesInfo = Tools_LeadGeolocationTools::prepareCoordinatesRange($lat, $lng, $radius);

        $whereQuery = $this->getDbTable()->getAdapter()->quoteInto(
            'pld.lat >= ?',
            $coordinatesInfo['latitudeStart']
        );
        $whereQuery .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto(
                'pld.lat <= ?',
                $coordinatesInfo['latitudeEnd']
            );
        $whereQuery .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto(
                'pld.lng >= ?',
                $coordinatesInfo['longitudeStart']
            );
        $whereQuery .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto(
                'pld.lng <= ?',
                $coordinatesInfo['longitudeEnd']
            );

        if (!empty($where)) {
            $where .= ' AND ('. $whereQuery.')';
        } else {
            $where = $whereQuery;
        }

        $select = $this->getDbTable()->getAdapter()
            ->select()->from(array('pld' => 'plugin_leads_data'), array(
                'pld.id',
                'latitude' => 'pld.lat',
                'longitude' => 'pld.lng',
                'firstName' => 'pld.lead_first_name',
                'lastName' => 'pld.lead_last_name',
                'organizationName' => 'plo.organization_name',
                'address' => 'pld.lead_address',
                'phone' => 'pld.lead_phone',
                'mobile' => 'pld.lead_mobile',
                'phoneCountryCodeValue' => 'pld.lead_phone_country_code_value',
                'mobileCountryCodeValue' => 'pld.lead_mobile_country_code_value'
            ));

        $select->joinLeft(array('plop' => 'plugin_leads_opportunity'), 'plop.lead_id=pld.id', array())
               ->joinLeft(array('plo' => 'plugin_lead_organization'), 'plo.id=pld.organization_id', array());

        if (!empty($order)) {
            $select->order($order);
        }

        $select->limit($limit, $offset);

        if (!empty($where)) {
            $select->where($where);
        }

        $select->group('pld.id');

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        return $data;

    }

    /**
     * Find leads in radius using coordinates
     *
     * @param string $latitudeStart latitude start
     * @param string $latitudeEnd latitude end
     * @param string $longitudeStart longitude start
     * @param string $longitudeEnd longitude end
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @return mixed
     * @throws Exception
     */
    public function findInRadiusPredefinedCoord($latitudeStart, $latitudeEnd, $longitudeStart, $longitudeEnd, $where = null, $order = null, $limit = null, $offset = null,
                                 $withoutCount = false, $singleRecord = false)
    {


        $whereQuery = $this->getDbTable()->getAdapter()->quoteInto(
            'pld.lat >= ?',
            (float) $latitudeStart
        );
        $whereQuery .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto(
                'pld.lat <= ?',
                (float) $latitudeEnd
            );
        $whereQuery .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto(
                'pld.lng >= ?',
                (float) $longitudeStart
            );
        $whereQuery .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto(
                'pld.lng <= ?',
                (float) $longitudeEnd
            );

        if (!empty($where)) {
            $where .= ' AND ('. $whereQuery.')';
        } else {
            $where = $whereQuery;
        }

        $select = $this->getDbTable()->getAdapter()
            ->select()->from(array('pld' => 'plugin_leads_data'), array(
                'pld.id',
                'latitude' => 'pld.lat',
                'longitude' => 'pld.lng',
                'firstName' => 'pld.lead_first_name',
                'lastName' => 'pld.lead_last_name',
                'organizationName' => 'plo.organization_name',
                'address' => 'pld.lead_address',
                'phone' => 'pld.lead_phone',
                'mobile' => 'pld.lead_mobile',
                'phoneCountryCodeValue' => 'pld.lead_phone_country_code_value',
                'mobileCountryCodeValue' => 'pld.lead_mobile_country_code_value'
            ));

        $select->joinLeft(array('plop' => 'plugin_leads_opportunity'), 'plop.lead_id=pld.id', array())
            ->joinLeft(array('plo' => 'plugin_lead_organization'), 'plo.id=pld.organization_id', array());

        if (!empty($order)) {
            $select->order($order);
        }

        $select->limit($limit, $offset);

        if (!empty($where)) {
            $select->where($where);
        }

        $select->group('pld.id');

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        return $data;

    }


}
