<?php

/**
 * LeadsMeetingLogMapper.php
 *
 *
 * @method Leads_Mapper_LeadsMeetingLogMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsMeetingLogMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsMeetingLogModel';

    protected $_dbTable = 'Leads_DbTable_LeadsMeetingLogDbTable';

    /**
     * Save meeting log model to DB
     * @param $model Leads_Model_LeadsMeetingLogModel
     * @return Leads_Model_LeadsMeetingLogModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'meeting_id' => $model->getMeetingId(),
            'action_log_id' => $model->getActionLogId(),
            'start_time_date' => $model->getStartTimeDate(),
            'end_time_date' => $model->getEndTimeDate(),
            'location' => $model->getLocation(),
            'invite_attendees' => $model->getInviteAttendees(),
            'period' => $model->getPeriod(),
            'subject' => $model->getSubject(),
            'details' => $model->getDetails(),
            'google_calendar' => $model->getGoogleCalendar(),
            'meeting_status' => $model->getMeetingStatus(),
            'creator_id' => $model->getCreatorId(),
            'lead_id' => $model->getLeadId(),
            'additional_content' => $model->getAdditionalContent()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Change lead id
     *
     * @param array $leadIds lead ids
     * @param int $newLeadId new lead id
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeLeadId($leadIds, $newLeadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id IN (?)', $leadIds);
        $data = array('lead_id' => $newLeadId);
        $this->getDbTable()->getAdapter()->update('plugin_leads_meeting_log', $data, $where);
    }


    /**
     * Change creator id for call
     *
     * @param int $creatorIdFrom creator id from (system user id)
     * @param int $creatorIdTo creator id  to (system user id)
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeCreatorId($creatorIdFrom, $creatorIdTo)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('creator_id = ?', $creatorIdFrom);
        $data = array('creator_id' => $creatorIdTo);
        $this->getDbTable()->getAdapter()->update('plugin_leads_meeting_log', $data, $where);
    }


    /**
     * Delete lead email log record
     *
     * @param int $id lead id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_meeting_log', $where);

    }

}
