<?php

/**
 * LeadsMobileAppConfigMapper.php
 *
 *
 * @method Leads_Mapper_LeadsMobileAppConfigMapper  getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsMobileAppConfigMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsMobileAppConfigModel';

    protected $_dbTable = 'Leads_DbTable_LeadsMobileAppConfigDbTable';

    /**
     * Save lead imap config to DB
     * @param $model Leads_Model_LeadsMobileAppConfigModel
     * @return Leads_Model_LeadsImapConfigModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'user_id' => $model->getUserId(),
            'login' => $model->getLogin(),
            'password' => $model->getPassword(),
            'status' => $model->getStatus(),
            'social_app_status' => $model->getSocialAppStatus(),
            'created_at' => $model->getCreatedAt(),
            'updated_at' => $model->getUpdatedAt(),
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Get lead mobile app config data with info
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $singleRow single row result
     * @return array
     */
    public function fetchAllData($where = null, $order = null, $limit = null, $offset = null, $singleRow = false)
    {
        $fieldsToSelect = array(
            'plmac.id',
            'plmac.login',
            'plmac.user_id',
            'plmac.password',
            'plmac.status',
            'plmac.social_app_status',
            'plmac.created_at',
            'plmac.updated_at'
        );

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plmac' => 'plugin_leads_mobile_app_config'),
                $fieldsToSelect
            );


        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRow === true) {
            return $this->getDbTable()->getAdapter()->fetchRow($select);
        }

        $data = $this->getDbTable()->getAdapter()->fetchAll($select);

        $select->reset(Zend_Db_Select::COLUMNS);
        $select->reset(Zend_Db_Select::FROM);
        $select->reset(Zend_Db_Select::LIMIT_OFFSET);
        $select->reset(Zend_Db_Select::GROUP);

        $select->from(array('plmac' => 'plugin_leads_mobile_app_config'), array('count' => 'COUNT(plmac.id)'));
        $count = $this->getDbTable()->getAdapter()->fetchRow($select);

        return array(
            'totalRecords' => $count['count'],
            'data' => $data,
            'offset' => $offset,
            'limit' => $limit
        );
    }

    /**
     * Find record by email address
     *
     * @param string $login email address
     * @return Leads_Model_LeadsMobileAppConfigModel
     */
    public function findByEmail($login)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('login = ?', $login);
        return $this->_findWhere($where);
    }

    /**
     * @return array
     */
    public function getUniqueLogins()
    {
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plmac' => 'plugin_leads_mobile_app_config'),
                array('login')
            );

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Delete lead type record
     *
     * @param int $id lead type id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_mobile_app_config', $where);

    }

}
