<?php

/**
 * LeadsOpportunityQuoteMapper.php
 *
 *
 * @method Leads_Mapper_LeadsOpportunityQuoteMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsOpportunityQuoteMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsOpportunityQuoteModel';

    protected $_dbTable = 'Leads_DbTable_LeadsOpportunityQuoteDbTable';

    /**
     * Save lead opportunity to DB
     * @param $model Leads_Model_LeadsOpportunityQuoteModel
     * @return Leads_Model_LeadsOpportunityQuoteModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'quote_id' => $model->getQuoteId(),
            'opportunity_id' => $model->getOpportunityId(),
            'lead_id' => $model->getLeadId(),
            'cart_id' => $model->getCartId(),
            'created_at' => $model->getCreatedAt()
        );

        $leadOpportunityQuoteModel = $this->getOpportunityQuote($data['quote_id'], $data['opportunity_id'], $data['lead_id']);
        if (!$leadOpportunityQuoteModel instanceof Leads_Model_LeadsOpportunityQuoteModel) {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount without count flag
     * @param bool $singleRecord if true return single record
     * @return array
     */
    public function searchQuotes($where = null, $order = null, $limit = null, $offset = null, $withoutCount = false, $singleRecord = false)
    {
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('sq' => 'shopping_quote'),
                array(
                    'sq.id',
                    'sq.title'
                )
            );

        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRecord === true) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::GROUP);

            $select->from(array('sq' => 'shopping_quote'), array('count' => 'COUNT(sq.id)'));
            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        }

        return $data;
    }


    /**
     * get opportunity by params
     *
     * @param int $quoteId quote id
     * @param int $opportunityId opportunity
     * @param int $leadId lead id
     * @return null
     */
    public function getOpportunityQuote($quoteId, $opportunityId, $leadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('quote_id = ?', $quoteId);
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('opportunity_id = ?', $opportunityId);
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);

        return $this->_findWhere($where);
    }

    /**
     * Get quote ids
     *
     * @param int $opportunityId opportunity id
     * @return null
     */
    public function getOpportunityQuotesIds($opportunityId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('opportunity_id = ?', $opportunityId);
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_opportunity_quote',
            array('quote_id', 'quote_id'))->where($where);

        return $this->getDbTable()->getAdapter()->fetchPairs($select);

    }

    /**
     * Get opportunities
     *
     * @param int $opportunityId opportunity id
     * @return null
     */
    public function getOpportunities($opportunityId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('opportunity_id = ?', $opportunityId);
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_opportunity_quote',
            array('plugin_leads_opportunity_quote.*'))->where($where);

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);

    }

    /**
     * Get by cart id
     *
     * @param int $cartId cart id
     * @return null
     */
    public function getByCartId($cartId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('cart_id = ?', $cartId);

        return $this->_findWhere($where);
    }

    public function getQuotesWithoutLeads(int $leadId, $where = null) {
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('sq' => 'shopping_quote'), array(
                'sq.id',
                'sq.title'
            ))
            ->join(array('pld' => 'plugin_leads_data'), 'pld.user_id = sq.user_id', array())
            ->joinLeft(array('ploq' => 'plugin_leads_opportunity_quote'), 'ploq.quote_id = sq.id', array());

        if(!empty($where)) {
            $where .= ' AND ';
        }

        $where .= $this->getDbTable()->getAdapter()->quoteInto('pld.id = ?', $leadId);
        $where .= ' AND ' . new Zend_Db_Expr('ploq.quote_id IS NULL');

        $select->where($where);

        $data = $this->getDbTable()->getAdapter()->fetchAssoc($select);

        return $data;
    }

    /**
     * @param $opportunityId
     * @param $leadId
     * @return array
     */
    public function getOppQuoteAndCartInfoByOppId($opportunityId, $leadId) {
        $where = $this->getDbTable()->getAdapter()->quoteInto('opportunity_id = ?', $opportunityId);
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('ploq' => 'plugin_leads_opportunity_quote'),
                array(
                    'quoteId' => 'sq.id',
                    'quoteTitle' => 'sq.title',
                    'quoteStatus' => 'sq.status',
                    'scs.total'

                )
            )
            ->join(array('sq' => 'shopping_quote'), 'sq.id=ploq.quote_id', array())
            ->join(array('scs' => 'shopping_cart_session'), 'scs.id=ploq.cart_id', array());

        $select->where($where);

        return $this->getDbTable()->getAdapter()->fetchAll($select);
    }

}
