<?php

/**
 * LeadsOpportunityStageMapper.php
 *
 *
 * @method Leads_Mapper_LeadsOpportunityStageMapper  getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsOpportunityStageMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsOpportunityStageModel';

    protected $_dbTable = 'Leads_DbTable_LeadsOpportunityStageDbTable';

    /**
     * Save lead opportunity model to DB
     * @param $model Leads_Model_LeadsOpportunityStageModel
     * @return Leads_Model_LeadsOpportunityStageModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            return false;
        }
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $model->getId());
        $this->getDbTable()->update(array(
            'description' => $model->getDescription()
        ), $where);

        return $model;
    }

    /**
     * Get all opportunity types pairs
     * @param bool $excludeEmpty exclude not set opportunity stages
     * @param array $order order by
     *
     * @return array
     */
    public function getOpportunityStages($excludeEmpty = false, $order = array())
    {
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_opportunity_stage', array('id', 'description'));

        if ($excludeEmpty) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('description <> ?', '');
            $select->where($where);
        }

        if (!empty($order)) {
            $select->order($order);
        }

        return $this->getDbTable()->getAdapter()->fetchPairs($select);
    }

    /**
     * Get all opportunity types pairs
     * @param bool $excludeEmpty exclude not set opportunity stages
     * @param array $order order by
     *
     * @return array
     */
    public function getOpportunityStagesData($excludeEmpty = false, $order = array())
    {
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_opportunity_stage', array('id', 'description'));

        if ($excludeEmpty) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('description <> ?', '');
            $select->where($where);
        }

        if (!empty($order)) {
            $select->order($order);
        }

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Get default stage
     *
     * @return mixed
     */
    public function getDefaultStage()
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('`default` = ?', '1');
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_opportunity_stage', array('id', 'description', 'default'))->where($where);
        return $this->getDbTable()->getAdapter()->fetchAll($select);
    }

    /**
     * Assign default stage
     *
     * @param int $stageId stage id
     * @throws Zend_Db_Adapter_Exception
     */
    public function assignDefaultStage($stageId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $stageId);
        $data = array('default' => '1');
        $this->getDbTable()->getAdapter()->update('plugin_leads_opportunity_stage', $data, $where);

    }

    /**
     * Disable all default stages
     *
     * @return int
     * @throws Zend_Db_Adapter_Exception
     */
    public function disableDefaultStage()
    {
        return $this->getDbTable()->getAdapter()->update('plugin_leads_opportunity_stage', array('default' => '0'));
    }

}
