<?php

/**
 * LeadsOpportunityTypeMapper.php
 *
 *
 * @method Leads_Mapper_LeadsOpportunityTypeMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsOpportunityTypeMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsOpportunityTypeModel';

    protected $_dbTable = 'Leads_DbTable_LeadsOpportunityTypeDbTable';

    /**
     * Save lead opportunity type to DB
     * @param $model Leads_Model_LeadsOpportunityTypeModel
     * @return Leads_Model_LeadsTagModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'type_name' => $model->getTypeName(),
            'type_label' => $model->getTypeLabel()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Get opportunity type data with info
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount without count flag
     * @param bool $singleRecord if true return single record
     * @return array
     */
    public function fetchAll($where = null, $order = null, $limit = null, $offset = null, $withoutCount = false, $singleRecord = false)
    {
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plot' => 'plugin_leads_opportunity_type'),
                array(
                    'plot.id',
                    'plot.type_name',
                    'plot.type_label'
                )
            );
        
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRecord === true) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::GROUP);

            $select->from(array('plot' => 'plugin_leads_opportunity_type'), array('count' => 'COUNT(plot.id)'));
            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        }

        return $data;
    }


    /**
     * Get all opportunity types pairs
     *
     * @return array
     */
    public function getOpportunityTypes()
    {
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_opportunity_type', array('id', 'type_name', 'type_label'));

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Get opportunity types pairs id -> type_label
     *
     * @param array $order order by
     *
     * @return array
     */
    public function getOpportunityTypesPairs($order = array())
    {
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_opportunity_type', array('id', 'type_label'));
        if (!empty($order)) {
            $select->order($order);
        }

        return $this->getDbTable()->getAdapter()->fetchPairs($select);
    }

    /**
     * Get opportunity type by tag name
     *
     * @param string $typeName tag name
     *
     * @return array
     */
    public function getOpportunityType($typeName)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('type_name = ?', $typeName);

        return $this->_findWhere($where);
    }

    /**
     * Delete lead opportunity type record
     *
     * @param int $id lead opportunity type id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_opportunity_type', $where);

    }

}
