<?php

/**
 * LeadsOrganizationCustomParamsMapper.php
 *
 *
 * @method Leads_Mapper_LeadsOrganizationCustomParamsMapper  getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsOrganizationCustomParamsMapper extends Application_Model_Mappers_Abstract implements Leads_LeadsCustomParamsMapperInterface
{

    protected $_model = 'Leads_Model_LeadsOrganizationCustomParamsModel';

    protected $_dbTable = 'Leads_DbTable_LeadsOrganizationCustomParamsDbTable';

    /**
     * Save organization custom params model to DB
     * @param $model Leads_Model_LeadsOrganizationCustomParamsModel
     * @return Leads_Model_LeadsOrganizationCustomParamsModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'param_id' => $model->getParamId(),
            'organization_id' => $model->getOrganizationId(),
            'param_value' => $model->getParamValue(),
            'params_option_id' => $model->getParamsOptionId()
        );

        $id = $model->getId();

        if (empty($id)) {
            $existingModel = $this->getParamByIdAndParamId($data['organization_id'], $data['param_id']);
            if ($existingModel instanceof Leads_Model_LeadsOrganizationCustomParamsModel) {
                $id = $existingModel->getId();
            }
        }

        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Get param by organization id and param id
     *
     * @param int $organizationId organization id
     * @param int $paramId param id
     * @return null
     */
    public function getParamByIdAndParamId($organizationId, $paramId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('organization_id = ?', $organizationId);
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('param_id = ?', $paramId);
        return $this->_findWhere($where);
    }

    /**
     * Get organization custom params
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @return array
     */
    public function fetchAll(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $withoutCount = false,
        $singleRecord = false
    ) {
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('ploccpd' => 'plugin_leads_organization_custom_params_data'),
                array(
                    'ploccpd.id',
                    'ploccpd.param_id',
                    'ploccpd.organization_id',
                    'ploccpd.param_value',
                )
            );
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::GROUP);

            $select->from(array('ploccpd' => 'plugin_leads_organization_custom_params_data'), array('count' => 'COUNT(ploccpd.id)'));
            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }

    /**
     * Get organization custom params and original config params
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @return array
     */
    public function fetchAllData(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $withoutCount = false,
        $singleRecord = false
    ) {
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('ploccpc' => 'plugin_leads_organization_custom_params_config'),
                array(
                    'param_id' => 'ploccpc.id',
                    'ploccpc.param_type',
                    'ploccpc.param_name',
                    'ploccpc.label',
                    'option_values' => new Zend_Db_Expr('GROUP_CONCAT(plocpod.option_value)')
                )
            )
            ->joinLeft(array('plocpod' => 'plugin_leads_organization_custom_params_options_data'), 'plocpod.custom_param_id = ploccpc.id', array())
            ->group('ploccpc.id');
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::GROUP);

            $select->from(array('ploccpc' => 'plugin_leads_organization_custom_params_config'), array('count' => 'COUNT(ploccpc.id)'));
            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }

    /**
     * Get params prepared for grid (associative array which has as a key - custom param config id)
     *
     * @param int $organizationId lead id
     * @return mixed
     */
    public function getParamsForGrid($organizationId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('organization_id = ?', $organizationId);
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pllcpd' => 'plugin_leads_organization_custom_params_data'), array('param_id', 'organization_id', 'id', 'param_value'))
            ->where($where);
        return  $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }


    /**
     * Get params (associative array which has as a key - custom param config id)
     *
     * @param int $organizationId lead id
     * @return mixed
     */
    public function getAllParams($organizationId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('organization_id = ?', $organizationId);
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pllcpd' => 'plugin_leads_organization_custom_params_data'), array('param_id', 'organization_id', 'id', 'param_value', 'params_option_id'))
            ->where($where);
        return  $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Update selections option param values
     *
     * @param $optionId
     * @param $optionValue
     */
    public function selectUpdateOptionsValues($optionId , $optionValue)
    {
        if(!empty($optionValue)){
            $data = array('param_value' => $optionValue);

            $where = $this->getDbTable()->getAdapter()->quoteInto('params_option_id = ?', $optionId);
            $select = $this->getDbTable()->getAdapter()->select()
                ->from(array('plocpd' => 'plugin_leads_organization_custom_params_data'), array('params_option_id'))
                ->where($where);
            $resultRow =  $this->getDbTable()->getAdapter()->fetchRow($select);

            if(!empty($resultRow)){
                $this->getDbTable()->update($data , $where);
            }
        }
    }

    /**
     * Delete organization custom param record
     *
     * @param int $id organization custom param id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_organization_custom_params_data', $where);

    }

}
