<?php

class Leads_Mapper_LeadsOrganizationDocumentsMapper extends Application_Model_Mappers_Abstract
{
    protected $_model = 'Leads_Model_LeadsOrganizationDocumentsModel';

    protected $_dbTable = 'Leads_DbTable_LeadsOrganizationDocumentsDbTable';

    /**
     * @param $model
     * @return mixed
     * @throws Exception
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'organization_id'    => $model->getOrganizationId(),
            'file_stored_name'   => $model->getFileStoredName(),
            'file_hash'          => $model->getFileHash(),
            'original_file_name' => $model->getOriginalFileName(),
            'display_file_name'  => $model->getDisplayFileName(),
            'uploaded_at'        => $model->getUploadedAt()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * @param $organizationId
     * @return array|null
     * @throws Exception
     */
    public function findDocumentsByOrganizationId($organizationId)
    {
        $select = $this->getDbTable()->getAdapter()->select(Zend_Db_Table::SELECT_WITHOUT_FROM_PART)->from(array('plodoc' => 'plugin_leads_organization_documents') ,
            array(
                'plodoc.id',
                'leadId'           => 'plodoc.organization_id',
                'fileStoredName'   => 'plodoc.file_stored_name',
                'fileHash'         => 'plodoc.file_hash',
                'originalFileName' => 'plodoc.original_file_name',
                'displayfilename'  => 'plodoc.display_file_name',
                'uploadedAt'       => 'plodoc.uploaded_at'
            ));
        $where = $this->getDbTable()->getAdapter()->quoteInto('plodoc.organization_id = ?', $organizationId);
        $select->where($where);
        $select->order('plodoc.uploaded_at DESC');

        return $this->getDbTable()->getAdapter()->fetchAll($select);
    }

    /**
     * @param $id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
        return $this->getDbTable()->getAdapter()->delete('plugin_leads_organization_documents', $where);
    }


}
