<?php

/**
 * LeadsOrganizationMapper.php
 *
 *
 * @method Leads_Mapper_LeadsOrganizationMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsOrganizationMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsOrganizationModel';

    protected $_dbTable = 'Leads_DbTable_LeadsOrganizationDbTable';

    /**
     * Save lead organization model to DB
     * @param $model Leads_Model_LeadsOrganizationModel
     * @return Leads_Model_LeadsOrganizationModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        if (empty($model->getSkipCoordinatesGeneration())) {
            $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
            $enableLeadOrgGeotagging = $configHelper->getConfig('enableLeadOrgGeotagging');
            if (!empty($enableLeadOrgGeotagging)) {
                $model = Tools_LeadGeolocationTools::generateAddressHashFromOrganizationModelAndAssignData($model);
            }
        }

        $data = array(
            'organization_name' => $model->getOrganizationName(),
            'organization_address' => $model->getOrganizationAddress(),
            'organization_address1' => $model->getOrganizationAddress1(),
            'organization_city' => $model->getOrganizationCity(),
            'organization_zip' => $model->getOrganizationZip(),
            'organization_country_code' => $model->getOrganizationCountryCode(),
            'organization_state_code' => $model->getOrganizationStateCode(),
            'customer_group_id' => $model->getCustomerGroupId(),
            'organization_website' => $model->getOrganizationWebsite(),
            'organization_website1' => $model->getOrganizationWebsite1(),
            'created_at' => $model->getCreatedAt(),
            'updated_at' => $model->getUpdatedAt(),
            'organization_industry_id' => $model->getOrganizationIndustryId(),
            'organization_phone' => $model->getOrganizationPhone(),
            'organization_email' => $model->getOrganizationEmail(),
            'organization_employees' => $model->getOrganizationEmployees(),
            'organization_domain' => $model->getOrganizationDomain(),
            'organization_domain1' => $model->getOrganizationDomain1(),
            'organization_phone_country_code' => $model->getOrganizationPhoneCountryCode(),
            'organization_phone_country_code_value' => $model->getOrganizationPhoneCountryCodeValue(),
            'organization_phone_extension' => $model->getOrganizationPhoneExtension(),
            'organization_mobile' => $model->getOrganizationMobile(),
            'organization_mobile_country_code' => $model->getOrganizationMobileCountryCode(),
            'organization_mobile_country_code_value' => $model->getOrganizationMobileCountryCodeValue(),
            'external_id' => $model->getExternalId(),
            'creation_type' => $model->getCreationType(),
            'primary_contact_id' => $model->getPrimaryContactId(),
            'first_contact_creation_date' => $model->getFirstContactCreationDate(),
            'organization_notes' => $model->getOrganizationNotes(),
            'address_hash'      => $model->getAddressHash(),
            'lat'               => $model->getLat(),
            'lng'               => $model->getLng(),
            'last_coordinates_processed_date'=> $model->getLastCoordinatesProcessedDate()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Find by organization name
     *
     * @param string $organizationName organization name
     * @return null
     */
    public function findByName($organizationName)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('organization_name = ?', $organizationName);
        return $this->_findWhere($where);
    }

    /**
     * Find by primary contact id
     *
     * @param int $leadId lead id
     * @return null
     */
    public function findByPrimaryContactId($leadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('primary_contact_id = ?', $leadId);
        return $this->_findWhere($where);
    }

    /**
     * Get lead organization data with info etc..
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param bool $withOpportunityAmount select with opportunity amount
     * @param string $having mysql having
     * @param bool $nextActivityDate enable search by next activity date
     * @param bool $withLeadsData leads data
     * @param array $additionalOrganizationParamsForSelect select query params
     * @param bool $withLeadsTags leads tags
     * @return array
     */
    public function fetchAll(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $withoutCount = false,
        $singleRecord = false,
        $withOpportunityAmount = false,
        $having = '',
        $nextActivityDate = false,
        $withLeadsData = false,
        $includesCustomOrganizationFields = false,
        $withSpentPurchaseAmount = false,
        $additionalOrganizationParamsForSelect = array(),
        $withLeadsTags = false

    )
    {

        $params = array(
            'plo.id',
            'plo.organization_name',
            'plo.organization_address',
            'plo.organization_city',
            'plo.organization_zip',
            'plo.organization_country_code',
            'plo.organization_state_code',
            'plo.organization_website',
            'plo.organization_website1',
            'plo.customer_group_id',
            'group_name' => new Zend_Db_Expr('GROUP_CONCAT(DISTINCT(shgr.groupName))'),
            'plo.created_at',
            'plo.updated_at',
            'plo.organization_industry_id',
            'plo.organization_phone',
            'plo.organization_email',
            'plo.organization_employees',
            'plo.organization_mobile',
            'plo.organization_mobile_country_code',
            'plo.organization_mobile_country_code_value',
            'plo.organization_phone_country_code',
            'plo.organization_phone_country_code_value',
            'plo.organization_phone_extension',
            'plo.organization_address1',
            'plo.organization_notes',
            'plo.external_id',
            'primary_contact_name' => new Zend_Db_Expr('CONCAT(pldm.lead_first_name, " ", pldm.lead_last_name)'),
            'plo.primary_contact_id',
            'plo.first_contact_creation_date',
            'plodoc.file_stored_name'
        );

        $params = array_merge($additionalOrganizationParamsForSelect, $params);

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plo' => 'plugin_lead_organization'),
               $params
            )
            ->joinLeft(array('shgr' => 'shopping_group'), 'shgr.id=plo.customer_group_id', array())
            ->joinLeft(array('pldm' => 'plugin_leads_data'), 'pldm.id=plo.primary_contact_id', array())
            ->joinLeft(array('plodoc' => 'plugin_leads_organization_documents'), 'plodoc.organization_id=plo.id', array());

        if ($withLeadsData === true || $withOpportunityAmount === true || $withSpentPurchaseAmount === true) {
            $select->joinLeft(array('pld' => 'plugin_leads_data'), 'plo.id=pld.organization_id', array());
        }

        if ($withLeadsTags === true) {
            $select->joinLeft(array('plht' => 'plugin_leads_has_tags'), 'pld.id=plht.lead_id', array());
        }

        if (true === $withOpportunityAmount) {
            $select->joinLeft(array('plop' => 'plugin_leads_opportunity'), 'plop.lead_id=pld.id', array());
        }

        if (true === $withSpentPurchaseAmount) {
            $select->joinLeft(array('scs' => 'shopping_cart_session'), 'scs.user_id=pld.user_id', array());
        }

        if (true === $includesCustomOrganizationFields) {
            $select->joinLeft(array('plocpd' => 'plugin_leads_organization_custom_params_data'), 'plocpd.organization_id=plo.id', array());
        }

        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->group('plo.id');

        if (!empty($having)) {
            $select->having($having);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::LIMIT_COUNT);

            $count = array('count' => new Zend_Db_Expr('COUNT(DISTINCT(plo.id))'));
            $count = array_merge($count, $additionalOrganizationParamsForSelect);

            $select->from(array('plo' => 'plugin_lead_organization'), $count)
                ->joinLeft(array('shgr' => 'shopping_group'), 'shgr.id=plo.customer_group_id', array())
                ->joinLeft(array('pldm' => 'plugin_leads_data'), 'pldm.id=plo.primary_contact_id', array());

            if ($withLeadsData === true || $withOpportunityAmount === true || $withSpentPurchaseAmount === true) {
                $select->joinLeft(array('pld' => 'plugin_leads_data'), 'plo.id=pld.organization_id', array());
            }

            if ($withLeadsTags === true) {
                $select->joinLeft(array('plht' => 'plugin_leads_has_tags'), 'pld.id=plht.lead_id', array());
            }

            if (true === $withOpportunityAmount) {
                $select->joinLeft(array('plop' => 'plugin_leads_opportunity'), 'plop.lead_id=pld.id', array());
            }

            if (true === $withSpentPurchaseAmount) {
                $select->joinLeft(array('scs' => 'shopping_cart_session'), 'scs.user_id=pld.user_id', array());
            }

            if (true === $includesCustomOrganizationFields) {
                $select->joinLeft(array('plocpd' => 'plugin_leads_organization_custom_params_data'), 'plocpd.organization_id=plo.id', array());
            }

            $select =  $this->getDbTable()->getAdapter()->select()
                ->from(
                    array('subres' => $select),
                    array('count' => 'SUM(count)')
                );

            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }

    /**
     * Get last contacted lead info
     *
     * @param int $organizationId organization id
     * @return mixed
     */
    public function getLastContactedLead($organizationId)
    {
        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');

        $createdAt = new Zend_Db_Expr("CONVERT_TZ(`plal`.`created_at`, '+00:00', '$mysqlOffset')");
        $now = Tools_System_Tools::convertDateFromTimezone('now');

        $where = $this->getDbTable()->getAdapter()->quoteInto('plo.id = ?', $organizationId);
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('plal.created_at <= ?', $now);
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plo' => 'plugin_lead_organization'),
                array('created_at' => $createdAt, 'plal.type', 'plal.action_type', 'plal.lead_type'))
            ->joinLeft(array('pld' => 'plugin_leads_data'), 'pld.organization_id=plo.id', array())
            ->join(array('plal' => 'plugin_leads_action_log'), 'plal.lead_id=pld.id', array())->where($where)->order('plal.created_at DESC')->limit(1);

        return $this->getDbTable()->getAdapter()->fetchRow($select);
    }

    /**
     * Get all leads under organization id
     *
     * @param int $organizationId organization id
     * @return array
     */
    public function getOrganizationLeads($organizationId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plo.id = ?', $organizationId);

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plo' => 'plugin_lead_organization'), array('pld.id', 'pld.lead_first_name', 'pld.lead_last_name'))
            ->join(array('pld' => 'plugin_leads_data'), 'pld.organization_id=plo.id', array())
            ->where($where);

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     *  Get first purchase date
     *
     * @param int $organizationId organization id
     * @return mixed
     */
    public function getFirstPurchaseDate($organizationId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('scs.status IN (?)', array(Models_Model_CartSession::CART_STATUS_COMPLETED, Models_Model_CartSession::CART_STATUS_SHIPPED, Models_Model_CartSession::CART_STATUS_DELIVERED));
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('scs.created_at <> ?', '0000-00-00 00:00:00');
        if (!empty($organizationId)) {
            $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('plo.id = ?', $organizationId);
        }
        $select = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_lead_organization'), array('scs.created_at'))
            ->join(array('pld' => 'plugin_leads_data'), 'pld.organization_id=plo.id', array())
            ->joinLeft(array('scs' => 'shopping_cart_session'), 'scs.user_id=pld.user_id', array());

        $select->where($where);
        $select->order('scs.created_at ASC');

        return $this->getDbTable()->getAdapter()->fetchRow($select);

    }

    /**
     * Delete lead record
     *
     * @param int $id lead id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_lead_organization', $where);

    }

    public function getOrganizationWithLeads()
    {
        $where = new Zend_Db_Expr('organization_id IS NOT NULL');
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_data', array('organization_id'));
        $select->where($where);
        $select->group('organization_id');

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

}
