<?php

/**
 * LeadsOwnerGoalsMapper.php
 *
 *
 * @method Leads_Mapper_LeadsOwnerGoalsMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsOwnerGoalsMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsOwnerGoalsModel';

    protected $_dbTable = 'Leads_DbTable_LeadsOwnerGoalsDbTable';

    /**
     * @param $model Leads_Model_LeadsOwnerGoalsModel
     * @return Leads_Model_LeadsOwnerGoalsModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'user_id' => $model->getUserId(),
            'quotes_amount' => $model->getQuotesAmount(),
            'quote_objective' => $model->getQuoteObjective(),
            'meetings_held' => $model->getMeetingsHeld(),
            'calls_placed' => $model->getCallsPlaced(),
            'emails_sent' => $model->getEmailsSent()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }


    /**
     * Get lead goal config data with info
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param string $groupBy mysql group by
     * @return array
     */
    public function fetchAllData($where = null, $order = null, $limit = null, $offset = null, $withoutCount = false,
        $singleRecord = false, $groupBy = '')
    {
        $fieldsToSelect = array(
            'plog.*',
            'full_name' => 'u.full_name'
        );

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plog' => 'plugin_leads_owner_goals'),
                $fieldsToSelect
            )->join(array('u' => 'user'), 'u.id=plog.user_id', array());


        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }
        $select->limit($limit, $offset);


        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::LIMIT_COUNT);
            if (empty($groupBy)) {
                $select->reset(Zend_Db_Select::GROUP);
            }

            $select->from(array('plog' => 'plugin_leads_owner_goals'),
                array('count' => 'COUNT(plog.id)'))->join(array('u' => 'user'), 'u.id=plog.user_id', array());

            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }

    /**
     * Get main goal config
     *
     * @param int $userId user id
     *
     * @return array
     */
    public function getMainConfigData($userId = 0)
    {
        $fieldsToSelect = array(
            'plog.user_id',
            'plog.*',
            'full_name' => 'u.full_name'
        );

        $where = null;

        if (!empty($userId)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('plog.user_id = ?', $userId);
        }

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plog' => 'plugin_leads_owner_goals'),
                $fieldsToSelect
            )->join(array('u' => 'user'), 'u.id=plog.user_id', array());

        if (!empty($where)) {
            $select->where($where);
        }

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Find by user id
     *
     * @param int $userId user id
     * @return null
     */
    public function findByUserId($userId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('user_id = ?', $userId);
        return $this->_findWhere($where);
    }


    /**
     * @param int $id row id
     * @return int
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_owner_goals', $where);
    }

}
