<?php

/**
 * LeadsOwnerGoalsOpportunitiesMapper.php
 *
 *
 * @method Leads_Mapper_LeadsOwnerGoalsOpportunitiesMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsOwnerGoalsOpportunitiesMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsOwnerGoalsOpportunitiesModel';

    protected $_dbTable = 'Leads_DbTable_LeadsOwnerGoalsOpportunitiesDbTable';

    /**
     * @param $model Leads_Model_LeadsOwnerGoalsOpportunitiesModel
     * @return Leads_Model_LeadsOwnerGoalsOpportunitiesModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'owner_goals_id' => $model->getOwnerGoalsId(),
            'opportunity_amount' => $model->getOpportunityAmount(),
            'opportunity_count' => $model->getOpportunityCount(),
            'opportunity_type_id' => $model->getOpportunityTypeId()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Find by user id
     *
     * @param int $userId user id
     * @return array
     */
    public function findByUserId($userId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('user_id = ?', $userId);
        $select = $this->getDbTable()->getAdapter()->select()->from(array('plogo' => 'plugin_leads_owner_goals_opportunities'))->where($where);

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);

    }

    /**
     * Find by  id
     *
     * @param int $id user id
     * @return array
     */
    public function findById($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
        $select = $this->getDbTable()->getAdapter()->select()->from(array('plogo' => 'plugin_leads_owner_goals_opportunities'))->where($where);

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);

    }

    /**
     * @param int $goalId goal id
     * @param int $oppId opportunity id
     * @return Leads_Model_LeadsOwnerGoalsOpportunitiesModel
     */
    public function findByGoalAndOppId($goalId, $oppId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('owner_goals_id = ?', $goalId);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('opportunity_type_id = ?', $oppId);

        return $this->_findWhere($where);
    }

    /**
     * @param int $goalId goal id
     * @param int $oppType opp type id
     * @return array
     */
    public function findByGoalId($goalId, $oppType = 0)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('owner_goals_id = ?', $goalId);

        if (!empty($oppType)) {
            $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('opportunity_type_id = ?', $oppType);
        }

        $select = $this->getDbTable()->getAdapter()->select()->from(array('plogo' => 'plugin_leads_owner_goals_opportunities'))->where($where);

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);

    }

    public function getAllOpportunities()
    {
        $select = $this->getDbTable()->getAdapter()->select()->from(array('plogo' => 'plugin_leads_owner_goals_opportunities'));

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }


}
