<?php

/**
 * LeadsPurchaseLogMapper.php
 *
 *
 * @method Leads_Mapper_LeadsPurchaseLogMapper  getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsPurchaseLogMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsPurchaseLogModel';

    protected $_dbTable = 'Leads_DbTable_LeadsPurchaseLogDbTable';

    /**
     * Save purchase log model to DB
     * @param $model Leads_Model_LeadsPurchaseLogModel
     * @return Leads_Model_LeadsPurchaseLogModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'cart_id' => $model->getCartId(),
            'created_at' => $model->getCreatedAt(),
            'action_log_id' => $model->getActionLogId(),
            'status' => $model->getStatus(),
            'lead_id' => $model->getLeadId(),
            'total' => $model->getTotal(),
            'is_partial' => $model->getIsPartial(),
            'is_manually_paid' => $model->getIsManuallyPaid(),
            'partially_paid_amount' => $model->getPartiallyPaidAmount(),
            'gateway' => $model->getGateway(),
            'signed_only_by' => $model->getSignedOnlyBy()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Find last purchase by cart id
     *
     * @param int $cartId cart id
     * @param string $status cart status
     * @return null
     */
    public function findByCartId($cartId, $status)
    {
       $where = $this->getDbTable()->getAdapter()->quoteInto('cart_id = ?', $cartId);
       $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('status = ?', $status);

       return $this->_findWhere($where);
    }

    /**
     * Change lead id
     *
     * @param array $leadIds lead ids
     * @param int $newLeadId new lead id
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeLeadId($leadIds, $newLeadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id IN (?)', $leadIds);
        $data = array('lead_id' => $newLeadId);
        $this->getDbTable()->getAdapter()->update('plugin_leads_purchase_log', $data, $where);
    }


    /**
     * Delete lead quote log record
     *
     * @param int $id lead id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_quote_log', $where);

    }

}
