<?php

/**
 * LeadsQuoteLogMapper.php
 *
 *
 * @method Leads_Mapper_LeadsQuoteLogMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsQuoteLogMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsQuoteLogModel';

    protected $_dbTable = 'Leads_DbTable_LeadsQuoteLogDbTable';

    /**
     * Save quote log model to DB
     * @param $model Leads_Model_LeadsQuoteLogModel
     * @return Leads_Model_LeadsQuoteLogModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'quote_id' => $model->getQuoteId(),
            'title' => $model->getTitle(),
            'status' => $model->getStatus(),
            'created_at' => $model->getCreatedAt(),
            'action_log_id' => $model->getActionLogId(),
            'creator_id' => $model->getCreatorId(),
            'quote_link' => $model->getQuoteLink(),
            'quote_action_type' => $model->getQuoteActionType(),
            'lead_id' => $model->getLeadId(),
            'total' => $model->getTotal(),
            'quote_sent' => $model->getQuoteSent(),
            'is_deleted' => $model->getIsDeleted(),
            'deleted_by' => $model->getDeletedBy(),
            'force_display' => $model->getForceDisplay(),
            'signed_only_by' => $model->getSignedOnlyBy()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Find last quote by quote id and quote action type
     *
     * @param int $quoteId quote id
     * @param string $quoteActionType quote action type (change|view)
     * @return null
     */
    public function findByLastQuoteId($quoteId, $quoteActionType = Leads_Model_LeadsQuoteLogModel::QUOTE_ACTION_TYPE_CHANGE)
    {
       $where = $this->getDbTable()->getAdapter()->quoteInto('quote_id = ?', $quoteId);
       $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('quote_action_type = ?', $quoteActionType);
       $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_quote_log')
           ->where($where)->order('created_at DESC');
       return $this->getDbTable()->getAdapter()->fetchRow($select);
    }

    /**
     * Delete lead quote log record
     *
     * @param int $id lead id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_quote_log', $where);

    }

    /**
     * Change lead id
     *
     * @param array $leadIds lead ids
     * @param int $newLeadId new lead id
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeLeadId($leadIds, $newLeadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id IN (?)', $leadIds);
        $data = array('lead_id' => $newLeadId);
        $this->getDbTable()->getAdapter()->update('plugin_leads_quote_log', $data, $where);
    }


    /**
     * Find the list of quotes by quote id
     *
     * @param $quoteId
     * @return array|null
     */
    public function findByQuoteId($quoteId){
        $where = $this->getDbTable()->getAdapter()->quoteInto('quote_id = ?', $quoteId);

        return $this->fetchAll($where);
    }

    /** Change creator id for
     *
     * @param int $creatorIdFrom creator id from (system user id)
     * @param int $creatorIdTo creator id  to (system user id)
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeCreatorId($creatorIdFrom, $creatorIdTo)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('creator_id = ?', $creatorIdFrom);
        $data = array('creator_id' => $creatorIdTo);
        $this->getDbTable()->getAdapter()->update('plugin_leads_quote_log', $data, $where);
    }

    /** Change creator id for original quote
     *
     * @param int $creatorIdFrom creator id from (system user id)
     * @param int $creatorIdTo creator id  to (system user id)
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeCreatorIdOriginalQuote($creatorIdFrom, $creatorIdTo)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('creator_id = ?', $creatorIdFrom);
        $data = array('creator_id' => $creatorIdTo);
        $this->getDbTable()->getAdapter()->update('shopping_quote', $data, $where);
    }

    /** Change editor id for original quote
     *
     * @param int $editorIdFrom creator id from (system user id)
     * @param int $editorIdTo creator id  to (system user id)
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeEditorIdOriginalQuote($editorIdFrom, $editorIdTo)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('editor_id = ?', $editorIdFrom);
        $data = array('editor_id' => $editorIdTo);
        $this->getDbTable()->getAdapter()->update('shopping_quote', $data, $where);
    }


}
