<?php

/**
 * LeadsRegistrationLogMapper.php
 *
 *
 * @method Leads_Mapper_LeadsRegistrationLogMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsRegistrationLogMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsRegistrationLogModel';

    protected $_dbTable = 'Leads_DbTable_LeadsRegistrationLogDbTable';

    /**
     * Save user website registration log model to DB
     * @param $model Leads_Model_LeadsRegistrationLogModel
     * @return Leads_Model_LeadsRegistrationLogModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'email' => $model->getEmail(),
            'first_name' => $model->getFirstName(),
            'last_name' => $model->getLastName(),
            'mobile' => $model->getMobile(),
            'action_log_id' => $model->getActionLogId(),
            'lead_id' => $model->getLeadId(),
            'user_role' => $model->getUserRole(),
            'registration_page_url' => $model->getRegistrationPageUrl(),
            'registration_page_id' => $model->getRegistrationPageId(),
            'user_id' => $model->getUserId()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Find by email
     *
     * @param string $email email
     * @return null
     */
    public function findByEmail($email)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('email = ?', $email);
        return $this->_findWhere($where);
    }


    /**
     * Find by lead id
     *
     * @param string $leadId lead id
     * @return null
     */
    public function findByLeadId($leadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);
        return $this->_findWhere($where);
    }

    /**
     * Change lead id
     *
     * @param array $leadIds lead ids
     * @param int $newLeadId new lead id
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeLeadId($leadIds, $newLeadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id IN (?)', $leadIds);
        $data = array('lead_id' => $newLeadId);
        $this->getDbTable()->getAdapter()->update('plugin_leads_registration_log', $data, $where);
    }

    /**
     * Delete lead form log record
     *
     * @param int $id lead id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_registration_log', $where);

    }

}
