<?php

/**
 * LeadsRemarketingAudiencesMapper.php
 */
class Leads_Mapper_LeadsRemarketingAudiencesMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsRemarketingAudiencesModel';

    protected $_dbTable = 'Leads_DbTable_LeadsRemarketingAudiencesDbTable';

    /**
     * Save remarketing audiences model
     * @param $model Leads_Model_LeadsRemarketingAudiencesModel
     * @return Leads_Model_LeadsRemarketingAudiencesModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'type'            => $model->getType(),
            'action'          => $model->getAction(),
            'audience_id'     => $model->getAudienceId(),
            'list_content'    => $model->getListContent(),
            'additional_info' => $model->getAdditionalInfo(),
            'created_at'      => $model->getCreatedAt()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    public function fetchAll(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $groupBy = ''
    ) {

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plra' => 'plugin_leads_remarketing_audiences'),
                array(
                    'plra.id',
                    'type'           => 'plra.type',
                    'action'         => 'plra.action',
                    'audienceId'     => 'plra.audience_id',
                    'listContent'    => 'plra.list_content',
                    'additionalInfo' => 'plra.additional_info',
                    'createdAt'      => 'plra.created_at',
                )
            );
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        if (!empty($groupBy)) {
            $select->group($groupBy);
        }

        $select->limit($limit, $offset);

        $data = $this->getDbTable()->getAdapter()->fetchAll($select);

        return $data;
    }

    /**
     * @param $ids
     * @return void
     * @throws Exception
     */
    public function delete($ids)
    {
        if(!empty($ids)) {
            if(is_array($ids)) {
                $where = $this->getDbTable()->getAdapter()->quoteInto('id IN (?)', $ids);
            } else {
                $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $ids);
            }

            return $this->getDbTable()->getAdapter()->delete('plugin_leads_remarketing_audiences', $where);
        }
    }


}
