<?php

/**
 * LeadsReportsMapper.php
 *
 *
 * @method Leads_Mapper_LeadsReportsMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsReportsMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsModel';

    protected $_dbTable = 'Leads_DbTable_LeadsDbTable';

    public function save($model)
    {

    }

    /**
     * Alternative method
     *
     * get leads by owner count
     *
     * @param string $createdFrom mysql date format
     * @param string $createdTo mysql date format
     * @return array
     */
    public function getLeadsByOwnerCount($createdFrom, $createdTo)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('created_at >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('created_at <= ?', $createdTo);
        $select = $this->getDbTable()->getAdapter()->select()->from(array('pld' => 'plugin_leads_data'),
            array('u.id', 'u.full_name', 'count' => new Zend_Db_Expr('COUNT(pld.id)')))
            ->join(array('u' => 'user'), 'pld.owner_id=u.id', array())
            ->where($where)
            ->group('u.id');


        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     *
     * get leads by owner count
     *
     * @param string $createdFrom mysql date format
     * @param string $createdTo mysql date format
     * @param string $createdFromCompare mysql date format
     * @param string $createdToCompare mysql date format
     * @return array
     * @throws Zend_Db_Select_Exception
     */
    public function getLeadsByOwnerCountAggregated(
        $createdFrom,
        $createdTo,
        $createdFromCompare = false,
        $createdToCompare = false,
        $restrictedUserId = 0
    ) {
        $where = $this->getDbTable()->getAdapter()->quoteInto('created_at >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('created_at <= ?', $createdTo);

        if(!empty($restrictedUserId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
        }

        $selectFrom = $this->getDbTable()->getAdapter()->select()->from(array('pld' => 'plugin_leads_data'), array(
            'uniqueId' => new Zend_Db_Expr("CONCAT(u.id, '_current')"),
            new Zend_Db_Expr("'current' as type"),
            'u.id',
            'u.full_name',
            'count' => new Zend_Db_Expr('COUNT(pld.id)')
        ))
            ->join(array('u' => 'user'), 'pld.owner_id=u.id', array())
            ->where($where)
            ->group('u.id');

        if ($createdFromCompare !== false && $createdToCompare !== false) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('created_at >= ?', $createdFromCompare);
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('created_at <= ?', $createdToCompare);

            if(!empty($restrictedUserId)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
            }

            $selectTo = $this->getDbTable()->getAdapter()->select()->from(array('pld' => 'plugin_leads_data'),
                array(
                    'uniqueId' => new Zend_Db_Expr("CONCAT(u.id, '_previous')"),
                    new Zend_Db_Expr("'previous' as type"),
                    'u.id',
                    'u.full_name',
                    'count' => new Zend_Db_Expr('COUNT(pld.id)')
                ))
                ->join(array('u' => 'user'), 'pld.owner_id=u.id', array())
                ->where($where)
                ->group('u.id');
            $select = $this->getDbTable()->select()
                ->union(array($selectFrom, $selectTo));
        } else {
            $select = $selectFrom;
        }

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);

    }

    /**
     *
     * get opportunities count by owner count
     *
     * @param string $createdFrom mysql date format
     * @param string $createdTo mysql date format
     * @param string $createdFromCompare mysql date format
     * @param string $createdToCompare mysql date format
     * @return array
     * @throws Zend_Db_Select_Exception
     */
    public function getOpportunitiesByOwnerCountAggregated (
        $createdFrom,
        $createdTo,
        $createdFromCompare = false,
        $createdToCompare = false,
        $restrictedUserId = 0
    ) {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plo.created_at >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plo.created_at <= ?', $createdTo);

        if(!empty($restrictedUserId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
        }

        $selectFrom = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'), array(
            'uniqueId' => new Zend_Db_Expr("CONCAT(u.id, '_current')"),
            new Zend_Db_Expr("'current' as type"),
            'u.id',
            'u.full_name',
            'count' => new Zend_Db_Expr('COUNT(plo.id)')
        ))
            ->join(array('u' => 'user'), 'plo.creator_id=u.id', array())
            ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plo.lead_id', array())
            ->where($where)
            ->group('u.id');

        if ($createdFromCompare !== false && $createdToCompare !== false) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('plo.created_at >= ?', $createdFromCompare);
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plo.created_at <= ?', $createdToCompare);

            if(!empty($restrictedUserId)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
            }

            $selectTo = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'),
                array(
                    'uniqueId' => new Zend_Db_Expr("CONCAT(u.id, '_previous')"),
                    new Zend_Db_Expr("'previous' as type"),
                    'u.id',
                    'u.full_name',
                    'count' => new Zend_Db_Expr('COUNT(plo.id)')
                ))
                ->join(array('u' => 'user'), 'plo.creator_id=u.id', array())
                ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plo.lead_id', array())
                ->where($where)
                ->group('u.id');
            $select = $this->getDbTable()->select()
                ->union(array($selectFrom, $selectTo));
        } else {
            $select = $selectFrom;
        }

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);

    }

    /**
     * Get sequences with opened email status
     *
     * @param $createdFrom
     * @param $createdTo
     * @param $createdFromCompare
     * @param $createdToCompare
     * @param $restrictedUserId
     * @param $sequenceId
     * @return array
     * @throws Zend_Db_Select_Exception
     */
    public function getSequenceOpenSendEmailsByOwnerCountAggregated(
        $createdFrom,
        $createdTo,
        $createdFromCompare = '',
        $createdToCompare = '',
        $restrictedUserId = 0,
        $emailType = 'open',
        $sequenceIds = array()
    ) {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plal.created_at >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plal.created_at <= ?', $createdTo);

        if(!empty($restrictedUserId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
        }

        if(!empty($sequenceIds)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pes.id IN (?)', $sequenceIds);
        }

        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plal.lead_type = ?', Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_EMAIL);

        if($emailType == 'open') {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plel.email_status = ?', Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_OPENED);
        } elseif ($emailType == 'send') {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plel.email_status = ?', Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_SENT);
        }

        $where .= ' AND (' . $this->getDbTable()->getAdapter()->quoteInto('plel.external_source_name <> ?', 'sambaEmailMarketing'). ' OR plel.external_source_name IS NULL)';

        //outbound_email
        $selectFrom = $this->getDbTable()->getAdapter()->select()->from(array('plel' => 'plugin_leads_email_log'), array(
            'uniqueId' => new Zend_Db_Expr("CONCAT(pes.id, '_current')"),
            new Zend_Db_Expr("'current' as type"),
            'pes.id',
            'pes.sequence_name',
            'count' => new Zend_Db_Expr('COUNT(plal.id)')
        ))
            ->join(array('pes' => 'plugin_emailsequence_settings'), 'plel.follow_up_sequences_id=pes.id', array())
            ->join(array('plal' => 'plugin_leads_action_log'), 'plel.action_log_id=plal.id', array())
            ->join(array('pld' => 'plugin_leads_data'), 'plel.lead_id=pld.id', array())
            ->where($where)
            ->group('pes.id');

        if (!empty($createdFromCompare) && !empty($createdToCompare)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('plal.created_at >= ?', $createdFromCompare);
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plal.created_at <= ?', $createdToCompare);

            if(!empty($restrictedUserId)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
            }

            if(!empty($sequenceIds)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pes.id IN (?)', $sequenceIds);
            }

            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plal.lead_type = ?', Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_EMAIL);

            if($emailType == 'open') {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plel.email_status = ?', Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_OPENED);
            } elseif($emailType == 'send') {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plel.email_status = ?', Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_SENT);
            }

            $where .= ' AND (' . $this->getDbTable()->getAdapter()->quoteInto('plel.external_source_name <> ?', 'sambaEmailMarketing'). ' OR plel.external_source_name IS NULL)';

            $selectTo = $this->getDbTable()->getAdapter()->select()->from(array('plel' => 'plugin_leads_email_log'), array(
                'uniqueId' => new Zend_Db_Expr("CONCAT(pes.id, '_previous')"),
                new Zend_Db_Expr("'previous' as type"),
                'pes.id',
                'pes.sequence_name',
                'count' => new Zend_Db_Expr('COUNT(plal.id)')
            ))
                ->join(array('pes' => 'plugin_emailsequence_settings'), 'plel.follow_up_sequences_id=pes.id', array())
                ->join(array('plal' => 'plugin_leads_action_log'), 'plel.action_log_id=plal.id', array())
                ->join(array('pld' => 'plugin_leads_data'), 'plel.lead_id=pld.id', array())
                ->where($where)
                ->group('pes.id');

            $select = $this->getDbTable()->select()->union(array($selectFrom, $selectTo));
        } else {
            $select = $selectFrom;
        }

        $select->order('count DESC');

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Get sequences with sent sms status
     *
     * @param $createdFrom
     * @param $createdTo
     * @param $createdFromCompare
     * @param $createdToCompare
     * @param $restrictedUserId
     * @param $sequenceIds
     * @return array
     * @throws Zend_Db_Select_Exception
     */
    public function getSequenceSendSmsByOwnerCountAggregated(
        $createdFrom,
        $createdTo,
        $createdFromCompare = '',
        $createdToCompare = '',
        $restrictedUserId = 0,
        $sequenceIds = array()
    ) {
        $where = $this->getDbTable()->getAdapter()->quoteInto('pcq.sentAt >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pcq.sentAt <= ?', $createdTo);

        if(!empty($restrictedUserId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
        }

        if(!empty($sequenceIds)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pes.id IN (?)', $sequenceIds);
        }

        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pcq.type = ?', Emailsequence::SEQUENCE_TYPE_SMS);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pcq.status = ?', Emailsequence::QUEUE_STATUS_SENT);

        //outbound_sms
        $selectFrom = $this->getDbTable()->getAdapter()->select()->from(array('pcq' => 'plugin_cartstatusemail_queue'), array(
            'uniqueId' => new Zend_Db_Expr("CONCAT(pes.id, '_current')"),
            new Zend_Db_Expr("'current' as type"),
            'pes.id',
            'pes.sequence_name',
            'count' => new Zend_Db_Expr('COUNT(pcq.id)')
        ))
            ->join(array('pes' => 'plugin_emailsequence_settings'), 'pcq.sequence_id=pes.id', array())
            ->join(array('pld' => 'plugin_leads_data'), 'pcq.externalId=pld.id', array())
            ->where($where)
            ->group('pes.id');

        if (!empty($createdFromCompare) && !empty($createdToCompare)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('pcq.sentAt >= ?', $createdFromCompare);
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pcq.sentAt <= ?', $createdToCompare);

            if(!empty($restrictedUserId)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
            }

            if(!empty($sequenceIds)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pes.id IN (?)', $sequenceIds);
            }

            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pcq.type = ?', Emailsequence::SEQUENCE_TYPE_SMS);
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pcq.status = ?', Emailsequence::QUEUE_STATUS_SENT);


            $selectTo = $this->getDbTable()->getAdapter()->select()->from(array('pcq' => 'plugin_cartstatusemail_queue'), array(
                'uniqueId' => new Zend_Db_Expr("CONCAT(pes.id, '_previous')"),
                new Zend_Db_Expr("'previous' as type"),
                'pes.id',
                'pes.sequence_name',
                'count' => new Zend_Db_Expr('COUNT(pcq.id)')
            ))
                ->join(array('pes' => 'plugin_emailsequence_settings'), 'pcq.sequence_id=pes.id', array())
                ->join(array('pld' => 'plugin_leads_data'), 'pcq.externalId=pld.id', array())
                ->where($where)
                ->group('pes.id');

            $select = $this->getDbTable()->select()->union(array($selectFrom, $selectTo));
        } else {
            $select = $selectFrom;
        }

        $select->order('count DESC');

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    public function getActivitiesOutboundEmailsByOwnerCountAggregated (
        $createdFrom,
        $createdTo,
        $createdFromCompare = '',
        $createdToCompare = '',
        $description = '',
        $restrictedUserId = 0
    ) {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plal.created_at >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plal.created_at <= ?', $createdTo);

        if(!empty($restrictedUserId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
        }

        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plal.lead_type = ?', Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_EMAIL);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plel.email_status = ?', Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_SENT);
        $where .= ' AND (' . $this->getDbTable()->getAdapter()->quoteInto('plel.external_source_name <> ?',
                'sambaEmailMarketing'). ' OR plel.external_source_name IS NULL)';
        //outbound_email
        $selectFrom = $this->getDbTable()->getAdapter()->select()->from(array('pld' => 'plugin_leads_data'), array(
            'uniqueId' => new Zend_Db_Expr("CONCAT(u.id, '_current_outbound_emails')"),
            new Zend_Db_Expr("'current' as type"),
            'u.id',
            'u.full_name',
            'activity_id' => 'plt.id',
            'barId' =>  new Zend_Db_Expr("'10'"),
            'description' => new Zend_Db_Expr("'$description'"),
            'count' => new Zend_Db_Expr('COUNT(pld.id)')
        ))
            ->join(array('u' => 'user'), 'pld.owner_id=u.id', array())
            ->join(array('plal' => 'plugin_leads_action_log'), 'pld.id=plal.lead_id', array())
            ->join(array('plt' => 'plugin_leads_type'), 'plt.type_name=plal.lead_type', array())
            ->join(array('plel' => 'plugin_leads_email_log'), 'plal.id=plel.action_log_id', array())
            ->where($where)
            ->group('u.id');


        if (!empty($createdFromCompare) && !empty($createdToCompare)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('plal.created_at >= ?', $createdFromCompare);
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plal.created_at <= ?', $createdToCompare);

            if(!empty($restrictedUserId)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
            }

            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plal.lead_type = ?', Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_EMAIL);
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plel.email_status = ?', Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_SENT);
            $where .= ' AND (' . $this->getDbTable()->getAdapter()->quoteInto('plel.external_source_name <> ?',
                    'sambaEmailMarketing'). ' OR plel.external_source_name IS NULL)';

            $selectTo = $this->getDbTable()->getAdapter()->select()->from(array('pld' => 'plugin_leads_data'), array(
                'uniqueId' => new Zend_Db_Expr("CONCAT(u.id, '_previous_outbound_emails')"),
                new Zend_Db_Expr("'previous' as type"),
                'u.id',
                'u.full_name',
                'activity_id' => 'plt.id',
                'barId' =>  new Zend_Db_Expr("'10'"),
                'description' => new Zend_Db_Expr("'$description'"),
                'count' => new Zend_Db_Expr('COUNT(pld.id)')
            ))
                ->join(array('u' => 'user'), 'pld.owner_id=u.id', array())
                ->join(array('plal' => 'plugin_leads_action_log'), 'pld.id=plal.lead_id', array())
                ->join(array('plt' => 'plugin_leads_type'), 'plt.type_name=plal.lead_type', array())
                ->join(array('plel' => 'plugin_leads_email_log'), 'plal.id=plel.action_log_id', array())
                ->where($where)
                ->group('u.id');

            $select = $this->getDbTable()->select()->union(array($selectFrom, $selectTo));
        } else {
            $select = $selectFrom;
        }

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    public function getActivitiesOutboundCallsByOwnerCountAggregated (
        $createdFrom,
        $createdTo,
        $createdFromCompare = '',
        $createdToCompare = '',
        $description = '',
        $restrictedUserId = 0
    ) {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plal.created_at >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plal.created_at <= ?', $createdTo);

        if(!empty($restrictedUserId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
        }

        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plt.type_name = ?', Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_CALL);//10
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plal.action_type = ?', Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND);
        //$where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plcl.status = ?', 'completed');
        //outbound_call

        $selectFrom = $this->getDbTable()->getAdapter()->select()->from(array('pld' => 'plugin_leads_data'), array(
            'uniqueId' => new Zend_Db_Expr("CONCAT(u.id, '_current_outbound_calls')"),
            new Zend_Db_Expr("'current' as type"),
            'u.id',
            'u.full_name',
            'activity_id' => 'plt.id',
            'barId' =>  new Zend_Db_Expr("'20'"),
            'description' => new Zend_Db_Expr("'$description'"),
            'count' => new Zend_Db_Expr('COUNT(pld.id)')
        ))
            ->join(array('u' => 'user'), 'pld.owner_id=u.id', array())
            ->join(array('plal' => 'plugin_leads_action_log'), 'pld.id=plal.lead_id', array())
            ->join(array('plt' => 'plugin_leads_type'), 'plt.type_name=plal.lead_type', array())
            ->join(array('plcl' => 'plugin_leads_call_log'), 'plal.id=plcl.action_log_id', array())
            ->where($where)
            ->group('u.id');


        if (!empty($createdFromCompare) && !empty($createdToCompare)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('plal.created_at >= ?', $createdFromCompare);
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plal.created_at <= ?', $createdToCompare);

            if(!empty($restrictedUserId)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
            }

            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plt.type_name = ?', Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_CALL);//10
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plal.action_type = ?', Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND);
            //$where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plcl.status = ?', 'completed');
            //outbound_call

            $selectTo = $this->getDbTable()->getAdapter()->select()->from(array('pld' => 'plugin_leads_data'), array(
                'uniqueId' => new Zend_Db_Expr("CONCAT(u.id, '_previous_outbound_calls')"),
                new Zend_Db_Expr("'previous' as type"),
                'u.id',
                'u.full_name',
                'activity_id' => 'plt.id',
                'barId' =>  new Zend_Db_Expr("'20'"),
                'description' => new Zend_Db_Expr("'$description'"),
                'count' => new Zend_Db_Expr('COUNT(pld.id)')
            ))
                ->join(array('u' => 'user'), 'pld.owner_id=u.id', array())
                ->join(array('plal' => 'plugin_leads_action_log'), 'pld.id=plal.lead_id', array())
                ->join(array('plt' => 'plugin_leads_type'), 'plt.type_name=plal.lead_type', array())
                ->join(array('plcl' => 'plugin_leads_call_log'), 'plal.id=plcl.action_log_id', array())
                ->where($where)
                ->group('u.id');

            $select = $this->getDbTable()->select()->union(array($selectFrom, $selectTo));
        } else {
            $select = $selectFrom;
        }

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    public function getActivitiesOutboundMeetingsByOwnerCountAggregated (
        $createdFrom,
        $createdTo,
        $createdFromCompare = '',
        $createdToCompare = '',
        $description = '',
        $restrictedUserId = 0
    ) {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plal.created_at >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plal.created_at <= ?', $createdTo);

        if(!empty($restrictedUserId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
        }

        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plt.type_name = ?', Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_MEETING);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plml.meeting_status = ?', 'created');//finished
        //outbound_meeting

        $selectFrom = $this->getDbTable()->getAdapter()->select()->from(array('pld' => 'plugin_leads_data'), array(
            'uniqueId' => new Zend_Db_Expr("CONCAT(u.id, '_current_outbound_meetings')"),
            new Zend_Db_Expr("'current' as type"),
            'u.id',
            'u.full_name',
            'activity_id' => 'plt.id',
            'barId' =>  new Zend_Db_Expr("'30'"),
            'description' => new Zend_Db_Expr("'$description'"),
            'count' => new Zend_Db_Expr('COUNT(pld.id)')
        ))
            ->join(array('u' => 'user'), 'pld.owner_id=u.id', array())
            ->join(array('plal' => 'plugin_leads_action_log'), 'pld.id=plal.lead_id', array())
            ->join(array('plt' => 'plugin_leads_type'), 'plt.type_name=plal.lead_type', array())
            ->join(array('plml' => 'plugin_leads_meeting_log'), 'plal.id=plml.action_log_id', array())
            ->where($where)
            ->group('u.id');


        if (!empty($createdFromCompare) && !empty($createdToCompare)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('plal.created_at >= ?', $createdFromCompare);
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plal.created_at <= ?', $createdToCompare);

            if(!empty($restrictedUserId)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
            }

            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plt.type_name = ?', Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_MEETING);
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plml.meeting_status = ?', 'created');//finished
            //outbound_meeting

            $selectTo = $this->getDbTable()->getAdapter()->select()->from(array('pld' => 'plugin_leads_data'), array(
                'uniqueId' => new Zend_Db_Expr("CONCAT(u.id, '_previous_outbound_meetings')"),
                new Zend_Db_Expr("'previous' as type"),
                'u.id',
                'u.full_name',
                'activity_id' => 'plt.id',
                'barId' =>  new Zend_Db_Expr("'30'"),
                'description' => new Zend_Db_Expr("'$description'"),
                'count' => new Zend_Db_Expr('COUNT(pld.id)')
            ))
                ->join(array('u' => 'user'), 'pld.owner_id=u.id', array())
                ->join(array('plal' => 'plugin_leads_action_log'), 'pld.id=plal.lead_id', array())
                ->join(array('plt' => 'plugin_leads_type'), 'plt.type_name=plal.lead_type', array())
                ->join(array('plml' => 'plugin_leads_meeting_log'), 'plal.id=plml.action_log_id', array())
                ->where($where)
                ->group('u.id');

            $select = $this->getDbTable()->select()->union(array($selectFrom, $selectTo));
        } else {
            $select = $selectFrom;
        }

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }


    /**
     *
     * get tasks count by owner count
     *
     * @param string $createdFrom mysql date format
     * @param string $createdTo mysql date format
     * @param string $createdFromCompare mysql date format
     * @param string $createdToCompare mysql date format
     * @param string $description mysql date format
     * @return array
     * @throws Zend_Db_Select_Exception
     */
    public function getTasksByOwnerCountAggregated (
        $createdFrom,
        $createdTo,
        $createdFromCompare = '',
        $createdToCompare = '',
        $description = '',
        $restrictedUserId = 0
    ) {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plt.expired_at >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plt.expired_at <= ?', $createdTo);

        if(!empty($restrictedUserId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
        }

        $selectFrom = $this->getDbTable()->getAdapter()->select()->from(array('plt' => 'plugin_leads_tasks'), array(
            'uniqueId' => new Zend_Db_Expr("CONCAT(u.id, '_current_assigned')"),
            new Zend_Db_Expr("'current' as type"),
            'u.id',
            'u.full_name',
            'barId' =>  new Zend_Db_Expr("'10'"),
            'description' => new Zend_Db_Expr("'$description'"),
            'count' => new Zend_Db_Expr('COUNT(plt.id)')
        ))
            ->join(array('u' => 'user'), 'plt.user_id=u.id', array())
            ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plt.lead_id', array())
            ->where($where)
            ->group('u.id');

        if (!empty($createdFromCompare) && !empty($createdToCompare)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('plt.expired_at >= ?', $createdFromCompare);
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plt.expired_at <= ?', $createdToCompare);

            if(!empty($restrictedUserId)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
            }

            $selectTo = $this->getDbTable()->getAdapter()->select()->from(array('plt' => 'plugin_leads_tasks'),
                array(
                    'uniqueId' => new Zend_Db_Expr("CONCAT(u.id, '_previous_assigned')"),
                    new Zend_Db_Expr("'previous' as type"),
                    'u.id',
                    'u.full_name',
                    'barId' =>  new Zend_Db_Expr("'10'"),
                    'description' => new Zend_Db_Expr("'$description'"),
                    'count' => new Zend_Db_Expr('COUNT(plt.id)')
                ))
                ->join(array('u' => 'user'), 'plt.user_id=u.id', array())
                ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plt.lead_id', array())
                ->where($where)
                ->group('u.id');
            $select = $this->getDbTable()->select()
                ->union(array($selectFrom, $selectTo));
        } else {
            $select = $selectFrom;
        }

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);

    }

    /**
     *
     * get completed tasks count by owner count
     *
     * @param string $createdFrom mysql date format
     * @param string $createdTo mysql date format
     * @param string $createdFromCompare mysql date format
     * @param string $createdToCompare mysql date format
     * @param string $description mysql date format
     * @return array
     * @throws Zend_Db_Select_Exception
     */
    public function getCompletedTasksByOwnerCountAggregated (
        $createdFrom,
        $createdTo,
        $createdFromCompare = '',
        $createdToCompare = '',
        $description = '',
        $restrictedUserId = 0
    ) {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plal.created_at >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plal.created_at <= ?', $createdTo);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plal.type = ?', 'task_completed');

        if(!empty($restrictedUserId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
        }

        $selectFrom = $this->getDbTable()->getAdapter()->select()->from(array('pltl' => 'plugin_leads_tasks_log'), array(
            'uniqueId' => new Zend_Db_Expr("CONCAT(u.id, '_current_completed')"),
            new Zend_Db_Expr("'current' as type"),
            'u.id',
            'u.full_name',
            'barId' =>  new Zend_Db_Expr("'20'"),
            'description' => new Zend_Db_Expr("'$description'"),
            'count' => new Zend_Db_Expr('COUNT(pltl.id)')
        ))
            ->join(array('u' => 'user'), 'pltl.user_id=u.id', array())
            ->join(array('plal' => 'plugin_leads_action_log'), 'plal.id=pltl.action_log_id', array())
            ->join(array('plt' => 'plugin_leads_tasks'), 'plt.id=pltl.task_id', array())
            ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plt.lead_id', array())
            ->where($where)
            ->group('u.id');

        if (!empty($createdFromCompare) && !empty($createdToCompare)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('plal.created_at >= ?', $createdFromCompare);
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plal.created_at <= ?', $createdToCompare);
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plal.type = ?', 'task_completed');

            if(!empty($restrictedUserId)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
            }

            $selectTo = $this->getDbTable()->getAdapter()->select()->from(array('pltl' => 'plugin_leads_tasks_log'), array(
                    'uniqueId' => new Zend_Db_Expr("CONCAT(u.id, '_previous_completed')"),
                    new Zend_Db_Expr("'previous' as type"),
                    'u.id',
                    'u.full_name',
                    'barId' =>  new Zend_Db_Expr("'20'"),
                    'description' => new Zend_Db_Expr("'$description'"),
                    'count' => new Zend_Db_Expr('COUNT(pltl.id)')
                ))
                ->join(array('u' => 'user'), 'pltl.user_id=u.id', array())
                ->join(array('plal' => 'plugin_leads_action_log'), 'plal.id=pltl.action_log_id', array())
                ->join(array('plt' => 'plugin_leads_tasks'), 'plt.id=pltl.task_id', array())
                ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plt.lead_id', array())
                ->where($where)
                ->group('u.id');
            $select = $this->getDbTable()->select()
                ->union(array($selectFrom, $selectTo));
        } else {
            $select = $selectFrom;
        }

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);

    }

   /**
     *
     * get tasks count by owner count
     *
     * @param string $createdTo mysql date format
     * @param string $description mysql date format
     * @param string $createdToCompare mysql date format
     * @return array
     * @throws Zend_Db_Select_Exception
     */
    public function getTasksByOwnerCountAggregatedFromBeginning (
        $createdTo,
        $description = '',
        $createdToCompare = '',
        $restrictedUserId = 0
    ) {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plt.expired_at <= ?', $createdTo);

        if(!empty($restrictedUserId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
        }

        $selectFrom = $this->getDbTable()->getAdapter()->select()->from(array('plt' => 'plugin_leads_tasks'), array(
            'uniqueId' => new Zend_Db_Expr("CONCAT(u.id, '_current_pending')"),
            new Zend_Db_Expr("'current' as type"),
            'u.id',
            'u.full_name',
            'barId' =>  new Zend_Db_Expr("'30'"),
            'description' => new Zend_Db_Expr("'$description'"),
            'count' => new Zend_Db_Expr('COUNT(plt.id)')
        ))
            ->join(array('u' => 'user'), 'plt.user_id=u.id', array())
            ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plt.lead_id', array())
            ->where($where)
            ->group('u.id');

        if (!empty($createdToCompare)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('plt.expired_at <= ?', $createdToCompare);

            if(!empty($restrictedUserId)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
            }

            $selectTo = $this->getDbTable()->getAdapter()->select()->from(array('plt' => 'plugin_leads_tasks'),
                array(
                    'uniqueId' => new Zend_Db_Expr("CONCAT(u.id, '_previous_pending')"),
                    new Zend_Db_Expr("'previous' as type"),
                    'u.id',
                    'u.full_name',
                    'barId' =>  new Zend_Db_Expr("'30'"),
                    'description' => new Zend_Db_Expr("'$description'"),
                    'count' => new Zend_Db_Expr('COUNT(plt.id)')
                ))
                ->join(array('u' => 'user'), 'plt.user_id=u.id', array())
                ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plt.lead_id', array())
                ->where($where)
                ->group('u.id');
            $select = $this->getDbTable()->select()
                ->union(array($selectFrom, $selectTo));
        } else {
            $select = $selectFrom;
        }

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);

    }


    /**
     *
     * get opportunities amount by owner
     *
     * @param string $createdFrom mysql date format
     * @param string $createdTo mysql date format
     * @param string $createdFromCompare mysql date format
     * @param string $createdToCompare mysql date format
     * @return array
     * @throws Zend_Db_Select_Exception
     */
    public function getOpportunitiesByOwnerAmountAggregated (
        $createdFrom,
        $createdTo,
        $createdFromCompare = false,
        $createdToCompare = false,
        $restrictedUserId = 0
    ) {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plo.created_at >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plo.created_at <= ?', $createdTo);

        if(!empty($restrictedUserId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
        }

        $selectFrom = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'), array(
            'uniqueId' => new Zend_Db_Expr("CONCAT(u.id, '_current')"),
            new Zend_Db_Expr("'current' as type"),
            'u.id',
            'u.full_name',
            'count' => new Zend_Db_Expr('SUM(plo.opportunity_value)')
        ))
            ->join(array('u' => 'user'), 'plo.creator_id=u.id', array())
            ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plo.lead_id', array())
            ->where($where)
            ->group('u.id');

        if ($createdFromCompare !== false && $createdToCompare !== false) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('plo.created_at >= ?', $createdFromCompare);
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plo.created_at <= ?', $createdToCompare);

            if(!empty($restrictedUserId)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
            }

            $selectTo = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'),
                array(
                    'uniqueId' => new Zend_Db_Expr("CONCAT(u.id, '_previous')"),
                    new Zend_Db_Expr("'previous' as type"),
                    'u.id',
                    'u.full_name',
                    'count' => new Zend_Db_Expr('SUM(plo.opportunity_value)')
                ))
                ->join(array('u' => 'user'), 'plo.creator_id=u.id', array())
                ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plo.lead_id', array())
                ->where($where)
                ->group('u.id');
            $select = $this->getDbTable()->select()
                ->union(array($selectFrom, $selectTo));
        } else {
            $select = $selectFrom;
        }

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);

    }

    /**
     * get opportunities stage by owner count
     *
     * @param string $createdFrom mysql date format
     * @param string $createdTo mysql date format
     * @param mixed $createdFromCompare mysql date format
     * @param mixed $createdToCompare mysql date format
     * @param array $excludeStages exclude stages ids
     * @param bool $changeStageDateComparison flag for change_stage_date instead of created_at
     * @return array
     * @throws Zend_Db_Select_Exception
     */
    public function getOpportunitiesStageByOwnerCountAggregated (
        $createdFrom,
        $createdTo,
        $createdFromCompare = false,
        $createdToCompare = false,
        $excludeStages = array(),
        $changeStageDateComparison = false,
        $restrictedUserId = 0
    ) {
        $createdAtComparisonField = 'plo.created_at';
        if ($changeStageDateComparison === true) {
            $createdAtComparisonField = 'plo.change_stage_date';
        }
        $where = $this->getDbTable()->getAdapter()->quoteInto($createdAtComparisonField.' >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto($createdAtComparisonField.' <= ?', $createdTo);
        if (!empty($excludeStages)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plos.id NOT IN (?)', $excludeStages);
        }

        if(!empty($restrictedUserId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
        }

        $selectFrom = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'), array(
            'uniqueId' => new Zend_Db_Expr("CONCAT(u.id, '_current_', plos.id)"),
            new Zend_Db_Expr("'current' as type"),
            'u.id',
            'u.full_name',
            'plos.description',
            'opportunityId' => 'plos.id',
            'count' => new Zend_Db_Expr('COUNT(plo.id)'),
            'amount' => new Zend_Db_Expr('SUM(plo.opportunity_value)')
        ))
            ->join(array('u' => 'user'), 'plo.creator_id=u.id', array())
            ->join(array('plos' => 'plugin_leads_opportunity_stage'), 'plos.id=plo.opportunity_stage_id', array())
            ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plo.lead_id', array())
            ->where($where)
            ->group(array('u.id', 'plos.id'));

        if ($createdFromCompare !== false && $createdToCompare !== false) {
            $where = $this->getDbTable()->getAdapter()->quoteInto($createdAtComparisonField.' > ?', $createdFromCompare);
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto($createdAtComparisonField.'< ?', $createdToCompare);
            if (!empty($excludeStages)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plos.id NOT IN (?)', $excludeStages);
            }

            if(!empty($restrictedUserId)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
            }

            $selectTo = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'), array(
                'uniqueId' => new Zend_Db_Expr("CONCAT(u.id, '_previous_', plos.id)"),
                new Zend_Db_Expr("'previous' as type"),
                'u.id',
                'u.full_name',
                'plos.description',
                'opportunityId' => 'plos.id',
                'count' => new Zend_Db_Expr('COUNT(plo.id)'),
                'amount' => new Zend_Db_Expr('SUM(plo.opportunity_value)')
            ))
                ->join(array('u' => 'user'), 'plo.creator_id=u.id', array())
                ->join(array('plos' => 'plugin_leads_opportunity_stage'), 'plos.id=plo.opportunity_stage_id', array())
                ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plo.lead_id', array())
                ->where($where)
                ->group(array('u.id', 'plos.id'));

            $select = $this->getDbTable()->select()
                ->union(array($selectFrom, $selectTo));
        } else {
            $select = $selectFrom;
        }

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);

    }

    /**
     *
     * get leads by group count
     *
     * @param string $createdFrom mysql date format
     * @param string $createdTo mysql date format
     * @param string $createdFromCompare mysql date format
     * @param string $createdToCompare mysql date format
     * @return array
     * @throws Zend_Db_Select_Exception
     */
    public function getLeadsByGroupCountAggregated(
        $createdFrom,
        $createdTo,
        $createdFromCompare = false,
        $createdToCompare = false,
        $restrictedUserId = 0
    ) {
        $where = $this->getDbTable()->getAdapter()->quoteInto('created_at >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('created_at <= ?', $createdTo);

        if(!empty($restrictedUserId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
        }

        $selectFrom = $this->getDbTable()->getAdapter()->select()->from(array('pld' => 'plugin_leads_data'), array(
            'uniqueId' => new Zend_Db_Expr("CONCAT(sg.id, '_current')"),
            new Zend_Db_Expr("'current' as type"),
            'sg.id',
            'full_name' => 'sg.groupName',
            'count' => new Zend_Db_Expr('COUNT(pld.id)')
        ))
            ->join(array('sg' => 'shopping_group'), 'pld.customer_group_id=sg.id', array())
            ->where($where)
            ->group('sg.id');

        if ($createdFromCompare !== false && $createdToCompare !== false) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('created_at >= ?', $createdFromCompare);
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('created_at <= ?', $createdToCompare);

            if(!empty($restrictedUserId)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
            }

            $selectTo = $this->getDbTable()->getAdapter()->select()->from(array('pld' => 'plugin_leads_data'),
                array(
                    'uniqueId' => new Zend_Db_Expr("CONCAT(sg.id, '_previous')"),
                    new Zend_Db_Expr("'previous' as type"),
                    'sg.id',
                    'full_name' => 'sg.groupName',
                    'count' => new Zend_Db_Expr('COUNT(pld.id)')
                ))
                ->join(array('sg' => 'shopping_group'), 'pld.customer_group_id=sg.id', array())
                ->where($where)
                ->group('sg.id');
            $select = $this->getDbTable()->select()
                ->union(array($selectFrom, $selectTo));
        } else {
            $select = $selectFrom;
        }

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);

    }

    /**
     *
     * get leads lifecycle by group count
     *
     * @param string $createdFrom mysql date format
     * @param string $createdTo mysql date format
     * @param string $createdFromCompare mysql date format
     * @param string $createdToCompare mysql date format
     * @return array
     * @throws Zend_Db_Select_Exception
     */
    public function getLeadsLifecycleByGroupCountAggregated(
        $createdFrom,
        $createdTo,
        $createdFromCompare = false,
        $createdToCompare = false,
        $restrictedUserId = 0
    ) {
        $where = $this->getDbTable()->getAdapter()->quoteInto('created_at >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('created_at <= ?', $createdTo);

        if(!empty($restrictedUserId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
        }

        $selectFrom = $this->getDbTable()->getAdapter()->select()->from(array('pld' => 'plugin_leads_data'), array(
            'uniqueId' => new Zend_Db_Expr("CONCAT(sg.id, '_current')"),
            new Zend_Db_Expr("'current' as type"),
            'sg.id',
            'full_name' => 'sg.groupName',
            'count' => new Zend_Db_Expr('COUNT(pld.id)'),
            'stage_id' => new Zend_Db_Expr('GROUP_CONCAT(DISTINCT(pls.id))')
        ))
            ->join(array('sg' => 'shopping_group'), 'pld.customer_group_id=sg.id', array())
            ->join(array('pls' => 'plugin_leads_stage'), 'pls.id=pld.stage_id', array())
            ->where($where)
            ->group('sg.id');

        if ($createdFromCompare !== false && $createdToCompare !== false) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('created_at >= ?', $createdFromCompare);
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('created_at <= ?', $createdToCompare);

            if(!empty($restrictedUserId)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
            }

            $selectTo = $this->getDbTable()->getAdapter()->select()->from(array('pld' => 'plugin_leads_data'),
                array(
                    'uniqueId' => new Zend_Db_Expr("CONCAT(sg.id, '_previous')"),
                    new Zend_Db_Expr("'previous' as type"),
                    'sg.id',
                    'full_name' => 'sg.groupName',
                    'count' => new Zend_Db_Expr('COUNT(pld.id)'),
                    'stage_id' => new Zend_Db_Expr('GROUP_CONCAT(DISTINCT(pls.id))')
                ))
                ->join(array('sg' => 'shopping_group'), 'pld.customer_group_id=sg.id', array())
                ->join(array('pls' => 'plugin_leads_stage'), 'pls.id=pld.stage_id', array())
                ->where($where)
                ->group('sg.id');
            $select = $this->getDbTable()->select()->union(array($selectFrom, $selectTo));
        } else {
            $select = $selectFrom;
        }

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     *
     * get leads by group opportunities count
     *
     * @param string $createdFrom mysql date format
     * @param string $createdTo mysql date format
     * @param string $createdFromCompare mysql date format
     * @param string $createdToCompare mysql date format
     * @return array
     * @throws Zend_Db_Select_Exception
     */
    public function getLeadsByGroupOpportunitiesCountAggregated(
        $createdFrom,
        $createdTo,
        $createdFromCompare = false,
        $createdToCompare = false,
        $restrictedUserId = 0
    ) {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plo.created_at >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plo.created_at <= ?', $createdTo);

        if(!empty($restrictedUserId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
        }

        $selectFrom = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'), array(
            'uniqueId' => new Zend_Db_Expr("CONCAT(sg.id, '_current')"),
            new Zend_Db_Expr("'current' as type"),
            'sg.id',
            'full_name' => 'sg.groupName',
            'count' => new Zend_Db_Expr('COUNT(plo.id)')
        ))
            ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plo.lead_id', array())
            ->join(array('sg' => 'shopping_group'), 'pld.customer_group_id=sg.id', array())
            ->where($where)
            ->group('sg.id');

        if ($createdFromCompare !== false && $createdToCompare !== false) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('plo.created_at >= ?', $createdFromCompare);
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plo.created_at <= ?', $createdToCompare);

            if(!empty($restrictedUserId)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
            }

            $selectTo = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'),
                array(
                    'uniqueId' => new Zend_Db_Expr("CONCAT(sg.id, '_previous')"),
                    new Zend_Db_Expr("'previous' as type"),
                    'sg.id',
                    'full_name' => 'sg.groupName',
                    'count' => new Zend_Db_Expr('COUNT(plo.id)')
                ))
                ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plo.lead_id', array())
                ->join(array('sg' => 'shopping_group'), 'pld.customer_group_id=sg.id', array())
                ->where($where)
                ->group('sg.id');
            $select = $this->getDbTable()->select()
                ->union(array($selectFrom, $selectTo));
        } else {
            $select = $selectFrom;
        }


        return $this->getDbTable()->getAdapter()->fetchAssoc($select);

    }


    /**
     *
     * get leads by group opportunities amount
     *
     * @param string $createdFrom mysql date format
     * @param string $createdTo mysql date format
     * @param string $createdFromCompare mysql date format
     * @param string $createdToCompare mysql date format
     * @return array
     * @throws Zend_Db_Select_Exception
     */
    public function getLeadsByGroupOpportunitiesAmountAggregated (
        $createdFrom,
        $createdTo,
        $createdFromCompare = false,
        $createdToCompare = false,
        $restrictedUserId = 0
    ) {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plo.created_at >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plo.created_at <= ?', $createdTo);

        if(!empty($restrictedUserId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
        }

        $selectFrom = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'), array(
            'uniqueId' => new Zend_Db_Expr("CONCAT(sg.id, '_current')"),
            new Zend_Db_Expr("'current' as type"),
            'sg.id',
            'full_name' => 'sg.groupName',
            'count' => new Zend_Db_Expr('SUM(plo.opportunity_value)')
        ))
            ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plo.lead_id', array())
            ->join(array('sg' => 'shopping_group'), 'pld.customer_group_id=sg.id', array())
            ->where($where)
            ->group('sg.id');

        if ($createdFromCompare !== false && $createdToCompare !== false) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('plo.created_at >= ?', $createdFromCompare);
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plo.created_at <= ?', $createdToCompare);

            if(!empty($restrictedUserId)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
            }

            $selectTo = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'),
                array(
                    'uniqueId' => new Zend_Db_Expr("CONCAT(sg.id, '_previous')"),
                    new Zend_Db_Expr("'previous' as type"),
                    'sg.id',
                    'full_name' => 'sg.groupName',
                    'count' => new Zend_Db_Expr('SUM(plo.opportunity_value)')
                ))
                ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plo.lead_id', array())
                ->join(array('sg' => 'shopping_group'), 'pld.customer_group_id=sg.id', array())
                ->where($where)
                ->group('sg.id');
            $select = $this->getDbTable()->select()
                ->union(array($selectFrom, $selectTo));
        } else {
            $select = $selectFrom;
        }

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);

    }


    /**
     * get opportunities stage by group count
     *
     * @param string $createdFrom mysql date format
     * @param string $createdTo mysql date format
     * @param mixed $createdFromCompare mysql date format
     * @param mixed $createdToCompare mysql date format
     * @param array $excludeStages exclude stages ids
     * @param bool $changeStageDateComparison flag for change_stage_date instead of created_at
     * @return array
     * @throws Zend_Db_Select_Exception
     */
    public function getOpportunitiesStageByGroupCountAggregated (
        $createdFrom,
        $createdTo,
        $createdFromCompare = false,
        $createdToCompare = false,
        $excludeStages = array(),
        $changeStageDateComparison = false,
        $restrictedUserId = 0
    ) {

        $createdAtComparisonField = 'plo.created_at';
        if ($changeStageDateComparison === true) {
            $createdAtComparisonField = 'plo.change_stage_date';
        }
        $where = $this->getDbTable()->getAdapter()->quoteInto($createdAtComparisonField.' >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto($createdAtComparisonField.' <= ?', $createdTo);

        if (!empty($excludeStages)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plos.id NOT IN (?)', $excludeStages);
        }

        if(!empty($restrictedUserId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
        }

        $selectFrom = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'), array(
            'uniqueId' => new Zend_Db_Expr("CONCAT(sg.id, '_current_', plos.id)"),
            new Zend_Db_Expr("'current' as type"),
            'sg.id',
            'full_name' => 'sg.groupName',
            'plos.description',
            'opportunityId' => 'plos.id',
            'count' => new Zend_Db_Expr('COUNT(plo.id)'),
            'amount' => new Zend_Db_Expr('SUM(plo.opportunity_value)')
        ))
            ->join(array('plos' => 'plugin_leads_opportunity_stage'), 'plos.id=plo.opportunity_stage_id', array())
            ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plo.lead_id', array())
            ->join(array('sg' => 'shopping_group'), 'pld.customer_group_id=sg.id', array())
            ->where($where)
            ->group(array('sg.id', 'plos.id'));

        if ($createdFromCompare !== false && $createdToCompare !== false) {
            $where = $this->getDbTable()->getAdapter()->quoteInto($createdAtComparisonField.' > ?', $createdFromCompare);
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto($createdAtComparisonField.' < ?', $createdToCompare);

            if (!empty($excludeStages)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plos.id NOT IN (?)', $excludeStages);
            }

            if(!empty($restrictedUserId)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
            }

            $selectTo = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'), array(
                'uniqueId' => new Zend_Db_Expr("CONCAT(sg.id, '_previous_', plos.id)"),
                new Zend_Db_Expr("'previous' as type"),
                'sg.id',
                'full_name' => 'sg.groupName',
                'plos.description',
                'opportunityId' => 'plos.id',
                'count' => new Zend_Db_Expr('COUNT(plo.id)'),
                'amount' => new Zend_Db_Expr('SUM(plo.opportunity_value)')
            ))
                ->join(array('plos' => 'plugin_leads_opportunity_stage'), 'plos.id=plo.opportunity_stage_id', array())
                ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plo.lead_id', array())
                ->join(array('sg' => 'shopping_group'), 'pld.customer_group_id=sg.id', array())
                ->where($where)
                ->group(array('sg.id', 'plos.id'));

            $select = $this->getDbTable()->select()
                ->union(array($selectFrom, $selectTo));
        } else {
            $select = $selectFrom;
        }

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);

    }


    /**
     *
     * get leads by source count
     *
     * @param string $createdFrom mysql date format
     * @param string $createdTo mysql date format
     * @param string $createdFromCompare mysql date format
     * @param string $createdToCompare mysql date format
     * @return array
     * @throws Zend_Db_Select_Exception
     */
    public function getLeadsBySourceCountAggregated(
        $createdFrom,
        $createdTo,
        $createdFromCompare = false,
        $createdToCompare = false,
        $restrictedUserId = 0
    ) {
        $where = $this->getDbTable()->getAdapter()->quoteInto('created_at >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('created_at <= ?', $createdTo);

        if(!empty($restrictedUserId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
        }

        $selectFrom = $this->getDbTable()->getAdapter()->select()->from(array('pld' => 'plugin_leads_data'), array(
            'uniqueId' => new Zend_Db_Expr("CONCAT(plt.id, '_current')"),
            new Zend_Db_Expr("'current' as type"),
            'plt.id',
            'full_name' => 'plt.type_label',
            'count' => new Zend_Db_Expr('COUNT(pld.id)')
        ))
            ->join(array('plt' => 'plugin_leads_type'), 'plt.id=pld.lead_type_id', array())
            ->where($where)
            ->group('plt.id');

        if ($createdFromCompare !== false && $createdToCompare !== false) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('created_at >= ?', $createdFromCompare);
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('created_at <= ?', $createdToCompare);

            if(!empty($restrictedUserId)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
            }

            $selectTo = $this->getDbTable()->getAdapter()->select()->from(array('pld' => 'plugin_leads_data'),
                array(
                    'uniqueId' => new Zend_Db_Expr("CONCAT(plt.id, '_previous')"),
                    new Zend_Db_Expr("'previous' as type"),
                    'plt.id',
                    'full_name' => 'plt.type_label',
                    'count' => new Zend_Db_Expr('COUNT(pld.id)')
                ))
                ->join(array('plt' => 'plugin_leads_type'), 'plt.id=pld.lead_type_id', array())
                ->where($where)
                ->group('plt.id');
            $select = $this->getDbTable()->select()
                ->union(array($selectFrom, $selectTo));
        } else {
            $select = $selectFrom;
        }

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);

    }

    /**
     * Get leads lifecycle by source count
     *
     * @param $createdFrom
     * @param $createdTo
     * @param bool $createdFromCompare
     * @param bool $createdToCompare
     * @return array
     * @throws Zend_Db_Select_Exception
     */
    public function getLeadsByLifecycleCountAggregated(
        $createdFrom,
        $createdTo,
        $createdFromCompare = false,
        $createdToCompare = false,
        $restrictedUserId = 0
    ) {
        $where = $this->getDbTable()->getAdapter()->quoteInto('created_at >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('created_at <= ?', $createdTo);

        if(!empty($restrictedUserId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
        }

        $selectFrom = $this->getDbTable()->getAdapter()->select()->from(array('pld' => 'plugin_leads_data'), array(
            'uniqueId' => new Zend_Db_Expr("CONCAT(plt.id, '_current')"),
            new Zend_Db_Expr("'current' as type"),
            'plt.id',
            'full_name' => 'plt.type_label',
            'count' => new Zend_Db_Expr('COUNT(pld.id)'),
            'stage_id' => new Zend_Db_Expr('GROUP_CONCAT(DISTINCT(pls.id))')
        ))
            ->join(array('plt' => 'plugin_leads_type'), 'plt.id=pld.lead_type_id', array())
            ->join(array('pls' => 'plugin_leads_stage'), 'pls.id=pld.stage_id', array())
            ->where($where)
            ->group('plt.id');

        if ($createdFromCompare !== false && $createdToCompare !== false) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('created_at >= ?', $createdFromCompare);
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('created_at <= ?', $createdToCompare);

            if(!empty($restrictedUserId)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
            }

            $selectTo = $this->getDbTable()->getAdapter()->select()->from(array('pld' => 'plugin_leads_data'),
                array(
                    'uniqueId' => new Zend_Db_Expr("CONCAT(plt.id, '_previous')"),
                    new Zend_Db_Expr("'previous' as type"),
                    'plt.id',
                    'full_name' => 'plt.type_label',
                    'count' => new Zend_Db_Expr('COUNT(pld.id)'),
                    'stage_id' => new Zend_Db_Expr('GROUP_CONCAT(DISTINCT(pls.id))')
                ))
                ->join(array('plt' => 'plugin_leads_type'), 'plt.id=pld.lead_type_id', array())
                ->join(array('pls' => 'plugin_leads_stage'), 'pls.id=pld.stage_id', array())
                ->where($where)
                ->group('plt.id');
            $select = $this->getDbTable()->select()
                ->union(array($selectFrom, $selectTo));
        } else {
            $select = $selectFrom;
        }

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }


    /**
     *
     * get leads by source opportunities count
     *
     * @param string $createdFrom mysql date format
     * @param string $createdTo mysql date format
     * @param string $createdFromCompare mysql date format
     * @param string $createdToCompare mysql date format
     * @return array
     * @throws Zend_Db_Select_Exception
     */
    public function getLeadsBySourceOpportunitiesCountAggregated(
        $createdFrom,
        $createdTo,
        $createdFromCompare = false,
        $createdToCompare = false,
        $restrictedUserId = 0
    ) {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plo.created_at >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plo.created_at <= ?', $createdTo);

        if(!empty($restrictedUserId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
        }

        $selectFrom = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'), array(
            'uniqueId' => new Zend_Db_Expr("CONCAT(plt.id, '_current')"),
            new Zend_Db_Expr("'current' as type"),
            'plt.id',
            'full_name' => 'plt.type_label',
            'count' => new Zend_Db_Expr('COUNT(plo.id)')
        ))
            ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plo.lead_id', array())
            ->join(array('plt' => 'plugin_leads_type'), 'plt.id=pld.lead_type_id', array())
            ->where($where)
            ->group('plt.id');

        if ($createdFromCompare !== false && $createdToCompare !== false) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('plo.created_at >= ?', $createdFromCompare);
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plo.created_at <= ?', $createdToCompare);

            if(!empty($restrictedUserId)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
            }

            $selectTo = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'),
                array(
                    'uniqueId' => new Zend_Db_Expr("CONCAT(plt.id, '_previous')"),
                    new Zend_Db_Expr("'previous' as type"),
                    'plt.id',
                    'full_name' => 'plt.type_label',
                    'count' => new Zend_Db_Expr('COUNT(plo.id)')
                ))
                ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plo.lead_id', array())
                ->join(array('plt' => 'plugin_leads_type'), 'plt.id=pld.lead_type_id', array())
                ->where($where)
                ->group('plt.id');
            $select = $this->getDbTable()->select()
                ->union(array($selectFrom, $selectTo));
        } else {
            $select = $selectFrom;
        }


        return $this->getDbTable()->getAdapter()->fetchAssoc($select);

    }


    /**
     *
     * get leads by source opportunities amount
     *
     * @param string $createdFrom mysql date format
     * @param string $createdTo mysql date format
     * @param string $createdFromCompare mysql date format
     * @param string $createdToCompare mysql date format
     * @return array
     * @throws Zend_Db_Select_Exception
     */
    public function getLeadsBySourceOpportunitiesAmountAggregated (
        $createdFrom,
        $createdTo,
        $createdFromCompare = false,
        $createdToCompare = false,
        $restrictedUserId = 0
    ) {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plo.created_at >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plo.created_at <= ?', $createdTo);

        if(!empty($restrictedUserId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
        }

        $selectFrom = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'), array(
            'uniqueId' => new Zend_Db_Expr("CONCAT(plt.id, '_current')"),
            new Zend_Db_Expr("'current' as type"),
            'plt.id',
            'full_name' => 'plt.type_label',
            'count' => new Zend_Db_Expr('SUM(plo.opportunity_value)')
        ))
            ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plo.lead_id', array())
            ->join(array('plt' => 'plugin_leads_type'), 'pld.lead_type_id=plt.id', array())
            ->where($where)
            ->group('plt.id');

        if ($createdFromCompare !== false && $createdToCompare !== false) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('plo.created_at >= ?', $createdFromCompare);
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plo.created_at <= ?', $createdToCompare);

            if(!empty($restrictedUserId)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
            }

            $selectTo = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'),
                array(
                    'uniqueId' => new Zend_Db_Expr("CONCAT(plt.id, '_previous')"),
                    new Zend_Db_Expr("'previous' as type"),
                    'plt.id',
                    'full_name' => 'plt.type_label',
                    'count' => new Zend_Db_Expr('SUM(plo.opportunity_value)')
                ))
                ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plo.lead_id', array())
                ->join(array('plt' => 'plugin_leads_type'), 'pld.lead_type_id=plt.id', array())
                ->where($where)
                ->group('plt.id');
            $select = $this->getDbTable()->select()
                ->union(array($selectFrom, $selectTo));
        } else {
            $select = $selectFrom;
        }

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);

    }


    /**
     * get opportunities stage by source count
     *
     * @param string $createdFrom mysql date format
     * @param string $createdTo mysql date format
     * @param mixed $createdFromCompare mysql date format
     * @param mixed $createdToCompare mysql date format
     * @param array $excludeStages exclude stages ids
     * @param bool $changeStageDateComparison flag for change_stage_date instead of created_at
     * @return array
     * @throws Zend_Db_Select_Exception
     */
    public function getOpportunitiesStageBySourceCountAggregated (
        $createdFrom,
        $createdTo,
        $createdFromCompare = false,
        $createdToCompare = false,
        $excludeStages = array(),
        $changeStageDateComparison = false,
        $restrictedUserId = 0
    ) {

        $createdAtComparisonField = 'plo.created_at';
        if ($changeStageDateComparison === true) {
            $createdAtComparisonField = 'plo.change_stage_date';
        }
        $where = $this->getDbTable()->getAdapter()->quoteInto($createdAtComparisonField.' >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto($createdAtComparisonField.' <= ?', $createdTo);

        if (!empty($excludeStages)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plos.id NOT IN (?)', $excludeStages);
        }

        if(!empty($restrictedUserId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
        }

        $selectFrom = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'), array(
            'uniqueId' => new Zend_Db_Expr("CONCAT(plt.id, '_current_', plos.id)"),
            new Zend_Db_Expr("'current' as type"),
            'plt.id',
            'full_name' => 'plt.type_label',
            'plos.description',
            'opportunityId' => 'plos.id',
            'count' => new Zend_Db_Expr('COUNT(plo.id)'),
            'amount' => new Zend_Db_Expr('SUM(plo.opportunity_value)')
        ))
            ->join(array('plos' => 'plugin_leads_opportunity_stage'), 'plos.id=plo.opportunity_stage_id', array())
            ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plo.lead_id', array())
            ->join(array('plt' => 'plugin_leads_type'), 'pld.lead_type_id=plt.id', array())
            ->where($where)
            ->group(array('plt.id', 'plos.id'));

        if ($createdFromCompare !== false && $createdToCompare !== false) {
            $where = $this->getDbTable()->getAdapter()->quoteInto($createdAtComparisonField.' > ?', $createdFromCompare);
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto($createdAtComparisonField.' < ?', $createdToCompare);

            if (!empty($excludeStages)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plos.id NOT IN (?)', $excludeStages);
            }

            if(!empty($restrictedUserId)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
            }

            $selectTo = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'), array(
                'uniqueId' => new Zend_Db_Expr("CONCAT(plt.id, '_previous_', plos.id)"),
                new Zend_Db_Expr("'previous' as type"),
                'plt.id',
                'full_name' => 'plt.type_label',
                'plos.description',
                'opportunityId' => 'plos.id',
                'count' => new Zend_Db_Expr('COUNT(plo.id)'),
                'amount' => new Zend_Db_Expr('SUM(plo.opportunity_value)')
            ))
                ->join(array('plos' => 'plugin_leads_opportunity_stage'), 'plos.id=plo.opportunity_stage_id', array())
                ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plo.lead_id', array())
                ->join(array('plt' => 'plugin_leads_type'), 'pld.lead_type_id=plt.id', array())
                ->where($where)
                ->group(array('plt.id', 'plos.id'));

            $select = $this->getDbTable()->select()
                ->union(array($selectFrom, $selectTo));
        } else {
            $select = $selectFrom;
        }

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);

    }

    /**
     * get opportunities stage by tag count
     *
     * @param $createdFrom
     * @param $createdTo
     * @param bool $createdFromCompare
     * @param bool $createdToCompare
     * @param array $excludeStages
     * @param bool $changeStageDateComparison
     * @return array
     * @throws Zend_Db_Select_Exception
     */
    public function getOpportunitiesStageByTagCountAggregated (
        $createdFrom,
        $createdTo,
        $createdFromCompare = false,
        $createdToCompare = false,
        $excludeStages = array(),
        $changeStageDateComparison = false,
        $restrictedUserId = 0
    ) {

        $createdAtComparisonField = 'plo.created_at';
        if ($changeStageDateComparison === true) {
            $createdAtComparisonField = 'plo.change_stage_date';
        }
        $where = $this->getDbTable()->getAdapter()->quoteInto($createdAtComparisonField.' >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto($createdAtComparisonField.' <= ?', $createdTo);

        if (!empty($excludeStages)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plos.id NOT IN (?)', $excludeStages);
        }

        if(!empty($restrictedUserId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
        }

        $selectFrom = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'), array(
            'uniqueId' => new Zend_Db_Expr("CONCAT(plt.id, '_current_', plos.id)"),
            new Zend_Db_Expr("'current' as type"),
            'plt.id',
            'full_name' => 'plt.label',
            'plos.description',
            'opportunityId' => 'plos.id',
            'count' => new Zend_Db_Expr('COUNT(plo.id)'),
            'amount' => new Zend_Db_Expr('SUM(plo.opportunity_value)')
        ))
            ->join(array('plos' => 'plugin_leads_opportunity_stage'), 'plos.id=plo.opportunity_stage_id', array())
            ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plo.lead_id', array())
            ->join(array('plht' => 'plugin_leads_has_tags'), 'plht.lead_id=pld.id', array())
            ->join(array('plt' => 'plugin_leads_tags'), 'plt.id=plht.tag_id', array())

            ->where($where)
            ->group(array('plt.id', 'plos.id'));

        if ($createdFromCompare !== false && $createdToCompare !== false) {
            $where = $this->getDbTable()->getAdapter()->quoteInto($createdAtComparisonField.' > ?', $createdFromCompare);
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto($createdAtComparisonField.' < ?', $createdToCompare);

            if (!empty($excludeStages)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plos.id NOT IN (?)', $excludeStages);
            }

            if(!empty($restrictedUserId)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
            }

            $selectTo = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'), array(
                'uniqueId' => new Zend_Db_Expr("CONCAT(plt.id, '_previous_', plos.id)"),
                new Zend_Db_Expr("'previous' as type"),
                'plt.id',
                'full_name' => 'plt.label',
                'plos.description',
                'opportunityId' => 'plos.id',
                'count' => new Zend_Db_Expr('COUNT(plo.id)'),
                'amount' => new Zend_Db_Expr('SUM(plo.opportunity_value)')
            ))
                ->join(array('plos' => 'plugin_leads_opportunity_stage'), 'plos.id=plo.opportunity_stage_id', array())
                ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plo.lead_id', array())
                ->join(array('plht' => 'plugin_leads_has_tags'), 'plht.lead_id=pld.id', array())
                ->join(array('plt' => 'plugin_leads_tags'), 'plt.id=plht.tag_id', array())
                ->where($where)
                ->group(array('plt.id', 'plos.id'));

            $select = $this->getDbTable()->select()
                ->union(array($selectFrom, $selectTo));
        } else {
            $select = $selectFrom;
        }

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);

    }

    /**
     * Get opportunity stages
     *
     * @param string $createdFrom mysql date format
     * @param string $createdTo mysql date format
     * @param string $dateRangeType date range type 'expected'
     * @param array $excludeStages exclude stages ids
     * @return array
     */
    public function getOpportunitiesStage (
        $createdFrom,
        $createdTo,
        $dateRangeType = 'expected',
        $excludeStages = array(),
        $restrictedUserId = 0
    ) {

        $dateRangeTypeValue = 'plo.expected_close_date';
        if ($dateRangeType === 'changed') {
            $dateRangeTypeValue = 'plo.change_stage_date';
        }
        if ($dateRangeType === 'created') {
            $dateRangeTypeValue = 'plo.created_at';
        }

        $where = $this->getDbTable()->getAdapter()->quoteInto($dateRangeTypeValue.' >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto($dateRangeTypeValue.' <= ?', $createdTo);

        if (!empty($excludeStages)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plos.id NOT IN (?)', $excludeStages);
        }

        if(!empty($restrictedUserId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
        }

        $select = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'), array(
            'plos.id',
            'opportunityStageDescription' => 'plos.description',
            'opportunityName' => 'plo.opportunity_name',
            'opportunityStageId' => 'plos.id',
            'count' => new Zend_Db_Expr('COUNT(DISTINCT(plo.id))'),
            'opportunityValue' => new Zend_Db_Expr('SUM(plo.opportunity_value)'),
            'leadsCount' => new Zend_Db_Expr('COUNT(DISTINCT(pld.id))')
        ))
            ->join(array('plos' => 'plugin_leads_opportunity_stage'), 'plos.id=plo.opportunity_stage_id', array())
            ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plo.lead_id', array())
            ->where($where)
            ->group(array('plos.id'))
            ->order('plos.id');

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }


    /**
     * Get opportunity stages amount
     *
     * @param string $createdFrom mysql date format
     * @param string $createdTo mysql date format
     * @param array $excludeStages exclude stages ids
     * @param bool $changeStageDateComparison flag for change_stage_date instead of created_at
     * @param int $userId user id
     * @param int $oppTypeId opp type id
     * @return array
     */
    public function getOpportunitiesStageAmount (
        $createdFrom,
        $createdTo,
        $excludeStages = array(),
        $changeStageDateComparison = false,
        $userId = 0,
        $oppTypeId = 0
    ) {

        $createdAtComparisonField = 'plo.created_at';
        if ($changeStageDateComparison === true) {
            //$createdAtComparisonField = 'plo.change_stage_date';
            $createdAtComparisonField = new Zend_Db_Expr('COALESCE(plo.change_stage_date, plo.created_at)');
        }
        $where = $this->getDbTable()->getAdapter()->quoteInto($createdAtComparisonField.' >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto($createdAtComparisonField.' <= ?', $createdTo);

        if (!empty($excludeStages)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plos.id NOT IN (?)', $excludeStages);
        }

        if(!empty($userId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $userId);
        }

        if (!empty($oppTypeId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plot.id = ?', $oppTypeId);
        }

        $select = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'), array(
            'plos.id',
            'opportunityStageDescription' => 'plos.description',
            'opportunityName' => 'plo.opportunity_name',
            'opportunityStageId' => 'plos.id',
            'amount' => new Zend_Db_Expr('SUM(plo.opportunity_value)'),
            'count' => new Zend_Db_Expr('COUNT(plo.id)')
        ))
            ->join(array('plos' => 'plugin_leads_opportunity_stage'), 'plos.id=plo.opportunity_stage_id', array())
            ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plo.lead_id', array());

        if (!empty($oppTypeId)) {
            $select->joinLeft(array('plot' => 'plugin_leads_opportunity_type'), 'plo.opportunity_type_id=plot.id', array());
        }

        $select->where($where)
            ->group(array('plos.id'))
            ->order('plos.id');

        $select->assemble();

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Get opportunity stages
     *
     * @param string $createdFrom mysql date format
     * @param string $createdTo mysql date format
     * @param array $excludeStages exclude stages ids
     * @param string $createdFromCompare mysql date format
     * @param string $createdToCompare mysql date format
     * @param bool $changeStageDateComparison flag for change_stage_date instead of created_at
     * @return array
     */
    public function getOpportunitiesStageCompareAmount (
        $createdFrom,
        $createdTo,
        $createdFromCompare,
        $createdToCompare,
        $excludeStages = array(),
        $changeStageDateComparison = false,
        $restrictedUserId = 0
    ) {

        $createdAtComparisonField = 'plo.created_at';
        if ($changeStageDateComparison === true) {
            $createdAtComparisonField = 'plo.change_stage_date';
        }
        $where = $this->getDbTable()->getAdapter()->quoteInto($createdAtComparisonField.' >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto($createdAtComparisonField.' <= ?', $createdTo);

        if (!empty($excludeStages)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plos.id NOT IN (?)', $excludeStages);
        }

        if(!empty($restrictedUserId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
        }

        $selectFrom = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'), array(
            'uniqueId' => new Zend_Db_Expr("CONCAT(plos.id, '_current')"),
            'plos.id',
            new Zend_Db_Expr("'current' as type"),
            'opportunityStageDescription' => 'plos.description',
            'opportunityName' => 'plo.opportunity_name',
            'opportunityStageId' => 'plos.id',
            'count' => new Zend_Db_Expr('SUM(plo.opportunity_value)')
        ))
            ->join(array('plos' => 'plugin_leads_opportunity_stage'), 'plos.id=plo.opportunity_stage_id', array())
            ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plo.lead_id', array())
            ->where($where)
            ->group(array('plos.id'));

        $where = $this->getDbTable()->getAdapter()->quoteInto($createdAtComparisonField.' > ?', $createdFromCompare);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto($createdAtComparisonField.' < ?', $createdToCompare);

        if (!empty($excludeStages)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plos.id NOT IN (?)', $excludeStages);
        }

        if(!empty($restrictedUserId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
        }

        $selectTo = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'), array(
            'uniqueId' => new Zend_Db_Expr("CONCAT(plos.id, '_previous')"),
            'plos.id',
            new Zend_Db_Expr("'previous' as type"),
            'opportunityStageDescription' => 'plos.description',
            'opportunityName' => 'plo.opportunity_name',
            'opportunityStageId' => 'plos.id',
            'count' => new Zend_Db_Expr('SUM(plo.opportunity_value)')
        ))
            ->join(array('plos' => 'plugin_leads_opportunity_stage'), 'plos.id=plo.opportunity_stage_id', array())
            ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plo.lead_id', array())
            ->where($where)
            ->group(array('plos.id'));


        $select = $this->getDbTable()->select()
            ->union(array($selectFrom, $selectTo));

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Get opportunity stages
     *
     * @param string $createdFrom mysql date format
     * @param string $createdTo mysql date format
     * @param array $excludeStages exclude stages ids
     * @param string $createdFromCompare mysql date format
     * @param string $createdToCompare mysql date format
     * @return array
     */
    public function getOpportunitiesStageCompare (
        $createdFrom,
        $createdTo,
        $createdFromCompare,
        $createdToCompare,
        $excludeStages = array(),
        $restrictedUserId = 0
    ) {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plo.created_at >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plo.created_at <= ?', $createdTo);

        if (!empty($excludeStages)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plos.id NOT IN (?)', $excludeStages);
        }

        if(!empty($restrictedUserId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
        }

        $selectFrom = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'), array(
            'uniqueId' => new Zend_Db_Expr("CONCAT(plos.id, '_current')"),
            'plos.id',
            new Zend_Db_Expr("'current' as type"),
            'opportunityStageDescription' => 'plos.description',
            'opportunityName' => 'plo.opportunity_name',
            'opportunityStageId' => 'plos.id',
            'count' => new Zend_Db_Expr('COUNT(plo.id)')
        ))
            ->join(array('plos' => 'plugin_leads_opportunity_stage'), 'plos.id=plo.opportunity_stage_id', array())
            ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plo.lead_id', array())
            ->where($where)
            ->group(array('plos.id'));

        $where = $this->getDbTable()->getAdapter()->quoteInto('plo.created_at >= ?', $createdFromCompare);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plo.created_at <= ?', $createdToCompare);

        if (!empty($excludeStages)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plos.id NOT IN (?)', $excludeStages);
        }

        if(!empty($restrictedUserId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $restrictedUserId);
        }

        $selectTo = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'), array(
            'uniqueId' => new Zend_Db_Expr("CONCAT(plos.id, '_previous')"),
            'plos.id',
            new Zend_Db_Expr("'previous' as type"),
            'opportunityStageDescription' => 'plos.description',
            'opportunityName' => 'plo.opportunity_name',
            'opportunityStageId' => 'plos.id',
            'count' => new Zend_Db_Expr('COUNT(plo.id)')
        ))
            ->join(array('plos' => 'plugin_leads_opportunity_stage'), 'plos.id=plo.opportunity_stage_id', array())
            ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plo.lead_id', array())
            ->where($where)
            ->group(array('plos.id'));


        $select = $this->getDbTable()->select()
            ->union(array($selectFrom, $selectTo));

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Get placed calls
     *
     * @param string $createdFrom mysql date format
     * @param string $createdTo mysql date format
     * @param string $actionType action type
     * @param int $ownerId owner id
     * @return array
     */
    public function getCallsStats($createdFrom, $createdTo, $actionType, $ownerId = 0, $subtype = '')
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plal.created_at > ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plal.created_at < ?', $createdTo);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plal.action_type = ?', $actionType);

        if (!empty($subtype)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plcl.type = ?', $subtype);
        }

        if (!empty($ownerId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $ownerId);
        }

        $select = $this->getDbTable()->getAdapter()->select()->from(array('plcl' => 'plugin_leads_call_log'),
            array(
               'quantity' => new Zend_Db_Expr('COUNT(plcl.id)')
            )
        )->join(array('plal' => 'plugin_leads_action_log'), 'plal.id=plcl.action_log_id', array());

        if (!empty($ownerId)) {
            $select->join(array('pld' => 'plugin_leads_data'), 'pld.id=plal.lead_id', array());
        }

        $select->where($where);

        return $this->getDbTable()->getAdapter()->fetchRow($select);

    }

    /**
     * Get sent emails
     *
     * @param string $createdFrom mysql date format
     * @param string $createdTo mysql date format
     * @param string $emailStatus email status Ex: (sent, received)
     * @param bool $withoutNewsletter flag without newsletter emails
     * @param string $externalSourceName external source name
     * @param int $ownerId owner id
     * @return array
     */
    public function getEmailsStats($createdFrom, $createdTo, $emailStatus, $externalSourceName = '', $withoutNewsletter = true, $ownerId = 0)
    {

        $where = $this->getDbTable()->getAdapter()->quoteInto('plal.created_at >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plal.created_at <= ?', $createdTo);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plel.email_status = ?', $emailStatus);
        if ($withoutNewsletter === true) {
            $where .= ' AND (' . $this->getDbTable()->getAdapter()->quoteInto('plel.external_source_name <> ?',
                    'sambaEmailMarketing'). ' OR plel.external_source_name IS NULL)';
        }

        if (!empty($ownerId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $ownerId);
        }

        if (!empty($externalSourceName)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plel.external_source_name = ?',
                    $externalSourceName);
        }

        $select = $this->getDbTable()->getAdapter()->select()->from(
            array('plel' => 'plugin_leads_email_log'),
            array(
                'quantity' => new Zend_Db_Expr('COUNT(plel.id)')
            )
        );

        $select->join(array('plal' => 'plugin_leads_action_log'), 'plal.id=plel.action_log_id', array());


        if (!empty($ownerId)) {
            $select->join(array('pld' => 'plugin_leads_data'), 'pld.id=plal.lead_id', array());
        }

        $select->where($where);

        return $this->getDbTable()->getAdapter()->fetchRow($select);

    }

    /**
     * Get manual sent emails
     *
     * @param string $createdFrom mysql date format
     * @param string $createdTo mysql date format
     * @param string $emailStatus email status Ex: (sent, received)
     * @param bool $withoutNewsletter flag without newsletter emails
     * @param string $externalSourceName external source name
     * @param int $ownerId owner id
     * @return array
     */
    public function getManualSentEmailsStats($createdFrom, $createdTo, $emailStatus, $externalSourceName = '', $withoutNewsletter = true, $ownerId = 0)
    {

        $where = $this->getDbTable()->getAdapter()->quoteInto('plal.created_at >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plal.created_at <= ?', $createdTo);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plel.email_status = ?', $emailStatus);

        if ($withoutNewsletter === true) {
            $where .= ' AND (' . $this->getDbTable()->getAdapter()->quoteInto('plel.external_source_name <> ?',
                    'sambaEmailMarketing'). ' OR plel.external_source_name IS NULL)';
        }

        if (!empty($ownerId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $ownerId);
        }

        if (!empty($externalSourceName)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plel.external_source_name = ?',
                    $externalSourceName);
        }

        $select = $this->getDbTable()->getAdapter()->select()->from(
            array('plel' => 'plugin_leads_email_log'),
            array(
                'quantity' => new Zend_Db_Expr('COUNT(plel.id)')
            )
        );

        $select->join(array('plal' => 'plugin_leads_action_log'), 'plal.id=plel.action_log_id', array());

        $where .= ' AND '. new Zend_Db_Expr('plq.externalId IS NULL');
        $select->join(array('pld' => 'plugin_leads_data'), 'pld.id=plal.lead_id', array());
        $select->joinLeft(array('plq' => 'plugin_cartstatusemail_queue'), 'pld.id=plq.externalId', array());

        $select->where($where);

        return $this->getDbTable()->getAdapter()->fetchRow($select);

    }


    /**
     * Get sequence emails which was sent to the lead
     *
     * @param string $createdFrom mysql date format
     * @param string $createdTo mysql date format
     * @param int $ownerId owner id
     * @return mixed
     */
    public function getEmailsSequenceStats($createdFrom, $createdTo, $ownerId = 0)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plq.sentAt >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plq.sentAt <= ?', $createdTo);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plq.status = ?', '1');

        if (!empty($ownerId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $ownerId);
        }

        $select = $this->getDbTable()->getAdapter()->select()->from(
            array('plq' => 'plugin_cartstatusemail_queue'),
            array(
                'quantity' => new Zend_Db_Expr('COUNT(plq.id)')
            )
        )->join(array('pld' => 'plugin_leads_data'), 'pld.id=plq.externalId', array())
            ->where($where);

        return $this->getDbTable()->getAdapter()->fetchRow($select);
    }

    /**
     * Get sequence emails which was sent and opened to the lead
     *
     * @param string $createdFrom mysql date format
     * @param string $createdTo mysql date format
     * @param int $ownerId owner id
     * @return mixed
     */
    public function getEmailsSequenceStatsOpened($createdFrom, $createdTo, $ownerId = 0)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plal.created_at >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plal.created_at <= ?', $createdTo);

        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pcq.sentAt >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pcq.sentAt <= ?', $createdTo);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pcq.status = ?', '1');

        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plel.email_status = ?', Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_OPENED);
        $where .= ' AND (' . $this->getDbTable()->getAdapter()->quoteInto('plel.external_source_name <> ?', 'sambaEmailMarketing'). ' OR plel.external_source_name IS NULL)';

        if (!empty($ownerId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $ownerId);
        }

        $select = $this->getDbTable()->getAdapter()->select()->from(
            array('plel' => 'plugin_leads_email_log'),
            array('plel.id')
        )
            ->joinLeft(array('plal' => 'plugin_leads_action_log'), 'plel.action_log_id=plal.id', array())
            ->joinLeft(array('pcq' => 'plugin_cartstatusemail_queue'), 'plal.lead_id=pcq.externalId', array());

        if (!empty($ownerId)) {
            $select->join(array('pld' => 'plugin_leads_data'), 'pld.id=plal.lead_id', array());
        }

        $select->where($where)
            ->group('plel.id');

        $result = $this->getDbTable()->getAdapter()->fetchAll($select);

        return array('quantity' => count($result));
    }

    /**
     * Get meetings
     *
     * @param string $createdFrom mysql date format
     * @param string $createdTo mysql date format
     * @param array $statuses optional status limitation
     * @param int $ownerId owner id
     * @return array
     */
    public function getMeetings($createdFrom, $createdTo, $statuses = array(), $ownerId = 0)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plal.created_at >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plal.created_at <= ?', $createdTo);
        if (!empty($statuses)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plml.meeting_status IN (?)', $statuses);
        }

        if (!empty($ownerId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plml.creator_id = ?', $ownerId);
        }

        $select = $this->getDbTable()->getAdapter()->select()->from(
            array('plml' => 'plugin_leads_meeting_log'),
            array(
                'quantity' => new Zend_Db_Expr('COUNT(plml.id)')
            )
        )->join(array('plal' => 'plugin_leads_action_log'), 'plml.action_log_id=plal.id', array());


        if (!empty($ownerId)) {
            $select->join(array('pld' => 'plugin_leads_data'), 'pld.id=plal.lead_id', array());
        }

        $select->where($where);

        return $this->getDbTable()->getAdapter()->fetchRow($select);

    }

    /**
     * Get completed tasks
     *
     * @param string $createdFrom mysql date format
     * @param string $createdTo mysql date format
     * @param array $statuses optional status limitation
     * @param array $types type action log type
     * @param int $ownerId owner id
     * @return array
     */
    public function getTasks($createdFrom, $createdTo, $statuses = array(), $types = array(), $ownerId = 0)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plal.created_at >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plal.created_at <= ?', $createdTo);

        if (!empty($statuses)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plal.event_type IN (?)', $statuses);
        }

        if (!empty($types)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plal.type IN (?)', $types);
        }

        if (!empty($ownerId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('pltl.creator_id = ?', $ownerId);
        }

        $select = $this->getDbTable()->getAdapter()->select()->from(
            array('pltl' => 'plugin_leads_tasks_log'),
            array(
                'quantity' => new Zend_Db_Expr('COUNT(pltl.id)')
            )
        )->join(array('plal' => 'plugin_leads_action_log'), 'pltl.action_log_id=plal.id', array());

        if (!empty($ownerId)) {
            $select->join(array('pld' => 'plugin_leads_data'), 'pld.id=plal.lead_id', array());
        }

        $select->where($where);

        return $this->getDbTable()->getAdapter()->fetchRow($select);

    }

    /**
     * @param $period
     * @param int $includeTax
     * @param int $includeShipping
     * @return mixed
     */
    public function salesamountByTypeSalesCart($createdFrom, $createdTo)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('scs.created_at >= ?', $createdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('scs.created_at <= ?', $createdTo);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('scs.status IN (?)', array(Models_Model_CartSession::CART_STATUS_COMPLETED, Models_Model_CartSession::CART_STATUS_DELIVERED, Models_Model_CartSession::CART_STATUS_SHIPPED));
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('scs.gateway <> ?', 'Quote');

        $select = $this->getDbTable()->getAdapter()->select()->from(array('scs' => 'shopping_cart_session'),
            array('amount' => new Zend_Db_Expr('SUM(scs.total)'),
                'count' => new Zend_Db_Expr('COUNT(scs.id)')))
            ->joinLeft(array('scp' => 'shopping_recurring_payment'), 'scp.cart_id=scs.id', array())
            ->where($where);

        return $this->getDbTable()->getAdapter()->fetchRow($select);
    }

}
