<?php

/**
 * LeadsReviewRequestMapper.php
 *
 *
 * @method Leads_Mapper_LeadsReviewRequestMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsReviewRequestMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsReviewRequestModel';

    protected $_dbTable = 'Leads_DbTable_LeadsReviewRequestDbTable';

    /**
     * Save model to DB
     * @param $model Leads_Model_LeadsReviewRequestModel
     * @return Leads_Model_LeadsReviewRequestModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'review_token' => $model->getReviewToken(),
            'lead_id' => $model->getLeadId(),
            'status' => $model->getStatus(),
            'score' => $model->getScore(),
            'sent_at' => $model->getSentAt(),
            'reviewed_at' => $model->getReviewedAt(),
            'review_status' => $model->getReviewStatus(),
            'review_request_sent_at' => $model->getReviewRequestSentAt(),
            'request_reviewed_facebook_date' => $model->getRequestReviewedFacebookDate(),
            'request_reviewed_gmb_date' => $model->getRequestReviewedGmbDate()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Find by review token
     *
     * @param string $token review token
     * @return null
     */
    public function findByToken($token)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('review_token = ?', $token);
        return $this->_findWhere($where);
    }

    /**
     * Find by lead id
     *
     * @param int $leadId lead di
     * @return null
     */
    public function findByLeadId($leadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);
        return $this->_findWhere($where);
    }

    /**
     * Re-enable sending review request
     *
     * @throws Zend_Db_Adapter_Exception
     */
    public function unblockSendingReviewRequest()
    {
        $date = Tools_System_Tools::convertDateFromTimezone(date('Y-m-d', strtotime('-1 month')). ' 00:00:00');
        $where = $this->getDbTable()->getAdapter()->quoteInto('review_status = ?', 'sent');
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('review_request_sent_at <= ?', $date);
        $data = array('review_status' => '');
        $this->getDbTable()->getAdapter()->update('plugin_leads_review_request', $data, $where);
    }


    /**
     * Delete record
     *
     * @param int $id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_review_request', $where);

    }

}
