<?php

/**
 * LeadsScoreRulesGeneralConfigMapper.php
 *
 *
 * @method Leads_Mapper_LeadsScoreRulesGeneralConfigMapper  getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsScoreRulesGeneralConfigMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsScoreRulesGeneralConfigModel';

    protected $_dbTable = 'Leads_DbTable_LeadsScoreRulesGeneralConfigDbTable';

    /**
     * Save model to DB
     * @param $model Leads_Model_LeadsScoreRulesGeneralConfigModel
     * @return Leads_Model_LeadsScoreRulesGeneralConfigModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'rule_name' => $model->getRuleName(),
            'created_at' => $model->getCreatedAt(),
            'creator_id' => $model->getCreatorId(),
            'updated_at' => $model->getUpdatedAt(),
            'editor_id' => $model->getEditorId(),
            'score' => $model->getScore(),
            'score_sign' => $model->getScoreSign()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Get score rules data
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param bool $withTimezoneOffset select data with processed dates based on logged user timezone
     * @return array
     */
    public function fetchAll(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $withoutCount = false,
        $singleRecord = false,
        $withTimezoneOffset = false
    ) {

        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');

        $createdAt = 'plsrgc.created_at';

        if ($withTimezoneOffset === true) {
            $createdAt = new Zend_Db_Expr("CONVERT_TZ(`plsrgc`.`created_at`, '+00:00', '$mysqlOffset')");
        }

        $updatedAt = 'plsrgc.updated_at';

        if ($withTimezoneOffset === true) {
            $updatedAt = new Zend_Db_Expr("CONVERT_TZ(`plsrgc`.`updated_at`, '+00:00', '$mysqlOffset')");
        }

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plsrgc' => 'plugin_leads_score_rules_general_config'),
                array(
                    'plsrgc.id',
                    'plsrgc.rule_name',
                    'plsrgc.creator_id',
                    'createdAt' => $createdAt,
                    'plsrgc.editor_id',
                    'updatedAt' => $updatedAt,
                    'creatorName' => 'u.full_name',
                    'editorName' => 'uc.full_name',
                    'plsrgc.score',
                    'plsrgc.score_sign'
                )
            )
            ->joinLeft(array('u' => 'user'), 'u.id=plsrgc.creator_id', array())
            ->joinLeft(array('uc' => 'user'), 'uc.id=plsrgc.editor_id', array());


        $select->group('plsrgc.id');
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::LIMIT_COUNT);

            $count = array('count' => new Zend_Db_Expr('COUNT(DISTINCT(plsrgc.id))'));

            $select->from(array('plsrgc' => 'plugin_leads_score_rules_general_config'), $count)
                ->joinLeft(array('u' => 'user'), 'u.id=plsrgc.creator_id', array())
                ->joinLeft(array('uc' => 'user'), 'uc.id=plsrgc.editor_id', array());


            $select = $this->getDbTable()->getAdapter()->select()
                ->from(
                    array('subres' => $select),
                    array('count' => 'SUM(count)')
                );

            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }

    /**
     * Get all score config data
     *
     * @return array
     */
    public function getScoreRulesConfigData()
    {
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plsrgc' => 'plugin_leads_score_rules_general_config'),
                array(
                    'plsrgc.id',
                    'plsrgc.rule_name',
                    'plsrgc.score',
                    'plsrgc.score_sign',
                    'plsrc.rule_type',
                    'plsrc.rule_subtype',
                    'plsrc.field_name',
                    'plsrc.rule_comparison_operator',
                    'plsrc.field_value'
                )
            )
            ->joinLeft(array('plsrc' => 'plugin_leads_score_rules_config'), 'plsrgc.id=plsrc.rule_id', array());

         $select->order('plsrgc.id');

         return $this->getDbTable()->getAdapter()->fetchAll($select);
    }


    /**
     * Check if rule already exists
     *
     * @param string $ruleName unique rule name
     * @return null
     */
    public function checkRuleExist($ruleName)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('rule_name = ?', $ruleName);

        return $this->_findWhere($where);
    }


    /**
     * Set not assigned creator id
     *
     * @param int $creatorId creator id
     * @throws Zend_Db_Adapter_Exception
     */
    public function setNotAssignedCreator($creatorId)
    {
        $where = new Zend_Db_Expr('creator_id IS NULL');

        $data = array('creator_id' => $creatorId);
        $this->getDbTable()->getAdapter()->update('plugin_leads_score_rules_general_config', $data, $where);
    }

    /**
     * Set not assigned editor id
     *
     * @param int $editorId editor id
     * @throws Zend_Db_Adapter_Exception
     */
    public function setNotAssignedEditor($editorId)
    {
        $where = new Zend_Db_Expr('editor_id IS NULL');

        $data = array('editor_id' => $editorId);
        $this->getDbTable()->getAdapter()->update('plugin_leads_score_rules_general_config', $data, $where);
    }

    /**
     * Delete score rule (general config record)
     *
     * @param int $id config id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_score_rules_general_config', $where);

    }

}
