<?php

/**
 * LeadsScoreRulesLogMapper.php
 *
 *
 * @method Leads_Mapper_LeadsScoreRulesLogMapper  getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsScoreRulesLogMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsScoreRulesLogModel';

    protected $_dbTable = 'Leads_DbTable_LeadsScoreRulesLogDbTable';

    /**
     * Save model to DB
     * @param $model Leads_Model_LeadsScoreRulesLogModel
     * @return Leads_Model_LeadsScoreRulesLogModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'lead_id' => $model->getLeadId(),
            'rule_id' => $model->getRuleId()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Delete by rule id
     *
     * @param int $ruleId rule id
     * @return int
     */
    public function deleteByRuleId($ruleId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('rule_id = ?', $ruleId);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_score_rules_log', $where);
    }

    /**
     * Delete by lead id
     *
     * @param int $leadId rule id
     * @return int
     */
    public function deleteByLeadId($leadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_score_rules_log', $where);
    }


    /**
     * Delete score rule config record
     *
     * @param int $id config id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_score_rules_log', $where);

    }

}
