<?php

/**
 * LeadsSocialServicesConfigMapper.php
 *
 *
 * @method Leads_Mapper_LeadsSocialServicesConfigMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsSocialServicesConfigMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsSocialServiceConfigModel';

    protected $_dbTable = 'Leads_DbTable_LeadsSocialServicesConfigDbTable';

    /**
     * Save lead tag to DB
     * @param $model Leads_Model_LeadsSocialServiceConfigModel
     * @return Leads_Model_LeadsSocialServiceConfigModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'service_name' => $model->getServiceName(),
            'service_label' => $model->getServiceLabel(),
            'image_name' => $model->getImageName()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Get services data with info
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount without count flag
     * @param bool $singleRecord return single record
     * @return array
     */
    public function fetchAll($where = null, $order = null, $limit = null, $offset = null, $withoutCount = false, $singleRecord = false)
    {
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plcsc' => 'plugin_leads_social_services_config'),
                array(
                    'plcsc.id',
                    'plcsc.service_name',
                    'plcsc.service_label',
                    'plcsc.image_name'
                )
            );
        
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::GROUP);

            $select->from(array('plcsc' => 'plugin_leads_social_services_config'), array('count' => 'COUNT(plcsc.id)'));
            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        }

        return $data;
    }


    /**
     * Get all services pairs
     *
     * @return array
     */
    public function getServices()
    {
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_social_services_config', array('id', 'service_label', 'service_name', 'image_name'));

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Get all services pairs
     *
     * @return array
     */
    public function getServicesData()
    {
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_social_services_config', array('id', 'service_label', 'service_name', 'image_name'));

        return $this->getDbTable()->getAdapter()->fetchAll($select);
    }


    /**
     * Delete service record
     *
     * @param int $id service id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_social_services_config', $where);

    }

}
