<?php

/**
 * LeadsSourceMapper.php
 *
 *
 * @method Leads_Mapper_LeadsSourceMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsSourceMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsSourceModel';

    protected $_dbTable = 'Leads_DbTable_LeadsSourceDbTable';

    /**
     * Save lead source to DB
     * @param $model Leads_Model_LeadsSourceModel
     * @return Leads_Model_LeadsSourceModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'source_name' => $model->getSourceName(),
            'system' => $model->getSystem()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Get lead source data with info
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount without count flag
     * @return array
     */
    public function fetchAllData($where = null, $order = null, $limit = null, $offset = null, $withoutCount = false)
    {
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pls' => 'plugin_leads_source'),
                array(
                    'pls.id',
                    'pls.source_name',
                    'pls.system'
                )
            );
        
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        $data = $this->getDbTable()->getAdapter()->fetchAll($select);

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::GROUP);

            $select->from(array('pls' => 'plugin_leads_source'), array('count' => 'COUNT(pls.id)'));
            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        }

        return $data;
    }


    /**
     * Delete lead source record
     *
     * @param int $id lead source id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_source', $where);

    }

}
