<?php

/**
 * LeadsStageMapper.php
 *
 *
 * @method Leads_Mapper_LeadsStageMapper  getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsStageMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsStageModel';

    protected $_dbTable = 'Leads_DbTable_LeadsStageDbTable';

    /**
     * Save lead stage model to DB
     * @param $model Leads_Model_LeadsStageModel
     * @return Leads_Model_LeadsStageModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'stage_name' => $model->getStageName()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Get lead stage data with info
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @return array
     */
    public function fetchAllData($where = null, $order = null, $limit = null, $offset = null)
    {
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pls' => 'plugin_leads_stage'),
                array(
                    'pls.id',
                    'pls.stage_name'
                )
            );
        
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        $data = $this->getDbTable()->getAdapter()->fetchAll($select);

        if(!empty($data)) {
            $stageSelect = $this->getDbTable()->getAdapter()->select()
                ->from(array('pls' => 'plugin_leads_stage'), array('pls.id', 'pls.id'))
                ->join(array('pld' => 'plugin_leads_data'), 'pls.id=pld.stage_id', array());
            $leadsCount = $this->getDbTable()->getAdapter()->fetchPairs($stageSelect);

            foreach ($data as $key => $stage) {
                if(in_array($stage['id'], $leadsCount)) {
                    $data[$key]['usedInLeads'] = '1';
                }
            }
        }

        $select->reset(Zend_Db_Select::COLUMNS);
        $select->reset(Zend_Db_Select::FROM);
        $select->reset(Zend_Db_Select::LIMIT_OFFSET);
        $select->reset(Zend_Db_Select::GROUP);

        $select->from(array('pls' => 'plugin_leads_stage'), array('count' => 'COUNT(pls.id)'));
        $count = $this->getDbTable()->getAdapter()->fetchRow($select);

        return array(
            'totalRecords' => $count['count'],
            'data' => $data,
            'offset' => $offset,
            'limit' => $limit
        );
    }

    /**
     * Get all stage pairs
     *
     * @return array
     */
    public function getStages()
    {
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_stage', array('id', 'stage_name'));

        return $this->getDbTable()->getAdapter()->fetchPairs($select);
    }

    /**
     * Find by stage name
     *
     * @param string $stageName stage name
     * @return null
     */
    public function findByStageName($stageName)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('stage_name = ?', $stageName);
        return $this->_findWhere($where);
    }


    /**
     * Delete lead record
     *
     * @param int $id lead id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_stage', $where);

    }

    /**
     * Disable all default stages
     *
     * @return int
     * @throws Zend_Db_Adapter_Exception
     */
    public function disableDefaultStage()
    {
        return $this->getDbTable()->getAdapter()->update('plugin_leads_stage', array('default' => '0'));
    }

    /**
     * Assign default stage
     *
     * @param int $stageId id
     * @throws Zend_Db_Adapter_Exception
     */
    public function assignDefaultStage($stageId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $stageId);
        $data = array('default' => '1');
        $this->getDbTable()->getAdapter()->update('plugin_leads_stage', $data, $where);

    }

    /**
     * get default type
     *
     * @return null
     */
    public function getDefaultStage()
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('`default` = ?', '1');
        return $this->_findWhere($where);
    }

}
