<?php

/**
 * LeadsTableColumnsOrderMapper.php
 *
 *
 * @method Leads_Mapper_LeadsTableColumnsOrderMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsTableColumnsOrderMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsTableColumnsOrderModel';

    protected $_dbTable = 'Leads_DbTable_LeadsTableColumnsOrderDbTable';

    /**
     * Save call log model to DB
     * @param $model Leads_Model_LeadsTableColumnsOrderModel
     * @return Leads_Model_LeadsTableColumnsOrderModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'table_name' => $model->getTableName(),
            'preset_name' => $model->getPresetName(),
            'is_default' => $model->getIsDefault(),
            'fields_data' => $model->getFieldsData(),
            'user_id' => $model->getUserId(),
            'apply_to_all' => $model->getApplyToAll(),
            'created_at' => $model->getCreatedAt(),
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Find by table name
     *
     * @param string $tableName table name
     * @param int $userId user id
     * @return null
     */
    public function findByTableName($tableName, $userId = 0)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('table_name = ?', $tableName);
        if (!empty($userId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('user_id = ?', $userId);
        }

        return $this->_findWhere($where);
    }

    /**
     * Find user preset
     *
     * @param string $tableName table name
     * @param string $presetName preset name
     * @param int $userId user id
     * @return null
     */
    public function findUserPreset($tableName, $presetName, $userId = 0)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('table_name = ?', $tableName);
        $where .= ' AND '.$this->getDbTable()->getAdapter()->quoteInto('preset_name = ?', $presetName);
        if (!empty($userId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('user_id = ?', $userId);
        }

        return $this->_findWhere($where);
    }


    /**
     * @param string $tableName table name
     * @return null
     */
    public function findByDefaultTableName($tableName) {
        $where = $this->getDbTable()->getAdapter()->quoteInto('table_name = ?', $tableName);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('apply_to_all = ?', '1');

        return $this->_findWhere($where);
    }

    /**
     * Get all user presets for the particular table
     *
     * @param string $tableName table name
     * @param int $userId user id
     * @return array
     */
    public function findUserPresets($tableName, $userId = 0)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('table_name = ?', $tableName);
        if (!empty($userId)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('user_id = ?', $userId);
        }

        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_table_columns_order',
            array('id', 'table_name', 'preset_name', 'is_default', 'fields_data', 'user_id', 'apply_to_all'))->where($where);
        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Get all user presets for the particular table
     *
     * @param string $tableName table name
     * @param int $userId user id
     * @return array
     */
    public function findDefaultPreset($tableName)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('table_name = ?', $tableName);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('apply_to_all = ?', '1');

        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_table_columns_order',
            array('id', 'table_name', 'preset_name', 'is_default', 'fields_data', 'user_id', 'apply_to_all'))->where($where);
        return $this->getDbTable()->getAdapter()->fetchRow($select);
    }

    /**
     * Get all user presets for the particular table
     *
     * @param string $tableName table name
     * @param int $userId user id
     * @return array
     */
    public function findDefaultPresetModel($tableName)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('table_name = ?', $tableName);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('apply_to_all = ?', '1');

        return $this->_findWhere($where);
    }

    public function disableDefaultPreset($userId, $tableName)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('user_id = ?', $userId);
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('table_name = ?', $tableName);
        $data = array('is_default' => '0');
        $this->getDbTable()->getAdapter()->update('plugin_leads_table_columns_order', $data, $where);
    }

    /**
     * Delete record
     *
     * @param int $id id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_table_columns_order', $where);

    }

}
