<?php

/**
 * LeadsUsersApiAccessMapper.php
 *
 *
 * @method Leads_Mapper_LeadsUsersApiAccessMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsUsersApiAccessMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsUsersApiAccessModel';

    protected $_dbTable = 'Leads_DbTable_LeadsUsersApiAccessDbTable';

    /**
     * Save action log model
     * @param $model Leads_Model_LeadsUsersApiAccessModel
     * @return Leads_Model_LeadsUsersApiAccessModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'user_id' => $model->getUserId(),
            'ip_addresses' => $model->getIpAddresses()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }


    /**
     * Get all access list
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param string $having mysql having
     *
     * @return array
     */
    public function getConfigInfo(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $having = null,
        $withoutCount = false,
        $singleRecord = false
    ) {

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pluac' => 'plugin_leads_users_api_access'),
                array('pluac.*'));

        if (!empty($where)) {
            $select->where($where);
        }

        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($having)) {
            $select->having($having);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::LIMIT_COUNT);


            $count = array('count' => new Zend_Db_Expr('COUNT(DISTINCT(pluac.id))'));

            $select->from(array('pluac' => 'plugin_leads_users_api_access'), $count);

            $select = $this->getDbTable()->getAdapter()->select()
                ->from(
                    array('subres' => $select),
                    array('count' => 'SUM(count)')
                );

            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }

    }


    /**
     * Find by user id
     *
     * @param int $userId user id
     * @return null
     */
    public function findByUserId($userId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('user_id = ?', $userId);
        return $this->_findWhere($where);
    }

    /**
     * @param int $id system user id
     * @return int
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_users_api_access', $where);
    }


}
