<?php

/**
 * LeadsUsersApiTokensMapper.php
 *
 *
 * @method Leads_Mapper_LeadsUsersApiTokensMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsUsersApiTokensMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsUsersApiTokensModel';

    protected $_dbTable = 'Leads_DbTable_LeadsUsersApiTokensDbTable';

    /**
     * @param $model Leads_Model_LeadsUsersApiTokensModel
     * @return Leads_Model_LeadsUsersApiTokensModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'api_user_id' => $model->getApiUserId(),
            'token' => $model->getToken(),
            'expiration_date' => $model->getExpirationDate(),
            'creation_date' => $model->getCreationDate(),
            'ip_address' => $model->getIpAddress()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }


        return $model;

    }

    /**
     * Get by user id and token
     *
     * @param int $userId api user id
     * @param string $token JWT token
     * @return null
     */
    public function findByUserIdToken($userId, $token)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('api_user_id = ?', $userId);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('token = ?', $token);

        return $this->_findWhere($where);
    }


    /**
     * Get by user id and token
     *
     * @param string $token JWT token
     * @return Leads_Model_LeadsUsersApiTokensModel
     */
    public function findByToken($token)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('token = ?', $token);

        return $this->_findWhere($where);
    }

    /**
     * @param int $userId api user id relation
     * @return int
     */
    public function deleteByApiUserId($userId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('api_user_id = ?', $userId);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_users_api_tokens', $where);
    }


    /**
     * @param string $token JWS token
     * @return int
     */
    public function deleteByToken($token)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('token = ?', $token);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_users_api_tokens', $where);
    }


}
