<?php

/**
 * LeadsWorkflowActionsMapper.php
 *
 *
 * @method Leads_Mapper_LeadsWorkflowActionsMapper  getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsWorkflowActionsMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsWorkflowActionModel';

    protected $_dbTable = 'Leads_DbTable_LeadsWorkflowActionsDbTable';

    /**
     * Save lead note model to DB
     * @param $model Leads_Model_LeadsWorkflowActionModel
     * @return Leads_Model_LeadsWorkflowActionModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'rule_id' => $model->getRuleId(),
            'action_type' => $model->getActionType(),
            'action_config' => $model->getActionConfig()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Get action by rule ids
     *
     * @param array $ruleIds rule ids
     * @return array
     */
    public function getActionByRuleIds($ruleIds)
    {
        if(!empty($ruleIds)) {
            if(is_array($ruleIds)) {
                $where = $this->getDbTable()->getAdapter()->quoteInto('rule_id IN (?)', $ruleIds);
            } else {
                $where = $this->getDbTable()->getAdapter()->quoteInto('rule_id = ?', $ruleIds);
            }
        }

        $select = $this->getDbTable()->getAdapter()->select()->from(array('plwa' => 'plugin_leads_workflow_actions'),
                   array('plwa.action_type', 'plwa.id', 'plwa.action_config'))->where($where);

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Delete by rule id
     *
     * @param int $ruleId rule id
     * @return int
     */
    public function deleteByRuleId($ruleId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('rule_id = ?', $ruleId);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_workflow_actions', $where);
    }


    /**
     * Delete form actions config record
     *
     * @param int $id config id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_workflow_actions', $where);

    }

}
