<?php

/**
 * LeadsWorkflowConfigMapper.php
 *
 *
 * @method Leads_Mapper_LeadsWorkflowConfigMapper  getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsWorkflowConfigMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsWorkflowConfigModel';

    protected $_dbTable = 'Leads_DbTable_LeadsWorkflowConfigDbTable';

    /**
     * Save model to DB
     * @param $model Leads_Model_LeadsWorkflowConfigModel
     * @return Leads_Model_LeadsWorkflowConfigModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'rule_id' => $model->getRuleId(),
            'field_name' => $model->getFieldName(),
            'rule_comparison_operator' => $model->getRuleComparisonOperator(),
            'field_value' => $model->getFieldValue(),
            'rule_type' => $model->getRuleType(),
            'rule_subtype' => $model->getRuleSubtype()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Get all score rules
     * @param int $ruleId rule id
     *
     * @return array
     */
    public function getScoreRules($ruleId = 0)
    {
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_workflow_config');
        if (!empty($ruleId)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('rule_id = ?', $ruleId);
            $select->where($where);
        }
        $select->order('rule_id');
        return $this->getDbTable()->getAdapter()->fetchAll($select);
    }

    /**
     * Delete by rule id
     *
     * @param int $ruleId rule id
     * @return int
     */
    public function deleteByRuleId($ruleId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('rule_id = ?', $ruleId);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_workflow_config', $where);
    }

    /**
     * Delete score rule config record
     *
     * @param int $id config id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_workflow_config', $where);

    }

}
