<?php

/**
 * Class Leads_Model_LeadsWorkflowConfigModel
 */
class Leads_Model_LeadsWorkflowConfigModel extends Application_Model_Models_Abstract
{

    /**
     * comparison operators
     */
    const RULE_COMPARISON_OPERATOR_EQUAL = 'equal';

    const RULE_COMPARISON_OPERATOR_NOTEQUAL = 'notequal';

    const RULE_COMPARISON_OPERATOR_LIKE = 'like';

    const RULE_COMPARISON_OPERATOR_IN = 'in';

    const RULE_COMPARISON_GREATER_THAN = 'greaterthan';

    const RULE_COMPARISON_LESS_THAN = 'lessthan';

    const RULE_COMPARISON_NOT_IN = 'notin';

    /**
     * Rule types
     */
    const RULE_TYPE_LEAD_ATTRIBUTE = 'lead_attribute';

    const RULE_TYPE_ORGANIZATION_ATTRIBUTE = 'organization_attribute';

    const RULE_TYPE_ORGANIZATION_CUSTOM_ATTRIBUTES = 'organization_custom_attributes';

    const RULE_TYPE_ACTION_TYPE = 'action_type';

    const RULE_TYPE_LEAD_OPPORTUNITIES = 'lead_opportunities';

    const RULE_TYPE_LEAD_CUSTOM_ATTRIBUTES = 'lead_custom_attributes';

    /**
     * Subtypes
     */
    const RULE_SUBTYPE_CALL_RECEIVED = 'call_received';

    const RULE_SUBTYPE_CALL_PLACED = 'call_placed';

    const RULE_SUBTYPE_REGISTRATION = 'registration';

    const RULE_SUBTYPE_QUOTE_VIEW = 'quote_view';

    const RULE_SUBTYPE_QUOTE_REQUESTED = 'quote_requested';

    const RULE_SUBTYPE_QUOTE_SENT = 'quote_sent';

    const RULE_SUBTYPE_QUOTE_EXPIRATION_DATE = 'quote_expiration_date';

    const RULE_SUBTYPE_QUOTE_AMOUNT = 'quote_amount';

    const RULE_SUBTYPE_DOCUMENT_DOWNLOADED = 'document_downloaded';

    const RULE_SUBTYPE_EMAIL_SENT = 'email_sent';

    const RULE_SUBTYPE_EMAIL_RECEIVED = 'email_received';

    const RULE_SUBTYPE_WEBSITE_VISIT = 'website_visit';

    const RULE_SUBTYPE_PURCHASE = 'purchase';

    const RULE_SUBTYPE_SMS_RECEIVED = 'sms_received';

    const RULE_SUBTYPE_SMS_SENT = 'sms_sent';

    /**
     * Subtype attributes
     */
    const RULE_SUBTYPE_ATTRIBUTE_DURATION_PLACED = 'duration_placed';

    const RULE_SUBTYPE_ATTRIBUTE_DURATION_RECEIVED = 'duration_received';

    const RULE_SUBTYPE_ATTRIBUTE_DOWNLOADED_DOCUMENT_FILE_NAME = 'file_name';

    const RULE_SUBTYPE_ATTRIBUTE_PAGE_URL_VIEWED = 'page_url_viewed';

    const RULE_SUBTYPE_ATTRIBUTE_PAGE_URL_VIEWED_LATEST = 'page_url_viewed_latest';

    const RULE_SUBTYPE_ATTRIBUTE_VISIT_DURATION = 'visit_duration';

    const RULE_SUBTYPE_ATTRIBUTE_VISIT_DURATION_LATEST = 'visit_duration_latest';

    const RULE_SUBTYPE_ATTRIBUTE_TIME_ACTION_NUMBER = 'action_number';

    const RULE_SUBTYPE_ATTRIBUTE_TIME_ACTION_NUMBER_LATEST = 'action_number_latest';

    const RULE_SUBTYPE_ATTRIBUTE_PURCHASE_AMOUNT_SINGLE = 'purchase_amount_single';

    const RULE_SUBTYPE_ATTRIBUTE_PURCHASE_AMOUNT_CUMULATIVE = 'purchase_amount_cumulative';

    const RULE_SUBTYPE_ATTRIBUTE_LAST_PURCHASE_DATE = 'purchase_date';

    const RULE_SUBTYPE_ATTRIBUTE_QUOTE_VIEW = 'quote_view';

    const RULE_SUBTYPE_ATTRIBUTE_QUOTE_REQUESTED = 'quote_requested';

    const RULE_SUBTYPE_ATTRIBUTE_QUOTE_SENT = 'quote_sent';

    const RULE_SUBTYPE_ATTRIBUTE_QUOTE_EXPIRATION_DATE = 'quote_expiration_date';

    const RULE_SUBTYPE_ATTRIBUTE_QUOTE_AMOUNT_SINGLE = 'quote_amount_single';

    const RULE_SUBTYPE_ATTRIBUTE_QUOTE_AMOUNT_CUMULATIVE = 'quote_amount_cumulative';

    const RULE_SUBTYPE_ATTRIBUTE_REGISTERED = 'registered';

    /**
     * Lead attributes
     */

    const LEAD_ATTRIBUTE_OWNER = 'owner_id';

    const LEAD_ATTRIBUTE_COUNTRY_CODE = 'lead_country_code';

    const LEAD_ATTRIBUTE_STATE_CODE = 'lead_state_code';

    const LEAD_ATTRIBUTE_CUSTOMER_GROUP = 'lead_customer_group_id';

    const LEAD_ATTRIBUTE_LEAD_TAGS = 'lead_tags';

    const LEAD_ATTRIBUTE_POSITION = 'lead_position';

    const LEAD_ATTRIBUTE_LIFECYCLE = 'stage_id';

    const LEAD_ATTRIBUTE_CREATED_AT = 'lead_created_at';

    const LEAD_ATTRIBUTE_LAST_ACTIVITY_INBOUND = 'last_activity_inbound';

    const LEAD_ATTRIBUTE_LAST_ACTIVITY_OUTBOUND = 'last_activity_outbound';

    /**
     * Organization attributes
     */

    const ORGANIZATION_ATTRIBUTE_INDUSTRY = 'organization_industry_id';

    const ORGANIZATION_ATTRIBUTE_GROUP = 'organization_customer_group_id';

    const ORGANIZATION_ATTRIBUTE_EMPLOYEES = 'organization_employees';

    const ORGANIZATION_ATTRIBUTE_COUNTRY_CODE = 'organization_country_code';

    const ORGANIZATION_ATTRIBUTE_STATE = 'organization_state_code';

    /**
     * Lead opportunities attributes
     */
    const LEAD_OPPORTUNITY_ATTRIBUTE_OPPORTUNITY_CLOSE_DATE = 'opportunity_close_date';

    const LEAD_OPPORTUNITY_ATTRIBUTE_OPPORTUNITY_TYPE = 'opportunity_type';

    const LEAD_OPPORTUNITY_ATTRIBUTE_OPPORTUNITY_STAGE = 'opportunity_stage';

    const LEAD_OPPORTUNITY_ATTRIBUTE_OPPORTUNITY_AMOUNT_SINGLE = 'opportunity_amount';

    const LEAD_OPPORTUNITY_ATTRIBUTE_OPPORTUNITY_AMOUNT_CUMULATIVE = 'opportunity_amount_cumulative';

    /**
     * Allowed lead attributes
     *
     * @var array
     */
    public static $_allowedLeadAttributes = array(
        self::LEAD_ATTRIBUTE_OWNER,
        self::LEAD_ATTRIBUTE_COUNTRY_CODE,
        self::LEAD_ATTRIBUTE_STATE_CODE,
        self::LEAD_ATTRIBUTE_CUSTOMER_GROUP,
        self::LEAD_ATTRIBUTE_LEAD_TAGS,
        self::LEAD_ATTRIBUTE_POSITION,
        self::LEAD_ATTRIBUTE_LIFECYCLE,
        self::LEAD_ATTRIBUTE_CREATED_AT,
        self::LEAD_ATTRIBUTE_LAST_ACTIVITY_INBOUND,
        self::LEAD_ATTRIBUTE_LAST_ACTIVITY_OUTBOUND
    );

    /**
     * Allowed organization attributes
     *
     * @var array
     */
    public static $_allowedOrganizationAttributes = array(
        self::ORGANIZATION_ATTRIBUTE_INDUSTRY,
        self::ORGANIZATION_ATTRIBUTE_GROUP,
        self::ORGANIZATION_ATTRIBUTE_EMPLOYEES,
        self::ORGANIZATION_ATTRIBUTE_COUNTRY_CODE,
        self::ORGANIZATION_ATTRIBUTE_STATE
    );

    /**
     * comparison operators
     *
     * @var array
     */
    public static $_allowedComparisonOperators = array(
        self::RULE_COMPARISON_OPERATOR_EQUAL,
        self::RULE_COMPARISON_OPERATOR_NOTEQUAL,
        self::RULE_COMPARISON_OPERATOR_LIKE,
        self::RULE_COMPARISON_OPERATOR_IN,
        self::RULE_COMPARISON_GREATER_THAN,
        self::RULE_COMPARISON_LESS_THAN,
        self::RULE_COMPARISON_NOT_IN
    );

    /**
     * Rule types
     *
     * @var array
     */
    public static $_allowedRuleTypes = array(
        self::RULE_TYPE_LEAD_ATTRIBUTE,
        self::RULE_TYPE_ORGANIZATION_ATTRIBUTE,
        self::RULE_TYPE_ACTION_TYPE
    );

    protected $_ruleId = '';

    protected $_fieldName = '';

    protected $_ruleComparisonOperator = self::RULE_COMPARISON_OPERATOR_EQUAL;

    protected $_fieldValue = '';

    protected $_ruleType = self::RULE_TYPE_LEAD_ATTRIBUTE;

    protected $_ruleSubtype = '';

    /**
     * @return string
     */
    public function getRuleId()
    {
        return $this->_ruleId;
    }

    /**
     * @param string $ruleId
     * @return string
     */
    public function setRuleId($ruleId)
    {
        $this->_ruleId = $ruleId;

        return $this;
    }

    /**
     * @return string
     */
    public function getFieldName()
    {
        return $this->_fieldName;
    }

    /**
     * @param string $fieldName
     * @return string
     */
    public function setFieldName($fieldName)
    {
        $this->_fieldName = $fieldName;

        return $this;
    }

    /**
     * @return string
     */
    public function getRuleComparisonOperator()
    {
        return $this->_ruleComparisonOperator;
    }

    /**
     * @param string $ruleComparisonOperator
     * @return string
     */
    public function setRuleComparisonOperator($ruleComparisonOperator)
    {
        $this->_ruleComparisonOperator = $ruleComparisonOperator;

        return $this;
    }

    /**
     * @return string
     */
    public function getFieldValue()
    {
        return $this->_fieldValue;
    }

    /**
     * @param string $fieldValue
     * @return string
     */
    public function setFieldValue($fieldValue)
    {
        $this->_fieldValue = $fieldValue;

        return $this;
    }

    /**
     * @return string
     */
    public function getRuleType()
    {
        return $this->_ruleType;
    }

    /**
     * @param string $ruleType
     * @return string
     */
    public function setRuleType($ruleType)
    {
        $this->_ruleType = $ruleType;

        return $this;
    }

    /**
     * @return string
     */
    public function getRuleSubtype()
    {
        return $this->_ruleSubtype;
    }

    /**
     * @param string $ruleSubtype
     * @return string
     */
    public function setRuleSubtype($ruleSubtype)
    {
        $this->_ruleSubtype = $ruleSubtype;

        return $this;
    }


}