<?php

/**
 * Class Tools_LeadApiTools
 */
class Tools_LeadApiTools
{

    /**
     * List of fields which use during automatic
     * lead and organization creation
     *
     * @var array
     */
    public static $_reservedFields = array(
        'lead_first_name',
        'lead_email',
        'lead_phone',
        'lead_source',
        'lead_notes',
        'organization_id',
        'user_id',
        'stage_id',
        'customer_group_id',
        'customer_group_name',
        'created_at',
        'updated_at',
        'lead_last_name',
        'lead_zip',
        'lead_address',
        'lead_type_id',
        'owner_id',
        'last_activity',
        'lead_mobile',
        'lead_position',
        'lead_country_code',
        'lead_state_code',
        'middle_name',
        'prefix',
        'lead_phone_country_code',
        'lead_phone_extension',
        'lead_mobile_country_code',
        'lead_mobile_country_code_value',
        'lead_phone_country_code_value',
        'lead_address1',
        'lead_city',
        'lead_tags',
        'external_id',
        'creation_type',
        'last_activity_type_id',
        'use_organization_address',
        'last_activity_subtype',
        'organization_name',
        'organization_address',
        'organization_city',
        'organization_zip',
        'organization_country_code',
        'organization_website',
        'organization_website1',
        'organization_industry_id',
        'organization_phone',
        'organization_email',
        'organization_employees',
        'organization_domain',
        'organization_domain1',
        'organization_state_code',
        'organization_phone_country_code',
        'organization_phone_country_code_value',
        'organization_phone_extension',
        'organization_mobile',
        'organization_mobile_country_code',
        'organization_mobile_country_code_value',
        'organization_address1',
        'organization_notes',
        'organization_industry_type',
        'primary_contact_id',
        'first_contact_creation_date',
        'lead_created_by_name',
        'lead_created_by_email',
        'rawData',
        'eventName',
        'eventDate',
        'eventDescription',
        'lead_social_service_name',
        'lead_social_service_link'
    );

    /**
     * @param $leadParams
     * @return int
     */
    public static function processLeadsData($leadParams)
    {

        if (empty($leadParams['lead_first_name'])) {
            return array('error' => 1, 'message' => Tools_LeadImportTools::$_errorTypes['1']);
        }

        $leadParams['lead_first_name'] = trim(str_replace(array('\'', '`'), ' ',
            $leadParams['lead_first_name']));

        if (empty($leadParams['lead_first_name'])) {
            return array('error' => 1, 'message' => Tools_LeadImportTools::$_errorTypes['1']);
        }

        $isNewLead = Tools_LeadImportTools::isNewLead($leadParams);
        $leadParams = self::prepareData($leadParams);

        $leadTags = array();
        if (!empty($leadParams['lead_tags']) && is_array($leadParams['lead_tags'])) {
            $leadTags = $leadParams['lead_tags'];
            unset($leadParams['lead_tags']);
        }

        //if (!empty($isNewLead) && $isNewLead['status'] === 'new') {
            if (!empty($leadParams['utmTagList'])) {
                $leadUtmTags = Tools_LeadTools::prepareUtmTagsString($leadParams['utmTagList']);
                if (!empty($leadUtmTags)) {
                    $leadParams['lead_source'] = $leadParams['lead_source'] . ' ' . $leadUtmTags;
                }
            }
        //}


        $leadModel = self::processLead($leadParams, $isNewLead['status'], $isNewLead['leadModel']);
        if ($leadModel instanceof Leads_Model_LeadsModel) {
            $leadId = $leadModel->getId();
            if (!empty($leadTags)) {
                Tools_LeadImportTools::processLeadTags($leadId, $leadTags);
            }

            if (!empty($leadParams['rawData'])) {
                if(!empty($leadParams['notcleanusedfields'])) {
                    $fieldsNotInUse = $leadParams['rawData'];
                } else {
                    $fieldsNotInUse = self::removeUsedFields($leadParams['rawData']);
                }

                if (!empty($fieldsNotInUse)) {
                    $notesContent = Tools_LeadTools::prepareFormDetailsHtml($fieldsNotInUse);
                    Tools_LeadTools::processNotes($leadId, $notesContent);
                }
            }

            if (!empty($leadTags)) {
                $leadParams['lead_tags'] = $leadTags;
            }
            if (!empty($leadParams['eventName']) && !empty($leadParams['eventDate'])) {
                $eventDescription = (!empty($leadParams['eventDescription']) ? $leadParams['eventDescription'] : '');

                $type = Leads_Model_LeadsActionLogModel::ACTION_EXTERNAL;
                $leadType = Leads_Model_LeadsTypeModel::LEAD_INBOUND_EXTERNAL;

                if(!empty($leadParams['specialInboundType'])) {
                    if($leadParams['specialInboundType'] === Leads_Model_LeadsTypeModel::LEAD_INBOUND_CHAT) {
                        $type = Leads_Model_LeadsActionLogModel::ACTION_CHAT;
                        $leadType = Leads_Model_LeadsTypeModel::LEAD_INBOUND_CHAT;
                    }
                }

                $leadsActionLogModel = Tools_LeadTools::logLeadAction($type, $leadId,
                    Leads_Model_LeadsActionLogModel::ACTION_TYPE_INBOUND, $leadParams['eventDate'], $leadType, Leads_Model_LeadsActionLogModel::EVENT_LOG_EXPIRED, $eventDescription);
                Tools_LeadEventTools::analyzeEventRules($leadParams['eventName'], $leadParams, $leadModel, $isNewLead['status']);
            }

            if(!empty($leadParams['lead_social_service_name']) && !empty($leadParams['lead_social_service_link'])) {
                $socialServiceNames = explode(',', $leadParams['lead_social_service_name']);
                $socialServiceLinks = explode(',', $leadParams['lead_social_service_link']);

                if(!empty($socialServiceNames) && !empty($socialServiceLinks)) {
                    $leadsSocialServicesConfigMapper = Leads_Mapper_LeadsSocialServicesConfigMapper::getInstance();
                    $leadMapper = Leads_Mapper_LeadsMapper::getInstance();

                    foreach ($socialServiceNames as $key => $serviceName) {
                        if(!empty($socialServiceLinks[$key])) {
                            $serviceName = preg_replace('~[^\w_]~ui', '', strtolower($serviceName));

                            $where = $leadsSocialServicesConfigMapper->getDbTable()->getAdapter()->quoteInto('plcsc.service_name = ?', $serviceName);
                            $leadsSocialService = $leadsSocialServicesConfigMapper->fetchAll($where, null, null, null, true, true);

                            if(!empty($leadsSocialService)) {
                                $leadMapper->addSocialService($leadId, $leadsSocialService['id'], trim($socialServiceLinks[$key]));
                            }
                        }
                    }
                }
            }

            return array('error' => 0, 'message' => 'Lead has been processed');
        } else {
            return array('error' => 1, 'message' => Tools_LeadImportTools::$_errorTypes[$leadModel['error']]);
        }

    }

    /**
     *
     * Prepare lead data
     *
     * @param array $leadParams raw lead data
     * @return array
     */
    public static function prepareData($leadParams)
    {
        $customerGroups = self::getCustomerGroups();
        $leadsIndustryMapper = Leads_Mapper_LeadsIndustryMapper::getInstance();
        $industryTypes = $leadsIndustryMapper->getIndustriesPairs(true);
        if (!empty($leadParams['customer_group_name'])) {
            $groupName = filter_var($leadParams['customer_group_name'], FILTER_SANITIZE_STRING);
            if (array_key_exists($groupName, $customerGroups)) {
                $leadParams['customer_group_id'] = $customerGroups[$groupName]['id'];
            }
        }

        $leadParams['lead_email'] = trim(strtolower($leadParams['lead_email']));

        $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
        $userDefaultMobileCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');

        if (empty($leadParams['lead_mobile_country_code']) && !empty($userDefaultMobileCountryCode)) {
            $leadParams['lead_mobile_country_code'] = $userDefaultMobileCountryCode;
        }

        if (empty($leadParams['lead_phone_country_code']) && !empty($userDefaultMobileCountryCode)) {
            $leadParams['lead_phone_country_code'] = $userDefaultMobileCountryCode;
        }

        $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();
        if (!empty($shoppingConfig['country'])) {
            $defaultCountryCode = $shoppingConfig['country'];
            if (empty($leadParams['lead_country_code'])) {
                $leadParams['lead_country_code'] = $defaultCountryCode;
            }
            if (empty($leadParams['organization_country_code'])) {
                $leadParams['organization_country_code'] = $defaultCountryCode;
            }
        }

        if (!empty($leadParams['organization_mobile'])) {
            $organizationMobile = Tools_LeadTools::cleanPhoneAddress($leadParams['organization_mobile']);
        }

        if (!empty($leadParams['organization_phone'])) {
            $organizationPhone = Tools_LeadTools::cleanPhoneAddress($leadParams['organization_phone']);
        }

        if (!empty($leadParams['lead_mobile'])) {
            $mobile = Tools_LeadTools::cleanPhoneAddress($leadParams['lead_mobile']);
        }

        if (!empty($phone)) {
            $phone = Tools_LeadTools::cleanPhoneAddress($leadParams['lead_phone']);
        }

        if (!empty($leadParams['lead_mobile_country_code']) && !empty($mobile)) {
            $mobileNumberExists = Tools_LeadTools::checkLeadMobileOrPhoneNumber($mobile,  $leadParams['lead_mobile_country_code']);
            $mobileCountryCodeValue = Tools_LeadTools::getCountryCodeValue($leadParams['lead_mobile_country_code']);
            if (!empty($mobileNumberExists)) {
                $leadParams['lead_mobile'] = '';
            } else {
                $leadParams['lead_mobile_country_code_value'] = '';
                $leadParams['lead_mobile_country_code_value'] = $mobileCountryCodeValue;
            }
        }

        if (!empty($leadParams['organization_mobile_country_code']) && !empty($organizationMobile)) {
            $organizationMobileCountryCodeValue = Tools_LeadTools::getCountryCodeValue($leadParams['organization_mobile_country_code']);
            if (!empty($organizationMobileNumberExists)) {
                $leadParams['organization_mobile'] = '';
            } else {
                $leadParams['organization_mobile_country_code_value'] = '';
                $leadParams['organization_mobile_country_code_value'] = $organizationMobileCountryCodeValue;
            }
        }


        if (!empty($leadParams['lead_phone_country_code']) && !empty($phone)) {
            $phoneNumberExists = Tools_LeadTools::checkLeadMobileOrPhoneNumber($phone, $leadParams['lead_phone_country_code']);
            $phoneCountryCodeValue = Tools_LeadTools::getCountryCodeValue($leadParams['lead_phone_country_code']);
            if (!empty($phoneNumberExists)) {
                $leadParams['lead_phone'] = '';
                $leadParams['lead_phone_country_code_value'] = '';
            } else {
                $leadParams['lead_phone_country_code_value'] = $phoneCountryCodeValue;
            }
        }

        if (!empty($leadParams['organization_phone_country_code']) && !empty($organizationPhone)) {
            $organizationPhoneCountryCodeValue = Tools_LeadTools::getCountryCodeValue($leadParams['organization_phone_country_code']);
            if (!empty($organizationPhoneNumberExists)) {
                $leadParams['organization_phone'] = '';
                $leadParams['organization_phone_country_code_value'] = '';
            } else {
                $leadParams['organization_phone_country_code_value'] = $organizationPhoneCountryCodeValue;
            }
        }


        if (!empty($leadParams['organization_industry_type'])) {
            $allowIndustriesCreation = Leads_Mapper_LeadsConfigMapper::getInstance()->getConfigParam('allowIndustriesCreation');
            $industryType = trim(filter_var($leadParams['organization_industry_type'], FILTER_SANITIZE_STRING));
            if (array_key_exists($industryType, $industryTypes)) {
                $leadParams['organization_industry_id'] = $industryTypes[$industryType];
            } elseif ($allowIndustriesCreation) {
                $label = $industryType;
                if(!empty($leadParams['organization_industry_type_label'])) {
                    $label = $leadParams['organization_industry_type_label'];
                }

                $leadsIndustryModel = new Leads_Model_LeadsIndustryModel();
                $leadsIndustryModel->setType($industryType);
                $leadsIndustryModel->setLabel($label);

                $leadsIndustryModel = $leadsIndustryMapper->save($leadsIndustryModel);

                $leadParams['organization_industry_id'] = $leadsIndustryModel->getId();
            }
        }


        if (!empty($leadParams['lifecycle_name'])) {
            $lifecycleName = filter_var($leadParams['lifecycle_name'], FILTER_SANITIZE_STRING);
        }

        $leadStageMapper = Leads_Mapper_LeadsStageMapper::getInstance();
        if(empty($lifecycleName)) {
            $leadStageModel = $leadStageMapper->getDefaultStage();
            if ($leadStageModel instanceof Leads_Model_LeadsStageModel) {
                $leadStageId = $leadStageModel->getId();
                $leadParams['stage_id'] = $leadStageId;
            }
        } else {
            $leadStageModel = $leadStageMapper->findByStageName($lifecycleName);
            if (!$leadStageModel instanceof Leads_Model_LeadsStageModel) {
                $leadStageModel = new Leads_Model_LeadsStageModel();
                $leadStageModel->setStageName($lifecycleName);

                $leadStageModel = $leadStageMapper->save($leadStageModel);

                $leadParams['stage_id'] = $leadStageModel->getId();
            } else {
                $leadParams['stage_id'] = $leadStageModel->getId();
            }
        }

        return $leadParams;
    }

    /**
     * Create lead based on lead data
     *
     * @param array $data lead model
     * @param string $leadStatus lead status new old
     * @param Leads_Model_LeadsModel $leadModel lead status new old
     * @return int id
     */
    public static function processLead($data, $leadStatus = Tools_LeadFormEventsTools::LEAD_MODEL_STATUS_OLD, $leadModel)
    {
        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();

        $userMapper = Application_Model_Mappers_UserMapper::getInstance();
        if (!empty($data['lead_email'])) {
            $userModel = $userMapper->findByEmail($data['lead_email']);
            if ($userModel instanceof Application_Model_Models_User) {
                $data['user_id'] = $userModel->getId();
            }
        }

        if (!empty($data['lead_email'])) {
            $leadEmail = $data['lead_email'];
        }

        if (!empty($data['lead_first_name'])) {
            $firstName = $data['lead_first_name'];
        }

        if (!empty($data['lead_last_name'])) {
            $lastName = $data['lead_last_name'];
        }

        if (!empty($data['lead_zip'])) {
            $zip = $data['lead_zip'];
        }

        if (!empty($data['lead_address'])) {
            $address = $data['lead_address'];
        }

        if (!empty($data['lead_address1'])) {
            $address1 = $data['lead_address1'];
        }

        if (!empty($data['lead_position'])) {
            $position = $data['lead_position'];
        }

        if (!empty($data['lead_mobile_country_code'])) {
            $mobileCountryCode = $data['lead_mobile_country_code'];
        }

        if (!empty($data['lead_phone_country_code'])) {
            $phoneCountryCode = $data['lead_phone_country_code'];
        }

        if (!empty($data['lead_mobile_country_code_value'])) {
            $mobileCountryCodeValue = $data['lead_mobile_country_code_value'];
        }

        if (!empty($data['lead_phone_country_code_value'])) {
            $phoneCountryCodeValue = $data['lead_phone_country_code_value'];
        }

        if (!empty($data['lead_mobile'])) {
            $mobile = $data['lead_mobile'];
        }

        if (!empty($data['lead_phone'])) {
            $phone = $data['lead_phone'];
        }

        if (!empty($data['lead_notes'])) {
            $leadNotes = $data['lead_notes'];
        }

        if (!empty($data['customer_group_id'])) {
            $groupId = $data['customer_group_id'];
        }

        if (!empty($data['lead_state_code'])) {
            $stateCode = $data['lead_state_code'];
        }

        if (!empty($data['prefix'])) {
            $prefix = $data['prefix'];
        }

        if (!empty($data['lead_city'])) {
            $city = $data['lead_city'];
        }

        if (!empty($data['middle_name'])) {
            $middleName = $data['middle_name'];
        }

        if (!empty($data['lead_country_code'])) {
            $countryCode = $data['lead_country_code'];
        }

        if (!empty($data['organization_name'])) {
            $organizationName = $data['organization_name'];
        }

        if (!empty($data['organization_website'])) {
            $organizationWebsite = $data['organization_website'];
        }

        if (!empty($data['organization_website1'])) {
            $organizationWebsite1 = $data['organization_website1'];
        }

        if (!empty($data['organization_address'])) {
            $organizationAddress = $data['organization_address'];
        }

        if (!empty($data['organization_address1'])) {
            $organizationAddress1 = $data['organization_address1'];
        }

        if (!empty($data['organization_city'])) {
            $organizationCity = $data['organization_city'];
        }

        if (!empty($data['organization_zip'])) {
            $organizationZip = $data['organization_zip'];
        }

        if (!empty($data['organization_country_code'])) {
            $organizationCountryCode = $data['organization_country_code'];
        }

        if (!empty($data['organization_phone'])) {
            $organizationPhone = $data['organization_phone'];
        }

        if (!empty($data['organization_phone_country_code'])) {
            $organizationPhoneCountryCode = $data['organization_phone_country_code'];
        }

        if (!empty($data['organization_phone_country_code_value'])) {
            $organizationPhoneCountryCodeValue = $data['organization_phone_country_code_value'];
        }

        if (!empty($data['organization_mobile'])) {
            $organizationMobile = $data['organization_mobile'];
        }

        if (!empty($data['organization_mobile_country_code'])) {
            $organizationMobileCountryCode = $data['organization_mobile_country_code'];
        }

        if (!empty($data['organization_mobile_country_code_value'])) {
            $organizationMobileCountryCodeValue = $data['organization_mobile_country_code_value'];
        }

        if (!empty($data['organization_state_code'])) {
            $organizationStateCode = $data['organization_state_code'];
        }

        if (!empty($data['organization_notes'])) {
            $organizationNotes = $data['organization_notes'];
        }

        if (!empty($data['organization_email'])) {
            $organizationEmail = $data['organization_email'];
        }

        if (!empty($data['organization_employees'])) {
            $organizationEmployees = $data['organization_employees'];
        }

        if (!empty($data['organization_industry_id'])) {
            $organizationIndustryId = $data['organization_industry_id'];
        }

        if (!empty($data['lead_ip_address'])) {
            $ip = $data['lead_ip_address'];
        }

        if (!empty($data['lead_source'])) {
            $source = $data['lead_source'];
        }

        if (!empty($data['stage_id'])) {
            $stageId = $data['stage_id'];
        }

        $ownerId = Tools_LeadTools::getDefaultLeadOwner();
        if (!empty($data['lead_owner_id'])) {
            $leadOwnerModel = Application_Model_Mappers_UserMapper::getInstance()->find($data['lead_owner_id']);
            if ($leadOwnerModel instanceof Application_Model_Models_User) {
                $ownerId = $data['lead_owner_id'];
            }
        }


        $leadType = Leads_Model_LeadsTypeModel::LEAD_INBOUND_EXTERNAL;

        if(!empty($data['specialInboundType'])) {
            $leadType = $data['specialInboundType'];
        }

        $createdAt = Tools_System_Tools::convertDateFromTimezone('now');
        if (!$leadModel instanceof Leads_Model_LeadsModel) {
            $leadAdditionAvailable = Tools_LeadLicenseTools::checkLimit();
            if ($leadAdditionAvailable['error'] === true) {
                $leadModel = new Leads_Model_LeadsModel();
                $leadModel->registerObserver(new Tools_Mail_Watchdog(array(
                    'trigger' => Tools_LeadsMailWatchdog::LEADS_LICENSE_LIMIT
                )));

                $leadModel->notifyObservers();

                return array('error' => '3');
            }
            $leadTypeId = Tools_LeadTools::getLeadTypeIdByLeadTypeName($leadType);
            $leadModel = new Leads_Model_LeadsModel();
            $leadModel->setCreatedAt($createdAt);
            if (!empty($leadEmail)) {
                $leadModel->setLeadEmail($leadEmail);
            }

            if (!empty($firstName)) {
                $leadModel->setLeadFirstName($firstName);
            }
            if (!empty($mobileCountryCode)) {
                $leadModel->setLeadMobileCountryCode($mobileCountryCode);
            }
            if (!empty($mobile)) {
                $leadModel->setLeadMobile($mobile);
                $leadModel->setLeadMobileCountryCodeValue($mobileCountryCodeValue);
            }
            if (!empty($phoneCountryCode)) {
                $leadModel->setLeadPhoneCountryCode($phoneCountryCode);
            }
            if (!empty($phone)) {
                $leadModel->setLeadPhone($phone);
                $leadModel->setLeadPhoneCountryCodeValue($phoneCountryCodeValue);
            }
            if (!empty($groupId)) {
                $leadModel->setCustomerGroupId($groupId);
            }
            if (!empty($prefix)) {
                $leadModel->setPrefix($prefix);
            }
            if (!empty($city)) {
                $leadModel->setLeadCity($city);
            }
            if (!empty($middleName)) {
                $leadModel->setMiddleName($middleName);
            }
            if (!empty($countryCode)) {
                $leadModel->setLeadCountryCode($countryCode);
            }
            if (!empty($stateCode)) {
                $leadModel->setLeadStateCode($stateCode);
            }

            if (!empty($lastName)) {
                $leadModel->setLeadLastName($lastName);
            }

            if (!empty($zip)) {
                $leadModel->setLeadZip($zip);
            }

            if (!empty($leadNotes)) {
                $leadModel->setLeadNotes($leadNotes);
            }

            if (!empty($address)) {
                $leadModel->setLeadAddress($address);
            }

            if (!empty($address1)) {
                $leadModel->setLeadAddress1($address1);
            }
            if (!empty($position)) {
                $leadModel->setLeadPosition($position);
            }

            $leadModel->setOwnerId($ownerId);

            if (!empty($leadTypeId)) {
                $leadModel->setLeadTypeId($leadTypeId);
            }
            if (!empty($source)) {
                $leadModel->setLeadSource($source);
            }

            if (!empty($stageId)) {
                $leadModel->setStageId($stageId);
            }

        } else {
            $currentLeadEmail = $leadModel->getLeadEmail();
            $currentFirstName = $leadModel->getLeadFirstName();
            $currentLeadLastName = $leadModel->getLeadLastName();
            $currentMobile = $leadModel->getLeadMobile();
            $currentPhone = $leadModel->getLeadPhone();
            $currentZip = $leadModel->getLeadZip();
            $currentAddress = $leadModel->getLeadAddress();
            $currentPosition = $leadModel->getLeadPosition();
            $currentPrefix = $leadModel->getPrefix();
            $currentCity = $leadModel->getLeadCity();
            $currentCountry = $leadModel->getLeadCountryCode();
            $currentState = $leadModel->getLeadStateCode();
            $currentAddress1 = $leadModel->getLeadAddress1();
            $currentCustomerGroupId = $leadModel->getCustomerGroupId();
            $currentMiddleName = $leadModel->getMiddleName();
            $currentLeadNotes = $leadModel->getLeadNotes();
            $currentLeadStageId = $leadModel->getStageId();
            $currentLeadSource = $leadModel->getLeadSource();

            if (empty($currentLeadEmail) && !empty($leadEmail)) {
                $leadModel->setLeadFirstName($leadEmail);
            }

            if (empty($currentFirstName) && !empty($firstName)) {
                $leadModel->setLeadFirstName($firstName);
            }
            if (empty($currentLeadLastName) && !empty($lastName)) {
                $leadModel->setLeadLastName($lastName);
            }

            if (empty($currentCustomerGroupId) && !empty($groupId)) {
                $leadModel->setCustomerGroupId($groupId);
            }

            if (empty($currentMobile) && !empty($mobile)) {
                $leadModel->setLeadMobile($mobile);
                $leadModel->setLeadMobileCountryCodeValue($mobileCountryCodeValue);
                $leadModel->setLeadMobileCountryCode($mobileCountryCode);
            }
            if (empty($currentPhone) && !empty($phone)) {
                $leadModel->setLeadPhone($phone);
                $leadModel->setLeadPhoneCountryCode($phoneCountryCode);
                $leadModel->setLeadPhoneCountryCodeValue($phoneCountryCodeValue);
            }

            if (empty($currentPrefix) && !empty($prefix)) {
                $leadModel->setPrefix($prefix);
            }

            if (empty($currentCity) && !empty($city)) {
                $leadModel->setLeadCity($city);
            }

            if (empty($currentMiddleName) && !empty($middleName)) {
                $leadModel->setMiddleName($middleName);
            }

            if (empty($currentCountry) && !empty($countryCode)) {
                $leadModel->setLeadCountryCode($countryCode);
            }

            if (empty($currentState) && !empty($stateCode)) {
                $leadModel->setLeadStateCode($stateCode);
            }

            if (empty($currentLeadNotes) && !empty($leadNotes)) {
                $leadModel->setLeadNotes($leadNotes);
            }

            if (empty($currentZip) && !empty($zip)) {
                $leadModel->setLeadZip($zip);
            }

            if (empty($currentAddress) && !empty($address)) {
                $leadModel->setLeadAddress($address);
            }

            if (empty($currentAddress1) && !empty($address1)) {
                $leadModel->setLeadAddress1($address1);
            }

            if (empty($currentPosition) && !empty($position)) {
                $leadModel->setLeadPosition($position);
            }

            if (empty($currentLeadStageId) && !empty($stageId)) {
                $leadModel->setStageId($stageId);
            }

            if (empty($currentLeadSource) && !empty($source)) {
                $leadModel->setLeadSource($source);
            }

        }

        $oldUserId = $leadModel->getUserId();

        if (empty($oldUserId) && !empty($data['user_id'])) {
            $leadModel->setUserId($data['user_id']);
        }

        $leadTypeModel = Leads_Mapper_LeadsTypeMapper::getInstance()->findByTypeName($leadType);
        if ($leadTypeModel instanceof Leads_Model_LeadsTypeModel) {
            $leadModel->setLastActivityTypeId($leadTypeModel->getId());
            $leadModel->setLastActivitySubtype('');
        }

        $currentOrganizationId = $leadModel->getOrganizationId();

        $organizationModel = null;
        $assignPrimaryContact = false;
        if (!empty($organizationName) && empty($currentOrganizationId)) {
            if (!empty($organizationName)) {
                $assignPrimaryContact = Tools_LeadTools::getOrganizationPrimaryContactStatus($organizationName);
            }

            $organizationModel = Tools_LeadTools::processOrganization($organizationName, true);
            if ($organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                $leadModel->setOrganizationId($organizationModel->getId());
            }
        }

        $leadModel->setLastActivity($createdAt);

        if (!empty($leadModel->getUserId()) && !empty($leadModel->getCustomerGroupId())) {
            Tools_LeadTools::syncUserGroup($leadModel->getUserId(), $leadModel->getCustomerGroupId());
        }

        try {
            $leadModel = $leadsMapper->save($leadModel);
        } catch (Exception $e) {
            return array('error' => '99');
        }

        $leadId = $leadModel->getId();

        if (!empty($organizationName) && $organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
            $leadOrganizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
            if ($assignPrimaryContact === true) {
                $organizationModel->setPrimaryContactId($leadId);
            }
            $currentOrganizationWebsite = $organizationModel->getOrganizationWebsite();
            $currentOrganizationWebsite1 = $organizationModel->getOrganizationWebsite1();
            $currentOrganizationAddress = $organizationModel->getOrganizationAddress();
            $currentOrganizationAddress1 = $organizationModel->getOrganizationAddress1();
            $currentOrganizationCity = $organizationModel->getOrganizationCity();
            $currentOrganizationZip = $organizationModel->getOrganizationZip();
            $currentOrganizationCountryCode = $organizationModel->getOrganizationCountryCode();
            $currentOrganizationPhone = $organizationModel->getOrganizationPhone();
            $currentOrganizationEmail = $organizationModel->getOrganizationEmail();
            $currentOrganizationEmployees = $organizationModel->getOrganizationEmployees();
            $currentOrganizationStateCode = $organizationModel->getOrganizationStateCode();
            $currentOrganizationPhoneCountryCode = $organizationModel->getOrganizationPhoneCountryCode();
            $currentOrganizationPhoneCountryCodeValue = $organizationModel->getOrganizationPhoneCountryCodeValue();
            $currentOrganizationMobile = $organizationModel->getOrganizationMobile();
            $currentOrganizationMobileCountryCode = $organizationModel->getOrganizationMobileCountryCode();
            $currentOrganizationMobileCountryCodeValue = $organizationModel->getOrganizationMobileCountryCodeValue();
            $currentOrganizationIndustryId = $organizationModel->getOrganizationIndustryId();
            $currentOrganizationNotes = $organizationModel->getOrganizationNotes();

            if (empty($currentOrganizationWebsite)) {
                $website = Tools_LeadTools::prepareWebsiteUrl($organizationWebsite);
                if (empty($currentOrganizationWebsite) && !empty($website)) {
                    $domain = Tools_LeadTools::getDomainFromUrl($website);
                    if (!empty($domain)) {
                        $organizationDomain = $domain;
                    } else {
                        $organizationDomain = '';
                    }
                    if (!empty($domain)) {
                        $organizationModel->setOrganizationDomain($organizationDomain);
                    }
                    $organizationModel->setOrganizationWebsite($website);
                }
            }

            if (empty($currentOrganizationWebsite1)) {
                $website = Tools_LeadTools::prepareWebsiteUrl($organizationWebsite1);
                if (empty($currentOrganizationWebsite1) && !empty($website)) {
                    $domain = Tools_LeadTools::getDomainFromUrl($website);
                    if (!empty($domain)) {
                        $organizationDomain = $domain;
                    } else {
                        $organizationDomain = '';
                    }
                    if (!empty($domain)) {
                        $organizationModel->setOrganizationDomain1($organizationDomain);
                    }
                    $organizationModel->setOrganizationWebsite1($website);
                }
            }

            if (empty($currentOrganizationAddress) && !empty($organizationAddress)) {
                $organizationModel->setOrganizationAddress($organizationAddress);
            }

            if (empty($currentOrganizationAddress1) && !empty($organizationAddress1)) {
                $organizationModel->setOrganizationAddress1($organizationAddress1);
            }

            if (empty($currentOrganizationCity) && !empty($organizationCity)) {
                $organizationModel->setOrganizationCity($organizationCity);
            }

            if (empty($currentOrganizationCountryCode) && !empty($organizationCountryCode)) {
                $organizationModel->setOrganizationCountryCode($organizationCountryCode);
            }

            if (empty($currentOrganizationZip) && !empty($organizationZip)) {
                $organizationModel->setOrganizationZip($organizationZip);
            }

            if (empty($currentOrganizationEmail) && !empty($organizationEmail)) {
                $organizationModel->setOrganizationEmail($organizationEmail);
            }

            if (empty($currentOrganizationEmployees) && !empty($organizationEmployees)) {
                $organizationModel->setOrganizationEmployees($organizationEmployees);
            }

            if (empty($currentOrganizationPhone) && !empty($organizationPhone)) {
                $organizationModel->setOrganizationPhone($organizationPhone);
            }

            if (empty($currentOrganizationPhoneCountryCode) && !empty($organizationPhoneCountryCode)) {
                $organizationModel->setOrganizationPhoneCountryCode($organizationPhoneCountryCode);
            }

            if (empty($currentOrganizationPhoneCountryCodeValue) && !empty($organizationPhoneCountryCodeValue)) {
                $organizationModel->setOrganizationPhoneCountryCodeValue($organizationPhoneCountryCodeValue);
            }

            if (empty($currentOrganizationMobile) && !empty($organizationMobile)) {
                $organizationModel->setOrganizationMobile($organizationMobile);
            }

            if (empty($currentOrganizationMobileCountryCode) && !empty($organizationMobileCountryCode)) {
                $organizationModel->setOrganizationMobileCountryCode($organizationMobileCountryCode);
            }

            if (empty($currentOrganizationMobileCountryCodeValue) && !empty($organizationMobileCountryCodeValue)) {
                $organizationModel->setOrganizationMobileCountryCodeValue($organizationMobileCountryCodeValue);
            }

            if (empty($currentOrganizationStateCode) && !empty($organizationStateCode)) {
                $organizationModel->setOrganizationStateCode($organizationStateCode);
            }

            if (empty($currentOrganizationIndustryId) && !empty($organizationIndustryId)) {
                $organizationModel->setOrganizationIndustryId($organizationIndustryId);
            }

            if (empty($currentOrganizationNotes) && !empty($organizationNotes)) {
                $organizationModel->setOrganizationNotes($organizationNotes);
            }

            $firstContactOrganizationDate = $organizationModel->getFirstContactCreationDate();
            if (empty($firstContactOrganizationDate)) {
                $organizationModel->setFirstContactCreationDate($leadModel->getCreatedAt());
            }
            $leadOrganizationMapper->save($organizationModel);
        }

        if (!empty($ip)) {
            $leadsIpsModel = Tools_LeadTools::addIpAddressToLead($leadId, $ip);
            if ($leadsIpsModel instanceof Leads_Model_LeadsIpsModel) {
                Tools_LeadTools::addIpToMojo($leadsIpsModel);
            }
        }

        return $leadModel;

    }

    /**
     * Get all customer groups
     *
     * @return array
     */
    public static function getCustomerGroups()
    {
        $dbTable = new Store_DbTable_Group();
        $select = $dbTable->select()->from('shopping_group',
            array('groupName', 'id', 'priceSign', 'priceType', 'priceValue'));

        return $dbTable->getAdapter()->fetchAssoc($select);
    }

    /**
     * Clean values which already in use
     *
     * @param array $data form fields to clean
     * @return array
     */
    public static function removeUsedFields(array $data)
    {
        $cleanedFields = array_diff_key($data, array_flip(self::$_reservedFields));

        return $cleanedFields;
    }

}
