<?php

class Tools_LeadDownloaderWatchdog implements Interfaces_Observer
{

    /**
     *
     * @param $downloaderFilesModel
     * @return string
     */
    public function notify($downloaderFilesModel)
    {
        if ($downloaderFilesModel instanceof Downloader_Models_Model_DownloaderFilesModel) {
            $formParams = Zend_Controller_Front::getInstance()->getRequest()->getParams();
            if (!empty($formParams['email'])) {
                $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();

                $pageId = $downloaderFilesModel->getPageId();
                $pathToFile = $downloaderFilesModel->getFilePath();
                $leadsDownloaderLogMapper = Leads_Mapper_LeadsDownloaderLogMapper::getInstance();

                $leadEmail = Tools_LeadTools::getValue($formParams, 'email');
                $formParams = filter_var_array($formParams, FILTER_SANITIZE_STRING);
                $firstName = '';
                if (!empty($_POST['name'])) {
                    $firstName = filter_var($_POST['name'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES);
                }

                $lastName = Tools_LeadTools::getValue($formParams, 'lastname');
                $formName = Tools_LeadTools::getValue($formParams, 'formName');
                $mobile = Tools_LeadTools::getValue($formParams, 'mobile');
                $phone = Tools_LeadTools::getValue($formParams, 'phone');
                $zip = Tools_LeadTools::getValue($formParams, 'zip');
                $address = Tools_LeadTools::getValue($formParams, 'address');
                $position = Tools_LeadTools::getValue($formParams, 'position');
                $countryCode = Tools_LeadTools::getValue($formParams,'country');
                $stateCode = Tools_LeadTools::getValue($formParams,'state');
                $city = Tools_LeadTools::getValue($formParams, 'city');
                $prefix = Tools_LeadTools::getValue($formParams, 'prefix');
                $address1 = Tools_LeadTools::getValue($formParams, 'address1');
                $organization = Tools_LeadTools::getValue($formParams,'organization');
                $employees = filter_var(Zend_Controller_Front::getInstance()->getRequest()->getParam('employees'), FILTER_SANITIZE_NUMBER_INT);
                $groupName = Tools_LeadTools::getValue($formParams,'group-name');
                $tagNames = Tools_LeadTools::getValue($formParams,'tag-name');
                if (!is_array($tagNames)) {
                    $tagNames = array_filter(explode(',' , $tagNames));
                }
                $mobileCountryCode =  Tools_LeadTools::getValue($formParams,'mobile-country-code');
                $phoneCountryCode =  Tools_LeadTools::getValue($formParams,'phone-country-code');
                $website =  Tools_LeadTools::getValue($formParams, 'website');

                $session = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
                $currentLoggedUserId = $session->getCurrentUser()->getId();
                $currentLoggedUserEmail =  $session->getCurrentUser()->getEmail();;

                $leadModel = Tools_LeadTools::getLeadModelByIdAndEmail($currentLoggedUserId, $currentLoggedUserEmail, $leadEmail);

                if (!Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
                    $ip = Tools_LeadTools::getIpAddress();
                }

                $ownerId = Tools_LeadTools::getDefaultLeadOwner();

                $createdAt = Tools_System_Tools::convertDateFromTimezone('now');
                $leadType = Leads_Model_LeadsTypeModel::LEAD_INBOUND_DOWNLOADER;

                $organizationModel = null;
                $mobileCountryCodeValue = '';
                $phoneCountryCodeValue = '';
                $withTags = false;

                if (!empty($groupName)) {
                    $groupMapper = Store_Mapper_GroupMapper::getInstance();
                    $whereGroup = $groupMapper->getDbTable()->getAdapter()->quoteInto('groupName = ?', $groupName);
                    $groupExists = $groupMapper->fetchAll($whereGroup);
                    if (!empty($groupExists)) {
                        $groupId = $groupExists[0]->getId();
                    }
                }

                if (!empty($tagNames)) {
                    $withTags = true;
                }

                $countryCodes = Tools_Geo::getCountries(true);
                if (!empty($countryCode)) {
                    if (!array_key_exists($countryCode, $countryCodes)) {
                        $countryCode = '';
                    }
                }

                if (empty($mobileCountryCode)) {
                    if (!array_key_exists($mobileCountryCode, $countryCodes)) {
                        $mobileCountryCode = '';
                    }
                }

                if (empty($phoneCountryCode)) {
                    if (!array_key_exists($phoneCountryCode, $countryCodes)) {
                        $phoneCountryCode = '';
                    }
                }

                $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();
                if (!empty($shoppingConfig['country'])) {
                    if (empty($countryCode)) {
                        $countryCode = $shoppingConfig['country'];
                    }
                }

                $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
                $userDefaultMobileCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');

                if (empty($mobileCountryCode) && !empty($userDefaultMobileCountryCode)) {
                    $mobileCountryCode = $userDefaultMobileCountryCode;
                }

                if (empty($phoneCountryCode) && !empty($userDefaultMobileCountryCode)) {
                    $phoneCountryCode = $userDefaultMobileCountryCode;
                }

                if (!empty($mobile)) {
                    $mobile = Tools_LeadTools::cleanPhoneAddress($mobile);
                }

                if (!empty($phone)) {
                    $phone = Tools_LeadTools::cleanPhoneAddress($phone);
                }

                if (!empty($mobileCountryCode) && !empty($mobile)) {
                    $mobileNumberExists = Tools_LeadTools::checkLeadMobileOrPhoneNumber($mobile, $mobileCountryCode);
                    $mobileCountryCodeValue = Tools_LeadTools::getCountryCodeValue($mobileCountryCode);
                    if (!empty($mobileNumberExists)) {
                        $mobile = '';
                    }
                }
                if (!empty($phoneCountryCode) && !empty($phone)) {
                    $phoneNumberExists = Tools_LeadTools::checkLeadMobileOrPhoneNumber($phone, $phoneCountryCode);
                    $phoneCountryCodeValue = Tools_LeadTools::getCountryCodeValue($phoneCountryCode);
                    if (!empty($phoneNumberExists)) {
                        $phone = '';
                    }
                }

                $leadModelStatus = Tools_LeadFormTools::LEAD_MODEL_STATUS_OLD;

                if (!$leadModel instanceof Leads_Model_LeadsModel) {
                    $leadModelStatus = Tools_LeadFormTools::LEAD_MODEL_STATUS_NEW;
                    $leadAdditionAvailable = Tools_LeadLicenseTools::checkLimit();
                    if ($leadAdditionAvailable['error'] === true) {
                        $leadModel = new Leads_Model_LeadsModel();
                        $leadModel->registerObserver(new Tools_Mail_Watchdog(array(
                            'trigger' => Tools_LeadsMailWatchdog::LEADS_LICENSE_LIMIT
                        )));

                        $leadModel->notifyObservers();
                        return '';
                    }
                    $leadTypeId = Tools_LeadTools::getLeadTypeIdByLeadTypeName($leadType);
                    $leadModel = new Leads_Model_LeadsModel();
                    $leadModel->setCreatedAt($createdAt);
                    if (!empty($currentLoggedUserEmail)) {
                        $leadModel->setLeadEmail($currentLoggedUserEmail);
                    } else {
                        $leadModel->setLeadEmail($leadEmail);
                    }

                    if (!empty($countryCode)) {
                        $leadModel->setLeadCountryCode($countryCode);
                    }
                    if (!empty($stateCode)) {
                        $leadModel->setLeadStateCode($stateCode);
                    }
                    $leadModel->setPrefix($prefix);
                    $leadModel->setLeadCity($city);
                    $leadModel->setLeadFirstName($firstName);
                    $leadModel->setLeadMobileCountryCode($mobileCountryCode);
                    if (!empty($mobile)) {
                        $leadModel->setLeadMobile($mobile);
                        $leadModel->setLeadMobileCountryCodeValue($mobileCountryCodeValue);
                    }

                    $leadModel->setLeadPhoneCountryCode($phoneCountryCode);
                    if (!empty($phone)) {
                        $leadModel->setLeadPhone($phone);
                        $leadModel->setLeadPhoneCountryCodeValue($phoneCountryCodeValue);
                    }
                    if (!empty($groupId)) {
                        $leadModel->setCustomerGroupId($groupId);
                    }
                    $leadModel->setLeadLastName($lastName);
                    $leadModel->setLeadZip($zip);
                    $leadModel->setLeadAddress($address);
                    $leadModel->setLeadPosition($position);
                    $leadModel->setLeadAddress1($address1);
                    $leadModel->setOwnerId($ownerId);
                    $leadModel->setLeadTypeId($leadTypeId);
                    $leadModel->setLeadSource(Tools_LeadTools::getRefererUrl().' '.Tools_LeadTools::getUtmTagsString());
                } else {
                    $currentFirstName = $leadModel->getLeadFirstName();
                    $currentLeadLastName = $leadModel->getLeadLastName();
                    $currentMobile = $leadModel->getLeadMobile();
                    $currentPhone = $leadModel->getLeadPhone();
                    $currentZip = $leadModel->getLeadZip();
                    $currentAddress = $leadModel->getLeadAddress();
                    $currentPosition = $leadModel->getLeadPosition();
                    $currentPrefix = $leadModel->getPrefix();
                    $currentCity = $leadModel->getLeadCity();
                    $currentCountryCode = $leadModel->getLeadCountryCode();
                    $currentState = $leadModel->getLeadStateCode();
                    $currentAddress1 = $leadModel->getLeadAddress1();
                    $currentCustomerGroupId = $leadModel->getCustomerGroupId();

                    if (empty($currentFirstName)) {
                        $leadModel->setLeadFirstName($firstName);
                    }
                    if (empty($currentLeadLastName)) {
                        $leadModel->setLeadLastName($lastName);
                    }

                    if (empty($currentCustomerGroupId) && !empty($groupId)) {
                        $leadModel->setCustomerGroupId($groupId);
                    }

                    if (empty($currentMobile) && !empty($mobile)) {
                        $leadModel->setLeadMobile($mobile);
                        $leadModel->setLeadMobileCountryCodeValue($mobileCountryCodeValue);
                        $leadModel->setLeadMobileCountryCode($mobileCountryCode);
                    }
                    if (empty($currentPhone) && !empty($phone)) {
                        $leadModel->setLeadPhone($phone);
                        $leadModel->setLeadPhoneCountryCode($phoneCountryCode);
                        $leadModel->setLeadPhoneCountryCodeValue($phoneCountryCodeValue);
                    }

                    if (empty($currentZip)) {
                        $leadModel->setLeadZip($zip);
                    }

                    if (empty($currentAddress)) {
                        $leadModel->setLeadAddress($address);
                    }

                    if (empty($currentPosition)) {
                        $leadModel->setLeadPosition($position);
                    }

                    if (empty($currentPrefix)) {
                        $leadModel->setPrefix($prefix);
                    }

                    if (empty($currentCity)) {
                        $leadModel->setLeadCity($city);
                    }

                    if (empty($currentCountryCode) && !empty($countryCode)) {
                        $leadModel->setLeadCountryCode($countryCode);
                    }

                    if (empty($currentState) && !empty($stateCode)) {
                        $leadModel->setLeadStateCode($stateCode);
                    }

                    if (empty($currentAddress1)) {
                        $leadModel->setLeadAddress1($address1);
                    }

                }

                $oldUserId = $leadModel->getUserId();

                if (empty($oldUserId)) {
                    $leadModel->setUserId($currentLoggedUserId);
                }

                if (!empty($leadModel->getUserId()) && !empty($leadModel->getCustomerGroupId())) {
                    Tools_LeadTools::syncUserGroup($leadModel->getUserId(), $leadModel->getCustomerGroupId());
                }

                $leadTypeModel = Leads_Mapper_LeadsTypeMapper::getInstance()->findByTypeName(Leads_Model_LeadsTypeModel::LEAD_INBOUND_DOWNLOADER);
                if ($leadTypeModel instanceof Leads_Model_LeadsTypeModel) {
                    $leadModel->setLastActivityTypeId($leadTypeModel->getId());
                    $leadModel->setLastActivitySubtype('');
                }

                $currentOrganizationId = $leadModel->getOrganizationId();

                $assignPrimaryContact = false;
                if (!empty($organization) && empty($currentOrganizationId)) {
                    if (!empty($organization)) {
                        $assignPrimaryContact = Tools_LeadTools::getOrganizationPrimaryContactStatus($organization);
                    }

                    $organizationModel = Tools_LeadTools::processOrganization($organization);
                    if ($organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                        $leadModel->setOrganizationId($organizationModel->getId());
                    }
                }

                $leadModel->setLastActivity($createdAt);

                if($withTags){
                    $currentLeadId = $leadModel->getId();
                    $tagMapper = Leads_Mapper_LeadsTagsMapper::getInstance();

                    $incomingTags = $tagMapper->getAllIncomingTags($tagNames);
                    if(!empty($incomingTags)){
                        $incomingTags = array_flip($incomingTags);

                        if(!empty($currentLeadId)){
                            $findLeadByTag = $tagMapper->findTagsByLeadId($leadModel->getId());
                            if(!empty($findLeadByTag)){
                                foreach ($incomingTags as $tag){
                                    if(!in_array($tag, $findLeadByTag)){
                                        array_push($findLeadByTag, $tag);
                                    }
                                }
                            }else{
                                $findLeadByTag = $incomingTags;
                            }
                            $leadModel->setLeadTags($findLeadByTag);
                        }else{
                            $leadModel->setLeadTags($incomingTags);
                        }
                    }
                }
                try {
                    $leadModel = $leadsMapper->save($leadModel);
                } catch (Exception $e) {
                    return false;
                }
                $leadId = $leadModel->getId();

                if (!empty($organization) && $assignPrimaryContact === true && $organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                    $leadOrganizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
                    $organizationModel->setPrimaryContactId($leadId);
                    $currentOrganizationWebsite = $organizationModel->getOrganizationWebsite();
                    $website = Tools_LeadTools::prepareWebsiteUrl($website);
                    if (empty($currentOrganizationWebsite) && !empty($website)) {
                        $domain = Tools_LeadTools::getDomainFromUrl($website);
                        if (!empty($domain)) {
                            $organizationDomain = $domain;
                        } else {
                            $organizationDomain = '';
                        }
                        if (!empty($domain)) {
                            $organizationModel->setOrganizationDomain($organizationDomain);
                        }
                        $organizationModel->setOrganizationWebsite($website);
                    }
                    $firstContactOrganizationDate = $organizationModel->getFirstContactCreationDate();
                    if (empty($firstContactOrganizationDate)) {
                        $organizationModel->setFirstContactCreationDate($leadModel->getCreatedAt());
                    }
                    if(!empty($employees)) {
                        $organizationModel->setOrganizationEmployees($employees);
                    }

                    $leadOrganizationMapper->save($organizationModel);
                }

                if (!empty($ip)) {
                    $leadsIpsModel = Tools_LeadTools::addIpAddressToLead($leadId, $ip);
                    if ($leadsIpsModel instanceof Leads_Model_LeadsIpsModel) {
                        Tools_LeadTools::addIpToMojo($leadsIpsModel);
                    }
                }

                $leadsActionLog = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_DOWNLOAD,
                    $leadId,
                    Leads_Model_LeadsActionLogModel::ACTION_TYPE_INBOUND, $createdAt, $leadType);

                $pageModel = Application_Model_Mappers_PageMapper::getInstance()->find($pageId);
                $formParams['name'] = $firstName;
                $cleanedFormParams = Tools_LeadTools::cleanFormData($formParams);
                $formDetails = Tools_LeadTools::prepareFormDetailsHtml($cleanedFormParams);
                $formFieldsNotInUse = Tools_LeadTools::removeUsedFormFields($cleanedFormParams);
                if (!empty($formFieldsNotInUse)) {
                    $notesContent = Tools_LeadTools::prepareFormDetailsHtml($formFieldsNotInUse);
                    Tools_LeadTools::processNotes($leadId, $notesContent);
                }

                $leadDownloaderLogModel = new Leads_Model_LeadsDownloaderLogModel();
                $leadDownloaderLogModel->setPageId($pageId);
                $leadDownloaderLogModel->setActionLogId($leadsActionLog->getId());
                $leadDownloaderLogModel->setFilePath($pathToFile);
                $leadDownloaderLogModel->setFileName(Tools_Filesystem_Tools::basename($pathToFile));
                $leadDownloaderLogModel->setPageUrl($pageModel->getUrl());
                $leadDownloaderLogModel->setContent($formDetails);
                $leadDownloaderLogModel->setLeadId($leadId);
                $leadsDownloaderLogMapper->save($leadDownloaderLogModel);

                if (!empty($formName)) {
                    Tools_LeadFormTools::analyzeFormRules($formName,
                        Leads_Model_LeadsFormRulesGeneralConfigModel::FORM_TYPE_INTERNAL, $formParams, $leadModel, $leadModelStatus);
                }
            }
        }

    }
}

