<?php

/**
 * Class Lead_LeadEventTools
 */
class Tools_LeadEventTools extends Tools_LeadFormEventsTools
{

    /**
     * Analyze assigned form rules
     *
     * @param string $eventName event name
     * @param array $eventData form data
     * @param Leads_Model_LeadsModel $leadModel lead Model
     * @param string $leadModelStatus status of the lead (new, old)
     * @return array
     */
    public static function analyzeEventRules(
        $eventName,
        $eventData,
        Leads_Model_LeadsModel $leadModel,
        $leadModelStatus = self::LEAD_MODEL_STATUS_OLD
    ) {

        $eventIds = Leads_Mapper_LeadsEventsMapper::getInstance()->getEventByExternalId($eventName);
        if (empty($eventIds)) {
            return array();
        }

        $eventRules = Leads_Mapper_LeadsEventsRulesGeneralConfigMapper::getInstance()->getEventRules(array_keys($eventIds));
        if (!empty($eventRules)) {
            $ruleIds = array_combine(array_keys($eventRules), array_keys($eventRules));
        } else {
            return array();
        }

        $eventRulesWithFields = Leads_Mapper_LeadsEventsRulesGeneralConfigMapper::getInstance()->getEventRulesWithRules(array_keys($eventIds));
        if (!empty($eventRulesWithFields)) {
           foreach ($eventRulesWithFields as $eventRulesWithField) {
               unset($ruleIds[$eventRulesWithField['id']]);
           }
           $ruleIdsToAdd = self::getRuleIdsBasedOnData($eventRulesWithFields, $eventData);
           if (!empty($ruleIdsToAdd)) {
               foreach ($ruleIdsToAdd as $ruleIdToAdd => $ruleToAdd) {
                   $ruleIds[$ruleIdToAdd] = $ruleIdToAdd;
               }
           }
        }

        if (!empty($ruleIds)) {
            self::applyRuleActions($ruleIds, $leadModel, $leadModelStatus, self::LEAD_RULES_TYPE_EVENT, $eventData);
        }

        return array();
    }


}