<?php

require_once(__DIR__ . '/../../library/vendor/autoload.php');

/**
 * Class Tools_LeadExternalApiTools
 */
class Tools_LeadExternalApiTools
{

    const API_SECRET_KEY = 'dee2e5737bfc339cf4778f0c7f50b3e140cd8725';

    const ENCODING_ALG = 'HS256';

    const LEADS_ENDPOINT = 'leads';

    const LEAD_TAGS_ENDPOINT = 'tags';

    const LEAD_TASK_ENDPOINT = 'task';

    const LEAD_OWNERS_ENDPOINT = 'owners';

    const LEAD_ORGANIZATIONS_ENDPOINT = 'lead_organization';

    const ORGANIZATION_INDUSTRIES_ENDPOINT = 'organization_industries';

    const CUSTOMER_GROUPS_ENDPOINT = 'customer_group';

    const LEAD_NOTES_ENDPOINT = 'lead_notes';

    const ORGANIZATION_CUSTOM_PARAMS_ENDPOINT = 'organization_custom_params';

    const ORGANIZATION_CUSTOM_PARAMS_CONFIG_ENDPOINT = 'organization_custom_params_config';

    const LEAD_CUSTOM_PARAMS_CONFIG_ENDPOINT = 'lead_custom_params_config';

    const LEAD_CUSTOM_PARAMS_ENDPOINT = 'lead_custom_params';


    /**
     * @param string $login login(system email address)
     * @param string $password password
     * @return bool|string
     * @throws Zend_Auth_Adapter_Exception
     * @throws Zend_Exception
     */
    public static function authorize($login, $password)
    {
        $authAdapter = new Zend_Auth_Adapter_DbTable(
            Zend_Registry::get('dbAdapter'),
            'user',
            'email',
            'password',
            'MD5(?)'
        );
        $authAdapter->setIdentity($login);
        $authAdapter->setCredential($password);
        $authResult = $authAdapter->authenticate();
        if (!$authResult->isValid()) {
            return false;
        }

        $authUserData = $authAdapter->getResultRowObject(null, 'password');
        $userId = $authUserData->id;

        $ipAddressAllowed = self::verifyIpAddress($userId);
        if ($ipAddressAllowed === false) {
            return false;
        }

        return self::processToken($userId);
    }

    /**
     * Verify JWT token
     *
     * @param string $token JWT token
     * @return bool|object
     */
    public static function verify($token)
    {
        try {
            $tokenData = \Firebase\JWT\JWT::decode($token, self::API_SECRET_KEY, array(self::ENCODING_ALG));
            $ipAddressAllowed = self::verifyIpAddress($tokenData->userId);
            if ($ipAddressAllowed === false) {
                return false;
            }
            $leadsUsersApiTokensMapper = Leads_Mapper_LeadsUsersApiTokensMapper::getInstance();
            $tokenModel = $leadsUsersApiTokensMapper->findByToken($token);
            if (!$tokenModel instanceof Leads_Model_LeadsUsersApiTokensModel) {
                return false;
            }
        } catch (Exception $e) {
            return false;
        }

        return $tokenData;
    }

    /**
     * Verify and refresh token if exists
     *
     * @param string $token JWT token
     * @return bool|void
     */
    public static function verifyAndRefreshToken($token)
    {
        try {
            $tokenData = \Firebase\JWT\JWT::decode($token, self::API_SECRET_KEY, array(self::ENCODING_ALG));
            $ipAddressAllowed = self::verifyIpAddress($tokenData->userId);
            if ($ipAddressAllowed === false) {
                return false;
            }
        } catch (Exception $e) {
            if ($e->getMessage() === 'Expired token') {
                return self::refreshToken($token);
            }

            return false;
        }

        return self::refreshToken($token);

    }

    /**
     * @param string $token JWT token
     * @return bool|string
     */
    public static function refreshToken($token)
    {
        $leadsUsersApiTokensMapper = Leads_Mapper_LeadsUsersApiTokensMapper::getInstance();
        $leadsUsersApiTokensModel = $leadsUsersApiTokensMapper->findByToken($token);
        if (!$leadsUsersApiTokensModel instanceof Leads_Model_LeadsUsersApiTokensModel) {
            return false;
        }

        $leadsUsersApiAccessMapper = Leads_Mapper_LeadsUsersApiAccessMapper::getInstance();
        $userApiAccessModel = $leadsUsersApiAccessMapper->find($leadsUsersApiTokensModel->getApiUserId());
        $leadsUsersApiTokensMapper->deleteByToken($token);

        return self::processToken($userApiAccessModel->getUserId());

    }

    /**
     * @param int $userId system user id
     * @return bool|string
     */
    public static function processToken($userId)
    {
        $leadsUsersApiAccessMapper = Leads_Mapper_LeadsUsersApiAccessMapper::getInstance();
        $userApiAccessModel = $leadsUsersApiAccessMapper->findByUserId($userId);
        if (!$userApiAccessModel instanceof Leads_Model_LeadsUsersApiAccessModel) {
            return false;
        }

        $apiUserId = $userApiAccessModel->getId();

        $creationDate = Tools_System_Tools::convertDateFromTimezone('now');
        $expirationDate = Tools_System_Tools::convertDateFromTimezone('+1 day');

        $tokenData = array(
            'userId' => $userId,
            'iat' => strtotime('now'),
            'exp' => strtotime('+1 day')
        );

        $token = \Firebase\JWT\JWT::encode($tokenData, self::API_SECRET_KEY, self::ENCODING_ALG);

        $ipAddress = Tools_LeadTools::getIpAddress();

        $leadsUsersApiTokensMapper = Leads_Mapper_LeadsUsersApiTokensMapper::getInstance();
        $leadsUsersApiTokensModel = new Leads_Model_LeadsUsersApiTokensModel();
        $leadsUsersApiTokensModel->setToken($token);
        $leadsUsersApiTokensModel->setApiUserId($apiUserId);
        $leadsUsersApiTokensModel->setCreationDate($creationDate);
        $leadsUsersApiTokensModel->setExpirationDate($expirationDate);
        $leadsUsersApiTokensModel->setIpAddress($ipAddress);
        $leadsUsersApiTokensMapper->save($leadsUsersApiTokensModel);

        return $token;

    }

    /**
     * Add api request data to the log
     *
     * @param string $token JWT token
     * @param array $requestData request data
     * @param string $endPoint end point name
     * @param string $requestType request type GET|POST|PUT|DELETE
     * @return Leads_Model_LeadsApiRequestLogModel
     */
    public static function logRequest($token, $requestData, $endPoint = '', $requestType = 'GET')
    {
        $leadsUsersApiTokensMapper = Leads_Mapper_LeadsUsersApiTokensMapper::getInstance();
        $leadsUsersApiTokensModel = $leadsUsersApiTokensMapper->findByToken($token);
        if (!$leadsUsersApiTokensModel instanceof Leads_Model_LeadsUsersApiTokensModel) {
            return null;
        }

        $apiUserId = $leadsUsersApiTokensModel->getApiUserId();

        $ipAddress = Tools_LeadTools::getIpAddress();
        $creationDate = Tools_System_Tools::convertDateFromTimezone('now');
        $leadsApiRequestLogMapper = Leads_Mapper_LeadsApiRequestLogMapper::getInstance();
        $leadsApiRequestLogModel = new Leads_Model_LeadsApiRequestLogModel();
        $leadsApiRequestLogModel->setIpAddress($ipAddress);
        $leadsApiRequestLogModel->setApiUserId($apiUserId);
        $leadsApiRequestLogModel->setToken($token);
        $leadsApiRequestLogModel->setCreatedAt($creationDate);
        $leadsApiRequestLogModel->setEndpoint($endPoint);
        $leadsApiRequestLogModel->setRequestType($requestType);
        $leadsApiRequestLogModel->setRequest(json_encode($requestData));

        return $leadsApiRequestLogMapper->save($leadsApiRequestLogModel);
    }

    /**
     * Check if user eligible
     *
     * @param int $userId system id
     * @return bool
     */
    public static function verifyIpAddress($userId)
    {
        $ipAddress = Tools_LeadTools::getIpAddress();
        $leadsUsersApiAccessMapper = Leads_Mapper_LeadsUsersApiAccessMapper::getInstance();
        $userApiAccessModel = $leadsUsersApiAccessMapper->findByUserId($userId);
        if (!$userApiAccessModel instanceof Leads_Model_LeadsUsersApiAccessModel) {
            return false;
        }

        $ipAddresses = $userApiAccessModel->getIpAddresses();
        if (!empty($ipAddresses)) {
            $ipAddresses = array_filter(explode(',', $userApiAccessModel->getIpAddresses()));
            if (!in_array($ipAddress, $ipAddresses)) {
                return false;
            }
        }

        return true;

    }

}
