<?php

/**
 * Class Lead_LeadExternalEmailsTools
 */
class Tools_LeadExternalEmailsTools
{

    /**
     * Allowed email statuses
     *
     * @var array
     */
    public static $_allowedStatuses = array(
        Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_SENT,
        Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_BOUNCE,
        Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_DEFERRED,
        Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_DROPPED,
        Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_RECEIVED,
        Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_CLICK,
        Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_OPENED,
        Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_UNSUBSCRIBED
    );

    /**
     * Process external email service
     *
     * @param array $emailData
     * @return string
     */
    public static function processEmailServiceLog($emailData)
    {
//        $emailData = array(
//            'subject' => 'my subject',
//            'content' => 'some content',
//            'fromEmail' => 'abrasnow1@yahoo.com',
//            'email' => 'abrasnow@yahoo.com',
//            'event' => 'click',
//            'internalEmailMessageId' => 'asd234234234234',
//            'url' => 'http://somelink.com/link.html',
//            'fromName' => 'test name',
//            'campaignName' => 'campaign name',
//            'emailLists' => array('5' => 'Crm list'),
//            'timestamp' => '1516804682',
//            'serviceName' => ''
//        );

        //$delivered = json_decode(' {"internalEmailMessageId":"3","useragent":null,"event":"delivered","email":"vladymyr@seosamba.com","ip":"167.89.106.64","timestamp":1516804682,"content":"<div style=\"color:red;\">CRM content! Email sent from MOJO\r\n<a class=\"qweeewq\" href=\'https:\/\/www.google.com\'>google<\/a> Search for MOJO platform <\/div> Also visit SeoToaster CMS\r\n<a href=\"http:\/\/www.i.ua\/\">SeoToaster<\/a>","fromEmail":"crm@service.com","fromName":"CRM department","subject":"Notification CRM service","campaignName":"CRM Email PR","lists":"{\"5\":\"CRM list\"}"}', true);
        //$open = json_decode('{"internalEmailMessageId":"3","useragent":"Mozilla\/5.0 (Windows NT 5.1; rv:11.0) Gecko Firefox\/11.0 (via ggpht.com GoogleImageProxy)","event":"open","email":"vladymyr@seosamba.com","ip":"66.249.81.153","timestamp":1516811227,"content":"<div style=\"color:red;\">CRM content! Email sent from MOJO\r\n<a class=\"qweeewq\" href=\'https:\/\/www.google.com\'>google<\/a> Search for MOJO platform <\/div> Also visit SeoToaster CMS\r\n<a href=\"http:\/\/www.i.ua\/\">SeoToaster<\/a>","fromEmail":"crm@service.com","fromName":"CRM department","subject":"Notification CRM service","campaignName":"CRM Email PR","lists":"{\"5\":\"CRM list\"}"}',true);
        //$click = json_decode('{"internalEmailMessageId":"3","useragent":"Mozilla\/5.0 (X11; Linux x86_64) AppleWebKit\/537.36 (KHTML, like Gecko) Chrome\/55.0.2883.75 Safari\/537.36","event":"click","email":"vladymyr@seosamba.com","ip":"80.249.233.43","timestamp":1516811277,"content":"<div style=\"color:red;\">CRM content! Email sent from MOJO\r\n<a class=\"qweeewq\" href=\'https:\/\/www.google.com\'>google<\/a> Search for MOJO platform <\/div> Also visit SeoToaster CMS\r\n<a href=\"http:\/\/www.i.ua\/\">SeoToaster<\/a>","fromEmail":"crm@service.com","fromName":"CRM department","subject":"Notification CRM service","campaignName":"CRM Email PR","lists":"{\"5\":\"CRM list\"}","url":"http:\/\/www.i.ua\/"}', true);
        //$emailData = $click;

        if (!empty($emailData) && !empty($emailData['email'])) {
            $leadDataMapper = Leads_Mapper_LeadsMapper::getInstance();
            $leadModel = $leadDataMapper->findByEmail($emailData['email']);
            if ($leadModel instanceof Leads_Model_LeadsModel) {
                $serverTimezone = date_default_timezone_get();
                if (empty($serverTimezone)) {
                    $serverTimezone = 'UTC';
                }

                $eventDate = Tools_System_Tools::convertDateFromTimezone(date('Y-m-d H:i:s', $emailData['timestamp']),
                    $serverTimezone, 'UTC');
                $leadTypeModel = Leads_Mapper_LeadsTypeMapper::getInstance()->findByTypeName(Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_EMAIL);
                $leadId = $leadModel->getId();
                $trackingCodeHash = hash('sha256', $emailData['internalEmailMessageId']);

                if (array_key_exists($emailData['event'], Tools_LeadTools::$_externalEmailStatusAssociation)) {
                    $emailStatus = Tools_LeadTools::$_externalEmailStatusAssociation[$emailData['event']];
                } elseif (in_array($emailData['event'], Tools_LeadTools::$_externalEmailStatusAssociation,
                    true)) {
                    $emailStatus = $emailData['event'];
                } else {
                    $emailStatus = Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_SENT;
                }

                if (!in_array($emailStatus, self::$_allowedStatuses, true)) {
                    return false;
                }

                $leadsEmailLogMapper = Leads_Mapper_LeadsEmailLogMapper::getInstance();
                if ($emailStatus === Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_SENT || $emailStatus === Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_OPENED || $emailStatus === Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_BOUNCE || $emailStatus === Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_UNSUBSCRIBED) {
                    $leadsEmailLogModel = $leadsEmailLogMapper->getByTrackingCode($trackingCodeHash, $emailStatus);
                    if ($leadsEmailLogModel instanceof Leads_Model_LeadsEmailLogModel) {
                        return false;
                    }
                }

                if ($emailStatus === Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_CLICK) {
                    $fileCodeHash = hash('sha256', $emailData['url']);
                    $leadsEmailLinkAttachmentLogMapper = Leads_Mapper_LeadsEmailLinkAttachmentLogMapper::getInstance();
                    $leadsEmailLinkAttachmentLogModel = $leadsEmailLinkAttachmentLogMapper->getByFileHashSentHash($trackingCodeHash,
                        $fileCodeHash);
                    if ($leadsEmailLinkAttachmentLogModel instanceof Leads_Model_LeadsEmailLinkAttachmentLogModel) {
                        return false;
                    }
                }

                if ($emailStatus === Leads_Model_LeadsTypeModel::LEAD_OUTBOUND_EMAIL_SUBTYPE_DEFERRED || $emailStatus === Leads_Model_LeadsTypeModel::LEAD_OUTBOUND_EMAIL_SUBTYPE_DROPPED) {
                    return false;
                }

                if ($leadTypeModel instanceof Leads_Model_LeadsTypeModel && ($emailStatus === Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_OPENED || $emailStatus === Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_CLICK)) {
                    $leadModel->setLastActivityTypeId($leadTypeModel->getId());

                    if ($emailStatus === Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_OPENED) {
                        $leadModel->setLastActivitySubtype(Leads_Model_LeadsTypeModel::LEAD_OUTBOUND_EMAIL_SUBTYPE_OPEN);
                    }

                    if ($emailStatus === Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_CLICK) {
                        $leadModel->setLastActivitySubtype(Leads_Model_LeadsTypeModel::LEAD_OUTBOUND_EMAIL_SUBTYPE_CLICK);
                    }

                    $leadModel->setLastActivity(Tools_System_Tools::convertDateFromTimezone('now'));
                    $leadDataMapper->save($leadModel);
                }


                $actionLogModel = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_EMAIL, $leadId,
                    Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND, $eventDate,
                    Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_EMAIL,
                    Leads_Model_LeadsActionLogModel::EVENT_LOG_EXPIRED, $emailData['content']);

                $leadsEmailLinkAttachmentLogModel = null;
                if ($emailStatus === Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_CLICK) {
                    $leadsEmailLinkAttachmentLogModel = $leadsEmailLinkAttachmentLogMapper->getByFileHashSentHash($trackingCodeHash,
                        $fileCodeHash);
                    if (!$leadsEmailLinkAttachmentLogModel instanceof Leads_Model_LeadsEmailLinkAttachmentLogModel) {
                        $leadsEmailLinkAttachmentLogModel = new Leads_Model_LeadsEmailLinkAttachmentLogModel();
                        $leadsEmailLinkAttachmentLogModel->setEmailLogSentHash($trackingCodeHash);
                        $leadsEmailLinkAttachmentLogModel->setFileHash($fileCodeHash);
                        $leadsEmailLinkAttachmentLogModel->setFileName($emailData['url']);
                        $leadsEmailLinkAttachmentLogModel->setFileFolderName('');
                        $leadsEmailLinkAttachmentLogModel->setCreatedDateTime(Tools_System_Tools::convertDateFromTimezone('now'));
                        $leadsEmailLinkAttachmentLogModel->setLinkType(Leads_Model_LeadsEmailLinkAttachmentLogModel::LINK_TYPE_CLICK);
                        $leadsEmailLinkAttachmentLogModel->setSourceType(Leads_Model_LeadsEmailLinkAttachmentLogModel::EMAIL_ATTACHMENT_TYPE_EXTERNAL);
                        $leadsEmailLinkAttachmentLogModel = $leadsEmailLinkAttachmentLogMapper->save($leadsEmailLinkAttachmentLogModel);
                    }
                }

                $leadsEmailLogModel = new Leads_Model_LeadsEmailLogModel();
                $leadsEmailLogModel->setActionLogId($actionLogModel->getId());
                $leadsEmailLogModel->setContent($emailData['content']);
                $leadsEmailLogModel->setEmailFrom($emailData['fromEmail']);
                $leadsEmailLogModel->setEmailFromName($emailData['fromEmail']);
                $leadsEmailLogModel->setEmailTo($emailData['email']);
                $leadsEmailLogModel->setEmailSubject($emailData['subject']);
                $leadsEmailLogModel->setEmailTemplate($emailData['content']);
                $leadsEmailLogModel->setTrackingCode($trackingCodeHash);
                $leadsEmailLogModel->setLeadId($leadId);
                $leadsEmailLogModel->setEmailBcc('');
                $leadsEmailLogModel->setEmailStatus($emailStatus);
                if (!empty($emailData['campaignName'])) {
                    $leadsEmailLogModel->setCampaignName($emailData['campaignName']);
                }

                if (!empty($emailData['emailLists'])) {
                    $leadsEmailLogModel->setListNames(implode(',',$emailData['emailLists']));
                }

                if (!empty($emailData['serviceName'])) {
                    $leadsEmailLogModel->setExternalServiceName($emailData['serviceName']);
                }

                if ($leadsEmailLinkAttachmentLogModel instanceof Leads_Model_LeadsEmailLinkAttachmentLogModel) {
                    $leadsEmailLogModel->setAttachmentId($leadsEmailLinkAttachmentLogModel->getId());
                }
                $leadsEmailLogModel->setEmailSourceType(Leads_Model_LeadsEmailLogModel::EMAIL_SOURCE_TYPE_EXTERNAL);
                $leadsEmailLogMapper->save($leadsEmailLogModel);

                return true;
            }
        }
    }
}