<?php

/**
 * Class Lead_LeadExternalFormTools
 */
class Tools_LeadExternalFormTools
{

    /**
     * Send emails
     *
     * @param Leads_Model_LeadsBuilderFormGeneralConfigModel $leadsFormBuilderGeneralConfigModel form builder model
     * @param array $processedFields form fields
     * @param array $customEmailsTo custom admin emails
     * @param bool $withoutAutoReply without auto reply flag
     * @throws Exceptions_SeotoasterException
     */
    public static function sendEmails($leadsFormBuilderGeneralConfigModel, $processedFields, $customEmailsTo = array(), $withoutAutoReply = false)
    {
        $sendEmail = $leadsFormBuilderGeneralConfigModel->getSendEmail();
        $sendAutoReply = $leadsFormBuilderGeneralConfigModel->getSendAutoReply();

        if (!empty($sendEmail)) {
            $sendEmailConfig = $leadsFormBuilderGeneralConfigModel->getSendEmailConfig();
            if (!empty($sendEmailConfig)) {
                $sendEmailConfig = json_decode($sendEmailConfig, true);
                if (!empty($sendEmailConfig)) {
                    if (!empty($sendEmailConfig['emailTo']) && !empty($sendEmailConfig['adminEmailTemplate']) && !empty($sendEmailConfig['adminEmailFrom'])
                        && !empty($sendEmailConfig['adminEmailSubject'])) {

                        if (!empty($customEmailsTo)) {
                            $emailsTo = $customEmailsTo;
                        } else {
                            $emailsTo = array_unique(array_filter(explode(',', $sendEmailConfig['emailTo'])));
                        }
                        foreach ($emailsTo as $email) {
                            $emailValidator = new Tools_System_CustomEmailValidator();
                            if ($emailValidator->isValid($email)) {
                                $emailsToSendFiltered[] = $email;
                            }
                        }

                        if (!empty($emailsToSendFiltered)) {
                            $emailTemplate = Application_Model_Mappers_TemplateMapper::getInstance()->find($sendEmailConfig['adminEmailTemplate']);
                            if ($emailTemplate instanceof Application_Model_Models_Template) {
                                $options = array();
                                $options['template'] = $sendEmailConfig['adminEmailTemplate'];
                                $options['trigger'] = 'ExternalAdmin';
                                $options['data'] = $processedFields;
                                $options['emailsTo'] = $emailsToSendFiltered;
                                $options['withoutPreparseTemplate'] = true;
                                $options['emailFrom'] = $sendEmailConfig['adminEmailFrom'];
                                $options['emailSubject'] = $sendEmailConfig['adminEmailSubject'];
                                $options['subject'] = $sendEmailConfig['adminEmailSubject'];
                                $options['withoutPreparseTemplate'] = true;
                                if (!empty($sendEmailConfig['adminEmailMessage'])) {
                                    $options['message'] = $sendEmailConfig['adminEmailMessage'];
                                }

                                $formModel = new Application_Model_Models_Form();
                                $systemMailWatchdog = new Tools_LeadsMailWatchdog($options);
                                $systemMailWatchdog->notify($formModel);
                            }
                        }

                    }
                }
            }
        }

        if (!empty($sendAutoReply) && $withoutAutoReply === false) {
            $sendAutoReplyConfig = $leadsFormBuilderGeneralConfigModel->getSendAutoReplyConfig();
            if (!empty($sendAutoReplyConfig)) {
                $sendAutoReplyConfig = json_decode($sendAutoReplyConfig, true);
                if (!empty($sendAutoReplyConfig)) {
                    if (!empty($sendAutoReplyConfig['replyFromEmail']) && !empty($sendAutoReplyConfig['autoReplyEmailTemplate']) && !empty($sendAutoReplyConfig['autoReplyFromName'])
                        && !empty($sendAutoReplyConfig['autoReplySubject'])) {

                        $emailTemplate = Application_Model_Mappers_TemplateMapper::getInstance()->find($sendAutoReplyConfig['autoReplyEmailTemplate']);
                        if ($emailTemplate instanceof Application_Model_Models_Template) {
                            $options = array();

                            $emailToName = '';
                            if (!empty($processedFields['lastname'])) {
                                $emailToName .= $processedFields['lastname'];
                            }
                            if (!empty($processedFields['name'])) {
                                $emailToName .= ' ' . $processedFields['name'];
                            }

                            $options['template'] = $sendAutoReplyConfig['autoReplyEmailTemplate'];
                            $options['trigger'] = 'ExternalAutoReply';
                            $options['data'] = $processedFields;
                            $options['emailTo'] = $processedFields['email'];
                            $options['emailToName'] = $emailToName;
                            $options['withoutPreparseTemplate'] = true;
                            $options['emailFrom'] = $sendAutoReplyConfig['replyFromEmail'];
                            $options['emailSubject'] = $sendAutoReplyConfig['autoReplySubject'];
                            $options['emailReplyFromName'] = $sendAutoReplyConfig['autoReplyFromName'];
                            $options['subject'] = $sendAutoReplyConfig['autoReplySubject'];
                            $options['withoutPreparseTemplate'] = true;
                            if (!empty($sendAutoReplyConfig['autoReplyMessage'])) {
                                $options['message'] = $sendAutoReplyConfig['autoReplyMessage'];
                            }

                            $formModel = new Application_Model_Models_Form();
                            $systemMailWatchdog = new Tools_LeadsMailWatchdog($options);
                            $systemMailWatchdog->notify($formModel);
                        }

                    }
                }

            }
        }
    }

    /**
     * Send sms
     *
     * @param Leads_Model_LeadsBuilderFormGeneralConfigModel $leadsFormBuilderGeneralConfigModel form builder model
     * @param array $processedFields form fields
     * @throws Exceptions_SeotoasterPluginException
     */
    public static function sendSms($leadsFormBuilderGeneralConfigModel, $processedFields)
    {
        $smsMobiles = $leadsFormBuilderGeneralConfigModel->getSmsMobile();
        $externalId = $leadsFormBuilderGeneralConfigModel->getExternalId();
        $leadsFormBuilderGeneralConfigModel->getFormDomain();
        $customFormParams = array();
        $subscriber['form_url'] = $processedFields['formUrl'];
        $subscriber['form_name'] = $processedFields['formName'];
        $subscriber['sms_from_type'] = Apps::SMS_FROM_TYPE_FORM;
        $subscriber['formType'] = 'external';
        foreach ($processedFields as $paramName => $paramValue) {
            $customFormParams[$paramName] = $paramValue;
        }

        $mobileCountryCode = Tools_LeadTools::getValue($processedFields, 'mobile-country-code');
        $phoneCountryCode = Tools_LeadTools::getValue($processedFields, 'phone-country-code');
        if (!empty($mobileCountryCode)) {
            $countryCodes = Tools_Geo::getCountries(true);
            if (!array_key_exists($mobileCountryCode, $countryCodes)) {
                $mobileCountryCode = '';
            }
        }

        if (!empty($phoneCountryCode)) {
            $countryCodes = Tools_Geo::getCountries(true);
            if (!array_key_exists($phoneCountryCode, $countryCodes)) {
                $phoneCountryCode = '';
            }
        }

        $mobile = Tools_LeadTools::getValue($processedFields, 'mobile');
        $phone = Tools_LeadTools::getValue($processedFields, 'phone');
        if (!empty($mobileCountryCode)) {
            $mobileCountryCodeValue = Tools_LeadTools::getCountryCodeValue($mobileCountryCode);
            $mobile = $mobileCountryCodeValue . $mobile;
            $customFormParams['mobile'] = $mobile;
        }
        if (!empty($phoneCountryCode)) {
            $phoneCountryCodeValue = Tools_LeadTools::getCountryCodeValue($phoneCountryCode);
            $phone = $phoneCountryCodeValue . $phone;
            $customFormParams['phone'] = $phone;
        }

        $domain = $leadsFormBuilderGeneralConfigModel->getFormDomain();
        $subscriber['domainName'] = $domain;
        $subscriber['external_id'] = $externalId;
        if (!empty($smsMobiles)) {
            $smsMobiles = array_unique(array_filter(explode(',', $smsMobiles)));
            if (!empty($smsMobiles)) {
                $mobilesToSend = array();
                foreach ($smsMobiles as $smsMobile) {
                    $smsMobile = Apps_Tools_Twilio::normalizePhoneNumberToE164($smsMobile);
                    if ($smsMobile) {
                        array_push($mobilesToSend, $smsMobile);
                    }
                }

                $messageAdmin = 'Hello you\'ve received a request from: ' . $processedFields['formUrl'] . PHP_EOL;

                foreach ($processedFields as $name => $value) {
                    if (!$value || $name === 'externalFormSignature') {
                        continue;
                    }
                    $messageAdmin .= $name . ': ' . (is_array($value) ? implode(', ', $value) : $value) . PHP_EOL;
                }

                if (!empty($mobilesToSend)) {
                    $subscriber['subscriber']['admin'] = array(
                        'phone' => $mobilesToSend,
                        'message' => $messageAdmin,
                        'owner_type' => Apps::SMS_OWNER_TYPE_ADMIN,
                        'custom_params' => $customFormParams
                    );
                }
            }
        }

        $autoReplyConfig = $leadsFormBuilderGeneralConfigModel->getSendAutoReplyConfig();
        if (!empty($autoReplyConfig)) {
            $autoReplyConfig = json_decode($autoReplyConfig, true);
        }

        if (!empty($autoReplyConfig['sendSms'])) {

            $mobilePhoneUser = Apps_Tools_Twilio::normalizePhoneNumberToE164($mobile);
            $phonePhoneUser = Apps_Tools_Twilio::normalizePhoneNumberToE164($phone);
            $messageUser = $autoReplyConfig['autoReplyMessage'];

            if ($phonePhoneUser) {
                $subscriber['subscriber']['user'] = array(
                    'phone' => array($phonePhoneUser),
                    'message' => $messageUser,
                    'owner_type' => Apps::SMS_OWNER_TYPE_USER,
                    'sms_from_type' => 'crm',
                    'custom_params' => $customFormParams
                );
            }

            if ($mobilePhoneUser) {
                $subscriber['subscriber']['user'] = array(
                    'phone' => array($mobilePhoneUser),
                    'message' => $messageUser,
                    'owner_type' => Apps::SMS_OWNER_TYPE_USER,
                    'sms_from_type' => 'crm',
                    'custom_params' => $customFormParams
                );
            }
        }

        if (!empty($subscriber['subscriber'])) {
            Apps::apiCall('POST', 'apps', array('twilioSms'), $subscriber);
        }
    }

}
