<?php

/**
 * Class Lead_LeadFormAbstractTools
 */
class Tools_LeadFormEventsTools
{

    /**
     * Lead model status new
     */
    const LEAD_MODEL_STATUS_NEW = 'new';

    /**
     * Lead model status old
     */
    const LEAD_MODEL_STATUS_OLD = 'old';

    /**
     * Lead action type form
     */
    const LEAD_RULES_TYPE_FORM = 'type_form';

    /**
     * Lead action type event
     */
    const LEAD_RULES_TYPE_EVENT = 'type_event';


    const WITHOUT_RULE_CONFIG_VALUE = 'without_rule_value';

    /**
     * Allowed administrative roles list
     *
     * @var array
     */
    public static $_allowedAdministrativeRoles = array(
        Tools_Security_Acl::ROLE_SUPERADMIN,
        Tools_Security_Acl::ROLE_ADMIN,
        Shopping::ROLE_SALESPERSON
    );

    /**
     * Get rule ids based on form data
     *
     * @param array $formRules form rules
     * @param array $formData form data
     * @return array
     */
    public static function getRuleIdsBasedOnData($formRules, $formData)
    {
        $ruleIds = array();
        $skipId = 0;
        foreach ($formRules as $formRule) {
            $skipStatus = false;
            $operator = $formRule['rule_comparison_operator'];
            $fieldName = $formRule['field_name'];
            $fieldNameAlternative = $formRule['field_name'].'[]';
            $fieldValue = mb_strtolower($formRule['field_value']);
            if (isset($formData[$fieldNameAlternative])) {
                $fieldName = $fieldNameAlternative;
            }

            if ($fieldName === self::WITHOUT_RULE_CONFIG_VALUE) {
                $ruleIds[$formRule['id']] = $formRule['id'];
                continue;
            } elseif (!isset($formData[$fieldName])) {
                continue;
            }

            if (is_array($formData[$fieldName])) {
                $formDataValue = array_map('mb_strtolower', $formData[$fieldName]);
            } else {
                $formDataValue = mb_strtolower($formData[$fieldName]);
            }
            $ruleId = $formRule['id'];
            if ($skipId === $ruleId) {
                continue;
            }
            if (array_key_exists($fieldName, $formData)) {
                if ($operator === Leads_Model_LeadsFormRulesConfigModel::RULE_COMPARISON_OPERATOR_EQUAL) {
                    if (is_array($formDataValue)) {
                        if (!in_array($fieldValue, $formDataValue)) {
                            $skipStatus = true;
                        }
                    } else {
                        if ($formDataValue !== $fieldValue) {
                            $skipStatus = true;
                        }
                    }
                }
                if ($operator === Leads_Model_LeadsFormRulesConfigModel::RULE_COMPARISON_OPERATOR_IN) {
                    $compareToArray = explode(',', $fieldValue);
                    $compareToArray = array_map("mb_strtolower", $compareToArray);
                    if (is_array($formDataValue)) {
                        $comparisonResult = array_intersect($compareToArray, $formDataValue);
                        if (count($comparisonResult) == 0) {
                            $skipStatus = true;
                        }

                    } else {
                        if (!in_array($formDataValue, $compareToArray)) {
                            $skipStatus = true;
                        }
                    }

                }
                if ($operator === Leads_Model_LeadsFormRulesConfigModel::RULE_COMPARISON_OPERATOR_LIKE) {
                    $compareToArray = explode(',', $fieldValue);
                    $compareToArray = array_map("mb_strtolower", $compareToArray);
                    $compareToArrayLength = count($compareToArray);
                    if ($compareToArrayLength > 1) {
                        if (is_array($formDataValue)) {
                            $comparisonResult = array_intersect($compareToArray, $formDataValue);
                            if (count($comparisonResult) == 0) {
                                $skipStatus = true;
                            }
                        } else {
                            $compValCounter = 0;
                            foreach ($compareToArray as $compVal) {
                                $matchFound = mb_strpos($formDataValue, $compVal);
                                if ($matchFound === false) {
                                    $compValCounter = $compValCounter+1;
                                }
                            }

                            if ($compValCounter === $compareToArrayLength) {
                                $skipStatus = true;
                            }
                        }
                    } else {
                        if (is_array($formDataValue)) {
                            $inputSearchPattern = preg_quote($fieldValue, '~');
                            $inputSearchResult = preg_grep('~' . $inputSearchPattern . '~', $formDataValue);
                            if (empty($inputSearchResult)) {
                                $skipStatus = true;
                            }
                        } else {
                            $matchFound = mb_strpos($formDataValue, $fieldValue);
                            if ($matchFound === false) {
                                $skipStatus = true;
                            }
                        }
                    }
                }
                if ($operator === Leads_Model_LeadsFormRulesConfigModel::RULE_COMPARISON_OPERATOR_NOTEQUAL) {
                    if (is_array($formDataValue)) {
                        if (in_array($fieldValue, $formDataValue)) {
                            $skipStatus = true;
                        }
                    } else {
                        if ($formDataValue === $fieldValue) {
                            $skipStatus = true;
                        }
                    }
                }
                if ($operator === Leads_Model_LeadsFormRulesConfigModel::RULE_COMPARISON_GREATER_THAN) {
                    if(is_numeric($formDataValue)) {
                        if($fieldValue > $formDataValue) {
                            $skipStatus = true;
                        }
                    } elseif (is_array($formDataValue)) {
                        foreach ($formDataValue as $value) {
                            if(is_numeric($value)) {
                                if($fieldValue > $formDataValue) {
                                    $skipStatus = true;
                                }
                            } else {
                                $skipStatus = true;
                            }
                        }
                    }
                    else {
                        $skipStatus = true;
                    }
                }
                if ($operator === Leads_Model_LeadsFormRulesConfigModel::RULE_COMPARISON_LESS_THAN) {
                    if(is_numeric($formDataValue)) {
                        if($fieldValue < $formDataValue) {
                            $skipStatus = true;
                        }
                    } elseif (is_array($formDataValue)) {
                        foreach ($formDataValue as $value) {
                            if(is_numeric($value)) {
                                if($fieldValue < $formDataValue) {
                                    $skipStatus = true;
                                }
                            } else {
                                $skipStatus = true;
                            }
                        }
                    } else {
                        $skipStatus = true;
                    }
                }

                if ($skipStatus === true) {
                    $skipId = $ruleId;
                    unset($ruleIds[$ruleId]);
                } else {
                    $ruleIds[$ruleId] = $ruleId;
                }
            }
        }

        return $ruleIds;
    }

    /**
     * Apply actions based on the matched form rules
     *
     * @param array $ruleIds rule ids to apply
     * @param Leads_Model_LeadsModel $leadModel lead Model
     * @param string $leadModelStatus status of the lead (new, old)
     * @param string $actionType action type type_form|type_event
     * @param array $formData form data
     * @throws Exceptions_SeotoasterException
     * @throws Zend_Exception
     * @throws Zend_File_Transfer_Exception
     */
    public static function applyRuleActions(
        $ruleIds = array(),
        Leads_Model_LeadsModel $leadModel,
        $leadModelStatus = self::LEAD_MODEL_STATUS_OLD,
        $actionType = self::LEAD_RULES_TYPE_FORM,
        $formData = array()
    ) {

        if ($actionType === self::LEAD_RULES_TYPE_FORM) {
            $actions = Leads_Mapper_LeadsFormRulesActionsMapper::getInstance()->getActionByRuleIds($ruleIds);
        }

        if ($actionType === self::LEAD_RULES_TYPE_EVENT) {
            $actions = Leads_Mapper_LeadsEventRulesActionsMapper::getInstance()->getActionByRuleIds($ruleIds);
        }

        $originalActionType = $actionType;

        if (!empty($actions)) {
            foreach ($actions as $action) {
                $actionType = $action['action_type'];
                $actionConfig = $action['action_config'];
                if (!empty($actionConfig)) {
                    $actionConfig = json_decode($actionConfig, true);
                    $triggerNewLeadOnly = false;
                    if (!empty($actionConfig['triggerNewLeadOnly'])) {
                        $triggerNewLeadOnly = true;
                    }

                    if ($leadModelStatus !== self::LEAD_MODEL_STATUS_NEW && $triggerNewLeadOnly === true) {
                        continue;
                    }

                    if ($actionType === Leads_Model_LeadsFormRulesActionModel::ACTION_TYPE_ASSIGN_OWNER) {
                        self::assignLeadOwnerAction($actionConfig, $leadModel);
                    }

                    if ($actionType === Leads_Model_LeadsFormRulesActionModel::ACTION_TYPE_ASSIGN_TASK) {
                        self::assignTaskAction($actionConfig, $leadModel);
                    }

                    if ($actionType === Leads_Model_LeadsFormRulesActionModel::ACTION_TYPE_ASSIGN_TAGS) {
                        self::assignLeadTagsAction($actionConfig, $leadModel);
                    }

                    if ($actionType === Leads_Model_LeadsFormRulesActionModel::ACTION_TYPE_ASSIGN_LIFECYCLE) {
                        self::assignLeadLifecycleAction($actionConfig, $leadModel);
                    }

                    if ($actionType === Leads_Model_LeadsFormRulesActionModel::ACTION_TYPE_ASSIGN_SEQUENCE) {
                        self::assignLeadSequenceAction($actionConfig, $leadModel);
                    }

                    if ($actionType === Leads_Model_LeadsFormRulesActionModel::ACTION_TYPE_ASSIGN_EMAIL_LIST) {
                        self::assignLeadEmailServiceAction($actionConfig, $leadModel->getId());
                    }

                    if ($actionType === Leads_Model_LeadsFormRulesActionModel::ACTION_TYPE_ASSIGN_EMAIL_COPY) {
                        self::assignLeadEmailCopyAction($actionConfig, $leadModel, $formData, $originalActionType);
                    }

                    if ($actionType === Leads_Model_LeadsFormRulesActionModel::ACTION_TYPE_ASSIGN_REMARKETING_LIST) {
                        self::assignLeadRemarketingServiceAction($actionConfig, array($leadModel->getId()));
                    }
                }
            }
        }
    }

    /**
     * Assign task action
     *
     * @param array $actionConfig action config
     * @param Leads_Model_LeadsModel $leadModel lead Model
     * @return bool
     */
    public static function assignTaskAction($actionConfig = array(), Leads_Model_LeadsModel $leadModel)
    {

        $leadsTaskMapper = Leads_Mapper_LeadsTaskMapper::getInstance();
        $leadsTaskModel = new Leads_Model_LeadsTaskModel();

        if (empty($actionConfig['creator_id']) || empty($actionConfig['user_id'])) {
            return false;
        }

        $creatorId = $actionConfig['creator_id'];
        $userId = $actionConfig['user_id'];
        $userMapper = Application_Model_Mappers_UserMapper::getInstance();
        $creatorUserModel = $userMapper->find($creatorId);
        if (!$creatorUserModel instanceof Application_Model_Models_User) {
            return false;
        }

        $creatorRoleId = $creatorUserModel->getRoleId();

        if (!in_array($creatorRoleId, self::$_allowedAdministrativeRoles)) {
            return false;
        }

        $userModel = $userMapper->find($userId);
        if (!$userModel instanceof Application_Model_Models_User) {
            return false;
        }

        $userRoleId = $userModel->getRoleId();
        if (!in_array($userRoleId, self::$_allowedAdministrativeRoles)) {
            return false;
        }

        $data['creator_id'] = $actionConfig['creator_id'];
        $data['user_id'] = $actionConfig['user_id'];
        $data['notes'] = $actionConfig['notes'];
        $data['title'] = $actionConfig['title'];

        $leadId = $leadModel->getId();
        $leadsData = $leadModel->toArray();
        $organizationData = array();
        $dictionary = array();
        $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
        $leadsData['leadlink'] = $websiteHelper->getUrl().'crmLogin/tabName/dashboard/subTab/leads/paramHashName/lead/paramHash/'.$leadModel->getId().'/';
        $dictionary = Tools_LeadsMailWatchdog::generateDictionary($dictionary, $leadsData, 'lead');
        $dictionary = Tools_LeadsMailWatchdog::generateDictionary($dictionary, ['organizationName' => '', 'orglink' => $leadsData['leadlink']], 'organization');
        if (!empty($leadsData['organizationId'])) {
            $organizationModel = Leads_Mapper_LeadsOrganizationMapper::getInstance()->find($leadsData['organizationId']);
            if ($organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                $organizationData = $organizationModel->toArray();
                $organizationData['orglink'] = $websiteHelper->getUrl() . 'crmLogin/tabName/dashboard/subTab/organizations/paramHashName/organization/paramHash/' . $organizationModel->getId() . '/';
                $dictionary = Tools_LeadsMailWatchdog::generateDictionary($dictionary, $organizationData, 'organization');
            }
        }

        $data['title'] = Tools_LeadsMailWatchdog::parseEmailLexems($data['title'], $dictionary);
        $data['notes'] = Tools_LeadsMailWatchdog::parseEmailLexems($data['notes'], $dictionary);
        $data['notes'] = Tools_LeadTools::parseLexemsAndContent($data['notes'], $leadModel);

        $data['lead_id'] = $leadId;

        $leadsTaskModel->setOptions($data);

        $leadTaskModel = $leadsTaskMapper->save($leadsTaskModel);

        $createdAt = Tools_System_Tools::convertDateFromTimezone('now');

        $actionLogExpiredModel = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_TASK, $leadId,
            Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND, $createdAt,
            Leads_Model_LeadsTypeModel::LEAD_OUTBOUND_TASK);

        if (!empty($actionConfig['taskImmediate'])) {
            $expiredAt = Tools_System_Tools::convertDateFromTimezone('now');
        } else {
            $shoppingConfigMapper =  Models_Mapper_ShoppingConfig::getInstance();
            $taskDelayUnits = $actionConfig['taskDelayUnits'];
            $storeTimezone = $shoppingConfigMapper->getConfigParam('timezone');
            if (!empty($actionConfig['taskTimezone'])) {
                $timezone = $actionConfig['taskTimezone'];
            } elseif (!empty($storeTimezone)) {
                $timezone = $storeTimezone;
            } else {
                $timezone = 'UTC';
            }
            if ($taskDelayUnits !== 'hour') {
                $expiredAtDate = date('Y-m-d',
                        strtotime('now' . '+' . $actionConfig['taskDelay'] . $actionConfig['taskDelayUnits'])) . ' ' . $actionConfig['assignTaskTime'];
                $hoursShift = Tools_EmailSequenceTools::getTimezoneShift('UTC', $timezone);
                if ($hoursShift > 0) {
                    $hoursShift = ' -' . $hoursShift;
                } else {
                    $hoursShift = ' +' . abs($hoursShift);
                }
                $expiredAt = date(Tools_System_Tools::DATE_MYSQL, strtotime($expiredAtDate . $hoursShift . 'hours'));
            } else {
                $expiredAt = date('Y-m-d H:i:s',
                        strtotime($createdAt . '+' . $actionConfig['taskDelay'] . $actionConfig['taskDelayUnits']));
            }
        }

        if (!empty($actionConfig['assignTaskDueTime'])) {
            $shoppingConfigMapper =  Models_Mapper_ShoppingConfig::getInstance();
            $taskDelayUnits = $actionConfig['taskDelayDueUnits'];
            $storeTimezone = $shoppingConfigMapper->getConfigParam('timezone');
            if (!empty($actionConfig['taskTimezone'])) {
                $timezone = $actionConfig['taskTimezone'];
            } elseif (!empty($storeTimezone)) {
                $timezone = $storeTimezone;
            } else {
                $timezone = 'UTC';
            }
            if ($taskDelayUnits !== 'hour') {
                $dueAtDate = date('Y-m-d',
                        strtotime('now' . '+' . $actionConfig['taskDueDelay'] . $actionConfig['taskDelayDueUnits'])) . ' ' . $actionConfig['assignDueTime'];
                $hoursShift = Tools_EmailSequenceTools::getTimezoneShift('UTC', $timezone);
                if ($hoursShift > 0) {
                    $hoursShift = ' -' . $hoursShift;
                } else {
                    $hoursShift = ' +' . abs($hoursShift);
                }
                $dueAt = date(Tools_System_Tools::DATE_MYSQL, strtotime($dueAtDate . $hoursShift . 'hours'));
            } else {
                $dueAt = date('Y-m-d H:i:s',
                    strtotime($createdAt . '+' . $actionConfig['taskDueDelay'] . $actionConfig['taskDelayDueUnits']));
            }
            $data['due_date'] = $dueAt;
            $leadTaskModel->setDueDate($data['due_date']);
        }

        $data['expired_at'] = $expiredAt;
        $leadTaskModel->setExpiredAt($data['expired_at']);
        $leadTaskModel = $leadsTaskMapper->save($leadsTaskModel);

        $actionLogUpcomingModel = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_TASK, $leadId,
            Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND, $expiredAt,
            Leads_Model_LeadsTypeModel::LEAD_OUTBOUND_TASK, Leads_Model_LeadsActionLogModel::EVENT_LOG_UPCOMING);

        $leadTaskId = $leadTaskModel->getId();

        $leadsTaskLogMapper = Leads_Mapper_LeadsTasksLogMapper::getInstance();
        $leadsTaskLogModel = new Leads_Model_LeadsTasksLogModel();
        $leadsTaskLogModel->setOptions($data);
        $leadsTaskLogModel->setActionLogId($actionLogExpiredModel->getId());
        $leadsTaskLogModel->setTaskId($leadTaskId);
        $leadsTaskLogModel->setLeadId($leadId);
        $leadsTaskLogModel->setDueDate($data['due_date']);
        $leadsTaskLogModel->setIsTaskOverdue('0');
        $leadsTaskLogMapper->save($leadsTaskLogModel);


        $leadsTaskLogModel = new Leads_Model_LeadsTasksLogModel();
        $leadsTaskLogModel->setOptions($data);
        $leadsTaskLogModel->setActionLogId($actionLogUpcomingModel->getId());
        $leadsTaskLogModel->setTaskId($leadTaskId);
        $leadsTaskLogModel->setLeadId($leadId);
        $leadsTaskLogModel->setDueDate($data['due_date']);
        $leadsTaskLogModel->setIsTaskOverdue('0');
        $leadsTaskLogMapper->save($leadsTaskLogModel);

        if (!empty($actionConfig['taskImmediate'])) {
            $leadTaskModel = Leads_Mapper_LeadsTaskMapper::getInstance()->find($leadTaskId);
            if ($leadTaskModel instanceof Leads_Model_LeadsTaskModel) {
                $leadTaskModel->registerObserver(new Tools_Mail_Watchdog(array(
                    'trigger' => Tools_LeadsMailWatchdog::LEADS_TASK_CREATED,
                    'leadData' => $leadsData,
                    'organizationData' => $organizationData
                )));

                $leadTaskModel->notifyObservers();
            }
        }

//        if (!empty($actionConfig['taskImmediate'])) {
//            $leadTaskQueueMapper = Leads_Mapper_LeadsTaskQueueMapper::getInstance();
//            $taskReminderInQueueModel = $leadTaskQueueMapper->findByTaskIdQueueType($leadTaskId, Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_TYPE_FIRST_REMINDER);
//            if (!$taskReminderInQueueModel instanceof Leads_Model_LeadsTaskQueueModel) {
//                $taskReminderInQueueModel = new Leads_Model_LeadsTaskQueueModel();
//                $taskReminderInQueueModel->setStatus(Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_STATUS_SENT);
//                $taskReminderInQueueModel->setQueueType(Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_TYPE_FIRST_REMINDER);
//                $taskReminderInQueueModel->setTaskId($leadTaskId);
//                $taskReminderInQueueModel->setToSendDate($expiredAt);
//                $taskReminderInQueueModel->setSentDate($expiredAt);
//                $leadTaskQueueMapper->save($taskReminderInQueueModel);
//            }
//            $taskReminderInQueueModel = $leadTaskQueueMapper->findByTaskIdQueueType($leadTaskId, Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_TYPE_SECOND_REMINDER);
//            if (!$taskReminderInQueueModel instanceof Leads_Model_LeadsTaskQueueModel) {
//                $taskReminderInQueueModel = new Leads_Model_LeadsTaskQueueModel();
//                $taskReminderInQueueModel->setStatus('processing');
//                $taskReminderInQueueModel->setQueueType(Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_TYPE_SECOND_REMINDER);
//                $taskReminderInQueueModel->setTaskId($leadTaskId);
//                $taskReminderInQueueModel->setToSendDate($expiredAt);
//                $taskReminderInQueueModel->setSentDate($expiredAt);
//                $taskReminderInQueueModel = $leadTaskQueueMapper->save($taskReminderInQueueModel);
//            }
//
//            $leadTaskModel = Leads_Mapper_LeadsTaskMapper::getInstance()->find($leadTaskId);
//            if ($leadTaskModel instanceof Leads_Model_LeadsTaskModel) {
//                $leadTaskModel->registerObserver(new Tools_Mail_Watchdog(array(
//                    'trigger' => Tools_LeadsMailWatchdog::LEADS_TASK_REMINDER,
//                    'queueId' => $taskReminderInQueueModel->getId(),
//                    'leadData' => $leadsData,
//                    'organizationData' => $organizationData
//                )));
//
//                $leadTaskModel->notifyObservers();
//            }
//        }

        return true;
    }

    /**
     * Assign lead owner
     *
     * @param array $actionConfig action config
     * @param Leads_Model_LeadsModel $leadModel lead Model
     */
    public static function assignLeadOwnerAction($actionConfig = array(), Leads_Model_LeadsModel $leadModel)
    {
        $assignToId = $actionConfig['assign_to'];
        $userModel = Application_Model_Mappers_UserMapper::getInstance()->find($assignToId);
        if ($userModel instanceof Application_Model_Models_User) {
            $roleId = $userModel->getRoleId();
            if (in_array($roleId, self::$_allowedAdministrativeRoles)) {
                $leadModel->setOwnerId($assignToId);
                Leads_Mapper_LeadsMapper::getInstance()->save($leadModel);
            }
        }
    }

    /**
     * Assign lead tags
     *
     * @param array $actionConfig action config
     * @param Leads_Model_LeadsModel $leadModel lead Model
     */
    public static function assignLeadTagsAction($actionConfig = array(), Leads_Model_LeadsModel $leadModel)
    {
        if (!empty($actionConfig['tagIds'])) {
            $tagMapper = Leads_Mapper_LeadsTagsMapper::getInstance();

            $originalLeadTags = $tagMapper->findTagsByLeadId($leadModel->getId());
            if (empty($originalLeadTags)) {
                $originalLeadTags = array();
            }
            $assignLeadTags = $actionConfig['tagIds'];
            $leadTags = array_unique(array_merge($originalLeadTags, $assignLeadTags));
            $leadModel->setLeadTags($leadTags);
            Leads_Mapper_LeadsMapper::getInstance()->save($leadModel);
        }
    }

    /**
     * Assign lead email sequence
     *
     * @param array $actionConfig
     * @param Leads_Model_LeadsModel $leadModel
     * @throws Exception
     */
    public static function assignLeadSequenceAction($actionConfig = array(), Leads_Model_LeadsModel $leadModel)
    {
        if (!empty($actionConfig['sequenceId'])) {
            $leadId = $leadModel->getId();
            $leadEmailHasSequenceMapper = Leads_Mapper_LeadsHasEmailSequenceMapper::getInstance();
            $emailSequenceSettingsMapper = Emailsequence_Models_Mapper_EmailSequenceSettingsMapper::getInstance();
            $sequenceModel = $emailSequenceSettingsMapper->find($actionConfig['sequenceId']);

            if ($sequenceModel instanceof Emailsequence_Models_Models_EmailSequenceSettingsModel && !empty($leadId)) {
                $archiveSequence = $sequenceModel->getArchive();

                if(!$archiveSequence) {
                    $leadEmailHasModel = $leadEmailHasSequenceMapper->findByLeadIdSequenceId($leadId, $sequenceModel->getId());

                    if (!empty($actionConfig['unassignSequence'])) {
                        $leadHasSequences = $leadEmailHasSequenceMapper->getLeadSequenceByLeadId($leadId);
                        if (!empty($leadHasSequences)) {
                            foreach ($leadHasSequences as $leadHasSequence) {
                                $sequenceId = $leadHasSequence['sequence_id'];
                                $leadEmailHasSequenceMapper->delete($leadId, $sequenceId);
                                $notSendEmailsInQueue = $leadEmailHasSequenceMapper->getFromCartStatusQueue($leadId, 'lead',
                                    $sequenceId, array('0'));
                                if (!empty($notSendEmailsInQueue)) {
                                    $leadEmailSequenceStatusMapper = Leads_Mapper_LeadsEmailSequenceStatusMapper::getInstance();
                                    $leadEmailSequenceStatusMapper->deleteByLeadIdSequenceId($leadId, $sequenceId);
                                    $leadEmailHasSequenceMapper->deleteFromCartStatusQueue($leadId, 'lead', $sequenceId,
                                        '0');
                                }
                            }
                        }
                    }

                    if (!$leadEmailHasModel instanceof Leads_Model_LeadsHasEmailSequenceModel) {
                        $sequenceAppliedDate = date(Tools_System_Tools::DATE_MYSQL, strtotime('now'));

                        $leadEmailHasModel = new Leads_Model_LeadsHasEmailSequenceModel();
                        $leadEmailHasModel->setSequenceId($sequenceModel->getId());
                        $leadEmailHasModel->setLeadId($leadId);
                        $leadEmailHasModel->setSequenceDate($sequenceAppliedDate);

                        $leadEmailHasSequenceMapper->save($leadEmailHasModel);
                    }
                }
            }
        }
    }

    /**
     * Insert Ignore sequences into plugin_leads_has_email_sequence Dbtable
     *
     * @param $sequenceId
     * @param array $leadIds
     * @return bool
     */
    public static function assignLeadSequenceMassAction($sequenceId, array $leadIds = array())
    {
        if(!empty($sequenceId) && !empty($leadIds)){
            $dbAdapter = Zend_Db_Table::getDefaultAdapter();

            $sql = "INSERT IGNORE INTO plugin_leads_has_email_sequence (lead_id, sequence_id, sequence_date) VALUES (:lead_id, :sequence_id, :sequence_date)";

            foreach ($leadIds as $leadId) {
                $sequenceAppliedDate = date(Tools_System_Tools::DATE_MYSQL, strtotime('now'));
                $dbAdapter->query($sql, array('lead_id' => $leadId, 'sequence_id' => $sequenceId, 'sequence_date' => $sequenceAppliedDate));
            }

            return true;
        }else{
            return false;
        }
    }

    /**
     *
     *
     * @param $sequenceId
     * @param array $leadIds
     * @return bool
     */
    public static function unAssignLeadSequenceMassAction($sequenceId, array $leadIds = array())
    {
        if(!empty($sequenceId) && !empty($leadIds)){
            $leadsHasEmailSequenceMapper = Leads_Mapper_LeadsHasEmailSequenceMapper::getInstance();

            foreach ($leadIds as $leadId) {
                $leadHasSequenceModel = $leadsHasEmailSequenceMapper->getLeadSequence($leadId, $sequenceId);

                if ($leadHasSequenceModel instanceof Leads_Model_LeadsHasEmailSequenceModel) {
                    $leadsHasEmailSequenceMapper->delete($leadId, $sequenceId);
                    $notSendEmailsInQueue = $leadsHasEmailSequenceMapper->getFromCartStatusQueue($leadId, 'lead', $sequenceId, array('0'));
                    if (!empty($notSendEmailsInQueue)) {
                        $leadEmailSequenceStatusMapper = Leads_Mapper_LeadsEmailSequenceStatusMapper::getInstance();
                        $leadEmailSequenceStatusMapper->deleteByLeadIdSequenceId($leadId, $sequenceId);
                        $leadsHasEmailSequenceMapper->deleteFromCartStatusQueue($leadId, 'lead', $sequenceId, '0');
                    }
                }
            }

            return true;
        }else{
            return false;
        }
    }

    /**
     * Assign lead email service
     *
     * @param array $actionConfig
     * @param $leadId
     * @throws Exceptions_SeotoasterPluginException
     */
    public static function assignLeadEmailServiceAction($actionConfig = array(), $leadId)
    {
        if(!empty($actionConfig) && !empty($leadId)) {
            $emailServiceName = $actionConfig['serviceName'];

            if(!empty($actionConfig['type']) && !empty($emailServiceName)) {
                $data['services'] = array($emailServiceName => array('type' => $actionConfig['type']));

                $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();

                $serviceInfo = Apps_Models_Mapper_AppsSettingsMapper::getInstance()->getByServiceName($actionConfig['serviceName']);
                if(!empty($serviceInfo)) {
                    $detailedInfo = $serviceInfo[0]->getDetailedInfo();

                    if (($actionConfig['type'] === Apps::SERVICE_TYPE_EMAIL) && !$detailedInfo) {
                        $leadsData = $leadsMapper->findByLeadIdsForEmailService(array($leadId));
                    } else {
                        $leadsData = $leadsMapper->getExtendedLeadInfoForEmailService(array($leadId));
                    }

                    if (!empty($leadsData)) {
                        $mojoClientAction = $emailServiceName . 'Client';

                        $data['services'][$emailServiceName]['clients'] = $leadsData;

                        $emailLists = array();
                        if(!empty($actionConfig['lists'])) {
                            $emailLists = explode(',', $actionConfig['lists']);
                        }
                        $data['services'][$emailServiceName]['lists'] = $emailLists;

                        if($actionConfig['requestType'] == 'unassign') {
                            Apps::apiCall('DELETE', 'apps', array($mojoClientAction), $data);
                        } else {
                            Apps::apiCall('POST', 'apps', array($mojoClientAction), $data);
                        }

                        if (isset($actionConfig['listsLabels'])) {
                            Tools_LeadScoreWorkflowTools::assignListNameTags([$leadId], $actionConfig['listsLabels']);
                        }
                    }
                }
            }
        }
    }

    /**
     * Send copy of the system form email
     *
     * @param array $actionConfig actions config
     * @param Leads_Model_LeadsModel $leadModel lead model
     * @param array $formData form data
     * @param string $actionType action type type_form|type_event
     * @throws Exceptions_SeotoasterException
     * @throws Zend_Exception
     * @throws Zend_File_Transfer_Exception
     */
    public static function assignLeadEmailCopyAction(
        $actionConfig = array(),
        Leads_Model_LeadsModel $leadModel,
        $formData,
        $actionType
    ) {
        if (!empty($formData) && !empty($actionConfig['emailsToCopy'])) {
            if ($actionConfig['emailsToCopy'] === '{leadowner:email}') {
                $leadOwnerModel = Application_Model_Mappers_UserMapper::getInstance()->find($leadModel->getOwnerId());
                if ($leadOwnerModel instanceof Application_Model_Models_User) {
                    $actionConfig['emailsToCopy'] = $leadOwnerModel->getEmail();
                }
            }
            $emailsToCopy = array_unique(array_filter(explode(',', $actionConfig['emailsToCopy'])));
            if ($actionType === self::LEAD_RULES_TYPE_FORM) {
                $formName = $formData['formName'];
                $formModel = Application_Model_Mappers_FormMapper::getInstance()->findByName($formName);
                $emailsToCopyFiltered = array();
                foreach ($emailsToCopy as $email) {
                    $emailValidator = new Tools_System_CustomEmailValidator();
                    if ($emailValidator->isValid($email)) {
                        $emailsToCopyFiltered[] = $email;
                    }
                }

                if (!empty($formData['externalFormSignature'])) {
                    $leadsFormBuilderGeneralConfigMapper = Leads_Mapper_LeadsBuilderFormGeneralConfigMapper::getInstance();
                    $leadsFormBuilderGeneralConfigModel = $leadsFormBuilderGeneralConfigMapper->findBySignature($formData['externalFormSignature']);
                    if (!$leadsFormBuilderGeneralConfigModel instanceof Leads_Model_LeadsBuilderFormGeneralConfigModel) {
                        return '';
                    }

                    Tools_LeadExternalFormTools::sendEmails($leadsFormBuilderGeneralConfigModel, $formData, $emailsToCopyFiltered, true);
                } else {
                    $xmlHttpRequest = Zend_Controller_Front::getInstance()->getRequest()->isXmlHttpRequest();
                    $options = array();
                    $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                    if (!$xmlHttpRequest) {
                        //Adding attachments to email
                        $websitePathTemp = $websiteHelper->getPath() . $websiteHelper->getTmp();
                        $uploader = new Zend_File_Transfer_Adapter_Http();
                        $uploader->setDestination($websitePathTemp);
                        $files = $uploader->getFileInfo();
                        if (!empty($files)) {
                            foreach ($files as $file => $fileInfo) {
                                if ($fileInfo['name'] != '') {
                                    $at = new Zend_Mime_Part(file_get_contents($uploader->getFileName($file)));
                                    $fInfo = finfo_open(FILEINFO_MIME_TYPE);
                                    $mime = finfo_file($fInfo, $uploader->getFileName($file));
                                    $at->type = $mime;
                                    $at->disposition = Zend_Mime::DISPOSITION_ATTACHMENT;
                                    $at->encoding = Zend_Mime::ENCODING_BASE64;
                                    $at->filename = $fileInfo['name'];
                                    $attachment[] = $at;
                                    unset($at);
                                }
                            }
                            if (!empty($attachment)) {
                                $options['attachment'] = $attachment;
                            }
                        }
                    }

                    $formModel->setAdminFrom(self::parseData($formModel->getAdminFrom()));
                    $formModel->setAdminSubject(html_entity_decode(self::parseData($formModel->getAdminSubject()), null,
                        'UTF-8'));
                    $formModel->setAdminFromName(html_entity_decode(self::parseData($formModel->getAdminFromName()),
                        null,
                        'UTF-8'));
                    $formModel->setAdminText(self::parseData($formModel->getAdminText()));
                    $formModel->setReplyText(self::parseData($formModel->getReplyText()));
                    $formModel->setContactEmail(self::parseData($formModel->getContactEmail()));
                    $formModel->setReplyFrom(self::parseData($formModel->getReplyFrom()));
                    $formModel->setReplyFromName(html_entity_decode(self::parseData($formModel->getReplyFromName()),
                        null,
                        'UTF-8'));
                    $formModel->setReplySubject(html_entity_decode(self::parseData($formModel->getReplySubject()), null,
                        'UTF-8'));
                    $formModel->setMobile(self::parseData($formModel->getMobile()));


                    $adminTemplateEmail = $formModel->getAdminMailTemplate();

                    if (!empty($adminTemplateEmail)) {
                        $options['template'] = $adminTemplateEmail;
                    }
                    $options['trigger'] = 'SystemEmailCopyForm';
                    $options['data'] = $formData;
                    $options['emailsTo'] = $emailsToCopyFiltered;
                    $options['withoutPreparseTemplate'] = true;

                    $systemMailWatchdog = new Tools_LeadsMailWatchdog($options);
                    $systemMailWatchdog->notify($formModel);
                }
            }

            if ($actionType === self::LEAD_RULES_TYPE_EVENT) {
                if (!empty($emailsToCopy) && !empty($actionConfig['emailTemplate']) && !empty($actionConfig['emailSubject'])
                    && !empty($actionConfig['emailSubject']) && !empty($actionConfig['emailFrom'])) {

                    foreach ($emailsToCopy as $email) {
                        $emailValidator = new Tools_System_CustomEmailValidator();
                        if ($emailValidator->isValid($email)) {
                            $emailsToCopyFiltered[] = $email;
                        }
                    }

                    if (!empty($emailsToCopyFiltered)) {
                        $emailTemplate = Application_Model_Mappers_TemplateMapper::getInstance()->find($actionConfig['emailTemplate']);
                        if ($emailTemplate instanceof Application_Model_Models_Template) {
                            $leadsData = $leadModel->toArray();
                            $organizationData = array();
                            if (!empty($leadsData['organizationId'])) {
                                $organizationModel = Leads_Mapper_LeadsOrganizationMapper::getInstance()->find($leadsData['organizationId']);
                                if ($organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                                    $organizationData = $organizationModel->toArray();
                                }
                            }
                            if ($actionConfig['emailFrom'] === '{leadowner:email}') {
                                $leadOwnerModel = Application_Model_Mappers_UserMapper::getInstance()->find($leadModel->getOwnerId());
                                if ($leadOwnerModel instanceof Application_Model_Models_User) {
                                    $actionConfig['emailFrom'] = $leadOwnerModel->getEmail();
                                }
                            }
                            $options = array();
                            $options['template'] = $actionConfig['emailTemplate'];
                            $options['trigger'] = 'EventEmailCopy';
                            $options['data'] = $formData;
                            $options['emailsTo'] = $emailsToCopyFiltered;
                            $options['withoutPreparseTemplate'] = true;
                            $options['emailFrom'] = $actionConfig['emailFrom'];
                            $options['emailSubject'] = $actionConfig['emailSubject'];
                            $options['subject'] = $actionConfig['emailSubject'];
                            $options['withoutPreparseTemplate'] = true;
                            $options['leadData'] = $leadsData;
                            $options['organizationData'] = $organizationData;
                            if (!empty($actionConfig['emailMessage'])) {
                                $options['message'] = $actionConfig['emailMessage'];
                            }

                            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                            $link = $websiteHelper->getUrl().'crmLogin/tabName/dashboard/subTab/leads/paramHashName/lead/paramHash/'.$leadModel->getId().'/';
                            $link = "<a href=".$link.">link</a>";
                            $options['leadData']['leadlink'] = $link;
                            if (!empty($organizationData) && $organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                                $orgLink = $websiteHelper->getUrl() . 'crmLogin/tabName/dashboard/subTab/organizations/paramHashName/organization/paramHash/' . $organizationModel->getId() . '/';
                                $orgLink = "<a href=" . $orgLink . ">organization</a>";
                                $options['organizationData']['orglink'] = $orgLink;
                            }

                            $formModel = new Application_Model_Models_Form();
                            $systemMailWatchdog = new Tools_LeadsMailWatchdog($options);
                            $systemMailWatchdog->notify($formModel);
                        }
                    }

                }

            }

        }

    }

    /**
     * * Assign lead Ad audience lists
     *
     * @param array $actionConfig
     * @param $leadId
     */
    public static function assignLeadRemarketingServiceAction($actionConfig = array(), array $leadIds)
    {
        if(!empty($actionConfig) && !empty($leadIds)) {
            $remarketingName = $actionConfig['serviceName'];

            if(!empty($remarketingName) && !empty($actionConfig['lists'])) {
                $lists = json_decode($actionConfig['lists'], true);
                $listsLabels = json_decode($actionConfig['listsLabels'], true);

                if(!empty($lists)) {
                    $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();

                    $leadsData = $leadsMapper->getExtendedLeadInfoForRemarketingLists($leadIds);

                    if (!empty($leadsData)) {
                        $existedLeadsIds = array();
                        foreach ($leadsData as $key => $leadData) {
                            $leadData['phone'] = '';
                            if(!empty($leadData['mobileNumberCode']) && !empty($leadData['mobileNumber'])) {
                                $leadData['phone'] = $leadData['mobileNumberCode'] . $leadData['mobileNumber'];
                            } elseif (!empty($leadData['phoneNumberCode']) && !empty($leadData['phoneNumber'])) {
                                $leadData['phone'] = $leadData['phoneNumberCode'] . $leadData['phoneNumber'];
                            }

                            $leadsData[$key] = $leadData;

                            if(in_array($leadData['id'], $leadIds)) {
                                $existedLeadsIds[] = $leadData['id'];
                            }
                        }

                        foreach ($lists as $marketingCampaignsName => $listsIds) {
                            if(!empty($listsIds)) {
                                $data['type'] = $marketingCampaignsName;
                                $data['audienceId'] = $listsIds;
                                $data['listContent'] = $leadsData;
                                $data['remove'] = false;

                                if($actionConfig['requestType'] == 'unassign') {
                                    $data['remove'] = true;
                                }

                                Tools_LeadRemarketingAudiencesTools::prepareAndSaveData($data['type'], $data['audienceId'], $data['remove'], $data['listContent']);

                                /*$response = Apps::apiCall('POST', 'sambastatsAddDataToRemarketingList', array(), $data);

                                if(!empty($response['errorList'])) {
                                    foreach ($response['errorList'] as $key => $error) {
                                        $errorListKey = array_search($key, $lists[$marketingCampaignsName]);
                                        unset($listsLabels[$marketingCampaignsName][$errorListKey]);
                                    }
                                }*/

                                if(!empty($listsLabels[$marketingCampaignsName])) {
                                    $leadTagsMapper = Leads_Mapper_LeadsTagsMapper::getInstance();

                                    foreach ($listsLabels[$marketingCampaignsName] as $listLabelKey => $listLabel) {
                                        $adsPrefix = 'Added to Google Audience';
                                        if($marketingCampaignsName == 'facebook') {
                                            $adsPrefix = 'Added to FB Audience';
                                        }

                                        $tag = $adsPrefix . ' ' . $listLabel .' - '. $lists[$marketingCampaignsName][$listLabelKey];//Added to Google Audience SMB CRM Leads - 243575394539

                                        $leadTagModel = $leadTagsMapper->getTag($tag);
                                        if (!$leadTagModel instanceof Leads_Model_LeadsTagModel) {
                                            $leadTagModel = new Leads_Model_LeadsTagModel();
                                            $leadTagModel->setLabel($tag);
                                            $leadTagModel->setTagName($tag);
                                            $leadTagModel = $leadTagsMapper->save($leadTagModel);
                                        }

                                        foreach ($existedLeadsIds as $leadId) {
                                            $leadsMapper->addTag($leadId, $leadTagModel->getId());
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * Parse widgets for template
     *
     * @param string $data
     * @return null
     * @throws Zend_Exception
     */
    public static function parseData($data)
    {
        $themeData = Zend_Registry::get('theme');
        $extConfig = Zend_Registry::get('extConfig');
        $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
        $parserOptions = array(
            'websiteUrl' => $websiteHelper->getUrl(),
            'websitePath' => $websiteHelper->getPath(),
            'currentTheme' => $extConfig['currentTheme'],
            'themePath' => Tools_Filesystem_Tools::cleanWinPath($themeData['path']),
        );
        $parser = new Tools_Content_Parser($data, array(), $parserOptions);

        return $parser->parseSimple();
    }

    /**
     * Assign lead lifecycle
     *
     * @param array $actionConfig action config
     * @param Leads_Model_LeadsModel $leadModel lead Model
     */
    public static function assignLeadLifecycleAction($actionConfig = array(), Leads_Model_LeadsModel $leadModel)
    {
        if (!empty($actionConfig['lifecycleId'])) {
            $leadModel->setStageId($actionConfig['lifecycleId']);
            Leads_Mapper_LeadsMapper::getInstance()->save($leadModel);
        }
    }

    public static function getDefaultFieldsList()
    {
        return array(
            'lead_first_name' => 'Lead first name',
            'lead_email' => 'Lead email',
            'lead_phone' => 'Lead phone',
            'lead_source' => 'Lead source',
            'lead_last_name' => 'Lead lastname',
            'lead_zip' => 'Lead zip',
            'lead_address' => 'Lead address',
            'lead_mobile' => 'Lead mobile',
            'lead_position' => 'Lead position',
            'lead_country_code' => 'Lead country code',
            'lead_state_code' => 'Lead state country code',
            'middle_name' => 'Lead middle name',
            'prefix' => 'Lead prefix',
            'lead_phone_country_code' => 'Lead phone country code',
            'lead_mobile_country_code' => 'Lead mobile country code',
            'lead_address1' => 'Lead address1',
            'lead_city' => 'Lead city',
            'lead_tags' => 'Lead tag',
            'organization_name' => 'Organization name',
            'organization_address' => 'Organization address',
            'organization_city' => 'Organization city',
            'organization_zip' => 'Organization zip',
            'organization_country_code' => 'Organization country code',
            'organization_website' => 'Organization website',
            'organization_website1' => 'Organization website 2',
            'organization_phone' => 'Organization phone',
            'organization_email' => 'Organization email',
            'organization_employees' => 'Organization employees',
            'organization_state_code' => 'Organization state code',
            'organization_phone_country_code' => 'Organization phone country code',
            'organization_mobile' => 'Organization mobile',
            'organization_mobile_country_code' => 'Organization mobile country code',
            'organization_address1' => 'Organization address1',
            'organization_industry_type' => 'Organization industry type'
        );
    }

}
