<?php

class Tools_LeadFormWatchdog implements Interfaces_Observer
{

    public function notify($formModel)
    {

        $externalForm = false;
        if (is_array($formModel) && !empty($formModel['email'])) {
            $externalForm = true;
        }

        if ($formModel instanceof Application_Model_Models_Form || $externalForm === true) {
            if ($externalForm === true) {
                $formParams = $formModel;
            } else {
                $formParams = Zend_Controller_Front::getInstance()->getRequest()->getParams();
            }
            if (!empty($formParams['email'])) {
                $leadEmail = $formParams['email'];
                $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
                if (!Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
                    $ip = Tools_LeadTools::getIpAddress();
                }
                $formParams = filter_var_array($formParams, FILTER_SANITIZE_STRING);
                $firstName = Tools_LeadTools::getValue($formParams, 'name');
                if (empty($firstName)) {
                    $firstName = 'no name';
                }
                $lastName = Tools_LeadTools::getValue($formParams, 'lastname');
                $mobile = Tools_LeadTools::getValue($formParams, 'mobile');
                $phone = Tools_LeadTools::getValue($formParams, 'phone');
                $zip = Tools_LeadTools::getValue($formParams, 'zip');
                $address = Tools_LeadTools::getValue($formParams, 'address');
                $address1 = Tools_LeadTools::getValue($formParams, 'address1');
                $position = Tools_LeadTools::getValue($formParams, 'position');
                $prefix = Tools_LeadTools::getValue($formParams, 'prefix');
                $city = Tools_LeadTools::getValue($formParams, 'city');
                $countryCode = Tools_LeadTools::getValue($formParams,'country');
                $stateCode = Tools_LeadTools::getValue($formParams,'state');
                $organization = Tools_LeadTools::getValue($formParams,'organization');
                $employees = filter_var(Zend_Controller_Front::getInstance()->getRequest()->getParam('employees'), FILTER_SANITIZE_NUMBER_INT);
                $groupName = Tools_LeadTools::getValue($formParams,'group-name');
                $tagNames = Tools_LeadTools::getValue($formParams,'tag-name');
                if (!is_array($tagNames)) {
                    $tagNames = array_filter(explode(',' , $tagNames));
                }
                $mobileCountryCode =  Tools_LeadTools::getValue($formParams,'mobile-country-code');
                $phoneCountryCode =  Tools_LeadTools::getValue($formParams,'phone-country-code');
                $website =  Tools_LeadTools::getValue($formParams, 'website');
                $session = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
                $currentLoggedUserId = $session->getCurrentUser()->getId();
                $currentLoggedUserEmail =  $session->getCurrentUser()->getEmail();

                $leadModel = Tools_LeadTools::getLeadModelByIdAndEmail($currentLoggedUserId, $currentLoggedUserEmail, $leadEmail);
                $ownerId = Tools_LeadTools::getDefaultLeadOwner();
                $cleanedFormParams = Tools_LeadTools::cleanFormData($formParams);
                $filteredFormParams = $cleanedFormParams;
                $formDetails = Tools_LeadTools::prepareFormDetailsHtml($cleanedFormParams);
                $createdAt = Tools_System_Tools::convertDateFromTimezone('now');
                $leadType = Leads_Model_LeadsTypeModel::LEAD_INBOUND_FORM;
                $organizationModel = null;
                $mobileCountryCodeValue = '';
                $phoneCountryCodeValue = '';
                $withTags = false;

                if (!empty($groupName)) {
                    $groupMapper = Store_Mapper_GroupMapper::getInstance();
                    $whereGroup = $groupMapper->getDbTable()->getAdapter()->quoteInto('groupName = ?', $groupName);
                    $groupExists = $groupMapper->fetchAll($whereGroup);
                    if (!empty($groupExists)) {
                        $groupId = $groupExists[0]->getId();
                    }
                }

                if (!empty($tagNames)) {
                    $withTags = true;
                }

                $countryCodes = Tools_Geo::getCountries(true);
                if (!empty($countryCode)) {
                    if (!array_key_exists($countryCode, $countryCodes)) {
                        $countryCode = '';
                    }
                }

                if (empty($mobileCountryCode)) {
                    if (!array_key_exists($mobileCountryCode, $countryCodes)) {
                        $mobileCountryCode = '';
                    }
                }

                if (empty($phoneCountryCode)) {
                    if (!array_key_exists($phoneCountryCode, $countryCodes)) {
                        $phoneCountryCode = '';
                    }
                }

                $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();
                if (!empty($shoppingConfig['country'])) {
                    if (empty($countryCode)) {
                        $countryCode = $shoppingConfig['country'];
                    }
                }

                if (!empty($shoppingConfig['state']) && empty($stateCode)) {
                    $defaultStateId = $shoppingConfig['state'];
                    $allStates = Tools_LeadTools::getStates($countryCode, true, false, 'state');
                    if (array_key_exists($defaultStateId, $allStates)) {
                        $stateCode = $allStates[$defaultStateId];
                    }
                }

                $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
                $userDefaultMobileCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');
                if (empty($mobileCountryCode) && !empty($userDefaultMobileCountryCode)) {
                    $mobileCountryCode = $userDefaultMobileCountryCode;
                }

                if (empty($phoneCountryCode) && !empty($userDefaultMobileCountryCode)) {
                    $phoneCountryCode = $userDefaultMobileCountryCode;
                }

                if (!empty($mobile)) {
                    $mobile = Tools_LeadTools::cleanPhoneAddress($mobile);
                }

                if (!empty($phone)) {
                    $phone = Tools_LeadTools::cleanPhoneAddress($phone);
                }

                if (!empty($mobileCountryCode) && !empty($mobile)) {
                    $mobileNumberExists = Tools_LeadTools::checkLeadMobileOrPhoneNumber($mobile, $mobileCountryCode);
                    $mobileCountryCodeValue = Tools_LeadTools::getCountryCodeValue($mobileCountryCode);
                    if (!empty($mobileNumberExists)) {
                        $mobile = '';
                    }
                }
                if (!empty($phoneCountryCode) && !empty($phone)) {
                    $phoneNumberExists = Tools_LeadTools::checkLeadMobileOrPhoneNumber($phone, $phoneCountryCode);
                    $phoneCountryCodeValue = Tools_LeadTools::getCountryCodeValue($phoneCountryCode);
                    if (!empty($phoneNumberExists)) {
                        $phone = '';
                    }
                }
                $leadModelStatus = Tools_LeadFormTools::LEAD_MODEL_STATUS_OLD;
                if (!$leadModel instanceof Leads_Model_LeadsModel) {
                    $leadModelStatus = Tools_LeadFormTools::LEAD_MODEL_STATUS_NEW;
                    $leadAdditionAvailable = Tools_LeadLicenseTools::checkLimit();
                    if ($leadAdditionAvailable['error'] === true) {
                        $leadModel = new Leads_Model_LeadsModel();
                        $leadModel->registerObserver(new Tools_Mail_Watchdog(array(
                            'trigger' => Tools_LeadsMailWatchdog::LEADS_LICENSE_LIMIT
                        )));

                        $leadModel->notifyObservers();

                        return '';
                    }
                    $leadTypeId = Tools_LeadTools::getLeadTypeIdByLeadTypeName($leadType);
                    $leadModel = new Leads_Model_LeadsModel();
                    $leadModel->setCreatedAt($createdAt);
                    if (!empty($currentLoggedUserEmail)) {
                        $leadModel->setLeadEmail($currentLoggedUserEmail);
                    } else {
                        $leadModel->setLeadEmail($leadEmail);
                    }
                    $leadModel->setLeadFirstName($firstName);
                    $leadModel->setLeadMobileCountryCode($mobileCountryCode);
                    if (!empty($mobile)) {
                        $leadModel->setLeadMobile($mobile);
                        $leadModel->setLeadMobileCountryCodeValue($mobileCountryCodeValue);
                    }
                    $leadModel->setLeadPhoneCountryCode($phoneCountryCode);
                    if (!empty($phone)) {
                        $leadModel->setLeadPhone($phone);
                        $leadModel->setLeadPhoneCountryCodeValue($phoneCountryCodeValue);
                    }
                    if (!empty($groupId)) {
                        $leadModel->setCustomerGroupId($groupId);
                    }
                    if (!empty($prefix)) {
                        $leadModel->setPrefix($prefix);
                    }
                    if (!empty($city)) {
                        $leadModel->setLeadCity($city);
                    }
                    if (!empty($countryCode)) {
                        $leadModel->setLeadCountryCode($countryCode);
                    }
                    if (!empty($stateCode)) {
                        $leadModel->setLeadStateCode($stateCode);
                    }
                    $leadModel->setLeadLastName($lastName);
                    $leadModel->setLeadZip($zip);
                    $leadModel->setLeadAddress($address);
                    $leadModel->setLeadAddress1($address1);
                    $leadModel->setLeadPosition($position);
                    $leadModel->setOwnerId($ownerId);
                    $leadModel->setLeadTypeId($leadTypeId);
                    $leadModel->setLeadSource(Tools_LeadTools::getRefererUrl().' '.Tools_LeadTools::getUtmTagsString());
                } else {
                    $currentFirstName = $leadModel->getLeadFirstName();
                    $currentLeadLastName = $leadModel->getLeadLastName();
                    $currentMobile = $leadModel->getLeadMobile();
                    $currentPhone = $leadModel->getLeadPhone();
                    $currentZip = $leadModel->getLeadZip();
                    $currentAddress = $leadModel->getLeadAddress();
                    $currentPosition = $leadModel->getLeadPosition();
                    $currentPrefix = $leadModel->getPrefix();
                    $currentCity = $leadModel->getLeadCity();
                    $currentCountry = $leadModel->getLeadCountryCode();
                    $currentState = $leadModel->getLeadStateCode();
                    $currentAddress1 = $leadModel->getLeadAddress1();
                    $currentCustomerGroupId = $leadModel->getCustomerGroupId();

                    if (empty($currentFirstName)) {
                        $leadModel->setLeadFirstName($firstName);
                    }
                    if (empty($currentLeadLastName)) {
                        $leadModel->setLeadLastName($lastName);
                    }

                    if (empty($currentCustomerGroupId) && !empty($groupId)) {
                        $leadModel->setCustomerGroupId($groupId);
                    }

                    if (empty($currentMobile) && !empty($mobile)) {
                        $leadModel->setLeadMobile($mobile);
                        $leadModel->setLeadMobileCountryCodeValue($mobileCountryCodeValue);
                        $leadModel->setLeadMobileCountryCode($mobileCountryCode);
                    }
                    if (empty($currentPhone) && !empty($phone)) {
                        $leadModel->setLeadPhone($phone);
                        $leadModel->setLeadPhoneCountryCode($phoneCountryCode);
                        $leadModel->setLeadPhoneCountryCodeValue($phoneCountryCodeValue);
                    }

                    if (empty($currentPrefix) && !empty($prefix)) {
                        $leadModel->setPrefix($prefix);
                    }

                    if (empty($currentCity) && !empty($city)) {
                        $leadModel->setLeadCity($city);
                    }

                    if (empty($currentCountry) && !empty($countryCode)) {
                        $leadModel->setLeadCountryCode($countryCode);
                    }

                    if (empty($currentState) && !empty($stateCode)) {
                        $leadModel->setLeadStateCode($stateCode);
                    }

                    if (empty($currentZip)) {
                        $leadModel->setLeadZip($zip);
                    }

                    if (empty($currentAddress)) {
                        $leadModel->setLeadAddress($address);
                    }

                    if (empty($currentAddress1)) {
                        $leadModel->setLeadAddress1($address1);
                    }

                    if (empty($currentPosition)) {
                        $leadModel->setLeadPosition($position);
                    }

                }

                $oldUserId = $leadModel->getUserId();

                if (empty($oldUserId)) {
                    $leadModel->setUserId($currentLoggedUserId);
                }

                if (!empty($leadModel->getUserId()) && !empty($leadModel->getCustomerGroupId())) {
                    Tools_LeadTools::syncUserGroup($leadModel->getUserId(), $leadModel->getCustomerGroupId());
                }

                $stageId = $leadModel->getStageId();
                if(empty($stageId)) {
                    $leadStageMapper = Leads_Mapper_LeadsStageMapper::getInstance();
                    $leadStageModel = $leadStageMapper->getDefaultStage();
                    if ($leadStageModel instanceof Leads_Model_LeadsStageModel) {
                        $leadStageId = $leadStageModel->getId();
                        $leadModel->setStageId($leadStageId);
                    }
                }


                $leadTypeModel = Leads_Mapper_LeadsTypeMapper::getInstance()->findByTypeName($leadType);
                if ($leadTypeModel instanceof Leads_Model_LeadsTypeModel) {
                    $leadModel->setLastActivityTypeId($leadTypeModel->getId());
                    $leadModel->setLastActivitySubtype('');
                }

                $currentOrganizationId = $leadModel->getOrganizationId();

                $assignPrimaryContact = false;
                if (!empty($organization) && empty($currentOrganizationId)) {
                    if (!empty($organization)) {
                        $assignPrimaryContact = Tools_LeadTools::getOrganizationPrimaryContactStatus($organization);
                    }

                    $organizationModel = Tools_LeadTools::processOrganization($organization);
                    if ($organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                        $leadModel->setOrganizationId($organizationModel->getId());
                    }
                }

                $leadModel->setLastActivity($createdAt);

                if($withTags){
                    $currentLeadId = $leadModel->getId();
                    $tagMapper = Leads_Mapper_LeadsTagsMapper::getInstance();

                    $incomingTags = $tagMapper->getAllIncomingTags($tagNames);
                    if(!empty($incomingTags)){
                        $incomingTags = array_flip($incomingTags);

                        if(!empty($currentLeadId)){
                            $findLeadByTag = $tagMapper->findTagsByLeadId($leadModel->getId());
                            if(!empty($findLeadByTag)){
                                foreach ($incomingTags as $tag){
                                    if(!in_array($tag, $findLeadByTag)){
                                        array_push($findLeadByTag, $tag);
                                    }
                                }
                            }else{
                                $findLeadByTag = $incomingTags;
                            }
                            $leadModel->setLeadTags($findLeadByTag);
                        }else{
                            $leadModel->setLeadTags($incomingTags);
                        }
                    }
                }
                try {
                    $leadModel = $leadsMapper->save($leadModel);
                } catch (Exception $e) {
                    return false;
                }
                $leadId = $leadModel->getId();

                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websitePathTemp = $websiteHelper->getPath().$websiteHelper->getTmp();

                $uploader = new Zend_File_Transfer_Adapter_Http();
                $uploader->setDestination($websitePathTemp);

                $files = $uploader->getFileInfo();

                $filesDataToNotify = array();

                if (!empty($files)) {
                    foreach ($files as $file => $fileInfo) {
                        $storedData = Tools_LeadDocumentsTools::generateStoredName($fileInfo);
                        if (!isset($storedData['error'])) {
                            $savePath = Tools_LeadDocumentsTools::getFilePath($storedData['fileStoredName']);
                            $fInfo = finfo_open(FILEINFO_MIME_TYPE);
                            $mimeType = finfo_file($fInfo, $uploader->getFileName($file));

                            if (!empty($mimeType) && in_array($mimeType,
                                    Tools_LeadDocumentsTools::$_acceptedFilesMimeTypes)) {
                                $file = file_get_contents($uploader->getFileName($file));
                                if (!empty($file)) {
                                    $isUploaded = file_put_contents($savePath, $file);
                                    if ($isUploaded) {
                                        $filesDataToNotifyInfo = array(
                                            'fileExtension' => $storedData['fileExtension'],
                                            'fileName' => $storedData['fileName'],
                                            'file' => base64_encode($file)
                                        );
                                        $filesDataToNotify[] = $filesDataToNotifyInfo;

                                        Tools_LeadDocumentsTools::processDocumentRecord($leadId,
                                            $storedData['fileStoredName'], $storedData['fileHash'],
                                            $storedData['fileName'], $storedData['fileExtension']);
                                    }
                                }
                            }
                        }
                    }
                }

                $leadCustomParams = Tools_LeadFormTools::processFormCustomParams($cleanedFormParams, $leadId);
                $cleanedFormParams = Tools_LeadTools::removeFields($cleanedFormParams, $leadCustomParams);
                $organizationId = $leadModel->getOrganizationId();
                if (!empty($organizationId)) {
                    $organizationCustomParams = Tools_LeadFormTools::processFormCustomParams($cleanedFormParams,
                        $organizationId, Tools_LeadFormTools::ORGANIZATION_TYPE_CUSTOM_FIELD);
                    $cleanedFormParams = Tools_LeadTools::removeFields($cleanedFormParams, $organizationCustomParams);
                }

                $formFieldsNotInUse = Tools_LeadTools::removeUsedFormFields($cleanedFormParams);
                if (!empty($formFieldsNotInUse)) {
                    $notesContent = Tools_LeadTools::prepareFormDetailsHtml($formFieldsNotInUse);
                    Tools_LeadTools::processNotes($leadId, $notesContent);
                }

                if (!empty($organization) && $assignPrimaryContact === true && $organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                    $leadOrganizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
                    $organizationModel->setPrimaryContactId($leadId);
                    $currentOrganizationWebsite = $organizationModel->getOrganizationWebsite();
                    $website = Tools_LeadTools::prepareWebsiteUrl($website);
                    if (empty($currentOrganizationWebsite) && !empty($website)) {
                        $domain = Tools_LeadTools::getDomainFromUrl($website);
                        if (!empty($domain)) {
                            $organizationDomain = $domain;
                        } else {
                            $organizationDomain = '';
                        }
                        if (!empty($domain)) {
                            $organizationModel->setOrganizationDomain($organizationDomain);
                        }
                        $organizationModel->setOrganizationWebsite($website);
                    }
                    $firstContactOrganizationDate = $organizationModel->getFirstContactCreationDate();
                    if (empty($firstContactOrganizationDate)) {
                        $organizationModel->setFirstContactCreationDate($leadModel->getCreatedAt());
                    }
                    if(!empty($employees)) {
                        $organizationModel->setOrganizationEmployees($employees);
                    }

                    $leadOrganizationMapper->save($organizationModel);
                }

                if (!empty($ip)) {
                    $leadsIpsModel = Tools_LeadTools::addIpAddressToLead($leadId, $ip);
                    if ($leadsIpsModel instanceof Leads_Model_LeadsIpsModel) {
                        Tools_LeadTools::addIpToMojo($leadsIpsModel);
                    }
                }

                $leadsActionLogModel = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_FORM, $leadId,
                    Leads_Model_LeadsActionLogModel::ACTION_TYPE_INBOUND, $createdAt, $leadType);

                $formType = Leads_Model_LeadsFormRulesGeneralConfigModel::FORM_TYPE_INTERNAL;
                if ($externalForm === true) {
                    $formType =  Leads_Model_LeadsFormRulesGeneralConfigModel::FORM_TYPE_EXTERNAL;
                }

                $pluginLeadsFormLogMapper = Leads_Mapper_LeadsFormLogMapper::getInstance();
                $pluginLeadsFormLogModel = new Leads_Model_LeadsFormLogModel();
                $pluginLeadsFormLogModel->setActionLogId($leadsActionLogModel->getId());
                $pluginLeadsFormLogModel->setFormName($formParams['formName']);
                $pluginLeadsFormLogModel->setContent($formDetails);
                $pluginLeadsFormLogModel->setFormUrl($formParams['formUrl']);
                $pluginLeadsFormLogModel->setLeadId($leadId);
                $pluginLeadsFormLogModel->setFormType($formType);
                $pluginLeadsFormLogMapper->save($pluginLeadsFormLogModel);

                $formName = $formParams['formName'];
                if (!empty($formName)) {
                    Tools_LeadFormTools::analyzeFormRules($formName,
                        $formType, $formParams, $leadModel, $leadModelStatus);
                }

                if ($externalForm === false) {
                    if (empty($ip)) {
                        $ip = '';
                    }
                    $filteredFormParams['ip'] = $ip;
                    $filteredFormParams['createdAt'] = $createdAt;
                    $filteredFormParams['formName'] = $formName;
                    Apps::apiCall('POST', 'appsSaveLeadFormData', array(), array('data' =>array(
                        'formParams' => $filteredFormParams,
                        'formFilesData' => $filesDataToNotify
                    )),1);
                }

            }
        }
    }

}

