<?php

/**
 * Class Tools_LeadImportTools
 */
class Tools_LeadImportTools
{

    /**
     * Import limit
     */
    const PLUGIN_LEADS_IMPORT_LIMIT = '5';

    /**
     * Leads import db table
     */
    const PLUGIN_LEADS_IMPORT_TABLE = 'plugin_leads_import_data';

    /**
     *  Path to sample file import
     */
    const SAMPLE_FILE_IMPORT_DIR = 'leads/web/sample/';

    /**
     * Sample lead import file name
     */
    const SAMPLE_LEAD_IMPORT_FILE_NAME = 'lead_import_sample_structure.csv';

    /**
     * Sample organizations import file name
     */
    const SAMPLE_ORGANIZATION_IMPORT_FILE_NAME = 'organization_import_sample_structure.csv';

    /**
     * Sample leads options import file name
     */
    const SAMPLE_LEAD_IMPORT_OPTION_FILE_NAME = 'lead_options_import_sample_structure.csv';

    /**
     * Sample leads options import file name
     */
    const SAMPLE_ORGANIZATION_IMPORT_OPTION_FILE_NAME = 'organization_options_import_sample_structure.csv';

    /**
     * Sample lead task import file name
     */
    const SAMPLE_LEAD_TASK_IMPORT_FILE_NAME = 'lead_tasks_import_sample_structure.csv';

    /**
     * Sample contacts data import file name
     */
    const SAMPLE_CONTACTS_DATA_IMPORT_FILE_NAME = 'import_contacts_data_sample_structure.csv';

    /**
     * sample type lead
     */
    const SAMPLE_TYPE_LEAD = 'lead';

    /**
     * sample type organization
     */
    const SAMPLE_TYPE_ORGANIZATION = 'organization';

    /**
     * sample type lead
     */
    const SAMPLE_TYPE_LEAD_OPTION = 'lead-option';

    /**
     * sample type organization
     */
    const SAMPLE_TYPE_ORGANIZATION_OPTION = 'organization-option';

    /**
     * Import lead tasks type
     */
    const SAMPLE_TYPE_LEAD_TASKS = 'lead-tasks';

    /**
     * Import contacts data type
     */
    const SAMPLE_TYPE_CONTACTS_DATA = 'contacts-data';

    /**
     * Import leads success code
     */
    const GET_LEADS_SUCCESS_CODE = '1';

    /**
     * Import leads error code
     */
    const IMPORT_LEADS_ERROR_CODE = '2';

    /**
     * Import leads error license error
     */
    const IMPORT_LEADS_LICENSE_LIMIT = '3';

    /**
     * Import leads error phone or mobile error
     */
    const IMPORT_LEADS_DUBLICATE_PHONE_OR_MOBILE = '10';

    /**
     * Lead tasks import error (empty lead id and external id)
     */
    const IMPORT_LEADS_TASKS_ERROR_CODE = '20';

    /**
     * Lead tasks import error (wrong assigned to email)
     */
    const IMPORT_LEADS_TASKS_EMAIL_ASSIGNED_ERROR_CODE = '21';

    /**
     * Lead task empty title error
     */
    const IMPORT_LEADS_TASKS_TITLE_EMPTY_ERROR_CODE = '22';

    /**
     * Lead task empty date error
     */
    const IMPORT_LEADS_TASKS_DATE_EMPTY_ERROR_CODE = '23';

    /**
     * Lead task lead doesn't exists error
     */
    const IMPORT_LEADS_TASKS_LEAD_DOES_NOT_EXISTS_ERROR_CODE = '24';

    /**
     * Lead task assigned user doesn't exist
     */
    const IMPORT_LEADS_TASKS_ASSIGNED_USER_EMPTY_ERROR_CODE = '25';

    /**
     * Lead task assigned to user doesn\'t exist
     */
    const IMPORT_LEADS_TASKS_EMAIL_ASSIGNED_DOES_NOT_EXIST_ERROR_CODE = '26';

    /**
     * Lead task assigned to user has wrong role
     */
    const IMPORT_LEADS_TASKS_EMAIL_ASSIGNED_WRONG_ROLE_ERROR_CODE = '27';

    /**
     * Lead task assigned by user has wrong role
     */
    const IMPORT_LEADS_TASKS_EMAIL_ASSIGNED_BY_WRONG_ROLE_ERROR_CODE = '28';

    /**
     * Lead or Organization options import error (empty lead or organization option type)
     */
    const IMPORT_LEADS_OR_ORGANIZATIONS_OPTION_TYPE_ERROR_CODE = '30';

    /**
     * Lead already exists and has another owner
     */
    const IMPORT_LEADS_EXISTING_LEAD_HAS_ANOTHER_OWNER_ERROR_CODE = '40';

    /**
     * Taks has due at
     */
    const IMPORT_LEADS_TASK_DUE_AT_MISSING = '41';

    /**
     * Attribute type text
     */
    const ATTRIBUTE_TYPE_TEXT = 'text';

    /**
     * Attribute type date
     */
    const ATTRIBUTE_TYPE_DATE = 'date';

    /**
     * Attribute type checkbox
     */
    const ATTRIBUTE_TYPE_CHECKBOX = 'checkbox';

    /**
     * Attribute type select
     */
    const ATTRIBUTE_TYPE_SELECT = 'select';

    /**
     * Attribute type radio
     */
    const ATTRIBUTE_TYPE_RADIO = 'radio';

    /**
     * select - dropdown alias
     */
    const ATTRIBUTE_SELECT_DROPDOWN_ALIAS = 'dropdown';

    /**
     * Allowed import types
     *
     * @var array
     */
    public static $_allowedImportTypes = array(
        self::SAMPLE_TYPE_ORGANIZATION,
        self::SAMPLE_TYPE_LEAD,
        self::SAMPLE_TYPE_LEAD_OPTION,
        self::SAMPLE_TYPE_ORGANIZATION_OPTION,
        self::SAMPLE_TYPE_LEAD_TASKS,
        self::SAMPLE_TYPE_CONTACTS_DATA
    );


    /**
     * Minimum required import fields leads
     *
     * @var array
     */
    public static $_leadsMinimumRequiredFields = array(
        'lead_first_name',
        'lead_email',
        'lead_type_name'
    );

    /**
     * Minimum required import fields options (leads or organizations)
     *
     * @var array
     */
    public static $_leadsOrOrganizationsOptionsMinimumRequiredFields = array(
        'custom_fields_names',
        'custom_fields_labels',
        'option_types',
        'option_value_names'
    );


    /**
     * Default column types for import leads or organizations options
     *
     * @var array
     */
    public static $_leadOrOrganizationOptionsImportFieldTypes = array(
        'id' => '`id` INT(11) NOT NULL AUTO_INCREMENT',
        'error_code' => "`error_code` TINYINT(3) NOT NULL DEFAULT '0'",
        'import_status' => "`import_status` ENUM('0', '1', '2') NOT NULL DEFAULT '0'",
        'import_id' => '`import_id` INT(11) NOT NULL',
        'import_url' => '`import_url` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'import_type' => '`import_type` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'custom_fields_names' => '`custom_fields_names` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'custom_fields_labels' => '`custom_fields_labels` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'option_types' => '`option_types` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'option_value_names' => '`option_value_names` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL'
    );

    /**
     * Minimum required import fields organizations
     *
     * @var array
     */
    public static $_organizationsMinimumRequiredFields = array(
        'organization_name'
    );

    /**
     * Minimum required import fields lead tasks
     *
     * @var array
     */
    public static $_leadTasksMinimumRequiredFields = array(
        'lead_id',
        'external_id',
        'task_title',
        'assigned_to',
        'assigned_by',
        'task_date_time'
    );


    /**
     * Default column types for import
     *
     * @var array
     */
    public static $_leadImportFieldTypes = array(
        'id' => '`id` INT(11) NOT NULL AUTO_INCREMENT',
        'error_code' => "`error_code` TINYINT(3) NOT NULL DEFAULT '0'",
        'import_status' => "`import_status` ENUM('0', '1', '2') NOT NULL DEFAULT '0'",
        'import_id' => '`import_id` INT(11) NOT NULL',
        'import_url' => '`import_url` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'import_type' => '`import_type` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'lead_first_name' => '`lead_first_name` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'lead_last_name' => '`lead_last_name` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'lead_email' => '`lead_email` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'lead_secondary_email' => '`lead_secondary_email` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'lead_created_by_name' => '`lead_created_by_name` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'lead_created_by_email' => '`lead_created_by_email` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'lead_phone' => '`lead_phone` VARCHAR(20) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'lead_source' => '`lead_source` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'lead_notes' => '`lead_notes` TEXT CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'organization_name' => '`organization_name` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'stage_name' => '`stage_name` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'customer_group_name' => '`customer_group_name` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'lead_zip' => '`lead_zip` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'lead_address' => '`lead_address` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'lead_address1' => '`lead_address1` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'lead_city' => '`lead_city` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'lead_type_name' => '`lead_type_name` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'lead_mobile' => '`lead_mobile` VARCHAR(20) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'lead_position' => '`lead_position` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'organization_website' => '`organization_website` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'organization_website1' => '`organization_website1` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'lead_country_code' => '`lead_country_code` CHAR(2) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'lead_state_code' => '`lead_state_code` VARCHAR(3) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'middle_name' => '`middle_name` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'prefix' => '`prefix` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'lead_phone_country_code' => '`lead_phone_country_code` CHAR(2) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'lead_phone_extension' => '`lead_phone_extension` VARCHAR(10) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'lead_mobile_country_code' => '`lead_mobile_country_code` CHAR(2) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'lead_mobile_country_code_value' => '`lead_mobile_country_code_value` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'organization_industry' => '`organization_industry` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'lead_phone_country_code_value' => '`lead_phone_country_code_value` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'external_id' => '`external_id` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_bin DEFAULT NULL',
        'custom' => 'TEXT CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'lead_tags' => '`lead_tags` TEXT CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'lifecycle_name' => '`lifecycle_name` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'created_at' => '`created_at` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'last_activity' => '`last_activity` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'last_activity_type' => '`last_activity_type` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'last_activity_type_action' => '`last_activity_type_action` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL'
    );

    /**
     * Default column types for import
     *
     * @var array
     */
    public static $_organizationImportFieldTypes = array(
        'id' => '`id` INT(11) NOT NULL AUTO_INCREMENT',
        'error_code' => "`error_code` TINYINT(3) NOT NULL DEFAULT '0'",
        'import_status' => "`import_status` ENUM('0', '1', '2') NOT NULL DEFAULT '0'",
        'import_id' => '`import_id` INT(11) NOT NULL',
        'import_url' => '`import_url` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'import_type' => '`import_type` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'organization_name' => '`organization_name` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'organization_address' => '`organization_address` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'organization_city' => '`organization_city` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'organization_zip' => '`organization_zip` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'customer_group_name' => '`customer_group_name` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'organization_notes' => '`organization_notes` TEXT CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'organization_website' => '`organization_website` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'organization_website1' => '`organization_website1` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'organization_industry' => '`organization_industry` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'organization_phone_country_code' => '`organization_phone_country_code` CHAR(2) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'organization_phone_extension' => '`organization_phone_extension` VARCHAR(10) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'organization_phone_country_code_value' => '`organization_phone_country_code_value` VARCHAR(10) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'organization_mobile_country_code' => '`organization_mobile_country_code` CHAR(2) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'organization_mobile_country_code_value' => '`organization_mobile_country_code_value` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'organization_email' => '`organization_email` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'organization_employees' => '`organization_employees` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'organization_country_code' => '`organization_country_code` CHAR(2) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'organization_state_code' => '`organization_state_code` VARCHAR(3) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'organization_mobile' => '`organization_mobile` VARCHAR(20) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'organization_phone' => '`organization_phone` VARCHAR(20) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'external_id' => '`external_id` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_bin DEFAULT NULL',
        'custom' => 'TEXT CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL'
    );

    /**
     * Default column types for import lead tasks
     *
     * @var array
     */
    public static $_leadTasksImportFieldTypes = array(
        'id' => '`id` INT(11) NOT NULL AUTO_INCREMENT',
        'lead_id' => '`lead_id` INT(11) DEFAULT NULL',
        'error_code' => "`error_code` TINYINT(3) NOT NULL DEFAULT '0'",
        'import_status' => "`import_status` ENUM('0', '1', '2') NOT NULL DEFAULT '0'",
        'import_id' => '`import_id` INT(11) NOT NULL',
        'import_url' => '`import_url` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'import_type' => '`import_type` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'external_id' => '`external_id` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_bin DEFAULT NULL',
        'task_title' => '`task_title` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'assigned_to' => '`assigned_to` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'assigned_by' => '`assigned_by` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'task_date_time' => '`task_date_time` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'due_at' => '`due_at` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'notes' => '`notes` TEXT CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'created_at' => '`created_at` TEXT CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL',
        'task_external_id' => '`task_external_id` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_bin DEFAULT NULL'
    );

    /**
     * Default delimiter
     *
     * @var array
     */
    public static $_csvSettings = array(
        'delimiter' => ','
    );

    /**
     * Errors
     *
     * @var array
     */
    public static $_errorTypes = array(
        '0' => 'invalid lead email',
        '1' => 'empty lead first name',
        '3' => 'Sorry, you need to upgrade your CRM to the proper subscription package before uploading these additional leads.',
        '10' => 'Dublicate phone or mobile number for Lead',
        '20' => 'lead id and external id are empty',
        '21' => 'invalid "assigned to" email',
        '22' => 'empty title',
        '23' => 'empty task date',
        '24' => 'lead doesn\'t exist',
        '25' => '"assigned to" can\'t be empty',
        '26' => '"assigned to" email doesn\'t exist',
        '27' => '"assigned to" a user with a wrong role',
        '28' => '"assigned by" a user with a wrong role',
        '30' => 'empty option type',
        '31' => 'secondary email wrong format',
        '40' => 'the lead already exists and has another owner',
        '41' => 'due at field is missing',
        '99' => 'unknown error during creation',
    );

    /**
     *
     * Storing csv data for future import into the database
     *
     * @param $csvInfo
     * @return array
     *
     */
    public static function putCsvToDatabase($csvInfo, $ownerEmail = null)
    {
        $setLeadOwner = false;
        $csvFile = fopen($csvInfo['file']['tmp_name'], 'r');
        $translator = Zend_Registry::get('Zend_Translate');
        $dataImportDbTable = new Leads_DbTable_LeadsImportDataDbtable();

        if ($csvFile !== false) {
            $assignHeaders = false;
            $importLeadsData = array();
            $excludeHeadersKeys = array();
            $headers = array();
            $columnNames = array();
            $cleanColumnNames = function($value) {
                return  strtolower(str_replace(' ', '_', mb_substr(preg_replace('[^\w\d]', '_', $value), 0, 63)));

            };
            while (($importData = fgetcsv($csvFile, self::$_csvSettings['delimiter'])) !== false) {
                if (!$assignHeaders) {
                    $assignHeaders = true;
                    if ($ownerEmail && !in_array('lead_created_by_email', $importData)) {
                        array_push($importData, 'lead_created_by_email');
                        $setLeadOwner = true;
                    }
                    $headers = array_flip(array_map($cleanColumnNames, $importData));

                    $importFieldTypes = array();
                    $minimumRequiredFields = array();

                    if ($csvInfo['importType'] === self::SAMPLE_TYPE_LEAD) {
                        $minimumRequiredFields = self::$_leadsMinimumRequiredFields;
                        $importFieldTypes = self::$_leadImportFieldTypes;
                    }
                    if ($csvInfo['importType'] === self::SAMPLE_TYPE_ORGANIZATION) {
                        $minimumRequiredFields = self::$_organizationsMinimumRequiredFields;
                        $importFieldTypes = self::$_organizationImportFieldTypes;
                    }

                    if ($csvInfo['importType'] === self::SAMPLE_TYPE_LEAD_TASKS) {
                        $minimumRequiredFields = self::$_leadTasksMinimumRequiredFields;
                        $importFieldTypes = self::$_leadTasksImportFieldTypes;
                    }

                    if (($csvInfo['importType'] === self::SAMPLE_TYPE_LEAD_OPTION) || ($csvInfo['importType'] === self::SAMPLE_TYPE_ORGANIZATION_OPTION)) {
                        $minimumRequiredFields = self::$_leadsOrOrganizationsOptionsMinimumRequiredFields;
                        $importFieldTypes = self::$_leadOrOrganizationOptionsImportFieldTypes;
                    }

                    if (empty($minimumRequiredFields) || empty($importFieldTypes)) {
                        return array(
                            'error' => 1,
                            'errorMessage' => $translator->translate(Leads::TRANSLATION_PREFIX.'wrong ')
                        );
                    }

                    $requiredFields = array_diff_key(array_flip($minimumRequiredFields),
                        $headers);
                    if (!empty($requiredFields)) {
                        $errorMessage = '';
                        foreach ($requiredFields as $fieldMissed => $key) {
                            $errorMessage .= $fieldMissed . '<br />';
                        }
                        fclose($csvFile);

                        return array(
                            'error' => 1,
                            'errorMessage' => $translator->translate(Leads::TRANSLATION_PREFIX. 'Required fields missed') . '<br />' . $errorMessage
                        );
                    }
                    $deleteImportTableSql = "DROP TABLE IF EXISTS " . self::PLUGIN_LEADS_IMPORT_TABLE;
                    $stmt = $dataImportDbTable->getAdapter()->prepare($deleteImportTableSql);
                    $stmt->execute();
                    $sql = "CREATE TABLE IF NOT EXISTS " . self::PLUGIN_LEADS_IMPORT_TABLE . " (";
                    $sql .= $importFieldTypes['id'] . ',';
                    $sql .= $importFieldTypes['import_status'] . ',';
                    $sql .= $importFieldTypes['import_id'] . ',';
                    $sql .= $importFieldTypes['error_code'] . ',';
                    $sql .= $importFieldTypes['import_url'] . ',';
                    $sql .= $importFieldTypes['import_type'] . ',';

                    $tmpHeaders = $headers;
                    foreach ($headers as $columnName => $columnKey) {
                        $tmpHeaders[$columnName] = $columnName;

                        if (array_key_exists($columnName, $importFieldTypes)) {
                            $sql .= ' ' . $importFieldTypes[$columnName] . ',';
                        } elseif (preg_match('~custom_~', $columnName)) {
                            $tmpHeaders[$columnName] = '`'.$columnName.'`';
                            $sql .= ' `' . $columnName . '` ' . $importFieldTypes['custom'] . ',';
                        } elseif (preg_match('~social_~', $columnName)) {
                            $sql .= ' `' . $columnName . '` ' . $importFieldTypes['custom'] . ',';
                        } else {
                            $excludeHeadersKeys[$columnKey] = $columnKey;
                            unset($headers[$columnName]);
                            unset($tmpHeaders[$columnName]);
                        }
                    }
                    $sql .= ' PRIMARY KEY (`id`)) ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;';
                    $stmt = $dataImportDbTable->getAdapter()->prepare($sql);

                    $stmt->execute();
                    $columnNames = implode(($tmpHeaders), ', ');
                    continue;
                }

                $keysData = array_diff_key($importData, $excludeHeadersKeys);
                if ($ownerEmail && $setLeadOwner) {
                    array_push($keysData, $ownerEmail);
                }
                $importLeadsData = array_merge($importLeadsData, $keysData);
            }
            fclose($csvFile);
            $quantity = 0;
            if (!empty($importLeadsData)) {
                $whereStm = substr(str_repeat('?,', count($headers)), 0, -1);
                $quantity = count($importLeadsData) / count($headers);
                $values = implode(
                    ',',
                    array_fill(0, $quantity, '(' . $whereStm . ')')
                );
                $importLeadsStmt = $dataImportDbTable->getAdapter()
                    ->prepare(
                        'INSERT INTO ' . self::PLUGIN_LEADS_IMPORT_TABLE . ' (' . $columnNames . ') VALUES ' . $values . ''
                    );
                try {
                    $importLeadsStmt->execute($importLeadsData);
                } catch (Exception $e) {
                    return array(
                        'error' => 1,
                        'errorMessage' => $e->getMessage()
                    );
                }
            }

            return array('error' => 0, 'quantity' => $quantity);

        } else {
            return array(
                'error' => 1,
                'errorMessage' => $translator->translate(Leads::TRANSLATION_PREFIX.'File error')
            );
        }
    }

    /**
     * Process custom attributes
     *
     * @param array $leadsData import data
     * @param string $type organization or lead
     * @param int $id organization or lead id
     */
    public static function processCustomAttributes($leadsData, $type, $id, $attributeType)
    {
        $customAttributesExists = preg_grep('~custom_.*~', array_keys($leadsData));
        if (!empty($customAttributesExists)) {
            $attributes = array();
            foreach ($customAttributesExists as $customAttributeName) {
                $customAttributeValue = filter_var($leadsData[$customAttributeName], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES);
                $customAttributeName = str_replace('custom_', '',
                    strtolower(preg_replace('~[^-\w\d]~', '', filter_var($customAttributeName, FILTER_SANITIZE_STRING))));
                $attributes[$customAttributeName] = $customAttributeValue;
            }

            self::saveAdditionalFieldsData($id, $type, $attributes, $attributeType);
        }
    }

    /**
     * Process lead social accounts
     *
     * @param array $leadsData import data
     * @param int $leadId lead id
     */
    public static function processSocialAccounts($leadsData, $leadId)
    {
        $socialAccounts = preg_grep('~social_.*~', array_keys($leadsData));
        if (!empty($socialAccounts)) {
            $leadsData = array_intersect_key($leadsData, array_flip($socialAccounts));
            if (!empty($leadsData)) {
                if (!empty($leadsData['social_name']) && !empty($leadsData['social_link'])) {
                    $socialName = 'social_' . trim(strtolower($leadsData['social_name']));
                    $leadsData[$socialName] = trim(strtolower($leadsData['social_link']));
                    unset($leadsData['social_name'], $leadsData['social_link'], $socialName);
                }
                $socialConfig = Leads_Mapper_LeadsSocialServicesConfigMapper::getInstance()->getServices();
                $socialsListConfig = array();
                $resultSocials = array();
                $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
                if (!empty($socialConfig) && is_array($socialConfig)) {
                    foreach ($socialConfig as $social) {
                        $socialsListConfig[$social['id']] = 'social_' . $social['service_name'];
                    }
                    foreach ($leadsData as $service => $link) {
                        $service = trim(strtolower($service));
                        if (in_array($service, $socialsListConfig)) {
                            $resultSocials[$service]['id'] = array_flip($socialsListConfig)[$service];
                            $resultSocials[$service]['link'] = $link;
                        }
                    }
                }
                foreach ($resultSocials as $service) {
                    $leadsMapper->addSocialService($leadId, $service['id'], $service['link']);
                }
            }
        }

    }

    /**
     * Save additional fields data
     *
     * @param int $id lead id
     * @param array $attributes additional data for save
     * @param string $type organization or lead
     * @param string $attributeType attribute name
     */
    public static function saveAdditionalFieldsData($id, $type, $attributes, $attributeType)
    {
        if (!empty($attributes)) {
            if ($type === self::SAMPLE_TYPE_LEAD) {
                $customAttributeConfigMapper = Leads_Mapper_LeadsCustomParamsConfigMapper::getInstance();
            }
            if ($type === self::SAMPLE_TYPE_ORGANIZATION) {
                $customAttributeConfigMapper = Leads_Mapper_LeadsOrganizationCustomParamsConfigMapper::getInstance();
            }

            $customParamsOptionsMapper = Leads_Mapper_LeadsCustomParamsOptionsDataMapper::getInstance();

            foreach ($attributes as $fieldName => $fieldVal) {
                $paramsOptionId = null;

                $fieldVal = filter_var($fieldVal, FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES);
                $customAttributeConfigModel = $customAttributeConfigMapper->getByName($fieldName);

                if (!$customAttributeConfigModel instanceof Leads_LeadsCustomParamsConfigModelInterface) {
                    if ($type === self::SAMPLE_TYPE_LEAD) {
                        $customAttributeConfigModel = new Leads_Model_LeadsCustomParamsConfigModel();
                    }
                    if ($type === self::SAMPLE_TYPE_ORGANIZATION) {
                        $customAttributeConfigModel = new Leads_Model_LeadsOrganizationCustomParamsConfigModel();
                    }
                    $customAttributeConfigModel->setParamName($fieldName);
                    $customAttributeConfigModel->setLabel($fieldName);
                    $customAttributeConfigModel->setParamType($attributeType);
                    $customAttributeConfigModel = $customAttributeConfigMapper->save($customAttributeConfigModel);
                }

                $customAttributeConfigId = $customAttributeConfigModel->getId();

                if ($type === self::SAMPLE_TYPE_LEAD) {
                    $customParamsMapper = Leads_Mapper_LeadsCustomParamsMapper::getInstance();
                }
                if ($type === self::SAMPLE_TYPE_ORGANIZATION) {
                    $customParamsMapper = Leads_Mapper_LeadsOrganizationCustomParamsMapper::getInstance();
                }

                $customAttributeModel = $customParamsMapper->getParamByIdAndParamId($id, $customAttributeConfigId);
                $paramType = $customAttributeConfigModel->getParamType();

                if($paramType == Tools_LeadImportTools::ATTRIBUTE_TYPE_SELECT || $paramType == Tools_LeadImportTools::ATTRIBUTE_TYPE_CHECKBOX || $paramType == Tools_LeadImportTools::ATTRIBUTE_TYPE_RADIO) {
                    $predefinedValues = $customParamsOptionsMapper->findByCustomParamId($customAttributeConfigModel->getId(), true);

                    if(!empty($predefinedValues) && in_array($fieldVal, $predefinedValues)) {
                        $paramsOptionId = array_search($fieldVal, $predefinedValues);
                    }

                    if(!empty($paramsOptionId)) {
                        if(!$customAttributeModel instanceof Leads_LeadsCustomParamsModelInterface) {
                            if ($type === self::SAMPLE_TYPE_LEAD) {
                                $customAttributeModel = new Leads_Model_LeadsCustomParamsModel();
                                $customAttributeModel->setLeadId($id);
                            }
                            if ($type === self::SAMPLE_TYPE_ORGANIZATION) {
                                $customAttributeModel = new Leads_Model_LeadsOrganizationCustomParamsModel();
                                $customAttributeModel->setOrganizationId($id);
                            }
                        }

                        $customAttributeModel->setParamId($customAttributeConfigId);
                        $customAttributeModel->setParamValue($fieldVal);
                        $customAttributeModel->setParamsOptionId($paramsOptionId);

                        $customParamsMapper->save($customAttributeModel);
                    } elseif (!empty($predefinedValues) && $customAttributeModel instanceof Leads_LeadsCustomParamsModelInterface) {
                        $customAttributeModel->setParamId($customAttributeConfigId);
                        $customAttributeModel->setParamValue('');
                        $customAttributeModel->setParamsOptionId(null);

                        $customParamsMapper->save($customAttributeModel);
                    }
                } else {
                    if (!$customAttributeModel instanceof Leads_LeadsCustomParamsModelInterface) {
                        if ($type === self::SAMPLE_TYPE_LEAD) {
                            $customAttributeModel = new Leads_Model_LeadsCustomParamsModel();
                            $customAttributeModel->setLeadId($id);
                        }
                        if ($type === self::SAMPLE_TYPE_ORGANIZATION) {
                            $customAttributeModel = new Leads_Model_LeadsOrganizationCustomParamsModel();
                            $customAttributeModel->setOrganizationId($id);
                        }
                    }

                    $customAttributeModel->setParamId($customAttributeConfigId);
                    $customAttributeModel->setParamValue($fieldVal);
                    $customParamsMapper->save($customAttributeModel);
                }
            }
        }
    }

    /**
     * process organizations data
     *
     * @return array
     */
    public static function processOrganizationsData()
    {
        $dataImportDbTable = new Leads_DbTable_LeadsImportDataDbtable();

        $where = $dataImportDbTable->getAdapter()->quoteInto('import_status = ?', '0');
        $select = $dataImportDbTable->getAdapter()->select()->from(
            self::PLUGIN_LEADS_IMPORT_TABLE
        )
            ->where($where)
            ->limit(self::PLUGIN_LEADS_IMPORT_LIMIT);
        $importDataAll = $dataImportDbTable->getAdapter()->fetchAll($select);
        $dbTable = new Store_DbTable_Group();
        $select = $dbTable->select()->from('shopping_group', array('groupName', 'id', 'priceSign', 'priceType', 'priceValue'));
        $customerGroups = $dbTable->getAdapter()->fetchAssoc($select);
        $industryTypes = Leads_Mapper_LeadsIndustryMapper::getInstance()->getIndustriesPairs(true);

        if (!empty($importDataAll)) {
            foreach ($importDataAll as $importData) {
                $importData['organization_name'] = trim(str_replace(array('\'', '`'), ' ',
                    $importData['organization_name']));
                $importDataId = $importData['id'];
                if (empty($importData['organization_name'])) {
                    self::updateImportStatus($importDataId, self::IMPORT_LEADS_ERROR_CODE, '0');
                    continue;
                }

                if (!empty($importData['customer_group_name'])) {
                    $groupName = filter_var($importData['customer_group_name'], FILTER_SANITIZE_STRING);
                    if (array_key_exists($groupName, $customerGroups)) {
                        $importData['customer_group_id'] = $customerGroups[$groupName]['id'];
                    }
                }

                if (!empty($importData['organization_industry'])) {
                    $industryType = filter_var($importData['organization_industry'], FILTER_SANITIZE_STRING);
                    if (array_key_exists($industryType, $industryTypes)) {
                        $importData['organization_industry_id'] = $industryTypes[$industryType];
                    }
                }

                if (empty($importData['organization_name'])) {
                    self::updateImportStatus($importDataId, self::IMPORT_LEADS_ERROR_CODE, '0');
                    continue;
                }else{
                    $leadsOrganizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
                    $leadsOrganizationModel = $leadsOrganizationMapper->findByName($importData['organization_name']);

                    if(empty($importData['external_id'])){
                        $importData['external_id'] = null;
                    }

                    if($leadsOrganizationModel instanceof Leads_Model_LeadsOrganizationModel){
                        $sessionHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
                        $useExtendedConfiguration = $sessionHelper->useExtendedConfiguration;

                        if(!empty($useExtendedConfiguration)){
                            $importUpdateFieldsConfig = $sessionHelper->importUpdateFieldsConfig;

                            if(!empty($importUpdateFieldsConfig)){
                                $importUpdateFieldsConfig = explode(',', $importUpdateFieldsConfig);
                            }
                            if(!in_array('external_id', $importUpdateFieldsConfig)){
                                $importData['external_id'] = $leadsOrganizationModel->getExternalId();
                            }
                            if(!in_array('organization_name', $importUpdateFieldsConfig)){
                                $importData['organization_name'] = $leadsOrganizationModel->getOrganizationName();
                            }
                            if(!in_array('organization_email', $importUpdateFieldsConfig)){
                                $importData['organization_email'] = $leadsOrganizationModel->getOrganizationEmail();
                            }
                            if(!in_array('organization_mobile_country_code', $importUpdateFieldsConfig)){
                                $importData['organization_mobile_country_code'] = $leadsOrganizationModel->getOrganizationMobileCountryCode();
                            }
                            if(!in_array('organization_mobile', $importUpdateFieldsConfig)){
                                $importData['organization_mobile'] = $leadsOrganizationModel->getOrganizationMobile();
                            }
                            if(!in_array('organization_phone', $importUpdateFieldsConfig)){
                                $importData['organization_phone'] = $leadsOrganizationModel->getOrganizationPhone();
                            }
                            if(!in_array('organization_phone_extension', $importUpdateFieldsConfig)){
                                $importData['organization_phone_extension'] = $leadsOrganizationModel->getOrganizationPhoneExtension();
                            }
                            if(!in_array('organization_phone_country_code', $importUpdateFieldsConfig)){
                                $importData['organization_phone_country_code'] = $leadsOrganizationModel->getOrganizationPhoneCountryCode();
                            }
                            if(in_array('organization_industry', $importUpdateFieldsConfig)){
                                $industryType = filter_var($importData['organization_industry'], FILTER_SANITIZE_STRING);
                                if (array_key_exists($industryType, $industryTypes)) {
                                    $importData['organization_industry_id'] = $industryTypes[$industryType];
                                }
                            }
                        }
                    }
                }

                $organizationModel = self::processOrganization($importData);
                self::processCustomAttributes($importData, self::SAMPLE_TYPE_ORGANIZATION, $organizationModel->getId(), self::ATTRIBUTE_TYPE_TEXT);
                self::updateImportStatus($importDataId, '1', 0, $organizationModel->getId(), self::SAMPLE_TYPE_ORGANIZATION);

            }

        }

        return array('error' => '0', 'importedCount' => count($importDataAll));

    }

    /**
     * Create organization based on organization data
     *
     * @param array $data organization data
     * @param bool $ignoreExternalId ignore external id
     * @return object Leads_Model_LeadsOrganizationModel
     */
    public static function processOrganization($data, $ignoreExternalId = false)
    {

        unset($data['id']);
        if ($ignoreExternalId === true) {
            unset($data['external_id']);
        }
        if (empty($data['customer_group_name'])) {
            $data['customer_group_id'] = null;
        }

        if (empty($data['organization_industry'])) {
            $data['organization_industry_id'] = null;
        }

        if (empty($data['organization_country_code'])) {
            $data['organization_country_code'] = null;
        }

        if (!empty($data['organization_website'])) {
            if (!preg_match('~http:\/\/~', $data['organization_website'])) {
                $data['organization_website'] = 'http://'.$data['organization_website'];
            }
            $domain = Tools_LeadTools::getDomainFromUrl($data['organization_website']);
            if (!empty($domain)) {
                $data['organization_domain'] = $domain;
            }
        }

        if (!empty($data['organization_website1'])) {
            if (!preg_match('~http:\/\/~', $data['organization_website1'])) {
                $data['organization_website1'] = 'http://'.$data['organization_website1'];
            }
            $domain = Tools_LeadTools::getDomainFromUrl($data['organization_website1']);
            if (!empty($domain)) {
                $data['organization_domain1'] = $domain;
            }
        }

        if (!empty($data['organization_email'])) {
            $data['organization_email'] = trim(strtolower($data['organization_email']));
        }
        if (!empty($data['organization_mobile'])) {
            $data['organization_mobile'] = Tools_LeadTools::cleanPhoneAddress($data['organization_mobile']);
        }
        if (!empty($data['organization_phone'])) {
            $data['organization_phone'] = Tools_LeadTools::cleanPhoneAddress($data['organization_phone']);
        }

        $leadsOrganizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
        $leadsOrganizationModel = $leadsOrganizationMapper->findByName($data['organization_name']);
        if (!$leadsOrganizationModel instanceof Leads_Model_LeadsOrganizationModel) {
            $leadsOrganizationModel = new Leads_Model_LeadsOrganizationModel();
            $createdAt = Tools_System_Tools::convertDateFromTimezone('now');
            $data['creation_type'] = Leads_Model_LeadsOrganizationModel::CREATION_TYPE_IMPORT;
        }

        $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
        $userDefaultMobileCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');
        if (empty($data['organization_phone_country_code']) && !empty($userDefaultMobileCountryCode)) {
            $data['organization_phone_country_code'] = $userDefaultMobileCountryCode;
        }

        if (empty($data['organization_mobile_country_code']) && !empty($userDefaultMobileCountryCode)) {
            $data['organization_mobile_country_code'] = $userDefaultMobileCountryCode;
        }

        $leadsOrganizationModel->setOptions($data);

        if (!empty($createdAt)) {
            $leadsOrganizationModel->setCreatedAt($createdAt);
        } else {
            $leadsOrganizationModel->setUpdatedAt(Tools_System_Tools::convertDateFromTimezone('now'));
        }

        $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();
        if (!empty($shoppingConfig['country'])) {
            $defaultCountryCode = $shoppingConfig['country'];
            if (empty($data['organization_country_code'])) {
                $leadsOrganizationModel->setOrganizationCountryCode($defaultCountryCode);
            }
        }

        $leadsOrganizationModel->setSkipCoordinatesGeneration(true);
        return $leadsOrganizationMapper->save($leadsOrganizationModel);
    }

    public static function processLeadsOrOrganizationsOptionsData($importType){
        $dataImportDbTable = new Leads_DbTable_LeadsImportDataDbtable();

        $select = $dataImportDbTable->getAdapter()->select()->distinct()->from(array(self::PLUGIN_LEADS_IMPORT_TABLE),
            array('custom_fields_names', 'custom_fields_labels', 'option_types'))->where('import_status = ?', '0');
        $importDistinctData = $dataImportDbTable->getAdapter()->fetchAssoc($select);

        $select->where('custom_fields_names = ?', '');
        $importCheckEmptyUniqueName = $dataImportDbTable->getAdapter()->fetchAll($select);
        if(!empty($importCheckEmptyUniqueName)){
            return array('error' => '1');
        };

        $importDataAll = array();

        if(!empty($importDistinctData)){
            $importDistinctData = array_shift($importDistinctData);
            $where = $dataImportDbTable->getAdapter()->quoteInto('custom_fields_names = ?', $importDistinctData['custom_fields_names']);
            $where .= ' AND '. $dataImportDbTable->getAdapter()->quoteInto('import_status = ?', '0');

            $select = $dataImportDbTable->getAdapter()->select()->from(self::PLUGIN_LEADS_IMPORT_TABLE)
                ->where($where);
            $importDataAll = $dataImportDbTable->getAdapter()->fetchAll($select);

            if($importType == Tools_LeadImportTools::SAMPLE_TYPE_LEAD_OPTION){
                $customParamConfigMapper = Leads_Mapper_LeadsCustomParamsConfigMapper::getInstance();
                $checkedInstanceObjectModel = new Leads_Model_LeadsCustomParamsConfigModel();
            }else{
                $customParamConfigMapper = Leads_Mapper_LeadsOrganizationCustomParamsConfigMapper::getInstance();
                $checkedInstanceObjectModel = new Leads_Model_LeadsOrganizationCustomParamsConfigModel();
            }

            if(empty($importDistinctData['option_types'])){
                $issetOptionTypeName = false;
                foreach ($importDataAll as $data){
                    if(!empty($data['option_types'])){
                        $issetOptionTypeName = true;
                        $importDistinctData['option_types'] = $data['option_types'];
                    }else{
                        self::updateImportStatus($data['id'], self::IMPORT_LEADS_ERROR_CODE, self::IMPORT_LEADS_OR_ORGANIZATIONS_OPTION_TYPE_ERROR_CODE);
                    }
                }
                if(!$issetOptionTypeName){
                    return array('error' => '0', 'importedCount' => count($importDataAll));
                }
            }

            $importDistinctData['option_types'] = str_replace('dropdown', 'select', $importDistinctData['option_types']);

            $customParamConfigModel = $customParamConfigMapper->getByTypeName($importDistinctData['option_types'],
                $importDistinctData['custom_fields_names']);
            if ($importType == Tools_LeadImportTools::SAMPLE_TYPE_LEAD_OPTION) {
                $leadCustomParamOptionsDataMapper = Leads_Mapper_LeadsCustomParamsOptionsDataMapper::getInstance();
            } else {
                $leadCustomParamOptionsDataMapper = Leads_Mapper_LeadsOrganizationCustomParamsOptionsDataMapper::getInstance();
            }
            if (!$customParamConfigModel instanceof $checkedInstanceObjectModel) {
                if ($importType == Tools_LeadImportTools::SAMPLE_TYPE_LEAD_OPTION) {
                    $customParamConfigModel = new Leads_Model_LeadsCustomParamsConfigModel();

                } else {
                    $customParamConfigModel = new Leads_Model_LeadsOrganizationCustomParamsConfigModel();
                }
            }

                if(($importDistinctData['option_types'] === self::ATTRIBUTE_TYPE_TEXT) || ($importDistinctData['option_types'] === self::ATTRIBUTE_TYPE_DATE)){
                    $importData = $importDataAll;
                    $importData = array_shift($importData);

                    $importDataId = $importData['id'];

                    $customParamConfigModel->setParamType($importData['option_types']);
                    $customParamConfigModel->setParamName($importData['custom_fields_names']);
                    $customParamConfigModel->setLabel((!empty($importData['custom_fields_labels']) ? $importData['custom_fields_labels'] : $importData['custom_fields_names']));

                    $customParamConfigMapper->save($customParamConfigModel);

                    self::updateImportStatus($importDataId, '1');

                }elseif(($importDistinctData['option_types'] === self::ATTRIBUTE_TYPE_SELECT) || ($importDistinctData['option_types'] === self::ATTRIBUTE_TYPE_CHECKBOX)){
                    $customParamConfigModel->setParamType($importDistinctData['option_types']);
                    $customParamConfigModel->setParamName($importDistinctData['custom_fields_names']);
                    $customParamConfigModel->setLabel((!empty($importDistinctData['custom_fields_labels']) ? $importDistinctData['custom_fields_labels'] : $importDistinctData['custom_fields_names']));

                    $customParamConfigMapper->save($customParamConfigModel);

                    if(!empty($importDataAll)){

                        $leadCustomParamId = $customParamConfigModel->getId();

                        foreach ($importDataAll as $dataKey => $data) {
                            if ($importDistinctData['option_types'] === self::ATTRIBUTE_TYPE_CHECKBOX && $dataKey > 1) {
                                continue;
                            }

                            $leadCustomParamOptionsDataModel = $leadCustomParamOptionsDataMapper->findOptionIdByCustomParamId($leadCustomParamId,
                                $data['option_value_names']);
                            if (!$leadCustomParamOptionsDataModel instanceof Leads_Model_LeadsCustomParamsOptionsDataModel && !$leadCustomParamOptionsDataModel instanceof Leads_Model_LeadsOrganizationCustomParamsOptionsDataModel) {
                                if ($importType == Tools_LeadImportTools::SAMPLE_TYPE_LEAD_OPTION) {
                                    $leadCustomParamOptionsDataModel = new Leads_Model_LeadsCustomParamsOptionsDataModel();
                                } else {
                                    $leadCustomParamOptionsDataModel = new Leads_Model_LeadsOrganizationCustomParamsOptionsDataModel();
                                }
                                $leadCustomParamOptionsDataModel->setCustomParamId($leadCustomParamId);
                                $optionValueName = (empty($data['option_value_names'])) ? $importDistinctData['option_types'].$dataKey : $data['option_value_names'];
                                $leadCustomParamOptionsDataModel->setOptionValue($optionValueName);
                                $leadCustomParamOptionsDataMapper->save($leadCustomParamOptionsDataModel);
                            }

                            self::updateImportStatus($data['id'], '1');
                        }
                    }
                }


        }

        return array('error' => '0', 'importedCount' => count($importDataAll));
    }

    /**
     * process leads data
     * @param $currentLoggedUserId string
     * @return array
     */
    public static function processLeadsData($currentLoggedUserId = null)
    {
        $sessionHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
        $dataImportDbTable = new Leads_DbTable_LeadsImportDataDbtable();

        $where = $dataImportDbTable->getAdapter()->quoteInto('import_status = ?', '0');
        $select = $dataImportDbTable->getAdapter()->select()->from(
            self::PLUGIN_LEADS_IMPORT_TABLE
        )
            ->where($where)
            ->limit(self::PLUGIN_LEADS_IMPORT_LIMIT);
        $importDataAll = $dataImportDbTable->getAdapter()->fetchAll($select);
        $dbTable = new Store_DbTable_Group();
        $select = $dbTable->select()->from('shopping_group', array('groupName', 'id', 'priceSign', 'priceType', 'priceValue'));
        $customerGroups = $dbTable->getAdapter()->fetchAssoc($select);
        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
        $industryTypes = Leads_Mapper_LeadsIndustryMapper::getInstance()->getIndustriesPairs(true);
        $leadTypeMapper = Leads_Mapper_LeadsTypeMapper::getInstance();

        if (!empty($importDataAll)) {
            foreach ($importDataAll as $importData) {
                $importData['lead_first_name'] = trim(str_replace(array('\'', '`'), ' ',
                    $importData['lead_first_name']));
                $importDataId = $importData['id'];
                if (empty($importData['lead_first_name'])) {
                    self::updateImportStatus($importDataId, self::IMPORT_LEADS_ERROR_CODE, '1');
                    continue;
                }

                if (!empty($importData['customer_group_name'])) {
                    $groupName = filter_var($importData['customer_group_name'], FILTER_SANITIZE_STRING);
                    if (array_key_exists($groupName, $customerGroups)) {
                        $importData['customer_group_id'] = $customerGroups[$groupName]['id'];
                    }
                }

                $importData['lead_email'] = trim(strtolower($importData['lead_email']));

                if (!empty($importData['lead_email'])) {
                    if (!filter_var($importData['lead_email'], FILTER_VALIDATE_EMAIL)) {
                        self::updateImportStatus($importDataId, self::IMPORT_LEADS_ERROR_CODE, '0');
                        continue;
                    }
                }

                $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
                $userDefaultMobileCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');
                if (empty($importData['lead_mobile_country_code']) && !empty($userDefaultMobileCountryCode)) {
                    $importData['lead_mobile_country_code'] = $userDefaultMobileCountryCode;
                }

                if (empty($importData['lead_phone_country_code']) && !empty($userDefaultMobileCountryCode)) {
                    $importData['lead_phone_country_code'] = $userDefaultMobileCountryCode;
                }

                $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();
                if (!empty($shoppingConfig['country'])) {
                    $defaultCountryCode = $shoppingConfig['country'];
                    if (empty($importData['lead_country_code'])) {
                        $importData['lead_country_code'] = $defaultCountryCode;
                    }
                }

                if (!empty($importData['organization_industry'])) {
                    $industryType = filter_var($importData['organization_industry'], FILTER_SANITIZE_STRING);
                    if (array_key_exists($industryType, $industryTypes)) {
                        $importData['organization_industry_id'] = $industryTypes[$industryType];
                    }
                }


                if ($currentLoggedUserId) {
                    $importData['currentLoggedUserId'] = $currentLoggedUserId;
                }
                $leadModel = self::processLead($importData);
                if ($leadModel instanceof Leads_Model_LeadsModel) {
                    if (!empty($leadModel->getUserId()) && !empty($leadModel->getCustomerGroupId())) {
                        Tools_LeadTools::syncUserGroup($leadModel->getUserId(), $leadModel->getCustomerGroupId());
                    }

                    $leadId = $leadModel->getId();
                    $lastActivityDate = $leadModel->getLastActivity();
                    $activityId = $leadModel->getLastActivityTypeId();
                    if (!empty($lastActivityDate) && !empty($activityId) && $importData['last_activity_type_action']) {
                        $leadTypeModel = $leadTypeMapper->find($activityId);
                        if ($leadTypeModel instanceof Leads_Model_LeadsTypeModel && (Leads_Model_LeadsActionLogModel::ACTION_TYPE_INBOUND || Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND)) {
                            $leadsActionLog = Tools_LeadTools::logLeadAction(str_replace($importData['last_activity_type_action'].'_', '', $leadTypeModel->getTypeName()),
                                $leadId,
                                $importData['last_activity_type_action'], $lastActivityDate, $leadTypeModel->getTypeName());
                        }
                    }
                    self::processCustomAttributes($importData, self::SAMPLE_TYPE_LEAD, $leadId, self::ATTRIBUTE_TYPE_TEXT);

                    $importUpdateFieldsConfig = $sessionHelper->importUpdateFieldsConfig;
                    $importConcatFieldsConfig = $sessionHelper->importConcatFieldsConfig;
                    $useExtendedConfiguration = $sessionHelper->useExtendedConfiguration;

                    if(!empty($useExtendedConfiguration)){
                        if(!empty($importUpdateFieldsConfig)){
                            $importUpdateFieldsConfig = explode(',', $importUpdateFieldsConfig);
                        }

                        if(!empty($importConcatFieldsConfig)){
                            $importConcatFieldsConfig = explode(',', $importConcatFieldsConfig);
                        }
                    }
                    if(!empty($useExtendedConfiguration) && !empty($importUpdateFieldsConfig)){
                        if(in_array('social_accounts', $importUpdateFieldsConfig)){
                            self::processSocialAccounts($importData, $leadId);
                        }
                        if(in_array('lead_tags', $importUpdateFieldsConfig) && !in_array('lead_tags', $importConcatFieldsConfig)){
                            $leadsMapper->deleteTag($leadId);

                            if (!empty($importData['lead_tags'])) {
                                self::processLeadTags($leadId, explode(',', $importData['lead_tags']));
                            }
                        }elseif(in_array('lead_tags', $importUpdateFieldsConfig) && in_array('lead_tags', $importConcatFieldsConfig)){
                            $leadsTagsMapper = Leads_Mapper_LeadsTagsMapper::getInstance();
                            $leadTags = $leadsTagsMapper->findTagsByLeadId($leadId);

                            if(!empty($leadTags)){
                                $leadTags = array_flip($leadTags);
                            }
                            if(!empty($importData['lead_tags'])){
                                $additionalTags = explode(',', $importData['lead_tags']);
                                if(is_array($additionalTags)){
                                    $importData['lead_tags'] = array_merge($leadTags, $additionalTags);
                                }else{
                                    $importData['lead_tags'] = $leadTags;
                                }
                                self::processLeadTags($leadId, $importData['lead_tags']);
                            }
                        }
                    }else{
                        self::processSocialAccounts($importData, $leadId);
                        $leadsMapper->deleteTag($leadId);
                        if (!empty($importData['lead_tags'])) {
                            self::processLeadTags($leadId, explode(',', $importData['lead_tags']));
                        }
                    }

                    self::updateImportStatus($importDataId, '1', 0, $leadId, self::SAMPLE_TYPE_LEAD);
                } else {
                    if (isset($leadModel['error'])) {
                        self::updateImportStatus($importDataId, self::IMPORT_LEADS_ERROR_CODE, $leadModel['error']);
                    } else {
                        self::updateImportStatus($importDataId, self::IMPORT_LEADS_ERROR_CODE, '99');
                    }
                }
            }
        }

        return array('error' => '0', 'importedCount' => count($importDataAll));
    }

    /**
     * Process lead tasks data
     *
     * @return array
     */
    public static function processLeadTasksData()
    {
        $dataImportDbTable = new Leads_DbTable_LeadsImportDataDbtable();
        $where = $dataImportDbTable->getAdapter()->quoteInto('import_status = ?', '0');
        $importDataAll = $dataImportDbTable->fetchAll($where, null, self::PLUGIN_LEADS_IMPORT_LIMIT);
        if (!empty($importDataAll)) {
            $userMapper = Application_Model_Mappers_UserMapper::getInstance();
            foreach ($importDataAll as $importData) {
                $importDataId = $importData['id'];
                if (empty($importData['lead_id']) && empty($importData['external_id'])) {
                    self::updateImportStatus($importDataId, self::IMPORT_LEADS_ERROR_CODE, self::IMPORT_LEADS_TASKS_ERROR_CODE);
                    continue;
                }

                $importData['assigned_to'] = trim(strtolower($importData['assigned_to']));

                if (empty($importData['assigned_to'])) {
                    self::updateImportStatus($importDataId, self::IMPORT_LEADS_ERROR_CODE, self::IMPORT_LEADS_TASKS_EMAIL_ASSIGNED_ERROR_CODE);
                    continue;
                }

                if (!filter_var($importData['assigned_to'], FILTER_VALIDATE_EMAIL)) {
                    self::updateImportStatus($importDataId, self::IMPORT_LEADS_ERROR_CODE, self::IMPORT_LEADS_TASKS_EMAIL_ASSIGNED_ERROR_CODE);
                    continue;
                }

                if (empty($importData['due_at'])) {
                    self::updateImportStatus($importDataId, self::IMPORT_LEADS_ERROR_CODE, self::IMPORT_LEADS_TASK_DUE_AT_MISSING);
                    continue;
                }

                $userModel = $userMapper->findByEmail($importData['assigned_to']);
                if (!$userModel instanceof Application_Model_Models_User) {
                    self::updateImportStatus($importDataId, self::IMPORT_LEADS_ERROR_CODE, self::IMPORT_LEADS_TASKS_EMAIL_ASSIGNED_DOES_NOT_EXIST_ERROR_CODE);
                    continue;
                }

                $assignedToRole = $userModel->getRoleId();
                if ($assignedToRole !== Tools_Security_Acl::ROLE_ADMIN && $assignedToRole !== Tools_Security_Acl::ROLE_SUPERADMIN && $assignedToRole !== Shopping::ROLE_SALESPERSON) {
                    self::updateImportStatus($importDataId, self::IMPORT_LEADS_ERROR_CODE,
                        self::IMPORT_LEADS_TASKS_EMAIL_ASSIGNED_WRONG_ROLE_ERROR_CODE);
                    continue;
                }


                $ownerModel = null;

                if (!empty($importData['assigned_by'])) {
                    $ownerModel = $userMapper->findByEmail($importData['assigned_by']);
                }

                if (!$ownerModel instanceof Application_Model_Models_User) {
                    $ownerId = Tools_LeadTools::getDefaultLeadOwner();
                    $ownerModel = $userMapper->find($ownerId);
                }

                if (!$ownerModel instanceof Application_Model_Models_User) {
                    $ownerModel = $userMapper->findByRole(Tools_Security_Acl::ROLE_SUPERADMIN);
                } else {
                    $ownerRole = $ownerModel->getRoleId();
                    if ($ownerRole !== Tools_Security_Acl::ROLE_ADMIN && $ownerRole !== Tools_Security_Acl::ROLE_SUPERADMIN && $ownerRole !== Shopping::ROLE_SALESPERSON) {
                        self::updateImportStatus($importDataId, self::IMPORT_LEADS_ERROR_CODE,
                            self::IMPORT_LEADS_TASKS_EMAIL_ASSIGNED_BY_WRONG_ROLE_ERROR_CODE);
                        continue;
                    }
                }

                $importData['user_id'] = $userModel->getId();
                $importData['creator_id'] = $ownerModel->getId();

                if (empty($importData['task_title'])) {
                    self::updateImportStatus($importDataId, self::IMPORT_LEADS_ERROR_CODE, self::IMPORT_LEADS_TASKS_TITLE_EMPTY_ERROR_CODE);
                    continue;
                }

                $importData['title'] = $importData['task_title'];

                if (empty($importData['task_date_time'])) {
                    self::updateImportStatus($importDataId, self::IMPORT_LEADS_ERROR_CODE, self::IMPORT_LEADS_TASKS_DATE_EMPTY_ERROR_CODE);
                    continue;
                }

                $leadTaskResult = self::processLeadTask($importData);
                if (!$leadTaskResult instanceof Leads_Model_LeadsTaskModel) {
                    self::updateImportStatus($importDataId, self::IMPORT_LEADS_ERROR_CODE, $leadTaskResult['error']);
                } else {
                    self::updateImportStatus($importDataId, '1', 0, $leadTaskResult->getLeadId(), self::SAMPLE_TYPE_LEAD);
                }

            }
        }

        return array('error' => '0', 'importedCount' => count($importDataAll));
    }

    /**
     * Process lead task
     *
     * @param array $data lead task data
     * @return array
     */
    public static function processLeadTask($data)
    {
        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
        $leadModel = null;
        unset($data['id']);
        unset($data['action_log_id']);
        if (!empty($data['lead_id'])) {
            $leadModel = $leadsMapper->find($data['lead_id']);
        }

        if (!$leadModel instanceof Leads_Model_LeadsModel && !empty($data['external_id'])) {
            $leadModel = $leadsMapper->findByExternalId($data['external_id']);
        }

        if (!$leadModel instanceof Leads_Model_LeadsModel) {
            return array('error' => '24');
        }

        $shoppingConfigMapper =  Models_Mapper_ShoppingConfig::getInstance();
        $leadsTaskLogMapper = Leads_Mapper_LeadsTasksLogMapper::getInstance();
        $storeTimezone = $shoppingConfigMapper->getConfigParam('timezone');
        $data['expired_at'] = Tools_System_Tools::convertDateFromTimezone($data['task_date_time'], $storeTimezone, 'UTC');

        if (empty($data['created_at'])) {
            $createdAt = Tools_System_Tools::convertDateFromTimezone('now', $storeTimezone, 'UTC');
        } else {
            $createdAt = Tools_System_Tools::convertDateFromTimezone($data['created_at'], $storeTimezone, 'UTC');
        }

        $leadTaskModel = null;
        $leadTaskMapper = Leads_Mapper_LeadsTaskMapper::getInstance();
        if (!empty($data['task_external_id'])) {
            $leadTaskModel = $leadTaskMapper->findByTaskExternalId($data['task_external_id']);
        } else {
            $data['task_external_id'] = '';
        }

        if ($leadTaskModel instanceof Leads_Model_LeadsTaskModel) {
            if ($leadTaskModel instanceof Leads_Model_LeadsTaskModel) {
                $leadId = $leadModel->getId();
                $leadTaskModel->setLeadId($leadId);
                $leadTaskModel->setCreationType(Leads_Model_LeadsTaskModel::CREATION_TYPE_IMPORT);
                $leadTaskModel->setTitle($data['title']);
                $leadTaskModel->setUserId($data['user_id']);
                $leadTaskModel->setCreatorId($data['creator_id']);
                $leadTaskModel->setTaskExternalId($data['task_external_id']);
                if (!empty($data['notes'])) {
                    $leadTaskModel->setNotes($data['notes']);
                }
                $leadTaskModel->setExpiredAt($data['expired_at']);
                $leadTaskId = $leadTaskModel->getId();
                $leadTaskLogModels = $leadsTaskLogMapper->findByTaskId($leadTaskId);
                if (!empty($leadTaskLogModels)) {
                    foreach ($leadTaskLogModels as $leadTaskLogModel) {
                        $actionLogId = $leadTaskLogModel->getActionLogId();
                        $actionLogMapper = Leads_Mapper_LeadsActionLogMapper::getInstance();
                        $actionLogModel = $actionLogMapper->find($actionLogId);
                        $leadTaskLogModel->setOptions($data);
                        $leadTaskLogModel->setExpiredAt($data['expired_at']);
                        $leadTaskLogModel->setLeadId($leadId);

                        if ($actionLogModel instanceof Leads_Model_LeadsActionLogModel) {
                            $eventType = $actionLogModel->getEventType();
                            if ($eventType === 'expired') {
                                $actionLogModel->setCreatedAt($createdAt);
                            }
                            if ($eventType === 'upcoming') {
                                $actionLogModel->setCreatedAt($data['expired_at']);
                            }
                            $actionLogModel->setLeadId($leadId);
                            $actionLogMapper->save($actionLogModel);
                            $leadsTaskLogMapper->save($leadTaskLogModel);

                        } else {
                            return array('error' => '99');
                        }
                    }

                    $leadTaskQueueMapper = Leads_Mapper_LeadsTaskQueueMapper::getInstance();
                    $taskQueueModel = $leadTaskQueueMapper->findByTaskIdQueueType($leadTaskId, Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_STATUS_PROCESSING);
                    if ($taskQueueModel instanceof Leads_Model_LeadsTaskQueueModel) {
                        $leadTaskQueueMapper->delete($taskQueueModel->getId());
                    }

                    $leadTaskModel = $leadTaskMapper->save($leadTaskModel);

                } else {
                    return array('error' => '99');
                }
            }

        } else {
            $leadTaskModel = new Leads_Model_LeadsTaskModel();
            $leadId = $leadModel->getId();
            $leadTaskModel->setLeadId($leadId);
            $leadTaskModel->setCreationType(Leads_Model_LeadsTaskModel::CREATION_TYPE_IMPORT);
            $leadTaskModel->setTitle($data['title']);
            $leadTaskModel->setUserId($data['user_id']);
            $leadTaskModel->setCreatorId($data['creator_id']);
            $leadTaskModel->setTaskExternalId($data['task_external_id']);
            $leadTaskModel->setDueDate($data['due_at']);
            if (!empty($data['notes'])) {
                $leadTaskModel->setNotes($data['notes']);
            }

            $leadTaskModel->setExpiredAt($data['expired_at']);

            $leadTaskModel = $leadTaskMapper->save($leadTaskModel);

            $actionLogExpiredModel = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_TASK, $leadId,
                Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND, $createdAt, Leads_Model_LeadsTypeModel::LEAD_OUTBOUND_TASK);

            $actionLogUpcomingModel = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_TASK, $leadId,
                Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND, $data['expired_at'], Leads_Model_LeadsTypeModel::LEAD_OUTBOUND_TASK, Leads_Model_LeadsActionLogModel::EVENT_LOG_UPCOMING);


            $leadsTaskLogModel = new Leads_Model_LeadsTasksLogModel();
            $leadsTaskLogModel->setOptions($data);
            $leadsTaskLogModel->setActionLogId($actionLogExpiredModel->getId());
            $leadsTaskLogModel->setTaskId($leadTaskModel->getId());
            $leadsTaskLogModel->setLeadId($leadId);
            $leadsTaskLogMapper->save($leadsTaskLogModel);


            $leadsTaskLogModel = new Leads_Model_LeadsTasksLogModel();
            $leadsTaskLogModel->setOptions($data);
            $leadsTaskLogModel->setActionLogId($actionLogUpcomingModel->getId());
            $leadsTaskLogModel->setTaskId($leadTaskModel->getId());
            $leadsTaskLogModel->setLeadId($leadId);
            $leadsTaskLogMapper->save($leadsTaskLogModel);
        }

        return $leadTaskModel;

    }

    /**
     * Create or assign tags to lead
     *
     * @param int $leadId lead id
     * @param array $tagsData tag name array
     */
    public static function processLeadTags($leadId, array $tagsData)
    {
        $tagsData = array_filter($tagsData);
        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
        $leadTagsMapper = Leads_Mapper_LeadsTagsMapper::getInstance();
        if (!empty($tagsData)) {
            foreach ($tagsData as $tag) {
                $tag = filter_var($tag, FILTER_SANITIZE_STRING);
                $leadTagModel = $leadTagsMapper->getTag($tag);
                if (!$leadTagModel instanceof Leads_Model_LeadsTagModel) {
                    $leadTagModel = new Leads_Model_LeadsTagModel();
                    $leadTagModel->setLabel($tag);
                    $leadTagModel->setTagName($tag);
                    $leadTagModel = $leadTagsMapper->save($leadTagModel);
                }
                $leadsMapper->addTag($leadId, $leadTagModel->getId());
            }
        }

    }

    /**
     * Check if it's a new lead
     *
     * @param array $data lead info data
     * @return array  status old | new and leadModel
     */
    public static function isNewLead($data)
    {
        if (empty($data['customer_group_name'])) {
            $data['customer_group_id'] = null;
        }

        if (empty($data['lead_country_code'])) {
            $data['lead_country_code'] = null;
        }

        if (empty($data['lead_mobile'])) {
            $data['lead_mobile'] = '';
        }

        if (empty($data['lead_phone'])) {
            $data['lead_phone'] = '';
        }

        $data['lead_mobile'] = Tools_LeadTools::cleanPhoneAddress($data['lead_mobile']);
        $data['lead_phone'] = Tools_LeadTools::cleanPhoneAddress($data['lead_phone']);

        if (empty($data['lead_mobile'])) {
            $data['lead_mobile'] = null;
        }

        if (empty($data['lead_mobile_country_code_value'])) {
            $data['lead_mobile_country_code_value'] = null;
        } else {
            $data['lead_mobile_country_code_value'] = self::prepareCountryCodeValue($data['lead_mobile_country_code_value']);
        }

        if (empty($data['lead_phone'])) {
            $data['lead_phone'] = null;
        }

        if (empty($data['lead_phone_country_code_value'])) {
            $data['lead_phone_country_code_value'] = null;
        } else {
            $data['lead_phone_country_code_value'] = self::prepareCountryCodeValue($data['lead_phone_country_code_value']);
        }

        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
        $leadsModel = null;
        $userMapper = Application_Model_Mappers_UserMapper::getInstance();
        if (!empty($data['lead_email'])) {
            $leadsModel = $leadsMapper->findByEmail($data['lead_email']);
            $userModel = $userMapper->findByEmail($data['lead_email']);
            if ($userModel instanceof Application_Model_Models_User) {
                $data['user_id'] = $userModel->getId();
            }
        }

        if (!$leadsModel instanceof Leads_Model_LeadsModel) {
            if (!empty($data['lead_mobile_country_code_value']) && !empty($data['lead_mobile'])) {
                $leadsModel = Tools_LeadTools::findLeadByPhoneMobileNumber($data['lead_mobile_country_code_value'].$data['lead_mobile'], true);
            }
        } else {
            $leadsModelCheck = null;
            if (!empty($data['lead_mobile_country_code_value']) && !empty($data['lead_mobile'])) {
                $leadsModelCheck = Tools_LeadTools::findLeadByPhoneMobileNumber($data['lead_mobile_country_code_value'] . $data['lead_mobile'],
                    true, $leadsModel->getId());
            }
            if ($leadsModelCheck instanceof Leads_Model_LeadsModel) {
                return array('status' => Tools_LeadFormEventsTools::LEAD_MODEL_STATUS_OLD, 'leadModel' => $leadsModelCheck);
            }
        }

        if (!$leadsModel instanceof Leads_Model_LeadsModel) {
            if (!empty($data['lead_phone_country_code_value']) && !empty($data['lead_phone'])) {
                $leadsModel = Tools_LeadTools::findLeadByPhoneMobileNumber($data['lead_phone_country_code_value'].$data['lead_phone'], true);
            }
        } else {
            $leadsModelCheck = null;
            if (!empty($data['lead_phone_country_code_value']) && !empty($data['lead_phone'])) {
                $leadsModelCheck = Tools_LeadTools::findLeadByPhoneMobileNumber($data['lead_phone_country_code_value'].$data['lead_phone'], true, $leadsModel->getId());
            }
            if ($leadsModelCheck instanceof Leads_Model_LeadsModel) {
                return array('status' => Tools_LeadFormEventsTools::LEAD_MODEL_STATUS_OLD, 'leadModel' => $leadsModelCheck);
            }
        }

        if (!$leadsModel instanceof Leads_Model_LeadsModel && !empty($data['lead_zip'])) {
            $leadsModel = $leadsMapper->findByFirstNameLastNameZip($data['lead_first_name'], $data['lead_last_name'], $data['lead_zip']);
        }

        if ($leadsModel instanceof Leads_Model_LeadsModel) {
            return array('status' => Tools_LeadFormEventsTools::LEAD_MODEL_STATUS_OLD, 'leadModel' => $leadsModel);
        }

        return array('status' => Tools_LeadFormEventsTools::LEAD_MODEL_STATUS_NEW, 'leadModel' => null);

    }

    /**
     * Create lead based on lead data
     *
     * @param array $data lead model
     * @return int id
     */
    public static function processLead($data)
    {
        $sessionHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('session');

        $importUpdateFieldsConfig = $sessionHelper->importUpdateFieldsConfig;
        $importConcatFieldsConfig = $sessionHelper->importConcatFieldsConfig;
        $useExtendedConfiguration = $sessionHelper->useExtendedConfiguration;

        if(!empty($useExtendedConfiguration)){
            if(!empty($importUpdateFieldsConfig)){
                $importUpdateFieldsConfig = explode(',', $importUpdateFieldsConfig);
            }

            if(!empty($importConcatFieldsConfig)){
                $importConcatFieldsConfig = explode(',', $importConcatFieldsConfig);
            }
        }

        unset($data['id']);

        $isNewLead = self::isNewLead($data);

        if (empty($data['customer_group_name'])) {
            $data['customer_group_id'] = null;
        }

        if (empty($data['lead_country_code'])) {
            $data['lead_country_code'] = null;
        }

        $data['lead_mobile'] = Tools_LeadTools::cleanPhoneAddress($data['lead_mobile']);
        $data['lead_phone'] = Tools_LeadTools::cleanPhoneAddress($data['lead_phone']);

        if (empty($data['lead_mobile'])) {
            $data['lead_mobile'] = null;
        }

        if (empty($data['lead_mobile_country_code_value'])) {
            $data['lead_mobile_country_code_value'] = null;
        } else {
            $data['lead_mobile_country_code_value'] = self::prepareCountryCodeValue($data['lead_mobile_country_code_value']);
        }

        if (empty($data['lead_phone'])) {
            $data['lead_phone'] = null;
        }

        if (empty($data['lead_phone_country_code_value'])) {
            $data['lead_phone_country_code_value'] = null;
        } else {
            $data['lead_phone_country_code_value'] = self::prepareCountryCodeValue($data['lead_phone_country_code_value']);
        }

        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
        $leadsModel = null;
        $userMapper = Application_Model_Mappers_UserMapper::getInstance();
        if (!empty($data['lead_email'])) {
            $leadsModel = $leadsMapper->findByEmail($data['lead_email']);
            $userModel = $userMapper->findByEmail($data['lead_email']);
            if ($userModel instanceof Application_Model_Models_User) {
                $data['user_id'] = $userModel->getId();
            }
        }

        if (!$leadsModel instanceof Leads_Model_LeadsModel) {
            if (!empty($data['lead_mobile_country_code_value']) && !empty($data['lead_mobile'])) {
                $leadsModel = Tools_LeadTools::findLeadByPhoneMobileNumber($data['lead_mobile_country_code_value'].$data['lead_mobile'], true);
            }
        } else {
            $leadsModelCheck = null;
            if (!empty($data['lead_mobile_country_code_value']) && !empty($data['lead_mobile'])) {
                $leadsModelCheck = Tools_LeadTools::findLeadByPhoneMobileNumber($data['lead_mobile_country_code_value'] . $data['lead_mobile'],
                    true, $leadsModel->getId());
            }
            if ($leadsModelCheck instanceof Leads_Model_LeadsModel) {
                return array('error' => '10');
            }
        }

        if (!$leadsModel instanceof Leads_Model_LeadsModel) {
            if (!empty($data['lead_phone_country_code_value']) && !empty($data['lead_phone'])) {
                $leadsModel = Tools_LeadTools::findLeadByPhoneMobileNumber($data['lead_phone_country_code_value'].$data['lead_phone'], true);
            }
        } else {
            $leadsModelCheck = null;
            if (!empty($data['lead_phone_country_code_value']) && !empty($data['lead_phone'])) {
                $leadsModelCheck = Tools_LeadTools::findLeadByPhoneMobileNumber($data['lead_phone_country_code_value'].$data['lead_phone'], true, $leadsModel->getId());
            }
            if ($leadsModelCheck instanceof Leads_Model_LeadsModel) {
                return array('error' => '10');
            }
        }

        if (!$leadsModel instanceof Leads_Model_LeadsModel && (!empty($data['lead_phone']) || !empty($data['lead_mobile']) || !empty($data['lead_email']))) {
            $leadAdditionAvailable = Tools_LeadLicenseTools::checkLimit();
            if ($leadAdditionAvailable['error'] === true) {
                return array('error' => '3');
            }
        }

        if (!$leadsModel instanceof Leads_Model_LeadsModel && !empty($data['lead_zip'])) {
            $leadsModel = $leadsMapper->findByFirstNameLastNameZip($data['lead_first_name'], $data['lead_last_name'], $data['lead_zip']);
        }

        if (!$leadsModel instanceof Leads_Model_LeadsModel) {
            $leadAdditionAvailable = Tools_LeadLicenseTools::checkLimit();
            if ($leadAdditionAvailable['error'] === true) {
                return array('error' => '3');
            }
        }
        if ($leadsModel instanceof Leads_Model_LeadsModel && !empty($data['currentLoggedUserId'])) {
            if ($leadsModel->getOwnerId() != $data['currentLoggedUserId']) {
                return array('error' => '40');
            }
        }

        if (empty($data['lead_email'])) {
            $data['lead_email'] = null;
        }

        $emailValidator = new Tools_System_CustomEmailValidator();
        if (!empty($data['lead_secondary_email'])) {
            if (!$emailValidator->isValid($data['lead_secondary_email'])) {
                return array('error' => '31');
            }
        }

        /**
         * assign owner to lead if exists
         */
        if (!empty($data['lead_created_by_email']) && $emailValidator->isValid($data['lead_created_by_email'])) {
            $ownerModel = $userMapper->findByEmail($data['lead_created_by_email']);
            if ($ownerModel instanceof Application_Model_Models_User) {
                    $data['owner_id'] = $ownerModel->getId();
            } else {
                $ownerModel = new Application_Model_Models_User();
                $ownerModel->setEmail($data['lead_created_by_email']);
                if (!empty($data['lead_created_by_name'])) {
                    $ownerModel->setFullName($data['lead_created_by_name']);
                } else {
                    $ownerModel->setFullName('Sales person');
                }
                $ownerModel->setPassword(microtime());
                $ownerModel->setRoleId(Shopping::ROLE_SALESPERSON);

                if($isNewLead['status'] == Tools_LeadFormEventsTools::LEAD_MODEL_STATUS_OLD && !empty($useExtendedConfiguration) && !empty($importUpdateFieldsConfig)){
                    if(in_array('lead_created_by_email', $importUpdateFieldsConfig)){
                        $data['owner_id'] = $userMapper->save($ownerModel);
                    }else{
                        $data['owner_id'] = $leadsModel->getOwnerId();
                    }
                }else{
                    $data['owner_id'] = $userMapper->save($ownerModel);
                }
            }

        }

        /**
         * Assign default owner
         */
        if (empty($data['owner_id']) && !$leadsModel instanceof Leads_Model_LeadsModel) {
            $superAdminUserModel = $leadsMapper->findSuperAdminUser();
            if ($superAdminUserModel instanceof Application_Model_Models_User) {
                $data['owner_id'] = $superAdminUserModel->getId();
            }
        }

        $assignPrimaryContact = false;
        $organizationModel = null;

        /**
         * create or attach organization
         */
        if (!empty($data['organization_name'])) {
            $leadsOrganizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
            $leadsOrganizationModel = $leadsOrganizationMapper->findByName($data['organization_name']);

            if (!$leadsOrganizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                $assignPrimaryContact = true;
            }

            if(!empty($useExtendedConfiguration) && !empty($importUpdateFieldsConfig)){
                $organizationData['organization_name'] = $data['organization_name'];

                if(in_array('organization_website', $importUpdateFieldsConfig)){
                    $organizationData['organization_website'] = $data['organization_website'];
                }

                if(in_array('organization_website1', $importUpdateFieldsConfig)){
                    $organizationData['organization_website1'] = $data['organization_website1'];
                }
                if(in_array('organization_industry', $importUpdateFieldsConfig)){
                    $industryTypes = Leads_Mapper_LeadsIndustryMapper::getInstance()->getIndustriesPairs(true);
                    $industryType = filter_var($data['organization_industry'], FILTER_SANITIZE_STRING);
                    if (array_key_exists($industryType, $industryTypes)) {
                        $organizationData['organization_industry_id'] = $data['organization_industry_id'];
                        $organizationData['organization_industry'] = $data['organization_industry_id'];
                    }
                }
                $organizationModel = self::processOrganization($organizationData, true);
            }else{
                $organizationModel = self::processOrganization($data, true);
            }

            if ($organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                $data['organization_id'] = $organizationModel->getId();
            }
        }

        /**
         * Create or assign lead type
         */

        if (!empty($data['lead_type_name'])) {
            $leadTypesMapper = Leads_Mapper_LeadsTypeMapper::getInstance();
            $leadTypeName = str_replace(' ', '_', filter_var($data['lead_type_name'], FILTER_SANITIZE_STRING));
            $leadTypeModel = $leadTypesMapper->findByTypeName($leadTypeName);
            if (!$leadTypeModel instanceof Leads_Model_LeadsTypeModel) {
                $leadTypeModel = new Leads_Model_LeadsTypeModel();
                $leadTypeModel->setTypeLabel($leadTypeName);
                $leadTypeModel->setTypeName($leadTypeName);
                $leadTypeModel = $leadTypesMapper->save($leadTypeModel);
            }
            if($isNewLead['status'] == Tools_LeadFormEventsTools::LEAD_MODEL_STATUS_OLD && !empty($useExtendedConfiguration) && !empty($importUpdateFieldsConfig)){
                if(in_array('lead_type_name', $importUpdateFieldsConfig)){
                    $data['lead_type_id'] = $leadTypeModel->getId();
                }else{
                    $data['lead_type_id'] = $leadsModel->getLeadTypeId();
                }
            }else{
                $data['lead_type_id'] = $leadTypeModel->getId();
            }
        }

        /**
         * Create or assign lead stage
         */
        $leadStageMapper = Leads_Mapper_LeadsStageMapper::getInstance();
        if (!empty($data['lifecycle_name'])) {
            $leadStageName = filter_var($data['lifecycle_name'], FILTER_SANITIZE_STRING);
            $leadStageModel = $leadStageMapper->findByStageName($leadStageName);
            if (!$leadStageModel instanceof Leads_Model_LeadsStageModel) {
                $leadStageModel = new Leads_Model_LeadsStageModel();
                $leadStageModel->setStageName($leadStageName);
                $leadStageModel = $leadStageMapper->save($leadStageModel);
            }

            if($isNewLead['status'] == Tools_LeadFormEventsTools::LEAD_MODEL_STATUS_OLD && !empty($useExtendedConfiguration) && !empty($importUpdateFieldsConfig)){
                if(in_array('lifecycle_name', $importUpdateFieldsConfig)){
                    $data['stage_id'] = $leadStageModel->getId();
                }else{
                    $data['stage_id'] = $leadsModel->getLeadTypeId();
                }
            }else{
                $data['stage_id'] = $leadStageModel->getId();
            }
        } else {
            $leadStageModel = $leadStageMapper->getDefaultStage();
            if ($leadStageModel instanceof Leads_Model_LeadsStageModel) {
                $data['stage_id']  = $leadStageModel->getId();
            }
        }

        if (!empty($data['created_at'])) {
            $storeTimezone = Models_Mapper_ShoppingConfig::getInstance()->getConfigParam('timezone');
            $data['created_at'] = date(Tools_System_Tools::DATE_MYSQL, strtotime($data['created_at'] .'+'.Tools_EmailSequenceTools::getTimezoneShift('UTC', $storeTimezone).'hours'));
            $createdAt = $data['created_at'];
        }

        if (!empty($data['last_activity'])) {
            $storeTimezone = Models_Mapper_ShoppingConfig::getInstance()->getConfigParam('timezone');
            $data['last_activity'] = date(Tools_System_Tools::DATE_MYSQL, strtotime($data['last_activity'] .'+'.Tools_EmailSequenceTools::getTimezoneShift('UTC', $storeTimezone).'hours'));
        }

        if (!empty($data['last_activity_type'])) {
            $leadTypesMapper = Leads_Mapper_LeadsTypeMapper::getInstance();
            $leadTypeName = str_replace(' ', '_', filter_var($data['last_activity_type'], FILTER_SANITIZE_STRING));
            $leadTypeModel = $leadTypesMapper->findByTypeName($leadTypeName);
            if ($leadTypeModel instanceof Leads_Model_LeadsTypeModel) {
                $data['last_activity_type_id'] = $leadTypeModel->getId();
            }
        }

        if (!$leadsModel instanceof Leads_Model_LeadsModel) {
            $leadsModel = new Leads_Model_LeadsModel();
            if (empty($data['created_at'])) {
                $createdAt = Tools_System_Tools::convertDateFromTimezone('now');
            }
            $data['creation_type'] = Leads_Model_LeadsModel::CREATION_TYPE_IMPORT;
        }

        if($isNewLead['status'] == Tools_LeadFormEventsTools::LEAD_MODEL_STATUS_OLD && !empty($useExtendedConfiguration) && !empty($importUpdateFieldsConfig)){
            if(!empty($importConcatFieldsConfig)){
                if(in_array('lead_notes', $importConcatFieldsConfig)){
                    $leadNotes = $leadsModel->getLeadNotes();
                    if(!empty($leadNotes)){
                        $leadNotes .= ' '.$data['lead_notes'];
                        $data['lead_notes'] = $leadNotes;
                    }
                }
            }

            if(!empty($useExtendedConfiguration)){
                $importUpdateFieldsConfig[] = 'created_at';
                $importUpdateFieldsConfig[] = 'last_activity';
            }

            if(in_array('organization_name', $importUpdateFieldsConfig)){
                $importUpdateFieldsConfig[] = 'organization_id';
            }

            if(in_array('lead_created_by_email', $importUpdateFieldsConfig)){
                $importUpdateFieldsConfig[] = 'owner_id';
            }

            if(in_array('lifecycle_name', $importUpdateFieldsConfig)){
                $importUpdateFieldsConfig[] = 'stage_id';
            }

            if(in_array('lead_type_name', $importUpdateFieldsConfig)){
                $importUpdateFieldsConfig[] = 'lead_type_id';
            }

            if(in_array('customer_group_name', $importUpdateFieldsConfig)){
                $importUpdateFieldsConfig[] = 'customer_group_id';
            }

            if(in_array('last_activity_type', $importUpdateFieldsConfig)){
                $importUpdateFieldsConfig[] = 'last_activity_type_id';
            }

            if(in_array('lead_email', $importUpdateFieldsConfig)){
                $importUpdateFieldsConfig[] = 'user_id';
            }

            if(in_array('lifecycle_name', $importUpdateFieldsConfig)){
                $importUpdateFieldsConfig[] = 'stage_id';
            }

            foreach ($data as $key => $value){
                if(!in_array($key, $importUpdateFieldsConfig)){
                    unset($data[$key]);
                }
            }
        }

        $leadsModel->setOptions($data);

        if (!empty($createdAt)) {
            $leadsModel->setCreatedAt($createdAt);
        } else {
            $leadsModel->setUpdatedAt(Tools_System_Tools::convertDateFromTimezone('now'));
        }

        $leadsModel->setSkipCoordinatesGeneration(true);
        $leadsModel = $leadsMapper->save($leadsModel);

        if ($assignPrimaryContact === true && $organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
            $organizationModel->setSkipCoordinatesGeneration(true);
            $leadsOrganizationMapper->save($organizationModel->setPrimaryContactId($leadsModel->getId()));
        }

        return $leadsModel;
    }

    /**
     * Add + in case if not exists
     *
     * @param string $number country code value
     * @return string
     */
    public static function prepareCountryCodeValue($number)
    {
        if (!preg_match('~\+~', $number)) {
            $number = '+'.$number;
        }

        return $number;
    }

    /**
     * Update import leads status
     *
     * @param int $id import id number
     * @param string $importStatus import status
     * @param int $errorCode error code id
     * @param int $importItemId import id
     * @param string $importType import type
     */
    public static function updateImportStatus($id, $importStatus = '0', $errorCode = 0, $importItemId = 0, $importType = '')
    {
        $importDataTable = new Leads_DbTable_LeadsImportDataDbtable();
        $data = array('import_status' => $importStatus);
        if ($errorCode) {
            $data['error_code'] = $errorCode;
        }
        if (!empty($importItemId)) {
            $data['import_id'] = $importItemId;
        }
        if (!empty($importType)) {
            $data['import_type'] = $importType;
        }
        $where = $importDataTable->getAdapter()->quoteInto('id = ?', $id);
        $importDataTable->update($data, $where);
    }

    /**
     * Download csv
     *
     * @param string $filePath file path
     * @param string $fileName file name
     */
    public static function downloadCsv($filePath, $fileName)
    {
        $response = Zend_Controller_Front::getInstance()->getResponse();
        $zipFile = Tools_System_Tools::zip($filePath, $fileName);
        $response->setHeader(
            'Content-Disposition',
            'attachment; filename=' . Tools_Filesystem_Tools::basename($zipFile)
        )
            ->setHeader('Content-type', 'application/force-download');
        readfile($zipFile);
        $response->sendResponse();
        exit;
    }

    /**
     * Get import log by status id
     *
     * @param string $importStatus import status
     * @param bool $single return single record flag
     * @return array
     */
    public static function getImportByStatusCode($importStatus = '0', $single = false)
    {
        $dataImportDbTable = new Leads_DbTable_LeadsImportDataDbtable();

        $where = $dataImportDbTable->getAdapter()->quoteInto('import_status = ?', $importStatus);
        $select = $dataImportDbTable->getAdapter()->select()->from(self::PLUGIN_LEADS_IMPORT_TABLE)->where($where);
        if ($single) {
            return $dataImportDbTable->getAdapter()->fetchRow($select);
        }

        return $dataImportDbTable->getAdapter()->fetchAll($select);
    }

    /**
     * @param $data
     * @param $leadsIds
     */
    public static function prepareLeadsDataForExport($data, $leadsIds, $useCustomParams = false, $disableUseCustomParamsPrefix = false)
    {
        $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
        unset($data['name']);
        unset($data['run']);
        unset($data['leadIds']);
        unset($data['controller']);
        unset($data['action']);
        unset($data['filters']);
        $excludeFields = array();
        foreach ($data as $exportFieldName => $exportFieldValue) {
            if (!preg_match('~checked~', $exportFieldName)) {
                if ($exportFieldValue === '') {
                    $exportFieldValue = $exportFieldName;
                }
                if (!isset($data[$exportFieldName . '-checked'])) {
                    $excludeFields[$exportFieldName] = $exportFieldName;
                } else {
                    $renamedFields[] = $exportFieldValue;
                }
            }
        }

        $dataToExport = Leads_Mapper_LeadsMapper::getInstance()->fetchLeadsForExport($leadsIds, $excludeFields, $useCustomParams);

        if (!empty($dataToExport)) {
            if(!empty($useCustomParams) && !empty($leadsIds)) {
                $customParams = Tools_LeadExportTools::procesExportCustomParams($data, $dataToExport, $disableUseCustomParamsPrefix);

                if(!empty($customParams['headers']) && !empty($customParams['dataToExport'])) {
                    $renamedFields = array_merge($renamedFields, $customParams['headers']);
                    $dataToExport = $customParams['dataToExport'];
                }
            }

            $headers[] = $renamedFields;
            $fileName = 'leadlist.' . date("Y-m-d", time()) . '.csv';
            $filePath = $websiteHelper->getPath() . $websiteHelper->getTmp() . $fileName;
            $expFile = fopen($filePath, 'w');
            $dataToExport = array_merge($headers, $dataToExport);
            foreach ($dataToExport as $data) {
                fputcsv($expFile, $data, ',', '"');
            }
            fclose($expFile);
            Tools_LeadImportTools::downloadCsv($filePath, $fileName);
        }

    }

}
