<?php

/**
 * Class Tools_LeadInboundEmailTools
 */
class Tools_LeadInboundEmailTools
{

    /**
     * imap
     */
    const EMAIL_PROCESS_EMAILS_PATH = 'plugins/leads/system/cronscript/processImap.php';

    const EMAIL_PROCESS_HISTORICAL_EMAILS_PATH = 'plugins/leads/system/cronscript/processHistoricalDataImap.php';

    const LEAD_ATTRIBUTE_EMAIL = 'lead_email';

    const LEAD_ATTRIBUTE_FIRST_NAME = 'lead_first_name';

    const LEAD_ATTRIBUTE_LAST_NAME = 'lead_last_name';

    const LEAD_ATTRIBUTE_ADDRESS = 'lead_address';

    const LEAD_ATTRIBUTE_ADDRESS1 = 'lead_address1';

    const LEAD_ATTRIBUTE_CITY = 'lead_city';

    const LEAD_ATTRIBUTE_ZIP = 'lead_zip';

    const LEAD_ATTRIBUTE_MOBILE = 'lead_mobile';

    const LEAD_ATTRIBUTE_MOBILE_COUNTRY_CODE = 'lead_mobile_country_code';

    const LEAD_ATTRIBUTE_PHONE = 'lead_phone';

    const LEAD_ATTRIBUTE_PHONE_COUNTRY_CODE = 'lead_phone_country_code';

    const LEAD_ATTRIBUTE_COUNTRY_CODE = 'lead_country_code';

    const LEAD_ATTRIBUTE_COUNTRY_NAME = 'lead_country_name';

    const LEAD_ATTRIBUTE_STATE = 'lead_state';

    const LEAD_ATTRIBUTE_NOTES = 'lead_notes';

    const LEAD_ATTRIBUTE_POSITION = 'lead_position';

    const LEAD_ATTRIBUTE_SOCIAL_SERVICE_LINKEDIN = 'lead_social_service_linkedin';

    const LEAD_ATTRIBUTE_ORGANIZATION_NAME = 'lead_organization_name';

    const LEAD_ATTRIBUTE_ORGANIZATION_WEBSITE = 'lead_organization_website';

    const LEAD_ATTRIBUTE_ADDITIONAL_NOTE1 = 'lead_additional_note1';

    const LEAD_ATTRIBUTE_ADDITIONAL_NOTE2 = 'lead_additional_note2';

    const LEAD_ATTRIBUTE_ADDITIONAL_NOTE3 = 'lead_additional_note3';

    const LEAD_ATTRIBUTE_ADDITIONAL_NOTE4 = 'lead_additional_note4';

    const LEAD_ATTRIBUTE_ADDITIONAL_NOTE5 = 'lead_additional_note5';

    public static $_listOfAdditionalFieldsList = array(
        self::LEAD_ATTRIBUTE_ADDITIONAL_NOTE1,
        self::LEAD_ATTRIBUTE_ADDITIONAL_NOTE2,
        self::LEAD_ATTRIBUTE_ADDITIONAL_NOTE3,
        self::LEAD_ATTRIBUTE_ADDITIONAL_NOTE4,
        self::LEAD_ATTRIBUTE_ADDITIONAL_NOTE5
    );

    const LEAD_CUSTOM_ATTRIBUTES_PREFIX = 'lead_custom_';

    const LEAD_ORGANIZATION_CUSTOM_ATTRIBUTES_PREFIX = 'lead_organization_custom_';

    const LEAD_CUSTOM_ATTRIBUTE_TYPE_PREFIX = 'text_';

    const LEAD_CUSTOM_ATTRIBUTE_TYPE_PREFIX_SPECIALWORD = 'spat';

    /**
     * @param string $imapHost hostname
     * @param string $imapSsl ssl, tls
     * @param string $user user login
     * @param string $password user password
     * @param string $boxName user password
     * @return bool|resource
     */
    public static function getImapConnection($imapHost, $imapSsl, $user, $password, $boxName = '')
    {
        $errorTestConnection = 0;
        $errorTestConnectionString = '';
        $testConnection = Tools_LeadInboundEmailTools::testConnection($imapHost, $imapSsl,
            $errorTestConnection, $errorTestConnectionString, 5);
        if ($testConnection !== false) {
            $connection = Tools_LeadInboundEmailTools::connectionImap($imapHost, $imapSsl, $user, $password, OP_READONLY, 1, $boxName);
            return $connection;
        }

        return false;
    }

    /**
     * @param string $imapHost hostname
     * @param string $imapSsl ssl, tls
     * @param $errorTestConnection
     * @param $errorTestConnectionString
     * @param $timeout
     * @return bool
     */
    public static function testConnection($imapHost, $imapSsl, $errorTestConnection, $errorTestConnectionString, $timeout)
    {
        $sslTestConnection = '';
        $imapPort = '143';
        $connectTo = $imapHost;
        if (!empty($imapSsl) && (strtolower($imapSsl) === 'tls' || strtolower($imapSsl) === 'ssl')) {
            $sslTestConnection = strtolower($imapSsl) . '://';
            $imapPort = '993';
            $connectTo = $connectTo . ':993/imap/ssl/novalidate-cert';
        }

        $testConnectionHost = $sslTestConnection . $connectTo;

        return self::checkDomainConnection($testConnectionHost, $imapPort,
            $errorTestConnection, $errorTestConnectionString, $timeout);
    }


    /**
     * @param string $hostname hostname
     * @param string $port port
     * @param $errorTestConnection
     * @param $errorTestConnectionString
     * @param $timeout
     * @return bool
     */
    public static function checkDomainConnection(
        $hostname,
        $port,
        $errorTestConnection,
        $errorTestConnectionString,
        $timeout
    ) {
        $testConnection = fsockopen($hostname, $port, $errorTestConnection, $errorTestConnectionString, $timeout);
        if ($testConnection !== false) {
            return true;
        }

        return false;
    }

    /**
     * Connect via the imap
     *
     * @param string $imapHost connection host
     * @param string $imapSsl ssl, tls
     * @param string $user user login
     * @param string $password user password
     * @param int $openOptionType imap option type
     * @param int $retries number of retries
     * @param string $boxName email box name
     * @return bool|resource
     */
    public static function connectionImap($imapHost, $imapSsl, $user, $password, $openOptionType = OP_READONLY, $retries = 1, $boxName ='')
    {
        $connectTo = $imapHost;
        if (!empty($imapSsl) && (strtolower($imapSsl) === 'tls' || strtolower($imapSsl) === 'ssl')) {
            $connectTo = $connectTo . ':993/imap/ssl/novalidate-cert';
        }

        if (!empty($boxName)) {
            $connection = imap_open('{' . $connectTo . '}'.$boxName, $user, $password, $openOptionType, $retries);
        } else {
            $connection = imap_open('{' . $connectTo . '}', $user, $password, $openOptionType, $retries);
        }
        if ($connection !== false) {
            return $connection;
        }

        return false;
    }

    /**
     * fetch non deleted/draft email detailed info
     *
     * @param resource $connection connection
     * @param string $emailNumber email number
     * @param int $fetchMode fetch mode
     * @param bool $withEmailLogCheck with email log check
     * @param bool $withEmailAdditionalLogCheck with email additional log check
     * @return array
     */
    public static function fetchEmailDetailedInfo($connection, $emailNumber, $fetchMode = FT_UID, $withEmailLogCheck = false, $withEmailAdditionalLogCheck = false)
    {
        /* get information specific to this email */
        $overview = imap_fetch_overview($connection, $emailNumber, $fetchMode);
        $deleted = $overview[0]->deleted;
        $draft = $overview[0]->draft;
        $result = array();
        if (empty($deleted) && empty($draft)) {

            $result['messageId'] = $overview[0]->message_id;

            if ($withEmailLogCheck === true) {
                $leadsEmailLogModel = Leads_Mapper_LeadsEmailLogMapper::getInstance()->findByMessageId($result['messageId']);
                if ($leadsEmailLogModel instanceof Leads_Model_LeadsEmailLogModel) {
                    return array();
                }
            }

            if ($withEmailAdditionalLogCheck === true) {
                $leadsEmailLogModel = Leads_Mapper_LeadsAdditionalEmailLogMapper::getInstance()->findByMessageId($result['messageId']);
                if ($leadsEmailLogModel instanceof Leads_Model_LeadsAdditionalEmailLogModel) {
                    return array();
                }
            }

            //email parts structure
            $emailStructure = imap_fetchstructure($connection, $emailNumber, $fetchMode);
            if (empty($emailStructure)) {
                return array();
            }

            $partsData = array();

            if (empty($emailStructure->parts)) {
                $partsData = self::getEmailPartData($connection, $emailNumber, $emailStructure, 1, $fetchMode);
            } else {
                foreach ($emailStructure->parts as $partNumber => $emailStruct) {
                    $partNumber = $partNumber + 1;
                    $partData = self::getEmailPartData($connection, $emailNumber, $emailStruct, $partNumber,
                        $fetchMode);
                    if (!empty($partData['attachment'])) {
                        $partsData['attachment'][] = $partData['attachment'];
                    } else {
                        $partsData = $partsData + $partData;
                    }
                }
            }

            if (!empty($partsData['attachment'])) {
                foreach ($partsData['attachment'] as $file) {
                    //$ifp = fopen($file['name'], "wb");
                    //fwrite($ifp, $file['file']);
                    //fclose($ifp);
                }
            }

            /* output the email header information
             * With an email message that only has a text body and does not have any mime attachments, imap-fetchbody() will return the following for each requested part number:
             *  (empty) - Entire message
             *  0 - Message header
             *  1 - Body text

             *  With an email message that is a multi-part message in MIME format, and contains the message text in plain text and HTML, and has a file.ext attachment, imap-fetchbody() will return something like the following for each requested part number:

             * (empty) - Entire message
             * 0 - Message header
             * 1 - MULTIPART/ALTERNATIVE
             * 1.1 - TEXT/PLAIN
             * 1.2 - TEXT/HTML
             *  2 - file.ext

             * Now if you attach the above email to an email with the message text in plain text and HTML, imap_fetchbody() will use this type of part number system:

             * (empty) - Entire message
             * 0 - Message header
             * 1 - MULTIPART/ALTERNATIVE
             * 1.1 - TEXT/PLAIN
             * 1.2 - TEXT/HTML
             * 2 - MESSAGE/RFC822 (entire attached message)
             * 2.0 - Attached message header
             * 2.1 - TEXT/PLAIN
             * 2.2 - TEXT/HTML
             * 2.3 - file.ext
             *
             */

            if (!empty($partsData['HTML'])) {
                $emailBodyPart = '';
                $emailBody = preg_replace('~.*<body[^>]*>~uis','',$partsData['HTML']);
                if (preg_last_error() === PREG_BAD_UTF8_ERROR) {
                    $emailBodyPart = mb_ereg_replace('.*<body[^>]*>','',$partsData['HTML']);
                    $emailBody = trim(preg_replace('~<\/body>.*~uis', '', $emailBodyPart));
                } else {
                    $emailBody = trim(preg_replace('~<\/body>.*~uis', '', $emailBody));
                }

                if (preg_last_error() === PREG_BAD_UTF8_ERROR && !empty($emailBodyPart)) {
                    $emailBody = trim(mb_ereg_replace('<\/body>.*', '', $emailBodyPart));
                }
            } else {
                if (!empty($partsData['PLAIN'])) {
                    $emailBody = $partsData['PLAIN'];
                } else {
                    $emailBody = '';
                }

            }

            if (!empty($overview[0]->from)) {
                $firstName = self::prepareNameBasedOnEmailFrom($overview[0]->from);
            } else {
                $firstName = '';
            }


            $result['seen'] = $overview[0]->seen;
            if (!empty($overview[0]->subject)) {
                $result['subject'] = self::prepareSubject($overview[0]->subject);
            } else {
                $result['subject'] = '';
            }

            if (!empty($overview[0]->from)) {
                $result['from'] = self::prepareFromTo($overview[0]->from);
            } else {
                $result['from'] = '';
            }

            $result['to'] = self::prepareFromTo($overview[0]->to);
            $result['date'] = $overview[0]->date;
            $result['size'] = $overview[0]->size;
            $result['answered'] = $overview[0]->answered;
            $result['emailBody'] = $emailBody;
            $result['firstName'] = $firstName;
            if (!empty($overview[0]->in_reply_to)) {
                $result['inReply'] = $overview[0]->in_reply_to;
            } else {
                $result['inReply'] = '';
            }
            if (!empty($overview[0]->references)) {
                $result['reference'] = $overview[0]->references;
            } else {
                $result['reference'] = '';
            }

            $emailOriginDate =  $result['date'];
            $dt = new DateTime($emailOriginDate);

            $emailUtcDate = Tools_System_Tools::convertDateFromTimezone($result['date'], $dt->getTimezone()->getName(), 'UTC');
            $result['emailUtcDate'] = $emailUtcDate;
            $result['uid'] = $overview[0]->uid;
            $result['messageNumber'] =  $overview[0]->msgno;
        }

        return $result;
    }

    /**
     * Get email content data
     *
     * @param resource $connection connection
     * @param string $emailNumber email number
     * @param object $emailStructure email structure
     * @param  string $partNumber Ex: '1', '2', '2.1', '2.1.3', etc for multipart, 0 if simple
     * @param int $fetchMode fetch mode
     * @param array $partsData current message
     *
     * @return array
     */
    public static function getEmailPartData(
        $connection,
        $emailNumber,
        $emailStructure,
        $partNumber,
        $fetchMode = FT_UID,
        $partsData = array()
    ) {

        $type = $emailStructure->subtype;
        if ($partNumber) {
            $partsData[$type] = imap_fetchbody($connection, $emailNumber, $partNumber, $fetchMode);
        } else {
            $partsData[$type]  = imap_body($connection, $emailNumber);
        }

        $encoding = '';
        // DECODE DATA
        // Any part may be encoded, even plain text messages, so check everything.
        if ($emailStructure->encoding == ENCQUOTEDPRINTABLE) {
            $partsData[$type]  = quoted_printable_decode($partsData[$type]);
            if (isset($emailStructure->parameters)) {
                foreach ($emailStructure->parameters as $encodingParam) {
                    if ($encodingParam->attribute === 'charset') {
                        $encoding = $encodingParam->value;
                    }
                }
            }
            if (!empty($encoding) && strtoupper($encoding) !== 'UTF-8') {
                $partsData[$type] = iconv($encoding, 'UTF-8', $partsData[$type]);
            }

        } elseif ($emailStructure->encoding == ENCBASE64) {
            $partsData[$type]  = base64_decode($partsData[$type]);
        }

        // PARAMETERS
        // get all parameters, like charset, filenames of attachments, etc.
        if ($emailStructure->ifdparameters) {
            if ($emailStructure->dparameters) {
                foreach ($emailStructure->dparameters as $parameter) {
                    $params[strtolower($parameter->attribute)] = $parameter->value;
                }
            }
        }

        if ($emailStructure->ifparameters) {
            $params = array();
            if ($emailStructure->parameters) {
                foreach ($emailStructure->parameters as $parameter) {
                    $params[strtolower($parameter->attribute)] = $parameter->value;
                }
            }
        }


        // ATTACHMENT
        // Any part with a filename is an attachment,
        // so an attached text file (type 0) is not mistaken as the message.
        if (!empty($params['filename']) || !empty($params['name'])) {
            // filename may be given as 'Filename' or 'Name' or both'
            $params['file'] = $partsData[$type];
            unset($partsData[$type]);
            $partsData['attachment'] = $params;
        }

        // SUBPART RECURSION
        if (isset($emailStructure->parts)) {
            foreach ($emailStructure->parts as $currentPartNumber => $partChunkStructure) {
                $partsDataRes =  self::getEmailPartData($connection, $emailNumber, $partChunkStructure,
                    $partNumber . '.' . ($currentPartNumber + 1), $fetchMode);

                $partsData = $partsData + $partsDataRes;
            }  // 1.2, 1.2.1, etc.
        }

        return $partsData;
    }

    /**
     * Search by condition
     *
     * @param resource $connection connection
     * @param string $searchQuery query condition Ex: ALL UNDELETED SINCE
     * @param int $fetchMode fetch mode
     * @return array
     */
    public static function searchBy($connection, $searchQuery, $fetchMode = SE_UID)
    {
        $emails = imap_search($connection, $searchQuery, $fetchMode);

        return $emails;
    }

    /**
     * Process lead based on inbound email
     *
     * @param array $emailDetailedInfo email detailed info
     * @param int $accountId account id
     * @param int $ownerId owner id
     * @param string $status email status
     * @param bool $allEmails process all emails
     * @throws Zend_Db_Table_Exception
     *
     * @return Leads_Model_LeadsModel
     *
     */
    public static function processLead(array $emailDetailedInfo, $accountId, $ownerId = 0, $status = Leads_Model_LeadsAdditionalEmailLogModel::EMAIL_STATUS_RECEIVED, $allEmails = false)
    {
        if (empty($emailDetailedInfo['messageId'])) {
            return null;
        }

        $leadsEmailLogModel = Leads_Mapper_LeadsEmailLogMapper::getInstance()->findByMessageId($emailDetailedInfo['messageId']);
        if ($leadsEmailLogModel instanceof Leads_Model_LeadsEmailLogModel) {
            return null;
        }

        $actionType = Leads_Model_LeadsActionLogModel::ACTION_TYPE_INBOUND;
        $leadType = Leads_Model_LeadsTypeModel::LEAD_TYPE_INBOUND_EMAIL;
        if ($status === Leads_Model_LeadsAdditionalEmailLogModel::EMAIL_STATUS_SENT) {
            $leadType = Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_EMAIL;
            $actionType = Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND;
        }

        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
        $leadEmailFrom = $emailDetailedInfo['from'];
        if ($status === Leads_Model_LeadsAdditionalEmailLogModel::EMAIL_STATUS_SENT) {
            $leadEmail = $emailDetailedInfo['to'];
        } else {
            $leadEmail = $emailDetailedInfo['from'];
        }

        $countryCode = '';
        $leadPosition = '';
        $leadNotes = '';
        $leadSocialServicesInfo = array();
        $leadStateCode = '';

        if (!empty($emailDetailedInfo['lead_last_name'])) {
            $leadLastName = $emailDetailedInfo['lead_last_name'];
        }

        if (!empty($emailDetailedInfo['lead_zip'])) {
            $leadZip = $emailDetailedInfo['lead_zip'];
        }

        if (!empty($emailDetailedInfo['lead_address'])) {
            $leadAddress = $emailDetailedInfo['lead_address'];
        }

        if (!empty($emailDetailedInfo['lead_address1'])) {
            $leadAddress1 = $emailDetailedInfo['lead_address1'];
        }

        if (!empty($emailDetailedInfo['lead_city'])) {
            $leadCity = $emailDetailedInfo['lead_city'];
        }

        if (!empty($emailDetailedInfo['lead_position'])) {
            $leadPosition = $emailDetailedInfo['lead_position'];
        }

        if (!empty($emailDetailedInfo['lead_phone'])) {
            $leadPhone = $emailDetailedInfo['lead_phone'];
        }

        if (!empty($emailDetailedInfo['lead_phone_country_code'])) {
            $phoneCountryCode = $emailDetailedInfo['lead_phone_country_code'];
        }

        if (!empty($emailDetailedInfo['lead_mobile'])) {
            $leadMobile = $emailDetailedInfo['lead_mobile'];
        }

        if (!empty($emailDetailedInfo['lead_mobile_country_code'])) {
            $mobileCountryCode = $emailDetailedInfo['lead_mobile_country_code'];
        }

        if (!empty($emailDetailedInfo['lead_country_code'])) {
            $countryCode = $emailDetailedInfo['lead_country_code'];
        }

        if (!empty($emailDetailedInfo['lead_notes'])) {
            $leadNotes = $emailDetailedInfo['lead_notes'];
        }

        if (!empty($emailDetailedInfo['socialServicesInfo'])) {
            $leadSocialServicesInfo = $emailDetailedInfo['socialServicesInfo'];
        }

        if (!empty($emailDetailedInfo['lead_state_code'])) {
            $leadStateCode = $emailDetailedInfo['lead_state_code'];
        }

        $leadModel = $leadsMapper->findByEmail($leadEmail);
        if (empty($ownerId)) {
            $ownerId = Tools_LeadTools::getDefaultLeadOwner();
        }
        $leadTypeModel = Leads_Mapper_LeadsTypeMapper::getInstance()->findByTypeName($leadType);

        $configTable   = new Application_Model_DbTable_Config();
        $config = $configTable->selectConfig();
        $userDefaultMobileCountryCode = '';
        if (!empty($config['userDefaultPhoneMobileCode'])) {
            $userDefaultMobileCountryCode = $config['userDefaultPhoneMobileCode'];
        }

        if (empty($mobileCountryCode)) {
            $mobileCountryCode = $userDefaultMobileCountryCode;
        }

        if (empty($phoneCountryCode)) {
            $phoneCountryCode = $userDefaultMobileCountryCode;
        }

        $leadModelStatus = Tools_LeadFormTools::LEAD_MODEL_STATUS_OLD;
        $mobileCountryCodeValue = Tools_LeadTools::getCountryCodeValue($mobileCountryCode);
        $phoneCountryCodeValue = Tools_LeadTools::getCountryCodeValue($phoneCountryCode);
        $createdAt = Tools_System_Tools::convertDateFromTimezone('now');
        $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();

        if (!empty($shoppingConfig['country'])) {
            if (empty($countryCode)) {
                $countryCode = $shoppingConfig['country'];
            }
        }

        $leadModelStatus = Tools_LeadFormTools::LEAD_MODEL_STATUS_OLD;

        if (!$leadModel instanceof Leads_Model_LeadsModel) {
            if ($status === Leads_Model_LeadsAdditionalEmailLogModel::EMAIL_STATUS_SENT) {
                return false;
            }
            if ($allEmails === false && empty($emailDetailedInfo['leadEmailFound'])) {
                return false;
            }
            $leadModelStatus = Tools_LeadFormTools::LEAD_MODEL_STATUS_NEW;
            $leadAdditionAvailable = Tools_LeadLicenseTools::checkLimit();
            if ($leadAdditionAvailable['error'] === true) {
                $leadModel = new Leads_Model_LeadsModel();
                $leadModel->registerObserver(new Tools_Mail_Watchdog(array(
                    'trigger' => Tools_LeadsMailWatchdog::LEADS_LICENSE_LIMIT
                )));

                $leadModel->notifyObservers();

                return '';
            }
            $leadTypeId = Tools_LeadTools::getLeadTypeIdByLeadTypeName($leadType);
            $leadModel = new Leads_Model_LeadsModel();
            $leadModel->setCreatedAt($createdAt);
            $leadModel->setLeadEmail($leadEmail);
            $leadModel->setLeadFirstName($emailDetailedInfo['firstName']);
            if (!empty($leadLastName)) {
                $leadModel->setLeadLastName($leadLastName);
            }
            if (!empty($leadZip)) {
                $leadModel->setLeadZip($leadZip);
            }
            if (!empty($leadAddress)) {
                $leadModel->setLeadAddress($leadAddress);
            }
            if (!empty($leadAddress1)) {
                $leadModel->setLeadAddress1($leadAddress1);
            }
            if (!empty($leadCity)) {
                $leadModel->setLeadCity($leadCity);
            }
            if (!empty($leadPosition)) {
                $leadModel->setLeadPosition($leadPosition);
            }
            if (!empty($leadMobile)) {
                $leadModel->setLeadMobile($leadMobile);
            }

            if (!empty($leadPhone)) {
                $leadModel->setLeadPhone($leadPhone);
            }

            if (!empty($leadNotes)) {
                $leadModel->setLeadNotes($leadNotes);
            }

            if (!empty($leadPosition)) {
                $leadModel->setLeadPosition($leadPosition);
            }

            if (!empty($leadStateCode)) {
                $leadModel->setLeadStateCode($leadStateCode);
            }

            $leadModel->setLeadMobileCountryCode($mobileCountryCode);
            $leadModel->setLeadMobileCountryCodeValue($mobileCountryCodeValue);
            $leadModel->setLeadPhoneCountryCode($phoneCountryCode);
            $leadModel->setLeadPhoneCountryCodeValue($phoneCountryCodeValue);
            $leadModel->setLeadCountryCode($countryCode);
            $leadModel->setOwnerId($ownerId);
            $leadModel->setLeadTypeId($leadTypeId);
        } else {
            $currentFirstName = $leadModel->getLeadFirstName();
            $currentLeadLastName = $leadModel->getLeadLastName();
            $currentMobile = $leadModel->getLeadMobile();
            $currentMobileCountryCode = $leadModel->getLeadMobileCountryCode();
            $currentPhone = $leadModel->getLeadPhone();
            $currentZip = $leadModel->getLeadZip();
            $currentAddress = $leadModel->getLeadAddress();
            $currentPosition = $leadModel->getLeadPosition();
            $currentPrefix = $leadModel->getPrefix();
            $currentCity = $leadModel->getLeadCity();
            $currentCountry = $leadModel->getLeadCountryCode();
            $currentState = $leadModel->getLeadStateCode();
            $currentAddress1 = $leadModel->getLeadAddress1();
            $currentCustomerGroupId = $leadModel->getCustomerGroupId();
            $currentLeadNotes = $leadModel->getLeadNotes();
            $currentLeadPosition = $leadModel->getLeadPosition();
            if (empty($currentFirstName)) {
                $leadModel->setLeadFirstName($emailDetailedInfo['firstName']);
            }
            if (empty($currentLeadLastName) && !empty($leadLastName)) {
                $leadModel->setLeadLastName($leadLastName);
            }
            if (empty($currentZip) && !empty($leadZip)) {
                $leadModel->setLeadZip($leadZip);
            }
            if (empty($currentAddress) && !empty($leadAddress)) {
                $leadModel->setLeadAddress($leadAddress);
            }
            if (empty($currentAddress1) && !empty($leadAddress1)) {
                $leadModel->setLeadAddress1($leadAddress1);
            }
            if (empty($currentCity) && !empty($leadCity)) {
                $leadModel->setLeadCity($leadCity);
            }
            if (empty($currentPosition) && !empty($leadPosition)) {
                $leadModel->setLeadPosition($leadPosition);
            }
            if (empty($currentMobile) && !empty($leadMobile)) {
                $leadModel->setLeadMobile($leadMobile);
            }
            if (empty($currentPhone) && !empty($leadPhone)) {
                $leadModel->setLeadPhone($leadPhone);
            }
            if (empty($currentMobileCountryCode) && !empty($mobileCountryCode)) {
                $leadModel->setLeadMobileCountryCode($mobileCountryCode);
                $leadModel->setLeadMobileCountryCodeValue($mobileCountryCodeValue);
            }
            if (empty($currentPhoneCountryCode) && !empty($phoneCountryCode)) {
                $leadModel->setLeadPhoneCountryCode($phoneCountryCode);
                $leadModel->setLeadPhoneCountryCodeValue($phoneCountryCodeValue);
            }
            if (empty($currentCountry) && !empty($countryCode)) {
                $leadModel->setLeadCountryCode($countryCode);
            }
            if (empty($currentLeadNotes) && !empty($leadNotes)) {
                $leadModel->setLeadNotes($leadNotes);
            }
            if (empty($currentLeadPosition) && !empty($leadPosition)) {
                $leadModel->setLeadPosition($leadPosition);
            }
            if (empty($currentState) && !empty($leadStateCode)) {
                $leadModel->setLeadStateCode($leadStateCode);
            }

        }

        if ($leadTypeModel instanceof Leads_Model_LeadsTypeModel) {
            $leadModel->setLastActivityTypeId($leadTypeModel->getId());
            $leadModel->setLastActivitySubtype('');
            if ($leadType === Leads_Model_LeadsTypeModel::LEAD_TYPE_INBOUND_EMAIL) {
                $leadModel->setLastActivity($emailDetailedInfo['emailUtcDate']);
            } else {
                $leadModel->setLastActivityOutbound($emailDetailedInfo['emailUtcDate']);
            }
        }

        try {
            $leadModel = $leadsMapper->save($leadModel);
        } catch (Exception $e) {
            return false;
        }

        $leadId = $leadModel->getId();

        if (!empty($leadSocialServicesInfo)) {
            $currentSocialServices = $leadsMapper->getLeadSocialServices($leadId);
            if (!empty($currentSocialServices)) {
                foreach ($leadSocialServicesInfo as $leadSocialServiceInfo) {
                    if (!array_key_exists($leadSocialServiceInfo['socialServiceId'], $currentSocialServices)) {
                        $leadsMapper->addSocialService($leadId, $leadSocialServiceInfo['socialServiceId'], $leadSocialServiceInfo['socialServiceValue']);
                    }
                }
            } else {
                foreach ($leadSocialServicesInfo as $leadSocialServiceInfo) {
                    $leadsMapper->addSocialService($leadId, $leadSocialServiceInfo['socialServiceId'], $leadSocialServiceInfo['socialServiceValue']);
                }
            }
        }

        $leadOrganizationModel = null;
        if (!empty($emailDetailedInfo['lead_organization_name'])) {
            $leadOrganizationModel = self::processOrganization($emailDetailedInfo, $leadModel);
            if ($leadOrganizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                $leadModel->setOrganizationId($leadOrganizationModel->getId());
                $leadModel = $leadsMapper->save($leadModel);
            }
        }

        if (!empty($emailDetailedInfo['additionalNotes'])) {
            self::processAdditionalLeadNotes($emailDetailedInfo['additionalNotes'], $leadModel);
        }

        if (!empty($emailDetailedInfo['customLeadAttributes'])) {
            self::processLeadCustomAttributes($emailDetailedInfo['customLeadAttributes'], $leadModel);
        }

        if (!empty($emailDetailedInfo['customLeadOrganizationAttributes']) && $leadOrganizationModel instanceof Leads_Model_LeadsOrganizationModel) {
            self::processLeadOrganizationCustomAttributes($emailDetailedInfo['customLeadOrganizationAttributes'], $leadOrganizationModel);
        }

        $leadsActionLogModel = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_EMAIL, $leadId,
            $actionType, $emailDetailedInfo['emailUtcDate'], $leadType);

        $pluginLeadsEmailLogMapper = Leads_Mapper_LeadsEmailLogMapper::getInstance();
        $leadsEmailLogModel = new Leads_Model_LeadsEmailLogModel();
        $leadsEmailLogModel->setLeadId($leadId);
        $leadsEmailLogModel->setActionLogId($leadsActionLogModel->getId());
        $leadsEmailLogModel->setContent('');
        if (!empty($emailDetailedInfo['originalEmailFrom'])) {
            $leadsEmailLogModel->setEmailFrom($emailDetailedInfo['originalEmailFrom']);
        } else {
            $leadsEmailLogModel->setEmailFrom($leadEmailFrom);
        }
        $leadsEmailLogModel->setEmailFromName($leadEmailFrom);
        $leadsEmailLogModel->setEmailTo($emailDetailedInfo['to']);
        $leadsEmailLogModel->setEmailSubject($emailDetailedInfo['subject']);
        $leadsEmailLogModel->setEmailTemplate('');
        $leadsEmailLogModel->setEncodedContent($emailDetailedInfo['emailBody']);
        $leadsEmailLogModel->setTrackingCode('');
        $leadsEmailLogModel->setLeadId($leadId);
        $leadsEmailLogModel->setEmailBcc('');
        if ($status === Leads_Model_LeadsAdditionalEmailLogModel::EMAIL_STATUS_SENT) {
            $leadsEmailLogModel->setCreatorId($ownerId);
        }
        $leadsEmailLogModel->setEmailStatus($status);
        $leadsEmailLogModel->setEmailSourceType(Leads_Model_LeadsEmailLogModel::EMAIL_SOURCE_TYPE_EXTERNAL);
        $leadsEmailLogModel->setMessageId($emailDetailedInfo['messageId']);
        $leadsEmailLogModel->setReplyId($emailDetailedInfo['inReply']);
        $leadsEmailLogModel->setReferences($emailDetailedInfo['reference']);
        $leadsEmailLogModel->setAccountId($accountId);
        $leadsEmailLogModel->setUid($emailDetailedInfo['uid']);
        $leadsEmailLogModel->setMessageNumber($emailDetailedInfo['messageNumber']);
        $leadsEmailLogModel->setReceivedAt($emailDetailedInfo['emailUtcDate']);
        $pluginLeadsEmailLogMapper->save($leadsEmailLogModel);

        if ($leadModelStatus === Tools_LeadFormTools::LEAD_MODEL_STATUS_NEW) {
            $leadWorkflowGeneralConfig = Leads_Mapper_LeadsWorkflowGeneralConfigMapper::getInstance()->getScoreRulesConfigData();
            if (!empty($leadWorkflowGeneralConfig)) {
                $where = $leadsMapper->getDbTable()->getAdapter()->quoteInto('pld.id = ?', $leadId);
                $leadInfo = $leadsMapper->fetchAll($where, null, null, null, true);
                if (!empty($leadInfo)) {
                    $leadInfo[0]['isImap'] = '1';
                    $leadInfo[0]['email_message_id'] = $emailDetailedInfo['messageId'];
                    $workflowGeneralConfigGroupedByRule = Tools_LeadScoreWorkflowTools::prepareScoreRules($leadWorkflowGeneralConfig);
                    Tools_LeadScoreWorkflowTools::processWorkflowScore($leadInfo, $leadWorkflowGeneralConfig,
                        $workflowGeneralConfigGroupedByRule);
                }
            }
        }

        return $leadModel;
    }

    /**
     * Process lead custom attributes
     *
     * @param array $customLeadParams
     * @param Leads_Model_LeadsModel $leadModel
     */
    public static function processLeadCustomAttributes($customLeadParams, Leads_Model_LeadsModel $leadModel)
    {
        if (!empty($customLeadParams)) {
            $leadId = $leadModel->getId();
            $leadsCustomParamsConfigMapper = Leads_Mapper_LeadsCustomParamsConfigMapper::getInstance();
            $leadsCustomParamsMapper = Leads_Mapper_LeadsCustomParamsMapper::getInstance();
            foreach ($customLeadParams as $customLeadParam) {
                $attributeName = $customLeadParam['customLeadAttribute'];
                $attributeValue = $customLeadParam['customLeadAttributeValue'];
                $attributeType = $customLeadParam['customLeadAttributeType'];
                $leadsCustomParamsConfigModel = $leadsCustomParamsConfigMapper->getByTypeName($attributeType, $attributeName);
                if ($leadsCustomParamsConfigModel instanceof Leads_Model_LeadsCustomParamsConfigModel) {
                    $paramId = $leadsCustomParamsConfigModel->getId();
                    $leadsCustomParamsModel = $leadsCustomParamsMapper->getParamByIdAndParamId($leadId, $paramId);
                    if (!$leadsCustomParamsModel instanceof Leads_Model_LeadsCustomParamsModel) {
                        $leadsCustomParamsModel = new Leads_Model_LeadsCustomParamsModel();
                        $leadsCustomParamsModel->setLeadId($leadId);
                        $leadsCustomParamsModel->setParamId($paramId);
                    } else {
                        $currentParamValue = $leadsCustomParamsModel->getParamValue();
                        if (!empty($currentParamValue)) {
                            continue;
                        }
                    }

                    $leadsCustomParamsModel->setParamValue($attributeValue);
                    $leadsCustomParamsMapper->save($leadsCustomParamsModel);
                }
            }
        }
    }

    /**
     * Process lead organization custom attributes
     *
     * @param array $customLeadOrganizationParams
     * @param Leads_Model_LeadsOrganizationModel $leadOrganizationModel
     */
    public static function processLeadOrganizationCustomAttributes(
        $customLeadOrganizationParams,
        Leads_Model_LeadsOrganizationModel $leadOrganizationModel
    ) {
        if (!empty($customLeadOrganizationParams)) {
            $organizationId = $leadOrganizationModel->getId();
            $leadsOrganizationCustomParamsConfigMapper = Leads_Mapper_LeadsOrganizationCustomParamsConfigMapper::getInstance();
            $leadsOrganizationCustomParamsMapper = Leads_Mapper_LeadsOrganizationCustomParamsMapper::getInstance();
            foreach ($customLeadOrganizationParams as $customLeadOrganizationParam) {
                $attributeName = $customLeadOrganizationParam['customLeadOrganizationAttribute'];
                $attributeValue = $customLeadOrganizationParam['customLeadOrganizationAttributeValue'];
                $attributeType = $customLeadOrganizationParam['customLeadAttributeType'];
                $leadsOrganizationCustomParamsConfigModel = $leadsOrganizationCustomParamsConfigMapper->getByTypeName($attributeType, $attributeName);
                if ($leadsOrganizationCustomParamsConfigModel instanceof Leads_Model_LeadsOrganizationCustomParamsConfigModel) {
                    $paramId = $leadsOrganizationCustomParamsConfigModel->getId();
                    $leadsOrganizationCustomParamsModel = $leadsOrganizationCustomParamsMapper->getParamByIdAndParamId($organizationId, $paramId);
                    if (!$leadsOrganizationCustomParamsModel instanceof Leads_Model_LeadsOrganizationCustomParamsModel) {
                        $leadsOrganizationCustomParamsModel = new Leads_Model_LeadsOrganizationCustomParamsModel();
                        $leadsOrganizationCustomParamsModel->setOrganizationId($organizationId);
                        $leadsOrganizationCustomParamsModel->setParamId($paramId);
                    } else {
                        $currentParamValue = $leadsOrganizationCustomParamsModel->getParamValue();
                        if (!empty($currentParamValue)) {
                            continue;
                        }
                    }

                    $leadsOrganizationCustomParamsModel->setParamValue($attributeValue);
                    $leadsOrganizationCustomParamsMapper->save($leadsOrganizationCustomParamsModel);
                }
            }
        }
    }

    /**
     * Process additional lead notes
     *
     * @param array $leadNotes
     * @param Leads_Model_LeadsModel $leadModel
     * @throws Zend_Db_Table_Exception
     */
    public static function processAdditionalLeadNotes($leadNotes, Leads_Model_LeadsModel $leadModel)
    {
        if (!empty($leadNotes)) {
            $leadsNoteMapper = Leads_Mapper_LeadsNoteMapper::getInstance();
            $leadId = $leadModel->getId();
            $defaultLeadOwnerId = Tools_LeadTools::getDefaultLeadOwner();
            foreach ($leadNotes as $leadNote) {
                $leadNoteModel = new Leads_Model_LeadsNoteModel();
                $leadNoteModel->setUserId($defaultLeadOwnerId);
                $leadNoteModel->setLeadId($leadId);
                $leadNoteModel->setDateCreated(Tools_System_Tools::convertDateFromTimezone('now'));
                $leadNoteModel->setNote($leadNote['additionalNote']);
                $leadsNoteMapper->save($leadNoteModel);
            }
        }
    }

    /**
     * Process organization based on IMAP emails rules data
     *
     * @param array $data
     * @param Leads_Model_LeadsModel $leadModel
     * @return Leads_Model_LeadsOrganizationModel $leadOrganizationModel
     */
    public static function processOrganization($data, Leads_Model_LeadsModel $leadModel)
    {
        $leadsOrganizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
        if (!empty($data['lead_organization_name'])) {
            $organizationName = filter_var($data['lead_organization_name'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES);
            $organizationModel = $leadsOrganizationMapper->findByName($organizationName);
            $organizationWebsite = '';
            if (!empty($data['lead_organization_website'])) {
                $organizationWebsite = $data['lead_organization_website'];
            }

            $leadOrganizationId = $leadModel->getOrganizationId();
            if ($organizationModel instanceof Leads_Model_LeadsOrganizationModel && !empty($leadOrganizationId)) {
                if ($organizationModel->getId() != $leadOrganizationId) {
                    return false;
                }
            }

            $currentDate = Tools_System_Tools::convertDateFromTimezone('now');
            $countryCode = '';

            if (!$organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                $organizationModel = new Leads_Model_LeadsOrganizationModel();
                $organizationModel->setOrganizationName($organizationName);
                if (!empty($organizationWebsite)) {
                    $organizationModel->setOrganizationWebsite($organizationWebsite);
                    $domain = Tools_LeadTools::getDomainFromUrl($organizationWebsite);
                    if (!empty($domain)) {
                        $organizationDomain = $domain;
                    } else {
                        $organizationDomain = '';
                    }
                    if (!empty($domain)) {
                        $organizationModel->setOrganizationDomain($organizationDomain);
                    }
                }
                $organizationModel->setCreatedAt($currentDate);
            } else {
                $currentOrganizationWebsite = $organizationModel->getOrganizationWebsite();
                if (empty($currentOrganizationWebsite) && !empty($organizationWebsite)) {
                    $organizationModel->setOrganizationWebsite($organizationWebsite);
                }

                $organizationModel->setUpdatedAt($currentDate);
            }

            if (empty($organizationModel->getPrimaryContactId())) {
                $organizationModel->setPrimaryContactId($leadModel->getId());
                $organizationModel->setFirstContactCreationDate($currentDate);
            }

            $configTable   = new Application_Model_DbTable_Config();
            $config = $configTable->selectConfig();
            $mobileCountryCode = '';
            if (!empty($config['userDefaultPhoneMobileCode'])) {
                $mobileCountryCode = $config['userDefaultPhoneMobileCode'];
            }

            $mobileCountryCodeValue = Tools_LeadTools::getCountryCodeValue($mobileCountryCode);
            $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();

            if (!empty($shoppingConfig['country'])) {
                if (empty($countryCode)) {
                    $countryCode = $shoppingConfig['country'];
                }
            }

            $organizationModel->setOrganizationCountryCode($countryCode);
            $organizationModel->setOrganizationPhoneCountryCode($mobileCountryCode);
            $organizationModel->setOrganizationPhoneCountryCodeValue($mobileCountryCodeValue);
            $organizationModel->setOrganizationMobileCountryCode($mobileCountryCode);
            $organizationModel->setOrganizationMobileCountryCodeValue($mobileCountryCodeValue);

            $leadOrganizationModel = $leadsOrganizationMapper->save($organizationModel);

            return $leadOrganizationModel;
        }
    }

    /**
     * Process lead based on inbound email
     *
     * @param array $emailDetailedInfo email detailed info
     * @param int $accountId account id
     * @param int $ownerId owner id
     * @param string $emailStatus email status
     * @throws Zend_Db_Table_Exception
     *
     * @return Leads_Model_LeadsAdditionalEmailLogModel
     *
     */
    public static function processAdditionalEmailLog(array $emailDetailedInfo, $accountId, $ownerId = 0, $emailStatus = Leads_Model_LeadsAdditionalEmailLogModel::EMAIL_STATUS_RECEIVED)
    {
        if (empty($emailDetailedInfo['messageId'])) {
            return null;
        }

        $leadsEmailLogModel = Leads_Mapper_LeadsAdditionalEmailLogMapper::getInstance()->findByMessageId($emailDetailedInfo['messageId']);
        if ($leadsEmailLogModel instanceof Leads_Model_LeadsAdditionalEmailLogModel) {
            return null;
        }

        $leadEmail = $emailDetailedInfo['from'];
        if (empty($ownerId)) {
            $ownerId = Tools_LeadTools::getDefaultLeadOwner();
        }

        $pluginLeadsAdditionalEmailLogMapper = Leads_Mapper_LeadsAdditionalEmailLogMapper::getInstance();
        $leadsAdditionalEmailLogModel = new Leads_Model_LeadsAdditionalEmailLogModel();
        $leadsAdditionalEmailLogModel->setContent('');
        $leadsAdditionalEmailLogModel->setEmailFrom($leadEmail);
        $leadsAdditionalEmailLogModel->setEmailFromName($leadEmail);
        $leadsAdditionalEmailLogModel->setEmailTo($emailDetailedInfo['to']);
        $leadsAdditionalEmailLogModel->setEmailSubject($emailDetailedInfo['subject']);
        $leadsAdditionalEmailLogModel->setEmailBcc('');
        $leadsAdditionalEmailLogModel->setEmailStatus($emailStatus);
        $leadsAdditionalEmailLogModel->setEmailSourceType(Leads_Model_LeadsAdditionalEmailLogModel::EMAIL_SOURCE_TYPE_EXTERNAL);
        $leadsAdditionalEmailLogModel->setMessageId($emailDetailedInfo['messageId']);
        $leadsAdditionalEmailLogModel->setReplyId($emailDetailedInfo['inReply']);
        $leadsAdditionalEmailLogModel->setReferences($emailDetailedInfo['reference']);
        $leadsAdditionalEmailLogModel->setAccountId($accountId);
        $leadsAdditionalEmailLogModel->setUid($emailDetailedInfo['uid']);
        $leadsAdditionalEmailLogModel->setMessageNumber($emailDetailedInfo['messageNumber']);
        $leadsAdditionalEmailLogModel->setReceivedAt($emailDetailedInfo['emailUtcDate']);
        $leadsAdditionalEmailLogModel->setEncodedContent($emailDetailedInfo['emailBody']);
        $pluginLeadsAdditionalEmailLogMapper->save($leadsAdditionalEmailLogModel);

        return $leadsAdditionalEmailLogModel;
    }


    /**
     * Alternative way to search
     * Imap extension doesn't provide ability by HEADERS
     * Use this function if you need to get email by header
     * or any other non imap native extension search
     *
     * @param string $host host
     * @param string $port port
     * @param string $user user name
     * @param string $password password
     * @param string $query search query without SEARCH word!!! Ex: HEADER Message-ID "<CA+9oPU3+ChB9h0i_FW+DVrFzPCujOmpV+mLap4F69b9MpdQUHg@mail.gmail.com>"
     * @param bool $ssl flag
     * @return array
     * @throws Zend_Mail_Protocol_Exception
     */
    public static function findByAlternative($host, $port, $user, $password, $query, $ssl = true, $box = 'INBOX')
    {
        $zendImap = new Zend_Mail_Protocol_Imap(
            $host,
            $port,
            $ssl
        );
        $zendImap->login($user, $password);
        $zendImap->select($box);
        $res = $zendImap->search(array($query));
        if (empty($res)) {
            return array();
        }

        return $res;
    }

    /**
     * Prepare email subject
     *
     * @param string $subject email subject
     * @return string
     */
    public static function prepareSubject($subject)
    {
        return imap_utf8($subject);
    }

    /**
     * Prepare email from to (clean from symbols)
     *
     * @param string $message
     * @return string
     */
    public static function prepareFromTo($message)
    {
        return strtolower(str_replace(array('<', '>'), '', preg_replace('~.*<~', '', $message)));
    }

    /**
     * Prepare name based on the email from part
     *
     * @param $from
     * @return mixed|string
     */
    public static function prepareNameBasedOnEmailFrom($from)
    {
        preg_match('~(.*<)~ui', $from, $name);

        if (!empty($name) && !empty($name[0])) {
            $name = str_replace('<', '', $name['0']);
        }

        if (empty($name)) {
            $name = 'Not provided';
        }

        return imap_utf8($name);
    }

    /**
     * AES key
     *
     * @return string
     */
    public static function getPasswordKey()
    {
        return strrev(md5(strrev(base64_encode('Leads'))));
    }

    /**
     * @param string $emailReplyId email reply id
     * @return array
     */
    public static function getEmailConversationDetails($emailReplyId)
    {
        $pluginLeadsEmailLogMapper = Leads_Mapper_LeadsEmailLogMapper::getInstance();
        $emailLogModel = $pluginLeadsEmailLogMapper->findByReplyId($emailReplyId);
        if (!$emailLogModel instanceof Leads_Model_LeadsEmailLogModel) {
            return array();
        }

        $accountId = $emailLogModel->getAccountId();
        $replyId = $emailLogModel->getReplyId();
        $emails =  array();
        for ($i = 1; $i < 10000; $i++) {
            $emailInfo = self::getOriginalReplyEmailInfo($replyId, $accountId);
            if (!empty($emailInfo) && !empty($emailInfo['emailFrom'])) {
                $emailInfo['emailDate'] = Tools_System_Tools::convertDateFromTimezone($emailInfo['emailDate']);
                $emails[] = $emailInfo;
            }
            if (empty($emailInfo['replyId'])) {
                break;
            } else {
                $replyId = $emailInfo['replyId'];
            }
        }

        return $emails;
    }

    /**
     * @param string $emailReplyId email reply id
     * @param int $accountId account id
     * @return array
     */
    public static function getOriginalReplyEmailInfo($emailReplyId, $accountId)
    {
        $pluginLeadsEmailLogMapper = Leads_Mapper_LeadsEmailLogMapper::getInstance();
        $emailLogData = $pluginLeadsEmailLogMapper->findByMessageIdDecoded($emailReplyId);
        if (empty($emailLogData)) {
            $emailLogData = Leads_Mapper_LeadsAdditionalEmailLogMapper::getInstance()->findByMessageIdDecoded($emailReplyId);
        }

        if (empty($emailLogData)) {
            if (empty($accountId)) {
                return array();
            }

            $leadImapDataMapper = Leads_Mapper_LeadsImapConfigMapper::getInstance();
            $where = $leadImapDataMapper->getDbTable()->getAdapter()->quoteInto('id = ?', $accountId);
            $configData = $leadImapDataMapper->fetchAllData($where, null, null, null, true, true);
            if (empty($configData)) {
                return array();
            }

            $imapPassword = $configData['password'];
            $imapHost = $configData['host'];
            $imapSsl = $configData['encryption'];
            $user = $configData['login'];
            $connection = self::getImapConnection($imapHost, $imapSsl, $user, $imapPassword);
            if ($connection === false) {
                return array();
            }

            $imapPort = '143';
            $ssl = false;
            if (!empty($imapSsl) && (strtolower($imapSsl) === 'tls' || strtolower($imapSsl) === 'ssl')) {
                $imapPort = '993';
                $ssl = true;
            }

            $query = 'HEADER Message-ID "'.$emailReplyId.'"';
            try {
                $emailExists = self::findByAlternative($imapHost, $imapPort, $user, $imapPassword, $query, $ssl);

                if (!empty($emailExists)) {
                    $roughEmailInfo = self::fetchEmailDetailedInfo($connection, imap_uid($connection , $emailExists[0]));
                }

                if (!empty($roughEmailInfo)) {
                    return array(
                        'subject' => $roughEmailInfo['subject'],
                        'emailFrom' => $roughEmailInfo['from'],
                        'emailContent' => $roughEmailInfo['emailBody'],
                        'emailMessageId' => $roughEmailInfo['messageId'],
                        'emailDate' => $roughEmailInfo['emailUtcDate'],
                        'replyId' => $roughEmailInfo['inReply']
                    );
                }

                return array();
            } catch (Exception $e) {
                return array();
            }
        }


        return array(
            'subject' => $emailLogData['email_subject'],
            'emailFrom' => $emailLogData['email_from'],
            'emailContent' => base64_decode($emailLogData['encoded_content']),
            'emailMessageId' => $emailLogData['message_id'],
            'emailDate' => $emailLogData['received_at'],
            'replyId' => $emailLogData['reply_id']
        );


    }

    /**
     * Get last imap error
     *
     * @return string
     */
    public static function getImapError()
    {
        return imap_last_error();
    }

    /**
     *
     * Check whether imap extension installed
     *
     * @return bool
     */
    public static function checkImapExtension()
    {
        if (!extension_loaded('imap')) {
            return false;
        }

        return true;
    }


    public static function unsubscribeSequence($email)
    {
        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
        $leadModel = $leadsMapper->findByEmail($email);
        if ($leadModel instanceof Leads_Model_LeadsModel) {
            $leadsHasEmailSequenceMapper = Leads_Mapper_LeadsHasEmailSequenceMapper::getInstance();
            $leadsHasEmailSequenceData = $leadsHasEmailSequenceMapper->getLeadSequenceByLeadId($leadModel->getId());
            if (!empty($leadsHasEmailSequenceData)) {
                foreach ($leadsHasEmailSequenceData as $assignedSequence) {
                    $leadsHasEmailSequenceMapper->deleteFromCartStatusQueue($leadModel->getId(), 'lead',
                        $assignedSequence['sequence_id'], '0');
                }
            }
        }
    }


    public static function prepareSubRuleLeadData($accountId, $emailDetailedInfo)
    {
        $leadsImapRulesConfigMapper = Leads_Mapper_LeadsImapRulesConfigMapper::getInstance();

        $where = $leadsImapRulesConfigMapper->getDbTable()->getAdapter()->quoteInto('plirc.imap_config_id', $accountId);
        $order = 'rule_email DESC';
        $leadsImapRulesConfigData = $leadsImapRulesConfigMapper->fetchAll($where, $order, null, null, true);

//        if (!$leadsImapRulesConfigModel instanceof Leads_Model_LeadsImapRulesConfigModel) {
//            return $emailDetailedInfo;
//        }

        if (empty($leadsImapRulesConfigData)) {
            return $emailDetailedInfo;
        }

        $resultForEmailDetails = array();

        foreach ($leadsImapRulesConfigData as $leadsImapRulesConfig) {

            $ruleEmail = $leadsImapRulesConfig['rule_email'];

            if (!empty($ruleEmail) && $ruleEmail !== $emailDetailedInfo['from']) {
                continue;
            }

            $subRulesConfig = $leadsImapRulesConfig['subRulesConfig'];
            if (empty($subRulesConfig)) {
                continue;
            }

            $subRulesConfig = json_decode($subRulesConfig, true);

            if (empty($subRulesConfig)) {
                continue;
            }

            $emailBody = $emailDetailedInfo['emailBody'];

            $translator = Zend_Registry::get('Zend_Translate');

            foreach ($subRulesConfig as $subRuleConfig) {
                $parseResult = self::parseEmailContentByRules($subRuleConfig, $emailBody);
                if (!empty($parseResult)) {
                    $resultForEmailDetails[$parseResult['name']] = $parseResult['value'];
                    if ($subRuleConfig['assignToAttribute'] === self::LEAD_ATTRIBUTE_EMAIL) {
                        $resultForEmailDetails['originalEmailFrom'] = $emailDetailedInfo['from'];
                        $resultForEmailDetails['from'] = $parseResult['value'];
                        if (!empty($parseResult['additionalParams'])) {
                            $resultForEmailDetails['leadEmailFound'] = '1';
                        }
                    }

                    if ($subRuleConfig['assignToAttribute'] === self::LEAD_ATTRIBUTE_FIRST_NAME) {
                        $resultForEmailDetails['firstName'] = $parseResult['value'];
                    }

                    if ($subRuleConfig['assignToAttribute'] === self::LEAD_ATTRIBUTE_MOBILE && !empty($parseResult['additionalParams'])) {
                        $resultForEmailDetails['lead_mobile_country_code'] = $parseResult['additionalParams']['lead_mobile_country_code'];
                    }

                    if ($subRuleConfig['assignToAttribute'] === self::LEAD_ATTRIBUTE_PHONE && !empty($parseResult['additionalParams'])) {
                        $resultForEmailDetails['lead_phone_country_code'] = $parseResult['additionalParams']['lead_phone_country_code'];
                    }

                    if ($subRuleConfig['assignToAttribute'] === self::LEAD_ATTRIBUTE_SOCIAL_SERVICE_LINKEDIN && !empty($parseResult['additionalParams'])) {
                        if (!empty($parseResult['additionalParams']['socialServiceId'])) {
                            if (!empty($resultForEmailDetails['socialServicesInfo'])) {
                                $resultForEmailDetails['socialServicesInfo'][] = array('socialServiceId' => $parseResult['additionalParams']['socialServiceId'], 'socialServiceValue' => $parseResult['value']);
                            } else {
                                $resultForEmailDetails['socialServicesInfo'][0] = array('socialServiceId' => $parseResult['additionalParams']['socialServiceId'], 'socialServiceValue' => $parseResult['value']);
                            }
                        }
                    }

                    if (in_array($subRuleConfig['assignToAttribute'], self::$_listOfAdditionalFieldsList) && !empty($parseResult['additionalParams'])) {
                        if (!empty($parseResult['additionalParams']['additionalNote'])) {
                            if (!empty($resultForEmailDetails['additionalNotes'])) {
                                $resultForEmailDetails['additionalNotes'][] = array('additionalNote' => $parseResult['value']);
                            } else {
                                $resultForEmailDetails['additionalNotes'][0] = array('additionalNote' => $parseResult['value']);
                            }
                        }
                    }

                    if ((strpos($subRuleConfig['assignToAttribute'], self::LEAD_CUSTOM_ATTRIBUTES_PREFIX) !== false) && !empty($parseResult['additionalParams'])) {
                        if (!empty($parseResult['additionalParams']['customLeadAttribute'])) {
                            $customAttributeName = str_replace(self::LEAD_CUSTOM_ATTRIBUTES_PREFIX, '', $subRuleConfig['assignToAttribute']);
                            if (strpos($customAttributeName, self::LEAD_CUSTOM_ATTRIBUTE_TYPE_PREFIX_SPECIALWORD.self::LEAD_CUSTOM_ATTRIBUTE_TYPE_PREFIX) !== false) {
                                $customAttributeName = str_replace(self::LEAD_CUSTOM_ATTRIBUTE_TYPE_PREFIX_SPECIALWORD.self::LEAD_CUSTOM_ATTRIBUTE_TYPE_PREFIX, '', $customAttributeName);
                                $customAttributeType = str_replace('_', '', self::LEAD_CUSTOM_ATTRIBUTE_TYPE_PREFIX);

                                if (!empty($resultForEmailDetails['customLeadAttributes'])) {
                                    $resultForEmailDetails['customLeadAttributes'][] = array(
                                        'customLeadAttribute' => $customAttributeName,
                                        'customLeadAttributeValue' => $parseResult['value'],
                                        'customLeadAttributeType' => $customAttributeType
                                    );
                                } else {
                                    $resultForEmailDetails['customLeadAttributes'][0] = array(
                                        'customLeadAttribute' => $customAttributeName,
                                        'customLeadAttributeValue' => $parseResult['value'],
                                        'customLeadAttributeType' => $customAttributeType
                                    );
                                }
                            }
                        }
                    }

                    if ((strpos($subRuleConfig['assignToAttribute'], self::LEAD_ORGANIZATION_CUSTOM_ATTRIBUTES_PREFIX) !== false) && !empty($parseResult['additionalParams'])) {
                        $customOrganizationAttributeName = str_replace(self::LEAD_ORGANIZATION_CUSTOM_ATTRIBUTES_PREFIX, '', $subRuleConfig['assignToAttribute']);
                        if (strpos($customOrganizationAttributeName, self::LEAD_CUSTOM_ATTRIBUTE_TYPE_PREFIX_SPECIALWORD.self::LEAD_CUSTOM_ATTRIBUTE_TYPE_PREFIX) !== false) {
                            $customOrganizationAttributeName = str_replace(self::LEAD_CUSTOM_ATTRIBUTE_TYPE_PREFIX_SPECIALWORD.self::LEAD_CUSTOM_ATTRIBUTE_TYPE_PREFIX, '', $customOrganizationAttributeName);
                            $customAttributeType = str_replace('_', '', self::LEAD_CUSTOM_ATTRIBUTE_TYPE_PREFIX);
                            if (!empty($parseResult['additionalParams']['customOrganizationLeadAttribute'])) {
                                if (!empty($resultForEmailDetails['customLeadOrganizationAttributes'])) {
                                    $resultForEmailDetails['customLeadOrganizationAttributes'][] = array(
                                        'customLeadOrganizationAttribute' => $customOrganizationAttributeName,
                                        'customLeadOrganizationAttributeValue' => $parseResult['value'],
                                        'customLeadAttributeType' => $customAttributeType
                                    );
                                } else {
                                    $resultForEmailDetails['customLeadOrganizationAttributes'][0] = array(
                                        'customLeadOrganizationAttribute' => $customOrganizationAttributeName,
                                        'customLeadOrganizationAttributeValue' => $parseResult['value'],
                                        'customLeadAttributeType' => $customAttributeType
                                    );
                                }
                            }
                        }
                    }
                }
            }

            if (empty($resultForEmailDetails[self::LEAD_ATTRIBUTE_FIRST_NAME]) && !empty($resultForEmailDetails[self::LEAD_ATTRIBUTE_EMAIL])) {
                $resultForEmailDetails['firstName'] = $translator->translate(Leads::TRANSLATION_PREFIX.'Not provided');
            }

            if (empty($resultForEmailDetails[self::LEAD_ATTRIBUTE_EMAIL])) {
                $resultForEmailDetails = array();
                continue;
            }

            foreach ($resultForEmailDetails as $key => $resultForEmailDetail) {
                $emailDetailedInfo[$key] = $resultForEmailDetail;
            }

            break;
        }

        return $emailDetailedInfo;

    }

    public static function parseEmailContentByRules($subRuleConfig, $emailBody)
    {

        $parseResult = array();
        $wordToMatch = $subRuleConfig['afterTheText'];
        $numberOfWords = $subRuleConfig['numberOfWords'];
        $stopWord = $subRuleConfig['stopWord'];
        $numberOfWordsPattern = '[\^\(\)\'\":\w;&\.\/\#№-]*';
        if (empty($numberOfWords)) {
            $numberOfWordsPattern = '(.*)';
        }
        if ($numberOfWords > 1 && $subRuleConfig['assignToAttribute'] != self::LEAD_ATTRIBUTE_EMAIL) {
            $numberOfWordsPattern = $numberOfWordsPattern.str_repeat('\s?[\^\(\)\'\"\w;&\.\/\#№-]*', $numberOfWords - 1);
        }

        if ($subRuleConfig['assignToAttribute'] === self::LEAD_ATTRIBUTE_EMAIL) {
            $emailBody = preg_replace('~<a[^>]*>~ui', '', $emailBody);
            $numberOfWordsPattern = $numberOfWordsPattern.'\w+([\.-]?\w+)*@\w+([\.-]?\w+)*(\.\w{2,3})';
        }

        if ($subRuleConfig['assignToAttribute'] === self::LEAD_ATTRIBUTE_MOBILE || $subRuleConfig['assignToAttribute'] === self::LEAD_ATTRIBUTE_PHONE) {
            $numberOfWordsPattern = '([+\d\s\(\)-]*)';
        }

        $emailBody = str_replace('&amp;', '&', strip_tags($emailBody));
        $emailBody = str_replace('&rsquo;', '’', $emailBody);
        $emailBody = str_replace('&lsquo;', '‘', $emailBody);
        $emailBody = str_replace('&cent;', '¢', $emailBody);
        $emailBody = str_replace('&pound;', '£', $emailBody);
        $emailBody = str_replace('&sect;', '§', $emailBody);
        $emailBody = str_replace('&copy;', '©', $emailBody);
        $emailBody = str_replace('&laquo;', '«', $emailBody);
        $emailBody = str_replace('&raquo;', '»', $emailBody);
        $emailBody = str_replace('&reg;', '®', $emailBody);
        $emailBody = str_replace('&deg;', '°', $emailBody);
        $emailBody = str_replace('&plusmn;', '±', $emailBody);
        $emailBody = str_replace('&para;', '¶', $emailBody);
        $emailBody = str_replace('&middot;', '·', $emailBody);
        $emailBody = str_replace('&frac12;', '½', $emailBody);
        $emailBody = str_replace('&ndash;', '–', $emailBody);
        $emailBody = str_replace('&mdash;', '—', $emailBody);
        $emailBody = str_replace('&sbquo;', '‚', $emailBody);
        $emailBody = str_replace('&ldquo;', '“', $emailBody);
        $emailBody = str_replace('&rdquo;', '”', $emailBody);
        $emailBody = str_replace('&bdquo;', '„', $emailBody);
        $emailBody = str_replace('&dagger;', '†', $emailBody);
        $emailBody = str_replace('&Dagger;', '‡', $emailBody);
        $emailBody = str_replace('&bull;', '•', $emailBody);
        $emailBody = str_replace('&hellip;', '…', $emailBody);
        $emailBody = str_replace('&prime;', '′', $emailBody);
        $emailBody = str_replace('&Prime;', '″', $emailBody);
        $emailBody = str_replace('&euro;', '€', $emailBody);
        $emailBody = str_replace('&trade;', '™', $emailBody);
        $emailBody = str_replace('&asymp;', '≈', $emailBody);
        $emailBody = str_replace('&ne;', '≠', $emailBody);
        $emailBody = str_replace('&le;', '≤', $emailBody);
        $emailBody = str_replace('&ge;', '≥', $emailBody);
        $emailBody = str_replace('&lt;', '<', $emailBody);
        $emailBody = str_replace('&gt;', '>', $emailBody);

        if (empty($numberOfWords)) {
            if ($subRuleConfig['assignToAttribute'] === self::LEAD_ATTRIBUTE_NOTES) {
                //in case if we going to store html in the future
                $numberOfWordsPattern = '((.*|\n|\r)*.)';
            } else {
                $emailBody = self::cleanResultData($emailBody);
            }
        }

        preg_match('~(?<='.preg_quote($wordToMatch).')\s*('.$numberOfWordsPattern.')~ui', $emailBody, $match);
        if (!empty($match) && !empty($match[0])) {
            $matchedWord = self::cleanResultData($match[0]);
            if (!empty($stopWord)) {
                preg_match('~(.+?(?='.preg_quote($stopWord).'\s?))~ui', $matchedWord, $matchWithStopWord);
                if (!empty($matchWithStopWord)) {
                    $matchedWord = self::cleanResultData($matchWithStopWord[0]);
                }
            }

            $parseResult['name'] = $subRuleConfig['assignToAttribute'];
            if ($subRuleConfig['assignToAttribute'] === self::LEAD_ATTRIBUTE_EMAIL) {
                $parseResult['value'] =  self::cleanResultData($matchedWord);
                $parseResult['additionalParams'] = array('leadEmailFound' => '1');
            } elseif ($subRuleConfig['assignToAttribute'] === self::LEAD_ATTRIBUTE_MOBILE) {
                $parseResult = self::prepareMobilePhoneNumber($matchedWord, $parseResult);
            } elseif ($subRuleConfig['assignToAttribute'] === self::LEAD_ATTRIBUTE_PHONE) {
                $parseResult = self::preparePhoneNumber($matchedWord, $parseResult);
            } elseif ($subRuleConfig['assignToAttribute'] === self::LEAD_ATTRIBUTE_COUNTRY_NAME) {
                $countryNameList = array_flip(Tools_LeadTools::countryListNames());
                if (array_key_exists($matchedWord, $countryNameList)) {
                    $parseResult['value'] = $countryNameList[$matchedWord];
                    $parseResult['name'] = 'lead_country_code';
                }
            } elseif ($subRuleConfig['assignToAttribute'] === self::LEAD_ATTRIBUTE_STATE) {
                $matchedWord = self::cleanResultData(self::removeWrongEncodedSymbols($matchedWord));
                $matchedWord = trim(preg_replace('~[^\w\s]~', '', $matchedWord));

                $statesAbbrList = Tools_LeadTools::getStatesList();
                $reverseStatesAbbrList = array_flip(Tools_LeadTools::getStatesList());
                $statesFullNameList = array_flip(Tools_LeadTools::getStatesList(true));
                if (array_key_exists($matchedWord, $statesAbbrList)) {
                    $parseResult['value'] = $matchedWord;
                    $parseResult['name'] = 'lead_state_code';
                }
                if (array_key_exists($matchedWord, $statesFullNameList)) {
                    $parseResult['value'] = $reverseStatesAbbrList[$matchedWord];
                    $parseResult['name'] = 'lead_state_code';
                }
            } elseif ($subRuleConfig['assignToAttribute'] === self::LEAD_ATTRIBUTE_SOCIAL_SERVICE_LINKEDIN) {
                $leadsSocialServicesConfigMapper = Leads_Mapper_LeadsSocialServicesConfigMapper::getInstance();
                $whereSocialService = $leadsSocialServicesConfigMapper->getDbTable()->getAdapter()->quoteInto('plcsc.service_name = ?', 'linkedin');
                $leadSocialServices = $leadsSocialServicesConfigMapper->fetchAll($whereSocialService, null, null, null, true, true);
                if (!empty($leadSocialServices)) {
                    $parseResult['value'] =  self::cleanResultData($matchedWord);
                    $parseResult['additionalParams'] = array('socialServiceId' => $leadSocialServices['id']);
                }
            } elseif (in_array($subRuleConfig['assignToAttribute'], self::$_listOfAdditionalFieldsList)) {
                $parseResult['value'] =  self::cleanResultData($matchedWord);
                $parseResult['additionalParams'] = array('additionalNote' => self::cleanResultData($matchedWord));
            } elseif (strpos($subRuleConfig['assignToAttribute'], self::LEAD_CUSTOM_ATTRIBUTES_PREFIX) !== false) {
                $parseResult['value'] =  self::cleanResultData($matchedWord);
                $parseResult['additionalParams'] = array('customLeadAttribute' => self::cleanResultData($matchedWord));
            } elseif (strpos($subRuleConfig['assignToAttribute'], self::LEAD_ORGANIZATION_CUSTOM_ATTRIBUTES_PREFIX) !== false) {
                $parseResult['value'] =  self::cleanResultData($matchedWord);
                $parseResult['additionalParams'] = array('customOrganizationLeadAttribute' => self::cleanResultData($matchedWord));
            }
            else {
                $parseResult['value'] = self::removeSpecialSymbols(self::cleanResultData($matchedWord));
            }
        }

        return $parseResult;

    }

    public static function prepareMobilePhoneNumber($number, $parseResult)
    {
        $number = trim(str_replace(array('-', '(', ')', '\s', ' '), '',  $number));
        if (strpos($number, '+') !== false) {
            $mobileInfo = Tools_LeadPhoneLibTools::parseRawNumber($number);
            if (!empty($mobileInfo)) {
                $parseResult['value'] = $mobileInfo['number'];
                if (!empty($mobileInfo['countryCode'])) {
                    $parseResult['additionalParams'] = array('lead_mobile_country_code' => $mobileInfo['countryCode']);
                }
            }
        } else {
            $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
            $userDefaultMobileCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');
            if (!empty($userDefaultMobileCountryCode)) {
                $allCountryCodes = Zend_Locale::getTranslationList('phoneToTerritory');
                $countryCodeValue = $allCountryCodes[$userDefaultMobileCountryCode];
                $number = '+' . $countryCodeValue . $number;
                $mobileInfo = Tools_LeadPhoneLibTools::parseRawNumber($number);
                if (!empty($mobileInfo)) {
                    $parseResult['value'] = $mobileInfo['number'];
                    if (!empty($mobileInfo['countryCode'])) {
                        $parseResult['additionalParams'] = array('lead_mobile_country_code' => $mobileInfo['countryCode']);
                    }
                }
            } else {
                $parseResult['value'] = self::cleanResultData($number);
            }
        }

        return $parseResult;
    }

    public static function preparePhoneNumber($number, $parseResult)
    {
        $number = trim(str_replace(array('-', '(', ')', '\s', ' '), '',  $number));
        if (strpos($number, '+') !== false) {
            $phoneInfo = Tools_LeadPhoneLibTools::parseRawNumber($number);
            if (!empty($phoneInfo)) {
                $parseResult['value'] = $phoneInfo['number'];
                if (!empty($phoneInfo['countryCode'])) {
                    $parseResult['additionalParams'] = array('lead_phone_country_code' => $phoneInfo['countryCode']);
                }
            }
        } else {
            $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
            $userDefaultMobileCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');
            if (!empty($userDefaultMobileCountryCode)) {
                $allCountryCodes = Zend_Locale::getTranslationList('phoneToTerritory');
                $countryCodeValue = $allCountryCodes[$userDefaultMobileCountryCode];
                $number = '+' . $countryCodeValue . $number;
                $phoneInfo = Tools_LeadPhoneLibTools::parseRawNumber($number);
                if (!empty($phoneInfo)) {
                    $parseResult['value'] = $phoneInfo['number'];
                    if (!empty($phoneInfo['countryCode'])) {
                        $parseResult['additionalParams'] = array('lead_phone_country_code' => $phoneInfo['countryCode']);
                    }
                }
            } else {
                $parseResult['value'] = self::cleanResultData($number);
            }
        }

        return $parseResult;
    }

    public static function cleanResultData($value)
    {
        $valResult = rtrim(trim($value));
        $valResult = str_replace(array("\r\n"), ' ', $valResult);
        $valResult = str_replace(array("\n"), ' ', $valResult);
        $valResult = str_replace(array("\r"), ' ', $valResult);
        $valResult = str_replace(array("  "), ' ', $valResult);

        return $valResult;
    }

    public static function removeSpecialSymbols($value)
    {
        $valResult = str_replace(array("\""), ' ', $value);
        return $valResult;
    }

    public static function removeWrongEncodedSymbols($valResult)
    {
        $valResult = str_replace(array("&nbsp;"), ' ', $valResult);
        return $valResult;
    }

    public static function getListOfAssignToAttributes()
    {
        $translator = Zend_Registry::get('Zend_Translate');

        $assignToAttributes = array(
            self::LEAD_ATTRIBUTE_ORGANIZATION_NAME => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead organization name'),
            self::LEAD_ATTRIBUTE_ORGANIZATION_WEBSITE => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead organization website'),
            self::LEAD_ATTRIBUTE_FIRST_NAME => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead first name').'*',
            self::LEAD_ATTRIBUTE_LAST_NAME => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead last name'),
            self::LEAD_ATTRIBUTE_EMAIL => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead email').'*',
            self::LEAD_ATTRIBUTE_ADDRESS => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead address'),
            self::LEAD_ATTRIBUTE_ADDRESS1 => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead address 2'),
            self::LEAD_ATTRIBUTE_CITY => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead city'),
            self::LEAD_ATTRIBUTE_ZIP => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead postal code'),
            self::LEAD_ATTRIBUTE_MOBILE => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead mobile'),
            self::LEAD_ATTRIBUTE_MOBILE_COUNTRY_CODE => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead mobile country code (Ex:US)'),
            self::LEAD_ATTRIBUTE_PHONE => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead phone'),
            self::LEAD_ATTRIBUTE_PHONE_COUNTRY_CODE => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead phone country code (Ex:US)'),
            self::LEAD_ATTRIBUTE_COUNTRY_CODE => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead country code (Ex:US)'),
            self::LEAD_ATTRIBUTE_COUNTRY_NAME => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead country (Ex:United States - in English only!)'),
            self::LEAD_ATTRIBUTE_STATE => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead state'),
            self::LEAD_ATTRIBUTE_NOTES => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead notes (Permanent sticker)'),
            self::LEAD_ATTRIBUTE_POSITION => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead job position'),
            self::LEAD_ATTRIBUTE_SOCIAL_SERVICE_LINKEDIN => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead social service Linkedin (You must have "linkedin" field in social service config.)'),
            self::LEAD_ATTRIBUTE_ADDITIONAL_NOTE1 => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead note 1'),
            self::LEAD_ATTRIBUTE_ADDITIONAL_NOTE2 => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead note 2'),
            self::LEAD_ATTRIBUTE_ADDITIONAL_NOTE3 => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead note 3'),
            self::LEAD_ATTRIBUTE_ADDITIONAL_NOTE4 => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead note 4'),
            self::LEAD_ATTRIBUTE_ADDITIONAL_NOTE5 => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead note 5')
        );

        $leadsCustomParamsConfigMapper = Leads_Mapper_LeadsCustomParamsConfigMapper::getInstance();
        $where = $leadsCustomParamsConfigMapper->getDbTable()->getAdapter()->quoteInto('param_type IN (?)', 'text');
        $leadCustomParams = $leadsCustomParamsConfigMapper->fetchAll($where, null, null, null, true);
        if (!empty($leadCustomParams)) {
            foreach ($leadCustomParams as $leadCustomParam) {
                $assignToAttributes[self::LEAD_CUSTOM_ATTRIBUTES_PREFIX.self::LEAD_CUSTOM_ATTRIBUTE_TYPE_PREFIX_SPECIALWORD.$leadCustomParam['param_type'].'_'.$leadCustomParam['param_name']] = $leadCustomParam['label'].' '.$translator->translate(Leads::TRANSLATION_PREFIX.'(Lead custom field)');
            }
        }

        $leadsOrganizationCustomParamsConfigMapper = Leads_Mapper_LeadsOrganizationCustomParamsConfigMapper::getInstance();
        $where = $leadsOrganizationCustomParamsConfigMapper->getDbTable()->getAdapter()->quoteInto('param_type IN (?)', 'text');
        $leadOrganizationCustomParams = $leadsOrganizationCustomParamsConfigMapper->fetchAll($where, null, null, null, true);
        if (!empty($leadOrganizationCustomParams)) {
            foreach ($leadOrganizationCustomParams as $leadOrganizationCustomParam) {
                $assignToAttributes[self::LEAD_ORGANIZATION_CUSTOM_ATTRIBUTES_PREFIX.self::LEAD_CUSTOM_ATTRIBUTE_TYPE_PREFIX_SPECIALWORD.$leadCustomParam['param_type'].'_'.$leadOrganizationCustomParam['param_name']] = $leadOrganizationCustomParam['label'].' '.$translator->translate(Leads::TRANSLATION_PREFIX.'(Lead organization custom field)');
            }
        }

        return $assignToAttributes;
    }

}
