<?php

/**
 * Class Lead_LeadLicenseTools
 */
class Tools_LeadLicenseTools
{
    /**
     * plugin class name
     */
    const PLUGIN_CLASS_NAME = 'Leads';

    /**
     * mojo registration url
     */
    const REGISTER_URL = 'https://mojo.seosamba.com/register.html';

    /**
     * plugin status new
     */
    const PLUGIN_STATUS_NEW = 'new';

    /**
     * plugin status infinite
     */
    const PLUGIN_STATUS_INFINITE = 'infinite';

    /**
     * plugin status active
     */
    const PLUGIN_STATUS_ACTIVE = 'active';

    /**
     * default license limit
     */
    const DEFAULT_FREE_LIMIT = 250;

    /**
     * Promotional email text for the free subscription
     */
    const PROMOTIONAL_HIDDEN_EMAIL_TEXT = 'Customer Relationship Management, Sales & Marketing Automation Software Provided by SeoSamba';

    /**
     * Check leads availability
     *
     * @return array
     */
    public static function checkLimit()
    {
        $eligibleLeadsAmount = Leads_Mapper_LeadsMapper::getInstance()->calculateLeadsWithNonEmptyEmailPhone();
        $quantity = 0;
        if (!empty($eligibleLeadsAmount['quantity'])) {
            $quantity = $eligibleLeadsAmount['quantity'];
        }

        if (self::DEFAULT_FREE_LIMIT <= $quantity) {
            $licenseInfo = self::localStatusCheck();
            if (empty($licenseInfo)) {
                $leadLicenseInfo = false;
            } else {
                $leadLicenseInfo = $licenseInfo;
            }

            if (!empty($leadLicenseInfo)) {
                $leadLicenseInfo = (array) $leadLicenseInfo;
                if (!empty($leadLicenseInfo['units'])) {
                    $units = (array) $leadLicenseInfo['units'];
                    if (!empty($units['lead_number'])) {
                        if ($units['lead_number'] > $quantity) {
                            return array('error' => false);
                        } else {
                            return array(
                                'error' => true,
                                'message' => 'Congratulations, you’re getting the hang of this. Now that you’ve reached your subscription limit, '.'<a target="_blank" href="' . $leadLicenseInfo['link'] . '">please upgrade your subscription</a> before adding more contacts.'                            );
                        }
                    }
                }
            } else {
                $pluginLicense = self::getPluginLicense();
                if (!empty($pluginLicense) && !empty($pluginLicense['license'])) {
                    $license = (array) json_decode($pluginLicense['license'], true);
                    if (!empty($license['link'])) {
                        return array(
                            'error' => true,
                            'message' => 'Congratulations, you’re getting the hang of this. Now that you’ve reached your subscription limit, '.'<a target="_blank" href="' . $license['link'] . '">please upgrade your subscription</a> before adding more contacts.'
                        );
                    }
                }
            }

            return array('error' => true, 'message' => 'Congratulations, you’re getting the hang of this. Now that you’ve reached your subscription limit, before adding more contacts.');

        } else {
            return array('error' => false);
        }

    }

    /**
     * Get subscription link
     *
     * @return mixed
     */
    public static function getLicenseSubscriptionLink()
    {
        $pluginLicense = self::getPluginLicense();
        if (!empty($pluginLicense) && !empty($pluginLicense['license'])) {
            $license = (array) json_decode($pluginLicense['license'], true);
            if (!empty($license['link'])) {
                return $license['link'];
            } else {
                return self::getMojoUpdateLink();
            }
        }

        return self::getMojoUpdateLink();
    }

    /**
     * Mojo update link
     *
     * @return string
     */
    public static function getMojoUpdateLink()
    {
        $websiteConfig = Zend_Controller_Action_HelperBroker::getStaticHelper('config')->getConfig();
        if (!empty($websiteConfig['websiteId'])) {
            return  'https://mojo.seosamba.com/plugin/api/run/paymentForm/websiteId/'.$websiteConfig['websiteId'].'/pluginNameHash/ae5361473c0038d7f8c4b4236c50b2da';
        }
        return 'https://mojo.seosamba.com/plugin/api/run/paymentForm/websiteId/0/pluginNameHash/ae5361473c0038d7f8c4b4236c50b2da';
    }

    /**
     * Get license current limit
     *
     * @return int
     */
    public static function getLicenseLimit()
    {
        $pluginLicense = self::getPluginLicense();
        if (!empty($pluginLicense) && !empty($pluginLicense['license'])) {
            $leadLicenseInfo = (array)json_decode($pluginLicense['license'], true);
            if (!empty($leadLicenseInfo['units'])) {
                $units = (array)$leadLicenseInfo['units'];
                if (!empty($units['lead_number'])) {
                    return $units['lead_number'];
                }
            }
        }

        return self::DEFAULT_FREE_LIMIT;
    }

    /**
     * get current leads amount
     *
     * @return int
     */
    public static function getCurrentLeadsAmount()
    {
        $quantity = 0;
        $eligibleLeadsAmount = Leads_Mapper_LeadsMapper::getInstance()->calculateLeadsWithNonEmptyEmailPhone();
        if (!empty($eligibleLeadsAmount['quantity'])) {
            $quantity = $eligibleLeadsAmount['quantity'];
        }

        return $quantity;
    }

    /**
     * Get license info
     *
     * @return mixed
     */
    public static function getLicenseInfoFromMojo()
    {
        $commands = array('license');
        $data = array('plugins' => array('leads'));

        return self::apiCall('GET', 'apps', $commands, $data);
    }

    /**
     * Prepare salt for the license
     *
     * @return string
     */
    public static function createSalt()
    {
        return strrev(md5(strrev(Api_Tools_System::getDomain()) . base64_encode(self::PLUGIN_CLASS_NAME)));
    }

    /**
     * get plugin license
     *
     * @return mixed
     */
    public static function getPluginLicense()
    {
        $pluginDbTable = new Application_Model_DbTable_Plugin();
        $where = $pluginDbTable->getAdapter()->quoteInto('name = ?', self::PLUGIN_CLASS_NAME);
        $pluginLicense = $pluginDbTable->getAdapter()->fetchRow(
            $pluginDbTable->getAdapter()->select()->from('plugin', array(
                'license' => new Zend_Db_Expr(
                    $pluginDbTable->getAdapter()->quoteInto("AES_DECRYPT(license,?)", self::createSalt())
                )
            ))->where($where)
        );
        $pluginLicense['license'] = stripslashes($pluginLicense['license']);

        return $pluginLicense;
    }

    /**
     * local status check
     *
     * @return bool
     */
    public static function localStatusCheck()
    {
        $date = null;
        $status = null;
        $plugin = new Application_Model_Models_Plugin(self::getPluginLicense());
        if ($plugin->getLicense() != null) {
            $license = json_decode($plugin->getLicense());
            if (!isset($license->status)) {
                return false;
            }
            if (($license->status == self::PLUGIN_STATUS_INFINITE) && ($license->nextBillDate == null) && ($license->websiteUrl == Api_Tools_System::getDomain())) {
                return $license;
            }
            $status = ($license->status != self::PLUGIN_STATUS_ACTIVE) ? false : true;
            if (isset($license->nextBillDate)) {
                $date = (date('Y-m-d') > $license->nextBillDate) ? false : true;
            } else {
                $date = false;
            }
            if ((($date == true) && ($status == true)) && ($license->websiteUrl == Api_Tools_System::getDomain())) {
                return $license;
            } else {
                $licenseOldData = (array) $license;
                $licenseOldData['next_license_check'] =  date('Y-m-d', strtotime('+1 day'));
                self::setPluginLicense(json_encode($licenseOldData));
                if (!empty($license->next_license_check)) {
                    if (strtotime($license->next_license_check) <= strtotime(date('Y-m-d'))) {
                        return self::remoteStatusCheck();
                    } else {
                        return false;
                    }
                } else {
                    return self::remoteStatusCheck();
                }
            }
        } else {
            $nextCheckDate = date('Y-m-d', time());
            $license = json_encode(array(
                'url' => Api_Tools_System::getDomain(),
                'next_check' => $nextCheckDate,
                'status' => self::PLUGIN_STATUS_NEW
            ));
            self::setPluginLicense($license);

            return self::remoteStatusCheck();
        }
    }

    /**
     * status check on mojo
     *
     * @return bool
     * @throws Exceptions_SeotoasterPluginException
     */
    public static function remoteStatusCheck()
    {
        $data['pluginNameHash'] = md5(self::PLUGIN_CLASS_NAME);
        $checkResult = self::apiCall('get', 'plugin', '', $data);
        if ($checkResult['done'] == true) {
            $license = (object)$checkResult['license'];
            $license->next_license_check =  date('Y-m-d', strtotime('+1 day'));
            self::setPluginLicense(json_encode($license));
            if ((strtolower($license->status) == self::PLUGIN_STATUS_ACTIVE) && ($license->nextBillDate > date('Y-m-d'))) {
                return $license;
            } else {
                if ((strtolower($license->status) == self::PLUGIN_STATUS_INFINITE) && ($license->nextBillDate == null)) {
                    return $license;
                } else {
                    return false;
                }
            }
        } else {
            $license = (object)$checkResult['license'];
            self::setPluginLicense(json_encode($license));
            return false;
        }

    }

    /**
     * Save plugin license
     *
     * @param string $license plugin license
     */
    public static function setPluginLicense($license)
    {
        $salt = self::createSalt();
        $pluginDbTable = new Application_Model_DbTable_Plugin();
        $pluginDbTable->update(array("license" => new Zend_Db_Expr("AES_ENCRYPT('" . addslashes($license) . "', '" . $salt . "')")),
            array('name = ?' => strtolower(self::PLUGIN_CLASS_NAME)));
    }


    /**
     * Call mojo.seosamba.com api
     *
     * @param string $methodType method type (GET, POST, PUT, DELETE)
     * @param string $methodName method name (apps)
     * @param array $command command list for mojo api
     * @param array $data data
     * @return mixed
     * @throws Exceptions_SeotoasterPluginException
     */
    public static function apiCall($methodType, $methodName, $command, $data = array())
    {
        $websiteConfig = Zend_Controller_Action_HelperBroker::getStaticHelper('config')->getConfig();
        $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
        $websiteUrl = $websiteHelper->getUrl();
        if (!empty($websiteConfig['sambaToken'])) {
            $data['sambaToken'] = $websiteConfig['sambaToken'];
        }

        if (!empty($websiteConfig['websiteId'])) {
            $data['websiteId'] = $websiteConfig['websiteId'];
        }

        $data['pluginNameHash'] = md5(self::PLUGIN_CLASS_NAME);
        $data['command'] = $command;
        $seosambaRequest = Tools_Factory_PluginFactory::createPlugin('api', array(),
            array('websiteUrl' => $websiteUrl));

        return $seosambaRequest::request($methodType, $methodName, $data);

    }

}