<?php

/**
 * Class Lead_LeadMergeTools
 */
class Tools_LeadMergeTools
{

    const MERGE_LEAD_BASED_ON_EMAIL = 'email';

    const MERGE_LEAD_BASED_ON_PHONE = 'phone';

    const MERGE_LEAD_BASED_ON_MOBILE = 'mobile';

    const SORT_TYPE_LEAD = 'leeds';

    const SORT_TYPE_ORGANIZATION = 'organization';

    const NOTES_DIVIDER = '============';

    /**
     * @var array
     */
    public static $_leadMergeBasedElements = array(
        self::MERGE_LEAD_BASED_ON_EMAIL,
        self::MERGE_LEAD_BASED_ON_PHONE,
        self::MERGE_LEAD_BASED_ON_MOBILE
    );

    /**
     * Get organization labels for merge
     *
     * @return array
     * @throws Zend_Exception
     */
    public static function getOrganizationMergeLabels()
    {
        $translator = Zend_Registry::get('Zend_Translate');

        return array(
            'address' => $translator->translate(Leads::TRANSLATION_PREFIX.'Address'),
            'address1' => $translator->translate(Leads::TRANSLATION_PREFIX.'Address 2'),
            'city' => $translator->translate(Leads::TRANSLATION_PREFIX.'City'),
            'zip' => $translator->translate(Leads::TRANSLATION_PREFIX.'Zip'),
            'countryCode' => $translator->translate(Leads::TRANSLATION_PREFIX.'Country'),
            'customerGroupId' => $translator->translate(Leads::TRANSLATION_PREFIX.'Customer group'),
            'website' => $translator->translate(Leads::TRANSLATION_PREFIX.'Website'),
            'industryId' => $translator->translate(Leads::TRANSLATION_PREFIX.'Industry'),
            'phone' => $translator->translate(Leads::TRANSLATION_PREFIX.'Phone'),
            'email' => $translator->translate(Leads::TRANSLATION_PREFIX.'Email'),
            'employees' => $translator->translate(Leads::TRANSLATION_PREFIX.'Employees'),
            'domain' => $translator->translate(Leads::TRANSLATION_PREFIX.'Domain'),
            'stateCode' => $translator->translate(Leads::TRANSLATION_PREFIX.'State'),
            'phoneCountryCode' => $translator->translate(Leads::TRANSLATION_PREFIX.'Phone country code'),
            'phoneCountryCodeValue' => $translator->translate(Leads::TRANSLATION_PREFIX.'Phone country code value'),
            'phoneExtension' => $translator->translate(Leads::TRANSLATION_PREFIX.'Phone extension'),
            'mobile' => $translator->translate(Leads::TRANSLATION_PREFIX.'Mobile'),
            'mobileCountryCode' => $translator->translate(Leads::TRANSLATION_PREFIX.'Mobile country code'),
            'mobileCountryCodeValue' => $translator->translate(Leads::TRANSLATION_PREFIX.'Mobile country code value'),
            'notes' => $translator->translate(Leads::TRANSLATION_PREFIX.'notes'),
            'primaryContactId' => $translator->translate(Leads::TRANSLATION_PREFIX.'Primary contact'),
            'firstContactCreationDate' => $translator->translate(Leads::TRANSLATION_PREFIX.'First contact creation date'),
            'externalId' => $translator->translate(Leads::TRANSLATION_PREFIX.'External id')
        );
    }

    /**
     * Get leads labels for merge
     *
     * @return array
     * @throws Zend_Exception
     */
    public static function getLeadMergeLabels()
    {
        $translator = Zend_Registry::get('Zend_Translate');

        return array(
            'prefix' => $translator->translate(Leads::TRANSLATION_PREFIX.'Prefix'),
            'firstname' => $translator->translate(Leads::TRANSLATION_PREFIX.'First name'),
            'lastname' => $translator->translate(Leads::TRANSLATION_PREFIX.'Last name'),
            'address' => $translator->translate(Leads::TRANSLATION_PREFIX.'Address'),
            'address1' => $translator->translate(Leads::TRANSLATION_PREFIX.'Address 2'),
            'city' => $translator->translate(Leads::TRANSLATION_PREFIX.'City'),
            'zip' => $translator->translate(Leads::TRANSLATION_PREFIX.'Zip'),
            'countryCode' => $translator->translate(Leads::TRANSLATION_PREFIX.'Country'),
            'phone' => $translator->translate(Leads::TRANSLATION_PREFIX.'Phone'),
            'email' => $translator->translate(Leads::TRANSLATION_PREFIX.'Email'),
            'stateCode' => $translator->translate(Leads::TRANSLATION_PREFIX.'State'),
            'phoneCountryCode' => $translator->translate(Leads::TRANSLATION_PREFIX.'Phone country code'),
            'phoneCountryCodeValue' => $translator->translate(Leads::TRANSLATION_PREFIX.'Phone country code value'),
            'phoneExtension' => $translator->translate(Leads::TRANSLATION_PREFIX.'Phone extension'),
            'mobile' => $translator->translate(Leads::TRANSLATION_PREFIX.'Mobile'),
            'mobileCountryCode' => $translator->translate(Leads::TRANSLATION_PREFIX.'Mobile country code'),
            'mobileCountryCodeValue' => $translator->translate(Leads::TRANSLATION_PREFIX.'Mobile country code value'),
            'notes' => $translator->translate(Leads::TRANSLATION_PREFIX.'notes'),
            'primaryContactId' => $translator->translate(Leads::TRANSLATION_PREFIX.'Primary contact'),
            'organizationId' => $translator->translate(Leads::TRANSLATION_PREFIX.'Organization name'),
            'externalId' => $translator->translate(Leads::TRANSLATION_PREFIX.'External id'),
            'position' => $translator->translate(Leads::TRANSLATION_PREFIX.'Position'),
            'userId' => $translator->translate(Leads::TRANSLATION_PREFIX.'User name'),
            'ownerId' => $translator->translate(Leads::TRANSLATION_PREFIX.'Owner name'),
            'middleName' => $translator->translate(Leads::TRANSLATION_PREFIX.'Middle name'),
            'leadSource' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead source'),
            'stageId' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lifecycle'),
            'typeId' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead type'),
            'customerGroupId' => $translator->translate(Leads::TRANSLATION_PREFIX.'Customer group'),
            'useOrganizationAddress' => $translator->translate(Leads::TRANSLATION_PREFIX.'Use organization\'s address'),
            'lastActivityOutbound' => $translator->translate(Leads::TRANSLATION_PREFIX.'Last activity outbound'),
            'lastActivityTypeId' => $translator->translate(Leads::TRANSLATION_PREFIX.'Last activity type')
        );
    }

    /**
     * Merge two organizations
     *
     * @param int $fromOrganizationId organization which will be merged id
     * @param int $toOrganizationId merge to organization id
     * @param bool $removeMergedOrganization flag merged organization
     * @return array
     * @throws Exception
     */
    public static function mergeOrganization($fromOrganizationId, $toOrganizationId, $removeMergedOrganization = false)
    {
        $translator = Zend_Registry::get('Zend_Translate');
        $mergedOrganizationModel = null;
        $leadOrganizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
        $leadOrganizationModelFrom = $leadOrganizationMapper->find($fromOrganizationId);
        $leadOrganizationModelTo = $leadOrganizationMapper->find($toOrganizationId);
        if (!$leadOrganizationModelFrom instanceof Leads_Model_LeadsOrganizationModel) {
            return array(
                'error' => 1,
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Organization model which you are going to merge doesn\'t exist')
            );
        }

        if (!$leadOrganizationModelTo instanceof Leads_Model_LeadsOrganizationModel) {
            return array(
                'error' => 1,
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Organization model where you are going to merge doesn\'t exist')
            );
        }

        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();

        $customParamsAddToNotes = self::mergeOrganizationCustomParams($fromOrganizationId, $toOrganizationId);

        $generalNotesInfo = array();

        //move all leads under merged organization
        $leadsMapper->changeOrganization($fromOrganizationId, $toOrganizationId);

        //merge general organization attributes
        $fromOrganizationAddress = $leadOrganizationModelFrom->getOrganizationAddress();
        $fromOrganizationAddress1 = $leadOrganizationModelFrom->getOrganizationAddress1();
        $fromOrganizationCity = $leadOrganizationModelFrom->getOrganizationCity();
        $fromOrganizationZip = $leadOrganizationModelFrom->getOrganizationZip();
        $fromOrganizationCountryCode = $leadOrganizationModelFrom->getOrganizationCountryCode();
        $fromOrganizationCustomerGroupId = $leadOrganizationModelFrom->getCustomerGroupId();
        $fromOrganizationWebsite = $leadOrganizationModelFrom->getOrganizationWebsite();
        $fromOrganizationIndustryId = $leadOrganizationModelFrom->getOrganizationIndustryId();
        $fromOrganizationPhone = $leadOrganizationModelFrom->getOrganizationPhone();
        $fromOrganizationEmail = $leadOrganizationModelFrom->getOrganizationEmail();
        $fromOrganizationEmployees = $leadOrganizationModelFrom->getOrganizationEmployees();
        $fromOrganizationDomain = $leadOrganizationModelFrom->getOrganizationDomain();
        $fromOrganizationStateCode = $leadOrganizationModelFrom->getOrganizationStateCode();
        $fromOrganizationPhoneCountryCode = $leadOrganizationModelFrom->getOrganizationPhoneCountryCode();
        $fromOrganizationPhoneCountryCodeValue = $leadOrganizationModelFrom->getOrganizationPhoneCountryCodeValue();
        $fromOrganizationPhoneExtension = $leadOrganizationModelFrom->getOrganizationPhoneExtension();
        $fromOrganizationMobile = $leadOrganizationModelFrom->getOrganizationMobile();
        $fromOrganizationMobileCountryCode = $leadOrganizationModelFrom->getOrganizationMobileCountryCode();
        $fromOrganizationMobileCountryCodeValue = $leadOrganizationModelFrom->getOrganizationMobileCountryCodeValue();
        $fromOrganizationNotes = $leadOrganizationModelFrom->getOrganizationNotes();
        $fromOrganizationPrimaryContactId = $leadOrganizationModelFrom->getPrimaryContactId();
        $fromOrganizationFirstContactCreationDate = $leadOrganizationModelFrom->getFirstContactCreationDate();
        $fromOrganizationExternalId = $leadOrganizationModelFrom->getExternalId();

        $toOrganizationAddress = $leadOrganizationModelTo->getOrganizationAddress();
        $toOrganizationAddress1 = $leadOrganizationModelTo->getOrganizationAddress1();
        $toOrganizationCity = $leadOrganizationModelTo->getOrganizationCity();
        $toOrganizationZip = $leadOrganizationModelTo->getOrganizationZip();
        $toOrganizationCountryCode = $leadOrganizationModelTo->getOrganizationCountryCode();
        $toOrganizationCustomerGroupId = $leadOrganizationModelTo->getCustomerGroupId();
        $toOrganizationWebsite = $leadOrganizationModelTo->getOrganizationWebsite();
        $toOrganizationIndustryId = $leadOrganizationModelTo->getOrganizationIndustryId();
        $toOrganizationPhone = $leadOrganizationModelTo->getOrganizationPhone();
        $toOrganizationEmail = $leadOrganizationModelTo->getOrganizationEmail();
        $toOrganizationEmployees = $leadOrganizationModelTo->getOrganizationEmployees();
        $toOrganizationDomain = $leadOrganizationModelTo->getOrganizationDomain();
        $toOrganizationStateCode = $leadOrganizationModelTo->getOrganizationStateCode();
        $toOrganizationPhoneCountryCode = $leadOrganizationModelTo->getOrganizationPhoneCountryCode();
        $toOrganizationPhoneCountryCodeValue = $leadOrganizationModelTo->getOrganizationPhoneCountryCodeValue();
        $toOrganizationPhoneExtension = $leadOrganizationModelTo->getOrganizationPhoneExtension();
        $toOrganizationMobile = $leadOrganizationModelTo->getOrganizationMobile();
        $toOrganizationMobileCountryCode = $leadOrganizationModelTo->getOrganizationMobileCountryCode();
        $toOrganizationMobileCountryCodeValue = $leadOrganizationModelTo->getOrganizationMobileCountryCodeValue();
        $toOrganizationNotes = $leadOrganizationModelTo->getOrganizationNotes();
        $toOrganizationPrimaryContactId = $leadOrganizationModelTo->getPrimaryContactId();
        $toOrganizationFirstContactCreationDate = $leadOrganizationModelTo->getFirstContactCreationDate();
        $toOrganizationExternalId = $leadOrganizationModelTo->getExternalId();

        if (empty($toOrganizationAddress)) {
            $leadOrganizationModelTo->setOrganizationAddress($fromOrganizationAddress);
        } elseif (!empty($fromOrganizationAddress)) {
            $generalNotesInfo['address'] = $fromOrganizationAddress;
        }

        if (empty($toOrganizationAddress1)) {
            $leadOrganizationModelTo->setOrganizationAddress1($fromOrganizationAddress1);
        } elseif (!empty($fromOrganizationAddress1)) {
            $generalNotesInfo['address1'] = $fromOrganizationAddress1;
        }

        if (empty($toOrganizationCity)) {
            $leadOrganizationModelTo->setOrganizationCity($fromOrganizationCity);
        } elseif (!empty($fromOrganizationCity)) {
            $generalNotesInfo['city'] = $fromOrganizationCity;
        }

        if (empty($toOrganizationZip)) {
            $leadOrganizationModelTo->setOrganizationZip($fromOrganizationZip);
        } elseif (!empty($fromOrganizationZip)) {
            $generalNotesInfo['zip'] = $fromOrganizationZip;
        }

        if (empty($toOrganizationCountryCode)) {
            $leadOrganizationModelTo->setOrganizationCountryCode($fromOrganizationCountryCode);
        } elseif (!empty($fromOrganizationCountryCode)) {
            $generalNotesInfo['countryCode'] = $fromOrganizationCountryCode;
        }

        if (empty($toOrganizationCustomerGroupId)) {
            $leadOrganizationModelTo->setCustomerGroupId($fromOrganizationCustomerGroupId);
        } elseif (!empty($fromOrganizationCustomerGroupId)) {
            $customerGroups = Store_Mapper_GroupMapper::getInstance()->fetchAssocAll();
            if (array_key_exists($fromOrganizationCustomerGroupId, $customerGroups)) {
                $generalNotesInfo['customerGroupId'] = $customerGroups[$fromOrganizationCustomerGroupId]['groupName'];
            } else {
                $generalNotesInfo['customerGroupId'] = $fromOrganizationCustomerGroupId;
            }
        }

        if (empty($toOrganizationWebsite)) {
            $leadOrganizationModelTo->setOrganizationWebsite($fromOrganizationWebsite);
        } elseif (!empty($fromOrganizationWebsite)) {
            $generalNotesInfo['website'] = $fromOrganizationWebsite;
        }


        if (empty($toOrganizationIndustryId)) {
            $leadOrganizationModelTo->setOrganizationIndustryId($fromOrganizationIndustryId);
        } elseif (!empty($fromOrganizationIndustryId)) {
            $industries = Leads_Mapper_LeadsIndustryMapper::getInstance()->getIndustriesPairs(false, array('label'));
            if (array_key_exists($fromOrganizationIndustryId, $industries)) {
                $generalNotesInfo['industryId'] = $industries[$fromOrganizationIndustryId];
            } else {
                $generalNotesInfo['industryId'] = $fromOrganizationIndustryId;
            }
        }

        if (empty($toOrganizationPhone)) {
            $leadOrganizationModelTo->setOrganizationPhone($fromOrganizationPhone);
        } elseif (!empty($fromOrganizationPhone)) {
            $generalNotesInfo['phone'] = $fromOrganizationPhone;
        }

        if (empty($toOrganizationEmail)) {
            $leadOrganizationModelTo->setOrganizationEmail($fromOrganizationEmail);
        } elseif (!empty($fromOrganizationEmail)) {
            $generalNotesInfo['email'] = $fromOrganizationEmail;
        }

        if (empty($toOrganizationEmployees)) {
            $leadOrganizationModelTo->setOrganizationEmployees($fromOrganizationEmployees);
        } elseif (!empty($fromOrganizationEmployees)) {
            $generalNotesInfo['employees'] = $fromOrganizationEmployees;
        }

        if (empty($toOrganizationDomain)) {
            $leadOrganizationModelTo->setOrganizationDomain($fromOrganizationDomain);
        } elseif (!empty($fromOrganizationDomain)) {
            $generalNotesInfo['domain'] = $fromOrganizationDomain;
        }

        if (empty($toOrganizationStateCode)) {
            $leadOrganizationModelTo->setOrganizationStateCode($fromOrganizationStateCode);
        } elseif (!empty($fromOrganizationStateCode)) {
            $generalNotesInfo['stateCode'] = $fromOrganizationStateCode;
        }

        if (empty($toOrganizationPhoneCountryCode)) {
            $leadOrganizationModelTo->setOrganizationPhoneCountryCode($fromOrganizationPhoneCountryCode);
        } elseif (!empty($fromOrganizationPhoneCountryCode)) {
            $generalNotesInfo['phoneCountryCode'] = $fromOrganizationPhoneCountryCode;
        }

        if (empty($toOrganizationPhoneCountryCodeValue)) {
            $leadOrganizationModelTo->setOrganizationPhoneCountryCodeValue($fromOrganizationPhoneCountryCodeValue);
        } elseif (!empty($fromOrganizationPhoneCountryCodeValue)) {
            $generalNotesInfo['phoneCountryCodeValue'] = $fromOrganizationPhoneCountryCodeValue;
        }

        if (empty($toOrganizationPhoneExtension)) {
            $leadOrganizationModelTo->setOrganizationPhoneExtension($fromOrganizationPhoneExtension);
        } elseif (!empty($fromOrganizationPhoneExtension)) {
            $generalNotesInfo['phoneExtension'] = $fromOrganizationPhoneExtension;
        }

        if (empty($toOrganizationMobile)) {
            $leadOrganizationModelTo->setOrganizationMobile($fromOrganizationMobile);
        } elseif (!empty($fromOrganizationMobile)) {
            $generalNotesInfo['mobile'] = $fromOrganizationMobile;
        }

        if (empty($toOrganizationMobileCountryCode)) {
            $leadOrganizationModelTo->setOrganizationMobileCountryCode($fromOrganizationMobileCountryCode);
        } elseif (!empty($fromOrganizationMobileCountryCode)) {
            $generalNotesInfo['mobileCountryCode'] = $fromOrganizationMobileCountryCode;
        }

        if (empty($toOrganizationMobileCountryCodeValue)) {
            $leadOrganizationModelTo->setOrganizationMobileCountryCodeValue($fromOrganizationMobileCountryCodeValue);
        } elseif (!empty($fromOrganizationMobileCountryCodeValue)) {
            $generalNotesInfo['mobileCountryCodeValue'] = $fromOrganizationMobileCountryCodeValue;
        }

        if (empty($toOrganizationNotes)) {
            $leadOrganizationModelTo->setOrganizationNotes($fromOrganizationNotes);
        } elseif (!empty($fromOrganizationNotes)) {
            $generalNotesInfo['notes'] = trim($fromOrganizationNotes);
        }

        if (empty($toOrganizationPrimaryContactId)) {
            $leadOrganizationModelTo->setPrimaryContactId($fromOrganizationPrimaryContactId);
        } elseif (!empty($fromOrganizationPrimaryContactId)) {
            $leadModel = $leadsMapper->find($fromOrganizationPrimaryContactId);
            if ($leadModel instanceof Leads_Model_LeadsModel) {
                $generalNotesInfo['primaryContactId'] = $leadModel->getLeadFirstName() . ' ' . $leadModel->getLeadLastName();
            } else {
                $generalNotesInfo['primaryContactId'] = $fromOrganizationPrimaryContactId;
            }
        }

        if (empty($toOrganizationFirstContactCreationDate)) {
            $leadOrganizationModelTo->setFirstContactCreationDate($fromOrganizationFirstContactCreationDate);
        } elseif (!empty($fromOrganizationFirstContactCreationDate)) {
            $generalNotesInfo['firstContactCreationDate'] = $fromOrganizationFirstContactCreationDate;
        }

        if (empty($toOrganizationExternalId)) {
            $leadOrganizationModelTo->setExternalId($fromOrganizationExternalId);
        } elseif (!empty($fromOrganizationExternalId)) {
            $generalNotesInfo['externalId'] = $fromOrganizationExternalId;
        }

        $mergeDifferenceInformation = PHP_EOL;
        $organizationMergeLabels = self::getOrganizationMergeLabels();

        if(!empty($generalNotesInfo)) {
            $generalNotesInfo = self::processNotesPosition($generalNotesInfo, self::SORT_TYPE_ORGANIZATION);
        }

        foreach ($generalNotesInfo as $noteName => $noteValue) {
            $labelName = $noteName;
            if (array_key_exists($noteName, $organizationMergeLabels)) {
                $labelName = $organizationMergeLabels[$noteName];
            }
            $mergeDifferenceInformation .= $labelName . ': ' . $noteValue . PHP_EOL;
        }

        if (!empty($customParamsAddToNotes)) {
            foreach ($customParamsAddToNotes as $customParamsAddToNote) {
                $value = $customParamsAddToNote['param_value'];
                if ($customParamsAddToNote['optionValue'] !== null) {
                    $value = $customParamsAddToNote['optionValue'];
                }
                $mergeDifferenceInformation .= $customParamsAddToNote['paramLabel'] . ': ' . $value . PHP_EOL;
            }
        }

        if (true === $removeMergedOrganization) {
            $leadOrganizationMapper->delete($fromOrganizationId);
        } else {
            //remove old primary contact id if we are going to keep organization
            $leadOrganizationModelFrom->setPrimaryContactId(null);
            $leadOrganizationMapper->save($leadOrganizationModelFrom);
        }

        $mergeDifferenceInformation .= self::NOTES_DIVIDER . PHP_EOL;
        $leadOrganizationModelTo->setOrganizationNotes(trim($leadOrganizationModelTo->getOrganizationNotes()) . PHP_EOL . trim($mergeDifferenceInformation));
        $leadOrganizationMapper->save($leadOrganizationModelTo);


        return array('error' => 0, 'model' => $mergedOrganizationModel);
    }


    /**
     * Merge lead
     *
     * @param int $fromLeadId from lead id
     * @param int $toLeadId to lead id
     * @param string $mergeBasedOn merge based on field ('email', 'mobile', 'phone')
     * @param bool $removeMergedLead remove merged lead
     * @return array
     * @throws Zend_Exception
     */
    public static function mergeLead($fromLeadId, $toLeadId, $mergeBasedOn, $removeMergedLead = false)
    {
        $translator = Zend_Registry::get('Zend_Translate');
        $mergedLeadModel = null;
        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
        $userMapper = Application_Model_Mappers_UserMapper::getInstance();
        $leadLeadModelFrom = $leadsMapper->find($fromLeadId);
        $leadLeadModelTo = $leadsMapper->find($toLeadId);
        if (!$leadLeadModelFrom instanceof Leads_Model_LeadsModel) {
            return array(
                'error' => 1,
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead model which you are going to merge doesn\'t exist')
            );
        }

        if (!$leadLeadModelTo instanceof Leads_Model_LeadsModel) {
            return array(
                'error' => 1,
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead model where you are going to merge doesn\'t exist')
            );
        }

        if (empty($mergeBasedOn)) {
            return array(
                'error' => 1,
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Merge based on is missing')
            );
        }

        if (!in_array($mergeBasedOn, self::$_leadMergeBasedElements)) {
            return array(
                'error' => 1,
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Wrong merge based on type')
            );
        }

        $tagMapper = Leads_Mapper_LeadsTagsMapper::getInstance();

        $formOriginalLeadTags = $tagMapper->findTagsByLeadId($leadLeadModelFrom->getId());
        if (empty($formOriginalLeadTags)) {
            $formOriginalLeadTags = array();
        }

        $toOriginalLeadTags = $tagMapper->findTagsByLeadId($leadLeadModelTo->getId());
        if (empty($toOriginalLeadTags)) {
            $toOriginalLeadTags = array();
        }

        $leadTags = array_unique(array_merge($formOriginalLeadTags, $toOriginalLeadTags));

        $leadLeadModelTo->setLeadTags($leadTags);

        //merge general lead attributes
        $fromLeadEmail = $leadLeadModelFrom->getLeadEmail();
        $fromLeadSecondaryEmail = $leadLeadModelFrom->getLeadSecondaryEmail();
        $fromLeadFirstName = $leadLeadModelFrom->getLeadFirstName();
        $fromLeadLastName = $leadLeadModelFrom->getLeadLastName();
        $fromLeadMiddleName = $leadLeadModelFrom->getMiddleName();
        $fromLeadPrefix = $leadLeadModelFrom->getPrefix();
        $fromLeadZip = $leadLeadModelFrom->getLeadZip();
        $fromLeadPosition = $leadLeadModelFrom->getLeadPosition();
        $fromLeadNotes = $leadLeadModelFrom->getLeadNotes();
        $fromLeadStateCode = $leadLeadModelFrom->getLeadStateCode();
        $fromLeadCity = $leadLeadModelFrom->getLeadCity();
        $fromLeadUserId = $leadLeadModelFrom->getUserId();
        $fromLeadOwnerId = $leadLeadModelFrom->getOwnerId();
        $fromLeadAddress = $leadLeadModelFrom->getLeadAddress();
        $fromLeadAddress1 = $leadLeadModelFrom->getLeadAddress1();
        $fromLeadExternalId = $leadLeadModelFrom->getExternalId();
        $fromLeadTypeId = $leadLeadModelFrom->getLeadTypeId();
        $fromLeadSource = $leadLeadModelFrom->getLeadSource();
        $fromLeadCustomerGroupId = $leadLeadModelFrom->getCustomerGroupId();
        $fromLeadCountryCode = $leadLeadModelFrom->getLeadCountryCode();
        $fromLeadOrganizationId = $leadLeadModelFrom->getOrganizationId();
        $fromLeadPhone = $leadLeadModelFrom->getLeadPhone();
        $fromLeadPhoneCountryCode = $leadLeadModelFrom->getLeadPhoneCountryCode();
        $fromLeadPhoneCountryCodeValue = $leadLeadModelFrom->getLeadPhoneCountryCodeValue();
        $fromLeadPhoneExtension = $leadLeadModelFrom->getLeadPhoneExtension();
        $fromLeadMobile = $leadLeadModelFrom->getLeadMobile();
        $fromLeadMobileCountryCode = $leadLeadModelFrom->getLeadMobileCountryCode();
        $fromLeadMobileCountryCodeValue = $leadLeadModelFrom->getLeadMobileCountryCodeValue();
        $fromUseOrganizationAddress = $leadLeadModelFrom->getUseOrganizationAddress();
        $fromLastActivityOutbound = $leadLeadModelFrom->getLastActivityOutbound();
        $fromStageId = $leadLeadModelFrom->getStageId();
        $fromLastActivityTypeId = $leadLeadModelFrom->getLastActivityTypeId();

        $toLeadEmail = $leadLeadModelTo->getLeadEmail();
        $toLeadSecondaryEmail = $leadLeadModelTo->getLeadSecondaryEmail();
        $toLeadFirstName = $leadLeadModelTo->getLeadFirstName();
        $toLeadLastName = $leadLeadModelTo->getLeadLastName();
        $toLeadMiddleName = $leadLeadModelTo->getMiddleName();
        $toLeadPrefix = $leadLeadModelTo->getPrefix();
        $toLeadZip = $leadLeadModelTo->getLeadZip();
        $toLeadPosition = $leadLeadModelTo->getLeadPosition();
        $toLeadNotes = $leadLeadModelTo->getLeadNotes();
        $toLeadStateCode = $leadLeadModelTo->getLeadStateCode();
        $toLeadCity = $leadLeadModelTo->getLeadCity();
        $toLeadUserId = $leadLeadModelTo->getUserId();
        $toLeadOwnerId = $leadLeadModelTo->getOwnerId();
        $toLeadAddress = $leadLeadModelTo->getLeadAddress();
        $toLeadAddress1 = $leadLeadModelTo->getLeadAddress1();
        $toLeadExternalId = $leadLeadModelTo->getExternalId();
        $toLeadTypeId = $leadLeadModelTo->getLeadTypeId();
        $toLeadSource = $leadLeadModelTo->getLeadSource();
        $toLeadCustomerGroupId = $leadLeadModelTo->getCustomerGroupId();
        $toLeadCountryCode = $leadLeadModelTo->getLeadCountryCode();
        $toLeadOrganizationId = $leadLeadModelTo->getOrganizationId();
        $toLeadPhone = $leadLeadModelTo->getLeadPhone();
        $toLeadPhoneCountryCode = $leadLeadModelTo->getLeadPhoneCountryCode();
        $toLeadPhoneExtension = $leadLeadModelTo->getLeadPhoneExtension();
        $toLeadPhoneCountryCodeValue = $leadLeadModelTo->getLeadPhoneCountryCodeValue();
        $toLeadMobile = $leadLeadModelTo->getLeadMobile();
        $toLeadMobileCountryCode = $leadLeadModelTo->getLeadMobileCountryCode();
        $toLeadMobileCountryCodeValue = $leadLeadModelTo->getLeadMobileCountryCodeValue();
        $toUseOrganizationAddress = $leadLeadModelTo->getUseOrganizationAddress();
        $toLastActivityOutbound = $leadLeadModelTo->getLastActivityOutbound();
        $toStageId = $leadLeadModelTo->getStageId();
        $toLastActivityTypeId = $leadLeadModelTo->getLastActivityTypeId();

        $generalNotesInfo = array();

        //prevent adding duplicate by zip
        $skipZip = false;
        if (!empty($fromLeadFirstName) && !empty($fromLeadLastName) && !empty($fromLeadZip) && (empty($toLeadZip) || empty($toLeadLastName))) {
            $leadsModelByZip = $leadsMapper->findByFirstNameLastNameZip($fromLeadFirstName, $fromLeadLastName,
                $fromLeadZip);
            if ($leadsModelByZip instanceof Leads_Model_LeadsModel) {
                $generalNotesInfo['zip'] = $fromLeadZip;
                $skipZip = true;
            }
        }

        if (empty($toLeadFirstName)) {
            $leadLeadModelTo->setLeadFirstName($fromLeadFirstName);
        } elseif (!empty($fromLeadFirstName)) {
            $generalNotesInfo['firstname'] = $fromLeadFirstName;
        }

        if (empty($toLeadLastName)) {
            $leadLeadModelTo->setLeadLastName($fromLeadLastName);
        } elseif (!empty($fromLeadLastName)) {
            $generalNotesInfo['lastname'] = $fromLeadLastName;
        }

        if (empty($toLeadMiddleName)) {
            $leadLeadModelTo->setMiddleName($fromLeadMiddleName);
        } elseif (!empty($fromLeadMiddleName)) {
            $generalNotesInfo['middleName'] = $fromLeadMiddleName;
        }

        if (empty($toLeadPrefix)) {
            $leadLeadModelTo->setPrefix($fromLeadPrefix);
        } elseif (!empty($fromLeadPrefix)) {
            $generalNotesInfo['prefix'] = $fromLeadPrefix;
        }

        if (empty($toUseOrganizationAddress)) {
            $leadLeadModelTo->setUseOrganizationAddress($fromUseOrganizationAddress);
        } elseif (!empty($fromUseOrganizationAddress)) {
            $generalNotesInfo['useOrganizationAddress'] = $fromUseOrganizationAddress;
        }

        if (empty($toLastActivityOutbound)) {
            $leadLeadModelTo->setLastActivityOutbound($fromLastActivityOutbound);
        } elseif (!empty($fromLastActivityOutbound)) {
            $generalNotesInfo['lastActivityOutbound'] = $fromLastActivityOutbound;
        }

        if (empty($toStageId)) {
            $leadLeadModelTo->setStageId($fromStageId);
        } elseif (!empty($fromStageId)) {
            $stageModel = Leads_Mapper_LeadsStageMapper::getInstance()->find($fromStageId);
            if ($stageModel instanceof Leads_Model_LeadsStageModel) {
                $generalNotesInfo['stageId'] = $stageModel->getStageName();
            } else {
                $generalNotesInfo['stageId'] = $fromStageId;
            }

        }

        if (empty($toLastActivityTypeId)) {
            $leadLeadModelTo->setLastActivityTypeId($fromLastActivityTypeId);
        } elseif (!empty($fromLastActivityTypeId)) {
            $leadTypeModel = Leads_Mapper_LeadsTypeMapper::getInstance()->find($fromLastActivityTypeId);
            if ($leadTypeModel instanceof Leads_Model_LeadsTypeModel) {
                $generalNotesInfo['lastActivityTypeId'] = $leadTypeModel->getTypeLabel();
            } else {
                $generalNotesInfo['lastActivityTypeId'] = $fromLastActivityTypeId;
            }
        }

        if ($skipZip === false) {
            if (empty($toLeadZip)) {
                $leadLeadModelTo->setLeadZip($fromLeadZip);
            } elseif (!empty($fromLeadZip)) {
                $generalNotesInfo['zip'] = $fromLeadZip;
            }
        }

        if (empty($toLeadPosition)) {
            $leadLeadModelTo->setLeadPosition($fromLeadPosition);
        } elseif (!empty($fromLeadPosition)) {
            $generalNotesInfo['position'] = $fromLeadPosition;
        }

        if (empty($toLeadNotes)) {
            $leadLeadModelTo->setLeadNotes($fromLeadNotes);
        } elseif (!empty($fromLeadNotes)) {
            $generalNotesInfo['notes'] = $fromLeadNotes;
        }

        $formCountriesStates = Tools_LeadTools::getStates($fromLeadCountryCode, true, true);
        if (empty($formCountriesStates) || !array_key_exists($fromLeadStateCode, $formCountriesStates)) {
            $fromLeadStateCode = '';
        }

        if (empty($toLeadStateCode)) {
            $leadLeadModelTo->setLeadStateCode($fromLeadStateCode);
        } elseif (!empty($fromLeadStateCode)) {
            $generalNotesInfo['stateCode'] = $fromLeadStateCode;
        }

        if (empty($toLeadCity)) {
            $leadLeadModelTo->setLeadCity($fromLeadCity);
        } elseif (!empty($fromLeadCity)) {
            $generalNotesInfo['city'] = $fromLeadCity;
        }

        if (empty($toLeadUserId)) {
            $leadLeadModelTo->setUserId($fromLeadUserId);
        } elseif (!empty($fromLeadUserId)) {
            $userModel = $userMapper->find($fromLeadUserId);
            if ($userModel instanceof Application_Model_Models_User) {
                $generalNotesInfo['userId'] = $userModel->getFullName();
            } else {
                $generalNotesInfo['userId'] = $fromLeadUserId;
            }
        }

        if (empty($toLeadOwnerId)) {
            $leadLeadModelTo->setOwnerId($fromLeadOwnerId);
        } elseif (!empty($fromLeadOwnerId)) {
            $ownerModel = $userMapper->find($fromLeadOwnerId);
            if ($ownerModel instanceof Application_Model_Models_User) {
                $generalNotesInfo['ownerId'] = $ownerModel->getFullName();
            } else {
                $generalNotesInfo['ownerId'] = $fromLeadOwnerId;
            }

        }

        if (empty($toLeadAddress)) {
            $leadLeadModelTo->setLeadAddress($fromLeadAddress);
        } elseif (!empty($fromLeadAddress)) {
            $generalNotesInfo['address'] = $fromLeadAddress;
        }

        if (empty($toLeadAddress1)) {
            $leadLeadModelTo->setLeadAddress1($fromLeadAddress1);
        } elseif (!empty($fromLeadAddress1)) {
            $generalNotesInfo['address1'] = $fromLeadAddress1;
        }

        if (empty($toLeadExternalId)) {
            $leadLeadModelTo->setExternalId($fromLeadExternalId);
        } elseif (!empty($fromLeadExternalId)) {
            $generalNotesInfo['externalId'] = $fromLeadExternalId;
        }

        if (empty($toLeadTypeId)) {
            $leadLeadModelTo->setLeadTypeId($fromLeadTypeId);
        } elseif (!empty($fromLeadTypeId)) {
            $leadTypeModel = Leads_Mapper_LeadsTypeMapper::getInstance()->find($fromLeadTypeId);
            if ($leadTypeModel instanceof Leads_Model_LeadsTypeModel) {
                $generalNotesInfo['leadTypeId'] = $leadTypeModel->getTypeLabel();
            } else {
                $generalNotesInfo['leadTypeId'] = $fromLeadTypeId;
            }
        }

        if (empty($toLeadSource)) {
            $leadLeadModelTo->setLeadSource($fromLeadSource);
        } elseif (!empty($fromLeadSource)) {
            $generalNotesInfo['leadSource'] = $fromLeadSource;
        }

        if (empty($toLeadCustomerGroupId)) {
            $leadLeadModelTo->setCustomerGroupId($fromLeadCustomerGroupId);
            if (!empty($leadLeadModelTo->getUserId())) {
                Tools_LeadTools::syncUserGroup($leadLeadModelTo->getUserId(), $fromLeadCustomerGroupId);
            }
        } elseif (!empty($fromLeadCustomerGroupId)) {
            $customGroups = $leadsMapper->getCustomerGroups();
            if (!empty($customGroups) && array_key_exists($fromLeadCustomerGroupId, $customGroups)) {
                $generalNotesInfo['customerGroup'] = $customGroups[$fromLeadCustomerGroupId];
            } else {
                $generalNotesInfo['customerGroup'] = $fromLeadCustomerGroupId;
            }
        }

        if (empty($toLeadCountryCode)) {
            $leadLeadModelTo->setLeadCountryCode($fromLeadCountryCode);
        } elseif (!empty($fromLeadCountryCode)) {
            $generalNotesInfo['countryCode'] = $fromLeadCountryCode;
        }

        if (empty($toLeadOrganizationId)) {
            $leadLeadModelTo->setOrganizationId($fromLeadOrganizationId);
        } elseif (!empty($fromLeadOrganizationId)) {
            $leadOrganizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
            $leadOrganizationModel = $leadOrganizationMapper->find($fromLeadOrganizationId);
            if ($leadOrganizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                $generalNotesInfo['organizationId'] = $leadOrganizationModel->getOrganizationName();
            } else {
                $generalNotesInfo['organizationId'] = $fromLeadOrganizationId;
            }
        }

        if (empty($toLeadPhoneExtension)) {
            $leadLeadModelTo->setLeadPhoneExtension($fromLeadPhoneExtension);
        } elseif (!empty($fromLeadPhoneExtension)) {
            $generalNotesInfo['phoneExtension'] = $fromLeadPhoneExtension;
        }

        if ($toLeadPhoneCountryCode !== $fromLeadPhoneCountryCode) {
            $generalNotesInfo['phoneCountryCode'] = $fromLeadPhoneCountryCode;
        }

        if ($fromLeadPhone !== $toLeadPhone && empty($toLeadPhone)) {
            if (empty($toLeadPhone) && !empty($fromLeadPhone)) {
                $leadsModelWithPhoneNumber = Tools_LeadTools::findLeadByPhoneMobileNumber($fromLeadPhoneCountryCodeValue . $fromLeadPhone,
                    true, $fromLeadId);
                if (!$leadsModelWithPhoneNumber instanceof Leads_Model_LeadsModel) {
                    $leadLeadModelTo->setLeadPhone($fromLeadPhone);
                    $leadLeadModelTo->setLeadPhoneCountryCode($fromLeadPhoneCountryCode);
                    $leadLeadModelTo->setLeadPhoneCountryCodeValue($fromLeadPhoneCountryCodeValue);
                }
            } elseif (!empty($fromLeadPhone) && !empty($toLeadPhone)) {
                $generalNotesInfo['phone'] = $fromLeadPhone;
            }
        } elseif (!empty($fromLeadPhone) && $fromLeadPhone !== $toLeadPhone) {
            $generalNotesInfo['phone'] = $fromLeadPhone;
        }

        if ($toLeadMobileCountryCode !== $fromLeadMobileCountryCode) {
            $generalNotesInfo['mobileCountryCode'] = $fromLeadMobileCountryCode;
        }

        if ($fromLeadMobile !== $toLeadMobile && empty($toLeadMobile)) {
            if (empty($toLeadMobile) && !empty($fromLeadMobile)) {
                $leadsModelWithMobileNumber = Tools_LeadTools::findLeadByPhoneMobileNumber($fromLeadPhoneCountryCodeValue . $fromLeadMobile,
                    true, $fromLeadId);
                if (!$leadsModelWithMobileNumber instanceof Leads_Model_LeadsModel) {
                    $leadLeadModelTo->setLeadMobile($fromLeadMobile);
                    $leadLeadModelTo->setLeadMobileCountryCode($fromLeadMobileCountryCode);
                    $leadLeadModelTo->setLeadMobileCountryCodeValue($fromLeadMobileCountryCodeValue);
                }
            } elseif (!empty($fromLeadMobile) && !empty($toLeadMobile)) {
                $generalNotesInfo['mobile'] = $fromLeadMobile;
            }
        } elseif (!empty($fromLeadMobile) && $fromLeadMobile !== $toLeadMobile) {
            $generalNotesInfo['mobile'] = $fromLeadMobile;
        }

        if(!empty($fromLeadEmail)) {
            if(empty($toLeadEmail)) {
                $leadLeadModelTo->setLeadEmail($fromLeadEmail);

                if(!empty($fromLeadSecondaryEmail) && empty($toLeadSecondaryEmail)) {
                    $leadLeadModelTo->setLeadSecondaryEmail($fromLeadSecondaryEmail);
                } elseif(!empty($fromLeadSecondaryEmail) && !empty($toLeadSecondaryEmail)) {
                    $generalNotesInfo['secondaryEmail'] = $fromLeadSecondaryEmail;
                }

            } elseif(empty($toLeadSecondaryEmail)) {
                $leadLeadModelTo->setLeadSecondaryEmail($fromLeadEmail);

                if(!empty($fromLeadSecondaryEmail)) {
                    $generalNotesInfo['secondaryEmail'] = $fromLeadSecondaryEmail;
                }

            } else {
                if(!empty($fromLeadSecondaryEmail)) {
                    $generalNotesInfo['mainEmail'] = $fromLeadEmail;
                    $generalNotesInfo['secondaryEmail'] = $fromLeadSecondaryEmail;
                } else {
                    $generalNotesInfo['mainEmail'] = $fromLeadEmail;
                }

            }
        } elseif(empty($fromLeadEmail) && !empty($fromLeadSecondaryEmail)) {
            if(empty($toLeadEmail)) {
                $leadLeadModelTo->setLeadEmail($fromLeadSecondaryEmail);

            } elseif(empty($toLeadSecondaryEmail)) {
                $leadLeadModelTo->setLeadSecondaryEmail($fromLeadSecondaryEmail);

            } else {
                $generalNotesInfo['secondaryEmail'] = $fromLeadSecondaryEmail;

            }
        }

        $mergeDifferenceInformation = PHP_EOL;
        $leadMergeLabels = self::getLeadMergeLabels();

        if(!empty($generalNotesInfo)) {
            $generalNotesInfo = self::processNotesPosition($generalNotesInfo, self::SORT_TYPE_LEAD);
        }

        foreach ($generalNotesInfo as $noteName => $noteValue) {
            $labelName = $noteName;
            if (array_key_exists($noteName, $leadMergeLabels)) {
                $labelName = $leadMergeLabels[$noteName];
            }
            $mergeDifferenceInformation .= $labelName . ': ' . $noteValue . PHP_EOL;
        }

        $customParamsAddToNotes = self::mergeLeadCustomParams($fromLeadId, $toLeadId);

        if (!empty($customParamsAddToNotes)) {
            foreach ($customParamsAddToNotes as $customParamsAddToNote) {
                $value = $customParamsAddToNote['param_value'];
                if ($customParamsAddToNote['optionValue'] !== null) {
                    $value = $customParamsAddToNote['optionValue'];
                }
                $mergeDifferenceInformation .= $customParamsAddToNote['paramLabel'] . ': ' . $value . PHP_EOL;
            }
        }

        $mergeDifferenceInformation .= self::NOTES_DIVIDER . PHP_EOL;

        $leadLeadModelTo->setLeadNotes(trim($leadLeadModelTo->getLeadNotes()) . PHP_EOL . trim($mergeDifferenceInformation));


        self::reassignLeadInfo(array($fromLeadId), $toLeadId);
        self::reassignLeadEmailSequence($fromLeadId, $toLeadId);
        self::reassignOpportunity(array($fromLeadId), $toLeadId);
        self::reassignSocialNetworks($fromLeadId, $toLeadId);

        if ($removeMergedLead === true) {
            $leadsMapper->delete($fromLeadId);
            Leads_Mapper_LeadsLastViewedMapper::getInstance()->delete($fromLeadId, 'lead');
        }

        $leadsMapper->save($leadLeadModelTo);

    }

    /**
     * Sorted position Notes
     * @param $notesInfo
     * @return array
     */
    public static function processNotesPosition($notesInfo, $type)
    {
        $sortedFields = array();

        $positionFields = array(
            0 => 'prefix',
            1 => 'firstname',
            2 => 'middleName',
            3 => 'lastname',
            4 => 'email',
            5 => 'mainEmail',
            6 => 'secondaryEmail',
            7 => 'phoneCountryCode',
            8 => 'phoneCountryCodeValue',
            9 => 'phone',
            10 => 'phoneExtension',
            11 => 'mobileCountryCode',
            12 => 'mobileCountryCodeValue',
            13 => 'mobile',
            14 => 'countryCode',
            15 => 'stateCode',
            16 => 'zip',
            17 => 'city',
            18 => 'address',
            19 => 'address1',
            20 => 'notes',
            21 => 'primaryContactId',
            22 => 'organizationId',
            23 => 'externalId',
            24 => 'position',
            25 => 'userId',
            26 => 'ownerId',
            27 => 'leadSource',
            28 => 'stageId',
            29 => 'typeId',
            30 => 'customerGroupId',
            31 => 'useOrganizationAddress',
            32 => 'lastActivityOutbound',
            33 => 'lastActivityTypeId'
        );

        if($type == self::SORT_TYPE_ORGANIZATION) {
            $positionFields = array(
                0 => 'address',
                1 => 'address1',
                2 => 'email',
                3 => 'domain',
                4 => 'mobileCountryCode',
                5 => 'mobileCountryCodeValue',
                6 => 'mobile',
                7 => 'phoneCountryCode',
                8 => 'phoneCountryCodeValue',
                9 => 'phone',
                10 => 'phoneExtension',
                11 => 'countryCode',
                12 => 'stateCode',
                13 => 'city',
                14 => 'zip',
                15 => 'employees',
                16 => 'customerGroupId',
                17 => 'website',
                18 => 'industryId',
                19 => 'notes',
                20 => 'primaryContactId',
                21 => 'firstContactCreationDate',
                22 => 'externalId'
            );
        }

        foreach ($notesInfo as $key => $note) {
            $positionKey = array_search($key, $positionFields);

            if($positionKey !== false) {
                $sortedFields[$positionKey] = $note;
            }
        }

        if(!empty($sortedFields)) {
            ksort($sortedFields, SORT_NUMERIC);
            $sortedFieldsResult = array();

            foreach ($sortedFields as $key => $field) {
                $sortedFieldsResult[$positionFields[$key]] = $notesInfo[$positionFields[$key]];
            }
            $notesInfo = $sortedFieldsResult;
        }

        return $notesInfo;
    }

    /**
     * Merge opportunity
     *
     * @param array $fromLeadId from lead id
     * @param int $toLeadId to lead id
     * @return bool
     * @throws Zend_Db_Adapter_Exception
     */
    public static function reassignOpportunity(array $fromLeadId, $toLeadId)
    {
        Leads_Mapper_LeadsOpportunityMapper::getInstance()->changeLeadId($fromLeadId, $toLeadId);

        return true;
    }

    /**
     * Merge social networks
     *
     * @param int $fromLeadId from lead id
     * @param int $toLeadId to lead id
     * @return mixed
     * @throws Zend_Db_Adapter_Exception
     */
    public static function reassignSocialNetworks($fromLeadId, $toLeadId)
    {
        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
        $leadSocialNetworksFrom = $leadsMapper->getLeadSocialServices($fromLeadId);
        $changeLeadIds = array();
        if (!empty($leadSocialNetworksFrom)) {
            $leadSocialNetworksTo = $leadsMapper->getLeadSocialServices($toLeadId);
            if (empty($leadSocialNetworksTo)) {
                $leadsMapper->changeSocialServiceLeadId(array($fromLeadId), $toLeadId);
            } else {
                foreach ($leadSocialNetworksFrom as $leadSocialNetworksFromId => $leadSocialNetworksFromData) {
                    if (!array_key_exists($leadSocialNetworksFromId, $leadSocialNetworksTo)) {
                        $changeLeadIds[$leadSocialNetworksFromId] = $leadSocialNetworksFromId;
                    }
                }
            }
        }

        if (!empty($changeLeadIds)) {
            $leadsMapper->changeSocialServiceLeadId(array($fromLeadId), $toLeadId, $changeLeadIds);
        }
    }


    /**
     * Merge organization custom fields
     *
     * @param int $fromOrganizationId from organization id
     * @param int $toOrganizationId to organization id
     * @return array not merged fields info
     */
    public static function mergeOrganizationCustomParams($fromOrganizationId, $toOrganizationId)
    {
        $leadsOrganizationCustomParamsMapper = Leads_Mapper_LeadsOrganizationCustomParamsMapper::getInstance();
        $fromOrganizationCustomParams = $leadsOrganizationCustomParamsMapper->getAllParams($fromOrganizationId);
        $customParamsAddToNotes = array();
        if (!empty($fromOrganizationCustomParams)) {
            $toOrganizationCustomParams = $leadsOrganizationCustomParamsMapper->getAllParams($toOrganizationId);
            $allCustomParams = Leads_Mapper_LeadsOrganizationCustomParamsConfigMapper::getInstance()->getCustomParamsConfig();
            $customParamsOptionsData = Leads_Mapper_LeadsOrganizationCustomParamsOptionsDataMapper::getInstance()->getCustomParamsOptionsDataConfig();
            foreach ($fromOrganizationCustomParams as $paramId => $fromOrganizationCustomParam) {
                $leadsOrganizationCustomParamsModel = null;
                if (!array_key_exists($paramId, $toOrganizationCustomParams)) {
                    $leadsOrganizationCustomParamsModel = new Leads_Model_LeadsOrganizationCustomParamsModel();
                } elseif (empty($toOrganizationCustomParams[$paramId]['param_value'])) {
                    $leadsOrganizationCustomParamsModel = new Leads_Model_LeadsOrganizationCustomParamsModel();
                    $leadsOrganizationCustomParamsModel->setId($toOrganizationCustomParams[$paramId]['id']);
                } else {
                    $optionValue = null;
                    if (!empty($fromOrganizationCustomParam['params_option_id'])) {
                        $optionValue = $customParamsOptionsData[$fromOrganizationCustomParam['params_option_id']]['option_value'];
                    }

                    $customParamsAddToNotes[$paramId] = array(
                        'param_value' => $fromOrganizationCustomParam['param_value'],
                        'paramLabel' => $allCustomParams[$paramId]['label'],
                        'optionValue' => $optionValue
                    );
                }

                if ($leadsOrganizationCustomParamsModel instanceof Leads_Model_LeadsOrganizationCustomParamsModel) {
                    $leadsOrganizationCustomParamsModel->setOrganizationId($toOrganizationId);
                    $leadsOrganizationCustomParamsModel->setParamId($paramId);
                    $leadsOrganizationCustomParamsModel->setParamsOptionId($fromOrganizationCustomParam['params_option_id']);
                    $leadsOrganizationCustomParamsModel->setParamValue($fromOrganizationCustomParam['param_value']);
                    $leadsOrganizationCustomParamsMapper->save($leadsOrganizationCustomParamsModel);
                }
            }
        }

        return $customParamsAddToNotes;
    }


    /**
     * Merge leads custom fields
     *
     * @param int $fromLeadId from lead id
     * @param int $toLeadId to lead id
     * @return array not merged fields info
     */
    public static function mergeLeadCustomParams($fromLeadId, $toLeadId)
    {
        $leadsCustomParamsMapper = Leads_Mapper_LeadsCustomParamsMapper::getInstance();
        $fromCustomParams = $leadsCustomParamsMapper->getAllParams($fromLeadId);
        $customParamsAddToNotes = array();
        if (!empty($fromCustomParams)) {
            $toCustomParams = $leadsCustomParamsMapper->getAllParams($toLeadId);
            $allCustomParams = Leads_Mapper_LeadsCustomParamsConfigMapper::getInstance()->getCustomParamsConfig();
            $customParamsOptionsData = Leads_Mapper_LeadsCustomParamsOptionsDataMapper::getInstance()->getCustomParamsOptionsDataConfig();
            foreach ($fromCustomParams as $paramId => $fromCustomParam) {
                $leadsCustomParamsModel = null;
                if (!array_key_exists($paramId, $toCustomParams)) {
                    $leadsCustomParamsModel = new Leads_Model_LeadsCustomParamsModel();
                } elseif (empty($toCustomParams[$paramId]['param_value'])) {
                    $leadsCustomParamsModel = new Leads_Model_LeadsCustomParamsModel();
                    $leadsCustomParamsModel->setId($toCustomParams[$paramId]['id']);
                } else {
                    $optionValue = null;
                    if (!empty($fromCustomParam['params_option_id'])) {
                        $optionValue = $customParamsOptionsData[$fromCustomParam['params_option_id']]['option_value'];
                    }

                    $customParamsAddToNotes[$paramId] = array(
                        'param_value' => $fromCustomParam['param_value'],
                        'paramLabel' => $allCustomParams[$paramId]['label'],
                        'optionValue' => $optionValue
                    );
                }

                if ($leadsCustomParamsModel instanceof Leads_Model_LeadsCustomParamsModel) {
                    $leadsCustomParamsModel->setLeadId($toLeadId);
                    $leadsCustomParamsModel->setParamId($paramId);
                    $leadsCustomParamsModel->setParamsOptionId($fromCustomParam['params_option_id']);
                    $leadsCustomParamsModel->setParamValue($fromCustomParam['param_value']);
                    $leadsCustomParamsMapper->save($leadsCustomParamsModel);
                }
            }
        }

        return $customParamsAddToNotes;
    }

    /**
     * Reassign lead email sequence
     *
     * @param int $fromLeadId from lead id
     * @param int $newLeadId new lead id
     * @throws Zend_Db_Adapter_Exception
     *
     */
    public static function reassignLeadEmailSequence($fromLeadId, $newLeadId)
    {
        $leadEmailHasSequenceMapper = Leads_Mapper_LeadsHasEmailSequenceMapper::getInstance();
        $leadSequenceData =  $leadEmailHasSequenceMapper->getLeadSequenceByLeadId($fromLeadId);
        $changeSequenceIds = array();
        if (!empty($leadSequenceData)) {
            $leadSequenceDataTo =  $leadEmailHasSequenceMapper->getLeadSequenceByLeadId($newLeadId);
            foreach ($leadSequenceData as $leadSequenceId => $leadSequence) {
                if (!array_key_exists($leadSequenceId, $leadSequenceDataTo)) {
                    $changeSequenceIds[$leadSequenceId] = $leadSequenceId;
                }
            }
        }

        if (!empty($changeSequenceIds)) {
            $leadEmailHasSequenceMapper->changeEmailSequenceLead($fromLeadId, $newLeadId, $changeSequenceIds);
            Leads_Mapper_LeadsEmailSequenceStatusMapper::getInstance()->changeEmailSequenceLead($fromLeadId, $newLeadId, $changeSequenceIds);
        }
    }

    /**
     * Reassign lead action log data, lead notes
     *
     * @param array $leadIds
     * @param $newLeadId
     * @throws Zend_Db_Adapter_Exception
     */
    public static function reassignLeadInfo(array $leadIds, $newLeadId)
    {
        $leadNotesMapper = Leads_Mapper_LeadsNoteMapper::getInstance();
        $leadNotesMapper->changeLeadId(array($leadIds), $newLeadId);

        Leads_Mapper_LeadsActionLogMapper::getInstance()->changeLeadId($leadIds, $newLeadId);
        Leads_Mapper_LeadsCallLogMapper::getInstance()->changeLeadId($leadIds, $newLeadId);
        Leads_Mapper_LeadsDownloaderLogMapper::getInstance()->changeLeadId($leadIds, $newLeadId);
        Leads_Mapper_LeadsEmailLogMapper::getInstance()->changeLeadId($leadIds, $newLeadId);
        Leads_Mapper_LeadsFormLogMapper::getInstance()->changeLeadId($leadIds, $newLeadId);
        Leads_Mapper_LeadsIpUserProfileLogMapper::getInstance()->changeLeadId($leadIds, $newLeadId);
        Leads_Mapper_LeadsMeetingLogMapper::getInstance()->changeLeadId($leadIds, $newLeadId);
        Leads_Mapper_LeadsMeetingMapper::getInstance()->changeLeadId($leadIds, $newLeadId);
        Leads_Mapper_LeadsPurchaseLogMapper::getInstance()->changeLeadId($leadIds, $newLeadId);
        Leads_Mapper_LeadsQuoteLogMapper::getInstance()->changeLeadId($leadIds, $newLeadId);
        Leads_Mapper_LeadsRegistrationLogMapper::getInstance()->changeLeadId($leadIds, $newLeadId);
        Leads_Mapper_LeadsTasksLogMapper::getInstance()->changeLeadId($leadIds, $newLeadId);
        Leads_Mapper_LeadsTaskMapper::getInstance()->changeLeadId($leadIds, $newLeadId);
        Leads_Mapper_LeadsDocumentsMapper::getInstance()->changeLeadId($leadIds, $newLeadId);

    }

}
