<?php

/**
 * Class Tools_LeadMobileApiTools
 */
class Tools_LeadMobileApiTools
{

    /**
     * Delete lead
     *
     * @param int $leadId lead id
     * @return mixed
     */
    public static function deleteLead($leadId)
    {
       $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
       $leadModel = $leadMapper->find($leadId);
       if ($leadModel instanceof Leads_Model_LeadsModel) {
            $emailSequenceEnabled = Tools_LeadTools::verifyPluginEnabled(Tools_LeadTools::SEQUENCE_PLUGIN);
            if($emailSequenceEnabled) {
                $leadEmailHasSequenceMapper = Leads_Mapper_LeadsHasEmailSequenceMapper::getInstance();
                $leadHasSequences = $leadEmailHasSequenceMapper->getLeadSequenceByLeadId($leadId);

                if (!empty($leadHasSequences)) {
                    foreach ($leadHasSequences as $leadHasSequence) {
                        $sequenceId = $leadHasSequence['sequence_id'];
                        $leadId = $leadHasSequence['lead_id'];

                        Emailsequence_Models_Mapper_EmailsequenceQueueMapper::getInstance()->deleteQueueBySequenceIdAndExternalId($sequenceId, $leadId, 'lead', array('0'));
                    }
                }
            }

            $pluginLeadsRegistrationLogMapper = Leads_Mapper_LeadsRegistrationLogMapper::getInstance();
            $leadRegistrationModel = $pluginLeadsRegistrationLogMapper->findByLeadId($leadId);
            if ($leadRegistrationModel instanceof Leads_Model_LeadsRegistrationLogModel) {
                $pluginLeadsRegistrationLogMapper->delete($leadRegistrationModel->getId());
            }

            $res = $leadMapper->delete($leadId);
            $lastViewedLeadsAndOrgMapper = Leads_Mapper_LeadsLastViewedMapper::getInstance();
            $lastViewedLeadsAndOrgMapper->delete($leadId, Leads_Model_LeadsLastViewedModel::LEAD_LAST_VIEWED_TYPE_LEAD);

            return $res;
        }
    }


    /**
     * @param $data
     * @return array
     * @throws Zend_Exception
     */
    public static function addLeadInfo($data)
    {
        $translator = Zend_Registry::get('Zend_Translate');

        if (isset($data['leadEmail'])) {
            $data['leadEmail'] = trim($data['leadEmail']);
        }


        if (empty($data['ownerId'])) {
            $superAdminUserModel = Application_Model_Mappers_UserMapper::getInstance()->findByRole('superadmin');
            $data['ownerId'] = $superAdminUserModel->getId();
        }

        $allowedToCreate = false;
        if (!empty($data['leadEmail'])) {
            $allowedToCreate = true;
        } elseif (!empty($data['leadPhone']) || !empty($data['leadMobile'])) {
            $allowedToCreate = true;
        } elseif (!empty($data['leadFirstName']) && !empty($data['leadLastName']) && !empty($data['leadZip'])) {
            $allowedToCreate = true;
        }

        if (!$allowedToCreate) {
            return array(
                'error' => '1',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Please specify either email or phone number or first|last name and zip code')
            );
        }

        $leadAdditionAvailable = Tools_LeadLicenseTools::checkLimit();
        if ($leadAdditionAvailable['error'] === true) {
            $message = 'Congratulations, you\'re getting the hang of this. Now that you\'ve reached your subscription limit, please upgrade your subscription before adding more contacts.';
            return array(
                'error' => '1',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.$message)
            );
        }

        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
        $leadsModel = new Leads_Model_LeadsModel();

        if (!empty($data['leadMobile'])) {
            $data['lead_mobile'] = Tools_LeadTools::cleanPhoneAddress($data['leadMobile']);
        }


        if (!empty($data['leadPhone'])) {
            $data['lead_phone'] = Tools_LeadTools::cleanPhoneAddress($data['leadPhone']);
        }

        if (!empty($data['leadPhoneCountryCode'])) {
            $phoneCountryPhoneCode = Zend_Locale::getTranslation($data['leadPhoneCountryCode'], 'phoneToTerritory');
            $data['lead_phone_country_code_value'] = '+' . $phoneCountryPhoneCode;
            $data['lead_phone_country_code'] = $data['leadPhoneCountryCode'];
        }

        if (!empty($data['leadMobileCountryCode'])) {
            $mobileCountryPhoneCode = Zend_Locale::getTranslation($data['leadMobileCountryCode'], 'phoneToTerritory');
            $data['lead_mobile_country_code_value'] = '+' . $mobileCountryPhoneCode;
            $data['lead_mobile_country_code'] = $data['leadMobileCountryCode'];
        }

        if (empty($data['leadMobile'])) {
            $data['lead_mobile'] = null;
            $data['lead_mobile_country_code_value'] = null;
        } else {
            $data['lead_mobile'] = $data['leadMobile'];
        }

        if (!empty($data['leadPhone'])) {
            $data['lead_phone'] = Tools_LeadTools::cleanPhoneAddress($data['leadPhone']);
        }

        if (empty($data['leadPhone'])) {
            $data['lead_phone'] = null;
            $data['lead_phone_country_code_value'] = null;
        } else {
            $data['lead_phone'] = $data['leadPhone'];
        }

        $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
        $userDefaultMobileCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');
        if (empty($data['lead_mobile_country_code']) && !empty($userDefaultMobileCountryCode)) {
            $data['lead_mobile_country_code'] = $userDefaultMobileCountryCode;
        }

        if (empty($data['lead_phone_country_code']) && !empty($userDefaultMobileCountryCode)) {
            $data['lead_phone_country_code'] = $userDefaultMobileCountryCode;
        }

        if (!empty($data['leadFirstName'])) {
            $data['lead_first_name'] = $data['leadFirstName'];
        } else {
            $data['lead_first_name'] = '';
        }

        if (!empty($data['leadLastName'])) {
            $data['lead_last_name'] = $data['leadLastName'];
        } else {
            $data['lead_last_name'] = '';
        }

        if (!empty($data['leadPosition'])) {
            $data['lead_position'] = $data['leadPosition'];
        } else {
            $data['lead_position'] = '';
        }

        if (!empty($data['leadAddress'])) {
            $data['lead_address'] = $data['leadAddress'];
        } else {
            $data['lead_address'] = '';
        }

        if (!empty($data['leadAddress1'])) {
            $data['lead_address1'] = $data['leadAddress1'];
        } else {
            $data['lead_address1'] = '';
        }

        if (!empty($data['leadCity'])) {
            $data['lead_city'] = $data['leadCity'];
        } else {
            $data['lead_city'] = '';
        }

        if (!empty($data['leadZip'])) {
            $data['lead_zip'] = $data['leadZip'];
        } else {
            $data['lead_zip'] = '';
        }

        if (!empty($data['stageId'])) {
            $data['stage_id'] = $data['stageId'];
        } else {
            $data['stage_id'] = null;
        }

        if (!empty($data['stage_id'])) {
            $leadStageModel = Leads_Mapper_LeadsStageMapper::getInstance()->find($data['stage_id']);
            if (!$leadStageModel instanceof Leads_Model_LeadsStageModel) {
                $data['stage_id'] = null;
            }
        }

        if (empty($data['customerGroupId'])) {
            $data['customer_group_id'] = null;
        } else {
            $groupId = filter_var($data['customerGroupId'], FILTER_SANITIZE_NUMBER_INT);
            if (!empty($groupId)) {
                $validator = new Zend_Validate_Db_NoRecordExists(array(
                    'table' => 'shopping_group',
                    'field' => 'id'
                ));

                if ($validator->isValid($groupId)) {
                    return array('error' => '1', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Group is invalid.'));
                }
            }
            $data['customer_group_id'] = $data['customerGroupId'];
        }

        $leadsModel->setOptions($data);


        $assignPrimaryContactId = false;
        $leadOrganizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
        $organizationModel = null;

        /**
         * Create organization if not exists
         */
        if (!empty($data['organizationName'])) {
            $organizationName = $data['organizationName'];
            $organizationModel = $leadOrganizationMapper->findByName($organizationName);
            if (!$organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                $assignPrimaryContactId = true;
                $organizationModel = new Leads_Model_LeadsOrganizationModel();
                $organizationModel->setOrganizationName($organizationName);
                $createdAt = Tools_System_Tools::convertDateFromTimezone('now');
                $organizationModel->setCreatedAt($createdAt);
                $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
                $userDefaultMobileCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');
                if (!empty($userDefaultMobileCountryCode)) {
                    $organizationModel->setOrganizationMobileCountryCode($userDefaultMobileCountryCode);
                    $organizationModel->setOrganizationPhoneCountryCode($userDefaultMobileCountryCode);
                }

                $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();
                if (!empty($shoppingConfig['country'])) {
                    $defaultCountryCode = $shoppingConfig['country'];
                    $organizationModel->setOrganizationCountryCode($defaultCountryCode);
                }
                $organizationModel = $leadOrganizationMapper->save($organizationModel);
            }
            $leadsModel->setOrganizationId($organizationModel->getId());

        }

        if (!empty($data['leadEmail'])) {
            $emailValidator = new Tools_System_CustomEmailValidator();

            if (!$emailValidator->isValid($data['leadEmail'])) {
                return array(
                    'error' => '1',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Not valid email address')
                );
            }

            $validator = new Zend_Validate_Db_NoRecordExists(array(
                'table' => 'plugin_leads_data',
                'field' => 'lead_email'
            ));

            if (!$validator->isValid($data['leadEmail'])) {
                return array(
                    'error' => '1',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Email already in use')
                );
            }
        } else {
            $leadsModel->setLeadEmail(null);
        }

        $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
        $websiteUrl = $websiteHelper->getUrl();

        if (!empty($data['lead_phone_country_code_value']) && !empty($data['lead_phone'])) {
            $leadsModelWithPhone = Tools_LeadTools::findLeadByPhoneMobileNumber($data['lead_phone_country_code_value'] . $data['lead_phone'], true);
            if ($leadsModelWithPhone instanceof Leads_Model_LeadsModel) {
                $leadLink = $translator->translate(Leads::TRANSLATION_PREFIX."Lead");
                return array(
                    'error' => '1',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX."You already have a"). ' '.$leadLink. ' '. $translator->translate(Leads::TRANSLATION_PREFIX."with such phone number")
                );
            }
        }

        if (!empty($data['lead_mobile_country_code_value']) && !empty($data['lead_mobile'])) {
            $leadsModelWithPhone = Tools_LeadTools::findLeadByPhoneMobileNumber($data['lead_mobile_country_code_value'] . $data['lead_mobile'], true);
            if ($leadsModelWithPhone instanceof Leads_Model_LeadsModel) {
                $leadLink = $translator->translate(Leads::TRANSLATION_PREFIX."Lead");
                return array(
                    'error' => '1',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX."You already have a"). ' '.$leadLink. ' '. $translator->translate(Leads::TRANSLATION_PREFIX."with such mobile number")
                );
            }
        }

        if(!empty($data['leadFirstName']) && !empty($data['leadLastName']) && !empty($data['leadZip'])) {
            $leadsModelWithFirstLastNameZip = $leadsMapper->findByFirstNameLastNameZip($data['leadFirstName'], $data['leadLastName'], $data['leadZip']);

            if($leadsModelWithFirstLastNameZip instanceof Leads_Model_LeadsModel) {
                $leadLink = $translator->translate(Leads::TRANSLATION_PREFIX."Lead");
                return array(
                    'error' => '1',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX."You already have a"). ' '.$leadLink. ' '. $translator->translate(Leads::TRANSLATION_PREFIX."with such name and zip")
                );
            }

            $leadsModel->setLeadZip($data['leadZip']);
        }

        $createdAt = Tools_System_Tools::convertDateFromTimezone('now');

        $leadsModel->setOwnerId($data['ownerId']);
        $leadsModel->setCreatedAt($createdAt);

        $leadTypeModel = Leads_Mapper_LeadsTypeMapper::getInstance()->getDefaultType();
        if ($leadTypeModel instanceof Leads_Model_LeadsTypeModel && empty($data['lead_type_id'])) {
            $leadsModel->setLeadTypeId($leadTypeModel->getId());
        }

        $leadStageModel = Leads_Mapper_LeadsStageMapper::getInstance()->getDefaultStage();
        if ($leadStageModel instanceof Leads_Model_LeadsStageModel && empty($data['stage_id'])) {
            $leadsModel->setStageId($leadStageModel->getId());
        }

        $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();

        if (!empty($data['lead_phone_country_code'])) {
            $leadsModel->setLeadCountryCode($data['lead_phone_country_code']);
        }

        if (!empty($shoppingConfig['country'])) {
            $defaultCountryCode = $shoppingConfig['country'];
            if (empty($data['leadCountryCode'])) {
                $leadsModel->setLeadCountryCode($defaultCountryCode);
            } else {
                $leadsModel->setLeadCountryCode($data['leadCountryCode']);
            }
        }

        if (!empty($shoppingConfig['state'])) {
            $defaultStateId = $shoppingConfig['state'];
            if (empty($data['leadStateCode'])) {
                $allStates = Tools_LeadTools::getStates($data['leadCountryCode'], true, false, 'state');
                if (array_key_exists($defaultStateId, $allStates)) {
                    $leadsModel->setLeadStateCode($allStates[$defaultStateId]);
                }
            }
        }

        if (!empty($data['useOrgAddress'])) {
            $leadsModel->setUseOrganizationAddress('1');
        } else {
            $leadsModel->setUseOrganizationAddress('0');
        }

        $leadsModel = $leadsMapper->save($leadsModel);


        /**
         * Assign primary contact id if it's new organization
         */
        if ($assignPrimaryContactId === true) {
            $organizationModel->setPrimaryContactId($leadsModel->getId());
            $leadOrganizationMapper->save($organizationModel);
        }

        if ($organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
            $firstContactOrganizationDate = $organizationModel->getFirstContactCreationDate();
            if (empty($firstContactOrganizationDate)) {
                $organizationModel->setFirstContactCreationDate($leadsModel->getCreatedAt());
                $leadOrganizationMapper->save($organizationModel);
            }
        }

        return array(
            'error' => '0',
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead has been created'),
            'leadId' => $leadsModel->getId()
        );

    }

    /**
     * @param $data
     * @return array
     * @throws Zend_Exception
     */
    public static function editLeadInfo($data)
    {

        if (empty($data) || empty($data['leadId'])) {
            return array('error' => '1', 'message' => 'Missing lead id');
        }

        $leadId = $data['leadId'];
        $translator = Zend_Registry::get('Zend_Translate');

        $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
        $leadModel = $leadMapper->find($leadId);
        if (!$leadModel instanceof Leads_Model_LeadsModel) {
            return array('error' => '1', 'message' => 'Lead not found');
        }

        $currentLeadEmail = $leadModel->getLeadEmail();
        $currentMobilePhone = $leadModel->getLeadMobile();
        $currentPhone = $leadModel->getLeadPhone();
        if (empty($currentLeadEmail) && empty($currentMobilePhone) && empty($currentPhone) && (!empty($data['leadEmail']) || !empty($data['leadMobile']) || !empty($data['leadPhone']))) {
            $leadAdditionAvailable = Tools_LeadLicenseTools::checkLimit();
            if ($leadAdditionAvailable['error'] === true) {
                return array('error' => '1', 'message' => $leadAdditionAvailable['message']);
            }
        }

        if (!empty($data['useOrgAddress'])) {
            $data['leadAddress'] = $leadModel->getLeadAddress();
            $data['leadAddress1'] = $leadModel->getLeadAddress1();
            $data['leadCity'] = $leadModel->getLeadCity();
            $data['leadZip'] = $leadModel->getLeadZip();
            $data['leadCountryCode'] = $leadModel->getLeadCountryCode();
            $data['leadStateCode'] = $leadModel->getLeadStateCode();
        }

        if (!empty($data['leadEmail'])) {
            $data['lead_email'] = $data['leadEmail'];
        }

        if (!empty($data['leadMobile'])) {
            $data['lead_mobile'] = $data['leadMobile'];
        }

        if (!empty($data['leadPhone'])) {
            $data['lead_phone'] = $data['leadPhone'];
        }

        if (!empty($data['leadFirstName'])) {
            $data['lead_first_name'] = $data['leadFirstName'];
        } else {
            $data['lead_first_name'] = '';
        }

        if (!empty($data['leadLastName'])) {
            $data['lead_last_name'] = $data['leadLastName'];
        } else {
            $data['lead_last_name'] = '';
        }

        if (!empty($data['leadPosition'])) {
            $data['lead_position'] = $data['leadPosition'];
        } else {
            $data['lead_position'] = '';
        }

        if (!empty($data['leadAddress'])) {
            $data['lead_address'] = $data['leadAddress'];
        } else {
            $data['lead_address'] = '';
        }

        if (!empty($data['leadAddress1'])) {
            $data['lead_address1'] = $data['leadAddress1'];
        } else {
            $data['lead_address1'] = '';
        }

        if (!empty($data['leadCity'])) {
            $data['lead_city'] = $data['leadCity'];
        } else {
            $data['lead_city'] = '';
        }

        if (!empty($data['leadZip'])) {
            $data['lead_zip'] = $data['leadZip'];
        } else {
            $data['lead_zip'] = '';
        }

        if (!empty($data['stageId'])) {
            $data['stage_id'] = $data['stageId'];
        } else {
            $data['stage_id'] = null;
        }

        if (!empty($data['stage_id'])) {
            $leadStageModel = Leads_Mapper_LeadsStageMapper::getInstance()->find($data['stage_id']);
            if (!$leadStageModel instanceof Leads_Model_LeadsStageModel) {
                $data['stage_id'] = null;
            }
        }

        $oldEmail = $leadModel->getLeadEmail();
        $currentEmail = $data['lead_email'];
        if (!empty($data['leadPhoneCountryCode'])) {
            $phoneCountryPhoneCode = Zend_Locale::getTranslation($data['leadPhoneCountryCode'],
                'phoneToTerritory');
            $data['lead_phone_country_code_value'] = '+' . $phoneCountryPhoneCode;
            $data['lead_phone_country_code'] = $data['leadPhoneCountryCode'];
        }

        if (!empty($data['leadMobileCountryCode'])) {
            $mobileCountryPhoneCode = Zend_Locale::getTranslation($data['leadMobileCountryCode'],
                'phoneToTerritory');
            $data['lead_mobile_country_code_value'] = '+' . $mobileCountryPhoneCode;
            $data['lead_mobile_country_code'] = $data['leadMobileCountryCode'];
        }

        if (empty($data['customerGroupId'])) {
            $data['customer_group_id'] = null;
        } else {
            $groupId = filter_var($data['customerGroupId'], FILTER_SANITIZE_NUMBER_INT);
            if (!empty($groupId)) {
                $validator = new Zend_Validate_Db_NoRecordExists(array(
                    'table' => 'shopping_group',
                    'field' => 'id'
                ));

                if ($validator->isValid($groupId)) {
                    return array('error' => '1', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Group is invalid.'));
                }
            }
            $data['customer_group_id'] = $data['customerGroupId'];
        }

        $clearStateCode = false;

        if (empty($data['leadCountryCode'])) {
            $data['lead_country_code'] = null;
        } else {
            $data['lead_country_code'] = $data['leadCountryCode'];
            $stateCodes = Tools_LeadTools::getStates($data['leadCountryCode'], true, true);

            if (!empty($stateCodes) && empty($data['leadStateCode'])) {
                $stateCodesKeys = array_keys($stateCodes);
                $data['lead_state_code'] = $stateCodesKeys[0];
            } elseif (empty($stateCodes)) {
                $clearStateCode = true;
            }
        }

        if (empty($data['leadStateCode'])) {
            $data['lead_state_code'] = null;
        } elseif ($clearStateCode) {
            $data['lead_state_code'] = null;
        } elseif (!empty($stateCodes) && !empty($data['leadStateCode']) && empty($stateCodes[$data['leadStateCode']])) {
            $stateCodesKeys = array_keys($stateCodes);
            $data['lead_state_code'] = $stateCodesKeys[0];
        }

        if (!empty($data['lead_mobile'])) {
            $data['lead_mobile'] = Tools_LeadTools::cleanPhoneAddress($data['lead_mobile']);
        }

        if (!empty($data['lead_phone'])) {
            $data['lead_phone'] = Tools_LeadTools::cleanPhoneAddress($data['lead_phone']);
        }

        if (empty($data['lead_mobile'])) {
            $data['lead_mobile'] = null;
            $data['lead_mobile_country_code_value'] = null;
        }

        if (empty($data['lead_phone'])) {
            $data['lead_phone'] = null;
            $data['lead_phone_country_code_value'] = null;
        }

        if (!empty($data['lead_phone_country_code_value']) && !empty($data['lead_phone'])) {
            $leadsModelWithPhone = Tools_LeadTools::findLeadByPhoneMobileNumber($data['lead_phone_country_code_value'] . $data['lead_phone'],
                true, $leadId);
            if ($leadsModelWithPhone instanceof Leads_Model_LeadsModel) {
                $message = '';

                if ($data['lead_last_name'] != $leadsModelWithPhone->getLeadLastName()) {
                    //$message = ' ' . $translator->translate('Also Lead has different last name') . ' "' . $leadsModelWithPhone->getLeadLastName() . '".';
                }

                return array(
                    'error' => '1',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'You have another lead with such phone.').' '.$message
                );

            }
        }

        if (!empty($data['lead_mobile_country_code_value']) && !empty($data['lead_mobile'])) {
            $leadsModelWithMobile = Tools_LeadTools::findLeadByPhoneMobileNumber($data['lead_mobile_country_code_value'] . $data['lead_mobile'],
                true, $leadId);
            if ($leadsModelWithMobile instanceof Leads_Model_LeadsModel) {
                $message = '';

                if ($data['lead_last_name'] != $leadsModelWithMobile->getLeadLastName()) {
                    //$message = ' ' . $translator->translate('Also Lead has different last name') . ' "' . $leadsModelWithMobile->getLeadLastName() . '".';
                }

                return array(
                    'error' => '1',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'You have another lead with such phone.').' '.$message
                );
            }
        }

        if (!empty($data['lead_phone_country_code'])) {
            $phoneCountryPhoneCode = Zend_Locale::getTranslation($data['lead_phone_country_code'],
                'phoneToTerritory');
            $data['lead_phone_country_code_value'] = '+' . $phoneCountryPhoneCode;
        }

        if (!empty($data['lead_mobile_country_code'])) {
            $mobileCountryPhoneCode = Zend_Locale::getTranslation($data['lead_mobile_country_code'],
                'phoneToTerritory');
            $data['lead_mobile_country_code_value'] = '+' . $mobileCountryPhoneCode;
        }

        $leadUserId = $leadModel->getUserId();

        if (!empty($currentEmail)) {
            $emailValidator = new Tools_System_CustomEmailValidator();

            if (!$emailValidator->isValid($currentEmail)) {
                return array('error' => '1', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Not valid email address'));
            }
        }

        if (empty($currentEmail)) {
            $currentEmail = null;
            $data['lead_email'] = null;
        }

        if ($oldEmail !== $currentEmail) {
            $validateEmailExists = new Zend_Validate_Db_RecordExists(array(
                'table' => 'plugin_leads_data',
                'field' => 'lead_email'
            ));
            if ($validateEmailExists->isValid($currentEmail)) {
                $duplicatedLeadModel = $leadMapper->findByEmail($currentEmail);
                if ($duplicatedLeadModel instanceof Leads_Model_LeadsModel) {

                    return array(
                        'error' => '1',
                        'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'You have another lead with such email')
                    );
                }
            }
        }

        if (!empty($leadUserId) && $oldEmail !== $currentEmail) {
            return array(
                'error' => '1',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'You can\'t change user email. Real user already assigned to the lead.')
            );
        }

        if (!empty($leadUserId)) {
            Tools_LeadTools::syncUserGroup($leadUserId, $data['customer_group_id']);
        }

        $leadModel->setOptions($data);
        $createdAt = Tools_System_Tools::convertDateFromTimezone('now');

        /**
         * Create organization if not exists
         */
        if (!empty($data['organizationName'])) {
            $organizationName = filter_var($data['organizationName'], FILTER_SANITIZE_STRING);
            if (!empty($organizationName)) {
                $leadOrganizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
                $organizationModel = $leadOrganizationMapper->findByName($organizationName);
                if (!$organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                    $organizationModel = new Leads_Model_LeadsOrganizationModel();
                    $organizationModel->setOrganizationName($organizationName);
                    $organizationModel->setCreatedAt($createdAt);

                    $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
                    $userDefaultMobileCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');
                    if (!empty($userDefaultMobileCountryCode)) {
                        $organizationModel->setOrganizationMobileCountryCode($userDefaultMobileCountryCode);
                        $organizationModel->setOrganizationPhoneCountryCode($userDefaultMobileCountryCode);
                    }

                    $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();
                    if (!empty($shoppingConfig['country'])) {
                        $defaultCountryCode = $shoppingConfig['country'];
                        $organizationModel->setOrganizationCountryCode($defaultCountryCode);
                    }

                    /**
                     * Assign primary contact id if it's new organization
                     */
                    $organizationModel->setPrimaryContactId($leadId);
                    $organizationModel->setFirstContactCreationDate($data['created_at']);
                    $organizationModel = $leadOrganizationMapper->save($organizationModel);
                }
                $leadModel->setOrganizationId($organizationModel->getId());
                $organizationModelWithThisLeadContactId = $leadOrganizationMapper->findByPrimaryContactId($leadModel->getId());
                if ($organizationModelWithThisLeadContactId instanceof Leads_Model_LeadsOrganizationModel) {
                    $previousPrimaryOrganizationId = $organizationModelWithThisLeadContactId->getId();
                    $currentOrganizationId = $leadModel->getOrganizationId();
                    if ($previousPrimaryOrganizationId != $currentOrganizationId) {
                        $organizationModelWithThisLeadContactId->setPrimaryContactId(null);
                        $firstContactOrganizationDate = $organizationModel->getFirstContactCreationDate();
                        if (empty($firstContactOrganizationDate)) {
                            $organizationModel->setFirstContactCreationDate($data['created_at']);
                        }
                        $leadOrganizationMapper->save($organizationModelWithThisLeadContactId);
                    }
                }

            }
        } else {
            $leadModel->setOrganizationId(null);
        }

        $updatedAt = Tools_System_Tools::convertDateFromTimezone('now');

        if (!empty($data['useOrgAddress'])) {
            $leadModel->setUseOrganizationAddress('1');
        } else {
            $leadModel->setUseOrganizationAddress('0');
        }

        $leadModel->setUpdatedAt($updatedAt);

        $leadModel = $leadMapper->save($leadModel);

        return array('error' => '0', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead data has been updated'));

    }

    /**
     * Search organization
     *
     * @param string $searchTerm search term
     * @return array
     */
    public static function searchOrganization($searchTerm)
    {
        $leadOrganizationDataMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
        $where = $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_name LIKE ?',
            $searchTerm . '%');
        return $leadOrganizationDataMapper->fetchAll($where, array(), null, null, true, false);
    }

    /**
     * Add tag to lead (mobile app)
     *
     * @param int $leadId lead id
     * @param int $tagId tag id
     * @return array
     */
    public static function addTagToLead($leadId, $tagId)
    {
        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
        $leadModel = $leadsMapper->find($leadId);
        $translator = Zend_Registry::get('Zend_Translate');
        if (!$leadModel instanceof Leads_Model_LeadsModel) {
            return array('error' => '1', 'message' => $translator->translate('lead not found'));
        }

        $leadsTagMapper = Leads_Mapper_LeadsTagsMapper::getInstance();
        $leadTagModel = $leadsTagMapper->find($tagId);
        if (!$leadTagModel instanceof Leads_Model_LeadsTagModel) {
            return array('error' => '1', 'message' => $translator->translate('tag not found'));
        }

        $leadsMapper->addTag($leadId, $tagId);
        return array('id' => $tagId, 'label' => $leadTagModel->getLabel());

    }

    /**
     * Add lead tag(process new tag) to lead (mobile app)
     *
     * @param int $leadId lead id
     * @param $tagLabel
     * @return array
     */
    public static function addLeadTagByLabel($leadId, $tagLabel)
    {
        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
        $translator = Zend_Registry::get('Zend_Translate');
        $leadModel = $leadsMapper->find($leadId);
        if (!$leadModel instanceof Leads_Model_LeadsModel) {
            return array('error' => '1', 'message' => $translator->translate('lead not found'));
        }

        $leadsTagMapper = Leads_Mapper_LeadsTagsMapper::getInstance();
        $validateTagExists = new Zend_Validate_Db_RecordExists(array(
            'table' => 'plugin_leads_tags',
            'field' => 'tag_name'
        ));

        if ($validateTagExists->isValid($tagLabel)) {
            return array('error' => '1', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX . 'You have another tag with such name'));
        }

        $leadsTagModel = new Leads_Model_LeadsTagModel();
        $leadsTagModel->setLabel($tagLabel);
        $leadsTagModel->setTagName($tagLabel);
        $leadsTagModel = $leadsTagMapper->save($leadsTagModel);

        return self::addTagToLead($leadId, $leadsTagModel->getId());

    }

}
