<?php

class Tools_LeadPurchaseWatchdog implements Interfaces_Observer
{

    /**
     * Cart statuses for the sold quote
     *
     * @var array
     */
    public static $_cartQuoteSoldStatuses = array(
        Models_Model_CartSession::CART_STATUS_COMPLETED,
        Models_Model_CartSession::CART_STATUS_SHIPPED,
        Models_Model_CartSession::CART_STATUS_DELIVERED
    );


    /**
     * Logs only orders with status completed and ordered!!!
     *
     * @param $cartSessionModel
     * @return string
     */
    public function notify($cartSessionModel)
    {
        if ($cartSessionModel instanceof Models_Model_CartSession) {
            $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
            $userMapper = Application_Model_Mappers_UserMapper::getInstance();
            $cartId = $cartSessionModel->getId();
            $cartStatus = $cartSessionModel->getStatus();
            $currentLoggedUserId = $cartSessionModel->getUserId();
            $userModel = $userMapper->find($currentLoggedUserId);
            $purchasedOn = $cartSessionModel->getPurchasedOn();
            if (($cartStatus === Models_Model_CartSession::CART_STATUS_CANCELED || $cartStatus === Models_Model_CartSession::CART_STATUS_REFUNDED) && !empty($purchasedOn)) {
                $leadsOpportunityQuoteMapper = Leads_Mapper_LeadsOpportunityQuoteMapper::getInstance();
                $leadsOpportunityQuoteModel = $leadsOpportunityQuoteMapper->getByCartId($cartId);
                if ($leadsOpportunityQuoteModel instanceof Leads_Model_LeadsOpportunityQuoteModel) {
                    $opportunityId = $leadsOpportunityQuoteModel->getOpportunityId();
                    $leadsOpportunityMapper = Leads_Mapper_LeadsOpportunityMapper::getInstance();
                    $leadsOpportunityModel = $leadsOpportunityMapper->find($opportunityId);
                    if ($leadsOpportunityModel instanceof Leads_Model_LeadsOpportunityModel) {
                        $leadsOpportunityModel->setOpportunityStageId('1');
                        $leadsOpportunityModel->setChangeStageDate(Tools_System_Tools::convertDateFromTimezone('now'));
                        $leadsOpportunityMapper->save($leadsOpportunityModel);
                    }

                }
            }
            if ($userModel instanceof Application_Model_Models_User && ($cartStatus === Models_Model_CartSession::CART_STATUS_COMPLETED || $cartStatus === Models_Model_CartSession::CART_STATUS_REFUNDED || $cartStatus === Models_Model_CartSession::CART_STATUS_PARTIAL || $cartStatus === Models_Model_CartSession::CART_STATUS_NOT_VERIFIED)) {
                $leadsPurchaseLogMapper = Leads_Mapper_LeadsPurchaseLogMapper::getInstance();
                $leadPurchaseModel = $leadsPurchaseLogMapper->findByCartId($cartId, $cartStatus);
                if ($cartStatus === Models_Model_CartSession::CART_STATUS_REFUNDED) {
                    $cartTotal = $cartSessionModel->getRefundAmount();
                } elseif($cartStatus === Models_Model_CartSession::CART_STATUS_PARTIAL) {
                    $cartTotal = $cartSessionModel->getPartialPaidAmount();
                } else {
                    $cartTotal = $cartSessionModel->getTotal();
                }
                if ($leadPurchaseModel instanceof Leads_Model_LeadsPurchaseLogModel) {
                    return '';
                }

                $currentLoggedUserEmail = $userModel->getEmail();

                $leadModel = Tools_LeadTools::getLeadModelByIdAndEmail($currentLoggedUserId, $currentLoggedUserEmail);

                if (!Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
                    $ip = Tools_LeadTools::getIpAddress();
                }

                $ownerId = Tools_LeadTools::getDefaultLeadOwner();

                $createdAt = Tools_System_Tools::convertDateFromTimezone('now');
                $leadType =  Leads_Model_LeadsTypeModel::LEAD_TYPE_INBOUND_PURCHASE;

                $billingAddressId = $cartSessionModel->getBillingAddressId();
                $billingAddress = Tools_ShoppingCart::getAddressById($billingAddressId);
                $firstName = $billingAddress['firstname'];
                $lastName = $billingAddress['lastname'];
                $zip = $billingAddress['zip'];
                $city = $billingAddress['city'];
                $address = $billingAddress['address1'];
                $address2 = $billingAddress['address2'];
                $leadCountyCode = $billingAddress['country'];
                $leadStateCode = $billingAddress['state'];
                $mobileCountryCode = $billingAddress['mobilecountrycode'];
                $mobile = $billingAddress['mobile'];
                $phone = $billingAddress['phone'];
                $phoneCountryCode = $billingAddress['phonecountrycode'];
                $mobileCountryCodeValue = '';
                $phoneCountryCodeValue = '';
                if (!empty($leadStateCode) && is_numeric($leadStateCode)) {
                    $stateInfo = Tools_Geo::getStateById($leadStateCode);
                    if (!empty($stateInfo)) {
                        $leadStateCode = $stateInfo['state'];
                    }
                }

                $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
                $userDefaultMobileCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');
                if (empty($mobileCountryCode) && !empty($userDefaultMobileCountryCode)) {
                    $mobileCountryCode = $userDefaultMobileCountryCode;
                }

                if (empty($phoneCountryCode) && !empty($userDefaultMobileCountryCode)) {
                    $phoneCountryCode = $userDefaultMobileCountryCode;
                }

                if (!empty($mobile)) {
                    $mobile = Tools_LeadTools::cleanPhoneAddress($mobile);
                }

                if (!empty($phone)) {
                    $phone = Tools_LeadTools::cleanPhoneAddress($phone);
                }

                if (!empty($mobileCountryCode) && !empty($mobile)) {
                    $mobileNumberExists = Tools_LeadTools::checkLeadMobileOrPhoneNumber($mobile, $mobileCountryCode);
                    $mobileCountryCodeValue = Tools_LeadTools::getCountryCodeValue($mobileCountryCode);
                    if (!empty($mobileNumberExists)) {
                        $mobile = '';
                    }
                }
                if (!empty($phoneCountryCode) && !empty($phone)) {
                    $phoneNumberExists = Tools_LeadTools::checkLeadMobileOrPhoneNumber($phone, $phoneCountryCode);
                    $phoneCountryCodeValue = Tools_LeadTools::getCountryCodeValue($phoneCountryCode);
                    if (!empty($phoneNumberExists)) {
                        $phone = '';
                    }
                }

                $mobileNumberExists = Tools_LeadTools::checkLeadMobileOrPhoneNumber($mobile);
                if (!empty($mobileNumberExists)) {
                    $mobile = '';
                }
                $phoneNumberExists = Tools_LeadTools::checkLeadMobileOrPhoneNumber($phone);
                if (!empty($phoneNumberExists)) {
                    $phone = '';
                }

                $customerMapper = Models_Mapper_CustomerMapper::getInstance();
                $customerModel = $customerMapper->find($userModel->getId());

                if (!$leadModel instanceof Leads_Model_LeadsModel) {
                    $leadAdditionAvailable = Tools_LeadLicenseTools::checkLimit();
                    if ($leadAdditionAvailable['error'] === true) {
                        $leadModel = new Leads_Model_LeadsModel();
                        $leadModel->registerObserver(new Tools_Mail_Watchdog(array(
                            'trigger' => Tools_LeadsMailWatchdog::LEADS_LICENSE_LIMIT
                        )));

                        $leadModel->notifyObservers();
                        return '';
                    }
                    $leadTypeId = Tools_LeadTools::getLeadTypeIdByLeadTypeName($leadType);
                    $leadModel = new Leads_Model_LeadsModel();
                    $leadModel->setCreatedAt($createdAt);
                    $leadModel->setLeadEmail($currentLoggedUserEmail);
                    $leadModel->setOwnerId($ownerId);
                    $leadModel->setLeadTypeId($leadTypeId);
                    $leadModel->setLeadZip($zip);
                    $leadModel->setLeadAddress($address);
                    $leadModel->setLeadAddress1($address2);
                    $leadModel->setLeadCountryCode($leadCountyCode);
                    $leadModel->setLeadStateCode($leadStateCode);
                    $leadModel->setLeadCity($city);
                    $leadModel->setLeadFirstName($firstName);
                    $leadModel->setLeadMobileCountryCode($mobileCountryCode);
                    if (!empty($mobile)) {
                        $leadModel->setLeadMobile($mobile);
                        $leadModel->setLeadMobileCountryCodeValue($mobileCountryCodeValue);
                    }
                    $leadModel->setLeadPhoneCountryCode($phoneCountryCode);

                    if ($customerModel instanceof Models_Model_Customer) {
                        $customerGroupId = $customerModel->getGroupId();
                        if (!empty($customerGroupId)) {
                            $leadModel->setCustomerGroupId($customerModel->getGroupId());
                        }
                    }

                    if (!empty($phone)) {
                        $leadModel->setLeadPhone($phone);
                        $leadModel->setLeadPhoneCountryCodeValue($phoneCountryCodeValue);
                    }
                    $leadModel->setLeadLastName($lastName);
                    $leadModel->setLeadSource(Tools_LeadTools::getRefererUrl().' '.Tools_LeadTools::getUtmTagsString());
                } else {
                    $currentFirstName = $leadModel->getLeadFirstName();
                    $currentLeadLastName = $leadModel->getLeadLastName();
                    $currentZip = $leadModel->getLeadZip();
                    $currentAddress = $leadModel->getLeadAddress();
                    $currentAddress1 = $leadModel->getLeadAddress1();
                    $currentLeadCity = $leadModel->getLeadCity();
                    $currentLeadState = $leadModel->getLeadStateCode();
                    $currentLeadCountryCode = $leadModel->getLeadCountryCode();
                    $currentMobile = $leadModel->getLeadMobile();
                    $currentPhone = $leadModel->getLeadPhone();

                    if (empty($currentFirstName)) {
                        $leadModel->setLeadFirstName($firstName);
                    }
                    if (empty($currentLeadLastName)) {
                        $leadModel->setLeadLastName($lastName);
                    }

                    if (empty($currentZip)) {
                        $leadModel->setLeadZip($zip);
                    }

                    if (empty($currentAddress)) {
                        $leadModel->setLeadAddress($address);
                    }

                    if(empty($currentAddress1)){
                        $leadModel->setLeadAddress1($address2);
                    }

                    if (empty($currentLeadCity) && !empty($city)) {
                        $leadModel->setLeadCity($city);
                    }

                    if (empty($currentLeadState) && !empty($leadStateCode)) {
                        $leadModel->setLeadStateCode($leadStateCode);
                    }

                    if (empty($currentLeadCountryCode) && !empty($leadCountyCode)) {
                        $leadModel->setLeadCountryCode($leadCountyCode);
                    }

                    if (empty($currentMobile) && !empty($mobile)) {
                        $leadModel->setLeadMobile($mobile);
                        $leadModel->setLeadMobileCountryCodeValue($mobileCountryCodeValue);
                        $leadModel->setLeadMobileCountryCode($mobileCountryCode);
                    }
                    if (empty($currentPhone) && !empty($phone)) {
                        $leadModel->setLeadPhone($phone);
                        $leadModel->setLeadPhoneCountryCode($phoneCountryCode);
                        $leadModel->setLeadPhoneCountryCodeValue($phoneCountryCodeValue);
                    }

                }
                $oldUserId = $leadModel->getUserId();
                if (empty($oldUserId)) {
                    $leadModel->setUserId($currentLoggedUserId);
                }

                $leadTypeModel = Leads_Mapper_LeadsTypeMapper::getInstance()->findByTypeName($leadType);
                if ($leadTypeModel instanceof Leads_Model_LeadsTypeModel) {
                    $leadModel->setLastActivityTypeId($leadTypeModel->getId());
                    $leadModel->setLastActivitySubtype('');
                }
                $leadModel->setLastActivity($createdAt);
                try {
                    $leadModel = $leadsMapper->save($leadModel);
                } catch (Exception $e) {
                    return false;
                }
                $leadId = $leadModel->getId();

                $processProductTags = Leads_Mapper_LeadsConfigMapper::getInstance()->getConfigParam(Api_Leads_Leadgeneralconfig::PROCESS_PRODUCT_TAGS);

                if(!empty($processProductTags)) {
                    $productMapper = Models_Mapper_ProductMapper::getInstance();
                    $cartContent = $cartSessionModel->getCartContent();

                    if(!empty($cartContent)) {
                        $productTags = array();
                        foreach ($cartContent as $key => $product) {
                            $productObject = $productMapper->find($product['product_id']);
                            if ($productObject instanceof Models_Model_Product) {
                                $prodTags = $productObject->getTags();

                                if(!empty($prodTags)) {
                                    foreach ($prodTags as $tag) {
                                        if(!in_array($tag['name'], $productTags)) {
                                            array_push($productTags, $tag['name']);
                                        }
                                    }
                                }
                            }
                        }

                        if(!empty($productTags)) {
                            Tools_LeadImportTools::processLeadTags($leadId, $productTags);
                        }
                    }
                }

                if (!empty($ip)) {
                    $leadsIpsModel = Tools_LeadTools::addIpAddressToLead($leadId, $ip);
                    if ($leadsIpsModel instanceof Leads_Model_LeadsIpsModel) {
                        Tools_LeadTools::addIpToMojo($leadsIpsModel);
                    }
                }

                $leadsActionLog = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_PURCHASE,
                    $leadId,
                    Leads_Model_LeadsActionLogModel::ACTION_TYPE_INBOUND, $createdAt, $leadType);

                $isManuallyPaid = '0';
                $partiallyPaidAmount = '0';

                if ($cartStatus === Models_Model_CartSession::CART_STATUS_PARTIAL) {
                    $partiallyPaidAmount = $cartSessionModel->getPartialPaidAmount();
                    if (!empty($cartSessionModel->getIsFirstPaymentManuallyPaid())) {
                        $isManuallyPaid = '1';
                    }
                }

                if ($cartStatus === Models_Model_CartSession::CART_STATUS_COMPLETED && !empty($cartSessionModel->getIsPartial())) {
                    $partiallyPaidAmount = $cartSessionModel->getTotal() - $cartSessionModel->getFirstPartialPaidAmount();
                    if (!empty($cartSessionModel->getIsSecondPaymentManuallyPaid())) {
                        $isManuallyPaid = '1';
                    }
                }

                $signedOnlyById = 0;
                if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
                    $session = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
                    $currentLoggedUserId = $session->getCurrentUser()->getId();
                    $creatorId = $currentLoggedUserId;
                    if ($cartStatus === Models_Model_CartSession::CART_STATUS_NOT_VERIFIED) {
                        $signedOnlyById  = $creatorId;
                    }
                }

                $leadPurchaseLogModel = new Leads_Model_LeadsPurchaseLogModel();
                $leadPurchaseLogModel->setStatus($cartStatus);
                $leadPurchaseLogModel->setCartId($cartId);
                $leadPurchaseLogModel->setCreatedAt($createdAt);
                $leadPurchaseLogModel->setActionLogId($leadsActionLog->getId());
                $leadPurchaseLogModel->setLeadId($leadId);
                $leadPurchaseLogModel->setTotal($cartTotal);
                $leadPurchaseLogModel->setIsPartial($cartSessionModel->getIsPartial());
                $leadPurchaseLogModel->setGateway($cartSessionModel->getGateway());
                $leadPurchaseLogModel->setIsManuallyPaid($isManuallyPaid);
                $leadPurchaseLogModel->setPartiallyPaidAmount($partiallyPaidAmount);
                $leadPurchaseLogModel->setSignedOnlyBy($signedOnlyById);
                $leadsPurchaseLogMapper->save($leadPurchaseLogModel);

                if (in_array($cartStatus, self::$_cartQuoteSoldStatuses, true)) {
                    $leadsOpportunityQuoteMapper = Leads_Mapper_LeadsOpportunityQuoteMapper::getInstance();
                    $leadsOpportunityQuoteModel = $leadsOpportunityQuoteMapper->getByCartId($cartId);
                    if ($leadsOpportunityQuoteModel instanceof Leads_Model_LeadsOpportunityQuoteModel) {
                        $opportunityId = $leadsOpportunityQuoteModel->getOpportunityId();
                        $leadsOpportunityMapper = Leads_Mapper_LeadsOpportunityMapper::getInstance();
                        $leadsOpportunityModel = $leadsOpportunityMapper->find($opportunityId);
                        if ($leadsOpportunityModel instanceof Leads_Model_LeadsOpportunityModel) {
                            $leadsOpportunityModel->setOpportunityStageId('100');
                            $leadsOpportunityModel->setChangeStageDate(Tools_System_Tools::convertDateFromTimezone('now'));
                            $leadsOpportunityMapper->save($leadsOpportunityModel);
                        }

                    }
                }

            }
        }

    }
}

