<?php

class Tools_LeadRegistrationWatchdog implements Interfaces_Observer
{

    public function notify($registrationModel)
    {
        if (!$registrationModel instanceof Application_Model_Models_User && !$registrationModel instanceof Models_Model_Customer) {
            return '';
        }

        $userMapper = Application_Model_Mappers_UserMapper::getInstance();
        $ownerIdFrom = $registrationModel->getId();
        $userModel = $userMapper->find($ownerIdFrom);
        $ownerIdTo = Tools_LeadTools::getDefaultLeadOwner();
        if (!$userModel instanceof Application_Model_Models_User) {
            Tools_LeadChangeOwnerTools::changeOwner($ownerIdFrom, $ownerIdTo);
        }

        $originalRegistrationDate = $registrationModel->getRegDate();
        $userId = $registrationModel->getId();
        $userRole = $registrationModel->getRoleId();
        if (!empty($originalRegistrationDate)) {
            $serverDateNow = date('Y-m-d', strtotime('now'));
            $originalRegistrationDateApplied = date('Y-m-d', strtotime($registrationModel->getRegDate()));
            if ($serverDateNow !== $originalRegistrationDateApplied) {
                return '';
            }
        }
        $registrationPageUrl ='';
        $registrationPageId = '';
        $params = Zend_Controller_Front::getInstance()->getRequest()->getParams();
        if (!empty($params['PageId'])) {
            $registrationPageId = $params['PageId'];
            $pageModel = Application_Model_Mappers_PageMapper::getInstance()->find($params['PageId']);
            if ($pageModel instanceof Application_Model_Models_Page) {
                $registrationPageUrl = $pageModel->getUrl();
            }
        } else {
            $checkoutPage = Tools_Misc::getCheckoutPage();
            if ($checkoutPage instanceof Application_Model_Models_Page) {
                $registrationPageUrl = $checkoutPage->getUrl();
                $registrationPageId = $checkoutPage->getId();
            }
        }


        if (!empty($originalRegistrationDate)) {
            $defaultTimezone = date_default_timezone_get();
            if (empty($defaultTimezone)) {
                $createdAt = Tools_System_Tools::convertDateFromTimezone('now');
            } else {
                $createdAt = Tools_System_Tools::convertDateFromTimezone($originalRegistrationDate, $defaultTimezone);
            }
        } else {
            $createdAt = Tools_System_Tools::convertDateFromTimezone('now');
        }
        $pluginLeadsRegistrationLogMapper = Leads_Mapper_LeadsRegistrationLogMapper::getInstance();
        $leadEmail = $registrationModel->getEmail();
        $pluginLeadsRegistrationLogModel = $pluginLeadsRegistrationLogMapper->findByEmail($leadEmail);
        if ($pluginLeadsRegistrationLogModel instanceof Leads_Model_LeadsRegistrationLogModel) {
            return '';
        }
        $fullNameData = explode(' ', $registrationModel->getFullName());
        $mobile = $registrationModel->getMobilePhone();
        $phone = '';
        $firstName = $fullNameData[0];
        $lastName = (!empty($fullNameData[1])) ? $fullNameData[1] : '';
        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();

        $leadModel = $leadsMapper->findByEmail($leadEmail);

        if(!$leadModel instanceof Leads_Model_LeadsModel && !empty($userId)) {
            $leadModel = $leadsMapper->findByUserId($userId);

            if($leadModel instanceof Leads_Model_LeadsModel) {
                $leadModel->setLeadEmail($leadEmail);

                try {
                    $leadsMapper->save($leadModel);
                    return true;
                } catch (Exception $e) {
                    return false;
                }
            }
        }

        $ip = Tools_LeadTools::getIpAddress();

        $leadType = Leads_Model_LeadsTypeModel::LEAD_INBOUND_REGISTRATION;

        $leadMobileCountryCode = '';
        $leadMobileCountryCodeValue = '';
        $leadPhoneCountryCode = '';
        $leadPhoneCountryCodeValue = '';

        if ($registrationModel instanceof Application_Model_Models_User) {
            $leadMobileCountryCodeValue = $registrationModel->getMobileCountryCodeValue();
            if (empty($leadMobileCountryCodeValue)) {
                $leadMobileCountryCodeValue = '';
            }
            $leadMobileCountryCode = $registrationModel->getMobileCountryCode();
            if (empty($leadMobileCountryCode)) {
                $leadMobileCountryCode = '';
            }
            $leadPhoneCountryCode = $registrationModel->getDesktopCountryCode();
            if (empty($leadPhoneCountryCode)) {
                $leadPhoneCountryCode = '';
            }
            $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
            $userDefaultMobileCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');
            if (empty($leadMobileCountryCode) && !empty($userDefaultMobileCountryCode)) {
                $leadMobileCountryCode = $userDefaultMobileCountryCode;
            }

            if (empty($leadPhoneCountryCode) && !empty($userDefaultMobileCountryCode)) {
                $leadPhoneCountryCode = $userDefaultMobileCountryCode;
            }

            $leadPhoneCountryCodeValue = $registrationModel->getDesktopCountryCodeValue();
            if (empty($leadPhoneCountryCodeValue)) {
                $leadPhoneCountryCodeValue = '';
            }
            $phone = $registrationModel->getDesktopPhone();

        } else {
            $leadMobileCountryCodeExists = Tools_LeadTools::getLoggedUserAttribute('mobilecountrycode', $userId);
            if (!empty($leadMobileCountryCodeExists['value'])) {
                $leadMobileCountryCode = $leadMobileCountryCodeExists['value'];
                $phoneCountryPhoneCode = Zend_Locale::getTranslation($leadMobileCountryCode, 'phoneToTerritory');
                if (!empty($phoneCountryPhoneCode)) {
                    $leadMobileCountryCodeValue = '+' . $phoneCountryPhoneCode;
                }
            }
        }

        if (!$leadModel instanceof Leads_Model_LeadsModel) {
            $leadAdditionAvailable = Tools_LeadLicenseTools::checkLimit();
            if ($leadAdditionAvailable['error'] === true) {
                $leadModel = new Leads_Model_LeadsModel();
                $leadModel->registerObserver(new Tools_Mail_Watchdog(array(
                    'trigger' => Tools_LeadsMailWatchdog::LEADS_LICENSE_LIMIT
                )));

                $leadModel->notifyObservers();
                return '';
            }
            $leadTypeId = Tools_LeadTools::getLeadTypeIdByLeadTypeName($leadType);
            $leadModel = new Leads_Model_LeadsModel();
            $leadModel->setCreatedAt($createdAt);
            $leadModel->setLeadEmail($leadEmail);
            $leadModel->setOwnerId($ownerIdTo);
            $leadModel->setLeadTypeId($leadTypeId);
            $leadModel->setLeadSource(Tools_LeadTools::getRefererUrl().' '.Tools_LeadTools::getUtmTagsString());
            $leadModel->setLeadPhoneCountryCode($leadPhoneCountryCode);
            $leadModel->setLeadMobileCountryCode($leadMobileCountryCode);
        }

        $currentMobileCountryCode = $leadModel->getLeadMobileCountryCode();
        $currentLeadMobileCountryCodeValue = $leadModel->getLeadMobileCountryCodeValue();
        $currentPhoneCountryCode = $leadModel->getLeadPhoneCountryCode();
        $currentPhoneCountryCodeValue = $leadModel->getLeadPhoneCountryCodeValue();
        $currentLeadMobile = $leadModel->getLeadMobile();
        $currentLeadPhone = $leadModel->getLeadPhone();

        $updateMobileNumber = false;
        $updatePhoneNumber = false;
        if (!empty($leadMobileCountryCodeValue) && !empty($mobile) && empty($currentLeadMobile)) {
            $updateMobileNumber = true;
            $mobile = str_replace($leadMobileCountryCodeValue, '',  $mobile);
            if (!$leadModel instanceof Leads_Model_LeadsModel) {
                $leadsModelWithPhone = Tools_LeadTools::findLeadByPhoneMobileNumber($leadMobileCountryCodeValue . $mobile, true);
            } else {
                $leadsModelWithPhone = Tools_LeadTools::findLeadByPhoneMobileNumber($leadMobileCountryCodeValue . $mobile, true, $leadModel->getId());
            }

            if ($leadsModelWithPhone instanceof Leads_Model_LeadsModel) {
               $updateMobileNumber = false;
            }
        }

        if (!empty($leadPhoneCountryCodeValue) && !empty($phone) && empty($currentLeadPhone)) {
            $updatePhoneNumber = true;
            $phone = str_replace($leadMobileCountryCodeValue, '',  $phone);
            if (!$leadModel instanceof Leads_Model_LeadsModel) {
                $leadsModelWithPhoneDesktop = Tools_LeadTools::findLeadByPhoneMobileNumber($leadMobileCountryCodeValue . $phone, true);
            } else {
                $leadsModelWithPhoneDesktop = Tools_LeadTools::findLeadByPhoneMobileNumber($leadMobileCountryCodeValue . $phone, true, $leadModel->getId());
            }

            if ($leadsModelWithPhoneDesktop instanceof Leads_Model_LeadsModel) {
                $updatePhoneNumber = false;
            }
        }

        if ($updateMobileNumber) {
            if (empty($currentMobileCountryCode) && !empty($leadMobileCountryCode)) {
                $leadModel->setLeadMobileCountryCode($leadMobileCountryCode);
            }

            if (empty($currentLeadMobileCountryCodeValue) && !empty($leadMobileCountryCodeValue)) {
                $leadModel->setLeadMobileCountryCodeValue($leadMobileCountryCodeValue);
            }
        } else {
            $leadsModelWithPhone = Tools_LeadTools::findLeadByPhoneMobileNumber($mobile, true, $leadModel->getId());
            if ($leadsModelWithPhone instanceof Leads_Model_LeadsModel) {
                $updateMobileNumber = false;
            }
        }

        if ($updatePhoneNumber) {
            if (empty($currentPhoneCountryCode) && !empty($leadPhoneCountryCode)) {
                $leadModel->setLeadPhoneCountryCode($leadPhoneCountryCode);
            }

            if (empty($currentPhoneCountryCodeValue) && !empty($leadPhoneCountryCodeValue)) {
                $leadModel->setLeadPhoneCountryCodeValue($leadPhoneCountryCodeValue);
            }
        } else {
            $leadsModelWithPhone = Tools_LeadTools::findLeadByPhoneMobileNumber($phone, true, $leadModel->getId());
            if ($leadsModelWithPhone instanceof Leads_Model_LeadsModel) {
                $updatePhoneNumber = false;
            }
        }

        $leadModel->setLeadFirstName($firstName);
        if ($updateMobileNumber) {
            $leadModel->setLeadMobile($mobile);
        }
        if ($updatePhoneNumber) {
            $leadModel->setLeadPhone($phone);
        }
        $leadModel->setLeadLastName($lastName);
        $leadModel->setUserId($userId);
        $leadTypeModel = Leads_Mapper_LeadsTypeMapper::getInstance()->findByTypeName($leadType);
        if ($leadTypeModel instanceof Leads_Model_LeadsTypeModel) {
            $leadModel->setLastActivityTypeId($leadTypeModel->getId());
            $leadModel->setLastActivitySubtype('');
        }
        $leadModel->setLastActivity($createdAt);

        $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();
        if (!empty($shoppingConfig['country'])) {
            if (empty($countryCode)) {
                $countryCode = $shoppingConfig['country'];
            }
        }

        if (!empty($countryCode)) {
            $currentLeadCountryCode = $leadModel->getLeadCountryCode();
            if (empty($currentLeadCountryCode)) {
                $leadModel->setLeadCountryCode($countryCode);
            }
        }

        if (!empty($shoppingConfig['state']) && empty($leadModel->getLeadStateCode())) {
            $defaultStateId = $shoppingConfig['state'];
             $allStates = Tools_LeadTools::getStates($leadModel->getLeadCountryCode(), true, false, 'state');
             if (array_key_exists($defaultStateId, $allStates)) {
                $leadModel->setLeadStateCode($allStates[$defaultStateId]) ;
             }

        }

        try {
            $leadModel = $leadsMapper->save($leadModel);
        } catch (Exception $e) {
            return false;
        }

        $processedUserModel = $userMapper->find($userId);
        if (!empty($processedUserModel)) {
            $userCustomAttributes = $processedUserModel->getAttributes();
            if (!empty($userCustomAttributes)) {
                $leadId = $leadModel->getId();
                $zip = Tools_LeadTools::getValue($userCustomAttributes, 'zip');
                $address = Tools_LeadTools::getValue($userCustomAttributes, 'address');
                $address1 = Tools_LeadTools::getValue($userCustomAttributes, 'address1');
                $position = Tools_LeadTools::getValue($userCustomAttributes, 'position');
                $prefix = Tools_LeadTools::getValue($userCustomAttributes, 'prefix');
                $city = Tools_LeadTools::getValue($userCustomAttributes, 'city');
                $countryCode = Tools_LeadTools::getValue($userCustomAttributes, 'country');
                $stateCode = Tools_LeadTools::getValue($userCustomAttributes, 'state');
                $organization = Tools_LeadTools::getValue($userCustomAttributes, 'organization');
                $organizationId = Tools_LeadTools::getValue($userCustomAttributes, 'organizationid');
                $employees = filter_var(Zend_Controller_Front::getInstance()->getRequest()->getParam('employees'), FILTER_SANITIZE_NUMBER_INT);
                $groupName = Tools_LeadTools::getValue($userCustomAttributes, 'groupname');
                $tagNames = Tools_LeadTools::getValue($userCustomAttributes, 'tagname');
                $website = Tools_LeadTools::getValue($userCustomAttributes, 'website');
                if (!is_array($tagNames)) {
                    $tagNames = array_filter(explode(',', $tagNames));
                }

                $withTags = false;

                if (!empty($groupName)) {
                    $groupMapper = Store_Mapper_GroupMapper::getInstance();
                    $whereGroup = $groupMapper->getDbTable()->getAdapter()->quoteInto('groupName = ?', $groupName);
                    $groupExists = $groupMapper->fetchAll($whereGroup);
                    if (!empty($groupExists)) {
                        $groupId = $groupExists[0]->getId();
                    }
                }

                if (!empty($tagNames)) {
                    $withTags = true;
                }

                $countryCodes = Tools_Geo::getCountries(true);
                if (!empty($countryCode)) {
                    if (!array_key_exists($countryCode, $countryCodes)) {
                        $countryCode = '';
                    }
                }

                $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();
                if (!empty($shoppingConfig['country'])) {
                    if (empty($countryCode)) {
                        $countryCode = $shoppingConfig['country'];
                    }
                }

                if ($withTags) {
                    $currentLeadId = $leadModel->getId();
                    $tagMapper = Leads_Mapper_LeadsTagsMapper::getInstance();

                    $incomingTags = $tagMapper->getAllIncomingTags($tagNames);
                    if (!empty($incomingTags)) {
                        $incomingTags = array_flip($incomingTags);

                        if (!empty($currentLeadId)) {
                            $findLeadByTag = $tagMapper->findTagsByLeadId($leadModel->getId());
                            if (!empty($findLeadByTag)) {
                                foreach ($incomingTags as $tag) {
                                    if (!in_array($tag, $findLeadByTag)) {
                                        array_push($findLeadByTag, $tag);
                                    }
                                }
                            } else {
                                $findLeadByTag = $incomingTags;
                            }
                            $leadModel->setLeadTags($findLeadByTag);
                        } else {
                            $leadModel->setLeadTags($incomingTags);
                        }
                    }
                }
                $currentFirstName = $leadModel->getLeadFirstName();
                $currentLeadLastName = $leadModel->getLeadLastName();
                $currentZip = $leadModel->getLeadZip();
                $currentAddress = $leadModel->getLeadAddress();
                $currentPosition = $leadModel->getLeadPosition();
                $currentPrefix = $leadModel->getPrefix();
                $currentCity = $leadModel->getLeadCity();
                $currentCountry = $leadModel->getLeadCountryCode();
                $currentState = $leadModel->getLeadStateCode();
                $currentAddress1 = $leadModel->getLeadAddress1();
                $currentCustomerGroupId = $leadModel->getCustomerGroupId();

                if (empty($currentFirstName)) {
                    $leadModel->setLeadFirstName($firstName);
                }
                if (empty($currentLeadLastName)) {
                    $leadModel->setLeadLastName($lastName);
                }

                if (empty($currentCustomerGroupId) && !empty($groupId)) {
                    $leadModel->setCustomerGroupId($groupId);
                }

                if (empty($currentPrefix) && !empty($prefix)) {
                    $leadModel->setPrefix($prefix);
                }

                if (empty($currentCity) && !empty($city)) {
                    $leadModel->setLeadCity($city);
                }

                if (empty($currentCountry) && !empty($countryCode)) {
                    $leadModel->setLeadCountryCode($countryCode);
                }

                if (empty($currentState) && !empty($stateCode)) {
                    $leadModel->setLeadStateCode($stateCode);
                }

                if (empty($currentZip)) {
                    $leadModel->setLeadZip($zip);
                }

                if (empty($currentAddress)) {
                    $leadModel->setLeadAddress($address);
                }

                if (empty($currentAddress1)) {
                    $leadModel->setLeadAddress1($address1);
                }

                if (empty($currentPosition)) {
                    $leadModel->setLeadPosition($position);
                }

                $assignPrimaryContact = false;
                if (!empty($organization) && empty($currentOrganizationId)) {
                    if (!empty($organization)) {
                        $assignPrimaryContact = Tools_LeadTools::getOrganizationPrimaryContactStatus($organization);
                    }

                    $organizationModel = Tools_LeadTools::processOrganization($organization);
                    if ($organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                        $leadModel->setOrganizationId($organizationModel->getId());
                    }
                }

                if (!empty($organizationId) && empty($currentOrganizationId)) {
                    $leadOrganizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
                    $organizationModel = $leadOrganizationMapper->find($organizationId);
                    if ($organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                        $organization = $organizationModel->getOrganizationName();
                        $leadModel->setOrganizationId($organizationModel->getId());
                        $assignPrimaryContact = Tools_LeadTools::getOrganizationPrimaryContactStatus($organization);
                    }
                }

                $leadCustomParams = Tools_LeadFormTools::processFormCustomParams($userCustomAttributes, $leadId);
                $userCustomAttributes = Tools_LeadTools::removeFields($userCustomAttributes, $leadCustomParams);

                $organizationId = $leadModel->getOrganizationId();
                if (!empty($organizationId)) {
                    $organizationCustomParams = Tools_LeadFormTools::processFormCustomParams($userCustomAttributes,
                        $organizationId, Tools_LeadFormTools::ORGANIZATION_TYPE_CUSTOM_FIELD);
                    $userCustomAttributes = Tools_LeadTools::removeFields($userCustomAttributes, $organizationCustomParams);
                }

                if (!empty($organization) && $assignPrimaryContact === true && $organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                    $leadOrganizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
                    $organizationModel->setPrimaryContactId($leadId);
                    $currentOrganizationWebsite = $organizationModel->getOrganizationWebsite();
                    $website = Tools_LeadTools::prepareWebsiteUrl($website);
                    if (empty($currentOrganizationWebsite) && !empty($website)) {
                        $domain = Tools_LeadTools::getDomainFromUrl($website);
                        if (!empty($domain)) {
                            $organizationDomain = $domain;
                        } else {
                            $organizationDomain = '';
                        }
                        if (!empty($domain)) {
                            $organizationModel->setOrganizationDomain($organizationDomain);
                        }
                        $organizationModel->setOrganizationWebsite($website);
                    }
                    $firstContactOrganizationDate = $organizationModel->getFirstContactCreationDate();
                    if (empty($firstContactOrganizationDate)) {
                        $organizationModel->setFirstContactCreationDate($leadModel->getCreatedAt());
                    }
                    if (!empty($employees)) {
                        $organizationModel->setOrganizationEmployees($employees);
                    }

                    $leadOrganizationMapper->save($organizationModel);
                }

                try {
                    $leadModel = $leadsMapper->save($leadModel);
                } catch (Exception $e) {
                    return false;
                }

            }
        }

        $leadId = $leadModel->getId();
        $leadsActionLogModel = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_REGISTRATION,
            $leadId,
            Leads_Model_LeadsActionLogModel::ACTION_TYPE_INBOUND, $createdAt, $leadType);

        $leadsIpsModel = Tools_LeadTools::addIpAddressToLead($leadId, $ip);
        if ($leadsIpsModel instanceof Leads_Model_LeadsIpsModel) {
            Tools_LeadTools::addIpToMojo($leadsIpsModel);
        }

        $pluginLeadsRegistrationLogModel = new Leads_Model_LeadsRegistrationLogModel();
        $pluginLeadsRegistrationLogModel->setActionLogId($leadsActionLogModel->getId());
        $pluginLeadsRegistrationLogModel->setUserId($registrationModel->getId());
        $pluginLeadsRegistrationLogModel->setEmail($leadEmail);
        $pluginLeadsRegistrationLogModel->setFirstName($firstName);
        $pluginLeadsRegistrationLogModel->setLastName($lastName);
        $pluginLeadsRegistrationLogModel->setMobile($mobile);
        $pluginLeadsRegistrationLogModel->setLeadId($leadId);
        $pluginLeadsRegistrationLogModel->setUserRole($userRole);
        $pluginLeadsRegistrationLogModel->setRegistrationPageUrl($registrationPageUrl);
        $pluginLeadsRegistrationLogModel->setRegistrationPageId($registrationPageId);
        $pluginLeadsRegistrationLogMapper->save($pluginLeadsRegistrationLogModel);

    }

}

