<?php

/**
 * Class Tools_LeadScoreTools
 */
class Tools_LeadScoreTools
{

    const PERIOD_LAST_DAY = 'last_day';
    const PERIOD_LAST_WEEK = 'last_week';
    const PERIOD_LAST_MONTH = 'last_month';
    const PERIOD_LAST_YEAR = 'last_year';
    const PERIOD_LAST_THREE_MONTH = 'last_three_month';
    const PERIOD_LAST_SIX_MONTH = 'last_six_month';
    const PERIOD_LAST_TWELVE_MONTH = 'last_twelve_month';

    const PERIOD_NEXT_DAY = 'next_day';
    const PERIOD_NEXT_WEEK = 'next_week';
    const PERIOD_NEXT_MONTH = 'next_month';

    const PERIOD_THIS_DAY = 'this_day';
    const PERIOD_THIS_WEEK = 'this_week';
    const PERIOD_THIS_MONTH = 'this_month';
    const PERIOD_THIS_YEAR = 'this_year';

    const LEAD_SCORE_PROCESS_PATH = 'plugins/leads/system/cronscript/processLeadScore.php';

    public static $_specialLeadAttributes = array(
        'last_activity_inbound',
        'last_activity_outbound',
        'lead_tags',
        'nps_review_score',
        'lead_phone'
    );

    /**
     * get score property/actions types with labels
     *
     * @return array
     * @throws Zend_Exception
     */
    public static function getScorePropertyActions()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        $scorePropertyActions = array(
            Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_LEAD_ATTRIBUTE =>
                $translator->translate(Leads::TRANSLATION_PREFIX.'Lead Properties'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_LEAD_CUSTOM_ATTRIBUTES =>
                $translator->translate(Leads::TRANSLATION_PREFIX.'Lead Custom attributes'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_ACTION_TYPE =>
                $translator->translate(Leads::TRANSLATION_PREFIX.'Lead Actions'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_LEAD_OPPORTUNITIES =>
                $translator->translate(Leads::TRANSLATION_PREFIX.'Lead Opportunities'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_ORGANIZATION_ATTRIBUTE =>
                $translator->translate(Leads::TRANSLATION_PREFIX.'Organization Properties'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_ORGANIZATION_CUSTOM_ATTRIBUTES =>
                $translator->translate(Leads::TRANSLATION_PREFIX.'Organization Custom attributes'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_FORM_NAME =>
                $translator->translate(Leads::TRANSLATION_PREFIX.'Forms'),
        );

        return $scorePropertyActions;
    }

    /**
     * Get score actions with labels
     *
     * @return array
     * @throws Zend_Exception
     */
    public static function getScoreActions()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        $scoreActions = array(
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_CALL_RECEIVED => $translator->translate(Leads::TRANSLATION_PREFIX.'Call received'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_CALL_PLACED => $translator->translate(Leads::TRANSLATION_PREFIX.'Call placed'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_SMS_SENT => $translator->translate(Leads::TRANSLATION_PREFIX.'Sms sent'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_SMS_RECEIVED => $translator->translate(Leads::TRANSLATION_PREFIX.'Sms received'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_REGISTRATION => $translator->translate(Leads::TRANSLATION_PREFIX.'Registration'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_QUOTE_VIEW => $translator->translate(Leads::TRANSLATION_PREFIX.'Quote view'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_QUOTE_REQUESTED => $translator->translate(Leads::TRANSLATION_PREFIX.'Quote requested'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_QUOTE_SENT => $translator->translate(Leads::TRANSLATION_PREFIX.'Quote sent'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_QUOTE_EXPIRATION_DATE => $translator->translate(Leads::TRANSLATION_PREFIX.'Quote expiration date'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_QUOTE_AMOUNT => $translator->translate(Leads::TRANSLATION_PREFIX.'Quote amount'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_DOCUMENT_DOWNLOADED => $translator->translate(Leads::TRANSLATION_PREFIX.'Document downloaded'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_SENT => $translator->translate(Leads::TRANSLATION_PREFIX.'Email sent'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_RECEIVED => $translator->translate(Leads::TRANSLATION_PREFIX.'Email received'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_WEBSITE_VISIT => $translator->translate(Leads::TRANSLATION_PREFIX.'Website visit'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_PURCHASE => $translator->translate(Leads::TRANSLATION_PREFIX.'Purchase')
        );

        return $scoreActions;
    }

    /**
     * Get lead score attributes with labels
     *
     * @return array
     * @throws Zend_Exception
     */
    public static function getLeadScoreLeadAttributes()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        $scoreActions = array(
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_OWNER => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead owner'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_COUNTRY_CODE => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead country'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_STATE_CODE => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead state'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_CUSTOMER_GROUP => $translator->translate(Leads::TRANSLATION_PREFIX.'Customer group'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_LEAD_TAGS => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead tag'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_POSITION => $translator->translate(Leads::TRANSLATION_PREFIX.'Position'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_LIFECYCLE => $translator->translate(Leads::TRANSLATION_PREFIX.'Lifecycle'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_NPS_REVIEW_SCORE => $translator->translate(Leads::TRANSLATION_PREFIX.'Nps review score'),
            //Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_CREATED_AT => $translator->translate('Creation date'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_LAST_ACTIVITY_INBOUND => $translator->translate(Leads::TRANSLATION_PREFIX.'Last activity inbound'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_LAST_ACTIVITY_OUTBOUND => $translator->translate(Leads::TRANSLATION_PREFIX.'Last activity outbound'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_LEAD_PHONE => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead phone'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_SOURCE => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead source')
        );

        return $scoreActions;
    }

    /**
     * Get lead score attributes with labels
     *
     * @return array
     * @throws Zend_Exception
     */
    public static function getFormsLabels()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        $forms = array(
            Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_FORM_NAME => $translator->translate(Leads::TRANSLATION_PREFIX.'Forms')
        );

        return $forms;
    }

    /**
     * Get score organization attributes with labels
     *
     * @return array
     * @throws Zend_Exception
     */
    public static function getLeadScoreOrganizationAttributes()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        $scoreActions = array(
            Leads_Model_LeadsScoreRulesConfigModel::ORGANIZATION_ATTRIBUTE_INDUSTRY => $translator->translate(Leads::TRANSLATION_PREFIX.'Industry'),
            Leads_Model_LeadsScoreRulesConfigModel::ORGANIZATION_ATTRIBUTE_GROUP => $translator->translate(Leads::TRANSLATION_PREFIX.'Group'),
            Leads_Model_LeadsScoreRulesConfigModel::ORGANIZATION_ATTRIBUTE_EMPLOYEES => $translator->translate(Leads::TRANSLATION_PREFIX.'Employees'),
            Leads_Model_LeadsScoreRulesConfigModel::ORGANIZATION_ATTRIBUTE_COUNTRY_CODE => $translator->translate(Leads::TRANSLATION_PREFIX.'Organization country'),
            Leads_Model_LeadsScoreRulesConfigModel::ORGANIZATION_ATTRIBUTE_STATE => $translator->translate(Leads::TRANSLATION_PREFIX.'Organization state')
        );

        return $scoreActions;
    }

    /**
     * Get lead score opportunity attributes with labels
     *
     * @return array
     * @throws Zend_Exception
     */
    public static function getLeadScoreOpportunitiesAttributes()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        $scoreActions = array(
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_OPPORTUNITY_ATTRIBUTE_OPPORTUNITY_TYPE => $translator->translate(Leads::TRANSLATION_PREFIX.'Opportunity type'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_OPPORTUNITY_ATTRIBUTE_OPPORTUNITY_STAGE => $translator->translate(Leads::TRANSLATION_PREFIX.'Opportunity stage'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_OPPORTUNITY_ATTRIBUTE_OPPORTUNITY_CLOSE_DATE => $translator->translate(Leads::TRANSLATION_PREFIX.'Opportunity close date'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_OPPORTUNITY_ATTRIBUTE_OPPORTUNITY_AMOUNT_SINGLE => $translator->translate(Leads::TRANSLATION_PREFIX.'Opportunity amount (single)'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_OPPORTUNITY_ATTRIBUTE_OPPORTUNITY_AMOUNT_CUMULATIVE => $translator->translate(Leads::TRANSLATION_PREFIX.'Opportunity amount (cumulated)')
        );

        return $scoreActions;
    }

    /**
     * Get lead custom attribute
     *
     * @param string $customAttributeType custom attribute type
     * @return array
     * @throws Zend_Exception
     */
    public static function getLeadScoreCustomAttributes($customAttributeType)
    {
        $scoreCustomAttributes = array();
        if ($customAttributeType === Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_LEAD_CUSTOM_ATTRIBUTES) {
            $leadsCustomParamsMapper = Leads_Mapper_LeadsCustomParamsConfigMapper::getInstance();
            $scoreCustomAttributes = $leadsCustomParamsMapper->getCustomParamsPairs();
        }

        if ($customAttributeType === Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_ORGANIZATION_CUSTOM_ATTRIBUTES) {
            $leadsOrganizationCustomParamsMapper = Leads_Mapper_LeadsOrganizationCustomParamsConfigMapper::getInstance();
            $scoreCustomAttributes = $leadsOrganizationCustomParamsMapper->getCustomParamsPairs();
        }

        return $scoreCustomAttributes;
    }

    public static function getForms($namesOnly = false)
    {
        $forms = Leads_Mapper_LeadsFormRulesGeneralConfigMapper::getInstance()->getAllForms();
        if (empty($forms)) {
            return array();
        }

        if ($namesOnly === true) {
            $formResult = array();
            foreach ($forms as $key => $formData) {
                $formResult[$key] = $formData['name'].' ('.$formData['formType'].')';
            }

            return $formResult;
        }

        return $forms;
    }


    /**
     * @return array
     * @throws Zend_Exception
     */
    public static function getAllActionAttributes()
    {
        $translator = Zend_Registry::get('Zend_Translate');

        $scoreActionsAttributes = array(
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_DURATION_RECEIVED => $translator->translate(Leads::TRANSLATION_PREFIX.'Duration (Call received)'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_DURATION_PLACED => $translator->translate(Leads::TRANSLATION_PREFIX.'Duration (Call placed)'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_PURCHASE_AMOUNT_SINGLE => $translator->translate(Leads::TRANSLATION_PREFIX.'Amount single'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_PURCHASE_AMOUNT_CUMULATIVE => $translator->translate(Leads::TRANSLATION_PREFIX.'Amount cumulated'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_PURCHASE_PRODUCTS_SKU => $translator->translate(Leads::TRANSLATION_PREFIX.'Products sku'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_QUOTE_VIEW => $translator->translate(Leads::TRANSLATION_PREFIX.'Quote view'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_QUOTE_SENT => $translator->translate(Leads::TRANSLATION_PREFIX.'Quote sent'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_SENT => $translator->translate(Leads::TRANSLATION_PREFIX.'Email sent'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_OPENED => $translator->translate(Leads::TRANSLATION_PREFIX.'Email opened'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_RECEIVED => $translator->translate(Leads::TRANSLATION_PREFIX.'Email received'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_RECEIVED_IMAP => $translator->translate(Leads::TRANSLATION_PREFIX.'Email via IMAP'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_RECEIVED_IMAP_FROM => $translator->translate(Leads::TRANSLATION_PREFIX.'Email from (IMAP)'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_SMS_KEYWORD_RECEIVED => $translator->translate(Leads::TRANSLATION_PREFIX.'Inbound message'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_SMS_KEYWORD_SENT => $translator->translate(Leads::TRANSLATION_PREFIX.'Outbound message'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_QUOTE_EXPIRATION_DATE => $translator->translate(Leads::TRANSLATION_PREFIX.'Quote expiration date'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_QUOTE_REQUESTED => $translator->translate(Leads::TRANSLATION_PREFIX.'Quote requested'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_QUOTE_AMOUNT_SINGLE => $translator->translate(Leads::TRANSLATION_PREFIX.'Quote amount single'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_QUOTE_AMOUNT_CUMULATIVE => $translator->translate(Leads::TRANSLATION_PREFIX.'Quote amount cumulated'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_REGISTERED => $translator->translate(Leads::TRANSLATION_PREFIX.'Registered'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_DOWNLOADED_DOCUMENT_FILE_NAME => $translator->translate(Leads::TRANSLATION_PREFIX.'Document name'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_VISIT_DURATION => $translator->translate(Leads::TRANSLATION_PREFIX.'Visit duration (cumulated sessions time)'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_VISIT_DURATION_LATEST => $translator->translate(Leads::TRANSLATION_PREFIX.'Visit duration (latest session only)'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_TIME_ACTION_NUMBER => $translator->translate(Leads::TRANSLATION_PREFIX.'Pages visited count (cumulated)'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_TIME_ACTION_NUMBER_LATEST => $translator->translate(Leads::TRANSLATION_PREFIX.'Pages visited (latest session only)'),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_WEBSITE_LAST_ACTION_DATE_TIME => $translator->translate(Leads::TRANSLATION_PREFIX.'Last visit date'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_TRACKING_NUMBER => $translator->translate(Leads::TRANSLATION_PREFIX.'Tracking number')
        );

        return $scoreActionsAttributes;
    }

    /**
     *  Actions attributes labels
     *
     * @param string $actionName action name
     * @return array
     * @throws Zend_Exception
     */
    public static function getActionAttributes($actionName)
    {
        $translator = Zend_Registry::get('Zend_Translate');
        $scoreActionsAttributes = array();
        if ($actionName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_CALL_RECEIVED) {
            $scoreActionsAttributes = array(
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_DURATION_RECEIVED => $translator->translate(Leads::TRANSLATION_PREFIX.'Duration (Call received)'),
                Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_TRACKING_NUMBER => $translator->translate(Leads::TRANSLATION_PREFIX.'Tracking number')
            );
        }
        if ($actionName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_CALL_PLACED) {
            $scoreActionsAttributes = array(
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_DURATION_PLACED => $translator->translate(Leads::TRANSLATION_PREFIX.'Duration (Call placed)')
            );
        }
        if ($actionName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_PURCHASE) {
            $scoreActionsAttributes = array(
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_PURCHASE_AMOUNT_SINGLE => $translator->translate(Leads::TRANSLATION_PREFIX.'Amount single'),
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_PURCHASE_AMOUNT_CUMULATIVE => $translator->translate(Leads::TRANSLATION_PREFIX.'Amount cumulated'),
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_PURCHASE_PRODUCTS_SKU => $translator->translate(Leads::TRANSLATION_PREFIX.'Products sku')
            );
        }

        if ($actionName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_QUOTE_VIEW) {
            $scoreActionsAttributes = array(
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_QUOTE_VIEW => $translator->translate(Leads::TRANSLATION_PREFIX.'Quote view')
            );
        }

        if ($actionName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_QUOTE_REQUESTED) {
            $scoreActionsAttributes = array(
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_QUOTE_REQUESTED => $translator->translate(Leads::TRANSLATION_PREFIX.'Quote requested')
            );
        }

        if ($actionName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_QUOTE_SENT) {
            $scoreActionsAttributes = array(
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_QUOTE_SENT => $translator->translate(Leads::TRANSLATION_PREFIX.'Quote sent')
            );
        }

        if ($actionName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_QUOTE_EXPIRATION_DATE) {
            $scoreActionsAttributes = array(
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_QUOTE_EXPIRATION_DATE => $translator->translate(Leads::TRANSLATION_PREFIX.'Quote expiration date')
            );
        }


        if ($actionName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_QUOTE_AMOUNT) {
            $scoreActionsAttributes = array(
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_QUOTE_AMOUNT_SINGLE => $translator->translate(Leads::TRANSLATION_PREFIX.'Quote amount single'),
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_QUOTE_AMOUNT_CUMULATIVE => $translator->translate(Leads::TRANSLATION_PREFIX.'Quote amount cumulated')
            );
        }

        if ($actionName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_REGISTRATION) {
            $scoreActionsAttributes = array(
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_REGISTERED => $translator->translate(Leads::TRANSLATION_PREFIX.'Registered')
            );
        }

        if ($actionName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_DOCUMENT_DOWNLOADED) {
            $scoreActionsAttributes = array(
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_DOWNLOADED_DOCUMENT_FILE_NAME => $translator->translate(Leads::TRANSLATION_PREFIX.'Document name')
            );
        }

        if ($actionName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_SENT) {
            $scoreActionsAttributes = array(
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_SENT => $translator->translate(Leads::TRANSLATION_PREFIX.'Email sent'),
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_OPENED => $translator->translate(Leads::TRANSLATION_PREFIX.'Email opened')
            );
        }

        if ($actionName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_RECEIVED) {
            $scoreActionsAttributes = array(
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_RECEIVED => $translator->translate(Leads::TRANSLATION_PREFIX.'Email received'),
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_RECEIVED_IMAP => $translator->translate(Leads::TRANSLATION_PREFIX.'Email via IMAP'),
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_RECEIVED_IMAP_FROM => $translator->translate(Leads::TRANSLATION_PREFIX.'Email from (IMAP)')
            );
        }

        if ($actionName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_WEBSITE_VISIT) {
            $scoreActionsAttributes = array(
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_VISIT_DURATION => $translator->translate(Leads::TRANSLATION_PREFIX.'Visit duration (cumulated sessions time)'),
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_VISIT_DURATION_LATEST => $translator->translate(Leads::TRANSLATION_PREFIX.'Visit duration (latest session only)'),
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_TIME_ACTION_NUMBER => $translator->translate(Leads::TRANSLATION_PREFIX.'Pages visited count (cumulated)'),
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_TIME_ACTION_NUMBER_LATEST => $translator->translate(Leads::TRANSLATION_PREFIX.'Pages visited (latest session only)'),
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_WEBSITE_LAST_ACTION_DATE_TIME => $translator->translate(Leads::TRANSLATION_PREFIX.'Last visit date')
            );
        }

        if ($actionName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_SMS_SENT) {
            $scoreActionsAttributes = array(
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_SMS_KEYWORD_SENT => $translator->translate(Leads::TRANSLATION_PREFIX.'Outbound message')
            );
        }

        if ($actionName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_SMS_RECEIVED) {
            $scoreActionsAttributes = array(
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_SMS_KEYWORD_RECEIVED => $translator->translate(Leads::TRANSLATION_PREFIX.'Inbound message')
            );

        }


        return $scoreActionsAttributes;
    }

    /**
     * Get lead predefined attributes
     *
     * @return array
     * @throws Zend_Exception
     */
    public static function getLeadAttributesPredefinedValuesList()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        $predefinedValuesList = array(
            'yes_no_block' => array('yes' => $translator->translate(Leads::TRANSLATION_PREFIX.'Yes'), 'no' => $translator->translate(Leads::TRANSLATION_PREFIX.'No')),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_LAST_ACTIVITY_INBOUND => array(
                self::PERIOD_THIS_DAY => $translator->translate(Leads::TRANSLATION_PREFIX.'Today'),
                self::PERIOD_THIS_WEEK => $translator->translate(Leads::TRANSLATION_PREFIX.'This week'),
                self::PERIOD_THIS_MONTH => $translator->translate(Leads::TRANSLATION_PREFIX.'This month'),
                self::PERIOD_THIS_YEAR => $translator->translate(Leads::TRANSLATION_PREFIX.'This year'),
                self::PERIOD_LAST_DAY => $translator->translate(Leads::TRANSLATION_PREFIX.'Yesterday'),
                self::PERIOD_LAST_WEEK => $translator->translate(Leads::TRANSLATION_PREFIX.'Last week'),
                self::PERIOD_LAST_MONTH => $translator->translate(Leads::TRANSLATION_PREFIX.'Last month'),
                self::PERIOD_LAST_YEAR => $translator->translate(Leads::TRANSLATION_PREFIX.'Last year')
            ),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_LAST_ACTIVITY_OUTBOUND => array(
                self::PERIOD_THIS_DAY => $translator->translate(Leads::TRANSLATION_PREFIX.'Today'),
                self::PERIOD_THIS_WEEK => $translator->translate(Leads::TRANSLATION_PREFIX.'This week'),
                self::PERIOD_THIS_MONTH => $translator->translate(Leads::TRANSLATION_PREFIX.'This month'),
                self::PERIOD_THIS_YEAR => $translator->translate(Leads::TRANSLATION_PREFIX.'This year'),
                self::PERIOD_LAST_DAY => $translator->translate(Leads::TRANSLATION_PREFIX.'Yesterday'),
                self::PERIOD_LAST_WEEK => $translator->translate(Leads::TRANSLATION_PREFIX.'Last week'),
                self::PERIOD_LAST_MONTH => $translator->translate(Leads::TRANSLATION_PREFIX.'Last month'),
                self::PERIOD_LAST_YEAR => $translator->translate(Leads::TRANSLATION_PREFIX.'Last year')
            ),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_QUOTE_EXPIRATION_DATE => array(
                self::PERIOD_NEXT_DAY => $translator->translate(Leads::TRANSLATION_PREFIX.'Tomorrow'),
                self::PERIOD_NEXT_WEEK => $translator->translate(Leads::TRANSLATION_PREFIX.'Next week'),
                self::PERIOD_NEXT_MONTH => $translator->translate(Leads::TRANSLATION_PREFIX.'Next month')
            ),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_OPPORTUNITY_ATTRIBUTE_OPPORTUNITY_CLOSE_DATE => array(
                self::PERIOD_NEXT_DAY => $translator->translate(Leads::TRANSLATION_PREFIX.'Tomorrow'),
                self::PERIOD_NEXT_WEEK => $translator->translate(Leads::TRANSLATION_PREFIX.'Next week'),
                self::PERIOD_NEXT_MONTH => $translator->translate(Leads::TRANSLATION_PREFIX.'Next month')
            ),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_WEBSITE_LAST_ACTION_DATE_TIME => array(
                self::PERIOD_THIS_DAY => $translator->translate(Leads::TRANSLATION_PREFIX.'Today'),
                self::PERIOD_LAST_DAY => $translator->translate(Leads::TRANSLATION_PREFIX.'Yesterday'),
                self::PERIOD_LAST_WEEK => $translator->translate(Leads::TRANSLATION_PREFIX.'Last week'),
                self::PERIOD_LAST_MONTH => $translator->translate(Leads::TRANSLATION_PREFIX.'Last month'),
                self::PERIOD_LAST_THREE_MONTH => $translator->translate(Leads::TRANSLATION_PREFIX.'Last 3 month'),
                self::PERIOD_LAST_SIX_MONTH => $translator->translate(Leads::TRANSLATION_PREFIX.'Last 6 month'),
                self::PERIOD_LAST_TWELVE_MONTH => $translator->translate(Leads::TRANSLATION_PREFIX.'Last 12 month')
            )
        );

        return $predefinedValuesList;
    }

    /**
     * Get lead attribute legends
     *
     * @return array
     * @throws Zend_Exception
     */
    public static function getLeadAttributeLegends()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        $leadAttributeLegends = array(
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_CREATED_AT => $translator->translate(''),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_CUSTOMER_GROUP => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead customer group'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_LEAD_TAGS => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead tags'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_LAST_ACTIVITY_INBOUND => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead last inbound activity'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_LAST_ACTIVITY_OUTBOUND => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead last outbound activity'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_COUNTRY_CODE => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead country'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_OWNER => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead owner'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_STATE_CODE => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead state'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_LIFECYCLE => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead lifecycle'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_POSITION => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead position'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_EXTERNAL_EVENTS => $translator->translate(Leads::TRANSLATION_PREFIX.'External events'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_ORGANIZATION_CUSTOM_TABS => $translator->translate(Leads::TRANSLATION_PREFIX.'Leads/Organization custom tabs'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_INBOUND_EMAILS_CONFIG => $translator->translate(Leads::TRANSLATION_PREFIX.'Inbound emails config'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_INBOUND_EMAILS_CONFIG => $translator->translate(Leads::TRANSLATION_PREFIX.'Configuration email IMAP'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_IMAP_RULES_TAB => $translator->translate(Leads::TRANSLATION_PREFIX.'Imap rules'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_CRM_API_ACCESS => $translator->translate(Leads::TRANSLATION_PREFIX.'Crm Api access')
        );

        return $leadAttributeLegends;
    }

    /**
     * Get organization attribute legends
     *
     * @return array
     * @throws Zend_Exception
     */
    public static function getOrganizationAttributeLegends()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        $organizationAttributeLegends = array(
            Leads_Model_LeadsScoreRulesConfigModel::ORGANIZATION_ATTRIBUTE_EMPLOYEES => $translator->translate(Leads::TRANSLATION_PREFIX.'Organization employees'),
            Leads_Model_LeadsScoreRulesConfigModel::ORGANIZATION_ATTRIBUTE_GROUP => $translator->translate(Leads::TRANSLATION_PREFIX.'Organization customer group'),
            Leads_Model_LeadsScoreRulesConfigModel::ORGANIZATION_ATTRIBUTE_INDUSTRY => $translator->translate(Leads::TRANSLATION_PREFIX.'Organization industry'),
            Leads_Model_LeadsScoreRulesConfigModel::ORGANIZATION_ATTRIBUTE_COUNTRY_CODE => $translator->translate(Leads::TRANSLATION_PREFIX.'Organization country'),
            Leads_Model_LeadsScoreRulesConfigModel::ORGANIZATION_ATTRIBUTE_STATE => $translator->translate(Leads::TRANSLATION_PREFIX.'Organization state')
        );

        return $organizationAttributeLegends;
    }

    /**
     * Get organization attribute legends
     *
     * @return array
     * @throws Zend_Exception
     */
    public static function getLeadOpportunitiesLegends()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        $leadOpportunitiesLegends = array(
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_OPPORTUNITY_ATTRIBUTE_OPPORTUNITY_TYPE => $translator->translate(Leads::TRANSLATION_PREFIX.'Opportunity type'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_OPPORTUNITY_ATTRIBUTE_OPPORTUNITY_STAGE => $translator->translate(Leads::TRANSLATION_PREFIX.'Opportunity stage'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_OPPORTUNITY_ATTRIBUTE_OPPORTUNITY_CLOSE_DATE => $translator->translate(Leads::TRANSLATION_PREFIX.'Opportunity close date'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_OPPORTUNITY_ATTRIBUTE_OPPORTUNITY_AMOUNT_SINGLE => $translator->translate(Leads::TRANSLATION_PREFIX.'Opportunity amount (single)'),
            Leads_Model_LeadsScoreRulesConfigModel::LEAD_OPPORTUNITY_ATTRIBUTE_OPPORTUNITY_AMOUNT_CUMULATIVE => $translator->translate(Leads::TRANSLATION_PREFIX.'Opportunity amount (cumulated)')
        );

        return $leadOpportunitiesLegends;
    }

    /**
     * Get comparison operators legends
     *
     * @return array
     * @throws Zend_Exception
     */
    public static function getComparisonOperatorLegend()
    {
        $translator = Zend_Registry::get('Zend_Translate');

        $comparisonOperatorLegend = array(
            'equal' => $translator->translate(Leads::TRANSLATION_PREFIX.'is'),
            'notequal' => $translator->translate(Leads::TRANSLATION_PREFIX.'other than'),
            'like' => $translator->translate(Leads::TRANSLATION_PREFIX.'includes'),
            'in' => $translator->translate(Leads::TRANSLATION_PREFIX.'one of'),
            'notin' => $translator->translate(Leads::TRANSLATION_PREFIX.'nothing of'),
            'greaterthan' => $translator->translate(Leads::TRANSLATION_PREFIX.'greater than'),
            'lessthan' => $translator->translate(Leads::TRANSLATION_PREFIX.'less than')
        );

        return $comparisonOperatorLegend;
    }

    /**
     * get action types attributes legends
     *
     * @return array
     * @throws Zend_Exception
     */
    public static function getActionTypesAttributeLegends()
    {
        $translator = Zend_Registry::get('Zend_Translate');

        $actionTypesAttributeLegends = array(
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_CALL_PLACED =>
                array(
                    Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_DURATION_PLACED
                    => $translator->translate(Leads::TRANSLATION_PREFIX.'Call placed')
                ),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_CALL_RECEIVED =>
                array(
                    Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_DURATION_RECEIVED
                    => $translator->translate(Leads::TRANSLATION_PREFIX.'Call received')
                ),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_SMS_SENT =>
                array(
                    Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_SMS_KEYWORD_SENT
                    => $translator->translate(Leads::TRANSLATION_PREFIX.'Sms text sent')
                ),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_SMS_RECEIVED =>
                array(
                    Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_SMS_KEYWORD_RECEIVED
                    => $translator->translate(Leads::TRANSLATION_PREFIX.'Sms text received')
                ),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_DOCUMENT_DOWNLOADED =>
                array(
                    Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_DOWNLOADED_DOCUMENT_FILE_NAME
                    => $translator->translate(Leads::TRANSLATION_PREFIX.'Document name')
                ),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_SENT =>
                array(
                    Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_SENT
                    => $translator->translate(Leads::TRANSLATION_PREFIX.'Email sent')
                ),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_RECEIVED =>
                array(
                    Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_RECEIVED
                    => $translator->translate(Leads::TRANSLATION_PREFIX.'Email received'),
                    Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_RECEIVED_IMAP
                    => $translator->translate(Leads::TRANSLATION_PREFIX.'Email via IMAP'),
                    Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_RECEIVED_IMAP_FROM
                    => $translator->translate(Leads::TRANSLATION_PREFIX.'Email from (IMAP)')
                ),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_PURCHASE => array(
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_PURCHASE_AMOUNT_SINGLE
                => $translator->translate(Leads::TRANSLATION_PREFIX.'Purchase amount single'),
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_PURCHASE_AMOUNT_CUMULATIVE
                => $translator->translate(Leads::TRANSLATION_PREFIX.'Purchase amount cumulated')
            ),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_QUOTE_REQUESTED =>
                array(
                    Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_QUOTE_REQUESTED
                    => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead requested a quote')
                ),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_QUOTE_VIEW =>
                array(
                    Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_QUOTE_VIEW =>
                        $translator->translate(Leads::TRANSLATION_PREFIX.'Lead viewed a quote')
                ),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_QUOTE_SENT =>
                array(
                    Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_QUOTE_SENT =>
                        $translator->translate(Leads::TRANSLATION_PREFIX.'Lead quote sent')
                ),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_QUOTE_EXPIRATION_DATE =>
                array(
                    Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_QUOTE_EXPIRATION_DATE =>
                        $translator->translate(Leads::TRANSLATION_PREFIX.'Quote expiration date')
                ),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_QUOTE_AMOUNT => array(
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_QUOTE_AMOUNT_SINGLE =>
                    $translator->translate(Leads::TRANSLATION_PREFIX.'Quote amount single'),
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_QUOTE_AMOUNT_CUMULATIVE =>
                    $translator->translate(Leads::TRANSLATION_PREFIX.'Quote amount cumulated')
            ),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_REGISTRATION =>
                array(
                    Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_REGISTERED =>
                        $translator->translate(Leads::TRANSLATION_PREFIX.'Lead registered')
                ),
            Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_WEBSITE_VISIT => array(
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_WEBSITE_LAST_ACTION_DATE_TIME =>
                    $translator->translate(Leads::TRANSLATION_PREFIX.'Last visit date'),
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_VISIT_DURATION =>
                    $translator->translate(Leads::TRANSLATION_PREFIX.'Visit duration (cumulated sessions time)'),
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_VISIT_DURATION_LATEST =>
                    $translator->translate(Leads::TRANSLATION_PREFIX.'Visit duration (latest session only)'),
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_TIME_ACTION_NUMBER =>
                    $translator->translate(Leads::TRANSLATION_PREFIX.'Pages visited count (cumulated)'),
                Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_TIME_ACTION_NUMBER_LATEST =>
                    $translator->translate(Leads::TRANSLATION_PREFIX.'Pages visited (latest session only)')
            )
        );

        return $actionTypesAttributeLegends;
    }

    /**
     * Prepare score rules
     *
     * @param array $scoreRules score rules
     * @return array
     */
    public static function prepareScoreRules($scoreRules)
    {
        $resultScoreRules = array();
        foreach ($scoreRules as $scoreRule) {
            $resultScoreRules[$scoreRule['id']][] = $scoreRule;
        }

        return $resultScoreRules;
    }

    /**
     * Process lead score for leads
     *
     * @param array $leadsData leads data
     * @param array $scoreRules score rules
     * @return bool
     */
    public static function processLeadScore($leadsData, $scoreRules)
    {
        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
        $leadsScoreRulesLogMapper = Leads_Mapper_LeadsScoreRulesLogMapper::getInstance();
        foreach ($leadsData as $leadData) {
            try {
                $score = 0;
                $ruleIds = array();
                foreach ($scoreRules as $scoreRule) {
                    $scoreResult = self::applyLeadScore($leadData, $scoreRule);
                    $score += $scoreResult['score'];
                    if (empty($scoreResult['error'])) {
                        $ruleIds[$scoreResult['ruleId']] = $scoreResult['ruleId'];
                    }
                }

                if ($score < 0) {
                    $score = 0;
                }

                $leadId = $leadData['id'];
                $leadsScoreRulesLogMapper->deleteByLeadId($leadId);
                foreach ($ruleIds as $ruleId) {
                    $leadsScoreRulesLogModel = new Leads_Model_LeadsScoreRulesLogModel();
                    $leadsScoreRulesLogModel->setLeadId($leadId);
                    $leadsScoreRulesLogModel->setRuleId($ruleId);
                    $leadsScoreRulesLogMapper->save($leadsScoreRulesLogModel);
                }

                $leadsMapper->updateScore($leadData['id'], $score);


            } catch (Exception $exception) {
                return false;
            }
        }

        return true;
    }

    /**
     * apply score rules
     *
     * @param array $leadData leadData
     * @param array $scoreRules score rules
     * @return array
     */
    public static function applyLeadScore($leadData, $scoreRules)
    {
        $currentScore = 0;
        $addScore = false;
        $scoreValue = 0;
        foreach ($scoreRules as $scoreRule) {
            $ruleType = $scoreRule['rule_type'];
            $ruleId = $scoreRule['id'];
            if ($scoreRule['score_sign'] === Leads_Model_LeadsScoreRulesGeneralConfigModel::SCORE_SIGN_POSITIVE) {
                $scoreValue = $scoreRule['score'];
            } else {
                $scoreValue = -$scoreRule['score'];
            }

            $fieldName = $scoreRule['field_name'];
            if ($fieldName === 'lead_customer_group_id' || $fieldName === 'organization_customer_group_id') {
                $fieldName = 'customer_group_id';
            }

            if ($ruleType === Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_LEAD_ATTRIBUTE) {
                $fieldValue = $scoreRule['field_value'];
                if (in_array($fieldName, self::$_specialLeadAttributes, true)) {
                    if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_LAST_ACTIVITY_INBOUND
                        || $fieldName === Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_LAST_ACTIVITY_OUTBOUND) {
                        $lastActivityDate = '';
                        $lastActivityToDate = '';
                        if ($fieldValue === self::PERIOD_LAST_DAY) {
                            $lastActivityDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('-1 day')), '', 'UTC', 'Y-m-d 00:00:00');
                            $lastActivityToDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('-1 day')), '', 'UTC', 'Y-m-d 23:59:59');
                        }
                        if ($fieldValue === self::PERIOD_LAST_WEEK) {
                            $lastActivityDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('monday last week - 1 day')), '', 'UTC', 'Y-m-d 00:00:00');
                            $lastActivityToDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('monday this week - 1 second - 1 day')), '', 'UTC', 'Y-m-d 23:59:59');
                        }
                        if ($fieldValue === self::PERIOD_LAST_MONTH) {
                            $lastActivityDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('first day of previous month')), '', 'UTC', 'Y-m-d 00:00:00');
                            $lastActivityToDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('last day of previous month')), '', 'UTC', 'Y-m-d 23:59:59');
                        }

                        if ($fieldValue === self::PERIOD_LAST_YEAR) {
                            $lastActivityDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('first day of january previous year')), '', 'UTC', 'Y-m-d 00:00:00');
                            $lastActivityToDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('last day of december previous year')), '', 'UTC', 'Y-m-d 23:59:59');
                        }

                        if ($fieldValue === self::PERIOD_THIS_DAY) {
                            $lastActivityDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('now')), '', 'UTC', 'Y-m-d 00:00:00');
                            $lastActivityToDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('now')), '', 'UTC', 'Y-m-d 23:59:59');
                        }

                        if ($fieldValue === self::PERIOD_THIS_WEEK) {
                            $lastActivityDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('this week monday - 1 day')), '', 'UTC', 'Y-m-d 00:00:00');
                            $lastActivityToDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('now')), '', 'UTC', 'Y-m-d 23:59:59');
                        }

                        if ($fieldValue === self::PERIOD_THIS_MONTH) {
                            $lastActivityDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('first day of this month')), '', 'UTC', 'Y-m-d 00:00:00');
                            $lastActivityToDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('now')), '', 'UTC', 'Y-m-d 23:59:59');
                        }

                        if ($fieldValue === self::PERIOD_THIS_YEAR) {
                            $lastActivityDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('first day of january this year')), '', 'UTC', 'Y-m-d 00:00:00');
                            $lastActivityToDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('now')), '', 'UTC', 'Y-m-d 23:59:59');
                        }

                        if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_LAST_ACTIVITY_INBOUND) {
                            $lastActivityCurrentLead = $leadData['last_activity'];
                        } else {
                            $lastActivityCurrentLead = $leadData['last_activity_outbound'];
                        }

                        if (empty($lastActivityCurrentLead) || empty($lastActivityDate)) {
                            $addScore = false;
                            break;
                        }

                        if (strtotime($lastActivityCurrentLead) >= strtotime($lastActivityDate) && strtotime($lastActivityCurrentLead) <= strtotime($lastActivityToDate)) {
                            $addScore = true;
                        } else {
                            $addScore = false;
                            break;
                        }
                    }

                    if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_LEAD_TAGS) {
                        $currentLeadTags = Leads_Mapper_LeadsTagsMapper::getInstance()->findTagsByLeadId($leadData['id']);
                        if (empty($currentLeadTags)) {
                            $addScore = false;
                            break;
                        }

                        $addScore = self::compareValue($scoreRule['rule_comparison_operator'],
                            $currentLeadTags, $scoreRule['field_value']);

                        if ($addScore === false) {
                            break;
                        } else {
                            $addScore = true;
                        }


                    }

                    if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_LEAD_PHONE) {
                        $fullCountryPhoneCodesList = Tools_System_Tools::getFullCountryPhoneCodesList(false, array(), true);

                        $leadPhone = $leadData['lead_phone'];
                        $codeValue = '';

                        if(empty($leadPhone)) {
                            $addScore = false;
                            break;
                        }

                        if(!empty($leadData['lead_phone_country_code']) || !empty($leadData['lead_phone_country_code_value'])) {
                            if(!empty($leadData['lead_phone_country_code_value'])) {
                                $codeValue = $leadData['lead_phone_country_code_value'];
                            } elseif (!empty($leadData['lead_phone_country_code'])){
                                $codeValue = $fullCountryPhoneCodesList[$leadData['lead_phone_country_code']];
                            }
                        }

                        if($scoreRule['rule_comparison_operator'] == Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_EQUAL ||
                            $scoreRule['rule_comparison_operator'] == Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_NOTEQUAL) {

                            if(empty($codeValue)) {
                                $addScore = false;
                                break;
                            }

                            $fieldValue = json_decode($fieldValue, true);

                            if(empty($fieldValue) || !is_array($fieldValue)) {
                                $addScore = false;
                                break;
                            }

                            $fieldValue = $fullCountryPhoneCodesList[$fieldValue['country_code']] . $fieldValue['lead_phone'];
                        }

                        $leadPhoneFull = $codeValue . $leadPhone;

                        $addScore = self::compareValue($scoreRule['rule_comparison_operator'], $leadPhoneFull, $fieldValue);

                        if ($addScore === false) {
                            break;
                        } else {
                            $addScore = true;
                        }

                    }

                } else {
                    if (isset($leadData[$fieldName])) {
                        $addScore = self::compareValue($scoreRule['rule_comparison_operator'],
                            $leadData[$fieldName], $scoreRule['field_value']);
                        if ($addScore === false) {
                            break;
                        } else {
                            $addScore = true;
                        }
                    } else {
                        $addScore = false;
                        break;
                    }
                }

            }

            if ($ruleType === Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_ORGANIZATION_ATTRIBUTE) {
                if (!empty($leadData['organization_id'])) {
                    $organizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
                    $where = $organizationMapper->getDbTable()->getAdapter()->quoteInto('plo.id = ?',
                        $leadData['organization_id']);
                    $organizationData = $organizationMapper->fetchAll($where, null, null, null, true, true);
                    if (empty($organizationData)) {
                        $addScore = false;
                        break;
                    }
                    if (isset($organizationData[$fieldName])) {
                        $addScore = self::compareValue($scoreRule['rule_comparison_operator'],
                            $organizationData[$fieldName], $scoreRule['field_value']);
                        if ($addScore === false) {
                            break;
                        } else {
                            $addScore = true;
                        }
                    } else {
                        $addScore = false;
                        break;
                    }

                } else {
                    $addScore = false;
                    break;
                }

            }

            if ($ruleType === Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_LEAD_OPPORTUNITIES) {
                $addScore = self::processLeadOpportunitiesTypes($scoreRule['rule_subtype'], $leadData['id'],
                    $scoreRule['rule_comparison_operator'], $scoreRule['field_name'], $scoreRule['field_value']);
                if ($addScore === false) {
                    break;
                } else {
                    $addScore = true;
                }
            }

            if ($ruleType === Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_LEAD_CUSTOM_ATTRIBUTES ||
                $ruleType === Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_ORGANIZATION_CUSTOM_ATTRIBUTES) {


                if ($ruleType === Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_LEAD_CUSTOM_ATTRIBUTES) {
                    $leadsCustomParamsMapper = Leads_Mapper_LeadsCustomParamsMapper::getInstance();
                    $scoreCustomAttributes = $leadsCustomParamsMapper->getParamByIdAndParamId($leadData['id'],
                        $scoreRule['field_name']);
                }

                if ($ruleType === Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_ORGANIZATION_CUSTOM_ATTRIBUTES) {
                    if (empty($leadData['organization_id'])) {
                        $addScore = false;
                        break;
                    }
                    $leadsOrganizationCustomParamsMapper = Leads_Mapper_LeadsOrganizationCustomParamsMapper::getInstance();
                    $scoreCustomAttributes = $leadsOrganizationCustomParamsMapper->getParamByIdAndParamId($leadData['organization_id'],
                        $scoreRule['field_name']);
                }

                if (empty($scoreCustomAttributes)) {
                    if ($scoreRule['field_value'] === '') {
                        $addScore = true;
                        break;
                    }
                    $addScore = false;
                    break;
                }

                $addScore = self::compareValue($scoreRule['rule_comparison_operator'],
                    $scoreCustomAttributes->getParamValue(), $scoreRule['field_value']);
                if ($addScore === false) {
                    break;
                } else {
                    $addScore = true;
                }
            }

            if ($ruleType === Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_ACTION_TYPE) {
                if (empty($scoreRule['rule_subtype'])) {
                    $addScore = false;
                    break;
                }

                $addScore = self::processActionRuleTypes($scoreRule['rule_subtype'], $leadData['id'],
                    $scoreRule['rule_comparison_operator'], $scoreRule['field_name'], $scoreRule['field_value']);
                if ($addScore === false) {
                    break;
                } else {
                    $addScore = true;
                }
            }
        }

        if ($addScore === true) {
            return array('error' => '0', 'score' => $scoreValue, 'ruleId' => $ruleId);
        }

        return array('error' => '1', 'score' => $currentScore, 'ruleId' => $ruleId);
    }

    /**
     * Check whether value meet criteria
     *
     * @param string $comparisonOperator comparison operator ('equal','notequal','like','in','greaterthan','lessthan', 'notin')
     * @param string $currentValue lead value
     * @param string $compareTo config value
     * @return bool
     */
    public static function compareValue($comparisonOperator, $currentValue, $compareTo)
    {
        $skipStatus = false;
        $compareTo = mb_strtolower($compareTo);
        if (is_array($currentValue)) {
            $dataValue = array_map('mb_strtolower', $currentValue);
        } else {
            $dataValue = mb_strtolower($currentValue);
        }

        if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_EQUAL) {
            if (is_array($dataValue)) {
                if (!in_array($compareTo, $dataValue)) {
                    $skipStatus = true;
                }
            } else {
                if ($dataValue !== $compareTo) {
                    $skipStatus = true;
                }
            }

        }

        if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_NOTEQUAL) {
            if (is_array($dataValue)) {
                if (in_array($compareTo, $dataValue)) {
                    $skipStatus = true;
                }
            } else {
                if ($dataValue === $compareTo) {
                    $skipStatus = true;
                }
            }
        }

        if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_LIKE) {
            $compareToArray = explode(',', $compareTo);
            $compareToArray = array_map("mb_strtolower", $compareToArray);
            $compareToArrayLength = count($compareToArray);
            if ($compareToArrayLength > 1) {
                if (is_array($dataValue)) {
                    $comparisonResult = array_intersect($compareToArray, $dataValue);
                    if (count($comparisonResult) == 0) {
                        $skipStatus = true;
                    }
                } else {
                    $compValCounter = 0;
                    foreach ($compareToArray as $compVal) {
                        $matchFound = mb_strpos($dataValue, $compVal);
                        if ($matchFound === false) {
                            $compValCounter = $compValCounter+1;
                        }
                    }

                    if ($compValCounter === $compareToArrayLength) {
                        $skipStatus = true;
                    }
                }
            } else {
                if (is_array($dataValue)) {
                    $inputSearchPattern = preg_quote($compareTo, '~');
                    $inputSearchResult = preg_grep('~' . $inputSearchPattern . '~', $dataValue);
                    if (empty($inputSearchResult)) {
                        $skipStatus = true;
                    }
                } else {
                    $matchFound = mb_strpos($dataValue, $compareTo);
                    if ($matchFound === false) {
                        $skipStatus = true;
                    }
                }
            }
        }

        if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_IN) {
            $compareToArray = explode(',', $compareTo);
            $compareToArray = array_map("mb_strtolower", $compareToArray);
            if (is_array($dataValue)) {
                $comparisonResult = array_intersect($compareToArray, $dataValue);
                if (count($comparisonResult) == 0) {
                    $skipStatus = true;
                }

            } else {
                if (!in_array($dataValue, $compareToArray)) {
                    $skipStatus = true;
                }
            }
        }

        if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_NOT_IN) {
            $compareToArray = explode(',', $compareTo);
            $compareToArray = array_map("mb_strtolower", $compareToArray);
            if (is_array($dataValue)) {
                $comparisonResult = array_intersect($compareToArray, $dataValue);
                if (count($comparisonResult) > 0) {
                    $skipStatus = true;
                }

            } else {
                if (in_array($dataValue, $compareToArray)) {
                    $skipStatus = true;
                }
            }
        }

        if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_GREATER_THAN) {
            if (is_numeric($dataValue)) {
                if ($compareTo > $dataValue) {
                    $skipStatus = true;
                }
            } elseif (is_array($dataValue)) {
                foreach ($dataValue as $value) {
                    if (is_numeric($value)) {
                        if ($compareTo > $dataValue) {
                            $skipStatus = true;
                        }
                    } else {
                        $skipStatus = true;
                    }
                }
            } else {
                $skipStatus = true;
            }
        }

        if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_LESS_THAN) {
            if (is_numeric($dataValue)) {
                if ($compareTo < $dataValue) {
                    $skipStatus = true;
                }
            } elseif (is_array($dataValue)) {
                foreach ($dataValue as $value) {
                    if (is_numeric($value)) {
                        if ($compareTo < $dataValue) {
                            $skipStatus = true;
                        }
                    } else {
                        $skipStatus = true;
                    }
                }
            } else {
                $skipStatus = true;
            }
        }

        if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_DATE_BEFORE) {
            $preparedDate = date_parse($dataValue);
            if ($preparedDate !== false) {
               if (!empty($preparedDate['year'])) {
                    $currentTime = Tools_System_Tools::convertDateFromTimezone('+ '.$compareTo.' days');
                    if (strtotime($currentTime) > strtotime($dataValue)) {
                        $skipStatus = false;
                    } else {
                        $skipStatus = true;
                    }
               } else {
                   $skipStatus = true;
               }
            } else {
                $skipStatus = true;
            }
        }

        if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_DATE_AFTER) {
            $preparedDate = date_parse($dataValue);
            if ($preparedDate !== false) {
                if (!empty($preparedDate['year'])) {
                    $currentTime = Tools_System_Tools::convertDateFromTimezone('- '.$compareTo.' days');
                    if (strtotime($currentTime) > strtotime($dataValue)) {
                        $skipStatus = false;
                    } else {
                        $skipStatus = true;
                    }
                } else {
                    $skipStatus = true;
                }
            } else {
                $skipStatus = true;
            }
        }

        if ($skipStatus === false) {
            return true;
        }

        return false;

    }


    /**
     * Process lead opportunities rules
     *
     * @param string $subtype rule subtype
     * @param int $leadId lead id
     * @param string $comparisonOperator comparison operator ('equal','notequal','like','in','greaterthan','lessthan')
     * @param string $fieldName specific field name
     * @param string $fieldValue value of the field
     * @return bool
     */
    public static function processLeadOpportunitiesTypes(
        $subtype,
        $leadId,
        $comparisonOperator,
        $fieldName,
        $fieldValue
    ) {

        $leadsOpportunityMapper = Leads_Mapper_LeadsOpportunityMapper::getInstance();
        if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::LEAD_OPPORTUNITY_ATTRIBUTE_OPPORTUNITY_CLOSE_DATE) {
            $expectedDateCloseFrom = '';
            $expectedDateCloseTo = '';
            if ($fieldValue === self::PERIOD_NEXT_DAY) {
                $expectedDateCloseFrom = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                    strtotime('tomorrow')), '', 'UTC', 'Y-m-d 00:00:00');
                $expectedDateCloseTo = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                    strtotime('tomorrow')), '', 'UTC', 'Y-m-d 23:59:59');
            }
            if ($fieldValue === self::PERIOD_NEXT_WEEK) {
                $expectedDateCloseFrom = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                    strtotime('next week - 1 day')), '', 'UTC', 'Y-m-d 00:00:00');
                $expectedDateCloseTo = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                    strtotime('saturday next week')), '', 'UTC', 'Y-m-d 23:59:59');
            }
            if ($fieldValue === self::PERIOD_NEXT_MONTH) {
                $expectedDateCloseFrom = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                    strtotime('first day of next month')), '', 'UTC', 'Y-m-d 00:00:00');
                $expectedDateCloseTo = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                    strtotime('last day of next month')), '', 'UTC', 'Y-m-d 23:59:59');
            }

            $where = $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.expected_close_date <= ?',
                $expectedDateCloseTo);
            $where .= ' AND ' . $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.expected_close_date >= ?',
                    $expectedDateCloseFrom);

            $data = $leadsOpportunityMapper->getOpportunitiesForTimeline($where, null, null, null, true, true);
        }

        if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::LEAD_OPPORTUNITY_ATTRIBUTE_OPPORTUNITY_STAGE) {
            $fieldValue = explode(',', $fieldValue);
            $fieldValue = array_map("mb_strtolower", $fieldValue);
            $where = $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.lead_id = ?',
                $leadId);

            if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_IN) {
                $where .= ' AND ' . $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.opportunity_stage_id IN (?)',
                        $fieldValue);
            }

            if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_NOT_IN) {
                $where .= ' AND ' . $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.opportunity_stage_id NOT IN (?)',
                        $fieldValue);
            }

            $data = $leadsOpportunityMapper->getOpportunitiesForTimeline($where, null, null, null, true, true);
        }

        if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::LEAD_OPPORTUNITY_ATTRIBUTE_OPPORTUNITY_TYPE) {
            $fieldValue = explode(',', $fieldValue);
            $fieldValue = array_map("mb_strtolower", $fieldValue);
            $where = $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.lead_id = ?',
                $leadId);

            if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_IN) {
                $where .= ' AND ' . $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.opportunity_type_id IN (?)',
                        $fieldValue);
            }

            if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_NOT_IN) {
                $where .= ' AND ' . $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.opportunity_type_id NOT IN (?)',
                        $fieldValue);
            }

            $data = $leadsOpportunityMapper->getOpportunitiesForTimeline($where, null, null, null, true, true);
        }

        if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::LEAD_OPPORTUNITY_ATTRIBUTE_OPPORTUNITY_AMOUNT_SINGLE
            || $fieldName === Leads_Model_LeadsScoreRulesConfigModel::LEAD_OPPORTUNITY_ATTRIBUTE_OPPORTUNITY_AMOUNT_CUMULATIVE) {
            $where = $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.lead_id = ?',
                $leadId);

            $groupBy = false;
            $having = false;
            if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::LEAD_OPPORTUNITY_ATTRIBUTE_OPPORTUNITY_AMOUNT_CUMULATIVE) {
                if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_GREATER_THAN) {
                    $having = new Zend_Db_Expr('SUM(plo.opportunity_value) >=' . $fieldValue);
                } else {
                    $having = new Zend_Db_Expr('SUM(plo.opportunity_value) <=' . $fieldValue);
                }
                $groupBy = 'plo.lead_id';
            }

            if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::LEAD_OPPORTUNITY_ATTRIBUTE_OPPORTUNITY_AMOUNT_SINGLE) {
                if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_GREATER_THAN) {
                    $where .= ' AND ' . $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.opportunity_value >= ?',
                            $fieldValue);
                } else {
                    $where .= ' AND ' . $leadsOpportunityMapper->getDbTable()->getAdapter()->quoteInto('plo.opportunity_value <= ?',
                            $fieldValue);
                }
            }

            $data = $leadsOpportunityMapper->getOpportunitiesForTimeline($where, null, null, null, true, true, $groupBy,
                $having);
        }

        if (!empty($data)) {
            return true;
        }

        return false;
    }


    public static function processForm($subtype, $leadId, $comparisonOperator, $fieldName, $fieldValue, $leadsData = array())
    {

        $fieldValue = mb_strtolower($fieldValue);
        $data = array();
        if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_IN || $comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_NOT_IN) {
            $fieldValue = explode(',', $fieldValue);
            $fieldValue = array_map("mb_strtolower", $fieldValue);
        }

        $leadsActionLogMapper = Leads_Mapper_LeadsActionLogMapper::getInstance();
        $where = $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plfl.lead_id = ?', $leadId);

        if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_IN || $comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_NOT_IN) {
            if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_IN) {
                $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('CONCAT(plfl.form_name, "_", plfl.form_type) IN (?)',
                        $fieldValue);
            }

            if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_NOT_IN) {
                $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('CONCAT(plfl.form_name, "_", plfl.form_type) NOT IN (?)',
                        $fieldValue);
            }

            $data = $leadsActionLogMapper->fetchAll($where, null, null, null, true, true);
        }

        if (!empty($data)) {
            return true;
        }


        return false;

    }

    /**
     * Process action type rules
     *
     * @param string $subtype rule subtype Ex: call_received, purchase etc..
     * @param int $leadId lead id
     * @param string $comparisonOperator comparison operator ('equal','notequal','like','in','greaterthan','lessthan')
     * @param string $fieldName specific field name
     * @param string $fieldValue value of the field
     * @param array $leadsData leads data
     * @param bool $returnFullResult return result instead of true|false
     * @return bool
     */
    public static function processActionRuleTypes($subtype, $leadId, $comparisonOperator, $fieldName, $fieldValue, $leadsData = array(), $returnFullResult = false)
    {
        $leadsActionLogMapper = Leads_Mapper_LeadsActionLogMapper::getInstance();
        $fieldValue = mb_strtolower($fieldValue);
        $data = array();
        if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_IN || $comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_NOT_IN || $comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_LIKE) {
            $fieldValue = explode(',', $fieldValue);
            $fieldValue = array_map("mb_strtolower", $fieldValue);
        }
        if ($subtype === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_CALL_RECEIVED || $subtype === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_CALL_PLACED) {
            $actionType = 'outbound';
            if ($subtype === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_CALL_RECEIVED) {
                $actionType = 'inbound';
            }
            if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_DURATION_PLACED ||
                $fieldName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_DURATION_RECEIVED) {
                if (!empty($fieldValue)) {
                    $fieldValue = (int)$fieldValue;
                    $where = $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plcl.lead_id = ?',
                        $leadId);
                    $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plal.action_type = ?',
                            $actionType);
                    $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plcl.type = ?',
                            'call');
                    if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_GREATER_THAN) {
                        $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plcl.call_duration >= ?',
                                $fieldValue);
                    } else {
                        $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plcl.call_duration <= ?',
                                $fieldValue);
                    }

                    $data = $leadsActionLogMapper->fetchAll($where, null, null, null, true, true);
                }

            }

            if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_TRACKING_NUMBER) {
                $fullCountryPhoneCodesList = Tools_System_Tools::getFullCountryPhoneCodesList(false, array(), true);

                if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_EQUAL ||
                    $comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_NOTEQUAL) {

                    $fieldValue = json_decode($fieldValue, true);

                    if(empty($fieldValue) || !is_array($fieldValue)) {
                        $data = array();
                    }

                    $fieldValue['country_code'] = mb_strtoupper($fieldValue['country_code']);
                    $fieldValue = $fullCountryPhoneCodesList[$fieldValue['country_code']] . $fieldValue['tracking_number'];
                }

                if (!empty($fieldValue)) {
                    $where = $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plcl.lead_id = ?', $leadId);
                    $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plcl.type = ?', 'call');
                    $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plal.action_type = ?', $actionType);
                    $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plal.lead_type = ?', Leads_Model_LeadsTypeModel::LEAD_TYPE_INBOUND_CALL);

                    if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_EQUAL) {
                        $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plcl.receiver = ?', $fieldValue);
                    }

                    if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_NOTEQUAL) {
                        $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plcl.receiver <> ?', $fieldValue);
                    }

                    if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_LIKE) {
                        $where .= ' AND (';
                        foreach ($fieldValue as $key => $fv) {
                            $where .= $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plcl.receiver LIKE  ?', '%' . $fv . '%');

                            if(!empty($fieldValue[$key+1])) {
                                $where .= ' OR ';
                            }
                        }
                        $where .= ')';
                    }

                    if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_IN) {
                        $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plcl.receiver IN (?)', $fieldValue);
                    }

                    $data = $leadsActionLogMapper->fetchAll($where, null, null, null, true, true);
                }
            }

        }

        if ($subtype === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_SMS_RECEIVED || $subtype === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_SMS_SENT) {
            $actionType = 'outbound';
            if ($subtype === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_SMS_RECEIVED) {
                $actionType = 'inbound';
            }

            if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_SMS_KEYWORD_SENT || $fieldName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_SMS_KEYWORD_RECEIVED) {
                $where = $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plcl.lead_id = ?',
                    $leadId);
                $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plal.action_type = ?',
                        $actionType);
                $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plcl.type = ?',
                        'sms');

                if (!empty($fieldValue)) {
                    if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_LIKE) {
                        $where .= ' AND (';
                        foreach ($fieldValue as $key => $fv) {
                            $where .= $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plcl.details LIKE  ?', '%' . $fv . '%');

                            if(!empty($fieldValue[$key+1])) {
                                $where .= ' OR ';
                            }
                        }
                        $where .= ')';
                    }
                }

                $data = $leadsActionLogMapper->fetchAll($where, null, null, null, true, true);
            }
        }


        if ($subtype === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_DOCUMENT_DOWNLOADED) {
            if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_DOWNLOADED_DOCUMENT_FILE_NAME) {
                $where = $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('pldl.lead_id = ?',
                    $leadId);

                if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_EQUAL) {
                    $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('pldl.file_name = ?',
                            $fieldValue);
                }

                if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_NOTEQUAL) {
                    $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('pldl.file_name <> ?',
                            $fieldValue);
                }

                if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_IN) {
                    $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('pldl.file_name IN (?)',
                            $fieldValue);
                }

                if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_LIKE) {
                    $where .= ' AND (';
                    foreach ($fieldValue as $key => $fv) {
                        $where .= $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('pldl.file_name LIKE  ?', '%' . $fv . '%');

                        if(!empty($fieldValue[$key+1])) {
                            $where .= ' OR ';
                        }
                    }
                    $where .= ')';
                }

                $data = $leadsActionLogMapper->fetchAll($where, null, null, null, true, true);

            }
        }

        if ($subtype === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_RECEIVED && $fieldName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_RECEIVED_IMAP) {
            $leadsActionEmailLogMapper = Leads_Mapper_LeadsActionLogMapper::getInstance();
            $where = $leadsActionEmailLogMapper->getDbTable()->getAdapter()->quoteInto('plel.message_id = ?', $leadsData['email_message_id']);
            $leadsEmailLogData = $leadsActionEmailLogMapper->fetchAll($where, null, null, null, true, true);
            if (!empty($leadsEmailLogData)) {
                $emailContent = $leadsEmailLogData['encodedContent'];
                if (!empty($emailContent)) {
                    $pattern = preg_quote($fieldValue);
                    $textFound = preg_match('~'.$pattern.'~ui', base64_decode($emailContent));
                    if (!empty($textFound)) {
                        return true;
                    }
                }
             }

            return false;
        }

        if ($subtype === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_RECEIVED && $fieldName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_RECEIVED_IMAP_FROM) {
            $leadsActionEmailLogMapper = Leads_Mapper_LeadsActionLogMapper::getInstance();
            $where = $leadsActionEmailLogMapper->getDbTable()->getAdapter()->quoteInto('plel.message_id = ?',
                $leadsData['email_message_id']);
            $leadsEmailLogData = $leadsActionEmailLogMapper->fetchAll($where, null, null, null, true, true);
            if (!empty($leadsEmailLogData)) {
                $emailFrom =  $leadsEmailLogData['emailFromEmail'];
                if ($emailFrom === $fieldValue) {
                    return true;
                }
            }

            return false;
        }

        if (($subtype === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_SENT
            || $subtype === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_RECEIVED) && $fieldName !== Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_RECEIVED_IMAP && $fieldName !== Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_RECEIVED_IMAP_FROM) {

            $where = $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plal.lead_id = ?',
                $leadId);

            $type = 'inbound';
            $typeEmail = 'received';
            if ($subtype === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_SENT) {
                $type = 'outbound';
                $typeEmail = 'sent';
            }

            if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_OPENED) {
                $typeEmail = 'opened';

                $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plal.action_type = ?',
                        $type);
                $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plal.type = ?',
                        'email');
                $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plel.email_status = ?',
                        $typeEmail);

                //current date
                //$currentDate = Tools_System_Tools::convertDateFromTimezone('now', 'UTC', Tools_System_Tools::getUserTimezone(), 'Y-m-d H:i:s');

                //previous date
                $dateBefore = date("Y-m-d H:i:s", strtotime("-". $fieldValue ." days"));
                $previousDate = Tools_System_Tools::convertDateFromTimezone($dateBefore, 'UTC', Tools_System_Tools::getUserTimezone(), 'Y-m-d H:i:s');

                $where .= ' AND ' .$leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plal.created_at >= ?', $previousDate);
            } else {
                $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plal.action_type = ?',
                        $type);
                $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plal.type = ?',
                        'email');

                $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plel.email_status = ?',
                        $typeEmail);

                $where .= ' AND (' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plql.status <> ?',
                        'new');
                $where .= ' OR ' . new Zend_Db_Expr('plql.status IS NULL');
                $where .= ' OR ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plql.force_display = ?',
                        '1') . ')';
            }

            if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_GREATER_THAN) {
                $having = new Zend_Db_Expr('COUNT(plal.lead_id) >=' . $fieldValue);
            } else {
                $having = new Zend_Db_Expr('COUNT(plal.lead_id) <=' . $fieldValue);
            }
            $groupBy = 'plal.lead_id';


            $data = $leadsActionLogMapper->fetchAll($where, null, null, null, true, true, false, $groupBy, $having);
        }

        if ($subtype === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_REGISTRATION) {
            $where = $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plrl.lead_id = ?',
                $leadId);

            $data = $leadsActionLogMapper->fetchAll($where, null, null, null, true, true);
        }

        if ($subtype === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_QUOTE_VIEW) {
            $where = $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plql.lead_id = ?',
                $leadId);
            $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plql.quote_action_type = ?',
                    'view');

            $data = $leadsActionLogMapper->fetchAll($where, null, null, null, true, true);

            if ($returnFullResult === true) {
                return $data;
            }
        }

        if ($subtype === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_QUOTE_REQUESTED) {
            $where = $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plql.lead_id = ?',
                $leadId);
            $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plal.type = ?',
                    'quote');
            $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plal.action_type = ?',
                    'inbound');
            $where .= ' AND (' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plql.status <> ?',
                    'new');
            $where .= ' OR ' . new Zend_Db_Expr('plql.status IS NULL');
            $where .= ' OR ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plql.force_display = ?',
                    '1') . ')';

            $data = $leadsActionLogMapper->fetchAll($where, null, null, null, true, true);

            if ($returnFullResult === true) {
                return $data;
            }
        }

        if ($subtype === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_QUOTE_SENT) {
            $where = $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plql.lead_id = ?',
                $leadId);
            $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plal.type = ?',
                    'quote');
            $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plal.action_type = ?',
                    'outbound');
            $where .= ' AND (' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plql.status <> ?',
                    'new');
            $where .= ' OR ' . new Zend_Db_Expr('plql.status IS NULL');
            $where .= ' OR ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plql.force_display = ?',
                    '1') . ')';

            $data = $leadsActionLogMapper->fetchAll($where, null, null, null, true, true);

            if ($returnFullResult === true) {
                return $data;
            }
        }

        if ($subtype === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_QUOTE_AMOUNT) {
            $where = $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('pld.id = ?',
                $leadId);

            $groupBy = false;
            $having = false;
            if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_QUOTE_AMOUNT_CUMULATIVE) {
                if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_GREATER_THAN) {
                    $having = new Zend_Db_Expr('SUM(scs.total) >=' . $fieldValue);
                } else {
                    $having = new Zend_Db_Expr('SUM(scs.total) <=' . $fieldValue);
                }
                $groupBy = 'pld.id';
            }

            if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_QUOTE_AMOUNT_SINGLE) {
                if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_GREATER_THAN) {
                    $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('scs.total >= ?',
                            $fieldValue);
                } else {
                    $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('scs.total <= ?',
                            $fieldValue);
                }
            }

            $data = $leadsActionLogMapper->getQuotesDataInfo($where, null, null, null, $groupBy, $having, true, true,
                false);

            if ($returnFullResult === true) {
                return $data;
            }

        }

        if ($subtype === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_QUOTE_EXPIRATION_DATE) {
            $lastActivityDateFrom = '';
            $lastActivityDateTo = '';
            if ($fieldValue === self::PERIOD_NEXT_DAY) {
                $lastActivityDateFrom = date('Y-m-d',
                    strtotime('tomorrow')). ' 00:00:00';
                $lastActivityDateTo = date('Y-m-d',
                    strtotime('tomorrow')). ' 23:59:59';
            }
            if ($fieldValue === self::PERIOD_NEXT_WEEK) {
                $lastActivityDateFrom = date('Y-m-d',
                    strtotime('next week - 1 day')). '00:00:00';
                $lastActivityDateTo = date('Y-m-d',
                    strtotime('saturday next week')).' 23:59:59';
            }
            if ($fieldValue === self::PERIOD_NEXT_MONTH) {
                $lastActivityDateFrom = date('Y-m-d',
                    strtotime('first day of next month')).' 00:00:00';
                $lastActivityDateTo = date('Y-m-d',
                    strtotime('last day of next month')). ' 23:59:59';
            }

            $where = $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('pld.id = ?',
                $leadId);
            $where .= ' AND ' .$leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('sq.expires_at <= ?',
                $lastActivityDateTo);
            $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('sq.expires_at >= ?',
                    $lastActivityDateFrom);

            $data = $leadsActionLogMapper->getQuotesDataInfo($where, null, null, null, false, false, true, true, false);

        }

        if ($subtype === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_WEBSITE_VISIT) {
            $leadsIpUserProfileLogActionDetailsMapper = Leads_Mapper_LeadsIpUserProfileLogActionDetailsMapper::getInstance();
            $where = $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('pliupl.lead_id = ?',
                $leadId);

            if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_WEBSITE_LAST_ACTION_DATE_TIME) {
                if ($fieldValue === self::PERIOD_LAST_TWELVE_MONTH) {
                    $lastActionDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                        strtotime('first day of -12 month')), '', 'UTC', 'Y-m-d 00:00:00');
                }

                if ($fieldValue === self::PERIOD_LAST_SIX_MONTH) {
                    $lastActionDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                        strtotime('first day of -6 month')), '', 'UTC', 'Y-m-d 00:00:00');
                }

                if ($fieldValue === self::PERIOD_LAST_THREE_MONTH) {
                    $lastActionDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                        strtotime('first day of -3 month')), '', 'UTC', 'Y-m-d 00:00:00');
                }

                if ($fieldValue === self::PERIOD_LAST_MONTH) {
                    $lastActionDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                        strtotime('first day of previous month')), '', 'UTC', 'Y-m-d 00:00:00');
                }

                if ($fieldValue === self::PERIOD_LAST_WEEK) {
                    $lastActionDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                        strtotime('monday last week - 1 day')), '', 'UTC', 'Y-m-d 00:00:00');;
                }

                if ($fieldValue === self::PERIOD_LAST_DAY) {
                    $lastActionDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                        strtotime('-1 day')), '', 'UTC', 'Y-m-d 00:00:00');
                }

                if ($fieldValue === self::PERIOD_THIS_DAY) {
                    $lastActionDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                        strtotime('now')), '', 'UTC', 'Y-m-d 00:00:00');;
                }

                $where .= ' AND '. $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plal.created_at >= ?',
                    $lastActionDate);
            }


            if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_PAGE_URL_VIEWED_LATEST
                || $fieldName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_TIME_ACTION_NUMBER_LATEST
                || $fieldName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_VISIT_DURATION_LATEST) {

                $where1 = $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('pliupl.lead_id = ?',
                    $leadId);
                $where1 .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plal.type = ?',
                        'visit');
                $lastInteraction = $leadsActionLogMapper->fetchAll($where1, array('pliupl.last_action_date_time DESC'),
                    null, null,
                    true, true);

                if (empty($lastInteraction)) {
                    return false;
                }

                $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('id_visit = ?',
                        $lastInteraction['idVisit']);
            }

            if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_PAGE_URL_VIEWED ||
                $fieldName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_PAGE_URL_VIEWED_LATEST) {
                if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_EQUAL) {
                    $queryParams = "'https://',`pliuplad`.`domain_name`,'/$fieldValue'";
                    $queryParams1 = "'http://',`pliuplad`.`domain_name`,'/$fieldValue'";
                    $where .= ' AND (' . new Zend_Db_Expr('page_url = CONCAT(' . $queryParams . ')');
                    $where .= ' OR ' . new Zend_Db_Expr('page_url = CONCAT(' . $queryParams1 . ')') . ')';
                }

                if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_NOTEQUAL) {
                    $queryParams = "'https://',`pliuplad`.`domain_name`,'/$fieldValue'";
                    $queryParams1 = "'http://',`pliuplad`.`domain_name`,'/$fieldValue'";
                    $where .= ' AND (' . new Zend_Db_Expr('page_url <> CONCAT(' . $queryParams . ')');
                    $where .= ' OR ' . new Zend_Db_Expr('page_url <> CONCAT(' . $queryParams1 . ')') . ')';
                }

                if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_IN) {
                    $where .= ' AND (';
                    foreach ($fieldValue as $fv) {
                        $queryParams = "'https://',`pliuplad`.`domain_name`,'/$fv'";
                        $queryParams1 = "'http://',`pliuplad`.`domain_name`,'/$fv'";
                        $where .= ' (' . new Zend_Db_Expr('page_url = CONCAT(' . $queryParams . ')');
                        $where .= ' OR ' . new Zend_Db_Expr('page_url = CONCAT(' . $queryParams1 . ')') . ')) OR (';
                    }

                    $where = rtrim($where, ' OR (');
                }

                if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_LIKE) {
                    $where .= ' AND (';

                    foreach ($fieldValue as $fv) {
                        $queryParams = "'https://',`pliuplad`.`domain_name`,'/','$fv','%'";
                        $queryParams1 = "'http://',`pliuplad`.`domain_name`,'/','$fv','%'";
                        $where .= ' (' . new Zend_Db_Expr('page_url LIKE CONCAT(' . $queryParams . ')');
                        $where .= ' OR ' . new Zend_Db_Expr('page_url LIKE CONCAT(' . $queryParams1 . ')') . ')) OR (';
                    }

                    $where = rtrim($where, ' OR (');
                }
            }

            if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_VISIT_DURATION ||
                $fieldName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_VISIT_DURATION_LATEST) {
                if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_GREATER_THAN) {
                    $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('pliupl.visit_duration >= ?',
                            $fieldValue);
                } else {
                    $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('pliupl.visit_duration <= ?',
                            $fieldValue);
                }
            }

            if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_TIME_ACTION_NUMBER ||
                $fieldName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_TIME_ACTION_NUMBER_LATEST) {
                if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_GREATER_THAN) {
                    $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('pliupl.actions >= ?',
                            $fieldValue);
                } else {
                    $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('pliupl.actions <= ?',
                            $fieldValue);
                }

            }
            if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_PAGE_URL_VIEWED ||
                $fieldName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_PAGE_URL_VIEWED_LATEST) {
                $data = $leadsIpUserProfileLogActionDetailsMapper->fetchAllForGridWithActionLog($where, null, null,
                    null, true, true);
            } else {
                $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plal.type = ?',
                        'visit');
                $data = $leadsActionLogMapper->fetchAll($where, null, null, null, true, true);
            }
        }

        if ($subtype === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_PURCHASE) {
            $where = $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('pld.id = ?',
                $leadId);
            $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('scs.status = ?',
                    'completed');
            $groupBy = false;
            $having = false;
            if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_PURCHASE_AMOUNT_CUMULATIVE) {
                if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_GREATER_THAN) {
                    $having = new Zend_Db_Expr('SUM(scs.total) >=' . $fieldValue);
                } else {
                    $having = new Zend_Db_Expr('SUM(scs.total) <=' . $fieldValue);
                }
                $groupBy = 'pld.id';
            }

            if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_PURCHASE_AMOUNT_SINGLE) {
                if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_GREATER_THAN) {
                    $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('scs.total >= ?',
                            $fieldValue);
                } else {
                    $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('scs.total <= ?',
                            $fieldValue);
                }
            }

            $withProducts = false;

            if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_PURCHASE_PRODUCTS_SKU) {
                $withProducts = true;
                $groupBy = 'pld.id';
                $productSkus = array_filter(array_unique(array_map('trim', $fieldValue)));
                $where .= ' AND ' . $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('sp.sku IN (?)',
                            $productSkus);
            }

            $data = $leadsActionLogMapper->getCartDataInfo($where, null, null, null, $groupBy, $having, true, true,
                false, $withProducts);

            if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_PURCHASE_PRODUCTS_SKU) {
                if ($comparisonOperator === Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_NOT_IN) {
                    if (!empty($data)) {
                        $data = array();
                    } else {
                        $data = array('lead_id' => $leadId);
                    }
                }
            }
        }

        if ($subtype === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_QUOTE_VIEW
            || $subtype === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_QUOTE_REQUESTED
            || $subtype === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_REGISTRATION
            || $subtype === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_QUOTE_SENT) {


            if ($fieldValue === 'no' && !empty($data)) {
                return false;
            } elseif ($fieldValue === 'no' && empty($data)) {
                return true;
            } elseif ($fieldValue === 'yes' && empty($data)) {
                return false;
            } else {
                return true;
            }

        }

        if (!empty($data)) {
            return true;
        }


        return false;
    }

}
