<?php

/**
 * Class Tools_LeadScoreWorkflowTools
 */
class Tools_LeadScoreWorkflowTools extends Tools_LeadScoreTools
{

    /**
     * Lead model status new
     */
    const LEAD_MODEL_STATUS_NEW = 'new';

    /**
     * Lead model status old
     */
    const LEAD_MODEL_STATUS_OLD = 'old';

    /**
     * Lead action type form
     */
    const LEAD_RULES_TYPE_FORM = 'type_form';

    /**
     * Lead action type event
     */
    const LEAD_RULES_TYPE_EVENT = 'type_event';

    /**
     * Lead action type lead workflow event
     */
    const LEAD_RULES_TYPE_WORKFLOW = 'type_workflow';

    /**
     * Allowed administrative roles list
     *
     * @var array
     */
    public static $_allowedAdministrativeRoles = array(
        Tools_Security_Acl::ROLE_SUPERADMIN,
        Tools_Security_Acl::ROLE_ADMIN,
        Shopping::ROLE_SALESPERSON
    );


    /**
     * Process workflow rules for leads
     *
     * @param array $leadsData leads data
     * @param array $workflowGroupedByRule grouped workflow rules
     * @param array $generalWorkflowRules general workflow rules
     * @return bool
     */
    public static function processWorkflowScore($leadsData, $generalWorkflowRules, $workflowGroupedByRule)
    {
        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
        $leadsWorkflowRulesLogMapper = Leads_Mapper_LeadsWorkflowRulesLogMapper::getInstance();


        foreach ($leadsData as $leadData) {
            try {
                foreach ($workflowGroupedByRule as $scoreRule) {
                    $operator = current($scoreRule)['score_comparison_operator'];
                    $score = current($scoreRule)['score'];
                    if (!empty($score)) {
                        $scoreResult = self::compareValue($operator, $leadData['score'], $score);
                        if ($scoreResult === false) {
                            continue;
                        }
                    }

                    $result = self::verifyWorkflowRule($leadData, $scoreRule);
                    if (empty($result['error'])) {
                        $ruleId = $result['ruleId'];
                        $leadId = $leadData['id'];
                        $leadsWorkflowRulesLogModel = $leadsWorkflowRulesLogMapper->findByRuleIdLeadId($ruleId, $leadId);
                        if ($leadsWorkflowRulesLogModel instanceof Leads_Model_LeadsWorkflowRulesLogModel) {
                            continue;
                        }

                        $leadModel = $leadsMapper->find($leadId);
                        if (!$leadModel instanceof Leads_Model_LeadsModel) {
                            continue;
                        }

                        self::applyRuleActions(array($ruleId), $leadModel, self::LEAD_MODEL_STATUS_OLD, self::LEAD_RULES_TYPE_WORKFLOW, $leadModel->toArray());

                        $leadsWorkflowRulesLogModel = new Leads_Model_LeadsWorkflowRulesLogModel();
                        $leadsWorkflowRulesLogModel->setRuleId($ruleId);
                        $leadsWorkflowRulesLogModel->setLeadId($leadId);
                        $leadsWorkflowRulesLogMapper->save($leadsWorkflowRulesLogModel);
                    }
                }

            } catch (Exception $exception) {
                return false;
            }
        }

        return true;
    }

    /**
     * Verify workflow rule condition
     *
     * @param array $leadData lead data
     * @param array $scoreRules workflow score rules
     * @return array
     */
    public static function verifyWorkflowRule($leadData, $scoreRules)
    {
        $addScore = false;
        foreach ($scoreRules as $scoreRule) {
            $ruleType = $scoreRule['rule_type'];
            $fieldName = $scoreRule['field_name'];
            $ruleId = $scoreRule['id'];
            if ($fieldName === 'lead_customer_group_id' || $fieldName === 'organization_customer_group_id') {
                $fieldName = 'customer_group_id';
            }

            if (!empty($scoreRule['rule_subtype']) && $scoreRule['rule_subtype'] === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_RECEIVED && ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_RECEIVED_IMAP || Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_RECEIVED_IMAP_FROM) && empty($leadData['isImap'])) {
                $addScore = false;
                break;

            }

            if ($ruleType === Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_LEAD_ATTRIBUTE) {
                $fieldValue = $scoreRule['field_value'];
                if (in_array($fieldName, self::$_specialLeadAttributes, true)) {
                    if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_LAST_ACTIVITY_INBOUND
                        || $fieldName === Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_LAST_ACTIVITY_OUTBOUND) {
                        $lastActivityDate = '';
                        $lastActivityToDate = '';
                        if ($fieldValue === self::PERIOD_LAST_DAY) {
                            $lastActivityDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('-1 day')), '', 'UTC', 'Y-m-d 00:00:00');
                            $lastActivityToDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('-1 day')), '', 'UTC', 'Y-m-d 23:59:59');
                        }
                        if ($fieldValue === self::PERIOD_LAST_WEEK) {
                            $lastActivityDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('monday last week - 1 day')), '', 'UTC', 'Y-m-d 00:00:00');
                            $lastActivityToDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('monday this week - 1 second - 1 day')), '', 'UTC', 'Y-m-d 23:59:59');
                        }
                        if ($fieldValue === self::PERIOD_LAST_MONTH) {
                            $lastActivityDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('first day of previous month')), '', 'UTC', 'Y-m-d 00:00:00');
                            $lastActivityToDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('last day of previous month')), '', 'UTC', 'Y-m-d 23:59:59');
                        }

                        if ($fieldValue === self::PERIOD_LAST_YEAR) {
                            $lastActivityDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('first day of january previous year')), '', 'UTC', 'Y-m-d 00:00:00');
                            $lastActivityToDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('last day of december previous year')), '', 'UTC', 'Y-m-d 23:59:59');
                        }

                        if ($fieldValue === self::PERIOD_THIS_DAY) {
                            $lastActivityDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('now')), '', 'UTC', 'Y-m-d 00:00:00');
                            $lastActivityToDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('now')), '', 'UTC', 'Y-m-d 23:59:59');
                        }

                        if ($fieldValue === self::PERIOD_THIS_WEEK) {
                            $lastActivityDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('this week monday - 1 day')), '', 'UTC', 'Y-m-d 00:00:00');
                            $lastActivityToDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('now')), '', 'UTC', 'Y-m-d 23:59:59');
                        }

                        if ($fieldValue === self::PERIOD_THIS_MONTH) {
                            $lastActivityDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('first day of this month')), '', 'UTC', 'Y-m-d 00:00:00');
                            $lastActivityToDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('now')), '', 'UTC', 'Y-m-d 23:59:59');
                        }

                        if ($fieldValue === self::PERIOD_THIS_YEAR) {
                            $lastActivityDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('first day of january this year')), '', 'UTC', 'Y-m-d 00:00:00');
                            $lastActivityToDate = Tools_System_Tools::convertDateFromTimezone(date(Tools_System_Tools::DATE_MYSQL,
                                strtotime('now')), '', 'UTC', 'Y-m-d 23:59:59');
                        }

                        if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_LAST_ACTIVITY_INBOUND) {
                            $lastActivityCurrentLead = $leadData['last_activity'];
                        } else {
                            $lastActivityCurrentLead = $leadData['last_activity_outbound'];
                        }

                        if (empty($lastActivityCurrentLead) || empty($lastActivityDate)) {
                            $addScore = false;
                            break;
                        }

                        if (strtotime($lastActivityCurrentLead) >= strtotime($lastActivityDate) && strtotime($lastActivityCurrentLead) <= strtotime($lastActivityToDate)) {
                            $addScore = true;
                        } else {
                            $addScore = false;
                            break;
                        }
                    }

                    if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_LEAD_TAGS) {
                        $currentLeadTags = Leads_Mapper_LeadsTagsMapper::getInstance()->findTagsByLeadId($leadData['id']);
                        if (empty($currentLeadTags)) {
                            $addScore = false;
                            break;
                        }

                        $addScore = self::compareValue($scoreRule['rule_comparison_operator'],
                            $currentLeadTags, $scoreRule['field_value']);

                        if ($addScore === false) {
                            break;
                        } else {
                            $addScore = true;
                        }

                    }

                    if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_NPS_REVIEW_SCORE) {
                        $leadsReviewRequestMapper = Leads_Mapper_LeadsReviewRequestMapper::getInstance();
                        $leadsReviewRequestModel = $leadsReviewRequestMapper->findByLeadId($leadData['id']);
                        if ($leadsReviewRequestModel instanceof Leads_Model_LeadsReviewRequestModel) {
                            $reviewScore = $leadsReviewRequestModel->getScore();
                            $reviewNpsStatus = $leadsReviewRequestModel->getStatus();
                            $reviewStatus = $leadsReviewRequestModel->getReviewStatus();
                            if (Leads_Model_LeadsReviewRequestModel::STATUS_REVIEWED === $reviewNpsStatus) {
                                $addScore = self::compareValue($scoreRule['rule_comparison_operator'],
                                    $reviewScore, $scoreRule['field_value']);
                                if ($addScore === false) {
                                    break;
                                } else {
                                    $addScore = true;
                                }
                            } else {
                                $addScore = false;
                                break;
                            }
                        }

                    }

                    if ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::LEAD_ATTRIBUTE_LEAD_PHONE) {
                        $fullCountryPhoneCodesList = Tools_System_Tools::getFullCountryPhoneCodesList(false, array(), true);

                        $leadPhone = $leadData['lead_phone'];
                        $codeValue = '';

                        if(empty($leadPhone)) {
                            $addScore = false;
                            break;
                        }

                        if(!empty($leadData['lead_phone_country_code']) || !empty($leadData['lead_phone_country_code_value'])) {
                            if(!empty($leadData['lead_phone_country_code_value'])) {
                                $codeValue = $leadData['lead_phone_country_code_value'];
                            } elseif (!empty($leadData['lead_phone_country_code'])){
                                $codeValue = $fullCountryPhoneCodesList[$leadData['lead_phone_country_code']];
                            }
                        }

                        if($scoreRule['rule_comparison_operator'] == Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_EQUAL ||
                            $scoreRule['rule_comparison_operator'] == Leads_Model_LeadsScoreRulesConfigModel::RULE_COMPARISON_OPERATOR_NOTEQUAL) {

                            if(empty($codeValue)) {
                                $addScore = false;
                                break;
                            }

                            $fieldValue = json_decode($fieldValue, true);

                            if(empty($fieldValue) || !is_array($fieldValue)) {
                                $addScore = false;
                                break;
                            }

                            $fieldValue = $fullCountryPhoneCodesList[$fieldValue['country_code']] . $fieldValue['lead_phone'];
                        }

                        $leadPhoneFull = $codeValue . $leadPhone;

                        $addScore = self::compareValue($scoreRule['rule_comparison_operator'], $leadPhoneFull, $fieldValue);

                        if ($addScore === false) {
                            break;
                        } else {
                            $addScore = true;
                        }

                    }

                } else {
                    if (isset($leadData[$fieldName])) {
                        $addScore = self::compareValue($scoreRule['rule_comparison_operator'],
                            $leadData[$fieldName], $scoreRule['field_value']);
                        if ($addScore === false) {
                            break;
                        } else {
                            $addScore = true;
                        }
                    } else {
                        $addScore = false;
                        break;
                    }
                }

            }

            if ($ruleType === Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_ORGANIZATION_ATTRIBUTE) {
                if (!empty($leadData['organization_id'])) {
                    $organizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
                    $where = $organizationMapper->getDbTable()->getAdapter()->quoteInto('plo.id = ?',
                        $leadData['organization_id']);
                    $organizationData = $organizationMapper->fetchAll($where, null, null, null, true, true);
                    if (empty($organizationData)) {
                        $addScore = false;
                        break;
                    }
                    if (isset($organizationData[$fieldName])) {
                        $addScore = self::compareValue($scoreRule['rule_comparison_operator'],
                            $organizationData[$fieldName], $scoreRule['field_value']);
                        if ($addScore === false) {
                            break;
                        } else {
                            $addScore = true;
                        }
                    } else {
                        $addScore = false;
                        break;
                    }

                } else {
                    $addScore = false;
                    break;
                }

            }

            if ($ruleType === Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_LEAD_OPPORTUNITIES) {
                $addScore = self::processLeadOpportunitiesTypes($scoreRule['rule_subtype'], $leadData['id'],
                    $scoreRule['rule_comparison_operator'], $scoreRule['field_name'], $scoreRule['field_value']);
                if ($addScore === false) {
                    break;
                } else {
                    $addScore = true;
                }
            }

            if ($ruleType === Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_LEAD_CUSTOM_ATTRIBUTES ||
                $ruleType === Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_ORGANIZATION_CUSTOM_ATTRIBUTES) {


                if ($ruleType === Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_LEAD_CUSTOM_ATTRIBUTES) {
                    $leadsCustomParamsMapper = Leads_Mapper_LeadsCustomParamsMapper::getInstance();
                    $scoreCustomAttributes = $leadsCustomParamsMapper->getParamByIdAndParamId($leadData['id'],
                        $scoreRule['field_name']);
                }

                if ($ruleType === Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_ORGANIZATION_CUSTOM_ATTRIBUTES) {
                    if (empty($leadData['organization_id'])) {
                        $addScore = false;
                        break;
                    }
                    $leadsOrganizationCustomParamsMapper = Leads_Mapper_LeadsOrganizationCustomParamsMapper::getInstance();
                    $scoreCustomAttributes = $leadsOrganizationCustomParamsMapper->getParamByIdAndParamId($leadData['organization_id'],
                        $scoreRule['field_name']);
                }

                if (empty($scoreCustomAttributes)) {
                    if ($scoreRule['field_value'] === '') {
                        $addScore = true;
                        break;
                    }
                    $addScore = false;
                    break;
                }

                $addScore = self::compareValue($scoreRule['rule_comparison_operator'],
                    $scoreCustomAttributes->getParamValue(), $scoreRule['field_value']);
                if ($addScore === false) {
                    break;
                } else {
                    $addScore = true;
                }
            }

            if ($ruleType === Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_ACTION_TYPE) {
                if (empty($scoreRule['rule_subtype'])) {
                    $addScore = false;
                    break;
                }

                $addScore = self::processActionRuleTypes($scoreRule['rule_subtype'], $leadData['id'],
                    $scoreRule['rule_comparison_operator'], $scoreRule['field_name'], $scoreRule['field_value'], $leadData);
                if ($addScore === false) {
                    break;
                } else {
                    $addScore = true;
                }
            }


            if ($ruleType === Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_FORM_NAME) {
                $addScore = self::processForm($scoreRule['rule_subtype'], $leadData['id'],
                    $scoreRule['rule_comparison_operator'], $scoreRule['field_name'], $scoreRule['field_value'], $leadData);
                if ($addScore === false) {
                    break;
                } else {
                    $addScore = true;
                }
            }
        }


        if ($addScore === true) {
            return array('error' => '0', 'ruleId' => $ruleId);
        }

        return array('error' => '1',  'ruleId' => $ruleId);
    }


    /**
     * Apply actions based on the matched form rules
     *
     * @param array $ruleIds rule ids to apply
     * @param Leads_Model_LeadsModel $leadModel lead Model
     * @param string $leadModelStatus status of the lead (new, old)
     * @param string $actionType action type type_form|type_event
     * @param array $formData form data
     * @throws Exceptions_SeotoasterException
     * @throws Zend_Exception
     * @throws Zend_File_Transfer_Exception
     */
    public static function applyRuleActions(
        $ruleIds = array(),
        Leads_Model_LeadsModel $leadModel,
        $leadModelStatus = self::LEAD_MODEL_STATUS_OLD,
        $actionType = self::LEAD_RULES_TYPE_WORKFLOW,
        $formData = array()
    ) {


        if ($actionType === self::LEAD_RULES_TYPE_WORKFLOW) {
            $leadsWorkflowActionsMapper = Leads_Mapper_LeadsWorkflowActionsMapper::getInstance();
            $actions = $leadsWorkflowActionsMapper->getActionByRuleIds($ruleIds);
            $leadsWorkflowConfigMapper = Leads_Mapper_LeadsWorkflowConfigMapper::getInstance();
            $rulesInfo = $leadsWorkflowConfigMapper->getScoreRules(current($ruleIds));
            foreach ($rulesInfo as $ruleInfo) {
                $ruleType = $ruleInfo['rule_type'];
                $fieldName = $ruleInfo['field_name'];
                if ($fieldName === 'lead_customer_group_id' || $fieldName === 'organization_customer_group_id') {
                    $fieldName = 'customer_group_id';
                }

                if (!empty($ruleInfo['rule_subtype']) && $ruleInfo['rule_subtype'] === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_RECEIVED && ($fieldName === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_RECEIVED_IMAP || Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_EMAIL_RECEIVED_IMAP_FROM) && empty($formData['isImap'])) {
                    break;

                }

                if ($ruleType === Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_ACTION_TYPE) {
                    if (empty($ruleInfo['rule_subtype'])) {
                        break;
                    }

                    $ruleSubtype = $ruleInfo['rule_subtype'];
                    if ($ruleSubtype === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_QUOTE_VIEW || $ruleSubtype === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_QUOTE_SENT || $ruleSubtype === Leads_Model_LeadsScoreRulesConfigModel::RULE_SUBTYPE_ATTRIBUTE_QUOTE_AMOUNT_SINGLE) {

                        $ruleResultInfo = self::processActionRuleTypes($ruleInfo['rule_subtype'], $formData['id'],
                            $ruleInfo['rule_comparison_operator'], $ruleInfo['field_name'], $ruleInfo['field_value'], $formData, true);

                        if (!empty($ruleResultInfo)) {
                            if (!empty($ruleResultInfo['quoteId'])) {
                                $formData['quoteId'] = $ruleResultInfo['quoteId'];
                            }
                        }
                    }
                }
            }
        }

        $originalActionType = $actionType;

        if (!empty($actions)) {
            foreach ($actions as $action) {
                $actionType = $action['action_type'];
                $actionConfig = $action['action_config'];
                if (!empty($actionConfig)) {
                    $actionConfig = json_decode($actionConfig, true);
                    $triggerNewLeadOnly = false;
                    if (!empty($actionConfig['triggerNewLeadOnly'])) {
                        $triggerNewLeadOnly = true;
                    }

                    if ($leadModelStatus !== self::LEAD_MODEL_STATUS_NEW && $triggerNewLeadOnly === true) {
                        continue;
                    }

                    if ($actionType === Leads_Model_LeadsFormRulesActionModel::ACTION_TYPE_ASSIGN_OWNER) {
                        self::assignLeadOwnerAction($actionConfig, $leadModel);
                    }

                    if ($actionType === Leads_Model_LeadsFormRulesActionModel::ACTION_TYPE_ASSIGN_TASK) {
                        self::assignTaskAction($actionConfig, $leadModel);
                    }

                    if ($actionType === Leads_Model_LeadsFormRulesActionModel::ACTION_TYPE_ASSIGN_TAGS) {
                        self::assignLeadTagsAction($actionConfig, $leadModel);
                    }

                    if ($actionType === Leads_Model_LeadsFormRulesActionModel::ACTION_TYPE_ASSIGN_LIFECYCLE) {
                        self::assignLeadLifecycleAction($actionConfig, $leadModel);
                    }

                    if ($actionType === Leads_Model_LeadsFormRulesActionModel::ACTION_TYPE_ASSIGN_SEQUENCE) {
                        self::assignLeadSequenceAction($actionConfig, $leadModel);
                    }

                    if ($actionType === Leads_Model_LeadsFormRulesActionModel::ACTION_TYPE_ASSIGN_EMAIL_LIST) {
                        self::assignLeadEmailServiceAction($actionConfig, $leadModel->getId());
                    }

                    if ($actionType === Leads_Model_LeadsFormRulesActionModel::ACTION_TYPE_ASSIGN_EMAIL_COPY) {
                        self::assignLeadEmailCopyAction($actionConfig, $leadModel, $formData, $originalActionType);
                    }

                    if ($actionType === Leads_Model_LeadsFormRulesActionModel::ACTION_TYPE_SEND_REVIEW_REQUEST) {
                        self::sendReviewRequestAction($actionConfig, $leadModel->getId());
                    }

                    if ($actionType === Leads_Model_LeadsFormRulesActionModel::ACTION_TYPE_ASSIGN_REMARKETING_LIST) {
                        self::assignLeadRemarketingServiceAction($actionConfig, array($leadModel->getId()));
                    }
                }
            }
        }
    }

    /**
     * Assign task action
     *
     * @param array $actionConfig action config
     * @param Leads_Model_LeadsModel $leadModel lead Model
     * @return bool
     */
    public static function assignTaskAction($actionConfig = array(), Leads_Model_LeadsModel $leadModel)
    {

        $leadsTaskMapper = Leads_Mapper_LeadsTaskMapper::getInstance();
        $leadsTaskModel = new Leads_Model_LeadsTaskModel();

        if (empty($actionConfig['creator_id']) || empty($actionConfig['user_id'])) {
            return false;
        }

        $creatorId = $actionConfig['creator_id'];
        $userId = $actionConfig['user_id'];
        $userMapper = Application_Model_Mappers_UserMapper::getInstance();
        $creatorUserModel = $userMapper->find($creatorId);
        if (!$creatorUserModel instanceof Application_Model_Models_User) {
            return false;
        }

        $creatorRoleId = $creatorUserModel->getRoleId();

        if (!in_array($creatorRoleId, self::$_allowedAdministrativeRoles)) {
            return false;
        }

        $userModel = $userMapper->find($userId);
        if (!$userModel instanceof Application_Model_Models_User) {
            return false;
        }

        $userRoleId = $userModel->getRoleId();
        if (!in_array($userRoleId, self::$_allowedAdministrativeRoles)) {
            return false;
        }

        $data['creator_id'] = $actionConfig['creator_id'];
        $data['user_id'] = $actionConfig['user_id'];
        $data['notes'] = $actionConfig['notes'];
        $data['title'] = $actionConfig['title'];

        $leadId = $leadModel->getId();
        $leadsData = $leadModel->toArray();
        $organizationData = array();
        $dictionary = array();
        $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
        $leadsData['leadlink'] = $websiteHelper->getUrl().'crmLogin/tabName/dashboard/subTab/leads/paramHashName/lead/paramHash/'.$leadModel->getId().'/';
        $dictionary = Tools_LeadsMailWatchdog::generateDictionary($dictionary, $leadsData, 'lead');
        $dictionary = Tools_LeadsMailWatchdog::generateDictionary($dictionary, ['organizationName' => '', 'orglink' => $leadsData['leadlink']], 'organization');
        if (!empty($leadsData['organizationId'])) {
            $organizationModel = Leads_Mapper_LeadsOrganizationMapper::getInstance()->find($leadsData['organizationId']);
            if ($organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                $organizationData = $organizationModel->toArray();
                $organizationData['orglink'] = $websiteHelper->getUrl() . 'crmLogin/tabName/dashboard/subTab/organizations/paramHashName/organization/paramHash/' . $organizationModel->getId() . '/';
                $dictionary = Tools_LeadsMailWatchdog::generateDictionary($dictionary, $organizationData, 'organization');
            }
        }
        $data['title'] = Tools_LeadsMailWatchdog::parseEmailLexems($data['title'], $dictionary);
        $data['notes'] = Tools_LeadsMailWatchdog::parseEmailLexems($data['notes'], $dictionary);
        $data['notes'] = Tools_LeadTools::parseLexemsAndContent($data['notes'], $leadModel);
        $data['lead_id'] = $leadId;

        $leadsTaskModel->setOptions($data);

        $leadTaskModel = $leadsTaskMapper->save($leadsTaskModel);

        $createdAt = Tools_System_Tools::convertDateFromTimezone('now');

        $actionLogExpiredModel = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_TASK, $leadId,
            Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND, $createdAt,
            Leads_Model_LeadsTypeModel::LEAD_OUTBOUND_TASK);

        if (!empty($actionConfig['taskImmediate'])) {
            $expiredAt = Tools_System_Tools::convertDateFromTimezone('now');
        } else {
            $shoppingConfigMapper =  Models_Mapper_ShoppingConfig::getInstance();
            $taskDelayUnits = $actionConfig['taskDelayUnits'];
            $storeTimezone = $shoppingConfigMapper->getConfigParam('timezone');
            if (!empty($actionConfig['taskTimezone'])) {
                $timezone = $actionConfig['taskTimezone'];
            } elseif (!empty($storeTimezone)) {
                $timezone = $storeTimezone;
            } else {
                $timezone = 'UTC';
            }
            if ($taskDelayUnits !== 'hour') {
                $expiredAtDate = date('Y-m-d',
                        strtotime('now' . '+' . $actionConfig['taskDelay'] . $actionConfig['taskDelayUnits'])) . ' ' . $actionConfig['assignTaskTime'];
                $hoursShift = Tools_EmailSequenceTools::getTimezoneShift('UTC', $timezone);
                if ($hoursShift > 0) {
                    $hoursShift = ' -' . $hoursShift;
                } else {
                    $hoursShift = ' +' . abs($hoursShift);
                }
                $expiredAt = date(Tools_System_Tools::DATE_MYSQL, strtotime($expiredAtDate . $hoursShift . 'hours'));
            } else {
                $expiredAt = date('Y-m-d H:i:s',
                    strtotime($createdAt . '+' . $actionConfig['taskDelay'] . $actionConfig['taskDelayUnits']));
            }
        }

        if (!empty($actionConfig['assignTaskDueTime'])) {
            $shoppingConfigMapper =  Models_Mapper_ShoppingConfig::getInstance();
            $taskDelayUnits = $actionConfig['taskDelayDueUnits'];
            $storeTimezone = $shoppingConfigMapper->getConfigParam('timezone');
            if (!empty($actionConfig['taskTimezone'])) {
                $timezone = $actionConfig['taskTimezone'];
            } elseif (!empty($storeTimezone)) {
                $timezone = $storeTimezone;
            } else {
                $timezone = 'UTC';
            }
            if ($taskDelayUnits !== 'hour') {
                $dueAtDate = date('Y-m-d',
                        strtotime('now' . '+' . $actionConfig['taskDueDelay'] . $actionConfig['taskDelayDueUnits'])) . ' ' . $actionConfig['assignDueTime'];
                $hoursShift = Tools_EmailSequenceTools::getTimezoneShift('UTC', $timezone);
                if ($hoursShift > 0) {
                    $hoursShift = ' -' . $hoursShift;
                } else {
                    $hoursShift = ' +' . abs($hoursShift);
                }
                $dueAt = date(Tools_System_Tools::DATE_MYSQL, strtotime($dueAtDate . $hoursShift . 'hours'));
            } else {
                $dueAt = date('Y-m-d H:i:s',
                    strtotime($createdAt . '+' . $actionConfig['taskDueDelay'] . $actionConfig['taskDelayDueUnits']));
            }
            $data['due_date'] = $dueAt;
            $leadTaskModel->setDueDate($data['due_date']);
        }

        $data['expired_at'] = $expiredAt;
        $leadTaskModel->setExpiredAt($data['expired_at']);
        $leadTaskModel = $leadsTaskMapper->save($leadsTaskModel);

        $actionLogUpcomingModel = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_TASK, $leadId,
            Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND, $expiredAt,
            Leads_Model_LeadsTypeModel::LEAD_OUTBOUND_TASK, Leads_Model_LeadsActionLogModel::EVENT_LOG_UPCOMING);

        $leadTaskId = $leadTaskModel->getId();

        $leadsTaskLogMapper = Leads_Mapper_LeadsTasksLogMapper::getInstance();
        $leadsTaskLogModel = new Leads_Model_LeadsTasksLogModel();
        $leadsTaskLogModel->setOptions($data);
        $leadsTaskLogModel->setActionLogId($actionLogExpiredModel->getId());
        $leadsTaskLogModel->setTaskId($leadTaskId);
        $leadsTaskLogModel->setLeadId($leadId);
        $leadsTaskLogModel->setDueDate($data['due_date']);
        $leadsTaskLogModel->setIsTaskOverdue('0');
        $leadsTaskLogMapper->save($leadsTaskLogModel);


        $leadsTaskLogModel = new Leads_Model_LeadsTasksLogModel();
        $leadsTaskLogModel->setOptions($data);
        $leadsTaskLogModel->setActionLogId($actionLogUpcomingModel->getId());
        $leadsTaskLogModel->setTaskId($leadTaskId);
        $leadsTaskLogModel->setLeadId($leadId);
        $leadsTaskLogModel->setDueDate($data['due_date']);
        $leadsTaskLogModel->setIsTaskOverdue('0');
        $leadsTaskLogMapper->save($leadsTaskLogModel);

        if (!empty($actionConfig['taskImmediate'])) {
            $leadTaskModel = Leads_Mapper_LeadsTaskMapper::getInstance()->find($leadTaskId);
            if ($leadTaskModel instanceof Leads_Model_LeadsTaskModel) {
                $leadTaskModel->registerObserver(new Tools_Mail_Watchdog(array(
                    'trigger' => Tools_LeadsMailWatchdog::LEADS_TASK_CREATED,
                    'leadData' => $leadsData,
                    'organizationData' => $organizationData
                )));

                $leadTaskModel->notifyObservers();
            }
        }

//        if (!empty($actionConfig['taskImmediate'])) {
//            $leadTaskQueueMapper = Leads_Mapper_LeadsTaskQueueMapper::getInstance();
//            $taskReminderInQueueModel = $leadTaskQueueMapper->findByTaskIdQueueType($leadTaskId, Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_TYPE_FIRST_REMINDER);
//            if (!$taskReminderInQueueModel instanceof Leads_Model_LeadsTaskQueueModel) {
//                $taskReminderInQueueModel = new Leads_Model_LeadsTaskQueueModel();
//                $taskReminderInQueueModel->setStatus(Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_STATUS_SENT);
//                $taskReminderInQueueModel->setQueueType(Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_TYPE_FIRST_REMINDER);
//                $taskReminderInQueueModel->setTaskId($leadTaskId);
//                $taskReminderInQueueModel->setToSendDate($expiredAt);
//                $taskReminderInQueueModel->setSentDate($expiredAt);
//                $leadTaskQueueMapper->save($taskReminderInQueueModel);
//            }
//            $taskReminderInQueueModel = $leadTaskQueueMapper->findByTaskIdQueueType($leadTaskId, Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_TYPE_SECOND_REMINDER);
//            if (!$taskReminderInQueueModel instanceof Leads_Model_LeadsTaskQueueModel) {
//                $taskReminderInQueueModel = new Leads_Model_LeadsTaskQueueModel();
//                $taskReminderInQueueModel->setStatus('processing');
//                $taskReminderInQueueModel->setQueueType(Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_TYPE_SECOND_REMINDER);
//                $taskReminderInQueueModel->setTaskId($leadTaskId);
//                $taskReminderInQueueModel->setToSendDate($expiredAt);
//                $taskReminderInQueueModel->setSentDate($expiredAt);
//                $taskReminderInQueueModel = $leadTaskQueueMapper->save($taskReminderInQueueModel);
//            }
//
//            $leadTaskModel = Leads_Mapper_LeadsTaskMapper::getInstance()->find($leadTaskId);
//            if ($leadTaskModel instanceof Leads_Model_LeadsTaskModel) {
//                $leadTaskModel->registerObserver(new Tools_Mail_Watchdog(array(
//                    'trigger' => Tools_LeadsMailWatchdog::LEADS_TASK_REMINDER,
//                    'queueId' => $taskReminderInQueueModel->getId(),
//                    'leadData' => $leadsData,
//                    'organizationData' => $organizationData
//                )));
//
//                $leadTaskModel->notifyObservers();
//            }
//        }

        return true;
    }

    /**
     * Assign lead owner
     *
     * @param array $actionConfig action config
     * @param Leads_Model_LeadsModel $leadModel lead Model
     */
    public static function assignLeadOwnerAction($actionConfig = array(), Leads_Model_LeadsModel $leadModel)
    {
        $assignToId = $actionConfig['assign_to'];
        $userModel = Application_Model_Mappers_UserMapper::getInstance()->find($assignToId);
        if ($userModel instanceof Application_Model_Models_User) {
            $roleId = $userModel->getRoleId();
            if (in_array($roleId, self::$_allowedAdministrativeRoles)) {
                $leadModel->setOwnerId($assignToId);
                Leads_Mapper_LeadsMapper::getInstance()->save($leadModel);
            }
        }
    }

    /**
     * Assign lead tags
     *
     * @param array $actionConfig action config
     * @param Leads_Model_LeadsModel $leadModel lead Model
     */
    public static function assignLeadTagsAction($actionConfig = array(), Leads_Model_LeadsModel $leadModel)
    {
        if (!empty($actionConfig['tagIds'])) {
            $tagMapper = Leads_Mapper_LeadsTagsMapper::getInstance();

            $originalLeadTags = $tagMapper->findTagsByLeadId($leadModel->getId());
            if (empty($originalLeadTags)) {
                $originalLeadTags = array();
            }
            $assignLeadTags = $actionConfig['tagIds'];
            $leadTags = array_unique(array_merge($originalLeadTags, $assignLeadTags));
            $leadModel->setLeadTags($leadTags);
            Leads_Mapper_LeadsMapper::getInstance()->save($leadModel);
        }
    }

    /**
     * Assign lead email sequence
     *
     * @param array $actionConfig
     * @param Leads_Model_LeadsModel $leadModel
     * @throws Exception
     */
    public static function assignLeadSequenceAction($actionConfig = array(), Leads_Model_LeadsModel $leadModel)
    {
        if (!empty($actionConfig['sequenceId'])) {
            $leadId = $leadModel->getId();
            $leadEmailHasSequenceMapper = Leads_Mapper_LeadsHasEmailSequenceMapper::getInstance();
            $emailSequenceSettingsMapper = Emailsequence_Models_Mapper_EmailSequenceSettingsMapper::getInstance();
            $sequenceModel = $emailSequenceSettingsMapper->find($actionConfig['sequenceId']);

            if ($sequenceModel instanceof Emailsequence_Models_Models_EmailSequenceSettingsModel && !empty($leadId)) {
                $archiveSequence = $sequenceModel->getArchive();

                if(!$archiveSequence) {
                    $leadEmailHasModel = $leadEmailHasSequenceMapper->findByLeadIdSequenceId($leadId, $sequenceModel->getId());

                    if (!empty($actionConfig['unassignSequence'])) {
                        $leadHasSequences = $leadEmailHasSequenceMapper->getLeadSequenceByLeadId($leadId);
                        if (!empty($leadHasSequences)) {
                            foreach ($leadHasSequences as $leadHasSequence) {
                                $sequenceId = $leadHasSequence['sequence_id'];
                                $leadEmailHasSequenceMapper->delete($leadId, $sequenceId);
                                $notSendEmailsInQueue = $leadEmailHasSequenceMapper->getFromCartStatusQueue($leadId, 'lead',
                                    $sequenceId, array('0'));
                                if (!empty($notSendEmailsInQueue)) {
                                    $leadEmailSequenceStatusMapper = Leads_Mapper_LeadsEmailSequenceStatusMapper::getInstance();
                                    $leadEmailSequenceStatusMapper->deleteByLeadIdSequenceId($leadId, $sequenceId);
                                    $leadEmailHasSequenceMapper->deleteFromCartStatusQueue($leadId, 'lead', $sequenceId,
                                        '0');
                                }
                            }
                        }
                    }

                    if (!$leadEmailHasModel instanceof Leads_Model_LeadsHasEmailSequenceModel) {
                        $sequenceAppliedDate = date(Tools_System_Tools::DATE_MYSQL, strtotime('now'));

                        $leadEmailHasModel = new Leads_Model_LeadsHasEmailSequenceModel();
                        $leadEmailHasModel->setSequenceId($sequenceModel->getId());
                        $leadEmailHasModel->setLeadId($leadId);
                        $leadEmailHasModel->setSequenceDate($sequenceAppliedDate);

                        $leadEmailHasSequenceMapper->save($leadEmailHasModel);
                    }
                }
            }
        }
    }

    /**
     * Insert Ignore sequences into plugin_leads_has_email_sequence Dbtable
     *
     * @param $sequenceId
     * @param array $leadIds
     * @return bool
     */
    public static function assignLeadSequenceMassAction($sequenceId, array $leadIds = array())
    {
        if(!empty($sequenceId) && !empty($leadIds)){
            $dbAdapter = Zend_Db_Table::getDefaultAdapter();

            $sql = "INSERT IGNORE INTO plugin_leads_has_email_sequence (lead_id, sequence_id, sequence_date) VALUES (:lead_id, :sequence_id, :sequence_date)";

            foreach ($leadIds as $leadId) {
                $sequenceAppliedDate = date(Tools_System_Tools::DATE_MYSQL, strtotime('now'));
                $dbAdapter->query($sql, array('lead_id' => $leadId, 'sequence_id' => $sequenceId, 'sequence_date' => $sequenceAppliedDate));
            }

            return true;
        }else{
            return false;
        }
    }

    /**
     * Assign lead email service
     *
     * @param array $actionConfig
     * @param $leadId
     * @throws Exceptions_SeotoasterPluginException
     */
    public static function assignLeadEmailServiceAction($actionConfig = array(), $leadId)
    {
        if(!empty($actionConfig) && !empty($leadId)) {
            $emailServiceName = $actionConfig['serviceName'];

            if(!empty($actionConfig['type']) && !empty($emailServiceName)) {
                $data['services'] = array($emailServiceName => array('type' => $actionConfig['type']));

                $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();

                $serviceInfo = Apps_Models_Mapper_AppsSettingsMapper::getInstance()->getByServiceName($actionConfig['serviceName']);
                if(!empty($serviceInfo)) {
                    $detailedInfo = $serviceInfo[0]->getDetailedInfo();

                    if (($actionConfig['type'] === Apps::SERVICE_TYPE_EMAIL) && !$detailedInfo) {
                        $leadsData = $leadsMapper->findByLeadIdsForEmailService(array($leadId));
                    } else {
                        $leadsData = $leadsMapper->getExtendedLeadInfoForEmailService(array($leadId));
                    }

                    if (!empty($leadsData)) {
                        $mojoClientAction = $emailServiceName . 'Client';

                        $data['services'][$emailServiceName]['clients'] = $leadsData;

                        $emailLists = array();
                        if(!empty($actionConfig['lists'])) {
                            $emailLists = explode(',', $actionConfig['lists']);
                        }
                        $data['services'][$emailServiceName]['lists'] = $emailLists;

                        if($actionConfig['requestType'] == 'unassign') {
                            Apps::apiCall('DELETE', 'apps', array($mojoClientAction), $data);
                        } else {
                            Apps::apiCall('POST', 'apps', array($mojoClientAction), $data);
                        }

                        if (isset($actionConfig['listsLabels'])) {
                            self::assignListNameTags([$leadId], $actionConfig['listsLabels']);
                        }
                    }
                }
            }
        }
    }

    /**
     * * Assign lead Ad audience lists
     *
     * @param array $actionConfig
     * @param $leadId
     */
    public static function assignLeadRemarketingServiceAction($actionConfig = array(), array $leadIds)
    {
        if(!empty($actionConfig) && !empty($leadIds)) {
            $remarketingName = $actionConfig['serviceName'];

            if(!empty($remarketingName) && !empty($actionConfig['lists'])) {
                $lists = json_decode($actionConfig['lists'], true);
                $listsLabels = json_decode($actionConfig['listsLabels'], true);

                if(!empty($lists)) {
                    $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();

                    $leadsData = $leadsMapper->getExtendedLeadInfoForRemarketingLists($leadIds);

                    if (!empty($leadsData)) {
                        $existedLeadsIds = array();
                        foreach ($leadsData as $key => $leadData) {
                            $leadData['phone'] = '';
                            if(!empty($leadData['mobileNumberCode']) && !empty($leadData['mobileNumber'])) {
                                $leadData['phone'] = $leadData['mobileNumberCode'] . $leadData['mobileNumber'];
                            } elseif (!empty($leadData['phoneNumberCode']) && !empty($leadData['phoneNumber'])) {
                                $leadData['phone'] = $leadData['phoneNumberCode'] . $leadData['phoneNumber'];
                            }

                            $leadsData[$key] = $leadData;

                            if(in_array($leadData['id'], $leadIds)) {
                                $existedLeadsIds[] = $leadData['id'];
                            }
                        }

                        foreach ($lists as $marketingCampaignsName => $listsIds) {
                            if(!empty($listsIds)) {
                                $data['type'] = $marketingCampaignsName;
                                $data['audienceId'] = $listsIds;
                                $data['listContent'] = $leadsData;
                                $data['remove'] = false;

                                if($actionConfig['requestType'] == 'unassign') {
                                    $data['remove'] = true;
                                }

                                Tools_LeadRemarketingAudiencesTools::prepareAndSaveData($data['type'], $data['audienceId'], $data['remove'], $data['listContent']);

                                /*$response = Apps::apiCall('POST', 'sambastatsAddDataToRemarketingList', array(), $data);

                                if(!empty($response['errorList'])) {
                                    foreach ($response['errorList'] as $key => $error) {
                                        $errorListKey = array_search($key, $lists[$marketingCampaignsName]);
                                        unset($listsLabels[$marketingCampaignsName][$errorListKey]);
                                    }
                                }*/

                                if(!empty($listsLabels[$marketingCampaignsName])) {
                                    $leadTagsMapper = Leads_Mapper_LeadsTagsMapper::getInstance();

                                    foreach ($listsLabels[$marketingCampaignsName] as $listLabelKey => $listLabel) {
                                        $adsPrefix = 'Added to Google Audience';
                                        if($marketingCampaignsName == 'facebook') {
                                            $adsPrefix = 'Added to FB Audience';
                                        }

                                        $tag = $adsPrefix . ' ' . $listLabel .' - '. $lists[$marketingCampaignsName][$listLabelKey];//Added to Google Audience SMB CRM Leads - 243575394539

                                        $leadTagModel = $leadTagsMapper->getTag($tag);
                                        if (!$leadTagModel instanceof Leads_Model_LeadsTagModel) {
                                            $leadTagModel = new Leads_Model_LeadsTagModel();
                                            $leadTagModel->setLabel($tag);
                                            $leadTagModel->setTagName($tag);
                                            $leadTagModel = $leadTagsMapper->save($leadTagModel);
                                        }

                                        foreach ($existedLeadsIds as $leadId) {
                                            $leadsMapper->addTag($leadId, $leadTagModel->getId());
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    public static function assignListNameTags(array $leadIds, $listsLabels)
    {
        $leadsTagMapper = Leads_Mapper_LeadsTagsMapper::getInstance();
        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
        $tagsIds = [];
        $listsLabels = explode(',', $listsLabels);
        if (!empty($listsLabels)) {
            foreach ($listsLabels as $tag) {
                $data['tag_name'] = rtrim(preg_replace('~[^\w\s\_]~ui', '', $tag));
                $existingTag = $leadsTagMapper->getTag($data['tag_name']);
                if ($existingTag instanceof Leads_Model_LeadsTagModel) {
                    $tagsIds[] = $existingTag->getId();
                } else {
                    $data['label'] = preg_replace('~[^\w\s\_]~ui', '', $tag);
                    $leadsTagModel = new Leads_Model_LeadsTagModel();
                    $leadsTagModel->setOptions($data);
                    $validator = new Zend_Validate_Db_NoRecordExists(array(
                        'table' => 'plugin_leads_tags',
                        'field' => 'tag_name'
                    ));
                    if ($validator->isValid($data['tag_name'])) {
                        $leadsTagModel = $leadsTagMapper->save($leadsTagModel);
                        $tagsIds[] = $leadsTagModel->getId();
                    }
                }
            }
        }
        if (!empty($tagsIds) && !empty($leadIds)) {
            foreach ($leadIds as $leadId) {
                foreach ($tagsIds as $tagId) {
                    $leadsMapper->addTag($leadId, $tagId);
                }
            }
        }
    }

    /**
     * Send review request
     *
     * @param array $actionConfig
     * @param $leadId
     * @return string
     */
    public static function sendReviewRequestAction(
        $actionConfig = array(),
        $leadId
    ) {
        if (!empty($actionConfig) && !empty($leadId)) {
            $leadsReviewRequestMapper = Leads_Mapper_LeadsReviewRequestMapper::getInstance();
            $leadsReviewRequestModel = $leadsReviewRequestMapper->findByLeadId($leadId);
            if (!$leadsReviewRequestModel instanceof Leads_Model_LeadsReviewRequestModel) {
                return '';
            }
            $leadsMapperLeadsMapper = Leads_Mapper_LeadsMapper::getInstance();
            $currentLeadModel = $leadsMapperLeadsMapper->find($leadId);
            $reviewStatus = $leadsReviewRequestModel->getReviewStatus();

            if ($leadsReviewRequestModel->getStatus() === Leads_Model_LeadsReviewRequestModel::STATUS_REVIEWED && empty($reviewStatus)) {
                $reviewLinks = array(
                    'reviewlink:facebook' => '',
                    'reviewlink:gmb' => ''
                );

                if ($currentLeadModel instanceof Leads_Model_LeadsModel) {
                    $currentLeadModel->registerObserver(new Tools_Mail_Watchdog(array(
                        'trigger' => Tools_LeadsMailWatchdog::LEADS_REVIEW_REQUEST,
                        'observableModel' => $currentLeadModel,
                        'leadData' => $currentLeadModel->toArray(),
                        'reviewLinks' => $reviewLinks
                    )));

                    $currentLeadModel->notifyObservers();
                }

                $leadsReviewRequestModel->setReviewRequestSentAt(Tools_System_Tools::convertDateFromTimezone('now'));
                $leadsReviewRequestModel->setReviewStatus(Leads_Model_LeadsReviewRequestModel::STATUS_SENT);
                $leadsReviewRequestMapper->save($leadsReviewRequestModel);
            }
        }
    }

    /**
     * Send copy of the system form email
     *
     * @param array $actionConfig actions config
     * @param Leads_Model_LeadsModel $leadModel lead model
     * @param array $formData form data
     * @param string $actionType action type type_form|type_event
     * @throws Exceptions_SeotoasterException
     * @throws Zend_Exception
     * @throws Zend_File_Transfer_Exception
     */
    public static function assignLeadEmailCopyAction(
        $actionConfig = array(),
        Leads_Model_LeadsModel $leadModel,
        $formData,
        $actionType
    ) {
        if (!empty($formData) && !empty($actionConfig['emailsToCopy'])) {
            if ($actionConfig['emailsToCopy'] === '{leadowner:email}') {
                $leadOwnerModel = Application_Model_Mappers_UserMapper::getInstance()->find($leadModel->getOwnerId());
                if ($leadOwnerModel instanceof Application_Model_Models_User) {
                    $actionConfig['emailsToCopy'] = $leadOwnerModel->getEmail();
                }
            }
            $emailsToCopy = array_unique(array_filter(explode(',', $actionConfig['emailsToCopy'])));
            if ($actionType === self::LEAD_RULES_TYPE_FORM) {
                $formName = $formData['formName'];
                $formModel = Application_Model_Mappers_FormMapper::getInstance()->findByName($formName);
                $emailsToCopyFiltered = array();
                foreach ($emailsToCopy as $email) {
                    $emailValidator = new Tools_System_CustomEmailValidator();
                    if ($emailValidator->isValid($email)) {
                        $emailsToCopyFiltered[] = $email;
                    }
                }

                if (!empty($formData['externalFormSignature'])) {
                    $leadsFormBuilderGeneralConfigMapper = Leads_Mapper_LeadsBuilderFormGeneralConfigMapper::getInstance();
                    $leadsFormBuilderGeneralConfigModel = $leadsFormBuilderGeneralConfigMapper->findBySignature($formData['externalFormSignature']);
                    if (!$leadsFormBuilderGeneralConfigModel instanceof Leads_Model_LeadsBuilderFormGeneralConfigModel) {
                        return '';
                    }

                    Tools_LeadExternalFormTools::sendEmails($leadsFormBuilderGeneralConfigModel, $formData, $emailsToCopyFiltered, true);
                } else {
                    $xmlHttpRequest = Zend_Controller_Front::getInstance()->getRequest()->isXmlHttpRequest();
                    $options = array();
                    $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                    if (!$xmlHttpRequest) {
                        //Adding attachments to email
                        $websitePathTemp = $websiteHelper->getPath() . $websiteHelper->getTmp();
                        $uploader = new Zend_File_Transfer_Adapter_Http();
                        $uploader->setDestination($websitePathTemp);
                        $files = $uploader->getFileInfo();
                        if (!empty($files)) {
                            foreach ($files as $file => $fileInfo) {
                                if ($fileInfo['name'] != '') {
                                    $at = new Zend_Mime_Part(file_get_contents($uploader->getFileName($file)));
                                    $fInfo = finfo_open(FILEINFO_MIME_TYPE);
                                    $mime = finfo_file($fInfo, $uploader->getFileName($file));
                                    $at->type = $mime;
                                    $at->disposition = Zend_Mime::DISPOSITION_ATTACHMENT;
                                    $at->encoding = Zend_Mime::ENCODING_BASE64;
                                    $at->filename = $fileInfo['name'];
                                    $attachment[] = $at;
                                    unset($at);
                                }
                            }
                            if (!empty($attachment)) {
                                $options['attachment'] = $attachment;
                            }
                        }
                    }

                    $formModel->setAdminFrom(self::parseData($formModel->getAdminFrom()));
                    $formModel->setAdminSubject(html_entity_decode(self::parseData($formModel->getAdminSubject()), null,
                        'UTF-8'));
                    $formModel->setAdminFromName(html_entity_decode(self::parseData($formModel->getAdminFromName()),
                        null,
                        'UTF-8'));
                    $formModel->setAdminText(self::parseData($formModel->getAdminText()));
                    $formModel->setReplyText(self::parseData($formModel->getReplyText()));
                    $formModel->setContactEmail(self::parseData($formModel->getContactEmail()));
                    $formModel->setReplyFrom(self::parseData($formModel->getReplyFrom()));
                    $formModel->setReplyFromName(html_entity_decode(self::parseData($formModel->getReplyFromName()),
                        null,
                        'UTF-8'));
                    $formModel->setReplySubject(html_entity_decode(self::parseData($formModel->getReplySubject()), null,
                        'UTF-8'));
                    $formModel->setMobile(self::parseData($formModel->getMobile()));


                    $adminTemplateEmail = $formModel->getAdminMailTemplate();

                    if (!empty($adminTemplateEmail)) {
                        $options['template'] = $adminTemplateEmail;
                    }
                    $options['trigger'] = 'SystemEmailCopyForm';
                    $options['data'] = $formData;
                    $options['emailsTo'] = $emailsToCopyFiltered;
                    $options['withoutPreparseTemplate'] = true;

                    $systemMailWatchdog = new Tools_LeadsMailWatchdog($options);
                    $systemMailWatchdog->notify($formModel);
                }
            }

            if ($actionType === self::LEAD_RULES_TYPE_EVENT || $actionType === self::LEAD_RULES_TYPE_WORKFLOW) {
                if (!empty($emailsToCopy) && !empty($actionConfig['emailTemplate']) && !empty($actionConfig['emailSubject'])
                    && !empty($actionConfig['emailSubject']) && !empty($actionConfig['emailFrom'])) {

                    foreach ($emailsToCopy as $email) {
                        $emailValidator = new Tools_System_CustomEmailValidator();
                        if ($emailValidator->isValid($email)) {
                            $emailsToCopyFiltered[] = $email;
                        }
                    }

                    if (!empty($emailsToCopyFiltered)) {
                        $emailTemplate = Application_Model_Mappers_TemplateMapper::getInstance()->find($actionConfig['emailTemplate']);
                        if ($emailTemplate instanceof Application_Model_Models_Template) {
                            $leadsData = $leadModel->toArray();
                            $organizationData = array();
                            if (!empty($leadsData['organizationId'])) {
                                $organizationModel = Leads_Mapper_LeadsOrganizationMapper::getInstance()->find($leadsData['organizationId']);
                                if ($organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                                    $organizationData = $organizationModel->toArray();
                                }
                            }
                            if ($actionConfig['emailFrom'] === '{leadowner:email}') {
                                $leadOwnerModel = Application_Model_Mappers_UserMapper::getInstance()->find($leadModel->getOwnerId());
                                if ($leadOwnerModel instanceof Application_Model_Models_User) {
                                    $actionConfig['emailFrom'] = $leadOwnerModel->getEmail();
                                }
                            }

                            $options = array();
                            $options['template'] = $actionConfig['emailTemplate'];
                            $options['trigger'] = 'EventEmailCopy';
                            $options['data'] = $formData;
                            $options['emailsTo'] = $emailsToCopyFiltered;
                            $options['withoutPreparseTemplate'] = true;
                            $options['emailFrom'] = $actionConfig['emailFrom'];
                            $options['emailSubject'] = $actionConfig['emailSubject'];
                            $options['subject'] = $actionConfig['emailSubject'];
                            $options['withoutPreparseTemplate'] = true;
                            $options['leadData'] = $leadsData;
                            $options['organizationData'] = $organizationData;
                            if (!empty($actionConfig['emailMessage'])) {
                                $options['message'] = $actionConfig['emailMessage'];
                            }

                            $options['leadData']['additionalInfo'] = $formData;

                            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                            $link = $websiteHelper->getUrl().'crmLogin/tabName/dashboard/subTab/leads/paramHashName/lead/paramHash/'.$leadModel->getId().'/';
                            $link = "<a href=".$link.">link</a>";
                            $options['leadData']['leadlink'] = $link;
                            if (!empty($organizationData) && $organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                                $orgLink = $websiteHelper->getUrl() . 'crmLogin/tabName/dashboard/subTab/organizations/paramHashName/organization/paramHash/' . $organizationModel->getId() . '/';
                                $orgLink = "<a href=" . $orgLink . ">organization</a>";
                                $options['organizationData']['orglink'] = $orgLink;
                            }


                            $formModel = new Application_Model_Models_Form();
                            $systemMailWatchdog = new Tools_LeadsMailWatchdog($options);
                            $systemMailWatchdog->notify($formModel);
                        }
                    }

                }

            }

        }

    }

    /**
     * Parse widgets for template
     *
     * @param string $data
     * @return null
     * @throws Zend_Exception
     */
    public static function parseData($data)
    {
        $themeData = Zend_Registry::get('theme');
        $extConfig = Zend_Registry::get('extConfig');
        $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
        $parserOptions = array(
            'websiteUrl' => $websiteHelper->getUrl(),
            'websitePath' => $websiteHelper->getPath(),
            'currentTheme' => $extConfig['currentTheme'],
            'themePath' => Tools_Filesystem_Tools::cleanWinPath($themeData['path']),
        );
        $parser = new Tools_Content_Parser($data, array(), $parserOptions);

        return $parser->parseSimple();
    }

    /**
     * Assign lead lifecycle
     *
     * @param array $actionConfig action config
     * @param Leads_Model_LeadsModel $leadModel lead Model
     */
    public static function assignLeadLifecycleAction($actionConfig = array(), Leads_Model_LeadsModel $leadModel)
    {
        if (!empty($actionConfig['lifecycleId'])) {
            $leadModel->setStageId($actionConfig['lifecycleId']);
            Leads_Mapper_LeadsMapper::getInstance()->save($leadModel);
        }
    }

}
