<?php

/**
 * Class Lead_LeadStatsTools
 */
class Tools_LeadStatsTools
{

    /**
     * Existing stat types
     */
    const LEAD_STAT_TYPE_CALLS_PLACED = 'callsPlaced';

    const LEAD_STAT_TYPE_CALLS_RECEIVED = 'callsReceived';

    const LEAD_STAT_TYPE_EMAILS_SENT = 'emailsSent';

    const LEAD_STAT_TYPE_EMAILS_RECEIVED = 'emailsReceived';

    const LEAD_STAT_TYPE_MEETINGS_HELD = 'meetingsHeld';

    const LEAD_STAT_TYPE_TASKS_COMPLETED = 'tasksCompleted';

    const LEAD_STAT_TYPE_OPP_AMOUNT = 'amount';

    const LEAD_STAT_TYPE_OPP_COUNT = 'count';

    const LEAD_STAT_TYPE_SUB_TYPE_WON = 'won';

    const LEAD_STAT_TYPE_SUB_TYPE_OPEN = 'open';

    /**
     *
     * Get general stats
     *
     * @param $leadReportsFilterStartDate
     * @param $leadReportsFilterEndDate
     * @param array $statsTypes
     * @param int $ownerId owner id
     * @param string $leadReportsCompare
     * @param string $leadReportsCompareFilterStartDate
     * @param string $leadReportsCompareFilterEndDate
     * @return array
     */
    public static function getGeneralStats(
        $leadReportsFilterStartDate,
        $leadReportsFilterEndDate,
        $statsTypes = array(),
        $ownerId = 0,
        $leadReportsCompare = '',
        $leadReportsCompareFilterStartDate = '',
        $leadReportsCompareFilterEndDate = ''
    ) {

        $data = array();
        $leadReportsMapper = Leads_Mapper_LeadsReportsMapper::getInstance();
        if (in_array(self::LEAD_STAT_TYPE_CALLS_PLACED, $statsTypes)) {
            $originalData = $leadReportsMapper->getCallsStats($leadReportsFilterStartDate, $leadReportsFilterEndDate,
                Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND, $ownerId, 'call');
            $compareData = array();
            if (!empty($leadReportsCompare)) {
                $compareData = $leadReportsMapper->getCallsStats($leadReportsCompareFilterStartDate,
                    $leadReportsCompareFilterEndDate, Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND, $ownerId, 'call');
            }

            $data['stats'][self::LEAD_STAT_TYPE_CALLS_PLACED] = Tools_LeadReportsTools::prepareStatsData($originalData,
                $leadReportsCompare, $compareData);
        }

        if (in_array(self::LEAD_STAT_TYPE_CALLS_RECEIVED, $statsTypes)) {
            $originalData = $leadReportsMapper->getCallsStats($leadReportsFilterStartDate, $leadReportsFilterEndDate,
                Leads_Model_LeadsActionLogModel::ACTION_TYPE_INBOUND, $ownerId, 'call');
            $compareData = array();
            if (!empty($leadReportsCompare)) {
                $compareData = $leadReportsMapper->getCallsStats($leadReportsCompareFilterStartDate,
                    $leadReportsCompareFilterEndDate, Leads_Model_LeadsActionLogModel::ACTION_TYPE_INBOUND, $ownerId, 'call');
            }

            $data['stats'][self::LEAD_STAT_TYPE_CALLS_RECEIVED] = Tools_LeadReportsTools::prepareStatsData($originalData,
                $leadReportsCompare, $compareData);
        }

        if (in_array(self::LEAD_STAT_TYPE_EMAILS_SENT, $statsTypes)) {
            //sent
//            $originalDataEmailSent = $leadReportsMapper->getEmailsStats($leadReportsFilterStartDate,
//                $leadReportsFilterEndDate, Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_SENT, '', true, $ownerId);

            $originalDataEmailSent = $leadReportsMapper->getManualSentEmailsStats($leadReportsFilterStartDate,
                $leadReportsFilterEndDate, Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_SENT, '', true, $ownerId);
            //sent
            $originalDataEmailSequenceSent = $leadReportsMapper->getEmailsSequenceStats($leadReportsFilterStartDate,
                $leadReportsFilterEndDate, $ownerId);
            //sent
            $originalDataEmailNewsletterSent = $leadReportsMapper->getEmailsStats($leadReportsFilterStartDate,
                $leadReportsFilterEndDate, Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_SENT, 'sambaEmailMarketing',
                false, $ownerId);

            //opened
            $originalDataEmailOpened = $leadReportsMapper->getEmailsStats($leadReportsFilterStartDate,
                $leadReportsFilterEndDate, Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_OPENED, '', true, $ownerId);
            //opened
            $originalDataEmailSequenceOpened = $leadReportsMapper->getEmailsSequenceStatsOpened($leadReportsFilterStartDate,
                $leadReportsFilterEndDate, $ownerId);
            //opened
            $originalDataEmailNewsletterOpened = $leadReportsMapper->getEmailsStats($leadReportsFilterStartDate,
                $leadReportsFilterEndDate, Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_OPENED, 'sambaEmailMarketing',
                false, $ownerId);

//            if (!empty($originalDataEmailSent) && !empty($originalDataEmailSent['quantity']) && !empty($originalDataEmailSequenceSent)) {
//                $originalDataEmailSent['quantity'] -= $originalDataEmailSequenceSent['quantity'];
//
//            }
//
//            if (!empty($originalDataEmailOpened) && !empty($originalDataEmailOpened['quantity']) && !empty($originalDataEmailSequenceOpened)) {
//                $originalDataEmailOpened['quantity'] -= $originalDataEmailSequenceOpened['quantity'];
//            }

            $compareDataSent = array();
            $originalData = array();
            $originalData['quantity'] = $originalDataEmailSent['quantity'] + $originalDataEmailSequenceSent['quantity'] + $originalDataEmailNewsletterSent['quantity'];
            if (!empty($leadReportsCompare)) {
                //sent
                $compareDataEmailSent = $leadReportsMapper->getEmailsStats($leadReportsCompareFilterStartDate,
                    $leadReportsCompareFilterEndDate, Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_SENT, '', true, $ownerId);
                //sent
                $compareDataEmailSequenceSent = $leadReportsMapper->getEmailsSequenceStats($leadReportsCompareFilterStartDate,
                    $leadReportsCompareFilterEndDate, $ownerId);
                //sent
                $compareDataEmailNewsletterSent = $leadReportsMapper->getEmailsStats($leadReportsCompareFilterStartDate,
                    $leadReportsCompareFilterEndDate, Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_SENT,
                    'sambaEmailMarketing', false, $ownerId);

                if (!empty($compareDataEmailSent) && !empty($compareDataEmailSent['quantity']) && !empty($compareDataEmailSequenceSent)) {
                    $compareDataEmailSent['quantity'] -= $compareDataEmailSequenceSent['quantity'];
                }

                $compareDataSent['quantity'] = $compareDataEmailSent['quantity'] + $compareDataEmailSequenceSent['quantity'] + $compareDataEmailNewsletterSent['quantity'];
            }

            //sent
            $result = Tools_LeadReportsTools::prepareStatsData($originalData,
                $leadReportsCompare, $compareDataSent);

            $result['manual'] = $originalDataEmailSent['quantity'];
            $result['auto'] = $originalDataEmailSequenceSent['quantity'];
            $result['newsletter'] = $originalDataEmailNewsletterSent['quantity'];

            //opened
            $compareDataOpened = array();
            $originalData['quantity'] = $originalDataEmailOpened['quantity'] + $originalDataEmailSequenceOpened['quantity'] + $originalDataEmailNewsletterOpened['quantity'];
            if (!empty($leadReportsCompare)) {
                //opened
                $compareDataEmailOpened = $leadReportsMapper->getEmailsStats($leadReportsCompareFilterStartDate,
                    $leadReportsCompareFilterEndDate, Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_OPENED, '', true, $ownerId);
                //opened
                $compareDataEmailSequenceOpened = $leadReportsMapper->getEmailsSequenceStatsOpened($leadReportsCompareFilterStartDate,
                    $leadReportsCompareFilterEndDate, $ownerId);
                //opened
                $compareDataEmailNewsletterOpened = $leadReportsMapper->getEmailsStats($leadReportsCompareFilterStartDate,
                    $leadReportsCompareFilterEndDate, Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_OPENED,
                    'sambaEmailMarketing', false, $ownerId);

                if (!empty($compareDataEmailOpened) && !empty($compareDataEmailOpened['quantity']) && !empty($compareDataEmailSequenceOpened)) {
                    $compareDataEmailOpened['quantity'] -= $compareDataEmailSequenceOpened['quantity'];
                }

                $compareDataOpened['quantity'] = $compareDataEmailOpened['quantity'] + $compareDataEmailSequenceOpened['quantity'] + $compareDataEmailNewsletterOpened['quantity'];
            }

            $quantityOpened = Tools_LeadReportsTools::prepareStatsData($originalData,
                $leadReportsCompare, $compareDataOpened);

            $result['incrementOpened'] = $quantityOpened['increment'];
            $result['incrementSymbolOpened'] = $quantityOpened['incrementSymbol'];
            $result['incrementSymbolLabelOpened'] = $quantityOpened['incrementSymbolLabel'];
            $result['quantityOpened'] = $quantityOpened['quantity'];

            $result['manualOpened'] = $originalDataEmailOpened['quantity'];
            $result['autoOpened'] = $originalDataEmailSequenceOpened['quantity'];
            $result['newsletterOpened'] = $originalDataEmailNewsletterOpened['quantity'];

            $data['stats'][self::LEAD_STAT_TYPE_EMAILS_SENT] = $result;
        }

        if (in_array(self::LEAD_STAT_TYPE_EMAILS_RECEIVED, $statsTypes)) {
            $originalData = $leadReportsMapper->getEmailsStats($leadReportsFilterStartDate, $leadReportsFilterEndDate,
                Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_RECEIVED, '', true, $ownerId);
            $compareData = array();
            if (!empty($leadReportsCompare)) {
                $compareData = $leadReportsMapper->getEmailsStats($leadReportsCompareFilterStartDate,
                    $leadReportsCompareFilterEndDate, Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_RECEIVED, '', true, $ownerId);
            }

            $data['stats'][self::LEAD_STAT_TYPE_EMAILS_RECEIVED] = Tools_LeadReportsTools::prepareStatsData($originalData,
                $leadReportsCompare, $compareData);
        }

        if (in_array(self::LEAD_STAT_TYPE_MEETINGS_HELD, $statsTypes)) {
            $originalData = $leadReportsMapper->getMeetings($leadReportsFilterStartDate, $leadReportsFilterEndDate,
                array('finished'), $ownerId);
            $compareData = array();
            if (!empty($leadReportsCompare)) {
                $compareData = $leadReportsMapper->getMeetings($leadReportsCompareFilterStartDate,
                    $leadReportsCompareFilterEndDate, array('finished'), $ownerId);
            }

            $data['stats'][self::LEAD_STAT_TYPE_MEETINGS_HELD] = Tools_LeadReportsTools::prepareStatsData($originalData,
                $leadReportsCompare, $compareData);
        }

        if (in_array(self::LEAD_STAT_TYPE_TASKS_COMPLETED, $statsTypes)) {
            $originalData = $leadReportsMapper->getTasks($leadReportsFilterStartDate, $leadReportsFilterEndDate,
                array(), array(Leads_Model_LeadsActionLogModel::ACTION_TASK_COMPLETED), $ownerId);
            $compareData = array();
            if (!empty($leadReportsCompare)) {
                $compareData = $leadReportsMapper->getTasks($leadReportsCompareFilterStartDate,
                    $leadReportsCompareFilterEndDate, array(),
                    array(Leads_Model_LeadsActionLogModel::ACTION_TASK_COMPLETED), $ownerId);
            }

            $data['stats'][self::LEAD_STAT_TYPE_TASKS_COMPLETED] = Tools_LeadReportsTools::prepareStatsData($originalData,
                $leadReportsCompare, $compareData);
        }

        return $data;
    }

    /**
     * Calculate goal opportunities
     *
     *
     * @param array $stats stats data
     * @param int $userId user id
     * @param string $filterPeriodMonths user id
     * @param string $filterDateFrom filter date from
     * @param string $filterDateTo filter date to
     * @return array
     */
    public static function calculateSalesGoals($stats, $filterPeriodMonths, $filterDateFrom, $filterDateTo, $userId = 0) {
        $goalsConfigData = Leads_Mapper_LeadsOwnerGoalsMapper::getInstance()->getMainConfigData($userId);
        if (empty($goalsConfigData)) {
            return $stats;
        }

        if (empty($userId)) {
            $configData = array('quote_objective' => 0);
            foreach ($goalsConfigData as $singleConfigData) {
                $configData['quote_objective'] += $singleConfigData['quote_objective'];
            }

            $stats = self::calculateSingleSalesStat($stats, $configData, $filterDateFrom, $filterDateTo,
                $filterPeriodMonths);
        } else {
            foreach ($goalsConfigData as $singleConfigData) {
                $stats = self::calculateSingleSalesStat($stats, $singleConfigData, $filterDateFrom, $filterDateTo,
                    $filterPeriodMonths);
            }
        }

        return $stats;

    }

    public static function calculateSingleSalesStat($stats, $singleConfigData, $filterDateFrom, $filterDateTo, $filterPeriodMonths)
    {

        if ($filterPeriodMonths != '0') {
            $stats['goal'] = $singleConfigData['quote_objective']*$filterPeriodMonths;
            if (empty($stats['totalAmount']) || empty($singleConfigData['quote_objective'])) {
                $stats['goalCompletion'] = 0;
            } else {
                $stats['goalCompletion'] = round(($stats['totalAmount']/ ($singleConfigData['quote_objective']*$filterPeriodMonths)*100), 2);
            }

        } else {
            $date1 = date_create($filterDateTo);
            $date2 = date_create($filterDateFrom);
            $dateDiff = date_diff($date1, $date2);
            $numberOfDays = $dateDiff->days;

            if ($numberOfDays === 0) {
                $numberOfDays = 1;
            }

            if ($numberOfDays <= 30) {
                $filterPeriodMonths = $numberOfDays/30;
            } else {
                $filterPeriodMonths = $numberOfDays/30;
            }

            $stats['goal'] = $singleConfigData['quote_objective'];
            if (empty($stats['totalAmount']) || empty($singleConfigData['quote_objective'])) {
                $stats['goalCompletion'] = 0;
            } else {
                $stats['goalCompletion'] = round(($stats['totalAmount'] / ($singleConfigData['quote_objective'] * $filterPeriodMonths) * 100),
                    2);
            }
        }

        return $stats;
    }

    /**
     * Calculate goal opportunities
     *
     *
     * @param array $stats stats data
     * @param int $userId user id
     * @param string $filterPeriodMonths user id
     * @param string $filterDateFrom filter date from
     * @param string $filterDateTo filter date to
     * @param string $type type amount|count
     * @param int $oppType opp type id
     * @return array
     */
    public static function calculateOpportunityGoals($stats, $filterPeriodMonths, $filterDateFrom, $filterDateTo, $userId = 0, $type = 'amount', $oppType = 0)
    {
        $goalsConfigData = Leads_Mapper_LeadsOwnerGoalsMapper::getInstance()->getMainConfigData($userId);
        if (empty($goalsConfigData)) {
            return $stats;
        }

        if (empty($userId)) {
            $allConfigData  = array();

            foreach ($goalsConfigData as $singleConfigData) {
                $goalsOpportunityConfigData = Leads_Mapper_LeadsOwnerGoalsOpportunitiesMapper::getInstance()->findByGoalId($singleConfigData['id'], $oppType);
                if (empty($allConfigData)) {
                    $allConfigData['goalsOpportunityConfigData'] = $goalsOpportunityConfigData;
                } else {
                    $allConfigData['goalsOpportunityConfigData'] = $allConfigData['goalsOpportunityConfigData']+$goalsOpportunityConfigData;
                }

            }

            $stats = self::calculateSingleUserOppStat($stats, $allConfigData, $filterDateFrom, $filterDateTo,
                $filterPeriodMonths, $type);
        } else {
            foreach ($goalsConfigData as $singleConfigData) {
                $goalsOpportunityConfigData = Leads_Mapper_LeadsOwnerGoalsOpportunitiesMapper::getInstance()->findByGoalId($singleConfigData['id'], $oppType);
                $singleConfigData['goalsOpportunityConfigData'] = $goalsOpportunityConfigData;
                $stats = self::calculateSingleUserOppStat($stats, $singleConfigData, $filterDateFrom, $filterDateTo,
                    $filterPeriodMonths, $type);
            }
        }

        return $stats;
    }

    public static function calculateSingleUserOppStat($stats, $singleConfigData, $filterDateFrom, $filterDateTo, $filterPeriodMonths, $type)
    {
        if ($filterPeriodMonths != '0') {
            if ($type === self::LEAD_STAT_TYPE_OPP_AMOUNT) {
                $stats['openGoalAmountCompletion'] = 0;
                $stats['wonGoalAmountCompletion'] = 0;
                $stats['goalAmount'] = 0;
                if (!empty($singleConfigData['goalsOpportunityConfigData'])) {
                    foreach ($singleConfigData['goalsOpportunityConfigData'] as $goalOppConfig) {
                        $stats['goalAmount'] += $goalOppConfig['opportunity_amount']*$filterPeriodMonths;
                    }

                    if (!empty($stats['goalAmount'])) {
                        if (empty($stats['openAmount']) || empty($stats['goalAmount'])) {
                            $stats['openGoalAmountCompletion'] = 0;
                        } else {
                            $stats['openGoalAmountCompletion'] = round((($stats['openAmount'] / $stats['goalAmount']) * 100),
                                2);
                        }

                        if (empty($stats['wonAmount']) || empty($stats['goalAmount'])) {
                            $stats['wonGoalAmountCompletion'] = 0;
                        } else {
                            $stats['wonGoalAmountCompletion'] = round((($stats['wonAmount'] / $stats['goalAmount']) * 100),
                                2);
                        }
                    }
                }
            } else {
                $stats['openGoalCountCompletion'] = 0;
                $stats['wonGoalCountCompletion'] = 0;
                $stats['goalCount'] = 0;
                if (!empty($singleConfigData['goalsOpportunityConfigData'])) {
                    foreach ($singleConfigData['goalsOpportunityConfigData'] as $goalOppConfig) {
                        $stats['goalCount'] += $goalOppConfig['opportunity_count']*$filterPeriodMonths;
                    }

                    if (!empty($stats['goalCount'])) {
                        if (empty($stats['openCount']) || empty($stats['goalCount'])) {
                            $stats['openGoalCountCompletion'] = 0;
                        } else {
                            $stats['openGoalCountCompletion'] = round((($stats['openCount'] / $stats['goalCount']) * 100),
                                2);
                        }

                        if (empty($stats['wonCount']) || empty($stats['goalCount'])) {
                            $stats['wonGoalCountCompletion'] = 0;
                        } else {
                            $stats['wonGoalCountCompletion'] = round((($stats['wonCount'] / $stats['goalCount']) * 100),
                                2);
                        }
                    }
                }
            }
        } else {
            $date1 = date_create($filterDateTo);
            $date2 = date_create($filterDateFrom);
            $dateDiff = date_diff($date1, $date2);
            $numberOfDays = $dateDiff->days;

            if ($numberOfDays === 0) {
                $numberOfDays = 1;
            }

            if ($numberOfDays <= 30) {
                $filterPeriodMonths = $numberOfDays/30;
            } else {
                $filterPeriodMonths = $numberOfDays/30;
            }

            if ($type === self::LEAD_STAT_TYPE_OPP_AMOUNT) {
                $stats['openGoalAmountCompletion'] = 0;
                $stats['wonGoalAmountCompletion'] = 0;
                $stats['goalAmount'] = 0;
                if (!empty($singleConfigData['goalsOpportunityConfigData'])) {
                    foreach ($singleConfigData['goalsOpportunityConfigData'] as $goalOppConfig) {
                        $stats['goalAmount'] += $goalOppConfig['opportunity_amount'] * $filterPeriodMonths;
                    }

                    if (!empty($stats['goalAmount'])) {
                        if (empty($stats['openAmount']) || empty($stats['goalAmount'])) {
                            $stats['openGoalAmountCompletion'] = 0;
                        } else {
                            $stats['openGoalAmountCompletion'] = round((($stats['openAmount'] / ($stats['goalAmount'])) * 100),
                                2);
                        }

                        if (empty($stats['wonAmount']) || empty($stats['goalAmount'])) {
                            $stats['wonGoalAmountCompletion'] = 0;
                        } else {
                            $stats['wonGoalAmountCompletion'] = round((($stats['wonAmount'] / ($stats['goalAmount'])) * 100),
                                2);
                        }
                    }
                }
            } else {
                $stats['openGoalCountCompletion'] = 0;
                $stats['wonGoalCountCompletion'] = 0;
                $stats['goalCount'] = 0;
                if (!empty($singleConfigData['goalsOpportunityConfigData'])) {
                    foreach ($singleConfigData['goalsOpportunityConfigData'] as $goalOppConfig) {
                        $stats['goalCount'] += $goalOppConfig['opportunity_count']* $filterPeriodMonths;
                    }

                    if (!empty($stats['goalCount'])) {
                        if (empty($stats['openCount']) || empty($stats['goalCount'])) {
                            $stats['openGoalCountCompletion'] = 0;
                        } else {
                            $stats['openGoalCountCompletion'] = round((($stats['openCount'] / ($stats['goalCount'])) * 100),
                                2);
                        }

                        if (empty($stats['wonCount']) || empty($stats['goalCount'])) {
                            $stats['wonGoalCountCompletion'] = 0;
                        } else {
                            $stats['wonGoalCountCompletion'] = round((($stats['wonCount'] / ($stats['goalCount'])) * 100),
                                2);
                        }
                    }
                }
            }
        }

        return $stats;
    }



    /**
     * Calculate goal stats
     *
     *
     * @param array $stats stats data
     * @param int $userId user id
     * @param string $filterPeriodMonths user id
     * @param string $filterDateFrom filter date from
     * @param string $filterDateTo filter date to
     * @return array
     */
    public static function calculateGoalStats($stats, $filterPeriodMonths, $filterDateFrom, $filterDateTo, $userId = 0)
    {
        $goalsConfigData = Leads_Mapper_LeadsOwnerGoalsMapper::getInstance()->getMainConfigData($userId);
        if (empty($goalsConfigData)) {
            return $stats;
        }

        if (empty($userId)) {
            $configData = array(
                'quotes_amount' => 0,
                'meetings_held' => 0,
                'calls_placed' => 0,
                'emails_sent' => 0,
                'quote_objective' => 0
            );
            foreach ($goalsConfigData as $singleConfigData) {
                $configData['quotes_amount'] += $singleConfigData['quotes_amount'];
                $configData['meetings_held'] += $singleConfigData['meetings_held'];
                $configData['calls_placed'] += $singleConfigData['calls_placed'];
                $configData['emails_sent'] += $singleConfigData['emails_sent'];
                $configData['quote_objective'] += $singleConfigData['quote_objective'];
            }

            $stats = self::calculateSingleUserStat($stats, $configData, $filterDateFrom, $filterDateTo,
                $filterPeriodMonths);
        } else {
            foreach ($goalsConfigData as $singleConfigData) {
                $stats = self::calculateSingleUserStat($stats, $singleConfigData, $filterDateFrom, $filterDateTo,
                    $filterPeriodMonths);
            }
        }

        return $stats;
    }


    public static function calculateSingleUserStat($stats, $singleConfigData, $filterDateFrom, $filterDateTo, $filterPeriodMonths)
    {

        if ($filterPeriodMonths != '0') {
            $stats['callsPlaced']['goal'] = $singleConfigData['calls_placed']*$filterPeriodMonths;
            if (empty($stats['callsPlaced']['quantity']) || empty($singleConfigData['calls_placed'])) {
                $stats['callsPlaced']['goalCompletion'] = 0;
            } else {
                $stats['callsPlaced']['goalCompletion'] = round(($stats['callsPlaced']['quantity'] / ($singleConfigData['calls_placed'] * $filterPeriodMonths) * 100),
                    2);
            }

            $stats['emailsSent']['goal'] = $singleConfigData['emails_sent']*$filterPeriodMonths;
            if (empty($stats['emailsSent']['quantity']) || empty($singleConfigData['emails_sent'])) {
                $stats['emailsSent']['goalCompletion'] = 0;
            } else {
                $stats['emailsSent']['goalCompletion'] = round(($stats['emailsSent']['manual']/ ($singleConfigData['emails_sent']*$filterPeriodMonths)*100), 2);
            }

            $stats['meetingsHeld']['goal'] = $singleConfigData['meetings_held']*$filterPeriodMonths;
            if (empty($stats['meetingsHeld']['quantity']) || empty($singleConfigData['meetings_held'])) {
                $stats['meetingsHeld']['goalCompletion'] = 0;
            } else {
                $stats['meetingsHeld']['goalCompletion'] = round(($stats['meetingsHeld']['quantity'] / ($singleConfigData['meetings_held'] * $filterPeriodMonths) * 100),
                    2);
            }
        } else {
            $date1 = date_create($filterDateTo);
            $date2 = date_create($filterDateFrom);
            $dateDiff = date_diff($date1, $date2);
            $numberOfDays = $dateDiff->days;

            if ($numberOfDays === 0) {
                $numberOfDays = 1;
            }

            if ($numberOfDays <= 30) {
                $filterPeriodMonths = $numberOfDays/30;
            } else {
                $filterPeriodMonths = $numberOfDays/30;
            }

            $stats['callsPlaced']['goal'] = $singleConfigData['calls_placed'];
            if (empty($stats['callsPlaced']['quantity']) || empty($stats['calls_placed']['quantity'])) {
                $stats['callsPlaced']['goalCompletion'] = 0;
            } else {
                $stats['callsPlaced']['goalCompletion'] = round(($stats['callsPlaced']['quantity']/ ($singleConfigData['calls_placed']*$filterPeriodMonths)*100), 2);
            }

            $stats['emailsSent']['goal'] = $singleConfigData['emails_sent'];
            if (empty($stats['emailsSent']['quantity']) || empty($stats['emails_sent']['quantity'])) {
                $stats['emailsSent']['goalCompletion'] = 0;
            } else {
                $stats['emailsSent']['goalCompletion'] = round(($stats['emailsSent']['quantity']/ ($singleConfigData['emails_sent']*$filterPeriodMonths)*100), 2);
            }

            $stats['meetingsHeld']['goal'] = $singleConfigData['meetings_held'];

            if (empty($stats['meetingsHeld']['quantity']) || empty($stats['meetings_held']['quantity'])) {
                $stats['meetingsHeld']['goalCompletion'] = 0;
            } else {
                $stats['meetingsHeld']['goalCompletion'] = round(($stats['meetingsHeld']['quantity']/ ($singleConfigData['meetings_held']*$filterPeriodMonths)*100), 2);
            }


        }

        return $stats;
    }

    /**
     *
     *
     * @param string $filterDateFrom filter from date
     * @param string $filterDateTo filter to date
     * @param int $userId user id
     * @param string $type
     * @param string $subType won|open
     * @param int $oppTypeId opportunity type id
     * @return int
     */
    public static function calculateSalesRank($filterDateFrom, $filterDateTo, $userId, $type = self::LEAD_STAT_TYPE_OPP_AMOUNT, $subType = self::LEAD_STAT_TYPE_SUB_TYPE_WON, $oppTypeId = 0)
    {
        $rank = 1;

        $excludeStatuses = array();
        if ($subType === self::LEAD_STAT_TYPE_SUB_TYPE_WON) {
            $excludeStatuses = array('1', '10', '20', '30', '40', '50', '60', '70', '80', '90');
        }

        if ($subType === self::LEAD_STAT_TYPE_SUB_TYPE_OPEN) {
            $excludeStatuses = array('1', '100');
        }

        $leadReportsMapper = Leads_Mapper_LeadsReportsMapper::getInstance();
        $leadReportsFilterStartDate = Tools_System_Tools::convertDateFromTimezone($filterDateFrom . ' 00:00:00',
            false, 'UTC', 'Y-m-d H:i:s');
        $leadReportsFilterEndDate = Tools_System_Tools::convertDateFromTimezone($filterDateTo . ' 23:59:59',
            false, 'UTC', 'Y-m-d H:i:s');

        $yourOpportunitiesSalesData = $leadReportsMapper->getOpportunitiesStageAmount($leadReportsFilterStartDate, $leadReportsFilterEndDate, $excludeStatuses, true, $userId, $oppTypeId);

        $leadOwners = Leads_Mapper_LeadsMapper::getInstance()->getLeadOwnersFullList($userId, array(), true);
        if (empty($leadOwners)) {
            return 1;
        }

        $yourAmount = 0;

        if (!empty($yourOpportunitiesSalesData)) {
            if ($type === self::LEAD_STAT_TYPE_OPP_AMOUNT) {
                if ($subType === self::LEAD_STAT_TYPE_SUB_TYPE_WON) {
                    $yourAmount = $yourOpportunitiesSalesData['100']['amount'];
                }

                if ($subType === self::LEAD_STAT_TYPE_SUB_TYPE_OPEN) {
                    foreach ($yourOpportunitiesSalesData as $yourOpportunitiesSalesDat) {
                        $yourAmount += $yourOpportunitiesSalesDat['amount'];
                    }
                }
            } else {
                if ($subType === self::LEAD_STAT_TYPE_SUB_TYPE_WON) {
                    $yourAmount = $yourOpportunitiesSalesData['100']['count'];
                }

                if ($subType === self::LEAD_STAT_TYPE_SUB_TYPE_OPEN) {
                    foreach ($yourOpportunitiesSalesData as $yourOpportunitiesSalesDat) {
                        $yourAmount += $yourOpportunitiesSalesDat['count'];
                    }
                }
            }
        }

        $allAdminsAmount = array();

        foreach ($leadOwners as $leadOwner) {
            $opportunitiesSalesData = $leadReportsMapper->getOpportunitiesStageAmount($leadReportsFilterStartDate, $leadReportsFilterEndDate, $excludeStatuses, true, $leadOwner['id'], $oppTypeId);
            if (!empty($opportunitiesSalesData)) {
                if ($type === self::LEAD_STAT_TYPE_OPP_AMOUNT) {
                    if ($subType === self::LEAD_STAT_TYPE_SUB_TYPE_WON) {
                        if ($opportunitiesSalesData['100']['amount'] > 0) {
                            $allAdminsAmount[] = $opportunitiesSalesData['100']['amount'];
                        }
                    }

                    if ($subType === self::LEAD_STAT_TYPE_SUB_TYPE_OPEN) {
                        $amount = 0;
                        foreach ($opportunitiesSalesData as $yourOpportunitiesSalesDat) {
                            $amount += $yourOpportunitiesSalesDat['amount'];
                        }

                        if ($amount > 0) {
                            $allAdminsAmount[] = $amount;
                        }
                    }
                } else {
                    if ($subType === self::LEAD_STAT_TYPE_SUB_TYPE_WON) {
                        if ($opportunitiesSalesData['100']['count'] > 0) {
                            $allAdminsAmount[] = $opportunitiesSalesData['100']['count'];
                        }
                    }
                    if ($subType === self::LEAD_STAT_TYPE_SUB_TYPE_OPEN) {
                        $count = 0;
                        foreach ($opportunitiesSalesData as $yourOpportunitiesSalesDat) {
                            $count += $yourOpportunitiesSalesDat['count'];
                        }

                        if ($count > 0) {
                            $allAdminsAmount[] = $count;
                        }
                    }
                }
            }
        }

        if (empty($allAdminsAmount)) {
            return 1;
        }

        $result = array_filter(
            $allAdminsAmount,
            function ($value) use($yourAmount) {
                return ($value >= $yourAmount);
            }
        );

        return $rank+count($result);

    }

}