<?php

/**
 * Class Lead_LeadFormTools
 */
class Tools_LeadTasksTools
{

    /**
     *
     * Change task status.
     * If status 1 status will be completed and users will be notified
     *
     * @param int $taskId task id
     * @param string $status task status (0, 1)
     * @param int $closedByUserId task closed by user id
     * @param array $notificationEmails emails to notify
     * @param string $completionMessage completion message
     * @return bool
     */
    public static function changeTaskStatus($taskId, $status, $closedByUserId, $notificationEmails, $completionMessage)
    {
        if (empty($taskId)) {
            return false;
        }

        $leadTaskMapper = Leads_Mapper_LeadsTaskMapper::getInstance();
        $leadTaskMapperLog = Leads_Mapper_LeadsTasksLogMapper::getInstance();
        $leadActionLogMapper = Leads_Mapper_LeadsActionLogMapper::getInstance();
        $leadTaskModel = $leadTaskMapper->find($taskId);

        if (!$leadTaskModel instanceof Leads_Model_LeadsTaskModel) {
            return false;
        }

        $leadTaskLogModel = $leadTaskMapperLog->findByTaskId($taskId, true);
        if (!$leadTaskLogModel instanceof Leads_Model_LeadsTasksLogModel) {
            return false;
        }

        $leadId = $leadTaskModel->getLeadId();

        if (!empty($status)) {
            $leadTaskMapper->closeTask($taskId, $closedByUserId, $completionMessage);
            $leadTaskMapperLog->closeTask($taskId, $closedByUserId, $completionMessage);

            $createdAt = Tools_System_Tools::convertDateFromTimezone('now');
            $leadActionLogDeletedExists = $leadTaskMapperLog->findByTypeTaskId(Leads_Model_LeadsActionLogModel::ACTION_TASK_COMPLETED,
                $taskId);
            if (!empty($leadActionLogDeletedExists)) {
                return false;
            }

            $leadActionLogModel = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_TASK_COMPLETED,
                $leadId,
                Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND, $createdAt,
                Leads_Model_LeadsTypeModel::LEAD_OUTBOUND_TASK,
                Leads_Model_LeadsActionLogModel::EVENT_LOG_EXPIRED);

            $leadTaskLogModel->setId(null);
            $leadTaskLogModel->setActionLogId($leadActionLogModel->getId());
            $leadTaskLogModel->setIsTaskCompleted('1');
            $leadTaskLogModel->setTaskCompletedBy($closedByUserId);
            if (!empty($completionMessage)) {
                $leadTaskLogModel->setCompletionComment($completionMessage);
            }
            $leadTaskMapperLog->save($leadTaskLogModel);

            $leadTaskModel = $leadTaskMapper->find($taskId);

            $leadId = $leadTaskModel->getLeadId();
            $leadModel = Leads_Mapper_LeadsMapper::getInstance()->find($leadId);
            $organizationData = array();
            $leadsData = array();
            if ($leadModel instanceof Leads_Model_LeadsModel) {
                $leadsData = $leadModel->toArray();
                if (!empty($leadsData['organizationId'])) {
                    $organizationModel = Leads_Mapper_LeadsOrganizationMapper::getInstance()->find($leadsData['organizationId']);
                    if ($organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                        $organizationData = $organizationModel->toArray();
                    }
                }
            }

            $leadTaskModel->registerObserver(new Tools_Mail_Watchdog(array(
                'trigger' => Tools_LeadsMailWatchdog::LEAD_TASK_COMPLETED,
                'emailsTo'=> $notificationEmails,
                'leadData' => $leadsData,
                'organizationData' => $organizationData
            )));

            $leadTaskModel->notifyObservers();

        } else {
            $leadTaskMapper->openTask($taskId, $closedByUserId, $completionMessage);
            $leadTaskMapperLog->openTask($taskId, $closedByUserId, $completionMessage);
            $leadActionLogDeletedExists = $leadTaskMapperLog->findByTypeTaskId(Leads_Model_LeadsActionLogModel::ACTION_TASK_COMPLETED,
                $taskId);
            if (!empty($leadActionLogDeletedExists)) {
                $leadActionLogMapper->delete($leadActionLogDeletedExists['action_log_id']);
                $leadTaskMapperLog->deleteByActionLogId($leadActionLogDeletedExists['action_log_id']);

            }

            $taskData = $leadTaskModel->toArray();
            $userIdsToNotify = array($taskData['creatorId'], $taskData['userId']);
            $usersInfo = Leads_Mapper_LeadsMapper::getInstance()->getLeadOwnersList($userIdsToNotify);
            if (!empty($usersInfo)) {
                $taskData['notificationEmails'] = '';
                foreach ($usersInfo as $userInfo) {
                    $taskData['notificationEmails'] .= $userInfo['email'].',';
                }
                $taskData['notificationEmails'] = rtrim($taskData['notificationEmails'], ',');
            }
            
            $leadTaskModel = $leadTaskMapper->find($taskId);
            $leadId = $leadTaskModel->getLeadId();
            $leadModel = Leads_Mapper_LeadsMapper::getInstance()->find($leadId);
            $organizationData = array();
            $leadsData = array();
            if ($leadModel instanceof Leads_Model_LeadsModel) {
                $leadsData = $leadModel->toArray();
                if (!empty($leadsData['organizationId'])) {
                    $organizationModel = Leads_Mapper_LeadsOrganizationMapper::getInstance()->find($leadsData['organizationId']);
                    if ($organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                        $organizationData = $organizationModel->toArray();
                    }
                }
            }

            $leadTaskModel->registerObserver(new Tools_Mail_Watchdog(array(
                'trigger' => Tools_LeadsMailWatchdog::LEAD_TASK_REOPENED,
                'emailsTo'=> explode(',', $taskData['notificationEmails']),
                'leadData' => $leadsData,
                'organizationData' => $organizationData
            )));

            $leadTaskModel->notifyObservers();
        }


        return true;


    }

    /**
     * Mark tasks as overdue
     */
    public static function markTasksAsOverdue()
    {
        $leadsTaskMapper = Leads_Mapper_LeadsTaskMapper::getInstance();
        $leadsTasksLogMapper = Leads_Mapper_LeadsTasksLogMapper::getInstance();
        $leadsTaskMapper->markTasksAsOverdue();
        $leadsTasksLogMapper->markTasksAsOverdue();
    }

    public static function sendStartTaskEmails()
    {
        $leadsTaskMapper = Leads_Mapper_LeadsTaskMapper::getInstance();
        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
        $beforeStartTime = '- 1 hour';
        $beforeStartTime = Tools_System_Tools::convertDateFromTimezone($beforeStartTime);
        $beforeTimeLimit = Tools_System_Tools::convertDateFromTimezone('+ 1 hour');
        $notSentStartEmailsTasks = $leadsTaskMapper->getNotSentTaskStartEmails($beforeTimeLimit, $beforeStartTime);
        $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
        if (!empty($notSentStartEmailsTasks)) {
            foreach ($notSentStartEmailsTasks as $notSentStartEmailsTask) {
                $leadTaskModel = $leadsTaskMapper->find($notSentStartEmailsTask['id']);
                if ($leadTaskModel instanceof Leads_Model_LeadsTaskModel) {
                    $leadModel = $leadsMapper->find($notSentStartEmailsTask['lead_id']);
                    if ($leadModel instanceof Leads_Model_LeadsModel) {
                        $leadsData = $leadModel->toArray();
                        $leadsData['leadlink'] = $websiteHelper->getUrl().'crmLogin/tabName/dashboard/subTab/leads/paramHashName/lead/paramHash/'.$leadModel->getId().'/';
                        $dictionary = array();
                        $organizationData = array();
                        $dictionary = Tools_LeadsMailWatchdog::generateDictionary($dictionary, $leadsData, 'lead');
                        $dictionary = Tools_LeadsMailWatchdog::generateDictionary($dictionary, ['organizationName' => '', 'orglink' => $leadsData['leadlink']], 'organization');
                        if (!empty($leadsData['organizationId'])) {
                            $organizationModel = Leads_Mapper_LeadsOrganizationMapper::getInstance()->find($leadsData['organizationId']);
                            if ($organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                                $organizationData = $organizationModel->toArray();
                                $organizationData['orglink'] = $websiteHelper->getUrl() . 'crmLogin/tabName/dashboard/subTab/organizations/paramHashName/organization/paramHash/' . $organizationModel->getId() . '/';
                                $dictionary = Tools_LeadsMailWatchdog::generateDictionary($dictionary, $organizationData, 'organization');
                            }
                        }

                        $leadTaskModel->registerObserver(new Tools_Mail_Watchdog(array(
                            'trigger' => Tools_LeadsMailWatchdog::LEAD_TASK_START,
                            'leadData' => $leadsData,
                            'organizationData' => $organizationData
                        )));

                        $leadTaskModel->notifyObservers();

                        $leadsTaskMapper->setStatusStartSent($notSentStartEmailsTask['id']);
                    }

                }
            }
        }

    }

    public static function processTaskRemindersEmails()
    {
        $leadTaskQueueMapper = Leads_Mapper_LeadsTaskQueueMapper::getInstance();
        $leadTaskStatusConfigData = Leads_Mapper_LeadsConfigMapper::getInstance()->getConfigParams();
        $leadTaskMapper = Leads_Mapper_LeadsTaskMapper::getInstance();
        if (!empty($leadTaskStatusConfigData['first_task_reminder'])) {
            $taskReminderHours = intval($leadTaskStatusConfigData['first_task_reminder']);
            $remindersListData = $leadTaskMapper->getTasksPreDueDate($taskReminderHours);
            if (!empty($remindersListData)) {
                foreach ($remindersListData as $reminderData) {
                    $taskId = $reminderData['id'];
                    $taskReminderInQueueModel = $leadTaskQueueMapper->findByTaskIdQueueType($taskId, Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_TYPE_FIRST_REMINDER);
                    if (!$taskReminderInQueueModel instanceof Leads_Model_LeadsTaskQueueModel) {
                        $taskReminderInQueueModel = new Leads_Model_LeadsTaskQueueModel();
                        $taskReminderInQueueModel->setStatus(Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_STATUS_PROCESSING);
                        $taskReminderInQueueModel->setQueueType(Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_TYPE_FIRST_REMINDER);
                        $taskReminderInQueueModel->setTaskId($taskId);
                        $taskReminderInQueueModel->setToSendDate(date("Y-m-d H:i:s", strtotime($reminderData['taskExpired'].'- '. $taskReminderHours. 'hour')));
                        $leadTaskQueueMapper->save($taskReminderInQueueModel);
                    }
                }
            }

        }

        if (!empty($leadTaskStatusConfigData['second_task_reminder'])) {
            $taskReminderHours = intval($leadTaskStatusConfigData['second_task_reminder']);
            $remindersListData = $leadTaskMapper->getTasksPreDueDate($taskReminderHours);
            foreach ($remindersListData as $reminderData) {
                $taskId = $reminderData['id'];
                $taskReminderInQueueModel = $leadTaskQueueMapper->findByTaskIdQueueType($taskId, Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_TYPE_SECOND_REMINDER);
                if (!$taskReminderInQueueModel instanceof Leads_Model_LeadsTaskQueueModel) {
                    $taskReminderInQueueModel = new Leads_Model_LeadsTaskQueueModel();
                    $taskReminderInQueueModel->setStatus(Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_STATUS_PROCESSING);
                    $taskReminderInQueueModel->setQueueType(Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_TYPE_SECOND_REMINDER);
                    $taskReminderInQueueModel->setTaskId($taskId);
                    $taskReminderInQueueModel->setToSendDate(date("Y-m-d H:i:s", strtotime($reminderData['taskExpired'].'- '. $taskReminderHours. 'hour')));
                    $leadTaskQueueMapper->save($taskReminderInQueueModel);
                }
            }
        }

        $remindersInQueue =  $leadTaskQueueMapper->getAllQueue(0, array(Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_TYPE_SECOND_REMINDER, Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_TYPE_FIRST_REMINDER));
        if (!empty($remindersInQueue)) {
            foreach ($remindersInQueue as $queueId => $reminder) {
                $leadTaskModel = Leads_Mapper_LeadsTaskMapper::getInstance()->find($reminder['task_id']);
                if ($leadTaskModel instanceof Leads_Model_LeadsTaskModel) {
                    $leadId = $leadTaskModel->getLeadId();
                    $leadModel = Leads_Mapper_LeadsMapper::getInstance()->find($leadId);
                    $organizationData = array();
                    $leadsData = array();
                    if ($leadModel instanceof Leads_Model_LeadsModel) {
                        $leadsData = $leadModel->toArray();
                        if (!empty($leadsData['organizationId'])) {
                            $organizationModel = Leads_Mapper_LeadsOrganizationMapper::getInstance()->find($leadsData['organizationId']);
                            if ($organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                                $organizationData = $organizationModel->toArray();
                            }
                        }
                    }
                    $leadTaskModel->registerObserver(new Tools_Mail_Watchdog(array(
                        'trigger' => Tools_LeadsMailWatchdog::LEADS_TASK_REMINDER,
                        'queueId' => $queueId,
                        'leadData' => $leadsData,
                        'organizationData' => $organizationData
                    )));

                    $leadTaskModel->notifyObservers();
                }
            }
        }

    }

    /**
     * Process overdue tasks emails
     */
    public static function sendOverdueEmails()
    {
        $leadTaskQueueMapper = Leads_Mapper_LeadsTaskQueueMapper::getInstance();
        $leadTaskStatusConfigData = Leads_Mapper_LeadsConfigMapper::getInstance()->getConfigParams();
        $leadTaskMapper = Leads_Mapper_LeadsTaskMapper::getInstance();
        if (!empty($leadTaskStatusConfigData['overdue_reminder'])) {
            $taskReminderHours = intval($leadTaskStatusConfigData['overdue_reminder']);
            $overdueTasks = $leadTaskMapper->getAllOverdueTasks();
            if (!empty($overdueTasks)) {
                foreach ($overdueTasks as $overdueTask) {
                    if ($overdueTask['taskExpired'] === '0000-00-00 00:00:00') {
                        continue;
                    }
                    $taskId = $overdueTask['id'];
                    $taskReminderInQueueModel = $leadTaskQueueMapper->findByTaskIdQueueType($taskId, Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_TYPE_CUSTOM_REMINDER);
                    if (!$taskReminderInQueueModel instanceof Leads_Model_LeadsTaskQueueModel) {
                        $taskReminderInQueueModel = new Leads_Model_LeadsTaskQueueModel();
                        $taskReminderInQueueModel->setStatus(Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_STATUS_PROCESSING);
                        $taskReminderInQueueModel->setQueueType(Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_TYPE_CUSTOM_REMINDER);
                        $taskReminderInQueueModel->setTaskId($taskId);
                        $taskReminderInQueueModel->setToSendDate(date("Y-m-d H:i:s", strtotime($overdueTask['taskExpired'].'+ '. $taskReminderHours. 'days')));
                        $leadTaskQueueMapper->save($taskReminderInQueueModel);
                    } else {
                        if ($taskReminderInQueueModel->getStatus() === Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_STATUS_PROCESSING) {
                            continue;
                        }
                        $previousToSentDate = $taskReminderInQueueModel->getToSendDate();
                        $newToSentDate = date("Y-m-d H:i:s", strtotime($previousToSentDate.'+ '. $taskReminderHours. 'days'));
                        $taskReminderInQueueModel->setStatus(Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_STATUS_PROCESSING);
                        $taskReminderInQueueModel->setToSendDate($newToSentDate);
                        $leadTaskQueueMapper->save($taskReminderInQueueModel);

                    }
                }
            }
        }

        $remindersInQueue =  $leadTaskQueueMapper->getAllQueue(0, array(Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_TYPE_CUSTOM_REMINDER));
        if (!empty($remindersInQueue)) {
            foreach ($remindersInQueue as $queueId => $reminder) {
                $leadTaskModel = Leads_Mapper_LeadsTaskMapper::getInstance()->find($reminder['task_id']);
                if ($leadTaskModel instanceof Leads_Model_LeadsTaskModel) {
                    $leadId = $leadTaskModel->getLeadId();
                    $leadModel = Leads_Mapper_LeadsMapper::getInstance()->find($leadId);
                    $organizationData = array();
                    $leadsData = array();
                    if ($leadModel instanceof Leads_Model_LeadsModel) {
                        $leadsData = $leadModel->toArray();
                        if (!empty($leadsData['organizationId'])) {
                            $organizationModel = Leads_Mapper_LeadsOrganizationMapper::getInstance()->find($leadsData['organizationId']);
                            if ($organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                                $organizationData = $organizationModel->toArray();
                            }
                        }
                    }
                    $leadTaskModel->registerObserver(new Tools_Mail_Watchdog(array(
                        'trigger' => Tools_LeadsMailWatchdog::LEAD_TASK_OVERDUE,
                        'queueId' => $queueId,
                        'leadData' => $leadsData,
                        'organizationData' => $organizationData
                    )));

                    $leadTaskModel->notifyObservers();
                }
            }
        }


    }

}