<?php

/**
 * Class Lead_LeadTools
 */
class Tools_LeadTools
{

    /**
     * Email sequence plugin name
     */
    const SEQUENCE_PLUGIN = 'emailsequence';

    /**
     * Current day log
     */
    const TIMELINE_TYPE_TODAY = 'today';

    /**
     * Upcoming events
     */
    const TIMELINE_TYPE_UPCOMING = 'upcoming';

    /**
     * Defined user prefixes
     * @var array
     */
    public static $userPrefixes = array(
        'Mr',
        'Mrs',
        'Ms',
        'Miss',
        'Dr'
    );


    /**
     * List of fields which use during automatic
     * lead and organization creation
     *
     * @var array
     */
    public static $_formFieldsInUse = array(
        'name',
        'email',
        'lastname',
        'mobile',
        'phone',
        'zip',
        'address',
        'address1',
        'position',
        'prefix',
        'city',
        'country',
        'state',
        'organization',
        'group-name',
        'mobile-country-code',
        'phone-country-code',
        'website'
    );

    /**
     * External emails statuses association
     *
     * @var array
     */
    public static $_externalEmailStatusAssociation = array(
        'click' => Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_CLICK,
        'open' => Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_OPENED,
        'delivered' => Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_SENT,
        'bounce' => Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_BOUNCE,
        'dropped' => Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_DROPPED,
        'deferred' => Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_DEFERRED,
        'unsubscribe' => Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_UNSUBSCRIBED
    );

    public static $_blockedFieldsRules = array(
        'crm_cvv' => 'replace',
        'crm_credit_card' => 'replace_not_full'
    );

    /**
     * Defined demo data
     * @var array
     */
    public static $_demoData = array(
        'firstname'                   => 'John',
        'lastname'                    => 'Doe',
        'middlename'                  => 'Michael',
        'position'                    => 'Client',
        'prefix'                      => 'Mr',
        'fullmobilephone'             => '+10123456789',
        'fulldesktopphone'            => '+19876543210',
        'organizationname'            => 'SeoSamba',
        'organizationwebsite'         => '',
        'organizationlogo'            => 'plugins/leads/web/images/corporate-logo.png',
        'leadownerfullname'           => 'Jim Fuller',
        'leadowneremail'              => 'jim.fuller@example.com',
        'leadownermobilecountrycode'  => 'US',
        'leadownermobilephone'        => '1122334455',
        'leadownerdesktopcountrycode' => 'US',
        'leadownerdesktopphone'       => '5544332211',
        'leadownerfulldesktopphone'   => '+15544332211',
        'leadownerfullmobilephone'    => '+11122334455',
        'leadownersignature'          => 'user signature',
        'id'                          => '1000',
        'source'                      => 'https://www.seosamba.com/',
        'personalcalendarurl'         => 'https://calendar.google.com/calendar/',
        'avatarlink'                  => 'https://www.seotoaster.com/plugins/widcard/system/userdata/CorporateLogo.png'
    );

    /**
     * Return advertising account
     *
     * @var string[]
     */
    public static $_advertisingTypes = array(
        'adwords'  => 'Adwords',
        'facebook' => 'Facebook'
    );

    /**
     * Return ip address
     *
     * @return mixed
     */
    public static function getIpAddress()
    {

        if (!empty($_SERVER['HTTP_CLIENT_IP'])) {
            $ip = $_SERVER['HTTP_CLIENT_IP'];
        } elseif (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
            $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
        } else {
            $ip = $_SERVER['REMOTE_ADDR'];
        }

        return $ip;
    }

    /**
     * Get element value
     *
     * @param array $array any array
     * @param string $name array element name
     * @return mixed
     */
    public static function getValue($array, $name)
    {
        if (!empty($array[$name])) {
            if(is_array($array[$name])){
                return filter_var_array($array[$name], FILTER_SANITIZE_STRING);
            }

            return filter_var($array[$name], FILTER_SANITIZE_STRING);
        }
    }

    /**
     * Log leads actions
     *
     * @param string $type lead action type
     * @param int $leadId lead id
     * @param string $actionType lead action type (inbound, outbound)
     * @param string $createdAt event passed at
     * @param string $leadType lead type
     * @param string $additionalContent additional information
     * @param string $eventType event log type (expired, upcoming)
     * @return Leads_Model_LeadsQuoteLogModel
     */
    public static function logLeadAction(
        $type,
        $leadId,
        $actionType,
        $createdAt,
        $leadType = '',
        $eventType = Leads_Model_LeadsActionLogModel::EVENT_LOG_EXPIRED,
        $additionalContent = ''
    ) {
        $leadActionLogMapper = Leads_Mapper_LeadsActionLogMapper::getInstance();
        $leadsActionLog = new Leads_Model_LeadsActionLogModel();
        $leadsActionLog->setCreatedAt($createdAt);
        $leadsActionLog->setLeadId($leadId);
        $leadsActionLog->setType($type);
        $leadsActionLog->setActionType($actionType);
        $leadsActionLog->setAdditionalContent($additionalContent);
        $leadsActionLog->setEventType($eventType);
        $leadsActionLog->setLeadType($leadType);

        return $leadActionLogMapper->save($leadsActionLog);
    }

    /**
     * Get lead model by email or user id
     *
     * @param string $leadEmail logged user email
     * @param int $userId user id
     * @param string $currentLoggedUserEmail
     * @return null
     */
    public static function getLeadModelByIdAndEmail($userId = null, $currentLoggedUserEmail = '', $leadEmail = '')
    {
        $leadModel = null;
        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
        if (!empty($userId)) {
            $leadModel = $leadsMapper->findByUserId($userId);
        }

        if (!$leadModel instanceof Leads_Model_LeadsModel && !empty($currentLoggedUserEmail)) {
            $leadModel = $leadsMapper->findByEmail($currentLoggedUserEmail);
        }

        if (!$leadModel instanceof Leads_Model_LeadsModel && !empty($leadEmail)) {
            $leadModel = $leadsMapper->findByEmail($leadEmail);
        }

        return $leadModel;

    }

    /**
     * Add ip address for lead
     *
     * @param int $leadId lead id
     * @param string $ip ip address
     * @return Leads_Model_LeadsIpsModel
     */
    public static function addIpAddressToLead($leadId, $ip)
    {
        $leadIpsMapper = Leads_Mapper_LeadsIpsMapper::getInstance();
        $leadIpModel = $leadIpsMapper->getByLeadIdAndIp($leadId, $ip);
        $leadIpQuantity = 1;
        if ($leadIpModel instanceof Leads_Model_LeadsIpsModel) {
            $leadIpQuantity = $leadIpModel->getIpQuantity() + 1;
        }
        $leadIpModel = new Leads_Model_LeadsIpsModel();
        $leadIpModel->setLeadId($leadId);
        $leadIpModel->setLeadIpAddress($ip);
        $leadIpModel->setIpQuantity($leadIpQuantity);

        return $leadIpsMapper->save($leadIpModel);
    }

    /**
     * Get lead type id by lead type name
     *
     * @param string $typeName lead type name
     * @return null
     */
    public static function getLeadTypeIdByLeadTypeName($typeName)
    {
        $leadTypeId = null;
        $leadTypeModel = Leads_Mapper_LeadsTypeMapper::getInstance()->findByTypeName($typeName);
        if ($leadTypeModel instanceof Leads_Model_LeadsTypeModel) {
            $leadTypeId = $leadTypeModel->getId();
        }

        return $leadTypeId;
    }

    /**
     * Get referer url
     *
     * @return string
     */
    public static function getRefererUrl()
    {
        $session = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
        if (!empty($session->refererLeadsUrl)) {
            return $session->refererLeadsUrl;
        }

        return '';
    }

    /**
     * Get utm tags string components
     *
     * @param bool $parsed
     * @return string
     */
    public static function getUtmTagsString($parsed = true)
    {
        $refererUrl = self::getRefererUrl();
        if (empty($refererUrl)) {
            return '';
        }
        $session = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
        if ($parsed === true && !empty($session->refererLeadsGetParams)) {
            $urlParams = $session->refererLeadsGetParams;
            if (!empty($urlParams)) {
                if (!empty($urlParams['utm_source'])) {
                    $finalUtmStringResult = 'Source: ' . $urlParams['utm_source'];
                    if (!empty($urlParams['utm_medium'])) {
                        $finalUtmStringResult .= ' Medium: ' . $urlParams['utm_medium'];
                    }
                    if (!empty($urlParams['utm_campaign'])) {
                        $finalUtmStringResult .= ' Campaign: ' . $urlParams['utm_campaign'];
                    }
                    if (!empty($urlParams['utm_term'])) {
                        $finalUtmStringResult .= ' Term: ' . $urlParams['utm_term'];
                    }
                    if (!empty($urlParams['utm_content'])) {
                        $finalUtmStringResult .= ' Content: ' . $urlParams['utm_content'];
                    }

                    return $finalUtmStringResult;
                }
            }
        }

        return '';

    }

    /**
     * Prepare utm tags screen
     *
     * @param array $params
     * @return string
     */
    public static function prepareUtmTagsString($params)
    {
        if (!empty($params)) {
            if (!empty($params['utm_source'])) {
                $finalUtmStringResult = 'Source: ' . $params['utm_source'];
                if (!empty($params['utm_medium'])) {
                    $finalUtmStringResult .= ' Medium: ' . $params['utm_medium'];
                }
                if (!empty($params['utm_campaign'])) {
                    $finalUtmStringResult .= ' Campaign: ' . $params['utm_campaign'];
                }
                if (!empty($params['utm_term'])) {
                    $finalUtmStringResult .= ' Term: ' . $params['utm_term'];
                }
                if (!empty($params['utm_content'])) {
                    $finalUtmStringResult .= ' Content: ' . $params['utm_content'];
                }

                return $finalUtmStringResult;
            }
        }
    }

    /**
     * Get attribute value for logged user
     *
     * @param string $attributeName user attribute name
     * @param int $userId user id
     * @return mixed
     */
    public static function getLoggedUserAttribute($attributeName, $userId = 0)
    {
        $sessionHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
        if (empty($userId)) {
            $userId = $sessionHelper->getCurrentUser()->getId();
        }
        $userMapper = Application_Model_Mappers_UserMapper::getInstance();
        $where = $userMapper->getDbTable()->getAdapter()->quoteInto('attribute = ?', $attributeName);
        $where .= ' AND ' . $userMapper->getDbTable()->getAdapter()->quoteInto('user_id = ?', $userId);
        $select = $userMapper->getDbTable()->getAdapter()->select()->from('user_attributes',
            'value')->where($where);

        return $userMapper->getDbTable()->getAdapter()->fetchRow($select);
    }


    /**
     * Method returns list of State/Province/Region for given countries
     * @static
     * @param string $country official ISO code of country
     * @param bool $pairs If true returns array of id => name
     * @param bool $stateCodeAsKey If true returns array of state => name
     * @param string $secondValue key pair value
     * @return array|null array with list of states or null if given country doesn't have any
     */
    public static function getStates($country = null, $pairs = false, $stateCodeAsKey = false, $secondValue = 'name')
    {
        $stateTable = new Zend_Db_Table('shopping_list_state');

        $where = null;
        if (!empty($country)) {
            $where = $stateTable->getAdapter()->quoteInto('country = ?', $country);
        }
        if ($pairs) {
            if ($stateCodeAsKey === true) {
                $select = $stateTable->select()->from($stateTable, array('state', $secondValue));
            } else {
                $select = $stateTable->select()->from($stateTable, array('id', $secondValue));
            }
            if ($where) {
                $select->where($where);
            }
            $data = $stateTable->getAdapter()->fetchPairs($select);
        } else {
            $select = $stateTable->select()->from($stateTable);
            if ($where) {
                $select->where($where);
            }
            $data = $stateTable->getAdapter()->fetchAll($select);
        }

        return $data;
    }

    /**
     * get country info by country code
     *
     * @param string $countryCode country code ISO Alpha-2
     * @return array
     */
    public static function getCountryByCode($countryCode)
    {
        $countryTable = new Models_DbTable_Country();
        $where = $countryTable->getAdapter()->quoteInto('country = ?', $countryCode);

        return $countryTable->fetchRow($where)->toArray();
    }

    /**
     * get country phone codes list
     *
     * @param bool $withCountryCode flag to use country code
     * @param array $intersect
     * @return array
     */
    public static function getCountryPhoneCodesList($withCountryCode = true, $intersect = array())
    {
        $phoneCodes = Zend_Locale::getTranslationList('phoneToTerritory');
        $countryCodes = Zend_Locale::getTranslationList('Territory');
        array_shift($phoneCodes);
        if (!empty($intersect)) {
            $phoneCodes = array_intersect_key($phoneCodes, array_flip($intersect));
        }
        array_walk($phoneCodes, function (&$item, $key) use ($withCountryCode, $countryCodes) {
            $item = ($withCountryCode) ? '+' . $item . ' ' . $countryCodes[$key] : '+' . $item;
        });

        return $phoneCodes;

    }

    /**
     * Get domain name from url
     *
     * @param string $url url
     * @return string
     */
    public static function getDomainFromUrl($url)
    {
        $urlParts = parse_url($url);
        if (!empty($urlParts['host'])) {
            if (preg_match('/(?<domain>[a-z0-9\-]{1,63}\.[a-z\.]{2,6})$/i', $urlParts['host'], $matches)) {
                if (!empty($matches['domain'])) {
                    return $matches['domain'];
                }

            }
        }

        return '';

    }

    /**
     * Prepare website url
     *
     * @param string $websiteUrl website url
     * @return string
     */
    public static function prepareWebsiteUrl($websiteUrl)
    {
        if (!preg_match('~(http|https:\/\/)~', $websiteUrl)) {
            $websiteUrl = 'http://'.$websiteUrl;
        }

        return $websiteUrl;
    }

    /**
     * Remove all non digital symbols
     *
     * @param string $phoneAddress any phone number
     * @return mixed
     */
    public static function cleanPhoneAddress($phoneAddress)
    {
        return preg_replace('~[^\d]~ui', '', $phoneAddress);
    }

    /**
     * @param string $stateAbbr state ISO code
     * @return array
     */
    public static function findStateByStateAbbr($stateAbbr)
    {
        $stateDbTable = new Models_DbTable_State();
        $where = $stateDbTable->getAdapter()->quoteInto('state = ?', $stateAbbr);
        $select = $stateDbTable->getAdapter()->select()->from('shopping_list_state')->where($where);

        return $stateDbTable->getAdapter()->fetchRow($select);
    }

    /**
     * @param $where
     * @return string
     */
    public static function addAndToStatement($where)
    {
        if (!empty($where)) {
            $where .= ' AND ';
        }

        return $where;
    }

    /**
     * Get either a Gravatar URL or complete image tag for a specified email address.
     *
     * @param string $email The email address
     * @param int $size Size in pixels, defaults to 80px [ 1 - 2048 ]
     * @param string $dimensions Default imageset to use [ 404 | mm | identicon | monsterid | wavatar ]
     * @param string $rating Maximum rating (inclusive) [ g | pg | r | x ]
     * @param bool $img True to return a complete IMG tag False for just the URL
     * @param array $attrs Optional, additional key/value attributes to include in the IMG tag
     * @return String containing either just a URL or a complete image tag
     * @source https://gravatar.com/site/implement/images/php/
     */
    public static function getGravatar(
        $email,
        $size = 80,
        $dimensions = 'mm',
        $rating = 'g',
        $img = false,
        $attrs = array()
    ) {
        $url = 'https://www.gravatar.com/avatar/' . md5(strtolower(trim($email))) . "?s=$size&d=$dimensions&r=$rating";
        if ($img) {
            $url = '<img src="' . $url . '"';
            foreach ($attrs as $key => $val) {
                $url .= ' ' . $key . '="' . $val . '"';
            }
            $url .= ' />';
        }

        return $url;
    }

    /**
     * Check if exists lead with such phone or mobile number
     *
     * @param int $number phone or mobile number
     * @param string $countryCode country code
     * @return null
     */
    public static function checkLeadMobileOrPhoneNumber($number, $countryCode = '')
    {
        $leadPhoneCountryCodeValue = '';
        if (!empty($countryCode)) {
            $phoneCountryPhoneCode = Zend_Locale::getTranslation($countryCode, 'phoneToTerritory');
            $leadPhoneCountryCodeValue = '+' . $phoneCountryPhoneCode;
        }

        return self::findLeadByPhoneMobileNumber($leadPhoneCountryCodeValue . $number, true);
    }

    /**
     * Get country code value
     *
     * @param string $countryCode country code
     * @return string
     */
    public static function getCountryCodeValue($countryCode)
    {
        $countryPhoneCode = Zend_Locale::getTranslation($countryCode, 'phoneToTerritory');
        $countryCodeValue = '+' . $countryPhoneCode;
        return $countryCodeValue;
    }

    /**
     * Add ip to mojo.seosamba.com
     *
     * @param Leads_Model_LeadsIpsModel $leadsIpsModel
     */
    public static function addIpToMojo(Leads_Model_LeadsIpsModel $leadsIpsModel)
    {
        $ipQuantity = $leadsIpsModel->getIpQuantity();
        $params = array('visitorIp' => $leadsIpsModel->getLeadIpAddress());
        if ($ipQuantity == '1') {
            $leadId = $leadsIpsModel->getLeadId();
            $ip = $params['visitorIp'];
            $leadsIpsModel = Leads_Mapper_LeadsIpsMapper::getInstance()->getByIpExcludeLeadId($leadId, $ip);
            if ($leadsIpsModel instanceof Leads_Model_LeadsIpsModel) {
                $params['flag'] = 'force';
            }
            Apps::apiCall('POST', 'sambastats', '', $params);
        }

    }

    /**
     * Get email sequence data key, sequence_name
     *
     * @return array
     */
    public static function getEmailSequenceList()
    {
        $sequenceData = array();
        $pluginExistsAndEnabled = self::verifyPluginEnabled(self::SEQUENCE_PLUGIN);
        if ($pluginExistsAndEnabled === true) {
            $emailSequenceMapper = Emailsequence_Models_Mapper_EmailSequenceSettingsMapper::getInstance();
            $where = $emailSequenceMapper->getDbTable()->getAdapter()->quoteInto('archive = ?', '0');
            $sequenceData = $emailSequenceMapper->getEmailSequenceList($where,
                'sequence_name', null, null, true);
            if (empty($sequenceData)) {
                $sequenceData = array();
            }
        }

        return $sequenceData;
    }

    /**
     * Get sms/email sequence list
     * @param array $order order by
     * @return array
     */
    public static function getEmailSequenceListData($order = array(), $archive = false)
    {
        $sequenceData = array();
        $pluginExistsAndEnabled = self::verifyPluginEnabled(self::SEQUENCE_PLUGIN);
        if ($pluginExistsAndEnabled === true) {
            $emailSequenceMapper = Emailsequence_Models_Mapper_EmailSequenceSettingsMapper::getInstance();

            $where = $emailSequenceMapper->getDbTable()->getAdapter()->quoteInto('archive = ?', '0');
            if(!empty($archive)) {
                $where = $emailSequenceMapper->getDbTable()->getAdapter()->quoteInto('archive = ?', '1');
            }

            $select = $emailSequenceMapper->getDbTable()->getAdapter()->select()->from('plugin_emailsequence_settings', array('id', 'sequence_name'))->where($where);

            if (!empty($order)) {
                $select->order($order);
            }

            $sequenceData = $emailSequenceMapper->getDbTable()->getAdapter()->fetchPairs($select);
            if (empty($sequenceData)) {
                $sequenceData = array();
            }
        }

        return $sequenceData;
    }

    /**
     * check plugin status
     *
     * @param string $pluginName plugin name
     * @return bool
     */
    public static function verifyPluginEnabled($pluginName)
    {
        $pluginExistsAndEnabled = false;
        $emailSequencePlugin = Application_Model_Mappers_PluginMapper::getInstance()->findByName($pluginName);
        if ($emailSequencePlugin instanceof Application_Model_Models_Plugin) {
            $emailSequencePluginStatus = $emailSequencePlugin->getStatus();
            if ($emailSequencePluginStatus === 'enabled') {
                $pluginExistsAndEnabled = true;
            }
        }

        return $pluginExistsAndEnabled;
    }

    /**
     * Get currency format
     *
     * @throws Zend_Exception
     */
    public static function getCurrencyFormat()
    {
        $currency = Zend_Registry::get('Zend_Currency');
        $moneyFormat = Tools_Misc::getCurrencyFormat();
        $moneyFormat['precision'] = 0;
        $moneyFormat['symbol'] =  preg_replace('~[\w]~', '', $currency->getSymbol());
        $moneyFormat['format'] =  '%s%v';
        $moneyFormat['thousand'] =  ',';
        $moneyFormat['decimal'] =  '.';

        return $moneyFormat;
    }

    /**
     * @param $data
     * @param $optionValues
     * @param $type
     * @param $customParamId
     */
    public static function processCustomParamsOptions($data, $optionValues, $type, $customParamId)
    {
        if($type === Leads::LEADS_TYPE){
            $customParamOptionsDataMapper = Leads_Mapper_LeadsCustomParamsOptionsDataMapper::getInstance();
            $leadCustomParamMapper = Leads_Mapper_LeadsCustomParamsMapper::getInstance();
        }elseif($type === Leads::ORGANIZATIONS_TYPE){
            $customParamOptionsDataMapper = Leads_Mapper_LeadsOrganizationCustomParamsOptionsDataMapper::getInstance();
            $leadCustomParamMapper = Leads_Mapper_LeadsOrganizationCustomParamsMapper::getInstance();
        }else{
            return;
        }

        if (empty($data)) {
            if (!empty($optionValues)) {
               self::_saveNewOptionsSelections($optionValues, $type, $customParamOptionsDataMapper, $customParamId);
            }
        } else {
            foreach ($data as $key => $optionsDataModel) {
                $optionDataId = $optionsDataModel->getId();
                $leadCustomParamMapper->selectUpdateOptionsValues($optionDataId, $optionValues[$key]);

                $optionsDataModel->setOptionValue($optionValues[$key]);
                $customParamOptionsDataMapper->save($optionsDataModel);
            }

            $newElSelection = array_diff_key($optionValues, $data);
            if (!empty($newElSelection)) {
                self::_saveNewOptionsSelections($newElSelection, $type, $customParamOptionsDataMapper, $customParamId);
            }
        }
        return;
    }

    /**
     * @param $optionValues
     * @param $type
     * @param $customParamOptionsDataMapper
     * @param $customParamId
     */
    private static function _saveNewOptionsSelections($optionValues, $type, $customParamOptionsDataMapper, $customParamId)
    {
        foreach ($optionValues as $key => $value) {
            if($type === Leads::LEADS_TYPE){
                $customParamOptionsDataModel = new Leads_Model_LeadsCustomParamsOptionsDataModel();
            }else{
                $customParamOptionsDataModel = new Leads_Model_LeadsOrganizationCustomParamsOptionsDataModel();
            }

            $customParamOptionsDataModel->setCustomParamId($customParamId);
            $customParamOptionsDataModel->setOptionValue($value);

            $customParamOptionsDataMapper->save($customParamOptionsDataModel);
        }
    }

    /**
     * Process organization
     *
     * @param string $organizationName organization name
     * @param bool $withoutDefaultSettings without default settings
     * @return Leads_Model_LeadsOrganizationModel|null
     */
    public static function processOrganization($organizationName, $withoutDefaultSettings = false)
    {
        $organizationModel = null;
        $leadOrganizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
        $organizationModel = $leadOrganizationMapper->findByName($organizationName);
        if (!$organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
            $organizationModel = new Leads_Model_LeadsOrganizationModel();
            $organizationModel->setOrganizationName($organizationName);
            $createdAt = Tools_System_Tools::convertDateFromTimezone('now');
            $organizationModel->setCreatedAt($createdAt);
            $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
            $userDefaultMobileCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');
            if (!empty($userDefaultMobileCountryCode) && $withoutDefaultSettings === false) {
                $organizationModel->setOrganizationMobileCountryCode($userDefaultMobileCountryCode);
                $organizationModel->setOrganizationPhoneCountryCode($userDefaultMobileCountryCode);
            }

            $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();
            if (!empty($shoppingConfig['country']) && $withoutDefaultSettings === false) {
                $defaultCountryCode = $shoppingConfig['country'];
                $organizationModel->setOrganizationCountryCode($defaultCountryCode);
            }
            $organizationModel = $leadOrganizationMapper->save($organizationModel);
        }

        return $organizationModel;
    }

    /**
     * Get primary contact status
     *
     * @param string $organizationName organization name
     * @return bool
     */
    public static function getOrganizationPrimaryContactStatus($organizationName)
    {
        $assignPrimaryContactId = false;
        $leadOrganizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
        $organizationModel = $leadOrganizationMapper->findByName($organizationName);
        if (!$organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
            $assignPrimaryContactId = true;
        }

        return $assignPrimaryContactId;
    }

    /**
     * Clean form data fields
     *
     * @param array $data form data
     * @return mixed
     */
    public static function cleanFormData($data)
    {
        if (!empty($data['formName'])) {
            $formName = $data['formName'];
            $formModel = Application_Model_Mappers_FormMapper::getInstance()->findByName($formName);
            if ($formModel instanceof Application_Model_Models_Form) {
                unset($data[md5($formName . $formModel->getId())]);
            }
        }

        $data = self::processBlockedFields($data);

        unset($data['controller']);
        unset($data['action']);
        unset($data['module']);
        unset($data['formName']);
        unset($data['captcha']);
        unset($data['captchaId']);
        unset($data['recaptcha']);
        unset($data['recaptcha_challenge_field']);
        unset($data['recaptcha_response_field']);
        unset($data['formPageId']);
        unset($data['submit']);
        unset($data['uploadLimitSize']);
        unset($data['g-recaptcha-response']);
        unset($data['run']);
        unset($data['handle']);
        unset($data['pid']);
        if (isset($data['conversionPageUrl'])) {
            unset($data['conversionPageUrl']);
        }
        return $data;
    }

    /**
     * TODO extend functional
     *
     * Process blocked fields and apply rules
     *
     * @param $data
     * @return mixed
     */
    public static function processBlockedFields($data)
    {
        foreach ($data as $key => $val) {
            if (array_key_exists($key, self::$_blockedFieldsRules)) {
                if (self::$_blockedFieldsRules[$key] === 'replace') {
                    $data[$key] = preg_replace('~.*~', '*', $val);
                }
                if (self::$_blockedFieldsRules[$key] === 'replace_not_full') {
                    $data[$key] = str_repeat('*', strlen($val) - 4) . substr($val, -4);
                }

            }
        }

        return $data;
    }

    /**
     * Prepare form fields info in html format
     *
     * @param array $formDetails form data
     * @return string
     */
    public static function prepareFormDetailsHtml(array $formDetails)
    {
        $formDetailsHtml = '';
        foreach ($formDetails as $name => $value) {
            if (!$value) {
                continue;
            }
            $formDetailsHtml .= $name . ': ' . (is_array($value) ? implode(', ', $value) : $value) . '<br />';
        }

        return $formDetailsHtml;
    }

    /**
     * Clean form fields from leads and organization fields which already in use
     *
     * @param array $formFields form fields to clean
     * @return array
     */
    public static function removeUsedFormFields(array $formFields)
    {
        $cleanedFormFields = array_diff_key($formFields, array_flip(self::$_formFieldsInUse));

        return $cleanedFormFields;
    }

    /**
     * @param array $formFields form fields to clean
     * @param array $fieldsToRemove fields to remove
     * @return array
     */
    public static function removeFields(array $formFields, $fieldsToRemove)
    {
        $cleanedFormFields = array_diff_key($formFields, $fieldsToRemove);

        return $cleanedFormFields;
    }

    /**
     * Process new note
     *
     * @param int $leadId lead id
     * @param string $notesContent note content
     * @param int $userId user id
     */
    public static function processNotes($leadId, $notesContent, $userId = 0)
    {
        $leadsNotesMapper = Leads_Mapper_LeadsNoteMapper::getInstance();
        $leadsNoteModel = new Leads_Model_LeadsNoteModel();
        $leadsNoteModel->setLeadId($leadId);
        $leadsNoteModel->setNote($notesContent);
        $leadsNoteModel->setDateCreated(Tools_System_Tools::convertDateFromTimezone('now'));
        if (empty($userId)) {
            $superAdminUserModel = Application_Model_Mappers_UserMapper::getInstance()->findByRole('superadmin');
            $userId = $superAdminUserModel->getId();
        }

        $leadsNoteModel->setUserId($userId);

        return $leadsNotesMapper->save($leadsNoteModel);
    }

    /**
     * @param $filters
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $preParseParams pre-parse params
     * @param bool $withTimezoneOffset without timezone offset
     * @param bool $searchFullNumber search full number
     * @return array
     */
    public static function getSearchOrganizationFilterData($filters , $limit = null, $offset = null, $preParseParams = true, $withTimezoneOffset = true, $searchFullNumber = false, $existMobileNumber = false, $organizationLeads = false){

        $additionalOrganizationParamsForSelect = array();
        $withOpportunityAmount = false;
        $withSpentPurchaseAmount = false;
        $withNextActivityDate = false;
        $having = null;
        $withLeadsData = false;
        $sortOrder = 'plo.created_at DESC';
        $includesCustomOrganizationFields = false;

        if ($preParseParams === true) {
            parse_str($filters, $filterParams);
        } else {
            $filterParams = $filters;
        }

        $restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
        if (!empty($restrictedUserId)) {
            $filterParams['owner_id'] = $restrictedUserId;
        }

        $leadOrganizationDataMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
        $leadDataMapper = Leads_Mapper_LeadsMapper::getInstance();

        $where = null;
        if (!empty($filterParams)) {
            $filterParams['lead-organization-attribute'] = str_replace('*-amp-*', '&', $filterParams['lead-organization-attribute']);
            $where = '';
            if (!empty($filterParams['lead-organization-attribute'])) {
                $filterParams['lead-organization-attribute'] = trim($filterParams['lead-organization-attribute'], " \t\n\r\x0B");

                $organizationPhone = Tools_LeadTools::cleanPhoneAddress($filterParams['lead-organization-attribute']);
                $organizationPhoneIsNumber = mb_substr(preg_replace('~[^0-9A-Za-z]~ui', '', $filterParams['lead-organization-attribute']), 0, 1);
                if (strpos($filterParams['lead-organization-attribute'], '*') !== false) {
                    $attributeValue = str_replace('*', '%', $filterParams['lead-organization-attribute']);
                    $where .= ' (' . $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_name LIKE ?',
                            $attributeValue . '%');
                    if(!empty($organizationPhone) && is_numeric($organizationPhoneIsNumber)) {
                        $where .= ' OR ' . $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_phone LIKE ?',
                                $organizationPhone . '%');
                        $where .= ' OR ' . $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_mobile LIKE ?',
                                $organizationPhone . '%');
                    }
                } else {
                    $where .= ' (' . $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_name LIKE ?',
                            '%' . $filterParams['lead-organization-attribute'] . '%');
                    if(!empty($organizationPhone) && is_numeric($organizationPhoneIsNumber)) {
                        $where .= ' OR ' . $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_phone LIKE ?',
                                $organizationPhone . '%');
                        $where .= ' OR ' . $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_mobile LIKE ?',
                                $organizationPhone . '%');
                    }
                }

                if (!empty($filterParams['search_includes_notes'])) {
                    if (strpos($filterParams['lead-organization-attribute'], '*') !== false) {
                        $attributeValue = str_replace('*', '%', $filterParams['lead-organization-attribute']);
                        $where .= ' OR ' . $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_notes LIKE ?',
                                $attributeValue . '%');
                    } else {
                        $where .= ' OR ' . $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_notes LIKE ?',
                                '%' . $filterParams['lead-organization-attribute'] . '%');
                    }
                }

                if (!empty($filterParams['search_includes_custom_organization_fields'])) {
                    $includesCustomOrganizationFields = true;
                    if (strpos($filterParams['lead-organization-attribute'], '*') !== false) {
                        $attributeValue = str_replace('*', '%', $filterParams['lead-organization-attribute']);
                        $where .= ' OR ' . $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plocpd.param_value LIKE ?',
                                $attributeValue . '%');
                    } else {
                        $where .= ' OR ' . $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plocpd.param_value LIKE ?',
                                '%' . $filterParams['lead-organization-attribute'] . '%');
                    }
                }
                $where .=  ')';
            }

            if (!empty($filterParams['owner_id'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= ' ('.  $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('pldm.owner_id = ?', $filterParams['owner_id']);
                $where .= ' OR pldm.owner_id IS NULL)';
            }

            if (!empty($filterParams['organization_without_leads'])) {
                $organizationWithLeads = $leadOrganizationDataMapper->getOrganizationWithLeads();
                if (!empty($organizationWithLeads)) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }

                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.id NOT IN (?)', $organizationWithLeads);
                }
            }

            if (!empty($filterParams['organization_zip'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }

                $organizationZips = explode(',', $filterParams['organization_zip']);
                $organizationZips = array_diff($organizationZips, array(''));
                $organizationZips = array_map('trim', $organizationZips);

                $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_zip IN (?)', $organizationZips);
            }

            if (!empty($filterParams['organization_city'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_city = ?', $filterParams['organization_city']);
            }

            if (!empty($filterParams['organization_country'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_country_code = ?', $filterParams['organization_country']);
            }

            if (!empty($filterParams['organization_country_state']) && is_array($filterParams['organization_country_state'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }

                $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_state_code IN (?)',
                    $filterParams['organization_country_state']);
            }

            if(!empty($filterParams['created_at_from'])) {
                $createdAt = Tools_System_Tools::convertDateFromTimezone($filterParams['created_at_from']);
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                if (!empty($filterParams['last_updated']) && $filterParams['last_updated'] === 'last_updated') {
                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.updated_at >= ?', $createdAt);
                } else {
                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.created_at >= ?', $createdAt);
                }

            }
            if(!empty($filterParams['created_at_to'])) {
                $updatedAt =  Tools_System_Tools::convertDateFromTimezone($filterParams['created_at_to'].' 23:59:59');
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                if (!empty($filterParams['last_updated']) && $filterParams['last_updated'] === 'last_updated') {
                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.updated_at <= ?', $updatedAt);
                } else {
                    $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.created_at <= ?', $updatedAt);
                }

            }

            if (!empty($filterParams['customer_group_id'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.customer_group_id = ?', $filterParams['customer_group_id']);
            }

            if (!empty($filterParams['industry_id'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_industry_id = ?', $filterParams['industry_id']);
            }

            if (!empty($filterParams['organization_domain'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_domain = ?', $filterParams['organization_domain']);
            }

            if (!empty($filterParams['organization_domain1'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_domain1 = ?', $filterParams['organization_domain1']);
            }

            if (!empty($filterParams['last_interaction_id'])) {
                $withLeadsData = true;
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('pld.last_activity_type_id = ?', $filterParams['last_interaction_id']);
            }

            if (!empty($filterParams['last_interaction_id']) && !empty($filterParams['outbound_emails_sub_type'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('pld.last_activity_subtype = ?', $filterParams['outbound_emails_sub_type']);
            }

            if (!empty($filterParams['total_opportunity_amount_from']) || !empty($filterParams['total_opportunity_amount_to']) || !empty($filterParams['lead_opportunity_stages']) || !empty($filterParams['opportunity_type_id'])) {
                $withOpportunityAmount = true;
            }

            if (!empty($filterParams['total_spent_purchase_amount_from']) || !empty($filterParams['total_spent_purchase_amount_to']) || !empty($filterParams['special_filter_type']) && $filterParams['special_filter_type'] === Api_Leads_Leads::FILTER_EXISTING_CLIENT) {
                $withSpentPurchaseAmount = true;

                if (!empty($where)) {
                    $where .= ' AND ';
                }

                $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('scs.status IN (?)', array(Models_Model_CartSession::CART_STATUS_COMPLETED, Models_Model_CartSession::CART_STATUS_SHIPPED, Models_Model_CartSession::CART_STATUS_DELIVERED));
            }

            if($filterParams['special_filter_type'] === Api_Leads_Leads::FILTER_PROSPECT) {
                $withSpentPurchaseAmount = true;

            }

            if (!empty($filterParams['lead_opportunity_stages'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }

                $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plop.opportunity_stage_id IN (?)', $filterParams['lead_opportunity_stages']);
            }

            if (!empty($filterParams['lead_score_from'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('pld.score >= ?',
                    (int) $filterParams['lead_score_from']);
                $withLeadsData = true;
            }

            if (!empty($filterParams['lead_score_to'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('pld.score <= ?',
                    (int) $filterParams['lead_score_to']);
                $withLeadsData = true;
            }

            if (!empty($filterParams['organization_tags']) && is_array($filterParams['organization_tags'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }

                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plht.tag_id IN (?)', $filterParams['organization_tags']);
                $withLeadsData = true;
                $withLeadsTags = true;
            }
            if (!empty($filterParams['organization_tags_exclude']) && is_array($filterParams['organization_tags_exclude'])) {
                $subSelect = $leadDataMapper->getDbTable()->select(Zend_Db_Table::SELECT_WITHOUT_FROM_PART)
                    ->distinct()
                    ->setIntegrityCheck(false)
                    ->from(array('ltags' => 'plugin_leads_has_tags'), array('ltags.lead_id'));
                $subSelectWhere = $leadDataMapper->getDbTable()->getAdapter()->quoteInto('tag_id IN (?)', $filterParams['organization_tags_exclude']);
                $subSelect->where($subSelectWhere);
                $subSelectIds = $leadDataMapper->getDbTable()->getAdapter()->fetchCol($subSelect);
                if (!empty($subSelectIds)) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.id NOT IN (?)', $subSelectIds);
                    $withLeadsData = true;
                    $withLeadsTags = true;
                }
            }

            if (!empty($filterParams['special_filter_type'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                if ($filterParams['special_filter_type'] === Api_Leads_Leads::FILTER_EXISTING_CLIENT) {
                    $where .= new Zend_Db_Expr('pld.user_id IS NOT NULL');
                }
                if ($filterParams['special_filter_type'] === Api_Leads_Leads::FILTER_PROSPECT) {
                    $where .= new Zend_Db_Expr('pld.user_id IS NULL');
                    $where .= ' AND '. new Zend_Db_Expr('pld.id IS NOT NULL');
                }
                if ($filterParams['special_filter_type'] === Api_Leads_Leads::FILTER_NO_EMAIL) {
                    $withLeadsData = true;
                    $where .= new Zend_Db_Expr('pld.lead_email IS NULL');
                    $where .= ' AND '. new Zend_Db_Expr('pld.id IS NOT NULL');
                }
            }

            if (!empty($filterParams['opportunity_type_id'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }

                $where .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('plop.opportunity_type_id = ?', $filterParams['opportunity_type_id']);
            }

            if(!empty($organizationLeads)) {
                $withLeadsData = true;
                $additionalOrganizationParamsForSelect = array_merge($additionalOrganizationParamsForSelect, array('countLeads' => new Zend_Db_Expr('COUNT(DISTINCT(pld.id))')));
                $additionalOrganizationParamsForSelect = array_merge($additionalOrganizationParamsForSelect, array('organizationLeadIds' => new Zend_Db_Expr('GROUP_CONCAT(DISTINCT(pld.id))')));
            }

            if (!empty($filterParams['total_opportunity_amount_from'])) {
                if (!empty($having)) {
                    $having .= ' AND ';
                }
                $having .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('SUM(plop.opportunity_value) >= ?', $filterParams['total_opportunity_amount_from']);
            }
            if (!empty($filterParams['total_opportunity_amount_to'])) {
                if (!empty($having)) {
                    $having .= ' AND ';
                }
                $having .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('SUM(plop.opportunity_value) <= ?', $filterParams['total_opportunity_amount_to']);
            }

            if (!empty($filterParams['total_spent_purchase_amount_from'])) {
                if (!empty($having)) {
                    $having .= ' AND ';
                }
                $having .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('SUM(scs.total) >= ?',
                    $filterParams['total_spent_purchase_amount_from']);
            }

            if (!empty($filterParams['total_spent_purchase_amount_to'])) {
                if (!empty($having)) {
                    $having .= ' AND ';
                }
                $having .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('SUM(scs.total) <= ?',
                    $filterParams['total_spent_purchase_amount_to']);
            }

            if (!empty($filterParams['custom_organization_params_search']) && is_array($filterParams['custom_organization_params_search'])) {
                $leadsOrganizationCustomParamsMapper = Leads_Mapper_LeadsOrganizationCustomParamsConfigMapper::getInstance();
                $tempParamCounter = 1;
                foreach ($filterParams['custom_organization_params_search'] as  $searchCustomParamsData) {
                    $searchParamId = $searchCustomParamsData['id'];
                    $organizationCustomParamsModel = $leadsOrganizationCustomParamsMapper->find($searchParamId);
                    if ($organizationCustomParamsModel instanceof Leads_Model_LeadsOrganizationCustomParamsConfigModel) {
                        $includesCustomOrganizationFields = true;
                        $tempParamName = 'customOrganizationParamVal' . $tempParamCounter;
                        if (!empty($having)) {
                            $having .= ' AND ';
                        }
                        $operator = $searchCustomParamsData['operator'];
                        $paramOperator = '=';
                        if ($operator === Api_Leads_Leadorganization::RULE_COMPARISON_OPERATOR_EQUAL) {
                            $paramOperator = '=';
                        }
                        if ($operator === Api_Leads_Leadorganization::RULE_COMPARISON_OPERATOR_NOTEQUAL) {
                            $paramOperator = '<>';
                        }
                        if ($operator === Api_Leads_Leadorganization::RULE_COMPARISON_GREATER_THAN) {
                            $paramOperator = '>';
                            $searchCustomParamsData['value'] = floatval($searchCustomParamsData['value']);
                        }
                        if ($operator === Api_Leads_Leadorganization::RULE_COMPARISON_LESS_THAN) {
                            $paramOperator = '<';
                            $searchCustomParamsData['value'] = floatval($searchCustomParamsData['value']);
                        }

                        if ($operator === Api_Leads_Leadorganization::RULE_COMPARISON_OPERATOR_LIKE) {
                            $paramOperator = 'LIKE';
                            $having .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('(' . $tempParamName . ') ' . $paramOperator . ' ?',
                                '%'. $searchCustomParamsData['value']. '%');
                        } elseif ($operator === Api_Leads_Leadorganization::RULE_COMPARISON_OPERATOR_EQUAL) {
                            if (empty($searchCustomParamsData['value'])) {
                                $having .= '('.$leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('(' . $tempParamName . ') ' . $paramOperator . ' ?',
                                        $searchCustomParamsData['value']).' OR ';
                                $having .= '(' . $tempParamName . ') '. 'IS NULL';

                                $having .=   ')';
                            } else {
                                $having .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('(' . $tempParamName . ') ' . $paramOperator . ' ?',
                                    $searchCustomParamsData['value']);
                            }
                        } elseif ($operator === Api_Leads_Leadorganization::RULE_COMPARISON_GREATER_THAN || $operator === Api_Leads_Leadorganization::RULE_COMPARISON_LESS_THAN) {
                            $having .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('CAST(' . $tempParamName . '  AS DECIMAL(10,2))' . $paramOperator . ' ?',
                                $searchCustomParamsData['value']);
                        } else {
                            $having .= $leadOrganizationDataMapper->getDbTable()->getAdapter()->quoteInto('(' . $tempParamName . ') ' . $paramOperator . ' ?',
                                $searchCustomParamsData['value']);
                        }
                        $additionalOrganizationParamsForSelect = array_merge($additionalOrganizationParamsForSelect,
                            array($tempParamName => new Zend_Db_Expr('GROUP_CONCAT(DISTINCT(CASE WHEN plocpd.param_id=' . $searchParamId . ' THEN plocpd.param_value END))')));
                        $tempParamCounter++;
                    }
                }
            }

            return $leadOrganizationDataMapper->fetchAll($where, $sortOrder, $limit, $offset, false, false, $withOpportunityAmount, $having, $withNextActivityDate, $withLeadsData, $includesCustomOrganizationFields, $withSpentPurchaseAmount, $additionalOrganizationParamsForSelect, $withLeadsTags);

        }
    }



    /**
     * @param $filters
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $preParseParams pre-parse params
     * @param bool $withTimezoneOffset without timezone offset
     * @param bool $searchFullNumber search full number
     * @return array
     */
    public static function getSearchFilterData($filters , $limit = null, $offset = null, $preParseParams = true, $withTimezoneOffset = true, $searchFullNumber = false, $existMobileNumber = false){
        $leadDataMapper = Leads_Mapper_LeadsMapper::getInstance();

        $withOpportunityAmount = false;
        $withNextActivityDate = false;
        $includePluginLeadsTypeCpecificColl = false;
        $having = null;
        $withTasks = false;
        $sortOrder = null;
        $callDuration = false;
        $includesNotes = false;
        $includesCustomLeadFields = false;
        $includesCustomOrganizationFields = false;
        $withOpportunityExpected = false;
        $withSpentPurchaseAmount = false;
        $withEmailLog = false;
        $withOrganizationIndustry = false;
        $additionalParamsForSelect = array();
        $additionalOrganizationParamsForSelect = array();
        $withEmailSmsSequence = false;

        if ($preParseParams === true) {
            parse_str($filters, $filterParams);
        } else {
            $filterParams = $filters;
        }

        $filterParams = filter_var_array($filterParams, FILTER_SANITIZE_STRING);
        $restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
        if (!empty($restrictedUserId)) {
            $filterParams['owner_id'] = $restrictedUserId;
        }
        $where = null;
        if (!empty($filterParams)) {
            $filterParams['lead-attribute'] = str_replace('*-amp-*', '&', $filterParams['lead-attribute']);
            $where = '';
            if (!empty($filterParams['lead-attribute']) && $filterParams['filter-by'] === 'attributes') {
                $filterParams['lead-attribute'] = trim($filterParams['lead-attribute'], " \t\n\r\x0B");
                if (strpos($filterParams['lead-attribute'], '*') !== false) {
                    $attributeValue = str_replace('*', '%', $filterParams['lead-attribute']);
                    $where .= ' (' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_first_name LIKE ?',
                            $attributeValue . '%');
                    $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_last_name LIKE ?',
                            $attributeValue . '%');
                    $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_email LIKE ?',
                            $attributeValue . '%');
                    $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_secondary_email LIKE ?',
                            $attributeValue . '%');

                    if ($searchFullNumber === true) {
                        $where .= ' OR ' .$leadDataMapper->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr('CONCAT(`lead_mobile_country_code_value`, `lead_mobile`) = ?'), $attributeValue);
                        $where .= ' OR ' .$leadDataMapper->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr('CONCAT(`lead_phone_country_code_value`, `lead_phone`) = ?'), $attributeValue);
                    } else {
                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_phone LIKE ?',
                                $attributeValue . '%');
                    }

                    $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr('CONCAT(pld.lead_first_name, " ", pld.lead_last_name) LIKE ?'),
                            $attributeValue. '%');
                    $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr('CONCAT(pld.lead_first_name, pld.lead_last_name) LIKE ?'),
                            $attributeValue . '%');
                } else {
                    $attributeValues = explode(' ', $filterParams['lead-attribute']);
                    $where .= ' ('.'(';
                    foreach ($attributeValues as $key => $attrVal) {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_first_name LIKE ?',
                            '%'. $attrVal . '%');

                        if (count($attributeValues) > $key+1) {
                            $where .= ' AND ';
                        }
                    }

                    $where .= ') OR ( ';

                    foreach ($attributeValues as $key => $attrVal) {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_last_name LIKE ?',
                            '%'. $attrVal . '%');

                        if (count($attributeValues) > $key+1) {
                            $where .= ' AND ';
                        }
                    }

                    $where .= ') OR ( ';
                    foreach ($attributeValues as $key => $attrVal) {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_email LIKE ?',
                            '%' .$attrVal . '%');

                        if (count($attributeValues) > $key+1) {
                            $where .= ' AND ';
                        }
                    }

                    $where .= ') OR ( ';
                    foreach ($attributeValues as $key => $attrVal) {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_secondary_email LIKE ?',
                            '%'. $attrVal . '%');

                        if (count($attributeValues) > $key+1) {
                            $where .= ' AND ';
                        }
                    }

                    $where .= ')';

                    if ($searchFullNumber === true) {
                        $where .= ' OR ' .$leadDataMapper->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr('CONCAT(`lead_mobile_country_code_value`, `lead_mobile`) = ?'), $filterParams['lead-attribute']);
                        $where .= ' OR ' .$leadDataMapper->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr('CONCAT(`lead_phone_country_code_value`, `lead_phone`) = ?'), $filterParams['lead-attribute']);
                    } else {
                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_phone LIKE ?',
                                $filterParams['lead-attribute'] . '%');
                    }

                    $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr('CONCAT(pld.lead_first_name, " ", pld.lead_last_name) LIKE ?'),
                            $filterParams['lead-attribute'] . '%');
                    $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr('CONCAT(pld.lead_first_name, pld.lead_last_name) LIKE ?'),
                            $filterParams['lead-attribute'] . '%');
                }

                if (!empty($filterParams['search_includes_notes'])) {
                    $includesNotes = true;
                    if (strpos($filterParams['lead-attribute'], '*') !== false) {
                        $attributeValue = str_replace('*', '%', $filterParams['lead-attribute']);
                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_notes LIKE ?',
                                $attributeValue . '%');
                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pln.note LIKE ?',
                                $attributeValue . '%');
                    } else {
                        $attributeValues = explode(' ', $filterParams['lead-attribute']);
                        $where .= ' OR ( ';
                        foreach ($attributeValues as $key => $attrVal) {
                            $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_notes LIKE ?',
                                '%' . $attrVal . '%');

                            if (count($attributeValues) > $key+1) {
                                $where .= ' AND ';
                            }
                        }

                        $where .= ') OR ( ';

                        foreach ($attributeValues as $key => $attrVal) {
                            $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pln.note LIKE ?',
                                '%' . $attrVal . '%');

                            if (count($attributeValues) > $key+1) {
                                $where .= ' AND ';
                            }
                        }
                        $where .= ')';
                    }
                }

                if (!empty($filterParams['search_includes_custom_lead_fields'])) {
                    $includesCustomLeadFields = true;
                    if (strpos($filterParams['lead-attribute'], '*') !== false) {
                        $attributeValue = str_replace('*', '%', $filterParams['lead-attribute']);
                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pllcpd.param_value LIKE ?',
                                $attributeValue . '%');

                    } else {
                        $attributeValues = explode(' ', $filterParams['lead-attribute']);

                        $where .= ' OR ( ';
                        foreach ($attributeValues as $key => $attrVal) {
                            $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pllcpd.param_value LIKE ?',
                                '%' . $attrVal . '%');
                            if (count($attributeValues) > $key + 1) {
                                $where .= ' AND ';
                            }
                        }
                        $where .= ')';
                    }
                }

                if ($searchFullNumber === false) {
                    $leadPhone = Tools_LeadTools::cleanPhoneAddress($filterParams['lead-attribute']);
                    $leadPhoneIsNumber = mb_substr(preg_replace('~[^0-9A-Za-z]~ui', '',
                        $filterParams['lead-attribute']), 0, 1);
                    if (strpos($filterParams['lead-attribute'], '*') !== false) {
                        $attributeValue = str_replace('*', '%', $filterParams['lead-attribute']);

                        if (!empty($leadPhone) && is_numeric($leadPhoneIsNumber)) {
                            $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_phone LIKE ?',
                                    $leadPhone . '%');
                            $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_mobile LIKE ?',
                                    $leadPhone . '%');
                        }

                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr('CONCAT(pld.lead_first_name, " ", pld.lead_last_name) LIKE ?'),
                                $attributeValue . '%');
                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr('CONCAT(pld.lead_first_name, pld.lead_last_name) LIKE ?'),
                                $attributeValue . '%') . ') ';
                    } else {
                        if (!empty($leadPhone) && is_numeric($leadPhoneIsNumber)) {
                            $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_phone LIKE ?',
                                    $leadPhone . '%');
                            $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_mobile LIKE ?',
                                    $leadPhone . '%');
                            $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr('CONCAT(pld.lead_mobile_country_code_value, pld.lead_mobile) LIKE ?'),
                                    '%' . $leadPhone . '%');
                            $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr('CONCAT(pld.lead_phone_country_code_value, pld.lead_phone) LIKE ?'),
                                    '%' . $leadPhone . '%');
                        }

                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr('CONCAT(pld.lead_first_name, " ", pld.lead_last_name) LIKE ?'),
                                $filterParams['lead-attribute'] . '%');
                        $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto(new Zend_Db_Expr('CONCAT(pld.lead_first_name, pld.lead_last_name) LIKE ?'),
                                $filterParams['lead-attribute'] . '%') . ') ';
                    }
                } else {
                    $where .= ') ';
                }
            }
            if (!empty($filterParams['lead-attribute']) && $filterParams['filter-by'] === 'organization') {
                $filterParams['lead-attribute'] = trim($filterParams['lead-attribute'], " \t\n\r\x0B");
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= ' (' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_name LIKE ?',
                    $filterParams['lead-attribute'] . '%');

                if (!empty($filterParams['search_includes_notes'])) {
                    $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_notes LIKE ?',
                            '%'. $filterParams['lead-attribute']. '%');
                }

                if (!empty($filterParams['search_includes_custom_lead_fields'])) {
                    $includesCustomOrganizationFields = true;
                    $where .= ' OR ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plocpd.param_value LIKE ?',
                            '%'. $filterParams['lead-attribute'] . '%');
                }

                $where .= ') ';
            }

            if (!empty($filterParams['lead_titles']) && !empty($filterParams['lead_title_operator'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }

                $operator = $filterParams['lead_title_operator'];

                if ($operator === Api_Leads_Leads::RULE_COMPARISON_OPERATOR_EQUAL) {
                    $paramOperator = '=';
                }
                if ($operator === Api_Leads_Leads::RULE_COMPARISON_OPERATOR_NOTEQUAL) {
                    $paramOperator = '<>';
                }

                if ($operator === Api_Leads_Leads::RULE_COMPARISON_OPERATOR_LIKE) {
                    $paramOperator = 'LIKE';
                    $filterParams['lead_titles'] = $filterParams['lead_titles'].'%';
                }

                if (!empty($paramOperator)) {
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_position ' . $paramOperator . ' ?',
                        $filterParams['lead_titles']);
                }
            }

            if (isset($filterParams['nps_score']) && is_numeric($filterParams['nps_score']) && !empty($filterParams['lead_filter_nps_score_operator'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }

                $operator = $filterParams['lead_filter_nps_score_operator'];

                if ($operator === Api_Leads_Leads::RULE_COMPARISON_OPERATOR_EQUAL) {
                    $paramOperator = '=';
                }
                if ($operator === Api_Leads_Leads::RULE_COMPARISON_GREATER_THAN) {
                    $paramOperator = '>=';
                }

                if ($operator === Api_Leads_Leads::RULE_COMPARISON_LESS_THAN) {
                    $paramOperator = '<=';
                }

                if (!empty($paramOperator)) {
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plrr.score ' . $paramOperator . ' ?',
                        $filterParams['nps_score']);
                    $where .= ' AND '. $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plrr.status = ?',
                            'reviewed');
                }
            }


            if (!empty($filterParams['lead_zip'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }

                $leadZips = explode(',', $filterParams['lead_zip']);
                $leadZips = array_diff($leadZips, array(''));
                $leadZips = array_map('trim', $leadZips);

                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_zip IN (?)', $leadZips);
            }

            if (!empty($filterParams['lead_ids'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.id IN (?)',
                    explode(',', $filterParams['lead_ids']));
            }

            if ($existMobileNumber === true) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }

                $where .=  '(('.new Zend_Db_Expr('pld.lead_mobile_country_code_value IS NOT NULL') . ' AND ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_mobile_country_code_value <> ?', '');
                $where .= ' AND ' .new Zend_Db_Expr('pld.lead_mobile IS NOT NULL')  . ' AND ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_mobile <> ?', '') .')';
                $where .= ' OR ';
                $where .=  '('.new Zend_Db_Expr('plo.organization_mobile_country_code_value IS NOT NULL') . ' AND ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_mobile_country_code_value <> ?', '');
                $where .= ' AND ' .new Zend_Db_Expr('plo.organization_mobile IS NOT NULL') . ' AND ' . $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plo.organization_mobile <> ?', '') .'))';
            }

            if (!empty($filterParams['not_empty_email_address'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= new Zend_Db_Expr('pld.lead_email IS NOT NULL');
                $where .= ' AND '.$leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_email <> ?',
                        '');
            }

            if (!empty($filterParams['lead_city'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_city = ?',
                    $filterParams['lead_city']);
            }

            if (!empty($filterParams['lead_country'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_country_code = ?',
                    $filterParams['lead_country']);
            }

            if (!empty($filterParams['lead_country_state']) && is_array($filterParams['lead_country_state'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }

                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_state_code IN (?)',
                    $filterParams['lead_country_state']);
            }

            if (!empty($filterParams['lead_score_from'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.score >= ?',
                    (int) $filterParams['lead_score_from']);
            }

            if (!empty($filterParams['lead_score_to'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.score <= ?',
                    (int) $filterParams['lead_score_to']);
            }

            if (!empty($filterParams['email_validation_status'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.email_validated_status = ?',
                    $filterParams['email_validation_status']);
                $where .= ' AND '.$leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_email <> ?',
                        '');
            }

            if (!empty($filterParams['created_at_from'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $createdAt =  Tools_System_Tools::convertDateFromTimezone($filterParams['created_at_from']);
                if (!empty($filterParams['last_updated']) && $filterParams['last_updated'] === 'last_updated') {
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.updated_at > ?',
                        $createdAt);
                } else {
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.created_at > ?',
                        $createdAt);
                }
            }
            if (!empty($filterParams['created_at_to'])) {
                $updatedAt =  Tools_System_Tools::convertDateFromTimezone($filterParams['created_at_to'].' 23:59:59');
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                if (!empty($filterParams['last_updated']) && $filterParams['last_updated'] === 'last_updated') {
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.updated_at < ?',
                        $updatedAt);
                } else {
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.created_at < ?',
                        $updatedAt);
                }
            }

            if (!empty($filterParams['customer_group_id'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.customer_group_id = ?',
                    $filterParams['customer_group_id']);
            }
            if (!empty($filterParams['stage_id'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.stage_id IN (?)', $filterParams['stage_id']);
            }
            if (!empty($filterParams['lead_source_type'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_type_id = ?',
                    $filterParams['lead_source_type']);

                $leadTypeModel = Leads_Mapper_LeadsTypeMapper::getInstance()->find($filterParams['lead_source_type']);
                $callDurationParam = false;
                if($leadTypeModel instanceof Leads_Model_LeadsTypeModel) {
                    if($leadTypeModel->getTypeName() == Leads_Model_LeadsTypeModel::LEAD_TYPE_INBOUND_CALL || $leadTypeModel->getTypeName() == Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_CALL) {
                        $callDurationParam = true;
                    }
                }

                if (!empty($filterParams['call_duration']) && !empty($callDurationParam)) {
                    $callDuration = true;
                    $filterParams['call_duration'] = (int)$filterParams['call_duration'];

                    $where .= ' AND ';

                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plcl.call_duration >= ?',
                        $filterParams['call_duration']);
                }
            }
            if (!empty($filterParams['lead_source'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_source LIKE ?',
                    '%'. $filterParams['lead_source']. '%');
            }
            if (!empty($filterParams['owner_id'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?',
                    $filterParams['owner_id']);
            }
            if (!empty($filterParams['excludeLeadIds'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.id NOT IN (?)',
                    $filterParams['excludeLeadIds']);
            }
            if (!empty($filterParams['opportunity_owner_id'])) {
                $withOpportunityExpected = true;
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plop.creator_id = ?',
                    $filterParams['opportunity_owner_id']);
            }

            if (!empty($filterParams['total_spent_purchase_amount_from']) || !empty($filterParams['total_spent_purchase_amount_to']) && !empty($filterParams['special_filter_type']) && $filterParams['special_filter_type'] === Api_Leads_Leads::FILTER_EXISTING_CLIENT) {
                $withSpentPurchaseAmount = true;

                if (!empty($where)) {
                    $where .= ' AND ';
                }

                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('scs.status IN (?)', array(Models_Model_CartSession::CART_STATUS_COMPLETED, Models_Model_CartSession::CART_STATUS_SHIPPED, Models_Model_CartSession::CART_STATUS_DELIVERED));
            }

            if (!empty($filterParams['email_status'])) {
                $withEmailLog = true;

                if (!empty($where)) {
                    $where .= ' AND ';
                }

                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plel.email_status = ? ', $filterParams['email_status']);
            }


            if (!empty($filterParams['special_filter_type'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                if ($filterParams['special_filter_type'] === Api_Leads_Leads::FILTER_EXISTING_CLIENT) {
                    $where .= new Zend_Db_Expr('pld.user_id IS NOT NULL');
                }
                if ($filterParams['special_filter_type'] === Api_Leads_Leads::FILTER_PROSPECT) {
                    $where .= new Zend_Db_Expr('pld.user_id IS NULL');
                }
                if ($filterParams['special_filter_type'] === Api_Leads_Leads::FILTER_NO_EMAIL) {
                    $where .= new Zend_Db_Expr('pld.lead_email IS NULL');
                }
            }

            if (!empty($filterParams['activity_id']) && is_array($filterParams['activity_id'])) {
                $withNextActivityDate = true;
                $includePluginLeadsTypeCpecificColl = true;
                if (!empty($where)) {
                    $where .= ' AND ';
                }

                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plt.id IN (?)', $filterParams['activity_id']);
            }

            if (!empty($filterParams['next_activity_from'])) {
                $withNextActivityDate = true;
                if (!empty($where)) {
                    $where .= ' AND ';
                }

                $currentDate = Tools_System_Tools::convertDateFromTimezone('now');
                $activityFromDate = Tools_System_Tools::convertDateFromTimezone($filterParams['next_activity_from']);

                if($activityFromDate > $currentDate) {
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plal.event_type = ?',
                        Leads_Model_LeadsActionLogModel::EVENT_LOG_UPCOMING);
                    $where .= ' AND ' .$leadDataMapper->getDbTable()->getAdapter()->quoteInto('plal.created_at > ?', $currentDate);

                } else {
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plal.created_at > ?', $activityFromDate);
                }

                $having = $leadDataMapper->getDbTable()->getAdapter()->quoteInto('MIN(plal.created_at) > ?', $activityFromDate);
            }

            if(!empty($filterParams['next_activity_to'])) {
                $withNextActivityDate = true;
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $activityToDate = Tools_System_Tools::convertDateFromTimezone($filterParams['next_activity_to'].' 23:59:59');
                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plal.created_at < ?', $activityToDate);

            }


            if(!empty($filterParams['industry_id'])) {
                $withOrganizationIndustry = true;
                if (!empty($where)) {
                    $where .= ' AND ';
                }

                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pli.id = ?', $filterParams['industry_id']);
            }

            if (!empty($filterParams['custom_organization_params_search']) && is_array($filterParams['custom_organization_params_search'])) {
                $leadsOrganizationCustomParamsMapper = Leads_Mapper_LeadsOrganizationCustomParamsConfigMapper::getInstance();
                $tempParamCounter = 1;
                foreach ($filterParams['custom_organization_params_search'] as  $searchCustomParamsData) {
                    $searchParamId = $searchCustomParamsData['id'];
                    $organizationCustomParamsModel = $leadsOrganizationCustomParamsMapper->find($searchParamId);
                    if ($organizationCustomParamsModel instanceof Leads_Model_LeadsOrganizationCustomParamsConfigModel) {
                        $includesCustomOrganizationFields = true;
                        $tempParamName = 'customOrganizationParamVal' . $tempParamCounter;
                        if (!empty($having)) {
                            $having .= ' AND ';
                        }
                        $operator = $searchCustomParamsData['operator'];
                        $paramOperator = '=';
                        if ($operator === Api_Leads_Leads::RULE_COMPARISON_OPERATOR_EQUAL) {
                            $paramOperator = '=';
                        }
                        if ($operator === Api_Leads_Leads::RULE_COMPARISON_OPERATOR_NOTEQUAL) {
                            $paramOperator = '<>';
                        }
                        if ($operator === Api_Leads_Leads::RULE_COMPARISON_GREATER_THAN) {
                            $paramOperator = '>';
                            $searchCustomParamsData['value'] = floatval($searchCustomParamsData['value']);
                        }
                        if ($operator === Api_Leads_Leads::RULE_COMPARISON_LESS_THAN) {
                            $paramOperator = '<';
                            $searchCustomParamsData['value'] = floatval($searchCustomParamsData['value']);
                        }

                        if ($operator === Api_Leads_Leads::RULE_COMPARISON_OPERATOR_LIKE) {
                            $paramOperator = 'LIKE';
                            $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('(' . $tempParamName . ') ' . $paramOperator . ' ?',
                                '%'. $searchCustomParamsData['value']. '%');
                        } elseif ($operator === Api_Leads_Leads::RULE_COMPARISON_OPERATOR_EQUAL) {
                            if (empty($searchCustomParamsData['value'])) {
                                $having .= '('.$leadDataMapper->getDbTable()->getAdapter()->quoteInto('(' . $tempParamName . ') ' . $paramOperator . ' ?',
                                        $searchCustomParamsData['value']).' OR ';
                                $having .= '(' . $tempParamName . ') '. 'IS NULL';

                                $having .=   ')';
                            } else {
                                $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('(' . $tempParamName . ') ' . $paramOperator . ' ?',
                                    $searchCustomParamsData['value']);
                            }
                        } elseif ($operator === Api_Leads_Leads::RULE_COMPARISON_GREATER_THAN || $operator === Api_Leads_Leads::RULE_COMPARISON_LESS_THAN) {
                            $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('CAST(' . $tempParamName . '  AS DECIMAL(10,2))' . $paramOperator . ' ?',
                                $searchCustomParamsData['value']);
                        } else {
                            $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('(' . $tempParamName . ') ' . $paramOperator . ' ?',
                                $searchCustomParamsData['value']);
                        }
                        $additionalOrganizationParamsForSelect = array_merge($additionalOrganizationParamsForSelect,
                            array($tempParamName => new Zend_Db_Expr('GROUP_CONCAT(DISTINCT(CASE WHEN plocpd.param_id=' . $searchParamId . ' THEN plocpd.param_value END))')));
                        $tempParamCounter++;
                    }
                }
            }


            if (!empty($filterParams['custom_params_search']) && is_array($filterParams['custom_params_search'])) {
                $leadsCustomParamsMapper = Leads_Mapper_LeadsCustomParamsConfigMapper::getInstance();
                $tempParamCounter = 1;
                foreach ($filterParams['custom_params_search'] as  $searchCustomParamsData) {
                    $searchParamId = $searchCustomParamsData['id'];
                    $leadsCustomParamsModel = $leadsCustomParamsMapper->find($searchParamId);
                    if ($leadsCustomParamsModel instanceof Leads_Model_LeadsCustomParamsConfigModel) {
                        $includesCustomLeadFields = true;
                        $tempParamName = 'customParamVal' . $tempParamCounter;
                        if (!empty($having)) {
                            $having .= ' AND ';
                        }
                        $operator = $searchCustomParamsData['operator'];
                        $paramOperator = '=';
                        if ($operator === Api_Leads_Leads::RULE_COMPARISON_OPERATOR_EQUAL) {
                            $paramOperator = '=';
                        }

                        if ($operator === Api_Leads_Leads::RULE_COMPARISON_OPERATOR_NOTEQUAL) {
                            $paramOperator = '<>';
                        }

                        if ($operator === Api_Leads_Leads::RULE_COMPARISON_GREATER_THAN) {
                            $paramOperator = '>';
                            $searchCustomParamsData['value'] = floatval($searchCustomParamsData['value']);
                        }

                        if ($operator === Api_Leads_Leads::RULE_COMPARISON_LESS_THAN) {
                            $paramOperator = '<';
                            $searchCustomParamsData['value'] = floatval($searchCustomParamsData['value']);
                        }

                        if ($operator === Api_Leads_Leads::RULE_COMPARISON_OPERATOR_LIKE) {
                            $paramOperator = 'LIKE';
                            $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('(' . $tempParamName . ') ' . $paramOperator . ' ?',
                                '%'. $searchCustomParamsData['value']. '%');
                        } elseif ($operator === Api_Leads_Leads::RULE_COMPARISON_OPERATOR_EQUAL) {
                            if (empty($searchCustomParamsData['value'])) {
                                $having .= '('.$leadDataMapper->getDbTable()->getAdapter()->quoteInto('(' . $tempParamName . ') ' . $paramOperator . ' ?',
                                        $searchCustomParamsData['value']).' OR ';
                                $having .= '(' . $tempParamName . ') '. 'IS NULL';

                                $having .=   ')';
                            } else {
                                $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('(' . $tempParamName . ') ' . $paramOperator . ' ?',
                                    $searchCustomParamsData['value']);
                            }
                        } elseif ($operator === Api_Leads_Leads::RULE_COMPARISON_GREATER_THAN || $operator === Api_Leads_Leads::RULE_COMPARISON_LESS_THAN) {
                            $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('CAST(' . $tempParamName . '  AS DECIMAL(10,2))' . $paramOperator . ' ?',
                                $searchCustomParamsData['value']);
                        } else {
                            $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('(' . $tempParamName . ') ' . $paramOperator . ' ?',
                                $searchCustomParamsData['value']);
                        }
                        $additionalParamsForSelect = array_merge($additionalParamsForSelect,
                            array($tempParamName => new Zend_Db_Expr('GROUP_CONCAT(DISTINCT(CASE WHEN pllcpd.param_id=' . $searchParamId . ' THEN pllcpd.param_value END))')));
                        $tempParamCounter++;
                    }
                }
            }

            if (!empty($filterParams['opportunity_date_from']) || !empty($filterParams['opportunity_date_to'])) {
                $withOpportunityExpected = true;
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $opportunityFromDate = Tools_System_Tools::convertDateFromTimezone($filterParams['opportunity_date_from']);
                if(!empty($filterParams['expected_at'])) {
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plop.expected_close_date > ?', $opportunityFromDate);
                } elseif (!empty($filterParams['creation_at'])) {
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plop.created_at > ?', $opportunityFromDate);
                } elseif (!empty($filterParams['state_change_at'])) {
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plop.change_stage_date > ?', $opportunityFromDate);
                }

                if (!empty($filterParams['opportunity_date_to'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $opportunityToDate = Tools_System_Tools::convertDateFromTimezone($filterParams['opportunity_date_to'] .' 23:59:59');

                    if(!empty($filterParams['expected_at'])) {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plop.expected_close_date < ?', $opportunityToDate);
                    } elseif (!empty($filterParams['creation_at'])) {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plop.created_at < ?', $opportunityToDate);
                    } elseif (!empty($filterParams['state_change_at'])) {
                        $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plop.change_stage_date < ?', $opportunityToDate);
                    }
                }
            }

            if (!empty($filterParams['email_sms_sequences']) && is_array($filterParams['email_sms_sequences'])) {
                $withEmailSmsSequence = true;
                if (!empty($where)) {
                    $where .= ' AND ';
                }

                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plhes.sequence_id IN (?)', $filterParams['email_sms_sequences']);

                if (!empty($filterParams['email_sequence_type']) && ($filterParams['email_sequence_type'] === 'sms' ||  $filterParams['email_sequence_type'] === 'email') &&
                    !empty($filterParams['email_sequence_type_action']) && ($filterParams['email_sequence_type_action'] === 'sent' || $filterParams['email_sequence_type_action'] === 'opened' || $filterParams['email_sequence_type_action'] === 'all')) {
                    $withNextActivityDate = true;
                    $emailSequenceTypeAction = $filterParams['email_sequence_type_action'];

                    if (!empty((int)$filterParams['email_sms_sequences_type_period'])) {
                        $periodDays = (int)$filterParams['email_sms_sequences_type_period'];
                        $periodDaysEnd = Tools_System_Tools::convertDateFromTimezone('now');
                        $periodDaysStart = Tools_System_Tools::convertDateFromTimezone('-'.$periodDays.' days');
                    }

                    if ($filterParams['email_sequence_type'] === 'sms') {
                        if (!empty($where)) {
                            $where .= ' AND ';
                        }

                        if ($emailSequenceTypeAction === 'all') {
                            $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.status IN (?)', array('0', '1', '2'));
                            $where .= ' AND '. $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.type = ?', 'sms');
                            $where .= ' AND '. $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.sequence_id IN (?)',  $filterParams['email_sms_sequences']);
                            if (!empty($periodDays)) {
                                if (!empty($where)) {
                                    $where .= ' AND ';
                                }

                                $where .= '(('.$leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.sentAt >= ?', $periodDaysStart);
                                $where .= ' AND '. $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.sentAt <= ?', $periodDaysEnd).')';
                                $where .= ' OR ('.$leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.delayedTo >= ?', $periodDaysStart);
                                $where .= ' AND '.$leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.delayedTo <= ?', $periodDaysEnd).'))';
                            }
                        } else {
                            $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.status = ?', '1');
                            $where .= ' AND '. $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.type = ?', 'sms');
                            $where .= ' AND '. $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.sequence_id IN (?)',  $filterParams['email_sms_sequences']);
                            if (!empty($periodDays)) {
                                if (!empty($where)) {
                                    $where .= ' AND ';
                                }

                                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.sentAt >= ?', $periodDaysStart);
                                $where .= ' AND '. $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.sentAt <= ?', $periodDaysEnd);
                            }
                        }


                    }

                    if ($filterParams['email_sequence_type'] === 'email') {
                        if ($emailSequenceTypeAction === 'all') {
                            if (!empty($where)) {
                                $where .= ' AND ';
                            }
                            $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.status IN (?)', array('0', '1', '2'));
                            $where .= ' AND '. $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.type = ?', 'email');
                        }
                        if ($emailSequenceTypeAction === 'sent') {
                            if (!empty($where)) {
                                $where .= ' AND ';
                            }
                            $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plel.email_status = ?', 'sent');
                            $where .= ' AND '. $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plel.follow_up_sequences_id IN (?)',  $filterParams['email_sms_sequences']);
                        }
                        if ($emailSequenceTypeAction === 'opened') {
                            if (!empty($where)) {
                                $where .= ' AND ';
                            }
                            $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plel.email_status = ?', 'opened');
                            $where .= ' AND '. $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plel.follow_up_sequences_id IN (?)',  $filterParams['email_sms_sequences']);

                        }

                        if (!empty($periodDays) && ($emailSequenceTypeAction === 'sent' || $emailSequenceTypeAction === 'opened')) {
                            if (!empty($where)) {
                                $where .= ' AND ';
                            }

                            $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plal.created_at  >= ?', $periodDaysStart);
                            $where .= ' AND '. $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plal.created_at  <= ?', $periodDaysEnd);
                        } elseif (!empty($periodDays)){
                            if (!empty($where)) {
                                $where .= ' AND ';
                            }

                            $where .= '(('.$leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.sentAt >= ?', $periodDaysStart);
                            $where .= ' AND '. $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.sentAt <= ?', $periodDaysEnd).')';
                            $where .= ' OR ('.$leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.delayedTo >= ?', $periodDaysStart);
                            $where .= ' AND '.$leadDataMapper->getDbTable()->getAdapter()->quoteInto('pcq.delayedTo <= ?', $periodDaysEnd).'))';
                        }

                    }
                }
            }

            if (!empty($filterParams['lead_tags']) && is_array($filterParams['lead_tags'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }

                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plht.tag_id IN (?)', $filterParams['lead_tags']);
            }

            if (!empty($filterParams['lead_tags_exclude']) && is_array($filterParams['lead_tags_exclude'])) {
                $subSelect = $leadDataMapper->getDbTable()->select(Zend_Db_Table::SELECT_WITHOUT_FROM_PART)
                    ->distinct()
                    ->setIntegrityCheck(false)
                    ->from(array('ltags' => 'plugin_leads_has_tags'), array('ltags.lead_id'));
                $subSelectWhere = $leadDataMapper->getDbTable()->getAdapter()->quoteInto('tag_id IN (?)', $filterParams['lead_tags_exclude']);
                $subSelect->where($subSelectWhere);
                $subSelectIds = $leadDataMapper->getDbTable()->getAdapter()->fetchCol($subSelect);
                if (!empty($subSelectIds)) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.id NOT IN (?)', $subSelectIds);
                }
            }

            if (!empty($filterParams['last_interaction_id'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.last_activity_type_id = ?', $filterParams['last_interaction_id']);
            }

            if (!empty($filterParams['last_interaction_id']) && !empty($filterParams['outbound_emails_sub_type'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.last_activity_subtype = ?', $filterParams['outbound_emails_sub_type']);
            }

            if (!empty($filterParams['total_opportunity_amount_from']) || !empty($filterParams['total_opportunity_amount_to']) || !empty($filterParams['lead_opportunity_stages']) || !empty($filterParams['opportunity_type_id'])) {
                $withOpportunityAmount = true;
            }

            if (!empty($filterParams['lead_opportunity_stages'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }

                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plop.opportunity_stage_id IN (?)', $filterParams['lead_opportunity_stages']);
            }

            if (!empty($filterParams['opportunity_type_id'])) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }

                $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plop.opportunity_type_id IN (?)', $filterParams['opportunity_type_id']);
            }

            if (!empty($filterParams['lead_task'])) {
                $withTasks = true;
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                if ($filterParams['lead_task'] === Api_Leads_Leads::FILTER_TASK_BOTH) {
                    $where .= new Zend_Db_Expr('plta.id IS NOT NULL');
                }
                if ($filterParams['lead_task'] === Api_Leads_Leads::FILTER_TASK_OPENED) {
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plta.is_task_completed = ?', '0');
                }
                if ($filterParams['lead_task'] === Api_Leads_Leads::FILTER_TASK_CLOSED) {
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plta.is_task_completed = ?', '1');
                }
                if ($filterParams['lead_task'] === Api_Leads_Leads::FILTER_TASK_OVERDUE) {
                    $where .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plta.is_task_overdue = ?', '1');
                    $where .= ' AND '. $leadDataMapper->getDbTable()->getAdapter()->quoteInto('plta.is_task_completed = ?', '0');
                }
                if ($filterParams['lead_task'] === Api_Leads_Leads::FILTER_TASK_NONE) {
                    $where .= new Zend_Db_Expr('plta.id IS NULL');
                }

            }

            if (!empty($filterParams['total_opportunity_amount_from'])) {
                if (!empty($having)) {
                    $having .= ' AND ';
                }
                if (empty($filterParams['next_activity_from'])) {
                    $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('SUM(plop.opportunity_value)/ COUNT(DISTINCT(ifnull(CONCAT(`plht`.`tag_id`, `plht`.`lead_id`), 1))) >= ?',
                        $filterParams['total_opportunity_amount_from']);
                } else {
                    $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('(SUM(plop.opportunity_value)/ COUNT(DISTINCT(ifnull(CONCAT(`plht`.`tag_id`, `plht`.`lead_id`), 1)))/COUNT(DISTINCT(ifnull(CONCAT(`plal`.`id`, `plal`.`lead_id`), 1)))) >= ?',
                        $filterParams['total_opportunity_amount_from']);
                }
            }
            if (!empty($filterParams['total_opportunity_amount_to'])) {
                if (!empty($having)) {
                    $having .= ' AND ';
                }
                if (empty($filterParams['next_activity_from'])) {
                    $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('SUM(plop.opportunity_value)/ COUNT(DISTINCT(ifnull(CONCAT(`plht`.`tag_id`, `plht`.`lead_id`), 1))) <= ?',
                        $filterParams['total_opportunity_amount_to']);
                } else {
                    $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('(SUM(plop.opportunity_value)/ COUNT(DISTINCT(ifnull(CONCAT(`plht`.`tag_id`, `plht`.`lead_id`), 1)))/COUNT(DISTINCT(ifnull(CONCAT(`plal`.`id`, `plal`.`lead_id`), 1)))) <= ?',
                        $filterParams['total_opportunity_amount_to']);
                }
            }

            if (!empty($filterParams['total_spent_purchase_amount_from'])) {
                if (!empty($having)) {
                    $having .= ' AND ';
                }
                $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('SUM(scs.total)/ COUNT(DISTINCT(ifnull(CONCAT(`plht`.`tag_id`, `plht`.`lead_id`), 1))) >= ?',
                    $filterParams['total_spent_purchase_amount_from']);
            }

            if (!empty($filterParams['total_spent_purchase_amount_to'])) {
                if (!empty($having)) {
                    $having .= ' AND ';
                }
                $having .= $leadDataMapper->getDbTable()->getAdapter()->quoteInto('SUM(scs.total)/ COUNT(DISTINCT(ifnull(CONCAT(`plht`.`tag_id`, `plht`.`lead_id`), 1))) <= ?',
                    $filterParams['total_spent_purchase_amount_to']);
            }

            if (!empty($filterParams['interaction_value'])) {
                if (in_array($filterParams['interaction_value'], Api_Leads_Leads::$_sortingTypes, true)) {
                    $sortOrder = 'pld.' . $filterParams['interaction_value'] . ' DESC';
                    if ($filterParams['interaction_value'] === Api_Leads_Leads::SORT_ORDER_LAST_ACTIVITY_INBOUND || $filterParams['interaction_value'] === Api_Leads_Leads::SORT_ORDER_LAST_ACTIVITY_OUTBOUND) {
                        $orderInteractionParam = $filterParams['interaction_value'];
                    }
                }
            }

        } elseif (!empty($searchTerm)) {
            $where = $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_email LIKE ?',
                $searchTerm . '%');
        } elseif (!empty($email)) {
            $where = $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.lead_email = ?', $email);
        }

        return $leadDataMapper->fetchAll($where, $sortOrder, $limit, $offset, false, false, $withOpportunityAmount,
            $having, $withNextActivityDate, $withTasks, $callDuration, $includesNotes, $includesCustomLeadFields,
            $includesCustomOrganizationFields, $withOpportunityExpected, $withSpentPurchaseAmount, $withEmailLog,
            $withOrganizationIndustry, $additionalParamsForSelect, $additionalOrganizationParamsForSelect, $withTimezoneOffset, $includePluginLeadsTypeCpecificColl, $withEmailSmsSequence);

    }

    /**
     * Get timezone shift
     *
     * @param $timezoneFrom
     * @param $timeZoneToShift
     * @return float
     */
    public static function getTimezoneShift($timezoneFrom, $timeZoneToShift)
    {
        $dateTimeZoneServer = new DateTimeZone($timezoneFrom);
        $dateTimeZoneStore = new DateTimeZone($timeZoneToShift);
        $dateTimeServer = new DateTime("now", $dateTimeZoneServer);

        $timeOffsetInHours = $dateTimeZoneStore->getOffset($dateTimeServer) / 3600;

        return $timeOffsetInHours;
    }

    /**
     * Is leads allowed for the user
     *
     * @param string $roleId role id
     * @return bool
     */
    public static function isLeadAllowed($roleId)
    {
        $session = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
        $userModel = $session->getCurrentUser();
        $userRole = $userModel->getRoleId();
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            if ($userRole === $roleId) {
                if ($roleId === Shopping::ROLE_SALESPERSON) {
                    $salesPersonLimitedAccess = Leads_Mapper_LeadsConfigMapper::getInstance()->getConfigParam(Api_Leads_Leadgeneralconfig::SALES_PERSON_LIMITED_ACCESS);
                    if (empty($salesPersonLimitedAccess)) {
                        return true;
                    }
                }

                return false;
            }

            return true;
        }

        return false;
    }

    /**
     * Is leads allowed for the user
     *
     * @param string $roleId role id
     * @param int $leadOwnerId lead owner id
     * @return bool
     * @throws Zend_Db_Table_Exception
     */
    public static function isAccessLimitedByUserId($roleId, $leadOwnerId)
    {
        $userModel = Application_Model_Mappers_UserMapper::getInstance()->find($leadOwnerId);
        if (!$userModel instanceof Application_Model_Models_User) {
            return true;
        }
        $userRole = $userModel->getRoleId();
        if (in_array($userRole, array(Shopping::ROLE_SALESPERSON, Tools_Security_Acl::ROLE_SUPERADMIN, Tools_Security_Acl::ROLE_ADMIN))) {
            if ($userRole === $roleId) {
                if ($roleId === Shopping::ROLE_SALESPERSON) {
                    $salesPersonLimitedAccess = Leads_Mapper_LeadsConfigMapper::getInstance()->getConfigParam(Api_Leads_Leadgeneralconfig::SALES_PERSON_LIMITED_ACCESS);
                    if (empty($salesPersonLimitedAccess)) {
                        return false;
                    }
                }

                return true;
            }

            return false;
        }

        return true;
    }

    /**
     * Get current logged user id
     *
     * @return mixed
     */
    public static function getLoggedUserId()
    {
        $session = Zend_Controller_Action_HelperBroker::getStaticHelper('session');

        return $session->getCurrentUser()->getId();
    }

    /**
     *
     * Get current logged id if action partially allowed
     * for example if full access allowed only for the admin and su-admin
     * and not allowed for the sales person
     *
     * @param string $roleId role id
     *
     * @return int
     */
    public static function getUserIdIfActionNotAllowed($roleId)
    {
        $isAllowed = self::isLeadAllowed($roleId);
        if ($isAllowed === false) {
            return self::getLoggedUserId();
        }

        return 0;
    }

    /**
     * Get default email template message.
     * return content inside the magic space
     * {defaultemailcontent} ... {/defaultemailcontent}
     *
     * @param string $templateId email template id
     * @return string
     */
    public static function getDefaultEmailMessage($templateId)
    {
        $result = '';
        $templateModel = Application_Model_Mappers_TemplateMapper::getInstance()->find($templateId);
        if ($templateModel instanceof Application_Model_Models_Template) {
            $templateContent = $templateModel->getContent();
            $websiteHelper = Zend_Controller_Action_HelperBroker::getExistingHelper('website');
            $themeData = Zend_Registry::get('theme');
            $extConfig = Zend_Registry::get('extConfig');
            $parserOptions = array(
                'websiteUrl' => $websiteHelper->getUrl(),
                'websitePath' => $websiteHelper->getPath(),
                'currentTheme' => $extConfig['currentTheme'],
                'themePath' => $themeData['path']
            );
            $pageModel = Tools_LeadTools::getTemplatePage($templateId);
            if ($pageModel instanceof Application_Model_Models_Page) {
                $pageMapper = Application_Model_Mappers_PageMapper::getInstance();
                $pageModel = $pageMapper->findByUrl($pageModel->getUrl());
                $parser = new Tools_Content_Parser($templateContent, $pageModel->toArray(), $parserOptions);
            } else {
                $parser = new Tools_Content_Parser($templateContent, array(), $parserOptions);
            }

            $emailTemplateDefaultMessage = $parser->parseWidgetsOnly();
            if (Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_CONTENT)) {
                $emailTemplateDefaultMessage = self::stripEditLinks($emailTemplateDefaultMessage);
            }

            preg_match('~{defaultemailcontent}(.*){\/defaultemailcontent}~uiUs', $emailTemplateDefaultMessage, $matches);
            if (!empty($matches[1])) {
                $result = $matches[1];
            }
        }

        return $result;
    }

    /**
     * Cut out edit links from contents.
     */
    public static function stripEditLinks($string) {
        if (Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_CONTENT)) {
            return preg_replace('/<a class=\"tpopup generator-links\".*>.*<\/a>/', '', $string);
        }
        return $string;
    }

    /**
     * Get first page with current template
     *
     * @param string $templateId template id
     * @return mixed|null
     */
    public static function getTemplatePage($templateId)
    {
        $pageMapper = Application_Model_Mappers_PageMapper::getInstance();
        $where = $pageMapper->getDbTable()->getAdapter()->quoteInto('template_id = ?', $templateId);
        $pageModels = Application_Model_Mappers_PageMapper::getInstance()->fetchAll($where, array(), true);
        if (!empty($pageModels)) {
            return current($pageModels);
        }
        return null;
    }

    /**
     * Gert default action email info
     *
     * @param string $triggerName trigger name
     * @return array
     */
    public static function getDefaultActionEmailInfo($triggerName)
    {
        $triggerActionsDbTable = new Application_Model_DbTable_TriggersActions();
        $where = $triggerActionsDbTable->getAdapter()->quoteInto('`trigger` = ?', $triggerName);
        $selectActiveActionEmailTriggers = $triggerActionsDbTable->select()->setIntegrityCheck(false)->from('email_triggers_actions',
            array('trigger', 'id', 'subject', 'template'))
            ->where($where)->limit(1);

        return $triggerActionsDbTable->getAdapter()->fetchRow($selectActiveActionEmailTriggers);
    }

    /**
     * Returned data with trim function
     *
     * @param $data
     * @param bool $uniqueArrayParam
     * @return array|string
     */
    public static function trimValues($data, $uniqueArrayParam = false)
    {
        if (is_string($data)) {
            $data = array_map('trim', explode(',', $data));
            if ($uniqueArrayParam) {
                $data = array_unique($data);
            }
            $data = implode(',', $data);

        } elseif (is_array($data)) {
            $data = array_map('trim', $data);
            if ($uniqueArrayParam) {
                $data = array_unique($data);
            }
        }

        return $data;
    }

    public static function getTimelineData($limit, $offset, $sortOrder, $id, $timeLineType, $leadId, $dateTodayEnd, $dateTodayStart, $filterParams, $withTimezoneOffset = true)
    {
        $leadsActionLogMapper = Leads_Mapper_LeadsActionLogMapper::getInstance();

        if ($id) {
            $where = $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plac.id = ?', $id);
            $data = $leadsActionLogMapper->fetchAll($where, null, null, null, true, true);
        } else {
            $where = null;
            if (!empty($leadId)) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }

                if(is_array($leadId)) {
                    $where .= $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plal.lead_id IN (?) ', $leadId);
                } else {
                    $where .= $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plal.lead_id = ? ', $leadId);
                }
            }
            if (!empty($filterParams)) {
                if (!empty($filterParams['filter-by-action-type']) && $filterParams['filter-by-action-type'] !== 'all') {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $where .= $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plal.action_type = ? ', $filterParams['filter-by-action-type']);
                }
                if (!empty($filterParams['filter-by-type'])) {
                    if (!empty($where)) {
                        $where .= ' AND ';
                    }
                    $filterByType = explode(',', $filterParams['filter-by-type']);
                    $where .= $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plal.type IN (?)', $filterByType);
                }
                if (!empty($filterParams['filter-by-sub-type'])) {
                    $filterSubType = $filterParams['filter-by-sub-type'];
                    if ($filterSubType && $filterParams['filter-by-type'] === 'email') {
                        if (!empty($where)) {
                            $where .= ' AND ';
                        }

                        $where .= $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plel.email_status = ? ', $filterParams['filter-by-sub-type']);
                    }

                    if ($filterSubType && $filterParams['filter-by-type'] === 'sms') {
                        if (!empty($where)) {
                            $where .= ' AND ';
                        }

                        $where .= $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plcl.sub_type = ? ', $filterParams['filter-by-sub-type']);
                    }

                    if ($filterSubType && in_array('task', $filterByType, true)) {
                        if (!empty($where)) {
                            $where .= ' AND ';
                        }

                        if ($filterParams['filter-by-sub-type'] === 'opened') {
                            $where .= $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('pltl.is_task_completed = ? ',
                                '0');
                        }
                        if ($filterParams['filter-by-sub-type'] === 'completed') {
                            $where .= $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('pltl.is_task_completed = ? ',
                                '1');
                        }

                        if ($filterParams['filter-by-sub-type'] === 'overdue') {
                            $where .= $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('pltl.is_task_completed = ? ',
                                '0');
                            $where .= ' AND '. $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('pltl.is_task_overdue = ? ','1');
                        }


                    }

                    if ($filterSubType && $filterParams['filter-by-type'] === 'purchase') {
                        if (!empty($where)) {
                            $where .= ' AND ';
                        }

                        if ($filterSubType === Models_Model_CartSession::CART_STATUS_PARTIAL) {
                            $where .= $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plpl.is_partial = ? ',
                                '1');
                        } else {
                            $where .= $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plpl.is_partial = ? ',
                                '0');
                            $where .= ' AND '. $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plpl.status = ? ',
                                $filterSubType);
                        }
                    }
                }
            }

            if (!empty($where)) {
                $where .= ' AND ';
            }

            $where .=  '(' .$leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plql.status <> ?', 'new');
            $where .=  ' OR '. new Zend_Db_Expr('plql.status IS NULL');
            $where .=  ' OR '. $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plql.force_display = ?', '1').')';

            if ($timeLineType === self::TIMELINE_TYPE_TODAY) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }

                $where .=  $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plal.created_at > ?', $dateTodayStart);
                $where .= ' AND '. $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plal.created_at < ?', $dateTodayEnd);
                $data = $leadsActionLogMapper->fetchAll($where, $sortOrder, $limit, $offset);
            } elseif($timeLineType === self::TIMELINE_TYPE_UPCOMING) {
                if (!empty($where)) {
                    $where .= ' AND ';
                }
                $where .= $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plal.event_type = ?', Leads_Model_LeadsActionLogModel::EVENT_LOG_UPCOMING);
                $where .= ' AND '. $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plal.created_at > ?', $dateTodayEnd);
                $data = $leadsActionLogMapper->fetchAll($where, $sortOrder, $limit, $offset);
            } else {
                if (!empty($where)) {
                    $where .= ' AND ';
                }

                $where .= $leadsActionLogMapper->getDbTable()->getAdapter()->quoteInto('plal.created_at < ?', $dateTodayStart);
                $data = $leadsActionLogMapper->fetchAll($where, $sortOrder, $limit, $offset, false, false, $withTimezoneOffset);
            }
        }

        if (empty($data)) {
            $data = array();
        }

        return $data;
    }

    /**
     * Check if plugin enabled
     *
     * @param string $pluginName plugin name
     *
     * @return bool
     */
    public static function isPluginEnabled($pluginName)
    {
        $plugin = Tools_Plugins_Tools::findPluginByName($pluginName);
        if ($plugin instanceof Application_Model_Models_Plugin) {
            return $plugin->getStatus() === 'enabled' ? true : false;
        }
        return false;
    }

    /**
     * Check if plugin installed
     *
     * @param string $pluginName plugin name
     *
     * @return bool
     */
    public static function isPluginInstalled($pluginName)
    {
        $availablePlugins = Tools_Plugins_Tools::findAvailablePlugins();
        if (!in_array(strtolower($pluginName), $availablePlugins, true)) {
            return false;
        }
        return true;
    }

    /**
     * Get default owner id
     *
     * @return int
     * @throws Zend_Db_Table_Exception
     */
    public static function getDefaultLeadOwner()
    {
        $userMapper = Application_Model_Mappers_UserMapper::getInstance();
        $leadConfigMapper = Leads_Mapper_LeadsConfigMapper::getInstance();
        $ownerIdTo = $leadConfigMapper->getConfigParam('defaultLeadOwner');
        $ownerToModel = null;
        if (!empty($ownerIdTo)) {
            $ownerToModel = $userMapper->find($ownerIdTo);
        }

        if ($ownerToModel instanceof Application_Model_Models_User) {
            $roleId = $ownerToModel->getRoleId();
            if (!in_array($roleId, Leads::$_administrativeRoles, true)) {
                $ownerToModel = null;
            }
        }

        if (!$ownerToModel instanceof Application_Model_Models_User) {
            $ownerToModel = $userMapper->findByRole(Tools_Security_Acl::ROLE_SUPERADMIN);
        }

        $ownerIdTo = $ownerToModel->getId();

        return (int) $ownerIdTo;
    }

    /**
     * Get lead screen links
     *
     * @param array $userIds lead ids
     * @return array
     * @throws Zend_Controller_Action_Exception
     */
    public static function getLeadLink($userIds)
    {
        $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
        $leadsData = $leadMapper->findByUserIds($userIds);
        $leadLinksData = array();
        if (!empty($leadsData)) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getExistingHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            foreach ($leadsData as $leadData) {
                $leadLinksData[$leadData['user_id']] = $websiteUrl.'dashboard/leads/#lead/'.$leadData['id'];
            }
        }

        return $leadLinksData;
    }

    /**
     * @param $fileName
     */
    public static function downloadFile($fileName)
    {
        $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
        $filePath = $websiteHelper->getPath() . $websiteHelper->getTmp() . $fileName;
        $response = Zend_Controller_Front::getInstance()->getResponse();
        $response->setHeader(
            'Content-Disposition',
            'attachment; filename=' . $fileName
        )
            ->setHeader('Content-type', 'application/force-download');
        readfile($filePath);
        $response->sendResponse();
        exit;
    }


    /**
     * Glue and normalize phone
     *
     * @param string $userMobileCode
     * @param string $userMobile
     * @param string $userPhoneCode
     * @param string $userPhone
     * @return bool|string
     */
    public static function glueUserPhone($userMobileCode = '', $userMobile = '', $userPhoneCode = '', $userPhone = '') {
        $mobilePhoneUser = '';

        if(!empty($userMobileCode) && !empty($userMobile)) {
            $mobilePhoneUser = $userMobileCode. $userMobile;
            $mobilePhoneUser = Apps_Tools_Twilio::normalizePhoneNumberToE164($mobilePhoneUser);
        } else if(!empty($userPhoneCode) && !empty($userPhone)) {
            $mobilePhoneUser = $userPhoneCode. $userPhone;
            $mobilePhoneUser = Apps_Tools_Twilio::normalizePhoneNumberToE164($mobilePhoneUser);
        }

        return $mobilePhoneUser;
    }

    /**
     * Allow sales access
     *
     * @param int $leadId lead id
     * @return bool
     */
    public static function allowSales($leadId)
    {
        $session = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
        $userModel = $session->getCurrentUser();
        $userId = $userModel->getId();
        $userRoleId = $userModel->getRoleId();

        $isAllow = Tools_LeadTools::isLeadAllowed($userRoleId);

        if(!$isAllow) {
            $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
            $leadModel = $leadMapper->find($leadId);
            if ($leadModel instanceof Leads_Model_LeadsModel) {
                if($userId == $leadModel->getOwnerId()) {
                    $isAllow = true;
                }
            }
        }

        return $isAllow;

    }

    /**
     * @param $serviceType
     * @return array
     */
    public static function getEnabledServices($serviceType)
    {
        if(!empty($serviceType)) {
            $enabledServices = Apps_Models_Mapper_AppsSettingsMapper::getInstance()->selectConfigByStatusCategory('1',
                $serviceType, array('additionalLists' => '1'));

            $categoryAndServices = Apps::apiCall('GET', 'apps', array('services'), array('withcrm' => 'withcrm', 'withSambaEmailMarketing' => 'withSambaEmailMarketing'));

            if (!empty($enabledServices)) {
                foreach ($enabledServices as $enableService)  {
                    if (!empty($categoryAndServices) && !empty($categoryAndServices['services'])
                        && !empty($categoryAndServices['services'][$enableService['category']])
                        && !empty($categoryAndServices['services'][$enableService['category']][$enableService['service_name']])) {

                        if (empty($categoryAndServices['services'][$enableService['category']][$enableService['service_name']]['alt_label'])) {
                            $enabledServices[$enableService['service_name']]['service_label'] = $categoryAndServices['services'][$enableService['category']][$enableService['service_name']]['name'];
                        } else {
                            $enabledServices[$enableService['service_name']]['service_label'] = $categoryAndServices['services'][$enableService['category']][$enableService['service_name']]['alt_label'];
                        }
                    } else {
                        $enabledServices[$enableService['service_name']]['service_label'] = $enabledServices['service_name'];
                    }
                }
                return $enabledServices;
            }
        }

        return array();
    }

    /**
     * Reprocess camel case keys to underscore
     *
     * @param array $dataArray
     * @return array
     */
    public static function camelCaseKeysToUnderscore($dataArray, $withValidateTextField = false)
    {
        $newArray = array();
        foreach ($dataArray as $key => $val) {
            if($withValidateTextField) {
                if (!is_array($val)) {
                    $val = self::validateTextField($val);
                }
            }

            $newArray[strtolower(preg_replace('/(?<!^)[A-Z]/', '_$0', $key))] = $val;
        }

        return $newArray;
    }

    /**
     * Filter string except quotes
     *
     * @param $param
     * @return mixed
     */
    public static function validateTextField($param) {
        if(!empty($param)) {
            $param = filter_var($param, FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES);
        }

        return $param;
    }

    /**
     * Allow single quote and white space
     *
     * @param $param
     * @param $customMask
     * @return bool
     */
    public static function customParamValidate($param, $customMask = '') {
        $notValidParam = false;
        if (!empty($customMask)) {
            if (preg_match($customMask, $param)) {
                $notValidParam = true;
            }
        } else {
            if (preg_match('~[^\'\s\w-]~ui', $param)) {
                $notValidParam = true;
            }
        }

        return $notValidParam;
    }

    /**
     * Sync user group
     *
     * @param int $userId user id
     * @param int $groupId group id
     */
    public static function syncUserGroup($userId, $groupId)
    {
        $customerMapper = Models_Mapper_CustomerMapper::getInstance();
        if (!empty($userId) && !empty($groupId)) {
            $userModel = Application_Model_Mappers_UserMapper::getInstance()->find($userId);
            if ($userModel instanceof Application_Model_Models_User) {
                $groupMapper = Store_Mapper_GroupMapper::getInstance();
                $groupModel = $groupMapper->find($groupId);
                if ($groupModel instanceof Store_Model_Group) {
                    $customerModel = $customerMapper->find($userId);
                    $customerModel->setGroupId($groupId);
                    $customerMapper->save($customerModel);
                }
            }

        } elseif(!empty($userId)) {
            $userModel = Application_Model_Mappers_UserMapper::getInstance()->find($userId);
            if ($userModel instanceof Application_Model_Models_User) {
                $customerModel = $customerMapper->find($userId);
                $customerModel->setGroupId(null);
                $customerMapper->save($customerModel);
            }
        }

    }

    /**
     * Get advertising account connection on mojo
     *
     * @param $advAccount
     * @return mixed|string[]
     * @throws Exceptions_SeotoasterPluginException
     */
    public static function sambastatsConnectionStatusGet($advAccount)
    {
        if(!empty($advAccount)) {
            $services = Apps::apiCall('GET', 'sambastatsConnectionStatus', array(), array('type' => $advAccount));

            return $services;
        }

        return array(
            'error' => '1',
            'message' => 'Advertising account not found'
        );
    }

    public static function parseHrefFromHtml($html)
    {
        $doc = new DOMDocument();
        /**
         * LIBXML_HTML_NOIMPLIED turns off the automatic adding of implied html/body elements
         * LIBXML_HTML_NODEFDTD prevents a default doctype being added when one is not found.
         */
        $doc->loadHTML(mb_convert_encoding('<html><body>' . $html . '</body></html>', 'HTML-ENTITIES', 'UTF-8'),
            LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD);

        $xpath = new DOMXpath($doc);
        foreach ($xpath->query('//a') as $fileLinkEl) {
            $hrefValue = $fileLinkEl->getAttribute('href');
            $htmlString = $doc->saveHTML($fileLinkEl);
            $html = str_replace($htmlString, $hrefValue, $html);
        }

        return $html;
    }

    /*
     * Return Organization logo by user ID
     */
    public static function getOrganizationLogo($userId)
    {
        $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
        $leadModel = $leadMapper->findByUserId($userId);

        if($leadModel instanceof Leads_Model_LeadsModel) {
            $organizationId = $leadModel->getOrganizationId();

            if(!empty($organizationId)) {
                $organizationDocumentData = $leadMapper->findOrganizationDocumentByUserId($userId);

                return $organizationDocumentData;
            }
        }

        return array();
    }

    /**
     * find lead by phone or mobile emulating leading zero number
     * Mobile or phone validated via libphonenumber library
     *
     * @param string $phone (phone or mobile number)
     * @param string $countryCode country code ISO code
     * @param bool $fullPhone full number flag
     * @param int $excludeId exclude lead id
     * @return null
     */
    public static function findLeadByPhoneMobileNumber($phone, $fullPhone = true, $excludeId = 0, $countryCode = '')
    {
        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();

        $mobilesWithAndWithoutLeadingNumber = Tools_LeadPhoneLibTools::getPhonesWithAndWithoutLeadingZero($phone, $countryCode);

        if (!empty($mobilesWithAndWithoutLeadingNumber)) {
            $leadModel = $leadsMapper->findByPhoneNumberOrMobile($mobilesWithAndWithoutLeadingNumber['phoneWithZero'], $fullPhone, $excludeId);
            if (!$leadModel instanceof Leads_Model_LeadsModel) {
                $leadModel = $leadsMapper->findByPhoneNumberOrMobile($mobilesWithAndWithoutLeadingNumber['phoneWithoutZero'], $fullPhone, $excludeId);
            }

            return $leadModel;
        } else {
            return $leadsMapper->findByPhoneNumberOrMobile($phone, $fullPhone, $excludeId);
        }
    }


    public static function leadUsefulShortcuts($disableForSms = 0, $withQuoteWidgets = false)
    {
        $widgets = array();
        $translator = Zend_Registry::get('Zend_Translate');
        array_push($widgets, array('alias' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead first name'), 'option' => '$lead:firstname'));
        array_push($widgets, array('alias' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead last name'), 'option' => '$lead:lastname'));
        if(!$disableForSms) {
            array_push($widgets, array('alias' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead full name (link)'), 'option' => '$lead:leadfullnamelink'));
        }
        array_push($widgets, array('alias' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead desktop'), 'option' => '$lead:fulldesktopphone'));
        array_push($widgets, array('alias' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead mobile'), 'option' => '$lead:fullmobilephone'));
        array_push($widgets, array('alias' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead prefix'), 'option' => '$lead:prefix'));
        array_push($widgets, array('alias' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead position'), 'option' => '$lead:position'));
        array_push($widgets, array('alias' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead middle name'), 'option' => '$lead:middlename'));
        array_push($widgets, array('alias' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead source'), 'option' => '$lead:source'));
        array_push($widgets, array('alias' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead owner full name'), 'option' => '$leadowner:fullname'));
        array_push($widgets, array('alias' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead owner email'), 'option' => '$leadowner:email'));
        array_push($widgets, array('alias' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead owner mobile phone'), 'option' => '$leadowner:fullmobilephone'));
        array_push($widgets, array('alias' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead owner desktop phone'), 'option' => '$leadowner:fulldesktopphone'));
        array_push($widgets, array('alias' => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead owner signature'), 'option' => '$leadowner:signature'));
        array_push($widgets, array('alias' => $translator->translate(Leads::TRANSLATION_PREFIX.'Organization name'), 'option' => '$lead:organizationname'));
        if(!$disableForSms) {
            array_push($widgets, array('alias' => $translator->translate(Leads::TRANSLATION_PREFIX.'Organization name (link)'), 'option' => '$lead:organizationnamelink'));
        }
        array_push($widgets, array('alias' => $translator->translate(Leads::TRANSLATION_PREFIX.'Organization logo'), 'option' => '$lead:organizationlogo'));
        array_push($widgets, array('alias' => $translator->translate(Leads::TRANSLATION_PREFIX.'Organization logo (src)'), 'option' => '$lead:organizationlogo:src'));
        array_push($widgets, array('alias' => $translator->translate(Leads::TRANSLATION_PREFIX.'Organization website'), 'option' => '$lead:organizationwebsite'));
        array_push($widgets, array('alias' => $translator->translate(Leads::TRANSLATION_PREFIX.'Personal calendar url'), 'option' => '$leadowner:personalcalendarurl'));
        if(!$disableForSms) {
            array_push($widgets, array('alias' => $translator->translate(Leads::TRANSLATION_PREFIX.'Avatar link'), 'option' => '$leadowner:avatarlink'));
        }
        array_push($widgets, array('alias' => $translator->translate(Leads::TRANSLATION_PREFIX.'Avatar link (src)'), 'option' => '$leadowner:avatarlink:src'));

        if ($withQuoteWidgets === true) {
            array_push($widgets, array('alias' => $translator->translate('Quote title'), 'option' => '$lead:quotetitlelink'));
            array_push($widgets, array('alias' => $translator->translate('Quote total'), 'option' => '$lead:quoteamount'));
        }

        return $widgets;
    }

    /**
     * @param bool $fullName flag to get states full names
     * @return array|null
     */
    public static function getStatesList($fullName = false)
    {
        if ($fullName === true) {
            return self::getStates(null, true, false, 'name');
        } else {
            return self::getStates(null, true, true, 'name');
        }
    }

    /**
     * Get country list names
     *
     * @return array
     */
    public static function countryListNames()
    {

        return array (
            'AC' => 'Ascension Island',
            'AD' => 'Andorra',
            'AE' => 'United Arab Emirates',
            'AF' => 'Afghanistan',
            'AG' => 'Antigua & Barbuda',
            'AI' => 'Anguilla',
            'AL' => 'Albania',
            'AM' => 'Armenia',
            'AO' => 'Angola',
            'AQ' => 'Antarctica',
            'AR' => 'Argentina',
            'AS' => 'American Samoa',
            'AT' => 'Austria',
            'AU' => 'Australia',
            'AW' => 'Aruba',
            'AX' => 'Åland Islands',
            'AZ' => 'Azerbaijan',
            'BA' => 'Bosnia & Herzegovina',
            'BB' => 'Barbados',
            'BD' => 'Bangladesh',
            'BE' => 'Belgium',
            'BF' => 'Burkina Faso',
            'BG' => 'Bulgaria',
            'BH' => 'Bahrain',
            'BI' => 'Burundi',
            'BJ' => 'Benin',
            'BL' => 'St. Barthélemy',
            'BM' => 'Bermuda',
            'BN' => 'Brunei',
            'BO' => 'Bolivia',
            'BQ' => 'Caribbean Netherlands',
            'BR' => 'Brazil',
            'BS' => 'Bahamas',
            'BT' => 'Bhutan',
            'BW' => 'Botswana',
            'BY' => 'Belarus',
            'BZ' => 'Belize',
            'CA' => 'Canada',
            'CC' => 'Cocos (Keeling) Islands',
            'CD' => 'Congo - Kinshasa',
            'CF' => 'Central African Republic',
            'CG' => 'Congo - Brazzaville',
            'CH' => 'Switzerland',
            'CI' => 'Côte d’Ivoire',
            'CK' => 'Cook Islands',
            'CL' => 'Chile',
            'CM' => 'Cameroon',
            'CN' => 'China',
            'CO' => 'Colombia',
            'CR' => 'Costa Rica',
            'CU' => 'Cuba',
            'CV' => 'Cape Verde',
            'CW' => 'Curaçao',
            'CX' => 'Christmas Island',
            'CY' => 'Cyprus',
            'CZ' => 'Czechia',
            'DE' => 'Germany',
            'DG' => 'Diego Garcia',
            'DJ' => 'Djibouti',
            'DK' => 'Denmark',
            'DM' => 'Dominica',
            'DO' => 'Dominican Republic',
            'DZ' => 'Algeria',
            'EA' => 'Ceuta & Melilla',
            'EC' => 'Ecuador',
            'EE' => 'Estonia',
            'EG' => 'Egypt',
            'EH' => 'Western Sahara',
            'ER' => 'Eritrea',
            'ES' => 'Spain',
            'ET' => 'Ethiopia',
            'FI' => 'Finland',
            'FJ' => 'Fiji',
            'FK' => 'Falkland Islands',
            'FM' => 'Micronesia',
            'FO' => 'Faroe Islands',
            'FR' => 'France',
            'GA' => 'Gabon',
            'GB' => 'United Kingdom',
            'GD' => 'Grenada',
            'GE' => 'Georgia',
            'GF' => 'French Guiana',
            'GG' => 'Guernsey',
            'GH' => 'Ghana',
            'GI' => 'Gibraltar',
            'GL' => 'Greenland',
            'GM' => 'Gambia',
            'GN' => 'Guinea',
            'GP' => 'Guadeloupe',
            'GQ' => 'Equatorial Guinea',
            'GR' => 'Greece',
            'GS' => 'South Georgia & South Sandwich Islands',
            'GT' => 'Guatemala',
            'GU' => 'Guam',
            'GW' => 'Guinea-Bissau',
            'GY' => 'Guyana',
            'HK' => 'Hong Kong SAR China',
            'HN' => 'Honduras',
            'HR' => 'Croatia',
            'HT' => 'Haiti',
            'HU' => 'Hungary',
            'IC' => 'Canary Islands',
            'ID' => 'Indonesia',
            'IE' => 'Ireland',
            'IL' => 'Israel',
            'IM' => 'Isle of Man',
            'IN' => 'India',
            'IO' => 'British Indian Ocean Territory',
            'IQ' => 'Iraq',
            'IR' => 'Iran',
            'IS' => 'Iceland',
            'IT' => 'Italy',
            'JE' => 'Jersey',
            'JM' => 'Jamaica',
            'JO' => 'Jordan',
            'JP' => 'Japan',
            'KE' => 'Kenya',
            'KG' => 'Kyrgyzstan',
            'KH' => 'Cambodia',
            'KI' => 'Kiribati',
            'KM' => 'Comoros',
            'KN' => 'St. Kitts & Nevis',
            'KP' => 'North Korea',
            'KR' => 'South Korea',
            'KW' => 'Kuwait',
            'KY' => 'Cayman Islands',
            'KZ' => 'Kazakhstan',
            'LA' => 'Laos',
            'LB' => 'Lebanon',
            'LC' => 'St. Lucia',
            'LI' => 'Liechtenstein',
            'LK' => 'Sri Lanka',
            'LR' => 'Liberia',
            'LS' => 'Lesotho',
            'LT' => 'Lithuania',
            'LU' => 'Luxembourg',
            'LV' => 'Latvia',
            'LY' => 'Libya',
            'MA' => 'Morocco',
            'MC' => 'Monaco',
            'MD' => 'Moldova',
            'ME' => 'Montenegro',
            'MF' => 'St. Martin',
            'MG' => 'Madagascar',
            'MH' => 'Marshall Islands',
            'MK' => 'North Macedonia',
            'ML' => 'Mali',
            'MM' => 'Myanmar (Burma)',
            'MN' => 'Mongolia',
            'MO' => 'Macao SAR China',
            'MP' => 'Northern Mariana Islands',
            'MQ' => 'Martinique',
            'MR' => 'Mauritania',
            'MS' => 'Montserrat',
            'MT' => 'Malta',
            'MU' => 'Mauritius',
            'MV' => 'Maldives',
            'MW' => 'Malawi',
            'MX' => 'Mexico',
            'MY' => 'Malaysia',
            'MZ' => 'Mozambique',
            'NA' => 'Namibia',
            'NC' => 'New Caledonia',
            'NE' => 'Niger',
            'NF' => 'Norfolk Island',
            'NG' => 'Nigeria',
            'NI' => 'Nicaragua',
            'NL' => 'Netherlands',
            'NO' => 'Norway',
            'NP' => 'Nepal',
            'NR' => 'Nauru',
            'NU' => 'Niue',
            'NZ' => 'New Zealand',
            'OM' => 'Oman',
            'PA' => 'Panama',
            'PE' => 'Peru',
            'PF' => 'French Polynesia',
            'PG' => 'Papua New Guinea',
            'PH' => 'Philippines',
            'PK' => 'Pakistan',
            'PL' => 'Poland',
            'PM' => 'St. Pierre & Miquelon',
            'PN' => 'Pitcairn Islands',
            'PR' => 'Puerto Rico',
            'PS' => 'Palestinian Territories',
            'PT' => 'Portugal',
            'PW' => 'Palau',
            'PY' => 'Paraguay',
            'QA' => 'Qatar',
            'RE' => 'Réunion',
            'RO' => 'Romania',
            'RS' => 'Serbia',
            'RU' => 'Russia',
            'RW' => 'Rwanda',
            'SA' => 'Saudi Arabia',
            'SB' => 'Solomon Islands',
            'SC' => 'Seychelles',
            'SD' => 'Sudan',
            'SE' => 'Sweden',
            'SG' => 'Singapore',
            'SH' => 'St. Helena',
            'SI' => 'Slovenia',
            'SJ' => 'Svalbard & Jan Mayen',
            'SK' => 'Slovakia',
            'SL' => 'Sierra Leone',
            'SM' => 'San Marino',
            'SN' => 'Senegal',
            'SO' => 'Somalia',
            'SR' => 'Suriname',
            'SS' => 'South Sudan',
            'ST' => 'São Tomé & Príncipe',
            'SV' => 'El Salvador',
            'SX' => 'Sint Maarten',
            'SY' => 'Syria',
            'SZ' => 'Eswatini',
            'TA' => 'Tristan da Cunha',
            'TC' => 'Turks & Caicos Islands',
            'TD' => 'Chad',
            'TF' => 'French Southern Territories',
            'TG' => 'Togo',
            'TH' => 'Thailand',
            'TJ' => 'Tajikistan',
            'TK' => 'Tokelau',
            'TL' => 'Timor-Leste',
            'TM' => 'Turkmenistan',
            'TN' => 'Tunisia',
            'TO' => 'Tonga',
            'TR' => 'Turkey',
            'TT' => 'Trinidad & Tobago',
            'TV' => 'Tuvalu',
            'TW' => 'Taiwan',
            'TZ' => 'Tanzania',
            'UA' => 'Ukraine',
            'UG' => 'Uganda',
            'UM' => 'U.S. Outlying Islands',
            'US' => 'United States',
            'UY' => 'Uruguay',
            'UZ' => 'Uzbekistan',
            'VA' => 'Vatican City',
            'VC' => 'St. Vincent & Grenadines',
            'VE' => 'Venezuela',
            'VG' => 'British Virgin Islands',
            'VI' => 'U.S. Virgin Islands',
            'VN' => 'Vietnam',
            'VU' => 'Vanuatu',
            'WF' => 'Wallis & Futuna',
            'WS' => 'Samoa',
            'XK' => 'Kosovo',
            'YE' => 'Yemen',
            'YT' => 'Mayotte',
            'ZA' => 'South Africa',
            'ZM' => 'Zambia',
            'ZW' => 'Zimbabwe',
        );
    }

    /**
     * Parse lexems and content
     *
     * @param string $content any content
     * @param object $observableModel
     * @return null|string
     * @throws Zend_Exception
     */
    public static function parseLexemsAndContent($content, $observableModel = null)
    {
        if (!empty($observableModel)) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getExistingHelper('website');
            $entityParser = new Tools_Content_EntityParser();
            $content = $entityParser->parse($content);
            $themeData = Zend_Registry::get('theme');
            $extConfig = Zend_Registry::get('extConfig');
            $parserOptions = array(
                'websiteUrl' => $websiteHelper->getUrl(),
                'websitePath' => $websiteHelper->getPath(),
                'currentTheme' => $extConfig['currentTheme'],
                'themePath' => $themeData['path'],
                'observableModel' => $observableModel
            );
            $parser = new Tools_Content_Parser($content, array(), $parserOptions);

            $content = $parser->parse();

            return $content;
        }
    }

}
