<?php

class Tools_LeadsMailWatchdog implements Interfaces_Observer
{

    /**
     * email sent from dashboard to lead
     */
    const LEADS_CONTACT = 'leads_contact';

    /**
     * task overdue reminder email
     */
    const LEADS_TASK_REMINDER = 'leads_taskreminder';

    /**
     * task created
     */
    const LEADS_TASK_CREATED = 'leads_taskcreated';

    /**
     * meeting  email
     */
    const LEADS_MEETING = 'leads_meeting';

    /**
     * License limit action email
     */
    const LEADS_LICENSE_LIMIT = 'leads_licenselimit';

    /**
     * Lead task completed
     */
    const LEAD_TASK_COMPLETED = 'leads_taskcompleted';

    /**
     *  task opened again
     */
    const LEAD_TASK_REOPENED = 'leads_taskreopened';

    /**
     *  task overdue
     */
    const LEAD_TASK_OVERDUE = 'leads_taskoverdue';

    /**
     *  task overdue
     */
    const LEAD_TASK_START = 'leads_taskstart';

    /**
     *  wrong imap credentials
     */
    const LEAD_IMAP_CREDENTIALS_NOTIFICATION = 'leads_imapcredentialsnotification';


    const LEADS_REVIEW_REQUEST = 'leads_reviewrequest';

    const LEADS_NPS_REVIEW_REQUEST = 'leads_npsreviewrequest';

    /**
     * meeting reminder email
     */
    const LEADS_MEETING_REMINDER = 'leads_meetingreminder';

    /**
     * @var array
     */
    private $_options;

    /**
     * @var Tools_Mail_Mailer instance of mailer
     */
    private $_mailer;

    /**
     * @var Helpers_Action_Config
     */
    private $_configHelper;

    /**
     * @var Helpers_Action_Website
     */
    private $_websiteHelper;

    /**
     * @var instance of watched object
     */
    private $_object;

    /**
     * Template html
     *
     * @var string
     */
    private $_template;

    /**
     * systemm translation
     *
     * @var Zend_Controller_Action_Helper_Abstract
     */
    private $_translator;

    private $_entityParser = null;

    /**
     * Default lexems structure for organization
     *
     * @var array
     */
    public static $_defaultOrganizationLexemsData = array(
        'organizationName' => '',
        'organizationAddress' => '',
        'organizationAddress1' => '',
        'organizationCity' => '',
        'organizationZip' => '',
        'organizationCountryCode' => '',
        'organizationUrl' => '',
        'customerGroupId' => '',
        'organizationWebsite' => '',
        'createdAt' => '',
        'updatedAt' => '',
        'organizationIndustryId' => '',
        'organizationPhone' => '',
        'organizationEmail' => '',
        'organizationEmployees' => '',
        'organizationDomain' => '',
        'organizationStateCode' => '',
        'organizationPhoneCountryCode' => '',
        'organizationPhoneCountryCodeValue' => '',
        'organizationPhoneExtension' => '',
        'organizationMobile' => '',
        'organizationMobileCountryCode' => '',
        'organizationMobileCountryCodeValue' => '',
        'organizationNotes' => '',
        'externalId' => '',
        'creationType' => '',
        'primaryContactId' => '',
        'firstContactCreationDate' => '',
        'id' => '',
        'orglink' => '',
        'leadlink' => '',
    );

    /**
     * @param array $options
     * @throws Zend_Controller_Action_Exception
     */
    public function __construct($options = array())
    {
        $this->_configHelper = Zend_Controller_Action_HelperBroker::getExistingHelper('config');
        $this->_websiteHelper = Zend_Controller_Action_HelperBroker::getExistingHelper('website');
        $this->_translator    = Zend_Controller_Action_HelperBroker::getStaticHelper('language');
        $this->_options = $options;
        $this->_initMailer();
        $this->_entityParser = new Tools_Content_EntityParser();
        $this->_initMailMessage();
    }

    /**
     * Init mailer
     */
    private function _initMailer()
    {
        $config = $this->_configHelper->getConfig();
        $this->_mailer = new Tools_Mail_Mailer();

        if ((bool)$config['useSmtp']) {
            $smtpConfig = array(
                'host' => $config['smtpHost'],
                'username' => $config['smtpLogin'],
                'password' => $config['smtpPassword']
            );
            if ((bool)$config['smtpSsl']) {
                $smtpConfig['ssl'] = $config['smtpSsl'];
            }
            if (!empty($config['smtpPort'])) {
                $smtpConfig['port'] = $config['smtpPort'];
            }
            $this->_mailer->setSmtpConfig($smtpConfig);
            $this->_mailer->setTransport(Tools_Mail_Mailer::MAIL_TYPE_SMTP);
        } else {
            $this->_mailer->setTransport(Tools_Mail_Mailer::MAIL_TYPE_MAIL);
        }
    }

    /**
     * set email message
     */
    protected function _initMailMessage()
    {
        $this->_options['message'] = (isset($this->_options['mailMessage']) ? $this->_options['mailMessage'] : $this->_options['message']);
        unset($this->_options['mailMessage']);
    }

    /**
     * @param $object instance of watched object
     * @return bool
     * @throws Exceptions_SeotoasterException
     */
    public function notify($object)
    {
        if (!$object) {
            return false;
        }

        $this->_object = $object;

        if (empty($this->_options['withoutPreparseTemplate'])) {
            if (isset($this->_options['template']) && !empty($this->_options['template'])) {
                $this->_template = $this->_preparseEmailTemplate();
            } else {
                return false;
            }
        }

        $this->_mailer->setMailFromLabel('')
            ->setSubject($this->_options['subject']);

        $wicEmail = $this->_configHelper->getConfig('wicEmail');
        $this->_entityParser->addToDictionary(array('widcard:BizEmail' => !empty($wicEmail) ? $wicEmail : $this->_configHelper->getConfig('adminEmail')));

        if (!empty($this->_options['from'])) {
            $this->_mailer->setMailFrom($this->_parseLexemsAndContent($this->_entityParser->parse($this->_options['from'])));
        } else {
            $this->_mailer->setMailFrom($this->_configHelper->getConfig('adminEmail'));
        }

        if (isset($this->_options['trigger'])) {
            $methodName = str_replace('leads_', '', $this->_options['trigger']);
            $methodName = '_send' . ucfirst(strtolower(preg_replace('/\s*/', '', $methodName))) . 'Mail';
            if (method_exists($this, $methodName)) {
                $this->$methodName();
            }
        }
    }

    /**
     * Send email to lead from the dashboard
     *
     * @return bool
     * @throws Exceptions_SeotoasterException
     */
    private function _sendImapcredentialsnotificationMail()
    {
        $systemConfig = $this->_configHelper->getConfig();
        $userEmail = $this->_options['userEmail'];
        $userMapper = Application_Model_Mappers_UserMapper::getInstance();
        $adminBccArray = array();
        $adminEmail = isset($systemConfig['adminEmail']) ? $systemConfig['adminEmail'] : 'admin@localhost';

        $this->_mailer->setMailToLabel('Admin')
            ->setMailTo($adminEmail);
        $where = $userMapper->getDbTable()->getAdapter()->quoteInto("role_id = ?",
            Tools_Security_Acl::ROLE_ADMIN);
        $adminUsers = $userMapper->fetchAll($where);
        if (!empty($adminUsers)) {
            foreach ($adminUsers as $admin) {
                array_push($adminBccArray, $admin->getEmail());
            }
            array_push($adminBccArray, $userEmail);
            if (!empty($adminBccArray)) {
                $this->_mailer->setMailBcc($adminBccArray);
            }
        }

        $this->_entityParser->addToDictionary(array('user:userEmail' => $userEmail));

        $emailStatus = $this->_send();
    }

    /**
     * Send email
     *
     * @return bool
     * @throws Exceptions_SeotoasterException
     */
    protected function _send()
    {
        if (!$this->_mailer->getMailFrom() || !$this->_mailer->getMailTo()) {
            throw new Exceptions_SeotoasterException('Missing required "from" and "to" fields');
        }
        $this->_mailer->setBody($this->_entityParser->parse($this->_template));

        return ($this->_mailer->send() !== false);
    }

    /**
     * Send email to lead from the dashboard
     *
     * @return bool
     * @throws Exceptions_SeotoasterException
     */
    private function _sendContactMail()
    {

        $firstName = $this->_object->getLeadFirstName();
        $lastName = $this->_object->getLeadLastName();
        $fullName = $firstName . ' ' . $lastName;
        $emailTo = $this->_options['emailTo'];
        $bccEmails = $this->_options['bccEmails'];
        $emailsTo = $this->_options['emailsTo'];
        $withoutTracking = false;
        if (!empty($this->_options['withoutTracking'])) {
            $withoutTracking = true;
        }

        $skipBcc = false;
        if (!empty($this->_options['skipBcc'])) {
            $skipBcc = true;
        }

        if (!empty($this->_options['from'])) {
            $userModel = Application_Model_Mappers_UserMapper::getInstance()->findByEmail($this->_options['from']);
            if ($userModel instanceof Application_Model_Models_User) {
                $this->_mailer->setMailFromLabel($userModel->getFullName());
            }
        }

        switch ($this->_options['recipient']) {
            case Tools_Security_Acl::ROLE_GUEST:
                        $this->_mailer->setMailToLabel($fullName)->setMailTo($emailTo);
                break;
            case Tools_Security_Acl::ROLE_ADMIN:
                $this->_mailer->setMailToLabel($fullName)->setMailTo($emailTo);
                break;
            case Shopping::ROLE_SALESPERSON:
                $this->_mailer->setMailToLabel($fullName)->setMailTo($emailTo);
                break;
            default:
                error_log('Unsupported recipient ' . $this->_options['recipient'] . ' given');

                return false;
                break;
        }

        if ($withoutTracking === true) {
            if (!empty($bccEmails) && $skipBcc === false){
                $this->_mailer->setMailBcc($bccEmails);
            }
        }

        $this->_entityParser->objectToDictionary($this->_object, 'lead');
        $trackingCodeHash = hash('sha256', microtime().rand(1,1000));
        $emailContent = $this->_template;
        if (!$withoutTracking) {
            $websiteUrl = $this->_websiteHelper->getUrl();
            $this->_template = $this->_template . ' ' . '<img src="' . $websiteUrl . 'plugin/leads/run/logPixelTrackingCode/trackingCode/' . $trackingCodeHash . '">';

            if (!empty($this->_template)) {
                $doc = new DOMDocument();
                /**
                 * LIBXML_HTML_NOIMPLIED turns off the automatic adding of implied html/body elements
                 * LIBXML_HTML_NODEFDTD prevents a default doctype being added when one is not found.
                 */

                $firePluginHiddenEmail = Tools_System_Tools::firePluginMethod('hiddenemail','crmHiddenEmail');
                if(!empty($firePluginHiddenEmail)){
                    $firePluginHiddenEmail = array_shift($firePluginHiddenEmail);

                    $this->_template .= $firePluginHiddenEmail;
                }

                $emailSentToMessage = '';
                if (count($emailsTo) > 1) {
                    $emailToMessageList = implode(',', $emailsTo);
                    $messageForEmailLine = $this->_translator->translate(Leads::TRANSLATION_PREFIX.'FYI: This email was also sent to');
                    $emailSentToMessage = '<table width="100%" border="0" cellspacing="0" cellpadding="0"><tr><td align="center"><p><b>' . $messageForEmailLine . '</b>' . ' ' . $emailToMessageList . '</p></td></tr></table><p></p>';
                }

                $doc->loadHTML(mb_convert_encoding('<html><body>'.$emailSentToMessage.$this->_template.'</body></html>', 'HTML-ENTITIES', 'UTF-8'), LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD);
                $xpath = new DOMXpath($doc);
                $leadsEmailLinkAttachmentLogMapper = Leads_Mapper_LeadsEmailLinkAttachmentLogMapper::getInstance();
                foreach ($xpath->query('//a[@class="file-link-attachment"]') as $fileLinkEl) {
                    $fileFolderName = $fileLinkEl->getAttribute('data-file-folder');
                    $fileLinkName = $fileLinkEl->getAttribute('data-file-name');
                    $fileCodeHash = hash('sha256', $fileFolderName . $fileLinkName);
                    $trackingLink = $websiteUrl . 'plugin/leads/run/openLink/fileCode/' . $fileCodeHash . '/emailCode/' . $trackingCodeHash;
                    $fileLinkEl->setAttribute('href', $trackingLink);
                    $leadsEmailLinkAttachmentLogModel = $leadsEmailLinkAttachmentLogMapper->getByFileHashSentHash($trackingCodeHash, $fileCodeHash);
                    if (!$leadsEmailLinkAttachmentLogModel instanceof Leads_Model_LeadsEmailLinkAttachmentLogModel) {
                        $leadsEmailLinkAttachmentLogModel = new Leads_Model_LeadsEmailLinkAttachmentLogModel();
                        $leadsEmailLinkAttachmentLogModel->setEmailLogSentHash($trackingCodeHash);
                        $leadsEmailLinkAttachmentLogModel->setFileHash($fileCodeHash);
                        $leadsEmailLinkAttachmentLogModel->setFileName($fileLinkName);
                        $leadsEmailLinkAttachmentLogModel->setFileFolderName($fileFolderName);
                        $leadsEmailLinkAttachmentLogMapper->save($leadsEmailLinkAttachmentLogModel);
                    }
                }

                $this->_template = strtr($doc->saveHTML(), array('<html>' => '', '<body>'  => '', '</body>'  => '', '</html>' => ''));
            }
        }

        $this->_mailer->setMessageId(true);
        $messageId = $this->_mailer->getMessageId();
        $this->_mailer->setSubject(html_entity_decode($this->_options['subject'], ENT_QUOTES, 'UTF-8'));

        $emailStatus = $this->_send();
        $loggedUserId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();
        if ($emailStatus && !$withoutTracking) {
            $leadId = $this->_object->getId();
            $createdAt = Tools_System_Tools::convertDateFromTimezone('now');
            $actionLogModel = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_EMAIL, $leadId,
                Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND, $createdAt, Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_EMAIL, Leads_Model_LeadsActionLogModel::EVENT_LOG_EXPIRED, $emailContent);
            $optionsMessage = $this->_preparseOptionMessage($this->_options['message']);

            $leadsEmailLogMapper = Leads_Mapper_LeadsEmailLogMapper::getInstance();
            $leadsEmailLogModel = new Leads_Model_LeadsEmailLogModel();
            $leadsEmailLogModel->setActionLogId($actionLogModel->getId());
            $leadsEmailLogModel->setContent($optionsMessage);
            $leadsEmailLogModel->setEmailFrom($this->_parseLexemsAndContent($this->_entityParser->parse($this->_options['from'])));
            $leadsEmailLogModel->setEmailTo($emailTo);
            $leadsEmailLogModel->setEmailSubject($this->_options['subject']);
            $leadsEmailLogModel->setEmailTemplate($emailContent);
            $leadsEmailLogModel->setTrackingCode($trackingCodeHash);
            $leadsEmailLogModel->setLeadId($leadId);
            $leadsEmailLogModel->setMessageId('<'.$messageId.'>');
            $leadsEmailLogModel->setCreatorId($loggedUserId);
            if (!empty($bccEmails)) {
                $leadsEmailLogModel->setEmailBcc(implode(',', $bccEmails));
            }
            $leadsEmailLogMapper->save($leadsEmailLogModel);

        }
    }

    private function _sendTaskreminderMail()
    {

        $subject = $this->_object->getTitle();
        $fromUserId = $this->_object->getCreatorId();
        $toUserId = $this->_object->getUserId();
        $queueId = $this->_options['queueId'];
        $userMapper = Application_Model_Mappers_UserMapper::getInstance();
        $fromUserModel = $userMapper->find($fromUserId);
        $toUserModel = $userMapper->find($toUserId);
        $translator = Zend_Registry::get('Zend_Translate');
        $leadTaskQueueMapper = Leads_Mapper_LeadsTaskQueueMapper::getInstance();
        if ($fromUserModel instanceof Application_Model_Models_User && $toUserModel instanceof Application_Model_Models_User && !empty($queueId)) {
            $toFullName = $toUserModel->getFullName();
            $toUserTimezone = $toUserModel->getTimezone();
            $dateFormat = 'd-M-Y h:i a';
            if (empty($toUserTimezone)) {
                $dueDate = date($dateFormat, strtotime($this->_object->getExpiredAt()));
            } else {
                $dueDate = Tools_System_Tools::convertDateFromTimezone($this->_object->getExpiredAt(), 'UTC', $toUserTimezone, $dateFormat);
            }
            $emailTo = $toUserModel->getEmail();
            $creatorFullName = $fromUserModel->getFullName();
            if ($fromUserId === $toUserId) {
                $creatorFullName = $translator->translate(Leads::TRANSLATION_PREFIX.'yourself');
            }
            $this->_mailer->setMailToLabel($toFullName)->setMailTo($emailTo);
            $this->_addLeadAndOrganizationLexemsToDictionary();
            $this->_mailer->setSubject($this->_parseLexemsAndContent($subject));
            $this->_entityParser->objectToDictionary($this->_object, 'task');
            $this->_entityParser->addToDictionary(array('task:assignedFullName' => $toFullName, 'task:creatorFullName' => $creatorFullName, 'task:dueDate' => $dueDate));
            $emailStatus = $this->_send();
            if ($emailStatus) {
                $leadTaskQueueMapper->updateQueueStatus($queueId, Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_STATUS_SENT);
                return true;
            }
        }

        if (!empty($queueId)) {
            $leadTaskQueueMapper->updateQueueStatus($queueId, Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_STATUS_ERROR);
        }

        return false;

    }


    private function _sendTaskcreatedMail()
    {

        $subject = $this->_object->getTitle();
        $fromUserId = $this->_object->getCreatorId();
        $toUserId = $this->_object->getUserId();
        $userMapper = Application_Model_Mappers_UserMapper::getInstance();
        $fromUserModel = $userMapper->find($fromUserId);
        $toUserModel = $userMapper->find($toUserId);
        $translator = Zend_Registry::get('Zend_Translate');
        if ($fromUserModel instanceof Application_Model_Models_User && $toUserModel instanceof Application_Model_Models_User) {
            $toFullName = $toUserModel->getFullName();
            $toUserTimezone = $toUserModel->getTimezone();
            $dateFormat = 'd-M-Y h:i a';
            $originalDueDate = $this->_object->getDueDate();
            if (empty($originalDueDate)) {
                $originalDueDate = $this->_object->getExpiredAt();
            }
            $startDate = $this->_object->getExpiredAt();
            if (empty($toUserTimezone)) {
                $dueDate = date($dateFormat, strtotime($originalDueDate));
                $startDate = date($dateFormat, strtotime($startDate));
            } else {
                $dueDate = Tools_System_Tools::convertDateFromTimezone($originalDueDate, 'UTC',
                    $toUserTimezone, $dateFormat);
                $startDate = Tools_System_Tools::convertDateFromTimezone($startDate, 'UTC',
                    $toUserTimezone, $dateFormat);
            }
            $emailTo = $toUserModel->getEmail();
            $creatorFullName = $fromUserModel->getFullName();
            if ($fromUserId === $toUserId) {
                $creatorFullName = $translator->translate(Leads::TRANSLATION_PREFIX.'yourself');
            }
            $this->_mailer->setMailToLabel($toFullName)->setMailTo($emailTo);
            $this->_addLeadAndOrganizationLexemsToDictionary();
            $this->_mailer->setSubject($this->_parseLexemsAndContent($subject));
            $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
            $leadModel = $leadsMapper->find($this->_object->getLeadId());

            $notes = '';
            if ($leadModel instanceof Leads_Model_LeadsModel) {
                $notes = $this->_parseLexemsAndContent($this->_object->getNotes(), $leadModel);
            }

            $this->_entityParser->objectToDictionary($this->_object, 'task');
            $this->_entityParser->addToDictionary(array('task:assignedFullName' => $toFullName, 'task:creatorFullName' => $creatorFullName, 'task:dueDate' => $dueDate, 'task:startDate' => $startDate, 'task:notes' => $notes));
            $this->_send();
        }

        return false;

    }

    private function _sendLicenselimitMail()
    {
        $systemConfig = $this->_configHelper->getConfig();
        $userMapper = Application_Model_Mappers_UserMapper::getInstance();
        $adminBccArray = array();
        $adminEmail = isset($systemConfig['adminEmail']) ? $systemConfig['adminEmail'] : 'admin@localhost';
        switch ($this->_options['recipient']) {
            case Tools_Security_Acl::ROLE_ADMIN:
                $this->_mailer->setMailToLabel('Admin')
                    ->setMailTo($adminEmail);
                $where = $userMapper->getDbTable()->getAdapter()->quoteInto("role_id = ?",
                    Tools_Security_Acl::ROLE_ADMIN);
                $adminUsers = $userMapper->fetchAll($where);
                if (!empty($adminUsers)) {
                    foreach ($adminUsers as $admin) {
                        array_push($adminBccArray, $admin->getEmail());
                    }
                    if (!empty($adminBccArray)) {
                        $this->_mailer->setMailBcc($adminBccArray);
                    }
                }
                break;
            default:
                error_log('Unsupported recipient ' . $this->_options['recipient'] . ' given');
                return false;
                break;
        }

        $subscriptionLink = Tools_LeadLicenseTools::getLicenseSubscriptionLink();
        $this->_entityParser->addToDictionary(array('lead:subscriptionlink' => $subscriptionLink));

        $emailStatus = $this->_send();

    }

    private function _sendMeetingMail()
    {

        $data = $this->_options['params'];
        $userTimezone = Tools_System_Tools::getUserTimezone();
        if (!empty($data['end_time_date']) && !empty($userTimezone)) {
            $data['end_time_date'] = $data['end_time_date'].' '.Tools_System_Tools::convertDateFromTimezone('now', 'UTC', Tools_System_Tools::getUserTimezone(), 'T');
        }
        if (!empty($data['start_time_date']) && !empty($userTimezone)) {
            $data['start_time_date'] = $data['start_time_date'].' '.Tools_System_Tools::convertDateFromTimezone('now', 'UTC', Tools_System_Tools::getUserTimezone(), 'T');
        }
        $leadModel = $this->_options['leadModel'];
        $sendSms = $this->_options['sendSms'];
        $smsPhoneNumber = $this->_options['smsPhoneNumber'];
        $leadEmail = $leadModel->getLeadEmail();
        $params = array();
        if (!empty($leadEmail)) {
            $leadFirstName = $leadModel->getLeadFirstName();
            $leadLastName = $leadModel->getLeadLastName();
            $leadFullName = '';
            if (!empty($leadFirstName)) {
                $leadFullName .= $leadFirstName.' ';
            }
            if (!empty($leadLastName)) {
                $leadFullName .= $leadLastName;
            }
            $leadFullName = rtrim($leadFullName, ' ');
        }

        $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');

        $this->_entityParser->objectToDictionary($this->_object, 'leadsmeeting');
        $this->_entityParser->addToDictionary(array('leadsmeeting:starttimedate' => $data['start_time_date'], 'leadsmeeting:endtimedate' =>$data['end_time_date']));

        if ($this->_options['service'] === 'sms') {
            if ($leadModel instanceof Leads_Model_LeadsModel) {
                $data['details'] = Tools_LeadSmsTools::addDictionarySmsFields($data['details'], $leadModel, $websiteHelper->getUrl(), '<a>');
                $data['details'] = Tools_LeadTools::parseHrefFromHtml($data['details']);
                $this->_entityParser->addToDictionary(array('leadsmeeting:details' => $data['details']));
            }

            if ($sendSms === true) {
                $subscriber['subscriber']['user'] = array(
                    'phone' => array($smsPhoneNumber),
                    'message' => $this->_entityParser->parse(strip_tags($this->_options['message'])),
                    'owner_type' => Apps::SMS_OWNER_TYPE_USER,
                    'custom_params' => array(),
                    'sms_from_type' => Apps::SERVICE_TYPE_CRM,
                    'lead_first_name' => $leadModel->getLeadFirstName(),
                    'lead_last_name' => $leadModel->getLeadLastName()
                );

                $response = Apps::apiCall('POST', 'apps', array('twilioSms'), $subscriber);

                return true;
            }
        } else {
            if ($leadModel instanceof Leads_Model_LeadsModel) {
                $data['details'] = $this->_parseLexemsAndContent($data['details'], $leadModel);
                $this->_entityParser->addToDictionary(array('leadsmeeting:details' => $data['details']));
            }

            $params['start'] = $this->_object->getStartTimeDate();
            $params['end'] = $this->_object->getEndTimeDate();
            $params['description'] = $data['details'];
            $params['title'] = $this->_object->getSubject();
            $params['location'] = $this->_object->getLocation();
            $params['summary'] = $this->_object->getSubject();
            $emailsTo = explode(',', $this->_object->getInviteAttendees());
            $currentUserEmail = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getEmail();
            array_push($emailsTo, $currentUserEmail);
            $emailsTo = array_filter(array_unique(array_map('trim', $emailsTo)));

            foreach ($emailsTo as $key => $emailTo) {
                $attendeesData[$key]['email'] = $emailTo;
                if (!empty($leadEmail) && $leadEmail === $emailTo && !empty($leadFullName)) {
                    $attendeesData[$key]['name'] = $leadFullName;
                }
                $attendeesData[$key]['name'] = '';
            }

            $params['description'] = nl2br($params['description']);
            $params['description'] = str_replace(array("\r", "\n"), '', $params['description']);
            $params['description'] = str_replace('<br />', '<br>', $params['description']);
            $params['location'] = html_entity_decode($params['location'], ENT_QUOTES, 'UTF-8');
            $params['summary'] = html_entity_decode($params['summary'], ENT_QUOTES, 'UTF-8');
            $params['description'] = html_entity_decode(strip_tags($params['description'], '<a><br>'), ENT_QUOTES, 'UTF-8');
            $params['title'] = html_entity_decode($params['title'], ENT_QUOTES, 'UTF-8');

            $params['attendees'] = $attendeesData;
            $params['organizer'] = $data['organizer'];
            $params['organizerEmail'] = $data['organizerEmail'];
            if (!empty($data['google_calendar'])) {
                $this->_mailer->addAttachment(Tools_System_ICalendar::generateEmailInviteAttachment($params));
            }
            $this->_mailer->setMailToLabel('')->setMailTo($emailsTo);

            return ($this->_send() !== false);
        }
    }

    private function _sendMeetingreminderMail()
    {
        $data = $this->_options['params'];
        $leadModel = $this->_options['leadModel'];
        $leadEmail = $leadModel->getLeadEmail();
        $sendSms = $this->_options['sendSms'];
        $smsPhoneNumber = $this->_options['smsPhoneNumber'];
        $adminSmsPhoneNumber = $this->_options['adminSmsPhoneNumber'];
        if (!empty($leadEmail)) {
            $leadFirstName = $leadModel->getLeadFirstName();
            $leadLastName = $leadModel->getLeadLastName();
            $leadFullName = '';
            if (!empty($leadFirstName)) {
                $leadFullName .= $leadFirstName.' ';
            }
            if (!empty($leadLastName)) {
                $leadFullName .= $leadLastName;
            }
            $leadFullName = rtrim($leadFullName, ' ');
        }

        $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');

        $this->_entityParser->objectToDictionary($this->_object, 'leadsmeeting');

        if ($this->_options['service'] === 'sms') {
            if ($sendSms === true) {
                if ($leadModel instanceof Leads_Model_LeadsModel) {
                    $data['details'] = Tools_LeadSmsTools::addDictionarySmsFields($data['details'], $leadModel, $websiteHelper->getUrl(), '<a>');
                    $data['details'] = Tools_LeadTools::parseHrefFromHtml($data['details']);
                    $this->_entityParser->addToDictionary(array('leadsmeeting:details' => $data['details']));
                }

                $ownerType = Apps::SMS_OWNER_TYPE_ADMIN;
                $phoneNumber = $adminSmsPhoneNumber;
                if ($this->_options['recipient'] === Tools_Security_Acl::ROLE_USER || $this->_options['recipient'] === Tools_Security_Acl::ROLE_GUEST || $this->_options['recipient'] === Shopping::ROLE_CUSTOMER) {
                    $ownerType = Apps::SMS_OWNER_TYPE_USER;
                    $phoneNumber = $smsPhoneNumber;
                }

                if (!empty($phoneNumber)) {
                    if ($ownerType === Apps::SMS_OWNER_TYPE_ADMIN) {
                        $subscriber['subscriber'][$ownerType] = array(
                            'phone' => array($phoneNumber),
                            'message' => $this->_entityParser->parse(strip_tags($this->_options['message'])),
                            'owner_type' => $ownerType,
                            'custom_params' => array()
                        );
                    } else {
                        $subscriber['subscriber'][$ownerType] = array(
                            'phone' => array($phoneNumber),
                            'message' => $this->_entityParser->parse(strip_tags($this->_options['message'])),
                            'owner_type' => $ownerType,
                            'custom_params' => array(),
                            'sms_from_type' => Apps::SERVICE_TYPE_CRM,
                            'lead_first_name' => $leadModel->getLeadFirstName(),
                            'lead_last_name' => $leadModel->getLeadLastName()
                        );
                    }

                    $response = Apps::apiCall('POST', 'apps', array('twilioSms'), $subscriber);
                }

                return true;
            }
        } else {
            if ($leadModel instanceof Leads_Model_LeadsModel) {
                $data['details'] = $this->_parseLexemsAndContent($data['details'], $leadModel);
                $this->_entityParser->addToDictionary(array('leadsmeeting:details' => $data['details']));
            }

            $params['start'] = $this->_object->getStartTimeDate();
            $params['end'] = $this->_object->getEndTimeDate();
            $params['description'] = $this->_object->getDetails();
            $params['title'] = $this->_object->getSubject();
            $params['location'] = $this->_object->getLocation();
            $params['summary'] = $this->_object->getSubject();
            $emailsTo = explode(',', $this->_object->getInviteAttendees());
            $creatorId = $this->_object->getCreatorId();
            if (!empty($creatorId)) {
                $userMapper = Application_Model_Mappers_UserMapper::getInstance();
                $userModel = $userMapper->find($creatorId);
                if ($userModel instanceof Application_Model_Models_User) {
                    $currentUserEmail = $userModel->getEmail();
                    array_push($emailsTo, $currentUserEmail);
                }
            }

            $emailsTo = array_filter(array_unique(array_map('trim', $emailsTo)));

            $this->_mailer->setMailToLabel('')->setMailTo($emailsTo);

            return ($this->_send() !== false);
        }
    }


    /**
     * Send copy of the system form email
     *
     * @return bool|Zend_Mail
     */
    private function _sendSystemEmailCopyFormMail()
    {

        $form = $this->_object;
        $formDetails = Tools_LeadTools::cleanFormData($this->_options['data']);
        $emailsTo = $this->_options['emailsTo'];

        $this->_mailer->setMailToLabel($this->_translator->translate(Leads::TRANSLATION_PREFIX.'Admin'))
            ->setMailTo($emailsTo[0]);
        if(count($emailsTo) > 1){
            array_shift($emailsTo);
            $this->_mailer->setMailBcc($emailsTo);
        }

        $adminTemplateEmail = $form->getAdminMailTemplate();

        if (!empty($adminTemplateEmail)) {
            $this->_options['template'] = $adminTemplateEmail;
        }

        $formAdminMessage  = $form->getAdminText();

        if (!empty($formAdminMessage)) {
            $this->_options['message'] = $formAdminMessage;
        }

        $formUrl = '';
        if (isset($formDetails['formUrl'])) {
            $formUrl = $formDetails['formUrl'];
            unset($formDetails['formUrl']);
        }

        if (($mailBody = $this->_prepareEmailBodySystemForm()) === false) {
            $mailBody = '{form:details}';
        }

        $formDetailsHtml = $this->_prepareFormDetailsHtml($formDetails);

        $this->_entityParser->setDictionary(array(
            'form:details' => $formDetailsHtml
        ));

        $formDetails['formUrl'] = $formUrl;
        $this->_addLeadAndOrganizationLexemsToDictionary();
        $this->_addToDictionaryLexemes($formDetails);
        $mailBody = $this->_entityParser->parse($mailBody);

        if ($formUrl && empty($adminTemplateEmail)) {
            $mailBody .= '<div style="background:#eee;padding:10px;">' . $this->_translator->translate(Leads::TRANSLATION_PREFIX.'This form was submitted from') . ': <a href="' . $formUrl . '">' . $formUrl . '</a></div>';
        }

        if (isset($this->_options['attachment']) && is_array($this->_options['attachment']) && !empty($this->_options['attachment'])) {
            $this->_mailer->addAttachment($this->_options['attachment']);
        }
        $this->_mailer->setBody($mailBody);

        $adminFormSubject = $form->getAdminSubject();
        if (empty($adminFormSubject)) {
            $senderFullName = (isset($formDetails['lastname'])) ? $formDetails['name'] . ' ' . $formDetails['lastname'] : $formDetails['name'];
            $adminFormSubject = $this->_translator->translate(Leads::TRANSLATION_PREFIX.'New') . ' ' . $form->getName() . ' ' . $this->_translator->translate(Leads::TRANSLATION_PREFIX.'form submitted from') . ' ' . $senderFullName;
        }

        $adminFromName = $form->getAdminFromName();
        if (empty($adminFormSubject)) {
            $adminFromName = $this->_translator->translate(Leads::TRANSLATION_PREFIX.'Notifications @ ') . $this->_websiteHelper->getUrl();
        }

        $adminFromEmail = $form->getAdminFrom();
        if (empty($adminFromEmail)) {
            $adminFromEmail = $this->_configHelper->getConfig('adminEmail');
        }

        $this->_mailer->setSubject($adminFormSubject)
            ->setMailFromLabel($adminFromName)
            ->setMailFrom($adminFromEmail);

        return $this->_mailer->send();
    }

    /**
     * Send event copy email
     *
     * @return bool|Zend_Mail
     */
    private function _sendEventEmailCopyMail()
    {
        $emailsTo = $this->_options['emailsTo'];
        $formDetails = Tools_LeadTools::cleanFormData($this->_options['data']);
        unset($formDetails['name']);

        $this->_mailer->setMailToLabel($this->_translator->translate(Leads::TRANSLATION_PREFIX.'Admin'))
            ->setMailTo($emailsTo[0]);
        if(count($emailsTo) > 1){
            array_shift($emailsTo);
            $this->_mailer->setMailBcc($emailsTo);
        }

        $formUrl = '';
        if (isset($formDetails['formUrl'])) {
            $formUrl = $formDetails['formUrl'];
            unset($formDetails['formUrl']);
        }

        if (($mailBody = $this->_prepareEmailBodySystemForm()) === false) {
            $mailBody = '{form:details}';
        }

        $formDetailsHtml = $this->_prepareFormDetailsHtml($formDetails);

        $this->_entityParser->setDictionary(array(
            'form:details' => $formDetailsHtml
        ));

        $formDetails['formUrl'] = $formUrl;

        $this->_addLeadAndOrganizationLexemsToDictionary();
        $this->_addToDictionaryLexemes($formDetails);

        if(!empty($this->_options['leadData'])) {
            $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
            $leadModel = $leadsMapper->find($this->_options['leadData']['id']);

            if ($leadModel instanceof Leads_Model_LeadsModel) {
                if (!empty($this->_options['leadData']['additionalInfo'])) {
                    $leadModel->setAdditionalInfo($this->_options['leadData']['additionalInfo']);
                }
                $mailBody = $this->_parseLexemsAndContent($mailBody, $leadModel);
            }
        }

        $mailBody = $this->_entityParser->parse($mailBody);

        $this->_mailer->setBody($mailBody);

        $this->_mailer->setSubject($this->_parseLexemsAndContent($this->_options['emailSubject']))
            ->setMailFrom($this->_parseLexemsAndContent($this->_options['emailFrom']));

        return $this->_mailer->send();
    }

    /**
     * Send event copy email
     *
     * @return bool|Zend_Mail
     */
    private function _sendExternalAdminMail()
    {
        $emailsTo = $this->_options['emailsTo'];
        $formDetails = Tools_LeadTools::cleanFormData($this->_options['data']);

        $this->_mailer->setMailToLabel($this->_translator->translate(Leads::TRANSLATION_PREFIX.'Admin'))
            ->setMailTo($emailsTo[0]);
        if(count($emailsTo) > 1){
            array_shift($emailsTo);
            $this->_mailer->setMailBcc($emailsTo);
        }

        $formUrl = '';
        if (isset($formDetails['formUrl'])) {
            $formUrl = $formDetails['formUrl'];
            unset($formDetails['formUrl']);
        }

        if (($mailBody = $this->_prepareEmailBodySystemForm()) === false) {
            $mailBody = '{form:details}';
        }

        $formDetailsHtml = $this->_prepareFormDetailsHtml($formDetails);

        $this->_entityParser->setDictionary(array(
            'form:details' => $formDetailsHtml
        ));

        $formDetails['formUrl'] = $formUrl;
        $this->_addLeadAndOrganizationLexemsToDictionary();
        $this->_addToDictionaryLexemes($formDetails);
        $mailBody = $this->_entityParser->parse($mailBody);

        $this->_mailer->setBody($mailBody);

        $this->_mailer->setSubject($this->_options['emailSubject'])
            ->setMailFrom($this->_entityParser->parse($this->_options['emailFrom']));

        return $this->_mailer->send();
    }


    /**
     * Send external auto reply
     *
     * @return bool|Zend_Mail
     */
    private function _sendExternalAutoReplyMail()
    {
        $emailTo = $this->_options['emailTo'];
        $emailToName = $this->_options['emailToName'];
        $formDetails = Tools_LeadTools::cleanFormData($this->_options['data']);
        unset($formDetails['name']);

        $this->_mailer->setMailToLabel($emailToName)
            ->setMailTo($emailTo);

        $formUrl = '';
        if (isset($formDetails['formUrl'])) {
            $formUrl = $formDetails['formUrl'];
            unset($formDetails['formUrl']);
        }

        if (($mailBody = $this->_prepareEmailBodySystemForm()) === false) {
            $mailBody = '{form:details}';
        }

        $formDetailsHtml = $this->_prepareFormDetailsHtml($formDetails);

        $this->_entityParser->setDictionary(array(
            'form:details' => $formDetailsHtml
        ));

        $formDetails['formUrl'] = $formUrl;
        $this->_addLeadAndOrganizationLexemsToDictionary();
        $this->_addToDictionaryLexemes($formDetails);
        $mailBody = $this->_entityParser->parse($mailBody);

        $this->_mailer->setBody($mailBody);

        $this->_mailer->setSubject($this->_options['emailSubject'])
            ->setMailFrom($this->_entityParser->parse($this->_options['emailFrom']))
            ->setMailFromLabel($this->_options['emailReplyFromName']);

        return $this->_mailer->send();
    }

    /**
     * Send emails when task is completed
     *
     * @throws Exceptions_SeotoasterException
     * @throws Zend_Exception
     */
    private function _sendTaskcompletedMail()
    {
        $emailsTo = $this->_options['emailsTo'];
        $emailsTo = array_filter(array_unique(array_map('trim', $emailsTo)));

        $fromUserId = $this->_object->getCreatorId();
        $toUserId = $this->_object->getUserId();
        $userMapper = Application_Model_Mappers_UserMapper::getInstance();
        $fromUserModel = $userMapper->find($fromUserId);
        $toUserModel = $userMapper->find($toUserId);
        $closedByUserModel = $userMapper->find($this->_object->getTaskCompletedBy());
        $translator = Zend_Registry::get('Zend_Translate');

        $toFullName = $toUserModel->getFullName();
        $toUserTimezone = $toUserModel->getTimezone();
        $dateFormat = 'd-M-Y h:i a';
        $dateFormatCompleted = 'd-M-Y h:i a';
        $originalDueDate = $this->_object->getDueDate();
        if (empty($originalDueDate)) {
            $originalDueDate = $this->_object->getExpiredAt();
        }
        $startDate = $this->_object->getExpiredAt();
        if (empty($toUserTimezone)) {
            $dueDate = date($dateFormat, strtotime($originalDueDate));
            $taskCompletedDate = date($dateFormatCompleted, strtotime('now'));
            $startDate = date($dateFormat, strtotime($startDate));
        } else {
            $dueDate = Tools_System_Tools::convertDateFromTimezone($originalDueDate, 'UTC',
                $toUserTimezone, $dateFormat);
            $taskCompletedDate = Tools_System_Tools::convertDateFromTimezone('now', 'UTC',
                $toUserTimezone, $dateFormatCompleted);
            $startDate = Tools_System_Tools::convertDateFromTimezone($startDate, 'UTC',
                $toUserTimezone, $dateFormat);
        }

        $creatorFullName = $fromUserModel->getFullName();
        if ($fromUserId === $toUserId) {
            $creatorFullName = $translator->translate(Leads::TRANSLATION_PREFIX.'yourself');
        }
        $this->_mailer->setMailToLabel('')->setMailTo($emailsTo);

        $this->_entityParser->objectToDictionary($this->_object, 'task');
        $this->_addLeadAndOrganizationLexemsToDictionary();
        $this->_mailer->setSubject($this->_parseLexemsAndContent($this->_mailer->getSubject()));
        $this->_entityParser->addToDictionary(array(
            'task:assignedFullName' => $toFullName,
            'task:creatorFullName' => $creatorFullName,
            'task:dueDate' => $dueDate,
            'task:startDate' => $startDate,
            'task:taskCompletedBy' => $closedByUserModel->getFullName(),
            'task:taskCompletedDate' => $taskCompletedDate
        ));

        $this->_send();

    }


    /**
     * Send emails when task is completed
     *
     * @throws Exceptions_SeotoasterException
     * @throws Zend_Exception
     */
    private function _sendTaskreopenedMail()
    {
        $emailsTo = $this->_options['emailsTo'];
        $emailsTo = array_filter(array_unique(array_map('trim', $emailsTo)));

        $fromUserId = $this->_object->getCreatorId();
        $toUserId = $this->_object->getUserId();
        $userMapper = Application_Model_Mappers_UserMapper::getInstance();
        $fromUserModel = $userMapper->find($fromUserId);
        $toUserModel = $userMapper->find($toUserId);
        $translator = Zend_Registry::get('Zend_Translate');

        $toFullName = $toUserModel->getFullName();
        $toUserTimezone = $toUserModel->getTimezone();
        $dateFormat = 'd-M-Y h:i a';
        $originalDueDate = $this->_object->getDueDate();
        if (empty($originalDueDate)) {
            $originalDueDate = $this->_object->getExpiredAt();
        }
        $startDate = $this->_object->getExpiredAt();
        if (empty($toUserTimezone)) {
            $dueDate = date($dateFormat, strtotime($originalDueDate));
            $startDate = date($dateFormat, strtotime($startDate));
        } else {
            $dueDate = Tools_System_Tools::convertDateFromTimezone($originalDueDate, 'UTC',
                $toUserTimezone, $dateFormat);
            $startDate = Tools_System_Tools::convertDateFromTimezone($startDate, 'UTC',
                $toUserTimezone, $dateFormat);
        }

        $creatorFullName = $fromUserModel->getFullName();
        if ($fromUserId === $toUserId) {
            $creatorFullName = $translator->translate(Leads::TRANSLATION_PREFIX.'yourself');
        }
        $this->_mailer->setMailToLabel('')->setMailTo($emailsTo);

        $this->_entityParser->objectToDictionary($this->_object, 'task');
        $this->_addLeadAndOrganizationLexemsToDictionary();
        $this->_mailer->setSubject($this->_parseLexemsAndContent($this->_mailer->getSubject()));
        $this->_entityParser->addToDictionary(array(
            'task:assignedFullName' => $toFullName,
            'task:creatorFullName' => $creatorFullName,
            'task:dueDate' => $dueDate,
            'task:startDate' => $startDate,
            'task:reopenedBy' => Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getFullName()
        ));

        $this->_send();

    }


    /**
     * Send emails when task is overdue
     *
     * @throws Exceptions_SeotoasterException
     * @throws Zend_Exception
     */
    private function _sendTaskoverdueMail()
    {
        $queueId = $this->_options['queueId'];
        $subject = $this->_object->getTitle();
        $fromUserId = $this->_object->getCreatorId();
        $toUserId = $this->_object->getUserId();
        $userMapper = Application_Model_Mappers_UserMapper::getInstance();
        $fromUserModel = $userMapper->find($fromUserId);
        $toUserModel = $userMapper->find($toUserId);
        $translator = Zend_Registry::get('Zend_Translate');
        $leadTaskQueueMapper = Leads_Mapper_LeadsTaskQueueMapper::getInstance();
        if ($fromUserModel instanceof Application_Model_Models_User && $toUserModel instanceof Application_Model_Models_User) {
            $toFullName = $toUserModel->getFullName();
            $toUserTimezone = $toUserModel->getTimezone();
            $dateFormat = 'd-M-Y h:i a';
            $originalDueDate = $this->_object->getDueDate();
            if (empty($originalDueDate)) {
                $originalDueDate = $this->_object->getExpiredAt();
            }
            $startDate = $this->_object->getExpiredAt();
            if (empty($toUserTimezone)) {
                $dueDate = date($dateFormat, strtotime($originalDueDate));
                $startDate = date($dateFormat, strtotime($startDate));
            } else {
                $dueDate = Tools_System_Tools::convertDateFromTimezone($originalDueDate, 'UTC',
                    $toUserTimezone, $dateFormat);
                $startDate = Tools_System_Tools::convertDateFromTimezone($startDate, 'UTC',
                    $toUserTimezone, $dateFormat);
            }
            $emailTo = $toUserModel->getEmail();
            $creatorFullName = $fromUserModel->getFullName();
            if ($fromUserId === $toUserId) {
                $creatorFullName = $translator->translate(Leads::TRANSLATION_PREFIX.'yourself');
            }
            $this->_mailer->setMailToLabel($toFullName)->setMailTo($emailTo);
            $this->_addLeadAndOrganizationLexemsToDictionary();
            $this->_mailer->setSubject($this->_parseLexemsAndContent($subject));
            $this->_entityParser->objectToDictionary($this->_object, 'task');
            $this->_entityParser->addToDictionary(array('task:assignedFullName' => $toFullName, 'task:creatorFullName' => $creatorFullName, 'task:dueDate' => $dueDate, 'task:startDate' => $startDate));
            $emailStatus = $this->_send();

            if ($emailStatus) {
                $leadTaskQueueMapper->updateQueueStatus($queueId, Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_STATUS_SENT);
                return true;
            }
        }

    }

    /**
     * Send emails when task is started
     *
     * @throws Exceptions_SeotoasterException
     * @throws Zend_Exception
     */
    private function _sendTaskstartMail()
    {
        $subject = $this->_object->getTitle();
        $fromUserId = $this->_object->getCreatorId();
        $toUserId = $this->_object->getUserId();
        $userMapper = Application_Model_Mappers_UserMapper::getInstance();
        $fromUserModel = $userMapper->find($fromUserId);
        $toUserModel = $userMapper->find($toUserId);
        $translator = Zend_Registry::get('Zend_Translate');
        if ($fromUserModel instanceof Application_Model_Models_User && $toUserModel instanceof Application_Model_Models_User) {
            $toFullName = $toUserModel->getFullName();
            $toUserTimezone = $toUserModel->getTimezone();
            $dateFormat = 'd-M-Y h:i a';
            $originalDueDate = $this->_object->getDueDate();
            if (empty($originalDueDate)) {
                $originalDueDate = $this->_object->getExpiredAt();
            }
            $startDate = $this->_object->getExpiredAt();
            if (empty($toUserTimezone)) {
                $dueDate = date($dateFormat, strtotime($originalDueDate));
                $startDate = date($dateFormat, strtotime($startDate));
            } else {
                $dueDate = Tools_System_Tools::convertDateFromTimezone($originalDueDate, 'UTC',
                    $toUserTimezone, $dateFormat);
                $startDate = Tools_System_Tools::convertDateFromTimezone($startDate, 'UTC',
                    $toUserTimezone, $dateFormat);
            }
            $emailTo = $toUserModel->getEmail();
            $creatorFullName = $fromUserModel->getFullName();
            if ($fromUserId === $toUserId) {
                $creatorFullName = $translator->translate(Leads::TRANSLATION_PREFIX.'yourself');
            }
            $this->_mailer->setMailToLabel($toFullName)->setMailTo($emailTo);
            $this->_addLeadAndOrganizationLexemsToDictionary();
            $this->_mailer->setSubject($this->_parseLexemsAndContent($subject));
            $this->_entityParser->objectToDictionary($this->_object, 'task');
            $this->_entityParser->addToDictionary(array('task:assignedFullName' => $toFullName, 'task:creatorFullName' => $creatorFullName, 'task:dueDate' => $dueDate, 'task:startDate' => $startDate));
            $this->_send();
        }

        return false;

    }

    private function _sendReviewRequestMail()
    {
        $leadModel = $this->_object;
        $emailTo = $leadModel->getLeadEmail();
        $fullName = $leadModel->getLeadFirstName(). $leadModel->getLeadLastName();
        $reviewLinks = $this->_options['reviewLinks'];

        $this->_mailer->setMailToLabel($fullName)->setMailTo($emailTo);
        $this->_mailer->setMailFrom($this->_parseLexemsAndContent($this->_options['from']));
        $this->_entityParser->addToDictionary($reviewLinks);
        $this->_addLeadAndOrganizationLexemsToDictionary();

        $this->_send();
    }

    private function _sendNpsReviewRequestMail()
    {
        $leadModel = $this->_object;
        $emailTo = $leadModel->getLeadEmail();
        $fullName = $leadModel->getLeadFirstName(). $leadModel->getLeadLastName();

        $this->_mailer->setMailFrom($this->_parseLexemsAndContent($this->_options['from']));

        $this->_mailer->setMailToLabel($fullName)->setMailTo($emailTo);
        $this->_addLeadAndOrganizationLexemsToDictionary();

        $this->_send();
    }



    /**
     * Adding additional lexemes
     *
     * @param array $params form params
     * @param string $lexemePrefix lexeme prefix
     */
    private function _addToDictionaryLexemes(array $params, $lexemePrefix = 'form')
    {
        $paramsDictionary = array();
        foreach ($params as $paramName => $paramValue) {
            if (is_array($paramValue)) {
                $paramValue = implode(',', $paramValue);
            }
            $paramsDictionary[$lexemePrefix . ':' . $paramName] = $paramValue;
        }

        $this->_entityParser->addToDictionary($paramsDictionary);
    }

    /**
     * Prepare form fields info in html format
     *
     * @param array $formDetails form data
     * @return string
     */
    private function _prepareFormDetailsHtml(array $formDetails)
    {
        $formDetailsHtml = '';
        foreach ($formDetails as $name => $value) {
            if (!$value) {
                continue;
            }
            $formDetailsHtml .= '<b>' . str_replace(array('_', '-'), ' ', ucfirst($name)) . '</b>' . ': ' . (is_array($value) ? implode(', ', $value) : $value) . '<br />';
        }

        return $formDetailsHtml;
    }

    /**
     * Prepare form system body
     *
     * @param int $pageId page id
     * @return bool|mixed
     * @throws Zend_Exception
     */
    protected function _prepareEmailBodySystemForm($pageId = 0) {
        $tmplMessage  = $this->_options['message'];
        $mailTemplate = Application_Model_Mappers_TemplateMapper::getInstance()->find($this->_options['template']);
        if (!empty($mailTemplate)){
            $this->_entityParser->setDictionary(array(
                'emailmessage' => !empty($tmplMessage) ? $tmplMessage : ''
            ));
            //pushing message template to email template and cleaning dictionary
            $mailTemplate = $this->_entityParser->parse($mailTemplate->getContent());
            $this->_entityParser->setDictionary(array());
            $mailTemplate = $this->_entityParser->parse($mailTemplate);

            $themeData = Zend_Registry::get('theme');
            $extConfig = Zend_Registry::get('extConfig');
            $parserOptions = array(
                'websiteUrl'   => $this->_websiteHelper->getUrl(),
                'websitePath'  => $this->_websiteHelper->getPath(),
                'currentTheme' => $extConfig['currentTheme'],
                'themePath'    => Tools_Filesystem_Tools::cleanWinPath($themeData['path']),
            );

            $cDbTable = new Application_Model_DbTable_Container();
            $select = $cDbTable->getAdapter()->select()->from('container', array(
                'uniqHash' => new Zend_Db_Expr("MD5(CONCAT_WS('-',`name`, COALESCE(`page_id`, 0), `container_type`))"),
                'id',
                'name',
                'page_id',
                'container_type',
                'content',
                'published',
                'publishing_date'
            ))
                ->where('(container_type = 2 OR container_type = 4)')
                ->where('page_id IS NULL');
            $stat   = $cDbTable->getAdapter()->fetchAssoc($select);
            if ($pageId) {
                $parserOptions['id'] = $pageId;
            }

            $parser = new Tools_Content_Parser($mailTemplate, array('containers' => $stat), $parserOptions);

            return Tools_Content_Tools::stripEditLinks($parser->parseSimple());
        }
        return false;
    }

    /**
     * Parse email template data
     *
     * @return bool|null
     * @throws Zend_Exception
     */
    private function _preparseEmailTemplate()
    {
        $tmplName = $this->_options['template'];
        $tmplMessage = $this->_options['message'];
        $mailTemplate = Application_Model_Mappers_TemplateMapper::getInstance()->find($tmplName);

        if (!empty($mailTemplate)) {
            if (!empty($this->_options['defaultMessageTemplate'])) {
                $emailContent = $mailTemplate->getContent();
                $emailContent = preg_replace('~{defaultemailcontent}(.*){\/defaultemailcontent}~uiUs', addcslashes($this->_options['message'], '$'), $emailContent);
                $mailTemplate->setContent($emailContent);
            }
            $this->_entityParser->setDictionary(
                array(
                    'emailmessage' => !empty($tmplMessage) ? $tmplMessage : ''
                )
            );
            //pushing message template to email template and cleaning dictionary
            $mailTemplate = $this->_entityParser->parse($mailTemplate->getContent());
            $this->_entityParser->setDictionary(array());

            $mailTemplate = $this->_entityParser->parse($mailTemplate);

            $themeData = Zend_Registry::get('theme');
            $extConfig = Zend_Registry::get('extConfig');
            $parserOptions = array(
                'websiteUrl' => $this->_websiteHelper->getUrl(),
                'websitePath' => $this->_websiteHelper->getPath(),
                'currentTheme' => $extConfig['currentTheme'],
                'themePath' => $themeData['path'],
                'observableModel' => $this->_object
            );

            $pageMapper = Application_Model_Mappers_PageMapper::getInstance();
            $where = $pageMapper->getDbTable()->getAdapter()->quoteInto('template_id = ?', $tmplName);
            $templatePageSystem = $pageMapper->fetchAll($where, array(), 1);
            $templatePageNotSystem = $pageMapper->fetchAll($where, array());
            if (!empty($templatePageSystem) || !empty($templatePageNotSystem)) {
                if (!empty($templatePageSystem)) {
                    $templatePageModel = array_pop($templatePageSystem);
                } else {
                    $templatePageModel = array_pop($templatePageNotSystem);
                }

                $templatePageUrl = $templatePageModel->getUrl();
                $templatePageModelWithContainers = $pageMapper->findByUrl($templatePageUrl);
                if ($templatePageModelWithContainers instanceof Application_Model_Models_Page) {
                    $parser = new Tools_Content_Parser($mailTemplate, $templatePageModelWithContainers->toArray(),
                        $parserOptions);
                } else {
                    $parser = new Tools_Content_Parser($mailTemplate, array(), $parserOptions);
                }

            } else {
                $parser = new Tools_Content_Parser($mailTemplate, array(), $parserOptions);
            }

            $emailTemplateMessage = $parser->parseSimple();

            if (Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_CONTENT)) {
                return Tools_LeadTools::stripEditLinks($emailTemplateMessage);
            }

            return $emailTemplateMessage;

        }

        return false;
    }

    /**
     * Parse $optionMessage data before save LeadsEmailLog
     * @param $optionMessage
     * @return null
     * @throws Zend_Exception
     */
    private function _preparseOptionMessage($optionMessage){
        $this->_entityParser->addToDictionary(
            array(
                'emailmessage' => !empty($optionMessage) ? $optionMessage : ''
            )
        );

        $mailTemplate = $this->_entityParser->parse($optionMessage);

        $themeData = Zend_Registry::get('theme');
        $extConfig = Zend_Registry::get('extConfig');
        $parserOptions = array(
            'websiteUrl' => $this->_websiteHelper->getUrl(),
            'websitePath' => $this->_websiteHelper->getPath(),
            'currentTheme' => $extConfig['currentTheme'],
            'themePath' => $themeData['path'],
            'observableModel' => $this->_object
        );
        $parser = new Tools_Content_Parser($mailTemplate, array(), $parserOptions);

        $content = $parser->parse();

        return $content;
    }

    /**
     * Parse lexems and content
     *
     * @param string $content any content
     * @param object $observableModel
     * @return null|string
     * @throws Zend_Exception
     */
    private function _parseLexemsAndContent($content, $observableModel = null)
    {
        $content = $this->_entityParser->parse($content);
        $themeData = Zend_Registry::get('theme');
        $extConfig = Zend_Registry::get('extConfig');
        $parserOptions = array(
            'websiteUrl' => $this->_websiteHelper->getUrl(),
            'websitePath' => $this->_websiteHelper->getPath(),
            'currentTheme' => $extConfig['currentTheme'],
            'themePath' => $themeData['path'],
            'observableModel' => $this->_object
        );

        if (!empty($observableModel)) {
            $parserOptions['observableModel'] = $observableModel;
        }

        $parser = new Tools_Content_Parser($content, array(), $parserOptions);

        $content = $parser->parse();

        return $content;
    }

    /**
     * Add lead and organization lexems to dictionary
     */
    private function _addLeadAndOrganizationLexemsToDictionary()
    {
        if (!empty($this->_options['leadData'])) {
            $this->_addToDictionaryLexemes($this->_options['leadData'], 'lead');
        }

        if (!empty($this->_options['organizationData'])) {
            $this->_addToDictionaryLexemes($this->_options['organizationData'], 'organization');
        } else {
            $this->_addToDictionaryLexemes(self::$_defaultOrganizationLexemsData, 'organization');
        }
    }


    /**
     * parse email lexems
     *
     * @param string $content parse email lexems
     * @param array $dictionary dictionary
     * @return string
     */
    public static function parseEmailLexems($content, $dictionary)
    {
        $entityParser = new Tools_Content_EntityParser();
        $entityParser->addToDictionary($dictionary);
        return $entityParser->parse($content);

    }

    /**
     * Generate dictionary
     *
     * @param array $dictionary current dictionary
     * @param array $params params to add to dictionary
     * @param string $lexemePrefix lexem prefix
     * @return array
     */
    public static function generateDictionary($dictionary, $params, $lexemePrefix)
    {
        $paramsDictionary = array();
        foreach ($params as $paramName => $paramValue) {
            if (is_array($paramValue)) {
                $paramValue = implode(',', $paramValue);
            }
            $paramsDictionary[$lexemePrefix . ':' . $paramName] = $paramValue;
        }

        return array_merge($dictionary, $paramsDictionary);
    }


    public static function domRemove(DOMNode $from)
    {
        $sibling = $from->firstChild;
        do {
            $next = $sibling->nextSibling;
            $from->parentNode->insertBefore($sibling, $from);
        } while ($sibling = $next);
        $from->parentNode->removeChild($from);
    }

}
