<?php

class Widgets_Lead_Lead extends Widgets_Abstract
{

    /**
     * lead observer key
     *
     * @var string
     */
    private static $_registryKey = 'leadObserverKey';

    /**
     * Website helper
     *
     * @var null
     */
    protected $_websiteHelper = null;

    /**
     * @var bool
     */
    protected $_cacheable = false;

    /**
     * Session helper
     *
     * @var bool
     */
    protected $_sessionHelper = false;

    /**
     * leads data model
     *
     * @var bool
     */
    protected $_leadDataModel = false;

    /**
     * @var null
     */
    protected $_layout = null;

    /**
     * Get Lead demo data by test sequence
     *
     * @var bool
     */
    protected $_isDemo = false;

    protected function _init()
    {
        parent::_init();
        $this->_view = new Zend_View(array(
            'scriptPath' => __DIR__ . '/views/'
        ));


        $this->_layout = new Zend_Layout();
        $this->_layout->setLayoutPath(Zend_Layout::getMvcInstance()->getLayoutPath());

        $this->_websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
        $this->_view->websiteUrl = $this->_websiteHelper->getUrl();
        $this->_websiteUrl = $this->_websiteHelper->getUrl();
        $this->_translator = Zend_Registry::get('Zend_Translate');

        $this->_view->setHelperPath(APPLICATION_PATH . '/views/helpers/');
        $this->_view->addHelperPath('ZendX/JQuery/View/Helper/', 'ZendX_JQuery_View_Helper');
        $this->_view->addScriptPath($this->_websiteHelper->getPath() . 'seotoaster_core/application/views/scripts/');
        // init helpers
        $this->_configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
        $this->_sessionHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
    }

    protected function _load()
    {
        if (empty($this->_options[0])) {
            return $this->_translator->translate(Leads::TRANSLATION_PREFIX.'Option not specified');
        }

        $leadMapper = Leads_Mapper_LeadsMapper::getInstance();

        if (!empty($this->_toasterOptions['observableModel']) && is_object($this->_toasterOptions['observableModel'])) {
            $observableModel = $this->_toasterOptions['observableModel'];
            if ($observableModel instanceof Emailsequence_Models_Models_EmailsequenceQueueModel) {
                $userEmail = $observableModel->getUserEmail();
                if (!empty($userEmail)) {
                    $userRegistryEmail = preg_replace("/[^a-zA-Z0-9-_]/", "", $userEmail);

                    $regKey = self::$_registryKey . '_' . $userRegistryEmail;
                    if (Zend_Registry::isRegistered($regKey)) {
                        $this->_leadDataModel = Zend_Registry::get($regKey);
                    } else {
                        $leadModel = $leadMapper->findByEmail($userEmail);
                        $this->_leadDataModel = $leadModel;
                        if (!empty($this->_leadDataModel)) {
                            Zend_Registry::set($regKey, $this->_leadDataModel);
                        }
                    }
                }
            } elseif ($observableModel instanceof Leads_Model_LeadsModel) {
                $this->_leadDataModel = $observableModel;
            }
        }

        if(!empty($this->_toasterOptions['isDemo'])) {
            $this->_isDemo = true;
        } elseif (!empty($this->_toasterOptions['observableModel']) && is_object($this->_toasterOptions['observableModel']) && !$this->_leadDataModel instanceof Leads_Model_LeadsModel) {
            return '';
        }

        $methodName = '_render' . ucfirst($this->_options[0]);
        if (method_exists($this, $methodName)) {
            return $this->$methodName();

        }

        return $this->_translator->translate(Leads::TRANSLATION_PREFIX.'Method doesn\'t exists');
    }

    /**
     * lead first name
     * @return string
     */
    protected function _renderFirstname()
    {
        if(!empty($this->_isDemo)) {
            return Tools_LeadTools::$_demoData['firstname'];
        } elseif ($this->_leadDataModel instanceof Leads_Model_LeadsModel) {
            return $this->_leadDataModel->getLeadFirstName();
        } else {
            return '{$lead:firstname}';
        }
    }

    /**
     * lead last name

     * @return string
     */
    protected function _renderLastname()
    {
        if(!empty($this->_isDemo)) {
            return Tools_LeadTools::$_demoData['lastname'];
        } elseif ($this->_leadDataModel instanceof Leads_Model_LeadsModel) {
            return $this->_leadDataModel->getLeadLastName();
        } else {
            return '{$lead:lastname}';
        }
    }

    /**
     * lead middle name
     * @return string
     */
    protected function _renderMiddlename()
    {
        if(!empty($this->_isDemo)) {
            return Tools_LeadTools::$_demoData['middlename'];
        } elseif ($this->_leadDataModel instanceof Leads_Model_LeadsModel) {
            return $this->_leadDataModel->getMiddleName();
        } else {
            return '{$lead:middlename}';
        }
    }

    /**
     * lead position
     * @return string
     */
    protected function _renderPosition()
    {
        if(!empty($this->_isDemo)) {
            return Tools_LeadTools::$_demoData['position'];
        } elseif ($this->_leadDataModel instanceof Leads_Model_LeadsModel) {
            return $this->_leadDataModel->getLeadPosition();
        } else {
            return '{$lead:position}';
        }
    }

    /**
     * lead prefix
     * @return string
     */
    protected function _renderPrefix()
    {
        if(!empty($this->_isDemo)) {
            return Tools_LeadTools::$_demoData['prefix'];
        } elseif ($this->_leadDataModel instanceof Leads_Model_LeadsModel) {
            return $this->_leadDataModel->getPrefix();
        } else {
            return '{$lead:prefix}';
        }
    }

    /**
     * lead suffix
     * todo: old method compatibility
     * @return string
     */
    protected function _renderSuffix()
    {
        if(!empty($this->_isDemo)) {
            return Tools_LeadTools::$_demoData['prefix'];
        } elseif ($this->_leadDataModel instanceof Leads_Model_LeadsModel) {
            return $this->_leadDataModel->getPrefix();
        } else {
            return '{$lead:suffix}';
        }
    }

    /**
     * lead full mobile
     * @return string
     */
    protected function _renderFullmobilephone()
    {
        if(!empty($this->_isDemo)) {
            return Tools_LeadTools::$_demoData['fullmobilephone'];
        } elseif ($this->_leadDataModel instanceof Leads_Model_LeadsModel) {
            $leadMobile = $this->_leadDataModel->getLeadMobile();
            if (!empty($leadMobile)) {
                return $this->_leadDataModel->getLeadMobileCountryCodeValue() . $leadMobile;
            }

            return '';
        } else {
            return '{$lead:fullmobilephone}';
        }
    }

    /**
     * lead full desktop phone

     * @return string
     */
    protected function _renderFulldesktopphone()
    {
        if(!empty($this->_isDemo)) {
            return Tools_LeadTools::$_demoData['fulldesktopphone'];
        } elseif ($this->_leadDataModel instanceof Leads_Model_LeadsModel) {
            $leadPhone = $this->_leadDataModel->getLeadPhone();
            if (!empty($leadPhone)) {
                return $this->_leadDataModel->getLeadPhoneCountryCodeValue() . $leadPhone;
            }

            return '';
        } else {
            return '{$lead:fulldesktopphone}';
        }
    }

    /**
     * lead organization name
     * @return string
     */
    protected function _renderOrganizationname()
    {
        if(!empty($this->_isDemo)) {

            return Tools_LeadTools::$_demoData['organizationname'];
        } elseif ($this->_leadDataModel instanceof Leads_Model_LeadsModel) {
            $organizationId = $this->_leadDataModel->getOrganizationId();
            $leadEmail = $this->_leadDataModel->getLeadEmail();
            if(!empty($organizationId) && !empty($leadEmail)){
                $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
                $leadOrganizationName = $leadMapper->findByLeadEmail($leadEmail);
                if(!empty($leadOrganizationName)){
                    return $leadOrganizationName['organization_name'];
                }
            }
            return '';
        } else {
            return '{$lead:organizationname}';
        }
    }

    /**
     * lead organization website
     * @return string
     */
    protected function _renderOrganizationwebsite()
    {
        if(!empty($this->_isDemo)) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            return  $websiteUrl;
        } elseif ($this->_leadDataModel instanceof Leads_Model_LeadsModel) {
            $organizationId = $this->_leadDataModel->getOrganizationId();
            $leadEmail = $this->_leadDataModel->getLeadEmail();
            if(!empty($organizationId) && !empty($leadEmail)){
                $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
                $leadOrganizationName = $leadMapper->findByLeadEmail($leadEmail);
                if(!empty($leadOrganizationName)){
                    return $leadOrganizationName['organization_website'];
                }
            }
            return '';
        } else {
            return '{$lead:organizationwebsite}';
        }
    }

    /**
     * lead remote authorization token
     * @return string
     */
    protected function _renderRemoteauthlink()
    {
        if ($this->_leadDataModel instanceof Leads_Model_LeadsModel) {
            $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
            $remoteAuthorizationToken = sha1(uniqid(microtime()));
            $this->_leadDataModel->setRemoteAuthorizationToken($remoteAuthorizationToken);
            $leadEmail = $this->_leadDataModel->getLeadEmail();
            if (empty($leadEmail)) {
                return '';
            }
            $leadMapper->save($this->_leadDataModel);
            $multiply = false;
            if (!empty($this->_options[1])) {
                $multiply = true;
            }
            $link = $this->_websiteHelper->getUrl().'plugin/leads/run/remoteAuthorization/leadAuthToken/'.$remoteAuthorizationToken.'/';
            if ($multiply === true) {
                $link .= 'multiply/1/';
            }

            return $link;
        } else {
            $multiply = false;
            if (!empty($this->_options[1])) {
                $multiply = true;
            }

            $lexem = '{$lead:remoteauthlink}';
            if ($multiply === true) {
                $lexem = '{$lead:remoteauthlink:multiply}';
            }
            return $lexem;
        }
    }

    protected function _renderGrid()
    {

        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $customerGroups = Store_Mapper_GroupMapper::getInstance()->fetchAll(null, array('groupName'));
            $leadStages = Leads_Mapper_LeadsStageMapper::getInstance()->fetchAll(null, array('stage_name'));
            $leadTypeMapper = Leads_Mapper_LeadsTypeMapper::getInstance();
            $where = $leadTypeMapper->getDbTable()->getAdapter()->quoteInto('type_name NOT IN (?)', Api_Leads_Leadtype::getExcludeSystemValues());
            $leadSourcesTypes = Leads_Mapper_LeadsTypeMapper::getInstance()->fetchAll($where, array('type_label'));
            $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
            $leadOwners = $leadMapper->getLeadOwners();
            $opportunityOwners = Leads_Mapper_LeadsOpportunityMapper::getInstance()->getOpportunityOwners();
            $currentUser = $this->_sessionHelper->getCurrentUser();
            $userId = $currentUser->getId();
            $userRole = $currentUser->getRoleId();
            $where = null;
            $leadFilterPresetMapper = Leads_Mapper_LeadsFilterPresetMapper::getInstance();
            $where = $leadFilterPresetMapper->getDbTable()->getAdapter()->quoteInto('filter_preset_type = ?', Leads_Model_LeadsFilterPresetModel::FILTER_PRESET_TYPE_LEAD);
            if ($userRole !== Tools_Security_Acl::ROLE_SUPERADMIN && $userRole !== Tools_Security_Acl::ROLE_ADMIN) {
                $where .= ' AND ('.$leadFilterPresetMapper->getDbTable()->getAdapter()->quoteInto('creator_id = ?', $userId);
                $where .= ' OR '.$leadFilterPresetMapper->getDbTable()->getAdapter()->quoteInto('access = ?', 'all').')';
                $where .= ' AND '. $leadFilterPresetMapper->getDbTable()->getAdapter()->quoteInto('filter_preset_type = ?', 'lead');

            }
            $leadFiltersPreset = $leadFilterPresetMapper->fetchAll($where, array('filter_preset_name'));
            $defaultPreset = $leadFilterPresetMapper->getDefaultPreset($userId, Leads_Model_LeadsFilterPresetModel::FILTER_PRESET_TYPE_LEAD);
            if ($defaultPreset instanceof Leads_Model_LeadsFilterPresetModel) {
                $presetData = $defaultPreset->getFilterPresetData();
                if (!empty($presetData)) {
                    $this->_view->leadPresetConfig = json_decode($presetData, true);
                }
                $this->_view->leadPresetDefaultId = $defaultPreset->getId();
                $this->_view->leadPresetDefaultName = $defaultPreset->getFilterPresetName();
                $this->_view->leadPresetDefaultAccess = $defaultPreset->getAccess();
                $this->_view->leadPresetDefault = $defaultPreset->getIsDefault();
            }
            $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();
            $this->_view->userRole = $this->_sessionHelper->getCurrentUser()->getRoleId();
            $this->_view->countriesList = Tools_Geo::getCountries(true);
            $this->_view->leadOwners = $leadOwners;
            $this->_view->opportunityOwners = $opportunityOwners;
            $this->_view->leadStages = $leadStages;
            $this->_view->leadSourcesTypes = $leadSourcesTypes;
            $this->_view->leadFiltersPreset = $leadFiltersPreset;
            $this->_view->defaultCountryCode = $shoppingConfig['country'];
            $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
            $userDefaultMobileCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');
            $this->_view->defaultMobileDesktopCountryCode = $userDefaultMobileCountryCode;
            $this->_view->noLayout = true;
            $this->_view->customerGroups = $customerGroups;
            $this->_view->websiteConfig = Zend_Registry::get('website');
            $this->_view->lastInteractionsData = Leads_Mapper_LeadsTypeMapper::getInstance()->getTypes(true, Api_Leads_Leadtype::getExcludeSystemValues(), array('type_label'));
            $this->_view->lastInteractionsFullInfo = Leads_Mapper_LeadsTypeMapper::getInstance()->getTypes(true,
                Api_Leads_Leadtype::getExcludeSystemValues(), array('type_label'),
                array('id', 'type_label', 'type_name'));
            $this->_view->activitiesData = Leads_Mapper_LeadsTypeMapper::getInstance()->getTypes(true, Api_Leads_Leadtype::getExcludeActivitiesValues());
            $this->_view->leadTags = Leads_Mapper_LeadsTagsMapper::getInstance()->getTags(array('label'));
            $this->_view->leadOpportunityStages = Leads_Mapper_LeadsOpportunityStageMapper::getInstance()->getOpportunityStages(true);
            $this->_view->leadOpportunityTypes = Leads_Mapper_LeadsOpportunityTypeMapper::getInstance()->getOpportunityTypesPairs(array('type_label'));
            $this->_view->orderingDateAndInteractionValues = Api_Leads_Leadtype::orderingDateAndInteractionValues();
            $leadTitles = $leadMapper->getLeadTitles();
            $this->_view->leadTitles = $leadTitles;

            $leadsCustomParamsConfigMapper = Leads_Mapper_LeadsCustomParamsConfigMapper::getInstance();
            $organizationCustomParamsConfigMapper = Leads_Mapper_LeadsOrganizationCustomParamsConfigMapper::getInstance();
            $customAttributeTypes = array('text', 'select', 'checkbox');
            $whereLimitAttributes = $leadsCustomParamsConfigMapper->getDbTable()->getAdapter()->quoteInto('param_type IN (?)', $customAttributeTypes);
            $this->_view->leadCustomAttributesNames = $leadsCustomParamsConfigMapper->fetchAll($whereLimitAttributes, null, null, null, true);
            $this->_view->organizationCustomAttributesNames = $organizationCustomParamsConfigMapper->fetchAll($whereLimitAttributes, null, null, null, true);

            $this->_view->restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
            $this->_view->userTime = Tools_System_Tools::convertDateFromTimezone('now');
            $listMasksMapper = Application_Model_Mappers_MasksListMapper::getInstance();
            $this->_view->desktopMasks = $listMasksMapper->getListOfMasksByType(Application_Model_Models_MaskList::MASK_TYPE_DESKTOP);

            $this->_view->organizationIndustries =  Leads_Mapper_LeadsIndustryMapper::getInstance()->getIndustriesPairs(false, array('label'));
            $this->_view->emailSmsSequences =  Tools_LeadTools::getEmailSequenceListData();
            $this->_view->emailSmsSequencesArchived =  Tools_LeadTools::getEmailSequenceListData(array(), true);

            return $this->_view->render('lead-grid.phtml');
        }

        return '';

    }


    protected function _renderLeadforms()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            return $this->_view->render('form-leads.phtml');
        }
    }

    protected function _renderLeadtools()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            return $this->_view->render('lead-tools.phtml');
        }
    }

    protected function _renderCheckimapcredentials()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            return $this->_view->render('check-imap-credentials.phtml');
        }
    }

    protected function _renderSalespersondashaccount()
    {
        $session = Zend_Controller_Action_HelperBroker::getExistingHelper('session');

        $roleId = $session->getCurrentUser()->getRoleId();
        $leadsConfigMapper = Leads_Mapper_LeadsConfigMapper::getInstance();
        $salesPersonLimitedAccess = $leadsConfigMapper->getConfigParam('sales_person_limited_access');
        if ($roleId === Shopping::ROLE_SALESPERSON && !empty($salesPersonLimitedAccess)) {
            $this->_view->loggedUserId = $session->getCurrentUser()->getId();
            return $this->_view->render('client-profile-tab.phtml');
        }

        return '';
    }

    /**
     * Review request link
     *
     * @return string
     *
     */
    protected function _renderNpsReviewRequest()
    {
        if ($this->_leadDataModel instanceof Leads_Model_LeadsModel) {
            $leadId = $this->_leadDataModel->getId();
            if (isset($this->_options[1])) {
                if (!in_array($this->_options[1], Leads_Model_LeadsReviewRequestModel::$_allowedScore, true)) {
                    return $this->_translator->translate(Leads::TRANSLATION_PREFIX.'Wrong option param. Please specify in range 1 - 10');
                }

                $token = sha1($leadId);
                $leadsReviewRequestMapper = Leads_Mapper_LeadsReviewRequestMapper::getInstance();
                $reviewRequestModel = $leadsReviewRequestMapper->findByToken($token);
                if (!$reviewRequestModel instanceof Leads_Model_LeadsReviewRequestModel) {
                    $leadsReviewRequestModel = new Leads_Model_LeadsReviewRequestModel();
                    $leadsReviewRequestModel->setReviewToken($token);
                    $leadsReviewRequestModel->setLeadId($leadId);
                    $leadsReviewRequestModel->setStatus(Leads_Model_LeadsReviewRequestModel::STATUS_SENT);
                    $leadsReviewRequestModel->setSentAt(Tools_System_Tools::convertDateFromTimezone('now'));
                    $leadsReviewRequestModel->setScore('0');
                    $leadsReviewRequestMapper->save($leadsReviewRequestModel);
                } else {
                    $reviewStatus = $reviewRequestModel->getStatus();
                    $reviewSentAt = $reviewRequestModel->getSentAt();
                    if (!empty($reviewStatus) && $reviewStatus === Leads_Model_LeadsReviewRequestModel::STATUS_SENT) {
                        if (!empty($reviewSentAt)) {
                            if (strtotime($reviewSentAt) < strtotime(Tools_System_Tools::convertDateFromTimezone(date('Y-m-d', strtotime('-1 month')). ' 00:00:00'))){
                                $reviewRequestModel->setSentAt(Tools_System_Tools::convertDateFromTimezone('now'));
                                $leadsReviewRequestMapper->save($reviewRequestModel);
                            }
                        }
                    }
                }

                return $this->_websiteHelper->getUrl() . 'plugin/leads/run/npsReviewRequest/score/' . $this->_options[1] . '/reviewToken/' . $token;
            }

            return '';

        } else {
            return '{$lead:reviewrequest}';
        }
    }

    /*
     * lead organization logo
     * @return string
     */
    protected function _renderOrganizationlogo()
    {
        if(!empty($this->_isDemo)) {
            if(in_array('src', $this->_options)) {
                return $this->_websiteHelper->getUrl() . Tools_LeadTools::$_demoData['organizationlogo'];
            }

            return '<img src="'. $this->_websiteHelper->getUrl() . Tools_LeadTools::$_demoData['organizationlogo'] .'" alt="organization logo">';
        } elseif ($this->_leadDataModel instanceof Leads_Model_LeadsModel) {
            $organizationId = $this->_leadDataModel->getOrganizationId();
            $leadEmail = $this->_leadDataModel->getLeadEmail();
            if(!empty($organizationId) && !empty($leadEmail)){
                $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
                $organizationDocumentData = $leadMapper->findOrganizationDocument($leadEmail);

                if(!empty($organizationDocumentData)){
                    if(in_array('src', $this->_options)) {
                        return $this->_websiteHelper->getUrl() . Leads::ORGANIZATION_LOGOS_IMAGES_PATH . DIRECTORY_SEPARATOR . $organizationDocumentData['file_stored_name'];
                    } else {
                        return '<img src="'. $this->_websiteHelper->getUrl() . Leads::ORGANIZATION_LOGOS_IMAGES_PATH . DIRECTORY_SEPARATOR . $organizationDocumentData['file_stored_name'] .'" alt="'. $organizationDocumentData['display_file_name'] .'">';
                    }
                }
            }
            return '';
        } elseif (in_array('userid', $this->_options)) {
            $key = array_search('userid', $this->_options);

            if(!empty($this->_options[$key+1])) {
                $userId = $this->_options[$key+1];

                $userModel = Application_Model_Mappers_UserMapper::getInstance()->find($userId);

                if ($userModel instanceof Application_Model_Models_User) {
                    $organizationDocumentData = Tools_LeadTools::getOrganizationLogo($userId);

                    if(!empty($organizationDocumentData)){
                        if(in_array('src', $this->_options)) {
                            return $this->_websiteHelper->getUrl() . Leads::ORGANIZATION_LOGOS_IMAGES_PATH . DIRECTORY_SEPARATOR . $organizationDocumentData['file_stored_name'];
                        } else {
                            return '<img src="'. $this->_websiteHelper->getUrl() . Leads::ORGANIZATION_LOGOS_IMAGES_PATH . DIRECTORY_SEPARATOR . $organizationDocumentData['file_stored_name'] .'" alt="'. $organizationDocumentData['display_file_name'] .'">';
                        }
                    }
                }
            }
        } else {
            if(in_array('src', $this->_options)) {
                return '{$lead:organizationlogo:src}';
            }
            return '{$lead:organizationlogo}';
        }
    }

    /**
     * lead id
     * @return string
     */
    protected function _renderId()
    {
        if(!empty($this->_isDemo)) {
            return Tools_LeadTools::$_demoData['id'];
        } elseif ($this->_leadDataModel instanceof Leads_Model_LeadsModel) {
            return $this->_leadDataModel->getId();
        } else {
            return '{$lead:id}';
        }
    }

    /**
     * lead source
     * @return string
     */
    protected function _renderSource()
    {
        if(!empty($this->_isDemo)) {
            return Tools_LeadTools::$_demoData['source'];
        } elseif ($this->_leadDataModel instanceof Leads_Model_LeadsModel) {
            $leadSource = $this->_leadDataModel->getLeadSource();
            if (!empty($leadSource)) {
                return $leadSource;
            }
            return '';
        } else {
            return '{$lead:source}';
        }
    }

    protected function _renderLeadfullnamelink() {
        if(!empty($this->_isDemo)) {
            $link = $this->_websiteHelper->getUrl().'dashboard/leads/';
            $link = '<a href="'.$link.'">'. Tools_LeadTools::$_demoData['firstname'].' '. Tools_LeadTools::$_demoData['lastname'] .'</a>';

            return $link;
        } elseif ($this->_leadDataModel instanceof Leads_Model_LeadsModel) {
            $link = $this->_websiteHelper->getUrl().'crmLogin/tabName/dashboard/subTab/leads/paramHashName/lead/paramHash/'.$this->_leadDataModel->getId().'/';
            $link = '<a href="'.$link.'">'. $this->_leadDataModel->getLeadFirstName().' '. $this->_leadDataModel->getLeadLastName() .'</a>';

            return $link;
        } else {
            return '{$lead:leadfullnamelink}';
        }
    }

    protected function _renderOrganizationnamelink() {
        if(!empty($this->_isDemo)) {
            $orgLink = $this->_websiteHelper->getUrl() . 'dashboard/organizations/';
            $orgLink = '<a href="' . $orgLink . '">'. Tools_LeadTools::$_demoData['organizationname'] .'</a>';

            return $orgLink;
        } elseif ($this->_leadDataModel instanceof Leads_Model_LeadsModel) {
            $organizationId = $this->_leadDataModel->getOrganizationId();
            $leadEmail = $this->_leadDataModel->getLeadEmail();
            if(!empty($organizationId) && !empty($leadEmail)){
                $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
                $leadOrganizationName = $leadMapper->findByLeadEmail($leadEmail);
                if(!empty($leadOrganizationName)){
                    $orgLink = $this->_websiteHelper->getUrl() . 'crmLogin/tabName/dashboard/subTab/organizations/paramHashName/organization/paramHash/' . $organizationId . '/';
                    $orgLink = '<a href="' . $orgLink . '">'. $leadOrganizationName['organization_name'] .'</a>';

                    return $orgLink;
                }
            }
            return '';
        } else {
            return '{$lead:organizationnamelink}';
        }
    }

    protected function _renderQuotetitlelink() {
        if(!empty($this->_isDemo)) {
            $quoteTitleLink = $this->_websiteHelper->getUrl() . 'dashboard/quotes/';

            return $quoteTitleLink;
        } elseif ($this->_leadDataModel instanceof Leads_Model_LeadsModel) {
            $additionalInfo = $this->_leadDataModel->getAdditionalInfo();
            if (!empty($additionalInfo['quoteId'])) {
                $quoteModel = Quote_Models_Mapper_QuoteMapper::getInstance()->find($additionalInfo['quoteId']);
                if ($quoteModel instanceof Quote_Models_Model_Quote) {
                    $quoteTitleLink = $this->_websiteHelper->getUrl().$quoteModel->getId().'.html';
                    $quoteTitleLink = '<a href="' . $quoteTitleLink . '">'. $quoteModel->getTitle().'</a>';
                    return $quoteTitleLink;
                }
            }
            return '';
        } else {
            return '{$lead:quotetitlelink}';
        }
    }

    protected function _renderQuoteamount() {
        if(!empty($this->_isDemo)) {
            if (!Zend_Registry::isRegistered('Zend_Currency')) {
                $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();

                $country = $shoppingConfig['country'];
                if(!empty($shoppingConfig['currencyCountry'])) {
                    $country = $shoppingConfig['currencyCountry'];
                }

                $locale = Zend_Locale::getLocaleToTerritory($country);
                try {
                    $currency = new Zend_Currency(
                        $shoppingConfig['currency'],
                        $locale
                    );
                } catch (Zend_Currency_Exception $e) {
                    $currency = new Zend_Currency();
                }

                return $currency->toCurrency(0);
            }

            return '';
        } elseif ($this->_leadDataModel instanceof Leads_Model_LeadsModel) {
            $additionalInfo = $this->_leadDataModel->getAdditionalInfo();
            if (!empty($additionalInfo['quoteId'])) {
                $quoteModel = Quote_Models_Mapper_QuoteMapper::getInstance()->find($additionalInfo['quoteId']);
                if ($quoteModel instanceof Quote_Models_Model_Quote) {
                    $cartModel = Models_Mapper_OrdersMapper::getInstance()->find($quoteModel->getCartId());
                    if ($cartModel instanceof Models_Model_CartSession) {
                        if (!Zend_Registry::isRegistered('Zend_Currency')) {
                            $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();

                            $country = $shoppingConfig['country'];
                            if(!empty($shoppingConfig['currencyCountry'])) {
                                $country = $shoppingConfig['currencyCountry'];
                            }

                            $locale = Zend_Locale::getLocaleToTerritory($country);
                            try {
                                $currency = new Zend_Currency(
                                    $shoppingConfig['currency'],
                                    $locale
                                );
                            } catch (Zend_Currency_Exception $e) {
                                $currency = new Zend_Currency();
                            }

                            return $currency->toCurrency($cartModel->getTotal());
                        }
                    }
                }
            }
            return '';
        } else {
            return '{$lead:quoteamount}';
        }
    }


}
