<?php

class Widgets_Leadorganization_Leadorganization extends Widgets_Abstract
{

    /**
     * Website helper
     *
     * @var null
     */
    protected $_websiteHelper = null;

    /**
     * @var bool
     */
    protected $_cacheable = false;

    /**
     * Session helper
     *
     * @var bool
     */
    protected $_sessionHelper = false;

    /**
     * @var null
     */
    protected $_layout = null;

    /**
     * Seotoaster config helper
     *
     * @var Helpers_Action_Config
     */
    protected $_configHelper = null;

    protected function _init()
    {
        parent::_init();
        $this->_view = new Zend_View(array(
            'scriptPath' => __DIR__ . '/views/'
        ));


        $this->_layout = new Zend_Layout();
        $this->_layout->setLayoutPath(Zend_Layout::getMvcInstance()->getLayoutPath());

        $this->_websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
        $this->_view->websiteUrl = $this->_websiteHelper->getUrl();
        $this->_websiteUrl = $this->_websiteHelper->getUrl();
        $this->_translator = Zend_Registry::get('Zend_Translate');

        $this->_view->setHelperPath(APPLICATION_PATH . '/views/helpers/');
        $this->_view->addHelperPath('ZendX/JQuery/View/Helper/', 'ZendX_JQuery_View_Helper');
        $this->_view->addScriptPath($this->_websiteHelper->getPath() . 'seotoaster_core/application/views/scripts/');
        // init helpers
        $this->_configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
        $this->_sessionHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
    }

    protected function _load()
    {
        if (empty($this->_options[0])) {
            return $this->_translator->translate(Leads::TRANSLATION_PREFIX.'Option not specified');
        }
        $methodName = '_render' . ucfirst($this->_options[0]);
        if (method_exists($this, $methodName)) {
            return $this->$methodName();

        }

        return $this->_translator->translate(Leads::TRANSLATION_PREFIX.'Method doesn\'t exists');
    }

    protected function _renderGrid()
    {

        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $customerGroups = Store_Mapper_GroupMapper::getInstance()->fetchAll(null, array('groupName'));
            $currentUser = $this->_sessionHelper->getCurrentUser();
            $userId = $currentUser->getId();
            $userRole = $currentUser->getRoleId();
            $where = null;
            $leadFilterPresetMapper = Leads_Mapper_LeadsFilterPresetMapper::getInstance();
            $where = $leadFilterPresetMapper->getDbTable()->getAdapter()->quoteInto('filter_preset_type = ?', Leads_Model_LeadsFilterPresetModel::FILTER_PRESET_TYPE_ORGANIZATION);
            if ($userRole !== Tools_Security_Acl::ROLE_SUPERADMIN && $userRole !== Tools_Security_Acl::ROLE_ADMIN) {
                $where .= ' AND ('.$leadFilterPresetMapper->getDbTable()->getAdapter()->quoteInto('creator_id = ?', $userId);
                $where .= ' OR '.$leadFilterPresetMapper->getDbTable()->getAdapter()->quoteInto('access = ?', 'all').')';
                $where .= ' AND '. $leadFilterPresetMapper->getDbTable()->getAdapter()->quoteInto('filter_preset_type = ?', 'organization');
            }
            $leadOrganizationFiltersPreset = $leadFilterPresetMapper->fetchAll($where, array('filter_preset_name'));
            $defaultPreset = $leadFilterPresetMapper->getDefaultPreset($userId, Leads_Model_LeadsFilterPresetModel::FILTER_PRESET_TYPE_ORGANIZATION);
            if ($defaultPreset instanceof Leads_Model_LeadsFilterPresetModel) {
                $presetData = $defaultPreset->getFilterPresetData();
                if (!empty($presetData)) {
                    $this->_view->leadOrganizationPresetConfig = json_decode($presetData, true);
                }
                $this->_view->leadOrganizationPresetDefaultId = $defaultPreset->getId();
                $this->_view->leadOrganizationPresetDefaultName = $defaultPreset->getFilterPresetName();
                $this->_view->leadOrganizationPresetDefaultAccess = $defaultPreset->getAccess();
                $this->_view->leadOrganizationPresetDefault = $defaultPreset->getIsDefault();
            }
            $this->_view->countriesList = Tools_Geo::getCountries(true);
            $this->_view->userRole = $this->_sessionHelper->getCurrentUser()->getRoleId();
            $this->_view->leadOrganizationFiltersPreset = $leadOrganizationFiltersPreset;
            $this->_view->organizationLastInteractions = Leads_Mapper_LeadsTypeMapper::getInstance()->getTypes(true, Api_Leads_Leadtype::getExcludeSystemValues(), array('type_label'));
            $this->_view->organizationIndustries =  Leads_Mapper_LeadsIndustryMapper::getInstance()->getIndustriesPairs(false, array('label'));
            $this->_view->leadOpportunityStages = Leads_Mapper_LeadsOpportunityStageMapper::getInstance()->getOpportunityStages(true);
            $this->_view->leadTags = Leads_Mapper_LeadsTagsMapper::getInstance()->getTags(array('label'));
            $this->_view->noLayout = true;
            $this->_view->customerGroups = $customerGroups;
            $this->_view->websiteConfig = Zend_Registry::get('website');
            $this->_view->leadOpportunityTypes = Leads_Mapper_LeadsOpportunityTypeMapper::getInstance()->getOpportunityTypesPairs(array('type_label'));
            $this->_view->userTime = Tools_System_Tools::convertDateFromTimezone('now');
            $listMasksMapper = Application_Model_Mappers_MasksListMapper::getInstance();
            $this->_view->desktopMasks = $listMasksMapper->getListOfMasksByType(Application_Model_Models_MaskList::MASK_TYPE_DESKTOP);
            $userDefaultMobileCountryCode = $this->_configHelper->getConfig('userDefaultPhoneMobileCode');
            $this->_view->restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
            $organizationCustomParamsConfigMapper = Leads_Mapper_LeadsOrganizationCustomParamsConfigMapper::getInstance();
            $customAttributeTypes = array('text', 'select', 'checkbox');
            $whereLimitAttributes = $organizationCustomParamsConfigMapper->getDbTable()->getAdapter()->quoteInto('param_type IN (?)', $customAttributeTypes);
            $this->_view->organizationCustomAttributesNames = $organizationCustomParamsConfigMapper->fetchAll($whereLimitAttributes, null, null, null, true);
            $this->_view->organizationLastInteractions = Leads_Mapper_LeadsTypeMapper::getInstance()->getTypes(true, Api_Leads_Leadtype::getExcludeSystemValues(), array('type_label'));
            $this->_view->lastInteractionsFullInfo = Leads_Mapper_LeadsTypeMapper::getInstance()->getTypes(true,
                Api_Leads_Leadtype::getExcludeSystemValues(), array('type_label'),
                array('id', 'type_label', 'type_name'));

            if(!empty($userDefaultMobileCountryCode)) {
                $this->_view->defaultCountryCode = $userDefaultMobileCountryCode;
            }

            return $this->_view->render('organization-grid.phtml');
        }

        return '';

    }


}
