<?php
class Widgets_Leadowner_Leadowner extends Widgets_Abstract
{

    private static $_zendRegistryKey = 'leadOwnerObserver';

    protected $_cacheable  = false;

    /**
     * @var null
     */
    protected $_ownerData = array();

    /**
     * Get Lead demo data by test sequence
     *
     * @var bool
     */
    protected $_isDemo = false;

    protected function _load()
    {
        if (empty($this->_options[0])) {
            return $this->_translator->translate(Leads::TRANSLATION_PREFIX.'Option not specified');
        }

        if(!empty($this->_toasterOptions['observableModel']) && is_object($this->_toasterOptions['observableModel'])){
            $observableModel = $this->_toasterOptions['observableModel'];
            if ($observableModel instanceof Emailsequence_Models_Models_EmailsequenceQueueModel){
                $userEmail = $observableModel->getUserEmail();

                if(!empty($userEmail)){
                    $userRegistryEmail = preg_replace ("/[^a-zA-Z0-9-_]/", "", $userEmail);

                    $regKey = self::$_zendRegistryKey.'_'.$userRegistryEmail;

                    if (Zend_Registry::isRegistered($regKey)) {
                        $this->_ownerData = Zend_Registry::get($regKey);
                    } else {

                        $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
                        $this->_ownerData = $leadMapper->findOwnerByUserEmail($userEmail);

                        if (!empty($this->_ownerData)){
                            Zend_Registry::set($regKey, $this->_ownerData);
                        }
                    }
                }
            } elseif ($observableModel instanceof Leads_Model_LeadsModel) {
                $ownerId = $observableModel->getOwnerId();
                if (!empty($ownerId)) {
                    $regKey = self::$_zendRegistryKey.'_'.$ownerId;
                    if (Zend_Registry::isRegistered($regKey)) {
                        $this->_ownerData = Zend_Registry::get($regKey);
                    } else {
                        $ownerModel = Application_Model_Mappers_UserMapper::getInstance()->find($ownerId);
                        if ($ownerModel instanceof Application_Model_Models_User) {
                            $this->_ownerData = array(
                                'fullname' => $ownerModel->getFullName(),
                                'email' => $ownerModel->getEmail(),
                                'mobilephone' => $ownerModel->getMobilePhone(),
                                'mobilecountrycode' => $ownerModel->getMobileCountryCode(),
                                'desktopcountrycode' => $ownerModel->getDesktopCountryCode(),
                                'desktopphone' => $ownerModel->getDesktopPhone(),
                                'fullmobilephone' => $ownerModel->getMobileCountryCodeValue().$ownerModel->getMobilePhone(),
                                'fulldesktopphone' => $ownerModel->getDesktopCountryCodeValue().$ownerModel->getDesktopPhone(),
                                'signature' => $ownerModel->getSignature(),
                                'personalcalendarurl' => $ownerModel->getPersonalCalendarUrl(),
                                'avatarlink' => $ownerModel->getAvatarLink(),
                            );
                            Zend_Registry::set($regKey, $this->_ownerData);
                        } else {
                            return '';
                        }
                    }
                } else {
                    return '';
                }
            }

            if(!empty($this->_toasterOptions['isDemo'])) {
                $this->_isDemo = true;
            }
        }

        $methodName = '_render' . ucfirst($this->_options[0]);

        if (method_exists($this, $methodName)) {
            return $this->$methodName($this->_options[0]);
        } else {
            return $this->_translator->translate(Leads::TRANSLATION_PREFIX.'Method doesn\'t exists');
        }
    }

    /**
     * lead owner fullname
     * @param string $option option name
     * @return string
     */
    protected function _renderFullname($option)
    {
        if(!empty($this->_isDemo)) {
            return Tools_LeadTools::$_demoData['leadownerfullname'];
        } elseif (!empty($this->_ownerData)) {
            return (!empty($this->_ownerData[$option])) ? $this->_ownerData[$option] : '';
        } else {
            return '{$leadowner:fullname}';
        }
    }

    /**
     * lead owner email
     * @param string $option option name
     * @return string
     */
    protected function _renderEmail($option)
    {
        if(!empty($this->_isDemo)) {
            return Tools_LeadTools::$_demoData['leadowneremail'];
        } elseif (!empty($this->_ownerData)) {
            return (!empty($this->_ownerData[$option])) ? $this->_ownerData[$option] : '';
        } else {
            return '{$leadowner:email}';
        }
    }

    /**
     *  lead owner mobilecountrycode
     * @param string $option option name
     * @return string
     */
    protected function _renderMobilecountrycode($option)
    {
        if(!empty($this->_isDemo)) {
            return Tools_LeadTools::$_demoData['leadownermobilecountrycode'];
        } elseif (!empty($this->_ownerData)) {
            return (!empty($this->_ownerData[$option])) ? $this->_ownerData[$option] : '';
        } else {
            return '{$leadowner:mobilecountrycode}';
        }
    }

    /**
     * lead owner mobilephone
     * @param string $option option name
     * @return string
     */
    protected function _renderMobilephone($option)
    {
        if(!empty($this->_isDemo)) {
            return Tools_LeadTools::$_demoData['leadownermobilephone'];
        } elseif (!empty($this->_ownerData)) {
            return (!empty($this->_ownerData[$option])) ? $this->_ownerData[$option] : '';
        } else {
            return '{$leadowner:mobilephone}';
        }
    }

    /**
     * lead owner desktopcountrycode
     * @param string $option option name
     * @return string
     */
    protected function _renderDesktopcountrycode($option)
    {
        if(!empty($this->_isDemo)) {
            return Tools_LeadTools::$_demoData['leadownerdesktopcountrycode'];
        } elseif (!empty($this->_ownerData)) {
            return (!empty($this->_ownerData[$option])) ? $this->_ownerData[$option] : '';
        } else {
            return '{$leadowner:desktopcountrycode}';
        }
    }

    /**
     * lead owner descktopphone
     * @param string $option option name
     * @return string
     */
    protected function _renderDesktopphone($option)
    {
        if(!empty($this->_isDemo)) {
            return Tools_LeadTools::$_demoData['leadownerdesktopphone'];
        } elseif (!empty($this->_ownerData)) {
            return (!empty($this->_ownerData[$option])) ? $this->_ownerData[$option] : '';
        } else {
            return '{$leadowner:desktopphone}';
        }
    }

    /**
     * Get full desktop number with country code
     *
     * @param string $option option name
     * @return string
     */
    protected function _renderFulldesktopphone($option)
    {
        if(!empty($this->_isDemo)) {
            return Tools_LeadTools::$_demoData['leadownerfulldesktopphone'];
        } elseif (!empty($this->_ownerData)) {
            return (!empty($this->_ownerData[$option])) ? $this->_ownerData[$option] : '';
        } else {
            return '{$leadowner:fulldesktopphone}';
        }
    }

    /**
     * Get full mobile number with country code
     * @param string $option option name
     * @return string
     */
    protected function _renderFullmobilephone($option)
    {
        if(!empty($this->_isDemo)) {
            return Tools_LeadTools::$_demoData['leadownerfullmobilephone'];
        } elseif (!empty($this->_ownerData)) {
            return (!empty($this->_ownerData[$option])) ? $this->_ownerData[$option] : '';
        } else {
            return '{$leadowner:fullmobilephone}';
        }
    }

    /**
     * Lead owner Signature
     * @param string $option option name
     * @return string
     */
    protected function _renderSignature($option)
    {
        if(!empty($this->_isDemo)) {
            $leadownersignature = Tools_LeadTools::$_demoData['leadownersignature'];

            $loggedUser = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser();
            if($loggedUser instanceof Application_Model_Models_User) {
                $userSignature = $loggedUser->getSignature();
                if(!empty($userSignature)) {
                    $leadownersignature = $userSignature;
                }
            }

            return $leadownersignature;
        } elseif (!empty($this->_ownerData)) {
            return (!empty($this->_ownerData[$option])) ? $this->_ownerData[$option] : '';
        } else {
            return '{$leadowner:signature}';
        }
    }

    /**
     * Lead owner personal calendar url
     * @param string $option option name
     * @return string
     */
    protected function _renderPersonalcalendarurl($option)
    {
        if(!empty($this->_isDemo)) {
            $leadownerpersonalcalendarurl = Tools_LeadTools::$_demoData['personalcalendarurl'];

            $loggedUser = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser();
            if($loggedUser instanceof Application_Model_Models_User) {
                $userpersonalcalendarurl = $loggedUser->getPersonalCalendarUrl();
                if(!empty($userpersonalcalendarurl)) {
                    $leadownerpersonalcalendarurl = $userpersonalcalendarurl;
                }
            }

            return $leadownerpersonalcalendarurl;
        } elseif (!empty($this->_ownerData)) {
            return (!empty($this->_ownerData[$option])) ? $this->_ownerData[$option] : '';
        } else {
            return '{$leadowner:personalcalendarurl}';
        }
    }

    /*
     * Lead owner avatar link
     * @return string
     */
    protected function _renderAvatarlink($option)
    {
        if(!empty($this->_isDemo)) {
            if(in_array('src', $this->_options)) {
                return Tools_LeadTools::$_demoData['avatarlink'];
            }

            return '<img src="'. Tools_LeadTools::$_demoData['avatarlink'] .'" alt="avatar link">';
        } elseif (!empty($this->_ownerData)) {
            if(!empty($this->_ownerData[$option])) {
                if(in_array('src', $this->_options)) {
                    return $this->_ownerData[$option];
                } else {
                    return '<img src="'. $this->_ownerData[$option] .'" alt="avatar link">';
                }
            }

            return '';
        } else {
            if(in_array('src', $this->_options)) {
                return '{$leadowner:avatarlink:src}';
            }
            return '{$leadowner:avatarlink}';
        }
    }

}
