import {mapGetters} from 'vuex';
import lodash from 'lodash';
import localeMapping from '../../../localizationLanguages';
import pagination from '../../pagination';
export default {
    data () {
        return {
            loadedScreen: false,
            queryType: 'tasks',
            subType: 'all',
            websiteUrl: $('#website_url').val(),
            hideTaskDialog: true,
            taskNotifyEmail:'',
            taskTitle:'',
            taskDialogId:0,
            checkedTasks:{},
            taskComment:''
        }
    },
    components: {
        pagination: pagination
    },
    computed: {
        ...mapGetters({
            formatDate: 'formatDate',
            formatOnlyDate: 'formatOnlyDate',
            formatTimeOnly: 'formatTimeOnly',
            configDataInfo:'getConfigDataInfo',
            additionalInfo:'getAdditionalInfo',
            sortByColumn: 'sortByColumn',
            truncateText: 'truncateText',
            selectedUserId: 'getSelectedUserId',
            leadMeetingsNotesData: 'getLeadMeetingsNotesData',
            cleanText: 'cleanText'
        })
    },
    watch: {
        selectedUserId (newUserId, oldUseId) {
            console.log(`Old user id ${oldUseId} - new user id ${newUserId}`);
            if (newUserId !== oldUseId) {
                this.$store.commit('setPaginationData',
                    {
                        leadTasksMeetingsNotes: {
                            currentPage: 1,
                            itemsPerPage: 10,
                            totalItems: 0,
                            visiblePages: 4
                        }
                    }
                );

                this.applyFilter();
            }
        }
    },
    methods: {
        filterByType(queryType) {
            this.queryType = queryType;
            this.subType = 'all';
            this.$store.commit('setPaginationData',
                {
                    leadTasksMeetingsNotes: {
                        currentPage: 1,
                        itemsPerPage: 10,
                        totalItems: 0,
                        visiblePages: 4
                    }
                }
            );
            this.applyFilter();
        },
        applySubFilter(querySubType) {
            if (querySubType !== this.subType) {
                this.$store.commit('setPaginationData',
                    {
                        leadTasksMeetingsNotes: {
                            currentPage: 1,
                            itemsPerPage: 10,
                            totalItems: 0,
                            visiblePages: 4
                        }
                    }
                );
            }
            this.subType = querySubType;
            this.applyFilter();
        },
        markTaskCompleted(event, id, index) {
            if (event.target.checked) {
                this.hideTaskDialog = false;
                this.checkedTasks = {};
                this.checkedTasks[id] = id;
                this.taskTitle = this.leadMeetingsNotesData[index]['title'];
                this.taskDialogId = id;
                if (this.leadMeetingsNotesData[index]['userEmail'] === this.leadMeetingsNotesData[index]['creatorEmail']) {
                    this.taskNotifyEmail = this.leadMeetingsNotesData[index]['userEmail'];
                } else {
                    this.taskNotifyEmail = this.leadMeetingsNotesData[index]['userEmail'] + ',' + this.leadMeetingsNotesData[index]['creatorEmail'];
                }
            } else {
                delete this.checkedTasks[id];
                this.hideTaskDialog = true;
            }
        },
        closeTaskDialog(id) {
            this.hideTaskDialog = true;
            delete this.checkedTasks[id];
            this.taskComment = '';
        },
        async completeTask(){
            const result = await this.$store.dispatch('completeTask', {'router':this.$router, 'taskNotifyEmails': this.taskNotifyEmail, 'taskId':this.taskDialogId, 'taskComment':this.taskComment});
            console.log(result);
            if(result.status === 'error') {
                showMessage('Please re-login', true, 3000);
            } else {
                showMessage(result.responseText.message, false, 3000);
                this.hideTaskDialog = true;
                this.taskComment = '';
                this.applyFilter();
            }
        },
        async applyFilter() {
            const result = await this.$store.dispatch('getTasksMeetingsNotesData', {'router':this.$router, 'queryType': this.queryType, 'userId':this.selectedUserId, 'querySubType':this.subType});
            if(result.status === 'error') {
                showMessage('Please re-login', true, 3000);
            } else {
                this.loadedScreen = true;
            }
        }
    },
    async created(){
        const result = await this.$store.dispatch('getTasksMeetingsNotesData', {'router':this.$router, 'queryType': this.queryType, 'userId':this.selectedUserId, 'querySubType':this.subType});
        if(result.status === 'error') {
            showMessage('Please re-login', true, 3000);
        } else {
            this.loadedScreen = true;
        }

    },
    async updated() {
        this.$nextTick(function () {
            if (typeof checkboxRadioStyle !== 'undefined' && typeof checkboxRadioStyle() === "function") {
                checkboxRadioStyle();
            }
        })
    }
}
