define([
    'backbone',
    'text!../templates/lead-task-detailed-tab.html',
    'i18n!../../../nls/dashboard_'+$('#main-dashboard-system-language').val()+'_ln',
    'tinyMCE',
    'leadshortcode',
], function(Backbone, leadTaskTmpl, i18n, tinymce, leadshortcode){
    var LeadTaskFormView = Backbone.View.extend({
        el: $('#lead-detailed-view-config-tab-task'),
        events: {
            'submit form': 'submit',
            'change #lead-task-start-immediate': 'switchImmediate',
            'keyup #lead-task-type-detailed-view': 'checkMandatory',
            'change #lead-task-date-detailed-view': 'checkMandatory',
            'change #lead-task-due-date-detailed-view': 'checkMandatory'
        },
        templates: {

        },
        initialize: function(leadModel, leadTaskDataModel){
            this.leadModel = leadModel;
            this.leadTaskDataModel = leadTaskDataModel;
        },
        render: function(){
            this.$el.empty();
            tinymce.remove();
            var presetedParams = this.getPresetedNotesParams(),
                defaultValues = '';

            this.$el.append(_.template(leadTaskTmpl, {'leadModel': this.leadModel, 'leadTaskDataModel': this.leadTaskDataModel, 'presetedParams' : presetedParams, 'i18n' : i18n}));
            this.$el.find('form').attr('action', $('#website_url').val()+'api/leads/leadtask');
            this.$el.find('#add-lead-task-action').prop('disabled', true);

            $('#lead-task-date-detailed-view').datepicker({
                dateFormat: 'd-M-yy',
                defaultDate: "+1w",
                changeMonth: true,
                changeYear: true,
                yearRange: "c-5:c+5",
                minDate: 0
            });

            $('#lead-task-due-date-detailed-view').datepicker({
                dateFormat: 'd-M-yy',
                defaultDate: "+1w",
                changeMonth: true,
                changeYear: true,
                yearRange: "c-5:c+5",
                minDate: 0
            });
            this.initTiny();
            if (presetedParams.length) {
                _.each(presetedParams, function (value, id) {
                    defaultValues += value + ' '
                })
            }
            setTimeout(function () {
                if (!_.isNull(tinymce.activeEditor) && '' === tinymce.activeEditor.getContent()) {
                    tinymce.activeEditor.setContent('Hello! ' +
                        'You have a task for this lead: {$lead:leadfullnamelink} from {$lead:organizationnamelink}.');
                }
            }, 500);
            return this;
        },
        getPresetedNotesParams: function(){
            var presetedParams = [],
                leadphone = $('.lead-phone-detailed-view').val(),
                leadMobile = $('.lead-mobile-detailed-view').val(),
                phoneCountryCodeText = $('#lead-phone-countries-codes option:selected').text(),
                mobileCountryCodeText = $('#lead-mobile-countries-codes option:selected').text();

            if(leadphone.replace(/\D+/g, '') != '') {
                var clearPhoneCountryCode = phoneCountryCodeText.replace(/[^-0-9+]/gim, '');
                var leadPhoneNumber =  clearPhoneCountryCode + leadphone.replace(/\D+/g, '');
                presetedParams.push(leadPhoneNumber);
            } else if(leadMobile.replace(/\D+/g, '') != '') {
                var clearMobileCountryCode = mobileCountryCodeText.replace(/[^-0-9+]/gim, '');
                var leadMobileNumber = clearMobileCountryCode + leadMobile.replace(/\D+/g, '');
                presetedParams.push(leadMobileNumber);
            }

            return presetedParams;
        },
        submit: function(e){
            e.preventDefault();

            var self = this,
                form = $(e.currentTarget),
                isValid = true,
                startTime = $('#lead-task-start-time').val(),
                dueTime = $('#lead-task-due-time').val(),
                startTimeMinutes = $('#lead-task-start-time-minutes').val(),
                endTimeMinutes = $('#lead-task-end-time-minutes').val();

            _.each(form.find('.required'), function(el){
                if (!$(el).val()){
                    isValid = false;
                }
            });

            if (!isValid){
                showMessage(_.isUndefined(i18n['Missing required field'])?'Missing required field':i18n['Missing required field'], true);
                return false;
            }


            form.find('input[name=expired_at]').val(($('#lead-task-date-detailed-view').val()) ? $.datepicker.formatDate('yy-mm-dd', $('#lead-task-date-detailed-view').datepicker("getDate"))+' '+startTime+':'+startTimeMinutes+':00': '');

            form.find('input[name=due_date]').val(($('#lead-task-due-date-detailed-view').val()) ? $.datepicker.formatDate('yy-mm-dd', $('#lead-task-due-date-detailed-view').datepicker("getDate"))+' '+dueTime+':'+endTimeMinutes+':00': '');

            var immediate = 0,
                immediateEmail = 0;

            if ($('#lead-task-start-immediate').is(':checked')) {
                immediate = 1;
            }

            if ($('#lead-task-immediate-notification').is(':checked')) {
                immediateEmail = 1;
            }

            var formParams = form.serialize()+'&secureToken='+$('#lead-grid-token').val()+'&immediate='+immediate+'&immediateEmail='+immediateEmail;


            $.ajax({
                url: form.attr('action'),
                type: 'POST',
                data: formParams,
                dataType: 'json'
            }).done(function(response) {
                showMessage(response.responseText, false, 5000);
                self.render();
            }).fail(function(response){
                showMessage(response.responseJSON, true, 5000);
            });
            return false;

        },
        switchImmediate: function(e)
        {
            var self = this;

            if ($(e.currentTarget).is(':checked')) {
                $('#lead-task-date-detailed-view').prop('disabled', true);
                $('#lead-task-start-time').prop('disabled', true);
                $('#lead-task-immediate-notification-block').addClass('hidden');
                $('#lead-task-start-time-minutes').prop('disabled', true);
                $.ajax({
                    url: $('#website_url').val()+'plugin/leads/run/getCurrentDate/',
                    type: 'POST',
                    data: {},
                    dataType: 'json'
                }).done(function(response) {
                    if ($('#lead-task-date-detailed-view').val() == '') {
                        $('#lead-task-date-detailed-view').val(response.responseText.currentDate);
                    }

                    if ($('#lead-task-due-date-detailed-view').val() == '') {
                        $('#lead-task-due-date-detailed-view').val(response.responseText.currentDate);
                    }

                    $('#lead-task-due-time').val(response.responseText.dateDue);

                    self.checkMandatory();
                }).fail(function(response){
                    showMessage(response.responseJSON, true, 5000);
                });

            } else {
                $('#lead-task-date-detailed-view').prop('disabled', false);
                $('#lead-task-start-time').prop('disabled', false);
                $('#lead-task-immediate-notification-block').removeClass('hidden');
                $('#lead-task-start-time-minutes').prop('disabled', false);
            }
            this.checkMandatory();
        },
        checkMandatory: function () {

            var elementStatus = true;

            if (($('#lead-task-type-detailed-view').val().trim() != '' && $('#lead-task-date-detailed-view').val() != '') || ($('#lead-task-type-detailed-view').val().trim() != '' && $('#lead-task-start-immediate').is(':checked'))) {
                elementStatus = false;
            }

            if (!$('#lead-task-start-immediate').is(':checked') && $('#lead-task-due-date-detailed-view').val() == '') {
                elementStatus = true;
            }

            if ($('#lead-task-start-immediate').is(':checked') && $('#lead-task-due-date-detailed-view').val() == '') {
                elementStatus = true;
            }

            $('#add-lead-task-action').prop('disabled', elementStatus);

        },
        initTiny: function (){
            var websiteUrl = $('#website_url').val(), self = this;
            tinymce.init({
                script_url              : websiteUrl+'system/js/external/tinymce/tinymce.gzip.php',
                selector                : "#task_notes",
                skin                    : 'seotoaster',
                menubar                 : false,
                browser_spellcheck      : true,
                resize                  : false,
                convert_urls            : false,
                relative_urls           : false,
                statusbar               : false,
                allow_script_urls       : false,
                force_p_newlines        : false,
                forced_root_block       : false,
                entity_encoding         : "raw",
                plugins                 :  ["leadshortcode fullscreen code link"],
                toolbar1                : "leadshortcode | fullscreen | code | link unlink ",
                image_advtab            : false,
                extended_valid_elements : false,
                setup                   : function(ed){
                    var keyTime = null;
                    ed.on('change blur keyup', function(ed, e){
                        self.dispatchEditorKeyup(ed, e, keyTime);
                        this.save();
                    });
                }
            });
            $('#task_notes').html('<span></span>').trigger('keyup');
            this.tinimce = tinymce;
        },
        dispatchEditorKeyup: function(editor, event, keyTime) {
            var keyTimer = keyTime;
            if(keyTimer === null) {
                keyTimer = setTimeout(function() {
                    keyTimer = null;
                }, 1000)
            }
        }
    });

    return LeadTaskFormView;
});
