var path = require('path');
var webpack = require('webpack');

const { VueLoaderPlugin } = require('vue-loader');

module.exports = {
    entry: {
        customfields: __dirname + "/custom-fields-config/",
        apiConfig: __dirname + "/modules/api-config/",
        salespersonimapconfig: __dirname + "/sales-person-imap-config/",
        mobileappconfig: __dirname + "/mobile-app-config/",
        importcontactsdata: __dirname + "/import-contacts-data/",
        advertiseraccountconfig: __dirname + "/advertiser-account-config/",
        imaprules: __dirname + "/imap-rules/",
        goalconfig: __dirname + "/goal-config/",
        dashboardwelcomescreen: __dirname + "/dashboard/welcomescreen/",
        opportunitiesscreen: __dirname + "/dashboard/opportunities/",
    },
    output: {
        path: __dirname,
        filename: '[name].js',
        library: '[name]'
    },
    module: {
        rules: [
            {
                test: /\.vue$/,
                loader: 'vue-loader',
                options: {
                    loaders: {}
                    // other vue-loader options go here
                }
            },
            {
                test: /\.js$/,
                exclude: /node_modules/,
                use: {
                    loader: 'babel-loader',
                    options: {
                        presets: ['@babel/preset-env']
                    }
                },
            },
            {
                resourceQuery: /blockType=i18n/,
                loader: '@kazupon/vue-i18n-loader'
            },
            {
                test: /.css$/,
                use: [
                    'style-loader',
                    'css-loader',
                ]
            },
        ]
    },
    watch: process.env.NODE_ENV === 'development',
    watchOptions: {
        poll: true,
        aggregateTimeout: 100
    },
    resolve: {
        alias: {
            'vue$': 'vue/dist/vue.esm.js'
        }
    },
    devServer: {
        historyApiFallback: true,
        noInfo: true
    },
    performance: {
        hints: false
    },
    devtool: '#eval-source-map'
};

module.exports.plugins = [new VueLoaderPlugin()];

if (process.env.NODE_ENV === 'production') {
    module.exports.devtool = '#source-map';
    // http://vue-loader.vuejs.org/en/workflow/production.html
    module.exports.plugins = (module.exports.plugins || []).concat([
        new webpack.DefinePlugin({
            'process.env': {
                NODE_ENV: '"production"'
            }
        }),
        new webpack.LoaderOptionsPlugin({
            minimize: true
        })
    ])
}
