<?php

class Leads extends Tools_Plugins_Abstract
{

    /**
     * Special prefix for unique tranlation
     */
    const TRANSLATION_PREFIX = 'leads_';

    /**
     * Tab name and action for crm plugin
     * key => tab name, value => plugin action
     */
    public static function crmConfigNameActions()
    {
        return array(
            'Import contacts data' => 'crmImportLeadsAndOrganizations',
            'Cloud tracking services' => 'crmLeadsCloudTrackingServices',
            'Opportunity stages' => 'crmConfigOpportunitystage',
            'Opportunity types' => 'crmConfigOpportunityTypes',
            'Lead config' => 'crmConfigLeadGeneral',
            'Goals and Quotas' => 'goalConfig',
            'Lead type' => 'crmConfigLead',
            'Lead source' => 'crmConfigLeadSource',
            'Lead lifecycle stage' => 'crmConfigStage',
            'Lead tags' => 'crmConfigLeadTags',
            'External events' => 'crmConfigLeadEvents',
            'Custom lead fields' => 'crmConfigLeadCustomParams',
            'Lead social services' => 'crmConfigSocialServices',
            'Organization\'s industries' => 'crmConfigIndustry',
            'Custom organization fields' => 'crmConfigOrganizationCustomParams',
            'Leads/Organization custom tabs' => 'crmCustomFieldsConfig',
            'Configuration email IMAP' => 'crmInboundEmailsConfig',
            'Imap inbound lead rules' => 'crmImapRulesConfig',
            'Crm Api access' => 'crmApiConfig'
        );
    }


    /**
     * Secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * Call lead device type mobile
     */
    const LEAD_CALL_DEVICE_TYPE_MOBILE = 'mobile';

    /**
     * Call lead device type desktop
     */
    const LEAD_CALL_DEVICE_TYPE_DESKTOP = 'desktop';

    /**
     * Call lead device voip phone
     */
    const LEAD_CALL_DEVICE_TYPE_VOIP_PHONE = 'voip-phone';

    /**
     * lead social services folder name
     */
    const SOCIAL_SERVICE_IMAGES_FOLDER = 'lead-social-services';

    /**
     * lead social services folder path
     */
    const SOCIAL_SERVICE_IMAGES_PATH = 'plugins/leads/web/images/lead-social-services';

    /**
     * organization logos folder path
     */
    const ORGANIZATION_LOGOS_IMAGES_PATH = 'plugins/leads/web/images/organization-logos';

    /**
     * Plugin cart status external type
     */
    const CART_STATUS_PLUGIN_EXTERNAL_TYPE = 'lead';

    /**
     * Leads type name
     */
    const LEADS_TYPE = 'leads';

    /**
     * Organizations type name
     */
    const ORGANIZATIONS_TYPE = 'organizations';

    /**
     * Import log error
     */
    const IMPORT_LOG_TYPE_ERROR = 'error';

    /**
     * Import log full
     */
    const IMPORT_LOG_TYPE_FULL = 'full';

    /**
     * Processed lead tags limit
     */
    const PROCESSED_LEAD_TAGS_LIMIT = 20;

    /**
     * Processed lead email/sms sequences limit
     */
    const PROCESSED_LEAD_SEQUENCES_LIMIT = 20;

    /**
     * Processed organization queue limit
     */
    const PROCESSED_ORGANIZATION_LIMIT = 5;

    /**
     * @var null|Zend_Layout
     */
    protected $_layout = null;

    /**
     * No cart statuses list
     *
     * @var array
     */
    public static $_noCartStatuses = array('no_cart');

    /**
     * Plugin api actions
     *
     * @return array
     */
    public static function pluginApiActions()
    {
        return array('callLog', 'smsLog', 'piwikUserVisitLog', 'processLead', 'externalEmailServiceLog',
            'addExternalIpLog', 'getLeadInfo', 'unsubscribeSms', 'searchLeadByParams', 'getLeadDetails',
            'addRemoteLeadNotes', 'deleteRemoteLeadNotes', 'deleteMobileAppRecord', 'editLeadInfo', 'addLeadInfo',
            'getGeneralInfo', 'searchOrganization', 'deleteLead', 'getMobileAppVersion', 'emailValidation',  'formsLog',
            'getTemplatesList', 'getImagesList', 'sendMobileEmail', 'getMobileEmailWidgets', 'getLeadInfoByCoordinates',
            'addLeadTag', 'deleteLeadTag', 'getLeadTags'
        );
    }

    /**
     * Administrative roles
     *
     * @var array
     */
    public static $_administrativeRoles = array(
        Tools_Security_Acl::ROLE_SUPERADMIN,
        Tools_Security_Acl::ROLE_ADMIN,
        Shopping::ROLE_SALESPERSON
    );

    /**
     * Unique leads and organizations mobile numbers
     *
     * @var array
     */
    public $uniqueMobilesNumbers = array();

    /**
     * @var array
     */
    private $_websiteConfig;

    /**
     * @param array $options plugin options array
     * @param array $seotoasterData system data array
     * @throws Zend_Exception
     */
    public function  __construct($options, $seotoasterData)
    {
        parent::__construct($options, $seotoasterData);

        $this->_layout = new Zend_Layout();
        $this->_layout->setLayoutPath(Zend_Layout::getMvcInstance()->getLayoutPath());

        if ($viewScriptPath = Zend_Layout::getMvcInstance()->getView()->getScriptPaths()) {
            $this->_view->setScriptPath($viewScriptPath);
        }
        $this->_view->addScriptPath(__DIR__ . '/system/views/');

        $this->_jsonHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('json');
        $this->_websiteConfig = Zend_Registry::get('website');

    }

    public function beforeRouter() {
        if (!isset($this->_sessionHelper->refererLeadsUrl)){
            $refererUrl = $this->_request->getHeader('referer');
            if (!empty($refererUrl)) {
                $params = $this->_request->getParams();
                $this->_sessionHelper->refererLeadsGetParams = $params;
            }
            $this->_sessionHelper->refererLeadsUrl = $refererUrl;
        }

        $router = Zend_Controller_Front::getInstance()->getRouter();

        //add crm login route
        $router->addRoute('crmLogin',
            new Zend_Controller_Router_Route('crmLogin/*', array(
                'controller' => 'backend_plugin',
                'action'     => 'fireaction',
                'page'       => 'index',
                'run'        => 'crmLogin',
                'name'      => 'leads'
            ))
        );

    }

    /**
     *
     * Crm login action
     *
     */
    public function crmLoginAction()
    {
        $params = $this->_request->getParams();
        $loggedUser = $this->_sessionHelper->getCurrentUser();
        $userRole = $loggedUser->getRoleId();

        $originalUrl = $this->_request->getRequestUri();

        $finalUrl = '';

        if (!empty($params['redirectTo']) && $params['redirectTo'] !== 'dashboard') {
            if (!preg_match('~\.html~ui', $params['redirectTo'])) {
                $finalUrl = $params['redirectTo'] . '/';
            } else {
                $finalUrl = $params['redirectTo'];
            }
        } elseif (!empty($params['tabName'])) {
            $finalUrl = $params['tabName'] . '/';

            if (!empty($params['subTab'])) {
                $finalUrl .= $params['subTab'] . '/';
            }

            if (!empty($params['paramHash']) && !empty($params['paramHashName'])) {
                $finalUrl .= '#' . $params['paramHashName'] . '/' . $params['paramHash'];
            }

        }

        $this->_sessionHelper->crmLoginRedirectUrl = $finalUrl;

        if ($userRole !== Tools_Security_Acl::ROLE_GUEST) {
            if (!empty($finalUrl)) {
                $this->_redirector->gotoUrl(Zend_Controller_Action_HelperBroker::getStaticHelper('website')->getUrl() . $finalUrl);
            } else {
                $this->_redirector->gotoUrl(Zend_Controller_Action_HelperBroker::getStaticHelper('website')->getUrl());
            }
        } else {
            $flashMessengerHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('flashMessenger');
            $loginForm = new Application_Form_Login();
            if ($this->_request->isPost()) {
                $secureToken = $this->_request->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
                $tokenValid = Tools_System_Tools::validateToken($secureToken, Tools_System_Tools::ACTION_PREFIX_LOGIN);
                if (!$tokenValid) {
                    $this->_redirector->gotoUrl(Zend_Controller_Action_HelperBroker::getStaticHelper('website')->getUrl());
                }
                $loginForm->removeElement(Tools_System_Tools::CSRF_SECURE_TOKEN);
                if ($loginForm->isValid($this->_request->getParams())) {
                    $authAdapter = new Zend_Auth_Adapter_DbTable(
                        Zend_Registry::get('dbAdapter'),
                        'user',
                        'email',
                        'password',
                        'MD5(?)'
                    );
                    $authAdapter->setIdentity($loginForm->getValue('email'));
                    $authAdapter->setCredential($loginForm->getValue('password'));
                    $authResult = $authAdapter->authenticate();
                    if ($authResult->isValid()) {
                        $authUserData = $authAdapter->getResultRowObject(null, 'password');
                        if (null !== $authUserData) {
                            $user = new Application_Model_Models_User((array)$authUserData);
                            $user->setLastLogin(date(Tools_System_Tools::DATE_MYSQL));
                            $user->setIpaddress($_SERVER['REMOTE_ADDR']);
                            $this->_sessionHelper->setCurrentUser($user);
                            Application_Model_Mappers_UserMapper::getInstance()->save($user);
                            unset($user);
                            Zend_Session::regenerateId();
                            $cacheHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('cache');
                            $cacheHelper->clean();
                            $this->_redirector->gotoUrl(Zend_Controller_Action_HelperBroker::getStaticHelper('website')->getUrl() . $finalUrl);
                        }
                    }
                    $flashMessengerHelper->addMessage(array('email' => $this->_translator->translate(self::TRANSLATION_PREFIX.'There is no user with such login and password.')));
                    $this->_redirector->gotoUrl($originalUrl);
                } else {
                    $flashMessengerHelper->addMessage(array('email' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Login should be a valid email address')));
                    $this->_redirector->gotoUrl($originalUrl);
                }
            } else {
                $languageHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('language');
                $this->_view->languages = $languageHelper->getLanguages();
                $this->_view->currentLanguage = strtolower(Zend_Registry::get('Zend_Locale')->getRegion());
                //getting messages
                $errorMessages = $flashMessengerHelper->getMessages();
                if (!empty($errorMessages)) {
                    foreach ($errorMessages as $message) {
                        foreach ($message as $elementName => $msg) {
                            $loginForm->getElement($elementName)->setAttribs(array(
                                'class' => 'notvalid'
                            ));
                        }
                    }
                }
                $this->_view->messages = $flashMessengerHelper->getMessages();
                //unset url redirect set from any login widget

                $loginForm->removeDecorator('HtmlTag');
                $loginForm->setElementDecorators(array(
                    'ViewHelper',
                    'Errors',
                    'Label',
                    array('HtmlTag', array('tag' => 'p'))
                ));
                $this->_view->loginForm = $loginForm;
                $loginForm->setAction('');

                $this->_layout->content = $this->_view->render('crm-login.phtml');
                echo $this->_layout->render();
            }
        }

    }

    /**
     * Handling quote views
     */
    public function beforeController()
    {
        //self::searchLeadByParams(array());
        //self::getLeadDetails(array());
        //self::editLeadInfo(array());
        //self::addLeadInfo(array());
        //self::getGeneralInfo();
        //self::searchOrganization(array());
        //self::deleteLead(array());
        //self::getMobileAppVersion();
        //self::formsLog(array());
        //self::getTemplatesList(array());
        //self::getImagesList(array());
        //self::sendMobileEmail(array());
        //self::getMobileEmailWidgets();
        //self::getLeadInfoByCoordinates(array());
        //self::addLeadTag(array());
        //self::deleteLeadTag(array());
        //self::getLeadTags(array());
        if (!Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $requestedUri = filter_var(Tools_System_Tools::getRequestUri(), FILTER_SANITIZE_STRING);
            $quoteMapper = Quote_Models_Mapper_QuoteMapper::getInstance();
            $quoteModel = $quoteMapper->find(
                Zend_Controller_Action_HelperBroker::getStaticHelper('page')->clean($requestedUri)
            );
            if ($quoteModel instanceof Quote_Models_Model_Quote) {
                $quoteId = $quoteModel->getId();
                $createdAt = Tools_System_Tools::convertDateFromTimezone('now');
                $leadsQuoteLogMapper = Leads_Mapper_LeadsQuoteLogMapper::getInstance();
                $leadQuoteLogExists = $leadsQuoteLogMapper->findByLastQuoteId($quoteId);
                if (!empty($leadQuoteLogExists)) {
                    $cartModel = Models_Mapper_CartSessionMapper::getInstance()->find($quoteModel->getCartId());
                    if ($cartModel instanceof Models_Model_CartSession) {
                        $cartTotal = $cartModel->getTotal();
                    }
                    $leadId = $leadQuoteLogExists['lead_id'];
                    $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
                    $leadModel = $leadMapper->find($leadId);
                    if ($leadModel instanceof Leads_Model_LeadsModel) {
                        $currentUser = $this->_sessionHelper->getCurrentUser();
                        $roleId = $currentUser->getRoleId();

                        if ($quoteModel->getStatus() == Quote_Models_Model_Quote::STATUS_SENT && ($roleId != Tools_Security_Acl::ROLE_SUPERADMIN || $roleId != Tools_Security_Acl::ROLE_ADMIN || $roleId != Shopping::ROLE_SALESPERSON)) {
                            $leadsActionLog = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_QUOTE_VIEW,
                                $leadId,
                                Leads_Model_LeadsActionLogModel::ACTION_TYPE_INBOUND, $createdAt,
                                Leads_Model_LeadsTypeModel::LEAD_TYPE_INBOUND_QUOTE_VIEW);

                            $leadsQuoteLogMapper = Leads_Mapper_LeadsQuoteLogMapper::getInstance();
                            $leadQuoteLogModel = new Leads_Model_LeadsQuoteLogModel();
                            $leadQuoteLogModel->setStatus($quoteModel->getStatus());
                            $leadQuoteLogModel->setQuoteId($quoteId);
                            $leadQuoteLogModel->setTitle($quoteModel->getTitle());
                            $leadQuoteLogModel->setCreatedAt($createdAt);
                            $leadQuoteLogModel->setActionLogId($leadsActionLog->getId());
                            $leadQuoteLogModel->setQuoteLink($quoteId . '.html');
                            $leadQuoteLogModel->setLeadId($leadId);
                            $leadQuoteLogModel->setCreatorId($leadQuoteLogExists['creator_id']);
                            $leadQuoteLogModel->setQuoteActionType(Leads_Model_LeadsQuoteLogModel::QUOTE_ACTION_TYPE_VIEW);
                            if (!empty($cartTotal)) {
                                $leadQuoteLogModel->setTotal($cartTotal);
                            }
                            $leadsQuoteLogMapper->save($leadQuoteLogModel);

                            $leadModel->setLastActivity($createdAt);
                            $leadTypeModel = Leads_Mapper_LeadsTypeMapper::getInstance()->findByTypeName(Leads_Model_LeadsTypeModel::LEAD_TYPE_INBOUND_QUOTE_VIEW);
                            if ($leadTypeModel instanceof Leads_Model_LeadsTypeModel) {
                                $leadModel->setLastActivityTypeId($leadTypeModel->getId());
                                $leadModel->setLastActivitySubtype('');
                            }
                            $leadMapper->save($leadModel);
                        }
                    }
                }

            }
        }

//        $actionDetailsArray = array(
//            '0' => array(
//                'type' => 'action',
//                'url' => 'http://www.centrosmedicossansebastian.com/',
//                'pageTitle' => 'Centro Médico en Donostia - San Sebastián | Centro Médico San Sebastián',
//                'pageIdAction' => '947847',
//                'serverTimePretty' => '4 Дек. 2016 г. 19:45:50',
//                'generationTime' => '0,91 с',
//                'icon' => null,
//                'timeSpent' => '27',
//                'timestamp' => '1480880750'
//            )
//        );
//
//        $generalPiwikDataArray = array(
//            '0' => array(
//                'visitIp' => '176.83.1.245',
//                'visitorId' => 'c245ddeb9f261e45',
//                'idVisit' => '18756293',
//                'idSite' => '1406',
//                'goalConversions' => '0',
//                'siteCurrency' => 'USD',
//                'siteCurrencySymbol' => '$',
//                'serverDate' => '2016-12-04',
//                'visitServerHour' => '18',
//                'lastActionTimestamp' => '1480877150',
//                'lastActionDateTime' => '2016-12-04 18:45:50',
//                'userId' => null,
//                'visitorType' => 'new',
//                'visitorTypeIcon' => null,
//                'visitConverted' => 0,
//                'visitConvertedIcon' => null,
//                'visitCount' => '1',
//                'firstActionTimestamp' => '1480877150',
//                'visitEcommerceStatus' => 'none',
//                'visitEcommerceStatusIcon' => null,
//                'daysSinceFirstVisit' => 0,
//                'daysSinceLastEcommerceOrder' => 0,
//                'visitDuration' => 10,
//                'visitDurationPretty' => '0 с',
//                'searches' => '0',
//                'actions' => '1',
//                'referrerType' => 'search',
//                'referrerTypeName' => 'Поисковые движки',
//                'referrerName' => 'Google',
//                'referrerKeyword' => 'Ключевое слово - не определено',
//                'referrerKeywordPosition' => null,
//                'referrerUrl' => 'http://piwik.org/faq/general/#faq_144',
//                'referrerSearchEngineUrl' => 'http://google.com',
//                'referrerSearchEngineIcon' => 'plugins/Referrers/images/searchEngines/google.com.png',
//                'languageCode' => 'es-es',
//                'language' => 'Код языка es-es',
//                'deviceType' => 'Смартфон',
//                'deviceTypeIcon' => 'plugins/DevicesDetection/images/screens/smartphone.png',
//                'deviceBrand' => 'Apple',
//                'deviceModel' => 'iPhone',
//                'operatingSystem' => 'iOS 10.1',
//                'operatingSystemName' => 'iOS',
//                'operatingSystemIcon' => 'plugins/DevicesDetection/images/os/IOS.gif',
//                'operatingSystemCode' => 'IOS',
//                'operatingSystemVersion' => '10.1',
//                'browserFamily' => 'WebKit',
//                'browserFamilyDescription' => 'WebKit (Safari, Chrome)',
//                'browser' => 'Mobile Safari 10.0',
//                'browserName' => 'Mobile Safari',
//                'browserIcon' => 'plugins/DevicesDetection/images/browsers/MF.gif',
//                'browserCode' => 'MF',
//                'browserVersion' => '10.0',
//                'events' => '0',
//                'continent' => 'Европа',
//                'continentCode' => 'eur',
//                'country' => 'Испания',
//                'countryCode' => 'es',
//                'countryFlag' => 'plugins/UserCountry/images/flags/es.png',
//                'region' => 'Andalucia',
//                'regionCode' => '51',
//                'city' => 'Cordoba',
//                'location' => 'Cordoba, Andalucia, Испания',
//                'latitude' => '37.883331',
//                'longitude' => '-4.766670',
//                'visitLocalTime' => '19:45:49',
//                'visitLocalHour' => '19',
//                'daysSinceLastVisit' => '0',
//                'customVariables' => '[]',
//                'resolution' => '750x1334',
//                'plugins' => '',
//                'pluginsIcons' => null,
//                'serverTimestamp' => '1480877150',
//                'serverTimePretty' => '19:45:50',
//                'serverDatePretty' => 'Воскресенье, 4 Декабря 2016 г.',
//                'serverDatePrettyFirstAction' => 'Воскресенье, 4 Декабря 2016 г.',
//                'serverTimePrettyFirstAction' => '19:45:50',
//                'actionDetails' => $actionDetailsArray
//            ),
//            '1' => array()
//        );

//        self::piwikUserVisitLog(array('data' => array('piwikData' => $generalPiwikDataArray, 'websiteId' => 2)));
//
//        $smsData = array('smsParams' => array(
//            'smsMessageSid' => 'SM4a864a08cec7a76eb78b49bd0501982d',
//            'smsStatus' => 'received',
//            'from' => '+18448131547',
//            'toState' => 'NJ',
//            'accountSid' => 'AC8fe044350418febc885381cdcfc75805',
//            'toZip' => '07662',
//            'fromCountry' => 'US',
//            'toCity' => 'ROCHELLE PARK',
//            'to' => '+12015828357',
//            'body' => 'Hey Vlad this is inbound sms Sent via SendaText',
//            'toCountry' => 'US',
//            'fromState' => '',
//            'timestamp' => ''
//         ));

        //self::smsLog($smsData);
        //$this->_addTaskReminderQueueEmail();
        $emailSequenceEnabled = Tools_LeadTools::verifyPluginEnabled(Tools_LeadTools::SEQUENCE_PLUGIN);
        if ($emailSequenceEnabled) {
            $this->_addEmailSequenceQueueEmail();
        }
    }

    protected function _addTaskReminderQueueEmail(){
        $this->_injectContent($this->_view->render('task-reminder-queue.phtml'));
    }


    protected function _addEmailSequenceQueueEmail(){
        $this->_injectContent($this->_view->render('email-sequence-queue.phtml'));
    }

    /**
     * @param array $visitLogData
     */
    public static function piwikUserVisitLog($visitLogData)
    {

        if (!empty($visitLogData['data']['websiteId']) && !empty($visitLogData['data']['piwikData'])) {
            Tools_LeadPiwikTools::processPiwikDataLog($visitLogData['data']['websiteId'], $visitLogData['data']['piwikData']);
        }

    }

    /**
     * @param array $formData
     * 'email' => 'andrei@seosamba.com',
     * 'formUrl' => 'http://toast.local.com/index.html',
     * 'form-referer' => 'http://toast.local.com/go',
     * 'ip' => '127.0.0.1',
     * 'createdAt' => '12-3-2021'
     *
     */
    public static function formsLog($formData)
    {

//        $formData = array();
//        $formData['data']['formParams'] = array(
//            'email' => 'andrei3456@seosamba.com',
//            'formUrl' => 'http://toast.local.com/index.html',
//            'form-referer' => 'http://toast.local.com/go',
//            'createdAt' => '2021-04-02 10:38:21'
//        );
//
//        $formData['data']['formFilesData'][] = array(
//            'fileExtension' => 'pdf',
//            'fileName' => 'Chrome M89 Enterprise Release Notes (updated)',
//            'file' => 'https://mojo-doc.s3.amazonaws.com/client_1293/1293_1617359905-Chrome_M89_Enterprise_Release_Notes__updated_.pdf'
//        );

        if (!empty($formData['data']['formParams'])) {
            Tools_LeadFormLogTools::processFormData($formData['data']['formParams'], $formData['data']['formFilesData']);
        }
    }

    /**
     * @param array $unsubscribeData unsubscribe data Ex: caller
     */
    public static function unsubscribeSms($unsubscribeData)
    {

        if (!empty($unsubscribeData) && $unsubscribeData['caller']) {
            $cartStatusSubscribeSmsMapper = Emailsequence_Models_Mapper_SubscribeSmsMapper::getInstance();
            $subscribeSmsModel = $cartStatusSubscribeSmsMapper->findByMobile($unsubscribeData['caller']);
            if ($subscribeSmsModel instanceof Emailsequence_Models_Models_SubscribeSmsModel) {
                $emailSequenceQueueMapper = Emailsequence_Models_Mapper_EmailsequenceQueueMapper::getInstance();
                $subscribeSmsModel->setStatus(Emailsequence_Models_Models_SubscribeSmsModel::STATUS_UNSUBSCRIBED);
                $subscribeSmsModel->setUnsubscribedAt($unsubscribeData['timestamp']);
                $cartStatusSubscribeSmsMapper->save($subscribeSmsModel);
                $emailSequenceQueueMapper->deleteByPhoneNumber($unsubscribeData['caller']);
            }

        }
    }


    /**
     * Response from api plugin
     *
     * ApiVersion	"2010-04-01"
     * SmsSid	"SM4a864a08cec7a76eb78b49bd0501982d"
     * SmsStatus	"received"
     * SmsMessageSid	"SM4a864a08cec7a76eb78b49bd0501982d"
     * NumSegments	"1"
     * From	"+18448131547"
     * ToState	"NJ"
     * MessageSid "SM4a864a08cec7a76eb78b49bd0501982d"
     * AccountSid	"AC8fe044350418febc885381cdcfc75805"
     * ToZip	"07662"
     * FromCountry	"US"
     * ToCity	"ROCHELLE PARK"
     * FromCity	""
     * To	"+12015828357"
     * FromZip	""
     * Body	"Hey Vlad this is inbound sms Sent via SendaText"
     * ToCountry	"US"
     * FromState	""
     * NumMedia	"0"
     *
     * @param $smsData
     * @return string
     */
    public static function smsLog($smsData)
    {
        $leadModel = null;
        $assetName = null;
        $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
        $isVoip = '0';

        $createdAt = Tools_System_Tools::convertDateFromTimezone('now');

        $leadType = Leads_Model_LeadsTypeModel::LEAD_TYPE_INBOUND_SMS;
        $type = Leads_Model_LeadsActionLogModel::ACTION_TYPE_INBOUND;
        $reverseData = false;
        if (!empty($smsData['smsParams']['type']) && $smsData['smsParams']['type'] === Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND) {
            $type = Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND;
            $leadType = Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_SMS;
            $reverseData = true;
        }

        $leadTypeId = Tools_LeadTools::getLeadTypeIdByLeadTypeName($leadType);
        $smsData['caller_name'] = 'Not provided';
        if (!empty($smsData['smsParams']['caller_name'])) {
            if (trim($smsData['smsParams']['caller_name']) !== 'Not provided') {
                $smsData['caller_name'] = $smsData['smsParams']['caller_name'];
                $callerNameArray = explode(' ', $smsData['caller_name']);
                if (!empty($callerNameArray)) {
                    $firstName = $callerNameArray['0'];
                    if (!empty($callerNameArray['1'])) {
                        $lastName = $callerNameArray['1'];
                    }
                }
            }
        }
        $creatorId = null;
        if ($smsData['smsParams']['smsStatus'] === 'received') {
            $smsData['smsParams']['smsStatus'] = 'completed';
        }

        if (!empty($smsData['smsParams']['utmTagList'])) {
            $leadUtmTags = Tools_LeadTools::prepareUtmTagsString($smsData['smsParams']['utmTagList']);
            if (!empty($leadUtmTags)) {
                $smsData['lead_source'] = $leadUtmTags;
            }
        }

        $leadModel = Tools_LeadTools::findLeadByPhoneMobileNumber($smsData['smsParams']['from']);

        if (!$leadModel instanceof Leads_Model_LeadsModel && !empty($smsData['smsParams']['from'])) {
            $leadAdditionAvailable = Tools_LeadLicenseTools::checkLimit();
            if ($leadAdditionAvailable['error'] === true) {
                return '';
            }

            $leadModel = new Leads_Model_LeadsModel();
            $leadName = $smsData['caller_name'];
            $countryCode = $smsData['smsParams']['fromCountry'];
            $city = $smsData['smsParams']['fromCity'];
            $zip = $smsData['smsParams']['fromZip'];
            $state = $smsData['smsParams']['fromState'];
            $ownerId = Tools_LeadTools::getDefaultLeadOwner();

            $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
            $userDefaultPhoneCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');
            $countryPhonesList = Tools_LeadTools::getCountryPhoneCodesList(false);
            if (array_key_exists($countryCode, $countryPhonesList)) {
                $countryCodeValue = $countryPhonesList[$countryCode];
                $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();
                if (!empty($countryCode) && empty($stateCode)) {
                    $defaultStateId = $shoppingConfig['state'];
                    $allStates = Tools_LeadTools::getStates($countryCode, true, false, 'state');
                    if (array_key_exists($defaultStateId, $allStates)) {
                        $stateCode = $allStates[$defaultStateId];
                    }
                }
                $leadModel->setLeadMobile(str_replace($countryCodeValue, '', $smsData['smsParams']['from']));
                $leadModel->setLeadMobileCountryCode($countryCode);
                $leadModel->setLeadMobileCountryCodeValue($countryCodeValue);
                $leadModel->setLeadCountryCode($countryCode);
                if (!empty($stateCode)) {
                    $leadModel->setLeadStateCode($stateCode);
                }

                if (!empty($firstName)) {
                    $leadModel->setLeadFirstName($firstName);
                } else {
                    $leadModel->setLeadFirstName($leadName);
                }

                if (!empty($lastName)) {
                    $leadModel->setLeadLastName($lastName);
                }

                $leadModel->setLeadCity($city);
                $leadModel->setLeadZip($zip);
                $leadModel->setLeadTypeId($leadTypeId);
                $leadModel->setCreatedAt($createdAt);
                $leadModel->setOwnerId($ownerId);
                if (!empty($state)) {
                    $leadModel->setLeadStateCode($state);
                }
                if (!empty($smsData['lead_source'])) {
                    $leadModel->setLeadSource($smsData['lead_source']);
                }

                if (!empty($userDefaultPhoneCountryCode)) {
                    $leadModel->setLeadPhoneCountryCode($userDefaultPhoneCountryCode);
                }

                $leadModel->setLastActivityTypeId($leadTypeId);
                $leadModel->setLastActivitySubtype('');
                if($type == Leads_Model_LeadsActionLogModel::ACTION_TYPE_INBOUND){
                    $leadModel->setLastActivity($createdAt);
                }else{
                    $leadModel->setLastActivityOutbound($createdAt);
                }

                try {
                    $leadModel = $leadMapper->save($leadModel);
                } catch (Exception $e) {
                    return false;
                }
            }
        } elseif($leadModel instanceof Leads_Model_LeadsModel && !empty($smsData['smsParams']['from'])) {
            $currentFirstName = $leadModel->getLeadFirstName();
            $currentLastName = $leadModel->getLeadLastName();
            $currentZip = $leadModel->getLeadZip();

            $leadName = $smsData['caller_name'];
            $countryCode = $smsData['smsParams']['fromCountry'];
            $city = $smsData['smsParams']['fromCity'];
            $zip = $smsData['smsParams']['fromZip'];
            $state = $smsData['smsParams']['fromState'];
            $currentLeadSource = $leadModel->getLeadSource();

            if (empty($currentFirstName)) {
                if (!empty($firstName)) {
                    $leadModel->setLeadFirstName($firstName);
                } else {
                    $leadModel->setLeadFirstName($leadName);
                }
            }

            if (empty($currentLastName)) {
                if (!empty($lastName)) {
                    $leadModel->setLeadLastName($lastName);
                }
            }

            if (empty($currentZip)) {
                $leadModel->setLeadZip($zip);
            }

            if (empty($currentLeadSource) && !empty($smsData['lead_source'])) {
                $leadModel->setLeadSource($smsData['lead_source']);
            }

            $leadModel->setLastActivityTypeId($leadTypeId);
            $leadModel->setLastActivitySubtype('');
            if($type == Leads_Model_LeadsActionLogModel::ACTION_TYPE_INBOUND){
                $leadModel->setLastActivity($createdAt);
            }else{
                $leadModel->setLastActivityOutbound($createdAt);
            }

            $leadModel = $leadMapper->save($leadModel);

        }

        if ($leadModel instanceof Leads_Model_LeadsModel) {
            if (!empty($leadTypeId)) {
                $leadModel->setLastActivityTypeId($leadTypeId);
                $leadModel->setLastActivitySubtype('');
                if($type == Leads_Model_LeadsActionLogModel::ACTION_TYPE_INBOUND){
                    $leadModel->setLastActivity($createdAt);
                }else{
                    $leadModel->setLastActivityOutbound($createdAt);
                }
                $leadMapper->save($leadModel);
            }

            $leadId = $leadModel->getId();
            $actionLedLogModel = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_SMS,
                $leadId, $type, $createdAt, $leadType);
            $callLogMapper = Leads_Mapper_LeadsCallLogMapper::getInstance();
            $callLogModel = new Leads_Model_LeadsCallLogModel();
            $callLogModel->setActionLogId($actionLedLogModel->getId());
            $callLogModel->setType(Leads_Model_LeadsCallLogModel::TYPE_SMS);
            $callLogModel->setCallSid($smsData['smsParams']['smsMessageSid']);
            $callLogModel->setCallDuration(0);
            $callLogModel->setAccountSid($smsData['smsParams']['accountSid']);
            if ($reverseData === true) {
                $callLogModel->setCaller($smsData['smsParams']['to']);
            } else {
                $callLogModel->setCaller($smsData['smsParams']['from']);
            }
            $callLogModel->setTimestamp($smsData['smsParams']['timestamp']);
            $callLogModel->setStatus($smsData['smsParams']['smsStatus']);
            $callLogModel->setScore('valid');
            $callLogModel->setLink('');
            $callLogModel->setCreatorId($creatorId);
            $callLogModel->setDetails($smsData['smsParams']['body']);
            $callLogModel->setLeadId($leadId);
            $callLogModel->setCallSource($assetName);
            if (!empty($smsData['smsParams']['subType']) && $smsData['smsParams']['subType'] === Leads_Model_LeadsCallLogModel::LEAD_CALL_SMS_SUBTYPE_SMS_WIDGET) {
                $callLogModel->setSubType($smsData['smsParams']['subType']);
            } else {
                $callLogModel->setSubType(null);
            }
            if ($reverseData === true) {
                $callLogModel->setReceiver($smsData['smsParams']['from']);
            } else {
                $callLogModel->setReceiver($smsData['smsParams']['to']);
            }
            $callLogModel->setIsVoip($isVoip);
            $callLogMapper->save($callLogModel);

            if (!empty($smsData['smsParams']['ipAddress'])) {
                $leadsIpsModel = Tools_LeadTools::addIpAddressToLead($leadId, $smsData['smsParams']['ipAddress']);
                if ($leadsIpsModel instanceof Leads_Model_LeadsIpsModel) {
                    Tools_LeadTools::addIpToMojo($leadsIpsModel);
                }
            }
        }
    }


    /**
     * Response from api plugin
     *
     * @param $callData
     * @return string
     */
    public static function callLog($callData)
    {
        $leadModel = null;
        $assetName = null;
        $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
        $isVoip = '0';
        if (!empty($callData['customData']['leadId'])) {
            $typeDirectCall = false;
            $leadId = $callData['customData']['leadId'];
            if (!empty($callData['customData']['isVoipPhone'])) {
                $isVoip = '1';
            }
            $leadModel = $leadMapper->find($leadId);
            $type = Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND;
            $leadType = Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_CALL;
            $creatorId = $callData['customData']['creatorId'];
        } elseif (!empty($callData['caller']) && $callData['callType'] === 'directCall') {
            $leadModel = Tools_LeadTools::findLeadByPhoneMobileNumber($callData['caller']);
            $type = Leads_Model_LeadsActionLogModel::ACTION_TYPE_INBOUND;
            $leadType = Leads_Model_LeadsTypeModel::LEAD_TYPE_INBOUND_CALL;
            $creatorId = null;
            if (!empty($callData['asset_name'])) {
                $assetName = $callData['asset_name'];
            }
            $typeDirectCall = true;
        } else {
            return '';
        }

        $createdAt = Tools_System_Tools::convertDateFromTimezone('now');

        if (empty($callData['call_duration'])) {
            $callDuration = null;
        } else {
            $callDuration = $callData['call_duration'];
        }

        if (!empty($callData['link'])) {
            $link = $callData['link'];
        } else {
            $link = '';
        }

        $leadTypeId = Tools_LeadTools::getLeadTypeIdByLeadTypeName($leadType);

        if (empty($callData['caller_name'])) {
            $callData['caller_name'] = 'Not provided';
        }

        if (!$leadModel instanceof Leads_Model_LeadsModel && $typeDirectCall === true && !empty($callData['caller_name'])) {

            $leadAdditionAvailable = Tools_LeadLicenseTools::checkLimit();
            if ($leadAdditionAvailable['error'] === true) {
                return '';
            }

            $leadModel = new Leads_Model_LeadsModel();
            $leadName = $callData['caller_name'];
            $countryCode = $callData['twilioParams']['FromCountry'];
            $city = $callData['twilioParams']['FromCity'];
            $zip = $callData['twilioParams']['FromZip'];
            $state = $callData['twilioParams']['FromState'];
            $superAdminUserModel = $leadMapper->findSuperAdminUser();
            $ownerId = 0;
            if ($superAdminUserModel instanceof Application_Model_Models_User) {
                $ownerId = $superAdminUserModel->getId();
            }

            $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
            $userDefaultPhoneCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');
            $countryPhonesList = Tools_LeadTools::getCountryPhoneCodesList(false);
            if (array_key_exists($countryCode, $countryPhonesList)) {
                $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();
                $countryCodeValue = $countryPhonesList[$countryCode];
                if (!empty($countryCode) && empty($stateCode)) {
                    $defaultStateId = $shoppingConfig['state'];
                    $allStates = Tools_LeadTools::getStates($countryCode, true, false, 'state');
                    if (array_key_exists($defaultStateId, $allStates)) {
                        $stateCode = $allStates[$defaultStateId];
                    }
                }

                $leadModel->setLeadMobile(str_replace($countryCodeValue, '', $callData['caller']));
                $leadModel->setLeadMobileCountryCode($countryCode);
                $leadModel->setLeadMobileCountryCodeValue($countryCodeValue);
                $leadModel->setLeadCountryCode($countryCode);
                if (!empty($stateCode)) {
                    $leadModel->setLeadStateCode($stateCode);
                }
                $leadModel->setLeadFirstName($leadName);
                $leadModel->setLeadCity($city);
                $leadModel->setLeadZip($zip);
                $leadModel->setLeadTypeId($leadTypeId);
                $leadModel->setCreatedAt($createdAt);
                $leadModel->setOwnerId($ownerId);
                if (!empty($state)) {
                    $leadModel->setLeadStateCode($state);
                }
                if (!empty($assetName)) {
                    $leadModel->setLeadSource($assetName);
                }

                if (!empty($userDefaultPhoneCountryCode)) {
                    $leadModel->setLeadPhoneCountryCode($userDefaultPhoneCountryCode);
                }

                $leadModel->setLastActivityTypeId($leadTypeId);
                $leadModel->setLastActivitySubtype('');
                if($type == Leads_Model_LeadsActionLogModel::ACTION_TYPE_INBOUND){
                    $leadModel->setLastActivity($createdAt);
                }else{
                    $leadModel->setLastActivityOutbound($createdAt);
                }

                try {
                    $leadMapper->save($leadModel);
                } catch (Exception $e) {
                    return false;
                }
            }
        } elseif($leadModel instanceof Leads_Model_LeadsModel && $typeDirectCall === true && !empty($callData['caller_name'])) {
            $currentFirstName = $leadModel->getLeadFirstName();
            $currentZip = $leadModel->getLeadZip();

            $leadName = $callData['caller_name'];
            $countryCode = $callData['twilioParams']['FromCountry'];
            $city = $callData['twilioParams']['FromCity'];
            $zip = $callData['twilioParams']['FromZip'];
            $state = $callData['twilioParams']['FromState'];

            if (empty($currentFirstName)) {
                $leadModel->setLeadFirstName($leadName);
            }

            if (empty($currentZip)) {
                $leadModel->setLeadZip($zip);
            }

            $leadModel->setLastActivityTypeId($leadTypeId);
            $leadModel->setLastActivitySubtype('');
            if($type == Leads_Model_LeadsActionLogModel::ACTION_TYPE_INBOUND){
                $leadModel->setLastActivity($createdAt);
            }else{
                $leadModel->setLastActivityOutbound($createdAt);
            }

            $leadMapper->save($leadModel);

        }

        if ($leadModel instanceof Leads_Model_LeadsModel) {
            if (!empty($leadTypeId)) {
                $leadModel->setLastActivityTypeId($leadTypeId);
                $leadModel->setLastActivitySubtype('');
                if($type == Leads_Model_LeadsActionLogModel::ACTION_TYPE_INBOUND){
                    $leadModel->setLastActivity($createdAt);
                }else{
                    $leadModel->setLastActivityOutbound($createdAt);
                }
                $leadMapper->save($leadModel);
            }

            $leadId = $leadModel->getId();
            $actionLedLogModel = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_CALL,
                $leadId, $type, $createdAt, $leadType);
            $callLogMapper = Leads_Mapper_LeadsCallLogMapper::getInstance();
            $callLogModel = new Leads_Model_LeadsCallLogModel();
            $callLogModel->setActionLogId($actionLedLogModel->getId());
            $callLogModel->setType(Leads_Model_LeadsCallLogModel::TYPE_CALL);
            $callLogModel->setCallSid($callData['call']);
            $callLogModel->setCallDuration($callDuration);
            $callLogModel->setAccountSid($callData['account_sid']);
            $callLogModel->setCaller($callData['caller']);
            $callLogModel->setTimestamp($callData['timestamp']);
            $callLogModel->setStatus($callData['status']);
            $callLogModel->setScore($callData['score']);
            $callLogModel->setLink($link);
            $callLogModel->setCreatorId($creatorId);
            $callLogModel->setDetails('');
            $callLogModel->setLeadId($leadId);
            $callLogModel->setCallSource($assetName);
            $callLogModel->setReceiver($callData['receiver']);
            $callLogModel->setIsVoip($isVoip);
            $callLogMapper->save($callLogModel);
        }
    }

//    /**
//     * Put in task reminder queue
//     *
//     * @throws Zend_Controller_Action_Exception
//     */
//    public function taskReminderQueueAction()
//    {
//        if ($this->_request->isPost()) {
//            $cacheHelper = Zend_Controller_Action_HelperBroker::getExistingHelper('cache');
//            $leadTaskQueueMapper = Leads_Mapper_LeadsTaskQueueMapper::getInstance();
//            if (null === ($leadTaskStatusConfigData = $cacheHelper->load('lead_task_queue_data', 'leads'))) {
//                $leadTaskStatusConfigData = Leads_Mapper_LeadsConfigMapper::getInstance()->getConfigParams();
//                $leadTaskMapper = Leads_Mapper_LeadsTaskMapper::getInstance();
//                if (!empty($leadTaskStatusConfigData['first_task_reminder'])) {
//                   $taskReminderHours = intval($leadTaskStatusConfigData['first_task_reminder']);
//                   $remindersListData = $leadTaskMapper->getTasksFromLogInDate($taskReminderHours);
//                   if (!empty($remindersListData)) {
//                       foreach ($remindersListData as $reminderData) {
//                           $taskId = $reminderData['id'];
//                           $taskReminderInQueueModel = $leadTaskQueueMapper->findByTaskIdQueueType($taskId, Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_TYPE_FIRST_REMINDER);
//                           if (!$taskReminderInQueueModel instanceof Leads_Model_LeadsTaskQueueModel) {
//                               $taskReminderInQueueModel = new Leads_Model_LeadsTaskQueueModel();
//                               $taskReminderInQueueModel->setStatus('processing');
//                               $taskReminderInQueueModel->setQueueType(Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_TYPE_FIRST_REMINDER);
//                               $taskReminderInQueueModel->setTaskId($taskId);
//                               $taskReminderInQueueModel->setToSendDate(date("Y-m-d H:i:s", strtotime($reminderData['taskExpired'].'- '. $taskReminderHours. 'hour')));
//                               $leadTaskQueueMapper->save($taskReminderInQueueModel);
//                           }
//                       }
//                   }
//                }
//
//                if (!empty($leadTaskStatusConfigData['second_task_reminder'])) {
//                    $taskReminderHours = intval($leadTaskStatusConfigData['second_task_reminder']);
//                    $remindersListData = $leadTaskMapper->getTasksFromLogInDate($taskReminderHours);
//                    foreach ($remindersListData as $reminderData) {
//                        $taskId = $reminderData['id'];
//                        $taskReminderInQueueModel = $leadTaskQueueMapper->findByTaskIdQueueType($taskId, Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_TYPE_SECOND_REMINDER);
//                        if (!$taskReminderInQueueModel instanceof Leads_Model_LeadsTaskQueueModel) {
//                            $taskReminderInQueueModel = new Leads_Model_LeadsTaskQueueModel();
//                            $taskReminderInQueueModel->setStatus('processing');
//                            $taskReminderInQueueModel->setQueueType(Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_TYPE_SECOND_REMINDER);
//                            $taskReminderInQueueModel->setTaskId($taskId);
//                            $taskReminderInQueueModel->setToSendDate(date("Y-m-d H:i:s", strtotime($reminderData['taskExpired'].'- '. $taskReminderHours. 'hour')));
//                            $leadTaskQueueMapper->save($taskReminderInQueueModel);
//                        }
//                    }
//                }
//
//                $cacheHelper->save('lead_task_queue_data', $leadTaskStatusConfigData, 'leads',
//                    Helpers_Action_Cache::CACHE_FLASH);
//                $this->_responseHelper->fail('');
//            } elseif (!Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_CONTENT)){
//                $limit = 3;
//                $remindersInQueue =  $leadTaskQueueMapper->getAllQueue($limit);
//                if (!empty($remindersInQueue)) {
//                    foreach ($remindersInQueue as $queueId => $reminder) {
//                        $leadTaskModel = Leads_Mapper_LeadsTaskMapper::getInstance()->find($reminder['task_id']);
//                        if ($leadTaskModel instanceof Leads_Model_LeadsTaskModel) {
//                            $leadId = $leadTaskModel->getLeadId();
//                            $leadModel = Leads_Mapper_LeadsMapper::getInstance()->find($leadId);
//                            $organizationData = array();
//                            $leadsData = array();
//                            if ($leadModel instanceof Leads_Model_LeadsModel) {
//                                $leadsData = $leadModel->toArray();
//                                if (!empty($leadsData['organizationId'])) {
//                                    $organizationModel = Leads_Mapper_LeadsOrganizationMapper::getInstance()->find($leadsData['organizationId']);
//                                    if ($organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
//                                        $organizationData = $organizationModel->toArray();
//                                    }
//                                }
//                            }
//                            $leadTaskModel->registerObserver(new Tools_Mail_Watchdog(array(
//                                'trigger' => Tools_LeadsMailWatchdog::LEADS_TASK_REMINDER,
//                                'queueId' => $queueId,
//                                'leadData' => $leadsData,
//                                'organizationData' => $organizationData
//                            )));
//
//                            $leadTaskModel->notifyObservers();
//                        }
//                    }
//
//                    $this->_responseHelper->success('');
//                }
//            }
//        }
//        $this->_responseHelper->fail('');
//    }

    /**
     * Assign lead owner for leads from dashboard
     */
    public function assignLeadOwnerAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
            if (empty($restrictedUserId)) {
                $this->_validateToken();
                $leadIds = filter_var($this->_request->getParam('leadIds'), FILTER_SANITIZE_STRING);
                $ownerId = filter_var($this->_request->getParam('leadOwnerId'), FILTER_SANITIZE_NUMBER_INT);
                $filters = $this->_request->getParam('filters');
                $allOwnerFilterLeads = $this->_request->getParam('allOwnerFilterLeads');

                if (!empty($leadIds) && !empty($ownerId)) {
                    $leadIds = array_filter(explode(',', $leadIds));
                    if (empty($leadIds)) {
                        $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Lead ids missing'));
                    }

                    $ownerUserModel = Application_Model_Mappers_UserMapper::getInstance()->find($ownerId);
                    if (!$ownerUserModel instanceof Application_Model_Models_User) {
                        $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Owner doesn\'t exists'));
                    }

                    if(!empty($allOwnerFilterLeads)) {
                        $leadsData = Tools_LeadTools::getSearchFilterData($filters, null);

                        if(!empty($leadsData)){
                            $leadsIds = array();
                            foreach ($leadsData['data'] as $lead){
                                array_push($leadsIds, $lead['id']);
                            }
                            $leadIds = $leadsIds;
                        }
                    }

                    Leads_Mapper_LeadsMapper::getInstance()->changeLeadOwner($leadIds, $ownerId);

                    $this->_responseHelper->success($this->_translator->translate(self::TRANSLATION_PREFIX.'Lead owner has been assigned'));
                }
            }
        }

        $this->_responseHelper->fail('');
    }

    /**
     * Process sending SMS for lead
     *
     */
    public function sendSmsAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);

            if (empty($restrictedUserId)) {
                $this->_validateToken();
                $leadIds = filter_var($this->_request->getParam('leadIds'), FILTER_SANITIZE_STRING);
                $smsMessage = strip_tags($this->_request->getParam('smsMessage'));
                $filters = $this->_request->getParam('filters');
                $matchingFilter = (bool) filter_var($this->_request->getParam('matchingFilter'), FILTER_SANITIZE_STRING);
                $filterQuantity = filter_var($this->_request->getParam('filterQuantity'), FILTER_SANITIZE_NUMBER_INT);
                $stepSendSms = filter_var($this->_request->getParam('stepSendSms'), FILTER_SANITIZE_NUMBER_INT);
                $uniqueMobilesNumbersData = $this->_request->getParam('uniqueMobilesNumbers');
                $limit = '5';
                $offset = null;
                if (!empty($stepSendSms)) {
                    $offset = $stepSendSms*$limit;
                    $finalLimit = ($stepSendSms+1)*$limit;
                    if ($filterQuantity < $finalLimit && empty($filterQuantity % $limit)) {
                        $this->_responseHelper->fail(array('quantity' => 0, 'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'All SMS messages have been sent')));
                    }
                }

                $userId = $this->_sessionHelper->getCurrentUser()->getId();
                $testSmsFlag = $this->_request->getParam('testSms');

                if(!empty($testSmsFlag)) {
                    $storeAdminUserModel = Application_Model_Mappers_UserMapper::getInstance()->find($userId);
                    if (!$storeAdminUserModel instanceof Application_Model_Models_User) {
                        $this->_responseHelper->fail(array('message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'user not authorized')));
                    }

                    $userMobileCode = $storeAdminUserModel->getMobileCountryCodeValue();
                    $userMobile = $storeAdminUserModel->getMobilePhone();

                    if(empty($userMobileCode) && empty($userMobile)) {
                        $this->_responseHelper->fail(array('message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Can\'t send test SMS. Wrong user mobile number')));
                    }

                    $leadOrganizationMobile = Tools_LeadTools::glueUserPhone($userMobileCode, $userMobile);

                    if(empty($leadOrganizationMobile)) {
                        $this->_responseHelper->fail(array('message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Can\'t send test SMS. Wrong user mobile number')));
                    }

                    $smsMessageTest = Tools_EmailSequenceTools::addDictionarySmsFields($smsMessage, null, $this->_websiteUrl, true);

                    $subscriber['subscriber']['admin'] = array(
                        'phone' => array($leadOrganizationMobile),
                        'message' => $smsMessageTest,
                        'owner_type' => Apps::SMS_OWNER_TYPE_USER,
                        'sms_from_type' => 'crm',
                        'custom_params' => array()
                    );

                    Apps::apiCall('POST', 'apps', array('twilioSms'), $subscriber);

                    $this->_responseHelper->fail(array('message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Test SMS has been sent')));
                }

                if (!empty($matchingFilter)) {
                    $leadsData = Tools_LeadTools::getSearchFilterData($filters, $limit, $offset, true, true, false, true);
                    if (!empty($leadsData)) {
                        $leadsData = $leadsData['data'];
                    }
                } else {
                    $leadIds = array_filter(explode(',', $leadIds));
                    $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
                    $leadsData = $leadsMapper->getLeadAndOrganizationMobileByLeadIds($leadIds, $limit, $offset);
                }

                if (empty($leadsData) && !empty($matchingFilter)) {
                    $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Leads have been added'));
                }

                if(!empty($leadsData) && !empty($smsMessage)) {
                    $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
                    $leadType = Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_SMS;
                    $type = Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND;
                    $leadTypeId = Tools_LeadTools::getLeadTypeIdByLeadTypeName($leadType);

                    if(!empty($uniqueMobilesNumbersData)) {
                        $this->uniqueMobilesNumbers = $uniqueMobilesNumbersData;
                    }

                    foreach ($leadsData as $data) {
                        $createdAt = Tools_System_Tools::convertDateFromTimezone('now');

                        $leadMobileCountryCodeValue = '';
                        $leadMobile = '';
                        $organizationMobileCountryCodeValue = '';
                        $organizationMobile = '';

                        if(!empty($data['lead_mobile_country_code_value']) && !empty($data['lead_mobile'])) {
                            $leadMobileCountryCodeValue = $data['lead_mobile_country_code_value'];
                            $leadMobile = $data['lead_mobile'];
                        } elseif (!empty($data['organization_mobile_country_code_value']) && !empty($data['organization_mobile'])) {
                            $organizationMobileCountryCodeValue = $data['organization_mobile_country_code_value'];
                            $organizationMobile = $data['organization_mobile'];
                        }

                        $leadOrganizationMobile = Tools_LeadTools::glueUserPhone(
                            $leadMobileCountryCodeValue,
                            $leadMobile,
                            $organizationMobileCountryCodeValue,
                            $organizationMobile
                        );

                        $skipSendSms = true;
                        if(!in_array($leadOrganizationMobile, $this->uniqueMobilesNumbers)) {
                            array_push($this->uniqueMobilesNumbers, $leadOrganizationMobile);
                            $skipSendSms = false;
                        }

                        $leadModel = $leadsMapper->find($data['id']);

                        if($leadModel instanceof Leads_Model_LeadsModel) {
                            if (!empty($leadTypeId)) {
                                $leadModel->setLastActivityTypeId($leadTypeId);
                                $leadModel->setLastActivitySubtype('');
                                if ($type == Leads_Model_LeadsActionLogModel::ACTION_TYPE_INBOUND) {
                                    $leadModel->setLastActivity($createdAt);
                                } else {
                                    $leadModel->setLastActivityOutbound($createdAt);
                                }
                                $leadsMapper->save($leadModel);
                            }

                            $smsMessage = Tools_LeadSmsTools::addDictionarySmsFields($smsMessage, $leadModel, $this->_websiteUrl);

                            if(!empty($leadOrganizationMobile)) {
                                $subscriber['subscriber']['admin'] = array(
                                    'phone' => array($leadOrganizationMobile),
                                    'message' => $smsMessage,
                                    'owner_type' => Apps::SMS_OWNER_TYPE_USER,
                                    'sms_from_type' => 'crm',
                                    'custom_params' => array()
                                );

                                if($skipSendSms === false) {
                                    Apps::apiCall('POST', 'apps', array('twilioSms'), $subscriber);
                                }

                                $actionLedLogModel = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_SMS,
                                    $leadModel->getId(), $type, $createdAt, $leadType);
                                $callLogMapper = Leads_Mapper_LeadsCallLogMapper::getInstance();
                                $callLogModel = new Leads_Model_LeadsCallLogModel();
                                $callLogModel->setActionLogId($actionLedLogModel->getId());
                                $callLogModel->setType(Leads_Model_LeadsCallLogModel::TYPE_SMS);
                                $callLogModel->setStatus('completed');
                                $callLogModel->setScore('valid');
                                $callLogModel->setCreatorId($userId);
                                $callLogModel->setDetails($smsMessage);
                                $callLogModel->setLeadId($leadModel->getId());
                                $callLogModel->setIsVoip('0');
                                $callLogModel->setCaller('');
                                $callLogModel->setReceiver($leadOrganizationMobile);
                                $callLogModel->setTimestamp($createdAt);
                                $callLogMapper->save($callLogModel);
                            }
                        }
                    }

                    if (count($leadsData) == $limit) {
                        $this->_responseHelper->success(array('quantity' => count($leadsData), 'uniqueMobilesNumbers' => $this->uniqueMobilesNumbers));
                    } else {
                        $meassageText = $this->_translator->translate(self::TRANSLATION_PREFIX.'All SMS messages have been sent');
                        if(count($this->uniqueMobilesNumbers) == 1) {
                            $meassageText = $this->_translator->translate(self::TRANSLATION_PREFIX.'SMS has been sent');
                        }

                        $this->_responseHelper->fail(array('quantity' => count($leadsData), 'message' => $meassageText));
                    }
                }
            }
        }
        $this->_responseHelper->fail('');
    }
    /**
     * Send emails to email service through mojo.seosamba.com
     */
    public function sendToEmailServiceAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $this->_validateToken();
            $emailServiceName = filter_var($this->_request->getParam('emailServiceName'), FILTER_SANITIZE_STRING);
            $emailLists = filter_var($this->_request->getParam('lists'), FILTER_SANITIZE_STRING);
            $listsLabels = filter_var($this->_request->getParam('listsLabels'), FILTER_SANITIZE_STRING);
            $leadIds = filter_var($this->_request->getParam('leadIds'), FILTER_SANITIZE_STRING);
            $detailedInfo = (bool) filter_var($this->_request->getParam('detailedInfo'), FILTER_SANITIZE_STRING);
            $matchingFilter = (bool) filter_var($this->_request->getParam('matchingFilter'), FILTER_SANITIZE_STRING);
            $filterQuantity = filter_var($this->_request->getParam('filterQuantity'), FILTER_SANITIZE_NUMBER_INT);
            $stepSendEmailService = filter_var($this->_request->getParam('stepSendEmailService'), FILTER_SANITIZE_NUMBER_INT);
            $filters = $this->_request->getParam('filters');
            $requestType = $this->_request->getParam('requestType');
            $limit = '100';
            $offset = null;
            if (!empty($stepSendEmailService)) {
                $offset = $stepSendEmailService*$limit;
                $finalLimit = ($stepSendEmailService+1)*$limit;
                if ($filterQuantity < $finalLimit && empty($filterQuantity % $limit)) {
                    $requestInvalidLeads = $this->_request->getParam('invalidLeads');
                    if(!empty($requestInvalidLeads)) {
                        $this->processErrorLogData($requestInvalidLeads, 0);
                    }
                    $this->_responseHelper->fail(array('quantity' => 0, 'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Leads have been added')));
                }
            }

            if (!empty($matchingFilter)) {
                $filters .= $filters.'&not_empty_email_address=1';
                $leadsData = Tools_LeadTools::getSearchFilterData($filters, $limit, $offset);
                if (!empty($leadsData)) {
                    $leadsIds = array();
                    foreach ($leadsData['data'] as $lead) {
                        array_push($leadsIds, $lead['id']);
                    }
                    $leadIds = implode(',', $leadsIds);
                }
            }

            if (empty($leadsData) && !empty($matchingFilter)) {
                $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Leads have been added'));
            }

            if (!empty($emailLists) && !empty($emailServiceName)) {
                $emailLists = array_filter(explode(',', $emailLists));
                $leadIds = array_filter(explode(',', $leadIds));
                if (!empty($emailLists) && !empty($leadIds)) {
                    $serviceInfo = Apps_Models_Mapper_AppsSettingsMapper::getInstance()->getByServiceName($emailServiceName);
                    if (empty($serviceInfo)) {
                       $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Service doesn\'t exists'));
                    }
                    $serviceType = $serviceInfo[0]->getCategory();

                    $data['services'] = array($emailServiceName => array('type' => $serviceType));
                    $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
                    if (($serviceType === Apps::SERVICE_TYPE_EMAIL) && !$detailedInfo) {
                        $leadsData = $leadsMapper->findByLeadIdsForEmailService($leadIds);
                    } else {
                        $leadsData = $leadsMapper->getExtendedLeadInfoForEmailService($leadIds);
                    }

                    if (!empty($leadsData)) {
                        $mojoClientAction = $emailServiceName . 'Client';
                        //$leadsData[1]['email'] = '';
                        //$leadsData[2]['firstName'] = '';
                        $data['services'][$emailServiceName]['clients'] = $leadsData;
                        $data['services'][$emailServiceName]['lists'] = $emailLists;

                        if($requestType == 'unassign') {
                            $response = Apps::apiCall('DELETE', 'apps', array($mojoClientAction), $data);
                        } else {
                            $response = Apps::apiCall('POST', 'apps', array($mojoClientAction), $data);
                        }
                        if ($response[$mojoClientAction]['done']) {
                          Tools_LeadScoreWorkflowTools::assignListNameTags($leadIds, $listsLabels);
                          if (count($leadsData) == $limit && !empty($matchingFilter)) {
                                $invalidLeads = array();
                                $requestInvalidLeads = $this->_request->getParam('invalidLeads');
                                if(!empty($response[$mojoClientAction]['invalidLeads']) && $detailedInfo) {
                                    if(!empty($requestInvalidLeads)) {
                                        $invalidLeads = $requestInvalidLeads;
                                    }
                                } else if(!empty($requestInvalidLeads)) {
                                    $invalidLeads = $requestInvalidLeads;
                                }
                                $invalidLeads = array_merge($invalidLeads, $response[$mojoClientAction]['invalidLeads']);

                                $this->_responseHelper->success(array('quantity' => count($leadIds), 'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Leads have been added'), 'invalidLeads' => $invalidLeads));
                            } else {
                                $requestInvalidLeads = $response[$mojoClientAction]['invalidLeads'];
                                $invalidLeads = $this->_request->getParam('invalidLeads');
                                if(is_null($invalidLeads)) {
                                    $invalidLeads = array();
                                }
                                $invalidLeads = array_merge($requestInvalidLeads, $invalidLeads);

                                if(!empty($invalidLeads)) {
                                    $this->processErrorLogData($invalidLeads, count($leadIds));
                                } else {
                                    $this->_responseHelper->fail(array('quantity' => count($leadIds), 'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Leads have been added')));
                                }
                            }
                        }
                        if (empty($response[$mojoClientAction]['message'])) {
                            $this->_responseHelper->response($this->_translator->translate(self::TRANSLATION_PREFIX.'Unknown error'), 1, 400);
                        }
                        $this->_responseHelper->response($response[$mojoClientAction]['message'], 1, 400);
                    }
                } else {
                    $requestInvalidLeads = $this->_request->getParam('invalidLeads');
                    if(!empty($requestInvalidLeads)) {
                        $this->processErrorLogData($requestInvalidLeads, 0);
                    }
                }
            }
        }
        $this->_responseHelper->fail('');
    }

    /**
     * Send emails to email service through mojo.seosamba.com in organizations
     */
    public function sendToEmailServiceOrganizationAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $this->_validateToken();
            $emailServiceName = filter_var($this->_request->getParam('emailServiceName'), FILTER_SANITIZE_STRING);
            $emailLists = filter_var($this->_request->getParam('lists'), FILTER_SANITIZE_STRING);
            $listsLabels = filter_var($this->_request->getParam('listsLabels'), FILTER_SANITIZE_STRING);
            $organizationIds = filter_var($this->_request->getParam('organizationIds'), FILTER_SANITIZE_STRING);
            $detailedInfo = (bool) filter_var($this->_request->getParam('detailedInfo'), FILTER_SANITIZE_STRING);
            $matchingFilter = (bool) filter_var($this->_request->getParam('matchingFilter'), FILTER_SANITIZE_STRING);
            $filterQuantity = filter_var($this->_request->getParam('filterQuantity'), FILTER_SANITIZE_NUMBER_INT);
            $stepSendEmailService = filter_var($this->_request->getParam('stepSendEmailService'), FILTER_SANITIZE_NUMBER_INT);
            $filters = $this->_request->getParam('filters');
            $limit = self::PROCESSED_ORGANIZATION_LIMIT;
            $requestType = $this->_request->getParam('requestType');
            $offset = null;
            if (!empty($stepSendEmailService)) {
                $offset = $stepSendEmailService*$limit;
                $finalLimit = ($stepSendEmailService+1)*$limit;
                if ($filterQuantity < $finalLimit && empty($filterQuantity % $limit)) {
                    $requestInvalidLeads = $this->_request->getParam('invalidLeads');
                    if(!empty($requestInvalidLeads)) {
                        $this->processErrorLogData($requestInvalidLeads, 0);
                    }
                    $this->_responseHelper->fail(array('quantity' => 0, 'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Leads have been added')));
                }
            }

            if (!empty($matchingFilter)) {
                $filters .= $filters.'&not_empty_email_address=1';
                $organizationsData = Tools_LeadTools::getSearchOrganizationFilterData($filters, $limit, $offset);
                if (!empty($organizationsData)) {
                    $organizationsIds = array();
                    foreach ($organizationsData['data'] as $lead) {
                        array_push($organizationsIds, $lead['id']);
                    }
                    $organizationIds = implode(',', $organizationsIds);
                }
            }

            $leadIdsFull = array();

            if(!empty($organizationIds)) {
                $organizationIds = explode(',', $organizationIds);
                $organizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
                foreach ($organizationIds as $orgId) {
                    $leadsOrganizations = $organizationMapper->getOrganizationLeads($orgId);

                    if(!empty($leadsOrganizations)) {
                        $leadIdsFull = array_merge($leadIdsFull, array_keys($leadsOrganizations));
                    }
                }
            }

            if (empty($organizationsData) && !empty($matchingFilter)) {
                $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Leads have been added'));
            }

            if (!empty($emailLists) && !empty($emailServiceName)) {
                $emailLists = array_filter(explode(',', $emailLists));
                if (!empty($emailLists) && !empty($leadIdsFull)) {
                    $serviceInfo = Apps_Models_Mapper_AppsSettingsMapper::getInstance()->getByServiceName($emailServiceName);
                    if (empty($serviceInfo)) {
                        $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Service doesn\'t exists'));
                    }
                    $serviceType = $serviceInfo[0]->getCategory();

                    $data['services'] = array($emailServiceName => array('type' => $serviceType));
                    $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
                    if (($serviceType === Apps::SERVICE_TYPE_EMAIL) && !$detailedInfo) {
                        $organizationsData = $leadsMapper->findByLeadIdsForEmailService($leadIdsFull);
                    } else {
                        $organizationsData = $leadsMapper->getExtendedLeadInfoForEmailService($leadIdsFull);
                    }

                    if (!empty($organizationsData)) {
                        $mojoClientAction = $emailServiceName . 'Client';
                        $data['services'][$emailServiceName]['clients'] = $organizationsData;
                        $data['services'][$emailServiceName]['lists'] = $emailLists;

                        if($requestType == 'unassign') {
                            $response = Apps::apiCall('DELETE', 'apps', array($mojoClientAction), $data);
                        } else {
                            $response = Apps::apiCall('POST', 'apps', array($mojoClientAction), $data);
                        }

                        if ($response[$mojoClientAction]['done']) {
                            Tools_LeadScoreWorkflowTools::assignListNameTags($leadIdsFull, $listsLabels);
                            if(!empty($matchingFilter)) {
                                $invalidLeads = array();
                                $requestInvalidLeads = $this->_request->getParam('invalidLeads');
                                if(!empty($response[$mojoClientAction]['invalidLeads']) && $detailedInfo) {
                                    if(!empty($requestInvalidLeads)) {
                                        $invalidLeads = $requestInvalidLeads;
                                    }
                                } else if(!empty($requestInvalidLeads)) {
                                    $invalidLeads = $requestInvalidLeads;
                                }
                                $invalidLeads = array_merge($invalidLeads, $response[$mojoClientAction]['invalidLeads']);

                                $this->_responseHelper->success(array('quantity' => count($leadIdsFull), 'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Leads have been added'), 'invalidLeads' => $invalidLeads));
                            } else {
                                $requestInvalidLeads = $response[$mojoClientAction]['invalidLeads'];
                                $invalidLeads = $this->_request->getParam('invalidLeads');
                                if(is_null($invalidLeads)) {
                                    $invalidLeads = array();
                                }
                                $invalidLeads = array_merge($requestInvalidLeads, $invalidLeads);

                                if(!empty($invalidLeads)) {
                                    $this->processErrorLogData($invalidLeads, count($leadIdsFull));
                                } else {
                                    $this->_responseHelper->fail(array('quantity' => count($leadIdsFull), 'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Leads have been added')));
                                }
                            }
                        }
                        if (empty($response[$mojoClientAction]['message'])) {
                            $this->_responseHelper->response($this->_translator->translate(self::TRANSLATION_PREFIX.'Unknown error'), 1, 400);
                        }
                        $this->_responseHelper->response($response[$mojoClientAction]['message'], 1, 400);
                    }
                } else {
                    $requestInvalidLeads = $this->_request->getParam('invalidLeads');
                    if(!empty($requestInvalidLeads)) {
                        $this->processErrorLogData($requestInvalidLeads, 0);
                    }

                    $this->_responseHelper->fail(array('quantity' => 0, 'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Leads have been added')));
                }
            }
        }
        $this->_responseHelper->fail('');
    }

    /**
     * Send Ad audience lists through mojo.seosamba.com
     */
    public function sendToRemarketingListAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $this->_validateToken();
            $lists = json_decode($this->_request->getParam('lists'), true);
            $listsLabels = json_decode($this->_request->getParam('listsLabels'), true);
            $leadIds = filter_var($this->_request->getParam('leadIds'), FILTER_SANITIZE_STRING);
            $matchingFilter = (bool) filter_var($this->_request->getParam('matchingFilter'), FILTER_SANITIZE_STRING);
            $filterQuantity = filter_var($this->_request->getParam('filterQuantity'), FILTER_SANITIZE_NUMBER_INT);
            $stepToRemarketingListService = filter_var($this->_request->getParam('stepToRemarketingListService'), FILTER_SANITIZE_NUMBER_INT);
            $filters = $this->_request->getParam('filters');
            $gridType = $this->_request->getParam('gridType');
            $limit = '100';
            $requestType = $this->_request->getParam('requestType');
            $offset = null;

            $deafaultMsg = $this->_translator->translate(self::TRANSLATION_PREFIX.'Added to Ad audience list(s)');
            if($requestType == 'unassign') {
                $deafaultMsg = $this->_translator->translate(self::TRANSLATION_PREFIX.'Removed from Ad audience list(s)');
            }

            $accountErrors = array();

            if (!empty($stepToRemarketingListService)) {
                $offset = $stepToRemarketingListService*$limit;
                $finalLimit = ($stepToRemarketingListService+1)*$limit;
                if ($filterQuantity < $finalLimit && empty($filterQuantity % $limit)) {
                    $requestAccountErrors = $this->_request->getParam('accountErrors');
                    if(!empty($requestAccountErrors)) {
                        $accountErrors = $requestAccountErrors;
                    }
                    $this->_responseHelper->fail(array('quantity' => 0, 'accountErrors' => $accountErrors, 'message' => $deafaultMsg));
                }
            }

            if (!empty($matchingFilter) && empty($gridType)) {
                $filters .= $filters.'&not_empty_email_address=1';
                $leadsData = Tools_LeadTools::getSearchFilterData($filters, $limit, $offset);
                if (!empty($leadsData)) {
                    $leadsIds = array();
                    foreach ($leadsData['data'] as $lead) {
                        array_push($leadsIds, $lead['id']);
                    }
                    $leadIds = implode(',', $leadsIds);
                }
            }

            if (empty($leadsData) && !empty($matchingFilter) && empty($gridType)) {
                $this->_responseHelper->fail($deafaultMsg);
            }

            if (!empty($lists) && !empty($leadIds)) {
                $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
                $leadIds = array_filter(explode(',', $leadIds));

                if($gridType == 'organizations') {
                    $where = $leadsMapper->getDbTable()->getAdapter()->quoteInto('pld.id IN (?)', $leadIds);
                    $lData = $leadsMapper->fetchAll($where, null, $limit, $offset, false, false);

                    $leadsData = array();
                    if(!empty($lData)) {
                        foreach ($lData['data'] as $key => $lDataParams) {
                            $leadsData[$key] = array(
                              'id' => $lDataParams['id'],
                              'email' => $lDataParams['lead_email'],
                              'phoneNumberCode' => $lDataParams['lead_phone_country_code_value'],
                              'phoneNumber' => $lDataParams['lead_phone'],
                              'mobileNumberCode' => $lDataParams['lead_mobile_country_code_value'],
                              'mobileNumber' => $lDataParams['lead_mobile'],
                              'city' => $lDataParams['lead_city'],
                              'state' => $lDataParams['lead_state_code'],
                              'country' => $lDataParams['lead_country_code'],
                              'zip' => $lDataParams['lead_zip'],
                              'firstName' => $lDataParams['lead_first_name'],
                              'lastName' => $lDataParams['lead_last_name']
                            );
                        }
                    }
                } else {
                    $leadsData = $leadsMapper->getExtendedLeadInfoForRemarketingLists($leadIds);
                }

                if (!empty($leadsData)) {
                    $existedLeadsIds = array();
                    foreach ($leadsData as $key => $leadData) {
                        $leadData['phone'] = '';
                        if(!empty($leadData['mobileNumberCode']) && !empty($leadData['mobileNumber'])) {
                            $leadData['phone'] = $leadData['mobileNumberCode'] . $leadData['mobileNumber'];
                        } elseif (!empty($leadData['phoneNumberCode']) && !empty($leadData['phoneNumber'])) {
                            $leadData['phone'] = $leadData['phoneNumberCode'] . $leadData['phoneNumber'];
                        }

                        $leadsData[$key] = $leadData;

                        if(in_array($leadData['id'], $leadIds)) {
                            $existedLeadsIds[] = $leadData['id'];
                        }
                    }

                    $requestAccountErrors = $this->_request->getParam('accountErrors');
                    $response['errorList'] = array();
                    $responseErrorList = array();

                    foreach ($lists as $marketingCampaignsName => $listsIds) {
                        if(!empty($listsIds)) {
                            $data['type'] = $marketingCampaignsName;
                            $data['audienceId'] = $listsIds;
                            $data['listContent'] = $leadsData;
                            $data['remove'] = false;

                            if($requestType == 'unassign') {
                                $data['remove'] = true;
                            }

                            /*$postLists = array(
                               //'websiteId' => null,
                               'type' => "facebook",
                               'audienceId' =>array(
                                   '23845629305980205',
                                   '34565745475745455'
                               ),
                               'listContent' => array(
                                   array(
                                       'email' => '', 'phone' => '+380...', 'city' => '', 'state' => '', 'country' => '', 'zip' => '', 'firstName' => '', 'lastName' => ''
                                   ),
                                   array(
                                       'email' => '', 'phone' => '+380...', 'city' => '', 'state' => '', 'country' => '', 'zip' => '', 'firstName' => '', 'lastName' => ''
                                   )
                               )
                            );*/

                            /*$response = array('errorList' => array(
                                '23850212804950651' => 'Audience doesn\'t belong to connected account',
                                '23850085689610651' => 'OTHER EEERRROR'
                            ));*/

                            $response = Apps::apiCall('POST', 'sambastatsAddDataToRemarketingList', array(), $data);

                            if(!empty($response['errorList'])) {
                                foreach ($response['errorList'] as $key => $error) {
                                    $errorListKey = array_search($key, $lists[$marketingCampaignsName]);
                                    unset($listsLabels[$marketingCampaignsName][$errorListKey]);
                                }
                            }

                            if(!empty($listsLabels[$marketingCampaignsName])) {
                                $leadTagsMapper = Leads_Mapper_LeadsTagsMapper::getInstance();

                                foreach ($listsLabels[$marketingCampaignsName] as $listLabelKey => $listLabel) {
                                    $adsPrefix = 'Added to Google Audience';
                                    if($marketingCampaignsName == 'facebook') {
                                        $adsPrefix = 'Added to FB Audience';
                                    }

                                    $tag = $adsPrefix . ' ' . $listLabel .' - '. $lists[$marketingCampaignsName][$listLabelKey];//Added to Google Audience SMB CRM Leads - 243575394539

                                    $leadTagModel = $leadTagsMapper->getTag($tag);
                                    if (!$leadTagModel instanceof Leads_Model_LeadsTagModel) {
                                        $leadTagModel = new Leads_Model_LeadsTagModel();
                                        $leadTagModel->setLabel($tag);
                                        $leadTagModel->setTagName($tag);
                                        $leadTagModel = $leadTagsMapper->save($leadTagModel);
                                    }

                                    foreach ($existedLeadsIds as $leadId) {
                                        $leadsMapper->addTag($leadId, $leadTagModel->getId());
                                    }
                                }
                            }

                            if(!empty($response['errorList'])) {
                                $responseErrorList[$marketingCampaignsName] = $response['errorList'];
                            }
                        }
                    }

                    if (count($leadsData) == $limit && !empty($matchingFilter)) {
                        if(!empty($responseErrorList)) {
                            if(!empty($requestAccountErrors)) {
                                foreach ($responseErrorList as $key => $error) {
                                    if(!array_key_exists($key, $requestAccountErrors)) {
                                        $requestAccountErrors[$key] = $error;
                                    }
                                }

                                $accountErrors = $requestAccountErrors;
                            } else {
                                $accountErrors = $responseErrorList;
                            }
                        }

                        $this->_responseHelper->success(array('quantity' => count($leadsData), 'message' => $deafaultMsg, 'accountErrors' => $accountErrors));
                    } else {
                        if(!empty($responseErrorList)) {
                            $accountErrors += $responseErrorList;
                        } else if(!empty($requestAccountErrors)) {
                            $accountErrors += $requestAccountErrors;
                        }

                        $this->_responseHelper->fail(array('quantity' => count($leadsData), 'accountErrors' => $accountErrors, 'message' => $deafaultMsg));
                    }
                }
            }
        }
        $this->_responseHelper->fail('');
    }

    /**
     * @param $errorsData
     * @param $quantity
     */
    public function processErrorLogData ($errorsData, $quantity) {
        $errorFileName = 'email-marketing-error-information-' . date('U') . '.csv';
        $errorFilePath = $this->_websiteHelper->getPath() . $this->_websiteHelper->getTmp() . $errorFileName;

        $fileWrite = fopen($errorFilePath, 'w');

        $errorsDataHeaders = array(
            'email',
            'firstName',
            'lastName',
            'city',
            'state',
            'country',
            'zip',
            'phoneCountryCode',
            'phoneNumber',
            'mobileCode',
            'mobileNumber',
            'externalId',
            'position',
            'organization',
            'websiteId',
            'errorMessage'
        );

        fputcsv($fileWrite, $errorsDataHeaders);

        foreach($errorsData as $leadInfoKey => $leadInfo){
            $result = array();
            foreach ($errorsDataHeaders as $key => $value) {
                if(array_key_exists($value, $leadInfo)) {
                    $result[$value] = $leadInfo[$value];
                } else {
                    $result[$value] = '';
                }
            }
            fputcsv($fileWrite, $result);
        }
        fclose($fileWrite);

        $this->_responseHelper->fail(array('message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'We\'ve processed your file, however some records could not be imported. Please check the error log.'), 'errorFileName' => $errorFileName, 'quantity' => $quantity));
    }

    /**
     * Change default opportunity stage
     */
    public function setDefaultOpportunityStageAction()
    {
        $currentUserRole = $this->_sessionHelper->getCurrentUser()->getRoleId();
        if ($currentUserRole === Tools_Security_Acl::ROLE_ADMIN || $currentUserRole === Tools_Security_Acl::ROLE_SUPERADMIN) {
            $this->_validateToken();
            $stageId = filter_var($this->_request->getParam('stageId'), FILTER_SANITIZE_NUMBER_INT);
            $leadOpportunityStageMapper = Leads_Mapper_LeadsOpportunityStageMapper::getInstance();
            $leadOpportunityStageMapper->disableDefaultStage();
            if (!empty($stageId)) {
                $leadOpportunityStageMapper->assignDefaultStage($stageId);
            }
            $this->_responseHelper->success($this->_translator->translate(self::TRANSLATION_PREFIX.'Default opportunity stage has been set to a new value'));
        }
        $this->_responseHelper->fail('');
    }

    /**
     * Tab general lead config
     */
    public function crmConfigLeadGeneralAction()
    {
        $currentUserRole = $this->_sessionHelper->getCurrentUser()->getRoleId();
        if ($currentUserRole === Tools_Security_Acl::ROLE_ADMIN || $currentUserRole === Tools_Security_Acl::ROLE_SUPERADMIN) {
            $this->_view->leadGeneralConfigParams = Leads_Mapper_LeadsConfigMapper::getInstance()->getConfigParams();
            $crmGoogleMapKey = Zend_Controller_Action_HelperBroker::getStaticHelper('config')->getConfig('crmGoogleMapKey');
            $this->_view->crmGoogleMapKey = $crmGoogleMapKey;
            $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
            $crmGoogleMapDefaultZoom = $configHelper->getConfig('crmGoogleMapDefaultZoom');
            $redirectAdminAfterLogin = $configHelper->getConfig('redirectAdminAfterLogin');
            $enableLeadOrgGeotagging = $configHelper->getConfig('enableLeadOrgGeotagging');
            $this->_view->redirectForAdminEnabled = false;
            if (!empty($redirectAdminAfterLogin) && $redirectAdminAfterLogin === 'dashboard') {
                $this->_view->redirectForAdminEnabled = true;
            }
            if (!empty($enableLeadOrgGeotagging) && $enableLeadOrgGeotagging === '1') {
                $this->_view->enableLeadOrgGeotagging = true;
            }

            $this->_view->crmGoogleMapDefaultZoom = $crmGoogleMapDefaultZoom;
            $currentUser = $this->_sessionHelper->getCurrentUser();
            $userRole = $currentUser->getRoleId();
            $this->_view->userRole = $userRole;

            $templatesAndDirectoriesMapper = Leads_Mapper_LeadsTemplatesAndDirectoriesConfigMapper::getInstance();

            $savedConfigFolders = $templatesAndDirectoriesMapper->findByType('folder');
            $listFolders = $this->_getFoldersList();
            $listFolders = array_diff($listFolders, $savedConfigFolders);
            $this->_view->listFolders = $listFolders;
            $this->_view->savedConfigFolders = $savedConfigFolders;

            $templates = Application_Model_Mappers_TemplateMapper::getInstance()->findByType('typemail');

            $savedConfigTemplates = $templatesAndDirectoriesMapper->findByType('template');
            $templateNames = array();

            foreach ($templates as $key => $value){
                $templateNames[] = $value->getName();
                if(in_array($value->getName(), $savedConfigTemplates)){
                    unset($templates[$key]);
                }
            }

            $templatesPagesUrls = array();

            if(!empty($templateNames)){
                foreach ($savedConfigTemplates as $key => $templateName){
                    $templateId = $templateName;
                    if ($templateId === 'default') {
                        $activeActionEmailTriggers = Tools_LeadTools::getDefaultActionEmailInfo(Tools_LeadsMailWatchdog::LEADS_CONTACT);
                        if (!empty($activeActionEmailTriggers)) {
                            $templateId = $activeActionEmailTriggers['template'];
                        } else {
                            $templateId = 0;
                        }
                    }

                    $pageModel = Tools_LeadTools::getTemplatePage($templateId);
                    if ($pageModel instanceof Application_Model_Models_Page) {
                        $pageUrl = $pageModel->getUrl();
                        $templatesPagesUrls[$key] = $pageUrl;
                    }


                    if(!in_array($templateName, $templateNames)){
                      $currentRow = $templatesAndDirectoriesMapper->findRow('template', $templateName);

                      if($currentRow instanceof Leads_Model_LeadsTemplatesAndDirectoriesConfigModel){
                          $templatesAndDirectoriesMapper->delete($currentRow->getId());
                          unset($savedConfigTemplates[$currentRow->getId()]);
                      }
                    }
                }
            }

            $pageMapper = Application_Model_Mappers_PageMapper::getInstance();

            $this->_view->templatesPagesUrls = $templatesPagesUrls;
            $this->_view->templates = $templates;
            $this->_view->savedConfigTemplates = $savedConfigTemplates;
            $this->_view->leadOwners = Leads_Mapper_LeadsMapper::getInstance()->getLeadOwnersFullList();
            $this->_view->regularPages = $pageMapper->fetchRegularPagesIdUrlPairs();

            echo $this->_view->render('crm-lead-general-config.phtml');
        }
    }

    /**
     * Refresh list directories in Lead config tab
     */
    public function crmConfigLeadGeneralRefreshDirectoriesAction()
    {
        $currentUserRole = $this->_sessionHelper->getCurrentUser()->getRoleId();
        if (($currentUserRole === Tools_Security_Acl::ROLE_ADMIN || $currentUserRole === Tools_Security_Acl::ROLE_SUPERADMIN) && $this->_request->isPost()) {
            $this->_validateToken();

            $type = filter_var($this->_request->getParam('type'), FILTER_SANITIZE_STRING);

            $templatesAndDirectoriesMapper = Leads_Mapper_LeadsTemplatesAndDirectoriesConfigMapper::getInstance();

            if($type == 'folder'){
                $folders = $this->_getFoldersList();
                if(!empty($folders)){
                    $removedFolders = array();
                    $savedConfigFolders = $templatesAndDirectoriesMapper->findByType('folder');
                    if(!empty($savedConfigFolders)){
                        foreach ($savedConfigFolders as $key => $folder){
                            if(!in_array($folder, $folders)){
                                $templatesAndDirectoriesMapper->delete($key);
                                $removedFolders[] = $folder;
                            }else{
                                unset($folders[$folder]);
                            }
                        }
                    }

                    $this->_responseHelper->success(array('type' => $type, 'listFolders' => $folders, 'removedItems' => $removedFolders));
                }

                $this->_responseHelper->fail('Empty directories list');
            }
        }

        $this->_responseHelper->fail('');
    }

    /**
     * Get media folders list
     *
     * @param bool $imagesOnly
     * @return array|bool
     */
    private function _getFoldersList($imagesOnly = false)
    {
        $listFolders = Tools_Filesystem_Tools::scanDirectoryForDirs(
            $this->_websiteConfig['path'] . $this->_websiteConfig['media']
        );
        if (!empty ($listFolders)) {
            if ($imagesOnly) {
                foreach ($listFolders as $key => $folder) {
                    if (!is_dir($this->_websiteConfig['path'] . $this->_websiteConfig['media'] . $folder . '/small')) {
                        unset($listFolders[$key]);
                    }
                }
            }
            $listFolders = array_combine($listFolders, $listFolders);

            natcasesort($listFolders);
        }
        return $listFolders;
    }

    /**
     * Get media folders list
     *
     * @param bool $imagesOnly
     * @return array|bool
     */
    public static function getFoldersList($imagesOnly = false)
    {
        $websiteConfig = Zend_Registry::get('website');
        $listFolders = Tools_Filesystem_Tools::scanDirectoryForDirs(
            $websiteConfig['path'] . $websiteConfig['media']
        );
        if (!empty ($listFolders)) {
            if ($imagesOnly) {
                foreach ($listFolders as $key => $folder) {
                    if (!is_dir($websiteConfig['path'] . $websiteConfig['media'] . $folder . '/small')) {
                        unset($listFolders[$key]);
                    }
                }
            }
            $listFolders = array_combine($listFolders, $listFolders);

            natcasesort($listFolders);
        }
        return $listFolders;
    }

    /**
     * Get Templates and directories from Leads_DbTable_LeadsTemplatesAndDirectoriesConfigDbtable
     */
    public function crmLeadsTemplatesAndDirectoriesAction(){
        $currentUserRole = $this->_sessionHelper->getCurrentUser()->getRoleId();
        if ($currentUserRole === Tools_Security_Acl::ROLE_ADMIN || $currentUserRole === Tools_Security_Acl::ROLE_SUPERADMIN) {
            if ($this->_request->isPost()) {
               $this->_validateToken();

               $templatesAndDirectoriesMapper = Leads_Mapper_LeadsTemplatesAndDirectoriesConfigMapper::getInstance();

               $data = filter_var_array($this->_request->getParams(), FILTER_SANITIZE_STRING);

                if(!empty($data['type']) && !empty($data['itemName'])){
                  $currentRow =  $templatesAndDirectoriesMapper->findRow($data['type'], $data['itemName']);

                  if(!$currentRow instanceof Leads_Model_LeadsTemplatesAndDirectoriesConfigModel){
                      $templatesAndDirectoriesModel = new Leads_Model_LeadsTemplatesAndDirectoriesConfigModel();

                      $templatesAndDirectoriesModel->setType($data['type']);
                      $templatesAndDirectoriesModel->setItemName($data['itemName']);

                    $templatesAndDirectories =  $templatesAndDirectoriesMapper->save($templatesAndDirectoriesModel);
                    $pageUrl = '';
                    if($data['type'] == 'template'){
                        $templateId = $data['itemName'];
                        if ($templateId === 'default') {
                            $activeActionEmailTriggers = Tools_LeadTools::getDefaultActionEmailInfo(Tools_LeadsMailWatchdog::LEADS_CONTACT);
                            if (!empty($activeActionEmailTriggers)) {
                                $templateId = $activeActionEmailTriggers['template'];
                            } else {
                                $templateId = 0;
                            }
                        }

                        $pageModel = Tools_LeadTools::getTemplatePage($templateId);
                        if ($pageModel instanceof Application_Model_Models_Page) {
                            $pageUrl = $pageModel->getUrl();
                        }
                    }

                    $this->_responseHelper->success(array('templatesAndDirectoriesType' => $templatesAndDirectories->getType(),
                        'templatesAndDirectoriesItemName' => $templatesAndDirectories->getItemName(),
                        'templatesAndDirectoriesItemId' => $templatesAndDirectories->getId(),
                        'pageUrl' => $pageUrl));
                  }
                }
                $this->_responseHelper->fail('');
            }
        }
        $this->_responseHelper->fail('');
    }

    /**
     * Remove template or directory from Leads_DbTable_LeadsTemplatesAndDirectoriesConfigDbtable
     */
    public function crmLeadsTemplatesAndDirectoriesRemoveItemAction(){
        $currentUserRole = $this->_sessionHelper->getCurrentUser()->getRoleId();
        if (($currentUserRole === Tools_Security_Acl::ROLE_ADMIN || $currentUserRole === Tools_Security_Acl::ROLE_SUPERADMIN) && $this->_request->isPost()) {
            $this->_validateToken();
            $itemId = filter_var($this->_request->getParam('itemId'), FILTER_SANITIZE_STRING);
            if(!empty($itemId)){
                $templatesAndDirectoriesMapper = Leads_Mapper_LeadsTemplatesAndDirectoriesConfigMapper::getInstance();

                $templatesAndDirectoriesModel = $templatesAndDirectoriesMapper->find($itemId);

                if($templatesAndDirectoriesModel instanceof Leads_Model_LeadsTemplatesAndDirectoriesConfigModel){
                    $templatesAndDirectoriesMapper->delete($templatesAndDirectoriesModel->getId());

                    $this->_responseHelper->success(array('itemType' => $templatesAndDirectoriesModel->getType(),
                        'itemName' => $templatesAndDirectoriesModel->getItemName()
                    ));
                }
            }
            $this->_responseHelper->fail('');
        }
        $this->_responseHelper->fail('');

    }

    /**
     * Cloud service tracking services tab
     */
    public function crmLeadsCloudTrackingServicesAction()
    {
        $currentUserRole = $this->_sessionHelper->getCurrentUser()->getRoleId();
        if ($currentUserRole === Tools_Security_Acl::ROLE_ADMIN || $currentUserRole === Tools_Security_Acl::ROLE_SUPERADMIN) {
            if ($this->_request->isPost()) {
                $this->_validateToken();
                $websites = $this->_request->getParam('websites');
                $type = $this->_request->getParam('type');

                if ($type === 'piwik') {
                    $websitesConfigurationDataResponse = Apps::apiCall('POST', 'sambastatsLeadconfig', '',
                        array('piwikWebsiteIds' => $websites));
                }

                if ($type === 'call') {
                    $websitesConfigurationDataResponse = Apps::apiCall('POST', 'appsLeadconfig', '',
                        array('websiteIds' => $websites));
                }

                if ($type === 'form') {
                    $websitesConfigurationDataResponse = Apps::apiCall('POST', 'appsLeadconfig', '',
                        array('websiteIds' => $websites, 'resourceType' => 'form'));
                }

                if (empty($websitesConfigurationDataResponse['done'])) {
                    $this->_responseHelper->fail($websitesConfigurationDataResponse['message']);
                }
                $this->_responseHelper->success($this->_translator->translate(self::TRANSLATION_PREFIX.'Configuration has been saved'));

            } else {
                $this->_view->leadGeneralConfigParams = Leads_Mapper_LeadsConfigMapper::getInstance()->getConfigParams();
                $configurationDataResponse = Apps::apiCall('GET', 'crmLeadconfig', '');
                $piwikWebsites = array();
                $callLogWebsites = array();

//                $appsLeadconfigPostActionArray = array('0' => array('website_id' => 444, 'url' => 'https://www.google.com.ua/', 'status' => '1'),
//                    '1' => array('website_id' => 445, 'url' => 'https://www.google.com.ua/', 'status' => '1'));
//                $configurationDataResponse = array(
//                    'leadConfig' => array('done' => true, 'data' => $appsLeadconfigPostActionArray),
//                    'callConfig' => array('done' => true, 'data' => $appsLeadconfigPostActionArray),
//                    'formConfig' => array('done' => true, 'data' => $appsLeadconfigPostActionArray)
//                );

                if (!empty($configurationDataResponse)) {
                    if (!empty($configurationDataResponse['leadConfig']) && $configurationDataResponse['leadConfig']['done'] !== false) {
                        $piwikWebsites = $configurationDataResponse['leadConfig']['data'];
                    }
                    if (!empty($configurationDataResponse['callConfig']) && $configurationDataResponse['callConfig']['done'] !== false) {
                        $callLogWebsites = $configurationDataResponse['callConfig']['data'];
                    }
                    if (!empty($configurationDataResponse['formConfig']) && $configurationDataResponse['formConfig']['done'] !== false) {
                        $formLogWebsites = $configurationDataResponse['formConfig']['data'];
                    }
                }

                if(!empty($piwikWebsites) && is_array($piwikWebsites)) {
                    $piwikWebsites = $this->_sortCloudTrackingServices($piwikWebsites);
                }

                if(!empty($callLogWebsites) && is_array($callLogWebsites)) {
                    $callLogWebsites = $this->_sortCloudTrackingServices($callLogWebsites);
                }

                if (!empty($formLogWebsites) && is_array($formLogWebsites)) {
                    $formLogWebsites = $this->_sortCloudTrackingServices($formLogWebsites);
                }

                $currentWebsiteId = false;
                $websiteConfig = Zend_Controller_Action_HelperBroker::getStaticHelper('config')->getConfig();
                if (isset($websiteConfig['sambaToken']) && (isset($websiteConfig['websiteId']))) {
                    $currentWebsiteId = $websiteConfig['websiteId'];
                }

                $this->_view->currentWebsiteId = $currentWebsiteId;
                $this->_view->piwikWebsites = $piwikWebsites;
                $this->_view->callLogWebsites = $callLogWebsites;
                $this->_view->formLogWebsites = $formLogWebsites;
                echo $this->_view->render('crm-cloud-tracking-services-config.phtml');
            }
        }
    }

    /**
     * Sort checked services on top
     *
     * @param $services
     * @return array
     */
    private function _sortCloudTrackingServices($services)
    {
        $servicesSorted = array();
        $servicesNotSortedTmp = array();

        foreach ($services as $service) {
            if(!empty($service['status'])) {
                array_push($servicesSorted, $service);
            } else {
                array_push($servicesNotSortedTmp, $service);
            }
        }
        return array_merge($servicesSorted, $servicesNotSortedTmp);
    }

    /**
     * Tab for stage crm config
     */
    public function crmConfigStageAction()
    {
        $currentUserRole = $this->_sessionHelper->getCurrentUser()->getRoleId();
        if ($currentUserRole === Tools_Security_Acl::ROLE_ADMIN || $currentUserRole === Tools_Security_Acl::ROLE_SUPERADMIN) {
            $leadsStageMapper = Leads_Mapper_LeadsStageMapper::getInstance();
            $this->_view->leadStages = $leadsStageMapper->fetchAll();

            echo $this->_view->render('crm-stage-config.phtml');
        }
    }

    /**
     * Tab for opportunity stage crm config
     */
    public function crmConfigOpportunitystageAction()
    {
        $currentUserRole = $this->_sessionHelper->getCurrentUser()->getRoleId();
        if ($currentUserRole === Tools_Security_Acl::ROLE_ADMIN || $currentUserRole === Tools_Security_Acl::ROLE_SUPERADMIN) {
            $this->_view->leadOpportunityStages = Leads_Mapper_LeadsOpportunityStageMapper::getInstance()->fetchAll();
            echo $this->_view->render('crm-opportunity-config.phtml');
        }
    }

    /**
     * Tab for industry types crm config
     */
    public function crmConfigIndustryAction()
    {
        $currentUserRole = $this->_sessionHelper->getCurrentUser()->getRoleId();
        if ($currentUserRole === Tools_Security_Acl::ROLE_ADMIN || $currentUserRole === Tools_Security_Acl::ROLE_SUPERADMIN) {
            $allowIndustriesCreation = Leads_Mapper_LeadsConfigMapper::getInstance()->getConfigParam('allowIndustriesCreation');
            $this->_view->allowIndustriesCreation = $allowIndustriesCreation;

            echo $this->_view->render('crm-industry-types-config.phtml');
        }
    }

    /**
     * Tab for lead crm config
     */
    public function crmConfigLeadAction()
    {
        $currentUserRole = $this->_sessionHelper->getCurrentUser()->getRoleId();
        if ($currentUserRole === Tools_Security_Acl::ROLE_ADMIN || $currentUserRole === Tools_Security_Acl::ROLE_SUPERADMIN) {
            $leadsTypeMapper = Leads_Mapper_LeadsTypeMapper::getInstance();
            $where = $leadsTypeMapper->getDbTable()->getAdapter()->quoteInto('type_name NOT IN (?)', Api_Leads_Leadtype::systemStatuses());
            $this->_view->leadTypes = Leads_Mapper_LeadsTypeMapper::getInstance()->fetchAll($where);
            echo $this->_view->render('crm-lead-type-config.phtml');
        }
    }

    /**
     * Tab organization custom params
     */
    public function crmConfigOrganizationCustomParamsAction()
    {
        $currentUserRole = $this->_sessionHelper->getCurrentUser()->getRoleId();
        if ($currentUserRole === Tools_Security_Acl::ROLE_ADMIN || $currentUserRole === Tools_Security_Acl::ROLE_SUPERADMIN) {
            echo $this->_view->render('crm-organization-custom-params-config.phtml');
        }
    }

    /**
     * Tab lead custom params
     */
    public function crmConfigLeadCustomParamsAction()
    {
        $currentUserRole = $this->_sessionHelper->getCurrentUser()->getRoleId();
        if ($currentUserRole === Tools_Security_Acl::ROLE_ADMIN || $currentUserRole === Tools_Security_Acl::ROLE_SUPERADMIN) {
            echo $this->_view->render('crm-lead-custom-params-config.phtml');
        }
    }

    /**
     * Tab lead/Organizations custom fields config
     */
    public function crmCustomFieldsConfigAction()
    {
        $currentUserRole = $this->_sessionHelper->getCurrentUser()->getRoleId();
        if ($currentUserRole === Tools_Security_Acl::ROLE_ADMIN || $currentUserRole === Tools_Security_Acl::ROLE_SUPERADMIN) {
            echo $this->_view->render('crm-lead-organization-custom-fields-config.phtml');
        }
    }

    /**
     * Tab Imap rules
     */
    public function crmImapRulesConfigAction()
    {
        $currentUserRole = $this->_sessionHelper->getCurrentUser()->getRoleId();
        if ($currentUserRole === Tools_Security_Acl::ROLE_ADMIN || $currentUserRole === Tools_Security_Acl::ROLE_SUPERADMIN) {
            //$this->_view->leadOwners = Leads_Mapper_LeadsMapper::getInstance()->getLeadOwnersFullList();
            echo $this->_view->render('crm-imap-rules-config.phtml');
        }
    }

    /**
     * Tab lead source config
     */
    public function crmConfigLeadSourceAction()
    {
        $currentUserRole = $this->_sessionHelper->getCurrentUser()->getRoleId();
        if ($currentUserRole === Tools_Security_Acl::ROLE_ADMIN || $currentUserRole === Tools_Security_Acl::ROLE_SUPERADMIN) {
            echo $this->_view->render('crm-lead-source-config.phtml');
        }
    }

    /**
     * Tab lead tags config
     */
    public function crmConfigLeadTagsAction()
    {
        $currentUserRole = $this->_sessionHelper->getCurrentUser()->getRoleId();
        if ($currentUserRole === Tools_Security_Acl::ROLE_ADMIN || $currentUserRole === Tools_Security_Acl::ROLE_SUPERADMIN) {
            echo $this->_view->render('crm-lead-tags-config.phtml');
        }
    }

    /**
     * Tab lead events config
     */
    public function crmConfigLeadEventsAction()
    {
        $currentUserRole = $this->_sessionHelper->getCurrentUser()->getRoleId();
        if ($currentUserRole === Tools_Security_Acl::ROLE_ADMIN || $currentUserRole === Tools_Security_Acl::ROLE_SUPERADMIN) {
            echo $this->_view->render('crm-lead-events-config.phtml');
        }
    }

    /**
     * Tab opportunity types
     */
    public function crmConfigOpportunityTypesAction()
    {
        $currentUserRole = $this->_sessionHelper->getCurrentUser()->getRoleId();
        if ($currentUserRole === Tools_Security_Acl::ROLE_ADMIN || $currentUserRole === Tools_Security_Acl::ROLE_SUPERADMIN) {
            echo $this->_view->render('crm-lead-opportunity-types-config.phtml');
        }
    }

    /**
     * Tab lead social services
     */
    public function crmConfigSocialServicesAction()
    {
        $currentUserRole = $this->_sessionHelper->getCurrentUser()->getRoleId();
        if ($currentUserRole === Tools_Security_Acl::ROLE_ADMIN || $currentUserRole === Tools_Security_Acl::ROLE_SUPERADMIN) {
            echo $this->_view->render('crm-lead-social-services-config.phtml');
        }
    }

    /**
     * Tab Configuration email IMAP
     */
    public function crmInboundEmailsConfigAction()
    {
        $currentUserRole = $this->_sessionHelper->getCurrentUser()->getRoleId();
        if ($currentUserRole === Tools_Security_Acl::ROLE_ADMIN || $currentUserRole === Tools_Security_Acl::ROLE_SUPERADMIN) {
            $this->_view->leadOwners = Leads_Mapper_LeadsMapper::getInstance()->getLeadOwnersFullList();
            echo $this->_view->render('lead-crm-imap-config.phtml');
        }
    }


    /**
     * Tab lead custom params
     */
    public function crmApiConfigAction()
    {
        $currentUserRole = $this->_sessionHelper->getCurrentUser()->getRoleId();
        if ($currentUserRole === Tools_Security_Acl::ROLE_ADMIN || $currentUserRole === Tools_Security_Acl::ROLE_SUPERADMIN) {
            echo $this->_view->render('crm-api-config.phtml');
        }
    }

    /**
     * @param $imapRowsIds
     * @return array
     * @throws Zend_Exception
     */
    public function checkConnection($imapRowsIds) {
        $translator = Zend_Registry::get('Zend_Translate');
        $leadImapDataMapper = Leads_Mapper_LeadsImapConfigMapper::getInstance();
        $connectionStatusArray = array();
        foreach ($imapRowsIds as $id) {
            $where = $leadImapDataMapper->getDbTable()->getAdapter()->quoteInto('plic.id = ?', $id);
            $data = $leadImapDataMapper->fetchAllData($where, null, null, null, true, true);
            if(!empty($data)) {
                $connection = Tools_LeadInboundEmailTools::getImapConnection($data['host'], $data['encryption'], $data['login'], $data['password']);
                if ($connection === false) {
                    $connectionStatus = $translator->translate(self::TRANSLATION_PREFIX.'Connection error');
                } else {
                    $connectionStatus = $translator->translate(self::TRANSLATION_PREFIX.'Connected');
                }

                $connectionStatusArray[$data['id']] = $connectionStatus;
            }
        }

        return $connectionStatusArray;
    }

    /**
     * Check connection status
     * Return array Connected|Connection error
     */
    public function checkImapConnectionsAction()
    {
        $currentUserRole = $this->_sessionHelper->getCurrentUser()->getRoleId();
        $salesOnly = $this->_request->getParam('salesOnly', false);

        if (($currentUserRole === Tools_Security_Acl::ROLE_ADMIN || $currentUserRole === Tools_Security_Acl::ROLE_SUPERADMIN) && $this->_request->isPost()) {
           $imapRowsIds = $this->_request->getParam('imapRowsIds', false);

           if(!empty($imapRowsIds)) {
               $connectionStatusArray = $this->checkConnection($imapRowsIds);
               $this->_responseHelper->success(array('connectionStatus' => $connectionStatusArray));
           }
        } elseif ($salesOnly && $currentUserRole === Shopping::ROLE_SALESPERSON && $this->_request->isPost()) {
            $imapRowsIds[] = $this->_request->getParam('imapRowsIds', false);

            if(!empty($imapRowsIds)) {
                $connectionStatusArray = $this->checkConnection($imapRowsIds);
                $this->_responseHelper->success(array('connectionStatus' => $connectionStatusArray));
            }
        }

        $this->_responseHelper->fail('');
    }

    /**
     * Tab import leads and organizations
     */
    public function crmImportLeadsAndOrganizationsAction()
    {
        $currentUserRole = $this->_sessionHelper->getCurrentUser()->getRoleId();
        if ($currentUserRole === Tools_Security_Acl::ROLE_ADMIN || $currentUserRole === Tools_Security_Acl::ROLE_SUPERADMIN) {
            $importDataDbTable = new Leads_DbTable_LeadsImportDataDbtable();
            $this->_view->logExists = false;
            $this->_view->fullLogExists = false;
            try {
                $result = $importDataDbTable->getAdapter()->describeTable(Tools_LeadImportTools::PLUGIN_LEADS_IMPORT_TABLE);
                if (!empty($result)) {
                    $importErrorsData = Tools_LeadImportTools::getImportByStatusCode(Tools_LeadImportTools::IMPORT_LEADS_ERROR_CODE,
                        true);
                    if (!empty($importErrorsData)) {
                        $this->_view->logExists = true;
                    }

                    $importFullData = Tools_LeadImportTools::getImportByStatusCode(Tools_LeadImportTools::GET_LEADS_SUCCESS_CODE,
                        true);

                    if (!empty($importFullData)) {
                        $this->_view->fullLogExists = true;
                    }
                }
            } catch (Exception $e) {
                $this->_view->logExists = false;
                $this->_view->fullLogExists = false;
            }
            echo $this->_view->render('crm-lead-import-leads-and-organizations-config.phtml');
        }
    }

    /**
     * Get useful short codes for lead email tab
     */
    public function getLeadShortCodeAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $disableForSms = filter_var($this->_request->getParam('disableForSms', false), FILTER_SANITIZE_NUMBER_INT);
            $withQuoteWidgets = filter_var($this->_request->getParam('withQuoteWidgets', false), FILTER_SANITIZE_NUMBER_INT);
            if (!empty($withQuoteWidgets)) {
                $withQuoteWidgets = true;
            }
            $widgets = Tools_LeadTools::leadUsefulShortcuts($disableForSms, $withQuoteWidgets);
            $this->_responseHelper->success($widgets);
        }
        $this->_responseHelper->fail('Access not allowed');
    }

    public function getLeadCallStatusesAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $widgets = array();
            array_push($widgets, array('alias' => 'No Answer', 'option' => 'no-answer'));
            array_push($widgets, array('alias' => 'Left Voicemail', 'option' => 'left-voicemail'));
            array_push($widgets, array('alias' => 'Bad Number', 'option' => 'bad-number'));
            $this->_responseHelper->success($widgets);
        }
        $this->_responseHelper->fail('Access not allowed');
    }

    /**
     * upload social service image action
     */
    public function uploadSocialServiceImageAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $this->_validateToken();
            $miscConfig = Zend_Registry::get('misc');
            $websitePath = $this->_websiteHelper->getPath();
            $uploadHandler = new Zend_File_Transfer_Adapter_Http();

            $uploadHandler->clearValidators()
                ->addValidator('Extension', false, array('jpeg', 'jpg', 'png', 'gif'))
                ->addValidator('ImageSize', false,
                    array('maxwidth' => $miscConfig['imgMaxWidth'], 'maxheight' => $miscConfig['imgMaxWidth']));

            if (extension_loaded('fileinfo')) {
                $uploadHandler->addValidator(new Validators_MimeType(array(
                    'image/gif',
                    'image/jpeg',
                    'image/jpg',
                    'image/png'
                )), false);
            }

            $socialServicePath = $websitePath . self::SOCIAL_SERVICE_IMAGES_PATH;

            if ($uploadHandler->isUploaded() && $uploadHandler->isValid()) {
                if (!is_dir($socialServicePath)) {
                    try {
                        Tools_Filesystem_Tools::mkDir($socialServicePath);
                    } catch (Exceptions_SeotoasterException $e) {
                        error_log($e->getMessage());
                    }
                }
                if (!$uploadHandler->hasFilter('Rename')) {
                    $filterChain = new Zend_Filter();
                    $filterChain->addFilter(new Zend_Filter_StringTrim())
                        ->addFilter(new Zend_Filter_StringToLower('UTF-8'))
                        ->addFilter(new Zend_Filter_PregReplace(array('match' => '/[^\w\d_]+/u', 'replace' => '-')));

                    $fileInfo = current($uploadHandler->getFileInfo());
                    $fullFileName = $fileInfo['name'];
                    $fileName = pathinfo($fullFileName, PATHINFO_FILENAME);
                    $fileExt = pathinfo($fullFileName, PATHINFO_EXTENSION);
                    if (!in_array($fileExt, array('jpeg', 'jpg', 'png', 'gif'))) {
                        $this->_responseHelper->fail("Wrong file extension");
                    }

                    $newName = $filterChain->filter($fileName);

                    if (false !== ($newName)) {
                        $uploadHandler->addFilter('Rename', array(
                            'target' => $socialServicePath . DIRECTORY_SEPARATOR . $newName . '.' . $fileExt,
                            'overwrite' => true
                        ));
                    } else {
                        $this->_responseHelper->fail("Wrong file name");
                    }
                }
                if ($uploadHandler->receive()) {
                    $this->_responseHelper->success($this->_translator->translate(self::TRANSLATION_PREFIX.'Image has been uploaded'));
                } else {
                    $this->_responseHelper->fail($uploadHandler->getMessages());
                }
            }

        }
        $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'System error'));
    }

    /**
     * upload organization logo action
     */
    public function uploadOrganizationLogoAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $this->_validateToken();
            $websitePath = $this->_websiteHelper->getPath();
            $uploadHandler = new Zend_File_Transfer_Adapter_Http();

            $uploadHandler->clearFilters();
            $uploadHandler->clearValidators();
            $fileInfo = $uploadHandler->getFileInfo();
            $organizationId = filter_var($this->_request->getParam('organizationId', false), FILTER_SANITIZE_NUMBER_INT);

            if (!empty($fileInfo) && !empty($organizationId)) {
                $file = reset($fileInfo);

                if (!empty($file)) {
                    $storedData = Tools_LeadDocumentsTools::generateStoredName($file);

                    if (isset($storedData['error'])) {
                        $this->_responseHelper->fail($storedData['error']);
                    }

                    $organizationLogosPath = $websitePath . self::ORGANIZATION_LOGOS_IMAGES_PATH;

                    $uploadHandler->addFilter('Rename', array(
                        'target' => $organizationLogosPath . DIRECTORY_SEPARATOR . $storedData['fileStoredName'],
                        'overwrite' => true
                    ));

                    $acceptedFileTypes = array(
                        'jpeg',
                        'jpg',
                        'png',
                        'gif'
                    );

                    $acceptedMimeTypes = array(
                        'image/gif',
                        'image/jpeg',
                        'image/jpg',
                        'image/png'
                    );

                    //Adding file extension validation
                    $uploadHandler->addValidator('Extension', false, $acceptedFileTypes);
                    //Adding mime types validation
                    $uploadHandler->addValidator('MimeType', true, $acceptedMimeTypes);

                    if ($uploadHandler->isUploaded() && $uploadHandler->isValid()) {
                        try {
                            $uploadHandler->receive();

                            $leadsOrganizationDocumentsMapper = Leads_Mapper_LeadsOrganizationDocumentsMapper::getInstance();

                            $this->_removeOrganizationLogo($organizationId);

                            $leadsOrganizationDocumentsModel = new Leads_Model_LeadsOrganizationDocumentsModel();
                            $leadsOrganizationDocumentsModel->setOrganizationId($organizationId);
                            $leadsOrganizationDocumentsModel->setFileStoredName($storedData['fileStoredName']);
                            $leadsOrganizationDocumentsModel->setFileHash($storedData['fileHash']);
                            $leadsOrganizationDocumentsModel->setOriginalFileName($storedData['fileName'] . '.' . $storedData['fileExtension']);
                            $leadsOrganizationDocumentsModel->setDisplayFileName($storedData['fileName']);
                            $leadsOrganizationDocumentsModel->setUploadedAt(date(Tools_System_Tools::DATE_MYSQL));

                            $leadsOrganizationDocumentsMapper->save($leadsOrganizationDocumentsModel);

                            $this->_responseHelper->success(array('fileStoredName' => $storedData['fileStoredName']));

                        } catch (Exceptions_SeotoasterException $e) {
                            $this->_error($e->getMessage());
                        }
                    } else {
                        $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'File type not supported'));
                    }
                }
            }
        }
    }

    public function removeOrganizationLogoAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $this->_validateToken();

            $organizationId = filter_var($this->_request->getParam('organizationId', false), FILTER_SANITIZE_NUMBER_INT);

            if(!empty($organizationId)) {
              $result = $this->_removeOrganizationLogo($organizationId);

              if(!empty($result)) {
                  $this->_responseHelper->success($this->_translator->translate(self::TRANSLATION_PREFIX.'Deleted'));
              }
            }
        }

        $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Can\'t delete logo'));
    }

    /**
     * @param $organizationId
     * @return |null
     */
    protected function _removeOrganizationLogo($organizationId)
    {
        $websitePath = $this->_websiteHelper->getPath();
        $organizationLogosPath = $websitePath . self::ORGANIZATION_LOGOS_IMAGES_PATH;

        $leadsOrganizationDocumentsMapper = Leads_Mapper_LeadsOrganizationDocumentsMapper::getInstance();

        $existedDocuments = $leadsOrganizationDocumentsMapper->findDocumentsByOrganizationId($organizationId);

        if(!empty($existedDocuments)) {
            foreach ($existedDocuments as $document) {
                $filePath = $organizationLogosPath . DIRECTORY_SEPARATOR . $document['fileStoredName'];

                if(is_file($filePath)) {
                    unlink($filePath);
                }
              return $leadsOrganizationDocumentsMapper->delete($document['id']);
            }
        }
        return null;
    }

    /**
     * log last viewed organization or lead action
     */
    public function logLastViewedAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT) && $this->_request->isPost()) {
            $this->_validateToken();

            $data = filter_var_array($this->_request->getParams(), FILTER_SANITIZE_STRING);

            if (empty($data['id']) || empty($data['type'])) {
                $this->_responseHelper->fail('missing mandatory attributes');
            }

            $type = $data['type'];

            if (!in_array($type, Leads_Model_LeadsLastViewedModel::$_allowedLeadLastViewed)) {
                $this->_responseHelper->fail('wrong type');
            }

            $userId = $this->_sessionHelper->getCurrentUser()->getId();
            $viewedAt = Tools_System_Tools::convertDateFromTimezone('now');
            $id = $data['id'];

            $lastViewedMapper = Leads_Mapper_LeadsLastViewedMapper::getInstance();
            $lastViewedModel = $lastViewedMapper->findBy($id, $type, $userId);
            if ($lastViewedModel instanceof Leads_Model_LeadsLastViewedModel) {
                $lastViewedMapper->delete($id, $type, $userId);
            }
            $lastViewedModel = new Leads_Model_LeadsLastViewedModel();
            $lastViewedModel->setId($data['id']);
            $lastViewedModel->setUserId($userId);
            $lastViewedModel->setLeadType($type);
            $lastViewedModel->setViewedAt($viewedAt);

            $lastViewedMapper->save($lastViewedModel);
        }
    }

    /**
     * Upload leads or organizations csv before import
     */
    public function uploadLeadsOrganizationsCsvAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT) && $this->_request->isPost()) {
            $this->_validateToken();
            $importType = $this->_request->getParam('importType', false);
            $importUpdateFieldsConfig = $this->_request->getParam('updateFields', false);
            $importConcatFieldsConfig = $this->_request->getParam('concatFields', false);
            $useExtendedConfiguration = $this->_request->getParam('useExtendedConfiguration', false);
            $isImportContactsData = filter_var($this->_request->getParam('isImportContactsData', false), FILTER_VALIDATE_BOOLEAN);

            $uploader = new Zend_File_Transfer_Adapter_Http();
            $csvInfo = $uploader->getFileInfo();

            if (!$uploader->isValid()) {
                $this->_responseHelper->fail('');
            }
            if (!in_array($importType, Tools_LeadImportTools::$_allowedImportTypes)) {
                $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Wrong import type.'));
            }

            unset($this->_sessionHelper->importUpdateFieldsConfig, $this->_sessionHelper->importConcatFieldsConfig, $this->_sessionHelper->useExtendedConfiguration);

            if(!empty($useExtendedConfiguration)){
                $this->_sessionHelper->useExtendedConfiguration = $useExtendedConfiguration;

                if(!empty($importUpdateFieldsConfig)){
                    $this->_sessionHelper->importUpdateFieldsConfig = $importUpdateFieldsConfig;
                }
                if(!empty($importConcatFieldsConfig)){
                    $this->_sessionHelper->importConcatFieldsConfig = $importConcatFieldsConfig;
                }
            }

            $csvInfo['importType'] = $importType;
            $currentLoggedUser = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser();
            if ($isImportContactsData && $currentLoggedUser instanceof Application_Model_Models_User) {
                $csvData = Tools_LeadImportTools::putCsvToDatabase($csvInfo, $currentLoggedUser->getEmail());
            } else {
                $csvData = Tools_LeadImportTools::putCsvToDatabase($csvInfo);
            }
            if ($csvData['error'] === 0) {
                $this->_responseHelper->success(array('quantity' => $csvData['quantity']));
            } else {
                $this->_responseHelper->fail($csvData['errorMessage']);
            }
        }
    }

    /**
     * Download sample file with import data for leads or organizations
     */
    public function getSampleImportDataAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $pluginPath = Tools_Plugins_Tools::getPluginsPath();
            $sampleType = $this->_request->getParam('type');
            if (!in_array($sampleType, Tools_LeadImportTools::$_allowedImportTypes)) {
                exit;
            }

            if ($sampleType === Tools_LeadImportTools::SAMPLE_TYPE_LEAD) {
                $fileName = Tools_LeadImportTools::SAMPLE_LEAD_IMPORT_FILE_NAME;
            }

            if ($sampleType === Tools_LeadImportTools::SAMPLE_TYPE_ORGANIZATION) {
                $fileName = Tools_LeadImportTools::SAMPLE_ORGANIZATION_IMPORT_FILE_NAME;
            }

            if($sampleType === Tools_LeadImportTools::SAMPLE_TYPE_LEAD_OPTION){
                $fileName = Tools_LeadImportTools::SAMPLE_LEAD_IMPORT_OPTION_FILE_NAME;
            }

            if($sampleType === Tools_LeadImportTools::SAMPLE_TYPE_ORGANIZATION_OPTION){
                $fileName = Tools_LeadImportTools::SAMPLE_ORGANIZATION_IMPORT_OPTION_FILE_NAME;
            }

            if ($sampleType === Tools_LeadImportTools::SAMPLE_TYPE_LEAD_TASKS) {
                $fileName = Tools_LeadImportTools::SAMPLE_LEAD_TASK_IMPORT_FILE_NAME;
            }

            if ($sampleType === Tools_LeadImportTools::SAMPLE_TYPE_CONTACTS_DATA) {
                $fileName = Tools_LeadImportTools::SAMPLE_CONTACTS_DATA_IMPORT_FILE_NAME;
            }

            if(!empty($fileName)){
                $filePath = $pluginPath . Tools_LeadImportTools::SAMPLE_FILE_IMPORT_DIR . $fileName;
                if (file_exists($filePath) && is_readable($filePath)) {
                    Tools_LeadImportTools::downloadCsv($filePath, pathinfo($fileName, PATHINFO_FILENAME));
                }
            }
        }
        exit;
    }

    /**
     * Import leads or organizations action
     */
    public function importLeadsAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT) && $this->_request->isPost()) {
            $this->_validateToken();
            $importType = $this->_request->getParam('importType', false);
            $isImportContactsData = filter_var($this->_request->getParam('isImportContactsData', false), FILTER_VALIDATE_BOOLEAN);

            if (!in_array($importType, Tools_LeadImportTools::$_allowedImportTypes)) {
                $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Wrong import type.'));
            }

            $leadsImportLimit = Tools_LeadImportTools::PLUGIN_LEADS_IMPORT_LIMIT;

            if ($importType === Tools_LeadImportTools::SAMPLE_TYPE_ORGANIZATION) {
                $importStatusInfo = Tools_LeadImportTools::processOrganizationsData();
            }

            if ($importType === Tools_LeadImportTools::SAMPLE_TYPE_LEAD) {
                $currentLoggedUser = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser();
                if ($isImportContactsData && $currentLoggedUser instanceof Application_Model_Models_User) {
                    $importStatusInfo = Tools_LeadImportTools::processLeadsData($currentLoggedUser->getId());
                } else {
                    $importStatusInfo = Tools_LeadImportTools::processLeadsData();
                }
            }

            if ($importType === Tools_LeadImportTools::SAMPLE_TYPE_LEAD_TASKS) {
                $importStatusInfo = Tools_LeadImportTools::processLeadTasksData();
            }

            if ($importType === Tools_LeadImportTools::SAMPLE_TYPE_LEAD_OPTION || $importType === Tools_LeadImportTools::SAMPLE_TYPE_ORGANIZATION_OPTION) {
                $importStatusInfo = Tools_LeadImportTools::processLeadsOrOrganizationsOptionsData($importType);
                if($importStatusInfo['error'] == 1){
                    $this->_responseHelper->fail('A unique name is missing in the file');
                }
                if(!empty($importStatusInfo['importedCount'])){
                    $leadsImportLimit = $importStatusInfo['importedCount'];
                }
            }

            if (!empty($importStatusInfo['error']) && $importStatusInfo['error'] === '1') {
                $this->_responseHelper->fail(array('generalError' => $importStatusInfo['message']));
            }

            $importedQuantity = $importStatusInfo['importedCount'];

            if ($importedQuantity == $leadsImportLimit) {
                $this->_responseHelper->success(array('importedQuantity' => $importedQuantity));
            } else {
                $importErrors = Tools_LeadImportTools::getImportByStatusCode(Tools_LeadImportTools::IMPORT_LEADS_ERROR_CODE,
                    true);
                $message = $this->_translator->translate(self::TRANSLATION_PREFIX.'Import finished');
                if (!empty($importErrors)) {
                    $message .= ' <a href="' . $this->_websiteUrl . 'plugin/leads/run/downloadImportLog/">' . $this->_translator->translate(self::TRANSLATION_PREFIX.'Import contains some errors. Click to download report.') . '</a>';
                    if ($isImportContactsData) {
                        $this->_responseHelper->fail(array('importedQuantity' => $importedQuantity, 'generalError' => $message, 'messageLink' => $this->_websiteUrl . 'plugin/leads/run/downloadImportLog/'));
                    } else {
                        $this->_responseHelper->fail(array('generalError' => $message));
                    }
                }
                if ($isImportContactsData) {
                    $this->_responseHelper->fail(array('importedQuantity' => $importedQuantity, 'message' => $message));
                } else {
                    $this->_responseHelper->fail($message);
                }

            }


        }
    }

    /**
     * download import log
     */
    public function downloadImportLogAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $importLogType = $this->_request->getParam('importLogType', self::IMPORT_LOG_TYPE_ERROR);
            $cacheHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('Cache');
            $cacheHelper->clean();
            $importDataDbTable = new Leads_DbTable_LeadsImportDataDbtable();
            $headers = $importDataDbTable->info(Zend_Db_Table_Abstract::COLS);
            unset($headers[0]);
            unset($headers[1]);
            unset($headers[2]);

            $fileName = 'import-log-.'. $importLogType.'-'. date("Y-m-d", time()) . '.csv';
            $filePath = $this->_websiteHelper->getPath() . $this->_websiteHelper->getTmp() . $fileName;
            $expFile = fopen($filePath, 'w');
            if ($importLogType === self::IMPORT_LOG_TYPE_FULL) {
                unset($headers[3]);
                unset($headers[5]);
            } else {
                unset($headers[4]);
                unset($headers[5]);
            }
            fputcsv($expFile, $headers, ',', '"');

            if ($importLogType === self::IMPORT_LOG_TYPE_ERROR) {
                $importErrorData = Tools_LeadImportTools::getImportByStatusCode(Tools_LeadImportTools::IMPORT_LEADS_ERROR_CODE,
                    false);
            } elseif ($importLogType === self::IMPORT_LOG_TYPE_FULL) {
                $importSuccessData = Tools_LeadImportTools::getImportByStatusCode(Tools_LeadImportTools::GET_LEADS_SUCCESS_CODE,
                    false);
            } else {
                $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Wrong import type.'));
            }

            if (!empty($importErrorData)) {
                foreach ($importErrorData as $errorData) {
                    unset($errorData['id']);
                    unset($errorData['import_status']);
                    unset($errorData['import_id']);
                    unset($errorData['import_url']);
                    unset($errorData['import_type']);
                    $errorCode = $errorData['error_code'];
                    $errorData['error_code'] = Tools_LeadImportTools::$_errorTypes[$errorCode];
                    fputcsv($expFile, $errorData, ',', '"');
                }
            } elseif(!empty($importSuccessData)) {
                foreach ($importSuccessData as $data) {
                    unset($data['id']);
                    unset($data['import_status']);
                    unset($data['error_code']);
                    if ($data['import_type'] === Tools_LeadImportTools::SAMPLE_TYPE_LEAD) {
                        $data['import_url'] = $this->_websiteUrl . 'dashboard/leads/#lead/' . $data['import_id'];
                    }
                    if ($data['import_type'] === Tools_LeadImportTools::SAMPLE_TYPE_ORGANIZATION) {
                        $data['import_url'] = $this->_websiteUrl . 'dashboard/organizations/#organization/' . $data['import_id'];
                    }
                    unset($data['import_id']);
                    unset($data['import_type']);
                    fputcsv($expFile, $data, ',', '"');
                }
            }
            fclose($expFile);
            Tools_LeadImportTools::downloadCsv($filePath, $fileName);

        }
    }

    private function _DomRemove(DOMNode $from)
    {
        $sibling = $from->firstChild;
        do {
            $next = $sibling->nextSibling;
            $from->parentNode->insertBefore($sibling, $from);
        } while ($sibling = $next);
        $from->parentNode->removeChild($from);
    }

    /**
     *
     * Send email from dashboard for lead
     *
     */
    public function sendEmailAction()
    {

        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $this->_validateToken();
            $leadId = filter_var($this->_request->getParam('leadId', false), FILTER_SANITIZE_NUMBER_INT);
            if (empty($leadId)) {
                $this->_responseHelper->response($this->_translator->translate(self::TRANSLATION_PREFIX.'Lead id missing'), 1, 400);
            }

            $leadEmailTemplate = $this->_request->getParam('lead-email-template', false);
            if (empty($leadEmailTemplate)) {
                $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'missed email template'));
            }

            $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
            $leadModel = $leadMapper->find($leadId);
            if (!$leadModel instanceof Leads_Model_LeadsModel) {
                $this->_responseHelper->fail('Lead doesn\'t exists');
            }

            $emailFrom = filter_var($this->_request->getParam('lead-email-from', false), FILTER_SANITIZE_STRING);
            $emailsTo = array_unique(array_map('trim',
                explode(',', filter_var($this->_request->getParam('lead-email-to', false), FILTER_SANITIZE_STRING))));

            $testEmailFlag = $this->_request->getParam('testEmail');

            if(!empty($testEmailFlag)){
                $currentUserEmail = $this->_sessionHelper->getCurrentUser()->getEmail();
                $emailsTo[0] = $currentUserEmail;
            }

            $subject = trim(filter_var($this->_request->getParam('lead-email-subject', false)), FILTER_SANITIZE_STRING);
            $emailContent = $this->_request->getParam('emailContent', false);
            $emailTemplate = $this->_request->getParam('lead-email-template', false);
            $emailSequenceId =  filter_var($this->_request->getParam('lead-email-follow-up-sequences', false), FILTER_SANITIZE_NUMBER_INT);
            $bccEmails = filter_var($this->_request->getParam('lead-email-bcc-to', false), FILTER_SANITIZE_STRING);

            if (empty($subject)) {
                $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Please specify email subject'));
            }

            if (preg_match("~email-default-message-template~ui", $emailContent)) {
                $doc = new DOMDocument();
                /**
                 * LIBXML_HTML_NOIMPLIED turns off the automatic adding of implied html/body elements
                 * LIBXML_HTML_NODEFDTD prevents a default doctype being added when one is not found.
                 */
                $doc->loadHTML(mb_convert_encoding('<html><body>'.$emailContent.'</body></html>', 'HTML-ENTITIES', 'UTF-8'), LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD);
                $xpath = new DOMXpath($doc);
                $div = $xpath->query("//*[@id='email-default-message-template']")->item(0);
                $this->_DomRemove($div);

                $emailContent = strtr($doc->saveHTML(), array('<html>' => '', '<body>'  => '', '</body>'  => '', '</html>' => ''));
                $emailContent = str_replace('%7B%24', '{$', $emailContent);
                $emailContent = str_replace('%7D', '}', $emailContent);
            }
            if (!empty($bccEmails)) {
                $bccEmails = array_filter(array_unique(array_map('trim', explode(',', $bccEmails))));
            }

            $emailValidate = new Tools_System_CustomEmailValidator();

            if (!$emailValidate->isValid($emailFrom)) {
                $this->_responseHelper->fail('Not valid email from');
            }

            if (!empty($bccEmails)) {
                foreach ($bccEmails as $bccEmail) {
                    if (!$emailValidate->isValid($bccEmail)) {
                        $this->_responseHelper->fail('Not valid bcc email');
                    }
                }
            }

            foreach ($emailsTo as $email) {
                if (!$emailValidate->isValid($email)) {
                    $this->_responseHelper->fail('Not valid email to');
                }
            }

            if (!empty($emailSequenceId) && empty($testEmailFlag)) {
                $emailSequenceMapper = Emailsequence_Models_Mapper_EmailSequenceSettingsMapper::getInstance();
                $emailSequenceModel = $emailSequenceMapper->find($emailSequenceId);
                $sequenceAppliedDate = date(Tools_System_Tools::DATE_MYSQL, strtotime('now'));
                if ($emailSequenceModel instanceof Emailsequence_Models_Models_EmailSequenceSettingsModel) {
                    $leadHasEmailSequenceMapper = Leads_Mapper_LeadsHasEmailSequenceMapper::getInstance();
                    $leadHasEmailSequenceModel = new Leads_Model_LeadsHasEmailSequenceModel();
                    $leadHasEmailSequenceModel->setLeadId($leadId);
                    $leadHasEmailSequenceModel->setSequenceId($emailSequenceId);
                    $leadHasEmailSequenceModel->setSequenceDate($sequenceAppliedDate);
                    $leadHasEmailSequenceMapper->save($leadHasEmailSequenceModel);
                }

            }

            $leadTypeModel = Leads_Mapper_LeadsTypeMapper::getInstance()->findByTypeName(Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_EMAIL);
            if ($leadTypeModel instanceof Leads_Model_LeadsTypeModel && empty($testEmailFlag)) {
                $leadModel->setLastActivityTypeId($leadTypeModel->getId());
                $leadModel->setLastActivitySubtype('');
                $leadModel->setLastActivityOutbound(Tools_System_Tools::convertDateFromTimezone('now'));
                $leadModel = $leadMapper->save($leadModel);
            }

            $userId = $this->_sessionHelper->getCurrentUser()->getId();

            foreach ($emailsTo as $userEmail) {
                $emailTriggerData = array(
                    'trigger' => Tools_LeadsMailWatchdog::LEADS_CONTACT,
                    'mailMessage' => $emailContent,
                    'from' => $emailFrom,
                    'emailTo' => $userEmail,
                    'subject' => $subject,
                    'userId' => $userId,
                    'template' => $emailTemplate,
                    'bccEmails' => $bccEmails,
                    'defaultMessageTemplate' => true,
                    'withoutTracking' => !empty($testEmailFlag),
                    'skipBcc' => true,
                    'emailsTo' => $emailsTo
                );

                if ($emailTemplate === 'default') {
                    unset($emailTriggerData['template']);
                }

                $leadModel->registerObserver(new Tools_Mail_Watchdog($emailTriggerData));
            }

            if (!empty($bccEmails) && empty($testEmailFlag)) {
                $bccExistedLeads = array();

                foreach ($bccEmails as $key => $email) {
                   $isLeadExist = $leadMapper->findByEmail($email);

                   if($isLeadExist instanceof Leads_Model_LeadsModel) {
                       $bccExistedLeads[] = array(
                           'leadEmail' => $isLeadExist->getLeadEmail(),
                           'leadModel' => $isLeadExist
                       );

                       unset($bccEmails[$key]);
                   }
                }

                if(!empty($bccEmails)) {
                    $mainBccEmail = array_shift($bccEmails);
                    $emailTriggerData = array(
                        'trigger' => Tools_LeadsMailWatchdog::LEADS_CONTACT,
                        'mailMessage' => $emailContent,
                        'from' => $emailFrom,
                        'emailTo' => $mainBccEmail,
                        'subject' => $subject,
                        'userId' => $userId,
                        'template' => $emailTemplate,
                        'bccEmails' => $bccEmails,
                        'withoutTracking' => true,
                        'defaultMessageTemplate' => true
                    );

                    if ($emailTemplate === 'default') {
                        unset($emailTriggerData['template']);
                    }

                    $leadModel->registerObserver(new Tools_Mail_Watchdog($emailTriggerData));
                }
            }

            $leadModel->notifyObservers();

            if(!empty($bccExistedLeads)) {
                foreach ($bccExistedLeads as $bccLead) {
                    $emailTriggerData = array(
                        'trigger' => Tools_LeadsMailWatchdog::LEADS_CONTACT,
                        'mailMessage' => $emailContent,
                        'from' => $emailFrom,
                        'emailTo' => $bccLead['leadEmail'],
                        'subject' => $subject,
                        'userId' => $userId,
                        'template' => $emailTemplate,
                        'bccEmails' => array(),
                        'withoutTracking' => false,
                        'defaultMessageTemplate' => true
                    );

                    if ($emailTemplate === 'default') {
                        unset($emailTriggerData['template']);
                    }

                    $bccLead['leadModel']->registerObserver(new Tools_Mail_Watchdog($emailTriggerData));

                    $bccLead['leadModel']->notifyObservers();
                }
            }

            $this->_responseHelper->success($this->_translator->translate(self::TRANSLATION_PREFIX.'Email has been sent.'));
        }
    }

    /**
     * Log manual call
     */
    public function manualCallLeadAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $this->_validateToken();
            $leadId = filter_var($this->_request->getParam('leadId', false), FILTER_SANITIZE_NUMBER_INT);
            if (empty($leadId)) {
                $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Lead id missing'));
            }

            $callDuration = filter_var($this->_request->getParam('call-duration-manual'), FILTER_SANITIZE_NUMBER_INT);
            if (empty($callDuration)) {
                $callDuration = filter_var($this->_request->getParam('call-duration-dropdown'), FILTER_SANITIZE_NUMBER_INT);
            }

            if (empty($callDuration)) {
                $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Please specify call duration'));
            }

            $callNotes = filter_var($this->_request->getParam('call-notes'), FILTER_SANITIZE_STRING);

            $receiverPhoneNumber = filter_var($this->_request->getParam('lead-call-number', ''),
                FILTER_SANITIZE_STRING);
            $receiverCountryCode = filter_var($this->_request->getParam('lead-call-country-code', ''),
                FILTER_SANITIZE_STRING);

            $countryPhoneCode = '+'.Zend_Locale::getTranslation($receiverCountryCode, 'phoneToTerritory');

            $receiverPhoneNumber = $countryPhoneCode.$receiverPhoneNumber;

            $receiverPhoneNumber = Apps_Tools_Twilio::normalizePhoneNumberToE164($receiverPhoneNumber);
            $callFromDeviceType = filter_var($this->_request->getParam('call-from-device-type'),
                FILTER_SANITIZE_STRING);
            $userId = $this->_sessionHelper->getCurrentUser()->getId();

            $storeAdminUserModel = Application_Model_Mappers_UserMapper::getInstance()->find($userId);
            if (!$storeAdminUserModel instanceof Application_Model_Models_User) {
                $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'user not authorized'));
            }

            $callerPhoneNumber = null;
            if ($callFromDeviceType === self::LEAD_CALL_DEVICE_TYPE_MOBILE) {
                $callerPhoneNumber = $storeAdminUserModel->getMobileCountryCodeValue().$storeAdminUserModel->getMobilePhone();
            }

            if ($callFromDeviceType === self::LEAD_CALL_DEVICE_TYPE_DESKTOP) {
                $callerPhoneNumber = $storeAdminUserModel->getDesktopCountryCodeValue().$storeAdminUserModel->getDesktopPhone();
            }

            if ($callFromDeviceType !== self::LEAD_CALL_DEVICE_TYPE_VOIP_PHONE) {
                $callerPhoneNumber = Apps_Tools_Twilio::normalizePhoneNumberToE164($callerPhoneNumber);
                $voipPhoneFlag = '0';
            } else {
                $callerPhoneNumber = $storeAdminUserModel->getVoipPhone();
                $voipPhoneFlag = '1';
            }

            if (empty($callerPhoneNumber)) {
                $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Wrong admin number'));
            }

            if (empty($receiverPhoneNumber)) {
                $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Wrong lead phone number'));
            }

            $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
            $leadModel = $leadMapper->find($leadId);
            $type = Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND;
            $leadType = Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_CALL;
            $createdAt = Tools_System_Tools::convertDateFromTimezone('now');
            $leadTypeId = Tools_LeadTools::getLeadTypeIdByLeadTypeName($leadType);

            if ($leadModel instanceof Leads_Model_LeadsModel) {
                if (!empty($leadTypeId)) {
                    $leadModel->setLastActivityTypeId($leadTypeId);
                    $leadModel->setLastActivitySubtype('');
                    if($type == Leads_Model_LeadsActionLogModel::ACTION_TYPE_INBOUND){
                        $leadModel->setLastActivity($createdAt);
                    }else{
                        $leadModel->setLastActivityOutbound($createdAt);
                    }
                    $leadMapper->save($leadModel);
                }

                $leadId = $leadModel->getId();
                $actionLedLogModel = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_CALL,
                    $leadId, $type, $createdAt, $leadType);
                $callLogMapper = Leads_Mapper_LeadsCallLogMapper::getInstance();
                $callLogModel = new Leads_Model_LeadsCallLogModel();
                $callLogModel->setActionLogId($actionLedLogModel->getId());
                $callLogModel->setType(Leads_Model_LeadsCallLogModel::TYPE_CALL);
                $callLogModel->setCallDuration($callDuration);
                $callLogModel->setStatus('completed');
                $callLogModel->setScore('valid');
                $callLogModel->setCreatorId($userId);
                $callLogModel->setDetails($callNotes);
                $callLogModel->setLeadId($leadId);
                $callLogModel->setIsVoip($voipPhoneFlag);
                $callLogModel->setCaller($receiverPhoneNumber);
                $callLogModel->setReceiver($callerPhoneNumber);
                $callLogModel->setTimestamp($createdAt);
                $callLogMapper->save($callLogModel);

                $this->_responseHelper->success($this->_translator->translate(self::TRANSLATION_PREFIX.'Call has been logged'));
            }

        }

        $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Lead not found'));
    }

    /**
     * Call lead action from lead dashboard
     */
    public function callLeadAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $this->_validateToken();
            $leadId = filter_var($this->_request->getParam('leadId', false), FILTER_SANITIZE_NUMBER_INT);
            if (empty($leadId)) {
                $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Lead id missing'));
            }

            $receiverPhoneNumber = filter_var($this->_request->getParam('lead-call-number', ''),
                FILTER_SANITIZE_STRING);
            $receiverCountryCode = filter_var($this->_request->getParam('lead-call-country-code', ''),
                FILTER_SANITIZE_STRING);

            $countryPhoneCode = '+'.Zend_Locale::getTranslation($receiverCountryCode, 'phoneToTerritory');

            $receiverPhoneNumber = $countryPhoneCode.$receiverPhoneNumber;

            $receiverPhoneNumber = Apps_Tools_Twilio::normalizePhoneNumberToE164($receiverPhoneNumber);
            $callFromDeviceType = filter_var($this->_request->getParam('call-from-device-type'),
                FILTER_SANITIZE_STRING);
            $userId = $this->_sessionHelper->getCurrentUser()->getId();

            $storeAdminUserModel = Application_Model_Mappers_UserMapper::getInstance()->find($userId);
            if (!$storeAdminUserModel instanceof Application_Model_Models_User) {
                $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'user not authorized'));
            }

            $callerPhoneNumber = null;
            if ($callFromDeviceType === self::LEAD_CALL_DEVICE_TYPE_MOBILE) {
                $callerPhoneNumber = $storeAdminUserModel->getMobileCountryCodeValue().$storeAdminUserModel->getMobilePhone();
            }

            if ($callFromDeviceType === self::LEAD_CALL_DEVICE_TYPE_DESKTOP) {
                $callerPhoneNumber = $storeAdminUserModel->getDesktopCountryCodeValue().$storeAdminUserModel->getDesktopPhone();
            }

            if ($callFromDeviceType !== self::LEAD_CALL_DEVICE_TYPE_VOIP_PHONE) {
                $callerPhoneNumber = Apps_Tools_Twilio::normalizePhoneNumberToE164($callerPhoneNumber);
                $voipPhoneFlag = '0';
            } else {
                $callerPhoneNumber = $storeAdminUserModel->getVoipPhone();
                $voipPhoneFlag = '1';
            }

            if (empty($callerPhoneNumber)) {
                $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Wrong admin number'));
            }

            if (empty($receiverPhoneNumber)) {
                $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Wrong lead phone number'));
            }

            $response = Apps::apiCall(
                'POST',
                'apps',
                array('twilioClickToCall'),
                array(
                    'callerPhoneNumber' => $receiverPhoneNumber,
                    'receiverPhoneNumber' => $callerPhoneNumber,
                    'customData' => array(
                        'leadId' => $leadId,
                        'creatorId' => $this->_sessionHelper->getCurrentUser()->getId(),
                        'isVoipPhone' => $voipPhoneFlag
                    )
                )
            );

            if (empty($response)) {
                $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Please open an account.'));
            }
            if ($response['twilioClickToCall']['done'] === false) {
                if(isset($response['twilioClickToCall']['serviceError'])){
                    $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'One of the phone numbers is unreachable at this time. Please double check phone numbers of the parties you\'re dialing'));
                }
                $this->_responseHelper->fail($response['twilioClickToCall']['message']);
            }

            $this->_responseHelper->success($this->_translator->translate(self::TRANSLATION_PREFIX.'Phone call'));
        }
    }

    /**
     * Get email tab additional data (images etc..)
     *
     * @throws Exceptions_SeotoasterException
     */
    public function getEmailTabDataAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $templates = Application_Model_Mappers_TemplateMapper::getInstance()->findByType('typemail');
            $leadId = filter_var($this->_request->getParam('lead_id'), FILTER_SANITIZE_NUMBER_INT);
            $templates = array_map(function ($model) {
                $modelData = $model->toArray();
                unset($modelData['content']);

                return $modelData;
            }, $templates);

            $templatesAndDirectoriesMapper = Leads_Mapper_LeadsTemplatesAndDirectoriesConfigMapper::getInstance();

            $savedConfigTemplates = $templatesAndDirectoriesMapper->findByType('template');

            if(!empty($savedConfigTemplates) && !empty($templates)){
                foreach ($templates as $tKey => $template){
                    if(!in_array($template['name'], $savedConfigTemplates)){
                       unset($templates[$tKey]);
                    }
                }
            }

            $listFolders = $this->_getFoldersList();
            $savedConfigFolders = $templatesAndDirectoriesMapper->findByType('folder');

            if(!empty($savedConfigFolders)){
                $listFolders = $savedConfigFolders;

                natcasesort($listFolders);
            }
            array_unshift($listFolders, $this->_translator->translate(self::TRANSLATION_PREFIX.'Select folder'));

            $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');

            $imagesConfigData = array(
                'small' => $configHelper->getConfig('imgSmall'),
                'medium' => $configHelper->getConfig('imgMedium'),
                'large' => $configHelper->getConfig('imgLarge')
            );

            $sequenceData = Tools_LeadTools::getEmailSequenceList();

            if (!empty($leadId)) {
                $sequenceInUse = Leads_Mapper_LeadsHasEmailSequenceMapper::getInstance()->getLeadSequenceByLeadId($leadId);
            }

            if (empty($sequenceInUse)) {
                $sequenceInUse = array();
            }
            $activeActionEmailTriggers = Tools_LeadTools::getDefaultActionEmailInfo(Tools_LeadsMailWatchdog::LEADS_CONTACT);
            $defaultSubject = '';
            $defaultEmailMessagePageUrl = '';
            if (!empty($activeActionEmailTriggers)) {
                $defaultSubject = $activeActionEmailTriggers['subject'];
                $templateId = $activeActionEmailTriggers['template'];
                $pageModel = Tools_LeadTools::getTemplatePage($templateId);
                if ($pageModel instanceof Application_Model_Models_Page) {
                    $defaultEmailMessagePageUrl = $pageModel->getUrl();
                }
            }

            echo json_encode(array(
                'templates' => $templates,
                'folders' => $listFolders,
                'imagesConfigData' => $imagesConfigData,
                'sequenceData' => $sequenceData,
                'sequenceInUse' => $sequenceInUse,
                'defaultSubject' => $defaultSubject,
                'defaultEmailMessagePageUrl' => $defaultEmailMessagePageUrl
            ));
        }
    }

    /**
     * Get states by country code
     */
    public function getStateByCountryCodeAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $countryCode = filter_var($this->_request->getParam('countryCode'), FILTER_SANITIZE_STRING);
            $this->_validateToken();

            if (!empty($countryCode)) {
                $statesData = Tools_LeadTools::getStates($countryCode, true, true);
                if (!empty($statesData)) {
                    $this->_responseHelper->success(array('statesList' => $statesData));
                }
            }
            $this->_responseHelper->fail('Wrong country code');
        }
    }

    /**
     * Get list of owners for tasks tab
     */
    public function getTaskTabDataAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $owners = Leads_Mapper_LeadsMapper::getInstance()->getLeadOwnersFullList();
            echo json_encode(array(
                'owners' => $owners,
            ));
        }
    }

    /**
     *  Get All leads count
     */
    public function getAllLeadsAction() {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            if ($this->_request->isGet()) {
                $allLeads = filter_var($this->_request->getParam('allLeads'), FILTER_SANITIZE_STRING);

                if(!empty($allLeads)) {
                    $allLeads = Tools_LeadTools::getSearchFilterData('', 1);

                    echo json_encode(array('totalRecords' => $allLeads['totalRecords']));
                    return;
                }
            }
        }
    }

    /**
     * @throws Zend_Db_Adapter_Exception
     */
    public function addTagsAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {

            if ($this->_request->isGet()) {
                $leadsIds = filter_var($this->_request->getParam('leadsIds'), FILTER_SANITIZE_STRING);
                $allLeadSelectedTags = array();

                $tagsMapper = Leads_Mapper_LeadsTagsMapper::getInstance();
                $leadTags = $tagsMapper->getTagsPairs();

                if (!empty($leadsIds)) {
                    $leadsIds = explode(',', $leadsIds);

                    foreach ($leadsIds as $leadId) {
                        $selectedLeadTags = $tagsMapper->findTagsByLeadId($leadId);

                        if (!empty($selectedLeadTags)) {
                            $allLeadSelectedTags = array_merge($allLeadSelectedTags, $selectedLeadTags);
                        }
                    }

                    $allLeadSelectedTags = array_flip($allLeadSelectedTags);
                }

                echo json_encode(array('tags' => $leadTags, 'selectedTags' => $allLeadSelectedTags));
                return;
            }

            if ($this->_request->isPost()) {
                $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
                $this->_validateToken();

                $leadIds = filter_var($this->_request->getParam('leadIds'), FILTER_SANITIZE_STRING);
                $tagsIds = filter_var_array($this->_request->getParam('tagsIds'), FILTER_SANITIZE_STRING);
                $tagsFunction = filter_var($this->_request->getParam('tagsFunction'), FILTER_SANITIZE_STRING);
                $allRecords = filter_var($this->_request->getParam('allRecords'), FILTER_SANITIZE_STRING);
                $tagsType = filter_var($this->_request->getParam('tagsType'), FILTER_SANITIZE_STRING);
                $filters = $this->_request->getParam('filters');
                $offset = 0;

                if(!empty($allRecords)) {
                    $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
                    $allLeads = Tools_LeadTools::getSearchFilterData('', self::PROCESSED_LEAD_TAGS_LIMIT, $offset);

                    $totalRecords = $allLeads['totalRecords'];
                    $leadsCount = 0;

                    if(!empty($allLeads['data'])) {
                        $leadIds = array();
                        $leadsCount = count($allLeads['data']);
                        foreach ($allLeads['data'] as $lead) {
                            $leadIds[] = $lead['id'];
                        }
                    }
                } elseif(!empty($tagsType)) {
                    $leadsData = Tools_LeadTools::getSearchFilterData($filters, null);

                    if(!empty($leadsData)){
                        $leadsIds = array();
                        foreach ($leadsData['data'] as $lead){
                            array_push($leadsIds, $lead['id']);
                        }
                        $leadIds = $leadsIds;
                    }
                }

                if (!empty($leadIds) && !empty($tagsFunction)) {
                    if(empty($allRecords) && empty($tagsType)) {
                        $leadsIds = explode(',', $leadIds);
                    } else {
                        $leadsIds =  $leadIds;
                    }

                    foreach ($leadsIds as $leadId) {
                        if($tagsFunction == 'reprocessing'){
                            if (!empty($tagsIds)) {
                                $leadMapper->processTags($leadId, $tagsIds);
                            } else {
                                $leadMapper->deleteTag($leadId, false);
                            }
                        } elseif (!empty($tagsIds)) {
                            foreach ($tagsIds as $tagId){
                                $leadMapper->addTag($leadId ,$tagId);
                            }
                        }
                    }

                    $this->_responseHelper->success(array('offset' => $offset + self::PROCESSED_LEAD_TAGS_LIMIT, 'totalRecords' => $totalRecords, 'leadsCount' => $leadsCount));
                }
                $this->_responseHelper->success($this->_translator->translate(self::TRANSLATION_PREFIX.'Done'));
            }
        }
    }

    /**
     * Multiple method for get and set email sequences
     */
    public function emailSequenceAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {

            //get email sequences list
            if($this->_request->isGet()){
                $emailSequenceMapper = Emailsequence_Models_Mapper_EmailSequenceSettingsMapper::getInstance();
                $emailSequences = $emailSequenceMapper->getSequencesPairs(true);
                echo json_encode(array(
                    'emailSequences' => $emailSequences,
                ));

                return;
            }

            //set email sequences for leads
            if($this->_request->isPost()) {
                $this->_validateToken();

                $sequenceId = filter_var($this->_request->getParam('leadEmailSequenceId'), FILTER_SANITIZE_NUMBER_INT);
                $leadIds = filter_var($this->_request->getParam('leadIds'), FILTER_SANITIZE_STRING);
                $status = false;
                $filters = $this->_request->getParam('filters');
                $allFilterLeads = $this->_request->getParam('allFilterLeads');
                $requestType = $this->_request->getParam('requestType');
                $offset = 0;

                if (!empty($sequenceId) && !empty($leadIds)) {
                    $leadIds = explode(',', $leadIds);
                    if (empty($leadIds)) {
                        $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX . 'Lead ids missing'));
                    }

                    if (!empty($allFilterLeads)) {
                        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
                        $leadsData = Tools_LeadTools::getSearchFilterData($filters, self::PROCESSED_LEAD_SEQUENCES_LIMIT, $offset);

                        $totalRecords = $leadsData['totalRecords'];
                        $leadsCount = 0;

                        if (!empty($leadsData)) {
                            $leadsIds = array();
                            $leadsCount = count($leadsData['data']);
                            foreach ($leadsData['data'] as $lead) {
                                array_push($leadsIds, $lead['id']);
                            }
                            $leadIds = $leadsIds;
                        }
                    }

                    if ($requestType === 'addnew') {
                        $status = Tools_LeadFormEventsTools::assignLeadSequenceMassAction($sequenceId, $leadIds);
                    } else {
                        $status = Tools_LeadFormEventsTools::unAssignLeadSequenceMassAction($sequenceId, $leadIds);
                    }

                    if($status){
                        if ($requestType === 'addnew') {
                            $message = $this->_translator->translate(self::TRANSLATION_PREFIX . 'sequencedone');
                        } else {
                            $message = $this->_translator->translate(self::TRANSLATION_PREFIX . 'Done');
                        }
                        $this->_responseHelper->success(array('message' => $message, 'offset' => $offset + self::PROCESSED_LEAD_SEQUENCES_LIMIT, 'totalRecords' => $totalRecords, 'leadsCount' => $leadsCount));
                    }
                }

                if ($requestType === 'addnew') {
                    $this->_responseHelper->success($this->_translator->translate(self::TRANSLATION_PREFIX . 'sequencedone'));
                } else {
                    $this->_responseHelper->success($this->_translator->translate(self::TRANSLATION_PREFIX . 'Done'));
                }
            }
        }
    }

    public function getExportLeadsTabDataAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $defaultLeadsExportConfig = Tools_LeadExportTools::getDefaultLeadExportConfig();
            $additionalLeadsExportConfig = Tools_LeadExportTools::getAdditionalLeadExportConfig();
            $additionalLeadsExportCustomParamsConfig = Tools_LeadExportTools::getAdditionalLeadsExportCustomParamsConfig();

            $this->_responseHelper->success(
                array(
                    'defaultConfig' => $defaultLeadsExportConfig,
                    'additionalLeadsExportConfig' => $additionalLeadsExportConfig,
                    'additionalLeadsExportCustomParamsConfig' => $additionalLeadsExportCustomParamsConfig
                )
            );
        }
    }

    public function exportLeadsAction()
    {
        $leadsIds = filter_var($this->_request->getParam('leadIds'), FILTER_SANITIZE_STRING);
        $data = $this->_request->getParams();
        $leadsIds = ($data['allLeads'] == 1 || empty($leadsIds)) ? array() : explode(',', $leadsIds);
        $useCustomParams = false;
        $disableUseCustomParamsPrefix = false;

        if(!empty($data['use-custom-params'])) {
            $useCustomParams = true;

            if(!empty($data['disable-custom-params-prefix'])) {
                $disableUseCustomParamsPrefix = true;
            }
        }

        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            if (!empty($leadsIds)) {
                Tools_LeadImportTools::prepareLeadsDataForExport($data, $leadsIds, $useCustomParams, $disableUseCustomParamsPrefix);
            } else {
                $leadsData = Tools_LeadTools::getSearchFilterData($data['filters'], null);

                if(!empty($leadsData)){
                    $leadsIds = array();
                    foreach ($leadsData['data'] as $lead){
                        array_push($leadsIds, $lead['id']);
                    }
                    $data['leadIds'] = implode(',' , $leadsIds);

                    Tools_LeadImportTools::prepareLeadsDataForExport($data, $leadsIds, $useCustomParams, $disableUseCustomParamsPrefix);
                }
            }
        }
    }

    public function getAllTagsFilterLeadsAction() {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $this->_validateToken();
            $filtersData = $this->_request->getParam('filters');
            $leadIdsData = explode(',', $this->_request->getParam('leadIds'));

            $allTagsFilterLeads = filter_var($this->_request->getParam('allTagsFilterLeads'), FILTER_SANITIZE_STRING);

            if(!empty($allTagsFilterLeads) && !empty($leadIdsData)) {
                $allTagsFilterLeads = Tools_LeadTools::getSearchFilterData($filtersData, 1);
                if(!empty($allTagsFilterLeads['totalRecords'])) {
                    echo json_encode(array('totalRecords' => $allTagsFilterLeads['totalRecords']));
                    return;
                }

                $this->_responseHelper->fail('');
            }
        }
    }

    public function countAllLeadsAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $this->_validateToken();
            $filtersData = $this->_request->getParam('filters');
            $leadIds= $this->_request->getParam('leadIds');
            $leadIdsData = explode(',', $this->_request->getParam('leadIds'));
            $uniqueEmail = $this->_request->getParam('uniqueEmail');
            $additionalParams = $this->_request->getParam('additionalParams');
            $additionalFiltersData = $filtersData;
            $emailStatus = $this->_request->getParam('emailStatus');
            $ignoreIds = $this->_request->getParam('ignoreIds');
            $getAdvertisingList = $this->_request->getParam('advertising');
            $existMobileNumber = $this->_request->getParam('existMobileNumber');
            $validateEmailBalance = $this->_request->getParam('validateEmailBalance');

            if (!empty($leadIdsData) && !empty($leadIds)) {
                if (empty($ignoreIds)) {
                    $filtersData = '&lead_ids=' . $leadIds;
                }
            }

            if (!empty($uniqueEmail)) {
                $filtersData .= '&not_empty_email_address=1';
                $additionalFiltersData .= '&not_empty_email_address=1';
            }
            if (!empty($emailStatus)) {
                $filtersData .= '&email_status='.$emailStatus;
            }

            if(!empty($additionalParams)) {
                $additionalFiltersData .= '&lead_ids='.$additionalParams;
            }

            $existMobileNumberFlag = false;
            if(!empty($existMobileNumber)) {
                $existMobileNumberFlag = true;
            }

            $quantityData = Tools_LeadTools::getSearchFilterData($filtersData, 1, null, true, true, false, $existMobileNumberFlag);

            $additionalQuantityData = Tools_LeadTools::getSearchFilterData($additionalFiltersData, 1, null, true, true, false, $existMobileNumberFlag);

            $advertisingList = array();
            if(!empty($getAdvertisingList)) {
                $advertisingList = Tools_LeadTools::$_advertisingTypes;

                foreach ($advertisingList as $advKey => $advAccount) {
                    $services = Tools_LeadTools::sambastatsConnectionStatusGet($advKey);

                    if (!empty($services['data'])) {
                        $adAccountId = $services['data']['adAccountId'];
                        if(empty($services['data']['connectionStatus'])) {
                            $adAccountId = 'disconnected';
                        }
                        $advertisingList[$advKey] .= ' (' . $adAccountId . ')';
                    }
                }

            }

            $leadGeneralConfigParams = Leads_Mapper_LeadsConfigMapper::getInstance()->getConfigParams();

            $emailValidateBalance = '0';
            $emailBalanceLink = '';
            if (!empty($validateEmailBalance)) {
                $response = Apps::apiCall('GET', 'appsValidationCredits', array(''), array('email' => $this->_sessionHelper->getCurrentUser()->getEmail()));
                if (empty($response['error'])) {
                    $emailValidateBalance = $response['balance'];
                    $emailBalanceLink = $response['link'];
                }
            }

            if(!empty($quantityData['totalRecords'])){
                $this->_responseHelper->success(array('quantity' => $quantityData['totalRecords'], 'additionalQuantity' => $additionalQuantityData['totalRecords'], 'advertisingList' => $advertisingList, 'leadGeneralConfigParams' => $leadGeneralConfigParams, 'emailValidateBalance' => $emailValidateBalance, 'emailBalanceLink' => $emailBalanceLink));
            }

            $this->_responseHelper->fail('');
        }
    }

    public function countAllOrganizationsAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $this->_validateToken();
            $filtersData = $this->_request->getParam('filters');
            $organizationIds= $this->_request->getParam('organizationIds');
            $organizationIdsData = explode(',', $this->_request->getParam('organizationIds'));
            $uniqueEmail = $this->_request->getParam('uniqueEmail');
            $additionalParams = $this->_request->getParam('additionalParams');
            $additionalFiltersData = $filtersData;
            $emailStatus = $this->_request->getParam('emailStatus');
            $ignoreIds = $this->_request->getParam('ignoreIds');
            $useMassAction = $this->_request->getParam('usemassaction');

            if (!empty($organizationIdsData) && !empty($organizationIds)) {
                if (empty($ignoreIds)) {
                    $filtersData = '&lead_ids=' . $organizationIds;
                }
            }

            if (!empty($uniqueEmail)) {
                $filtersData .= '&not_empty_email_address=1';
                $additionalFiltersData .= '&not_empty_email_address=1';
            }
            if (!empty($emailStatus)) {
                $filtersData .= '&email_status='.$emailStatus;
            }

            if(!empty($additionalParams)) {
                $additionalFiltersData .= '&lead_ids='.$additionalParams;
            }

            $organizationLeadsCount = 0;
            $allOrganizationIdsMachingFilter = array();
            $allOrganizatiosLeadsIdsMachingFilter = array();
            if(!empty($useMassAction)) {
                $quantityData = Tools_LeadTools::getSearchOrganizationFilterData($filtersData, null, null, true, true, false, false, true);

                if(!empty($quantityData['data'])) {
                    foreach ($quantityData['data'] as $lead) {
                        if(!empty($lead['countLeads'])) {
                            $organizationLeadsCount += $lead['countLeads'];
                            $allOrganizationIdsMachingFilter[] = $lead['id'];
                            $organizationLeadIds = explode(',' , $lead['organizationLeadIds']);
                            if(!empty($organizationLeadIds)) {
                                foreach($organizationLeadIds as $lId){
                                    if(!in_array($lId, $allOrganizatiosLeadsIdsMachingFilter)){
                                        array_push($allOrganizatiosLeadsIdsMachingFilter, $lId);
                                    }
                                }
                            }
                        }
                    }
                }

            } else {
                $quantityData = Tools_LeadTools::getSearchOrganizationFilterData($filtersData, 1, null, true, true, false);
            }
            $additionalQuantityData = Tools_LeadTools::getSearchOrganizationFilterData($additionalFiltersData, 1, null, true, true, false);

            if(!empty($quantityData['totalRecords'])){
                $this->_responseHelper->success(array(
                    'quantity' => $quantityData['totalRecords'],
                    'additionalQuantity' => $additionalQuantityData['totalRecords'],
                    'organizationLeadsCount' => $organizationLeadsCount,
                    'allOrganizationIdsMachingFilter' => $allOrganizationIdsMachingFilter,
                    'allOrganizatiosLeadsIdsMachingFilter' => $allOrganizatiosLeadsIdsMachingFilter,
                ));
            }

            $this->_responseHelper->fail('');
        }
    }

    public function remarketingOrganizationAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $getAdvertisingList = $this->_request->getParam('advertising');

            $organizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();

            if($this->_request->isGet()){
                $organizationIds = filter_var($this->_request->getParam('organizationIds'), FILTER_SANITIZE_STRING);
                if(!empty($organizationIds)) {
                    $organizationIds = explode(',', $organizationIds);

                    if (empty($organizationIds)) {
                        $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Organization ids missing'));
                    }

                    $countAllLeads = array();
                    $allOrganizatiosLeadsIdsMachingFilter = array();
                    foreach ($organizationIds as $orgId) {
                        $leadsOrganizations = $organizationMapper->getOrganizationLeads($orgId);

                        if(!empty($leadsOrganizations)) {
                            foreach ($leadsOrganizations as $lead) {
                                if(!in_array($lead['id'], $allOrganizatiosLeadsIdsMachingFilter)) {
                                    array_push($allOrganizatiosLeadsIdsMachingFilter, $lead['id']);
                                }
                            }
                            $countAllLeads = array_merge($countAllLeads, array_keys($leadsOrganizations));
                        }
                    }

                    $advertisingList = array();
                    if(!empty($getAdvertisingList)) {
                        $advertisingList = Tools_LeadTools::$_advertisingTypes;

                        foreach ($advertisingList as $advKey => $advAccount) {
                            $services = Tools_LeadTools::sambastatsConnectionStatusGet($advKey);

                            if (!empty($services['data'])) {
                                $adAccountId = $services['data']['adAccountId'];
                                if(empty($services['data']['connectionStatus'])) {
                                    $adAccountId = 'disconnected';
                                }
                                $advertisingList[$advKey] .= ' (' . $adAccountId . ')';
                            }
                        }

                    }

                    echo json_encode(array(
                        'countOrganizationLeads' => count($countAllLeads),
                        'advertisingList' => $advertisingList,
                        'allOrganizatiosLeadsIdsMachingFilter' => $allOrganizatiosLeadsIdsMachingFilter
                    ));
                }

                return;
            }
        }
    }

    /**
     * Multiple method for get and set email sequences in organization
     */
    public function emailSequenceOrganizationAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {

            $organizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();

            //get email sequences list
            if($this->_request->isGet()){
                $organizationIds = filter_var($this->_request->getParam('organizationIds'), FILTER_SANITIZE_STRING);
                if(!empty($organizationIds)) {
                    $organizationIds = explode(',', $organizationIds);

                    if (empty($organizationIds)) {
                        $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Organization ids missing'));
                    }

                    $countAllLeads = array();
                    foreach ($organizationIds as $orgId) {
                        $leadsOrganizations = $organizationMapper->getOrganizationLeads($orgId);

                        if(!empty($leadsOrganizations)) {
                            $countAllLeads = array_merge($countAllLeads, array_keys($leadsOrganizations));
                        }
                    }
                }

                $emailSequenceMapper = Emailsequence_Models_Mapper_EmailSequenceSettingsMapper::getInstance();
                $emailSequences = $emailSequenceMapper->getSequencesPairs(true);
                echo json_encode(array(
                    'emailSequences' => $emailSequences,
                    'countOrganizationLeads' => count($countAllLeads)
                ));

                return;
            }

            //set email sequences for leads
            if($this->_request->isPost()){
                $this->_validateToken();

                $sequenceId = filter_var($this->_request->getParam('organizationEmailSequenceId'), FILTER_SANITIZE_NUMBER_INT);
                $organizationIds = filter_var($this->_request->getParam('organizationIds'), FILTER_SANITIZE_STRING);
                $status = false;
                $filters = $this->_request->getParam('filters');
                $allFilterOrganizations = $this->_request->getParam('allFilterOrganizations');
                $quantityAllLeadsData = filter_var($this->_request->getParam('quantityAllLeadsData'), FILTER_SANITIZE_NUMBER_INT);
                $requestType = $this->_request->getParam('requestType');
                $offset = 0;

                if(!empty($sequenceId) && !empty($organizationIds)){
                    $organizationIds = explode(',', $organizationIds);
                    if (empty($organizationIds)) {
                        $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Organization ids missing'));
                    }

                    if(!empty($allFilterOrganizations)) {
                        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
                        $organizationsData = Tools_LeadTools::getSearchOrganizationFilterData($filters, self::PROCESSED_ORGANIZATION_LIMIT, $offset);

                        if(!empty($organizationsData)){
                            $organizationsIds = array();
                            foreach ($organizationsData['data'] as $lead){
                                array_push($organizationsIds, $lead['id']);
                            }
                            $organizationIds = $organizationsIds;
                        }
                    }

                    $leadIdsCount = array();

                    if(!empty($organizationIds)) {
                        foreach ($organizationIds as $orgId) {
                            $leadsOrganizations = $organizationMapper->getOrganizationLeads($orgId);

                            if(!empty($leadsOrganizations)) {
                                $leadIdsCount = array_merge($leadIdsCount, array_keys($leadsOrganizations));
                            }
                        }
                    }

                    if ($requestType === 'addnew') {
                        $status = Tools_LeadFormEventsTools::assignLeadSequenceMassAction($sequenceId, $leadIdsCount);
                    } else {
                        $status = Tools_LeadFormEventsTools::unAssignLeadSequenceMassAction($sequenceId, $leadIdsCount);
                    }

                    $leadIdsCount = count($leadIdsCount);

                    if($status){
                        $this->_responseHelper->success(array('offset' => $offset + self::PROCESSED_ORGANIZATION_LIMIT, 'totalRecords' => $quantityAllLeadsData, 'leadsIdsCount' => $leadIdsCount));
                    }
                }

                $this->_responseHelper->success($this->_translator->translate(self::TRANSLATION_PREFIX.'Done'));
            }
        }
    }


    /**
     * pixel tracking code (track open emails action)
     */
    public function logPixelTrackingCodeAction()
    {
        $params = filter_var_array($this->_request->getParams(), FILTER_SANITIZE_STRING);
        if (!empty($params) && !empty($params['trackingCode'])) {
            $leadsEmailLog = Leads_Mapper_LeadsEmailLogMapper::getInstance();
            $trackingCode = $params['trackingCode'];
            $alreadyLoggedEmailLogModel = $leadsEmailLog->getByTrackingCode($trackingCode,
                Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_OPENED);
            if (!$alreadyLoggedEmailLogModel instanceof Leads_Model_LeadsEmailLogModel) {
                $leadsEmailLogModel = $leadsEmailLog->getByTrackingCode($trackingCode, Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_SENT);
                if ($leadsEmailLogModel instanceof Leads_Model_LeadsEmailLogModel) {
                    $createdAt = Tools_System_Tools::convertDateFromTimezone('now');
                    $leadId = $leadsEmailLogModel->getLeadId();
                    $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
                    $leadModel = $leadMapper->find($leadId);
                    if ($leadModel instanceof Leads_Model_LeadsModel) {
                        $leadTypeId = Tools_LeadTools::getLeadTypeIdByLeadTypeName(Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_EMAIL);
                        if(!empty($leadTypeId)){
                            $leadModel->setLastActivityOutbound($createdAt);
                        }else{
                            $leadModel->setLastActivity($createdAt);
                        }
                        $leadModel->setLastActivityTypeId($leadTypeId);
                        $leadModel->setLastActivitySubtype(Leads_Model_LeadsTypeModel::LEAD_OUTBOUND_EMAIL_SUBTYPE_OPEN);
                        $leadMapper->save($leadModel);
                    }
                    $actionLogModel = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_EMAIL,
                        $leadsEmailLogModel->getLeadId(),
                        Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND, $createdAt,
                        Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_EMAIL,
                        Leads_Model_LeadsActionLogModel::EVENT_LOG_EXPIRED, $leadsEmailLogModel->getContent());
                    $leadsEmailLogModel->setEmailStatus(Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_OPENED);
                    $leadsEmailLogModel->setId(null);
                    $leadsEmailLogModel->setActionLogId($actionLogModel->getId());
                    $leadsEmailLog->save($leadsEmailLogModel);
                }
            }

            $image = imagecreate(1, 1);
            $white = imagecolorallocate($image, 255, 255, 255);
            imagesetpixel($image, 1, 1, $white);
            header("content-type:image/jpg");
            imagejpeg($image);
            imagedestroy($image);
            echo $image;
        }
    }

    /**
     * email attachment tracking code (track open attachments to the email action)
     */
    public function openLinkAction()
    {
        $params = filter_var_array($this->_request->getParams(), FILTER_SANITIZE_STRING);
        if (!empty($params) && !empty($params['fileCode']) && !empty($params['emailCode'])) {
            $leadsEmailLog = Leads_Mapper_LeadsEmailLogMapper::getInstance();
            $emailSentCode = $params['emailCode'];
            $fileCode = $params['fileCode'];
            $leadsEmailLinkAttachmentLogMapper = Leads_Mapper_LeadsEmailLinkAttachmentLogMapper::getInstance();
            $leadsEmailLinkAttachmentLogModel = $leadsEmailLinkAttachmentLogMapper->getByFileHashSentHash($emailSentCode, $fileCode);
            if (!$leadsEmailLinkAttachmentLogModel instanceof Leads_Model_LeadsEmailLinkAttachmentLogModel) {
                $this->_redirector->gotoUrl($this->_websiteUrl);
            }

            $linkType = $leadsEmailLinkAttachmentLogModel->getLinkType();
            //backward compatibility
            if (empty($linkType)) {
                $linkType = Leads_Model_LeadsEmailLinkAttachmentLogModel::LINK_TYPE_ATTACHMENT;
            }

            $attachmentId = $leadsEmailLinkAttachmentLogModel->getId();
            $alreadyOpened = $leadsEmailLog->findByAttachmentId($attachmentId);
            $createdAt = Tools_System_Tools::convertDateFromTimezone('now');

            if (!$alreadyOpened instanceof Leads_Model_LeadsEmailLogModel && $linkType === Leads_Model_LeadsEmailLinkAttachmentLogModel::LINK_TYPE_ATTACHMENT) {
                $leadsEmailLogModel = $leadsEmailLog->getByTrackingCode($emailSentCode, Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_SENT);
                if ($leadsEmailLogModel instanceof Leads_Model_LeadsEmailLogModel) {
                    $leadId = $leadsEmailLogModel->getLeadId();
                    $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
                    $leadModel = $leadMapper->find($leadId);
                    if ($leadModel instanceof Leads_Model_LeadsModel) {
                        $leadTypeId = Tools_LeadTools::getLeadTypeIdByLeadTypeName(Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_EMAIL);
                        if(!empty($leadTypeId)){
                            $leadModel->setLastActivityOutbound($createdAt);
                        }else{
                            $leadModel->setLastActivity($createdAt);
                        }
                        $leadModel->setLastActivityTypeId($leadTypeId);
                        $leadModel->setLastActivitySubtype(Leads_Model_LeadsTypeModel::LEAD_OUTBOUND_EMAIL_SUBTYPE_ATTACHMENT);
                        $leadMapper->save($leadModel);
                    }
                    $actionLogModel = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_EMAIL,
                        $leadsEmailLogModel->getLeadId(),
                        Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND, $createdAt,
                        Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_EMAIL,
                        Leads_Model_LeadsActionLogModel::EVENT_LOG_EXPIRED, $leadsEmailLogModel->getContent());
                    $leadsEmailLogModel->setEmailStatus(Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_OPENED_ATTACHMENT);
                    $leadsEmailLogModel->setId(null);
                    $leadsEmailLogModel->setActionLogId($actionLogModel->getId());
                    $leadsEmailLogModel->setAttachmentId($leadsEmailLinkAttachmentLogModel->getId());
                    $leadsEmailLog->save($leadsEmailLogModel);
                }

                $folderName = $leadsEmailLinkAttachmentLogModel->getFileFolderName();
                $fileName = $leadsEmailLinkAttachmentLogModel->getFileName();
                $leadsEmailLinkAttachmentLogModel->setArchived('1');
                $leadsEmailLinkAttachmentLogMapper->save($leadsEmailLinkAttachmentLogModel);
                $leadsEmailLinkAttachmentLogMapper->setArchivedByEmailHashed($leadsEmailLinkAttachmentLogModel->getEmailLogSentHash());
                $this->_redirector->gotoUrl($this->_websiteUrl.'media/'.$folderName.'/'.$fileName);
            }

            if (!$alreadyOpened instanceof Leads_Model_LeadsEmailLogModel && $linkType === Leads_Model_LeadsEmailLinkAttachmentLogModel::LINK_TYPE_CLICK) {
                $leadsEmailLogModel = $leadsEmailLog->getByTrackingCode($emailSentCode, Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_SENT);
                if ($leadsEmailLogModel instanceof Leads_Model_LeadsEmailLogModel) {
                    $leadId = $leadsEmailLogModel->getLeadId();
                    $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
                    $leadModel = $leadMapper->find($leadId);
                    if ($leadModel instanceof Leads_Model_LeadsModel) {
                        $leadTypeId = Tools_LeadTools::getLeadTypeIdByLeadTypeName(Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_EMAIL);
                        if(!empty($leadTypeId)){
                            $leadModel->setLastActivityOutbound($createdAt);
                        }else{
                            $leadModel->setLastActivity($createdAt);
                        }
                        $leadModel->setLastActivityTypeId($leadTypeId);
                        $leadModel->setLastActivitySubtype(Leads_Model_LeadsTypeModel::LEAD_OUTBOUND_EMAIL_SUBTYPE_CLICK);
                        $leadMapper->save($leadModel);
                    }
                    $actionLogModel = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_EMAIL,
                        $leadsEmailLogModel->getLeadId(),
                        Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND, $createdAt,
                        Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_EMAIL,
                        Leads_Model_LeadsActionLogModel::EVENT_LOG_EXPIRED, $leadsEmailLogModel->getContent());
                    $leadsEmailLogModel->setEmailStatus(Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_CLICK);
                    $leadsEmailLogModel->setId(null);
                    $leadsEmailLogModel->setActionLogId($actionLogModel->getId());
                    $leadsEmailLogModel->setAttachmentId($leadsEmailLinkAttachmentLogModel->getId());
                    $leadsEmailLog->save($leadsEmailLogModel);
                }

                $fileLink = $leadsEmailLinkAttachmentLogModel->getFileLink();
                $leadsEmailLinkAttachmentLogModel->setArchived('1');
                $leadsEmailLinkAttachmentLogMapper->save($leadsEmailLinkAttachmentLogModel);
                $leadsEmailLinkAttachmentLogMapper->setArchivedByEmailHashed($leadsEmailLinkAttachmentLogModel->getEmailLogSentHash());
                $this->_redirector->gotoUrl($fileLink);
            }

            if ($linkType === Leads_Model_LeadsEmailLinkAttachmentLogModel::LINK_TYPE_ATTACHMENT) {
                $folderName = $leadsEmailLinkAttachmentLogModel->getFileFolderName();
                $fileName = $leadsEmailLinkAttachmentLogModel->getFileName();
                $this->_redirector->gotoUrl($this->_websiteUrl.'media/'.$folderName.'/'.$fileName);
            }

            if ($linkType === Leads_Model_LeadsEmailLinkAttachmentLogModel::LINK_TYPE_CLICK) {
                $fileLink = $leadsEmailLinkAttachmentLogModel->getFileLink();
                $this->_redirector->gotoUrl($fileLink);
            }

        }
    }

    /**
     * Load images for email setup
     */
    public function loadImagesAction()
    {
        if ($this->_request->isPost()) {
            $imagesData = array();
            $folderName = $this->_request->getParam('folderName');
            $imagesPath = $this->_websiteHelper->getPath() . $this->_websiteHelper->getMedia() . $folderName;
            try {
                $imagesData = array(
                    'small' => '<div class="images-preview list-images">' . $this->_proccessImages(Tools_Filesystem_Tools::scanDirectory($imagesPath . '/' . 'small'),
                            $imagesPath, $folderName, 'small') . '</div>',
                    'medium' => '<div class="images-preview list-images">' . $this->_proccessImages(Tools_Filesystem_Tools::scanDirectory($imagesPath . '/' . 'medium'),
                            $imagesPath, $folderName, 'medium') . '</div>',
                    'large' => '<div class="images-preview list-images">' . $this->_proccessImages(Tools_Filesystem_Tools::scanDirectory($imagesPath . '/' . 'large'),
                            $imagesPath, $folderName, 'large') . '</div>',
                    'original' => '<div class="images-preview list-images">' . $this->_proccessImages(Tools_Filesystem_Tools::scanDirectory($imagesPath . '/' . 'original'),
                            $imagesPath, $folderName, 'original') . '</div>'
                );
            } catch (Exceptions_SeotoasterException $se) {
                $imagesData = array(
                    'small' => '<div class="images-preview list-images">' . $this->_translator->translate(self::TRANSLATION_PREFIX.'No images were found') . '</div>',
                    'medium' => '<div class="images-preview list-images">' . $this->_translator->translate(self::TRANSLATION_PREFIX.'No images were found') . '</div>',
                    'large' => '<div class="images-preview list-images">' . $this->_translator->translate(self::TRANSLATION_PREFIX.'No images were found') . '</div>',
                    'original' => '<div class="images-preview list-images">' . $this->_translator->translate(self::TRANSLATION_PREFIX.'No images were found') . '</div>'
                );
            }
            $this->_responseHelper->success($imagesData);
        }
        exit;
    }

    /**
     * load files list for the tinymce editor
     */
    public function loadFilesAction()
    {
        if ($this->_request->isPost()) {
            $folder = $this->_request->getParam('folder');
            $filesPathUrl = $this->_websiteHelper->getUrl() . $this->_websiteHelper->getMedia() . $folder . DIRECTORY_SEPARATOR;
            $filesPath = $this->_websiteHelper->getPath() . $this->_websiteHelper->getMedia() . $folder;
            $this->_view->files = ((is_dir($filesPath))) ? Tools_Filesystem_Tools::findFilesByExtension($filesPath,
                '.*', false, false, false) : array();
            $this->_view->fileFolder = $folder;
            $this->_view->filesPathUrl = $filesPathUrl;
            $html = (($folder) ? $this->_view->render('email-files.phtml') : '<h3 class="text-center mt10px">' . $this->_translator->translate(self::TRANSLATION_PREFIX.'Please, select a folder') . '</h3>');
            $this->_responseHelper->success($html);
        }
    }

    /**
     * Refresh image folders
     *
     * @throws Exceptions_SeotoasterException
     * @throws Zend_Exception
     */
    public function refreshfoldersAction()
    {
        $listFolders = $this->_getFoldersList();
        $templatesAndDirectoriesMapper = Leads_Mapper_LeadsTemplatesAndDirectoriesConfigMapper::getInstance();
        $savedConfigFolders = $templatesAndDirectoriesMapper->findByType('folder');

        if(!empty($savedConfigFolders)){
            $listFolders = $savedConfigFolders;

            natcasesort($listFolders);
        }
        array_unshift($listFolders, $this->_translator->translate(self::TRANSLATION_PREFIX.'Select folder'));

        $this->_responseHelper->success($listFolders);
    }

    /**
     * validate secure token for all actions
     */
    private function _validateToken()
    {
        $secureToken = $this->_request->getParam('secureToken', false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $this->_responseHelper->response($this->_translator->translate(self::TRANSLATION_PREFIX.'Invalid token'), 1, 400);
        }
    }

    /**
     * Prepare images for the tinymce editor
     *
     * @param array $images array of the image names
     * @param string $path path
     * @param string $folder image folder
     * @param string $type image folder type small, medium, large
     * @return string
     */
    private function _proccessImages(array $images, $path, $folder, $type)
    {
        if (!empty ($images)) {
            $imagesContent = '';
            $srcPath = $this->_websiteUrl . $this->_websiteHelper->getMedia() . $folder;
            $sourcePartPath = $srcPath;

            foreach ($images as $key => $image) {
                $srcPath = Tools_Content_Tools::applyMediaServers($sourcePartPath);
                $imageName = preg_replace('~\.(jpg|png|gif|jpeg)~i', '', $image);
                $imageSize = getimagesize($path . '/' . $type . '/' . $image);
                $imageElement = htmlspecialchars('<a class="_lbox" href="' . $srcPath . '/' . 'original' . '/' . $image . '" title="' . str_replace('-',
                        '&nbsp;', $imageName) . '"><img border="0" alt="' . str_replace('-', '&nbsp;',
                        $imageName) . '" src="' . $srcPath . '/' . $type . '/' . $image . '" width="' . $imageSize[0] . '" height="' . $imageSize[1] . '" /></a>');
                $imagesContent .= '<a href="javascript:;" onmousedown="tinymce.activeEditor.execCommand(\'mceInsertContent\', false, \'' . $imageElement . '\');">';
                $imagesContent .= '<img title="' . $image . '" border="0" width="80" src="' . $srcPath . '/'. $type .'/' . $image . '" /></a>';
            }

            return $imagesContent;
        }
    }

    /**
     *
     * Add lead email to the cart status email queue (only for cart statuses)
     *
     * @throws Zend_Controller_Action_Exception
     */
    public function addCartSequenceEmailToQueueAction()
    {
        if ($this->_request->isPost()) {
            $cacheHelper = Zend_Controller_Action_HelperBroker::getExistingHelper('cache');
            $emailSequenceEnabled = Tools_LeadTools::verifyPluginEnabled(Tools_LeadTools::SEQUENCE_PLUGIN);
            if ($emailSequenceEnabled) {
                if (null === ($cartStatusConfigData = $cacheHelper->load('email_sequence_lead_status_data',
                        'email_sequence_lead_status'))
                ) {
                    Tools_LeadEmailSequenceTools::addToQueueCartEmails();
                }
                $cartStatusConfigData = 1;
                $cacheHelper->save('email_sequence_lead_status_data', $cartStatusConfigData,
                    'email_sequence_lead_status',
                    Helpers_Action_Cache::CACHE_SHORT);
                $this->_responseHelper->fail('');
            }
        }
    }

    /**
     *
     * Add lead email to the cart status email queue (except cart statuses)
     *
     * @throws Zend_Controller_Action_Exception
     */
    public function addNoCartSequenceEmailToQueueAction()
    {
        if ($this->_request->isPost()) {
            $cacheHelper = Zend_Controller_Action_HelperBroker::getExistingHelper('cache');
            $emailSequenceEnabled = Tools_LeadTools::verifyPluginEnabled(Tools_LeadTools::SEQUENCE_PLUGIN);
            $partially = $this->_request->getParam('partially');
            if ($emailSequenceEnabled) {
//                if (null === ($cartStatusConfigData = $cacheHelper->load('email_sequence_lead_status_data_no_cart',
//                        'email_sequence_lead_status_no_cart'))
//                ) {
                    Tools_LeadEmailSequenceTools::addToQueueNoCartEmails($partially);

               // }
                $cartStatusConfigData = 1;
//                $cacheHelper->save('email_sequence_lead_status_data_no_cart', $cartStatusConfigData,
//                    'email_sequence_lead_status_no_cart',
//                    Helpers_Action_Cache::CACHE_SHORT);
                $this->_responseHelper->fail('');
            }
        }
    }

    /**
     * Get license amount info information
     */
    public function getLicenceAmountInfoAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $licenceAmountData = array();
            $leadsAmount =  Tools_LeadLicenseTools::getCurrentLeadsAmount();
            $licenceLimit = Tools_LeadLicenseTools::getLicenseLimit();
            $licenceUpgradeLink = Tools_LeadLicenseTools::getLicenseSubscriptionLink();
            $licenceAmountData['leadsAmount'] = $leadsAmount;
            $licenceAmountData['licenceLimit'] = $licenceLimit;
            $licenceAmountData['licenceUpgradeLink'] = $licenceUpgradeLink;

            $this->_responseHelper->success($licenceAmountData);
        }
    }

    /**
     * Get today month and year for the email sequence filter
     */
    public function getCurrentMonthAndYearAction()
    {
        if ($this->_request->isPost() && Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $defaultTimezone = date_default_timezone_get();
            if (empty($defaultTimezone)) {
                $defaultTimezone = 'UTC';
            }

            $currentYear = Tools_System_Tools::convertDateFromTimezone('now', $defaultTimezone, Tools_System_Tools::getUserTimezone(), 'Y');
            $currentMonth = Tools_System_Tools::convertDateFromTimezone('now', $defaultTimezone, Tools_System_Tools::getUserTimezone(), 'm');
            $currentMonthYear = array('month' => $currentMonth, 'year' => $currentYear);
            $this->_responseHelper->success($currentMonthYear);
        }
    }

    /**
     * Send email on logged user email from the Email sequence tab on the lead detailed view
     */
    public function sendCartStatusQueueEmailAction()
    {
        if ($this->_request->isPost() && Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $this->_validateToken();
            $queueId = filter_var($this->_request->getParam('queueId'), FILTER_SANITIZE_NUMBER_INT);
            if (empty($queueId)) {
                $this->_responseHelper->response($this->_translator->translate(self::TRANSLATION_PREFIX.'Missing queue id'), 1, '400');
            }

            $emailSequenceEnabled = Tools_LeadTools::verifyPluginEnabled(Tools_LeadTools::SEQUENCE_PLUGIN);
            if (empty($emailSequenceEnabled)) {
                $this->_responseHelper->response($this->_translator->translate(self::TRANSLATION_PREFIX.'Please install plugin email sequence'), 1,
                    '400');
            }

            $emailModel= Emailsequence_Models_Mapper_EmailsequenceQueueMapper::getInstance()->find($queueId);
            if ($emailModel instanceof Emailsequence_Models_Models_EmailsequenceQueueModel) {
                if($emailModel->getType() == 'sms') {
                    $sessionHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
                    $userSession = $sessionHelper->getCurrentUser();
                    $mobilePhoneUser = Tools_LeadTools::glueUserPhone(
                        $userSession->getMobileCountryCodeValue(),
                        $userSession->getMobilePhone(),
                        $userSession->getDesktopCountryCodeValue(),
                        $userSession->getDesktopPhone()
                    );

                    $smsMessage = $emailModel->getEmailMessage();
                    $currentLoggedUserEmail = $userSession->getEmail();

                    $leadData = Tools_LeadTools::getLeadModelByIdAndEmail($userSession->getId(), $currentLoggedUserEmail);

                    if($leadData instanceof Leads_Model_LeadsModel) {
                        $smsMessage = Tools_EmailSequenceTools::addDictionarySmsFields($smsMessage, $leadData, $this->_websiteUrl);
                    }

                    if(!empty($mobilePhoneUser)) {
                        $subscriber['subscriber']['admin'] = array(
                            'phone' => array($mobilePhoneUser),
                            'message' => $smsMessage,
                            'owner_type' => Apps::SMS_OWNER_TYPE_USER,
                            'sms_from_type' => 'crm',
                            'custom_params' => array()
                        );

                        $response = Apps::apiCall('POST', 'apps', array('twilioSms'), $subscriber);

                        if($response['twilioSms']['done']) {
                            $this->_responseHelper->success($this->_translator->translate(self::TRANSLATION_PREFIX.'SMS has been sent'));
                        } else {
                            $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Can\'t send SMS'));
                        }
                    } else {
                        $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Lead mobile phone doesn\'t exist'));
                    }
                } else {
                    $response = Tools_LeadEmailSequenceTools::sendCartStatusEmail($emailModel);
                    if (empty($response['error'])) {
                        $this->_responseHelper->success($this->_translator->translate(self::TRANSLATION_PREFIX.'Email has been sent'));
                    } else {
                        $this->_responseHelper->fail($response['error']['message']);
                    }
                }
            }

            $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Record doesn\'t exist'));
        }
        $this->_responseHelper->fail('');
    }


    /**
     * Gert form fields by form name
     */
    public function getFormFieldsListAction()
    {
        if ($this->_request->isGet() && Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $formName = filter_var($this->_request->getParam('formName'), FILTER_SANITIZE_STRING);
            $formType = filter_var($this->_request->getParam('formType'), FILTER_SANITIZE_STRING);
            if (empty($formName) || empty($formType)) {
                $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Missing queue id'));
            }
            if ($formType === Leads_Model_LeadsFormRulesGeneralConfigModel::FORM_TYPE_INTERNAL) {
                $formFields = Tools_LeadFormTools::getSystemFormFields($formName);
            }
            if ($formType === Leads_Model_LeadsFormRulesGeneralConfigModel::FORM_TYPE_EXTERNAL) {
                $formFields = Tools_LeadFormTools::getExternalFormFields($formName);
            }
            if (!empty($formFields)) {
                $this->_responseHelper->success(array('formFields' => $formFields));
            }
            $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Fields not found'));
        }
        $this->_responseHelper->fail('');
    }

    /**
     * Get event fields
     */
    public function getEventFieldsListAction()
    {
        if ($this->_request->isGet() && Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $fields = Tools_LeadFormTools::getDefaultFieldsList();
            if (!empty($fields)) {
                $fields = array_flip($fields);
                ksort($fields);
                $this->_responseHelper->success(array('fields' => $fields));
            }
            $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Fields not found'));
        }
        $this->_responseHelper->fail('');
    }

    /**
     * Get default email template message for the email tab on the dashboard
     */
    public function getEmailDefaultMessageAction()
    {
        $templateId = filter_var($this->_request->getParam('templateId'), FILTER_SANITIZE_STRING);
        $leadId = filter_var($this->_request->getParam('leadId'), FILTER_SANITIZE_STRING);
        if ($this->_request->isGet() && Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            if ($templateId === 'default') {
                $activeActionEmailTriggers = Tools_LeadTools::getDefaultActionEmailInfo(Tools_LeadsMailWatchdog::LEADS_CONTACT);
                if (!empty($activeActionEmailTriggers)) {
                    $templateId = $activeActionEmailTriggers['template'];
                } else {
                    $templateId = 0;
                }
            }

            $emailTemplateDefaultMessage = Tools_LeadTools::getDefaultEmailMessage($templateId);
            $pageModel = Tools_LeadTools::getTemplatePage($templateId);
            $pageUrl = '';
            if ($pageModel instanceof Application_Model_Models_Page) {
                $pageUrl = $pageModel->getUrl();
                $websiteHelper = Zend_Controller_Action_HelperBroker::getExistingHelper('website');
                $themeData = Zend_Registry::get('theme');
                $extConfig = Zend_Registry::get('extConfig');
                $observableModel = Leads_Mapper_LeadsMapper::getInstance()->find($leadId);
                $parserOptions = array(
                    'websiteUrl' => $websiteHelper->getUrl(),
                    'websitePath' => $websiteHelper->getPath(),
                    'currentTheme' => $extConfig['currentTheme'],
                    'themePath' => $themeData['path'],
                    'observableModel' => $observableModel
                );

                $parser = new Tools_Content_Parser($emailTemplateDefaultMessage, array(), $parserOptions);

                $emailTemplateDefaultMessage = $parser->parse();
            }
            $this->_responseHelper->success(array(
                'emailDefaultMessage' => $emailTemplateDefaultMessage,
                'pageUrl' => $pageUrl
            ));
        }
        $this->_responseHelper->fail('');
    }

    public function getFormFieldsCodeAction()
    {
        if ($this->_request->isPost() && Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $formFieldsData =  json_decode($this->_request->getParam('formFieldsData'), true);
            $formType =  $this->_request->getParam('formType');
            $content = $this->generatePreviewCode($formFieldsData, $formType);
            $this->_responseHelper->success($content);
        }
        $this->_responseHelper->fail('');
    }

    /**
     * Change task status
     */
    public function changeTaskStatusAction()
    {
        if ($this->_request->isPost() && Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $secureToken = $this->_request->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
            $dashboardResponseFormat = $this->_request->getParam('dashboardResponseFormat', false);
            $tokenValid = Tools_System_Tools::validateToken($secureToken, Api_Leads_Leads::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $this->_responseHelper->fail('Invalid token');
            }

            $taskId =  filter_var($this->_request->getParam('taskId', false), FILTER_SANITIZE_NUMBER_INT);
            $isClosed =  filter_var($this->_request->getParam('isClosed', false), FILTER_SANITIZE_NUMBER_INT);
            $additionalComment =  filter_var($this->_request->getParam('additionalComment', false), FILTER_SANITIZE_STRING);
            $notificationEmails =  filter_var_array($this->_request->getParam('notificationEmails', array()), FILTER_SANITIZE_STRING);
            if (empty($taskId)) {
                $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Task id is missing'));
            }

            $closedByUserId = $this->_sessionHelper->getCurrentUser()->getId();

            $status = Tools_LeadTasksTools::changeTaskStatus($taskId, $isClosed, $closedByUserId, $notificationEmails, $additionalComment);
            if ($status === true) {
                $taskMessage = $this->_translator->translate(self::TRANSLATION_PREFIX.'closed');
                if (empty($isClosed)) {
                    $taskMessage = $this->_translator->translate(self::TRANSLATION_PREFIX.'re-open');
                }

                if (!empty($dashboardResponseFormat)) {
                    $this->_responseHelper->success(array('error' => '0', 'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Task has been') . ' ' . $taskMessage));
                } else {
                    $this->_responseHelper->success($this->_translator->translate(self::TRANSLATION_PREFIX.'Task has been') . ' ' . $taskMessage);
                }
            }

        }

        if (!empty($dashboardResponseFormat)) {
            $this->_responseHelper->success(array('error' => '0', 'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Task not found')));
        } else {

            $this->_responseHelper->fail('');
        }
    }

    /**
     *
     * Get task info for the task complete dialog in the timeline
     *
     */
    public function getTaskInfoAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $taskId =  filter_var($this->_request->getParam('taskId', false), FILTER_SANITIZE_NUMBER_INT);
            if (!empty($taskId)) {
                $leadTaskMapper = Leads_Mapper_LeadsTaskMapper::getInstance();
                $leadTaskModel = $leadTaskMapper->find($taskId);
                if ($leadTaskModel instanceof Leads_Model_LeadsTaskModel) {
                    $taskData = $leadTaskModel->toArray();
                    $userIdsToNotify = array($taskData['creatorId'], $taskData['userId']);
                    $usersInfo = Leads_Mapper_LeadsMapper::getInstance()->getLeadOwnersList($userIdsToNotify);
                    if (!empty($usersInfo)) {
                        $taskData['notificationEmails'] = '';
                        foreach ($usersInfo as $userInfo) {
                            $taskData['notificationEmails'] .= $userInfo['email'].',';
                        }
                        $taskData['notificationEmails'] = rtrim($taskData['notificationEmails'], ',');
                        $this->_responseHelper->success($taskData);
                    } else {
                        $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'User in task has no permission. Please check user roles in task.'));
                    }
                }
            }
            $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Task not found'));
        }

        $this->_responseHelper->fail('');
    }

    /**
     * Generate form preview code
     *
     * @param array $formFieldsData form fields data
     * @param string $formType form type
     * @param string $formSignature form signature
     * @param array $data additional form data
     * @return string
     */
    public function generatePreviewCode($formFieldsData, $formType = 'internal', $formSignature = '', $data = array())
    {
        $this->_view->formFieldsData = $formFieldsData;
        $this->_view->formType = $formType;
        $this->_view->formSignature = $formSignature;
        $this->_view->additionalFormData = $data;
        $uniqueFieldNames = array();
        foreach ($formFieldsData as $fieldData) {
            $uniqueFieldNames[$fieldData['fieldName']] = $fieldData['fieldName'];
        }

        $additionalFieldGroupParams = array();
        $leadDataMapper = Leads_Mapper_LeadsMapper::getInstance();
        $tagMapper = Leads_Mapper_LeadsTagsMapper::getInstance();
        $additionalFieldGroupParams['phoneCountryCodes'] =  Tools_System_Tools::getFullCountryPhoneCodesList(true, array(), true);
        $customerGroups = $leadDataMapper->getCustomerGroups();
        $leadTags = $tagMapper->getTagsPairs();
        asort($customerGroups);
        asort($leadTags);

        $defaultCountryCode = 'US';
        $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();
        if (!empty($shoppingConfig['country'])) {
            if (empty($countryCode)) {
                $defaultCountryCode = $shoppingConfig['country'];
            }
        }

        $stateCodes = Tools_LeadTools::getStates($defaultCountryCode, true, true);
        asort($stateCodes);
        $additionalFieldGroupParams['customerGroups'] = $customerGroups;
        $additionalFieldGroupParams['countryCodes'] = Tools_Geo::getCountries(true);
        $additionalFieldGroupParams['leadTags'] = $leadTags;
        $additionalFieldGroupParams['stateCodes'] = $stateCodes;
        $this->_view->additionalFieldGroupParams = $additionalFieldGroupParams;


        $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
        $userDefaultPhoneMobileCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');
        if (!empty($userDefaultPhoneMobileCountryCode)) {
            $mobilePhoneCountryCode = $userDefaultPhoneMobileCountryCode;
        } else {
            $mobilePhoneCountryCode = 'US';
        }




        $this->_view->defaultCountryCode = $defaultCountryCode;
        $this->_view->defaultPhoneMobileCountryCode = $mobilePhoneCountryCode;

        $this->_view->uniqueFieldNames = $uniqueFieldNames;
        return $this->_view->render('form-builder-form-code.phtml');
    }

    /**
     * check list for the crm config
     */
    public static function crmCheckList()
    {
        $checkListMapper = Crm_Mapper_ChecklistMapper::getInstance();
        $checkListModel = $checkListMapper->findByName('mojo_registartion');
        if ($checkListModel instanceof Crm_Model_ChecklistModel) {
            $websiteConfig = Zend_Controller_Action_HelperBroker::getStaticHelper('config')->getConfig();
            $checkListModel->setClosedBy(Crm_Model_ChecklistModel::CHECKLIST_CLOSED_BY_AUTOMATICALLY);
            if (!empty($websiteConfig['websiteId']) && !empty($websiteConfig['sambaToken'])) {
                $checkListModel->setChecklistStatus('1');
            } else {
                $checkListModel->setChecklistStatus('0');
            }

            $checkListMapper->save($checkListModel);
        }

        $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
        $websiteUrl = $websiteHelper->getUrl();
        $translator = Zend_Registry::get('Zend_Translate');

        $checklistElementsData = array(
            'leads_import' => '<a class="highlight-crm-tab" data-tab-number="1" href="javascript:;">'. $translator->translate(self::TRANSLATION_PREFIX.'Import data').'</a>'.' '. $translator->translate(self::TRANSLATION_PREFIX.'from your previous solution or spreadsheets'),
            'add_products_screen' => '<a target="_blank" href="javascript:;" data-url="'.$websiteUrl.'plugin/shopping/run/product/" data-pheight="560" class="tpopup page">'
                .$translator->translate(self::TRANSLATION_PREFIX.'Add products and services').'</a> '.$translator->translate(self::TRANSLATION_PREFIX.'to build quotes easily'),
            'paypal_account_screen' => '<a href="javascript:;" class="tpopup page" data-pwidth="480" data-pheight="421" data-url="'.$websiteUrl.'plugin/paypal/run/config/">'.$translator->translate(self::TRANSLATION_PREFIX.'Add your PayPal').'</a> <a target="_blank" href="https://www.paypal.com/us/webapps/mpp/referral/paypal-payments-standard?partner_id=4E7GQK85JKSBA">('.$translator->translate(self::TRANSLATION_PREFIX.'open an account').') </a></a>'.$translator->translate(self::TRANSLATION_PREFIX.'credentials to collect online payments from your clients'),
            'dashboard_leads_tab' => $translator->translate(self::TRANSLATION_PREFIX.'Adjust other tabs presets, including').' <a class="checklist-tab-highlight" href="javascript:;" >'.$translator->translate(self::TRANSLATION_PREFIX.'opportunities and leads').'</a> '.$translator->translate(self::TRANSLATION_PREFIX.'as you see fit'),
            'create_groups' => '<a href="javascript:;" class="tpopup page" data-url="'.$websiteUrl.'plugin/shopping/run/merchandising/#group-pricing">'.$translator->translate(self::TRANSLATION_PREFIX.'Create groups').'</a>'.' ('.$translator->translate(self::TRANSLATION_PREFIX.'e.g. wholesale vs. retail: do you sell the same stuff at different prices?').')',
            'manage_users_screen' => '<a target="_blank" class="tpopup page" href="javascript:;" data-pwidth="960" data-pheight="560" data-url="'.$websiteUrl.'backend/backend_user/manage/" title="Manage users">'.$translator->translate(self::TRANSLATION_PREFIX.'Create sales users').'</a> '.$translator->translate(self::TRANSLATION_PREFIX.'and add their desktop and mobile numbers so that you can dial contacts out'),
            'mojo_registartion' => '<a target="_blank" href="https://mojo.seosamba.com/register.html" title="">'.$translator->translate(self::TRANSLATION_PREFIX.'Open a SeoSamba free account').'</a> '.$translator->translate(self::TRANSLATION_PREFIX.'and secure one or more toll-free or local sales phone numbers anywhere in the U.S and around the world to track inbound calls'),
            'imap_service' => $translator->translate(self::TRANSLATION_PREFIX.'Add your') . ' <a class="checklist-imap-tab-highlight" data-tab-number="34" href="javascript:;">'. $translator->translate(self::TRANSLATION_PREFIX.'IMAP credentials').'</a>'.' '. $translator->translate(self::TRANSLATION_PREFIX.'to sync your email client with your CRM contacts timeline')
        );

        return $checklistElementsData;

    }

    /**
     * @return string
     */
    public static function crmHiddenEmail(){
        $translator = Zend_Registry::get('Zend_Translate');
        $checkLicense = Tools_LeadLicenseTools::checkLimit();

        if($checkLicense['error']){
            return '<table width="100%"><tr><td align="center"><a target="_blank" href="https://www.seosamba.com" style="font-size:14px;">'. $translator->translate(self::TRANSLATION_PREFIX.Tools_LeadLicenseTools::PROMOTIONAL_HIDDEN_EMAIL_TEXT) .'</a></td></tr></table>';
        }

        return '';
    }

    /**
     * Create/Update Lead using third party service API
     *
     * @param $params
     * @return array
     */
    public static function processLead($params)
    {
        if (!empty($params)) {
            $leadResult = Tools_LeadApiTools::processLeadsData($params);
        } else {
           return array('error' => 1, 'message' => 'Empty data');
        }

        if ($leadResult['error'] == '1') {
            return array('error' => 1, 'message' => $leadResult['message']);
        }

        return array('error' => 0, 'message' => 'Lead has been processed');

    }

    /**
     * Get apps enabled services
     */
    public function getEnabledAppsServicesAction()
    {
        if (Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS)) {
            $data = $this->_request->getParams();

            $enabledServices = Tools_LeadTools::getEnabledServices($data['serviceType']);
            //$categoryAndServices = json_decode('{"services":{"email":{"constantContact":{"name":"Constant Contact","info":"<span class=\"serviceInfo\">Constant Contact, Inc. is an online marketing company offering email marketing, social media marketing, online survey, event marketing, digital storefronts, and local deals tools, primarily to small businesses, nonprofit organizations, and membership associations. <a class=\"readMore\" target=\"_blank\" href=\"http:\/\/www.seotoaster.com\/cms-applications.html\">Read more<\/a><\/span>","status":true,"params":{"registerLink":"<div style=\"margin-top: 10px; text-align: center;\">\n            <a href=\"http:\/\/diariovasco.localdigitalkit.com\/mail\/?id=438&w=816\" target=\"_blank\">Access your email marketing configuration here.<\/a>\n            <\/div>"}},"sambaEmailMarketing":{"name":"SEO Samba email marketing","info":"Create email campaigns","status":true, "detailedInfo": "1"}},"sms":{"cloud_communication":{"name":"SMS & Phone service","info":"<span class=\"serviceInfo\">Notify your clients wherever they are, using SMS service. <a class=\"readMore\" target=\"_blank\" href=\"http:\/\/www.seotoaster.com\/cms-applications.html\">Read more<\/a><\/span>","status":true,"params":{"registerLink":"<div style=\"margin-top: 10px; text-align: center;\">\n            <a href=\"http:\/\/diariovasco.localdigitalkit.com\/sms\/?id=438\" target=\"_blank\">Buy additional phone numbers around the world to track your campaigns.<\/a>\n            <\/div>"}}},"crm":{"infusionSoft":{"name":"InfusionSoft","info":"<span class=\"serviceInfo\">Notify your clients wherever they are, using SMS service. <a class=\"readMore\" target=\"_blank\" href=\"http:\/\/www.seotoaster.com\/cms-applications.html\">Read more<\/a><\/span>","status":"http:\/\/diariovasco.localdigitalkit.com\/crm\/?id=438&w=816"},"salesForce":{"name":"SalesForce","info":"<span class=\"serviceInfo\">Notify your clients wherever they are, crm. <a class=\"readMore\" target=\"_blank\" href=\"http:\/\/www.seotoaster.com\/cms-applications.html\">Read more<\/a><\/span>","status":"http:\/\/diariovasco.localdigitalkit.com\/crm\/?id=438&w=816"}},"categoryInfo":{"email":"<span class=\"categoryInfo\">Email marketing providers help you reach your audience with mass emails. Reputation is key to reach your audience when using bulk emails.\nHence, why you won\u2019t find a built-in newsletter facility around here. Instead, your system will  add emails right onto your mailing lists from\nyour website forms, as well as from your store dashboard\u2019s client tab (E-commerce version) in a snap.\n    <a class=\"readMore\"\n       target=\"_blank\"\n       href=\"http:\/\/www.seotoaster.com\/cms-applications.html\">Read\n        more<\/a><\/span>","sms":"<span class=\"categoryInfo\"><p>Buy phone numbers internationally for a low monthly fee to deliver superior\n        marketing & customer services.<\/p><p>You can use a single phone number to receive or send SMS when an order has\n        been received or has shipped (eCommerce edition) for instance, and\/or offer web-to-call services to your website\n        visitors, and\/or record and receive\n        calls over regular phone lines to track sales leads generated by your website.<\/p><p>Heck, you can even buy more\n        numbers to record and track phone leads for any\n        online or offline marketing campaign. <a class=\"readMore\"\n                                                target=\"_blank\"\n                                                href=\"http:\/\/www.seotoaster.com\/cms-applications.html\">Read\n            more<\/a><\/p><\/span>","crm":"<span class=\"categoryInfo\"><p>CRM information<\/p><p><a class=\"readMore\"\n                                                target=\"_blank\"\n                                                href=\"http:\/\/www.seotoaster.com\/cms-applications.html\">Read\n            more<\/a><\/p><\/span>"}}}', true);

            $organizationIds = filter_var($this->_request->getParam('organizationIds'), FILTER_SANITIZE_STRING);

            $countAllLeads = array();
            if(!empty($organizationIds)) {
                $organizationIds = explode(',', $organizationIds);

                if (empty($organizationIds)) {
                    $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Organization ids missing'));
                }

                $organizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();

                foreach ($organizationIds as $orgId) {
                    $leadsOrganizations = $organizationMapper->getOrganizationLeads($orgId);

                    if(!empty($leadsOrganizations)) {
                        $countAllLeads = array_merge($countAllLeads, array_keys($leadsOrganizations));
                    }
                }
            }

            if (!empty($enabledServices)) {
                $this->_responseHelper->success(array(
                    'clients' => $data['customers'],
                    'enabledServices' => $enabledServices,
                    'countOrganizationLeads' => count($countAllLeads)
                ));
            }
            $this->_responseHelper->fail();
        }
    }


    /**
     * Get Ad audience list Data
     *
     * @throws Exceptions_SeotoasterPluginException
     */
    public function getAdvertisingServiceAction(){
        if(Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS)) {
            $data = $this->_request->getParams();

            if(isset($data['serviceName'])){
                $services = Apps::apiCall('GET', 'sambastatsRemarketingList', array(), array('type' => $data['serviceName']));

                //@todo remove this emulation
                /*if($data['serviceName'] == 'adwords') {
                    $services = array(
                        'error' => 0,
                        'data' => array(
                            0 => array(
                                'id' => 944670158,
                                'name' => 'adwords Convention IFA 2020 list'
                            ),
                            1 => array(
                                'id' => 6492983775,
                                'name' => 'adwords adsGoogleList TEST'
                            ),
                            2 => array(
                                'id' => '6517075412',
                                'name' => 'adwords test list'
                            )
                        )
                    );
                }*/

                /*$services = array(
                    'error' => 0,
                    'data' => array(
                        0 => array(
                            'id' => 944670158,
                            'name' => 'Convention IFA 2020 list'
                        ),
                        1 => array(
                            'id' => 6492983775,
                            'name' => 'adsGoogleList TEST'
                        ),
                        2 => array(
                            'id' => '6517075412',
                            'name' => 'test list'
                        )
                    )
                );*/

                /*$postLists = array('websiteId' => null,
                    'type' => "facebook",
                    'audienceId' =>array(
                        '23845629305980205',
                        '34565745475745455'
                    ),
                    'listContent' => array(
                        array(
                            'email' => '', 'phone' => '+380...', 'city' => '', 'state' => '', 'country' => '', 'zip' => '', 'firstName' => '', 'lastName' => ''
                        ),
                        array(
                            'email' => '', 'phone' => '+380...', 'city' => '', 'state' => '', 'country' => '', 'zip' => '', 'firstName' => '', 'lastName' => ''
                        )
                    )
                );

                $errorList =  array(
                    '23845629305980205' => 'Audience doesn\'t belong to connected account',
                    '34565745475745455' => 'OTHER EEERRROR'
                );*/

                if(!$services['error']) {
                    $this->_responseHelper->success(array('list' => $services['data']));
                } else {
                    $message = $services['message'];

                    if(!empty($services[0])) {
                        $message = $services[0];
                    }
                }

                $this->_responseHelper->fail(array('status' => 'error', 'errorCode' => '1', 'message' => $message));

            }
            $this->_responseHelper->fail(array('status' => 'error', 'errorCode' => '1', 'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Error')));
        }
    }

    /**
     * Add Ad audience list
     *
     * @throws Exceptions_SeotoasterPluginException
     */
    public function addAdvertiserlistAction()
    {
        if(Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS)) {
            $this->_validateToken();
            $data = $this->_request->getParams();

            if(!empty($data['account']) && !empty($data['listName'])) {
                $services = Apps::apiCall('POST', 'sambastatsRemarketingList', array(), array('type' => $data['account'], 'listName' => $data['listName']));

                if(!empty($services['error'])) {
                    $this->_responseHelper->fail(array('status' => 'error', 'errorCode' => '1', 'message' => $services['message']));
                } else {
                    if(empty($services['listId'])) {
                        $this->_responseHelper->fail(array('status' => 'error', 'errorCode' => '1', 'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Can\'t create list. Check your account settings')));
                    }
                    $this->_responseHelper->success(array('error' => '0', 'message' => $services['message'], 'listId' => $services['listId']));
                }
            }

            $this->_responseHelper->fail(array('status' => 'error', 'errorCode' => '1', 'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Can\'t create list')));
        }
    }

    /**
     * Check connection account on Mojo and synchronize data
     *
     * @throws Exceptions_SeotoasterPluginException
     */
    public function checkAdvertiserConnectionAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $advertiserId = filter_var($this->_request->getParam('advertiserId'), FILTER_SANITIZE_NUMBER_INT);

            if(!empty($advertiserId)) {
                $leadsAdvertiserAccountsConfigMapper = Leads_Mapper_LeadsAdvertiserAccountsConfigMapper::getInstance();

                $advertiserAccountData = $leadsAdvertiserAccountsConfigMapper->findById($advertiserId);

                if($advertiserAccountData instanceof Leads_Model_LeadsAdvertiserAccountsConfigModel) {
                    $services = Apps::apiCall('POST', 'sambastatsConnectionRequest', array(), array('type' => $advertiserAccountData->getAdvType()));

                    /*$services = array(
                        'error' => 0,
                        'data' => array(
                            'status' => '1',
                            'adAccountId' => '1234567',
                            'name' => 'Some company name'),
                        'link' => 'http://google.com/');*/

                    /*$services = array(
                        'error' => 1,
                        'message' => 'This Facebook Ad account has been already connected to another agency'
                    );*/

                    if(!empty($services['error'])) {
                        $this->_responseHelper->fail(array('status' => 'error', 'errorCode' => '1', 'message' => $services['message']));
                    } else {
                        if(!empty($services['data']['status'])) {
                            $advertiserAccountData->setAdvStatus('connected');

                            $leadsAdvertiserAccountsConfigMapper->save($advertiserAccountData);

                            $this->_responseHelper->success(array('error' => '0', 'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Advertiser account successful verified.')));
                        } else {
                            $advertiserAccountData->setAdvStatus('pending');

                            $leadsAdvertiserAccountsConfigMapper->save($advertiserAccountData);
                            $this->_responseHelper->fail(array('status' => 'error', 'errorCode' => '1', 'accountVerificationLink' => $advertiserAccountData->getAdvVerificationLink(), 'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Your account is not verified yet. Try again later.')));
                        }
                    }
                }

                $this->_responseHelper->fail(array('status' => 'error', 'errorCode' => '1', 'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Advertiser account doesn\'t exists')));
            }
        }
    }

    /**
     * Check accounts connection on Mojo
     *
     * @throws Exceptions_SeotoasterPluginException
     */
    public function checkAdvertiserConnectionAccountsAction()
    {
        $advertiserAccountTypes = Tools_LeadTools::$_advertisingTypes;

        $advertiserAccountStatuses = array();

        $leadsAdvertiserAccountsConfigMapper = Leads_Mapper_LeadsAdvertiserAccountsConfigMapper::getInstance();
        foreach ($advertiserAccountTypes as $account => $accountLabel) {
            $services = Tools_LeadTools::sambastatsConnectionStatusGet($account);

            /*$services = array(
                        'error' => 0,
                        'data' => array(
                            'status' => '1',
                            'adAccountId' => '1234567',
                            'name' => 'Some company name'),
                        'link' => 'http://google.com/');*/

            /*$services = array(
                'error' => 1,
                'message' => 'This Facebook Ad account has been already connected to another agency'
            );*/

            /*$services = array(
                'error' => 1,
                'message' => 'Invitation has been declined'
            );*/

            $advertiserAccountStatuses[$account] = 'disconnected';

            if(!empty($services['data']['status'])) {
                if(!empty($services['data']['connectionStatus'])) {
                    $advertiserAccountStatuses[$account] = 'connected';
                }
            } elseif (empty($services['data']['status']) && (!empty($services['data']['adAccountId']))) {
                $advertiserAccountData = $leadsAdvertiserAccountsConfigMapper->findByAdvId($services['data']['adAccountId']);

                if($advertiserAccountData instanceof Leads_Model_LeadsAdvertiserAccountsConfigModel) {
                    $servicesConnection = Apps::apiCall('POST', 'sambastatsConnectionRequest', array(), array('type' => $advertiserAccountData->getAdvType()));

                    if(!empty($servicesConnection['data']['status'])) {
                        if(!empty($services['data']['connectionStatus'])) {
                            $advertiserAccountStatuses[$account] = 'connected';
                        }
                        $advertiserAccountData->setAdvStatus('connected');
                        $leadsAdvertiserAccountsConfigMapper->save($advertiserAccountData);
                    }
                }
            }
        }

        $this->_responseHelper->success(array('error' => '0', 'accountsStatuses' => $advertiserAccountStatuses));
    }

    /**
     * Get advertising account connection on mojo
     *
     * @throws Exceptions_SeotoasterPluginException
     */
    public function getAdvAccountConnectionStatusAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $advertiserType = filter_var($this->_request->getParam('advertiserType'), FILTER_SANITIZE_STRING);

            if(!empty($advertiserType)) {
                $services = Tools_LeadTools::sambastatsConnectionStatusGet($advertiserType);

                /*$services = array(
                        'error' => 0,
                        'data' => array(
                    //  'status' => '1',
                    //  'adAccountId' => '1234567',
                    //  'name' => 'Some company name'
                        ),
                        'link' => 'http://google.com/');*/

                /*$services = array(
                    'error' => 1,
                    'message' => 'Invitation has been declined'
                );*/

                if(!empty($services['error'])) {
                    $this->_responseHelper->fail(array('status' => 'error', 'errorCode' => '1', 'message' => $services['message']));
                } else{
                    $link = $services['link'];

                    if($advertiserType == 'facebook' && !empty($services['link'])) {
                        $link .= '&facebookauth=1';
                    }

                    $this->_responseHelper->success(array('status' => '0', 'message' => '', 'type' => $advertiserType, 'link' => $link));
                }
            }
        }
    }

    /**
     * Get apps services action
     */
    public function getAppsServiceAction(){

        if(Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS)) {
            $data = $this->_request->getParams();
            $lists = array();
            if(isset($data['serviceName'])){
                $detailedInfo = '0';
                $serviceMethodKey = $data['serviceName'].'List';
                //$services = json_decode('{"sambaEmailMarketingList":{"done":true,"data":{"sambaEmailMarketing":[{"id":"47","user_id":"1293","name":"Brat list","type":"private","creation_date":"2018-01-15 16:53:39","totalRecipient":0},{"id":"46","user_id":"1293","name":"vssdgsgsdgsd","type":"managed","creation_date":"2018-01-15 16:53:31","totalRecipient":0},{"id":"39","user_id":"1293","name":"123","type":"shared","creation_date":"2017-12-29 19:14:18","totalRecipient":0},{"id":"38","user_id":"1293","name":"vasya11","type":"shared","creation_date":"2017-12-29 19:13:49","totalRecipient":0}]},"detailedInfo":1}}', true);
                $userEmail = $this->_sessionHelper->getCurrentUser()->getEmail();
                $services = Apps::apiCall('GET', 'apps', array($data['serviceName'] . 'List'), array('userEmail' => $userEmail));

                /*$services = array('constantContactList' => array(
                    '0' => array(
                        'id' => '1791339256',
                        'name' => 'Test',
                        'status' => 'HIDDEN',
                        'contact_count' => '1',
                    ),
                    '1' => array(
                        'id' => '1399533545',
                        'name' => 'General Interest',
                        'status' => 'ACTIVE',
                        'contact_count' => '1',
                    )
                ));*/

                if(isset($services[$serviceMethodKey])){
                    if($serviceMethodKey == 'constantContactList') {
                        if(isset($services[$serviceMethodKey]['done']) && $services[$serviceMethodKey]['done'] == false) {
                            $message = '';
                            if(!empty($services[$serviceMethodKey]['message'])) {
                                $message = $services[$serviceMethodKey]['message'];
                            }
                            $this->_responseHelper->success(array('detailedInfo' => $detailedInfo, 'hasError' => '1', 'message' => $message));
                        } else {
                            if(!empty($services[$serviceMethodKey])) {
                                foreach($services[$serviceMethodKey] as $list){
                                    if(isset($list['id'])) {
                                        $lists[$list['id']] = $list['name'];
                                    }
                                }
                            }

                            $this->_responseHelper->success(array('list' => $lists, 'detailedInfo' => $detailedInfo));
                        }
                    } else {
                        if (!empty($services[$serviceMethodKey]['data'])) {
                            if (!empty($services[$serviceMethodKey]['detailedInfo'])) {
                                $detailedInfo = '1';
                            }

                            if (empty($services[$serviceMethodKey]['data'][$data['serviceName']])) {
                                $this->_responseHelper->success(array('detailedInfo' => $detailedInfo));
                            }

                            foreach($services[$serviceMethodKey]['data'][$data['serviceName']] as $list){
                                if(isset($list['id'])) {
                                    $lists[$list['id']] = $list['name'];
                                }
                            }
                        } else {
                            if (empty($services[$serviceMethodKey])) {
                                $this->_responseHelper->fail();
                            }
                            foreach($services[$serviceMethodKey] as $list){
                                if(isset($list['id'])) {
                                    $lists[$list['id']] = $list['name'];
                                }
                            }
                        }

                        $this->_responseHelper->success(array('list' => $lists, 'detailedInfo' => $detailedInfo));
                    }
                }
                $this->_responseHelper->fail();
            }
            $this->_responseHelper->fail();
        }
    }

    /**
     * Get lead/organization attributes list for the tools->score screen
     *
     * @throws Zend_Exception
     */
    public function getAttributesForScoreAction()
    {
        if (Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS)) {
            $attributesType = $this->_request->getParam('attributesType');
            $attributes = array();
            if (!empty($attributesType)) {
                if ($attributesType === Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_LEAD_ATTRIBUTE) {
                    $attributes = Tools_LeadScoreTools::getLeadScoreLeadAttributes();
                }
                if ($attributesType === Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_ORGANIZATION_ATTRIBUTE) {
                    $attributes = Tools_LeadScoreTools::getLeadScoreOrganizationAttributes();
                }

                if ($attributesType === Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_LEAD_OPPORTUNITIES) {
                    $attributes = Tools_LeadScoreTools::getLeadScoreOpportunitiesAttributes();
                }

                if ($attributesType === Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_LEAD_CUSTOM_ATTRIBUTES) {
                    $attributes = Tools_LeadScoreTools::getLeadScoreCustomAttributes(Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_LEAD_CUSTOM_ATTRIBUTES);
                }

                if ($attributesType === Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_ORGANIZATION_CUSTOM_ATTRIBUTES) {
                    $attributes = Tools_LeadScoreTools::getLeadScoreCustomAttributes(Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_ORGANIZATION_CUSTOM_ATTRIBUTES);
                }

                if ($attributesType === Leads_Model_LeadsScoreRulesConfigModel::RULE_TYPE_FORM_NAME) {
                    $attributes = array('form_name' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Form name'));
                }

                asort($attributes);
                $this->_responseHelper->success($attributes);
            }

        }

        $this->_responseHelper->fail();
    }

    public function getActionAttributesForScoreAction()
    {
        if (Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS)) {
            $actionType = $this->_request->getParam('actionType');
            $excludeField = explode(',', $this->_request->getParam('excludeField'));
            $attributes = Tools_LeadScoreTools::getActionAttributes($actionType);
            if (!empty($excludeField)) {
                foreach ($excludeField as $exclude) {
                    if (isset($attributes[$exclude])) {
                        unset($attributes[$exclude]);
                    }
                }
            }
            asort($attributes);
            $this->_responseHelper->success($attributes);
        }

        $this->_responseHelper->fail();
    }

    /**
     * Add lead ip address
     *
     * @param array $params email, lead_ip_address
     */
    public static function addExternalIpLog($params)
    {
        if (!empty($params['lead_email']) && !empty($params['lead_ip_address'])) {
            $ip = $params['lead_ip_address'];
            $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
            $leadModel = $leadsMapper->findByEmail($params['lead_email']);
            if ($leadModel instanceof Leads_Model_LeadsModel) {
                $leadId = $leadModel->getId();
                $leadsIpsModel = Tools_LeadTools::addIpAddressToLead($leadId, $ip);
                if ($leadsIpsModel instanceof Leads_Model_LeadsIpsModel) {
                        Tools_LeadTools::addIpToMojo($leadsIpsModel);
                }

            }
        }

    }

    /**
     * Response from api plugin for external email service
     *
     * @param array $emailData
     * @return string
     */
    public static function externalEmailServiceLog($emailData)
    {

        return Tools_LeadExternalEmailsTools::processEmailServiceLog($emailData);
    }

    /*
     * generate email tracking link
     *
     * @return string
     * @throws Zend_Controller_Action_Exception
     */
    public static function generateEmailTrackingLink()
    {
        return Tools_LeadEmailTools::prepareEmailTrackingLink();
    }

    /**
     * @return array
     * @throws Zend_Controller_Action_Exception
     */
    public static function getEmailTrackingLink()
    {
        return Tools_LeadEmailTools::prepareEmailTrackingLink();
    }

    /**
     * @param array $params params
     * @throws Zend_Exception
     * @return array
     */
    public static function addToEmailLog($params)
    {
        $mandatoryParams = array(
            'leadId',
            'emailContent',
            'emailMessage',
            'emailTo'
        );

        $fieldDataMissing = array_filter($mandatoryParams, function ($param) use ($params) {
            if (!array_key_exists($param, $params) || empty($params[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            return array('error' => 1, 'message' => 'Mandatory fields are missing');
        }

        if (empty($params['bccEmails'])) {
            $params['bccEmails'] = array();
        }

        if (empty($params['createdAt'])) {
            $params['createdAt'] = '';
        }

        if (empty($params['dictionary'])) {
            $params['dictionary'] = array();
        }

        if (empty($params['observableModel'])) {
            $params['observableModel'] = null;
        }

        if (empty($params['trackingCodeHash'])) {
            $params['trackingCodeHash'] = '';
        }

        if (empty($params['sequenceId'])) {
            $params['sequenceId'] = '';
        }

        return Tools_LeadEmailTools::logEmailSent($params['leadId'], $params['emailContent'], $params['emailMessage'],
            $params['trackingCodeHash'], $params['emailFrom'], $params['emailTo'], $params['emailSubject'],
            $params['bccEmails'], $params['createdAt'], $params['dictionary'], $params['observableModel'], $params['sequenceId']);

    }

    public static function addTrackableLinks($data)
    {
        if (empty($data['template'])) {
            return '';
        }

        $templateContent = $data['template'];

        $trackingCodeHash = '';
        if (!empty($data['trackingCodeHash'])) {
            $trackingCodeHash = $data['trackingCodeHash'];
        }

        $templateContent = Tools_LeadEmailTools::addTrackableLinks($templateContent, $trackingCodeHash);
        return $templateContent;
    }

    /**
     * Before system user delete message
     *
     * @return array
     * @throws Zend_Exception
     */
    public static function systemUserDeleteMessage()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
        $websiteUrl = $websiteHelper->getUrl();
        $beforeUserDeleteMessage = $translator->translate(self::TRANSLATION_PREFIX.'You are about to remove an item. Are you sure? Leads will be reassigned to the default lead owner.') . '<a target=\'blank\' href=\'' . $websiteUrl . 'dashboard/leads/\'>' . ' ' . $translator->translate(self::TRANSLATION_PREFIX.'Go to the dashboard') . '</a>';

        return array(
            'message' => $beforeUserDeleteMessage,
            'userRolesApplyTo' => self::$_administrativeRoles
        );
    }

    /**
     * Get Email Conversation details
     */
    public function getEmailConversationAction()
    {

        if (!Tools_LeadInboundEmailTools::checkImapExtension()) {
            $imapMissingMessage = $this->_translator->translate(self::TRANSLATION_PREFIX.'To use this functional you must have').' <a target="_blank" href="http://php.net/manual/en/book.imap.php">'.$this->_translator->translate(self::TRANSLATION_PREFIX.'php imap extension').'</a> '.$this->_translator->translate(self::TRANSLATION_PREFIX.'installed').'.';
            $this->_responseHelper->fail($imapMissingMessage);
        }

        if ($this->_request->isPost() && Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $secureToken = $this->_request->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
            $tokenValid = Tools_System_Tools::validateToken($secureToken, Api_Leads_Leads::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $this->_responseHelper->fail('Invalid token');
            }

            $emailReplyId = $this->_request->getParam('emailReplyId');
            if (empty($emailReplyId)) {
                $this->_responseHelper->fail('Missing reply id');
            }


            $emailConversationDetails = Tools_LeadInboundEmailTools::getEmailConversationDetails($emailReplyId);
            if (empty($emailConversationDetails)) {
                $this->_responseHelper->fail('Can\'t get conversation details');
            }

            $this->_responseHelper->success($emailConversationDetails);


        }
    }

    public function calculateLeadScoreAction()
    {
        if ($this->_request->isPost() && Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $this->_validateToken();
            $leadId = $this->_request->getParam('leadId');
            if (empty($leadId)) {
                $this->_responseHelper->fail('Missing lead id');
            }

            $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
            $scoreGeneralConfig = Leads_Mapper_LeadsScoreRulesGeneralConfigMapper::getInstance()->getScoreRulesConfigData();
            $where = $leadsMapper->getDbTable()->getAdapter()->quoteInto('pld.id = ?', $leadId);
            $leadData = $leadsMapper->fetchAll($where, null, null, null, true);
            if (!empty($scoreGeneralConfig) && !empty($leadData)) {
                $scoreGeneralConfigGroupedByRule = Tools_LeadScoreTools::prepareScoreRules($scoreGeneralConfig);
                Tools_LeadScoreTools::processLeadScore($leadData, $scoreGeneralConfigGroupedByRule);
                $leadModel = $leadsMapper->find($leadId);
                $leadScore = $leadModel->getScore();
                $this->_responseHelper->success(array(
                    'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'score has been calculated'),
                    'score' => $leadScore
                ));
            }

        }

        $this->_responseHelper->fail('');
    }

    /**
     * Get information about the leads
     */
    public function getLeadsInfoAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
            if (empty($restrictedUserId)) {
                $leadIds = explode(',', filter_var($this->_request->getParam('checkedLeads'), FILTER_SANITIZE_STRING));
                if (empty($leadIds)) {
                    return $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Please specify at least one lead id'));
                }

                $leadDataMapper = Leads_Mapper_LeadsMapper::getInstance();
                $where = $leadDataMapper->getDbTable()->getAdapter()->quoteInto('pld.id IN (?)', $leadIds);
                $leadsData = $leadDataMapper->fetchAll($where, null, null, null, true);
                if (empty($leadsData)) {
                    return $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'No leads have been found'));
                }

                return $this->_responseHelper->success($leadsData);

            }
        }
    }

    /**
     * Get information about the organizations
     */
    public function getOrganizationsInfoAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
            if (empty($restrictedUserId)) {
                $organizationIds = explode(',',
                    filter_var($this->_request->getParam('checkedOrganizations'), FILTER_SANITIZE_STRING));
                if (empty($organizationIds)) {
                    return $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Please specify at least one organization id'));
                }

                $organizationDataMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
                $where = $organizationDataMapper->getDbTable()->getAdapter()->quoteInto('plo.id IN (?)',
                    $organizationIds);
                $organizationsData = $organizationDataMapper->fetchAll($where, null, null, null, true);
                if (empty($organizationsData)) {
                    return $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'No organizations have been found'));
                }

                return $this->_responseHelper->success($organizationsData);

            }
        }
    }


    /**
     *
     * Merge bunch of organizations
     *
     */
    public function mergeMassOrganizationsAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
            $this->_validateToken();
            if (empty($restrictedUserId)) {
                $organizationIds = explode(',',
                    filter_var($this->_request->getParam('organizationIds'), FILTER_SANITIZE_STRING));
                $organizationMergeToId = filter_var($this->_request->getParam('organizationMergeToId'),
                    FILTER_SANITIZE_NUMBER_INT);
                if (empty($organizationIds)) {
                    return $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Please specify at least one organization id'));
                }

                if (empty($organizationIds) || empty($organizationMergeToId)) {
                    return $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Please specify at least one organization id and merged to id'));
                }

                foreach ($organizationIds as $organizationId) {
                    if ($organizationId != $organizationMergeToId) {
                        Tools_LeadMergeTools::mergeOrganization($organizationId, $organizationMergeToId, true);
                    }
                }

                return $this->_responseHelper->success($this->_translator->translate(self::TRANSLATION_PREFIX.'Organizations have been merged'));

            }
        }
    }

    /**
     *
     * Merge bunch of leads
     *
     */
    public function mergeMassLeadsAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
            $this->_validateToken();
            if (empty($restrictedUserId)) {
                $leadIds = explode(',', filter_var($this->_request->getParam('leadIds'), FILTER_SANITIZE_STRING));
                $leadMergeToId = filter_var($this->_request->getParam('leadMergeToId'), FILTER_SANITIZE_NUMBER_INT);
                if (empty($leadIds)) {
                    return $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Please specify at least one lead id'));
                }

                if (empty($leadIds) || empty($leadMergeToId)) {
                    return $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Please specify at least one lead id and merged to id'));
                }

                foreach ($leadIds as $leadId) {
                    if ($leadId != $leadMergeToId) {
                        Tools_LeadMergeTools::mergeLead($leadId, $leadMergeToId, 'email', true);
                    }
                }

                return $this->_responseHelper->success($this->_translator->translate(self::TRANSLATION_PREFIX.'Leads have been merged'));

            }
        }
    }

    /**
     * Mass organizations delete action
     *
     * @throws Exception
     */
    public function deleteMassOrganizationsAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
            $this->_validateToken();
            if (empty($restrictedUserId)) {
                $organizationIds = explode(',', filter_var($this->_request->getParam('organizationIds'), FILTER_SANITIZE_STRING));
                $matchingFilter = (bool) filter_var($this->_request->getParam('matchingFilter'), FILTER_SANITIZE_STRING);
                $filterQuantity = filter_var($this->_request->getParam('filterQuantity'), FILTER_SANITIZE_NUMBER_INT);
                $step = filter_var($this->_request->getParam('step'), FILTER_SANITIZE_NUMBER_INT);
                $filters = $this->_request->getParam('filters');

                $limit = '10';
                $organizationMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
                if (empty($organizationIds)) {
                    return $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Please specify at least one organization id'));
                }

                if (!empty($step)) {
                    $finalLimit = ($step+1)*$limit;
                    if ($filterQuantity < $finalLimit && empty($filterQuantity % $limit)) {
                        $this->_responseHelper->fail(array('quantity' => 0, 'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Organizations have been deleted')));
                    }
                }

                if (!empty($matchingFilter)) {
                    $organizationsData = Tools_LeadTools::getSearchOrganizationFilterData($filters, $limit);
                    if (!empty($organizationsData)) {
                        $organizationIds = array();
                        foreach ($organizationsData['data'] as $organization) {
                            array_push($organizationIds, $organization['id']);
                        }
                        $organizationIds = array_filter($organizationIds);
                    }
                }

                if (empty($organizationsData) && !empty($matchingFilter)) {
                    $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Organizations have been deleted'));
                }

                $leadsOrganizationDocumentsMapper = Leads_Mapper_LeadsOrganizationDocumentsMapper::getInstance();
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websitePath = $websiteHelper->getPath();
                $organizationLogosPath = $websitePath . Leads::ORGANIZATION_LOGOS_IMAGES_PATH;
                $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();

                foreach ($organizationIds as $organizationId) {
                    $organizationModel = $organizationMapper->find($organizationId);
                    if ($organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                        $existedDocuments = $leadsOrganizationDocumentsMapper->findDocumentsByOrganizationId($organizationModel->getId());
                        if (!empty($existedDocuments)) {
                            foreach ($existedDocuments as $document) {
                                $filePath = $organizationLogosPath . DIRECTORY_SEPARATOR . $document['fileStoredName'];

                                if(is_file($filePath)) {
                                    unlink($filePath);
                                }
                            }
                        }

                        $leadsMapper->removeOrganizationId($organizationId);
                        $organizationMapper->delete($organizationId);
                        $lastViewedLeadsAndOrgMapper = Leads_Mapper_LeadsLastViewedMapper::getInstance();
                        $lastViewedLeadsAndOrgMapper->delete($organizationId,
                            Leads_Model_LeadsLastViewedModel::LEAD_LAST_VIEWED_TYPE_ORGANIZATION);
                    }
                }

                if (count($organizationIds) == $limit && !empty($matchingFilter)) {
                    $this->_responseHelper->success(array(
                        'quantity' => count($organizationIds),
                        'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Organizations have been deleted')
                    ));
                } else {

                    $this->_responseHelper->fail(array(
                        'quantity' => count($organizationIds),
                        'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Organizations have been deleted')
                    ));

                }

                return $this->_responseHelper->success($this->_translator->translate(self::TRANSLATION_PREFIX.'Organizations have been deleted'));

            }
        }
    }


    /**
     * Mass leads delete action
     *
     * @throws Exception
     */
    public function deleteMassLeadsAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
            $this->_validateToken();
            if (empty($restrictedUserId)) {
                $leadsIds = explode(',', filter_var($this->_request->getParam('leadIds'), FILTER_SANITIZE_STRING));
                $matchingFilter = (bool) filter_var($this->_request->getParam('matchingFilter'), FILTER_SANITIZE_STRING);
                $filterQuantity = filter_var($this->_request->getParam('filterQuantity'), FILTER_SANITIZE_NUMBER_INT);
                $step = filter_var($this->_request->getParam('step'), FILTER_SANITIZE_NUMBER_INT);
                $markAsSpam = filter_var($this->_request->getParam('markAsSpam'), FILTER_SANITIZE_NUMBER_INT);
                $filters = $this->_request->getParam('filters');

                $limit = '10';
                $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
                if (empty($leadsIds)) {
                    return $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Please specify at least one lead id'));
                }

                if (!empty($step)) {
                    $finalLimit = ($step+1)*$limit;
                    if ($filterQuantity < $finalLimit && empty($filterQuantity % $limit)) {
                        $this->_responseHelper->fail(array('quantity' => 0, 'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Leads have been deleted')));
                    }
                }

                if (!empty($matchingFilter)) {
                    $leadsData = Tools_LeadTools::getSearchFilterData($filters, $limit);
                    if (!empty($leadsData)) {
                        $leadsIds = array();
                        foreach ($leadsData['data'] as $lead) {
                            array_push($leadsIds, $lead['id']);
                        }
                        $leadsIds = array_filter($leadsIds);
                    }
                }

                if (empty($leadsData) && !empty($matchingFilter)) {
                    $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Leads have been deleted'));
                }

                foreach ($leadsIds as $leadId) {
                    $leadModel = $leadsMapper->find($leadId);
                    if ($leadModel instanceof Leads_Model_LeadsModel) {
                        $emailSequenceEnabled = Tools_LeadTools::verifyPluginEnabled(Tools_LeadTools::SEQUENCE_PLUGIN);
                        if ($emailSequenceEnabled) {
                            $leadEmailHasSequenceMapper = Leads_Mapper_LeadsHasEmailSequenceMapper::getInstance();
                            $leadHasSequences = $leadEmailHasSequenceMapper->getLeadSequenceByLeadId($leadId);

                            if (!empty($leadHasSequences)) {
                                foreach ($leadHasSequences as $leadHasSequence) {
                                    $sequenceId = $leadHasSequence['sequence_id'];
                                    Emailsequence_Models_Mapper_EmailsequenceQueueMapper::getInstance()->deleteQueueBySequenceIdAndExternalId($sequenceId,
                                        $leadId, 'lead', array('0'));
                                }
                            }
                        }

                        $leadsMapper->delete($leadId);
                        $lastViewedLeadsAndOrgMapper = Leads_Mapper_LeadsLastViewedMapper::getInstance();
                        $lastViewedLeadsAndOrgMapper->delete($leadId,
                            Leads_Model_LeadsLastViewedModel::LEAD_LAST_VIEWED_TYPE_LEAD);
                    }

                    if (!empty($markAsSpam) && $leadModel instanceof Leads_Model_LeadsModel) {
                        $leadEmail = $leadModel->getLeadEmail();
                        $phoneNumber = '';
                        $ipAddress = array();
                        if (!empty($leadModel->getLeadMobile())) {
                            $phoneNumber = $leadModel->getLeadMobileCountryCodeValue().$leadModel->getLeadMobile();
                        }

                        $leadIpsMapper = Leads_Mapper_LeadsIpsMapper::getInstance();
                        $leadIpRecordExists = $leadIpsMapper->getByLeadId($leadId);
                        if ($leadIpRecordExists instanceof Leads_Model_LeadsIpsModel) {
                            $ipAddress = $leadIpRecordExists->getLeadIpAddress();
                        }

                        if (!empty($leadEmail) || !empty($phoneNumber) || !empty($ipAddress)) {
                            $response = Apps::apiCall('POST', 'appsAddLeadToSpam', array(), array(
                                'data' => array(
                                    'email' => $leadEmail,
                                    'phoneNumber' => $phoneNumber,
                                    'ipAddress' => $ipAddress
                                )
                            ), 1);
                        }
                    }
                }

                if (count($leadsIds) == $limit && !empty($matchingFilter)) {
                    $this->_responseHelper->success(array(
                        'quantity' => count($leadsIds),
                        'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Leads have been deleted')
                    ));
                } else {
                    $this->_responseHelper->fail(array(
                        'quantity' => count($leadsIds),
                        'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Leads have been deleted')
                    ));

                }

                return $this->_responseHelper->success($this->_translator->translate(self::TRANSLATION_PREFIX.'Leads have been deleted'));

            }
        }
    }



    /**
     * Get lead screen links
     *
     * @param array $userIds user ids
     * @return array
     * @throws Zend_Controller_Action_Exception
     */
    public static function getLeadLink($userIds)
    {
        return Tools_LeadTools::getLeadLink($userIds);
    }

    public function sendNotificationImapCredentialsAction()
    {
        $leadImapDataMapper = Leads_Mapper_LeadsImapConfigMapper::getInstance();
        $configsData = $leadImapDataMapper->fetchAllData(null, null, null, null, false, true);
        if (!empty($configsData) && !empty($configsData['data'])) {
            foreach ($configsData['data'] as $configData) {
                $imapPassword = $configData['password'];
                $imapHost = $configData['host'];
                $imapSsl = $configData['encryption'];
                $user = $configData['login'];
                $notificationSent = $configData['notification_sent'];
                $sendNotification = false;

                if (empty($notificationSent)) {
                    $sendNotification = true;
                }

                if (!empty($notificationSent)) {
                    $notificationSent = Tools_System_Tools::convertDateFromTimezone($notificationSent);
                    $timeNow = Tools_System_Tools::convertDateFromTimezone('-1 day');
                    if (strtotime($timeNow) >= strtotime($notificationSent)) {
                        $sendNotification = true;
                    }
                }

                if ($sendNotification === true) {

                    //process incoming emails
                    $connection = Tools_LeadInboundEmailTools::getImapConnection($imapHost, $imapSsl, $user,
                        $imapPassword);
                    if ($connection === false) {
                        $leadModel = new Leads_Model_LeadsModel();
                        $leadModel->registerObserver(new Tools_Mail_Watchdog(array(
                            'trigger' => Tools_LeadsMailWatchdog::LEAD_IMAP_CREDENTIALS_NOTIFICATION,
                            'userEmail' => $user
                        )));

                        $leadModel->notifyObservers();

                        $leadImapDataModel = $leadImapDataMapper->find($configData['id']);
                        if ($leadImapDataModel instanceof Leads_Model_LeadsImapConfigModel) {
                            $leadImapDataModel->setNotificationSent(Tools_System_Tools::convertDateFromTimezone('now'));
                            $leadImapDataModel->setPassword($imapPassword);
                            $leadImapDataMapper->save($leadImapDataModel);
                        }
                    }
                }
            }


        }
    }


    public function checkImapCredentialsAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $leadImapDataMapper = Leads_Mapper_LeadsImapConfigMapper::getInstance();
            $currentUserEmail = $this->_sessionHelper->getCurrentUser()->getEmail();
            $where = $leadImapDataMapper->getDbTable()->getAdapter()->quoteInto('plic.login = ?', $currentUserEmail);
            $configsData = $leadImapDataMapper->fetchAllData($where, null, null, null, false, true);
            if (!empty($configsData) && !empty($configsData['data'])) {
                $imapPassword = $configsData['data'][0]['password'];
                $imapHost = $configsData['data'][0]['host'];
                $imapSsl = $configsData['data'][0]['encryption'];
                $user = $configsData['data'][0]['login'];

                $connection = Tools_LeadInboundEmailTools::getImapConnection($imapHost, $imapSsl, $user,
                    $imapPassword);
                if ($connection === false) {
                    $this->_responseHelper->fail('');
                }

                $this->_responseHelper->success('');
            }
        }
    }

    /**
     * Download .csv file wirh errors
     *
     * @param $fileName
     */
    public function downloadFileAction() {
        if ($this->_request->isGet() && Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS)) {
            $fileName = $this->_request->getParam('fileName');
            if(!empty($fileName)) {
                return Tools_LeadTools::downloadFile($fileName);
            }
        }
    }

    /*
     * Check if site already connected to Mojo
     *
     */
    public function checkLeadProfileInfoAction() {
        if ($this->_request->isPost() && Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $this->_validateToken();
            $hasError = 0;
            $errorLinkMsg = '';

            $leadId = filter_var($this->_request->getParam('leadId', false), FILTER_SANITIZE_NUMBER_INT);

            $configMapper = Application_Model_Mappers_ConfigMapper::getInstance();
            $toasterConfig = $configMapper->getConfig();

            $leadsProfileMapper = Leads_Mapper_LeadsIpUserProfileLogMapper::getInstance();

            $leadsProfileData = $leadsProfileMapper->findProfileDataByLeadId($leadId);
            if(empty($leadsProfileData)) {
                if(empty($toasterConfig['websiteId']) || empty($toasterConfig['registered'])) {
                    $hasError = 1;
                    $errorMsg = $this->_translator->translate(self::TRANSLATION_PREFIX.'No web browsing data for this lead -');
                    $errorLinkMsg = $errorMsg . '<a href="https://mojo.seosamba.com/register.html" title="" target="_blank"> ' . $this->_translator->translate(self::TRANSLATION_PREFIX."Open a SeoSamba free account") . '</a>';
                } elseif (!empty($toasterConfig['websiteId']) && !empty($toasterConfig['registered'])) {
                    $leadIpsData = Leads_Mapper_LeadsIpsMapper::getInstance()->getByLeadId($leadId);
                    $hasError = 1;
                    if(!$leadIpsData instanceof Leads_Model_LeadsIpsModel) {
                        $errorLinkMsg = $this->_translator->translate(self::TRANSLATION_PREFIX.'No web browsing data for this lead');
                    } else {
                        $errorLinkMsg = $this->_translator->translate(self::TRANSLATION_PREFIX.'Please come back later. We are gathering data for that profile. This process can take up to 24 hours.');
                    }
                }
            }

            $this->_responseHelper->response(array('hasError' => $hasError, 'errorMsg' => $errorLinkMsg));
        }
    }

    /**
     * Remote authorization
     *
     * @throws Exception
     * @throws Exceptions_SeotoasterException
     */
    public function remoteAuthorizationAction()
    {
        $token = filter_var(trim($this->_request->getParam('leadAuthToken')), FILTER_SANITIZE_STRING);
        if (!empty($token) && mb_strlen($token) == 40) {
            $multiplyUsage = filter_var($this->_request->getParam('multiply', false), FILTER_SANITIZE_NUMBER_INT);
            $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
            $leadsModel = $leadsMapper->findByRemoteAuthToken($token);
            $pageModel = Application_Model_Mappers_PageMapper::getInstance()->fetchByOption(
                Shopping::OPTION_STORE_CLIENT_LOGIN,
                true
            );
            $currentUser = $this->_sessionHelper->getCurrentUser();
            if ($currentUser->getRoleId() !== Tools_Security_Acl::ROLE_GUEST) {
                if ($pageModel instanceof Application_Model_Models_Page) {
                    $this->_redirector->gotoUrl($this->_websiteUrl . $pageModel->getUrl());
                }
                $this->_redirector->gotoUrl($this->_websiteUrl);
            }
            if ($leadsModel instanceof Leads_Model_LeadsModel) {
                $userId = $leadsModel->getUserId();
                if (empty($multiplyUsage)) {
                    $leadsModel->setRemoteAuthorizationToken('');
                    $leadsMapper->save($leadsModel);
                }

                $leadEmail = $leadsModel->getLeadEmail();
                $userMapper = Application_Model_Mappers_UserMapper::getInstance();
                $userModel = null;
                if (!empty($userId)) {
                    $userModel = $userMapper->find($userId);
                } elseif (!empty($leadEmail)) {
                    $userModel = $userMapper->findByEmail($leadEmail);
                    if ($userModel instanceof Application_Model_Models_User) {
                        $userRoleId = $userModel->getRoleId();
                        if ($userRoleId === Tools_Security_Acl::ROLE_SUPERADMIN
                            || $userRoleId === Tools_Security_Acl::ROLE_ADMIN
                            || $userRoleId === Shopping::ROLE_SALESPERSON) {
                            $this->_redirector->gotoUrl($this->_websiteUrl);
                        }
                    }
                } else {
                    $this->_redirector->gotoUrl($this->_websiteUrl);
                }

                $newUser = false;
                if (!$userModel instanceof Application_Model_Models_User) {
                    $userModel = new Application_Model_Models_User();
                    $userModel->setEmail($leadEmail);
                    $userModel->setFullName($leadsModel->getLeadFirstName() . $leadsModel->getLeadLastName());
                    $userModel->setRoleId(Shopping::ROLE_CUSTOMER);
                    $userModel->setPassword(md5(microtime()));
                    $newUser = true;
                }

                $userModel->setLastLogin(date(Tools_System_Tools::DATE_MYSQL));
                $userModel->setIpaddress($_SERVER['REMOTE_ADDR']);
                if ($newUser === true) {
                    $userId = $userMapper->save($userModel);
                    $userModel->setId($userId);
                } else {
                    $userMapper->save($userModel);
                    $userModel = $userMapper->find($userModel->getId());
                    $userId = $userModel->getId();
                }

                $this->_sessionHelper->setCurrentUser($userModel);
                if ($newUser === true) {
                    $leadsModel->setUserId($userId);
                    $leadsMapper->save($leadsModel);
                }
                $cacheHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('cache');
                $cacheHelper->clean();
                if ($pageModel instanceof Application_Model_Models_Page) {
                    $this->_redirector->gotoUrl($this->_websiteUrl . $pageModel->getUrl());
                }
            }
        }

        $this->_redirector->gotoUrl($this->_websiteUrl);
    }

    /**
     * Verify whether lead with email exists
     *
     * @param array $emailData
     * @return array
     */
    public static function getLeadInfo($emailData)
    {
        if(empty($emailData['emailList'])){
            return array('emailExists' => '0');
        }
        $emailData = $emailData['emailList'];
        $emails = array_keys($emailData);
        $leadDataMapper = Leads_Mapper_LeadsMapper::getInstance();
        $leadData = $leadDataMapper->findByEmails($emails);
        if (!empty($leadData)) {
            $emailData = array_intersect_key($emailData, $leadData);

            return array('emailExists' => '1', 'emailsData' => $emailData);
        }

        return array('emailExists' => '0');
    }

    /**
     * Download uploaded document
     */
    public function downloadDocumentAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT) && $this->_request->isGet()) {
            $documentId = filter_var($this->_request->getParam('did', false), FILTER_SANITIZE_NUMBER_INT);
            $leadId = filter_var($this->_request->getParam('lid', false), FILTER_SANITIZE_NUMBER_INT);
            $userRoleId = $this->_sessionHelper->getCurrentUser()->getRoleId();

            if (!empty($documentId) && !empty($leadId)) {
                $isAllow = true;
                if($userRoleId == Shopping::ROLE_SALESPERSON) {
                    $isAllow = Tools_LeadTools::allowSales($leadId);
                }

                if($isAllow) {
                    $leadsDocumentsMapper = Leads_Mapper_LeadsDocumentsMapper::getInstance();
                    $documentData = $leadsDocumentsMapper->find($documentId);
                    if ($documentData instanceof Leads_Model_LeadsDocumentsModel) {
                        $externalUrl = $documentData->getExternalUrl();
                        if (!empty($externalUrl)) {
                            $response = Zend_Controller_Front::getInstance()->getResponse();
                            $fileToDownload = $documentData->getDisplayFileName().'.'. pathinfo($documentData->getOriginalFileName(),
                                    PATHINFO_EXTENSION);
                            $response->setHeader('Content-Disposition',
                                "attachment; filename=\"$fileToDownload\"")
                                ->setHeader('Content-type', 'application/force-download');
                            readfile($externalUrl);
                            $response->sendResponse();
                            exit;
                        } else {
                            $fileStoredName = $documentData->getFileStoredName();
                            if (!empty($fileStoredName)) {
                                $filePath = Tools_LeadDocumentsTools::getFilePath($fileStoredName);
                                if (file_exists($filePath)) {
                                    $response = Zend_Controller_Front::getInstance()->getResponse();
                                    $fileToDownload = $documentData->getDisplayFileName().'.'. pathinfo($fileStoredName,
                                            PATHINFO_EXTENSION);
                                    $response->setHeader('Content-Disposition',
                                        "attachment; filename=\"$fileToDownload\"")
                                        ->setHeader('Content-type', 'application/force-download');
                                    readfile($filePath);
                                    $response->sendResponse();
                                    exit;
                                }
                            }
                        }

                    }
                }
            }
        }
    }

    /**
     *  Delete queue for lead
     */
    public function deleteSequenceAction()
    {
        if ($this->_request->isPost() && Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
            if (empty($restrictedUserId)) {
                $this->_validateToken();
                $sequenceId = filter_var($this->_request->getParam('sequenceId'), FILTER_SANITIZE_NUMBER_INT);
                if (empty($sequenceId)) {
                    $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Missing sequence id'));
                }

                $emailSequenceEnabled = Tools_LeadTools::verifyPluginEnabled(Tools_LeadTools::SEQUENCE_PLUGIN);
                if (empty($emailSequenceEnabled)) {
                    $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Please install plugin email sequence'));
                }

                $queueMapper = Emailsequence_Models_Mapper_EmailsequenceQueueMapper::getInstance();

                $queueModel= $queueMapper->find($sequenceId);
                if ($queueModel instanceof Emailsequence_Models_Models_EmailsequenceQueueModel) {
                    if (method_exists($queueMapper, 'delete')) {
                        $queueMapper->delete($sequenceId);

                        $this->_responseHelper->success($this->_translator->translate(self::TRANSLATION_PREFIX.'Sequence has been deleted'));
                    }
                } else {
                    $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Sequence doesn\'t exist'));
                }
            }
        }
    }

    /**
     * @throws Exception
     */
    public function subscribeAction()
    {
        if ($this->_request->isPost() && Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);

            if (empty($restrictedUserId)) {
                $this->_validateToken();

                $leadId = filter_var($this->_request->getParam('leadId'), FILTER_SANITIZE_NUMBER_INT);
                if (empty($leadId)) {
                    $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Missing lead id'));
                }

                $leadsMapperLeadsMapper = Leads_Mapper_LeadsMapper::getInstance();

                $currentLead = $leadsMapperLeadsMapper->find($leadId);

                $leadEmail = $currentLead->getLeadEmail();

                $emailSequenceEnabled = Tools_LeadTools::verifyPluginEnabled(Tools_LeadTools::SEQUENCE_PLUGIN);
                if (empty($emailSequenceEnabled)) {
                    $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Please install plugin email sequence'));
                }

                $cartStatusSubscribeMapper = Emailsequence_Models_Mapper_SubscribeMapper::getInstance();
                $subscribeExists = $cartStatusSubscribeMapper->findByExternalLeadEmail($leadEmail);

                if(!empty($subscribeExists) && $subscribeExists['status'] == Emailsequence::SUBSCRIBED_STATUS_UNSUBSCRIBED) {
                    $cartStatusSubscribeMapper->updateStatus($subscribeExists['code'], Emailsequence::SUBSCRIBED_STATUS_SUBSCRIBED);

                    $this->_responseHelper->success($this->_translator->translate(self::TRANSLATION_PREFIX.'Subscribed'));
                }
            }
        }
    }

    public function sendNpsReviewRequestAction()
    {
        if ($this->_request->isPost() && Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);

            if (empty($restrictedUserId)) {
                $this->_validateToken();
                $leadId = filter_var($this->_request->getParam('leadId'), FILTER_SANITIZE_NUMBER_INT);
                if (empty($leadId)) {
                    $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Missing lead id'));
                }

                $sendRequestAgain = $this->_request->getParam('sendRequestAgain');
                if (!empty($sendRequestAgain) && $sendRequestAgain === 'yes') {
                    $token = sha1($leadId);
                    $leadsReviewRequestMapper = Leads_Mapper_LeadsReviewRequestMapper::getInstance();
                    $reviewRequestModel = $leadsReviewRequestMapper->findByToken($token);
                    if ($reviewRequestModel instanceof Leads_Model_LeadsReviewRequestModel) {
                        $reviewRequestModel->setSentAt(Tools_System_Tools::convertDateFromTimezone(date('Y-m-d', strtotime('-2 month')). ' 00:00:00'));
                        $leadsReviewRequestMapper->save($reviewRequestModel);
                    }
                }

                $leadsMapperLeadsMapper = Leads_Mapper_LeadsMapper::getInstance();
                $currentLeadModel = $leadsMapperLeadsMapper->find($leadId);
                $leadEmail = $currentLeadModel->getLeadEmail();
                if (empty($leadEmail)) {
                    $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Lead doesn\'t have email address'));
                }
                if ($currentLeadModel instanceof Leads_Model_LeadsModel) {
                    $currentLeadModel->registerObserver(new Tools_Mail_Watchdog(array(
                        'trigger' => Tools_LeadsMailWatchdog::LEADS_NPS_REVIEW_REQUEST,
                        'observableModel' => $currentLeadModel,
                        'leadData' => $currentLeadModel->toArray()
                    )));

                    $currentLeadModel->notifyObservers();
                }
                $where = $leadsMapperLeadsMapper->getDbTable()->getAdapter()->quoteInto('pld.id = ?', $currentLeadModel->getId());
                $data = $leadsMapperLeadsMapper->fetchAll($where, null, null, null, true, true);

                $restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
                $data['limitedAccess'] =  $restrictedUserId;
                $this->_responseHelper->success(array(
                    'leadData' => $data,
                    'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Request has been sent')
                ));

            }

            $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Access not allowed'));
        }
    }

    public function sendReviewRequestAction()
    {
        if ($this->_request->isPost() && Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);

            if (empty($restrictedUserId)) {
                $this->_validateToken();
                $leadId = filter_var($this->_request->getParam('leadId'), FILTER_SANITIZE_NUMBER_INT);
                if (empty($leadId)) {
                    $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Missing lead id'));
                }

                $leadModel = Leads_Mapper_LeadsMapper::getInstance()->find($leadId);
                if (!$leadModel instanceof Leads_Model_LeadsModel) {
                    $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Lead doesn\'t exist'));
                }

                $leadsReviewRequestMapper = Leads_Mapper_LeadsReviewRequestMapper::getInstance();
                $leadsReviewRequestModel  = $leadsReviewRequestMapper->findByLeadId($leadId);
                if (!$leadsReviewRequestModel instanceof Leads_Model_LeadsReviewRequestModel) {
                    $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Review request not found'));
                }
                $leadsMapperLeadsMapper = Leads_Mapper_LeadsMapper::getInstance();
                $currentLeadModel = $leadsMapperLeadsMapper->find($leadId);
                $sendRequestAgain = $this->_request->getParam('sendRequestAgain');

                if ($leadsReviewRequestModel->getReviewStatus() === Leads_Model_LeadsReviewRequestModel::STATUS_SENT && $sendRequestAgain === 'no') {
                    $where = $leadsMapperLeadsMapper->getDbTable()->getAdapter()->quoteInto('pld.id = ?', $currentLeadModel->getId());
                    $data = $leadsMapperLeadsMapper->fetchAll($where, null, null, null, true, true);

                    $this->_responseHelper->success(array(
                        'leadData' => $data,
                        'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'You can\'t send request once per month if you hasn\'t specified date yet.')
                    ));
                }

                $reviewLinks = array();
                $reviewLinksData = Apps::apiCall('GET', 'reviewRequestLink', '');
                if (empty($reviewLinksData)) {
                    $reviewLinks = array(
                        'reviewlink:facebook' => '',
                        'reviewlink:gmb' => ''
                    );
                } else {
                    $reviewLinks['reviewlink:facebook'] = $reviewLinksData['facebookLink'];
                    $reviewLinks['reviewlink:gmb'] = $reviewLinksData['gmbLink'];
                }

                if ($currentLeadModel instanceof Leads_Model_LeadsModel) {
                    $currentLeadModel->registerObserver(new Tools_Mail_Watchdog(array(
                        'trigger' => Tools_LeadsMailWatchdog::LEADS_REVIEW_REQUEST,
                        'observableModel' => $currentLeadModel,
                        'leadData' => $currentLeadModel->toArray(),
                        'reviewLinks' => $reviewLinks
                    )));

                    $currentLeadModel->notifyObservers();
                }

                $leadsReviewRequestModel->setReviewRequestSentAt(Tools_System_Tools::convertDateFromTimezone('now'));
                $leadsReviewRequestModel->setReviewStatus(Leads_Model_LeadsReviewRequestModel::STATUS_SENT);
                $leadsReviewRequestMapper->save($leadsReviewRequestModel);

                $where = $leadsMapperLeadsMapper->getDbTable()->getAdapter()->quoteInto('pld.id = ?', $currentLeadModel->getId());
                $data = $leadsMapperLeadsMapper->fetchAll($where, null, null, null, true, true);
                $data['reviewSystems'] = array(
                    'fb' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Facebook'),
                    'gmb' => $this->_translator->translate(self::TRANSLATION_PREFIX.'GMB')
                );

                $restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
                $data['limitedAccess'] =  $restrictedUserId;

                $this->_responseHelper->success(array(
                    'leadData' => $data,
                    'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Request has been sent')
                ));

            }

            $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Access not allowed'));
        }
    }

    public function npsReviewRequestAction()
    {
        $score = filter_var(trim($this->_request->getParam('score')), FILTER_SANITIZE_STRING);
        $token = filter_var(trim($this->_request->getParam('reviewToken')), FILTER_SANITIZE_STRING);
        if ($this->_request->isGet() && isset($score) && !empty($token)) {
            $leadsReviewRequestMapper = Leads_Mapper_LeadsReviewRequestMapper::getInstance();
            $reviewRequestModel = $leadsReviewRequestMapper->findByToken($token);
            if ($reviewRequestModel instanceof Leads_Model_LeadsReviewRequestModel && in_array($score,
                    Leads_Model_LeadsReviewRequestModel::$_allowedScore, true)) {
                if ($reviewRequestModel->getStatus() === Leads_Model_LeadsReviewRequestModel::STATUS_SENT) {
                    $reviewRequestModel->setStatus(Leads_Model_LeadsReviewRequestModel::STATUS_REVIEWED);
                    $reviewRequestModel->setScore($score);
                    $reviewRequestModel->setReviewedAt(Tools_System_Tools::convertDateFromTimezone('now'));
                    $leadsReviewRequestMapper->save($reviewRequestModel);
                    $leadsConfigParams = Leads_Mapper_LeadsConfigMapper::getInstance()->getConfigParams();
                    $redirectUrl = $this->_websiteUrl;
                    if (!empty($leadsConfigParams['reviewRequestRedirect'])) {
                        $pageModel = Application_Model_Mappers_PageMapper::getInstance()->find($leadsConfigParams['reviewRequestRedirect']);
                        if ($pageModel instanceof Application_Model_Models_Page) {
                            $redirectUrl .=  $pageModel->getUrl();
                        }
                    }
                    $this->_redirector->gotoUrl($redirectUrl);
                }
            }

            $this->_redirector->gotoUrl($this->_websiteUrl);
        }

    }


    public function saveReviewRequestManualAction()
    {
        if ($this->_request->isPost() && Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
            $reviewedAt = filter_var(trim($this->_request->getParam('reviewDate')), FILTER_SANITIZE_STRING);
            $reviewPlatform = filter_var(trim($this->_request->getParam('reviewPlatform')), FILTER_SANITIZE_STRING);
            if (empty($reviewedAt)) {
                $this->_responseHelper->fail(array('message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Please specify reviewed at date')));
            }

            if (empty($reviewPlatform)) {
                $this->_responseHelper->fail(array('message' =>$this->_translator->translate(self::TRANSLATION_PREFIX.'Please specify review platform')));
            }

            if (empty($restrictedUserId)) {
                $this->_validateToken();
                $leadId = filter_var($this->_request->getParam('leadId'), FILTER_SANITIZE_NUMBER_INT);
                if (empty($leadId)) {
                    $this->_responseHelper->fail(array('message' =>$this->_translator->translate(self::TRANSLATION_PREFIX.'Missing lead id')));
                }

                $reviewedAt =  Tools_System_Tools::convertDateFromTimezone($reviewedAt);

                $leadModel = Leads_Mapper_LeadsMapper::getInstance()->find($leadId);
                if (!$leadModel instanceof Leads_Model_LeadsModel) {
                    $this->_responseHelper->fail(array('message' =>$this->_translator->translate(self::TRANSLATION_PREFIX.'Lead doesn\'t exist')));
                }

                $leadsMapperLeadsMapper = Leads_Mapper_LeadsMapper::getInstance();
                $currentLeadModel = $leadsMapperLeadsMapper->find($leadId);

                $leadsReviewRequestMapper = Leads_Mapper_LeadsReviewRequestMapper::getInstance();
                $leadsReviewRequestModel  = $leadsReviewRequestMapper->findByLeadId($leadId);
                if (!$leadsReviewRequestModel instanceof Leads_Model_LeadsReviewRequestModel) {
                    $this->_responseHelper->fail(array('message' =>$this->_translator->translate(self::TRANSLATION_PREFIX.'Review request not found')));
                }

                if (Leads_Model_LeadsReviewRequestModel::PLATFORM_FACEBOOK === $reviewPlatform) {
                    $leadsReviewRequestModel->setRequestReviewedFacebookDate($reviewedAt);
                }

                if (Leads_Model_LeadsReviewRequestModel::PLATFORM_GMB === $reviewPlatform) {
                    $leadsReviewRequestModel->setRequestReviewedGmbDate($reviewedAt);
                }

                $leadsReviewRequestModel->setReviewStatus(Leads_Model_LeadsReviewRequestModel::STATUS_REVIEWED);
                $leadsReviewRequestMapper->save($leadsReviewRequestModel);

                $where = $leadsMapperLeadsMapper->getDbTable()->getAdapter()->quoteInto('pld.id = ?', $currentLeadModel->getId());
                $data = $leadsMapperLeadsMapper->fetchAll($where, null, null, null, true, true);
                $data['reviewSystems'] = array(
                    'fb' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Facebook'),
                    'gmb' => $this->_translator->translate(self::TRANSLATION_PREFIX.'GMB')
                );
                $restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
                $data['limitedAccess'] =  $restrictedUserId;
                $this->_responseHelper->success(array(
                    'leadData' => $data,
                    'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Request data has been saved')
                ));
            }
        }
    }

    public function validateEmailAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $this->_validateToken();
            $leadId = filter_var($this->_request->getParam('leadId'), FILTER_SANITIZE_NUMBER_INT);
            if (!empty($leadId)) {
                $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
                $leadModel = $leadsMapper->find($leadId);
                if ($leadModel instanceof Leads_Model_LeadsModel) {
                    $leadEmail = $leadModel->getLeadEmail();
                    if (!empty($leadEmail)) {
                        $validateData = array();
                        $validateData[] = array('leadId' => $leadModel->getId(), 'email' => $leadEmail);
                        $data = array('data' => $validateData, 'email' => $this->_sessionHelper->getCurrentUser()->getEmail());
                        $response = Apps::apiCall('POST', 'appsValidateEmail', array(), $data);
                        if (empty($response)) {
                            $this->_responseHelper->fail(array('message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Seosamba SaaS account isn\'t connected. Go to Website ID Card to connect it or ').'<u><a target="_blank" href="'.'https://mojo.seosamba.com/register.html'.'">'.$this->_translator->translate(self::TRANSLATION_PREFIX.'get one'). '</a></u>'));
                        }

                        if (!empty($response['error'])) {
                            $this->_responseHelper->fail(array('message' => ''));
                        }

                        if (empty($response['balance'])) {
                            $response = Apps::apiCall('GET', 'appsValidationCredits', array(''), array('email' => $this->_sessionHelper->getCurrentUser()->getEmail()));
                            if (empty($response['error'])) {
                                $emailBalanceLink = $response['link'];
                                $this->_responseHelper->fail(array('message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'You don\'t have enough credits.').' '.'<u><a target="_blank" href="'.$emailBalanceLink.'">'. $this->_translator->translate(self::TRANSLATION_PREFIX.'Please purchase additional credits.').'</a></u>'));
                            } else {
                                $this->_responseHelper->fail(array('message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'You don\'t have enough credits. Please purchase additional credits.')));
                            }

                        }

                        if (!empty($responseData['error'])) {
                            $this->_responseHelper->fail(array('message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'You don\'t have enough credits. Please purchase additional credits.')));
                        }

                        $responseData = current($response['data']);
                        $status = $responseData['status'];
                        $validatedDate = $responseData['timestamp'];
                        $score = $responseData['score'];
                        if (in_array($status, Leads_Model_LeadsModel::$emailValidateStatuses)) {
                            $leadModel->setEmailValidatedStatus($status);
                            $leadModel->setEmailValidatedDate($validatedDate);
                            $leadModel->setEmailValidatedScore($score);
                            $leadModel = $leadsMapper->save($leadModel);
                            $data = $leadsMapper->fetchAll($leadsMapper->getDbTable()->getAdapter()->quoteInto('pld.id = ?', $leadModel->getId()), null, null, null, true, true);
                            $this->_responseHelper->response(array('leadModel' => $data, 'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Email has been validated')));
                        }

                    }
                }
            }
        }
    }


    /**
     * Send emails to email validate service via mojo.seosamba.com
     */
    public function validateLeadEmailsAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $this->_validateToken();
            $leadIds = filter_var($this->_request->getParam('leadIds'), FILTER_SANITIZE_STRING);
            $matchingFilter = (bool) filter_var($this->_request->getParam('matchingFilter'), FILTER_SANITIZE_STRING);
            $filterQuantity = filter_var($this->_request->getParam('filterQuantity'), FILTER_SANITIZE_NUMBER_INT);
            $stepSendEmailService = filter_var($this->_request->getParam('stepSendEmailService'), FILTER_SANITIZE_NUMBER_INT);
            $filters = $this->_request->getParam('filters');
            $limit = '10';
            $offset = null;
            $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
            if (!empty($stepSendEmailService)) {
                $offset = $stepSendEmailService*$limit;
                $finalLimit = ($stepSendEmailService+1)*$limit;
                if ($filterQuantity < $finalLimit && empty($filterQuantity % $limit)) {
                    $filterResponse = array(
                        'quantity' => 0,
                        'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Lead emails have been validated')
                    );
                    $this->_responseHelper->fail($filterResponse);
                }
            }

            if (!empty($matchingFilter)) {
                $filters .= $filters.'&not_empty_email_address=1';
                $leadsData = Tools_LeadTools::getSearchFilterData($filters, $limit, $offset);
            } else {
                $leadIds = explode(',', $leadIds);
                $where = $leadsMapper->getDbTable()->getAdapter()->quoteInto('pld.id IN (?)', $leadIds);
                $where .= ' AND '. new Zend_Db_Expr("pld.lead_email <> ''");
                $leadsData = $leadsMapper->fetchAll($where, null, $limit, $offset, false, false);
            }

            if (!empty($leadsData)) {
                $leadsIds = array();
                foreach ($leadsData['data'] as $lead) {
                    array_push($leadsIds, $lead['id']);
                }
                $leadIds = implode(',', $leadsIds);
            }

            if (empty($leadsData) && !empty($matchingFilter)) {
                $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Lead emails have been added'));
            }

            $leadIds = array_filter(explode(',', $leadIds));
            if (!empty($leadIds)) {
                $leadsData = $leadsMapper->findByLeadIds($leadIds);

                if (!empty($leadsData)) {
                    $toSentData = array();
                    foreach ($leadsData as $lead) {
                        $toSentData[] = array('id' => $lead['id'], 'email' => $lead['lead_email']);
                    }

                    $data = array('data' => $toSentData, 'email' => $this->_sessionHelper->getCurrentUser()->getEmail());
                    $response = Apps::apiCall('POST', 'appsValidateEmail', array(), $data);

                    if (empty($response['error']) && !empty($response['data'])) {
                        $pointsBalance = $response['balance'];
                        $isError = false;
                        $currentLeadsQuantity = count($leadsData);
                        foreach ($response['data'] as $leadInfo) {
                            if (empty($leadInfo['error'])) {
                                $status = $leadInfo['status'];
                                $leadId = $leadInfo['id'];
                                $validatedDate = $leadInfo['timestamp'];
                                $score = $leadInfo['score'];
                                $leadModel = $leadsMapper->find($leadId);
                                if ($leadModel instanceof Leads_Model_LeadsModel && in_array($status,
                                        Leads_Model_LeadsModel::$emailValidateStatuses)) {
                                    $leadModel->setEmailValidatedStatus($status);
                                    $leadModel->setEmailValidatedDate($validatedDate);
                                    $leadModel->setEmailValidatedScore($score);
                                    $leadsMapper->save($leadModel);
                                }
                            } else {
                                $isError = true;
                                $currentLeadsQuantity = $currentLeadsQuantity - 1;
                            }
                        }

                        if (count($leadsData) == $limit && !empty($pointsBalance)) {
                            $this->_responseHelper->success(array(
                                'quantity' => count($leadIds),
                                'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Leads emails have been verified'),
                                'pointsBalance' => $pointsBalance
                            ));
                        } elseif ($isError === true) {
                            $this->_responseHelper->fail(array(
                                'quantity' => $currentLeadsQuantity,
                                'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'You don\'t have enough credits. Please purchase more credits.'),
                                'pointsBalance' => $pointsBalance
                            ));
                        } else {
                            $this->_responseHelper->fail(array(
                                'quantity' => count($leadIds),
                                'message' => $this->_translator->translate(self::TRANSLATION_PREFIX.'Leads emails have been verified'),
                                'pointsBalance' => $pointsBalance
                            ));
                        }
                    }
                    if (empty($response['data'])) {
                        $this->_responseHelper->response($this->_translator->translate(self::TRANSLATION_PREFIX.'Unknown error'), 1, 400);
                    }
                    $this->_responseHelper->response($response['message'], 1, 400);
                }
            } else {
                $requestInvalidLeads = $this->_request->getParam('invalidLeads');
                if (!empty($requestInvalidLeads)) {
                    $this->processErrorLogData($requestInvalidLeads, 0);
                }
            }
        }

        $this->_responseHelper->fail('');
    }




    public static function addRemoteLeadNotes($data)
    {

//        $data = array(
//            'leadId' => '134',
//            'notes' => 'some text here',
//            'ownerId' => '1'
//        );

        if (empty($data) || empty($data['leadId'])) {
            return array('error' => '1', 'message' => 'Missing lead id');
        }

        $userId = '0';

        if (!empty($data['ownerId'])) {
            $userId = $data['ownerId'];
        }

        $noteModel = Tools_LeadTools::processNotes($data['leadId'], $data['notes'], $userId);

        return array('id' => $noteModel->getId());
    }

    public static function deleteRemoteLeadNotes($data)
    {

//        $data = array(
//            'noteId' => '13444'
//        );

        if (empty($data) || empty($data['noteId'])) {
            return array('error' => '1', 'message' => 'Missing note id');
        }

        $leadsNoteMapper = Leads_Mapper_LeadsNoteMapper::getInstance();
        $leadNoteModel = $leadsNoteMapper->find($data['noteId']);
        if ($leadNoteModel instanceof Leads_Model_LeadsNoteModel) {
            $leadsNoteMapper->delete($data['noteId']);
        }

    }

    public static function deleteMobileAppRecord($data)
    {

//        $data = array(
//            'email' => 'andrei@seosamba.com'
//        );

        if (empty($data) || empty($data['email'])) {
            return array('error' => '1', 'message' => 'Missing email address');
        }

        $leadMobileAppDataMapper = Leads_Mapper_LeadsMobileAppConfigMapper::getInstance();
        $leadsMobileAppConfigModel = $leadMobileAppDataMapper->findByEmail($data['email']);
        if ($leadsMobileAppConfigModel instanceof Leads_Model_LeadsMobileAppConfigModel) {
            $leadMobileAppDataMapper->delete($leadsMobileAppConfigModel->getId());
        }

    }


    /**
     *
     *
     * @param array $data
     */
    public static function searchLeadByParams($data)
    {

//        $data = array(
//            'searchParams' =>
//                array(
//                    'lead-attribute' => '+5755555555555',
//                    'filter-by' => 'attributes',
//                    //'ownerId' => '10',
//                    'fullMobile' => '1',
//                    'excludeLeadIds' => array('134, 145')
//                ),
//            'limit' => '5',
//            'offset' => '0'
//        );

        $searchParams = $data['searchParams'];
        $limit = null;
        $offset = null;
        if (!empty($data['limit'])) {
            $limit = $data['limit'];
        }

        if (isset($data['offset'])) {
            $offset = $data['offset'];
        }

        if (!empty($searchParams['ownerId'])) {
            $restrictedAccess = Tools_LeadTools::isAccessLimitedByUserId(Shopping::ROLE_SALESPERSON, $searchParams['ownerId']);
            if ($restrictedAccess === true) {
                $searchParams['owner_id'] = $searchParams['ownerId'];
            } else {
                unset($searchParams['owner_id']);
                unset($searchParams['ownerId']);
            }
        }

        $searchFullNumberFlag = false;
        if (!empty($data['searchParams']) && !empty($data['searchParams']['fullMobile'])) {
            $searchFullNumberFlag = true;
        }

        $result = Tools_LeadTools::getSearchFilterData($searchParams, $limit, $offset, false, false, $searchFullNumberFlag);

        if (!empty($result) && !empty($result['data']) && $searchFullNumberFlag === true) {
            $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
            $currency = Zend_Registry::get('Zend_Currency');
            $moneyFormat = Tools_LeadTools::getCurrencyFormat();
            foreach ($result['data'] as $key => $res) {
                $where = $leadsMapper->getDbTable()->getAdapter()->quoteInto('plo.lead_id = ?', $res['id']);
                $leadOpportunityDataMapper = Leads_Mapper_LeadsOpportunityMapper::getInstance();
                $opportunityData = $leadOpportunityDataMapper->getOpportunitiesForTimeline($where, null, null, null,
                    true, false, '', '', false);
                $result['data'][$key]['opportunityData'] = $opportunityData;
                $result['data'][$key]['currencySymbol'] = preg_replace('~[\w]~', '', $currency->getSymbol());
                $result['data'][$key]['moneyFormat'] = json_encode($moneyFormat);
            }
        }

        $configHelper = new Helpers_Action_Config();
        $configHelper->init();
        $userDefaultMobileCountryCode = $configHelper->getConfig('userDefaultPhoneMobileCode');
        $result['mobileCountryCode'] = $userDefaultMobileCountryCode;
        $shoppingConfig = Models_Mapper_ShoppingConfig::getInstance()->getConfigParams();
        if (!empty($shoppingConfig['country'])) {
            $result['countryCode'] = $shoppingConfig['country'];
        } else {
            $result['countryCode'] = '';
        }

        return $result;
    }

    public static function getGeneralInfo()
    {
        $result = array();
        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
        $leadStageDataMapper = Leads_Mapper_LeadsStageMapper::getInstance();
        $result['customerGroups'] = $leadsMapper->getCustomerGroups();
        $result['lifecycleStages'] = $leadStageDataMapper->getStages();

        return $result;
    }


    public static function addLeadInfo($data)
    {

//        $data = array(
//            'ownerId' => '10',
//            'leadEmail' => 'andrei555testlead@seosamba.com',
//            'leadMobile' => '4123777122',
//            'leadPhone'  => '5543888534',
//            'leadPhoneCountryCode' => 'US',
//            'leadMobileCountryCode' => 'FR',
//            'customerGroupId' => '10',
//            'leadCountryCode' => 'US',
//            'leadStateCode' => 'AL',
//            'organizationName' => 'my test org',
//            'leadLastName' => 'last name',
//            'leadFirstName' => 'first name',
//            'leadPosition' => 'doctor',
//            'leadAddress' => 'address',
//            'leadAddress1' => 'address 1',
//            'leadCity' => 'new york',
//            'leadZip' => '123123',
//            'stageId' => '36'
//        );

        return Tools_LeadMobileApiTools::addLeadInfo($data);


    }

    public static function editLeadInfo($data)
    {
//        $data = array(
//            'leadId' => '134',
//            'leadEmail' => 'andrei@seosamba.com',
//            'leadMobile' => '4123123122',
//            'leadPhone'  => '5543534534',
//            'leadPhoneCountryCode' => 'US',
//            'leadMobileCountryCode' => 'FR',
//            'customerGroupId' => '10',
//            'leadCountryCode' => 'US',
//            'leadStateCode' => 'AL',
//            'organizationName' => 'my test org',
//            'leadLastName' => 'last name',
//            'leadFirstName' => 'first name',
//            'leadPosition' => 'doctor',
//            'leadAddress' => 'address',
//            'leadAddress1' => 'address 1',
//            'leadCity' => 'new york',
//            'leadZip' => '123123',
//            'stageId' => '36'
//        );

        return Tools_LeadMobileApiTools::editLeadInfo($data);

    }

    public static function addLeadTag($data)
    {

//        $data = array(
//            'leadId' => '281', 'tagId' => array('3', '6')
//        );
        //$data = array('leadId' => '281', 'label' => 'test tag label 1');
        if (empty($data['leadId'])) {
            return array('error' => '1', 'message' => 'Missing lead id');
        }

        if (empty($data['label']) && empty($data['tagId'])) {
            return array('error' => '1', 'message' => 'You must specify tag label or tag id');
        }

        $leadId = $data['leadId'];

        if (!empty($data['label'])) {
            $tagLabel = preg_replace('~[^\w\s\_]~ui', '', trim($data['label']));
            if (empty($tagLabel)) {
                return array('error' => '1', 'message' => 'tag label must be not empty');
            }

            $res = Tools_LeadMobileApiTools::addLeadTagByLabel($leadId, $tagLabel);
            $res['id'] = (string) $res['id'];
            return $res;
        }

        $tagId = $data['tagId'];
        $tagResponse = array();
        if (is_array($tagId)) {
            $leadsTagsMapper = Leads_Mapper_LeadsTagsMapper::getInstance();
            $numberOfTags = count($tagId);
            $where = $leadsTagsMapper->getDbTable()->getAdapter()->quoteInto('plt.id IN (?)', $tagId);
            $foundTags = $leadsTagsMapper->fetchAll($where);
            if ($numberOfTags != $foundTags['totalRecords']) {
                return array('error' => '1', 'message' => 'Wrong tag id specified.');
            }

            foreach ($tagId as $id){
                $res = Tools_LeadMobileApiTools::addTagToLead($leadId, $id);
                $res['id'] = (string) $res['id'];
                $tagResponse[]  = $res;
            }

            return $tagResponse;
        }

        $result =  Tools_LeadMobileApiTools::addTagToLead($leadId, $tagId);
        $result['id'] =  (string) $result['id'];
        return $result;
    }

    public static function deleteLeadTag($data)
    {
        //$data = array('leadId' => '281', 'tagId' => '6');
        if (empty($data['leadId'])) {
            return array('error' => '1', 'message' => 'Missing lead id');
        }

        if (empty($data['tagId'])) {
            return array('error' => '1', 'message' => 'Missing tag id');
        }

        $leadId = $data['leadId'];
        $tagId = $data['tagId'];

        $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
        $leadTagExists = $leadMapper->getByLeadIdTagId($leadId, $tagId);
        if (!empty($leadTagExists)) {
            $leadMapper->deleteTag($leadId, $tagId);
            return array('error' => '0', 'message' => 'Tag has been deleted');
        } else {
            return array('error' => '1', 'message' => 'Lead does not have this tag');
        }

    }


        public static function deleteLead($data)
    {
//        $data = array(
//            'leadId' => '9288'
//        );

        if (empty($data) && empty($data['leadId'])) {
                return array('error' => '1', 'message' => 'Missing lead id');
        }

        $leadId = $data['leadId'];

        return Tools_LeadMobileApiTools::deleteLead($leadId);
    }

    public static function getLeadTags($data)
    {
        //$data = array('leadId' => 11);
        if (!empty($data) && !empty($data['leadId'])) {
            $leadId = $data['leadId'];
        }

        if (!empty($leadId)) {
            $tagsList = Leads_Mapper_LeadsTagsMapper::getInstance()->getTagsForLeadApi($leadId);
        } else {
            $tagsList = Leads_Mapper_LeadsTagsMapper::getInstance()->getTagsForApi();
        }

        return array(
            'tags' => $tagsList
        );
    }

    public static function getMobileAppVersion()
    {

        $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
        $enableLeadOrgGeotagging = $configHelper->getConfig('enableLeadOrgGeotagging');
        $configMapper = Application_Model_Mappers_ConfigMapper::getInstance();
        $configParams = $configMapper->getConfig();
        if (!empty($configParams['crmGoogleMapKey'])) {
            $crmGoogleMapKey = $configParams['crmGoogleMapKey'];
        }

        $mapGoogleApiStatus = 0;
        if (empty($crmGoogleMapKey)) {
            $crmGoogleMapKey = '';
            $mapGoogleApiStatus = 1;
        }

        if (empty($enableLeadOrgGeotagging)) {
            $crmGoogleMapKey = '';
            $mapGoogleApiStatus = 2;
        }

        $leadsOpportunityStageMapper = Leads_Mapper_LeadsOpportunityStageMapper::getInstance();
        $opportunityTypes = $leadsOpportunityStageMapper->getOpportunityStages();
        $tagsList = Leads_Mapper_LeadsTagsMapper::getInstance()->getTagsForApi();

        return array(
            'version' => '1.7',
            'apiVersion' => '2',
            'mapGoogleApiKey' => $crmGoogleMapKey,
            'mapGoogleApiStatus' => $mapGoogleApiStatus,
            'opportunityTypes' => $opportunityTypes,
            'tags' => $tagsList
        );
    }

    public static function searchOrganization($data){
//        $data = array(
//            'organizationName' => 'test'
//        );

        if (empty($data) || empty($data['organizationName'])) {
            return array('error' => '1', 'message' => 'Missing organization name');
        }


        $orgNames = array();
        $result = Tools_LeadMobileApiTools::searchOrganization($data['organizationName']);

        if (!empty($result)) {
            foreach ($result as $orgRes) {
                $orgNames[$orgRes['id']] = $orgRes;
            }
        }

        return $orgNames;
    }

    public static function getLeadInfoByCoordinates($data)
    {
//        $data = array(
//            //'ownerId' => '10',
//            'lat' => '36.77',
//            'lng' => '-119',
//            'type' => 'opportunity',
//            //'opportunityTypes' => array('10', '20', '50'),
//            'radius' => '50',
//            'latitudeStart' => '51.014804883423714',
//            'latitudeEnd' => '16.993936188519',
//            'longitudeStart' => '51.15857551239104',
//            'longitudeEnd' => '17.11753237992525',
//        );

        $limit = null;
        $byRadius = true;
        if (!empty($data['latitudeStart']) && !empty($data['latitudeEnd']) && !empty($data['longitudeStart']) && !empty($data['longitudeEnd'])) {
            $byRadius = false;
            $latitudeStart = $data['latitudeStart'];
            $latitudeEnd = $data['latitudeEnd'];
            $longitudeStart = $data['longitudeStart'];
            $longitudeEnd = $data['longitudeEnd'];
            $limit = $data['limit'];
        } elseif (!empty($data['lat']) && !empty($data['lng'])) {
            $byRadius = true;
        } elseif (empty($data) || empty($data['lat']) || empty($data['lng'])) {
            return array('error' => '1', 'message' => 'Missing coordinates');
        }

        $opportunityTypes = array();

        if (!empty($data['opportunityTypes'])) {
            $opportunityTypes = $data['opportunityTypes'];
        }

        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();
        $where = $leadsMapper->getDbTable()->getAdapter()->quoteInto('pld.lat <> ?', '');
        $where .= ' AND ' . $leadsMapper->getDbTable()->getAdapter()->quoteInto('pld.lng <> ?', '');

        $type = '';
        if (!empty($data['type'])) {
            $type = $data['type'];
        }


        if ($byRadius === true) {
            $lat = $data['lat'];
            $lng = $data['lng'];

            $radius = '100';

            if (!empty($data['radius'])) {
                $radius = $data['radius'];
            }
        }

        if (!empty($data['ownerId'])) {
            $restrictedAccess = Tools_LeadTools::isAccessLimitedByUserId(Shopping::ROLE_SALESPERSON, $data['ownerId']);
            if ($restrictedAccess === true) {
                if (!empty($where)) {
                    $where .= ' AND '. $leadsMapper->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $data['ownerId']);
                } else {
                    $where .= $leadsMapper->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $data['ownerId']);
                }

            }
        }

        $currentTime = Tools_System_Tools::convertDateFromTimezone('now', false, 'UTC', 'Y-m-d');

        if (!empty($type) && $type !== 'all') {
            if (!empty($where)) {
                $where .= ' AND '. new Zend_Db_Expr('plop.id IS NOT NULL');
            } else {
                $where .= new Zend_Db_Expr('plop.id IS NOT NULL');
            }

            $where .= ' AND '. $leadsMapper->getDbTable()->getAdapter()->quoteInto('plop.expected_close_date >= ?', $currentTime);
        }

        if (!empty($opportunityTypes)) {
            if (!empty($where)) {
                $where .= ' AND '. $leadsMapper->getDbTable()->getAdapter()->quoteInto('plop.opportunity_stage_id IN(?)', $opportunityTypes);
            } else {
                $where .= $leadsMapper->getDbTable()->getAdapter()->quoteInto('plop.opportunity_stage_id IN(?)', $opportunityTypes);
            }
        }

        $order = null;

        if ($byRadius === true) {
            $result = $leadsMapper->findInRadius($lat, $lng, $radius, $where);
        } else {
            $result = $leadsMapper->findInRadiusPredefinedCoord($latitudeStart, $latitudeEnd, $longitudeStart, $longitudeEnd, $where, $order);
            if (!empty($result)) {
                if (count($result) >= $limit) {
                    shuffle($result);
                    $result = array_slice($result, 0, (int) $limit);
                }
            }
        }
        if (empty($result)) {
            return array();
        }

        //if (!empty($type) && $type !== 'all') {
            foreach ($result as $key =>  $leadData) {
                $leadOpportunityDataMapper = Leads_Mapper_LeadsOpportunityMapper::getInstance();
                $where = $leadsMapper->getDbTable()->getAdapter()->quoteInto('plo.lead_id = ?', $leadData['id']);
                $where .= ' AND '. $leadsMapper->getDbTable()->getAdapter()->quoteInto('plo.opportunity_stage_id NOT IN (?)', array('1', '100'));
                $where .= ' AND '. $leadsMapper->getDbTable()->getAdapter()->quoteInto('plo.expected_close_date >= ?', $currentTime);
                if (!empty($opportunityTypes)) {
                    $where .= ' AND '.$leadsMapper->getDbTable()->getAdapter()->quoteInto('plo.opportunity_stage_id IN(?)', $opportunityTypes);
                }
                $opportunityData = $leadOpportunityDataMapper->getOpportunitiesForTimeline($where, null, null, null, true);
                if (!empty($opportunityData)) {
                    $result[$key]['opportunityData'] = $opportunityData;
                } else {
                    $result[$key]['opportunityData'] = array();
                }

            }
        //}

        return $result;

    }


    public static function getLeadDetails($data)
    {
//        $data = array(
//            'leadId' => '134',
//            'ownerId' => '10'
//        );

        if (empty($data) || empty($data['leadId'])) {
            return array('error' => '1', 'message' => 'Missing lead id');
        }

        $leadId = $data['leadId'];

        $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();

        $where = $leadsMapper->getDbTable()->getAdapter()->quoteInto('pld.id = ?', $leadId);
        if (!empty($data['ownerId'])) {
            $restrictedAccess = Tools_LeadTools::isAccessLimitedByUserId(Shopping::ROLE_SALESPERSON, $data['ownerId']);
            if ($restrictedAccess === true) {
                $where .= ' AND '. $leadsMapper->getDbTable()->getAdapter()->quoteInto('pld.owner_id = ?', $data['ownerId']);
            }
        }

        $result = $leadsMapper->fetchAll($where, array(), null, null, true, true, false, '', false, false, false,false, false, false,false, false,
            false, false, array(), array(), false);
        if (empty($result)) {
            return array();
        }

        if (!empty($result['stage_id'])) {
            $leadStageModel = Leads_Mapper_LeadsStageMapper::getInstance()->find($result['stage_id']);
            if ($leadStageModel instanceof Leads_Model_LeadsStageModel) {
                $result['stageName'] = $leadStageModel->getStageName();
            }
        }

        if (empty($result['lead_state_code'])) {
            $result['lead_state_code'] = '';
        }

        if (!empty($result['lead_address'])) {
            $result['lead_location_address'] = $result['lead_address'] . ', ' . $result['lead_city'] . ', ' . $result['lead_state_code'] . ' ' . $result['lead_zip'] . ', ' . $result['lead_country_code'];
        } else {
            $result['lead_location_address'] = null;
        }

        $result['organization_location_address'] = null;

        if (!empty($result['organization_id'])) {
            $leadOrganizationDataMapper = Leads_Mapper_LeadsOrganizationMapper::getInstance();
            $organizationModel = $leadOrganizationDataMapper->find($result['organization_id']);
            if ($organizationModel instanceof Leads_Model_LeadsOrganizationModel) {
                $organizationModelData = $organizationModel->toArray();
                if (!empty($organizationModelData['organizationAddress'])) {
                    $result['organization_location_address'] = $organizationModelData['organizationAddress'] . ', ' . $organizationModelData['organizationCity'] . ', ' . $organizationModelData['organizationStateCode'] . ' ' . $organizationModelData['organizationZip'] . ', ' . $organizationModelData['organizationCountryCode'];
                }

                $result['organization_address'] = $organizationModelData['organizationAddress'];
                $result['organization_address1'] = $organizationModelData['organizationAddress1'];
                $result['organization_city'] = $organizationModelData['organizationCity'];
                $result['organization_zip'] = $organizationModelData['organizationZip'];
                $result['organization_state'] = $organizationModelData['organizationStateCode'];
                $result['organization_country_code'] = $organizationModelData['organizationCountryCode'];
            }

        }

        $where = $leadsMapper->getDbTable()->getAdapter()->quoteInto('pln.lead_id = ?', $leadId);
        $leadNotesDataMapper = Leads_Mapper_LeadsNoteMapper::getInstance();
        $orderNotes = 'pln.date_created DESC';
        $notesData = $leadNotesDataMapper->fetchAllData($where, $orderNotes, null, null, false);
        if (!empty($notesData)) {
            $result['notes'] = $notesData['data'];
        } else {
            $result['notes'] = array();
        }

        $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
        $result['websiteUrl'] = $websiteHelper->getUrl();
        $result['socialServices'] = Leads_Mapper_LeadsSocialServicesConfigMapper::getInstance()->getServicesData();
        $result['leadSocialServices'] = $leadsMapper->getLeadSocialServicesData($leadId);
        $result['leadSocialServicesNames'] = $leadsMapper->getLeadSocialServicesNamesData($leadId);
        $leadOpportunityDataMapper = Leads_Mapper_LeadsOpportunityMapper::getInstance();
        $where = $leadsMapper->getDbTable()->getAdapter()->quoteInto('plo.lead_id = ?', $leadId);
        $opportunityData = $leadOpportunityDataMapper->getOpportunitiesForTimeline($where, null, null, null, true, false, '', '', false);
        $result['opportunityData'] = $opportunityData;
        $result['groups'] = $leadsMapper->getCustomerGroups();

        $timeLineType = '';
        $dateTodayEnd = Tools_System_Tools::convertDateFromTimezone(date('Y-m-d', strtotime('+1 day')). ' 00:00:00');
        $serverTimezone = date_default_timezone_get();
        if (empty($serverTimezone)) {
            $serverTimezone = 'UTC';
        }
        $dateTodayStart = Tools_System_Tools::convertDateFromTimezone('now', $serverTimezone, 'UTC');
        $currency = Zend_Registry::get('Zend_Currency');

        $sortOrder = 'plal.created_at DESC';
        $limit = 500;

        $timelineData = Tools_LeadTools::getTimelineData($limit, null, $sortOrder, null, $timeLineType, $leadId, $dateTodayEnd, $dateTodayStart, array(), false);

        if (!empty($timelineData)) {
            $timelineData['users'] = Leads_Mapper_LeadsMapper::getInstance()->getAllUsers(true);
            $timelineData['currentUserId'] = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();
            $timelineData['currencySymbol'] = preg_replace('~[\w]~', '', $currency->getSymbol());
            $moneyFormat = Tools_LeadTools::getCurrencyFormat();
            $timelineData['moneyFormat'] = json_encode($moneyFormat);
        } else {
            $timelineData = array();
        }

        if (!empty($timelineData)) {
            if (!empty($timelineData['data']) && is_array($timelineData['data'])) {
                foreach ($timelineData['data'] as $key => $dataEl) {
                    if (!empty($timelineData['data'][$key]['emailUid'])) {
                        $canBeEncoded = json_encode(base64_decode($timelineData['data'][$key]['encodedContent']));
                        if ($canBeEncoded === false) {
                            if (function_exists('utf8_decode')) {
                                if (empty($timelineData['data'][$key]['emailContent'])) {
                                    $timelineData['data'][$key]['emailContent'] = utf8_decode(base64_decode($timelineData['data'][$key]['encodedContent']));
                                }
                            } else {
                                $timelineData['data'][$key]['encodedContent'] = '';
                            }
                        } else {
                            if (empty($timelineData['data'][$key]['emailContent'])) {
                                $timelineData['data'][$key]['emailContent'] = base64_decode($timelineData['data'][$key]['encodedContent']);
                            }
                        }
                    }
                }
            }

            $isQuizzInstalled = Tools_LeadTools::isPluginInstalled('Quizz');
            if ($isQuizzInstalled === true) {
                $isQuizzEnabled = Tools_LeadTools::isPluginEnabled('Quizz');
                if (!$isQuizzEnabled) {
                    $timelineData['data'] = self::removeQuizzItem($timelineData['data']);
                    $timelineData['totalRecords'] = count($timelineData['data']);
                }
            }

            $timelineData = $timelineData['data'];
        }

        $result['timelineData'] = $timelineData;
        return $result;

    }

    public static function removeQuizzItem($data)
    {
        $result = array();
        if (!empty($data) && is_array($data)) {
            foreach ($data as $item) {
                if ($item['type'] != 'quizz') {
                    $result[] = $item;
                }
            }
        }
        return $result;
    }


    public function salesImapConfigAction()
    {
        if (Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS)) {
            if ($this->_request->isGet()) {
                $this->_layout->content = $this->_view->render('sales-imap-config.phtml');
                echo $this->_layout->render();
            }
        }
    }

    public function mobileAppConfigAction()
    {
        if (Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS)) {
            if ($this->_request->isGet()) {
                $connection = Apps::apiCall('get', 'websiteConnection', '');
                $this->_view->connected = false;
                if (!empty($connection['done'])) {
                    $this->_view->connected = true;
                }
                $this->_layout->content = $this->_view->render('mobile-app-config.phtml');
                echo $this->_layout->render();
            }
        }
    }

    public function importContactsDataConfigAction()
    {
        if (Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS)) {
            if ($this->_request->isGet()) {
                $this->_layout->content = $this->_view->render('import-contacts-data-config.phtml');

                echo $this->_layout->render();
            }

        }
    }

    public function advertiserAccountConfigAction()
    {
        if (Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS)) {
            if ($this->_request->isGet()) {
                $connection = Apps::apiCall('get', 'websiteConnection', '');
                $this->_view->connected = false;
                if (!empty($connection['done'])) {
                    $this->_view->connected = true;
                }
                $this->_layout->content = $this->_view->render('advertiser-account-config.phtml');
                echo $this->_layout->render();
            }
        }
    }

    public function goalConfigAction()
    {
        if (Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS)) {
            if ($this->_request->isGet()) {
                echo $this->_view->render('goal-config.phtml');
            }
        }
    }

    public function getQuoteNamesAction()
    {
        if (Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS)) {
            if ($this->_request->isGet()) {
                $leadsOpportunityQuoteMapper = Leads_Mapper_LeadsOpportunityQuoteMapper::getInstance();
                $searchTerm = filter_var($this->_request->getParam('searchTerm'), FILTER_SANITIZE_STRING);
                $where = $leadsOpportunityQuoteMapper->getDbTable()->getAdapter()->quoteInto('sq.title LIKE ?',
                    $searchTerm . '%');
                $data = $leadsOpportunityQuoteMapper->searchQuotes($where, null, null, null, true);

                echo json_encode($data);
            }
        }
    }

    public function getIndependentQuoteNamesAction()
    {
        if (Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS)) {
            if ($this->_request->isGet()) {
                $leadsOpportunityQuoteMapper = Leads_Mapper_LeadsOpportunityQuoteMapper::getInstance();
                $leadId = filter_var($this->_request->getParam('leadId'), FILTER_SANITIZE_NUMBER_INT);

                $searchTerm = filter_var($this->_request->getParam('searchTerm'), FILTER_SANITIZE_STRING);
                $where = $leadsOpportunityQuoteMapper->getDbTable()->getAdapter()->quoteInto('sq.title LIKE ?', $searchTerm . '%');

                $data = $leadsOpportunityQuoteMapper->getQuotesWithoutLeads($leadId, $where);

                echo json_encode($data);
            }
        }
    }

    public function assignExistedQuoteOpportunutyAction()
    {
        if ($this->_request->isPost() && Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $secureToken = $this->_request->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
            $tokenValid = Tools_System_Tools::validateToken($secureToken, Api_Leads_Leads::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $this->_responseHelper->fail('Invalid token');
            }

            $quoteId = filter_var($this->_request->getParam('quote_id'), FILTER_SANITIZE_STRING);
            $opportunityId = filter_var($this->_request->getParam('opportunity_id'), FILTER_SANITIZE_NUMBER_INT);
            $leadId = filter_var($this->_request->getParam('lead_id'), FILTER_SANITIZE_NUMBER_INT);

            if (empty($quoteId)) {
                $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Missing quote id'));
            }

            if (empty($opportunityId)) {
                $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Missing opportunity id'));
            }

            if (empty($leadId)) {
                $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Missing lead id'));
            }

            $quoteMapper = Quote_Models_Mapper_QuoteMapper::getInstance();
            $quoteModel = $quoteMapper->find($quoteId);

            if ($quoteModel instanceof Quote_Models_Model_Quote) {
                $createdAt = Tools_System_Tools::convertDateFromTimezone('now');
                $leadsOpportunityQuoteMapper = Leads_Mapper_LeadsOpportunityQuoteMapper::getInstance();
                $leadOpportunityQuoteModel = new Leads_Model_LeadsOpportunityQuoteModel();
                $leadOpportunityQuoteModel->setQuoteId($quoteId);
                $leadOpportunityQuoteModel->setOpportunityId($opportunityId);
                $leadOpportunityQuoteModel->setLeadId($leadId);
                $leadOpportunityQuoteModel->setCartId($quoteModel->getCartId());
                $leadOpportunityQuoteModel->setCreatedAt($createdAt);
                $leadsOpportunityQuoteMapper->save($leadOpportunityQuoteModel);

                $this->_responseHelper->success($this->_translator->translate(self::TRANSLATION_PREFIX.'Done'));
            } else {
                $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Quote not found'));
            }
        }
    }

    public static function assignLeadGroup($data)
    {
        if (!empty($data['userId'])) {
            $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
            $leadModel = $leadMapper->findByUserId($data['userId']);
            if ($leadModel instanceof Leads_Model_LeadsModel) {
                if (!empty($data['groupId'])) {
                    $leadModel->setCustomerGroupId($data['groupId']);
                } else {
                    $leadModel->setCustomerGroupId(null);
                }
                $leadMapper->save($leadModel);
            }
        }
    }

    public function getOpportunityStatsOnlyAction()
    {
        $secureToken = $this->_request->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $this->_redirector->gotoUrl(Zend_Controller_Action_HelperBroker::getStaticHelper('website')->getUrl());
        }

        $leadId = $this->_request->getParam('leadId');

        $leadOpportunityDataMapper = Leads_Mapper_LeadsOpportunityMapper::getInstance();
        $data['leadOpportunityStats'] = $leadOpportunityDataMapper->getLeadOpportunityQuoteStatSalesDetailedView($leadId);

        return $this->_responseHelper->success($data);
    }

    /**
     * Allow Industries Creation through external services
     *
     * @throws Exceptions_SeotoasterPluginException
     */
    public function allowIndustriesCreationAction()
    {
        if ($this->_request->isPost() && Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
            if (empty($restrictedUserId)) {
                $this->_validateToken();
                $allowIndustriesCreation = filter_var($this->_request->getParam('allowIndustriesCreation'), FILTER_SANITIZE_NUMBER_INT);

                $leadGeneralConfigMapper = Leads_Mapper_LeadsConfigMapper::getInstance();
                $leadGeneralConfigMapper->save(array('allowIndustriesCreation' => $allowIndustriesCreation));

                $this->_responseHelper->success();
            }
        }
    }

    public function getCurrentDateAction()
    {
        $currentDate = Tools_System_Tools::convertDateFromTimezone('now', false, 'UTC', 'd-M-Y');
        $sessionHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
        $userId = $sessionHelper->getCurrentUser()->getId();
        $userModel = Application_Model_Mappers_UserMapper::getInstance()->find($userId);
        $userTimeZone = $userModel->getTimezone();
        $currentDateMinutes = Tools_System_Tools::convertDateFromTimezone('+1 hour', false, $userTimeZone, 'i');
        $currentHourPlus = '+1 hour';
        if ($currentDateMinutes > 30) {
            $currentHourPlus = '+2 hour';
        }
        $currentDateHour = Tools_System_Tools::convertDateFromTimezone($currentHourPlus, false, $userTimeZone, 'H');

        return $this->_responseHelper->success(array('currentDate' => $currentDate, 'dateDue' => $currentDateHour));
    }


    /**
     * @param $data
     */
    public static function emailValidation($data)
    {
        if (!empty($data) && !empty($data['data'])) {
            $leadsMapper = Leads_Mapper_LeadsMapper::getInstance();

            foreach ($data['data'] as $dataInfo) {
                $status = $dataInfo['status'];
                $email = $dataInfo['email'];
                $validatedDate = $dataInfo['timestamp'];
                $score = $dataInfo['score'];
                $leadModel = $leadsMapper->findByEmail($email);
                if ($leadModel instanceof Leads_Model_LeadsModel && in_array($status,
                        Leads_Model_LeadsModel::$emailValidateStatuses)) {
                    $leadModel->setEmailValidatedStatus($status);
                    $leadModel->setEmailValidatedDate($validatedDate);
                    $leadModel->setEmailValidatedScore($score);
                    $leadsMapper->save($leadModel);
                }
            }

        }
    }

    public static function getTemplatesList($data)
    {
        $templateId = '';

//        $data['templateId'] = 'course_2';
//        $data['leadId'] = '9395';

        if (!empty($data['templateId']) && !empty($data['leadId'])) {
            $templateId = $data['templateId'];
            if ($templateId === 'default') {
                $activeActionEmailTriggers = Tools_LeadTools::getDefaultActionEmailInfo(Tools_LeadsMailWatchdog::LEADS_CONTACT);
                if (!empty($activeActionEmailTriggers)) {
                    $templateId = $activeActionEmailTriggers['template'];
                } else {
                    $templateId = 0;
                }
            }

            if (!Zend_Registry::isRegistered('extConfig')) {
                $configTable   = new Application_Model_DbTable_Config();
                Zend_Registry::set('extConfig', $configTable->selectConfig());
            }

            $emailTemplateDefaultMessage = Tools_LeadTools::getDefaultEmailMessage($templateId);
            $pageModel = Tools_LeadTools::getTemplatePage($templateId);
            $pageUrl = '';
            if ($pageModel instanceof Application_Model_Models_Page) {
                $pageUrl = $pageModel->getUrl();
                $websiteHelper = Zend_Controller_Action_HelperBroker::getExistingHelper('website');
                $themeData = Zend_Registry::get('theme');
                $extConfig = Zend_Registry::get('extConfig');
                $observableModel = Leads_Mapper_LeadsMapper::getInstance()->find($data['leadId']);
                $parserOptions = array(
                    'websiteUrl' => $websiteHelper->getUrl(),
                    'websitePath' => $websiteHelper->getPath(),
                    'currentTheme' => $extConfig['currentTheme'],
                    'themePath' => $themeData['path'],
                    'observableModel' => $observableModel
                );

                $parser = new Tools_Content_Parser($emailTemplateDefaultMessage, array(), $parserOptions);

                $emailTemplateDefaultMessage = $parser->parse();
                $emailTemplateDefaultMessage = '<div id="email-default-message-template">'.$emailTemplateDefaultMessage.'</div>';
            }

            return array('content' => $emailTemplateDefaultMessage);

        } else {
            $templates = Application_Model_Mappers_TemplateMapper::getInstance()->findByType('typemail');
            $templates = array_map(function ($model) {
                $modelData = $model->toArray();
                unset($modelData['content']);
                unset($modelData['oldName']);
                unset($modelData['id']);
                unset($modelData['type']);

                return $modelData;
            }, $templates);

            $templatesAndDirectoriesMapper = Leads_Mapper_LeadsTemplatesAndDirectoriesConfigMapper::getInstance();

            $savedConfigTemplates = $templatesAndDirectoriesMapper->findByType('template');

            if (!empty($savedConfigTemplates) && !empty($templates)) {
                foreach ($templates as $tKey => $template) {
                    if (!in_array($template['name'], $savedConfigTemplates)) {
                        unset($templates[$tKey]);
                    }
                }
            }

            $activeActionEmailTriggers = Tools_LeadTools::getDefaultActionEmailInfo(Tools_LeadsMailWatchdog::LEADS_CONTACT);
            $defaultEmailMessagePageUrl = '';
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();

            if (!empty($activeActionEmailTriggers)) {
                $defaultTemplateId = $activeActionEmailTriggers['template'];
                $pageModel = Tools_LeadTools::getTemplatePage($defaultTemplateId);
                if ($pageModel instanceof Application_Model_Models_Page) {
                    $defaultEmailMessagePageUrl = $websiteUrl.$pageModel->getUrl();
                }
            }

            $templatesData = array('0' => array('id' => 'default', 'preview' => $defaultEmailMessagePageUrl));
            if (!empty($templates)) {
                foreach ($templates as $template) {
                    $emailMessagePageUrl = '';
                    $pageModel = Tools_LeadTools::getTemplatePage($template['name']);
                    if ($pageModel instanceof Application_Model_Models_Page) {
                        $emailMessagePageUrl = $websiteUrl.$pageModel->getUrl();
                    }

                    array_push($templatesData, array('id' => $template['name'], 'preview' => $emailMessagePageUrl));
                }
            }

            return array('templates' => $templatesData);
        }

    }

    public static function getImagesList($data)
    {

        $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');

        if (empty($data['folderName'])) {
            $templatesAndDirectoriesMapper = Leads_Mapper_LeadsTemplatesAndDirectoriesConfigMapper::getInstance();
            $savedConfigFolders = $templatesAndDirectoriesMapper->findByType('folder');

            if (!empty($savedConfigFolders)) {
                $listFolders = array_values($savedConfigFolders);
            } else {
                $listFolders = self::getFoldersList();
                if (!empty($listFolders)){
                    $listFolders = array_values($listFolders);
                }
            }

            if (empty($listFolders)) {
                $listFolders = array();
            }

            return $listFolders;
        } else {
            $folderName = $data['folderName'];
            $filesPathUrl = $websiteHelper->getUrl() . $websiteHelper->getMedia() . $folderName . DIRECTORY_SEPARATOR;
            $filesPath = $websiteHelper->getPath() . $websiteHelper->getMedia() . $folderName;
            $files = ((is_dir($filesPath))) ? Tools_Filesystem_Tools::findFilesByExtension($filesPath,
                '.*', false, false, false) : array();

            $filesResult = array();
            foreach ($files as $file) {
                $filesResult[] = $filesPathUrl . $file;
            }

            $imagesPath = $websiteHelper->getPath() . $websiteHelper->getMedia() . $folderName;
            try {
                $imagesData = array(
                    'small' => self::prepareImageLinks(
                        Tools_Filesystem_Tools::scanDirectory($imagesPath . '/' . 'small'), $imagesPath, $folderName,
                        'small'),
                    'medium' => self::prepareImageLinks(Tools_Filesystem_Tools::scanDirectory($imagesPath . '/' . 'medium'),
                        $imagesPath, $folderName, 'medium'),
                    'large' => self::prepareImageLinks(Tools_Filesystem_Tools::scanDirectory($imagesPath . '/' . 'large'),
                        $imagesPath, $folderName, 'large'),
                    'original' => self::prepareImageLinks(Tools_Filesystem_Tools::scanDirectory($imagesPath . '/' . 'original'),
                        $imagesPath, $folderName, 'original')
                );
            } catch (Exceptions_SeotoasterException $se) {
                $imagesData = array(
                    'small' => array(),
                    'medium' => array(),
                    'large' => array(),
                    'original' => array()
                );
            }

            $result = array('files' => $filesResult, 'images' => $imagesData);

            return $result;
        }

    }

    public static function sendMobileEmail($data)
    {

//        $data = array(
//            'leadId' => '134',
//            'emailTemplate' => 'course_2',
//            'testEmail' => 'andreibudchenko@gmail.com',
//            'leadEmailFrom' => 'andreibudchenko@gmail.com',
//            'leadEmailTo' => 'andreibudchenko1@gmail.com,andreibudchenko2@gmail.com',
//            'leadEmailSubject' => 'My subject',
//            'emailContent' => '<div id="email-default-message-template">Some content here</div>',
//            'leadEmailBccTo' => '',
//        );

        $leadId = $data['leadId'];
        if (empty($leadId)) {
            return array('error' => 1, 'message' => 'Lead id is empty');
        }

        if (empty($data['ownerId'])) {
           $leadModel = Leads_Mapper_LeadsMapper::getInstance()->find($leadId);
           if (!$leadModel instanceof Leads_Model_LeadsModel) {
               return array('error' => 1, 'message' => 'Lead is not found');
           }
           $leadOwnerId = $leadModel->getOwnerId();
           $data['ownerId'] = $leadOwnerId;
        }


        if (!Zend_Registry::isRegistered('extConfig')) {
            $configHelper = new Helpers_Action_Config();
            $configHelper->init();
            Zend_Controller_Action_HelperBroker::addHelper($configHelper);
        }

        $userId = $data['ownerId'];

        $emailTemplate = $data['emailTemplate'];
        if (empty($emailTemplate)) {
            return array('error' => 1, 'message' => 'Email template is missing');
        }

        $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
        $leadModel = $leadMapper->find($leadId);
        if (!$leadModel instanceof Leads_Model_LeadsModel) {
            return array('error' => 1, 'message' => 'Lead is not found');
        }

        $testEmailFlag = false;

        if (!empty($data['testEmail'])) {
            $testEmailFlag = true;
        }

        $emailFrom = filter_var($data['leadEmailFrom'], FILTER_VALIDATE_EMAIL);
        if (empty($emailFrom)) {
            return array('error' => 1, 'message' => 'Missing email from');
        }

        $emailsTo = array_unique(array_map('trim',
            explode(',', filter_var($data['leadEmailTo'], FILTER_SANITIZE_STRING))));

        if (empty($emailsTo)) {
            return array('error' => 1, 'message' => 'Missing email to');
        }

        if (!empty($testEmailFlag) && empty($data['testEmail'])) {
            return array('error' => 1, 'message' => 'Missing test email');
        } elseif (!empty($data['testEmail'])) {
            if (empty(filter_var($data['testEmail'], FILTER_VALIDATE_EMAIL))) {
                return array('error' => 1, 'message' => 'Not valid test email address');
            }
            $emailsTo[0] = $data['testEmail'];
        }

        $subject = trim(filter_var($data['leadEmailSubject'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES));
        $emailContent = $data['emailContent'];
        if (empty($emailTemplate)) {
            return array('error' => 1, 'message' => 'Missing email template');
        }

        $bccEmails = filter_var($data['leadEmailBccTo'], FILTER_SANITIZE_STRING);
        if (!empty($bccEmails)) {
            $bccEmails = array_filter(array_unique(array_map('trim', explode(',', $bccEmails))));
        }

        if (preg_match("~email-default-message-template~ui", $emailContent)) {
            $doc = new DOMDocument();
            /**
             * LIBXML_HTML_NOIMPLIED turns off the automatic adding of implied html/body elements
             * LIBXML_HTML_NODEFDTD prevents a default doctype being added when one is not found.
             */
            $doc->loadHTML(mb_convert_encoding('<html><body>' . $emailContent . '</body></html>', 'HTML-ENTITIES',
                'UTF-8'), LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD);
            $xpath = new DOMXpath($doc);
            $div = $xpath->query("//*[@id='email-default-message-template']")->item(0);
            Tools_LeadsMailWatchdog::DomRemove($div);

            $emailContent = strtr($doc->saveHTML(),
                array('<html>' => '', '<body>' => '', '</body>' => '', '</html>' => ''));
            $emailContent = str_replace('%7B%24', '{$', $emailContent);
            $emailContent = str_replace('%7D', '}', $emailContent);
        }

        $emailValidate = new Tools_System_CustomEmailValidator();

        if (!empty($bccEmails)) {
            foreach ($bccEmails as $bccEmail) {
                if (!$emailValidate->isValid($bccEmail)) {
                    return array('error' => 1, 'message' => 'Not valid bcc email');
                }
            }
        }

        foreach ($emailsTo as $email) {
            if (!$emailValidate->isValid($email)) {
                return array('error' => 1, 'message' => 'Not valid email to');
            }
        }

        $leadTypeModel = Leads_Mapper_LeadsTypeMapper::getInstance()->findByTypeName(Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_EMAIL);
        if ($leadTypeModel instanceof Leads_Model_LeadsTypeModel && empty($testEmailFlag)) {
            $leadModel->setLastActivityTypeId($leadTypeModel->getId());
            $leadModel->setLastActivitySubtype('');
            $leadModel->setLastActivityOutbound(Tools_System_Tools::convertDateFromTimezone('now'));
            $leadModel = $leadMapper->save($leadModel);
        }

        foreach ($emailsTo as $userEmail) {
            $emailTriggerData = array(
                'trigger' => Tools_LeadsMailWatchdog::LEADS_CONTACT,
                'mailMessage' => $emailContent,
                'from' => $emailFrom,
                'emailTo' => $userEmail,
                'subject' => $subject,
                'userId' => $userId,
                'template' => $emailTemplate,
                'bccEmails' => $bccEmails,
                'defaultMessageTemplate' => true,
                'withoutTracking' => !empty($testEmailFlag),
                'skipBcc' => true,
                'emailsTo' => $emailsTo
            );

            if ($emailTemplate === 'default') {
                unset($emailTriggerData['template']);
            }

            $leadModel->registerObserver(new Tools_Mail_Watchdog($emailTriggerData));
        }

        if (!empty($bccEmails) && empty($testEmailFlag)) {
            $bccExistedLeads = array();

            foreach ($bccEmails as $key => $email) {
                $isLeadExist = $leadMapper->findByEmail($email);

                if ($isLeadExist instanceof Leads_Model_LeadsModel) {
                    $bccExistedLeads[] = array(
                        'leadEmail' => $isLeadExist->getLeadEmail(),
                        'leadModel' => $isLeadExist
                    );

                    unset($bccEmails[$key]);
                }
            }

            if (!empty($bccEmails)) {
                $mainBccEmail = array_shift($bccEmails);
                $emailTriggerData = array(
                    'trigger' => Tools_LeadsMailWatchdog::LEADS_CONTACT,
                    'mailMessage' => $emailContent,
                    'from' => $emailFrom,
                    'emailTo' => $mainBccEmail,
                    'subject' => $subject,
                    'userId' => $userId,
                    'template' => $emailTemplate,
                    'bccEmails' => $bccEmails,
                    'withoutTracking' => true,
                    'defaultMessageTemplate' => true
                );

                if ($emailTemplate === 'default') {
                    unset($emailTriggerData['template']);
                }

                $leadModel->registerObserver(new Tools_Mail_Watchdog($emailTriggerData));
            }
        }

        $leadModel->notifyObservers();

        if (!empty($bccExistedLeads)) {
            foreach ($bccExistedLeads as $bccLead) {
                $emailTriggerData = array(
                    'trigger' => Tools_LeadsMailWatchdog::LEADS_CONTACT,
                    'mailMessage' => $emailContent,
                    'from' => $emailFrom,
                    'emailTo' => $bccLead['leadEmail'],
                    'subject' => $subject,
                    'userId' => $userId,
                    'template' => $emailTemplate,
                    'bccEmails' => array(),
                    'withoutTracking' => false,
                    'defaultMessageTemplate' => true
                );

                if ($emailTemplate === 'default') {
                    unset($emailTriggerData['template']);
                }

                $bccLead['leadModel']->registerObserver(new Tools_Mail_Watchdog($emailTriggerData));

                $bccLead['leadModel']->notifyObservers();
            }
        }

        return array('error' => 0);

    }


    public static function prepareImageLinks(array $images, $path, $folder, $type)
    {
        if (!empty ($images)) {
            $imgResult = array();
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $srcPath = $websiteHelper->getUrl() . $websiteHelper->getMedia() . $folder;

            foreach ($images as $key => $image) {
                $imgResult[] = $srcPath . DIRECTORY_SEPARATOR . $type . DIRECTORY_SEPARATOR . $image;
            }

            return $imgResult;
        }
    }

    public static function getMobileEmailWidgets()
    {
        $mobileEmailWidgets = Tools_LeadTools::leadUsefulShortcuts();

        return array('mobileEmailWidgets' => $mobileEmailWidgets);
    }

    public function saveFiscalYearStartAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $shoppingConfigMapper = Models_Mapper_ShoppingConfig::getInstance();
            $fiscalYearStartMonth = filter_var($this->_request->getParam('fiscalYearStartMonth'), FILTER_SANITIZE_NUMBER_INT);

            if(!empty($fiscalYearStartMonth)) {
                $shoppingConfigMapper->save(array('fiscalYearStart' => $fiscalYearStartMonth));

                $this->_responseHelper->success($this->_translator->translate(self::TRANSLATION_PREFIX.'Saved'));
            }
        }

        $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Can\'t save fiscal year start month'));
    }

    public function changeMarketingEmailsAndSendSmsSequenceAction()
    {
        if ($this->_request->isPost()) {
            $secureToken = $this->_request->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
            $tokenValid = Tools_System_Tools::validateToken($secureToken, Api_Leads_Leadreports::LEAD_REPORTS_SECURE_TOKEN);
            if (!$tokenValid) {
                $this->_responseHelper->fail('Invalid token');
            }

            $sequenceType = filter_var($this->_request->getParam('sequenceType'), FILTER_SANITIZE_STRING);
            $sequenceAction = filter_var($this->_request->getParam('sequenceAction'), FILTER_SANITIZE_STRING);
            $translator = Zend_Registry::get('Zend_Translate');
            $selectedSequenceIds =  filter_var_array($this->_request->getParam('selectedSequence'), FILTER_SANITIZE_STRING);
            $leadReportsCompare = filter_var($this->_request->getParam('leadReportsCompare'), FILTER_SANITIZE_NUMBER_INT);
            $leadReportsFilterStartDateOrigin = filter_var($this->_request->getParam('leadReportsFilterStartDate'), FILTER_SANITIZE_STRING);
            $leadReportsFilterEndDateOrigin = filter_var($this->_request->getParam('leadReportsFilterEndDate'), FILTER_SANITIZE_STRING);
            $leadReportsCompareFilterStartDateOrigin = filter_var($this->_request->getParam('leadReportsCompareFilterStartDate'), FILTER_SANITIZE_STRING);
            $leadReportsCompareFilterEndDateOrigin = filter_var($this->_request->getParam('leadReportsCompareFilterEndDate'), FILTER_SANITIZE_STRING);

            $leadReportsFilterStartDateOriginForLabel = $leadReportsFilterStartDateOrigin;
            if (empty($leadReportsFilterStartDateOriginForLabel)) {
                $leadReportsFilterStartDateOriginForLabel = Tools_System_Tools::convertDateFromTimezone('now');
            }
            $leadReportsFilterEndDateOriginForLabel = $leadReportsFilterEndDateOrigin;
            if (empty($leadReportsFilterEndDateOriginForLabel)) {
                $leadReportsFilterEndDateOriginForLabel = Tools_System_Tools::convertDateFromTimezone('now');
            }

            $leadReportsCompareFilterStartDateOriginForLabel = $leadReportsCompareFilterStartDateOrigin;
            if (empty($leadReportsCompareFilterStartDateOriginForLabel)) {
                $leadReportsCompareFilterStartDateOriginForLabel = Tools_System_Tools::convertDateFromTimezone('now');
            }

            $leadReportsCompareFilterEndDateOriginForLabel = $leadReportsCompareFilterEndDateOrigin;
            if (empty($leadReportsCompareFilterEndDateOriginForLabel)) {
                $leadReportsCompareFilterEndDateOriginForLabel = Tools_System_Tools::convertDateFromTimezone('now');
            }

            $leadReportsFilterDateOriginForLabel = date('d-M-Y', strtotime($leadReportsFilterStartDateOriginForLabel)).' to '. date('d-M-Y', strtotime($leadReportsFilterEndDateOriginForLabel));
            $leadReportsFilterDateCompareForLabel = date('d-M-Y', strtotime($leadReportsCompareFilterStartDateOriginForLabel)).' to '. date('d-M-Y', strtotime($leadReportsCompareFilterEndDateOriginForLabel));

            $leadReportsFilterStartDate = Tools_System_Tools::convertDateFromTimezone($leadReportsFilterStartDateOrigin. ' 00:00:00', false, 'UTC', 'Y-m-d H:i:s');
            $leadReportsFilterEndDate = Tools_System_Tools::convertDateFromTimezone($leadReportsFilterEndDateOrigin. ' 23:59:59', false, 'UTC', 'Y-m-d H:i:s');
            $leadReportsCompareFilterStartDate = Tools_System_Tools::convertDateFromTimezone($leadReportsCompareFilterStartDateOrigin. ' 00:00:00', false, 'UTC', 'Y-m-d H:i:s');
            $leadReportsCompareFilterEndDate = Tools_System_Tools::convertDateFromTimezone($leadReportsCompareFilterEndDateOrigin. ' 23:59:59', false, 'UTC', 'Y-m-d H:i:s');

            $sequenceIds = array();
            if (!empty($selectedSequenceIds)) {
                $sequenceIds = $selectedSequenceIds;
            }

            $leadReportsMapper = Leads_Mapper_LeadsReportsMapper::getInstance();
            $restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);

            if($sequenceType == 'email') {
                $graphResultLabel = Tools_LeadReportsTools::GRAPH_RESULT_LABEL_MARKETING_SEQUENCES_EMAILS_COUNT_BAR_OPEN;

                if(!empty($sequenceAction) && $sequenceAction == 'send') {
                    $graphResultLabel = Tools_LeadReportsTools::GRAPH_RESULT_LABEL_MARKETING_SEQUENCES_EMAILS_COUNT_BAR_SEND;
                }

            } elseif($sequenceType == 'sms') {
                $graphResultLabel = Tools_LeadReportsTools::GRAPH_RESULT_LABEL_MARKETING_SEQUENCES_SMS_COUNT_BAR_SEND;
            }

            if (!empty($leadReportsCompare)) {
                if($sequenceType == 'email') {
                    $sequencesData = $leadReportsMapper->getSequenceOpenSendEmailsByOwnerCountAggregated($leadReportsFilterStartDate, $leadReportsFilterEndDate, $leadReportsCompareFilterStartDate, $leadReportsCompareFilterEndDate, $restrictedUserId, $sequenceAction, $sequenceIds);
                } elseif ($sequenceType == 'sms') {
                    $sequencesData = $leadReportsMapper->getSequenceSendSmsByOwnerCountAggregated($leadReportsFilterStartDate, $leadReportsFilterEndDate, $leadReportsCompareFilterStartDate, $leadReportsCompareFilterEndDate, $restrictedUserId, $sequenceIds);
                }

                $sequencesDataReportResult = Tools_LeadReportsTools::prepareSequenceOpenEmailsAndSendSmsByOwnerDataAggregated($sequencesData, true, $graphResultLabel);
            } else {
                if($sequenceType == 'email') {
                    $sequencesData = $leadReportsMapper->getSequenceOpenSendEmailsByOwnerCountAggregated($leadReportsFilterStartDate, $leadReportsFilterEndDate, false, false, $restrictedUserId, $sequenceAction, $sequenceIds);
                } elseif ($sequenceType == 'sms') {
                    $sequencesData = $leadReportsMapper->getSequenceSendSmsByOwnerCountAggregated($leadReportsFilterStartDate, $leadReportsFilterEndDate, false, false, $restrictedUserId, $sequenceIds);
                }
                $sequencesDataReportResult = Tools_LeadReportsTools::prepareSequenceOpenEmailsAndSendSmsByOwnerDataAggregated($sequencesData, false, $graphResultLabel);
            }

            if (!empty($sequencesDataReportResult)) {
                /*if($sequenceType == 'email') {
                    $sequencesDataReportResult['title'] =  $translator->translate(self::TRANSLATION_PREFIX.Tools_LeadReportsTools::GRAPH_RESULT_LABEL_MARKETING_SEQUENCES_EMAILS_COUNT_BAR_OPEN);

                    if(!empty($sequenceAction) && $sequenceAction == 'send') {
                        $sequencesDataReportResult['title'] =  $translator->translate(self::TRANSLATION_PREFIX.Tools_LeadReportsTools::GRAPH_RESULT_LABEL_MARKETING_SEQUENCES_EMAILS_COUNT_BAR_SEND);
                    }
                } elseif($sequenceType == 'sms') {
                    $sequencesDataReportResult['title'] =  $translator->translate(self::TRANSLATION_PREFIX.Tools_LeadReportsTools::GRAPH_RESULT_LABEL_MARKETING_SEQUENCES_SMS_COUNT_BAR_SEND);
                }*/

                if (!empty($sequencesDataReportResult['data'])) {
                    $sequencesDataReportResult['data'][0] = $leadReportsFilterDateOriginForLabel;
                }

                if (!empty($sequencesDataReportResult['dataToCompare'])) {
                    $sequencesDataReportResult['dataToCompare'][0] = $leadReportsFilterDateCompareForLabel;
                }

                $this->_responseHelper->success(array('sequencesDataReportResult' => $sequencesDataReportResult));
            }

        }
    }

    /**
     * @return void
     */
    public function getUnusedFieldsAction()
    {
        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
            if (empty($restrictedUserId) && $this->_request->isPost()) {
                $secureToken = $this->_request->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
                $tokenValid = Tools_System_Tools::validateToken($secureToken, Api_Leads_Leads::LEADS_SECURE_TOKEN);
                if (!$tokenValid) {
                    $this->_responseHelper->fail('Invalid token');
                }

                $type = filter_var($this->_request->getParam('type'), FILTER_SANITIZE_STRING);

                if(!empty($type)) {
                    if($type == 'lead-custom-fields') {
                        $leadsCustomParamsConfigMapper = Leads_Mapper_LeadsCustomParamsConfigMapper::getInstance();

                        $data = $leadsCustomParamsConfigMapper->getAllCustomFields();

                        if(!empty($data)) {
                            $data = $this->_processEmptyUnusedFields($data);
                        }

                        $unusedFieldsCount = count($data);

                        $this->_responseHelper->success(array('unusedFields' => $data, 'unusedFieldsCount' => $unusedFieldsCount));
                    } elseif($type == 'organization-custom-fields') {
                        $leadsOrganizationCustomParamsConfigMapper = Leads_Mapper_LeadsOrganizationCustomParamsConfigMapper::getInstance();
                        $data = $leadsOrganizationCustomParamsConfigMapper->getAllCustomFields();

                        if(!empty($data)) {
                            $data = $this->_processEmptyUnusedFields($data);
                        }

                        $unusedFieldsCount = count($data);

                        $this->_responseHelper->success(array('unusedFields' => $data, 'unusedFieldsCount' => $unusedFieldsCount));
                    } elseif($type == 'lead-tags') {
                        $leadsTagsMapper = Leads_Mapper_LeadsTagsMapper::getInstance();
                        $data = $leadsTagsMapper->getUnusedTags();
                        $unusedFieldsCount = count($data);

                        $this->_responseHelper->success(array('unusedFields' => $data, 'unusedFieldsCount' => $unusedFieldsCount));
                    }
                }
            }
        }

        $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Can\'t get unused fields'));
    }

    /**
     * Find not empty custom params and clear them
     *
     * @param $data
     * @return array
     */
    protected function _processEmptyUnusedFields($data) {
        $clearData = array();

        foreach ($data as $key => $params) {
            if(empty($params['param_value'])) {
                $clearData[$params['id']] = $params;
            }
        }

        if(!empty($clearData)) {
            foreach ($clearData as $key => $params) {
                foreach ($data as $dataParams) {
                    if($params['id'] == $dataParams['id'] && !empty($dataParams['param_value']) && array_key_exists($key, $clearData)) {
                        unset($clearData[$key]);
                    }
                }
            }
        }

        return $clearData;
    }

    /**
     * @return void
     * @throws Exception
     */
    public function deleteUnusedFieldsAction()
    {

        if (Tools_Security_Acl::isAllowed(Shopping::RESOURCE_STORE_MANAGEMENT)) {
            $restrictedUserId = Tools_LeadTools::getUserIdIfActionNotAllowed(Shopping::ROLE_SALESPERSON);
            if (empty($restrictedUserId) && $this->_request->isPost()) {
                $secureToken = $this->_request->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
                $tokenValid = Tools_System_Tools::validateToken($secureToken, Api_Leads_Leads::LEADS_SECURE_TOKEN);
                if (!$tokenValid) {
                    $this->_responseHelper->fail('Invalid token');
                }

                $type = filter_var($this->_request->getParam('type'), FILTER_SANITIZE_STRING);
                $checkedUnusedFields = $this->_request->getParam('checkedUnusedFields');

                if(!empty($type) && !empty($checkedUnusedFields)) {
                    $leadsCleanUpFieldsLogMapper = Leads_Mapper_LeadsCleanUpFieldsLogMapper::getInstance();
                    $loggedUserId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();
                    $ipAddress = $_SERVER['REMOTE_ADDR'];
                    $createdAt = Tools_System_Tools::convertDateFromTimezone('now');
                    $filteredDataParams = array();

                    $deletedCounter = 0;
                    if($type == 'lead-custom-fields') {
                        $leadsCustomParamsConfigMapper = Leads_Mapper_LeadsCustomParamsConfigMapper::getInstance();
                        $where = $leadsCustomParamsConfigMapper->getDbTable()->getAdapter()->quoteInto('pllcpc.id IN (?)', $checkedUnusedFields);
                        $data = $leadsCustomParamsConfigMapper->fetchAll($where, null, null, null, true);

                        if(!empty($data)) {
                            foreach ($data as $dataParam) {
                                $param = array();
                                $deletedCounter ++;

                                $param['param_type'] = $dataParam['param_type'];
                                $param['param_name'] = $dataParam['param_name'];
                                $param['label'] = $dataParam['label'];

                                $filteredDataParams[] = $param;

                                $leadsCustomParamsConfigMapper->delete($dataParam['id']);
                            }

                            $filteredDataParams = json_encode($filteredDataParams);
                        }

                    } elseif ($type == 'organization-custom-fields') {
                        $leadsOrganizationCustomParamsConfigMapper = Leads_Mapper_LeadsOrganizationCustomParamsConfigMapper::getInstance();
                        $where = $leadsOrganizationCustomParamsConfigMapper->getDbTable()->getAdapter()->quoteInto('plocpc.id IN (?)', $checkedUnusedFields);
                        $data = $leadsOrganizationCustomParamsConfigMapper->fetchAll($where, null, null, null, true);

                        if(!empty($data)) {
                            foreach ($data as $dataParam) {
                                $param = array();
                                $deletedCounter ++;

                                $param['param_type'] = $dataParam['param_type'];
                                $param['param_name'] = $dataParam['param_name'];
                                $param['label'] = $dataParam['label'];

                                $filteredDataParams[] = $param;

                                $leadsOrganizationCustomParamsConfigMapper->delete($dataParam['id']);
                            }

                            $filteredDataParams = json_encode($filteredDataParams);
                        }
                    } elseif ($type == 'lead-tags') {
                        $leadsTagsMapper = Leads_Mapper_LeadsTagsMapper::getInstance();
                        $where = $leadsTagsMapper->getDbTable()->getAdapter()->quoteInto('plt.id IN (?)', $checkedUnusedFields);
                        $data = $leadsTagsMapper->fetchAll($where, null, null, null, true);

                        if(!empty($data)) {
                            foreach ($data as $dataParam) {
                                $param = array();
                                $deletedCounter ++;

                                $param['tag_name'] = $dataParam['tag_name'];
                                $param['label'] = $dataParam['label'];

                                $filteredDataParams[] = $param;

                                $leadsTagsMapper->delete($dataParam['id']);
                            }

                            $filteredDataParams = json_encode($filteredDataParams);
                        }
                    }

                    if($deletedCounter) {
                        $leadsCleanUpFieldsLogModel = new Leads_Model_LeadsCleanUpFieldsLogModel();
                        $leadsCleanUpFieldsLogModel->setCreatedAt($createdAt);
                        $leadsCleanUpFieldsLogModel->setLoggedUserId($loggedUserId);
                        $leadsCleanUpFieldsLogModel->setType($type);
                        $leadsCleanUpFieldsLogModel->setIpAddress($ipAddress);
                        $leadsCleanUpFieldsLogModel->setDeletedItemsInfo($filteredDataParams);

                        $leadsCleanUpFieldsLogMapper->save($leadsCleanUpFieldsLogModel);

                        $this->_responseHelper->success(array('deletedItems' => $deletedCounter));
                    } else {
                        $this->_responseHelper->success($this->_translator->translate(self::TRANSLATION_PREFIX.'Nothing to delete'));
                    }
                }
            }
        }

        $this->_responseHelper->fail($this->_translator->translate(self::TRANSLATION_PREFIX.'Can\'t delete fields'));
    }

}
